using System;
using com.plumtree.openlog;
using com.plumtree.portalnavigation.common.constants;
using com.plumtree.portalpages.common.mediator;
using com.plumtree.portalpages.common.plugnav;
using com.plumtree.portalpages.common.uiparts.navpagetype;
using com.plumtree.portaluiinfrastructure.compoundlist;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalnavigation.views
{
  /// <summary>
  /// Accessible navigation view (used for 508, and low bandwidth).
  /// This view displays actions for the My Pages, My Communities, Directory 
  /// sections. 
  /// This view is used by NavTypeAccessible in the below body section. 
  /// No javascript is used.
  /// </summary>
  /// <author>MichaelD</author>
  public class NavigationAccessibleBelowBodyView : IView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
     "com.plumtree.portalnavigation.views.NavigationAccessibleBelowBodyView");
    
    /// <summary>
    /// MVC object name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "NavigationAccessibleBelowBody";
    
    /// <summary>
    /// The maximum length of strings before truncation *
    /// </summary>
    public const int ACTION_TEXT_MAX_LENGTH = 31;
    
    /// <summary>
    /// Action Divider 
    /// </summary>
    public static readonly String STR_ACTION_DIVIDER = CommonHTMLStrings.SPACE + "|" + CommonHTMLStrings.SPACE;
    protected AActivitySpace m_asOwner;
    protected IPluggableNavModelRO m_model;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Display()"></seealso>
    public virtual HTMLElement Display()
    {
      HTMLElementCollection result = null;
      try
      {
        result = WritePageOptions();
      }
      catch (Exception e)
      {
        log.Error(e, "Could not display accessible below body navigation view.");
        }
      return result;
    }
    
    /// <summary>
    /// This method returns the HTMLScript that encompasses the Javascript on the page.
    /// </summary>
    /// <returns>HTMLScript</returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    public virtual HTMLScript DisplayJavascript()
    {
      return null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Init(IModelRO,AActivitySpace)"></seealso>
    public virtual void Init(IModelRO model, AActivitySpace parent)
    {
      m_model = (IPluggableNavModelRO) model;
      m_asOwner = parent;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new NavigationAccessibleBelowBodyView();
    }
    
    /// <summary>
    /// Adds an HTMLTable to the HTMLElementCollection with the Mypage links section
    /// </summary>
    /// <param name="col">The HTMLElementCollection to add the HTMLTable too</param>
    /// <throws>HTMLException</throws>
    /// <throws>Exception</throws>
    protected virtual HTMLElementCollection WritePageOptions()
    {
      // Determine whether to show actions and which actions to show.
      // Also determine if we need the Page Options header. Don't need it in the directory, it has its own.
      NavCategoryType navCatType = null;
      bool bShowHeader = true;
      if (NavPageType.COMMUNITY == m_model.GetCurNavPageType())
      {
        navCatType = NavCategoryType.COMMUNITY;
      } else
      if (NavPageType.MYPAGE == m_model.GetCurNavPageType())
      {
        navCatType = NavCategoryType.MYPAGE;
      } else
      if (m_model.GetSubPortalEnabled(PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_COMMUNITIES_ACCESS) && 0 == m_model.GetCategoryLinks(NavCategoryType.MYCOMMUNITIES,
       false).GetTotalCount())
      {
        bShowHeader = false;
      } else
      {
        return null;
      }
      HTMLElementCollection col = new HTMLElementCollection();
      HTMLTable table = new HTMLTable();
      col.AddInnerHTMLElement(table);
      table.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
      HTMLTableRow row = new HTMLTableRow();
      table.AddInnerHTMLElement(row);
      HTMLTableCell cell = new HTMLTableCell();
      row.AddInnerHTMLElement(cell);
      if (bShowHeader)
      {
        row.SetStyleClass(PTStyleClass.OBJECT_HEADER_BG);
        cell.SetStyleClass(PTStyleClass.OBJECT_HEADER);
        // Page Options Anchor
        HTMLAnchor anchor = new HTMLAnchor(CommonHTMLStrings.POUND_SYMBOL);
        cell.AddInnerHTMLElement(anchor);
        anchor.SetTitle(m_asOwner.GetString(764, com.plumtree.portalnavigation.common.constants.ResourceConstants.PORTALCOMMONMSGS));
        anchor.SetName(NavigationAccessibleAboveBodyView.STR_OPTIONS_ANCHOR);
        // Page Options:
        cell.AddInnerHTMLString(m_asOwner.GetString(763, com.plumtree.portalnavigation.common.constants.ResourceConstants.PORTALCOMMONMSGS));
        row = new HTMLTableRow();
        table.AddInnerHTMLElement(row);
        cell = new HTMLTableCell();
        row.AddInnerHTMLElement(cell);
      }
      row.SetStyleClass(PTStyleClass.NAV_SIDEBARSECTION_BG);
      if (NavPageType.COMMUNITY == m_model.GetCurNavPageType())
      {
        AddParentCommLinkToCell(cell);
      }
      int nCategoryActionsAdded = 0;
      if (navCatType != null)
      {
        // Category actions
        nCategoryActionsAdded = AddCategoryActionsToCell(cell, navCatType);
        }
      if (NavPageType.COMMUNITY != m_model.GetCurNavPageType() && m_model.GetSubPortalEnabled(PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_COMMUNITIES_ACCESS) && 0 == m_model.GetCategoryLinks(NavCategoryType.MYCOMMUNITIES,
       false).GetTotalCount())
      {
        if (navCatType != null && nCategoryActionsAdded > 0)
        {
          cell.AddInnerHTMLString(STR_ACTION_DIVIDER);
        }
        AddCategoryActionsToCell(cell, NavCategoryType.COMMUNITY);
      }
      return col;
    }
    
    /// <summary>
    /// Adds the actions for a category to the cell passed in.
    /// </summary>
    /// <param name="cell">Cell that the actions will be added to.</param>
    /// <param name="navCategoryType">Category type of the actions to be added.</param>
    /// <returns>number of Category Actions added</returns>
    protected virtual int AddCategoryActionsToCell(HTMLTableCell cell, NavCategoryType navCategoryType)
    {
      ICPListEntryIterator iterActions = m_model.GetCategoryLinks(navCategoryType,
       true);
      CListURLMediator med = new CListURLMediator(m_asOwner, iterActions);
      med.SetLabelMaxLength(ACTION_TEXT_MAX_LENGTH);
      med.SetShowImages(false);
      int i = iterActions.GetTotalCount();
      while (med.Next())
      {
        i--;
        HTMLAnchor anchor = (HTMLAnchor) med.GetEntry();
        if (anchor != null)
        {
          // Put action anchor in cell
          HTMLSpan span = new HTMLSpan();
          span.SetStyleClass(PTStyleClass.NAV_SIDEBAR_TEXT);
          span.AddInnerHTMLElement(anchor);
          cell.AddInnerHTMLElement(span);
          if (i > 0)
          {
            cell.AddInnerHTMLString(STR_ACTION_DIVIDER);
          }
        }
      }
      return iterActions.GetTotalCount();
    }
    protected virtual void AddParentCommLinkToCell(HTMLTableCell cell)
    {
      ICPListEntryIterator iterParentComm = ((CommunityNavPage) m_model.GetPageSpecificMethods()).GetParentComm();
      // Can't use the CListURLTemplateLinkMediator because we would get an anchor with displau label,
      // only want the url, construct our own anchor from the template string array instead
      CListURLMediator mediator = new CListURLMediator(m_asOwner, iterParentComm);
      mediator.SetLabelMaxLength(ACTION_TEXT_MAX_LENGTH);
      mediator.SetShowImages(false);
      // Go up to parent community link
      if (mediator.Next())
      {
        HTMLAnchor tempanchor = (HTMLAnchor) mediator.GetEntry();
        if (tempanchor != null)
        {
          HTMLAnchor anchor = new HTMLAnchor(tempanchor.GetHRef());
          anchor.SetTitle(m_asOwner.GetString(767, ResourceConstants.PORTALCOMMONMSGS));
          anchor.AddInnerHTMLString(m_asOwner.GetString(767, ResourceConstants.PORTALCOMMONMSGS));
          // Put action anchor in cell
          HTMLSpan span = new HTMLSpan();
          span.SetStyleClass(PTStyleClass.NAV_SIDEBAR_TEXT);
          span.AddInnerHTMLElement(anchor);
          cell.AddInnerHTMLElement(span);
          cell.AddInnerHTMLString(STR_ACTION_DIVIDER);
        }
      }
    }
  }
}