using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalnavigation.common.constants;
using com.plumtree.portalpages.common.mediator;
using com.plumtree.portalpages.common.plugnav;
using com.plumtree.portalpages.common.uiparts;
using com.plumtree.portaluiinfrastructure.application.varpacks;
using com.plumtree.portaluiinfrastructure.compoundlist;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalnavigation.views
{
  /// <summary>
  /// View for listing main navigation tabs using HTML Select elements
  /// This navigation view displays "My Pages", "My Communities", "Directory" and mandatory links 
  /// combobox menus. Mandatory communities are displayed in a separate row as tabs.
  /// This view is used by the NavTypeHorizontalComboBox navigation scheme in the above banner
  /// section, although it can be used in any horizontal space since it doesn't have any external
  /// dependencies except for the mandatory community tabs which only needs navigationhorizontalcommon.js
  /// (included in NavTypeHorizontalComboBox.java).
  /// Combobox menus are native HTML elements, making them simple to create and use and they work in all
  /// browsers. The downside is that they are not very customizable. All entries in the list use the 
  /// same style and they don't have support for images or other HTML in the menus. 
  /// </summary>
  /// <author>robertz</author>
  public class NavigationHorizontalComboBoxView : IView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
     "com.plumtree.portalnavigation.views.NavigationHorizontalComboBoxView");
    private AActivitySpace m_asOwner;
    private IPluggableNavModelRO m_model;
    
    /// <summary>
    /// MVC name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "HorizontalComboBoxMenu";
    
    /// <summary>
    /// Truncation width of the strings in the HTML Selects *
    /// </summary>
    public const int MENU_TEXT_MAX_LENGTH = 29;
    
    /// <summary>
    /// Index for each of the navigation tabs 
    /// </summary>
    public const int MYPAGES_TAB = 0;
    public const int COMMUNITY_TAB = 1;
    public const int DOCUMENTS_TAB = 2;
    public const int PREFERENCES_TAB = 3;
    
    /// <summary>
    /// Constant definitions of common strings *
    /// </summary>
    public const String m_spacerOptionRow = "-------------------------";
    public const String m_spacerOptionRowFiller = "&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;";
    public const String m_spacerOptionRowShort = "&nbsp; ";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new NavigationHorizontalComboBoxView();
    }
    
    /// <summary>
    /// This method returns the HTMLScript that encompasses the Javascript on the page.
    /// </summary>
    /// <returns>HTMLScript</returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    public virtual HTMLScript DisplayJavascript()
    {
      return null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Display()"></seealso>
    public virtual HTMLElement Display()
    {
      HTMLElementCollection myGroup = new HTMLElementCollection();
      HTMLElement result = myGroup;
      try
      {
        result.AddInnerHTMLElement(DisplayViewJavascript());
        // Spacer row
        HTMLTable spacerTable = new HTMLTable();
        HTMLTableRow tRow = new HTMLTableRow();
        HTMLTableCell tCell = new HTMLTableCell();
        spacerTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        spacerTable.SetBorder(CommonHTMLStrings.ZERO);
        spacerTable.SetCellPadding(CommonHTMLStrings.ZERO);
        spacerTable.SetCellSpacing(CommonHTMLStrings.ZERO);
        result.AddInnerHTMLElement(spacerTable);
        spacerTable.AddInnerHTMLElement(tRow);
        tRow.AddInnerHTMLElement(tCell);
        tRow.SetStyleClass(PTStyleClass.BAN_HEADER_BG);
        tCell.SetStyleClass(PTStyleClass.NAV_MIDTABSPACER_BG);
        tCell.AddInnerHTMLElement(NavigationCommonHelpers.GetSpacerImage("1",
         "4"));
        // main table with combo boxes -- this is a top-level HTMLElement,
        // so the language needs to be set on it.
        HTMLTable table = new HTMLTable();
        String strLang = (String) PersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE,
         m_asOwner);
        table.SetLangGlobal(strLang);
        table.SetBorder(CommonHTMLStrings.ZERO);
        table.SetCellPadding(CommonHTMLStrings.ZERO);
        table.SetCellSpacing(CommonHTMLStrings.ZERO);
        HTMLTableRow row = new HTMLTableRow();
        row.SetStyleClass(PTStyleClass.BAN_HEADER_BG);
        HTMLTableCell tSpacer = new HTMLTableCell();
        tSpacer.SetStyleClass(PTStyleClass.NAV_MIDTABSPACER_BG);
        tSpacer.AddInnerHTMLElement(NavigationCommonHelpers.GetSpacerImage("4",
         "1"));
        if (NavPageType.COMMUNITYPREVIEW != m_model.GetCurNavPageType())
        {
          // Only display the comboboxes for each section if they are enabled in the subportal
          if (m_model.GetSubPortalEnabled(PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_MYPAGES_ACCESS))
          {
            row.AddInnerHTMLElement(tSpacer);
            WriteMyPagesSection(row);
          }
          if (m_model.GetSubPortalEnabled(PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_COMMUNITIES_ACCESS))
          {
            row.AddInnerHTMLElement(tSpacer);
            WriteCommunitiesSection(row);
          }
          if (m_model.GetSubPortalEnabled(PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_DIRECTORY_ACCESS))
          {
            row.AddInnerHTMLElement(tSpacer);
            WriteDocumentsSection(row);
          }
          // Spacer
          row.AddInnerHTMLElement(tSpacer);
          // Add the combobox for subportal mandatory links (if any)
          WriteMandLinksSection(row);
          // Spacer
          row.AddInnerHTMLElement(tSpacer);
        }
        // Spacer cell to fill out the row
        tCell = new HTMLTableCell();
        tCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        tCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        row.AddInnerHTMLElement(tCell);
        table.AddInnerHTMLElement(row);
        // Call the javascript function to write out the mandatory community tabs, which
        // are displayed in the next row.
        HTMLScript script = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
        script.AddInnerHTMLString("writeTabsRows(" + JSHelper.GetSafeJSVariableWarn(NavigationCommonHelpers.JSCRIPT_VAR_MANDTAB_ARRAY) + ");");
        // Need to get the HTML from the table html element as a string and 
        // enclose it in an HTML form pad it with special HTML to avoid the problem
        // where Netscape inserts extra whitespace after forms.
        // Set the image server url in the table manually since we are extracting the html
        // in the table element as a string.  
        table.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_asOwner));
        // This is an awful hack but only way I found to get rid of the extra form spacing in Netscape. 
        result.AddInnerHTMLElement(new HTMLGenericElement("<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\" width=\"100%\"><tr><td>" + "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\" width=\"100%\"><form name=\"filler\">" + table.GetInnerHTMLDisplayString() + "</form></table></td></tr></table>"));
        result.AddInnerHTMLElement(script);
      }
      catch (Exception he)
      {
        log.Error(he, "Unable displaying Combobox menus: " + he.Message);
        return new HTMLGenericElement("There was a problem displaying the Combobox menu bar.");
        }
      return result;
    }
    
    /// <summary>
    /// This method returns the HTMLScript that encompasses the Javascript on the page.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    /// <returns>HTMLScript</returns>
    public virtual HTMLScriptCollection DisplayViewJavascript()
    {
      // Javascript function used as combobox menu onchange event handler
      HTMLScriptCollection col = new HTMLScriptCollection();
      String sOut = null;
      sOut = "\tfunction NavSelectMenuChange(strAction, objSelect)\n" + "\t\t{  \n" + "\t\t  if (strAction==\'\') {\n" + "\t\t    objSelect[0].selected=true;\n" + "\t\t  } else {\n" + "\t\t    " + NavigationCommonHelpers.URLTEMPLATE_REDIRECT_JSFUNCNAME + "(strAction);\n" + "\t\t  }\n" + "\t\t}  \n";
      HTMLScript script = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      col.AddInnerHTMLElement(script);
      try
      {
        script.AddInnerHTMLString(sOut);
        // Add the help javascript function used by the display mandatory community tab function
        // (writeMandatoryTabs in navigationhorizontalcommon.js on the imageserver). This function
        // is called in the Display method.
        col.AddInnerHTMLElement(NavigationCommonHelpers.GenerateJSHTMLHelperFunctions(m_asOwner));
        // Create the javascript array with the mandatory community tabs. Used by writeMandatoryTabs 
        // when displaying the mandatory tabs. 
        col.AddInnerHTMLElement(NavigationCommonHelpers.CreateMandatoryTabsArray(m_model.GetCategoryLinks(NavCategoryType.MANDATORYTABS,
         true), m_asOwner));
      }
      catch (Exception e)
      {
        log.Error(e, "Unable displaying Javascript for Combobox menus: " + e.Message);
        }
      return col;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IView#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IView#Init(IModel,AActivitySpace)"></seealso>
    public virtual void Init(IModelRO model, AActivitySpace parent)
    {
      m_model = (IPluggableNavModelRO) model;
      m_asOwner = parent;
    }
    
    /// <summary>
    /// Helper function to populate a select box with options from an
    /// ASCompoundList
    /// </summary>
    /// <param name="select">HTML Select to populate</param>
    /// <param name="cpList">The list of links</param>
    protected virtual void AddSelectOptions(HTMLSelect select, ICPListEntryIterator cpList,
     String strLabelPrefix)
    {
      ICPListIterator mediator = new CListURLTemplateMediator(m_asOwner, cpList);
      mediator.SetLabelMaxLength(MENU_TEXT_MAX_LENGTH);
      if (null == strLabelPrefix)
      {
        strLabelPrefix = "";
      }
      while (mediator.Next())
      {
        HTMLOption option = new HTMLOption("");
        String[] arStrings = (String[]) mediator.GetEntry();
        // Label
        String pageName = arStrings[CListURLTemplateMediator.TEMPLATEARR_INDEX_LABEL];
        // URL Template
        if (arStrings.Length >= 2)
        {
          String pageURL = arStrings[CListURLTemplateMediator.TEMPLATEARR_INDEX_URL];
          option.SetValue(pageURL);
        }
        option.AddInnerHTMLString(strLabelPrefix + " " + pageName);
        select.AddInnerHTMLElement(option);
      }
    }
    
    /// <summary>
    /// Helper function that adds a row with dashes to the HTML Select menu
    /// </summary>
    /// <param name="select">HTML Select to add the option row to</param>
    /// <throws>HTMLException</throws>
    public static void AddSpacerSelectBoxOption(HTMLSelect select)
    {
      HTMLOption option = new HTMLOption("");
      option.SetValue("");
      option.AddInnerHTMLString(m_spacerOptionRow);
      select.AddInnerHTMLElement(option);
    }
    
    /// <summary>
    /// Helper function that adds a padding row to the HTML Select menu 
    /// </summary>
    /// <param name="select">HTML Select to add the option row to</param>
    /// <throws>HTMLException</throws>
    public static void AddSpacerSelectBoxOptionFiller(HTMLSelect select)
    {
      HTMLOption option = new HTMLOption("");
      option.SetValue("");
      option.AddInnerHTMLString(m_spacerOptionRowFiller);
      select.AddInnerHTMLElement(option);
    }
    
    /// <summary>
    /// Helper function that adds an empty row to the HTML Select menu 
    /// </summary>
    /// <param name="select">HTML Select to add the option to</param>
    /// <throws>HTMLException</throws>
    public static void AddSpacerSelectBoxOptionShort(HTMLSelect select)
    {
      HTMLOption option = new HTMLOption("");
      option.SetValue("");
      option.AddInnerHTMLString(m_spacerOptionRowShort);
      select.AddInnerHTMLElement(option);
    }
    
    /// <summary>
    /// Helper function that adds an row with a label to the HTML Select menu.
    /// The label is prefixed and suffixed with dashes. Will look like:
    /// '- Label -' 
    /// </summary>
    /// <param name="select">HTML Select to add the option to</param>
    /// <param name="strTitle">The option row display</param>
    /// <throws>HTMLException</throws>
    protected virtual void AddTitleSelectBoxOption(HTMLSelect select, String strTitle)
    {
      HTMLOption option = new HTMLOption("");
      option.SetValue("");
      option.AddInnerHTMLEncodedString("- " + strTitle + " -");
      select.AddInnerHTMLElement(option);
    }
    
    /// <summary>
    /// Helper function that creates an HTML Select and adds it to the
    /// provided HTMLTableCell
    /// </summary>
    /// <param name="tCell">The HTMLTable cell to add the HTML Select too</param>
    /// <param name="strSelectName">The HTML ID of the select</param>
    /// <returns>							The created HTML Select element</returns>
    /// <throws>HTMLException</throws>
    protected virtual HTMLSelect CreateSelectEleNavMenu(HTMLTableCell tCell,
     String strSelectName)
    {
      HTMLSelect select = new HTMLSelect(strSelectName, strSelectName);
      select.SetStyleClass(PTStyleClass.NAV_PULLDOWN_TEXT);
      // Assign the javascript eventhandler function
      select.SetOnChange("NavSelectMenuChange(options[selectedIndex].value, options)");
      tCell.AddInnerHTMLElement(select);
      return select;
    }
    
    /// <summary>
    /// Adds a HTML Select to the specified HTMLTableRow with Community actions
    /// and My Community for the current user
    /// </summary>
    /// <param name="tRow">The HTMLTableRow to add the HTML Select to</param>
    /// <throws>HTMLException </throws>
    protected virtual void WriteCommunitiesSection(HTMLTableRow tRow)
    {
      HTMLTableCell tCell = new HTMLTableCell();
      tRow.AddInnerHTMLElement(tCell);
      HTMLSelect select = CreateSelectEleNavMenu(tCell, "in_se_commnav");
      // In this community
      AddTitleSelectBoxOption(select, m_asOwner.GetString(252, ResourceConstants.PORTALCOMMONMSGS));
      AddSpacerSelectBoxOptionFiller(select);
      // Community actions
      ICPListEntryIterator cpList = m_model.GetCategoryLinks(NavCategoryType.COMMUNITY,
       true);
      // Prefix label actions with "*"
      AddSelectOptions(select, cpList, "*");
      if (cpList.GetTotalCount() != 0)
      {
        AddSpacerSelectBoxOptionShort(select);
      }
      // My Communities
      cpList = m_model.GetCategoryLinks(NavCategoryType.MYCOMMUNITIES, false);
      AddSelectOptions(select, cpList, "");
      // Spacer cell after the combobox
      AddSpacerSelectBoxOptionShort(select);
    }
    
    /// <summary>
    /// Adds a HTML Select to the specified HTMLTableRow with directory links
    /// </summary>
    /// <param name="tRow">The HTMLTableRow to add the HTML Select to</param>
    /// <throws>HTMLException </throws>
    protected virtual void WriteDocumentsSection(HTMLTableRow tRow)
    {
      HTMLTableCell tCell = new HTMLTableCell();
      tRow.AddInnerHTMLElement(tCell);
      HTMLSelect select = CreateSelectEleNavMenu(tCell, "in_se_docnav");
      ICPListEntryIterator cpList = m_model.GetCategoryLinks(NavCategoryType.DIRECTORY,
       false);
      // My documents
      AddTitleSelectBoxOption(select, m_asOwner.GetString(254, ResourceConstants.PORTALCOMMONMSGS));
      AddSpacerSelectBoxOptionFiller(select);
      AddSelectOptions(select, cpList, "");
      // Spacer cell after the combobox
      AddSpacerSelectBoxOptionShort(select);
    }
    
    /// <summary>
    /// Adds a HTML Select to the specified HTMLTableRow with the Mandatory Links for
    /// the current user
    /// </summary>
    /// <param name="tRow">The HTMLTableRow to add the HTML Select to</param>
    /// <throws>HTMLException </throws>
    protected virtual void WriteMandLinksSection(HTMLTableRow tRow)
    {
      ICPListEntryIterator cpList = m_model.GetCategoryLinks(NavCategoryType.MANDATORYLINK,
       false);
      String strMandLinksName = m_model.GetCategoryTitle(NavCategoryType.MANDATORYLINK);
      // bjf -- don't show the mandatory links if there aren't any
      if ((null != strMandLinksName) && !("").Equals(strMandLinksName) && (cpList.GetTotalCount() > 0))
      {
        HTMLTableCell tCell = new HTMLTableCell();
        tRow.AddInnerHTMLElement(tCell);
        HTMLSelect select = CreateSelectEleNavMenu(tCell, "in_se_mandlinks");
        int nTrunkWidth = 20;
        String strMaxLength = (String) m_asOwner.GetVarPack(PTNavSettingsVarPack.VARPACK_ID).GetVarPackValue(PTNavSettingsVarPack.INTHORIZONTALNAVTAB_TRUNCATION_WIDTH);
        if (strMaxLength != null || "".Equals(strMaxLength))
        {
          nTrunkWidth = XPConvert.ToInteger(nTrunkWidth);
        }
        String strTrunkTitle = GeneralHelpers.TruncateLongString(strMandLinksName,
         nTrunkWidth, false);
        AddTitleSelectBoxOption(select, strTrunkTitle);
        AddSpacerSelectBoxOptionFiller(select);
        AddSelectOptions(select, cpList, "");
        AddSpacerSelectBoxOptionShort(select);
      }
    }
    
    /// <summary>
    /// Adds a HTML Select to the specified HTMLTableRow with Mypage actions and
    /// My pages for the current user.
    /// </summary>
    /// <param name="tRow">The HTMLTableRow to add the HTML Select to</param>
    /// <throws>HTMLException </throws>
    protected virtual void WriteMyPagesSection(HTMLTableRow tRow)
    {
      HTMLTableCell tCell = new HTMLTableCell();
      tRow.AddInnerHTMLElement(tCell);
      HTMLSelect select = CreateSelectEleNavMenu(tCell, "in_se_mypagenav");
      // My Pages
      AddTitleSelectBoxOption(select, m_asOwner.GetString(260, ResourceConstants.PORTALCOMMONMSGS));
      AddSpacerSelectBoxOptionFiller(select);
      // Mypage actions
      ICPListEntryIterator cpList = m_model.GetCategoryLinks(NavCategoryType.MYPAGE,
       true);
      // Prefix label actions with "*"
      AddSelectOptions(select, cpList, "*");
      if (cpList.GetTotalCount() != 0)
      {
        AddSpacerSelectBoxOptionShort(select);
      }
      // Mypages
      cpList = m_model.GetCategoryLinks(NavCategoryType.MYPAGE, false);
      AddSelectOptions(select, cpList, "");
      AddSpacerSelectBoxOptionShort(select);
    }
  }
}