using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.editor;
using com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory;
using com.plumtree.portaluiinfrastructure.resultwrapper;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.errordisplay;
namespace com.plumtree.portalpages.admin.editors.remoteserver
{
  /// <author>adolfoc</author>
  public class RemoteServerModel : ObjEditorModel,IRemoteServerModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
     "com.plumtree.portalpages.admin.editors.remoteserver.RemoteServerModel");
    public const String STR_MVC_CLASS_NAME = "RemoteServer";
    
    /// <summary>
    /// The following statics are used by the Section as values returned by the
    /// Authentication Type radio buttons. The HTMLRADIO_BASE_AUTH_TYPE input
    /// must return one and only one of these. The value of these is irrelevant -
    /// they merely must be unique and URL/HTML safe.
    /// </summary>
    public const int AUTHTYPE_NONE = 0;
    public const int AUTHTYPE_USER = 1;
    public const int AUTHTYPE_ADMIN = 2;
    
    /// <summary>
    /// This string is used in lieu of returning the actual password to the client.
    /// If this string is unchanged - we will assume that the client has not changed
    /// the password and not reset it.
    /// </summary>
    public const String NULL_PASSWORD = "************";
    
    /// <summary>
    /// The following members are used to store data that will eventually be translated
    /// and stored in the Remote Server object.
    /// </summary>
    private String m_strBaseURL = "";
    private String m_strLoginName = "";
    private String m_strPublicKey = "";
    
    /// <summary>
    /// This member is used in determining whether the login name has changed (if the object
    /// was edited, say). We use this in order to determine when to blank the password when
    /// the object is edited.
    /// </summary>
    private String m_strOriginalLoginName = "";
    private String m_strPassword = "";
    private int m_intAuthTypeRadio;
    public RemoteServerModel() : base()
    {
      m_intAuthTypeRadio = AUTHTYPE_NONE;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new RemoteServerModel();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskBeforeStore()"></seealso>
    public override void DoTaskBeforeStore()
    {
      base.DoTaskBeforeStore();
      IPTGadgetContentServer ptRemoteServer = (IPTGadgetContentServer) base.m_ptObject;
      // Set the Base URL to what we collected.
      ptRemoteServer.SetURL(m_strBaseURL);
      ptRemoteServer.SetPublicKey(m_strPublicKey);
      // Set the Authentication Info
      switch (m_intAuthTypeRadio){
      case AUTHTYPE_USER:
      try
      {
        ptRemoteServer.SetPassword(null);
        // AGC- Order seems to be important. Because of the possibility of corrupted passwords
        // causing the setlogin method to fail it is preferrable to call SetLogin immediately
        // AFTER calling SetPassword.
        ptRemoteServer.SetLogin(null);
      }
      catch (Exception e)
      {
        log.Error(e, "RemoteServerModel::DoTaskBeforeStore() AUTHTYPE_USER: Could not set login or password to null - the current password may be corrupt!");
        }
      ptRemoteServer.SetSendUserBasicAuthentication(true);
      break;
      case AUTHTYPE_ADMIN:
      ptRemoteServer.SetLogin(m_strLoginName);
      // Make sure we don't overwrite any existing password if it (and the Login Name)
      // wasn't changed.
      // AGC- This logic may reset the password to a bogus NULL_PASSWORD if
      // the login name is changed (but not the password). This is probably
      // a good idea - to avoid throwing passwords around that don't match
      // the username specified. If both are changed, this should store correctly.
      if ((0 != NULL_PASSWORD.CompareTo(m_strPassword)) || (0 != m_strLoginName.CompareTo(m_strOriginalLoginName)))
      {
        ptRemoteServer.SetPassword(m_strPassword);
      }
      ptRemoteServer.SetSendUserBasicAuthentication(false);
      break;
      //AUTHTYPE_NONE
      default:
      try
      {
        ptRemoteServer.SetPassword(null);
        // AGC- Order seems to be important. Because of the possibility of corrupted passwords
        // causing the setlogin method to fail it is preferrable to call SetLogin immediately
        // AFTER calling SetPassword.
        ptRemoteServer.SetLogin(null);
      }
      catch (Exception e)
      {
        log.Error(e, "RemoteServerModel::DoTaskBeforeStore() AUTHTYPE_NONE: Could not set login or password to null - the current password may be corrupt!");
        }
      ptRemoteServer.SetSendUserBasicAuthentication(false);
      break;
      }
      // If this object is ever accessed again before this model goes away, we want to
      // make sure that the original login name is updated correctly.
      m_strOriginalLoginName = ptRemoteServer.GetLogin();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskOnStartEditor(_htFormData)"></seealso>
    public override void DoTaskOnStartEditor(XPHashtable _htFormData)
    {
      base.DoTaskOnStartEditor(_htFormData);
      if (EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType())
      {
        m_strBaseURL = "";
        m_strLoginName = "";
        m_strOriginalLoginName = "";
        m_strPassword = "";
        m_intAuthTypeRadio = AUTHTYPE_NONE;
        // AGC- In create mode, there will be no objects associated with the
        // newly created object (duh). So there's no reason to show the associated
        // objects page.
        ((EditorAS) m_asOwner).GetLeftMenu().HideLink(true, AssociatedObjectsDP.STR_MVC_CLASS_NAME);
        // Hide migration status in Create mode
        ((EditorAS) m_asOwner).GetLeftMenu().HideLink(true, MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);
        } else
      {
        IPTGadgetContentServer ptRemoteServer = (IPTGadgetContentServer) base.m_ptObject;
        m_strBaseURL = ptRemoteServer.GetURL();
        m_strLoginName = ptRemoteServer.GetLogin();
        m_strOriginalLoginName = m_strLoginName;
        m_strPassword = NULL_PASSWORD;
        m_strPublicKey = ptRemoteServer.GetPublicKey();
        if (ptRemoteServer.GetSendUserBasicAuthentication())
        {
          m_strLoginName = "";
          m_strOriginalLoginName = "";
          m_intAuthTypeRadio = AUTHTYPE_USER;
        } else
        if (0 == m_strLoginName.Length)
        {
          m_intAuthTypeRadio = AUTHTYPE_NONE;
        } else
        {
          m_intAuthTypeRadio = AUTHTYPE_ADMIN;
        }
      }
    }
    
    /// <returns></returns>
    public virtual IASQueryResult GetAssocWebServices()
    {
      if (EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType())
      {
        return null;
      } else
      {
        IPTSession ptSession = GetPTSession();
        IPTObjectManager ptWebServices = (IPTObjectManager) ptSession.GetWebServices();
        PTObjectManagerQueryWrapper omqr = new PTObjectManagerQueryWrapper(ptWebServices);
        // Query filter
        Object[][] ptqf = new Object[3][];
        for (int j2c_index = 0;j2c_index < 3;j2c_index++)
        {
          ptqf[j2c_index] = new Object[1];
        }
        ptqf[0][0] = PT_PROPIDS.PT_PROPID_WEBSERVICE_REMOTESERVERID;
        ptqf[1][0] = PT_FILTEROPS.PT_FILTEROP_EQ;
        ptqf[2][0] = m_ptObject.GetObjectID();
        omqr.Query(-1, -1, null, ptqf);
        return omqr;
      }
    }
    
    /// <summary>
    /// Returns the strAuthTypeRadio.
    /// </summary>
    /// <returns>String</returns>
    public virtual int GetAuthType()
    {
      return m_intAuthTypeRadio;
    }
    
    /// <summary>
    /// Returns the strBaseURL.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetBaseURL()
    {
      return m_strBaseURL;
    }
    
    /// <summary>
    /// Returns the strLoginName.
    /// Note that there is no getter for the Password (it is never
    /// sent back to the client).
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetLoginName()
    {
      return m_strLoginName;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.IRepostModel#SavePage(String,XPHashtable)"></seealso>
    /// <returns>int Page status for server side validation</returns>
    public override int SavePage(String _sPageName, XPHashtable _htFormData)
    {
      int nPageStatus = base.SavePage(_sPageName, _htFormData);
      if (_sPageName.Equals(MainDP.STR_MVC_CLASS_NAME))
      {
        SetBaseURL(RSModelHelper.GetFormElementString(_htFormData, RemoteServerPropsView.HTMLINPUT_BASE_URL,
         m_strBaseURL));
        SetLoginName(RSModelHelper.GetFormElementString(_htFormData, RemoteServerPropsView.HTMLINPUT_LOGIN_NAME,
         m_strLoginName));
        SetAuthType(RSModelHelper.GetFormElementInt(_htFormData, RemoteServerPropsView.HTMLRADIO_BASE_AUTH_TYPE,
         m_intAuthTypeRadio));
        m_strPublicKey = RSModelHelper.GetFormElementString(_htFormData, RemoteServerPropsView.HTMLTEXTAREA_PUBLICKEY,
         m_strPublicKey);
        // Validation
        String password = RSModelHelper.GetFormElementString(_htFormData,
         RemoteServerPropsView.HTMLINPUT_PASSWORD, m_strPassword);
        String passconf = RSModelHelper.GetFormElementString(_htFormData,
         RemoteServerPropsView.HTMLINPUT_PASSWORD_CONFIRM, m_strPassword);
        if (password == null || passconf == null || !password.Equals(passconf))
        {
          nPageStatus = RemoteServerRepostControl.PAGE_STATUS_INVALID;
          if (log.IsWarnEnabled())
          {
            log.Warn("WSModelHelper::ValidateNotEmptyString: Null or empty string found. ");
            }
          String strError = m_asOwner.GetString(366, "ptmsgs_portaladminmsgs");
          UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
           strError, strError);
          UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
          } else
        {
          SetPassword(password);
        }
        if (AUTHTYPE_ADMIN == m_intAuthTypeRadio)
        {
          nPageStatus = RSModelHelper.ValidateLoginPassword(m_strLoginName,
           m_strPassword, NULL_PASSWORD, m_asOwner, m_asOwner.GetString(732,
           "ptmsgs_portaladminmsgs"), nPageStatus);
        }
        nPageStatus = RSModelHelper.ValidateIsURLString(m_strBaseURL, m_asOwner,
         m_asOwner.GetString(733, "ptmsgs_portaladminmsgs"), nPageStatus);
        }
      return nPageStatus;
    }
    
    /// <summary>
    /// Sets the strAuthTypeRadio.
    /// </summary>
    /// <param name="strAuthTypeRadio">The strAuthTypeRadio to set</param>
    public virtual void SetAuthType(int intAuthTypeRadio)
    {
      // We must receive one of the Radio value strings above (AUTHTYPE_NONE, ...USER or ...ADMIN).
      // These will determine what gets stored in the Remote Server object.
      switch (intAuthTypeRadio){
      case AUTHTYPE_NONE:
      case AUTHTYPE_USER:
      case AUTHTYPE_ADMIN:
      this.m_intAuthTypeRadio = intAuthTypeRadio;
      break;
      default:
      log.Error("RemoteServerModel::SetAuthType(" + intAuthTypeRadio + ") an invalid AuthType was received. The Authtype will be set to NONE.");
      break;
      }
    }
    
    /// <summary>
    /// Sets the strBaseURL.
    /// </summary>
    /// <param name="strBaseURL">The strBaseURL to set</param>
    public virtual void SetBaseURL(String strBaseURL)
    {
      // Validate that the Base URL ends in a slash (silent correction).
      strBaseURL = XPStringUtility.Trim(strBaseURL);
      if (XPStringUtility.EndsWith(strBaseURL, "\\"))
      {
        strBaseURL = XPStringUtility.Substring(strBaseURL, 0, strBaseURL.Length - 1) + "/";
        } else
      if (!XPStringUtility.EndsWith(strBaseURL, "/"))
      {
        strBaseURL+="/";
      }
      m_strBaseURL = strBaseURL;
    }
    
    /// <summary>
    /// Sets the strLoginName.
    /// </summary>
    /// <param name="strLoginName">The strLoginName to set</param>
    public virtual void SetLoginName(String strLoginName)
    {
      m_strLoginName = strLoginName;
    }
    
    /// <summary>
    /// Sets the strPassword.
    /// Note that there is no getter for the Password.
    /// </summary>
    /// <param name="strPassword">The strPassword to set</param>
    public virtual void SetPassword(String strPassword)
    {
      if (XPStringUtility.Compare(NULL_PASSWORD, strPassword) != 0)
      {
        m_strPassword = strPassword;
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.portalpages.admin.editors.remoteserver.IRemoteServerModelRO#GetPublicKey()
     */
     
    public virtual String GetPublicKey()
    {
      return m_strPublicKey;
    }
  }
}