using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.admin.editors.webservice
{
  /// <author>
  /// adolfoc
  /// This class is used to automatically identify and display
  /// the appropriate views based on the list of views in the WebServiceModel
  /// This should help avoid mistakes with forgetting to display javascript and
  /// missing spacers, etc.
  /// </author>
  public abstract class WSPluginDP : EditorDP
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
     "com.plumtree.portalpages.admin.editors.webservice.WSPluginDP");
    
    /// <summary>
    /// Returns the name of the Help Topic to be used for this
    /// page.
    /// </summary>
    /// <returns></returns>
    public abstract String GetPluginHelpTopic();
    
    /// <summary>
    /// Returns a list of all of the possible views this page can show.
    /// (Used to generate javascript for each view).
    /// </summary>
    public abstract XPArrayList GetAllPossibleViews();
    
    /// <summary>
    /// Returns an ordered list of the views for this page, in this mode.
    /// </summary>
    public abstract XPArrayList GetOrderedViewList(int nWSMode);
    public virtual void InitPluginDP()
    {
      //AGC - we must set the validation functions from our submodels.
      WebServiceModel myModel = (WebServiceModel) m_asOwner.GetModel(WebServiceModel.STR_MVC_CLASS_NAME);
      XPArrayList xpalWSViews = GetOrderedViewList(myModel.GetWSMode());
      IXPEnumerator xpEnumWSSubModels = xpalWSViews.GetEnumerator();
      while (xpEnumWSSubModels.MoveNext())
      {
        String strSubModelKeyCurrent = (String) xpEnumWSSubModels.GetCurrent();
        try
        {
          IWSSubModel wsSubModelCurrent = (IWSSubModel) m_asOwner.GetModel(strSubModelKeyCurrent);
          String strValidationFunc = wsSubModelCurrent.GetValidationFunction();
          if ((null != strValidationFunc) && (XPStringUtility.Trim(strValidationFunc).Length > 0))
          {
            AddToValidationFunction(strValidationFunc);
          }
        }
        catch (Exception e)
        {
          log.Error(e, GetName() + "::PageDisplay() The following error occured when trying to display the Javascript for Section (" + strSubModelKeyCurrent + ").");
          throw new XPException(m_asOwner.GetString(1197, "ptmsgs_portaladminmsgs"),
           e);
        }
      }
    }
    
    /// <summary>
    /// Determines whether a specific view is enabled in the suggested mode.
    /// </summary>
    public virtual bool IsEnabled(int nWSMode, String strViewName)
    {
      XPArrayList xpalWSViews = GetOrderedViewList(nWSMode);
      IXPEnumerator xpEnumWSSubModels = xpalWSViews.GetEnumerator();
      while (xpEnumWSSubModels.MoveNext())
      {
        String strSubModelKeyCurrent = (String) xpEnumWSSubModels.GetCurrent();
        if (0 == strSubModelKeyCurrent.CompareTo(strViewName))
        {
          return true;
        }
      }
      return false;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.AFormDP#PageDisplay()"></seealso>
    /// <returns>HTMLElement</returns>
    public override HTMLElement PageDisplay()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      // Insert super class HTML
      result.AddInnerHTMLElement(base.PageDisplay());
      HTMLElementCollection views = new HTMLElementCollection();
      WebServiceModel myModel = (WebServiceModel) m_asOwner.GetModel(WebServiceModel.STR_MVC_CLASS_NAME);
      XPArrayList xpalWSViews = GetOrderedViewList(myModel.GetWSMode());
      IXPEnumerator xpEnumWSSubModels = xpalWSViews.GetEnumerator();
      while (xpEnumWSSubModels.MoveNext())
      {
        String strSubModelKeyCurrent = (String) xpEnumWSSubModels.GetCurrent();
        // Don't display the Gateway prefix section in pagelet editor even though we added the view to the list of views to display. 
        // We need the view in the list to be able to save a hard-coded prefix for all pagelets.
        if (strSubModelKeyCurrent.Equals(WebServiceEditorAS.strStorage_GatewayPrefixPluginView) && myModel.GetWSMode() == WebServiceModel.WSMODE_PAGELET)
        {
          // skip this view
          continue;
        }
        try
        {
          EditorView wsSubViewCurrent = (EditorView) m_asOwner.GetView(strSubModelKeyCurrent);
          views.AddInnerHTMLElement(wsSubViewCurrent.Display());
          views.AddInnerHTMLElement(EditorView.GetSpacerTable());
        }
        catch (Exception e)
        {
          log.Error(e, GetName() + "::PageDisplay() The following error occurred when trying to display the HTML for Section (" + strSubModelKeyCurrent + ").");
          throw new XPException(m_asOwner.GetString(1197, "ptmsgs_portaladminmsgs"),
           e);
        }
      }
      result.AddInnerHTMLElement(CreateLeftNavBar(views));
      return result;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.AFormDP#DisplayJavaScriptFromChild()"></seealso>
    protected override HTMLScriptCollection DisplayJavaScriptFromChild()
    {
      HTMLScriptCollection result = base.DisplayJavaScriptFromChild();
      if (null == result)
      {
        result = new HTMLScriptCollection();
      }
      // AGC- Note, we actually show all of our javascript all of the time. This is because
      // the Page-level validation function is determined at editor start-up. If we
      // don't supply all of the possible validation functions, we will cause javascript errors
      // if the WSMODE is ever changed. In order to avoid validating components that don't show,
      // we rely on the views returning blank/alwaystrue validation when not in the appropriate
      // mode.
      XPArrayList xpalWSViews = GetAllPossibleViews();
      IXPEnumerator xpEnumWSSubModels = xpalWSViews.GetEnumerator();
      while (xpEnumWSSubModels.MoveNext())
      {
        String strSubModelKeyCurrent = (String) xpEnumWSSubModels.GetCurrent();
        try
        {
          EditorView wsSubViewCurrent = (EditorView) m_asOwner.GetView(strSubModelKeyCurrent);
          HTMLScript scriptCurrent = wsSubViewCurrent.DisplayJavascript();
          if (null != scriptCurrent)
          {
            result.AddInnerHTMLElement(scriptCurrent);
          }
        }
        catch (Exception e)
        {
          log.Error(e, GetName() + "::PageDisplay() The following error occured when trying to display the Javascript for Section (" + strSubModelKeyCurrent + ").");
          throw new XPException(m_asOwner.GetString(1197, "ptmsgs_portaladminmsgs"),
           e);
        }
      }
      return result;
    }
  }
}