using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.portalpages.admin.editors.webservice;
using com.plumtree.portaluiinfrastructure.classtypedescriptors.providerframework;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.form;
namespace com.plumtree.portalpages.admin.editors.webservice.plugins
{
  /// <summary>
  /// Store the activity space for the intrinsic portlet preference pages.
  /// Even though there is only one prefs plugin model and one prefs plugin
  /// view, there are actually three instances of this model in existence.
  /// Each of these models will contain information about one preference type,
  /// whether it be admin portlet prefs, user portlet prefs, or community
  /// portlet prefs.
  /// </summary>
  /// <author>josephc</author>
  public class IntrinsicUserPrefsPluginModel : EditorHelperModel,IIntrinsicUserPrefsPluginModelRO,
  IWSSubModel
  {
    
    /// <summary>
    /// Class name for IntrinsicUserPrefsPluginModel 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "IntrinsicUserPrefsPlugin";
    
    /// <summary>
    /// User portlet preferences 
    /// </summary>
    public const int UPPM_PREFMODE_USERPREFS = 0;
    
    /// <summary>
    /// Community portlet preferences 
    /// </summary>
    public const int UPPM_PREFMODE_COMMUNITYPREFS = 1;
    
    /// <summary>
    /// Admin portlet preferences 
    /// </summary>
    public const int UPPM_PREFMODE_ADMINPREFS = 2;
    
    /// <summary>
    /// The Web Service object being edited by our parent Web Service editor 
    /// </summary>
    internal IPTWebService m_ptWebService = null;
    
    /// <summary>
    /// Prefs activity space 
    /// </summary>
    private String m_strPrefsSpace = "";
    
    /// <summary>
    /// Admin prefs template activity space 
    /// </summary>
    private String m_strTemplatePrefsSpace = "";
    
    /// <summary>
    /// Preference mode 
    /// </summary>
    private int m_nPrefMode = 0;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new IntrinsicUserPrefsPluginModel();
    }
    
    /// <param name="nSupportsPersonalization"></param>
    /// <param name="strPrefsSpace"></param>
    /// <param name="nPortletPersonalizationRole"></param>
    /// <returns></returns>
    private int UpdatePersonalizationFlag(int nSupportsPersonalization, String strPrefsSpace,
     int nPortletPersonalizationRole)
    {
      if ((null == strPrefsSpace) || (0 >= XPStringUtility.Trim(strPrefsSpace).Length))
      {
        nSupportsPersonalization&=~nPortletPersonalizationRole;
      } else
      {
        nSupportsPersonalization|=nPortletPersonalizationRole;
      }
      return nSupportsPersonalization;
    }
    
    /// <summary>
    /// This method updates the personalization flag of the web service. This model
    /// only represents one type of preference, either user prefs, community prefs,
    /// or admin prefs.  So we just need to check if the text box has a value, then
    /// OR the value so that the flag is set.  If the text box does not have a valid
    /// value, then AND the value so the flag is unset.
    /// </summary>
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskBeforeWSStore()"></seealso>
    public virtual void DoTaskBeforeWSStore()
    {
      IXPPropertyBag ptProviderInfo = (IXPPropertyBag) m_ptWebService.GetProviderInfo();
      String strPrefsSpace = GetPrefsSpace();
      int nSupportsPersonalization = m_ptWebService.GetSupportsPersonalization();
      // Determine which type of preference our model is holding info about
      switch (m_nPrefMode){
      case UPPM_PREFMODE_USERPREFS:
      nSupportsPersonalization = UpdatePersonalizationFlag(nSupportsPersonalization,
       strPrefsSpace, PT_GADGETPERSONALIZATION_ROLES.PT_GADGETPERSONALIZATION_ENDUSER);
      WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_PREFS_SPACE,
       strPrefsSpace);
      break;
      case UPPM_PREFMODE_COMMUNITYPREFS:
      nSupportsPersonalization = UpdatePersonalizationFlag(nSupportsPersonalization,
       strPrefsSpace, PT_GADGETPERSONALIZATION_ROLES.PT_GADGETPERSONALIZATION_COMMUNITYOWNER);
      WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_COMMUNITYPREFS_SPACE,
       strPrefsSpace);
      break;
      case UPPM_PREFMODE_ADMINPREFS:
      nSupportsPersonalization = UpdatePersonalizationFlag(nSupportsPersonalization,
       strPrefsSpace, PT_GADGETPERSONALIZATION_ROLES.PT_GADGETPERSONALIZATION_ADMINISTRATOR);
      WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_ADMINPREFS_SPACE,
       strPrefsSpace);
      nSupportsPersonalization = UpdatePersonalizationFlag(nSupportsPersonalization,
       GetTemplatePrefsSpace(), PT_GADGETPERSONALIZATION_ROLES.PT_GADGETPERSONALIZATION_ADMINTEMPLATE);
      WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_ADMINPREFS_TEMPLT_SPACE,
       GetTemplatePrefsSpace());
      break;
      }
      m_ptWebService.SetSupportsPersonalization(nSupportsPersonalization);
      }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskOnRemoteServerUpdate(String,String,String)"></seealso>
    public virtual void DoTaskOnRemoteServerUpdate(String strRemoteServerID,
     String strRemoteServerName, String strRemoteServerBaseURL)
    {
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskOnWSModeUpdate(int)"></seealso>
    public virtual void DoTaskOnWSModeUpdate(int nNewWSMode)
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// This method checks whether this model is used to set community
    /// preferences or user preferences. SetPrefMode can be used to set this
    /// when the model is initialized. GetPrefMode can be used to get which
    /// mode this model is in.
    /// </summary>
    /// <returns>Preference mode</returns>
    public virtual int GetPrefMode()
    {
      return m_nPrefMode;
    }
    
    /// <summary>
    /// Gets the preference space value for the current type of preference.
    /// </summary>
    /// <returns>Preference space</returns>
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.plugins.IIntrinsicUserPrefsPluginModelRO#GetPrefsSpace()"></seealso>
    public virtual String GetPrefsSpace()
    {
      return m_strPrefsSpace;
    }
    
    /// <summary>
    /// Gets the preference space value for portlet template admin preferences.
    /// </summary>
    /// <returns>Portlet template admin preference space</returns>
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.plugins.IIntrinsicUserPrefsPluginModelRO#GetTemplatePrefsSpace()"></seealso>
    public virtual String GetTemplatePrefsSpace()
    {
      return m_strTemplatePrefsSpace;
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#GetValidationFunction()"></seealso>
    public virtual String GetValidationFunction()
    {
      return null;
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#InitWSModel()"></seealso>
    public virtual void InitWSModel(IPTWebService ptWebService)
    {
      m_ptWebService = ptWebService;
      IXPPropertyBag ptProviderInfo = (IXPPropertyBag) ptWebService.GetProviderInfo();
      switch (m_nPrefMode){
      case UPPM_PREFMODE_USERPREFS:
      SetPrefsSpace(WSModelHelper.PBReadString(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_PREFS_SPACE,
       m_strPrefsSpace));
      break;
      case UPPM_PREFMODE_COMMUNITYPREFS:
      SetPrefsSpace(WSModelHelper.PBReadString(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_COMMUNITYPREFS_SPACE,
       m_strPrefsSpace));
      break;
      case UPPM_PREFMODE_ADMINPREFS:
      SetPrefsSpace(WSModelHelper.PBReadString(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_ADMINPREFS_SPACE,
       m_strPrefsSpace));
      SetTemplatePrefsSpace(WSModelHelper.PBReadString(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_ADMINPREFS_TEMPLT_SPACE,
       m_strTemplatePrefsSpace));
      break;
      }
    }
    
    /// <summary>
    /// Verify that the specified activity space exists and has been loaded by
    /// dynamic discovery.
    /// </summary>
    /// <param name="strSpaceName">Activity space name</param>
    /// <param name="strErrorMsg">Error message to display for invalid space</param>
    /// <returns>Repost control flag</returns>
    private int ValidatePrefsSpace(String strSpaceName, String strErrorMsg)
    {
      if ((null == strSpaceName) || (0 == strSpaceName.Length))
      {
        // The space can be empty or null if the portlet doesn't support that type of pref.
        return RepostControl.PAGE_STATUS_VALID;
      } else
      {
        // If the space cannot be retrieved using the portlet class identifier, then
        // it must be invalid.
        if (!ASManager.INSTANCE.GetSpaceExists(strSpaceName))
        {
          UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
           strErrorMsg, strErrorMsg);
          UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
          return RepostControl.PAGE_STATUS_INVALID;
        }
        return RepostControl.PAGE_STATUS_VALID;
      }
    }
    
    /// <summary>
    /// Saves the preference spaces.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.editor.EditorHelperModel#SaveSection(XPHashtable)"></seealso>
    public override int SaveSection(XPHashtable _htFormData)
    {
      int nStatus = RepostControl.PAGE_STATUS_VALID;
      String strErrorMsg;
      switch (m_nPrefMode){
      case UPPM_PREFMODE_USERPREFS:
      SetPrefsSpace(WSModelHelper.GetFormElementString(_htFormData, IntrinsicUserPrefsPluginView.HTMLINPUT_USER_PREFS_SPACE,
       GetPrefsSpace()));
      strErrorMsg = XPResourceManager.FormatMessage(m_asOwner.GetString(2386,
       "ptmsgs_portaladminmsgs"), GetPrefsSpace());
      nStatus = ValidatePrefsSpace(GetPrefsSpace(), strErrorMsg);
      break;
      case UPPM_PREFMODE_COMMUNITYPREFS:
      SetPrefsSpace(WSModelHelper.GetFormElementString(_htFormData, IntrinsicUserPrefsPluginView.HTMLINPUT_COMMUNITY_PREFS_SPACE,
       GetPrefsSpace()));
      strErrorMsg = XPResourceManager.FormatMessage(m_asOwner.GetString(2387,
       "ptmsgs_portaladminmsgs"), GetPrefsSpace());
      nStatus = ValidatePrefsSpace(GetPrefsSpace(), strErrorMsg);
      break;
      case UPPM_PREFMODE_ADMINPREFS:
      SetPrefsSpace(WSModelHelper.GetFormElementString(_htFormData, IntrinsicUserPrefsPluginView.HTMLINPUT_ADMIN_PREFS_SPACE,
       GetPrefsSpace()));
      SetTemplatePrefsSpace(WSModelHelper.GetFormElementString(_htFormData,
       IntrinsicUserPrefsPluginView.HTMLINPUT_TEMPLT_ADMIN_PREFS_SPACE, GetTemplatePrefsSpace()));
      strErrorMsg = XPResourceManager.FormatMessage(m_asOwner.GetString(2384,
       "ptmsgs_portaladminmsgs"), GetPrefsSpace());
      nStatus = ValidatePrefsSpace(GetPrefsSpace(), strErrorMsg);
      // Validate template space only if prefs space is valid.			
      if (nStatus == RepostControl.PAGE_STATUS_VALID)
      {
        strErrorMsg = XPResourceManager.FormatMessage(m_asOwner.GetString(2385,
         "ptmsgs_portaladminmsgs"), GetTemplatePrefsSpace());
        nStatus = ValidatePrefsSpace(GetTemplatePrefsSpace(), strErrorMsg);
        }
      break;
      }
      return nStatus;
    }
    
    /// <summary>
    /// This method specifies whether this model is used to set community
    /// preferences or user preferences. SetPrefMode can be used to set this
    /// when the model is initialized. GetPrefMode can be used to get which
    /// mode this model is in.
    /// </summary>
    /// <param name="nPrefMode">Preference mode</param>
    public virtual void SetPrefMode(int nPrefMode)
    {
      m_nPrefMode = nPrefMode;
    }
    
    /// <summary>
    /// Sets the prefs space
    /// </summary>
    /// <param name="strPrefsSpace">Activity space for preferences</param>
    public virtual void SetPrefsSpace(String strPrefsSpace)
    {
      m_strPrefsSpace = strPrefsSpace;
    }
    
    /// <summary>
    /// Sets the Template Admin Prefs space
    /// </summary>
    /// <param name="strTemplatePrefsSpace">
    /// Activity space for admin template
    /// preferences
    /// </param>
    public virtual void SetTemplatePrefsSpace(String strTemplatePrefsSpace)
    {
      m_strTemplatePrefsSpace = strTemplatePrefsSpace;
    }
  }
}