using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.admin.editors.webservice;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.server.treeiterator;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.tree;
namespace com.plumtree.portalpages.admin.editors.webservice.userinfotree
{
  /// <summary>
  /// This version of the tree allows for browsing and
  /// selecting of objects with no hierarchical structure
  /// </summary>
  /// <author>AdolfoC</author>
  public class UserInfoTreeModel : ATreeModel,ITreeModel,ITreeModelRO,IRepostModel
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
     "com.plumtree.portalpages.admin.editors.webservice.userinfotree.UserInfoTreeModel");
    
    /// <summary>
    /// object name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "UserInfoTree";
    
    /// <summary>
    /// Tree iterator 
    /// </summary>
    protected IPTTreeIterator m_ptIterator = null;
    
    /// <summary>
    /// Storage for the list of all known user infos. This is used in order to
    /// get the UserInfo string based on the index into this array (the index is
    /// used in place of an object ID to uniquely identify each User Info).
    /// </summary>
    protected XPArrayList m_xpalKnownUserInfos;
    protected XPArrayList m_xpalSelected;
    public UserInfoTreeModel() : base()
    {
      m_xpalKnownUserInfos = new XPArrayList();
      m_xpalSelected = new XPArrayList();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new UserInfoTreeModel();
    }
    
    /// <summary>
    /// This is called when a node is expanded.  Since User Info
    /// is a flat list, there will be no recursive thoughts here.  This simply
    /// retrieves all the User Info available to the user and puts it on
    /// the tree as leaf items.
    /// </summary>
    /// <param name="bExpand">expands the branch</param>
    /// <param name="nNodeID">the uniqueid for this node</param>
    public override void ExpandNode(bool bExpand, int nNodeID)
    {
      base.ExpandNode(bExpand, nNodeID);
      // Always expand the dummy title to show all rights
      TreeCompositeNode node = (TreeCompositeNode) m_Root.GetNode(nNodeID);
      node.m_bExpanded = true;
      // Populate with User Info
      try
      {
        IPTProfileManager ptProfileManager = (IPTProfileManager) ((IPTSession) m_asOwner.GetUserSession()).OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_PROFILE_MANAGER,
         false);
        Object[][] arrMap;
        arrMap = (Object[][]) ptProfileManager.GetUserInfoMap();
        TreeLeafNode myLeaf;
        int nNumUserInfos = 0;
        if (null != arrMap)
        {
          nNumUserInfos = arrMap[0].Length;
        }
        // A stored array of User infos. We need this in order to be able to 
        // uniquely identify each UserInfo with an integer (rather than re-write the whole
        // damn Tree to support indexing by string).
        m_xpalKnownUserInfos = new XPArrayList();
        for (int nMapIndex = 0;nMapIndex < nNumUserInfos;nMapIndex++)
        {
          String strCurrentUserInfo = (String) arrMap[1][nMapIndex];
          if (!WSModelHelper.IsHardcodedUserInfo(strCurrentUserInfo))
          {
            // AGC- we need to keep track of the mapped items so we can do
            // selection properly.	        	
            int nCurrentUserInfoIndex = m_xpalKnownUserInfos.Add(strCurrentUserInfo);
            myLeaf = new TreeLeafNode(GetNextUniqueID());
            myLeaf.SetName(strCurrentUserInfo);
            //AGC- the most unique ID we can find is the index into the stored array.
            myLeaf.m_nObjectID = nCurrentUserInfoIndex;
            //AGC- Not Used.
            myLeaf.m_nClassID = -1;
            // TODOCR: donh myLeaf.GetName() returns the name truncated to 50,
            // and myLeaf.GetEncodedName() returns it truncated and HTMLEncoded
            // The data is stored in the encoded format, so GetEncodedName
            // is faster than GetName().  The truncation may cause problems
            // here.
            if (m_xpalSelected.Contains(myLeaf.GetName()))
            {
              myLeaf.m_bSelected = true;
              m_xpmSelectedItems.PutElement(myLeaf.m_nClassID + "," + myLeaf.m_nObjectID,
               myLeaf);
            }
            node.AddLeafChild(myLeaf);
          }
        }
      }
      catch (Exception ex)
      {
        log.Error("UserInfoTreeModel::Expand Node Unable to load User Info tree.  The following exception occured: " + ex);
        throw new XPException(m_asOwner.GetString(1938, "ptmsgs_portaladminmsgs"),
         ex);
      }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Returns the list of UserInfo strings that the Tree knows about (is
    /// displaying).
    /// </summary>
    /// <returns>XPArrayList</returns>
    public virtual XPArrayList GetTreeUserInfos()
    {
      return m_xpalKnownUserInfos;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#Init(AActivitySpace)"></seealso>
    public override void Init(AActivitySpace parent)
    {
      base.Init(parent);
    }
    
    /// <summary>
    /// Initializes and begins the loading of the tree.
    /// </summary>
    public virtual void LoadUserInfo()
    {
      // Root is simply a dummy title
      m_Root = new TreeCompositeNode(GetNextUniqueID());
      m_Root.SetName(m_asOwner.GetString(1939, "ptmsgs_portaladminmsgs"));
      if (!m_bShowRoot)
      {
        ExpandNode(true, m_Root.m_nTreeNodeID);
      }
    }
    
    /// <summary>
    /// If there are multiple class ids, determine which
    /// ones to display.  For User Info, we only
    /// have 1 class id, so this method is unimplemented.
    /// </summary>
    /// <param name="nIDs">determines which classids to display</param>
    public override void SetClassIDsToDisplay(int[] nIDs)
    {
    }
    // User Info do not need this currently.
    
    /// <summary>
    /// Determines what items in the tree are to be displayed.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModel#SetObjectIDsToDisplay(int[])"></seealso>
    /// <param name="nIDs">determines which nodes are not hidden.</param>
    public override void SetObjectIDsToDisplay(int[] nIDs)
    {
    }
    
    /// <summary>
    /// Determines what items in the tree are hidden.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModel#SetObjectIDsToHide(int[])"></seealso>
    /// <param name="nIDs">determines which nodes are hidden</param>
    public override void SetObjectIDsToHide(int[] nIDs)
    {
    }
    // User Info do not need this currently.
    
    /// <summary>
    /// Determines what items in the tree are selected.
    /// </summary>
    /// <param name="nIDs">determines which nodes are selected</param>
    public override void SetObjectIDsToSelect(int[] nIDs)
    {
    }
    // AGC- Not used by User Info
    
    /// <summary>
    /// Sets the userinfos that are currently selected.
    /// </summary>
    /// <param name="arsUserInfos"></param>
    public virtual void SetUserInfosToSelect(XPArrayList xpalUserInfos)
    {
      ClearSelectedItems();
      m_xpalSelected = xpalUserInfos;
    }
    
    /// <summary>
    /// Sets the userinfos that are currently selected.
    /// </summary>
    /// <param name="arsUserInfos"></param>
    public virtual void SetUserInfosToSelect(String[] arsUserInfos)
    {
      SetUserInfosToSelect(WSModelHelper.StringArrayToXPArray(arsUserInfos));
      }
  }
}