using System;
using com.plumtree.portaluiinfrastructure.resultwrapper;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.arraywrapper;
namespace com.plumtree.portalpages.admin.objmgr
{
  /// <author>DonH</author>
  public class ObjectManagerHolder
  {
    public readonly IPTObjectManager m_ptObjMgr;
    public int[] m_nSelectedObjectIDs;
    // pagination variables (-1 implies not yet set)
    private int m_nFirstObjectCount = -1;
    private int m_nLastObjectCount = -1;
    private int m_nSkippedRows = 0;
    private int[][] m_nSortOrder;
    // (-1 implies not yet set)
    
    /// <summary>
    /// Current pagination size 
    /// </summary>
    private int m_nPaginationSize = -1;
    private int m_nDefaultPaginationSize = -1;
    
    /// <summary>
    /// Creates a new ObjectManagerHolder object.
    /// </summary>
    /// <param name="ptObjMgr"></param>
    public ObjectManagerHolder(IPTObjectManager ptObjMgr)
    {
      m_ptObjMgr = ptObjMgr;
      m_nSelectedObjectIDs = null;
      m_nSortOrder = new int[1][];
      for (int j2c_index = 0;j2c_index < 1;j2c_index++)
      {
        m_nSortOrder[j2c_index] = new int[2];
      }
      m_nSortOrder[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID] = PT_PROPIDS.PT_PROPID_NAME;
      m_nSortOrder[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
      }
    
    /// <summary>
    /// This resets the selected items and pagination for this object manager.
    /// This is usually due to changing folders.  Sort order does not get reset.
    /// </summary>
    public virtual void Reset()
    {
      m_nFirstObjectCount = -1;
      m_nLastObjectCount = -1;
      m_nSkippedRows = 0;
      m_nSelectedObjectIDs = null;
      m_nPaginationSize = -1;
    }
    
    /// <summary>
    /// This gets the number of rows that are being
    /// skipped in the query.
    /// </summary>
    /// <returns>int the row to start on (default 0)</returns>
    public virtual int GetSkippedRows()
    {
      return m_nSkippedRows;
    }
    
    /// <summary>
    /// This gets the first object count
    /// </summary>
    /// <returns>int the first object count</returns>
    public virtual int GetFirstObjectCount()
    {
      return m_nFirstObjectCount;
    }
    
    /// <summary>
    /// This gets the last object count
    /// </summary>
    /// <returns>int the last object count</returns>
    public virtual int GetLastObjectCount()
    {
      return m_nLastObjectCount;
    }
    
    /// <summary>
    /// This gets the sort property ID.
    /// </summary>
    /// <returns>int the property id to sort on</returns>
    public virtual int GetSortDirection()
    {
      return m_nSortOrder[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION];
      }
    
    /// <summary>
    /// This gets the sort property ID.
    /// </summary>
    /// <returns>int the property id to sort on</returns>
    public virtual int GetSortPropID()
    {
      return m_nSortOrder[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID];
      }
    
    /// <summary>
    /// This returns whether or not the object is selected
    /// </summary>
    /// <param name="int">nID the object ID of the object to check</param>
    /// <returns>boolean true implies selected, false implies not selected</returns>
    public virtual bool IsObjectSelected(int nID)
    {
      if (m_nSelectedObjectIDs != null)
      {
        for (int i = 0;i < m_nSelectedObjectIDs.Length;i++)
        {
          if (nID == m_nSelectedObjectIDs[i])
          {
            return true;
          }
        }
      }
      return false;
    }
    
    /// <summary>
    /// This method returns a query of the objects with the correct sorting.
    /// </summary>
    /// <returns>IASQueryResult</returns>
    public virtual IASQueryResult QueryObjects(int nFolderID)
    {
      // Bug 27566: Admin hierarchy displaying a large number of users.
      // Bug 21969 -- We should calculate how many items we actually
      // want to view, and call result.SetMaxRows() with that.  This requires
      // some changes in the ObjMgrModel as well.  This is causing us to
      // double or triple our temporary memory usage when viewing objects
      // where there are close to 1000 objects of that type.
      int nPropIDMask = PT_PROPIDS.PT_PROPID_ALL & ~PT_PROPIDS.PT_PROPID_LOCKED;
      PTObjectManagerQueryWrapper result = new PTObjectManagerQueryWrapper(this.m_ptObjMgr);
      int nMaxRows = m_nDefaultPaginationSize;
      if (-1 != m_nPaginationSize)
      {
        nMaxRows = m_nPaginationSize;
      }
      result.SetMaxRows(nMaxRows);
      result.Query(nPropIDMask, nFolderID, m_nSortOrder, m_nSkippedRows, null);
      return result;
    }
    
    /// <summary>
    /// This sets the pagination.
    /// This assumes that the pagination bounds are not greater than the
    /// number of items we can query for.
    /// </summary>
    /// <param name="int">the first object count</param>
    /// <param name="int">the second object count</param>
    public virtual void SetPagination(int nFirstObjectCount, int nLastObjectCount)
    {
      m_nFirstObjectCount = nFirstObjectCount;
      m_nLastObjectCount = nLastObjectCount;
      m_nSkippedRows = m_nFirstObjectCount - 1;
      m_nPaginationSize = nLastObjectCount - m_nFirstObjectCount + 1;
    }
    public virtual void SetDefaultPaginationSize(int nDefaultPaginationSize)
    {
      m_nDefaultPaginationSize = nDefaultPaginationSize;
    }
    
    /// <summary>
    /// This sets the sort order.
    /// </summary>
    /// <param name="int">the property id to sort on</param>
    /// <param name="int">the sort direction (PTSortedArrayWrapper.PTSAW_SORT_*)</param>
    public virtual void SetSortOrder(int nPropID, int nSortDirection)
    {
      m_nSortOrder[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID] = nPropID;
      m_nSortOrder[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = nSortDirection;
      }
  }
}