using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.admin.ptutilities.migrationutility;
using com.plumtree.portaluiinfrastructure.resultwrapper;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.server.impl.core;
using com.plumtree.server.impl.migration;
using com.plumtree.uiinfrastructure.arraywrapper;
namespace com.plumtree.portalpages.admin.ptutilities.migrationutility.exp
{
  /// <summary>
  /// This class is what calls the actual server migration code.
  /// It runs in a separate thread to prevent UI timeouts.
  /// </summary>
  /// <author>jennifersu</author>
  public class PTExportMigrationTask : PTMigrationTask
  {
    private const String OUR_NAME = "PTExportMigrationTask";
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
     "com.plumtree.portalpages.admin.ptutilities.migrationutility.exp.PTExportMigrationTask");
    public const String PACKAGE_NAME = "Package_Name";
    public const String PACKAGE_DESCRIPTION = "Package_Description";
    public const String PUBLISHER_NAME = "Publisher_Name";
    
    /// <summary>
    /// ArrayWrapper of items to export
    /// </summary>
    private PTGrowableSortedArrayWrapper m_saData;
    
    /// <summary>
    /// Package name
    /// </summary>
    private String m_strName;
    
    /// <summary>
    /// Package description
    /// </summary>
    private String m_strDescription;
    
    /// <summary>
    /// Package publisher
    /// </summary>
    private String m_strPublisher;
    
    /// <summary>
    /// Array of UUIDs that have been migrated.
    /// </summary>
    private XPArrayList m_xpalStrMigrated;
    
    /// <summary>
    /// Array of class IDs that have been exported.
    /// Collab/Content need community ids and portlet ids.
    /// </summary>
    private XPArrayList exportedClassIDsAsIntObj;
    
    /// <summary>
    /// Array of object IDs that have been exported.
    /// Collab/Content need community ids and portlet ids.
    /// </summary>
    private XPArrayList exportedObjectIDsAsIntObj;
    
    /// <summary>
    /// Hashtable of migrated objects
    /// Key=UUID; Value=IPTMigrationElement
    /// </summary>
    protected XPHashtable m_xphtUUIDs;
    
    /// <summary>
    /// IPTSession
    /// </summary>
    protected IPTSession m_ptSession;
    
    /// <summary>
    /// Hashtable of MWS property bags to send.
    /// Key=MWS ID; Value=IXPPropertyBag 
    /// </summary>
    protected XPHashtable m_htMWSPropBags;
    
    /// <summary>
    /// When true, export parent folder dependencies as
    /// objects and pull in their dependencies as well
    /// </summary>
    private bool exportParentFolders = false;
    
    /// <summary>
    /// Store return value from MWS ExportPackage + info about which MWS it is
    /// </summary>
    private XPArrayList mwsMessages = null;
    
    /// <param name="exportParentFoldersWhenExportDeps">If true, export parent folders when exporting dependencies</param>
    public PTExportMigrationTask(IPTMigrationManager _MigrationManager, IPTSession _ptSession,
     PTGrowableSortedArrayWrapper _saData, String _strName, String _strDescription,
     String _strPublisher, bool exportParentFoldersWhenExportDeps) : base(_MigrationManager)
    {
      m_ptSession = _ptSession;
      m_saData = _saData;
      m_strName = _strName;
      m_strDescription = _strDescription;
      m_strPublisher = _strPublisher;
      m_xpalStrMigrated = new XPArrayList();
      m_xphtUUIDs = new XPHashtable();
      this.exportedClassIDsAsIntObj = new XPArrayList();
      this.exportedObjectIDsAsIntObj = new XPArrayList();
      this.exportParentFolders = exportParentFoldersWhenExportDeps;
      this.mwsMessages = new XPArrayList();
    }
    
    /// <summary>
    /// Performs the actual migration.
    /// </summary>
    public override void run()
    {
      m_intStatus = INPROGRESS;
      log.Info("Migration Task in progress.");
      try
      {
        for (int indObj = 0;indObj < m_saData.GetCount();indObj++)
        {
          // First migrate the object
          int nClassID = XPConvert.ToInteger(m_saData.GetItem(indObj, MigrationSelectForExportModel.EXPLIST_COL_CLASSID));
          int nObjectID = XPConvert.ToInteger(m_saData.GetItem(indObj, MigrationSelectForExportModel.EXPLIST_COL_OBJECTID));
          bool bDependencies = XPConvert.ToBoolean(m_saData.GetItem(indObj,
           MigrationSelectForExportModel.EXPLIST_COL_DEPENDENCIES));
          ExportObject(nClassID, nObjectID, bDependencies);
          if (m_intStatus == CANCEL)
          {
            log.Info("Migration Task cancelled.");
            return;
          }
          // Then migrate folder contents
          bool bFolderContents = XPConvert.ToBoolean(m_saData.GetItem(indObj,
           MigrationSelectForExportModel.EXPLIST_COL_FOLDER_CONTENTS));
          if (bFolderContents)
          {
            if (nClassID == PT_CLASSIDS.PT_ADMIN_FOLDER_ID)
            {
              IPTAdminCatalog adminCatalog = m_ptSession.GetAdminCatalog();
              IPTAdminFolder adminFolder = adminCatalog.OpenAdminFolder(nObjectID,
               false);
              ExportAdminFolderContents(adminCatalog, adminFolder, bDependencies);
              } else
            if (nClassID == PT_CLASSIDS.PT_CATALOGFOLDER_ID)
            {
              IPTCatalog dirCatalog = m_ptSession.GetCatalog();
              IPTFolder dirFolder = dirCatalog.OpenFolder(nObjectID, false);
              ExportDirectoryFolderContents(dirCatalog, dirFolder, bDependencies);
              }
          }
        }
        WriteExportPackage();
        if (m_intStatus != FAILED)
        {
          log.Info("Migration Task completed.");
          m_intStatus = DONE;
        } else
        {
          log.Error("Migration Task failed.");
        }
      }
      catch (Exception e)
      {
        m_intStatus = FAILED;
        log.Error(e, "PTExportMigrationTask::WriteExportPackage Unknown error writing export package.");
        }
    }
    private IXPPropertyBag CreateMigrationConfigBag(bool exportDependencies)
    {
      IXPPropertyBag migrationConfigBag = PortalObjectsFactory.CreatePropertyBag();
      // mel: Fix for 45700, should optionally be able to export parent folders when exporting dependencies
      if (exportDependencies && exportParentFolders)
      {
        try
        {
          PTBasePersist.PXINT_SAVE(migrationConfigBag, PTMigrationConstants.ExportParentFolders,
           XPConvert.ToInteger(exportParentFolders));
        }
        catch (Exception anyException)
        {
          log.Error(anyException, Global.ReplaceTokens("{0}.CreateMigrationConfigBag failed to save export parent folders option. Parent folders will not be exported.",
           OUR_NAME));
        }
      }
      return migrationConfigBag;
    }
    
    /// <param name="uuid">This parameter is always required and may not match ptME.GetUUID due to the way dependencies are exported. See ExportDependentObjects</param>
    private void TrackExportedObject(IPTMigrationElement ptME, String uuid,
     int classID, int objectID)
    {
      m_xpalStrMigrated.Add(uuid);
      m_xphtUUIDs.PutElement(uuid, ptME);
      exportedClassIDsAsIntObj.Add(classID);
      exportedObjectIDsAsIntObj.Add(objectID);
    }
    
    /// <summary>
    /// Export one object and its dependencies, if the
    /// flag is set.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_nObjectID"></param>
    /// <returns></returns>
    private IPTMigrationElement ExportObject(int _nClassID, int _nObjectID,
     bool _bExportDependencies)
    {
      IXPPropertyBag migrationConfigBag = CreateMigrationConfigBag(_bExportDependencies);
      IPTMigrationElement ptME = m_MigrationManager.ExportObject(_nClassID,
       _nObjectID, migrationConfigBag);
      // If it's not already in the package, export it.
      String strUUID = ptME.GetUUID();
      if (!m_MigrationManager.ObjectExistsInPackage(strUUID))
      {
        m_MigrationManager.StoreObjectInExportPackage(ptME);
        TrackExportedObject(ptME, strUUID, _nClassID, _nObjectID);
      }
      String strObjectName = ptME.GetName();
      log.Info("Migrated object: " + strObjectName + "; classID=" + _nClassID + "; objectID=" + _nObjectID);
      if (m_intStatus == CANCEL)
      {
        log.Info("Migration Task cancelled.");
        return ptME;
      }
      DependencyBundle db = new DependencyBundle(ptME, _nObjectID);
      // Then migrate its dependencies recursively
      ExportDependentObjects(ptME, _bExportDependencies, db);
      if (m_intStatus == CANCEL)
      {
        log.Info("Migration Task cancelled.");
        return ptME;
      }
      return ptME;
    }
    
    /// <summary>
    /// Convert an XPArrayList of Integer to int[].
    /// TODO This should be rolled into openfoundation, minus the error handling.
    /// </summary>
    /// <returns>Always returns a non-null array. Returns an empty error and logs if an error occurred.</returns>
    private int[] GetIntArrayFromIntegerObjects(XPArrayList integerObjects)
    {
      int[] emptyArray = {};
      if (integerObjects == null || integerObjects.IsEmpty())
      {
        return emptyArray;
      }
      int[] rval = null;
      try
      {
        rval = new int[integerObjects.GetSize()];
        IXPEnumerator enumer = integerObjects.GetEnumerator();
        int index = 0;
        while (enumer.MoveNext())
        {
          int intAsObj = (int) enumer.GetCurrent();
          rval[index] = XPConvert.ToInteger(intAsObj);
          ++index;
        }
      }
      catch (Exception anyException)
      {
        log.Error(anyException, OUR_NAME + " Error while converting object/class IDs to int[]. Collaboration/Content/Studio resources will not be properly exported for this MWS.");
        return emptyArray;
      }
      return rval;
    }
    
    /// <summary>
    /// Write package content to file
    /// </summary>
    public virtual void WriteExportPackage()
    {
      // Write our basic data
      m_MigrationManager.WriteUserDataItemToPackage(PACKAGE_NAME, m_strName);
      m_MigrationManager.WriteUserDataItemToPackage(PACKAGE_DESCRIPTION, m_strDescription);
      m_MigrationManager.WriteUserDataItemToPackage(PUBLISHER_NAME, m_strPublisher);
      if (m_htMWSPropBags != null && !m_htMWSPropBags.IsEmpty())
      {
        int[] exportedClassIDs = GetIntArrayFromIntegerObjects(exportedClassIDsAsIntObj);
        int[] exportedObjectIDs = GetIntArrayFromIntegerObjects(exportedObjectIDsAsIntObj);
        // assert: exportedClassIDs != null && exportedObjectIDs != null
        if (exportedClassIDs.Length != exportedObjectIDs.Length)
        {
          log.Error(Global.ReplaceTokens(OUR_NAME + "The quantity of exported class IDs {0} does not match the quantity of object IDs {1}. This is an internal error and may cause unexpected behavior in the remote MWS.",
           exportedClassIDs.Length, exportedObjectIDs.Length));
        }
        log.Info("Beginning remote export from " + m_htMWSPropBags.GetSize() + " Migration Web Services");
        IXPEnumerator enumKeys = m_htMWSPropBags.GetKeys();
        while (enumKeys.MoveNext())
        {
          Object currWSID = enumKeys.GetCurrent();
          String mwsName = GetMWSName(currWSID);
          String mwsLogMessagePrefix = Global.ReplaceTokens("Migration Web Service \"{0}\" (objectID={1}) ",
           mwsName, currWSID);
          IXPPropertyBag currPropBag = (IXPPropertyBag) m_htMWSPropBags.GetElement(currWSID);
          IPTMigrationProvider migProvider = m_MigrationManager.GetProvider(XPConvert.ToInteger(currWSID));
          // mel: Added exception handling and better logging
          try
          {
            String strPackageData = migProvider.ExportPackage((IXPPropertyBag) currPropBag,
             exportedClassIDs, exportedObjectIDs);
            log.Info(mwsLogMessagePrefix + "returned package data: " + strPackageData);
            m_MigrationManager.WriteMWSDataItemToPackage(XPConvert.ToInteger(currWSID),
             strPackageData);
            log.Info("Successfully exported resources from Migration Web Service ID " + XPConvert.ToInteger(currWSID));
            AddMWSMessage(mwsName, strPackageData);
          }
          catch (Exception anyException)
          {
            log.Error(anyException, OUR_NAME + ".WriteExportPackage received an error from " + mwsLogMessagePrefix);
            }
          if (m_intStatus == CANCEL)
          {
            log.Info("Migration Task cancelled.");
            return;
          }
        }
      }
      // End the export
      m_MigrationManager.EndExport();
      m_MigrationManager.BulkSetMigratedStatusByUUID(m_xpalStrMigrated.ToArray());
      }
    // mel: Fix for 46247, When a user exports objects from Collab or Publisher, no feedback is given that this was done. At least provide the UUID of the zip file in the final migration status.
    
    /// <returns>XPArrayList of String each of which represents the return value from an MWS ExportPackage call with info about the MWS. If no MWS's are present or none responded, returns an empty list.</returns>
    public override XPArrayList GetMWSMessages()
    {
      if (mwsMessages == null)
      {
        return new XPArrayList();
      }
      return mwsMessages;
    }
    
    /// <param name="webServiceIDAsObject">A string representing the web service object ID</param>
    /// <returns>On error returns "" and logs a warning, on success returns the localized name of the MWS, e.g. "Collaboration Migration Web Service"</returns>
    private String GetMWSName(Object webServiceIDAsObject)
    {
      String mwsName = "";
      try
      {
        int mwsObjectID = XPConvert.ToInteger(webServiceIDAsObject);
        IPTLocalizable mwsAsLocalizable = (IPTLocalizable) m_ptSession.GetObjectManagers(PT_CLASSIDS.PT_WEBSERVICE_ID).Open(mwsObjectID,
         false);
        mwsName = mwsAsLocalizable.GetLocalizedName();
      }
      catch (Exception anyException)
      {
        log.Warn(anyException, OUR_NAME + " failed to determine Migration Web Service name.");
        }
      return mwsName;
    }
    private void AddMWSMessage(String mwsName, String returnValueFromExport)
    {
      // Refuse to add a message if its empty
      if (mwsName == null || returnValueFromExport == null)
      {
        return;
      }
      String returnValueTrimmed = XPStringUtility.Trim(returnValueFromExport);
      if (returnValueTrimmed == null || returnValueTrimmed.Length == 0)
      {
        return;
      }
      // mel: We make the assumption that the Collab/Content/Studio file needed is the return value + ".zip". (46247)
      mwsMessages.Add("" + mwsName + " : " + returnValueFromExport + ".zip");
      }
    // mel: WARNING: Code duplication, here and PTExportMigrationTask, see bug# 45264
    // mel: Fix for 45935, exporting the plumtree auth source with export dependencies enabled when SSO auth partner is enabled results in "Error in exporting dependency with UUID {SSO-INTRINSIC-UUID}
    private bool isDependencyExcludedFromExport(String elementUUID)
    {
      if (elementUUID == null)
      {
        return false;
      }
      // Exclude SSO-INTRINSIC-UUID from export
      if (PTMigrationConstants.SSO_INTRINSIC_UUID.Equals(elementUUID))
      {
        return true;
      }
      return false;
    }
    private bool isDependencyExcludedFromExport(int dependencyType)
    {
      // Never export ACLs
      if ((PT_OBJECT_DEPENDENCY_TYPES.PT_OBJECT_DEPENDENCY_ACL & dependencyType) != 0)
      {
        return true;
      }
      // NEVER_EXPORT if explicitly flagged by the object.
      // For example PTUser might flag parent groups as NEVER_EXPORT to prevent bringing in its parent groups, e.g. Administrators group.
      // Fix for bug# 37311, 41478, 45207
      if ((PT_OBJECT_DEPENDENCY_TYPES.PT_OBJECT_DEPENDENCY_NEVER_EXPORT & dependencyType) != 0)
      {
        return true;
      }
      return false;
    }
    private bool isExportableDependency(int dependencyType)
    {
      if ((PT_OBJECT_DEPENDENCY_TYPES.PT_OBJECT_DEPENDENCY_ALWAYS_EXPORT & dependencyType) != 0)
      {
        return true;
      }
      return false;
    }
    // mel: WARNING: Discovered code duplication, here and ExportContext, see bug# 45264
    
    /// <summary>
    /// Recursively export all dependencies of the given 
    /// IPTMigrationElement
    /// </summary>
    /// <param name="ptME"></param>
    /// <param name="bExportAllDependencies"></param>
    /// <param name="dbOrig"></param>
    public virtual void ExportDependentObjects(IPTMigrationElement ptME, bool bExportAllDependencies,
     DependencyBundle dbOrig)
    {
      Object o = ptME.GetDependencies();
      if (null == o)
      {
        return;
      }
      Object[][] arDependencies = (Object[][]) o;
      int nDependencies = arDependencies[0].Length;
      String strOriginalMigrationComment = ptME.GetMigrationComment();
      for (int i = 0;i < nDependencies;i++)
      {
        int nDependencyType = XPConvert.ToInteger(arDependencies[PT_OBJECT_DEPENDENCY_COLS.PT_ODC_DEPENDENCY_TYPE][i]);
        String strUUID = XPConvert.ToString(arDependencies[PT_OBJECT_DEPENDENCY_COLS.PT_ODC_UUID][i]);
        if (isDependencyExcludedFromExport(nDependencyType) || isDependencyExcludedFromExport(strUUID))
        {
          // ACL dependency, don't bring in
          continue;
        } else
        if (bExportAllDependencies || isExportableDependency(nDependencyType))
        {
          // export the dependent object as well, if it is not already in the package
          IPTMigrationElement ptMEDependency;
          ptMEDependency = (IPTMigrationElement) m_xphtUUIDs.GetElement(strUUID);
          if (null == ptMEDependency)
          {
            Object[] arObject;
            int nClassID = 0;
            int nObjectID = -1;
            try
            {
              arObject = (Object[]) m_MigrationManager.UUIDToObjectID(strUUID);
              nClassID = XPConvert.ToInteger(arObject[0]);
              nObjectID = XPConvert.ToInteger(arObject[1]);
              if (IsNonMigratableObject(nClassID, nObjectID))
              {
                // Don't bring in intrinsic objects that can't be modified.
                continue;
              }
              ptMEDependency = m_MigrationManager.ExportObject(nClassID, nObjectID,
               PortalObjectsFactory.CreatePropertyBag());
              ptMEDependency.SetMigrationComment(strOriginalMigrationComment);
              m_MigrationManager.StoreObjectInExportPackage(ptMEDependency);
              TrackExportedObject(ptMEDependency, strUUID, nClassID, nObjectID);
              }
            catch (Exception e)
            {
              ptMEDependency = null;
              if (nObjectID < 0)
              {
                log.Error("Error in exporting dependency with UUID " + strUUID);
                } else
              {
                log.Error("Error in exporting dependent classID " + nClassID + " with ID " + nObjectID);
                }
            }
          }
          if (m_intStatus == CANCEL)
          {
            log.Info("Migration Task cancelled.");
            return;
          }
          // Now get the list of dependencies right for this DependencyBundle
          // so it can be displayed in the UI.
          if (null != ptMEDependency && !dbOrig.DependencyExistsInBundle(strUUID))
          {
            dbOrig.AddDependentObject(ptMEDependency);
            // Recursive call.  Recursion stops when dependencies already in the
            // dbOrig DependencyBundle have been visited, or when there are no
            // dependencies left.
            ExportDependentObjects(ptMEDependency, bExportAllDependencies,
             dbOrig);
          }
        }
      }
    }
    
    /// <summary>
    /// Recursively export all contents of the given 
    /// IPTAdminFolder
    /// </summary>
    /// <param name="ptME"></param>
    /// <param name="bExportAllDependencies"></param>
    /// <param name="dbOrig"></param>
    public virtual void ExportAdminFolderContents(IPTAdminCatalog ptAC, IPTAdminFolder ptAF,
     bool bExportAllDependencies)
    {
      IASQueryResult queryLeaves = new ASQueryResultWrapper(ptAF.QueryObjects(false));
      int nLeaves = queryLeaves.GetCount();
      for (int indLeaf = 0;indLeaf < nLeaves;indLeaf++)
      {
        int nClassID = XPConvert.ToInteger(queryLeaves.GetFields(indLeaf,
         PT_PROPIDS.PT_PROPID_CLASSID));
        int nObjectID = XPConvert.ToInteger(queryLeaves.GetFields(indLeaf,
         PT_PROPIDS.PT_PROPID_OBJECTID));
        String strUUID = m_MigrationManager.ObjectIDToUUID(nClassID, nObjectID);
        // export the child object, if it is not already in the package
        IPTMigrationElement ptME = (IPTMigrationElement) m_xphtUUIDs.GetElement(strUUID);
        if (null == ptME)
        {
          ptME = ExportObject(nClassID, nObjectID, bExportAllDependencies);
          if (m_intStatus == CANCEL)
          {
            log.Info("Migration Task cancelled.");
            return;
          }
        }
      }
      IASQueryResult queryFolders = new ASQueryResultWrapper(ptAF.QuerySubfolders(PT_PROPIDS.PT_PROPID_OBJECTID,
       0, PT_PROPIDS.PT_PROPID_NAME, -1, -1, null));
      int nFolders = queryFolders.GetCount();
      if (nFolders == 0)
      {
        return;
      }
      for (int indFolder = 0;indFolder < nFolders;indFolder++)
      {
        int nFolderID = XPConvert.ToInteger(queryFolders.GetFields(indFolder,
         PT_PROPIDS.PT_PROPID_OBJECTID));
        ExportObject(PT_CLASSIDS.PT_ADMIN_FOLDER_ID, nFolderID, bExportAllDependencies);
        if (m_intStatus == CANCEL)
        {
          log.Info("Migration Task cancelled.");
          return;
        }
        IPTAdminFolder childFolder = ptAC.OpenAdminFolder(nFolderID, false);
        ExportAdminFolderContents(ptAC, childFolder, bExportAllDependencies);
        }
    }
    
    /// <summary>
    /// Recursively export all contents of the given 
    /// IPTFolder.
    /// </summary>
    /// <param name="ptME"></param>
    /// <param name="bExportAllDependencies"></param>
    /// <param name="dbOrig"></param>
    public virtual void ExportDirectoryFolderContents(IPTCatalog ptCatalog,
     IPTFolder ptFolder, bool bExportAllDependencies)
    {
      IASQueryResult queryCards = new ASQueryResultWrapper(ptFolder.QueryCards(PT_PROPIDS.PT_PROPID_OBJECTID,
       true, PT_PROPIDS.PT_PROPID_OBJECTID, -1, -1, null));
      int nCards = queryCards.GetCount();
      for (int indCard = 0;indCard < nCards;indCard++)
      {
        int nObjectID = XPConvert.ToInteger(queryCards.GetFields(indCard,
         PT_PROPIDS.PT_PROPID_OBJECTID));
        String strUUID = m_MigrationManager.ObjectIDToUUID(PT_CLASSIDS.PT_CATALOGCARD_ID,
         nObjectID);
        // export the child object, if it is not already in the package
        IPTMigrationElement ptME = (IPTMigrationElement) m_xphtUUIDs.GetElement(strUUID);
        if (null == ptME)
        {
          ptME = ExportObject(PT_CLASSIDS.PT_CATALOGCARD_ID, nObjectID, bExportAllDependencies);
          if (m_intStatus == CANCEL)
          {
            log.Info("Migration Task cancelled.");
            return;
          }
        }
      }
      IASQueryResult queryFolders = new ASQueryResultWrapper(ptFolder.QuerySubfolders(PT_PROPIDS.PT_PROPID_OBJECTID,
       0, PT_PROPIDS.PT_PROPID_OBJECTID, -1, -1, null));
      int nFolders = queryFolders.GetCount();
      if (nFolders == 0)
      {
        return;
      }
      for (int indFolder = 0;indFolder < nFolders;indFolder++)
      {
        int nFolderID = XPConvert.ToInteger(queryFolders.GetFields(indFolder,
         PT_PROPIDS.PT_PROPID_OBJECTID));
        ExportObject(PT_CLASSIDS.PT_CATALOGFOLDER_ID, nFolderID, bExportAllDependencies);
        if (m_intStatus == CANCEL)
        {
          log.Info("Migration Task cancelled.");
          return;
        }
        IPTFolder childFolder = ptCatalog.OpenFolder(nFolderID, false);
        ExportDirectoryFolderContents(ptCatalog, childFolder, bExportAllDependencies);
        }
    }
    
    /// <summary>
    /// Check for any migration restrictions
    /// </summary>
    /// <param name="nClassID"></param>
    /// <param name="nObjectID"></param>
    /// <returns></returns>
    public static bool IsNonMigratableObject(int nClassID, int nObjectID)
    {
      bool bRet = false;
      switch (nClassID){
      case PT_CLASSIDS.PT_CATALOGFOLDER_ID:
      {
        if (PT_INTRINSICS.PT_FOLDER_ROOTFOLDER == nObjectID || PT_INTRINSICS.PT_FOLDER_UNCLASSIFIEDCARDS == nObjectID)
        {
          bRet = true;
        }
        break;
      }
      case PT_CLASSIDS.PT_ADMIN_FOLDER_ID:
      {
        if (PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER == nObjectID || PT_INTRINSICS.PT_ADMIN_FOLDER_UNCLASSIFIEDOBJECTS == nObjectID)
        {
          bRet = true;
        }
        break;
      }
      case PT_CLASSIDS.PT_USERGROUP_ID:
      {
        if (PT_INTRINSICS.PT_USERGROUP_EVERYONE == nObjectID)
        {
          bRet = true;
        }
        break;
      }
      case PT_CLASSIDS.PT_PROPERTY_ID:
      {
        if (nObjectID >= PT_SERVERCONSTANTS.PT_MIN_READONLYINTRINSIC_OBJECTID && nObjectID <= PT_SERVERCONSTANTS.PT_MAX_READONLYINTRINSIC_OBJECTID)
        {
          bRet = true;
        }
        break;
      }
      case PT_CLASSIDS.PT_ACTIVITY_ID:
      {
        if (nObjectID <= PT_SERVERCONSTANTS.PT_MAX_READONLYINTRINSIC_OBJECTID)
        {
          bRet = true;
        }
        break;
      }
      }
      return bRet;
    }
    
    /// <summary>
    /// Set the hashtable of property bags to send to MWS's.
    /// Key=MWS ID; Value=IXPPropertyBag.
    /// </summary>
    /// <param name="_htMWSPropBags"></param>
    public virtual void SetMWSPropBags(XPHashtable _htMWSPropBags)
    {
      m_htMWSPropBags = _htMWSPropBags;
    }
  }
}