using System;
using com.plumtree.openfoundation.util;
using com.plumtree.server;
namespace com.plumtree.portalpages.browsing.editors.objectdelivery.common
{
  /// <author>
  /// adolfoc
  /// This class is a simple representation of a dependency returned by the ObjectMigration API,
  /// with some bonus features.
  /// </author>
  public class ODDependency
  {
    
    /// <summary>
    /// These constants are used as a mask with the GetODDependency call to
    /// select what type of dependencies to return.
    /// </summary>
    public const int ODD_GET_ALL = -1;
    public const int ODD_GET_SUPPORTED = 1;
    public const int ODD_GET_UNSUPPORTED = 2;
    public const int ODD_GET_HIDDEN = 4;
    protected IPTMigrationElement m_ptMEParent = null;
    
    /// <summary>
    /// Our local information
    /// </summary>
    protected String m_strName = "";
    protected String m_strUUID = "";
    protected int m_nCLASSID = -1;
    protected int m_nDependencyType = 0;
    
    /// <summary>
    /// Constructor for ODDependency.
    /// </summary>
    public ODDependency(String strName, String strUUID, int nCLASSID, int nDependencyType,
     IPTMigrationElement ptMEParent)
    {
      m_strName = strName;
      m_strUUID = strUUID;
      m_nCLASSID = nCLASSID;
      m_nDependencyType = nDependencyType;
      m_ptMEParent = ptMEParent;
    }
    
    /// <summary>
    /// Returns the CLASSID.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetCLASSID()
    {
      return m_nCLASSID;
    }
    
    /// <summary>
    /// Returns the dependencyType.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetDependencyType()
    {
      return m_nDependencyType;
    }
    
    /// <summary>
    /// Returns the mEParent.
    /// </summary>
    /// <returns>IPTMigrationElement</returns>
    public virtual IPTMigrationElement GetMEParent()
    {
      return m_ptMEParent;
    }
    
    /// <summary>
    /// Returns the name.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetName()
    {
      return m_strName;
    }
    // Static methods;
    
    /// <summary>
    /// Static method which creates a bunch of ODDependencies given a Migration Element and a 
    /// selection criteria (see the mask defined above).
    /// </summary>
    public static XPArrayList GetODDependencies(IPTMigrationElement ptME,
     int nSelectMask)
    {
      XPArrayList xpalResult = new XPArrayList();
      Object objDepends = ptME.GetDependencies();
      if (null == objDepends)
      {
        // This is one end point for the recursion (no dependencies found).
        return xpalResult;
      }
      Object[][] arDependencies = (Object[][]) objDepends;
      int nDependencies = arDependencies[0].Length;
      for (int nIndex = 0;nIndex < nDependencies;nIndex++)
      {
        String strName = ((String) arDependencies[PT_OBJECT_DEPENDENCY_COLS.PT_ODC_NAME][nIndex]);
        String strUUID = ((String) arDependencies[PT_OBJECT_DEPENDENCY_COLS.PT_ODC_UUID][nIndex]);
        int nCLASSID = XPConvert.ToInteger(arDependencies[PT_OBJECT_DEPENDENCY_COLS.PT_ODC_CLASSID][nIndex]);
        int nDependencyType = XPConvert.ToInteger(arDependencies[PT_OBJECT_DEPENDENCY_COLS.PT_ODC_DEPENDENCY_TYPE][nIndex]);
        bool bAdd = ((0 != (nSelectMask & ODD_GET_SUPPORTED)) && ObjectDeliveryHelper.IsSupported(nCLASSID)) || ((0 != (nSelectMask & ODD_GET_UNSUPPORTED)) && !ObjectDeliveryHelper.IsSupported(nCLASSID) && !(ObjectDeliveryHelper.IsHidden(nCLASSID) || ObjectDeliveryHelper.IsHiddenDependencyType(nDependencyType))) || ((0 != (nSelectMask & ODD_GET_HIDDEN)) && (ObjectDeliveryHelper.IsHidden(nCLASSID) || ObjectDeliveryHelper.IsHiddenDependencyType(nDependencyType)));
        if (bAdd)
        {
          xpalResult.Add(new ODDependency(strName, strUUID, nCLASSID, nDependencyType,
           ptME));
        }
      }
      return xpalResult;
    }
    
    /// <summary>
    /// Static method which creates a bunch of ODDependencies, given a MigrationElement.
    /// </summary>
    public static XPArrayList GetODDependencies(IPTMigrationElement ptME)
    {
      return GetODDependencies(ptME, ODD_GET_ALL);
    }
    
    /// <summary>
    /// Static method which creates a bunch of ODDependencies, given a MigrationElement.
    /// Returns an array of only those ODDependencies that are supported (as listed in
    /// ObjectDeliveryHelper.m_arSupportedCLASSIDS); This method does not return
    /// Folders as supported dependencies.
    /// </summary>
    public static XPArrayList GetSupportedODDependencies(IPTMigrationElement ptME)
    {
      return GetODDependencies(ptME, ODD_GET_SUPPORTED);
    }
    
    /// <summary>
    /// Returns the uUID.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetUUID()
    {
      return m_strUUID;
    }
    
    /// <summary>
    /// Static method which creates a bunch of ODDependencies, given a MigrationElement.
    /// Returns an array of only those ODDependencies that are NOT supported (as listed in
    /// ObjectDeliveryHelper.m_arSupportedCLASSIDS); This method DOES NOT return Folder
    /// Dependencies or Users as UnSupported.
    /// </summary>
    public static XPArrayList GetUnImportableODDependencies(IPTMigrationElement ptME)
    {
      return GetODDependencies(ptME, ODD_GET_UNSUPPORTED);
    }
    
    /// <summary>
    /// Static method which creates a bunch of ODDependencies, given a MigrationElement.
    /// Returns an array of only those ODDependencies that are NOT supported (as listed in
    /// ObjectDeliveryHelper.m_arSupportedCLASSIDS); This method DOES return Folder
    /// Dependencies or Users as UnSupported.
    /// </summary>
    public static XPArrayList GetAllUnImportableODDependencies(IPTMigrationElement ptME)
    {
      return GetODDependencies(ptME, ODD_GET_UNSUPPORTED | ODD_GET_HIDDEN);
      }
    
    /// <summary>
    /// Returns whether or not this dependency is of this dependency type:
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsDependentBy(int nDependencyMask)
    {
      if (0 == (nDependencyMask & m_nDependencyType))
      {
        return false;
      } else
      {
        return true;
      }
    }
    
    /// <seealso cref="java.lang.Object#equals(Object)"></seealso>
    public override bool Equals(Object arg0)
    {
      try
      {
        ODDependency odEquand = (ODDependency) arg0;
        if (0 == m_strUUID.CompareTo(odEquand.GetUUID()))
        {
          return true;
        }
      }
      catch (Exception e)
      {
      }
      return false;
    }
  }
}