using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.pei;
using com.plumtree.portaluiinfrastructure.search;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.server.portalpagesperf;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portalpages.browsing.search.results
{
  /// <author>CraigS</author>
  public class AdvancedStartSearchControl : StartSearchControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.search.results.AdvancedStartSearchControl");
    
    /// <summary>
    /// Name of this control. 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "advancedstart";
    
    /// <summary>
    /// Creates an instance
    /// </summary>
    /// <returns>new instance</returns>
    public override Object Create()
    {
      return new AdvancedStartSearchControl();
    }
    
    /// <summary>
    /// Returns class name
    /// </summary>
    /// <returns>class name</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    // the user's original search string- we will log this for the sake
    // of search log reports
    private String _logString;
    
    /// <summary>
    /// This method is called by StartSearchControl to allow customizations to
    /// the query; it simply turns around and calls any advanced search PEIs that have been defined.
    /// </summary>
    /// <param name="query">the user's query</param>
    protected override Redirect CustomizeQuery(QueryArguments query)
    {
      // Bump up the advanced search counter
      PerfCounters.INSTANCE.IncrementCounter(PORTALPAGESPERF_COUNTERS.PORTALPAGESPERF_ADVANCED_SEARCHES);
      // Make sure PEIs are loaded
      LoadAdvancedSearchPEIs();
      // Call out to each PEI in turn
      if (null != _advancedSearchPEIs)
      {
        IXPEnumerator e = _advancedSearchPEIs.GetEnumerator();
        while (e.MoveNext())
        {
          IAdvancedSearchActions pei = (IAdvancedSearchActions) e.GetCurrent();
          pei.CustomizeQueryOnBeforeSearch(_parent, (IPTSession) _parent.GetUserSession(),
           query);
        }
      }
      // Save the user's query for logging in the search server logs
      IPTFilter filter = query.advancedFilter;
      if (filter != null)
      {
        String searchString = filter.GetSearchString();
        if (searchString != null)
        {
          _logString = searchString;
          // Log this query to the Analytics Server
          if (_parent.GetModelExists(LoggingHelperModel.STR_MVC_CLASS_NAME))
          {
            LoggingHelperModel logHelper = (LoggingHelperModel) _parent.GetModel(LoggingHelperModel.STR_MVC_CLASS_NAME);
            if (null != logHelper)
            {
              logHelper.LogStartSearchEvent(query.userQuery, LoggingHelperModel.SEARCH_TYPE_ADVANCED);
              }
          }
        }
      }
      // Return null to stay on the search results page
      return null;
    }
    
    /// <summary>
    /// This method is called just before the initial query is
    /// executed for advanced search... it simply calls out to any PEIs that have
    /// been defined to allow them to customize search settings.
    /// </summary>
    protected override SearchSettingCollection GetCustomSearchSettings()
    {
      SearchSettingCollection settings = new SearchSettingCollection();
      // Make sure any PEIs are loaded
      LoadAdvancedSearchPEIs();
      if (null != _advancedSearchPEIs)
      {
        IXPEnumerator e = _advancedSearchPEIs.GetEnumerator();
        while (e.MoveNext())
        {
          IAdvancedSearchActions pei = (IAdvancedSearchActions) e.GetCurrent();
          SearchSettingCollection customSettings = pei.GetCustomSettingsOnBeforeSearch(_parent,
           (IPTSession) _parent.GetUserSession());
          if (null != customSettings)
          {
            settings.merge(customSettings);
          }
        }
      }
      // All done
      return settings;
    }
    private IXPCollection _advancedSearchPEIs = null;
    private bool _loadedPEIs = false;
    
    /// <summary>
    /// BrianH/CraigS : This function will pull in all of the available
    /// IAdvancedSearchActions PEIs.
    /// </summary>
    private void LoadAdvancedSearchPEIs()
    {
      // if already loaded
      if (_loadedPEIs)
      {
        // don't do it again
        return;
      }
      _loadedPEIs = true;
      IApplication application = _parent.GetApplication();
      Object[] actions = (Object[]) application.GetCachingManager().GetEntry("AdvancedSearchActions");
      if (null != actions)
      {
        _advancedSearchPEIs = new XPArrayList();
        for (int x = 0;x < actions.Length;x++)
        {
          Object oAction = actions[x];
          if ((null != oAction) && (oAction is IAdvancedSearchActions))
          {
            _advancedSearchPEIs.Add(oAction);
          } else
          {
            log.Warn("AdvancedStartSearchControl.LoadAdvancedSearchPEIs: element " + XPConvert.ToString(x) + " is not an instance of IAdvancedSearchActions");
            }
        }
      }
    }
    
    /// <summary>
    /// For advanced searches, the user's query is logged in the search server logs
    /// </summary>
    protected override String GetLogString()
    {
      return _logString;
    }
  }
}