using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.server;
namespace com.plumtree.portalpages.browsing.search.results.assocobjects
{
  /// <author>CraigS</author>
  public class AssociatedObjectsModel : IModel
  {
    
    /// <summary>
    /// Name of this class 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "SRAssociatedObjects";
    
    /// <summary>
    /// Owning session 
    /// </summary>
    private IPTSession _session;
    
    /// <summary>
    /// Owning activityspace 
    /// </summary>
    private AActivitySpace m_asOwner;
    
    /// <summary>
    /// Name of the item we are dealing with 
    /// </summary>
    private String _itemName;
    
    /// <summary>
    /// Names of portlets the item is associated with (and that the user can see)
    /// </summary>
    private String[] _portletNames;
    
    /// <summary>
    /// Mapping of portlet names (from the _portletNames list) to IDs
    /// </summary>
    private IXPDictionary _portletIDMap;
    
    /// <summary>
    /// Names of communities the item is associated with (and that the user can see)
    /// </summary>
    private String[] _communityNames;
    
    /// <summary>
    /// Mapping of community names (from the _communityNames list) to IDs
    /// </summary>
    private IXPDictionary _communityIDMap;
    public virtual void Init(AActivitySpace parent)
    {
      m_asOwner = parent;
      _session = (IPTSession) m_asOwner.GetUserSession();
      _itemName = null;
      _portletNames = null;
      _portletIDMap = null;
      _communityNames = null;
      _communityIDMap = null;
    }
    public virtual void CleanupTempData()
    {
    }
    public virtual void CleanupAllData()
    {
      _itemName = null;
      _portletNames = null;
      _portletIDMap = null;
      _communityNames = null;
      _communityIDMap = null;
    }
    public virtual Object Create()
    {
      return new AssociatedObjectsModel();
    }
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    public virtual String GetItemName()
    {
      return _itemName;
    }
    public virtual void SetItemName(String itemName)
    {
      _itemName = itemName;
    }
    public virtual void SetPortletList(int[] portletList)
    {
      if ((null == portletList) || (0 == portletList.Length))
      {
        _portletNames = null;
        _portletIDMap = null;
        _communityNames = null;
        _communityIDMap = null;
        return;
      }
      XPArrayList pNames = new XPArrayList();
      XPArrayList cNames = new XPArrayList();
      _portletIDMap = new XPHashtable();
      _communityIDMap = new XPHashtable();
      IPTObjectManager pMgr = _session.GetGadgets();
      IPTCommunityManager cMgr = (IPTCommunityManager) (_session.GetCommunities());
      // For each portlet in list
      for (int j = 0;j < portletList.Length;j++)
      {
        String portletName = null;
        // Check whether the user can see this portlet
        try
        {
          /* lock= */
          IPTObject ptObj = (IPTObject) (pMgr.Open(portletList[j], false));
          portletName = ptObj.GetLocalizedName();
        }
        catch (Exception e)
        {
          portletName = null;
        }
        if ((null == portletName) || (0 == portletName.Length))
        {
          // user apparently can't see this portlet
          continue;
        }
        pNames.Add(portletName);
        _portletIDMap.PutElement(portletName, portletList[j]);
        // Look up communities the portlet is associated with
        IPTQueryResult qr = cMgr.QueryCommunitiesForGadget(portletList[j]);
        int cCount = qr.RowCount();
        for (int k = 0;k < cCount;k++)
        {
          int communityID = qr.ItemAsInt(k, PT_PROPIDS.PT_PROPID_OBJECTID);
          String communityName = qr.ItemAsString(k, PT_PROPIDS.PT_PROPID_NAME);
          // See if the _communityIDMap already contains this community.  Have to use the community
          // name, since that is the key for the map, but it's unlikely that there will be
          // two communities with the same name.
          if (!_communityIDMap.ContainsKey(communityName))
          {
            cNames.Add(communityName);
            _communityIDMap.PutElement(communityName, communityID);
          }
        }
      }
      // end k-loop over communities for portlet j
      // end j-loop over portlets
      // If no portlets, couldn't possibly have gotten communities either, so null out everything
      if (0 == pNames.GetSize())
      {
        _portletNames = null;
        _portletIDMap = null;
        _communityNames = null;
        _communityIDMap = null;
        return;
      }
      // At this point, we know we have something in the pNames list.  Call helper to sort it
      _portletNames = listToSortedStringArray(pNames);
      // It's possible that we got a list of portlets, but not communities
      if (0 == cNames.GetSize())
      {
        _communityNames = null;
        _communityIDMap = null;
      } else
      {
        // Else, sort the community names list too
        _communityNames = listToSortedStringArray(cNames);
      }
    }
    
    /// <summary>
    /// Converts an unsorted IXPList of Strings into a sorted array of strings. 
    /// </summary>
    protected virtual String[] listToSortedStringArray(IXPList list)
    {
      int n = list.GetSize();
      String[] sa = new String[n];
      for (int j = 0;j < n;j++)
      {
        sa[j] = (String) (list.GetElement(j));
      }
      XPArray.Sort(sa);
      return sa;
    }
    public virtual int GetNumPortlets()
    {
      if (null == _portletNames)
      {
        return 0;
      }
      return _portletNames.Length;
    }
    public virtual int GetNumCommunities()
    {
      if (null == _communityNames)
      {
        return 0;
      }
      return _communityNames.Length;
    }
    public virtual String GetPortletName(int idx)
    {
      return _portletNames[idx];
    }
    public virtual String GetCommunityName(int idx)
    {
      return _communityNames[idx];
    }
    public virtual int GetPortletID(int idx)
    {
      return XPConvert.ToInteger(_portletIDMap.GetElement(_portletNames[idx]));
      }
    public virtual int GetCommunityID(int idx)
    {
      return XPConvert.ToInteger(_communityIDMap.GetElement(_communityNames[idx]));
      }
  }
}