using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.login;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.login;
namespace com.plumtree.portalpages.browsing.sso.ensemble
{
  /// <summary>
  /// Controls process the events generated by Ensemble for SSO purposes. 
  /// Controls should encapsulate the business logic of Ensemble SSO.  When Ensemble sends a request to
  /// the WCI verifies if there is a current session and Internal SSO is turned on then this Control creates
  /// redirect url with login_token as query string and sends HTTP redirect.
  /// e.g:  
  /// req:  http://<portalserver_hostname>:<port>/portal/server.pt?space=EnsembleSSO&redirect_url=<orig_req_url>
  /// or
  /// req:  http://<portalserver_hostname>:<port>/portal/server.pt?space=EnsembleSSO&redirect_url=<orig_req_url>&login_token=<login_token>
  /// resp: 302 <orig_req_url>?login_token=<login_token>
  /// Query String Parameter Names:
  /// redirect_url - is the Ensemble url to which WCI redirects after successful response.
  /// login_token - is the login token that either Ensemble sends in request url to WCI or WCI sends in redirect url after processing to Ensemble.
  /// </summary>
  /// <author>Ravi Tunuguntla</author>
  public class EnsembleHandlerControl : IControl,ILoginControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.sso.ensemble.EnsembleHandlerControl");
    public const String STR_MVC_CLASS_NAME = "EnsembleSSO";
    public const String EN_URL_SSO_SUFIX = "WCIPortalSSO";
    public const String ES_POST_REDIRECT_URL = "redirect_url";
    public const String WCI_TOKEN = "login_token";
    public const String NO_TOKEN_FOUND = "did_not_find_token";
    public const String SSO_ACTION = "sso_action";
    public const String LOGOUT_VALUE = "logout";
    public const String LOGIN_VALUE = "login";
    public const String SSO_STATUS = "sso_status";
    public const String SSO_STATUS_SUCCESS = "sso_logout_success";
    public const String SSO_STATUS_ERR = "sso_logout_error";
    public const String SSO_STATUS_FOUND = "found_token";
    public const String SSO_STATUS_NO = "no_token";
    public const String LOGOUT_RESP_PARAM = "ensemblesso302";
    public const String LOGOUT_RESP_SUCCESS = "sso_logout_success";
    public const String LOGOUT_RESP_ERR = "sso_logout_error";
    
    /// <summary>
    /// This string is used to specify to which url the user has to be redirected after logout 
    /// </summary>
    public const String LOGOUT_REDIRECT_URL = "lourl";
    protected AActivitySpace m_asOwner;
    private bool m_bNewSession = false;
    private IPTSession m_Session = null;
    internal bool bLogout;
    
    /// <summary>
    /// This boolean is used to disable custom SSO solution between Ensemble and WCI 
    /// </summary>
    public const bool FEATURE_DISABLED = true;
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.activityspace.IControl#Execute()"></seealso>
    public virtual Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      //RT - This custom sso feature between Ensemble and WCI is disabled and not supported in NEO release.
      // To achieve SSO between WCI and Ensemble or other WebCenter products like spaces it is suggested to use
      // a generic SSO setup which includes thirdparty SSO solution like OSSO, OAM etc to achieve SSO between WCI
      // and Ensemble. For setting up generic SSO it is suggested to follow WCI documentation.
      // If any one invokes this feature a UI Error message will be displayed saying this feature is disabled and suggesting
      // to use generic SSO solution.
      if (FEATURE_DISABLED)
      {
        RaiseError(m_asOwner.GetString(2011, "ptmsgs_portalbrowsingmsgs"));
        return null;
      }
      Redirect redirect;
      String[] sValues = (String[]) arguments.GetElement(SSO_ACTION);
      // If sso_action is logout then we set logout flag to true
      // else sso_action is to do login.
      if (sValues != null)
      {
        bLogout = (XPStringUtility.CompareIgnoreCase(XPStringUtility.Trim(sValues[0]),
         LOGOUT_VALUE) == 0);
      } else
      {
        RaiseError(SSO_ACTION + " is not in the querystring parameters, So unable to process Ensemble SSO request. ");
        return null;
      }
      // Getting the return url from the ensemble request 
      String strReURL = null;
      sValues = (String[]) arguments.GetElement(ES_POST_REDIRECT_URL);
      // Verify if there is post_redirect_url
      if (sValues != null)
      {
        // URL is encrypted, we need to decrypt it using seed value.
        strReURL = EnsembleHandlerTasks.DecryptUsingSeedToken(XPStringUtility.Trim(sValues[0]));
        log.Info("Decrypted Redirect URL from Ensemble is :" + strReURL);
        } else
      {
        RaiseError(ES_POST_REDIRECT_URL + " is not in the querystring parameters, So unable to redirect SSO request. ");
        return null;
      }
      if (null != strReURL)
      {
        redirect = new Redirect();
        // If it is a logout from ensemble then we just invalidate session
        // and redirect to the external url
        if (bLogout)
        {
          log.Info("Got logout request from Ensemble, Initiating WCI logout");
          // Invalidate entire portal session, by triggering Logout done by portal
          String strFinalURL;
          try
          {
            redirect.SetLinkGetSpaceIfCached(LoginAS.STR_MVC_CLASS_NAME, m_asOwner);
            redirect.SetControl(LoginControl.STR_MVC_CLASS_NAME);
            redirect.AddControlArgument(LoginHelper.PARAM_DOLOGOUT, "true");
            redirect.AddControlArgument(ASConstants.CLEARSESSION, "true");
            strFinalURL = EnsembleHandlerTasks.AddQueryStringParam(strReURL,
             SSO_STATUS, LOGOUT_RESP_SUCCESS);
          }
          catch (XPException ex)
          {
            strFinalURL = EnsembleHandlerTasks.AddQueryStringParam(strReURL,
             SSO_STATUS, LOGOUT_RESP_ERR);
          }
          log.Info("Adding final url to internal redirect that will handle logout and redirection to Ensemble: " + strFinalURL);
          redirect.AddControlArgument(LOGOUT_REDIRECT_URL, strFinalURL);
          return redirect;
        }
        // Setting Redirect				
        redirect.SetIsHTTPRedirect(true);
        log.Info("Setting Redirect to External URL");
        // Verify if SSO is turned on and get token timeout			
        int tokenTimeout = EnsembleHandlerTasks.GetSSOTimeout();
        log.Info("Reading the " + ConfigConstants.INTERNAL_SSO_TOKEN_TIMEOUT + " value to be: " + tokenTimeout);
        // Verifying Token Setting if it is <= 0 then no login token is appended.			
        if (tokenTimeout <= 0)
        {
          redirect.SetLinkToExternalURL(EnsembleHandlerTasks.AddQueryStringParam(strReURL,
           WCI_TOKEN, NO_TOKEN_FOUND));
          log.Info("Token Timeout is 0, So Ensemble SSO is turned off. Redirecting with token value as " + NO_TOKEN_FOUND + " To turn on Ensemble SSO change " + ConfigConstants.INTERNAL_SSO_TOKEN_TIMEOUT + " to a positive value.");
          return redirect;
        }
        IPTSession session = (IPTSession) m_asOwner.GetUserSession();
        log.Info("Current user that portal has session with is: " + session.GetUserInfo().GetFullName());
        // Verifying if user is guest.
        if (!session.GetUser().IsGuestUser())
        {
          log.Info("User has session and is not guest user, using this session");
          String strToken = session.GetToken(tokenTimeout);
          // Base64 encoding the token value
          strToken = XPBase64Coder.Encode(XPStringUtility.GetBytes(strToken));
          // URL Encoding the resulting token value.
          // strToken = XPHttpUtility.UrlEncode(session.GetToken(tokenTimeout), LinkConstants.UTF_8_ENCODING);					
          String strFinalURL = EnsembleHandlerTasks.AddQueryStringParam(strReURL,
           WCI_TOKEN, strToken);
          strFinalURL = EnsembleHandlerTasks.AddQueryStringParam(strFinalURL,
           SSO_STATUS, SSO_STATUS_FOUND);
          redirect.SetLinkToExternalURL(strFinalURL);
        } else
        {
          // User is a guest user and Ensemble is requesting Token, That means 
          // user did not yet login to both Ensemble and Portal. So portal sends response
          // with login_token=did_not_find_token and sso_status=no_token
          //Create redirect.				
          String strFinalURL = EnsembleHandlerTasks.AddQueryStringParam(strReURL,
           WCI_TOKEN, NO_TOKEN_FOUND);
          strFinalURL = EnsembleHandlerTasks.AddQueryStringParam(strFinalURL,
           SSO_STATUS, SSO_STATUS_NO);
          redirect.SetLinkToExternalURL(strFinalURL);
          log.Info("No token available since request came without user loging in. Redirecting with token value as " + NO_TOKEN_FOUND);
          }
      } else
      {
        // Ensemble Redirect url is null. We just throw an error.
        RaiseError(ES_POST_REDIRECT_URL + " is in null, So unable to redirect Ensemble SSO request. ");
        return null;
      }
      return redirect;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new EnsembleHandlerControl();
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.activityspace.IControl#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.activityspace.IControl#Init(IModel,AActivitySpace)"></seealso>
    public virtual void Init(IModel model, AActivitySpace space)
    {
      m_asOwner = space;
    }
    
    /// <summary>
    /// Helper method that throws an UI Error
    /// </summary>
    /// <param name="strError">Error string to be shown in UI.</param>
    public virtual void RaiseError(String strError)
    {
      UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
       strError, strError);
      myError.SetActivitySpace(m_asOwner.GetName());
      myError.SetDisplayOnSeparatePage(false);
      myError.SetClearWithActivitySpaceChange(true);
      myError.SetImmediateClear(true);
      UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
      }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.ILoginControl#DoGetSession()"></seealso>
    public virtual bool DoGetSession()
    {
      return m_bNewSession;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.ILoginControl#GetSession()"></seealso>
    public virtual Object GetSession()
    {
      return m_Session;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.ILoginControl#SetRedirectForUseAfterLogin(Redirect)"></seealso>
    public virtual void SetRedirectForUseAfterLogin(Redirect r)
    {
    }
  }
}