using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portalpages.common.editpage
{
  /// <summary>
  /// This class manages getting and setting portlet columns count for adaptive layouts 
  /// </summary>
  /// <author>rozhang</author>
  public class LayoutColumnStore
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     typeof(LayoutColumnStore));
    //protected static final String APP_CACHEMAN_KEY = "PortletLayoutColumns";
    protected const String CACHEKEY_PREFIX = "LAYOUTCOLENTRY";
    protected const String DISABLE_DRAGDROPLAYOUTEDITOR = "DISABLE_DRAGDROPLAYOUTEDITOR";
    
    /// <summary>
    /// Get number of portlet columns for specified layout (should be a negative value for adaptive layout)
    /// </summary>
    /// <param name="layoutId"></param>
    /// <returns>0 if the adaptive layout for the specified id was not found  </returns>
    public virtual int GetLayoutColumnCount(int layoutId)
    {
      CachingManager cacheMan = GetCacheStore();
      int storeCount = LookupColumnCount(cacheMan, layoutId);
      return storeCount;
    }
    
    /// <summary>
    /// Helper method to get container to store layout id/column count pairs 
    /// </summary>
    /// <returns></returns>
    protected virtual CachingManager GetCacheStore()
    {
      IApplication app = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
      CachingManager cacheMan = app.GetCachingManager();
      return cacheMan;
    }
    
    /// <summary>
    /// Create cache key from layout id
    /// </summary>
    protected virtual String CreateCacheKey(int layoutId)
    {
      return CACHEKEY_PREFIX + layoutId;
    }
    
    /// <summary>
    /// Gets the number of columns assigned to specified layout id
    /// </summary>
    /// <param name="colStore"></param>
    /// <param name="layoutId"></param>
    /// <returns>0 if the layout was not found</returns>
    protected virtual int LookupColumnCount(CachingManager colStore, int layoutId)
    {
      Object val = colStore.GetEntry(CreateCacheKey(layoutId));
      if (val != null && val is int)
      {
        return XPConvert.ToInteger(val);
      } else
      {
        return 0;
      }
    }
    
    /// <summary>
    /// Updates column count for specified layout. New value is stored only
    /// if column count for layout is not stored before or if stored value 
    /// is different from new value 
    /// </summary>
    /// <param name="layoutId"></param>
    /// <param name="colCount"></param>
    /// <returns>true if column count was updated, false if the value already was set</returns>
    public virtual bool SetLayoutColumnCount(int layoutId, int newCount)
    {
      CachingManager cache = GetCacheStore();
      int storedCount = LookupColumnCount(cache, layoutId);
      if (storedCount == newCount)
      {
        // don't do anything, no need to update
        return false;
      } else
      {
        cache.SetEntry(CreateCacheKey(layoutId), newCount);
        return true;
      }
    }
    
    /// <summary>
    /// Sets flag on Application to disable Portlet layout editor used for IE browsers
    /// Called by LayoutMetaDataTag to resolve bug 9862302. Made optional for existing 
    /// customers who want to keep the drag and drop layout editor 
    /// </summary>
    public virtual void SetDisableDragDropLayoutEditor()
    {
      CachingManager cache = GetCacheStore();
      cache.SetEntry(DISABLE_DRAGDROPLAYOUTEDITOR, "true");
    }
    
    /// <summary>
    /// Checks whether drag drop layout editor for IE is disabled or not
    /// </summary>
    /// <returns></returns>
    public virtual bool IsDragDropLayoutEditorDisabled()
    {
      CachingManager cache = GetCacheStore();
      return (null != cache.GetEntry(DISABLE_DRAGDROPLAYOUTEDITOR));
    }
  }
}