using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.portalpages.common.helpers
{
  /// <author>
  /// adolfoc
  /// This is a simple class used to help translate between Durations used by the
  /// PTDuractionCombo construct and Milliseconds. Although this class uses a long
  /// for the internal representation of the duration, it is possible that certain
  /// methods will incorrectly handle durations that are more the int milliseconds
  /// long (roughly 22 days). This class does not check for overruns.
  /// </author>
  public class PTDuration
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
     "com.plumtree.portalpages.common.helpers.PTDuration");
    protected int m_nMASK_PossibleUnits;
    protected long m_lMilliseconds = 0;
    
    /// <summary>
    /// Constructor. Takes in a Duration and Duration Unit.
    /// The unit must be one of the UNITS defined in PTDurationCombo.
    /// </summary>
    /// <param name="lDuration"></param>
    /// <param name="nDurationUnits"></param>
    public PTDuration(long lDuration, int nDurationUnits, int nMASKPossibleUnits)
    {
      m_nMASK_PossibleUnits = PTDurationCombo.DISPLAYSECONDS | PTDurationCombo.DISPLAYMINUTES;
      m_lMilliseconds = PTDurationCombo.DurationToMilliseconds(lDuration,
       nDurationUnits);
      m_nMASK_PossibleUnits = nMASKPossibleUnits;
      if (0 == (nDurationUnits & nMASKPossibleUnits))
      {
        log.Error("PTDuration::PTDuration: Error - Default Unit passed in not one of the nMASKPossibleUnits. Adding to mask of possible units.");
        m_nMASK_PossibleUnits|=nDurationUnits;
      }
    }
    
    /// <summary>
    /// Constructor. Takes in the milliseconds.
    /// </summary>
    /// <param name="lMilliseconds"></param>
    public PTDuration(long lMilliseconds, int nMASKPossibleUnits)
    {
      m_nMASK_PossibleUnits = PTDurationCombo.DISPLAYSECONDS | PTDurationCombo.DISPLAYMINUTES;
      m_lMilliseconds = lMilliseconds;
      m_nMASK_PossibleUnits = nMASKPossibleUnits;
    }
    
    /// <summary>
    /// Returns the best duration unit possible from the MASK of available
    /// durations. If the current duration is zero or does not match any
    /// available durations, this will return nDefaultDuration units.
    /// </summary>
    /// <param name="nMASKSupportedUnits"></param>
    /// <param name="nDefaultUnits"></param>
    /// <returns>int</returns>
    public virtual int GetBestDurationUnit(int nDefaultUnits)
    {
      if (0 != m_lMilliseconds)
      {
        int nBestUnits = PTDurationCombo.BestDurationTypeForMilliseconds(m_nMASK_PossibleUnits,
         m_lMilliseconds);
        if (0 != nBestUnits)
        {
          return nBestUnits;
        }
      }
      return nDefaultUnits;
    }
    
    /// <summary>
    /// Returns the best duration unit possible from the MASK of available
    /// durations. If the current duration is zero or does not match any
    /// available durations, this will return the shortest supported Unit.
    /// </summary>
    /// <param name="nMASKSupportedUnits"></param>
    /// <param name="nDefaultUnits"></param>
    /// <returns>int</returns>
    public virtual int GetBestDurationUnit()
    {
      if (0 != m_lMilliseconds)
      {
        int nBestUnits = PTDurationCombo.BestDurationTypeForMilliseconds(m_nMASK_PossibleUnits,
         m_lMilliseconds);
        if (0 != nBestUnits)
        {
          return nBestUnits;
        }
      }
      return GetShortestSupportedUnit(m_nMASK_PossibleUnits);
    }
    
    /// <summary>
    /// Returns the shortest time unit described in nMASK
    /// </summary>
    /// <param name="nDurationUnits"></param>
    /// <returns>int</returns>
    public static int GetShortestSupportedUnit(int nMASKSupportedUnits)
    {
      if (0 != (PTDurationCombo.DISPLAYMILLISECONDS & nMASKSupportedUnits))
      {
        return PTDurationCombo.DISPLAYMILLISECONDS;
      }
      if (0 != (PTDurationCombo.DISPLAYSECONDS & nMASKSupportedUnits))
      {
        return PTDurationCombo.DISPLAYSECONDS;
      }
      if (0 != (PTDurationCombo.DISPLAYMINUTES & nMASKSupportedUnits))
      {
        return PTDurationCombo.DISPLAYMINUTES;
      }
      if (0 != (PTDurationCombo.DISPLAYHOURS & nMASKSupportedUnits))
      {
        return PTDurationCombo.DISPLAYHOURS;
      }
      if (0 != (PTDurationCombo.DISPLAYDAYS & nMASKSupportedUnits))
      {
        return PTDurationCombo.DISPLAYDAYS;
      }
      if (0 != (PTDurationCombo.DISPLAYWEEKS & nMASKSupportedUnits))
      {
        return PTDurationCombo.DISPLAYWEEKS;
      }
      if (0 != (PTDurationCombo.DISPLAYMONTHS & nMASKSupportedUnits))
      {
        return PTDurationCombo.DISPLAYMONTHS;
      }
      if (0 != (PTDurationCombo.DISPLAYYEARS & nMASKSupportedUnits))
      {
        return PTDurationCombo.DISPLAYYEARS;
      }
      log.Error("PTDuration::GetShortestSupportedUnit: Error - nMASKPossibleUnits - does not contain any supported units. Returning zero.");
      return 0;
    }
    
    /// <summary>
    /// Gets the duration in the longest supported units of time.
    /// (You can get this duration by calling GetBestDurationUnit).
    /// If the duration does not evenly divide into the supported units,
    /// this will return the duration in the shortest supported units.
    /// Notice that this returns an integer.
    /// </summary>
    /// <param name="nDurationUnits"></param>
    /// <returns>int</returns>
    public virtual int GetDurationInBestUnits()
    {
      return GetDurationInUnits(GetBestDurationUnit());
    }
    
    /// <summary>
    /// Gets the duration in the longest supported units of time or
    /// in nDefaultUnits if the duration does not equally divide into the
    /// supported units.
    /// Notice that this returns an integer.
    /// </summary>
    /// <param name="nDurationUnits"></param>
    /// <returns>int</returns>
    public virtual int GetDurationInBestUnits(int nDefaultUnits)
    {
      return GetDurationInUnits(GetBestDurationUnit(nDefaultUnits));
    }
    
    /// <summary>
    /// Gets the duration in DurationUnits. Returns the current duration in the
    /// units specified. If the units do not divide equally, the duration is
    /// integer divided (read floored) to the specified unit.
    /// Notice that this returns an integer.
    /// </summary>
    /// <param name="nDurationUnits"></param>
    /// <returns>int</returns>
    public virtual int GetDurationInUnits(int nDurationUnits)
    {
      return XPConvert.ToInteger(XPConvert.ToString(PTDurationCombo.DurationFromMilliseconds(m_lMilliseconds,
       nDurationUnits)));
    }
    
    /// <summary>
    /// Returns the duration in milliseconds.
    /// Notice that this returns an integer.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetMilliseconds()
    {
      return XPConvert.ToInteger(XPConvert.ToString(m_lMilliseconds));
    }
    
    /// <summary>
    /// Returns the mask of supported units.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetSupportedUnitsMask()
    {
      return m_nMASK_PossibleUnits;
    }
  }
}