using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.myportal;
using com.plumtree.portalpages.browsing.portalsettings;
using com.plumtree.portalpages.browsing.sitemap.sitemappage;
using com.plumtree.portaluiinfrastructure.compoundlist;
using com.plumtree.portaluiinfrastructure.login;
using com.plumtree.portaluiinfrastructure.opener;
using com.plumtree.portaluiinfrastructure.ptlink;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.taskapi.portalui;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.common.mediator
{
  /// <summary>
  /// NOTE: Use CListURLFullLinkMediator to get HTMLAnchors, this class
  /// might get changed in the future.
  /// Generates HTMLAnchors with target URLs in href, display string
  /// and image(s) if assigned. 
  /// This class is similar to CListURLTemplateMediator, except that the HREF 
  /// is used to go to the URL instead of onlick evens. Hence, it is 508 compatible
  /// since no browser Javascript capability is required.
  /// Example use:
  /// ICPListIterator mediator = new CListURLFullMediator(m_asOwner, cpList1);
  /// mediator.SetLabelMaxLength(50);    //set the maximum label length to 50 characters
  /// while (mediator.Next())
  /// {
  /// HTMLAnchor anchor = (HTMLAnchor) mediator.GetEntry();
  /// ...
  /// }
  /// </summary>
  /// <author>robertz</author>
  public class CListURLFullMediator : ICPListIterator
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
     "com.plumtree.portalpages.common.mediator.CListURLFullMediator");
    private AActivitySpace m_aspace;
    private ICPListEntryIterator m_list;
    private String m_strOpenDocInNewWindow;
    // The constant amount of space in pixels images should take up.
    // A spacer image will be added as padding if images doesn't fill
    // up the entire space.  
    public const int IMAGE_SPACE_WIDTH = 24;
    public const int ACTIONIMAGE_SPACE_WIDTH = 30;
    // minimum image sizes; overriden by entry settings if larger
    private int m_nLabelMaxLength = 20;
    private bool m_bShowImages = true;
    
    /// <summary>
    /// Constructor for CompoundListASURLMediator. Get the iterator interface for the list
    /// </summary>
    /// <param name="aspace">The current activityspace</param>
    /// <param name="list">The ASCompoundlist for this mediator</param>
    public CListURLFullMediator(AActivitySpace aspace, ICPListEntryIterator list)
    {
      m_list = list;
      m_aspace = aspace;
      list.GetIterator();
      // DCA - figure out if user wants links to open in a new window or in the main window
      m_strOpenDocInNewWindow = "0";
      try
      {
        Object objSetting = PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.OPEN_DOC_NEW_WINDOW,
         m_aspace);
        if (null != objSetting)
        {
          m_strOpenDocInNewWindow = XPConvert.ToString(objSetting);
        }
      }
      catch (Exception e)
      {
        // Setting has not been set, log as warning.
        log.Warn(e, "Document preference has not been set for this user, documents will be opened on the main window.");
        }
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#GetEntry()"></seealso>
    /// <returns>
    /// 						HTMLAnchor with target url as href target. 
    /// Also returns HTMLAnchors with null as href target, which
    /// will only generate a plain HTML string without the anchor.
    /// </returns>
    public virtual Object GetEntry()
    {
      ListEntry entry = (ListEntry) m_list.GetEntry();
      return GetURLFromListEntry(entry);
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#GetEntryAtIndex(int)"></seealso>
    public virtual Object GetEntryAtIndex(int nIndex)
    {
      ListEntry entry = (ListEntry) m_list.GetEntryAtIndex(nIndex);
      return GetURLFromListEntry(entry);
    }
    
    /// <summary>
    /// Returns the entry type of the current listentry
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetEntryType()
    {
      ListEntry entry = (ListEntry) m_list.GetEntry();
      return entry.GetType();
    }
    
    /// <summary>
    /// Returns the entry type of the specified index
    /// </summary>
    /// <param name="nIndex">Index of the listentry</param>
    /// <returns>					Type of the specified entry</returns>
    public virtual int GetEntryTypeAtIndex(int nIndex)
    {
      ListEntry entry = (ListEntry) m_list.GetEntryAtIndex(nIndex);
      return entry.GetType();
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#GetList()"></seealso>
    public virtual ICPListEntryIterator GetList()
    {
      return m_list;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#Next()"></seealso>
    public virtual bool Next()
    {
      return m_list.Next();
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#SetLabelMaxLength(int)"></seealso>
    public virtual void SetLabelMaxLength(int nMax)
    {
      m_nLabelMaxLength = nMax;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#SetShowImages(boolean)"></seealso>
    public virtual void SetShowImages(bool bShow)
    {
      m_bShowImages = bShow;
    }
    
    /// <summary>
    /// Creates an ASURL to a Community page
    /// </summary>
    /// <param name="entry">Current listentry object</param>
    /// <returns>							ASURL to the target URL</returns>
    protected virtual HTMLAnchor CreateCommPageASURL(ListEntry entry)
    {
      String label = entry.GetLabel();
      int nCommID = entry.GetCommunityID();
      int nPageID = entry.GetPageID();
      int nProfileID = entry.GetProfileID();
      HTMLAnchor anchor = new HTMLAnchor("");
      try
      {
        IPTURL url = TaskAPIUICommunity.CreateCommunityPageCommonOpenerURL(m_aspace,
         nCommID, nPageID, label);
        if (-1 != nProfileID)
        {
          url.AddControlParameter(SetCommunityControl.PROFILEID, nProfileID);
          }
        //			asURL.SetLinkGetSpaceIfCached(CommunityPageAS.STR_MVC_CLASS_NAME, m_aspace);
        //			asURL.SetControl(SetCommunityControl.STR_MVC_CLASS_NAME);
        //			asURL.AddControlArgument(SetCommunityControl.STR_COMMUNITY_ID, XPConvert.ToString(nCommID));
        //			asURL.AddControlArgument(SetCommunityControl.STR_PAGE_ID, XPConvert.ToString(nPageID));
        //			if ( -1 != nProfileID ) {
        //				asURL.AddControlArgument(SetCommunityControl.PROFILEID, nProfileID);
        //			}
        anchor.SetHRef(url.GetFullURL());
        SetImage(anchor, entry);
        anchor.AddInnerHTMLString(label);
      }
      catch (HTMLException he)
      {
        log.Error("Error adding String to ASURL");
      }
      return anchor;
    }
    
    /// <summary>
    /// Creates an ASURL to a Community Knowledge Directory Root page
    /// </summary>
    /// <param name="entry">Current listentry object</param>
    /// <returns>							HTMLAnchor linking to the target URL in the href</returns>
    protected virtual ASURL CreateSiteMapRootASURL(ListEntry entry)
    {
      String label = entry.GetLabel();
      int nCommID = entry.GetCommunityID();
      int nMode = entry.GetMode();
      bool bIsPreview = entry.GetIsPreview();
      ASURL asURL = new ASURL(m_aspace);
      try
      {
        asURL.SetLinkCreateNewSpace(SiteMapAS.STR_MVC_CLASS_NAME, m_aspace);
        asURL.SetControl(SetSiteMapControl.STR_MVC_CLASS_NAME);
        asURL.AddControlArgument(SetSiteMapControl.m_CommunityID, nCommID);
        asURL.AddControlArgument(SetSiteMapControl.m_Mode, nMode);
        if (bIsPreview)
        {
          asURL.AddControlArgument(SetSiteMapControl.PREVIEW);
        }
        SetImage(asURL, entry);
        asURL.AddInnerHTMLString(label);
      }
      catch (HTMLException he)
      {
        log.Error("Error adding String to ASURL");
      }
      return asURL;
    }
    
    /// <summary>
    /// Creates an ASURL to a directory page
    /// </summary>
    /// <param name="entry">Current listentry object</param>
    /// <returns>							ASURL to the target URL</returns>
    protected virtual HTMLAnchor CreateDirPageASURL(ListEntry entry)
    {
      String label = entry.GetLabel();
      int nMode = entry.GetMode();
      int nFolderID = entry.GetFolderID();
      HTMLAnchor anchor = new HTMLAnchor("");
      try
      {
        String strURL = TaskAPIUIDirectory.CreateCommonOpenerDirectoryFolderURL(m_aspace,
         label, nMode, nFolderID).GetFullURL();
        anchor.SetHRef(strURL);
        SetImage(anchor, entry);
        anchor.AddInnerHTMLString(label);
      }
      catch (HTMLException he)
      {
        log.Error("Error adding String to ASURL");
      }
      return anchor;
    }
    
    /// <summary>
    /// Creates an HTMLAnchor to a directory document page
    /// </summary>
    /// <param name="entry">Current listentry object</param>
    /// <returns>							HTMLAnchor linking to the target URL in the href</returns>
    protected virtual HTMLAnchor CreateDocHTMLAnchor(ListEntry entry)
    {
      String label = entry.GetLabel();
      int nDocID = entry.GetPageID();
      HTMLAnchor anchor = new HTMLAnchor((PTOpenerLinks.GetOpenerURLOpenObjID(PT_CLASSIDS.PT_CATALOGCARD_ID,
       nDocID, IPTOpenerConstants.OPENER_MODE_VIEW, m_aspace)));
      SetImage(anchor, entry);
      anchor.AddInnerHTMLString(label);
      if (null != m_strOpenDocInNewWindow && m_strOpenDocInNewWindow.Equals("1"))
      {
        anchor.SetTarget("_blank");
      }
      return anchor;
    }
    
    /// <summary>
    /// Creates an ASURL to an editor page
    /// </summary>
    /// <param name="entry">Current listentry object</param>
    /// <returns>							HTMLAnchor linking to the target URL in the href</returns>
    protected virtual ASURL CreateEditorASURL(ListEntry entry)
    {
      String label = entry.GetLabel();
      String editorSpace = entry.GetEditorAS();
      String control = entry.GetControl();
      String strPage = entry.GetPage();
      String strEditorType = entry.GetEditorType();
      String[] paramNames = entry.GetParamNames();
      String[] paramValues = entry.GetParamValues();
      ASURL asURL = new ASURL(m_aspace);
      try
      {
        asURL.SetLinkCreateNewSpace(editorSpace, m_aspace);
        if (null != control)
        {
          asURL.SetControl(control);
        }
        if (null != strPage)
        {
          asURL.SetDisplayPage(strPage);
        }
        if (null != strEditorType)
        {
          asURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE, strEditorType);
          }
        if (null != paramNames)
        {
          for (int i = 0;i < paramNames.Length;i++)
          {
            asURL.AddControlArgument(paramNames[i], paramValues[i]);
          }
        }
        SetImage(asURL, entry);
        asURL.AddInnerHTMLString(label);
      }
      catch (Exception he)
      {
        log.Error("Error adding String to ASURL");
      }
      return asURL;
    }
    
    /// <summary>
    /// Creates an HTMLAnchor that links to a portal URL
    /// </summary>
    /// <param name="entry"></param>
    /// <returns></returns>
    protected virtual HTMLAnchor CreateFullInternalURLHTMLAnchor(ListEntry entry)
    {
      String label = entry.GetLabel();
      String url = entry.GetFullURL();
      HTMLAnchor anchor = new HTMLAnchor(url);
      SetImage(anchor, entry);
      anchor.AddInnerHTMLString(label);
      return anchor;
    }
    
    /// <summary>
    /// Creates an HTMLAnchor that links to an external URL
    /// </summary>
    /// <param name="entry">Current listentry object</param>
    /// <returns>							HTMLAnchor linking to the target URL in the href</returns>
    protected virtual HTMLAnchor CreateFullURLHTMLAnchor(ListEntry entry)
    {
      String label = entry.GetLabel();
      String url = entry.GetFullURL();
      HTMLAnchor anchor = new HTMLAnchor(url);
      SetImage(anchor, entry);
      anchor.AddInnerHTMLString(label);
      if (null != m_strOpenDocInNewWindow && m_strOpenDocInNewWindow.Equals("1"))
      {
        anchor.SetTarget("_blank");
      }
      return anchor;
    }
    
    /// <summary>
    /// Creates an ASURL for generic Portal 50 pages
    /// </summary>
    /// <param name="entry">Current listentry object</param>
    /// <returns>							ASURL to the target URL</returns>
    protected virtual ASURL CreateGenericASURL(ListEntry entry)
    {
      ASURL asURL = new ASURL(m_aspace);
      String label = entry.GetLabel();
      String strAS = entry.GetEditorAS();
      String[] paramNames = entry.GetParamNames();
      String[] paramValues = entry.GetParamValues();
      int nNewASType = entry.GetNewASType();
      switch (nNewASType){
      case ASCompoundList.NEWASTYPE_CREATENEW:
      asURL.SetLinkCreateNewSpace(strAS, m_aspace);
      break;
      case ASCompoundList.NEWASTYPE_GETIFCACHED:
      asURL.SetLinkGetSpaceIfCached(strAS, m_aspace);
      break;
      }
      if (null != paramNames)
      {
        for (int i = 0;i < paramNames.Length;i++)
        {
          asURL.AddControlArgument(paramNames[i], paramValues[i]);
        }
      }
      SetImage(asURL, entry);
      asURL.AddInnerHTMLString(label);
      return asURL;
    }
    
    /// <summary>
    /// Creatse ASURL to a mypage
    /// </summary>
    /// <param name="entry">Current listentry object</param>
    /// <returns>							ASURL to the target URL</returns>
    protected virtual HTMLAnchor CreateMyPageASURL(ListEntry entry)
    {
      String label = entry.GetLabel();
      int nPageID = entry.GetPageID();
      String strURL = null;
      HTMLAnchor anchor = new HTMLAnchor("");
      try
      {
        strURL = TaskAPIUIMyPages.CreateMyPageCommonOpenerURL(m_aspace, nPageID,
         label).GetFullURL();
        //			asURL.SetLinkGetSpaceIfCached(MyPageAS.STR_MVC_CLASS_NAME, m_aspace);
        //			asURL.SetControl(SetPageControl.STR_MVC_CLASS_NAME);
        //			asURL.AddControlArgument(SetPageControl.STR_PAGE_ID,
        //				XPConvert.ToString(nPageID));
        SetImage(anchor, entry);
        //			
        anchor.SetHRef(strURL);
        anchor.AddInnerHTMLString(label);
      }
      catch (HTMLException he)
      {
        log.Error("Error adding String to ASURL");
      }
      return anchor;
    }
    
    /// <summary>
    /// Creates ASURL to a user preference page
    /// </summary>
    /// <param name="entry">Current listentry object</param>
    /// <returns>							ASURL to the target URL</returns>
    protected virtual ASURL CreatePrefsASURL(ListEntry entry)
    {
      ASURL asURL = new ASURL(m_aspace);
      String label = entry.GetLabel();
      String strPage = entry.GetPage();
      try
      {
        asURL.SetLinkGetSpaceIfCached(PortalSettingsAS.STR_MVC_CLASS_NAME,
         m_aspace);
        asURL.SetDisplayPage(strPage);
        SetImage(asURL, entry);
        asURL.AddInnerHTMLString(label);
      }
      catch (HTMLException he)
      {
        log.Error("Error adding String to ASURL");
      }
      return asURL;
    }
    
    /// <summary>
    /// Creates an ASURL with just a label, no URL
    /// </summary>
    /// <param name="entry">Current listentry object</param>
    /// <returns>							HTMLAnchor linking to the target URL in the href</returns>
    protected virtual ASURL CreateStringLabelASURL(ListEntry entry)
    {
      ASURL asURL = new ASURL(m_aspace);
      String label = entry.GetLabel();
      SetImage(asURL, entry);
      asURL.SetNoLinkLabelString(label);
      return asURL;
    }
    
    /// <summary>
    /// Creates an HTMLAnchor to a user profile page
    /// </summary>
    /// <param name="entry">Current listentry object</param>
    /// <returns>							HTMLAnchor linking to the target URL in the href</returns>
    protected virtual HTMLAnchor CreateUserHTMLAnchor(ListEntry entry)
    {
      String label = entry.GetLabel();
      int nUserID = entry.GetPageID();
      HTMLAnchor anchor = new HTMLAnchor(PTOpenerLinks.GetOpenerURLOpenObjID(PT_CLASSIDS.PT_USER_ID,
       nUserID, IPTOpenerConstants.OPENER_MODE_VIEW, m_aspace));
      SetImage(anchor, entry);
      anchor.AddInnerHTMLString(label);
      return anchor;
    }
    
    /// <summary>
    /// Set image(s) on the ASURL from the assigned images in the Listentry object.
    /// </summary>
    /// <param name="url"></param>
    /// <param name="entry"></param>
    protected virtual void SetImage(ASURL url, ListEntry entry)
    {
      PTImageType image = entry.GetImage();
      PTImageType image2 = entry.GetImage2();
      /*	String strImageHeight = entry.GetImageHeight();
       String strImageWidth  = entry.GetImageWidth();
       
       */
       
      if (null == image || !m_bShowImages)
      {
        return;
      }
      try
      {
        HTMLImg htmlImg = null;
        htmlImg = (HTMLImg) UIConstructFactory.GetConstruct(image, true, this.m_aspace).GetHTMLElement();
        htmlImg.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_aspace));
        htmlImg.SetAlt(entry.GetImageAltTextEncoded());
        htmlImg.SetAlign(CommonHTMLStrings.TOP);
        htmlImg.SetBorder(CommonHTMLStrings.ZERO);
        url.AddInnerHTMLElement(htmlImg);
        if (null != image2)
        {
          htmlImg = (HTMLImg) UIConstructFactory.GetConstruct(image2, true,
           this.m_aspace).GetHTMLElement();
          htmlImg.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_aspace));
          htmlImg.SetAlt(entry.GetImageAltTextEncoded());
          htmlImg.SetAlign(CommonHTMLStrings.TOP);
          htmlImg.SetBorder(CommonHTMLStrings.ZERO);
          url.AddInnerHTMLElement(htmlImg);
        }
        //anchor.SetTitle(entry.GetImageAltTextEncoded());
        url.SetLinkTitle(entry.GetImageAltTextEncoded());
      }
      catch (Exception e)
      {
        log.Error(e, "Exception forming HTMLImg component of editor link: " + e.Message);
        }
    }
    
    /// <summary>
    /// Sets the image(s) on the HTMLAnchor from the Listentry object.
    /// Adds spacer images to get straight marigin between images and text 
    /// </summary>
    /// <param name="anchor">The HTMLAnchor</param>
    /// <param name="entry">The Listentry object</param>
    protected virtual void SetImage(HTMLAnchor anchor, ListEntry entry)
    {
      PTImageType image = entry.GetImage();
      PTImageType image2 = entry.GetImage2();
      int nTotalWidth = 0;
      if (null == image || !m_bShowImages)
      {
        return;
      }
      try
      {
        HTMLImg htmlImg = null;
        htmlImg = (HTMLImg) UIConstructFactory.GetConstruct(image, true, this.m_aspace).GetHTMLElement();
        nTotalWidth+=XPConvert.ToInteger(htmlImg.GetWidth());
        htmlImg.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_aspace));
        htmlImg.SetAlt(entry.GetImageAltTextEncoded());
        htmlImg.SetAlign(CommonHTMLStrings.TOP);
        htmlImg.SetBorder(CommonHTMLStrings.ZERO);
        anchor.AddInnerHTMLElement(htmlImg);
        anchor.SetTitle(entry.GetImageAltTextEncoded());
        if (null != image2)
        {
          htmlImg = (HTMLImg) UIConstructFactory.GetConstruct(image2, true,
           this.m_aspace).GetHTMLElement();
          nTotalWidth+=XPConvert.ToInteger(htmlImg.GetWidth());
          htmlImg.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_aspace));
          htmlImg.SetAlt(entry.GetImageAltTextEncoded());
          htmlImg.SetAlign(CommonHTMLStrings.TOP);
          htmlImg.SetBorder(CommonHTMLStrings.ZERO);
          anchor.AddInnerHTMLElement(htmlImg);
        }
        //anchor.SetTitle(entry.GetImageAltTextEncoded());
        if (nTotalWidth < IMAGE_SPACE_WIDTH)
        {
          htmlImg = PlumtreeHelpers.GetSpacerImage(XPConvert.ToString(IMAGE_SPACE_WIDTH - nTotalWidth),
           "1");
          htmlImg.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_aspace));
          anchor.AddInnerHTMLElement(htmlImg);
        } else
        if (nTotalWidth >= IMAGE_SPACE_WIDTH)
        {
          htmlImg = PlumtreeHelpers.GetSpacerImage(XPConvert.ToString(ACTIONIMAGE_SPACE_WIDTH - nTotalWidth),
           "1");
          htmlImg.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_aspace));
          anchor.AddInnerHTMLElement(htmlImg);
        }
      }
      catch (Exception e)
      {
        log.Warn(e, "Exception forming HTMLImg component of editor link: " + e.Message);
        }
    }
    
    /// <summary>
    /// Generates HTMLAnchors with the URL in the href by delegating Listentries to the respective method
    /// that will create the HTMLAnchor based on the URL type
    /// </summary>
    /// <param name="entry">The current listentry object</param>
    /// <returns>						HTMLAnchor with the full url in the href, name and image if assigned</returns>
    private HTMLElement GetURLFromListEntry(ListEntry entry)
    {
      entry.SetTruncationLength(m_nLabelMaxLength);
      HTMLElement url = null;
      switch (entry.GetType()){
      case ASCompoundList.ENTRYTYPE_MYPAGETEMPL:
      url = CreateMyPageASURL(entry);
      break;
      case ASCompoundList.ENTRYTYPE_COMMPTEMPL:
      url = CreateCommPageASURL(entry);
      break;
      case ASCompoundList.ENTRYTYPE_DIRTEMPL:
      url = CreateDirPageASURL(entry);
      break;
      case ASCompoundList.ENTRYTYPE_EDITORTEMPL:
      url = CreateEditorASURL(entry).GetURLAsHTMLElement();
      break;
      case ASCompoundList.ENTRYTYPE_STRINGLABEL:
      url = CreateStringLabelASURL(entry).GetURLAsHTMLElement();
      break;
      case ASCompoundList.ENTRYTYPE_FULLINTERNALURL:
      url = CreateFullInternalURLHTMLAnchor(entry);
      break;
      case ASCompoundList.ENTRYTYPE_FULLURL:
      url = CreateFullURLHTMLAnchor(entry);
      break;
      case ASCompoundList.ENTRYTYPE_PREFSTEMPL:
      url = CreatePrefsASURL(entry).GetURLAsHTMLElement();
      break;
      case ASCompoundList.ENTRYTYPE_DOCTEMPL:
      // using PageID for DocID
      url = CreateDocHTMLAnchor(entry);
      break;
      case ASCompoundList.ENTRYTYPE_USERTEMPL:
      // using PageID from UserID
      url = CreateUserHTMLAnchor(entry);
      break;
      case ASCompoundList.ENTRYTYPE_GENERICTEMPL:
      url = CreateGenericASURL(entry).GetURLAsHTMLElement();
      break;
      case ASCompoundList.ENTRYTYPE_SITEMAPROOTTEMPL:
      url = CreateSiteMapRootASURL(entry).GetURLAsHTMLElement();
      break;
      default:
      log.Error("Entry created with unsupported link type.");
      break;
      }
      return url;
    }
  }
}