using System;
using com.plumtree.portaluiinfrastructure.compoundlist;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
namespace com.plumtree.portalpages.common.mediator
{
  /// <summary>
  /// This class uses delegation to pass on link generation requests to the
  /// appropriate link mediator.  If in 508 or low bandwidth access style, the
  /// class delegates to CListURLFullMediator.  If not, it delegates to
  /// CListURLTemplateLinkMediator.
  /// This class is not a real CListURLMediator in itself. It dynamically decides 
  /// which CListURLMediator to delegate to given the Display Option from the HTML
  /// source.
  /// From a practical point of view, use the Model methods to get an 
  /// ICPListEntryIterator appropriate for the navigation you're constructing, 
  /// and then use a mediator to quickly form HTMLAnchors for those links. 
  /// </summary>
  /// <author>DavidP</author>
  public class CListURLMediator : ICPListIterator
  {
    
    /// <summary>
    /// copy of the AActivitySpace reference; used to determine access style 
    /// </summary>
    private AActivitySpace m_aspace;
    private ICPListIterator m_linkDelegator;
    // Never gets used - private boolean m_bShowImages = true;
    
    /// <summary>
    /// Constructor for CompoundListASURLMediator. Get the iterator interface for the list
    /// </summary>
    /// <param name="aspace">Activity space the links will be generated for</param>
    /// <param name="list">List of "link entries" used to create anchors</param>
    public CListURLMediator(AActivitySpace aspace, ICPListEntryIterator list)
    {
      m_aspace = aspace;
      // instantiate the appropriate delegate
      if (m_aspace.GetAccessStyle() == AccessStyles.STANDARD)
      {
        // javascript-enabled template-based mediator
        m_linkDelegator = (ICPListIterator) (new CListURLTemplateLinkMediator(aspace,
         list));
      } else
      {
        // full URL, no link templating
        m_linkDelegator = (ICPListIterator) (new CListURLFullLinkMediator(aspace,
         list));
      }
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#GetEntry()"></seealso>
    public virtual Object GetEntry()
    {
      //VerifyAccess();
      return m_linkDelegator.GetEntry();
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#GetEntryAtIndex(int)"></seealso>
    public virtual Object GetEntryAtIndex(int nIndex)
    {
      //VerifyAccess();
      return m_linkDelegator.GetEntryAtIndex(nIndex);
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#GetList()"></seealso>
    public virtual ICPListEntryIterator GetList()
    {
      return m_linkDelegator.GetList();
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#Next()"></seealso>
    public virtual bool Next()
    {
      //VerifyAccess();
      return m_linkDelegator.Next();
    }
    /* (non-Javadoc)
     * @see com.plumtree.portalpages.common.mediator.ICPListIterator#SetLabelMaxLength(int)
     */
     
    public virtual void SetLabelMaxLength(int nMax)
    {
      //VerifyAccess();
      m_linkDelegator.SetLabelMaxLength(nMax);
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#SetShowImages(boolean)"></seealso>
    public virtual void SetShowImages(bool bShow)
    {
      m_linkDelegator.SetShowImages(bShow);
    }
  }
}
//	/**
//	 * Method VerifyAccess.  Swaps the underlying delegate as needed, sharing
//	 * the original list.
//	 * DEPRECATED: it's unlikely the mode will change during the lifetime of
//	 * this helper instance.
//	 */
//	private void VerifyAccess()
//	{
//		// Switch delegates on the fly as needed
//		if (b_is508 && (m_aspace.GetAccessStyle() == AccessStyles.STANDARD))
//		{
//			ICPListEntryIterator list = linkDelegator.GetList();
//			linkDelegator     = null;
//			linkDelegator     = (ICPListIterator) (new CListURLTemplateLinkMediator(m_aspace,
//					list));
//			b_is508			  = false;
//		}
//		else if (!b_is508 &&
//				(m_aspace.GetAccessStyle() != AccessStyles.STANDARD))
//		{
//			ICPListEntryIterator list = linkDelegator.GetList();
//			linkDelegator     = null;
//			linkDelegator     = (ICPListIterator) (new CListURLFullLinkMediator(m_aspace,
//					list));
//			b_is508			  = true;
//		}
//	}
