using System;
using com.plumtree.openlog;
using com.plumtree.portalpages.common.uiparts;
using com.plumtree.portaluiinfrastructure.compoundlist;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.xpshared.htmlelements;
using com.plumtree.openfoundation.util;
namespace com.plumtree.portalpages.common.mediator
{
  /// <summary>
  /// This class extends CListURLTemplateMediator generating HTMLAnchors using URL Templates.
  /// Clicking on the URL will construct the full URL and redirect to it. The downside with these
  /// links is users can't open the link in a new window with "Open in New Window" option since
  /// a javascript onclick event is used to trigger the redirect and not the HREF. See javadoc
  /// for CListURLTemplateMediator for more information on URL Templates.
  /// </summary>
  /// <seealso cref="CListURLTemplateMediator">
  /// Example use:
  /// ICPListIterator mediator = new CListURLTemplateLinkMediator(m_asOwner, cpList1);
  /// mediator.SetLabelMaxLength(50);    //set the maximum label length to 50 characters
  /// while (mediator.Next())
  /// {
  /// HTMLAnchor anchor = (HTMLAnchor) mediator.GetEntry();
  /// ...
  /// }
  /// </seealso>
  /// <author>robertz</author>
  public class CListURLTemplateLinkMediator : CListURLTemplateMediator,ICPListIterator
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
     "com.plumtree.portalpages.common.mediator.CListURLTemplateLinkMediator");
    
    /// <summary>
    /// Constructor for CListURLTemplateLinkMediator.
    /// </summary>
    /// <param name="aspace">The current Activityspace</param>
    /// <param name="list">The ASCompoundlist used by this mediator</param>
    public CListURLTemplateLinkMediator(AActivitySpace aspace, ICPListEntryIterator list) : base(aspace,
     list)
    {
      // Set to top by default
      m_strImageAlign = CommonHTMLStrings.ABSMIDDLE;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#GetEntry()"></seealso>
    /// <returns>
    /// 						An HTMLAnchor with the URL Template as onclick action
    /// and poundsign as href. Also includes image if one is
    /// set on the link.
    /// </returns>
    public override Object GetEntry()
    {
      HTMLAnchor anchor = null;
      String[] arrTemplate = (String[]) base.GetEntry();
      try
      {
        // No URL, set the HREF to be null. An HTMLAnchor with null HREF will just output its innerHTML. 
        // Since we are adding just a string (and sometimes an image), the generated HTML will just be a string
        // and an image if one is set.
        if (1 == arrTemplate.Length || null == arrTemplate[CListURLTemplateMediator.TEMPLATEARR_INDEX_URL])
        {
          anchor = new HTMLAnchor(null);
        } else
        {
          anchor = new HTMLAnchor("#");
          anchor.SetOnClick(NavigationCommonHelpers.URLTEMPLATE_REDIRECT_JSFUNCNAME + "(\'" + arrTemplate[CListURLTemplateMediator.TEMPLATEARR_INDEX_URL] + "\'); return false;");
          }
        // Has image, add the html for the image.
        if (IsImageEntry(arrTemplate))
        {
          anchor.AddInnerHTMLString(arrTemplate[CListURLTemplateMediator.TEMPLATEARR_INDEX_IMGURL]);
          }
        String label = arrTemplate[CListURLTemplateMediator.TEMPLATEARR_INDEX_LABEL];
        // Add the label as innerHTML.
        anchor.AddInnerHTMLString(label);
        // Make a tooltip with the original label for truncated label strings.
        if (XPStringUtility.EndsWith(label, "..."))
        {
          anchor.SetTitle(base.GetNonEncodedEntryOrigLabel());
        }
      }
      catch (HTMLException e)
      {
        log.Error("Error adding String to anchor tag, the String was: " + arrTemplate[0]);
        }
      return anchor;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#GetEntry()"></seealso>
    /// <returns>
    /// 						An HTMLAnchor with the URL Template as onclick action
    /// and poundsign as href.
    /// </returns>
    public override Object GetEntryAtIndex(int nIndex)
    {
      HTMLAnchor anchor = null;
      String[] arrTemplate = (String[]) base.GetEntryAtIndex(nIndex);
      try
      {
        // No URL, set the HREF to be null. An HTMLAnchor with null HREF will just output its innerHTML. 
        // Since we are adding just a string (and sometimes an image), the generated HTML will just be a string
        // and an image if one is set.
        if (1 == arrTemplate.Length)
        {
          anchor = new HTMLAnchor(null);
        } else
        {
          // Set the onclick action to be a call to the templateURLRedirect which will construct the
          // full URL and redirect to it. The HREF is actually not used.
          anchor = new HTMLAnchor("#");
          anchor.SetOnClick(NavigationCommonHelpers.URLTEMPLATE_REDIRECT_JSFUNCNAME + "(\'" + arrTemplate[TEMPLATEARR_INDEX_URL] + "\'); return false;");
          }
        // Has image, add the html for the image.
        if (arrTemplate.Length == 3)
        {
          anchor.AddInnerHTMLString(arrTemplate[TEMPLATEARR_INDEX_IMGURL]);
          }
        // Add the label as innerHTML.
        anchor.AddInnerHTMLString(arrTemplate[TEMPLATEARR_INDEX_LABEL]);
      }
      catch (HTMLException e)
      {
        log.Error("Error adding String to anchor tag, the String was: " + arrTemplate[0]);
        }
      return anchor;
    }
  }
}