using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.directory;
using com.plumtree.portaluiinfrastructure.compoundlist;
using com.plumtree.portaluiinfrastructure.friendlyurl;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.taskapi.portalui;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.common.mediator
{
  /// <summary>
  /// Created on Oct 29, 2002
  /// Mediator for ASCompoundList, takes values from ASCompoundList and returns
  /// URL Templates, comma separated String with the type of URL and its parameters.
  /// A CommunityPage URL would look like 'C,201,-201' where C stands for Community
  /// Template, 201 for the community id and -201 for page id.
  /// URL Templates are used to lessen the data being sent down to the client. The full 
  /// URL is constructed by calling the buildURLFromTemplate javascript function (use the
  /// NavigationCommonHelpers.JSCRIPT_FUNC_BUILD_URL_FROM_TEMPL constant). 
  /// The templateURLRedirect javascript function (use the NavigationCommonHelpers.URLTEMPLATE_REDIRECT_JSFUNCNAME)
  /// to construct the full url and redirect to it.
  /// This mediator is for menus with their own onclick event handling. Use the CListURLTemplateLinkMediator 
  /// to get an HTMLAnchor elements that redirects to the url in the template.
  /// Example use:
  /// ICPListIterator mediator = new CListURLTemplateMediator(m_asOwner, cpList1);
  /// mediator.SetLabelMaxLength(50);    //set the maximum label length to 50 characters
  /// while (mediator.Next())
  /// {
  /// String[] arURLTempl = (String[]) mediator.GetEntry();
  /// String strLabel \u0009= arURLTempl[CListURLTemplateMediator.TEMPLATEARR_INDEX_LABEL];
  /// String strURL\u0009\u0009= arURLTempl[CListURLTemplateMediator.TEMPLATEARR_INDEX_URL];
  /// String strImageURL\u0009= arURLTempl[CListURLTemplateMediator.TEMPLATEARR_INDEX_IMGURL];
  /// ...
  /// }
  /// </summary>
  /// <author>robertz</author>
  public class CListURLTemplateMediator : ICPListIterator
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
     "com.plumtree.portalpages.common.mediator.CListURLTemplateMediator");
    
    /// <summary>
    /// Index constants for the elements in the array generated by this mediator 
    /// </summary>
    public const int TEMPLATEARR_INDEX_LABEL = 0;
    public const int TEMPLATEARR_INDEX_URL = 1;
    public const int TEMPLATEARR_INDEX_IMGURL = 2;
    
    /// <summary>
    /// The constant amount of space in pixels images should take up.
    /// A spacer image will be added as padding if images doesn't fill
    /// up the entire space. 
    /// </summary>
    public const int IMAGE_SPACE_WIDTH = 24;
    public const int ACTIONIMAGE_SPACE_WIDTH = 30;
    private AActivitySpace m_aspace;
    private ICPListEntryIterator m_list;
    // Default values
    private int m_nLabelMaxLength = 20;
    private bool m_bShowImages = true;
    // Needs to be aligned to top to show up decent in dropdown menus in Netscape
    // Should be aigned to absmiddle in all other cases, set in child classes
    protected String m_strImageAlign = "";
    
    /// <summary>
    /// Constructor for CompoundListASURLMediator. Get the iterator interface for the list
    /// </summary>
    /// <param name="aspace">Current Activityspace</param>
    /// <param name="list">ICPListEntryIterator to wrap</param>
    public CListURLTemplateMediator(AActivitySpace aspace, ICPListEntryIterator list)
    {
      m_strImageAlign = CommonHTMLStrings.ABSMIDDLE;
      m_list = list;
      m_aspace = aspace;
      list.GetIterator();
    }
    
    /// <summary>
    /// Late added method, get the untruncated label of the current entry.
    /// </summary>
    public virtual String GetEntryOrigLabel()
    {
      ListEntry entry = (ListEntry) m_list.GetEntry();
      return XPHttpUtility.HtmlEncode(entry.GetOrigLabel());
    }
    
    /// <summary>
    /// Late added method, get the untruncated label of the current entry (non encoded).
    /// </summary>
    public virtual String GetNonEncodedEntryOrigLabel()
    {
      ListEntry entry = (ListEntry) m_list.GetEntry();
      return entry.GetOrigLabel();
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#GetEntry()"></seealso>
    /// <returns>
    /// A String array. 
    /// First element with the label of the link.
    /// Second element with the URL Template String, a comma separated string
    /// with the link type and parameters for link.
    /// Third (optional), HTML string with image or images.
    /// </returns>
    public virtual Object GetEntry()
    {
      ListEntry entry = (ListEntry) m_list.GetEntry();
      return GetURLFromListEntry(entry);
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#GetEntryAtIndex(int)"></seealso>
    public virtual Object GetEntryAtIndex(int nIndex)
    {
      ListEntry entry = (ListEntry) m_list.GetEntryAtIndex(nIndex);
      return GetURLFromListEntry(entry);
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#GetList()"></seealso>
    public virtual ICPListEntryIterator GetList()
    {
      return m_list;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#Next()"></seealso>
    public virtual bool Next()
    {
      return m_list.Next();
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#SetLabelMaxLength(int)"></seealso>
    public virtual void SetLabelMaxLength(int nMax)
    {
      m_nLabelMaxLength = nMax;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.mediator.ICPListIterator#SetShowImages(boolean)"></seealso>
    public virtual void SetShowImages(bool bShow)
    {
      m_bShowImages = bShow;
    }
    
    /// <summary>
    /// Creates a Template URL for Community pages
    /// </summary>
    /// <param name="entry">Current listentry element</param>
    /// <returns>				String array with string label, url template string and image string if assigned</returns>
    protected virtual String[] CreateCommPageURLTemplate(ListEntry entry)
    {
      String label = entry.GetLabel();
      int nCommID = entry.GetCommunityID();
      int nPageID = entry.GetPageID();
      String imageString = SetListEntryImage(entry);
      bool bUseFriendlyURL = ((-1 != entry.GetProfileID()) ? TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_USER_ID) : TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_COMMUNITY_ID));
      String strProfileID = ((-1 != entry.GetProfileID()) ? "," + entry.GetProfileID() : "");
      if (bUseFriendlyURL)
      {
        FriendlyURL friendly;
        if (-1 != entry.GetProfileID())
        {
          friendly = new FriendlyURL(PT_CLASSIDS.PT_USER_ID, entry.GetProfileID());
          } else
        {
          friendly = new FriendlyURL(PT_CLASSIDS.PT_COMMUNITY_ID, nCommID);
          }
        if (nPageID > 0)
        {
          friendly.SetChildId(nPageID);
        }
        if (imageString == null)
        {
          return new String[]{label, "IF," + friendly.GetFullFriendlyURL(m_aspace)};
          } else
        {
          return new String[]{label, "IF," + friendly.GetFullFriendlyURL(m_aspace),
           imageString};
        }
      } else
      {
        if (imageString == null)
        {
          return new String[]{label, "C," + nCommID + "," + nPageID + strProfileID};
          } else
        {
          return new String[]{label, "C," + nCommID + "," + nPageID + strProfileID,
           imageString};
        }
      }
    }
    
    /// <summary>
    /// Creates a Template URL for Community Knowledge Directory Root
    /// </summary>
    /// <param name="entry">Current listentry element</param>
    /// <returns>				String array with string label, url template string and image string if assigned</returns>
    protected virtual String[] CreateSiteMapRootURLTemplate(ListEntry entry)
    {
      String label = entry.GetLabel();
      int nCommID = entry.GetCommunityID();
      int nMode = entry.GetMode();
      bool bIsPreview = entry.GetIsPreview();
      String imageString = SetListEntryImage(entry);
      if (imageString == null)
      {
        return new String[]{label, "SM," + nCommID + "," + nMode + "," + XPConvert.ToString(bIsPreview)};
        } else
      {
        return new String[]{label, "SM," + nCommID + "," + nMode + "," + XPConvert.ToString(bIsPreview),
         imageString};
      }
    }
    
    /// <summary>
    /// Create a Template URL for Directory pages
    /// </summary>
    /// <param name="entry">Current listentry element</param>
    /// <returns>				String array with string label, url template string and image string if assigned</returns>
    protected virtual String[] CreateDirPageURLTemplate(ListEntry entry)
    {
      String label = entry.GetLabel();
      int nMode = entry.GetMode();
      int nFolderID = entry.GetFolderID();
      String imageString = SetListEntryImage(entry);
      bool bUseFriendlyURL = TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_CATALOGFOLDER_ID);
      if (bUseFriendlyURL)
      {
        FriendlyURL friendly = new FriendlyURL(PT_CLASSIDS.PT_CATALOGFOLDER_ID,
         nFolderID);
        String strDirMode = "";
        if (nMode > 1)
        {
          strDirMode = "?" + DirConstants.QS_DIRECTORY_MODE + "=" + nMode;
          }
        if (imageString == null)
        {
          return new String[]{label, "IF," + friendly.GetFullFriendlyURL(m_aspace) + strDirMode};
          } else
        {
          return new String[]{label, "IF," + friendly.GetFullFriendlyURL(m_aspace) + strDirMode,
           imageString};
        }
      } else
      {
        if (imageString == null)
        {
          return new String[]{label, "D," + nMode + "," + nFolderID};
        } else
        {
          return new String[]{label, "D," + nMode + "," + nFolderID, imageString};
          }
      }
    }
    
    /// <summary>
    /// Creates a Template URL for document pages
    /// </summary>
    /// <param name="entry">Current listentry element</param>
    /// <returns>				String array with string label, url template string and image string if assigned</returns>
    protected virtual String[] CreateDocURL(ListEntry entry)
    {
      String label = entry.GetLabel();
      int nDocID = entry.GetPageID();
      String imageString = SetListEntryImage(entry);
      if (imageString == null)
      {
        return new String[]{label, "DO," + nDocID};
      } else
      {
        return new String[]{label, "DO," + nDocID, imageString};
      }
    }
    
    /// <summary>
    /// Create a Template URL for editor pages
    /// </summary>
    /// <param name="entry">Current listentry element</param>
    /// <returns>				String array with string label, url template string and image string if assigned</returns>
    protected virtual String[] CreateEditorURLTemplate(ListEntry entry)
    {
      String label = entry.GetLabel();
      String editorSpace = entry.GetEditorAS();
      String control = entry.GetControl();
      String strPage = entry.GetPage();
      String strEditorType = entry.GetEditorType();
      String[] paramNames = entry.GetParamNames();
      String[] paramValues = entry.GetParamValues();
      XPStringBuilder buf = new XPStringBuilder(40);
      buf.Append("E," + editorSpace);
      if (null != control)
      {
        buf.Append("," + ASConstants.CONTROL + "=" + control);
      }
      if (null != strPage)
      {
        buf.Append("," + ASConstants.PAGE_INPUT + "=" + strPage);
      }
      if (null != strEditorType)
      {
        buf.Append("," + EditorStartControl.QS_EDITOR_TYPE + "=" + strEditorType);
        }
      if (paramNames != null)
      {
        for (int i = 0;i < paramNames.Length;i++)
        {
          buf.Append("," + paramNames[i] + "=" + paramValues[i]);
        }
      }
      String imageString = SetListEntryImage(entry);
      if (imageString == null)
      {
        return new String[]{label, buf.ToString()};
      } else
      {
        return new String[]{label, buf.ToString(), imageString};
      }
    }
    
    /// <param name="entry"></param>
    /// <returns></returns>
    protected virtual String[] CreateFullInternalURL(ListEntry entry)
    {
      String label = entry.GetLabel();
      String url = entry.GetFullURL();
      String imageString = SetListEntryImage(entry);
      if (imageString == null)
      {
        return new String[]{label, "IF," + url};
      } else
      {
        return new String[]{label, "IF," + url, imageString};
      }
    }
    
    /// <summary>
    /// Creates a full url with associated label and image if one
    /// </summary>
    /// <param name="entry">Current listentry element</param>
    /// <returns>				String array with string label, full url string and image string if assigned</returns>
    protected virtual String[] CreateFullURL(ListEntry entry)
    {
      String label = entry.GetLabel();
      String url = entry.GetFullURL();
      String imageString = SetListEntryImage(entry);
      if (imageString == null)
      {
        return new String[]{label, "F," + url};
      } else
      {
        return new String[]{label, "F," + url, imageString};
      }
    }
    
    /// <summary>
    /// Creates a generic Plumtree Portal 50 URL
    /// </summary>
    /// <param name="entry">Current listentry element</param>
    /// <returns>				String array with string label, url template string and image string if assigned</returns>
    protected virtual String[] CreateGenericURL(ListEntry entry)
    {
      XPStringBuilder buf = new XPStringBuilder(40);
      String label = entry.GetLabel();
      String strAS = entry.GetEditorAS();
      String[] paramNames = entry.GetParamNames();
      String[] paramValues = entry.GetParamValues();
      int nASType = entry.GetNewASType();
      if (nASType == ASCompoundList.NEWASTYPE_CREATENEW)
      {
        buf.Append("G1," + strAS);
      } else
      {
        log.Error("This Activityspace creation type is currently not supported for this mediator.");
        }
      if (null != paramNames)
      {
        for (int i = 0;i < paramNames.Length;i++)
        {
          buf.Append("&" + paramNames[i] + "=" + paramValues[i]);
        }
      }
      String imageString = SetListEntryImage(entry);
      if (imageString == null)
      {
        return new String[]{label, buf.ToString()};
      } else
      {
        return new String[]{label, buf.ToString(), imageString};
      }
    }
    
    /// <summary>
    /// Creates a Template URL for mypages
    /// </summary>
    /// <param name="entry">Current listentry element</param>
    /// <returns>				String array with string label, url template string and image string if assigned</returns>
    protected virtual String[] CreateMyPageURLTemplate(ListEntry entry)
    {
      String label = entry.GetLabel();
      int nPageID = entry.GetPageID();
      String imageString = SetListEntryImage(entry);
      bool bUseFriendlyURL = TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_MYPAGESCONTAINER_ID);
      if (bUseFriendlyURL)
      {
        FriendlyURL friendly = new FriendlyURL(PT_CLASSIDS.PT_MYPAGESCONTAINER_ID,
         nPageID);
        if (imageString == null)
        {
          return new String[]{label, "IF," + friendly.GetFullFriendlyURL(m_aspace)};
          } else
        {
          return new String[]{label, "IF," + friendly.GetFullFriendlyURL(m_aspace),
           imageString};
        }
      } else
      {
        if (imageString == null)
        {
          return new String[]{label, "M," + nPageID};
        } else
        {
          return new String[]{label, "M," + nPageID, imageString};
        }
      }
    }
    
    /// <summary>
    /// Creates a Template URL for users preferences
    /// </summary>
    /// <param name="entry">Current listentry element</param>
    /// <returns>				String array with string label, url template string and image string if assigned</returns>
    protected virtual String[] CreatePrefsURL(ListEntry entry)
    {
      String label = entry.GetLabel();
      String strPage = entry.GetPage();
      String imageString = SetListEntryImage(entry);
      if (imageString == null)
      {
        return new String[]{label, "P," + strPage};
      } else
      {
        return new String[]{label, "P," + strPage, imageString};
      }
    }
    
    /// <summary>
    /// Creates an array with just a label without url but possible image
    /// </summary>
    /// <param name="entry">Current listentry element</param>
    /// <returns>				String array with string label, url template string and image string if assigned</returns>
    protected virtual String[] CreateStringLabel(ListEntry entry)
    {
      String label = entry.GetLabel();
      String imageString = SetListEntryImage(entry);
      if (imageString == null)
      {
        return new String[]{label};
      } else
      {
        return new String[]{label, null, imageString};
      }
    }
    
    /// <summary>
    /// Creates a Template URL for user profile pages
    /// </summary>
    /// <param name="entry">Current listentry element</param>
    /// <returns>				String array with string label, url template string and image string if assigned</returns>
    protected virtual String[] CreateUserURL(ListEntry entry)
    {
      String label = entry.GetLabel();
      int nUserID = entry.GetPageID();
      String imageString = SetListEntryImage(entry);
      if (imageString == null)
      {
        return new String[]{label, "UP," + nUserID};
      } else
      {
        return new String[]{label, "UP," + nUserID, imageString};
      }
    }
    
    /// <summary>
    /// Creates a string containing the html for the assigned images
    /// </summary>
    /// <param name="entry">Current listentry element</param>
    /// <returns>				String with the html for the assigned images</returns>
    protected virtual String GetImageString(ListEntry entry)
    {
      PTImageType image = entry.GetImage();
      PTImageType image2 = entry.GetImage2();
      String strImageAlt = entry.GetImageAltTextEncoded();
      int nTotalWidth = 0;
      HTMLImg htmlImg = null;
      String imageString = "";
      if (null == image || !m_bShowImages)
      {
        return imageString;
      }
      try
      {
        htmlImg = (HTMLImg) UIConstructFactory.GetConstruct(image, true, this.m_aspace).GetHTMLElement();
        nTotalWidth+=XPConvert.ToInteger(htmlImg.GetWidth());
        htmlImg.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_aspace));
        htmlImg.SetAlt(strImageAlt);
        htmlImg.SetAlign(m_strImageAlign);
        htmlImg.SetBorder(CommonHTMLStrings.ZERO);
        imageString = htmlImg.GetDisplayString();
        if (null != image2)
        {
          htmlImg = (HTMLImg) UIConstructFactory.GetConstruct(image2, true,
           this.m_aspace).GetHTMLElement();
          nTotalWidth+=XPConvert.ToInteger(htmlImg.GetWidth());
          htmlImg.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_aspace));
          htmlImg.SetAlt(strImageAlt);
          htmlImg.SetAlign(m_strImageAlign);
          htmlImg.SetBorder(CommonHTMLStrings.ZERO);
          // HTMLEncode the string, might contain "
          //imageString = XPHttpUtility.HtmlEncode( htmlImg.GetDisplayString() );
          imageString+=htmlImg.GetDisplayString();
        }
        if (nTotalWidth < IMAGE_SPACE_WIDTH)
        {
          htmlImg = PlumtreeHelpers.GetSpacerImage(XPConvert.ToString(IMAGE_SPACE_WIDTH - nTotalWidth),
           "1");
          htmlImg.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_aspace));
          imageString+=htmlImg.GetDisplayString();
        } else
        if (nTotalWidth >= IMAGE_SPACE_WIDTH)
        {
          htmlImg = PlumtreeHelpers.GetSpacerImage(XPConvert.ToString(ACTIONIMAGE_SPACE_WIDTH - nTotalWidth),
           "1");
          htmlImg.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_aspace));
          imageString+=htmlImg.GetDisplayString();
        }
      }
      catch (Exception e)
      {
      }
      return imageString;
    }
    
    /// <summary>
    /// Creates a string containing the image source for the assigned image
    /// </summary>
    /// <returns>				String with the image source for the assigned images</returns>
    public virtual String GetImageSource()
    {
      ListEntry entry = (ListEntry) m_list.GetEntry();
      PTImageType image = entry.GetImage();
      String imageString = "";
      if (null == image || !m_bShowImages)
      {
        return imageString;
      }
      try
      {
        imageString = ConfigHelper.GetImageServerBaseURL(m_aspace) + image.GetFileName() + "\', \'" + image.GetWidth();
        }
      catch (Exception e)
      {
      }
      // TODO CR DLH: Why do we ignore this?
      return imageString;
    }
    
    /// <summary>
    /// Checks if an entry has image assigned.
    /// </summary>
    /// <param name="entry">Template entry from this mediator.</param>
    /// <returns>					true if image(s) exist, false otherwise</returns>
    public virtual bool IsImageEntry(String[] entry)
    {
      return (entry.Length == 3);
    }
    
    /// <summary>
    /// Get the image string for the specified entry
    /// </summary>
    /// <param name="entry"></param>
    /// <returns>					Image HTML String </returns>
    protected virtual String SetListEntryImage(ListEntry entry)
    {
      if (!m_bShowImages)
      {
        return null;
      }
      PTImageType image = entry.GetImage();
      String imageString = null;
      if (image != null)
      {
        imageString = GetImageString(entry);
      }
      return imageString;
    }
    
    /// <summary>
    /// Creates the URL Template by delegating to the respective method
    /// based on the URL type.
    /// </summary>
    /// <param name="entry">Current listentry</param>
    /// <returns>						An URL template</returns>
    private Object GetURLFromListEntry(ListEntry entry)
    {
      entry.SetTruncationLength(m_nLabelMaxLength);
      String[] template = null;
      switch (entry.GetType()){
      case ASCompoundList.ENTRYTYPE_MYPAGETEMPL:
      template = CreateMyPageURLTemplate(entry);
      break;
      case ASCompoundList.ENTRYTYPE_COMMPTEMPL:
      template = CreateCommPageURLTemplate(entry);
      break;
      case ASCompoundList.ENTRYTYPE_SITEMAPROOTTEMPL:
      template = CreateSiteMapRootURLTemplate(entry);
      break;
      case ASCompoundList.ENTRYTYPE_DIRTEMPL:
      template = CreateDirPageURLTemplate(entry);
      break;
      case ASCompoundList.ENTRYTYPE_EDITORTEMPL:
      template = CreateEditorURLTemplate(entry);
      break;
      case ASCompoundList.ENTRYTYPE_STRINGLABEL:
      template = CreateStringLabel(entry);
      break;
      case ASCompoundList.ENTRYTYPE_FULLINTERNALURL:
      template = CreateFullInternalURL(entry);
      break;
      case ASCompoundList.ENTRYTYPE_FULLURL:
      template = CreateFullURL(entry);
      break;
      case ASCompoundList.ENTRYTYPE_PREFSTEMPL:
      template = CreatePrefsURL(entry);
      break;
      case ASCompoundList.ENTRYTYPE_DOCTEMPL:
      //DCA - GetPage returns EditorPage not PageID
      // using PageID for DocID
      template = CreateDocURL(entry);
      break;
      case ASCompoundList.ENTRYTYPE_USERTEMPL:
      //DCA - GetPage returns EditorPage not PageID
      // using PageID from UserID
      template = CreateUserURL(entry);
      break;
      case ASCompoundList.ENTRYTYPE_GENERICTEMPL:
      template = CreateGenericURL(entry);
      break;
      default:
      log.Error("Entry created with unsupported link type.");
      break;
      }
      return template;
    }
  }
}