using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portalpages.common.mediator;
using com.plumtree.portaluiinfrastructure.application.varpacks;
using com.plumtree.portaluiinfrastructure.compoundlist;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.common.uiparts
{
  /// <summary>
  /// NavigationJSPortalMenuHelpers provides helper methods to 
  /// pluggable navigations that use the Plumtree JS Portal Menu
  /// Drop Downs. The available helper methods are MakeDDMenu and
  /// GetPlumtreeJSFrameworkJavaScript.
  /// MakeDDMenu will write to javascript the list of links to be
  /// displayed in a drop down menu in the format that the javascript
  /// framework for these drop downs will recognize. It uses the String 
  /// strMenuID to determine which tab the drop down menu is to drop
  /// down from, and it uses the ICPListEntryIterator cpList to iterate
  /// through and write out each of these links.
  /// MakeDDMenu should be called from inside the drop down view, once
  /// for each drop down menu. See 
  /// com.plumtree.portalnavigation.NavigationJSPortalMenuDropDownView
  /// to see how MakeDDMenu is used there.
  /// GetPlumtreeJSFrameworkJavaScript returns a HTMLScriptCollection
  /// of the javascript that is needed for use by the jsportalmenus.
  /// This includes several .js files.
  /// GetPlumtreeJSFrameworkJavaScript should be called in the drop
  /// down NavType's JavaScriptIncludes method. See
  /// com.plumtree.portalnavigation.NavTypeJSPortalMenuDropDown
  /// to see how GetPlumtreeJSFrameworkJavaScript is used there.
  /// </summary>
  /// <author>MichaelD</author>
  public class NavigationJSPortalMenuHelpers
  {
    
    /// <summary>
    /// Default width of text in the ISC dropdown menus *
    /// </summary>
    public const int DEFAULT_DDMENU_TEXT_MAX_LENGTH = 24;
    
    /// <summary>
    /// Prefix of the Javascript array names 
    /// </summary>
    public const String DATA_ARRAY_ID_PREFIX = "DataAR";
    public const String COL_ARRAY_ID_PREFIX = "ColAR";
    public const String LISTVIEW_ID_PREFIX = "ListVW";
    
    /// <summary>
    /// IDs of the ISC ListViewer objects 
    /// </summary>
    public const String m_strCommID = "comms";
    public const String m_strDocID = "docs";
    public const String m_strMandLinksID = "mandlinks";
    public const String m_strMyPageID = "mypage";
    public const String m_strPrefsID = "prefs";
    public const String m_strSubCommID = "subcomms";
    
    /// <summary>
    /// Internal function that creates the data array used to populate the JS Portal menus
    /// </summary>
    /// <param name="parent">Activityspace</param>
    /// <param name="cpList">List with links to populate the menu with</param>
    /// <param name="strMenuID">ID of the menu</param>
    /// <returns>HTMLScript	HTMLScript with the menu defined</returns>
    private static HTMLScript BuildMenuDataArray(AActivitySpace parent, ICPListEntryIterator cpList,
     String strMenuID)
    {
      XPStringBuilder buf = new XPStringBuilder(1000);
      CListURLTemplateMediator mediator = new CListURLTemplateMediator(parent,
       cpList);
      // Get the truncation width from the navigationsettings file		
      String strMaxLength = (String) parent.GetVarPack(PTNavSettingsVarPack.VARPACK_ID).GetVarPackValue(PTNavSettingsVarPack.INTISCDROPDOWNMENU_TRUNCATION_WIDTH);
      mediator.SetLabelMaxLength(((null == strMaxLength) ? DEFAULT_DDMENU_TEXT_MAX_LENGTH : XPConvert.ToInteger(strMaxLength)));
      int index = 0;
      buf.Append("[");
      // Need to replace all ' with \' in names and urls. Used to construct javascript object and will be treated
      // as string denominators. 
      while (mediator.Next())
      {
        String[] arStrings = (String[]) mediator.GetEntry();
        String strLabel = arStrings[CListURLTemplateMediator.TEMPLATEARR_INDEX_LABEL];
        String strURL = ((arStrings.Length > 1) ? arStrings[CListURLTemplateMediator.TEMPLATEARR_INDEX_URL] : "");
        String imgURL = "";
        // Sets images for the entries in the menu
        if (3 == arStrings.Length)
        {
          imgURL = mediator.GetImageSource();
        } else
        if (m_strCommID.Equals(strMenuID))
        {
          imgURL = NavigationCommonHelpers.GetCommunityImageSource(parent);
          } else
        if (m_strMyPageID.Equals(strMenuID))
        {
          imgURL = NavigationCommonHelpers.GetMyPageImageSource(parent);
        }
        if (index++ != 0)
        {
          buf.Append(",");
        }
        // Sets tooltip for the menu entry if the label was truncated
        HTMLSpan span = new HTMLSpan();
        if (mediator.GetNonEncodedEntryOrigLabel().Length > XPConvert.ToInteger(strMaxLength))
        {
          span.SetTitle(NavigationCommonHelpers.EscapeSingleQuotes(mediator.GetNonEncodedEntryOrigLabel()));
          }
        // Make the label of the current page bold.
        if ("".Equals(strURL))
        {
          HTMLB htmlB = new HTMLB();
          span.AddInnerHTMLElement(htmlB);
          htmlB.AddInnerHTMLString(NavigationCommonHelpers.EscapeSingleQuotes(strLabel));
          } else
        {
          span.AddInnerHTMLString(NavigationCommonHelpers.EscapeSingleQuotes(strLabel));
          }
        // Create the array for each entry
        // Element 0: Image URL
        // Element 1: HTML string for the label
        // Element 2: URL of the entry
        // This is a little tricky, use either ' or " as javascript lineseparators.
        // ' works better for urls and html since the " in them won't clash with ' but doesn't work for names, which can have 's
        // " works for name since " in names are htmlencoded, but clashes with " in html strings
        // Need to double encode the url since the url is used in an html string with an onclick action that also needs to be encoded 
        buf.Append("[\'" + XPStringUtility.Trim(imgURL) + "\',\'" + span.GetDisplayString() + "\',\"" + JSHelper.escapeJS(JSHelper.escapeJS(strURL)) + "\"]");
        }
      buf.Append("\t\t" + "]\n");
      // Add the array as a SafeJSVar value
      return JSHelper.AddSafeJSVariable(DATA_ARRAY_ID_PREFIX + strMenuID,
       buf.ToString());
    }
    
    /// <summary>
    /// Helper function to populate and create a new JS Portal Menu ListViewer object used for the
    /// dropdown menu. Inserts javascript into the HTMLScriptCollection passed to the method.
    /// </summary>
    /// <param name="col">HTMLScriptCollection</param>
    /// <param name="parent">Activityspace</param>
    /// <param name="cpList">List with links to populate the menu with</param>
    /// <param name="strMenuID">ID of the menu</param>
    public static void MakeDDMenu(HTMLScriptCollection col, AActivitySpace parent,
     ICPListEntryIterator cpList, String strMenuID)
    {
      col.AddInnerHTMLElement(NavigationJSPortalMenuHelpers.BuildMenuDataArray(parent,
       cpList, strMenuID));
    }
    
    /// <summary>
    /// Defines the required javascript includes for the navigation schemes using
    /// Plumtree's jsportalmenu JavaScript framework
    /// </summary>
    /// <returns>HTMLScriptCollection of the script needed for the jsportalmenus</returns>
    public static HTMLScriptCollection GetPlumtreeJSFrameworkJavaScript(AActivitySpace owner)
    {
      HTMLScriptCollection result = new HTMLScriptCollection();
      String strPortalImageServerJavaScriptURL = ConfigHelper.GetPortalImageServerJavaScriptURL(owner);
      // Include for js portal menu engine js functions
      HTMLScript jspmScript = new HTMLScript("text/javaScript");
      jspmScript.SetSrc(strPortalImageServerJavaScriptURL + "navjsportalmenufuncs.js");
      result.AddInnerHTMLElement(jspmScript);
      // Include for community section js functions
      HTMLScript tabScript = new HTMLScript("text/javaScript");
      tabScript.SetSrc(strPortalImageServerJavaScriptURL + "navtabsections.js");
      result.AddInnerHTMLElement(tabScript);
      // Include for mandatory tabs js functions
      HTMLScript commonScript = new HTMLScript("text/javaScript");
      commonScript.SetSrc(strPortalImageServerJavaScriptURL + "navjsportalmenucommon.js");
      result.AddInnerHTMLElement(commonScript);
      return result;
    }
    
    /// <summary>
    /// Defines the required javascript includes for the navigation schemes using
    /// Plumtree's jsportalmenu JavaScript framework
    /// </summary>
    /// <returns>HTMLScriptCollection of the script needed for the jsportalmenus</returns>
    public static HTMLScriptCollection GetPlumtreeJSFrameworkNoCommunityTabJavaScript(AActivitySpace owner)
    {
      HTMLScriptCollection result = new HTMLScriptCollection();
      String strPortalImageServerJavaScriptURL = ConfigHelper.GetPortalImageServerJavaScriptURL(owner);
      // Include for js portal menu engine js functions
      HTMLScript jspmScript = new HTMLScript("text/javaScript");
      jspmScript.SetSrc(strPortalImageServerJavaScriptURL + "navjsportalmenufuncs.js");
      result.AddInnerHTMLElement(jspmScript);
      // Include for mandatory tabs js functions
      HTMLScript commonScript = new HTMLScript("text/javaScript");
      commonScript.SetSrc(strPortalImageServerJavaScriptURL + "navjsportalmenucommon.js");
      result.AddInnerHTMLElement(commonScript);
      return result;
    }
    
    /// <summary>
    /// Defines the required javascript includes for the navigation schemes using
    /// Mandatory Tabs.
    /// </summary>
    /// <returns>HTMLScript of the script needed for the tabs (Mandatory Links, Administration, Mandatory Communities, My Pages, My Communities)</returns>
    public static HTMLScript GetPlumtreeMandatoryTabJavaScript(AActivitySpace owner)
    {
      String strPortalImageServerJavaScriptURL = ConfigHelper.GetPortalImageServerJavaScriptURL(owner);
      // Include for mandatory tabs js functions
      HTMLScript commonScript = new HTMLScript("text/javaScript");
      commonScript.SetSrc(strPortalImageServerJavaScriptURL + "navjsportalmenucommon.js");
      return commonScript;
    }
    
    /// <summary>
    /// Defines the required javascript includes for the navigation schemes using
    /// Community Tabs.
    /// </summary>
    /// <returns>HTMLScript of the script needed for the community tabs (Community Pages, Related Communities, Subcommunities)</returns>
    public static HTMLScript GetPlumtreeCommunityTabJavaScript(AActivitySpace owner)
    {
      String strPortalImageServerJavaScriptURL = ConfigHelper.GetPortalImageServerJavaScriptURL(owner);
      // Include for community section js functions
      HTMLScript tabScript = new HTMLScript("text/javaScript");
      tabScript.SetSrc(strPortalImageServerJavaScriptURL + "navtabsections.js");
      return tabScript;
    }
  }
}