using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.condition;
using com.plumtree.server;
using com.plumtree.server.condition;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.expandablelist;
using com.plumtree.uiinfrastructure.expandablelist.growablelist;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.portalpages.condition.conditiontypes
{
  /// <summary>
  /// This basic implementation of AGuestConditionType defines how to retrieve and evaluate
  /// a current user's browser type. It also defines methods to display conditions that have 
  /// been defined
  /// </summary>
  public class ConditionTypeUserAgent : AGuestConditionType
  {
    
    /// <summary>
    /// Returns the unique ID of this condition type, which is used
    /// to store the list of condition types implemented.
    /// </summary>
    /// <seealso cref="com.plumtree.server.condition.IConditionServerType#GetTypeID()"></seealso>
    public override int GetTypeID()
    {
      return ConditionTypeConstants.CONDITIONTYPE_ID_USER_AGENT;
    }
    
    /// <summary>
    /// Returns the result of comparing the condition value specified by conditionValue to
    /// the current value of the condition type found in htUserEnvironment.
    /// </summary>
    /// <seealso cref="com.plumtree.server.condition.IConditionServerType#Compare(com.plumtree.openfoundation.util.XPHashtable,com.plumtree.server.condition.IValue,com.plumtree.openfoundation.util.XPStringBuilder)"></seealso>
    public override bool Compare(XPHashtable htUserEnvironment, IValue conditionValue,
     XPStringBuilder sbDebugText)
    {
      // check that objValue is of type string and that the user agent is in the current environment
      if (conditionValue.GetType() != ValueTypeEnum.STRING || !htUserEnvironment.ContainsKey(GetTypeID()))
      {
        if (null != sbDebugText)
        {
          sbDebugText.Append("Condition on User Agent returning false because either the condition value is of the wrong type,").Append(" or the User Agent was not found in the user\'s environment<br>");
          }
        return false;
      }
      // cast value to type: String
      String strUserAgent = (String) conditionValue.GetValue();
      String envUserAgent = (String) htUserEnvironment.GetElement(GetTypeID());
      // NOTE: indexOf is much much faster than regexp pattern matchin.
      // Tests have shown a 10-20% reduction in throughput on all requests when using
      // regexp.  Since many rules will just look for a particular string, we can use
      // indexOf for most cases, which is 40x faster.
      // compare values
      // DCA - check for exact match
      bool result = XPStringUtility.EqualsIgnoreCase(envUserAgent, strUserAgent);
      if (!result && envUserAgent.Length > 0)
      {
        // DCA - do patterm matching
        result = (XPStringUtility.IndexOf(envUserAgent, strUserAgent) >= 0);
        }
      // NOTE: Bug#70069 - We should also support Regexp patterns for greater control.
      // That would be done like this:
      //String strURLPattern = PreparePattern(strUserAgent);
      //XPRegex urlPattern = new XPRegex(strURLPattern, XPRegex.IGNORE_CASE);
      //result = urlPattern.IsMatch(envUserAgent);
      if (result)
      {
        if (null != sbDebugText)
        {
          sbDebugText.Append("Condition on user-agent returning true because the user-agent string pattern: ").Append(strUserAgent).Append(" matches the one found in the user\'s environment: ").Append(envUserAgent).Append("<br>");
          }
        return true;
      } else
      {
        if (null != sbDebugText)
        {
          sbDebugText.Append("Condition on user-agent returning false because the user-agent string pattern: ").Append(strUserAgent).Append(" does not match the one found in the user\'s environment: ").Append(envUserAgent).Append("<br>");
          }
        return false;
      }
    }
    private String PreparePattern(String sSource)
    {
      // TODO: Bug#70069 - this should be shared with the URL domain condition if possible.
      //    We will convert occurrences of
      //    '?' to '.'
      //    '.' to '\.'
      //    '*' to '(.*)
      // This allows the user to enter standard file system expressions
      //    like 'Michael ? S*.doc' (i.e. any middle initial, and 
      //    shortened last name), and pass it to the engine in the correct
      //    form (in this case, '^Michael . S(.*)\.doc$')
      if (sSource == null)
      {
        return "";
      }
      int n = sSource.Length;
      XPStringBuilder sb = new XPStringBuilder(n);
      sb.Append("^");
      for (int i = 0;i < n;i++)
      {
        char c = XPStringUtility.CharAt(sSource, i);
        switch (c){
        case '(':
        sb.Append("\\(");
        break;
        case ')':
        sb.Append("\\)");
        break;
        case '.':
        sb.Append("\\.");
        break;
        case '?':
        sb.Append(".");
        break;
        case '*':
        sb.Append("(.*)");
        break;
        case '[':
        sb.Append("\\[");
        break;
        case ']':
        sb.Append("\\]");
        break;
        case '/':
        sb.Append("\\/");
        break;
        default:
        sb.Append(c);
        break;
        }
      }
      sb.Append("$");
      return sb.ToString();
    }
    
    /// <summary>
    /// Determines the condition type's current value, and adds it to the htUserEnvironment.
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.condition.AGuestConditionType#GetCurrentValue(com.plumtree.portaluiinfrastructure.condition.XPLimitedRequest,com.plumtree.server.IPTSession,com.plumtree.openfoundation.util.XPHashtable)"></seealso>
    public override void GetCurrentValue(XPLimitedRequest xpRequest, IPTSession guestReadOnlySession,
     XPHashtable htUserEnvironment)
    {
      htUserEnvironment.PutElement(GetTypeID(), xpRequest.GetHeader("User-Agent"));
      }
    
    /// <summary>
    /// Returns the String that will be displayed as the Section Title for this condition type
    /// in the Subportal Rules Editor. The title can be localized to strLanguage by using the 
    /// resource manager: xpRM.
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.condition.AConditionType#GetMyConditionsListSectionTitle(com.plumtree.openfoundation.internationalization.XPResourceManager,java.lang.String)"></seealso>
    public override String GetMyConditionsListSectionTitle(XPResourceManager xpRM,
     String strLanguage)
    {
      String strResult;
      try
      {
        strResult = xpRM.GetString(3226, "ptmsgs_portaladminmsgs", strLanguage);
        }
      catch (Exception e)
      {
        strResult = "The user-agent string matches ANY of the following patterns:";
        }
      return strResult;
    }
    
    /// <summary>
    /// Returns the String that will be displayed as the Section SubTitle for this condition type
    /// in the Subportal Rules Editor. The subtitle can be localized to strLanguage by using the 
    /// resource manager: xpRM.
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.condition.AConditionType#GetMyConditionsListSectionSubTitle(com.plumtree.openfoundation.internationalization.XPResourceManager,java.lang.String)"></seealso>
    public override String GetMyConditionsListSectionSubTitle(XPResourceManager xpRM,
     String strLanguage)
    {
      String strResult;
      try
      {
        strResult = xpRM.GetString(3227, "ptmsgs_portaladminmsgs", strLanguage);
        }
      catch (Exception e)
      {
        strResult = "User-Agent Strings";
      }
      return strResult;
    }
    
    /// <summary>
    /// Returns the String that will be displayed as the Add Label for this condition type
    /// in the Subportal Rules Editor. The label can be localized to strLanguage by using the 
    /// resource manager: xpRM.
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.condition.AConditionType#GetMyConditionsListAddLabel(com.plumtree.openfoundation.internationalization.XPResourceManager,java.lang.String)"></seealso>
    public override String GetMyConditionsListAddLabel(XPResourceManager xpRM,
     String strLanguage)
    {
      String strResult;
      try
      {
        strResult = xpRM.GetString(3228, "ptmsgs_portaladminmsgs", strLanguage);
        }
      catch (Exception e)
      {
        strResult = "Add a User-Agent String";
      }
      return strResult;
    }
    
    /// <summary>
    /// Returns the Icon to be associated with this condition type.
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.condition.AConditionType#GetMyConditionsListIcon()"></seealso>
    public override PTImageType GetMyConditionsListIcon()
    {
      return PTImageType.ICON_ROW;
    }
    
    /// <summary>
    /// Returns the value type of the codition expected by this condition type.
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.condition.AConditionType#GetConditionValueType()"></seealso>
    public override int GetConditionValueType()
    {
      return ValueTypeEnum.STRING;
    }
    
    /// <summary>
    /// Returns the condition value stored in the nRow row of saData.
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.condition.AConditionType#GetConditionValue(int,com.plumtree.uiinfrastructure.arraywrapper.IPTGrowableSortedArrayWrapperRO)"></seealso>
    public override Object GetConditionValue(int nRow, IPTGrowableSortedArrayWrapperRO saData)
    {
      Object result = saData.GetItem(nRow, GrowableListModel.EXPLIST_SORTEDARRAY_PROPID_INPUTTEXT);
      return result;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.condition.AConditionType#GetValidationFailedString(com.plumtree.openfoundation.internationalization.XPResourceManager, java.lang.String)
     */
     
    public override String GetValidationFailedString(XPResourceManager xpRM,
     String strLanguage)
    {
      return "";
    }
    
    /// <summary>
    /// Adds objItem to the list of conditions of this condition type that is stored in myListModel
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.condition.AConditionType#AddItemToMyConditionsList(java.lang.Object,com.plumtree.uiinfrastructure.expandablelist.ExpListModel,com.plumtree.server.IPTSession)"></seealso>
    public override void AddItemToMyConditionsList(Object objItem, ExpListModel myListModel,
     IPTSession ptSession)
    {
      GrowableListModel myGrowableListModel = (GrowableListModel) myListModel;
      myGrowableListModel.AddRowsToList(new String[]{XPConvert.ToString(objItem)});
      }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.condition.AConditionType#GetDescriptionString(com.plumtree.server.condition.IValue, com.plumtree.server.IPTSession)
     */
     
    public override String GetDescriptionString(IValue ConditionValue, IPTSession ptSession)
    {
      XPStringBuilder sbURLDomain = new XPStringBuilder();
      sbURLDomain.Append("User-Agent string contains string: ").Append(ConditionValue.GetValue());
      return sbURLDomain.ToString();
    }
  }
}