using System;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.portalpages.pei
{
  /// <summary>
  /// This interface was created to allow developers the ability to create
  /// functions that can be dynamically discovered by the Plumtree application
  /// for the purposes of performing some functionality when a particular
  /// directory based event occurs.  
  /// <br /><br />
  /// NOTE: Should this interface be implemented more than once within the
  /// Dynamic Discovery process, the implementations will be processed in the
  /// order that they are displayed within the representative XML file. All
  /// implementations will be called.
  /// <br /><br />
  /// NOTE: In the case where a Redirect object is returned, the first 
  /// implementation of the interface that actually returns a valid Redirect will 
  /// be the one that will be processed.
  /// <br /><br />
  /// NOTE: In the case where multiple implementations return a String object, all 
  /// the valid Strings (not null and not empty) will be appended together 
  /// (separated by a blank space) and the final String will be the error message 
  /// displayed on the page.
  /// </summary>
  public interface IDirectoryActions
  {
    
    /// <summary>
    /// This function will perform some functionality after an ABO Job is created 
    /// to delete a Directory Object.  
    /// <br /><br />
    /// More info on ABO Jobs and when this PEI is called:<br />
    /// When the user deletes an Directory Folder, since this action might
    /// require a lot of processing time, he will have the choice between
    /// deleting the folder now or  scheduling an ABO Job to do it later and
    /// minimize the impact on the Portal's performance. If he decides to do it
    /// now, the OnDeleteDirectoryFolder PEI is called, if he decides to have
    /// it done later by a Job, the OnBeforeCreateABOJob and
    /// OnAfterCreateABOJob PEIs are called.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_nJobID">- ID of the ABO Job created</param>
    /// <param name="_nClassID">- Class ID of the Object that the Job will delete</param>
    /// <param name="_nObjectID">- ID of the Object that the Job will delete</param>
    void OnAfterCreateABOJob(AActivitySpace _asCurrentSpace, IPTSession _ptUserSession,
     int _nJobID, int _nClassID, int _nObjectID);
    
    /// <summary>
    /// This function will perform some functionality after a Directory Folder
    /// has been created.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_nFolderID">- The specific folder being created</param>
    void OnAfterCreateDirectoryFolder(AActivitySpace _asCurrentSpace, IPTSession _ptUserSession,
     int _nFolderID);
    
    /// <summary>
    /// This function will perform some functionality after a Directory Folder
    /// has been deleted.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_nFolderID">- The specific folder being deleted</param>
    void OnAfterDeleteDirectoryFolder(AActivitySpace _asCurrentSpace, IPTSession _ptUserSession,
     int _nFolderID);
    
    /// <summary>
    /// This function will perform some functionality before an ABO Job is created 
    /// to delete a Directory Object.  
    /// <br /><br />
    /// More info on ABO Jobs and when this PEI is called:<br />
    /// When the user deletes an Directory Folder, since this action might
    /// require a lot of processing time, he will have the choice between
    /// deleting the folder now or  scheduling an ABO Job to do it later and
    /// minimize the impact on the Portal's performance. If he decides to do it
    /// now, the OnDeleteDirectoryFolder PEI is called, if he decides to have
    /// it done later by a Job, the OnBeforeCreateABOJob and
    /// OnAfterCreateABOJob PEIs are called.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_nClassID">- Class ID of the Object that the Job will delete</param>
    /// <param name="_nObjectID">- ID of the Object that the Job will delete</param>
    /// <returns>
    /// String - Error message if the creation of the ABO Job should not 
    /// continue, empty string or null if it should.
    /// </returns>
    String OnBeforeCreateABOJob(AActivitySpace _asCurrentSpace, IPTSession _ptUserSession,
     int _nClassID, int _nObjectID);
    
    /// <summary>
    /// This function will perform some functionality before the creation
    /// process of a Directory Folder.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_strFolderName">- Name of the folder being created</param>
    /// <param name="_strFolderDescription">- Description of the folder being created</param>
    /// <returns>
    /// String - Error message if the creation should not continue,
    /// empty string or null if it should.
    /// </returns>
    String OnBeforeCreateDirectoryFolder(AActivitySpace _asCurrentSpace, IPTSession _ptUserSession,
     String _strFolderName, String _strFolderDescription);
    
    /// <summary>
    /// This function will perform some functionality before the deletion
    /// process of a Directory Folder.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_nFolderID">- The specific folder being deleted</param>
    /// <returns>
    /// String - Error message if the deletion should not continue,
    /// empty string or null if it should.
    /// </returns>
    String OnBeforeDeleteDirectoryFolder(AActivitySpace _asCurrentSpace, IPTSession _ptUserSession,
     int _nFolderID);
    
    /// <summary>
    /// This function will perform some functionality before the deletion
    /// process of a Document.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_nDocumentID">- The specific document being deleted</param>
    /// <returns>
    /// String - Error message if the deletion should not continue,
    /// empty string or null if it should.
    /// </returns>
    String OnBeforeDeleteDocument(AActivitySpace _asCurrentSpace, IPTSession _ptUserSession,
     int _nDocumentID);
    
    /// <summary>
    /// This function will perform some functionality during the click through
    /// of a particular document and then perform a Redirect or return Null.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_nDocumentID">- Object ID of the Document trying to be viewed.</param>
    /// <returns>
    /// Redirect - Location that the User will be redirected after the
    /// functionality has completed.
    /// </returns>
    Redirect OnClickThroughToDoc(AActivitySpace _asCurrentSpace, IPTSession _ptUserSession,
     int _nDocumentID);
    
    /// <summary>
    /// This function will perform some functionality during the Opening of a
    /// folder within the directory structure and then perform a Redirect or
    /// return Null.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_nFolderID">- Object ID of the Folder trying to be viewed.</param>
    /// <returns>
    /// Redirect - Location that the User will be redirected after the
    /// functionality has completed.
    /// </returns>
    Redirect OnOpenFolder(AActivitySpace _asCurrentSpace, IPTSession _ptUserSession,
     int _nFolderID);
  }
}