using System;
using com.plumtree.portalpages.browsing.myportal;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.portalpages.pei
{
  /// <summary>
  /// This interface was created to allow developers the ability to create
  /// functions that can be dynamically discovered by the Plumtree application
  /// for the purposes of performing some functionality when a particular
  /// mypage based event occurs.  
  /// <br /><br />
  /// NOTE: Should this interface be implemented more than once within the
  /// Dynamic Discovery process, the implementations will be processed in the
  /// order that they are displayed within the representative XML file. All
  /// implementations will be called.
  /// <br /><br />
  /// NOTE: Be careful if implementing this interface more than once,
  /// as there could be very significant problems if multiple functions are
  /// making changes to the same page name or page type. 
  /// <br /><br />
  /// NOTE: In the case where multiple implementations return a String object, all 
  /// the valid Strings (not null and not empty) will be appended together 
  /// (separated by a blank space) and the final String will be the error message 
  /// displayed on the page.
  /// </summary>
  public interface IMyPortalPageActions
  {
    
    /// <summary>
    /// This function will perform some functionality after a new MyPage has been
    /// created.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_strPageName">- Name of the new MyPage</param>
    /// <param name="_nPageTypeID">- Type of the new MyPage</param>
    void OnAfterAddMyPortalPage(AActivitySpace _asCurrentSpace, IPTSession _ptUserSession,
     String _strPageName, int _nPageTypeID);
    
    /// <summary>
    /// This function will perform some functionality after a MyPage has been
    /// edited.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_strPageName">- Name of the MyPortal Page.</param>
    /// <param name="_nPageTypeID">- Page Type of the MyPortal page.</param>
    void OnAfterEditMyPortalPage(AActivitySpace _asCurrentSpace, IPTSession _ptUserSession,
     String _strPageName, int _nPageTypeID);
    
    /// <summary>
    /// This function will perform some functionality after a user has deleted a
    /// page.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_nPageID">- ID of the MyPage that the user has removed.</param>
    void OnAfterRemoveMyPortalPage(AActivitySpace _asCurrentSpace, IPTSession _ptUserSession,
     int _nPageID);
    
    /// <summary>
    /// This function will perform some validation based on the page name and
    /// page type when a user tries to add a new page to their available listing
    /// of pages.  
    /// <br /><br />
    /// NOTE: The int item of the MyPortalPageArguments needs to be
    /// a valid PageTypeConstant or there will be errors when processed.  
    /// <br /><br />
    /// NOTE: The String item of the MyPortalPageArguments needs to be non-null and
    /// with a length greater than 0 but less than or equal to 20. Otherwise
    /// the original _strPageName will superscede this value.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_strPageName">
    /// - Name that the user is trying to assign to this new
    /// MyPortal Page.
    /// </param>
    /// <param name="_nPageTypeID">
    /// - Page Type that the user wants to use for the new
    /// MyPortal page.
    /// </param>
    /// <returns>
    /// MyPortalPageArguments - This object will contain a String
    /// pertaining to the  modified name being returned and a int
    /// pertaining to the modified page layout.
    /// </returns>
    MyPortalPageArguments OnBeforeAddMyPortalPage(AActivitySpace _asCurrentSpace,
     IPTSession _ptUserSession, String _strPageName, int _nPageTypeID);
    
    /// <summary>
    /// This function will perform some validation based on the page name and
    /// page type when a user tries to edit a page.  
    /// <br /><br />
    /// NOTE: The int item of the MyPortalPageArguments needs to be a valid 
    /// PageTypeConstant or there will be errors when processed.  
    /// <br /><br />
    /// NOTE: The String item of the MyPortalPageArguments needs to be non-null 
    /// and with a length greater than 0 but less than or equal to 20. Otherwise 
    /// the original _strPageName will superscede this value.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_strPageName">
    /// - Name that the user is trying to assign to this
    /// MyPortal Page.
    /// </param>
    /// <param name="_nPageTypeID">
    /// - Page Type that the user wants to use for the
    /// MyPortal page.
    /// </param>
    /// <returns>
    /// MyPortalPageArguments - This object will contain a String
    /// pertaining to the  modified name being returned and a int
    /// pertaining to the modified page layout.
    /// </returns>
    MyPortalPageArguments OnBeforeEditMyPortalPage(AActivitySpace _asCurrentSpace,
     IPTSession _ptUserSession, String _strPageName, int _nPageTypeID);
    
    /// <summary>
    /// This function will perform some functionality when a user tries to remove
    /// a page from their available listing of pages.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_ptUserSession">- Plumtree session object for the current user</param>
    /// <param name="_nPageID">- Page ID that the user is trying to remove.</param>
    /// <returns>
    /// String - Error message if the deletion should not continue,
    /// empty string or null if it should.
    /// </returns>
    String OnBeforeRemoveMyPortalPage(AActivitySpace _asCurrentSpace, IPTSession _ptUserSession,
     int _nPageID);
  }
}