using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.portalsettings.password;
using com.plumtree.portaluiinfrastructure.login;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.server.portalpagesperf;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using Interpreter=com.plumtree.uiinfrastructure.interpreter.Interpreter;
using com.plumtree.uiinfrastructure.login;
using com.plumtree.uiinfrastructure.pei;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portalpages.pei
{
  /// <author>
  /// Ben Geislinger
  /// This class is automatically called during logins and caches
  /// the user's personal settings from the SessionInfo object.
  /// </author>
  public class PTLoginActions : ILoginActions
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
     "com.plumtree.portalpages.pei.PTLoginActions");
    // The name of the password last changed user property.
    public const String PASSWORD_LAST_CHANGED = "Password Last Changed";
    
    /// <summary>
    /// This function will take a given userSession and grab all of the users
    /// personal settings and take care of caching them.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.login.ILoginActions#OnAfterLogin(Object,ApplicationData)"></seealso>
    public virtual Redirect OnAfterLogin(Object _oUserSession, ApplicationData _appData)
    {
      IPTSession ptSession = (IPTSession) _oUserSession;
      IApplication application = _appData.GetApplication();
      ISessionManager sessionManager = _appData.GetSubSession();
      ISessionManager persistentSessionManager = _appData.GetPersistentSubSession();
      XPResourceManager rm = (XPResourceManager) application.GetCachingManager().GetEntry("ResourceManager");
      // This will cache the users settings on the http session for us.  The
      // locale may come back as UseBrowser, which requires us to determine
      // the proper locale from the browser settings.
      PTPersonalSettingsHelper.CachePersonalSettings(sessionManager, application,
       ptSession, rm, _appData);
      // Check if the default user locale string is UseBrowser
      if (0 == XPStringUtility.CompareIgnoreCase((String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LOCALE,
       sessionManager, application, ptSession), PTPersonalSettingsHelper.LOCALE_USE_BROWSER_SETTING))
      {
        // We need to set the language setting from the request header.
        // This needs to be done after CachePersonalSettings, otherwise we
        // will always get the default language.  CachePersonalSettings can
        // return UserBrowser either because the user has explicitly chosen
        // it, or if the user did not choose anything.
        SetBrowserLocaleSetting(_appData, ptSession, sessionManager, persistentSessionManager,
         application, rm);
      }
      if (!LoginHelper.INSTANCE.GetIsGuestUser(ptSession))
      {
        // Only track non-guest logins.
        PerfCounters.INSTANCE.IncrementCounter(PORTALPAGESPERF_COUNTERS.PORTALPAGESPERF_LOGINS_SUCCESSFUL);
        }
      // JF- Set the content-language in the header so it can be used by braille devices
      String sContentLanguageHeader = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LOCALE,
       sessionManager, application, ptSession);
      _appData.SetContentLanguage(sContentLanguageHeader);
      // JF- Also set the content language in the Session so it can be set again 
      // in the header next request by the interpreter.
      sessionManager.SetAttribute(Interpreter.USERLOCALE, sContentLanguageHeader);
      // Check if the password has expired or is about to expire.
      Redirect expiredPasswordRedirect = CheckPasswordExpiration(ptSession,
       _appData);
      if (expiredPasswordRedirect != null)
      {
        // The password has expired, redirect to the change password page
        return expiredPasswordRedirect;
      }
      return null;
    }
    
    /// <summary>
    /// This internal helper method checks if the password has expired or not.
    /// It is public for unit test purposes only and should not be used outside of this class.
    /// </summary>
    /// <param name="_ptSession">the current user session.</param>
    /// <param name="_appData">the application data for this PEI</param>
    /// <returns>a Redirect if the password has expired, null if the password is valid.</returns>
    public virtual Redirect CheckPasswordExpiration(IPTSession _ptSession,
     ApplicationData _appData)
    {
      if (_ptSession.GetSessionInfo().CheckCurrentUserIsGuest() || 
      // Default profile users are all stored in the same admin folder
      (_ptSession.GetSessionInfo().GetCurrentUserAdminFolderID() == PT_INTRINSICS.PT_ADMIN_FOLDER_DEFAULTPROFILES) || (_ptSession.GetSessionInfo().GetCurrentUserID() == PT_INTRINSICS.PT_USER_SYSADMIN))
      {
        // We don't do password expirations for the guest or default profile users,
        // since they don't really have a password.  We also don't expire the
        // intrinsic admin password to make sure that you can always get into the
        // portal with at least one user.
        return null;
      }
      IPTServerConfigSettings config = (IPTServerConfigSettings) _ptSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
       false);
      // This is a cached value, so it should be quick
      int nPasswordExpiration = config.GetSettingAsInt(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_EXPIRATION);
      // If password expiration is turned on
      if (nPasswordExpiration > 0)
      {
        // For performance, open cached property and check for expiration for all users.
        // Then, if it's expired, check if it's a native user.  If it's not, set the exipration time
        // to way in the future.  That way we only need to check it every once in a while and we
        // save a db query on login.
        // Check if the user's password needs to be changed.  This will be cached (or, will
        // need to be cached anyways), so we can consider it cheap.
        String strLastPasswordChange = _ptSession.GetUserInfo().GetSetting(PASSWORD_LAST_CHANGED);
        if (strLastPasswordChange == null)
        {
          // Users created before password policies were put in place won't have the Password
          // Last Changed property, so we need to set it for them.  This also applies to LDAP
          // users.  We check for expiration and then only check if they are a native user if
          // their password has expired.  This minimizes the number of DB queries on login.
          SetPasswordChangedTimeToNow(_ptSession);
        } else
        {
          // If they had a valid Last Password Change property, check
          // if their password has expired.
          // Convert it to a long (in milliseconds)
          long msLastPasswordChange;
          try
          {
            msLastPasswordChange = XPConvert.ToLong(strLastPasswordChange);
            }
          catch (Exception e)
          {
            log.Warn(e, "Unable to convert Last Password Change to a long.  This should never happen.  Aborting password expiration check.");
            // Invalid value - reset it.
            SetPasswordChangedTimeToNow(_ptSession);
            return null;
          }
          // Make sure to correctly convert everything to longs otherwise
          // you can cause the data to wrap if you use integer arithmetic.
          long lPasswordExpiration = XPConvert.ToLong(nPasswordExpiration);
          // Convert days to milliseconds
          long msExpiration = lPasswordExpiration * (long) (1000 * 60 * 60 * 24);
          long msNow = XPSystem.GetCurrentTimeMilliSec();
          long msPasswordAge = msNow - msLastPasswordChange;
          long msTimeUntilExpiration = msExpiration - msPasswordAge;
          bool bPasswordExpired = msTimeUntilExpiration <= 0;
          // This is a cached value, so it should be quick
          int nPasswordExpirationWarning = config.GetSettingAsInt(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_EXPIRATION_WARNING);
          // Convert days to milliseconds
          long msExpirationWarning = 1000 * 60 * 60 * 24 * nPasswordExpirationWarning;
          bool bPasswordExpirationWarning = msTimeUntilExpiration <= msExpirationWarning;
          if (bPasswordExpired || bPasswordExpirationWarning)
          {
            // If their password has expired, check if they are a native user,
            // since we only handle password expiration for native users.
            // NOTE: This requires a non-cached DB query, so we do this after
            // checking the expiration, since that's a cached user property.
            if (_ptSession.GetUser().GetAuthenticationSourceID() == PT_INTRINSICS.PT_AUTHSOURCE_PLUMTREE)
            {
              Redirect result = new Redirect();
              if (bPasswordExpired)
              {
                // Redirect to the change password editor.
                // This control will hide the user session so they can't go
                // anywhere else besides the change password page.
                result.SetLinkCreateNewSpace(PasswordAS.STR_MVC_CLASS_NAME,
                 null);
                result.SetControl(ExpiredPasswordControl.STR_MVC_CLASS_NAME);
                // Make this a true 302 so that the new user is fully logged in
                // when they get to the expired password control.  Otherwise things
                // like the login token cookie will not have been set yet.
                result.SetIsHTTPRedirect(true);
              } else
              if (bPasswordExpirationWarning)
              {
                // Redirect to the change password editor.
                // Since this is just a warning, we can go directly to the space and
                // not worry about hiding the user session.
                result.SetLinkCreateNewSpace(PasswordAS.STR_MVC_CLASS_NAME,
                 null);
                result.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
                result.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE,
                 PasswordModel.EDITOR_START_FLAG_EXPIRATION_WARNING);
                // Calculate number of days until the password expires.  This gets
                // truncated, so the number is not necessarily exactly correct
                // (i.e. 2.9 days is 2 days), but we don't really care about that.
                int nExpirationDays = (int) (msTimeUntilExpiration / (1000 * 60 * 60 * 24));
                if (nExpirationDays <= 0)
                {
                  // This catches the case where there is half a day left.
                  nExpirationDays = 1;
                }
                result.AddControlArgument(PasswordModel.PASSWORD_EXPIRATION,
                 nExpirationDays);
              }
              return result;
            } else
            {
              // Since this is a non-native user and their passwords don't expire, reset the expiration
              // time so that we don't check this for them for a while.
              SetPasswordChangedTimeToNow(_ptSession);
            }
          }
        }
      }
      // Valid unexpired password
      return null;
    }
    
    /// <summary>
    /// This helper method sets the Last Password Change property to the
    /// current time.
    /// </summary>
    /// <param name="ptSession">The current user.</param>
    private void SetPasswordChangedTimeToNow(IPTSession ptSession)
    {
      long lNow = XPSystem.GetCurrentTimeMilliSec();
      IPTObjectProperties props = ptSession.GetUser().GetObjectProperties();
      Object[][] propdata = new Object[2][];
      for (int j2c_index = 0;j2c_index < 2;j2c_index++)
      {
        propdata[j2c_index] = new Object[1];
      }
      propdata[PT_EDIT_PROPDATA_COLUMNS.PT_EDIT_PROPDATA_PROPERTYID][0] = PT_INTRINSICS.PT_PASSWORD_LAST_CHANGED;
      propdata[PT_EDIT_PROPDATA_COLUMNS.PT_EDIT_PROPDATA_VALUE][0] = lNow;
      props.SetPropertyData(propdata);
    }
    public virtual Redirect OnBeforeLogin(ApplicationData _appData, String _strUserName)
    {
      // Note that the user name will be "Guest" for the guest user and blank
      // for auto-login users.
      // This method cannot redirect internally for the guest user, only for
      // users that have already been logged in as guest.
      return null;
    }
    public virtual Redirect OnBeforeLogout(Object _oUserSession, ApplicationData _appData)
    {
      // TODO: This should get automatically cleared now
      // mj: clear the SSOInfo object off of the session
      ISessionManager sessionManager = _appData.GetSubSession();
      sessionManager.RemoveAttribute(PlumtreeExtensibility.PT_SSOINFO_ON_STATE);
      return null;
    }
    public virtual String OnFailedLogin(ApplicationData _appData, String _strUserName,
     XPException _xpError)
    {
      PerfCounters.INSTANCE.IncrementCounter(PORTALPAGESPERF_COUNTERS.PORTALPAGESPERF_LOGINS_FAILURE);
      return "";
    }
    
    /// <summary>
    /// This method gets a supported locale from the browser settings, and
    /// sets it on the http session and as a runtime setting.
    /// </summary>
    /// <param name="_appData"></param>
    /// <param name="_oUserSession"></param>
    /// <param name="_sessionManager"></param>
    /// <param name="_application"></param>
    /// <param name="_rm"></param>
    protected virtual void SetBrowserLocaleSetting(ApplicationData _appData,
     IPTSession _ptSession, ISessionManager _sessionManager, ISessionManager _persistentSessionManager,
     IApplication _application, XPResourceManager _rm)
    {
      String sBrowserLocaleList = _appData.GetHeader("accept-language");
      String strLocale = InternalGetBrowserLocale(sBrowserLocaleList, _application,
       _rm);
      if (strLocale != null)
      {
        log.Info("Setting user language based on supported locale of " + strLocale + " from accept-language header " + sBrowserLocaleList);
        InternalSetBrowserLocaleSetting(strLocale, _ptSession, _sessionManager,
         _persistentSessionManager, _application, _rm);
      }
    }
    
    /// <summary>
    /// This method gets a supported locale from the browser settings, but does
    /// not set it anywhere.
    /// </summary>
    /// <param name="_asOwner"></param>
    /// <returns>String The browser locale.</returns>
    public static String GetBrowserLocaleSetting(AActivitySpace _asOwner)
    {
      String sBrowserLocaleList = _asOwner.GetCurrentHTTPRequest().GetHeader("accept-language");
      return InternalGetBrowserLocale(sBrowserLocaleList, _asOwner.GetApplication(),
       _asOwner.GetResourceManager());
    }
    
    /// <summary>
    /// This method gets a supported locale from the browser settings, and
    /// sets it on the http session and as a runtime setting.
    /// </summary>
    /// <param name="_appData"></param>
    /// <param name="_oUserSession"></param>
    /// <param name="_sessionManager"></param>
    /// <param name="_application"></param>
    /// <param name="_rm"></param>
    protected static String InternalGetBrowserLocale(String sBrowserLocaleList,
     IApplication _application, XPResourceManager _rm)
    {
      String sBrowserLocale = null;
      if ((sBrowserLocaleList != null) && (0 != sBrowserLocaleList.Length))
      {
        try
        {
          // This method maps the accept header to a supported culture non-neutral locale
          // for us (non-neutral for .NET data formatting).
          sBrowserLocale = _rm.MapLocaleToUISupportedLocaleID(sBrowserLocaleList,
           PTPersonalSettingsHelper.GetDefaultLocale(_application));
        }
        catch (Exception e)
        {
          log.Error(e, "Unable to map locale to ui supported locale from accept-language header " + sBrowserLocaleList + ".");
          sBrowserLocale = null;
        }
        if ((null == sBrowserLocale) || (0 == sBrowserLocale.Length))
        {
          // We should never get here, because MapLocaleToUISupportedLocaleID
          // should always at least return the default locale, but just in case ...
          // This is worth considering, because if the locale is NOT set to a legit
          // value, the user is REALLY hosed- can't see anything.  Hence, even though 
          // we can "never" get here, it's STILL worth making triple sure we set the 
          // locale to something legit.
          log.Warn("Unable to map locale to ui supported locale from accept-language header " + sBrowserLocaleList + ". Using default system locale.");
          sBrowserLocale = PTPersonalSettingsHelper.GetDefaultLocale(_application);
          }
      } else
      {
        log.Warn("Unable to find browser locale list to map locale to ui supported locale from accept-language header " + sBrowserLocaleList + ". Using default system locale.");
        sBrowserLocale = PTPersonalSettingsHelper.GetDefaultLocale(_application);
        }
      return sBrowserLocale;
    }
    
    /// <summary>
    /// This method gets a supported locale from the browser settings, and
    /// sets it on the http session and as a runtime setting.
    /// </summary>
    /// <param name="_appData"></param>
    /// <param name="_oUserSession"></param>
    /// <param name="_sessionManager"></param>
    /// <param name="_application"></param>
    /// <param name="_rm"></param>
    protected static void InternalSetBrowserLocaleSetting(String sLocale,
     IPTSession _ptSession, ISessionManager _sessionManager, ISessionManager _persistentSessionManager,
     IApplication _application, XPResourceManager _rm)
    {
      // Some general rules/guidelines regarding locale settings
      //
      // 1) "Use Default", the zero length string, and not set should all act equivalently.
      // 2) If a non-guest/non-default profile user logs in with USE_DEFAULT or USE_BROWSER
      // set for it's locale, an appropriate locale will be determined, and then promoted
      // and stored in the DB as the user's new locale setting.
      // 3) There is a strict hierarchy of settings (i.e. it uses the user setting, but if
      // that's default, then it uses the XML setting, but if that's empty it uses the browser.
      // 4) guest user and default profile user locale settings never get promoted.
      // Do not store the locale setting the DB for the guest user or default profile users.
      if (LoginHelper.INSTANCE.GetIsGuestUser(_ptSession) || PlumtreeHelpers.IsEditingDefaultProfile(_persistentSessionManager))
      {
        // This will cache the locale for the UI and as a runtime setting, but not
        // save it to the DB.
        PTPersonalSettingsHelper.SetPersonalSetting(PersonalSettingNames.LOCALE,
         sLocale, _ptSession, _sessionManager, _application, _rm);
      } else
      {
        // This will cache the locale and language for the UI and as a
        // runtime setting, as well as store it in the DB.
        PTPersonalSettingsHelper.SetAndCachePersonalSetting(PersonalSettingNames.LOCALE,
         sLocale, _ptSession, _sessionManager, _application, _rm);
      }
    }
  }
}