using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.admin.serversettings.general;
using com.plumtree.portalpages.common.constants;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.portalpages.pei.util
{
  /// <summary>
  /// This helper class handles password validation for native users.
  /// </summary>
  public class PasswordValidator
  {
    
    /// <summary>
    /// Validate that the new password complies with the password management rules.
    /// </summary>
    /// <param name="_strNewPassword">The proposed new password.</param>
    /// <param name="_strUserName">The user name.</param>
    /// <param name="_asCurrentSpace">The current Activity Space.</param>
    /// <param name="_log">The logger to use for error logging.</param>
    /// <returns>An i18nized error string.  Null if the password successfully validates.</returns>
    public virtual String ValidatePassword(String _strNewPassword, String _strUserName,
     AActivitySpace _asCurrentSpace, OpenLogger _log)
    {
      IPTSession session = (IPTSession) _asCurrentSpace.GetUserSession();
      IPTServerConfigSettings config = (IPTServerConfigSettings) session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
       false);
      int nMinimumLength = config.GetSettingAsInt(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_LENGTH);
      if (_strNewPassword.Length < nMinimumLength)
      {
        _log.Info("New password is too short. Must be at least " + nMinimumLength + " characters.");
        return XPResourceManager.FormatMessage(_asCurrentSpace.GetString(1628,
         ResourceConstants.PORTALBROWSINGMSGS), nMinimumLength);
      }
      // Check new password length
      if (_strNewPassword.Length > SetServerSettingsModel.MAX_PASSWORD_LENGTH)
      {
        _log.Info("New password is too long.  Cannot be longer than " + SetServerSettingsModel.MAX_PASSWORD_LENGTH + ".");
        return _asCurrentSpace.GetString(1291, ResourceConstants.PORTALBROWSINGMSGS);
        }
      if (config.GetSettingAsBoolean(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_NO_USER_IDS))
      {
        if (XPStringUtility.IndexOf(XPStringUtility.ToLowerCase(_strNewPassword),
         XPStringUtility.ToLowerCase(_strUserName)) >= 0)
        {
          _log.Info("New password cannot contain user ID.");
          return _asCurrentSpace.GetString(1982, ResourceConstants.PORTALBROWSINGMSGS);
          }
      }
      if (config.GetSettingAsBoolean(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_CASING))
      {
        bool bUpperCase = false;
        bool bLowerCase = false;
        for (int i = 0;i < _strNewPassword.Length;i++)
        {
          char c = XPStringUtility.CharAt(_strNewPassword, i);
          // NOTE: This does not support i18nized character casing.
          // Character.isUpperCase(), etc... support this, but
          // OpenFoundation does not have isUpperCase.
          if ((c >= 'a') && (c <= 'z'))
          {
            bLowerCase = true;
          } else
          if ((c >= 'A') && (c <= 'Z'))
          {
            bUpperCase = true;
          }
        }
        if (!(bUpperCase && bLowerCase))
        {
          _log.Info("New password must contain both upper and lower case letters.");
          return _asCurrentSpace.GetString(1983, ResourceConstants.PORTALBROWSINGMSGS);
          }
      }
      if (config.GetSettingAsBoolean(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_NUMBERS))
      {
        bool bNumber = false;
        for (int i = 0;i < _strNewPassword.Length;i++)
        {
          char c = XPStringUtility.CharAt(_strNewPassword, i);
          if ((c >= '0') && (c <= '9'))
          {
            bNumber = true;
          }
        }
        if (!bNumber)
        {
          _log.Info("New password must contain a number.");
          return _asCurrentSpace.GetString(1984, ResourceConstants.PORTALBROWSINGMSGS);
          }
      }
      if (config.GetSettingAsBoolean(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_PUNCTUATION))
      {
        bool bPunctuation = false;
        for (int i = 0;i < _strNewPassword.Length;i++)
        {
          char c = XPStringUtility.CharAt(_strNewPassword, i);
          if ((c >= '!') && (c <= '/') || (c >= '[') && (c <= '`') || (c >= '{') && (c <= '~') || (c >= ':') && (c <= '@'))
          {
            bPunctuation = true;
          }
        }
        if (!bPunctuation)
        {
          _log.Info("New password must contain punctuation.");
          return _asCurrentSpace.GetString(1985, ResourceConstants.PORTALBROWSINGMSGS);
          }
      }
      int nReuseHistory = config.GetSettingAsInt(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_REUSE_HISTORY);
      if (nReuseHistory > 0)
      {
        IPTUserManager users = (IPTUserManager) session.GetUsers();
        bool bReused = users.IsPasswordInHistory(session.GetSessionInfo().GetCurrentUserID(),
         _strNewPassword, nReuseHistory);
        if (bReused)
        {
          _log.Info("New password was used previously.  Passwords cannot be reused for " + nReuseHistory + " times.");
          return XPResourceManager.FormatMessage(_asCurrentSpace.GetString(1998,
           ResourceConstants.PORTALBROWSINGMSGS), nReuseHistory);
        }
      }
      // The password appears to be valid.
      return null;
    }
  }
}