using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.rest;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portalpages.rest.dojo
{
  /// <summary>
  /// Abstract class for for QueryStoreModels
  /// A QueryStore is a dojo toolkit concept for potentially large dataset where smaller partial data chunks
  /// are lazily requested by the client. 
  /// Dojo QueryStore models will requests for pieces of the data store per request.
  /// Besides the main query, requests will also sends parameters for:
  /// - number of results to return
  /// - index to start at
  /// - property to sort results by
  /// - specific query 
  /// These parameters are set by the <c>QueryStoreControl</c> on the model with the 
  /// accessors defined on this interface. 
  /// USAGE:
  /// - QueryStoreControl or child control sets parameters parsed from the REST Request
  /// - DisplayPage calls GetQueryStoreResult 
  /// IMPLEMENTING CHILD MODEL
  /// - Implement GetQueryStoreResult to perform search / query with the results
  /// - Add property name to property id mapping by adding a static block to initialize mapping entries to  
  /// COLNAME_TO_PROPID_MAP
  /// </summary>
  /// <author>rozhang</author>
  public abstract class AQueryStoreModel : ARESTModel
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_REST),
     "com.plumtree.portalpages.rest.dojo.AQueryStoreModel");
    // Query store parameters
    // default search for all 
    private String m_strQuery = "*";
    // default size 
    private int m_resultSize = 20;
    private int m_startIndex = 0;
    
    /// <summary>
    /// negative column/property id for descending, positive for ascending *
    /// </summary>
    private int m_orderByColId = PT_INTRINSICS.PT_PROPERTY_OBJECTNAME;
    public static int INVALID_PROPID = -1;
    public static int NO_PROPTYPE = -1;
    public const int PROP_GRIDTYPE_ICON = 200;
    public const int PROP_GRIDTYPE_URL = 201;
    public const String COLNAME_PREFIX = "pid";
    // @TODO maybe refactor these constants to a more generic datastore class
    
    /// <summary>
    /// Property names for the JSON object returned in the response, don't change without also updating client javascript *
    /// </summary>
    public static readonly String COL_OBJECTID = COLNAME_PREFIX + PT_INTRINSICS.PT_PROPERTY_OBJECTID;
    public static readonly String COL_NAME = COLNAME_PREFIX + PT_INTRINSICS.PT_PROPERTY_OBJECTNAME;
    public static readonly String COL_DESCRIPTION = COLNAME_PREFIX + PT_INTRINSICS.PT_PROPERTY_OBJECTSUMMARY;
    public static readonly String COL_CREATED = COLNAME_PREFIX + PT_INTRINSICS.PT_PROPERTY_OBJECTCREATED;
    public static readonly String COL_LASTMODIFIED = COLNAME_PREFIX + PT_INTRINSICS.PT_PROPERTY_OBJECTLASTMODIFIED;
    //public static String COL_CHILDREN 				= "children"; // special flag for used with dojo tree widget 
    // @TODO maybe refactor these constants to a more generic datastore class 
    
    /// <summary>
    /// datastore response property containing the result set *
    /// </summary>
    public static String DATASTORE_PROP_ITEMS = "items";
    
    /// <summary>
    /// datastore response property for total result set (not just returned in the response *
    /// </summary>
    public static String DATASTORE_PROP_TOTALRESULTS = "numRows";
    
    /// <summary>
    /// datastore response property with the column/property name in each result entry that serves as the unqiue id *
    /// </summary>
    public static String DATASTORE_PROP_IDENTIFIER = "identifier";
    
    /// <summary>
    /// datastore response property with the column/property name in each result entry that serves as the label *
    /// </summary>
    public static String DATASTORE_PROP_LABEL = "label";
    
    /// <summary>
    /// Stores mapping between JSON object property name and search property ids *
    /// </summary>
    protected static XPHashtable COLNAME_TO_PROPID_MAP = new XPHashtable();
    
    /// <summary>
    /// Stores mapping of property ids and their types *
    /// </summary>
    protected static XPHashtable PROPID_PROPTYPE_MAP = new XPHashtable();
    public const String CUSTOMPROP_TYPE_SESSIONMAP_NAME = "QSMSERVICE_PROPTYPE_MAP";
    
    /// <summary>
    /// Lock to synchronize the PROPID_PROPTYPE_MAP hashtable *
    /// </summary>
    protected static Object PROPTYPE_LOCK = new Object();
    static AQueryStoreModel()
    {
      AddColNameToSearchPropIdMapping(COL_OBJECTID, PT_INTRINSICS.PT_PROPERTY_OBJECTID);
      AddColNameToSearchPropIdMapping(COL_NAME, PT_INTRINSICS.PT_PROPERTY_OBJECTNAME);
      AddColNameToSearchPropIdMapping(COL_DESCRIPTION, PT_INTRINSICS.PT_PROPERTY_OBJECTSUMMARY);
      AddColNameToSearchPropIdMapping(COL_CREATED, PT_INTRINSICS.PT_PROPERTY_OBJECTCREATED);
      AddColNameToSearchPropIdMapping(COL_LASTMODIFIED, PT_INTRINSICS.PT_PROPERTY_OBJECTCREATED);
      AddPropIdToPropTypeMapping(PT_INTRINSICS.PT_PROPERTY_OBJECTID, PT_PROPERTY_TYPES.PT_PROPTYPE_DOUBLE);
      AddPropIdToPropTypeMapping(PT_INTRINSICS.PT_PROPERTY_OBJECTNAME, PT_PROPERTY_TYPES.PT_PROPTYPE_STRING);
      AddPropIdToPropTypeMapping(PT_INTRINSICS.PT_PROPERTY_OBJECTSUMMARY,
       PT_PROPERTY_TYPES.PT_PROPTYPE_STRING);
      AddPropIdToPropTypeMapping(PT_INTRINSICS.PT_PROPERTY_OBJECTCREATED,
       PT_PROPERTY_TYPES.PT_PROPTYPE_DATE);
      AddPropIdToPropTypeMapping(PT_INTRINSICS.PT_PROPERTY_OBJECTLASTMODIFIED,
       PT_PROPERTY_TYPES.PT_PROPTYPE_DATE);
      AddPropIdToPropTypeMapping(PT_INTRINSICS.PT_PROPERTY_OBJECTIMAGEUUID,
       PROP_GRIDTYPE_ICON);
      AddPropIdToPropTypeMapping(PT_INTRINSICS.PT_PROPERTY_OBJECTIMAGEURL,
       PROP_GRIDTYPE_URL);
      // keywords
      AddPropIdToPropTypeMapping(101, PT_PROPERTY_TYPES.PT_PROPTYPE_STRING);
      // subject
      AddPropIdToPropTypeMapping(102, PT_PROPERTY_TYPES.PT_PROPTYPE_STRING);
      // author
      AddPropIdToPropTypeMapping(103, PT_PROPERTY_TYPES.PT_PROPTYPE_STRING);
      // created
      AddPropIdToPropTypeMapping(104, PT_PROPERTY_TYPES.PT_PROPTYPE_DATE);
      //doc title
      AddPropIdToPropTypeMapping(105, PT_PROPERTY_TYPES.PT_PROPTYPE_STRING);
      // url
      AddPropIdToPropTypeMapping(106, PROP_GRIDTYPE_URL);
      // category
      AddPropIdToPropTypeMapping(107, PT_PROPERTY_TYPES.PT_PROPTYPE_STRING);
      }
    
    /// <summary>
    /// Main method
    /// </summary>
    /// <returns></returns>
    public abstract XPHashtable GetQueryStoreResult();
    
    /// <summary>
    /// Helper to construct the DataStore response object
    /// </summary>
    /// <param name="dsData">The hashtable to set the properties on</param>
    /// <param name="dataItems"></param>
    /// <param name="totalRows"></param>
    protected virtual void AddDataStoreMetaData(XPHashtable dsData, IXPList dataItems,
     int totalRows)
    {
      dsData.PutElement(DATASTORE_PROP_ITEMS, dataItems);
      dsData.PutElement(DATASTORE_PROP_TOTALRESULTS, totalRows);
      dsData.PutElement(DATASTORE_PROP_IDENTIFIER, COL_OBJECTID);
      dsData.PutElement(DATASTORE_PROP_LABEL, COL_NAME);
    }
    
    /// <summary>
    /// Helper method to lookup property id from property name
    /// </summary>
    /// <param name="colName"></param>
    /// <returns>INVALID_PROPID (-1) if the column name was not found</returns>
    public virtual int GetSearchPropIdFromColumnName(String colName)
    {
      if (colName == null)
      {
        return INVALID_PROPID;
      }
      Object oPropId = COLNAME_TO_PROPID_MAP.GetElement(colName);
      if (oPropId == null)
      {
        return INVALID_PROPID;
      } else
      {
        return XPConvert.ToInteger(oPropId);
      }
    }
    
    /// <summary>
    /// Helper to add property name to property id mapping entry
    /// </summary>
    /// <param name="colName"></param>
    /// <param name="searchPropId"></param>
    /// <throws>XPIllegalStateException if mapping already exists</throws>
    protected static void AddColNameToSearchPropIdMapping(String colName,
     int searchPropId)
    {
      if (COLNAME_TO_PROPID_MAP.ContainsKey(colName))
      {
        throw new XPIllegalStateException("Attempting to add mapping with name: " + colName + " with value" + searchPropId + "but the mapping already exists with value: " + COLNAME_TO_PROPID_MAP.GetElement(colName));
        }
      COLNAME_TO_PROPID_MAP.PutElement(colName, searchPropId);
      COLNAME_TO_PROPID_MAP.PutElement("-" + colName, -searchPropId);
    }
    
    /// <summary>
    /// Helper to add property id to property type mapping entry
    /// </summary>
    /// <param name="propId"></param>
    /// <param name="propType"></param>
    /// <throws>XPIllegalStateException if mapping already exists</throws>
    protected static void AddPropIdToPropTypeMapping(int propId, int propType)
    {
      PROPID_PROPTYPE_MAP.PutElement(propId, propType);
    }
    protected static int GetPropType(int propId, AActivitySpace space)
    {
      XPHashtable curr = PROPID_PROPTYPE_MAP;
      int oPropId = propId;
      if (curr.ContainsKey(oPropId))
      {
        return XPConvert.ToInteger(curr.GetElement(oPropId));
      } else
      {
        return NO_PROPTYPE;
      }
    }
    
    /// <summary>
    /// Custom properties are cached on the user session
    /// Only cache types for custom properties are stored since the
    /// default doc properties don't change
    /// </summary>
    /// <param name="propId"></param>
    /// <param name="space"></param>
    /// <returns></returns>
    protected static int GetCustomPropType(int propId, AActivitySpace space)
    {
      // cache the custom proptypes on user's session so that we don't have to requery
      // for them each request. 
      ISessionManager userSession = space.GetSubSession();
      Object sessionMap = userSession.GetAttribute(CUSTOMPROP_TYPE_SESSIONMAP_NAME);
      if (sessionMap == null)
      {
        sessionMap = new XPHashtable();
        userSession.SetAttribute(CUSTOMPROP_TYPE_SESSIONMAP_NAME, sessionMap);
        }
      XPHashtable curr = (XPHashtable) sessionMap;
      int oPropId = propId;
      if (curr.ContainsKey(oPropId))
      {
        return XPConvert.ToInteger(curr.GetElement(oPropId));
      } else
      {
        int type = NO_PROPTYPE;
        try
        {
          IPTSession ptSession = ((IPTSession) space.GetUserSession());
          IPTObjectManager propMgr = ptSession.GetProperties();
          IPTProperty propObj = (IPTProperty) propMgr.Open(propId, false);
          type = propObj.GetType();
          curr.PutElement(oPropId, type);
        }
        catch (XPException e)
        {
          // default to string if property was removed or not found due to some other error
          type = PT_PROPERTY_TYPES.PT_PROPTYPE_STRING;
          log.Warn(e, "Unable to find property with id: " + propId + ". Defaulting to type as string.");
          }
        return type;
      }
    }
    
    /// <summary>
    /// Getters and setters for common properties
    /// </summary>
    public virtual String GetQuery()
    {
      return m_strQuery;
    }
    public virtual void SetQuery(String query)
    {
      m_strQuery = query;
    }
    public virtual int GetResultSize()
    {
      return m_resultSize;
    }
    public virtual void SetResultSize(int resultSize)
    {
      m_resultSize = resultSize;
    }
    public virtual int GetStartIndex()
    {
      return m_startIndex;
    }
    public virtual void SetStartIndex(int startIndex)
    {
      m_startIndex = startIndex;
    }
    public virtual int GetOrderById()
    {
      return m_orderByColId;
    }
    public virtual void SetOrderById(int orderByColId)
    {
      m_orderByColId = orderByColId;
    }
  }
}