using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.rest;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.application.varpacks;
namespace com.plumtree.portalpages.rest.ldap
{
  /// <summary>
  /// Model that looks up user object id from LDAP auth name
  /// </summary>
  /// <author>rozhang</author>
  public class UserIdLookupModel : ARESTModel,IModel
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_REST),
     "com.plumtree.portalpages.rest.ldap.UserIdLookupModel");
    public const String STR_MVC_CLASS_NAME = "UserIdLookupModel";
    private static bool REMOTEAUTHSRC_QUERIED = false;
    // lock object for flag
    private static Object QUERY_LOCK = new Object();
    // fix for ensemble bug #10241080, return 2 instead of -1 so that ensemble will
    // authenticate as quest user for kdportlet request. -1 was causing 500 error
    // in ensemble rc6
    public const int NO_USERID_FOUND = 2;
    protected AActivitySpace m_space;
    protected String m_authName = null;
    private static String m_defaultAuthenticationSourcePrefix = null;
    
    /// <summary>
    /// Main method to lookup user id from auth name set by control
    /// </summary>
    /// <returns></returns>
    public virtual int GetUserIdFromCurrentAuthName()
    {
      return LookupUserIdFromAuthName(GetAuthName());
    }
    
    /// <param name="authName">LDAP authentication name (without Portal Auth source prefix)</param>
    /// <returns></returns>
    protected virtual int LookupUserIdFromAuthName(String authName)
    {
      IPTSession session = (IPTSession) m_space.GetUserSession();
      String defaultAuthSrcPrefix = GetSingleRemoteAuthSourceObjectId(session);
      // if there are more than one remote auth source
      if (defaultAuthSrcPrefix == null)
      {
        ConfigVarPack configVarPack = (ConfigVarPack) m_space.GetApplication().GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);
        defaultAuthSrcPrefix = configVarPack.GetConfigValueAsString(ConfigConstants.AUTHENTICATION_SECTION,
         "DefaultAuthSourcePrefix", ConfigConstants.VALUE_ATTRIBUTE);
        if (defaultAuthSrcPrefix == null || "".Equals(defaultAuthSrcPrefix))
        {
          log.Error("More than one Authentication Sources were found but no default prefix was set in portalconfig.xml (DefaultAuthSourcePrefix setting). Reverting to Guest User.");
          return NO_USERID_FOUND;
        }
        if (log.IsDebugEnabled())
        {
          log.Debug("Default Authentication Source prefix configured to " + defaultAuthSrcPrefix);
          }
      } else
      {
        if (log.IsDebugEnabled())
        {
          log.Debug("Only a single Authentication Source with prefix " + defaultAuthSrcPrefix + " was found. ");
          }
      }
      String fullLoginName = defaultAuthSrcPrefix + "\\" + authName;
      IPTObjectManager userMgr = session.GetUsers();
      int[] userId = null;
      try
      {
        userId = userMgr.LookupObjectIdFromName(fullLoginName);
      }
      catch (Exception e)
      {
        // didn't find name, don't do anything
        log.Debug(e, "Not able to lookup user id for user with name: " + fullLoginName);
        }
      if (userId != null && userId.Length == 1)
      {
        return userId[0];
      } else
      {
        log.Info("User id not found for user " + fullLoginName);
        return NO_USERID_FOUND;
      }
    }
    
    /// <param name="session"></param>
    /// <returns>null if more than one remote auth source was found</returns>
    private String GetSingleRemoteAuthSourceObjectId(IPTSession session)
    {
      if (XPConvert.ToBoolean(REMOTEAUTHSRC_QUERIED))
      {
        return m_defaultAuthenticationSourcePrefix;
      } else
      {
        int nPropIDsToQuery = PT_PROPIDS.PT_PROPID_NAME | PT_PROPIDS.PT_PROPID_AUTHSOURCE_PREFIXSTRING | PT_PROPIDS.PT_PROPID_AUTHSOURCE_AUTHTYPE;
        Object[][] ptqf = new Object[3][];
        for (int j2c_index = 0;j2c_index < 3;j2c_index++)
        {
          ptqf[j2c_index] = new Object[1];
        }
        ptqf[0][0] = PT_PROPIDS.PT_PROPID_AUTHSOURCE_AUTHTYPE;
        ptqf[1][0] = PT_FILTEROPS.PT_FILTEROP_EQ;
        ptqf[2][0] = PT_AUTHTYPES.PT_AUTHTYPE_EXTENSIBLE;
        int[][] arrSort = new int[1][];
        for (int j2c_index = 0;j2c_index < 1;j2c_index++)
        {
          arrSort[j2c_index] = new int[2];
        }
        /*PTSAW_SORT_ROW_PROPID*/
        arrSort[0][0] = PT_PROPIDS.PT_PROPID_OBJECTID;
        /*PTSAW_SORT_ROW_DIRECTION*/
        arrSort[0][1] = 1;
        lock (QUERY_LOCK) {
          // check if someone initialized after we checked
          if (XPConvert.ToBoolean(REMOTEAUTHSRC_QUERIED))
          {
            return m_defaultAuthenticationSourcePrefix;
          }
          IPTObjectManager authSrcMan = session.GetAuthSources();
          IPTQueryResult qr = authSrcMan.Query(nPropIDsToQuery, -1, arrSort,
           0, -1, ptqf);
          if (qr.RowCount() == 1)
          {
            m_defaultAuthenticationSourcePrefix = qr.ItemAsString(0, PT_PROPIDS.PT_PROPID_AUTHSOURCE_PREFIXSTRING);
            }
          REMOTEAUTHSRC_QUERIED = true;
        }
        // end of synchronized block
        return m_defaultAuthenticationSourcePrefix;
      }
    }
    public virtual void SetAuthName(String name)
    {
      m_authName = name;
    }
    public virtual String GetAuthName()
    {
      return m_authName;
    }
    /*
     * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
     */
     
    public virtual void CleanupAllData()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()"></seealso>
    public virtual void CleanupTempData()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#Init(com.plumtree.uiinfrastructure.activityspace.AActivitySpace)"></seealso>
    public virtual void Init(AActivitySpace parent)
    {
      m_space = parent;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new UserIdLookupModel();
    }
  }
}