using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portalpages.rest;
using com.plumtree.portalpages.rest.utils;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.taskapi.portalui;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.login;
namespace com.plumtree.portalpages.rest.user
{
  /// <summary>
  /// This control handles processing the User REST API requests.
  /// </summary>
  public class UserRESTControl : IControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_REST),
     "com.plumtree.portalpages.rest.user.UserRESTControl");
    public const String STR_MVC_CLASS_NAME = "UserREST";
    
    /// <summary>
    /// The control argument for the user ID passed in from the friendly URL.
    /// </summary>
    public const String USER_ID = "user";
    
    /// <summary>
    /// The default value - current user.
    /// </summary>
    public const String CURRENT_USER = "current";
    
    /// <summary>
    /// The owning space for this control.
    /// </summary>
    protected AActivitySpace m_asOwner;
    
    /// <summary>
    /// The User REST model.
    /// </summary>
    protected IUserRestModel m_model;
    public virtual Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      try
      {
        // Users must be logged in to access this API.  The only time guest
        // users can use the API is when an admin user is editing the guest
        // profile.
        if (LoginHelper.INSTANCE.GetIsGuestUser(m_asOwner.GetUserSession()) && !TaskAPIUICommon.IsCurrentPageInEditDefaultProfileMode(m_asOwner))
        {
          log.Warn("User must be logged in to access the User REST API.");
          // Must be logged in to use this API. 
          return ProcessError(CommonRestErrors.NOT_LOGGED_IN);
        }
        // This originally comes from the friendly URL and is put into this format.
        String[] sValues = (String[]) arguments.GetElement(USER_ID);
        if ((sValues == null) || (sValues.Length == 0))
        {
          log.Warn("Could not find user ID in User REST API URL.");
          return ProcessError(UserRestErrors.INVALID_USER_ID);
        }
        String strUserID = sValues[0];
        // Current user is the only valid ID supported right now.
        if (XPStringUtility.CompareIgnoreCase(CURRENT_USER, strUserID) != 0)
        {
          log.Warn("Found invalid user ID in User REST API URL.  Expecting " + CURRENT_USER + ", but found [" + strUserID + "].  This value is not " + "currently supported.");
          return ProcessError(UserRestErrors.INVALID_USER_ID);
        }
        // NOTE: This is currently fairly secure because it only accepts POSTs.
        // However, if in the future this accepts GETs and other verbs, then we
        // need to be more careful.  If a user is logged into the portal and
        // goes to a hacker site, that site can use a javascript src include tag
        // on their page to the portal JSON page.  This script request (a GET)
        // will be passed onto portal with the user's portal cookie, allowing them
        // to access the data.  We are safe from this right now because the GET
        // request will be ignored by the control.
        // One way to secure against this kind of attack is to require the JSON request
        // to generate a hash of the contents of the login cookie and add it to the
        // request as an argument of some sort.
        IXPRequest request = m_asOwner.GetCurrentHTTPRequest();
        String strMethod = request.GetMethod();
        if (XPStringUtility.CompareIgnoreCase(strMethod, "POST") != 0)
        {
          log.Warn("This User REST API only supports the POST method.");
          // This KD REST API only supports the POST method.
          return ProcessError(CommonRestErrors.NOT_POST);
        }
        // Get the JSON input and parse it.
        // NOTE: We currently only support JSON POSTs, so we assume all requests
        // are the same.  We could eventually support GETs and XML as well.
        if (request.GetContentLength() == 0)
        {
          log.Warn("This User REST API requires POST content, but none was found.");
          // This User REST API requires POST content, but none was found.
          return ProcessError(CommonRestErrors.NO_POST_CONTENT);
        }
        // Get the POST body JSON string.
        String strJSON = new RESTHelper().GetPOSTBody(request);
        if ((strJSON == null) || (strJSON.Length == 0))
        {
          log.Warn("This User REST API requires POST content, but none was found.");
          // This User REST API requires POST content, but none was found.
          return ProcessError(CommonRestErrors.NO_JSON_POST_CONTENT);
        }
        IJSONParser parser = JSONParser.GetInstance();
        parser.ParseJSONObject(strJSON);
        if (XPStringUtility.CompareIgnoreCase(parser.GetObjectName(), UserRESTConstants.USER_FORMAT_NAME) != 0)
        {
          log.Warn("This User REST API only supports user objects, not " + parser.GetObjectName() + " objects.");
          // This User REST API only supports document objects.
          return ProcessError(UserRestErrors.NO_JSON_USER_OBJECT);
        }
        // Parse the JSON and execute the command.
        Redirect error = ProcessAction(parser);
        if (error != null)
        {
          return error;
        }
      }
      catch (Exception e)
      {
        log.Error(e, "Error processing User REST API request.");
        // General error.
        return ProcessError(CommonRestErrors.GENERAL);
      }
      // Redirect to DirRESTDP.
      return GetDPRedirect();
    }
    
    /// <summary>
    /// This method handles parsing the JSON body and executing the
    /// appropriate command.
    /// </summary>
    /// <param name="_parser">The parser with the JSON object already loaded.</param>
    /// <returns>An error redirect, if necessary.  Null if there were no errors.</returns>
    public virtual Redirect ProcessAction(IJSONParser _parser)
    {
      // Assume this is for the current user.
      // Join communities
      String strJoinCommunityID = _parser.GetMember(UserRESTConstants.USER_JOIN_COMMUNITIES);
      if (strJoinCommunityID != null)
      {
        int nJoinCommunityID;
        try
        {
          nJoinCommunityID = XPConvert.ToInteger(strJoinCommunityID);
        }
        catch (Exception e)
        {
          log.Error(e, "Error retrieving ID of community to join from JSON data.");
          return ProcessError(UserRestErrors.NO_JOIN_COMMUNITY_ID);
        }
        try
        {
          m_model.JoinCommunity(nJoinCommunityID);
        }
        catch (Exception e)
        {
          log.Error(e, "Error joining community " + nJoinCommunityID + ".");
          return ProcessError(UserRestErrors.JOIN_COMMUNITY_ERROR);
        }
      }
      // Leave communities
      String strLeaveCommunityID = _parser.GetMember(UserRESTConstants.USER_LEAVE_COMMUNITIES);
      if (strLeaveCommunityID != null)
      {
        int nLeaveCommunityID;
        try
        {
          nLeaveCommunityID = XPConvert.ToInteger(strLeaveCommunityID);
        }
        catch (Exception e)
        {
          log.Error(e, "Error retrieving ID of community to leave from JSON data.");
          return ProcessError(UserRestErrors.NO_LEAVE_COMMUNITY_ID);
        }
        try
        {
          m_model.LeaveCommunity(nLeaveCommunityID);
        }
        catch (Exception e)
        {
          log.Error(e, "Error leaving community " + nLeaveCommunityID + ".");
          return ProcessError(UserRestErrors.LEAVE_COMMUNITY_ERROR);
        }
      }
      // No errors
      return null;
    }
    
    /// <summary>
    /// This helper method sets the error on the model and redirects to the
    /// REST API display page.
    /// </summary>
    /// <param name="_nErrorID">the current error code.</param>
    /// <returns>The redirect to the User REST API display page.</returns>
    public virtual Redirect ProcessError(IRESTError _error)
    {
      Redirect redirect = GetDPRedirect();
      m_model.SetError(_error);
      return redirect;
    }
    
    /// <returns>the redirect to the User REST API display page.</returns>
    public virtual Redirect GetDPRedirect()
    {
      Redirect redirect = new Redirect();
      redirect.SetLinkGetCachedSpace(m_asOwner);
      redirect.SetDisplayPage(UserRESTDP.STR_MVC_CLASS_NAME);
      return redirect;
    }
    public virtual void Init(IModel model, AActivitySpace space)
    {
      m_asOwner = space;
      m_model = (IUserRestModel) model;
    }
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    public virtual Object Create()
    {
      return new UserRESTControl();
    }
  }
}