using System;
using com.plumtree.openfoundation.util;
using com.plumtree.taskapi.portalui;
namespace com.plumtree.portalpages.rest.utils
{
  /// <author>
  /// rozhang
  /// Serializes an XPHashtable or IXP list to JSON notation
  /// </author>
  public class JSONSerializer
  {
    //public static String[] INVALID_JSPROPNAME_CHARS = new String[] { "-", "&", "\\", "/", "#", "%, "", "!"};
    protected XPHashtable m_object = null;
    public JSONSerializer(XPHashtable jsonObject)
    {
      m_object = jsonObject;
    }
    
    /// <summary>
    /// Main parsing function
    /// </summary>
    /// <param name="addPrefix">
    /// Protects the JSON string from rewrites/pollution/hijacking hacks by prefixing
    /// the generated JSON with '{} && '. The prefix is skipped by the eval function
    /// when interpreting the JSON as data, i.e. transforming the JSON into an object
    /// with properties but turns the JSON into unexecutable javascript to protect against
    /// various CSRF vunerabilities
    /// </param>
    /// <returns></returns>
    public virtual String GenerateStringRepresentation(bool addPrefix)
    {
      XPStringBuilder buf = new XPStringBuilder();
      // add json hacking protection prefix 
      buf.Append((addPrefix) ? "{} && " : "");
      buf.Append(ParseHashTable(m_object, 1));
      return buf.ToString();
    }
    
    /// <summary>
    /// Generate JSON String representation for node with child nodes.
    /// The generated output will be in the following format:
    /// {
    /// property1: value2,
    /// property2: value2,
    /// property3: value3,
    /// ...
    /// }
    /// </summary>
    /// <param name="map"></param>
    /// <param name="level">The depth from the top where top is 0</param>
    /// <returns></returns>
    protected virtual String ParseHashTable(XPHashtable map, int level)
    {
      XPStringBuilder buf = new XPStringBuilder();
      buf.Append(AddPadding(level - 1)).Append("{\n");
      // iterate over the xphashtable
      IXPEnumerator keys = map.GetKeys();
      bool isFirstEntry = true;
      while (keys.MoveNext())
      {
        if (!isFirstEntry)
        {
          buf.Append(",\n");
        }
        String curName = TaskAPIUIJavascript.EscapeJavaScriptString((String) keys.GetCurrent());
        Object val = map.GetElement(curName);
        if (val is XPHashtable)
        {
          buf.Append(AddPadding(level)).Append(curName).Append(":");
          buf.Append(ParseHashTable((XPHashtable) val, level + 1));
        } else
        if (val is XPArrayList)
        {
          buf.Append(AddPadding(level)).Append(curName).Append(":\n");
          buf.Append(ParseArray((XPArrayList) val, level + 1));
        } else
        {
          buf.Append(AddPadding(level)).Append(curName).Append(":");
          buf.Append(" ").Append(GetStringValue(val, curName));
        }
        isFirstEntry = false;
      }
      buf.Append("\n").Append(AddPadding(level - 1)).Append("}");
      return buf.ToString();
    }
    
    /// <summary>
    /// Generate JSON string representation for Array node
    /// The generated output will be in the following format:
    /// [
    /// value1, value2, value3, ...
    /// ]
    /// </summary>
    /// <param name="values"></param>
    /// <param name="level"></param>
    /// <returns></returns>
    protected virtual String ParseArray(IXPList values, int level)
    {
      XPStringBuilder buf = new XPStringBuilder();
      // want the brackets to display in parent's level
      buf.Append(AddPadding(level)).Append("[ ");
      bool addIndent = false;
      for (int i = 0;i < values.GetSize();i++)
      {
        if (i != 0)
        {
          buf.Append((addIndent) ? AddPadding(level + 1) : " ");
          buf.Append(",");
          addIndent = false;
        }
        Object val = values.GetElement(i);
        if (val is XPHashtable)
        {
          buf.Append("\n");
          buf.Append(ParseHashTable((XPHashtable) val, level + 1));
        } else
        if (val is XPArrayList)
        {
          buf.Append("\n");
          buf.Append(ParseArray((XPArrayList) val, level + 1));
          buf.Append("\n");
          addIndent = true;
        } else
        // add support for arrays
        //			else if ( val instanceof Object[] ) 
        //			{
        //				buf.Append("\n");
        //				buf.Append(  ParseArray((Object[])val, level+1 ) );
        //				buf.Append("\n");
        //				addIndent = true;
        //			} 
        {
          buf.Append(GetStringValue(val, null));
        }
      }
      buf.Append(AddPadding(level - 1)).Append(" ]");
      return buf.ToString();
    }
    
    /// <summary>
    /// Helper for adding the proper amount of indentation
    /// </summary>
    /// <param name="level"></param>
    /// <returns></returns>
    protected virtual String AddPadding(int level)
    {
      XPStringBuilder buf = new XPStringBuilder();
      for (int i = 0;i < level;i++)
      {
        // 2 spaces
        buf.Append("  ");
      }
      return buf.ToString();
    }
    
    /// <summary>
    /// Returns JSON string representation of a value. 
    /// Strings are escape encoded
    /// </summary>
    /// <param name="val"></param>
    /// <param name="name"></param>
    /// <returns></returns>
    protected virtual String GetStringValue(Object val, String name)
    {
      String valString = "";
      if (val is String || val is char)
      {
        valString = "\"" + TaskAPIUIJavascript.EscapeJavaScriptString(val.ToString()) + "\"";
        } else
      if (val is int || val is float || val is double || val is long)
      {
        valString+=val;
      } else
      if (val is bool)
      {
        valString+=val;
      } else
      if (val is XPDateTime)
      {
        // dates are converted to millisecs since utc sent as a number
        valString+=XPConvert.ToString(((XPDateTime) val).GetTimeInMilliSeconds());
        } else
      if (val == null)
      {
        valString = "null";
      } else
      {
        if (name == null)
        {
          throw new XPIllegalArgumentException("Unsupported value type found for value of type: " + val);
          } else
        {
          throw new XPIllegalArgumentException("Unsupported value type found for value " + name + " of type: " + val);
          }
      }
      return valString;
    }
  }
}
//	protected boolean ValidateJSPropertyName(String propName)
//	{
//		// @TODO work in progress - add invalid variable/property name strings
//		
//		XPArrayList errors = new XPArrayList();
//		for ( int i = 0; i < INVALID_JSPROPNAME_CHARS.length; i++ ) {
//			if ( propName.indexOf(INVALID_JSPROPNAME_CHARS[i]) != -1 ) {
//				throw new XPIllegalArgumentException("Invalid Javascript property name: "+propName+". Contains following invalid characters: "+INVALID_JSPROPNAME_CHARS[i] );
//			}
//		}
//		return true;
//		
//	}
