using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portalpages.browsing.directory;
using com.plumtree.portaluiinfrastructure.ptlink;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.taskapi.portalui
{
  public interface ITaskAPIUIDirectory
  {
    
    /// <summary>
    /// Checks if the Knowledge Directory feature is enabled in the current Experience Definition.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> Knowledge Directory feature is enabled, <c>false</c> otherwise.</returns>
    bool IsDirectoryFeaturedEnabledInSubPortal(AActivitySpace owner);
    
    /// <summary>
    /// Gets the URL to the specified Knowledge Directory in Browse Mode.  
    /// The folder will not be accessible if the current user does not have sufficient access to it. 
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nFolderID">Object ID of a Knowledge Directory folder to create an URL to.</param>
    /// <returns>
    /// 		An <c>IPTURL</c> with the URL to the specified folder in Knowledge Directory Browse Mode,
    /// or <c>null</c> if the URL is not valid in the current context.
    /// </returns>
    IPTURL GetURLForBrowseDirectory(AActivitySpace owner, int nFolderID);
    
    /// <summary>
    /// Gets the URL to the specified Knowledge Directory in Edit Mode.  
    /// The folder will not be accessible if the current user does not have sufficient access it.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nFolderID">Object ID of a Knowledge Directory folder to create an URL to.</param>
    /// <returns>
    /// 		An <c>IPTURL</c> with the URL to the specified folder in Knowledge Directory Edit Mode,
    /// or <c>null</c> if the URL is not valid in the current context.
    /// </returns>
    IPTURL GetURLForEditDirectory(AActivitySpace owner, int nFolderID);
    
    /// <summary>
    /// Gets an URL to open a specified Knowledge Directory folder in the selected mode.
    /// The folder will not be accessible if the current user does not have sufficient access it.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="strTitle">Title of the URL, normally the name of the folder to open.</param>
    /// <param name="nDirMode">DirConstants.BROWSE_MODE or DirConstants.EDIT_MODE</param>
    /// <param name="nFolderID">Object ID of the Knowledge Directory Folder to get an URL to.</param>
    /// <returns>An <c>IPTURL</c> to a folder in the Knowledge Directory - no image is assigned to the URL.</returns>
    /// <seealso cref="DirConstants"></seealso>
    IPTURL CreateCommonOpenerDirectoryFolderURL(AActivitySpace owner, String strTitle,
     int nDirMode, int nFolderID);
    
    /// <summary>
    /// Gets an URL to open a specified Knowledge Directory folder in the selected mode.
    /// The folder will not be accessible if the current user does not have sufficient access it.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="strTitle">Title of the URL, normally the name of the folder to open.</param>
    /// <param name="nDirMode">DirConstants.BROWSE_MODE or DirConstants.EDIT_MODE</param>
    /// <param name="nFolderID">Object ID of the Knowledge Directory Folder to get an URL to.</param>
    /// <returns>An <c>IPTURL</c> to a folder in the Knowledge Directory - no image is assigned to the URL.</returns>
    /// <seealso cref="DirConstants"></seealso>
    IPTURL CreateCommonOpenerDirectoryFolderURL(AActivitySpace owner, String strTitle,
     int nDirMode, int nFolderID, String strFolderName);
    
    /// <summary>
    /// Gets an URL to open a specified Document in the Knowledge Directory.
    /// The document will not be accessible if the current user does not have sufficient access it.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="strTitle">Title of the URL, normally the name of the Document to open.</param>
    /// <param name="nDocumentID">Object ID of the Document to get an URL to.</param>
    /// <returns>	</returns>
    IPTURL CreateCommonOpenerDocumentURL(AActivitySpace owner, String strTitle,
     int nDocumentID);
    
    /// <summary>
    /// Gets an URL to open a specified Document in the Knowledge Directory.
    /// The document will not be accessible if the current user does not have sufficient access it.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="strTitle">Title of the URL, normally the name of the Document to open.</param>
    /// <param name="nDocumentID">Object ID of the Document to get an URL to.</param>
    /// <param name="strDocumentName">Name of the document.</param>
    /// <returns>	The URL to the document</returns>
    IPTURL CreateCommonOpenerDocumentURL(AActivitySpace owner, String strTitle,
     int nDocumentID, String strDocumentName);
    
    /// <summary>
    /// Checks if the current page is a Knowledge Directory page.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current page us a Knowledge Directory page, <c>false</c> otherwise.</returns>
    bool IsCurrentPageDirectoryPage(AActivitySpace owner);
    
    /// <summary>
    /// Gets the Data Sources used for submitting documents to the Knowledge Directory.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// the list of Data Sources used for submitting documents to the Knowledge
    /// Directory.  Each entry in the list will be a HashTableDataObject with entries
    /// for "name" and "id", as described in the SubmitDataSourceConstants class.
    /// </returns>
    IXPList GetSubmitDataSources(AActivitySpace owner);
    
    /// <summary>
    /// Adds the upload data for the Web Services associated with the
    /// Data Sources used for submitting documents to the Knowledge Directory.
    /// This will also filter the list for invalid submit data sources.  Data sources
    /// should be either web submit, or remote file upload.  Any other kinds of data
    /// sources will be removed from the list.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="_sources">
    /// List of HashtableDataObjects of the Data Sources to lookup.
    /// This is of the same format as the result of the GetSubmitDataSources method.
    /// </param>
    /// <returns>
    /// the original list of Data Sources with additional data added to each entry.
    /// The entries in the list will have additional entries for "type" (file or web),
    /// "url" (remote upload URL), and "wsid" (ID of the associated web service), as
    /// described in the SubmitDataSourceConstants class.  "url" will be null for
    /// data sources that don't support remote upload.
    /// </returns>
    IXPList AddUploadData(AActivitySpace owner, IXPList _sources);
    
    /// <summary>
    /// Check if the provider for this data source supports web paths.
    /// </summary>
    /// <param name="_dataSource">The data source to check.</param>
    /// <param name="_session">the current user session</param>
    /// <returns>whether or not this data source supports web paths.</returns>
    bool IsWebDataSource(IPTDataSource _dataSource, IPTSession _session);
    }
}