/*
 */
 
using System;
using com.plumtree.portalpages.browsing.directory;
using com.plumtree.portaluiinfrastructure.opener;
using com.plumtree.portaluiinfrastructure.ptlink;
using com.plumtree.server;
using com.plumtree.taskapi.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.taskapi.portalui
{
  /// <summary>
  /// Knowledge Directory related TaskAPI functions.
  /// Use TaskAPIDirectoryService.GetInstance() instead.
  /// </summary>
  public class TaskAPIUIDirectory
  {
    
    /// <summary>
    /// Checks if the Knowledge Directory feature is enabled in the current Experience Definition.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> Knowledge Directory feature is enabled, <c>false</c> otherwise.</returns>
    public static bool IsDirectoryFeaturedEnabledInSubPortal(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      return TaskAPIServerSubPortal.IsSubPortalFeatureEnabled(ptSession, PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_DIRECTORY_ACCESS);
      }
    
    /// <summary>
    /// Gets the URL to the specified Knowledge Directory in Browse Mode.  
    /// The folder will not be accessible if the current user does not have sufficient access to it. 
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nFolderID">Object ID of a Knowledge Directory folder to create an URL to.</param>
    /// <returns>
    /// 		An <c>IPTURL</c> with the URL to the specified folder in Knowledge Directory Browse Mode,
    /// or <c>null</c> if the URL is not valid in the current context.
    /// </returns>
    public static IPTURL GetURLForBrowseDirectory(AActivitySpace owner, int nFolderID)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTURL url = null;
      if (IsDirectoryFeaturedEnabledInSubPortal(owner) && !TaskAPIUICommon.IsCurrentPageInPreviewMode(owner))
      {
        String strTitle = owner.GetString(277, "ptmsgs_portalcommonmsgs");
        String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
        int nDirMode = DirConstants.BROWSE_MODE;
        url = CreateCommonOpenerDirectoryFolderURL(owner, strTitle, nDirMode,
         nFolderID);
        url.SetURLData(PTURLConstants.KEY_ICONIMG, strImageServerURL + PTImageType.ICON_BROWSE_MENU.GetFileName());
        url.SetURLData(PTURLConstants.KEY_ICONIMGHEIGHT, PTImageType.ICON_BROWSE_MENU.GetHeight());
        url.SetURLData(PTURLConstants.KEY_ICONIMGWIDTH, PTImageType.ICON_BROWSE_MENU.GetWidth());
        }
      return url;
    }
    
    /// <summary>
    /// Gets the URL to the specified Knowledge Directory in Edit Mode.  
    /// The folder will not be accessible if the current user does not have sufficient access it.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nFolderID">Object ID of a Knowledge Directory folder to create an URL to.</param>
    /// <returns>
    /// 		An <c>IPTURL</c> with the URL to the specified folder in Knowledge Directory Edit Mode,
    /// or <c>null</c> if the URL is not valid in the current context.
    /// </returns>
    public static IPTURL GetURLForEditDirectory(AActivitySpace owner, int nFolderID)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTURL url = null;
      if (IsDirectoryFeaturedEnabledInSubPortal(owner) && !TaskAPIUICommon.IsCurrentPageInPreviewMode(owner))
      {
        if (TaskAPIUIUser.IsAccessStyleStandard(owner) && TaskAPIUIUser.HasAccessToActivityRight(owner,
         PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_EDIT_DOCUMENT_DIRECTORY))
        {
          String strTitle = owner.GetString(279, "ptmsgs_portalcommonmsgs");
          String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
          int nDirMode = DirConstants.EDIT_MODE;
          url = CreateCommonOpenerDirectoryFolderURL(owner, strTitle, nDirMode,
           nFolderID);
          // TODOCR DONH: Shouldn't this image URLData be removed like everywhere else
          // and replaced with an ImageType?
          url.SetURLData(PTURLConstants.KEY_ICONIMG, strImageServerURL + PTImageType.ICON_EDIT_MENU.GetFileName());
          url.SetURLData(PTURLConstants.KEY_ICONIMGHEIGHT, PTImageType.ICON_EDIT_MENU.GetHeight());
          url.SetURLData(PTURLConstants.KEY_ICONIMGWIDTH, PTImageType.ICON_EDIT_MENU.GetWidth());
          }
      }
      return url;
    }
    
    /// <summary>
    /// Gets an URL to open a specified Knowledge Directory folder in the selected mode.
    /// The folder will not be accessible if the current user does not have sufficient access it.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="strTitle">Title of the URL, normally the name of the folder to open.</param>
    /// <param name="nDirMode">DirConstants.BROWSE_MODE or DirConstants.EDIT_MODE</param>
    /// <param name="nFolderID">Object ID of the Knowledge Directory Folder to get an URL to.</param>
    /// <returns>An <c>IPTURL</c> to a folder in the Knowledge Directory - no image is assigned to the URL.</returns>
    /// <seealso cref="DirConstants"></seealso>
    public static IPTURL CreateCommonOpenerDirectoryFolderURL(AActivitySpace owner,
     String strTitle, int nDirMode, int nFolderID)
    {
      return CreateCommonOpenerDirectoryFolderURL(owner, strTitle, nDirMode,
       nFolderID, null);
    }
    
    /// <summary>
    /// Gets an URL to open a specified Knowledge Directory folder in the selected mode.
    /// The folder will not be accessible if the current user does not have sufficient access it.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="strTitle">Title of the URL, normally the name of the folder to open.</param>
    /// <param name="nDirMode">DirConstants.BROWSE_MODE or DirConstants.EDIT_MODE</param>
    /// <param name="nFolderID">Object ID of the Knowledge Directory Folder to get an URL to.</param>
    /// <returns>An <c>IPTURL</c> to a folder in the Knowledge Directory - no image is assigned to the URL.</returns>
    /// <seealso cref="DirConstants"></seealso>
    public static IPTURL CreateCommonOpenerDirectoryFolderURL(AActivitySpace owner,
     String strTitle, int nDirMode, int nFolderID, String strFolderName)
    {
      String strImageServerBaseURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
      IPTURL url = null;
      if (TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_CATALOGFOLDER_ID) && DirConstants.BROWSE_MODE == nDirMode)
      {
        url = new CommonOpenerFriendlyURL(strTitle, PT_CLASSIDS.PT_CATALOGFOLDER_ID,
         nFolderID, strFolderName, owner);
      } else
      {
        String strAppBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
        int nUserID = TaskAPIUICommon.GetPTSession(owner).GetSessionInfo().GetCurrentUserID();
        int nMode = IPTOpenerConstants.OPENER_MODE_VIEW;
        url = new CommonOpenerOpenObjIDURL(strTitle, strAppBaseURL, PT_CLASSIDS.PT_CATALOGFOLDER_ID,
         nFolderID, nMode, nUserID);
        url.AddControlParameter(DirConstants.QS_DIRECTORY_MODE, nDirMode);
        }
      ((ACommonOpenerURL) url).SetURLImageType(PTImageType.ICON_FOLDER, strImageServerBaseURL);
      return url;
    }
    
    /// <summary>
    /// Gets an URL to open a specified Document in the Knowledge Directory.
    /// The document will not be accessible if the current user does not have sufficient access it.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="strTitle">Title of the URL, normally the name of the Document to open.</param>
    /// <param name="nDocumentID">Object ID of the Document to get an URL to.</param>
    /// <returns>	</returns>
    public static IPTURL CreateCommonOpenerDocumentURL(AActivitySpace owner,
     String strTitle, int nDocumentID)
    {
      return CreateCommonOpenerDocumentURL(owner, strTitle, nDocumentID, null);
      }
    
    /// <summary>
    /// Gets an URL to open a specified Document in the Knowledge Directory.
    /// The document will not be accessible if the current user does not have sufficient access it.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="strTitle">Title of the URL, normally the name of the Document to open.</param>
    /// <param name="nDocumentID">Object ID of the Document to get an URL to.</param>
    /// <param name="strDocumentName">Name of the document.</param>
    /// <returns>	The URL to the document</returns>
    public static IPTURL CreateCommonOpenerDocumentURL(AActivitySpace owner,
     String strTitle, int nDocumentID, String strDocumentName)
    {
      String strAppBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
      String strImageServerBaseURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
      int nUserID = TaskAPIUICommon.GetPTSession(owner).GetSessionInfo().GetCurrentUserID();
      IPTURL url = null;
      if (TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_CATALOGCARD_ID))
      {
        url = new CommonOpenerFriendlyURL(strTitle, PT_CLASSIDS.PT_CATALOGCARD_ID,
         nDocumentID, strDocumentName, owner);
      } else
      {
        url = new CommonOpenerOpenObjIDURL(strTitle, strAppBaseURL, PT_CLASSIDS.PT_CATALOGCARD_ID,
         nDocumentID, IPTOpenerConstants.OPENER_MODE_VIEW, nUserID);
      }
      ((ACommonOpenerURL) url).SetURLImageType(PTImageType.ICON_DOCUMENT,
       strImageServerBaseURL);
      return url;
    }
    
    /// <summary>
    /// Checks if the current page is a Knowledge Directory page.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current page us a Knowledge Directory page, <c>false</c> otherwise.</returns>
    public static bool IsCurrentPageDirectoryPage(AActivitySpace owner)
    {
      return DirAS.STR_MVC_CLASS_NAME.Equals(TaskAPIUICommon.GetActivitySpaceName(owner));
      }
  }
}