using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.admin.editors.page;
using com.plumtree.portalpages.browsing.editors.editportletprefs;
using com.plumtree.portalpages.browsing.myportal;
using com.plumtree.portalpages.browsing.myportal.editpage;
using com.plumtree.portalpages.browsing.myportal.mypages;
using com.plumtree.portalpages.browsing.objectselection;
using com.plumtree.portalpages.browsing.objectselection.flyout;
using com.plumtree.portalpages.browsing.objectselection.portlets;
using com.plumtree.portalpages.common.editpage;
using com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes;
using com.plumtree.portaluiinfrastructure.opener;
using com.plumtree.portaluiinfrastructure.ptlink;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.taskapi.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.taskapi.portalui
{
  /// <summary>
  /// MyPages related TaskAPI functions.
  /// </summary>
  public class TaskAPIUIMyPages
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.taskapi.portalui.TaskAPIUIMyPages");
    
    /// <summary>
    /// Checks if MyPages feature is enabled in the current Experience Definition.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>true if the MyPages feature is enabled in the current Experience Definition, false otherwise.</returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    internal static bool IsMyPageFeatureEnabledInSubPortal(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      return TaskAPIServerSubPortal.IsSubPortalFeatureEnabled(ptSession, PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_MYPAGES_ACCESS);
      }
    
    /// <summary>
    /// Gets an <c>IPTURL</c> to the specified MyPage.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nMyPageID">Object ID of the MyPage to create an URL to.</param>
    /// <param name="strURLTitle">
    /// Title of the URL, i.e. what a link using the URL will display. 
    /// Most often, the MyPage name should be used.
    /// </param>
    /// <returns><c>IPTURL</c> with an URL to the specified MyPage.</returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static IPTURL CreateMyPageCommonOpenerURL(AActivitySpace owner,
     int nMyPageID, String strURLTitle)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      String strAppBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
      int nUserID = TaskAPIUICommon.GetPTSession(owner).GetSessionInfo().GetCurrentUserID();
      String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
      IPTURL url = null;
      if (TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_MYPAGESCONTAINER_ID))
      {
        url = new CommonOpenerFriendlyURL(strURLTitle, PT_CLASSIDS.PT_MYPAGESCONTAINER_ID,
         nMyPageID, owner);
      } else
      {
        url = new CommonOpenerOpenObjIDURL(strURLTitle, strAppBaseURL, PT_CLASSIDS.PT_MYPAGESCONTAINER_ID,
         nUserID, IPTOpenerConstants.OPENER_MODE_VIEW, nUserID);
        url.AddControlParameter(PTMyPages.STR_PAGE_ID, nMyPageID);
      }
      ((ACommonOpenerURL) url).SetURLImageType(PTImageType.ICON_MYPAGE, strImageServerURL);
      if (nMyPageID == TaskAPIUIMyPages.GetCurrentMyPageID(owner))
      {
        url.SetURLData(PTURLConstants.KEY_CURRENTMARKER, "");
      }
      url.SetURLData(TaskAPIUICommunity.URLDATA_CLASSID, XPConvert.ToString(PT_CLASSIDS.PT_MYPAGESCONTAINER_ID));
      url.SetURLData(TaskAPIUICommunity.URLDATA_OBJECTID, XPConvert.ToString(nMyPageID));
      url.SetURLData(TaskAPIUICommunity.URLDATA_PAGEID, XPConvert.ToString(nMyPageID));
      url.SetURLData(TaskAPIUICommunity.URLDATA_PAGENAME, strURLTitle);
      return url;
    }
    
    /// <summary>
    /// Gets the list of the current user's MyPages
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// <c>XPArrayList</c> with <c>IPTURL</c> entries to the current user's MyPages, or <c>null</c>
    /// when MyPages should not be displayed, i.e. if MyPages feature is disabled in the current Experience Definition or on Preview
    /// pages.
    /// </returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static XPArrayList GetMyPagesAsURLs(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      XPArrayList list = null;
      if (IsMyPageFeatureEnabledInSubPortal(owner) && !TaskAPIUICommon.IsCurrentPageInPreviewMode(owner))
      {
        IPTQueryResult qrMyPages = TaskAPIServerMyPages.GetMyPages(ptSession);
        list = new XPArrayList();
        for (int i = 0;i < qrMyPages.RowCount();i++)
        {
          int nPageID = qrMyPages.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
          String strPageName = qrMyPages.ItemAsString(i, PT_PROPIDS.PT_PROPID_NAME);
          IPTURL url = CreateMyPageCommonOpenerURL(owner, nPageID, strPageName);
          list.Add(url);
        }
      }
      return list;
    }
    
    /// <summary>
    /// Gets the list of MyPages actions for the current user.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// <c>XPArrayList</c> with <c>IPTURL</c> entries to the current user's MyPage Actions.
    /// Actions Edit MyPage Editor, Add MyPage Portlets Editor, and Edit MyPage Portlet Preferences Editor,
    /// and Delete Current MyPage are only displayed when on a MyPage.  
    /// </returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static XPArrayList GetMyPageActionsAsURLs(AActivitySpace owner)
    {
      return GetMyPageActionsAsURLs(owner, false);
    }
    
    /// <summary>
    /// Gets the list of MyPages actions for the current user.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="bIncludeOneClick">indicates if the URL for one-click page creation should be included in the list.  default is false.</param>
    /// <returns>
    /// <c>XPArrayList</c> with <c>IPTURL</c> entries to the current user's MyPage Actions.
    /// Actions Edit MyPage Editor, Add MyPage Portlets Editor, and Edit MyPage Portlet Preferences Editor,
    /// and Delete Current MyPage are only displayed when on a MyPage.  
    /// </returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static XPArrayList GetMyPageActionsAsURLs(AActivitySpace owner,
     bool bIncludeOneClick)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      XPArrayList listURLs = new XPArrayList();
      IPTURL url = null;
      if (bIncludeOneClick)
      {
        url = GetURLForCreateMyPageOneClick(owner);
        if (null != url)
        {
          listURLs.Add(url);
        }
      }
      url = GetURLForCreateMyPageEditor(owner);
      if (null != url)
      {
        listURLs.Add(url);
      }
      url = GetURLForEditMyPageEditor(owner);
      if (null != url)
      {
        listURLs.Add(url);
      }
      url = GetURLForEditLayoutEditor(owner);
      if (null != url)
      {
        listURLs.Add(url);
      }
      url = GetURLForAddMyPagePortletsEditor(owner);
      if (null != url)
      {
        listURLs.Add(url);
      }
      url = GetURLForEditMyPagePortletPreferencesEditor(owner);
      if (null != url)
      {
        listURLs.Add(url);
      }
      url = GetURLForDeleteCurrentMyPageAction(owner);
      if (null != url)
      {
        listURLs.Add(url);
      }
      return listURLs;
    }
    
    /// <summary>
    /// Checks if the current user already has created the maximum number of MyPages (defined in
    /// <c>PT_SERVERCONSTANTS.PT_MAX_MYPORTAL_PAGES</c>.
    /// </summary>
    /// <param name="owner">Current Activityspace</param>
    /// <returns><c>true</c> if the maximum number of MyPages has been created for the current user, <c>false</c> otherwise.</returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static bool IsMaxMyPageLimitReached(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      IPTQueryResult qrMyPages = TaskAPIServerMyPages.GetMyPages(ptSession);
      return (PT_SERVERCONSTANTS.PT_MAX_MYPORTAL_PAGES < qrMyPages.RowCount());
      }
    
    /// <summary>
    /// Checks if the current page is a MyPage
    /// </summary>
    /// <param name="owner">Current Activityspace</param>
    /// <returns><c>true</c> if current page is a MyPage, false otherwise.</returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided AActivityspace is null</throws>
    public static bool IsCurrentPageMyPage(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      String strASName = TaskAPIUICommon.GetActivitySpaceName(owner);
      return MyPageAS.STR_MVC_CLASS_NAME.Equals(strASName);
    }
    
    /// <summary>
    /// Gets the current MyPage name. If current page is not a MyPage, a null value is returned.
    /// </summary>
    /// <param name="owner"></param>
    /// <returns>	MyPage Name of the current MyPage or <c>null</c> value if not currently on a MyPage. </returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static String GetCurrentMyPageName(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      if (owner.GetModelExists(MyPortalModel.STR_MVC_CLASS_NAME))
      {
        MyPortalModel model = (MyPortalModel) owner.GetModel(MyPortalModel.STR_MVC_CLASS_NAME);
        return model.GetPageName();
      } else
      if (FlyoutPortletSelectionAS.STR_MVC_CLASS_NAME.Equals(TaskAPIUICommon.GetActivitySpaceName(owner)))
      {
        // rob: The flyout AS is always preceded by community or mypage AS, so call the same function with the parent space  
        return GetCurrentMyPageName(owner.GetParentSpace());
      }
      return null;
    }
    
    /// <summary>
    /// Gets the URL for the Create MyPage Editor.
    /// </summary>
    /// <param name="owner">Current Activityspace</param>
    /// <returns>
    /// 	<c>IPTURL</c> with the URL to the Create MyPage Editor, or <c>null</c> if it URL is not valid
    /// in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor, or
    /// the maximum number of MyPages has been reached.
    /// </returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static IPTURL GetURLForCreateMyPageEditor(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not MyPageAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace myPageAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      // not on a mypage or community as, don't return anything
      if (myPageAS == null)
      {
        return null;
      }
      CommonOpenerOpenASURL url = null;
      // Special editor in Edit User Profile Mode
      if (TaskAPIUICommon.IsCurrentPageInEditDefaultProfileMode(myPageAS))
      {
        String strURLTitle = myPageAS.GetString(302, "ptmsgs_portalcommonmsgs");
        // Use current AActivitySpace for the following calls
        String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
        String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
        String strASName = PageEditorAS.STR_MVC_CLASS_NAME;
        int nUserID = TaskAPIUIUser.GetCurrentUserID(myPageAS);
        url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
         nUserID);
        url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
        url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_CREATE);
        url.AddControlParameter(PageEditorModel.EDITOR_QS_INT_QS_FOLDER_ID,
         -nUserID);
        url.AddControlParameter(PageEditorModel.EDITOR_QS_INT_CLASS_ID, PT_CLASSIDS.PT_PAGE_ID);
        url.AddControlParameter(PTPages.PAGE_EDITOR_QS_INT_DEFAULT_PROFILE_ID,
         nUserID);
        url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
        url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());
        // Set image
        url.SetURLImageType(PTImageType.ACTION_MYPAGE_NEW, strImageServerURL);
        } else
      {
        if (IsMyPageActionsDisplayable(myPageAS, false))
        {
          if (!IsMaxMyPageLimitReached(myPageAS))
          {
            String strURLTitle = myPageAS.GetString(302, "ptmsgs_portalcommonmsgs");
            String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
            String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
            String strASName = EditMyPortalPageAS.STR_MVC_CLASS_NAME;
            int nUserID = TaskAPIUIUser.GetCurrentUserID(myPageAS);
            url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
             nUserID);
            url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
            url.AddControlParameter(EditPageQueryArguments.HTMLINPUT_EDITPAGE_CREATEMYPAGE,
             XPConvert.ToString(true));
            url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
            url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
            url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());
            // Set image
            url.SetURLImageType(PTImageType.ACTION_MYPAGE_NEW, strImageServerURL);
            }
        }
      }
      return url;
    }
    
    /// <summary>
    /// Gets the URL for the one-click MyPage creation
    /// </summary>
    /// <param name="owner">Current Activityspace</param>
    /// <returns>
    /// 	<c>IPTURL</c> with the URL to the one-click MyPage control, or <c>null</c> if it URL is not valid
    /// in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor, or
    /// the maximum number of MyPages has been reached.
    /// </returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static IPTURL GetURLForCreateMyPageOneClick(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not MyPageAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace myPageAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (myPageAS == null)
      {
        return null;
      }
      CommonOpenerOpenASURL url = null;
      if (IsMyPageActionsDisplayable(myPageAS, false))
      {
        if (!IsMaxMyPageLimitReached(myPageAS))
        {
          String strURLTitle = myPageAS.GetString(802, "ptmsgs_portalcommonmsgs");
          String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
          String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
          String strASName = EditMyPortalPageAS.STR_MVC_CLASS_NAME;
          int nUserID = TaskAPIUIUser.GetCurrentUserID(myPageAS);
          url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
           nUserID);
          url.AddControlParameter(ASConstants.CONTROL, EditPageCreateDefaultControl.STR_MVC_CLASS_NAME);
          url.AddControlParameter(EditPageQueryArguments.HTMLINPUT_EDITPAGE_CREATEMYPAGE,
           XPConvert.ToString(true));
          url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
          url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());
          // Set image
          url.SetURLImageType(PTImageType.ACTION_MYPAGE_NEW, strImageServerURL);
          }
      }
      return url;
    }
    
    /// <summary>
    /// Gets the URL to the Edit current MyPage Editor for the Standard Access Style mode. 
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// <c>IPTURL</c> with the URL to the current MyPage Editor, or <c>null</c>if the URL is invalid 
    /// in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor.
    /// </returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static IPTURL GetURLForEditMyPageEditor(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not MyPageAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace myPageAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (myPageAS == null)
      {
        return null;
      }
      CommonOpenerOpenASURL url = null;
      // Special editor in Edit User Profile Mode
      if (TaskAPIUICommon.IsCurrentPageInEditDefaultProfileMode(myPageAS))
      {
        String strURLTitle = myPageAS.GetString(301, "ptmsgs_portalcommonmsgs");
        // Use current AActivitySpace for the following calls
        String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
        String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
        String strASName = PageEditorAS.STR_MVC_CLASS_NAME;
        int nUserID = TaskAPIUIUser.GetCurrentUserID(myPageAS);
        int nPageID = GetCurrentMyPageID(myPageAS);
        url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
         nUserID);
        url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
        url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_EDIT);
        url.AddControlParameter(PageEditorModel.EDITOR_QS_INT_QS_FOLDER_ID,
         -nUserID);
        url.AddControlParameter(PageEditorModel.EDITOR_QS_INT_QS_OBJECT_ID,
         nPageID);
        url.AddControlParameter(PageEditorModel.EDITOR_QS_INT_CLASS_ID, PT_CLASSIDS.PT_PAGE_ID);
        url.AddControlParameter(PTPages.PAGE_EDITOR_QS_INT_DEFAULT_PROFILE_ID,
         nUserID);
        url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
        url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());
        // Set image
        url.SetURLImageType(PTImageType.ACTION_MYPAGE_NEW, strImageServerURL);
        } else
      {
        if (IsMyPageActionsDisplayable(myPageAS, true) && TaskAPIUIUser.IsAccessStyleStandard(myPageAS))
        {
          String strURLTitle = myPageAS.GetString(301, "ptmsgs_portalcommonmsgs");
          String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
          String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
          String strASName = EditMyPortalPageAS.STR_MVC_CLASS_NAME;
          int nUserID = TaskAPIUIUser.GetCurrentUserID(myPageAS);
          int nPageID = GetCurrentMyPageID(myPageAS);
          url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
           nUserID);
          url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
          url.AddControlParameter(EditPageQueryArguments.HTMLINPUT_EDITPAGE_OPENERPAGEID,
           nPageID);
          url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
          url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
          url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());
          // Set image
          url.SetURLImageType(PTImageType.ICON_EDIT_MENU, strImageServerURL);
          }
      }
      return url;
    }
    
    /// <summary>
    /// Gets the URL to the Edit current MyPage Editor for the Low-Bandwidth and Accessibility Style mode. 
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// <c>IPTURL</c> with the URL to the current MyPage Editor, or <c>null</c>if the URL is invalid 
    /// in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor.
    /// </returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static IPTURL GetURLForEditLayoutEditor(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not MyPageAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace myPageAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (myPageAS == null)
      {
        return null;
      }
      CommonOpenerOpenASURL url = null;
      if (IsMyPageActionsDisplayable(myPageAS, true))
      {
        if (!TaskAPIUIUser.IsAccessStyleStandard(myPageAS))
        {
          String strURLTitle = myPageAS.GetString(301, "ptmsgs_portalcommonmsgs");
          // Use current AActivitySpace for the following calls
          String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
          String strImgSvrURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
          String strASName = EditMyPagePortletLayoutAS_D.STR_MVC_CLASS_NAME;
          int nUserID = TaskAPIUIUser.GetCurrentUserID(myPageAS);
          int nPageID = GetCurrentMyPageID(myPageAS);
          url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
           nUserID);
          url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
          url.AddControlParameter(EditMyPagePortletLayoutModel_D.QS_PAGEID,
           nPageID);
          url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
          url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
          url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());
          // Set image
          url.SetURLImageType(PTImageType.ICON_EDIT_MENU, strImgSvrURL);
        }
      }
      return url;
    }
    
    /// <summary>
    /// INTERNAL HELPER FUNCTION
    /// Performs the common checks whether a MyPage actions should be displayed or not.  
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns><c>true</c> if MyPage actions should be displayed in the current context, <c>false</c> if not. </returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    internal static bool IsMyPageActionsDisplayable(AActivitySpace owner,
     bool bRequireMyPage)
    {
      // Don't display mypage actions for guest user - except when editing default user profile
      if (TaskAPIUIUser.IsCurrentUserGuestUser(owner) && !TaskAPIUICommon.IsCurrentPageInEditDefaultProfileMode(owner))
      {
        return false;
      }
      if (IsMyPageFeatureEnabledInSubPortal(owner) && !TaskAPIUICommon.IsCurrentPageInPreviewMode(owner))
      {
        if (bRequireMyPage)
        {
          return IsCurrentPageMyPage(owner);
        } else
        {
          return true;
        }
      }
      return false;
    }
    
    /// <summary>
    /// Gets the URL to the MyPage Add Portlets Editor.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// <c>IPTURL</c> with the URL to the MyPage App Portlets Editor, or <c>null</c>if the URL is invalid 
    /// in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor.
    /// </returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static IPTURL GetURLForAddMyPagePortletsEditor(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not MyPageAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace myPageAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (myPageAS == null)
      {
        return null;
      }
      CommonOpenerOpenASURL url = null;
      if (IsMyPageActionsDisplayable(myPageAS, true))
      {
        String strURLTitle = myPageAS.GetString(300, "ptmsgs_portalcommonmsgs");
        // Use current AActivitySpace for the following calls
        String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
        String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
        String strASName = PortletSelectionEditorAS.STR_MVC_CLASS_NAME;
        int nUserID = TaskAPIUIUser.GetCurrentUserID(myPageAS);
        int nPageID = GetCurrentMyPageID(myPageAS);
        url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
         nUserID);
        url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
        url.AddControlParameter(ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTOROBJECTID,
         nPageID);
        url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
        url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
        url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());
        // Set image
        url.SetURLImageType(PTImageType.ACTION_PORTLET_ADD, strImageServerURL);
        }
      return url;
    }
    
    /// <summary>
    /// Gets the URL to the MyPage Add Portlets Flyout.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// <c>IPTURL</c> with the URL to the MyPage App Portlets Flyout, or <c>null</c>if the URL is invalid 
    /// in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor.
    /// </returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static IPTURL GetURLForAddMyPagePortletsFlyout(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      CommonOpenerOpenASURL url = null;
      if (IsMyPageActionsDisplayable(owner, true))
      {
        String strURLTitle = owner.GetString(300, "ptmsgs_portalcommonmsgs");
        String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
        String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
        String strASName = FlyoutPortletSelectionAS.STR_MVC_CLASS_NAME;
        int nUserID = TaskAPIUIUser.GetCurrentUserID(owner);
        int nPageID = GetCurrentMyPageID(owner);
        url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
         nUserID);
        url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
        url.AddControlParameter(ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTOROBJECTID,
         nPageID);
        url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
        url.AddControlParameter(ASConstants.PARENTSPACEID, owner.GetSpaceID());
        url.AddControlParameter(ASConstants.PARENTSPACENAME, owner.GetName());
        // Set image
        url.SetURLImageType(PTImageType.ACTION_PORTLET_ADD, strImageServerURL);
        }
      return url;
    }
    
    /// <summary>
    /// Gets the URL to the MyPage Edit Portlet Preference Editor.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// <c>IPTURL</c> with the URL to the MyPage Edit Portlet Preferences Editor, or <c>null</c>if the URL is invalid 
    /// in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor.
    /// </returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static IPTURL GetURLForEditMyPagePortletPreferencesEditor(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not MyPageAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace myPageAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (myPageAS == null)
      {
        return null;
      }
      CommonOpenerOpenASURL url = null;
      if (IsMyPageActionsDisplayable(myPageAS, true))
      {
        String strURLTitle = owner.GetString(306, "ptmsgs_portalcommonmsgs");
        // Use current AActivitySpace for the following calls
        String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
        String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
        String strASName = EditPortletPrefsAS.STR_MVC_CLASS_NAME;
        int nUserID = TaskAPIUIUser.GetCurrentUserID(myPageAS);
        int nPageID = GetCurrentMyPageID(myPageAS);
        url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
         nUserID);
        url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
        url.AddControlParameter(EditPortletPrefsAS.QS_PARAM_PAGEID, nPageID);
        url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
        url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
        url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());
        // Set image
        url.SetURLImageType(PTImageType.ACTION_PORTLET_PREF, strImageServerURL);
        }
      return url;
    }
    
    /// <summary>
    /// Gets the URL for Deleting the current MyPage.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// <c>IPTURL</c> with the URL to delete the current MyPage, or <c>null</c>if the URL is invalid 
    /// in the current context. For example when MyPages feature is disabled in the current Experience Definition Editor, or
    /// if the current MyPage is the only MyPage.
    /// </returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static IPTURL GetURLForDeleteCurrentMyPageAction(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not MyPageAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace myPageAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (myPageAS == null)
      {
        return null;
      }
      CommonOpenerOpenASURL url = null;
      if (IsMyPageActionsDisplayable(myPageAS, true))
      {
        //			mrc: fixed bug where delete link would should up even if there was only 1 page left			
        //			if ( GetCurrentMyPageID(myPageAS) > 0)
        IPTQueryResult qrMyPages = TaskAPIServerMyPages.GetMyPages(TaskAPIUICommon.GetPTSession(owner));
        if (qrMyPages.RowCount() > 1)
        {
          String strURLTitle = myPageAS.GetString(303, "ptmsgs_portalcommonmsgs");
          // Use current AActivitySpace for the following calls
          String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
          String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
          String strASName = MyPageAS.STR_MVC_CLASS_NAME;
          int nUserID = TaskAPIUIUser.GetCurrentUserID(myPageAS);
          int nPageID = GetCurrentMyPageID(myPageAS);
          url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
           nUserID);
          url.AddControlParameter(ASConstants.CONTROL, DeletePageControl.STR_MVC_CLASS_NAME);
          url.AddControlParameter(DeletePageControl.STR_PAGE_ID, nPageID);
          url.AddControlParameter(ASConstants.PARENTSPACEID, myPageAS.GetSpaceID());
          url.AddControlParameter(ASConstants.PARENTSPACENAME, myPageAS.GetName());
          // Set image
          url.SetURLImageType(PTImageType.ACTION_DELETE_MENU, strImageServerURL);
          }
      }
      return url;
    }
    
    /// <summary>
    /// Gets the object ID of the current MyPage.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>Object ID of the MyPage, or 0 if not currently on a MyPage.</returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static int GetCurrentMyPageID(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      int nCurrPageID = 0;
      if (TaskAPIUICommon.GetMyPortalPageModelExists(owner))
      {
        MyPortalModel ptModel = TaskAPIUICommon.GetMyPortalPageModel(owner);
        nCurrPageID = ptModel.GetPageID();
      } else
      if (FlyoutPortletSelectionAS.STR_MVC_CLASS_NAME.Equals(TaskAPIUICommon.GetActivitySpaceName(owner)))
      {
        // rob: The flyout AS is always preceded by community or mypage AS, so call the same function with the parent space  
        nCurrPageID = GetCurrentMyPageID(owner.GetParentSpace());
      }
      return nCurrPageID;
    }
    
    /// <summary>
    /// Helper function to retrieve the MyPage Object ID from a MyPage <c>CommonOpenerOpenObjIDURL</c> to a MyPage.
    /// </summary>
    /// <param name="ptURL">An <c>IPTURL</c> to a MyPage</param>
    /// <returns>
    /// The MyPage ID as an <c>int</c> or 0 if the provided URL is not a 
    /// <c>CommonOpenerOpenObjIDURL</c>or MyPage URL.
    /// </returns>
    /// <throws><c>XPIllegalArgumentException</c> if the provided <c>AActivityspace</c> is <c>null</c> or not valid.</throws>
    public static int GetMyPageIDFromPTURL(IPTURL ptURL)
    {
      if (null == ptURL)
      {
        return 0;
      }
      int nPageID = 0;
      String strPageID = ptURL.GetData(PTMyPages.STR_PAGE_ID);
      if (null != strPageID)
      {
        try
        {
          nPageID = XPConvert.ToInteger(strPageID);
        }
        catch (XPFormatException e)
        {
        }
      }
      // Don't do anything, we will just return the default page id
      return nPageID;
    }
  }
}