using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.common.uiparts;
using com.plumtree.portaluiinfrastructure.common.constants;
using com.plumtree.portaluiinfrastructure.helpers;
using com.plumtree.portaluiinfrastructure.ptlink;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taskapi.portalui
{
  /// <summary>
  /// TaskAPI functions related to Portal Configuration, includes values in portalconfig.xml
  /// </summary>
  public class TaskAPIUIPortalConfig
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.taskapi.portalui.TaskAPIUIPortalConfig");
    
    /// <summary>
    /// Returns the Application URL according to the URL Mapping.
    /// Warning: In mode 1, it returns the non secure URL by default.
    /// </summary>
    /// <param name="<code>AActivitySpace</code>">the current activity space.</param>
    /// <returns><c>String</c> with the Application Base URL</returns>
    public static String GetApplicationBaseURL(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      return PTConfigService.getPTInstance().GetApplicationBaseURL(owner.GetCurrentNormalizedBaseURL());
      }
    
    /// <summary>
    /// Same as <c>TaskAPIUIPortalConfig.GetApplicationBaseURL</c> but without the
    /// trailing question mark.
    /// </summary>
    /// <param name="<code>AActivitySpace</code>">the current activity space.</param>
    /// <returns><c>String</c> with the Application Base URL and no trailing question mark </returns>
    public static String GetApplicationBaseURLForFriendlyURL(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // remove the trailing question mark
      String baseURL = PTConfigService.getPTInstance().GetApplicationBaseURL(owner.GetCurrentNormalizedBaseURL());
      baseURL = GeneralHelpers.Replace(baseURL, "?", "");
      return baseURL;
    }
    
    /// <summary>
    /// Returns the Application URL according to the URL Mapping and
    /// determines the security according to the space name in argument.
    /// </summary>
    /// <param name="strNormalizedBaseURL">
    /// Normalized Base URL from the AActivity Space.
    /// (or ConfigHelper.GetNormalizedBaseURL).
    /// </param>
    /// <param name="strASName">Name of the destination Activity Space.</param>
    /// <param name="bIsSubSpace">Whether the target Activityspace a sub space or not?</param>
    /// <returns><c>String</c> with the Application Base URL.</returns>
    public static String GetApplicationBaseURLToAS(String strNormalizedBaseURL,
     String strASName, bool bIsSubSpace)
    {
      return PTConfigService.getPTInstance().GetApplicationBaseURLToAS(strNormalizedBaseURL,
       strASName, bIsSubSpace);
    }
    
    /// <summary>
    /// This method to access an integer variable from portalconfig.xml.
    /// </summary>
    /// <param name="strSection">One of the Section constants defined in <c>ConfigConstants</c>.</param>
    /// <param name="strSubSection">One of the Subsection constants defined in <c>ConfigConstants</c>.</param>
    /// <param name="strAttribute">One of the Attributes constants defined in <c>ConfigConstants</c>.</param>
    /// <param name="owner">current Activityspace.</param>
    /// <returns>The requested attribute value as <c>int</c>  </returns>
    /// <throws><c>XPException</c> If the requested variable is not found or cannot be retrieved.</throws>
    /// <seealso cref="com.plumtree.uiinfrastructure.application.varpacks.ConfigConstants"></seealso>
    public static int GetPTConfigVariableAsInt(String strSection, String strSubSection,
     String strAttribute, AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      try
      {
        return PTConfigService.getPTInstance().GetConfigVariableAsInt(strSection,
         strSubSection, strAttribute, owner);
      }
      catch (XPException e)
      {
        throw new XPException("Unable to retrieve the requested attribute: " + strAttribute,
         e);
      }
    }
    
    /// <summary>
    /// This method to access a <c>String</c> variable from portalconfig.xml.
    /// </summary>
    /// <param name="strSection">One of the Section constants defined in <c>ConfigConstants</c>.</param>
    /// <param name="strSubSection">One of the Subsection constants defined in <c>ConfigConstants</c>.</param>
    /// <param name="strAttribute">One of the Attributes constants defined in <c>ConfigConstants</c>.</param>
    /// <param name="owner">current Activityspace.</param>
    /// <returns><c>String</c> with the requested variable value, or <c>null</c> if the variable is not found, or cannot be retrieved.</returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.application.varpacks.ConfigConstants"></seealso>
    public static String GetPTConfigVariableAsString(String strSection, String strSubSection,
     String strAttribute, AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      try
      {
        return PTConfigService.getPTInstance().GetConfigVariableAsString(strSection,
         strSubSection, strAttribute, owner);
      }
      catch (XPException e)
      {
        log.Warn("Unable to retrieve the requested attribute: " + strAttribute + ".",
         e);
        return null;
      }
    }
    
    /// <summary>
    /// This method to access a variable with multiple values from portalconfig.xml.
    /// </summary>
    /// <param name="strSection">One of the Section constants defined in <c>ConfigConstants</c>.</param>
    /// <param name="strSubSection">One of the Subsection constants defined in <c>ConfigConstants</c>.</param>
    /// <param name="strAttribute">One of the Attributes constants defined in <c>ConfigConstants</c>.</param>
    /// <param name="owner">current Activityspace.</param>
    /// <returns>
    /// Object Array with values of the requested attribute, or <c>null</c> if the attribute was not found or could 
    /// not be retrieved.
    /// </returns>
    public static Object[] GetPTConfigVariables(String strSection, String strSubSection,
     String strAttribute, AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      try
      {
        return PTConfigService.getPTInstance().GetConfigVariables(strSection,
         strSubSection, strAttribute, owner);
      }
      catch (XPException e)
      {
        log.Warn("Unable to retrieve the requested attribute: " + strAttribute + ".",
         e);
        return null;
      }
    }
    
    /// <summary>
    /// This method returns the main application name.
    /// </summary>
    /// <returns><c>String</c> with the main Portal Application name.</returns>
    public static String GetMainApplicationName()
    {
      return ConfigHelper.GetMainApplicationName();
    }
    
    /// <summary>
    /// This method returns the Image Server Base URL.
    /// </summary>
    /// <param name="AActivitySpace">Current Activityspace.</param>
    /// <returns><c>String</c> with the image server base URL.</returns>
    public static String GetImageServerBaseURL(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      return ConfigHelper.GetImageServerBaseURL(owner);
    }
    
    /// <summary>
    /// This method returns the Portal Image Server JavaScript URL.
    /// </summary>
    /// <param name="AActivitySpace">Current Activityspace.</param>
    /// <returns><c>String</c> with the Portal image server JavaScript URL.</returns>
    public static String GetPortalImageServerJavaScriptURL(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      return ConfigHelper.GetPortalImageServerJavaScriptURL(owner.GetIsSecuredSpace());
      }
    
    /// <summary>
    /// Returns the base URL to the style sheets on the image server.
    /// </summary>
    /// <param name="AActivitySpace">Current Activityspace.</param>
    /// <returns><c>String</c> with the URL to the style sheets on the image server</returns>
    public static String GetStyleSheetsBaseURL(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      return ConfigHelper.GetStyleSheetsBaseURL(owner);
    }
    
    /// <summary>
    /// This method returns the Portal Image Server Help URL.
    /// </summary>
    /// <param name="AActivitySpace">Current Activityspace.</param>
    /// <returns><c>String</c> with the the Portal image server Help URL.</returns>
    public static String GetPortalImageServerHelpURL(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      return ConfigHelper.GetPortalImageServerHelpURL(owner);
    }
    
    /// <summary>
    /// Returns the security mode value as defined in portalconfig.xml.
    /// </summary>
    /// <returns>integer with the current Portal Security mode.</returns>
    public static int GetSecurityMode()
    {
      return ConfigHelper.GetSecurityMode();
    }
    
    /// <summary>
    /// Gets a Navigation settings from portalconfig.xml (what used to be NavigationSettings.xml in 5.x) as an integer value.
    /// Navigation Settings  are defined in 'portal:Navigation' the component and with the type defined in the 
    /// <c>PTNavSettingsVarPack.PORTAL_NAVIGATION_COMPONENT_TYPE</c> constant.
    /// </summary>
    /// <param name="strKey">Navigation setting attribute.</param>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	integer with the attribute value or, -1 if the key was not found or not an integer value.</returns>
    public static int GetNavSettingValueAsInt(String strKey, AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      return NavigationCommonHelpers.GetNavSettingsValueAsInt(strKey, owner);
      }
    
    /// <summary>
    /// Gets a Navigation specific settings from portalconfig.xml (what used to be NavigationSettings.xml in 5.x) as a <c>String</c> value.
    /// Navigation Settings  are defined in 'portal:Navigation' the component and with the type defined in the 
    /// <c>PTNavSettingsVarPack.PORTAL_NAVIGATION_COMPONENT_TYPE</c> constant.
    /// </summary>
    /// <param name="strKey">Navigation setting attribute.</param>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>String</c> with the attribute value, or <c>null</c> if the attribute was not found.</returns>
    public static String GetNavSettingsValueAsString(String strKey, AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      return NavigationCommonHelpers.GetNavSettingsValueAsString(strKey, owner);
      }
    
    /// <summary>
    /// Gets a Navigation settings from portalconfig.xml (what used to be NavigationSettings.xml in 5.x) which has a comma separated
    /// strings value. Example:. <SomeKey value="val1,val2,val3"/>
    /// Navigation Settings  are defined in 'portal:Navigation' the component and with the type defined in the 
    /// <c>PTNavSettingsVarPack.PORTAL_NAVIGATION_COMPONENT_TYPE</c> constant.
    /// </summary>
    /// <param name="strKey">Key to a name/value pair in NavigationSettings.xml with a comma separated string values.</param>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 					Array of <c>String</c>s with the comman separated values, or null if the attribute was not found, or 
    /// the comma separated values were in an invalid format.
    /// </returns>
    public static String[] GetNavSettingsValueAsStringArray(String strKey,
     AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      String strVal = NavigationCommonHelpers.GetNavSettingsValueAsString(strKey,
       owner);
      if (null == strVal)
      {
        return null;
      }
      String[] result = null;
      try
      {
        XPStringTokenizer tok = new XPStringTokenizer(strVal, ",");
        result = new String[tok.GetTokenCount()];
        int i = 0;
        while (tok.HasMoreTokens())
        {
          result[i++] = tok.GetNextToken();
        }
      }
      catch (Exception e)
      {
        log.Warn("Unable to parse the attribute value. Expecting a comma separated String, read value:" + strVal,
         e);
        return null;
      }
      return result;
    }
    
    /// <summary>
    /// Gets a Navigation settings from portalconfig.xml (what used to be NavigationSettings.xml in 5.x) which has a comma separated
    /// integer values. Example:. <SomeKey value="100,200,300"/>
    /// Navigation Settings  are defined in 'portal:Navigation' the component and with the type defined in the 
    /// <c>PTNavSettingsVarPack.PORTAL_NAVIGATION_COMPONENT_TYPE</c> constant.
    /// </summary>
    /// <param name="strKey">Key to a name/value pair in NavigationSettings.xml with a comma separated integer values.</param>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 					Array of <c>int</c>s with the comman separated values, or null if the attribute was not found, or 
    /// the comma separated values were in an invalid format.
    /// </returns>
    public static int[] GetNavSettingsValueAsIntArray(String strKey, AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      String strVal = NavigationCommonHelpers.GetNavSettingsValueAsString(strKey,
       owner);
      if (null == strVal)
      {
        return null;
      }
      int[] result = null;
      try
      {
        XPStringTokenizer tok = new XPStringTokenizer(strVal, ",");
        result = new int[tok.GetTokenCount()];
        int i = 0;
        while (tok.HasMoreTokens())
        {
          result[i++] = XPConvert.ToInteger(tok.GetNextToken());
        }
      }
      catch (Exception e)
      {
        throw new XPException("Error parsing NavigationSetting value - need to be a comma separated list of integers. Value: " + strVal);
        }
      return result;
    }
    
    /// <summary>
    /// Gets the URL to the Help Page for the current page.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 	<c>IPTURL</c> with URL to the Help Page for the current page. In standard display mode
    /// the url is just '#', instead a javascript call to open help in a pop-up is used. Get the javascript call with the
    /// <c>PTURLConstants.KEY_JAVASCRIPT_ACTION</c>.
    /// In 508 and low-bandwidth mode, a regular URL is used.
    /// </returns>
    public static IPTURL GetURLForCurrentPageHelp(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      String strText = owner.GetString(337, ResourceConstants.PORTALCOMMONMSGS);
      HTMLAnchor helpAnchor = HTMLConstructHelpers.GetHelpAnchor(owner);
      String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
      FullURL url = new FullURL(strText, helpAnchor.GetHRef());
      // Get the onclick action, which is what is really used in standard display mode
      url.SetURLData(PTURLConstants.KEY_JAVASCRIPT_ACTION, helpAnchor.GetOnClick());
      url.SetURLData(PTURLConstants.KEY_ICONIMG, strImageServerURL + PTImageType.ICON_HELP.GetFileName());
      url.SetURLData(PTURLConstants.KEY_ICONIMGWIDTH, PTImageType.ICON_HELP.GetWidth());
      url.SetURLData(PTURLConstants.KEY_ICONIMGHEIGHT, PTImageType.ICON_HELP.GetHeight());
      return url;
    }
  }
}