/*
 */
 
using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.directory;
using com.plumtree.portalpages.browsing.objectselection.communities;
using com.plumtree.portalpages.portlets.sitemap.prefs;
using com.plumtree.portaluiinfrastructure.ptlink;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.taskapi.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.taskapi.portalui
{
  /// <summary>
  /// Experience Definition related TaskAPI functions. 
  /// </summary>
  public class TaskAPIUISubPortal
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.taskapi.portalui.TaskAPIUISubportal");
    
    /// <summary>
    /// Checks if the specified Feature type is enabled in the current Experience Definition.
    /// The three feature types, also known as Access Types, are MyPage access, 
    /// Communities access and Directory access.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nAccessType">
    /// One of the experience definition features defined in 
    /// <c>com.plumtree.server.PT_SUBPORTAL_ACCESS_TYPES</c>.
    /// </param>
    /// <returns><c>true</c> if the current user has access to the specified feature, <c>false</c> in all other cases.</returns>
    /// <seealso cref="PT_SUBPORTAL_ACCESS_TYPES"></seealso>
    public static bool IsSubPortalEnabledForAccessType(AActivitySpace owner,
     int nAccessType)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      IPTSubPortalInfo ptSubPortal = TaskAPIServerSubPortal.GetCurrentSubPortalInfoObject(ptSession);
      if ((nAccessType != PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_MYPAGES_ACCESS) && (nAccessType != PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_COMMUNITIES_ACCESS) && (nAccessType != PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_DIRECTORY_ACCESS))
      {
        return false;
      }
      try
      {
        int nCurrAccessType = ptSubPortal.GetAccessType();
        return ((nAccessType & nCurrAccessType) != 0);
      }
      catch (Exception e)
      {
        log.Warn(e, "Unable to access the Subportal to retrieve the accesstype.");
        return false;
      }
    }
    
    /// <summary>
    /// Gets the URL to the c urrent user's Home Page as defined in the current Experience Definition.
    /// Examples of Home Pages are a My Page, Community Pages or Knowledge Directory Folder.
    /// </summary>
    /// <param name="owner">Current Activityspace</param>
    /// <returns>
    /// 	<c>IPTURL</c> with URL to current user's Home Page or <c>null</c> if the Home Page
    /// could not be retrieved for any reason.
    /// </returns>
    public static IPTURL GetURLForMyHome(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      IPTSubPortalInfo ptSubPortal = TaskAPIServerSubPortal.GetCurrentSubPortalInfoObject(ptSession);
      int nDefaultPageType = ptSubPortal.GetDefaultFirstPageType();
      int nDefaultPageID = ptSubPortal.GetDefaultFirstPageID();
      ACommonOpenerURL ptURL = null;
      String label = owner.GetString(698, ResourceConstants.PORTALCOMMONMSGS);
      String strAppBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
      String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
      int nUserID = TaskAPIUIUser.GetCurrentUserID(owner);
      switch (nDefaultPageType){
      case PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_COMMUNITIES_ACCESS:
      // 1. Go to comm page
      // 2. If no access to comm page -> Go to default comm
      // 3. If no default comm -> Join comm page
      // @TODOMARTOK make a taskapi function for this
      if (!ptSession.GetCommunities().IsObjectAccessible(nDefaultPageID, nUserID,
       PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ))
      {
        nDefaultPageID = TaskAPIUICommunity.GetDefaultCommunityID(owner);
        }
      if (nDefaultPageID != -1)
      {
        ptURL = (ACommonOpenerURL) TaskAPIUICommunity.CreateCommunityCommonOpenerURL(owner,
         nDefaultPageID, label);
      } else
      {
        CommonOpenerOpenASURL url2 = new CommonOpenerOpenASURL(label, strAppBaseURL,
         CommunitySelectionEditorAS.STR_MVC_CLASS_NAME, nUserID);
        url2.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
        url2.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
        ptURL = url2;
      }
      break;
      case PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_DIRECTORY_ACCESS:
      ptURL = (ACommonOpenerURL) TaskAPIUIDirectory.CreateCommonOpenerDirectoryFolderURL(owner,
       label, DirConstants.BROWSE_MODE, nDefaultPageID);
      break;
      case PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_MYPAGES_ACCESS:
      // DCA - MyPage PageID should be set to 0
      ptURL = (ACommonOpenerURL) TaskAPIUIMyPages.CreateMyPageCommonOpenerURL(owner,
       0, label);
      break;
      default:
      return null;
      break;
      }
      ptURL.SetURLImageType(PTImageType.ICON_HOME, strImageServerURL);
      return ptURL;
    }
    
    /// <summary>
    /// Gets a list of Mandatory Links URLs defined in the current Experience Definition.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// <c>XPArrayList</c> with <c>IPTURL</c> entries to current user's Mandatory Experience Definition  Links. 
    /// An empty list is returned if there are no entries. 
    /// </returns>
    public static XPArrayList GetSubPortalMandatoryLinksAsURLs(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      if (TaskAPIUICommon.IsCurrentPageInPreviewMode(owner))
      {
        return new XPArrayList();
      }
      String strImgSvrURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
      IPTQueryResult qrMandLinks = TaskAPIServerSubPortal.GetSubPortalMyMandatoryLinks(ptSession);
      if (qrMandLinks == null)
      {
        return new XPArrayList();
      }
      //		int[][]arrSort = new int[1][2];
      //		arrSort[0][0] = PT_PROPIDS.PT_PROPID_SITEMAP_LINKCLASSID;
      //		arrSort[0][1] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
      PTSortedArrayWrapper arrSortedLinks = CreateSortedArray(qrMandLinks,
       "", owner);
      XPArrayList list = new XPArrayList();
      int nNumLinks = arrSortedLinks.GetCount();
      for (int i = 0;i < nNumLinks;i++)
      {
        String strName = (String) arrSortedLinks.GetItem(i, PT_PROPIDS.PT_PROPID_NAME);
        String strLinkURL = (String) arrSortedLinks.GetItem(i, PT_PROPIDS.PT_PROPID_SITEMAP_LINKURL);
        int lLinkClassID = XPConvert.ToInteger(arrSortedLinks.GetItem(i, PT_PROPIDS.PT_PROPID_SITEMAP_LINKCLASSID));
        int lLinkObjectID = XPConvert.ToInteger(arrSortedLinks.GetItem(i,
         PT_PROPIDS.PT_PROPID_SITEMAP_LINKOBJECTID));
        int lLinkCommID = XPConvert.ToInteger(arrSortedLinks.GetItem(i, PT_PROPIDS.PT_PROPID_SITEMAP_LINKPAGEID));
        IPTURL url = null;
        switch (lLinkClassID){
        case PT_CLASSIDS.PT_SITEMAPWEBLINK_ID:
        //				cpList.SetImageForLastAddedURL(PTImageType.ICON_URL, 
        //					owner.GetString(1790, "ptmsgs_portalbrowsingmsgs"));
        APTURL currURL = new FullURL(strName, strLinkURL);
        currURL.SetURLImageType(PTImageType.ICON_URL, strImgSvrURL);
        currURL.SetURLData(PTURLConstants.KEY_ICONIMGALT, owner.GetString(1790,
         "ptmsgs_portalbrowsingmsgs"));
        url = currURL;
        break;
        case PT_CLASSIDS.PT_PAGE_ID:
        url = TaskAPIUICommunity.CreateCommunityPageCommonOpenerURL(owner,
         lLinkCommID, lLinkObjectID, strName);
        // The CreateCommunityPageCommonOpenerURL method sets document image as Community Page icon but we want Community image	
        // TODOCR DONH: Shouldn't this image URLData be removed like everywhere else
        // and replaced with an ImageType?
        url.SetURLData(PTURLConstants.KEY_ICONIMG, strImgSvrURL + PTImageType.ICON_COMMUNITY.GetFileName());
        url.SetURLData(PTURLConstants.KEY_ICONIMGWIDTH, PTImageType.ICON_COMMUNITY.GetWidth());
        url.SetURLData(PTURLConstants.KEY_ICONIMGHEIGHT, PTImageType.ICON_COMMUNITY.GetHeight());
        url.SetURLData(PTURLConstants.KEY_ICONIMGALT, owner.GetString(1787,
         "ptmsgs_portalbrowsingmsgs"));
        break;
        // User Profile page
        case PT_CLASSIDS.PT_USER_ID:
        url = TaskAPIUICommunity.CreateUserProfilePageCommonOpenerURL(owner,
         lLinkObjectID, strName);
        // TODOCR DONH: Does this need to have the image type set on it?  If not, this should
        // probably have a comment stating that the image is already set.
        url.SetURLData(PTURLConstants.KEY_ICONIMGALT, owner.GetString(1789,
         "ptmsgs_portalbrowsingmsgs"));
        break;
        case PT_CLASSIDS.PT_CATALOGFOLDER_ID:
        // Image set in the create method
        url = TaskAPIUIDirectory.CreateCommonOpenerDirectoryFolderURL(owner,
         strName, DirConstants.BROWSE_MODE, lLinkObjectID);
        // TODOCR DONH: Does this need to have the image type set on it?  If not, this should
        // probably have a comment stating that the image is already set.
        url.SetURLData(PTURLConstants.KEY_ICONIMGALT, owner.GetString(1791,
         "ptmsgs_portalbrowsingmsgs"));
        break;
        case PT_CLASSIDS.PT_CATALOGCARD_ID:
        // Image set in the create method
        url = TaskAPIUIDirectory.CreateCommonOpenerDocumentURL(owner, strName,
         lLinkObjectID);
        // TODOCR DONH: Does this need to have the image type set on it?  If not, this should
        // probably have a comment stating that the image is already set.
        url.SetURLData(PTURLConstants.KEY_ICONIMGALT, owner.GetString(1788,
         "ptmsgs_portalbrowsingmsgs"));
        break;
        }
        list.Add(url);
      }
      return list;
    }
    
    /// <summary>
    /// Internal helper function to create a <c>PTSortedArrayWrapper</c> object from a <c>IPTQueryResult</c> 
    /// with Mandatory Experience Definition Link Data in a specified order .
    /// </summary>
    /// <param name="ptSiteMapFolderInfo">
    /// IPTQueryResult with Mandatory Experience Definition Links from 
    /// <c>TaskAPIServerSubPortal.GetSubPortalMyMandatoryLinks</c>.
    /// </param>
    /// <param name="arrSort">
    /// A 2D int array of PropertyIDs to sort by.  It should contain a column for each sort.
    /// <p />  
    /// The first row holds the PropertyID. The list of supported  property IDs defined in <c>PT_PROPIDS</c>:
    /// PT_PROPIDS.PT_PROPID_OBJECTID,
    /// PT_PROPIDS.PT_PROPID_SITEMAP_GID,
    /// PT_PROPIDS.PT_PROPID_NAME,
    /// PT_PROPIDS.PT_PROPID_DESCRIPTION, 
    /// PT_PROPIDS.PT_PROPID_SITEMAP_LINKURL,
    /// PT_PROPIDS.PT_PROPID_SITEMAP_LINKCLASSID,
    /// PT_PROPIDS.PT_PROPID_SITEMAP_LINKOBJECTID,
    /// PT_PROPIDS.PT_PROPID_SITEMAP_LINKPAGEID;
    /// <p />
    /// The second row holds the sort direction, PTSortedArrayWrapper. PTSAW_SORT_ASCENDING or PTSortedArrayWrapper.PTSAW_SORT_DESCENDING
    /// </param>
    /// <param name="owner">Current Activityspace</param>
    /// <returns>A <c>PTSortedArrayWrapper</c> object containing Mandatory Experience Definition Links in the specified order</returns>
    public static PTSortedArrayWrapper CreateSortedArray(IPTQueryResult qrMandLinks,
     String strLinkOrder, AActivitySpace owner)
    {
      XPStringTokenizer strTok = new XPStringTokenizer(strLinkOrder, ",");
      XPHashtable tableLinkOrder = new XPHashtable();
      int nCount = 0;
      while (strTok.HasMoreElements())
      {
        tableLinkOrder.PutElement(XPConvert.ToInteger(strTok.GetNextToken()),
         nCount++);
      }
      int nNumLinks = qrMandLinks.RowCount();
      XPArrayList xparrLinks = new XPArrayList(nNumLinks);
      for (int i = 0;i < nNumLinks;i++)
      {
        XPArrayList linkProperties = new XPArrayList(9);
        Object intObjectID = qrMandLinks.ItemAsObject(i, PT_PROPIDS.PT_PROPID_OBJECTID);
        String strGID = qrMandLinks.ItemAsString(i, PT_PROPIDS.PT_PROPID_SITEMAP_GID);
        String strName = qrMandLinks.ItemAsString(i, PT_PROPIDS.PT_PROPID_NAME);
        String strDesc = qrMandLinks.ItemAsString(i, PT_PROPIDS.PT_PROPID_DESCRIPTION);
        String strURL = qrMandLinks.ItemAsString(i, PT_PROPIDS.PT_PROPID_SITEMAP_LINKURL);
        Object intClassID = qrMandLinks.ItemAsObject(i, PT_PROPIDS.PT_PROPID_SITEMAP_LINKCLASSID);
        Object intLinkObjectID = qrMandLinks.ItemAsObject(i, PT_PROPIDS.PT_PROPID_SITEMAP_LINKOBJECTID);
        Object intLinkCommID = qrMandLinks.ItemAsObject(i, PT_PROPIDS.PT_PROPID_SITEMAP_LINKPAGEID);
        linkProperties.Add(intObjectID);
        linkProperties.Add(strGID);
        linkProperties.Add(strName);
        linkProperties.Add(strDesc);
        linkProperties.Add(strURL);
        linkProperties.Add(intClassID);
        linkProperties.Add(intLinkObjectID);
        linkProperties.Add(intLinkCommID);
        Object oIndex = tableLinkOrder.GetElement(intObjectID);
        if (oIndex != null)
        {
          linkProperties.Add(oIndex);
        } else
        {
          linkProperties.Add(nCount++);
        }
        xparrLinks.Add(linkProperties);
      }
      int[] arrColIDs = new int[9];
      arrColIDs[0] = PT_PROPIDS.PT_PROPID_OBJECTID;
      arrColIDs[1] = PT_PROPIDS.PT_PROPID_SITEMAP_GID;
      // we're setting this to the gadget title bar state
      arrColIDs[2] = PT_PROPIDS.PT_PROPID_NAME;
      // we're setting this to true if there are comm prefs
      arrColIDs[3] = PT_PROPIDS.PT_PROPID_DESCRIPTION;
      arrColIDs[4] = PT_PROPIDS.PT_PROPID_SITEMAP_LINKURL;
      arrColIDs[5] = PT_PROPIDS.PT_PROPID_SITEMAP_LINKCLASSID;
      arrColIDs[6] = PT_PROPIDS.PT_PROPID_SITEMAP_LINKOBJECTID;
      arrColIDs[7] = PT_PROPIDS.PT_PROPID_SITEMAP_LINKPAGEID;
      arrColIDs[8] = SiteMapPortletPrefsSortedLinksModel.LINKS_COL_LINK_INDEX;
      int[][] arrSort;
      if (("").Equals(strLinkOrder))
      {
        arrSort = new int[1][];
        for (int j2c_index = 0;j2c_index < 1;j2c_index++)
        {
          arrSort[j2c_index] = new int[2];
        }
        arrSort[0][0] = PT_PROPIDS.PT_PROPID_NAME;
        arrSort[0][1] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
      } else
      {
        arrSort = new int[1][];
        for (int j2c_index = 0;j2c_index < 1;j2c_index++)
        {
          arrSort[j2c_index] = new int[2];
        }
        arrSort[0][0] = SiteMapPortletPrefsSortedLinksModel.LINKS_COL_LINK_INDEX;
        arrSort[0][1] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
      }
      Object[][] arrLinks = new Object[9][];
      for (int j2c_index = 0;j2c_index < 9;j2c_index++)
      {
        arrLinks[j2c_index] = new Object[nNumLinks];
      }
      for (int j = 0;j < nNumLinks;j++)
      {
        arrLinks[0][j] = ((XPArrayList) xparrLinks.GetElement(j)).GetElement(0);
        arrLinks[1][j] = ((XPArrayList) xparrLinks.GetElement(j)).GetElement(1);
        arrLinks[2][j] = ((XPArrayList) xparrLinks.GetElement(j)).GetElement(2);
        arrLinks[3][j] = ((XPArrayList) xparrLinks.GetElement(j)).GetElement(3);
        arrLinks[4][j] = ((XPArrayList) xparrLinks.GetElement(j)).GetElement(4);
        arrLinks[5][j] = ((XPArrayList) xparrLinks.GetElement(j)).GetElement(5);
        arrLinks[6][j] = ((XPArrayList) xparrLinks.GetElement(j)).GetElement(6);
        arrLinks[7][j] = ((XPArrayList) xparrLinks.GetElement(j)).GetElement(7);
        arrLinks[8][j] = ((XPArrayList) xparrLinks.GetElement(j)).GetElement(8);
        }
      PTSortedArrayWrapper arrSortedLinks = new PTSortedArrayWrapper();
      arrSortedLinks.Initialize(arrColIDs, arrLinks, arrSort, true);
      return arrSortedLinks;
    }
  }
}