using System;
using junit.framework;
using com.plumtree.openfoundation.mockobjects;
using com.plumtree.openfoundation.util;
using com.plumtree.portalpages.browsing.portalsettings.password;
using com.plumtree.portalpages.common.constants;
using com.plumtree.portalpages.pei;
using com.plumtree.portaluiinfrastructure.mock.friendlyurl;
using com.plumtree.portaluiinfrastructure.mock.tags;
using com.plumtree.server;
using com.plumtree.server.mock.core;
using com.plumtree.server.mock.users;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portalpages.test.pei
{
  /// <summary>
  /// This class handles testing the PTPasswordActions and PTCreateAccountActions PEIs.
  /// In reality, it is testing the underlying PasswordValidator that they both use,
  /// and so is not really a unit test.
  /// </summary>
  /// <author>dhayler</author>
  public class PasswordPEIsTest : TestCase
  {
    //  Test Data
    public static readonly String I18N_FILE = ResourceConstants.PORTALBROWSINGMSGS;
    public const String USER_NAME = "testuser";
    protected IApplication m_application = null;
    protected AActivitySpace m_asSpace = null;
    protected MockEnvironment m_mockSpace = null;
    protected ISessionManager m_subSession = null;
    protected MockPTSession m_userSession = null;
    protected MockPTServerConfigSettings m_serverConfigSettings = null;
    /*
     * @see junit.framework.TestCase#setUp()
     */
     
    public override void setUp()
    {
      m_application = new Application("", "");
      ApplicationManager.SetTestInstance(new MockApplicationManager());
      ((MockApplicationManager) ApplicationManager.GetInstance()).SetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString(),
       m_application);
      // helper member that shadows the real Activity Space for convenience purposes.
      m_mockSpace = new MockEnvironment();
      m_asSpace = m_mockSpace;
      m_subSession = SessionManagerFactory.GetSessionManager(new XPSessionSimulator());
      m_asSpace.SetSubSession(m_subSession);
      m_userSession = (MockPTSession) m_asSpace.GetUserSession();
      ((MockPTSessionInfo) m_userSession.GetSessionInfo()).m_LoginName = USER_NAME;
      m_serverConfigSettings = (MockPTServerConfigSettings) m_userSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
       true);
      // Default settings
      m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_LENGTH,
       7);
      m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_CASING,
       false);
      m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_NUMBERS,
       false);
      m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_PUNCTUATION,
       false);
      m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_NO_USER_IDS,
       true);
    }
    /*
     * @see junit.framework.TestCase#tearDown()
     */
     
    public override void tearDown()
    {
      m_application = null;
      m_asSpace = null;
      m_mockSpace = null;
      m_subSession = null;
      m_userSession = null;
      m_serverConfigSettings = null;
    }
    
    /// <summary>
    /// This method tests the various password rules implemented by the PTPasswordActions PEI.
    /// </summary>
    public virtual void testBasicPasswordRulesInPasswordPEI()
    {
      internalTestBasicPasswordRules(new PasswordPEI(new PTPasswordActions()));
      }
    
    /// <summary>
    /// This method tests the various password rules implemented by the PTCreateAccountActions PEI.
    /// </summary>
    public virtual void testBasicPasswordRulesInCreateAccountPEI()
    {
      internalTestBasicPasswordRules(new PasswordPEI(new PTCreateAccountActions()));
      }
    
    /// <summary>
    /// This helper method tests the various password rules implemented by the PEI.
    /// </summary>
    /// <param name="_pei">the PEI to test.</param>
    private void internalTestBasicPasswordRules(PasswordPEI _pei)
    {
      String strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession,
       "123456");
      // Only check that it returns an i18nized string.  We will check the string IDs later.
      Assert.assertEquals("Too short password length not rejected correctly.",
       MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);
      Assert.assertEquals("Incorrect i18n string ID returned for too short password error.",
       1628, m_mockSpace.m_nLastI18NStringIndex);
      Assert.assertEquals("Incorrect i18n string File returned for too short password error.",
       I18N_FILE, m_mockSpace.m_strLastI18NStringFile);
      strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "12345678901234567890123");
      // Only check that it returns an i18nized string.  We will check the string IDs later.
      Assert.assertEquals("Too long password length not rejected correctly.",
       MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);
      Assert.assertEquals("Incorrect i18n string ID returned for too long password error.",
       1291, m_mockSpace.m_nLastI18NStringIndex);
      Assert.assertEquals("Incorrect i18n string File returned for too long password error.",
       I18N_FILE, m_mockSpace.m_strLastI18NStringFile);
      strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, USER_NAME);
      // Only check that it returns an i18nized string.  We will check the string IDs later.
      Assert.assertEquals("Password using the user ID not rejected correctly.",
       MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);
      Assert.assertEquals("Incorrect i18n string ID returned for password using the user ID error.",
       1982, m_mockSpace.m_nLastI18NStringIndex);
      Assert.assertEquals("Incorrect i18n string File returned for password using the user ID error.",
       I18N_FILE, m_mockSpace.m_strLastI18NStringFile);
      strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "12345678");
      Assert.assertNull("Valid password rejected incorrectly.", strResult);
      }
    
    /// <summary>
    /// This method tests the password length rule implemented by the Password Actions PEI.
    /// </summary>
    public virtual void testPasswordLengthRuleInPasswordPEI()
    {
      internalTestPasswordLengthRule(new PasswordPEI(new PTPasswordActions()));
      }
    
    /// <summary>
    /// This method tests the password length rule implemented by the Create Account Actions PEI.
    /// </summary>
    public virtual void testPasswordLengthRuleInCreateAccountPEI()
    {
      internalTestPasswordLengthRule(new PasswordPEI(new PTCreateAccountActions()));
      }
    
    /// <summary>
    /// This method tests the password length rule implemented by the PEI.
    /// </summary>
    private void internalTestPasswordLengthRule(PasswordPEI _pei)
    {
      // Set password length requirement
      m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_LENGTH,
       8);
      String strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession,
       "1234567");
      // Only check that it returns an i18nized string.  We will check the string IDs later.
      Assert.assertEquals("Too short password length not rejected correctly.",
       MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);
      Assert.assertEquals("Incorrect i18n string ID returned for too short password error.",
       1628, m_mockSpace.m_nLastI18NStringIndex);
      Assert.assertEquals("Incorrect i18n string File returned for too short password error.",
       I18N_FILE, m_mockSpace.m_strLastI18NStringFile);
      strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "12345678");
      Assert.assertNull("Proper length password rejected incorrectly.", strResult);
      }
    
    /// <summary>
    /// This method tests the password casing rule implemented by the Password Actions PEI.
    /// </summary>
    public virtual void testPasswordCasingRuleInPasswordPEI()
    {
      internalTestPasswordCasingRule(new PasswordPEI(new PTPasswordActions()));
      }
    
    /// <summary>
    /// This method tests the password casing rule implemented by the Create Account Actions PEI.
    /// </summary>
    public virtual void testPasswordCasingRuleInCreateAccountPEI()
    {
      internalTestPasswordCasingRule(new PasswordPEI(new PTCreateAccountActions()));
      }
    
    /// <summary>
    /// This method tests the password casing rule implemented by the PEI.
    /// </summary>
    private void internalTestPasswordCasingRule(PasswordPEI _pei)
    {
      // Set password casing requirement to true
      m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_CASING,
       true);
      String strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession,
       "testcasing");
      // Only check that it returns an i18nized string.  We will check the string IDs later.
      Assert.assertEquals("All lower case password not rejected correctly.",
       MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);
      Assert.assertEquals("Incorrect i18n string ID returned for all lower case password error.",
       1983, m_mockSpace.m_nLastI18NStringIndex);
      Assert.assertEquals("Incorrect i18n string File returned for all lower case password error.",
       I18N_FILE, m_mockSpace.m_strLastI18NStringFile);
      strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "TESTCASING");
      // Only check that it returns an i18nized string.  We will check the string IDs later.
      Assert.assertEquals("All upper case password not rejected correctly.",
       MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);
      Assert.assertEquals("Incorrect i18n string ID returned for all upper case password error.",
       1983, m_mockSpace.m_nLastI18NStringIndex);
      Assert.assertEquals("Incorrect i18n string File returned for all upper case password error.",
       I18N_FILE, m_mockSpace.m_strLastI18NStringFile);
      strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "1234567");
      // Only check that it returns an i18nized string.  We will check the string IDs later.
      Assert.assertEquals("All number password not rejected correctly.", MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING,
       strResult);
      Assert.assertEquals("Incorrect i18n string ID returned for all number password error.",
       1983, m_mockSpace.m_nLastI18NStringIndex);
      Assert.assertEquals("Incorrect i18n string File returned for all number password error.",
       I18N_FILE, m_mockSpace.m_strLastI18NStringFile);
      strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "!@#$%^&*");
      // Only check that it returns an i18nized string.  We will check the string IDs later.
      Assert.assertEquals("All punctuation password not rejected correctly.",
       MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);
      Assert.assertEquals("Incorrect i18n string ID returned for all punctuation password error.",
       1983, m_mockSpace.m_nLastI18NStringIndex);
      Assert.assertEquals("Incorrect i18n string File returned for all punctuation password error.",
       I18N_FILE, m_mockSpace.m_strLastI18NStringFile);
      strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "TestCasing");
      Assert.assertNull("Properly cased password rejected incorrectly.", strResult);
      }
    
    /// <summary>
    /// This method tests the password number rule implemented by the Password Actions PEI.
    /// </summary>
    public virtual void testPasswordNumberRuleInPasswordPEI()
    {
      internalTestPasswordNumberRule(new PasswordPEI(new PTPasswordActions()));
      }
    
    /// <summary>
    /// This method tests the password number rule implemented by the Create Account Actions PEI.
    /// </summary>
    public virtual void testPasswordNumberRuleInCreateAccountPEI()
    {
      internalTestPasswordNumberRule(new PasswordPEI(new PTCreateAccountActions()));
      }
    
    /// <summary>
    /// This method tests the password number rule implemented by the PEI.
    /// </summary>
    private void internalTestPasswordNumberRule(PasswordPEI _pei)
    {
      // Set password number requirement to true
      m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_NUMBERS,
       true);
      String strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession,
       "testnumbers");
      // Only check that it returns an i18nized string.  We will check the string IDs later.
      Assert.assertEquals("No number password not rejected correctly.", MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING,
       strResult);
      Assert.assertEquals("Incorrect i18n string ID returned for no casing password error.",
       1984, m_mockSpace.m_nLastI18NStringIndex);
      Assert.assertEquals("Incorrect i18n string File returned for no casing password error.",
       I18N_FILE, m_mockSpace.m_strLastI18NStringFile);
      strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "test1Number");
      Assert.assertNull("Properly numberalized password rejected incorrectly.",
       strResult);
    }
    
    /// <summary>
    /// This method tests the password punctuation rule implemented by the Password Actions PEI.
    /// </summary>
    public virtual void testPasswordPunctuationRuleInPasswordPEI()
    {
      internalTestPasswordPunctuationRule(new PasswordPEI(new PTPasswordActions()));
      }
    
    /// <summary>
    /// This method tests the password punctuation rule implemented by the CreateAccount Actions PEI.
    /// </summary>
    public virtual void testPasswordPunctuationRuleInCreateAccountPEI()
    {
      internalTestPasswordPunctuationRule(new PasswordPEI(new PTCreateAccountActions()));
      }
    
    /// <summary>
    /// This method tests the password punctuation rule implemented by the PEI.
    /// </summary>
    private void internalTestPasswordPunctuationRule(PasswordPEI _pei)
    {
      // Set password number requirement to true
      m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_PUNCTUATION,
       true);
      String strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession,
       "testpunctuation");
      // Only check that it returns an i18nized string.  We will check the string IDs later.
      Assert.assertEquals("No punctuation password not rejected correctly.",
       MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);
      Assert.assertEquals("Incorrect i18n string ID returned for no punctuation password error.",
       1985, m_mockSpace.m_nLastI18NStringIndex);
      Assert.assertEquals("Incorrect i18n string File returned for no punctuation password error.",
       I18N_FILE, m_mockSpace.m_strLastI18NStringFile);
      strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "testpunctuat!on");
      Assert.assertNull("Properly punctuated password rejected incorrectly.",
       strResult);
    }
    
    /// <summary>
    /// This method tests the password reuse history rule implemented by the Create Account PEI.
    /// </summary>
    public virtual void testPasswordReuseHistoryInCreateAccountPEI()
    {
      // Set password reuse history to 12 (max)
      m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_REUSE_HISTORY,
       12);
      // We don't need to set any password history here because newly created accounts
      // do not have a history.
      PasswordPEI pei = new PasswordPEI(new PTCreateAccountActions());
      String strResult = pei.CallPEIPasswordMethod(m_asSpace, m_userSession,
       "testpassword");
      Assert.assertNull("Newly created accounts should not fail due to password reuse.",
       strResult);
    }
    
    /// <summary>
    /// This method tests the password reuse history rule implemented by the Change Password PEI.
    /// </summary>
    public virtual void testPasswordReuseHistoryInPasswordPEI()
    {
      // Set password reuse history to 12 (max)
      m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_REUSE_HISTORY,
       12);
      String strPreviousPassword = "usedPass";
      // Set password in mock history
      ((MockPTUserManager) m_userSession.GetUsers()).m_PasswordInHistory = strPreviousPassword;
      PasswordPEI pei = new PasswordPEI(new PTPasswordActions());
      String strResult = pei.CallPEIPasswordMethod(m_asSpace, m_userSession,
       strPreviousPassword);
      // Only check that it returns an i18nized string.  We will check the string IDs later.
      Assert.assertEquals("Reused password not rejected correctly.", MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING,
       strResult);
      Assert.assertEquals("Incorrect i18n string ID returned for reused password error.",
       1998, m_mockSpace.m_nLastI18NStringIndex);
      Assert.assertEquals("Incorrect i18n string File returned for reused password error.",
       I18N_FILE, m_mockSpace.m_strLastI18NStringFile);
      strResult = pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "unusedPass");
      Assert.assertNull("Unused password rejected incorrectly.", strResult);
      }
    
    /// <summary>
    /// This method tests the behavior if the password is missing in the Password Actions PEI.
    /// This is not relevant in the Create Account PEI, so it is not tested.
    /// </summary>
    public virtual void testNoPasswordInPasswordPEI()
    {
      PasswordPEI pei = new PasswordPEI(new PTPasswordActions());
      String strResult = pei.CallPEIPasswordMethod(m_asSpace, m_userSession,
       null);
      // Test defaults
      Assert.assertNull("PEI should return null if password is missing.",
       strResult);
    }
    
    /// <summary>
    /// Private inner class to help us to run the same test on 2 PEIs with different
    /// method signatures.
    /// </summary>
    private class PasswordPEI
    {
      private IPasswordActions m_pwPEI = null;
      private ICreateAccountActions m_caPEI = null;
      public PasswordPEI(IPasswordActions _pei)
      {
        m_pwPEI = _pei;
      }
      public PasswordPEI(ICreateAccountActions _pei)
      {
        m_caPEI = _pei;
      }
      public virtual String CallPEIPasswordMethod(AActivitySpace _asCurrentSpace,
       IPTSession _ptUserSession, String _strPassword)
      {
        if (m_pwPEI != null)
        {
          // Fill out required control arguments
          XPHashtable data = new XPHashtable();
          if (_strPassword != null)
          {
            data.PutElement(PasswordView.HTML_INPUT_NEW_PASSWORD, new String[]{_strPassword});
            }
          return m_pwPEI.OnBeforeChangePassword(_asCurrentSpace, _ptUserSession,
           data);
        }
        return m_caPEI.OnBeforeCreateAccount(_asCurrentSpace, _ptUserSession,
         USER_NAME, _strPassword);
      }
    }
  }
}