using System;
using junit.framework;
using com.plumtree.openfoundation.util;
using com.plumtree.portalpages.rest.utils;
namespace com.plumtree.portalpages.test.rest.utils
{
  /// <summary>
  /// This tests the simple JSON parser that's used for the
  /// KD REST API calls.
  /// It is very limited, and does not handle arrays, or non-string
  /// object types.  It also only handles escaped quotes and slashes.
  /// </summary>
  public class JSONParserTest : TestCase
  {
    public virtual void testParseError()
    {
      String strErrorJSON = "blah blah blah";
      IJSONParser parser = JSONParser.GetInstance();
      bool bException = false;
      try
      {
        parser.ParseJSONObject(strErrorJSON);
      }
      catch (XPFormatException xpfe)
      {
        bException = true;
      }
      if (!bException)
      {
        fail("Parser should fail when parsing non-JSON object.");
      }
    }
    public virtual void testMissingCommasParseError()
    {
      String strErrorJSON = "{ \"document\": { \"name\": \"test\" \"description\": \"test test\" } }";
      IJSONParser parser = JSONParser.GetInstance();
      bool bException = false;
      try
      {
        parser.ParseJSONObject(strErrorJSON);
      }
      catch (XPFormatException xpfe)
      {
        bException = true;
      }
      if (!bException)
      {
        fail("Parser should fail when parsing non-JSON object.");
      }
    }
    public virtual void testSimpleObject()
    {
      String strDocumentJSON = "{ \"document\": { \"name\": \"test\", \"description\": \"test test\" } }";
      IJSONParser parser = JSONParser.GetInstance();
      parser.ParseJSONObject(strDocumentJSON);
      assertEquals("The parsed object name was incorrect.", "document", parser.GetObjectName());
      assertEquals("The parsed name attribute was incorrect.", "test", parser.GetMember("name"));
      assertEquals("The parsed description attribute was incorrect.", "test test",
       parser.GetMember("description"));
    }
    public virtual void testSimpleObjectCommentFiltered()
    {
      // Comment filtering is often used to thwart JSON security attacks.  We wrap the text in /* */.
      String strDocumentJSON = "/* { \"document\": { \"name\": \"test\", \"description\": \"test test\" } } */";
      IJSONParser parser = JSONParser.GetInstance();
      parser.ParseJSONObject(strDocumentJSON);
      assertEquals("The parsed object name was incorrect.", "document", parser.GetObjectName());
      assertEquals("The parsed name attribute was incorrect.", "test", parser.GetMember("name"));
      assertEquals("The parsed description attribute was incorrect.", "test test",
       parser.GetMember("description"));
    }
    public virtual void testSimpleObjectCommentFilteredWithWhitespace()
    {
      // Comment filtering is often used to thwart JSON security attacks.  We wrap the text in /* */.
      String strDocumentJSON = " /* { \"document\": { \"name\": \"test\", \"description\": \"test test\" } } */ ";
      IJSONParser parser = JSONParser.GetInstance();
      parser.ParseJSONObject(strDocumentJSON);
      assertEquals("The parsed object name was incorrect.", "document", parser.GetObjectName());
      assertEquals("The parsed name attribute was incorrect.", "test", parser.GetMember("name"));
      assertEquals("The parsed description attribute was incorrect.", "test test",
       parser.GetMember("description"));
    }
    public virtual void testSimpleObjectWithWhiteSpace()
    {
      String strDocumentJSON = "{     \"error\":   {   \"text\":  \"test\",   \"code\":" + "\"123\"  }      }";
      IJSONParser parser = JSONParser.GetInstance();
      parser.ParseJSONObject(strDocumentJSON);
      assertEquals("The parsed object name was incorrect.", "error", parser.GetObjectName());
      assertEquals("The parsed text attribute was incorrect.", "test", parser.GetMember("text"));
      assertEquals("The parsed code attribute was incorrect.", "123", parser.GetMember("code"));
      }
    public virtual void testSimpleObjectWithNoWhiteSpace()
    {
      String strDocumentJSON = "{\"error\":{\"text\":\"test\",\"code\":\"123\"}}";
      IJSONParser parser = JSONParser.GetInstance();
      parser.ParseJSONObject(strDocumentJSON);
      assertEquals("The parsed object name was incorrect.", "error", parser.GetObjectName());
      assertEquals("The parsed text attribute was incorrect.", "test", parser.GetMember("text"));
      assertEquals("The parsed code attribute was incorrect.", "123", parser.GetMember("code"));
      }
    public virtual void testSimpleObjectWithEscapedQuotes()
    {
      String strDocumentJSON = "{\"error\":{\"text\":\"test \\\"me\\\" and a slash like \\\\\",\"code\":\"123\"}}";
      IJSONParser parser = JSONParser.GetInstance();
      parser.ParseJSONObject(strDocumentJSON);
      assertEquals("The parsed object name was incorrect.", "error", parser.GetObjectName());
      assertEquals("The parsed text attribute with escaped quotes was incorrect.",
       "test \"me\" and a slash like \\", parser.GetMember("text"));
      assertEquals("The parsed code attribute was incorrect.", "123", parser.GetMember("code"));
      }
    public virtual void testArrayValues()
    {
      String strDocumentJSON = "{ \"test\": { \"numarray\": [200, 300, 40000000], \"strarray\": [\"a\", \"ab\",\"abc\"], \"boolarray\": [true,false, true]} }";
      JSONParser parser = new JSONParser();
      parser.ParseJSONObject(strDocumentJSON);
      assertEquals("The parsed object name was incorrect.", "test", parser.GetObjectName());
      XPArrayList numList = (XPArrayList) parser.GetMemberAsList("numarray");
      XPArrayList strList = (XPArrayList) parser.GetMemberAsList("strarray");
      XPArrayList boolList = (XPArrayList) parser.GetMemberAsList("boolarray");
      assertNotNull("The \'numarray\' member was not found in the parsed JSON\'",
       numList);
      assertNotNull("The \'strarray\' member was not found in the parsed JSON\'",
       strList);
      assertNotNull("The \'boolarray\' member was not found in the parsed JSON\'",
       boolList);
      assertEquals("Parsed numeric array value was incorrect.", 200, (int) numList.GetElement(0));
      assertEquals("Parsed numeric array value was incorrect.", 300, (int) numList.GetElement(1));
      assertEquals("Parsed numeric array value was incorrect.", 40000000,
       (int) numList.GetElement(2));
      assertEquals("Parsed numeric array value was incorrect.", "a", (String) strList.GetElement(0));
      assertEquals("Parsed numeric array value was incorrect.", "ab", (String) strList.GetElement(1));
      assertEquals("Parsed numeric array value was incorrect.", "abc", (String) strList.GetElement(2));
      assertEquals("Parsed numeric array value was incorrect.", true, (bool) boolList.GetElement(0));
      assertEquals("Parsed numeric array value was incorrect.", false, (bool) boolList.GetElement(1));
      assertEquals("Parsed numeric array value was incorrect.", true, (bool) boolList.GetElement(2));
      }
  }
}