package com.plumtree.portalpages.admin.editors.crawler;


// Plumtree Server

import com.plumtree.openfoundation.util.*;
import com.plumtree.openkernel.config.IOKComponent;
import com.plumtree.openkernel.config.IOKContext;
import com.plumtree.openkernel.factory.OKConfigFactory;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.editors.common.setjob.*;
import com.plumtree.portalpages.admin.editors.doctypemap.*;
import com.plumtree.portalpages.common.documentlanguage.*;
import com.plumtree.portalpages.common.linksettings.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.providerframework.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity.*;
import com.plumtree.portaluiinfrastructure.expandablelist.selectptobject.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.crawlers.IPTCrawlerConfig;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.UIError;
import com.plumtree.uiinfrastructure.errordisplay.UIErrorHandler;
import com.plumtree.uiinfrastructure.errordisplay.UIErrorLevel;
import com.plumtree.uiinfrastructure.errordisplay.UIErrorScope;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.tree.*;
import com.plumtree.xpshared.config.ConfigPathResolver;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * This is the main Editor Model for the Crawler Editor.
 *
 * @author Julie Fournier
 */
public abstract class CrawlerModel extends ObjEditorModel
	implements ICrawlerModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.editors.crawler.CrawlerModel");

	/**
	 * Query String param: data source ID
	 */
	public static final String QS_CRAWLEREDITOR_DATASOURCE_ID = "dataSourceID";

	/**
	 * When the mirroring check box is checked, the destination folders selection becomes single select.
	 * That's why we need to notify the User.
	 */
	public boolean m_bDisplayMirroringAlert = false;
	
	/**
	 * Determines whether refresh settings should be applied to previously imported cards
	 */
	public boolean m_bRefreshExistingCards = false;

	/**
	 * Number of deleted cards in the deletion history.
	 */
	public int m_nDeletionCards = 0;

	/**
	 * Number of deleted cards in the rejection history.
	 */
	public int m_nRejectionCards = 0;
	
	/**
	 * Name of the Data Source
	 */
	public String m_sDataSourceName = "";

	/** Crawler Start Location Property Bag */
	protected IXPPropertyBag m_pbStartLocation;
	
	
	/** Crawler Runtime Configuration Property Bag 
	 *  Put infomation about the number of threads, etc. here*/
	protected IXPPropertyBag crawlerRuntimeConfig;

	/**
	 * Flag to know whether the Crawler's DS supports mirroring.
	 */
	protected boolean m_bDSSupportMirroring = false;
	
	/**
	 * Flag to know whether this Crawler supports ACL import or no.
	 */
	protected boolean m_bSupportsACLImport = false;

	/**
	 * Flag to know whether this Crawler is configured to import only new links.
	 */
	protected boolean m_bImportOnlyNewLinks;

	/**
	 * Number of sections to display on the Document Type Map page.
	 */
	private int m_nDocMapSectionCount;

	/**
	 * Clears the crawler's deletion history.
	 *
	 */
	public void ClearDeletionHistory()
	{
		m_nDeletionCards = ((IPTCrawler) m_ptObject).ClearDeletedCardsMemory(true, false);
	}
	
	/**
	 * Returns the Data Source name.
	 *
	 */
	public String GetDataSourceName()
	{
		return m_sDataSourceName;
	}

	/**
	 * Clears the crawler's rejection history.
	 *
	 */
	public void ClearRejectionHistory()
	{
		m_nRejectionCards = ((IPTCrawler) m_ptObject).ClearCardRejectsMemory(true);
	}
	
	/**
	 * Performs the change mirroring action
	 *
	 */
	public void ChangeMirroringStatus()
	{
		int nCrawlerSettings = GetCrawlerSettings();
		
		SelectPTObjectModel mySelectDestFoldersModel = (SelectPTObjectModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_SELECT_FOLDER);
		mySelectDestFoldersModel.ResetTreeID();		
		
		// JF- We have decided to mirror, we need to change the tree to multi select to single.
		if (0 != (PT_CRAWLER_SETTINGS.PT_CRAWLER_MIRROR & nCrawlerSettings))
		{
			mySelectDestFoldersModel.SetTreeMultiSelect(false);
			
			PTGrowableSortedArrayWrapper saItems = mySelectDestFoldersModel.GetItems();
			
			if (saItems.GetCount() > 1)
			{
				int[] arrRowsToDelete = new int[saItems.GetCount() - 1];
				
				for (int i = 0; i < saItems.GetCount() - 1; i ++)
				{
					arrRowsToDelete[i] = i + 1;
				}
				
				saItems.RemoveRows(arrRowsToDelete);
				m_bDisplayMirroringAlert = true;
			}
		}
		else
		{
			// JF- We have decided to not mirror, we need to change the tree to single select to multi.
			mySelectDestFoldersModel.SetTreeMultiSelect(true);
		}

	}

	/**
	 * Code executed after the object is stored.
	 *
	 */
	public void DoTaskAfterStore()
	{
		((SetJobModel) m_asOwner.GetModel(SetJobModel.STR_MVC_CLASS_NAME)).DoTaskAfterStoreSetJob();
		
		if (m_bRefreshExistingCards)
			ApplyRefreshSettingsToExistingCards();
	}

	/**
	 * Code executed before the object is stored.
	 *
	 */
	public void DoTaskBeforeStore()
	{
		StoreDestFolders();
		StoreContentLanguage();
		StoreLinkSettings();
		StoreAdvSettings();

		// Call Finish on the Card Security page
		((ObjectSecurityModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_CARD_SECURITY)).Finish();

		((SetJobModel) m_asOwner.GetModel(SetJobModel.STR_MVC_CLASS_NAME)).DoTaskBeforeStoreSetJob();

		GbDocTypeListModel myGbDocTypeModel;

		for (int i = 0; i < m_nDocMapSectionCount; i++)
		{
			myGbDocTypeModel = (GbDocTypeListModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_TYPE_MAP +
					i);
			myGbDocTypeModel.Finish();
		}
	}

	/**
	 * This method will be called when you come back to the main Editor from a sub Editor.
	 *
	 */
	public void DoTaskOnComeBackFromSubEditor()
	{
		super.DoTaskOnComeBackFromSubEditor();
		
		((SelectPTObjectModel) m_asOwner.GetModel(SetJobModel.STR_MVC_STORAGE_NAME_SELECT_JOB)).DoTaskOnComeBackFromSubEditor();

		GbDocTypeListModel myGbDocTypeModel;

		for (int i = 0; i < m_nDocMapSectionCount; i++)
		{
			myGbDocTypeModel = (GbDocTypeListModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_TYPE_MAP +
					i);
			myGbDocTypeModel.DoTaskOnComeBackFromSubEditor(i);
		}
	}

	/**
	 * Code executed when the Editor is starting.
	 *
	 * @param _htFormData
	 */
	public void DoTaskOnStartEditor(XPHashtable _htFormData)
	{
		// JF- Check if the Crawler's DS supports mirroring and ACL import
		InitMirroringAndACLImportFlags();
		
		InitDataSourceName();

		// JF- Init the select Folder array
		StartSelectFolder();

		// JF- Init the Set Job plug-in page
		((SetJobModel) m_asOwner.GetModel(SetJobModel.STR_MVC_CLASS_NAME)).DoTaskOnStartSetJob(m_ptObject,
			m_Session, GetEditorType());

		// JF- Init the language Section
		StartContentLanguage();

		// JF- Init the link settings page
		StartLinkSettings();

		// JF- Init the Card Security page
		StartCardSecurity();

		// JF- Init the Importing Docs view
		StartImportSettings();

		// JF- Init the Document Type Map page
		StartGbDocTypeList();

		// JF- Hide the Security page in create mode
		if (EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType())
		{
			((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
				ObjectSecurityDP.STR_MVC_CLASS_NAME);
			((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
				MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);
		}
	}
	
	/**
	 * Helper method to get the Data Source name.
	 */
	private void InitDataSourceName()
	{
		IPTDataSource ptDataSource = (IPTDataSource) m_Session.GetDataSources()
			.Open(((IPTCrawler) m_ptObject).GetDataSourceID(), false);
		m_sDataSourceName = ptDataSource.GetLocalizedName();
	}

	/**
	 * Returns crawler settings
	 *
	 * @return int
	 */
	public int GetCrawlerSettings()
	{
		return ((IPTCrawler) m_ptObject).GetSettings();
	}

	/**
	 * Returns the number of Sections to display on the Document Type Map page.
	 *
	 * @return int
	 */
	public int GetDocMapSectionCount()
	{
		return m_nDocMapSectionCount;
	}

	/**
	 * Returns the flag m_bImportOnlyNewLinks.
	 *
	 * @return boolean
	 */
	public boolean GetImportOnlyNewLinks()
	{
		return m_bImportOnlyNewLinks;
	}

	/**
	 * Tells whether the Crawler's DS supports mirroring or no.
	 *
	 * @return boolean
	 */
	public boolean GetSupportMirroring()
	{
		return m_bDSSupportMirroring;
	}
	
	/**
	 * Tells whether the Crawler supports ACL import.
	 *
	 * @return boolean
	 */
	public boolean GetSupportsACLImport()
	{
		return m_bSupportsACLImport;
	}

	/**
	 * Returns taxonomist settings
	 *
	 * @return int
	 */
	public int GetTaxSettings()
	{
		return ((IPTCrawler) m_ptObject).GetTaxonomist().GetSettings();
	}

	/**
	 * Helper methods to save the settings on MainDP.
	 *
	 * @param _htFormData
	 *
	 * @return int Page status for server side validation
	 */
	public int SaveMainDP(XPHashtable _htFormData)
	{
		// JF- Save destination folders list
		((SelectPTObjectModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_SELECT_FOLDER)).SaveSection(_htFormData);

		// JF- Save the content security list	
		((ObjectSecurityModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_CARD_SECURITY)).SaveSection(_htFormData);

		// JF- Save settings
		int nCrawlerSettings = GetCrawlerSettings();
		int nTaxSettings     = GetTaxSettings();

		// JF- Mirroring checkbox
		if ((m_bDSSupportMirroring) &&
				(EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType()))
		{
			if (!_htFormData.ContainsKey(
						DestFolderSettingsView.HTMLCHECKBOX_MIRROR))
			{
				nCrawlerSettings = nCrawlerSettings & ~PT_CRAWLER_SETTINGS.PT_CRAWLER_MIRROR;	
			}
			else
			{
				nCrawlerSettings = nCrawlerSettings | PT_CRAWLER_SETTINGS.PT_CRAWLER_MIRROR;
			}
		}

		if (!_htFormData.ContainsKey(
					DestFolderSettingsView.HTMLCHECKBOX_APPROVE_DOCS))
		{
			nCrawlerSettings = nCrawlerSettings &
				~PT_CRAWLER_SETTINGS.PT_CRAWLER_IMPORT_ALL_CARDS_AS_APPROVED;
		}
		else
		{
			nCrawlerSettings = nCrawlerSettings |
				PT_CRAWLER_SETTINGS.PT_CRAWLER_IMPORT_ALL_CARDS_AS_APPROVED;
		}

		if (0 == (PT_CRAWLER_SETTINGS.PT_CRAWLER_MIRROR & nCrawlerSettings))
		{
			if (!_htFormData.ContainsKey(
						DestFolderSettingsView.HTMLCHECKBOX_APPLY_FILTERS))
			{
				nTaxSettings = nTaxSettings |
					PT_TAXONOMIST_SETTINGS.PT_TAXONOMIST_ONLY_APPLY_SUBFOLDER_FILTERS;
			}
			else
			{
				nTaxSettings = nTaxSettings &
					~PT_TAXONOMIST_SETTINGS.PT_TAXONOMIST_ONLY_APPLY_SUBFOLDER_FILTERS;
			}
		}
		
		if (m_bSupportsACLImport)
		{
			if (!_htFormData.ContainsKey(DestFolderSettingsView.HTMLCHECKBOX_IMPORT_SECURITY))
			{
				nCrawlerSettings = nCrawlerSettings & ~PT_CRAWLER_SETTINGS.PT_CRAWLER_ACL_IMPORT;
				
				/* since we aren't importing ACLs, we should reset the ACL Refresh Flag */
				nCrawlerSettings = nCrawlerSettings & ~PT_CRAWLER_SETTINGS.PT_CRAWLER_FORCE_ACL_REFRESH;
			}
			else
			{
				nCrawlerSettings = nCrawlerSettings | PT_CRAWLER_SETTINGS.PT_CRAWLER_ACL_IMPORT;
			}
		}

		try
		{
			((IPTCrawler) m_ptObject).SetSettings(nCrawlerSettings);
			((IPTCrawler) m_ptObject).GetTaxonomist().SetSettings(nTaxSettings);
		}
		catch (Exception e)
		{
			
				log.Error("CrawlerModel::SaveMainDP The following exception happened " +
					"when trying to set the new Crawler settings: " + e);
			
		}

		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * Calls the right save function according to the page name passed in argument.
	 *
	 * @param _sPageName
	 * @param _htFormData
	 *
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		int nPageStatus = super.SavePage(_sPageName, _htFormData);

		if (_sPageName.equals(SetJobDP.STR_MVC_CLASS_NAME))
		{
			((SetJobModel) m_asOwner.GetModel(SetJobModel.STR_MVC_CLASS_NAME)).SaveSection(_htFormData);
		}
		else if (_sPageName.equals(AdvSettingsDP.STR_MVC_CLASS_NAME))
		{
		    nPageStatus = SaveAdvSettingsPage(_htFormData);
		}
		else if (_sPageName.equals(LinkSettingsDP.STR_MVC_CLASS_NAME))
		{
			((LinkSettingsPlugInModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_LINK_SETTINGS)).SaveSection(_htFormData);
			PutApplyExisting(_htFormData);
		}

		return nPageStatus;
	}

	/**
	 * Helper method to start the security page.
	 */
	public void StartCardSecurity()
	{
		ObjectSecurityModel myCardSecurityModel = (ObjectSecurityModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_CARD_SECURITY);
		
		myCardSecurityModel.InitializeSecurityPage(m_ptObject, m_Session);

		myCardSecurityModel.SetSectionTitle(m_asOwner.GetString(231, "ptmsgs_portaladminmsgs"));
		// JF- We need to change to list key otherwise it will be the same as the security page.
		myCardSecurityModel.SetListKey(21);
	}

	/**
	 * Helper method to init the Content language common section.
	 *
	 */
	public void StartContentLanguage()
	{
		String sLanguage = ((IPTCrawler) m_ptObject).GetContentLanguage();

		((CardLanguagePlugInModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_LANGUAGE)).
			InitializePlugIn(sLanguage,
			m_asOwner.GetString(232, "ptmsgs_portaladminmsgs"),
			m_asOwner.GetString(233, "ptmsgs_portaladminmsgs"), false);
	}

	/**
	 * Helper method to init the Importing Docs view
	 *
	 */
	public void StartImportSettings()
	{
		int nCrawlerSettings = GetCrawlerSettings();

		if ((0 == (nCrawlerSettings &
				PT_CRAWLER_SETTINGS.PT_CRAWLER_IMPORT_ONLY_CRAWLER_NEW_CARDS)) &&
				(0 == (nCrawlerSettings &
				PT_CRAWLER_SETTINGS.PT_CRAWLER_IMPORT_ONLY_DATA_SOURCE_NEW_CARDS)))
		{
			m_bImportOnlyNewLinks = false;
		}
		else
		{
			m_bImportOnlyNewLinks = true;
		}
	}

	/**
	 * Helper method to init the Link Settings page.
	 *
	 */
	public void StartLinkSettings()
	{
		IPTCrawler ptCrawler = (IPTCrawler) m_ptObject;

		((LinkSettingsPlugInModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_LINK_SETTINGS)).InitializePlugInForCrawlers(ptCrawler.GetCardRefreshRateUnits(),
			ptCrawler.GetCardRefreshRateValue(), ptCrawler.GetCheckLinkOnly(),
			ptCrawler.GetCardMissingDocumentDeletionDelayUnits(),
			ptCrawler.GetCardMissingDocumentDeletionDelayValue(),
			ptCrawler.GetCardExpirationDelayUnits(),
			ptCrawler.GetCardExpirationDelayValue());
	}

	/**
	 * Helper method to start the Select Folder List.
	 *
	 */
	public void StartSelectFolder()
	{
		SelectPTObjectModel mySelectDestFoldersModel = (SelectPTObjectModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_SELECT_FOLDER);

		mySelectDestFoldersModel.InitializeSelectPTObject(m_asOwner.GetString(
				224, "ptmsgs_portaladminmsgs"),
		m_asOwner.GetString(225, "ptmsgs_portaladminmsgs"),
			PTImageType.ICON_FOLDER,
		m_asOwner.GetString(226, "ptmsgs_portaladminmsgs"),
			new int[] { PT_CLASSIDS.PT_CATALOGFOLDER_ID }, 20);
			
		mySelectDestFoldersModel.SetDefaultRowIcon(PTImageType.ICON_FOLDER);

		// JF- Set Strings to display when list is empty
		mySelectDestFoldersModel.SetEmptyListMessages(m_asOwner.GetString(227,
				"ptmsgs_portaladminmsgs"),
			m_asOwner.GetString(228, "ptmsgs_portaladminmsgs"));

		// Set the tree leaf type
		mySelectDestFoldersModel.SetTreeLeafType(TreeConstants.N_COMPOSITESELECT);

		// requiring edit access on directory folder; fixing bug 33284 =mgs
		mySelectDestFoldersModel.SetAccessLevel(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT);
		
		// Set title and sub title for the tree
		mySelectDestFoldersModel.SetTreeTitle(m_asOwner.GetString(1795, "ptmsgs_portaladminmsgs"));
		mySelectDestFoldersModel.SetTreeSubTitle(m_asOwner.GetString(1794, "ptmsgs_portaladminmsgs"));

		// JF- Set JS validation to prevent the list from being empty.
		String sJSFunctionName = ((SelectPTObjectModel) mySelectDestFoldersModel).
			SetEnableEmptyListClientValidation(m_asOwner.GetString(229,
					"ptmsgs_portaladminmsgs"));

		// JF- Then add this JavaScript function name to the Display Page validation functions
		AMainDP myMainDP = (AMainDP) m_asOwner.GetPage(GetMainDPName());
		myMainDP.AddToValidationFunction(sJSFunctionName);

		// JF- Populate the list in Edit mode 	
		if (EditorStartControl.EDITOR_START_FLAG_EDIT == GetEditorType())
		{
			Object[] arrDestFolders = (Object[]) ((IPTCrawler) m_ptObject).GetTaxonomist()
				.GetDestinationFolders();
			if (null != arrDestFolders)
			{
				IPTFolder ptFolder;
				int k				    = 0;
				XPHashtable htFolders = new XPHashtable();
	
				for (int i = 0; i < arrDestFolders.length; i++)
				{
					try
					{
						ptFolder = m_Session.GetCatalog().
							OpenFolder(XPConvert.ToInteger(arrDestFolders[i]), false);
						htFolders.PutElement(arrDestFolders[i], ptFolder);
	
					}
					catch (Exception e)
					{
						
							log.Error("StartSelectFolder Unable to open admin foler with ID: " +
								arrDestFolders[i] + ". The exception was: " + e);
						
					}
				}
				
				Object[][] oRows = new Object[htFolders.GetSize()][SelectPTObjectModel.EXPLIST_COL_MAX];
				IXPEnumerator enumKeys = htFolders.GetKeys();
				
				while (enumKeys.MoveNext())
				{
					ptFolder = (IPTFolder) htFolders.GetElement(enumKeys.GetCurrent());
					
					oRows[k][SelectPTObjectModel.EXPLIST_COL_CLASSID] = new Integer(PT_CLASSIDS.PT_CATALOGFOLDER_ID);
					oRows[k][SelectPTObjectModel.EXPLIST_COL_OBJECTID] = enumKeys.GetCurrent();
					oRows[k][SelectPTObjectModel.EXPLIST_COL_STRING] = ptFolder.GetPath();
					k++;
				}
	
				mySelectDestFoldersModel.SetDefaultObjects(oRows);
				
				
				/* If this is a mirror crawl, then we want to use single-select */
				int nCrawlerSettings = GetCrawlerSettings();
				if (0 != (PT_CRAWLER_SETTINGS.PT_CRAWLER_MIRROR & nCrawlerSettings))
				{
					mySelectDestFoldersModel.SetTreeMultiSelect(false);
				}		
				else
				{
					mySelectDestFoldersModel.SetTreeMultiSelect(true);
				}

			}
		}
	}

	/**
	 * Helper method to get back the data from the content language page and set them on the crawler.
	 *
	 */
	public void StoreContentLanguage()
	{
		String sLanguage = ((CardLanguagePlugInModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_LANGUAGE)).GetCardLanguageCode();

		((IPTCrawler) m_ptObject).SetContentLanguage(sLanguage);
	}

	/**
	 * Helper method to get back the data from the link settings page and set them on the crawler.
	 *
	 */
	public void StoreLinkSettings()
	{
		LinkSettingsPlugInModel myModel = (LinkSettingsPlugInModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_LINK_SETTINGS);
		IPTCrawler ptCrawler		    = (IPTCrawler) m_ptObject;

		ptCrawler.SetCardExpirationDelay(myModel.GetExpireAfterUnits(),
			myModel.GetExpireAfterValue());
		ptCrawler.SetCardMissingDocumentDeletionDelay(myModel.GetBrokenLinksDeleteUnits(),
			myModel.GetBrokenLinksDeleteValue());
		ptCrawler.SetCardRefreshRate(myModel.GetRefreshUnits(),
			myModel.GetRefreshValue());
		ptCrawler.SetCheckLinkOnly(myModel.GetCheckLinkOnly());
	}

	/**
	 * Returns the name of the MainDP display page.
	 *
	 * @return String
	 */
	abstract protected String GetMainDPName();

	/**
	 * Retrieves the DS ID and initializes the Crawler.
	 *
	 * @param _htFormData
	 */
	protected void InitPTObject(XPHashtable _htFormData)
	{
		super.InitPTObject(_htFormData);

		if (EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType())
		{
			int nDataSourceID = GetDataSourceIDFromQS(_htFormData);

			if (-1 == nDataSourceID)
			{
				throw new XPException(m_asOwner.GetString(230,
						"ptmsgs_portaladminmsgs"));
			}

			((IPTCrawler) m_ptObject).Initialize(nDataSourceID);
			m_pbStartLocation = PortalObjectsFactory.CreatePropertyBag();
			InitProviderPropertyBag(m_pbStartLocation);
			crawlerRuntimeConfig = PortalObjectsFactory.CreatePropertyBag();
		}
		else
		{
			m_pbStartLocation = ((IPTCrawler) m_ptObject).GetStartLocation();
			crawlerRuntimeConfig = ((IPTCrawler) m_ptObject).GetCrawlerRuntimeConfiguration(); 
		}
	}

	/**
	 * Initializes the Crawler property bag.
	 * (Called in create mode ONLY)
	 *
	 * @param _pbStartLocation
	 */
	abstract protected void InitProviderPropertyBag(
		IXPPropertyBag _pbStartLocation);

	/**
	 * Helper method to store the advanced settings.
	 *
	 */
	protected void StoreAdvSettings()
	{
		if (!m_bImportOnlyNewLinks)
		{
			int nCrawlerSettings = GetCrawlerSettings();

			nCrawlerSettings     = nCrawlerSettings &
				~PT_CRAWLER_SETTINGS.PT_CRAWLER_IMPORT_ONLY_DATA_SOURCE_NEW_CARDS;
			nCrawlerSettings     = nCrawlerSettings &
				~PT_CRAWLER_SETTINGS.PT_CRAWLER_IMPORT_ONLY_CRAWLER_NEW_CARDS;
			nCrawlerSettings     = nCrawlerSettings &
				~PT_CRAWLER_SETTINGS.PT_CRAWLER_REFRESH_RECRAWLED_CARDS;
			nCrawlerSettings     = nCrawlerSettings &
				~PT_CRAWLER_SETTINGS.PT_CRAWLER_RETAXONOMIZE_RECRAWLED_CARDS;
			nCrawlerSettings     = nCrawlerSettings &
				~PT_CRAWLER_SETTINGS.PT_CRAWLER_RECRAWL_DELETED_CARDS;

			((IPTCrawler) m_ptObject).SetSettings(nCrawlerSettings);
		}
	}

	/**
	 * Helper method to store the destination folders.
	 *
	 */
	protected void StoreDestFolders()
	{
		IPTTaxonomist ptTaxonomist = ((IPTCrawler) m_ptObject).GetTaxonomist();
		Object[] arrDestFolders    = (Object[]) ptTaxonomist.GetDestinationFolders();

		if (null != arrDestFolders)
		{
			// JF- First we remove all the old destination folders
			for (int i = 0; i < arrDestFolders.length; i++)
			{
				try
				{
					ptTaxonomist.RemoveDestinationFolder(XPConvert.ToInteger(arrDestFolders[i]));
				}
				catch (Exception e)
				{
					
						log.Error("CrawlerModel::StoreDestFolders Unable to delete the desctination folder with ID: " +
							arrDestFolders[i] +
							"The following exception happened: " + e);
					
				}
			}
		}

		// JF- Then we had the new ones
		PTGrowableSortedArrayWrapper aDestFolersValues = ((SelectPTObjectModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_SELECT_FOLDER)).GetItems();
		int nObjectID;

		for (int i = 0; i < aDestFolersValues.GetCount(); i++)
		{
			nObjectID = XPConvert.ToInteger(aDestFolersValues.GetItem(i,
					SelectPTObjectModel.EXPLIST_COL_OBJECTID));
			ptTaxonomist.AddDestinationFolder(nObjectID);
		}
	}

	/**
	  * Helper method to retrieve the the Data Source ID from the QS.
	  *
	  * @param _htFormData
	  */
	private int GetDataSourceIDFromQS(XPHashtable _htFormData)
	{
		int nDataSourceID = -1;

		if (!_htFormData.ContainsKey(QS_CRAWLEREDITOR_DATASOURCE_ID))
		{
			
				log.Error("CrawlerModel::GetDataSourceIDFromQS The data source ID was " +
					"not found in the Query String. The Editor cannot be started.");
			
		}
		else
		{
			String[] data = (String[]) _htFormData.GetElement(QS_CRAWLEREDITOR_DATASOURCE_ID);

			try
			{
				nDataSourceID = XPConvert.ToInteger(data[0]);
			}
			catch (Exception e)
			{
				
					log.Error("CrawlerModel::GetDataSourceIDFromQS The data source ID was invalid: " +
						data +
						". The Editor cannot be started. The following exception happened:" +
						e);
				
			}
		}

		return nDataSourceID;
	}

	/**
	 * Helper method to init the member variable m_bDSSupportMirroring.
	 *
	 */
	private void InitMirroringAndACLImportFlags()
	{
		int nDataSourceID		   = ((IPTCrawler) m_ptObject).GetDataSourceID();
		IPTDataSource ptDataSource = (IPTDataSource) m_Session.GetDataSources()
			.Open(nDataSourceID, false);

		int nWebServiceID = ptDataSource.GetWebServiceID();
		
		if (0 != nWebServiceID)
		{
			IPTWebService ptWebService = (IPTWebService) m_Session.GetWebServices().Open(nWebServiceID, false);	 
			IXPPropertyBag ptProvInfo = ptWebService.GetProviderInfo();
			if (1 == ptProvInfo.ReadAsInt(AExtensibilityConstants.PTC_HTTPCWS_SUPPORTSMIRRORING))
			{
				m_bDSSupportMirroring = true;
			}
			
			if (1 == ptProvInfo.ReadAsInt(AExtensibilityConstants.PTC_HTTPCWS_SUPPORTSACLIMPORT))
			{
				m_bSupportsACLImport = true;	
			}
		}	
	}
	
	/**
	 * Returns the Query Result of the Unmapped External Domains filtered by the Data Source of this Crawler.
	 *
	 * @return IPTQueryResult
	 */
	public IPTQueryResult GetUnmappedExternalDomains()
	{
		try
		{
			IPTACLSyncMap ptAclSynMap = (IPTACLSyncMap) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_ACLSYNCMAP, false);
			int nDataSourceID = ((IPTCrawler) m_ptObject).GetDataSourceID();	
			
			Object[][] arrFilter = new Object[3][1];
			arrFilter[0][0] = new Integer(PT_PROPIDS.PT_PROPID_ACLSYNCUNMAPPED_DATASOURCEID);
			arrFilter[1][0] = new Integer(PT_FILTEROPS.PT_FILTEROP_EQ);
			arrFilter[2][0] = new Integer(nDataSourceID);
			
			return ptAclSynMap.QueryUnmappedExternalDomains(arrFilter);
		}
		catch (Exception e)
		{
			
				log.Error("CrawlerModel::GetUnmappedExternalDomains The following exception happened " +
					"when trying to set the open and query the ACL Synch map: " + e);
			
		}
	
		return null;
	}
	
	public String GetCrawlerTag() {
		return ((IPTCrawler) m_ptObject).GetCrawlerTag();
	}
	
	/**
	 * Tells whether previously imported cards should be updated for new refresh settings.
	 *
	 * @return boolean
	 */
	public boolean GetApplyExisting()
	{
		return m_bRefreshExistingCards;
	}

    /** 
     * Get the runtime configuration property bag
     * @see com.plumtree.portalpages.admin.editors.crawler.ICrawlerModelRO#GetRuntimeConfiguration()
     */
    public IXPPropertyBag GetRuntimeConfiguration() {
        
        return crawlerRuntimeConfig;
    }
	/**
	 * Helper method to save the Advance Settings page.
	 *
	 * @param _htFormData
	 *
	 * @return int Page status for server side validation
	 */
	private int SaveAdvSettingsPage(XPHashtable _htFormData)
	{
		int nPageStatus = ((CardLanguagePlugInModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_LANGUAGE)).SaveSection(_htFormData);

		if (_htFormData.ContainsKey(RejectedDocsView.HTMLRADIO_REJECTED_DOCS))
		{
			String[] data = (String[]) _htFormData.GetElement(RejectedDocsView.HTMLRADIO_REJECTED_DOCS);

			try
			{
				int nCheckBoxValue = XPConvert.ToInteger(data[0]);
				int nTaxSettings   = GetTaxSettings();

				if (0 == nCheckBoxValue)
				{
					nTaxSettings     = nTaxSettings |
						PT_TAXONOMIST_SETTINGS.PT_TAXONOMIST_PUT_REJECTED_CARDS_IN_UNCLASSIFIED;
					nTaxSettings     = nTaxSettings &
						~PT_TAXONOMIST_SETTINGS.PT_TAXONOMIST_DELETE_REJECTED_CARDS;
				}
				else if (1 == nCheckBoxValue)
				{
					nTaxSettings     = nTaxSettings &
						~PT_TAXONOMIST_SETTINGS.PT_TAXONOMIST_PUT_REJECTED_CARDS_IN_UNCLASSIFIED;
					nTaxSettings     = nTaxSettings |
						PT_TAXONOMIST_SETTINGS.PT_TAXONOMIST_DELETE_REJECTED_CARDS;
				}

				((IPTCrawler) m_ptObject).GetTaxonomist().SetSettings(nTaxSettings);
			}
			catch (Exception e)
			{
				
					log.Error("CrawlerModel::SaveAdvSettingsPage The following exception happened " +
						"when trying to set the new Crawler settings: " + e);
				
			}
		}

		if (EditorStartControl.EDITOR_START_FLAG_EDIT == GetEditorType())
		{
			int nCrawlerSettings = GetCrawlerSettings();

			if (m_bImportOnlyNewLinks)
			{
				if (_htFormData.ContainsKey(
							ImportingDocsView.HTMLRADIO_ALREADY_IMPORTED))
				{
					String[] data = (String[]) _htFormData.GetElement(ImportingDocsView.HTMLRADIO_ALREADY_IMPORTED);
	
					try
					{
						int nAlreadyImportedDocs = XPConvert.ToInteger(data[0]);
	
						if (PT_CRAWLER_SETTINGS.PT_CRAWLER_IMPORT_ONLY_DATA_SOURCE_NEW_CARDS == nAlreadyImportedDocs)
						{
							nCrawlerSettings     = nCrawlerSettings |
								PT_CRAWLER_SETTINGS.PT_CRAWLER_IMPORT_ONLY_DATA_SOURCE_NEW_CARDS;
							nCrawlerSettings     = nCrawlerSettings &
								~PT_CRAWLER_SETTINGS.PT_CRAWLER_IMPORT_ONLY_CRAWLER_NEW_CARDS;
						}
						else if (PT_CRAWLER_SETTINGS.PT_CRAWLER_IMPORT_ONLY_CRAWLER_NEW_CARDS == nAlreadyImportedDocs)
						{
							nCrawlerSettings     = nCrawlerSettings |
								PT_CRAWLER_SETTINGS.PT_CRAWLER_IMPORT_ONLY_CRAWLER_NEW_CARDS;
							nCrawlerSettings     = nCrawlerSettings &
								~PT_CRAWLER_SETTINGS.PT_CRAWLER_IMPORT_ONLY_DATA_SOURCE_NEW_CARDS;
						}
					}
					catch (Exception e)
					{
						
							log.Error("CrawlerModel::SaveAdvSettingsPage The following exception happened " +
								"when trying to get the value of " +
								ImportingDocsView.HTMLRADIO_ALREADY_IMPORTED +
								": " + e);
						
					}
				}
	
				if (0 != (nCrawlerSettings &
						PT_CRAWLER_SETTINGS.PT_CRAWLER_ACL_IMPORT))
				{
					if (!_htFormData.ContainsKey(
								ImportingDocsView.HTMLCHECKBOX_REFRESH_ACLS))
					{
						nCrawlerSettings = nCrawlerSettings &
							~PT_CRAWLER_SETTINGS.PT_CRAWLER_FORCE_ACL_REFRESH;
					}
					else
					{
						nCrawlerSettings = nCrawlerSettings |
							PT_CRAWLER_SETTINGS.PT_CRAWLER_FORCE_ACL_REFRESH;
					}
				}
	
				if (!_htFormData.ContainsKey(
							ImportingDocsView.HTMLCHECKBOX_REFRESH_THEM))
				{
					nCrawlerSettings = nCrawlerSettings &
						~PT_CRAWLER_SETTINGS.PT_CRAWLER_REFRESH_RECRAWLED_CARDS;
				}
				else
				{
					nCrawlerSettings = nCrawlerSettings |
						PT_CRAWLER_SETTINGS.PT_CRAWLER_REFRESH_RECRAWLED_CARDS;
				}
	
				if (0 == (nCrawlerSettings & PT_CRAWLER_SETTINGS.PT_CRAWLER_MIRROR))
				{
					if (!_htFormData.ContainsKey(
								ImportingDocsView.HTMLCHECKBOX_SORT_THEM))
					{
						nCrawlerSettings = nCrawlerSettings &
							~PT_CRAWLER_SETTINGS.PT_CRAWLER_RETAXONOMIZE_RECRAWLED_CARDS;
					}
					else
					{
						nCrawlerSettings = nCrawlerSettings |
							PT_CRAWLER_SETTINGS.PT_CRAWLER_RETAXONOMIZE_RECRAWLED_CARDS;
					}
				}

				if (!_htFormData.ContainsKey(
							ImportingDocsView.HTMLCHECKBOX_REGENERATE_LINKS))
				{
					nCrawlerSettings = nCrawlerSettings &
						~PT_CRAWLER_SETTINGS.PT_CRAWLER_RECRAWL_DELETED_CARDS;
				}
				else
				{
					nCrawlerSettings = nCrawlerSettings |
						PT_CRAWLER_SETTINGS.PT_CRAWLER_RECRAWL_DELETED_CARDS;
				}
			}
	
			if (0 == (nCrawlerSettings & PT_CRAWLER_SETTINGS.PT_CRAWLER_MIRROR))
			{
				if (_htFormData.ContainsKey(RejectedDocsView.HTMLRADIO_REIMPORT_DOCS))
				{
					String[] data = (String[]) _htFormData.
						GetElement(RejectedDocsView.HTMLRADIO_REIMPORT_DOCS);
	
					try
					{
						int nCheckBoxValue = XPConvert.ToInteger(data[0]);
	
						if (0 == nCheckBoxValue)
						{
							nCrawlerSettings = nCrawlerSettings &
								~PT_CRAWLER_SETTINGS.PT_CRAWLER_IGNORE_PREVIOUSLY_REJECTED_CARDS;
						}
						else
						{
							nCrawlerSettings = nCrawlerSettings |
								PT_CRAWLER_SETTINGS.PT_CRAWLER_IGNORE_PREVIOUSLY_REJECTED_CARDS;
						}
					}
					catch (Exception e)
					{
						
							log.Error("CrawlerModel::SaveAdvSettingsPage The following exception happened " +
								"when trying to get the value of " +
								RejectedDocsView.HTMLRADIO_REIMPORT_DOCS +
								": " + e);
						
					}
				}
				else
				{
					nCrawlerSettings = nCrawlerSettings &
						PT_CRAWLER_SETTINGS.PT_CRAWLER_IGNORE_PREVIOUSLY_REJECTED_CARDS;
				}
	
				if (!_htFormData.ContainsKey(
							ImportingDocsView.HTMLCHECKBOX_IMPORT_ONLY_NEW_DOCS))
				{
					m_bImportOnlyNewLinks = false;
				}
				else
				{
					m_bImportOnlyNewLinks = true;
				}
			}

			((IPTCrawler) m_ptObject).SetSettings(nCrawlerSettings);
		}
		
		// olegs - added
		
		if (_htFormData.ContainsKey(CrawlerTagView.HTMLTEXT_CRAWLERTAG))
		{
			String[] data = (String[]) _htFormData.GetElement(CrawlerTagView.HTMLTEXT_CRAWLERTAG);

			try
			{
				String tag=data[0];
				if(tag==null) tag="";
				((IPTCrawler) m_ptObject).SetCrawlerTag(tag);
			}
			catch (Exception e)
			{
				
					log.Error("CrawlerModel::SaveAdvSettingsPage The following exception happened " +
						"when trying to set the new Crawler settings: " + e);
				
				// and that's all??
			}
		}
		
		// end
		
		/* Now, get the thread configuration. Set the numbers of theads that fetch documents and threads that
		 * process(index, set properties, set Acl's , etc) the nodes.
		 */
		
		IPTCrawlerConfig crawlConfig= ((IPTCrawlerManager)m_Session.GetCrawlers()).GetCrawlerConfig();

		/* Set the maximum number of pulling(fetching) threads */
        if (_htFormData.ContainsKey(CrawlerThreadConfigView.HTMLTEXT_NUM_PULL_THREADS)) {
            try {
                String[] data = (String[]) _htFormData.GetElement(CrawlerThreadConfigView.HTMLTEXT_NUM_PULL_THREADS);
                IXPPropertyBag crawlRuntimeConfig = ((IPTCrawler) m_ptObject).GetCrawlerRuntimeConfiguration();
                String strNumPullThreads = data[0];
                if (strNumPullThreads != null && strNumPullThreads.length() > 0) {
                    int numPullThreads = XPConvert.ToInteger(strNumPullThreads);
                    if (crawlRuntimeConfig == null) {
                        crawlRuntimeConfig = PortalObjectsFactory.CreatePropertyBag();
                    }
                    //make sure the value is in config-allowable range
                    int minimumAllowed = crawlConfig.getMinPullingThreads();
                    int maximumAllowed = crawlConfig.getMaxPullingThreads();
                    //make sure the value is in config-allowable range
                    if (numPullThreads < minimumAllowed) {
                        RaiseError(m_asOwner.GetString(2914, "ptmsgs_portaladminmsgs") + "cannot be set below " + minimumAllowed);
                        numPullThreads = minimumAllowed;
                        nPageStatus = RepostControl.PAGE_STATUS_INVALID;
                   } else if (numPullThreads > maximumAllowed) {
                       	RaiseError(m_asOwner.GetString(2914, "ptmsgs_portaladminmsgs") + "cannot be set above " + maximumAllowed);
                       	numPullThreads = maximumAllowed;
                        nPageStatus = RepostControl.PAGE_STATUS_INVALID;
                   }
                    crawlRuntimeConfig.Write(PT_CRAWL_INFO.MAX_NUMBER_PULLING_THREADS, numPullThreads);
                } else {
                    crawlRuntimeConfig.Write(PT_CRAWL_INFO.MAX_NUMBER_PULLING_THREADS, crawlConfig.getDefaultPullingThreads());
                }
                ((IPTCrawler) m_ptObject).SetCrawlerRuntimeConfiguration(crawlRuntimeConfig);
            } catch (XPFormatException xfe) {
                RaiseError("Invalid parameter for " + m_asOwner.GetString(2914, "ptmsgs_portaladminmsgs"));
                nPageStatus = RepostControl.PAGE_STATUS_INVALID;
            } catch (Exception e) {
                log.Error(e, "CrawlerModel::SaveAdvSettingsPage The following exception happened "
                        + "when trying to set the new value for the number of pulling threads: ");
                nPageStatus = RepostControl.PAGE_STATUS_INVALID;
           }
        }
        /* now set the maximum namber of processing threads */
        if (_htFormData.ContainsKey(CrawlerThreadConfigView.HTMLTEXT_NUM_PROCESS_THREADS)) {
            try {
                String[] data = (String[]) _htFormData.GetElement(CrawlerThreadConfigView.HTMLTEXT_NUM_PROCESS_THREADS);
                IXPPropertyBag crawlRuntimeConfig = ((IPTCrawler) m_ptObject).GetCrawlerRuntimeConfiguration();
                String strNumProcessinglThreads = data[0];
                if (strNumProcessinglThreads != null && strNumProcessinglThreads.length() > 0) {
                    int numProccessThreads = XPConvert.ToInteger(strNumProcessinglThreads);
                    if (crawlRuntimeConfig == null) {
                        crawlRuntimeConfig = PortalObjectsFactory.CreatePropertyBag();
                    }
                    int minimumAllowed = crawlConfig.getMinProcessingThreads();
                    int maximumAllowed = crawlConfig.getMaxProcessingThreads();
                    //make sure the value is in config-allowable range
                    if (numProccessThreads < minimumAllowed) {
                        RaiseError(m_asOwner.GetString(2915, "ptmsgs_portaladminmsgs") + "cannot be set below " + minimumAllowed);
                        numProccessThreads = minimumAllowed;
                        nPageStatus = RepostControl.PAGE_STATUS_INVALID;
                    } else if (numProccessThreads > maximumAllowed) {
                        RaiseError(m_asOwner.GetString(2915, "ptmsgs_portaladminmsgs") + "cannot be set above " + maximumAllowed);
                        numProccessThreads = maximumAllowed;
                        nPageStatus = RepostControl.PAGE_STATUS_INVALID;
                   }
                    crawlRuntimeConfig.Write(PT_CRAWL_INFO.MAX_NUMBER_PROCESSING_THREADS, numProccessThreads);
                } else {
                    crawlRuntimeConfig.Write(PT_CRAWL_INFO.MAX_NUMBER_PROCESSING_THREADS, crawlConfig.getDefaultProcessingThreads());
                }
                ((IPTCrawler) m_ptObject).SetCrawlerRuntimeConfiguration(crawlRuntimeConfig);
            } catch (XPFormatException xfe) {
                RaiseError("Invalid parameter for " + m_asOwner.GetString(2915, "ptmsgs_portaladminmsgs"));
                nPageStatus = RepostControl.PAGE_STATUS_INVALID;
           } catch (Exception e) {
                log.Error(e, "CrawlerModel::SaveAdvSettingsPage The following exception happened "
                        + "when trying to set the new value for the number of processing threads: ");
                nPageStatus = RepostControl.PAGE_STATUS_INVALID;
           }
        }

		return nPageStatus;
	}

	/**
     * Helper method to init the Document Type Map page.
     *  
     */
	private void StartGbDocTypeList()
	{
		GbDocTypeListModel myGbDocTypeModel;
		IPTDocumentTypeMap ptDocTypeMap = ((IPTCrawler) m_ptObject).GetDocumentTypeMap();
		IASQueryResult qrSections	    = new ASQueryResultWrapper(ptDocTypeMap.GetSections());
		m_nDocMapSectionCount		    = qrSections.GetCount();

		((CrawlerEditorAS) m_asOwner).RegisterClassesForDocumentType(m_nDocMapSectionCount);

		for (int i = 0; i < m_nDocMapSectionCount; i++)
		{
			myGbDocTypeModel = (GbDocTypeListModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_TYPE_MAP +
					i);
			String strSectionName	    = (String) qrSections.GetFields(i, PT_PROPIDS.PT_PROPID_DTMSECTION_NAME);
			myGbDocTypeModel.InitializeGbDocTypeList(ptDocTypeMap, i, strSectionName);
			myGbDocTypeModel.SetFolderID(GetFolderID());
		}
	}

	/**
	 * Helper method to set "apply to existing cards" value.
	 *
	 */
	private void PutApplyExisting(XPHashtable _htFormData)
	{
		if (_htFormData.ContainsKey(ExistingCardsView.HTMLCHECKBOX_APPLYTOEXISTING))
			m_bRefreshExistingCards = true;
		else
			m_bRefreshExistingCards = false;
	}
	
	/**
	 * Helper method to actually apply the refresh settings to existing cards.
	 *
	 */
	private void ApplyRefreshSettingsToExistingCards()
	{
		try
		{
			IPTCardRefreshInfo ptCardRefreshInfo = (IPTCardRefreshInfo) (PortalObjectsFactory.CreateCardRefreshInfo());
			LinkSettingsPlugInModel myModel = (LinkSettingsPlugInModel) m_asOwner.GetModel(CrawlerEditorAS.STR_MVC_STORAGE_NAME_LINK_SETTINGS);
	
			ptCardRefreshInfo.SetCheckLinkOnly(myModel.GetCheckLinkOnly());
			ptCardRefreshInfo.SetCardRefreshRate(	myModel.GetRefreshUnits(),
													myModel.GetRefreshValue());
													
			ptCardRefreshInfo.SetCardMissingDocumentDeletionDelay(	myModel.GetBrokenLinksDeleteUnits(),
																	myModel.GetBrokenLinksDeleteValue());
			ptCardRefreshInfo.SetCardExpirationDate(CalculateDateFromInterval( myModel.GetExpireAfterUnits(),
																				myModel.GetExpireAfterValue()));

			ptCardRefreshInfo.SetCardNextRefreshDate(CalculateDateFromInterval( myModel.GetRefreshUnits(),
																				myModel.GetRefreshValue()));

			IPTCardRefreshAgent ptCardRefreshAgent = (IPTCardRefreshAgent)m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_CARD_REFRESH_AGENT, false);
			IPTCardRefreshAdmin ptCardRefreshAdmin = (IPTCardRefreshAdmin)ptCardRefreshAgent.GetInterfaces("IPTCardRefreshAdmin");
	
			ptCardRefreshAdmin.SetRefreshInfo(PT_CLASSIDS.PT_CRAWLER_ID, m_nObjectID, ptCardRefreshInfo);
		}
		catch (Exception e)
		{
			
				log.Error("CrawlerModel::SaveAdvSettingsPage The following exception happened " +
					"when trying to set the new Crawler settings: " + e);
			
		}
	}

	/**
	 * Helper method to calculate the next action date from the present time.
	 *
	 */	
	private XPDateTime CalculateDateFromInterval(int nUnits, int nInterval)
	{
		XPDateTime dtLastDate = new XPDateTime();
		XPCalendar xpcDateNext = XPCalendar.GetInstance();
		
		int units = 0;
	
		switch (nUnits)
		{
		case PT_SCHEDULETYPES.PT_SCHEDULETYPE_ANNUALY:
			units = XPCalendar.YEAR;
			break;
	
		case PT_SCHEDULETYPES.PT_SCHEDULETYPE_MONTHLY:
			units = XPCalendar.MONTH;
			break;
			
		case PT_SCHEDULETYPES.PT_SCHEDULETYPE_WEEKLY:
			units = XPCalendar.WEEK_OF_YEAR;
			break;
			
		case PT_SCHEDULETYPES.PT_SCHEDULETYPE_DAILY:
			units = XPCalendar.DATE;
			break;
			
		case PT_SCHEDULETYPES.PT_SCHEDULETYPE_HOURLY:
			units = XPCalendar.HOUR;
			break;
			
		case PT_SCHEDULETYPES.PT_SCHEDULETYPE_MINUTELY:
			units = XPCalendar.MINUTE;
			break;
		}
		
		if (0 == units)
			return null;

		xpcDateNext.SetTime(dtLastDate);
		xpcDateNext.Add(units, nInterval);
		
		XPDateTime dtNextDate = xpcDateNext.GetTime();
		
		return dtNextDate;
	
	}

	/**
	 * Helper method to raise an error
	 *
	 * @param strError - Error message
	 * @param strError - Extended error message
	 */
	public void RaiseError(String strError, String strExtended) /*"borrowed" from DirModel */
	{
	   
		UIError myError = new UIError(UIErrorLevel.FRIENDLY,
				UIErrorScope.CURRENT_USER, strError, strExtended);
				
		myError.SetActivitySpace(m_asOwner.GetName());
		myError.SetDisplayOnSeparatePage(false);
		myError.SetClearWithActivitySpaceChange(true);
		myError.SetImmediateClear(false);
		
		UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
			m_asOwner.GetApplication());
	}

	/**
	 * Helper method to raise an error
	 *
	 * @param strError - Error message
	 */
	public void RaiseError(String strError)/*"borrowed" from DirModel */
	{
		RaiseError(strError, strError);
	}

}



