package com.plumtree.portalpages.admin.editors.job;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.common.joblogs.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity.*;
import com.plumtree.portaluiinfrastructure.expandablelist.selectptobject.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.marshalers.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.pagecontrols.datepicker.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker.*;
import com.plumtree.uiinfrastructure.pagecontrols.timepicker.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.jsutil.*;

/**
 * This class implements the model for the job editor
 * 
 * @author JohnH
 */
public class JobModel extends ObjEditorModel implements IJobModelRO {
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.editors.job.JobModel");

	
	/** Unique identifier for the class */
	public static final String STR_MVC_CLASS_NAME = "Job";

	//Constant for recurring job type

	/** This constant is returned by the UI when the job type is recurring.
	 * It signals the model to request further information (the schedule) to
	 * properly populate m_intScheduleType with the correct value.
	 */
	public static final int SCHEDULE_TYPE_RECURRING = -1;

	/** This constant is returned by the UI when the job type is set to RunOnceNow 
	 */
	public static final int SCHEDULE_TYPE_RUN_ONCE_NOW = -2;


	//Some constants for editing operations

	/** Constant used internally */
	public static final int c_intDict = 0;

	/** Constant used internally */
	public static final int c_intDictObjClass = 1;

	/** Constant used internally */
	public static final int c_intDictObjName = 2;
	
	// Member variables
	/** An array containing the operations for this job */
	private PTGrowableSortedArrayWrapper m_arrOperationValues;

	/** Member variable containing the editing user's locale */
	private XPLocale m_Locale; //the user locale

	/** Member variable containing the editing user's time zone */
	private XPTimeZone m_Timezone; //the user time zone
	
	/** Member variable dictating whether the add operation feature should be disabled */
	private boolean m_bDisableAddOperation = false;
	
	/** Member variable indicating whether to ignore errors and continue (1) or to stop on errors (0) */
	private int m_intContinue = 0; //Ignore errors and continue
	
	/** Member variable contining the log level - default is 2 (PT_LOG_NORMAL) */
	private int m_intLogLevel = 2; //PT_LOG_NORMAL

	/** Member variable indicating the recurrence interval for the job. For example
	 * this job will run every N minutes, where N is the recurrence interval. */
	private int m_intRecurrenceInterval = 0;

	/** Member variable for restrictions on the running of the job */
	private int m_intRestrictions = 0; //None
	
	/** The schedule type: 0=NEVER, 1=ONCE, >1=PERIODICALLY.  Valid values are:
	 * PT_SCHEDULETYPE_NEVER = 0 PT_SCHEDULETYPE_ONCE = 1
	 * PT_SCHEDULETYPE_MINUTELY = 2 PT_SCHEDULETYPE_HOURLY = 3
	 * PT_SCHEDULETYPE_DAILY = 4 PT_SCHEDULETYPE_WEEKLY = 5
	 * PT_SCHEDULETYPE_MONTHLY = 6 PT_SCHEDULETYPE_ANNUALLY = 7 */
	private int m_intScheduleType = 0; //0=NEVER, 1=ONCE, >1=PERIODICALLY, -2=RUN_ONCE_NOW

	/** Member variable for the suspention checkpoint interval. Default: 0=Never */
	private int m_intSuspendCheckpointInterval = 0; //Never
	
	/** Member variable for the suspension mode - one of PT_SCHEDULE_SUSPEND_MODES:
	 * PT_SCHEDULE_WORK_CONTINUOUSLY (Default), PT_SCHEDULE_SUSPEND_ON,
	 * PT_SCHEDULE_WORK_ON
	 * 
	 */
	private int m_intSuspendMode = PT_SCHEDULE_SUSPEND_MODES.PT_SCHEDULE_WORK_CONTINUOUSLY;
	
	/** Member variable for the timeout */
	private int m_intTimeout = 0; //0 means run until finished
	
	/** Member variable for the timeout units */
	private int m_intTimeoutUnits = 0;

	/** Date format string to be passed into the date picker control */
	private String m_strDateFormatPattern;	
	
	/** DateTime date picker manager */
	private JSDateTimePickerMgr m_ptJSDateTimePickerMgr;

	/** Page Manager object */
	private PageMgr m_ptPageMgr;
	
	/** used by JobDependenciesView */
	private XPDateTime m_xpdtRunOnceNow = null;

	// BEGIN ***************** Framework methods *****************

	public JobModel()
	{	
		super();
	}

	/**
	 * Code executed when the Editor is starting.
	 *
	 * @param _htFormData
	 */
	public void DoTaskOnStartEditor(XPHashtable _htFormData) 
	{
	    if (EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType())
		{
			// no security page when creating
			((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
				ObjectSecurityDP.STR_MVC_CLASS_NAME);
			((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
				MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);
		}

		try
		{
			//Get the locale
			String strLocale =
				(String) PTPersonalSettingsHelper.GetPersonalSettingValue(
					PersonalSettingNames.LOCALE,
					m_asOwner);

			m_Locale = new XPLocale(strLocale);
			//m_Locale = PTPersonalSettingsHelper.GetPersonalSettingValue_Locale(m_asOwner);
	
			if (null == m_Locale) 
			{
				m_Locale = XPLocale.GetDefault();
			}
	
			//Get the timezone
			m_Timezone = PTPersonalSettingsHelper.GetPersonalSettingValue_TimeZone(m_asOwner);
			if (null == m_Timezone)
			{
				m_Timezone = XPTimeZone.GetCurrentTimeZone();
			}
	
			//Get a format object so we can pass format strings into javascript functions
			XPDateFormat xpDateFormat =
				XPDateFormat.GetDateInstance(
				XPDateFormat.MEDIUM,
				m_Locale,
				m_Timezone);
	
			m_strDateFormatPattern = xpDateFormat.ToPattern();
	
			//Initialize the page and component manager framework
			InitializeDatePickerMgrs();
			
			// JF- If the Job Editor is a sub Editor, we don't display the "add an Operation" table.
			// Instead, we are displaying a warning to let the user know that the operation being created will be automatically
			// added to the Job.
			if (m_asOwner.IsSubSpace()) 
			{
				m_bDisableAddOperation = true;
			} 
			else 
			{
				/* check to see if the create new with existing op ids are set */
				int op_clsid = HTTPHelpers.GetArgumentAsInt(_htFormData, PTJobs.JOBOP_CLSID, -1);
				int op_objid = HTTPHelpers.GetArgumentAsInt(_htFormData, PTJobs.JOBOP_OBJID, -1);

				int[] arrNewOp = null;
				if (-1!=op_clsid && -1!=op_objid)
				{
					arrNewOp = new int[]{op_clsid, op_objid};
				}
				
				StartSelectPTObject(arrNewOp);
			}

			//Set the job ID to an invalid value (-1) initially so no logs will be displayed
			//if we are actually in edit mode, the correct job id will be set
			(
				(JobLogModel) m_asOwner.GetModel(
					JobLogModel.STR_MVC_CLASS_NAME)).SetJobID(
				-1);
		}
		catch (Exception e)
		{
			GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskOnStartEditor", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(2415, "ptmsgs_portaladminmsgs"));			
		}
		
		//This local variable is used to initialize the date picker objects 
		ADatePickerComponent ptJSDateTimePicker=null;
		
		if (EditorStartControl.EDITOR_START_FLAG_EDIT == GetEditorType()) 
		{
			//Check to see if this job is locked - most likely because it is running..
			try
			{
				IPTJob myTempJob = (IPTJob) m_Session.GetObjectManagers(PT_CLASSIDS.PT_JOB_ID).Open(this.GetObjectID(), true);
				((IPTServerContext) myTempJob.GetInterfaces("IPTServerContext")).UnlockObject();
				myTempJob = null;
			}
			catch (Exception e)
			{
				if (e instanceof PTException) // locked
				{
					if (((PTException) e).GetErrorCode() == -2147205113) //locked
						m_bIsEditorReadOnly = true;
				}
			}
			
			//Set the job ID on the Job log class
			(
				(JobLogModel) m_asOwner.GetModel(
					JobLogModel.STR_MVC_CLASS_NAME)).SetJobID(
				this.GetObjectID());

			//The view classes will draw from these member variables
			//to populate the HTML controls on each page.
			try {
				IPTSchedule ptSchedule = ((IPTJob) m_ptObject).GetSchedule();

				//Get the Schedule type
				try 
				{
					m_intScheduleType = ptSchedule.GetScheduleType();
				} 
				catch (Exception e) 
				{
					GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskOnStartEditor", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(473, "ptmsgs_portaladminmsgs"));
				}

				//Get the Recurrence Interval
				try {
					m_intRecurrenceInterval =
						ptSchedule.GetRecurrenceInterval();
				} 
				catch (Exception e) 
				{
					GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskOnStartEditor", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(474, "ptmsgs_portaladminmsgs"));
				}

				//Get the ignore errors settings
				try {
					m_intContinue =
						(((IPTJob) m_ptObject)
							.GetSettingsAsInt(PT_JOB_SETTINGS.PT_JOB_CONTINUEONERRORS));
				} 
				catch (Exception e) 
				{
					GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskOnStartEditor", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(475, "ptmsgs_portaladminmsgs"));
				}

				//Get the timeout setting
				try {
					m_intTimeout =
						( ((IPTJob) m_ptObject)
							.GetSettingsAsInt(PT_JOB_SETTINGS.PT_JOB_TIMEOUT));
				} 
				catch (Exception e) 
				{
					GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskOnStartEditor", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(476, "ptmsgs_portaladminmsgs"));
				}

				//The timeout units setting
				try {
					m_intTimeoutUnits =
						DurationFromScheduleType(
							((IPTJob) m_ptObject)
								.GetSettingsAsInt(PT_JOB_SETTINGS.PT_JOB_TIMEOUT_UNITS));
				} 
				catch (Exception e) 
				{
					GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskOnStartEditor", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(477, "ptmsgs_portaladminmsgs"));
				}

				//The next run/date after/restrictions information
				XPDateTime dtNextRun = new XPDateTime();
				if (PT_SCHEDULETYPES.PT_SCHEDULETYPE_NEVER
					!= m_intScheduleType) {
					try 
					{
						dtNextRun =
							((IPTJob) m_ptObject).GetSettingsAsXPDateTime(
							PT_JOB_SETTINGS.PT_JOB_NEXTRUNTIME);
								
					} catch (Exception e) {
						GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskOnStartEditor", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(478, "ptmsgs_portaladminmsgs"));
					}
				}
				else //Never
				{
					ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");	
					ptJSDateTimePicker.Init(AFormDP.HTMLFORM_NAME, m_ptPageMgr, JobEditorAS.RUN_ONCE_DATEPICKER);
					m_ptJSDateTimePickerMgr.RegisterComponent(ptJSDateTimePicker);
						
					ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");	
					ptJSDateTimePicker.Init(AFormDP.HTMLFORM_NAME, m_ptPageMgr, JobEditorAS.NEXT_RUN_DATEPICKER);
					m_ptJSDateTimePickerMgr.RegisterComponent(ptJSDateTimePicker);

					ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");	
					ptJSDateTimePicker.Init(AFormDP.HTMLFORM_NAME, m_ptPageMgr, JobEditorAS.NOT_AFTER_DATEPICKER);
					m_ptJSDateTimePickerMgr.RegisterComponent(ptJSDateTimePicker);
				}

				if (PT_SCHEDULETYPES.PT_SCHEDULETYPE_ONCE
					== m_intScheduleType) {
					try 
					{
						ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");	
						ptJSDateTimePicker.Init(AFormDP.HTMLFORM_NAME, dtNextRun, m_ptPageMgr, JobEditorAS.RUN_ONCE_DATEPICKER);
						m_ptJSDateTimePickerMgr.RegisterComponent(ptJSDateTimePicker);
						
						ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");	
						ptJSDateTimePicker.Init(AFormDP.HTMLFORM_NAME, m_ptPageMgr, JobEditorAS.NEXT_RUN_DATEPICKER);
						m_ptJSDateTimePickerMgr.RegisterComponent(ptJSDateTimePicker);

						ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");
						ptJSDateTimePicker.Init(AFormDP.HTMLFORM_NAME, m_ptPageMgr, JobEditorAS.NOT_AFTER_DATEPICKER);
						m_ptJSDateTimePickerMgr.RegisterComponent(ptJSDateTimePicker);

					} catch (Exception e) 
					{
						GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskOnStartEditor", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(479, "ptmsgs_portaladminmsgs"));
					}
				} 
				else if (1 < m_intScheduleType) 
				{
					try 
					{
						//initialize the run once date picker with a null inital start date
						ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");
						ptJSDateTimePicker.Init(AFormDP.HTMLFORM_NAME, m_ptPageMgr, JobEditorAS.RUN_ONCE_DATEPICKER);
						m_ptJSDateTimePickerMgr.RegisterComponent(ptJSDateTimePicker);

						//Now initialize the next run date picker
						ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");
						ptJSDateTimePicker.Init(AFormDP.HTMLFORM_NAME, dtNextRun, m_ptPageMgr, JobEditorAS.NEXT_RUN_DATEPICKER);
						m_ptJSDateTimePickerMgr.RegisterComponent(ptJSDateTimePicker);

						XPDateTime xpdtNotAfterDate = ptSchedule.GetUntilTimestamp(); 

						//Initialize the not after date picker
						ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");
						ptJSDateTimePicker.Init(AFormDP.HTMLFORM_NAME, xpdtNotAfterDate, m_ptPageMgr, JobEditorAS.NOT_AFTER_DATEPICKER);
						m_ptJSDateTimePickerMgr.RegisterComponent(ptJSDateTimePicker);
						
						m_intRestrictions =
							(ptSchedule
								.GetSettingsAsInt(
									PT_SCHEDULE_SETTINGS
										.PT_SCHEDULE_BLOCKEDTIMES));
					} catch (Exception e) {
						GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskOnStartEditor", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(480, "ptmsgs_portaladminmsgs"));
					}
				}

				//Log Level
				try 
				{
					m_intLogLevel =
						(((IPTJob) m_ptObject)
							.GetSettingsAsInt(PT_JOB_SETTINGS.PT_JOB_LOGLEVEL));
				} 
				catch (Exception e) 
				{
					GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskOnStartEditor", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(481, "ptmsgs_portaladminmsgs"));
				}

				//Suspend mode and related settings
				try {
					m_intSuspendMode =
						( ptSchedule
							.GetSettingsAsInt(
								PT_SCHEDULE_SETTINGS.PT_SCHEDULE_SUSPEND_MODE));
				} catch (Exception e) {
					GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskOnStartEditor", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(482, "ptmsgs_portaladminmsgs"));
				}

				//Start and end suspend times
				if (PT_SCHEDULE_SUSPEND_MODES.PT_SCHEDULE_WORK_CONTINUOUSLY != m_intSuspendMode) 
				{
					// SE: Bug 57319 - When restoring the suspension times that were saved, we should
					// use US locale and the system's timezone when calling TransformTimeToNewLocale.
					// This is because the times were saved in that format. Note we shouldn't use the
					// user's timezone when calling TransformTimeToNewLocale to transform the locale. 
					// This causes the time to be adjusted to the user's timezone each time and is bad
					// when the user's timezone is not the same as the system's timezone. The TimePicker 
					// widget will handle the conversion when it renders.
					XPLocale myUSLocale = new XPLocale("EN", "us");
					XPTimeZone systemTimeZone = XPTimeZone.GetCurrentTimeZone();
					try 
					{
						String strSuspendWeekdayBeginTime =
								ptSchedule.GetSettingsAsString(
								PT_SCHEDULE_SETTINGS.PT_SCHEDULE_SUSPEND_WEEKDAY_BEGIN_TIME);
						
						log.Debug("JobModel::DoTaskOnStartEditor - strSuspendWeekdayBeginTime before TransformTimeToNewLocale: [" + strSuspendWeekdayBeginTime + "]");
						// SE: bug 57319 - Don't use XPLocale.GetDefault here. We know that the time string
						// that was saved is in US locale format
						//strSuspendWeekdayBeginTime = TimePicker.TransformTimeToNewLocale(strSuspendWeekdayBeginTime, XPLocale.GetDefault(), m_Locale, m_Timezone);
						// SE: Bug 57319 - We need to call TransformTimeToNewLocale so that we can pass in the
						// proper time format for the user's locale to the TimePicker or else it will not
						// be able to parse it correctly.
						strSuspendWeekdayBeginTime = TimePicker.TransformTimeToNewLocale(strSuspendWeekdayBeginTime, myUSLocale, m_Locale, systemTimeZone /* m_Timezone */);
						log.Debug("JobModel::DoTaskOnStartEditor - strSuspendWeekdayBeginTime after TransformTimeToNewLocale: [" + strSuspendWeekdayBeginTime + "]");

						ATimePicker ptBeginTimePicker =  (ATimePicker) m_ptPageMgr.AddPageControl("TimePicker");
						ptBeginTimePicker.Init( m_ptPageMgr, strSuspendWeekdayBeginTime, JobEditorAS.SUSPEND_BEGIN_TIMEPICKER);
						m_ptJSDateTimePickerMgr.RegisterComponent(ptBeginTimePicker);

						String strSuspendWeekdayEndTime =
							ptSchedule.GetSettingsAsString(
								PT_SCHEDULE_SETTINGS.PT_SCHEDULE_SUSPEND_WEEKDAY_END_TIME);
									
						log.Debug("JobModel::DoTaskOnStartEditor - strSuspendWeekdayEndTime before TransformTimeToNewLocale: [" + strSuspendWeekdayEndTime + "]");
						// SE: bug 57319 - Don't use XPLocale.GetDefault here. We know that the time string
						// that was saved is in US locale format
						//strSuspendWeekdayEndTime = TimePicker.TransformTimeToNewLocale(strSuspendWeekdayEndTime, XPLocale.GetDefault(), m_Locale, m_Timezone);
						// SE: Bug 57319 - We need to call TransformTimeToNewLocale so that we can pass in the
						// proper time format for the user's locale to the TimePicker or else it will not
						// be able to parse it correctly.
						strSuspendWeekdayEndTime = TimePicker.TransformTimeToNewLocale(strSuspendWeekdayEndTime, myUSLocale, m_Locale, systemTimeZone /* m_Timezone */);
						log.Debug("JobModel::DoTaskOnStartEditor - strSuspendWeekdayEndTime after TransformTimeToNewLocale: [" + strSuspendWeekdayEndTime + "]");
						
						ATimePicker ptEndTimePicker = (ATimePicker) m_ptPageMgr.AddPageControl("TimePicker");
						ptEndTimePicker.Init(m_ptPageMgr, strSuspendWeekdayEndTime, JobEditorAS.SUSPEND_END_TIMEPICKER);
						m_ptJSDateTimePickerMgr.RegisterComponent(ptEndTimePicker);

					} catch (Exception e) {
						GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskOnStartEditor", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(483, "ptmsgs_portaladminmsgs"));
					}
				} 
				else
				{
				    ATimePicker ptTimePickerBegin = (ATimePicker) m_ptPageMgr.AddPageControl("TimePicker");
				    ptTimePickerBegin.Init(m_ptPageMgr, "", JobEditorAS.SUSPEND_BEGIN_TIMEPICKER);
					m_ptJSDateTimePickerMgr.RegisterComponent(ptTimePickerBegin);

				    ATimePicker ptTimePickerEnd = (ATimePicker) m_ptPageMgr.AddPageControl("TimePicker");
				    ptTimePickerEnd.Init(m_ptPageMgr, "", JobEditorAS.SUSPEND_END_TIMEPICKER);
					m_ptJSDateTimePickerMgr.RegisterComponent(ptTimePickerEnd);
				}

				//Checkpoint interval
				try {
					m_intSuspendCheckpointInterval =
						(ptSchedule
							.GetSettingsAsInt(
								PT_SCHEDULE_SETTINGS
									.PT_SCHEDULE_SUSPEND_CHECKPOINT_INTERVAL));
				} catch (Exception e) {
					GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskOnStartEditor", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(484, "ptmsgs_portaladminmsgs"));
				}
			} 
			catch (Exception e) 
			{
				GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskOnStartEditor", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(485, "ptmsgs_portaladminmsgs"));
			}
		}
		else //Editor is in create mode
		{
			ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");
			ptJSDateTimePicker.Init(AFormDP.HTMLFORM_NAME, m_ptPageMgr, JobEditorAS.RUN_ONCE_DATEPICKER);
			m_ptJSDateTimePickerMgr.RegisterComponent(ptJSDateTimePicker);
						
			ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");
			ptJSDateTimePicker.Init(AFormDP.HTMLFORM_NAME, m_ptPageMgr, JobEditorAS.NEXT_RUN_DATEPICKER);
			m_ptJSDateTimePickerMgr.RegisterComponent(ptJSDateTimePicker);

			ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");
			ptJSDateTimePicker.Init(AFormDP.HTMLFORM_NAME, m_ptPageMgr, JobEditorAS.NOT_AFTER_DATEPICKER);
			m_ptJSDateTimePickerMgr.RegisterComponent(ptJSDateTimePicker);
			
			ATimePicker ptBeginTimePicker = (ATimePicker) m_ptPageMgr.AddPageControl("TimePicker");
			ptBeginTimePicker.Init(m_ptPageMgr, "", JobEditorAS.SUSPEND_BEGIN_TIMEPICKER);
			m_ptJSDateTimePickerMgr.RegisterComponent(ptBeginTimePicker);

			ATimePicker ptEndTimePicker = (ATimePicker) m_ptPageMgr.AddPageControl("TimePicker");
			ptEndTimePicker.Init(m_ptPageMgr, "", JobEditorAS.SUSPEND_END_TIMEPICKER);
			m_ptJSDateTimePickerMgr.RegisterComponent(ptEndTimePicker);
		}
	}


	/**
	 * Code executed before the object is stored.
	 *
	 */
	public void DoTaskBeforeStore() {
		//Set the properties that apply whether this is a new OR edited object
		//Continue on errors setting
		
		try 
		{
			//The Settings method expects an object argument, so create an Integer object from
			//the primitive type to pass into the method
			((IPTJob) m_ptObject).SetSettings(
				PT_JOB_SETTINGS.PT_JOB_CONTINUEONERRORS,m_intContinue);
		} 
		catch (Exception e) 
		{
			GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskBeforeStore", e, false, UIErrorLevel.SERIOUS, m_asOwner.GetString(468, "ptmsgs_portaladminmsgs"));
		}

		//Timeout period for the job
		try 
		{
			((IPTJob) m_ptObject).SetSettings(
				PT_JOB_SETTINGS.PT_JOB_TIMEOUT,
				m_intTimeout);
				
			if (0 != m_intTimeoutUnits)
			{
				((IPTJob) m_ptObject).SetSettings(
					PT_JOB_SETTINGS.PT_JOB_TIMEOUT_UNITS,
					ScheduleTypeFromDuration(m_intTimeoutUnits));
			}
		} 
		catch (Exception e) 
		{
			GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskBeforeStore", e, false, UIErrorLevel.SERIOUS, m_asOwner.GetString(469, "ptmsgs_portaladminmsgs"));
		}

		//Schedule frequency for the job
		try {
			//Get the schedule object
			IPTSchedule ptSchedule = ((IPTJob) m_ptObject).GetSchedule();
			JSDateTimePicker ptJSDateTimePicker;
			
			if (PT_SCHEDULETYPES.PT_SCHEDULETYPE_NEVER == m_intScheduleType) 
			{
				ptSchedule.SetScheduleType(
					PT_SCHEDULETYPES.PT_SCHEDULETYPE_NEVER);

				((IPTJob) m_ptObject).SetSettings(PT_JOB_SETTINGS.PT_JOB_NEXTRUNTIME,0);
			} 
			if (SCHEDULE_TYPE_RUN_ONCE_NOW == m_intScheduleType) 
			{
				long nMillisOneDay = 1000 * 60 * 60 * 24;
				long nMillisYesterday = new XPDateTime().GetTimeInMilliSeconds() - nMillisOneDay;
				
				m_xpdtRunOnceNow = new XPDateTime(nMillisYesterday);

				ptSchedule.SetScheduleType(
					PT_SCHEDULETYPES.PT_SCHEDULETYPE_ONCE);
				((IPTJob) m_ptObject).SetSettings(
					PT_JOB_SETTINGS.PT_JOB_NEXTRUNTIME,
					m_xpdtRunOnceNow);
				ptSchedule.SetStartTimestamp(m_xpdtRunOnceNow);
			}
			else if ( PT_SCHEDULETYPES.PT_SCHEDULETYPE_ONCE == m_intScheduleType) 
			{
				ptJSDateTimePicker = (JSDateTimePicker) m_ptJSDateTimePickerMgr.GetComponent(JobEditorAS.RUN_ONCE_DATEPICKER);
					
				XPDateTime xpdtRunOnce = ptJSDateTimePicker.GetDateTimeAsDateTime();

				ptSchedule.SetScheduleType(
					PT_SCHEDULETYPES.PT_SCHEDULETYPE_ONCE);
				((IPTJob) m_ptObject).SetSettings(
					PT_JOB_SETTINGS.PT_JOB_NEXTRUNTIME,
					xpdtRunOnce);
				ptSchedule.SetStartTimestamp(xpdtRunOnce);
			} 
			else if (1 < m_intScheduleType) 
			{
				ptJSDateTimePicker = (JSDateTimePicker) m_ptJSDateTimePickerMgr.GetComponent(JobEditorAS.NEXT_RUN_DATEPICKER);
					
				XPDateTime xpdtNextRun = ptJSDateTimePicker.GetDateTimeAsDateTime();

				if (null == xpdtNextRun)
				{
					GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskBeforeStore", null, false, UIErrorLevel.WARNING, m_asOwner.GetString(2412, "ptmsgs_portaladminmsgs"));
					SetPageStatus(RepostControl.PAGE_STATUS_INVALID);
					return;
				}

				ptSchedule.SetScheduleType(m_intScheduleType);
				((IPTJob) m_ptObject).SetSettings(
					PT_JOB_SETTINGS.PT_JOB_NEXTRUNTIME,
				xpdtNextRun);
				ptSchedule.SetStartTimestamp(xpdtNextRun);

				ptJSDateTimePicker = (JSDateTimePicker) m_ptJSDateTimePickerMgr.GetComponent(JobEditorAS.NOT_AFTER_DATEPICKER);

				XPDateTime xpdtNotAfter = ptJSDateTimePicker.GetDateTimeAsDateTime();
				
				if (null == xpdtNotAfter)
				{
					GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskBeforeStore", null, false, UIErrorLevel.WARNING, m_asOwner.GetString(2413, "ptmsgs_portaladminmsgs"));
					SetPageStatus(RepostControl.PAGE_STATUS_INVALID);
					return;
				}

				ptSchedule.SetUntilTimestamp(xpdtNotAfter);

				ptSchedule.SetRecurrenceInterval(m_intRecurrenceInterval);

				ptSchedule.SetSettings(
					PT_SCHEDULE_SETTINGS.PT_SCHEDULE_BLOCKEDTIMES,
					new Integer(m_intRestrictions));
			}
		} catch (Exception e) {
			GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskBeforeStore", e, false, UIErrorLevel.SERIOUS, m_asOwner.GetString(470, "ptmsgs_portaladminmsgs"));
		}

		//Suspension options for the job
		try {
			IPTSchedule ptSchedule = ((IPTJob) m_ptObject).GetSchedule();
			TimePicker ptTimePicker;
			
			ptSchedule.SetSettings(
				PT_SCHEDULE_SETTINGS.PT_SCHEDULE_SUSPEND_MODE,
				new Integer(m_intSuspendMode));

			if (PT_SCHEDULE_SUSPEND_MODES.PT_SCHEDULE_WORK_CONTINUOUSLY != m_intSuspendMode) 
			{
				ptTimePicker = (TimePicker) m_ptJSDateTimePickerMgr.GetComponent(JobEditorAS.SUSPEND_BEGIN_TIMEPICKER);
				// SE: 57319 - Fix for Tepco. Using XPLocale.GetDefault() is not right as it gets the locale of whatever the
				// ASP.NET user is set up as in IIS and whatever the system locale is that the JVM is running on. Since we need 
				// to have the time string as US_en when we save it, we can just call GetTimeAsString which gets us the 
				// "standard" time format for English. We can also skip converting to the user's locale as that's not needed here.
				String myDefaultBeginTimeString = ptTimePicker.GetTimeAsString();
				/*
				// this returns the default time string not 'english'.
				String myDefaultBeginTimeString = ptTimePicker.GetTimeAsEnglishString();
				XPLocale myUSLocale = new XPLocale("EN", "us");
				// need to convert time to US_en to satisfy SetSettings method
				myDefaultBeginTimeString = TimePicker.TransformTimeToNewLocale(myDefaultBeginTimeString, XPLocale.GetDefault(), myUSLocale, m_Timezone);
				*/
				
				ptSchedule.SetSettings(
					PT_SCHEDULE_SETTINGS.PT_SCHEDULE_SUSPEND_WEEKDAY_BEGIN_TIME,
					myDefaultBeginTimeString);
				log.Debug("JobModel::DoTaskBeforeStore - Setting myDefaultBeginTimeString: [" + myDefaultBeginTimeString + "]");
				
				ptTimePicker = (TimePicker) m_ptJSDateTimePickerMgr.GetComponent(JobEditorAS.SUSPEND_END_TIMEPICKER);
				// SE: 57319 - Fix for Tepco. Using XPLocale.GetDefault() is not right as it gets the locale of whatever the
				// ASP.NET user is set up as in IIS and whatever the system locale is that the JVM is running on. Since we need 
				// to have the time string as US_en when we save it, we can just call GetTimeAsString which gets us the 
				// "standard" time format for English. We can also skip converting to the user's locale as that's not needed here.
				String myDefaultEndTimeString = ptTimePicker.GetTimeAsString();
				/*
				// this returns the default time string not 'english'.
				String myDefaultEndTimeString = ptTimePicker.GetTimeAsEnglishString();
				myDefaultEndTimeString = TimePicker.TransformTimeToNewLocale(myDefaultEndTimeString, XPLocale.GetDefault(), myUSLocale, m_Timezone);
				*/
				
				ptSchedule.SetSettings(
					PT_SCHEDULE_SETTINGS.PT_SCHEDULE_SUSPEND_WEEKDAY_END_TIME,
					myDefaultEndTimeString);
				log.Debug("JobModel::DoTaskBeforeStore - Setting myDefaultEndTimeString: [" + myDefaultEndTimeString + "]");
			}
		} catch (Exception e) {
			GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskBeforeStore", e, false, UIErrorLevel.SERIOUS, m_asOwner.GetString(471, "ptmsgs_portaladminmsgs"));
		}

		//Logging level for the job
		try {
			((IPTJob) m_ptObject).SetSettings(
				PT_JOB_SETTINGS.PT_JOB_LOGLEVEL,
				m_intLogLevel);

			IPTSchedule ptSchedule = ((IPTJob) m_ptObject).GetSchedule();
			ptSchedule.SetSettings(
				PT_SCHEDULE_SETTINGS.PT_SCHEDULE_SUSPEND_CHECKPOINT_INTERVAL,
				new Integer(m_intSuspendCheckpointInterval));
		} catch (Exception e) {
			GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskBeforeStore", e, false, UIErrorLevel.SERIOUS, m_asOwner.GetString(471, "ptmsgs_portaladminmsgs"));
		}
	}


	/**
	 * Code executed after the object is stored.
	 *
	 */
	public void DoTaskAfterStore() 
	{
		if (!m_bDisableAddOperation) 
		{
			try 
			{
				IPTServerContext ptscJob =
					(IPTServerContext) ((IPTJob) m_ptObject).GetInterfaces(
						"IPTServerContext");

				// Operations on "intrinsic" jobs cannot be modified
				int nJobID = ptscJob.GetObjectID();

				if (nJobID <= PT_SERVERCONSTANTS.PT_MAX_READONLYINTRINSIC_OBJECTID) 
				{
					return;
				}

				try 
				{
					// Lock the object and delete the old entries if we are editing an object
					ptscJob.LockObject();
				} 
				catch (Exception e) 
				{
					GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskAfterStore", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(1841, "ptmsgs_portaladminmsgs"));
				}

				if (EditorStartControl.EDITOR_START_FLAG_EDIT
					== GetEditorType()) 
				{
					ASQueryResultWrapper ptrOperations =
						new ASQueryResultWrapper(
							((IPTJob) m_ptObject).QueryJobList());
					int intOpCount = ptrOperations.GetCount();

					for (int nIndex = intOpCount - 1; nIndex >= 0; nIndex--) 
					{
						Object listIndex =
							ptrOperations.GetFields(
								nIndex,
								PT_PROPIDS.PT_PROPID_JOB_LISTINDEX);

						if (listIndex != null) 
						{
							((IPTJob) m_ptObject).RemoveJobListEntry(
							XPConvert.ToInteger(listIndex));
						}
					}
				}

				for (int i = 0; i < m_arrOperationValues.GetCount(); i++) {
					int nClassID = XPConvert.ToInteger(m_arrOperationValues.GetItem(
							i,
							SelectPTObjectModel.EXPLIST_COL_CLASSID));
					int nObjectID = XPConvert.ToInteger(m_arrOperationValues.GetItem(
							i,
							SelectPTObjectModel.EXPLIST_COL_OBJECTID));
					((IPTJob) m_ptObject).AddJobListEntry(
						-1,
						nClassID,
						nObjectID);
				}
			} 
			catch (Exception e) 
			{
				GeneralHelpers.SetUIError(m_asOwner,OpenLogService.GetComponent("Portal_Admin"),"JobModel::DoTaskAfterStore", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(467, "ptmsgs_portaladminmsgs"));
			} 
			finally 
			{
				IPTServerContext ptscJob =
					(IPTServerContext) ((IPTJob) m_ptObject).GetInterfaces(
						"IPTServerContext");
				if (1 == ptscJob.GetLockState()) {
					ptscJob.UnlockObject();
				}
			}
		}
	}


	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create() {
		return new JobModel();
	}
	// END ***************** Framework methods *****************


	// BEGIN ***************** save DP methods *****************

	/**
	 * Calls the right save function according to the page name passed in argument.
	 *
	 * @param _sPageName
	 * @param _htFormData
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData) 
	{	
		int nPageStatus = super.SavePage(_sPageName, _htFormData);

		if (_sPageName.equals(MainDP.STR_MVC_CLASS_NAME)) 
		{
			nPageStatus = SaveMainPage(_htFormData);
		} 
		else if (_sPageName.equals(HistoryDP.STR_MVC_CLASS_NAME)) 
		{
			nPageStatus = (	(JobLogModel) m_asOwner.GetModel(JobLogModel.STR_MVC_CLASS_NAME))
				.SavePage(_sPageName, _htFormData);
		}

		return nPageStatus;
	}

	/**
	 * Saves the data on the Main page.
	 *
	 * @param _htFormData
	 * @return int Page status for server side validation
	 */
	private int SaveMainPage(XPHashtable _htFormData) {
		int nPageStatus = RepostControl.PAGE_STATUS_VALID;
		String[] data;

		try
		{
			m_ptPageMgr.StoreAllUserInput(_htFormData);
		}
		catch (Exception e)
		{
			nPageStatus = JobRepostControl.PAGE_STATUS_INVALID;
			
			GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Common"), "JobModel::SaveMainPage", e, false, UIErrorLevel.SERIOUS, 
				m_asOwner.GetString(2390, "ptmsgs_portaladminmsgs"));
		}

		try
		{
			//Get the Timeout information from the TimeoutView
			data =
				(String[]) _htFormData.GetElement(
					TimeoutView.HTMLINPUT_TIMEOUT_DURATION);
	
			if ((null != data) && !(data[0].equals(""))) {
				//Set the timeout time and units
				m_intTimeout = XPConvert.ToInteger(data[0]);
				data =
					(String[]) _htFormData.GetElement(
						TimeoutView.HTMLSELECT_TIMEOUT_UNITS);
				m_intTimeoutUnits = XPConvert.ToInteger(data[0]);
			} else {
				m_intTimeout = 0;
				m_intTimeoutUnits = 0;
			}
	
			data =
				(String[]) _htFormData.GetElement(TimeoutView.HTMLCHECKBOX_IGNORE);
	
			if ((null != data) && !(data[0].equals(""))) {
				//The box was checked
				m_intContinue = 1;
			} else {
				m_intContinue = 0;
			}
	
			//Get the Schedule information from the ScheduleView
			data =
				(String[]) _htFormData.GetElement(ScheduleView.HTMLINPUT_SCHEDULE);
	
			if ((null != data) && !(data[0].equals(""))) {
				//Set the schedule
				m_intScheduleType = XPConvert.ToInteger(data[0]);
	
				//If the number is SCHEDULE_TYPE_RECURRING, then get more information and set the periodicity.
				//Otherwise, just set it
				if (SCHEDULE_TYPE_RECURRING == m_intScheduleType) {
					data =
						(String[]) _htFormData.GetElement(
							ScheduleView.HTMLSELECT_RUN_UNITS);
	
					if ((null != data) && !(data[0].equals(""))) {
						m_intScheduleType =
							ScheduleTypeFromDuration(XPConvert.ToInteger(data[0]));
					}
	
					data =
						(String[]) _htFormData.GetElement(
							ScheduleView.HTMLINPUT_RUN_EVERY);
	
					if ((null != data) && !(data[0].equals(""))) {
						m_intRecurrenceInterval = XPConvert.ToInteger(data[0]);
					}
	
					if (0 == m_intRecurrenceInterval) //User did not enter run every information
					{
						nPageStatus = JobRepostControl.PAGE_STATUS_INVALID;
			
						GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Common"), "JobModel::SaveMainPage", null, false, UIErrorLevel.SERIOUS, 
							m_asOwner.GetString(1840, "ptmsgs_portaladminmsgs"));

						return nPageStatus;
					}
	
					//Get the restrictions setting
					data =
						(String[]) _htFormData.GetElement(
							ScheduleView.HTMLSELECT_RESTRICTIONS);
	
					if ((null != data) && !(data[0].equals(""))) {
						m_intRestrictions = XPConvert.ToInteger(data[0]);
					}
				}
			}
	
			//Get the Suspension information
			data =
				(String[]) _htFormData.GetElement(
					ScheduleView.HTMLCHECKBOX_SUSPENDOPTIONS);
	
			if ((null != data) && !(data[0].equals(""))) {
				m_intSuspendMode = XPConvert.ToInteger(data[0]);
			}
			else
			{
				m_intSuspendMode = PT_SCHEDULE_SUSPEND_MODES.PT_SCHEDULE_WORK_CONTINUOUSLY;
			}
	
			//Get the Logging information from the Logging Section
			data = (String[]) _htFormData.GetElement(LoggingView.HTMLSELECT_LEVEL);
	
			if ((null != data) && !(data[0].equals(""))) {
				m_intLogLevel = XPConvert.ToInteger(data[0]);
			}
	
			//The checkpoint information
			data =
				(String[]) _htFormData.GetElement(
					LoggingView.HTMLINPUT_CHECKPOINTMINUTES);
	
			if ((null != data) && !(data[0].equals(""))) {
				m_intSuspendCheckpointInterval = XPConvert.ToInteger(data[0]);
			}
	
			if (!m_bDisableAddOperation) {
				// if this is the display page where you have your select ptobject you need to
				// call the associated saveSection method.
				(
					(SelectPTObjectModel) m_asOwner.GetModel(
						JobEditorAS.m_strStorageSelectPTObject)).SaveSection(
					_htFormData);
	
				m_arrOperationValues =
					((SelectPTObjectModel) m_asOwner
						.GetModel(JobEditorAS.m_strStorageSelectPTObject))
						.GetItems();
			}
		}
		catch (Exception e)
		{
			nPageStatus = JobRepostControl.PAGE_STATUS_INVALID;
			
			GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Common"), "JobModel::SaveMainPage", e, false, UIErrorLevel.SERIOUS, 
				m_asOwner.GetString(2414, "ptmsgs_portaladminmsgs"));
		}
		
		return nPageStatus;
	}

	// END ***************** Save DP Methods *****************


	// BEGIN ***************** Helper methods *****************
	
	/**
	 * This function initializes the manager objects (Page and datetime) for the date pickers
	 */
	public void InitializeDatePickerMgrs()
	{
		// Deal with the page manager
		String strImageServerBaseURL = ImgSvrURLHelper.GetInstance().GetRootURL(m_asOwner.GetIsSecuredSpace());
		m_ptPageMgr     = new PageMgr (m_asOwner.GetAccessStyle(), m_Locale, m_Timezone, strImageServerBaseURL, ConfigHelper.GetPortalImageServerConnectionURL(m_asOwner));

		// Deal with the date/time-picker
		JSRegistry ptJSRegistry = JSRegistry.GetRegistry(strImageServerBaseURL, ConfigHelper.GetPortalImageServerConnectionURL(m_asOwner));

		m_ptJSDateTimePickerMgr = new JSDateTimePickerMgr(ptJSRegistry, new JSDatePickerVersionMgr(), m_ptPageMgr);
		m_ptPageMgr.RegisterComponentMgr(m_ptJSDateTimePickerMgr);

	}
	
	/**
	 * Delete the job history from the passed in date and earlier
	 *
	 * @param xpdtDeleteFrom
	 *
	 */
	public void DeleteJobHistory(XPDateTime xpdtDeleteFrom) {
		try 
		{
			IPTScheduler ptScheduler = m_Session.GetScheduler();
			ptScheduler.DeleteJobHistory(xpdtDeleteFrom);
		} 
		catch (Exception e) 
		{
			GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Common"), "JobModel::DeleteJobHistory", e, true, UIErrorLevel.SERIOUS, 
				m_asOwner.GetString(491, "ptmsgs_portaladminmsgs"));
		}
	}

	/**
	 * Helper method to start the Select PTObject List.
	 * 
	 * String[] _arrNewOp 	contains class/object id of a new operation that
	 * 						launched this editor else null or empty 
	 */
	public void StartSelectPTObject(int[] _arrNewOp) {
		// Initialize the Select PTObject list.
		//      _sSectionTitle: Main title.
		//      _sSectionSubTitle: Sub title.
		//      _imgAddIcon: Icon for the add action.
		//      _sAddLabel: Text to display next to the add icon.
		//      _arrObjectClassIDs Class ID of the Objects displayed on the list.
		//      _nListKey: In case you have multiple Expandable Lists on
		//                 the same DP, they should have a different key 
		//                 (otherwise, just set it to 1).
		SelectPTObjectModel mySelectPTObjectModel = (SelectPTObjectModel) m_asOwner.GetModel(
						JobEditorAS.m_strStorageSelectPTObject);
		
		ASQueryResultWrapper myQueryResult;
		myQueryResult =
			new ASQueryResultWrapper(
				(IPTQueryResult) m_Session.OpenGlobalObject(
					PT_GLOBALOBJECTS.PT_GLOBAL_SCHEDULED_CLASSES,
					false));

		int lCount = myQueryResult.GetCount();
		int[] arrClassIDs = new int[lCount];

		for (int i = 0; i < lCount; i++) {
			arrClassIDs[i] =
				XPConvert.ToInteger(myQueryResult
					.GetFields(i, PT_PROPIDS.PT_PROPID_CLASSID));
		}

		mySelectPTObjectModel.InitializeSelectPTObject(
			m_asOwner.GetString(487, "ptmsgs_portaladminmsgs"),
			m_asOwner.GetString(488, "ptmsgs_portaladminmsgs"),
			PTImageType.ICON_OPERATION,
			m_asOwner.GetString(489, "ptmsgs_portaladminmsgs"), 
			arrClassIDs, 
			1);

		// Optional setting
		//     Use the method bellow if you want to have the move up, 
		//     down, first and last small arrows at the end of each row.
		//     Warning: This will disable the sort by column (we can NOT have both).
		mySelectPTObjectModel.SetEnableMoveRowActions(true);
		mySelectPTObjectModel.SetTreeSubTitle(m_asOwner.GetString(1842, "ptmsgs_portaladminmsgs"));
		mySelectPTObjectModel.SetTreeTitle(m_asOwner.GetString(1843, "ptmsgs_portaladminmsgs"));
		mySelectPTObjectModel.SetTreeMultiSelect(true);

		//Set the message and instructions to display when the list is empty.
		mySelectPTObjectModel.SetEmptyListMessages(m_asOwner.GetString(1844, "ptmsgs_portaladminmsgs"), m_asOwner.GetString(1845, "ptmsgs_portaladminmsgs"));

		//Get the job operations if we are in edit mode, else, if in create and we were passed
		// new operations to associate, then do so
		if (EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType() && 
			(null != _arrNewOp && _arrNewOp.length>0))
		{
			try 
			{
				int op_clsid = _arrNewOp[0];
				int	op_objid = _arrNewOp[1];
				String strObjName; 
				if (op_clsid != 20) //Admin Folders
				{
					strObjName = m_Session.GetObjectManagers(op_clsid).
						QuerySingleObject(op_objid).ItemAsString(0,PT_PROPIDS.PT_PROPID_NAME);
				} 
				else // this will probably never ever happen, but, what do i know.
				{
					//In the case of admin folders, we just have to open the folder and add it
					IPTAdminFolder myFolder = m_Session.GetAdminCatalog().OpenAdminFolder(op_objid, false);
					strObjName = myFolder.GetLocalizedName();
				}

				// create arr object and add as default rows
				Object[][] arrOperations = new Object[1][];
				arrOperations[0] = new Object[SelectPTObjectModel.EXPLIST_COL_MAX];
				arrOperations[0][SelectPTObjectModel.EXPLIST_SORTEDARRAY_PROPID_INDEX] = 
					new Integer(mySelectPTObjectModel.m_nNextIndex);
				mySelectPTObjectModel.m_nNextIndex++;

				arrOperations[0][SelectPTObjectModel.EXPLIST_COL_CLASSID] = new Integer(op_clsid);
				arrOperations[0][SelectPTObjectModel.EXPLIST_COL_OBJECTID] = new Integer(op_objid);
				arrOperations[0][SelectPTObjectModel.EXPLIST_COL_STRING] = strObjName;

				mySelectPTObjectModel.SetDefaultObjects(arrOperations);
			} 
			catch (Exception e) 
			{
				GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Common"), 
					"JobModel::StartSelectPTObject", e, true, UIErrorLevel.SERIOUS, 
					m_asOwner.GetString(486, "ptmsgs_portaladminmsgs"));
			}

		}
		
		else if (EditorStartControl.EDITOR_START_FLAG_EDIT == GetEditorType()) 
		{
			try {
				//Since you can't query for the names, have to do some query filters for each of the dictionaries.
				//Get the total number of schedulable classids
				//Get the operations that can be displayed
				ASQueryResultWrapper ptqrOperations =
					new ASQueryResultWrapper(
						(IPTQueryResult) m_Session.OpenGlobalObject(
							PT_GLOBALOBJECTS.PT_GLOBAL_SCHEDULED_CLASSES,
							false));
				int intCount = ptqrOperations.GetCount();
				Object[][] arrDictionaries = new Object[3][intCount];

				//Dictionary/Map to hold the system agent information
				XPHashtable dicSysAg;

				//Dictionaries for the names/ids of the operations
				//Get all the possible classes that can be used as operations
				int intObjCID;
				int intObjID;

				for (int i = 0; i < intCount; i++) {
					intObjCID =
						XPConvert.ToInteger(ptqrOperations.GetFields(
							i,
							PT_PROPIDS.PT_PROPID_CLASSID));

					String strObjName =
						(String) ptqrOperations.GetFields(
							i,
							PT_PROPIDS.PT_PROPID_NAME);

					if (0 == intObjCID)
						//0 is system agents, and it's a special case..
					{
						//Dictionary for the names/ids of the system agents
						IPTQueryResult ptqrSysAg;
						int intSysAgIndex = 0;
						ptqrSysAg =
							(IPTQueryResult) m_Session.OpenGlobalObject(
								PT_GLOBALOBJECTS.PT_GLOBAL_SCHEDULED_AGENTS,
								false);
						dicSysAg = new XPHashtable();

						for (intSysAgIndex = 0;
							intSysAgIndex < ptqrSysAg.RowCount();
							intSysAgIndex++) 
						{
							int nScheduledAgentObjectID = ptqrSysAg.ItemAsInt(intSysAgIndex, PT_PROPIDS.PT_PROPID_OBJECTID);
							String strAgentName = GetLocalizedNameForGlobalObject(nScheduledAgentObjectID);
							if(null == strAgentName)
							{
								strAgentName = ptqrSysAg.ItemAsString(intSysAgIndex, PT_PROPIDS.PT_PROPID_NAME);
							}
							
							dicSysAg.PutElement(
								new Integer(nScheduledAgentObjectID),
								strAgentName);
						}

						//Add it to the array.
						int intGlobalObjectID = PT_CLASSIDS.PT_GLOBALOBJECT_ID;
						arrDictionaries[c_intDict][i] = dicSysAg;
						arrDictionaries[c_intDictObjClass][i] = new Integer(intGlobalObjectID);

						arrDictionaries[c_intDictObjName][i] =
							m_asOwner.GetString(466, "ptmsgs_portaladminmsgs");
					} else {
						arrDictionaries[c_intDict][i] = new XPHashtable();
						arrDictionaries[c_intDictObjClass][i] = new Integer(intObjCID);
						arrDictionaries[c_intDictObjName][i] = strObjName;
					}
				}

				//Operations of this job.
				ASQueryResultWrapper ptqrJobOps =
					new ASQueryResultWrapper(
						((IPTJob) m_ptObject).QueryJobList());
				int intOpCount = ptqrJobOps.GetCount();

				//String strOperations = "";
				//Load all the information so that the pages can display it
				//Put all the IDs of the ops that are going to be displayed into a dictionary.  Cannot 
				//query for their names.  
				for (int i = 0; i < intOpCount; i++) {
					intObjCID =
						XPConvert.ToInteger(ptqrJobOps.GetFields(
							i,
							PT_PROPIDS.PT_PROPID_JOB_OPERATIONCLASSID));

					if (intObjCID
						!= PT_CLASSIDS.PT_GLOBALOBJECT_ID) {
						intObjID =
							XPConvert.ToInteger(ptqrJobOps.GetFields(
								i,
								PT_PROPIDS.PT_PROPID_JOB_OPERATIONOBJECTID));
						(
							(XPHashtable) arrDictionaries[c_intDict][GetDictionary(
								intObjCID,
								arrDictionaries)]).PutElement(
							new Integer(intObjID),
							"");
					}
				}


				//Since you can't query for the names, have to do some query filters for each of the dictionaries.
				// Query filter
				Object[][] ptqf = new Object[3][1];
				ASQueryResultWrapper ptqrTempNames;

				for (int i = 0; i < intCount; i++) 
				{
					int[] arrKeys =
						new int[((XPHashtable) arrDictionaries[c_intDict][i])
							.GetSize()];

					IXPEnumerator myEnum;

					myEnum = ((XPHashtable) arrDictionaries[c_intDict][i]).GetKeys();

					int intTemp;

					int lIndex = -1;

					while (myEnum.MoveNext()) {
						lIndex++;
						intTemp = XPConvert.ToInteger(myEnum.GetCurrent());
						arrKeys[lIndex] = intTemp;
					}

					if ((arrKeys.length == 0) || (null == arrKeys)) 
					{
						arrDictionaries[c_intDict][i] = new XPHashtable();
					} else 
					{
						intTemp = XPConvert.ToInteger(arrDictionaries[c_intDictObjClass][i]);

						if (intTemp > 0) 
						{
							if (intTemp != 20 ) //Admin Folders
							{
								//Create the query filter
								ptqf[0][0] =
									new Integer(PT_PROPIDS.PT_PROPID_OBJECTID);
								ptqf[1][0] =
									new Integer(PT_FILTEROPS.PT_FILTEROP_IN);
								ptqf[2][0] = arrKeys;
								
								ptqrTempNames =
									new ASQueryResultWrapper(
										m_Session.GetObjectManagers(
											XPConvert.ToInteger(arrDictionaries[c_intDictObjClass][i]))
											.Query(
												PT_PROPIDS.PT_PROPID_OBJECTID | PT_PROPIDS.PT_PROPID_NAME,
												-1,
												(Object)null,
												0,
												-1,
												ptqf));
								for (int j = 0; j < ptqrTempNames.GetCount(); j++)
								{
									( (XPHashtable) arrDictionaries[c_intDict][i])
										.PutElement(ptqrTempNames.GetFields(
										j,
										PT_PROPIDS.PT_PROPID_OBJECTID),
										ptqrTempNames.GetFields(
										j,
										PT_PROPIDS.PT_PROPID_NAME));
								}
							} else
							{
								//In the case of admin folders, we just have to open the folder and add it
								for (int k=0; k< arrKeys.length; k++)
								{
									IPTAdminFolder myFolder = m_Session.GetAdminCatalog().OpenAdminFolder(arrKeys[k], false);
									((XPHashtable) arrDictionaries[c_intDict][i]).PutElement(new Integer(arrKeys[k]), myFolder.GetLocalizedName() );
								}
							}
						}
					}
				}

				//Store the results into strings that can be passed in the form.
				Object[][] arrOperations = new Object[intOpCount][];
				SelectPTObjectModel mySelectModel =
					(SelectPTObjectModel) m_asOwner.GetModel(
						JobEditorAS.m_strStorageSelectPTObject);

				for (int i = 0; i < intOpCount; i++) {
					intObjCID =
						XPConvert.ToInteger(ptqrJobOps.GetFields(
							i,
							PT_PROPIDS.PT_PROPID_JOB_OPERATIONCLASSID));
					intObjID =
					XPConvert.ToInteger(ptqrJobOps.GetFields(
							i,
							PT_PROPIDS.PT_PROPID_JOB_OPERATIONOBJECTID));
					String strObjName =
						(String)
							(
								(XPHashtable) arrDictionaries[c_intDict][GetDictionary(
									intObjCID,
									arrDictionaries)]).GetElement(
							new Integer(intObjID));

					arrOperations[i] =
						new Object[SelectPTObjectModel.EXPLIST_COL_MAX];

					arrOperations[i][SelectPTObjectModel
						.EXPLIST_SORTEDARRAY_PROPID_INDEX] =
						new Integer(mySelectModel.m_nNextIndex);
					mySelectModel.m_nNextIndex++;

					arrOperations[i][SelectPTObjectModel.EXPLIST_COL_CLASSID] =
						new Integer(intObjCID);
					arrOperations[i][SelectPTObjectModel.EXPLIST_COL_OBJECTID] =
						new Integer(intObjID);
					arrOperations[i][SelectPTObjectModel.EXPLIST_COL_STRING] =
						strObjName;
				}

				// Add default rows.
				(
					(SelectPTObjectModel) m_asOwner.GetModel(
						JobEditorAS
							.m_strStorageSelectPTObject))
							.SetDefaultObjects(
					arrOperations);

				//m_arrOperationValues.AddRows(arrOperations);
			} 
			catch (Exception e) 
			{
				GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Common"), "JobModel::StartSelectPTObject", e, true, UIErrorLevel.SERIOUS, 
					m_asOwner.GetString(486, "ptmsgs_portaladminmsgs"));
			}
		}
	}

	private String GetLocalizedNameForGlobalObject(int nGlobalObjectID)
	{
		String strAgentName;
		switch(nGlobalObjectID)
		{
			case PT_GLOBALOBJECTS.PT_GLOBAL_CARD_REFRESH_AGENT:
				strAgentName = m_asOwner.GetString(2680, "ptmsgs_portaladminmsgs");
				break;
			case PT_GLOBALOBJECTS.PT_GLOBAL_CATALOG_SEARCH_UPDATE_AGENT:
				strAgentName = m_asOwner.GetString(2681, "ptmsgs_portaladminmsgs");
				break;
			case PT_GLOBALOBJECTS.PT_GLOBAL_WEEKLY_HOUSEKEEPING_AGENT:
				strAgentName = m_asOwner.GetString(2682, "ptmsgs_portaladminmsgs");
				break;
			case PT_GLOBALOBJECTS.PT_GLOBAL_AUDITMANAGER:
				strAgentName = m_asOwner.GetString(2683, "ptmsgs_portaladminmsgs");
				break;
			case PT_GLOBALOBJECTS.PT_GLOBAL_DYNAMIC_MEMBERSHIP_UPDATE_AGENT:
				strAgentName = m_asOwner.GetString(2736, "ptmsgs_portaladminmsgs");
				break;
			default:
				strAgentName = null;
				break;
		}
		
		return strAgentName;
	}

	/**
	 * This helper function translates PTSCHEDULETYPES units into PTDurationCombo values.
	 * If it returns 0, it means the schedule type does not translate into a
	 * known PTDurationCombo duration.
	 */
	public static int DurationFromPTScheduleType(int nScheduleType) {
		switch (nScheduleType) {
			case PT_SCHEDULETYPES.PT_SCHEDULETYPE_MINUTELY :
				return PTDurationCombo.DISPLAYMINUTES;

			case PT_SCHEDULETYPES.PT_SCHEDULETYPE_DAILY :
				return PTDurationCombo.DISPLAYDAYS;

			case PT_SCHEDULETYPES.PT_SCHEDULETYPE_HOURLY :
				return PTDurationCombo.DISPLAYHOURS;

			case PT_SCHEDULETYPES.PT_SCHEDULETYPE_WEEKLY :
				return PTDurationCombo.DISPLAYWEEKS;

			case PT_SCHEDULETYPES.PT_SCHEDULETYPE_MONTHLY :
				return PTDurationCombo.DISPLAYMONTHS;

			case PT_SCHEDULETYPES.PT_SCHEDULETYPE_ANNUALY :
				return PTDurationCombo.DISPLAYYEARS;

			case PT_SCHEDULETYPES.PT_SCHEDULETYPE_NEVER :
			case PT_SCHEDULETYPES.PT_SCHEDULETYPE_ONCE :
			default :
				return 0;
		}
	}


	/**
	 * This helper function translates PTSCHEDULETYPES units into PTDurationCombo values
	 */
	private int DurationFromScheduleType(int nScheduleType) {
		int nDuration = JobModel.DurationFromPTScheduleType(nScheduleType);

		if (nDuration == 0) 
		{
			GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Admin"), "JobModel::DurationFromScheduleType", null, false, UIErrorLevel.SERIOUS, 
				m_asOwner.GetString(2416, "ptmsgs_portaladminmsgs"));
		}

		return nDuration;
	}

	
	/**
	 * Takes in a class ID, and returns the index in the dictionaries (XPMaps) array
	 * for that class.  From there you can either retrieve the dictionary or
	 * the name of the class.
	 * */
	private int GetDictionary(int intCID, Object[][] arrDictionaries) 
	{
		int i;
		int intReturn;
		intReturn = -1;

		for (i = 0; i < arrDictionaries[c_intDictObjClass].length; i++) {
			if (XPConvert.ToInteger(arrDictionaries[c_intDictObjClass][i]) == intCID) {
				intReturn = i;

				return intReturn;
			}
		}

		return intReturn;
	}


	// AGC-

	/**
	 * This helper function translates PTDurationCombo units into PTSCHEDULETYPES values
	 */
	private int ScheduleTypeFromDuration(int nDuration) {
		switch (nDuration) {
			case PTDurationCombo.DISPLAYMINUTES :
				return PT_SCHEDULETYPES.PT_SCHEDULETYPE_MINUTELY;

			case PTDurationCombo.DISPLAYDAYS :
				return PT_SCHEDULETYPES.PT_SCHEDULETYPE_DAILY;

			case PTDurationCombo.DISPLAYHOURS :
				return PT_SCHEDULETYPES.PT_SCHEDULETYPE_HOURLY;

			case PTDurationCombo.DISPLAYWEEKS :
				return PT_SCHEDULETYPES.PT_SCHEDULETYPE_WEEKLY;

			case PTDurationCombo.DISPLAYMONTHS :
				return PT_SCHEDULETYPES.PT_SCHEDULETYPE_MONTHLY;

			case PTDurationCombo.DISPLAYYEARS :
				return PT_SCHEDULETYPES.PT_SCHEDULETYPE_ANNUALY;

			case PTDurationCombo.DISPLAYSECONDS :
			case PTDurationCombo.DISPLAYMILLISECONDS :
			default :

				GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Admin"), "JobModel::ScheduleTypeFromDuration", null, false, UIErrorLevel.SERIOUS, 
					m_asOwner.GetString(2416, "ptmsgs_portaladminmsgs"));

				return PT_SCHEDULETYPES.PT_SCHEDULETYPE_NEVER;
		}
	}

	/**
	 * This helper function checks whether an admin folder has been assigned to any job server
	 */
	public boolean IsFolderRegistered(int nAdminFolderID)
	{
		boolean bAssigned = false;
		
		IPTScheduler ptScheduler = m_Session.GetScheduler();
		
	    IPTQueryResult ptQRServers = ptScheduler.QueryJobServerStatus("*");
	    int nNumServers = ptQRServers.RowCount();
	    
	    // Check each server to see if the folder is assigned that server
	    for(int i=0; i<nNumServers; i++)
	    {
	    	String strServerName = ptQRServers.ItemAsString(i, PT_PROPIDS.PT_PROPID_JOBSERVER_SERVERNAME);
			
		    IPTQueryResult ptQRAssignedFolders = ptScheduler.QueryJobAssignmentsByServer(strServerName);
		    int nNumAssignedFolders = ptQRAssignedFolders.RowCount();
		    
		    // Check each server to see if the folder is assigned that server
		    for(int j=0; j<nNumAssignedFolders; j++)
		    {
	            int nAssignedFolderID = ptQRAssignedFolders.ItemAsInt(j, PT_PROPIDS.PT_PROPID_FOLDERID);
	            
	            if(nAdminFolderID == nAssignedFolderID)
	            {
	            	bAssigned = true;
	            }
		    }
	    }
	    
	    return bAssigned;
	}


	// END ***************** Helper methods *****************

	
	// BEGIN ***************** Get and Set Methods *****************
	/**
	 * Returns Continue on errors setting.
	 *
	 * @return int
	 */
	public int GetContinue() {
		return m_intContinue;
	}

	/**
	 * Returns Log Level setting.
	 *
	 * @return int
	 */
	public int GetLogLevel() {
		return m_intLogLevel;
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName() {
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns Array of Operations.
	 *
	 * @return PTGrowableSortedArrayWrapper
	 */
	public PTGrowableSortedArrayWrapper GetOperations() {
		return m_arrOperationValues;
	}

	/**
	 * Returns Recurrence Interval setting.  For example
	 * this job will run every N minutes, where N is the recurrence interval.
	 *
	 * @return int
	 */
	public int GetRecurrenceInterval() {
		return m_intRecurrenceInterval;
	}

	/**
	 * Returns Restrictions setting. (0 = none (default), 1 = No Weekday, 2 = No weekend)
	 *
	 * @return int
	 */
	public int GetRestrictions() {
		return m_intRestrictions;
	}

	/**
	 * Returns Schedule Type setting. 0=NEVER, 1=ONCE, >1=PERIODICALLY.  Valid
	 * values are:
	 * PT_SCHEDULETYPE_NEVER = 0 PT_SCHEDULETYPE_ONCE = 1
	 * PT_SCHEDULETYPE_MINUTELY = 2 PT_SCHEDULETYPE_HOURLY = 3
	 * PT_SCHEDULETYPE_DAILY = 4 PT_SCHEDULETYPE_WEEKLY = 5
	 * PT_SCHEDULETYPE_MONTHLY = 6 PT_SCHEDULETYPE_ANNUALLY = 7
	 * 
	 * @return int
	 */
	public int GetScheduleType() {
		return m_intScheduleType;
	}

	/**
	 * Returns Suspend Checkpoint Interval setting.
	 *
	 * @return int
	 */
	public int GetSuspendCheckpointInterval() {
		return m_intSuspendCheckpointInterval;
	}

	/**
	 * Returns Suspend Mode setting:  One of PT_SCHEDULE_SUSPEND_MODES:
	 * PT_SCHEDULE_WORK_CONTINUOUSLY (Default),
	 * PT_SCHEDULE_SUSPEND_ON, PT_SCHEDULE_WORK_ON
	 * @return int
	 */
	public int GetSuspendMode() {
		return m_intSuspendMode;
	}

	/**
	 * Returns timeout setting.
	 *
	 * @return int
	 */
	public int GetTimeout() {
		return m_intTimeout;
	}

	/**
	 * Returns timeout units setting.
	 *
	 * @return int
	 */
	public int GetTimeoutUnits() {
		return m_intTimeoutUnits;
	}

	/**
	 * Returns true if the Add an Operation table on the Main DP should not be displayed.
	 *
	 * @return boolean
	 */
	public boolean IsAddOperationDisabled() {
		return m_bDisableAddOperation;
	}

	/**
	 * @return XPLocale
	 */
	public XPLocale GetLocale()
	{
		return m_Locale;
	}

	/**
	 * @return XPTimeZone
	 */
	public XPTimeZone GetTimezone()
	{
		return m_Timezone;
	}

	/**
	 * Returns the format pattern for the current locale so that it can be passed into
	 * the date picker control
	 * 
	 * @return String
	 */
	public String GetDateFormatPattern()
	{
		return m_strDateFormatPattern;
	}

	/**
	 * Get the Date Time picker control manager
	 *
	 * @return JSDateTimePickerMgr
	 */
	public JSDateTimePickerMgr GetDateTimePickerControlManager()
	{
		return m_ptJSDateTimePickerMgr;
	}

	public XPDateTime GetRunOnceNowDateTime()
	{
		return m_xpdtRunOnceNow;
	}

	/**
	 * Returns whether the Folder containing this Job is assigned to any Automation Server
	 *
	 * @return boolean
	 */
	public boolean IsJobFolderRegistered()
	{
		return IsFolderRegistered(m_nFolderID);
	}

	// END ***************** Get and Set Methods *****************


}


