package com.plumtree.portalpages.admin.editors.property;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.marshalers.*;
import com.plumtree.server.treeiterator.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.expandablelist.growablelist.*;
import com.plumtree.uiinfrastructure.expandablelist.multicolumnlist.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * Model for property editor
 *
 * @author josephc
 */
public class PropModel extends ObjEditorModel implements IPropModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.editors.property.PropModel");

	/** Class name for PropModel */
	public static final String STR_MVC_CLASS_NAME = "PropModel";

	/** Property object */
	private IPTProperty m_ptProperty = null;

	/** Managed list validation function */
	private String m_strManagedListValidation = "";

	/** Pick table */
	private String m_strPickTable = null;

	/** True if property is always card property */
	private boolean m_bAlwaysCardProp = false;

	/** True if property is card property */
	private boolean m_bCardProp = true;

	/** True if property is mandatory */
	private boolean m_bMandatory = false;

	/** True if property is multi-valued */
	private boolean m_bMultiValued = false;

	/** True if property is searchable */
	private boolean m_bSearchable = false;

	/** True if proeprty is an URL */
	private boolean m_bURL = false;

	/** True if property is visible */
	private boolean m_bVisible = true;

	/** True if property is read only */
	private boolean m_bReadOnly = false;

	/** Property pick type */
	private int m_nPickType;

	/** Reference class */
	private int m_nReferenceClass = -1;

	/** Property type */
	private int m_nPropType;

	/** Key for growable pick list value list */
	private static final int m_nPickListValueListKey = 1;

	/** Key for growable pick table column list */
	private static final int m_nPickTableColListKey = 2;

	/** Temporary pick list property for validating unmanaged dropdowns and picktrees */
	private IPTProperty m_tempPickListProperty = null;

	/**
	 * Creates a new PropModel object.
	 */
	public PropModel()
	{
		m_nPickType     = PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_NONE;
		m_nPropType     = PT_PROPERTY_TYPES.PT_PROPTYPE_STRING;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new PropModel();
	}

	/**
	 * Code executed after the object is stored.
	 */
	public void DoTaskAfterStore()
	{
		// Store the Managed PickList values
		if (PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_MANAGED_PICKLIST == m_nPickType)
		{
			SaveManagedPickListValues();
		}
	}

	/**
	 * Code executed before the object is stored.
	 */
	public void DoTaskBeforeStore()
	{
		// Property Type (only in CREATE mode)
		if (EditorStartControl.EDITOR_START_FLAG_EDIT != GetEditorType())
		{
			m_ptProperty.Initialize(m_nPropType);
		}

		m_ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_HREF, (m_bURL ? 1 : 0));
		m_ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_REFCLASS, m_nReferenceClass);
		m_ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_ISCARDPROP, (m_bCardProp ? 1 : 0));
		m_ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_VISIBLE, (m_bVisible ? 1 : 0));
		m_ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_READONLYVALUE, (m_bReadOnly ? 1 : 0));
		m_ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_MANDATORY, (m_bMandatory ? 1 : 0));
		m_ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_SEARCHABLE, (m_bSearchable ? 1 : 0));
		m_ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_MULTIVALUED, (m_bMultiValued ? 1 : 0));
		m_ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_PICKTYPE, (m_nPickType));

		switch (m_nPickType)
		{
		case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_UNMANAGED_PICKLIST:
		case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_PICKTREE:
		{
			m_ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_PICKTABLE, m_strPickTable);

			// Save pick columns and sort columns
			Object[][] pickTableColumns = ((PickTreeColumnsListModel) m_asOwner.GetModel(PropEditorAS.strPickTableColumnList)).GetListValues();

			m_ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_PICKTABLECOLS, pickTableColumns);
		}

		break;

		default:
			break;
		}
	}

	/**
	 * Code to perform tasks specific to Property Editor
	 *
	 * @param _htFormData
	 */
	public void DoTaskOnStartEditor(XPHashtable _htFormData)
	{
		m_ptProperty = (IPTProperty) m_ptObject;

		Object[][] arPickTableColumns = null;

		// Retrieve current settings only in EDIT mode
		if (EditorStartControl.EDITOR_START_FLAG_EDIT == GetEditorType())
		{
			try
			{
				// Value type
				m_nPropType = m_ptProperty.GetType();

				// URL
				if (PT_PROPERTY_TYPES.PT_PROPTYPE_STRING == m_nPropType)
				{
					m_bURL = (1 == m_ptProperty.GetCharacteristicsAsInt(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_HREF));
				}
				else
				{
					m_bURL = false;
				}

				// Reference class
				if (PT_PROPERTY_TYPES.PT_PROPTYPE_REF == m_nPropType)
				{
					m_nReferenceClass = m_ptProperty.GetCharacteristicsAsInt(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_REFCLASS);
				}
				else
				{
					m_nReferenceClass = -1;
				}

				// Card property
				if (PT_PROPERTY_TYPES.PT_PROPTYPE_REF != m_nPropType)
				{
					m_bCardProp = (1 == m_ptProperty.GetCharacteristicsAsInt(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_ISCARDPROP));
				}
				else
				{
					// Card prop must be set to false, because reference types cannot be used with cards
					m_bCardProp = false;
				}

				// Visible
				m_bVisible     = (1 == m_ptProperty.GetCharacteristicsAsInt(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_VISIBLE));

				// Read-only
				m_bReadOnly = (1 == m_ptProperty.GetCharacteristicsAsInt(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_READONLYVALUE));

				// Mandatory
				if (m_bVisible && !m_bReadOnly)
				{
					m_bMandatory = (1 == m_ptProperty.GetCharacteristicsAsInt(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_MANDATORY));
				}
				else
				{
					m_bMandatory = false;
				}

				// Searchable
				if (m_bVisible)
				{
					m_bSearchable = (1 == m_ptProperty.GetCharacteristicsAsInt(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_SEARCHABLE));
				}
				else
				{
					m_bSearchable = false;
				}

				// Multivalued
				m_bMultiValued = (1 == m_ptProperty.GetCharacteristicsAsInt(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_MULTIVALUED));

				// Picktype
				if ((PT_PROPERTY_TYPES.PT_PROPTYPE_REF != m_nPropType) && (PT_PROPERTY_TYPES.PT_PROPTYPE_DATE != m_nPropType))
				{
					m_nPickType = m_ptProperty.GetCharacteristicsAsInt(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_PICKTYPE);

					// Update pick table and pick columns
					switch (m_nPickType)
					{
					case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_UNMANAGED_PICKLIST:
					case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_PICKTREE:
					{
						// Load pick table
						m_strPickTable     = m_ptProperty.GetCharacteristicsAsString(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_PICKTABLE);

						// Load pick columns
						arPickTableColumns = m_ptProperty.GetCharacteristicsAsObject2DArray(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_PICKTABLECOLS);
					}

					default:
						break;
					}
				}
				else
				{
					m_nPickType		   = PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_NONE;
					m_strPickTable     = null;
				}
			}
			catch (Exception e)
			{
				log.Error(e, STR_MVC_CLASS_NAME + "::DoTaskOnStartEditor(): Unable to retrieve current settings:: " + e.getMessage());
			}
		}
		else
		{
			// Create mode
			if (this.m_asOwner.IsSubSpace())
			{
				m_bAlwaysCardProp = true;
			}
			else
			{
				m_bAlwaysCardProp = false;
			}

			((EditorAS) m_asOwner).GetLeftMenu().HideLink(true, MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);
		}

		StartPickListValuesList();
		StartPickTableColList(arPickTableColumns);
	}

	/**
	 * Return pick list validation function
	 *
	 * @return Pick list validation function
	 */
	public String GetManagedPicklistValidationFunction()
	{
		return m_strManagedListValidation;
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns name of database table used for pick table
	 *
	 * @return Name of database table used for pick table
	 */
	public String GetPickTable()
	{
		return m_strPickTable;
	}

	/**
	 * Returns pick table types (PT_PICKTABLE_TYPES)
	 *
	 * @return Pick type
	 */
	public int GetPickType()
	{
		return m_nPickType;
	}

	/**
	 * Returns property type (PT_PROPERTY_TYPES)
	 *
	 * @return Property type
	 */
	public int GetPropertyType()
	{
		return m_nPropType;
	}

	/**
	 * Returns reference class
	 *
	 * @return Reference class
	 */
	public int GetReferenceClass()
	{
		return m_nReferenceClass;
	}

	/**
	 * Returns true if property is always a card property
	 *
	 * @return True if property is always a card property
	 */
	public boolean IsAlwaysCardProperty()
	{
		return m_bAlwaysCardProp;
	}

	/**
	 * Returns true if property is a card property
	 *
	 * @return True if property is a card property
	 */
	public boolean IsCardProperty()
	{
		return m_bCardProp || m_bAlwaysCardProp;
	}

	/**
	 * Returns true if property is mandatory
	 *
	 * @return True if property is mandatory
	 */
	public boolean IsMandatory()
	{
		return m_bMandatory;
	}

	/**
	 * Returns true if property is multi-valued
	 *
	 * @return True if property is multi-valued
	 */
	public boolean IsMultiValued()
	{
		return m_bMultiValued;
	}

	/**
	 * Returns true if property is searchable
	 *
	 * @return True if property is searchable
	 */
	public boolean IsSearchable()
	{
		return m_bSearchable;
	}

	/**
	 * Returns true if property is an URL
	 *
	 * @return True if property is an URL
	 */
	public boolean IsURL()
	{
		return m_bURL;
	}

	/**
	 * Returns true if property is visible
	 *
	 * @return True if property is visible
	 */
	public boolean IsVisible()
	{
		return m_bVisible;
	}

	/**
	 * Returns true if property is read-only
	 *
	 * @return True if property is read-only
	 */
	public boolean IsReadOnly()
	{
		return m_bReadOnly;
	}

	/**
	 * Calls the right save function according to the page name passed in
	 * argument.
	 *
	 * @param _sPageName
	 * @param _htFormData
	 *
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		int nPageStatus = super.SavePage(_sPageName, _htFormData);

		if (_sPageName.equals(MainDP.STR_MVC_CLASS_NAME))
		{
			int nMainPageStatus = SaveMainPage(_htFormData);

			if (RepostControl.PAGE_STATUS_INVALID == nMainPageStatus)
			{
				nPageStatus = RepostControl.PAGE_STATUS_INVALID;
			}

			int nPickListValueListStatus = ((PickListValuesListModel) m_asOwner.GetModel(PropEditorAS.strPickListValuesList)).SaveSection(_htFormData);

			if (RepostControl.PAGE_STATUS_INVALID == nPickListValueListStatus)
			{
				nPageStatus = RepostControl.PAGE_STATUS_INVALID;
			}

			int nPickTableColumnListStatus = ((MultiColumnListModel) m_asOwner.GetModel(PropEditorAS.strPickTableColumnList)).SaveSection(_htFormData);

			if (RepostControl.PAGE_STATUS_INVALID == nPickTableColumnListStatus)
			{
				nPageStatus = RepostControl.PAGE_STATUS_INVALID;
			}

			switch (m_nPickType)
			{
			case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_UNMANAGED_PICKLIST:
			case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_PICKTREE:
			{
				if (null == m_strPickTable)
				{
					// If this is the first time we have selected to have an unmanaged picklist/picktree
					// then the pick table will be null because we haven't set it yet. In this case
					// we don't want to validate values.
					break;
				}

				// Lazy initialization of temporary property. We only want to create temporary property if we want to validate pick table values.
				if (null == m_tempPickListProperty)
				{
					m_tempPickListProperty = (IPTProperty) m_Session.GetProperties().Create(PT_INTRINSICS.PT_ADMIN_FOLDER_UNCLASSIFIEDOBJECTS);
					m_tempPickListProperty.Initialize(m_nPropType);
				}

				m_tempPickListProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_PICKTYPE, m_nPickType);
				m_tempPickListProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_PICKTABLE, m_strPickTable);

				// Get pick columns and sort columns
				Object[][] pickTableColumns = ((PickTreeColumnsListModel) m_asOwner.GetModel(PropEditorAS.strPickTableColumnList)).GetListValues();

				m_tempPickListProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_PICKTABLECOLS, pickTableColumns);

				boolean bErrorOccurred = false;

				try
				{
					IPTPickListEditor ptPickList = m_tempPickListProperty.GetPickList();

					if (null == ptPickList)
					{
						bErrorOccurred = true;
					}
					else
					{
						// Get tree iterator from pick list
						IPTTreeIterator ptTreeIterator = (IPTTreeIterator) ptPickList.GetInterfaces("IPTTreeIterator");

						ObjPropModel.ValidateTree(ptTreeIterator, 0);
					}
				}
				catch (PTException e)
				{
					bErrorOccurred = true;
				}

				if (bErrorOccurred)
				{
					String strType;

					if (PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_UNMANAGED_PICKLIST == m_nPickType)
					{
						// Unmanaged dropdown
						strType = m_asOwner.GetString(684, "ptmsgs_portaladminmsgs");
					}
					else
					{
						// Tree
						strType = m_asOwner.GetString(683, "ptmsgs_portaladminmsgs");
					}

					String strErrorMsg = XPResourceManager.FormatMessage(m_asOwner.GetString(2671, "ptmsgs_portaladminmsgs"), strType);
					UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER, strErrorMsg, strErrorMsg);
					UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());

					nPageStatus = RepostControl.PAGE_STATUS_INVALID;
				}
			}

			break;

			default:
				break;
			}
		}

		return nPageStatus;
	}

	/**
	 * Helper method to start the growable list for pick list values list.
	 */
	public void StartPickListValuesList()
	{
		PickListValuesListModel myPickListValuesList = (PickListValuesListModel) m_asOwner.GetModel(PropEditorAS.strPickListValuesList);

		myPickListValuesList.InitializeGrowableList(m_asOwner.GetString(673, "ptmsgs_portaladminmsgs"), m_asOwner.GetString(674, "ptmsgs_portaladminmsgs"), PTImageType.ICON_VALUE, // TODO: is there an image for this?
			m_asOwner.GetString(669, "ptmsgs_portaladminmsgs"), m_nPickListValueListKey);

		//Set the property type
		myPickListValuesList.ResetPickList(m_nPropType);
		myPickListValuesList.SetEnableMoveRowActions(true);

		// Retrieve current picklist values only in EDIT mode
		if ((EditorStartControl.EDITOR_START_FLAG_EDIT == GetEditorType()) && (PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_MANAGED_PICKLIST == m_nPickType))
		{
			String[] arValues;
			IPTPickListEditor ptPickList = (IPTPickListEditor) m_ptProperty.GetPickList();
			IPTTreeIterator ptTreeIterator = (IPTTreeIterator) ptPickList.GetInterfaces("IPTTreeIterator");
			Object[][] childNodes = (Object[][]) ptTreeIterator.Children(0, -1);

			if (null != childNodes)
			{
				int numChildren = childNodes[0].length;
				arValues = new String[numChildren];

				for (int i = 0; i < numChildren; i++)
				{
					String childName = (String) childNodes[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_NAME][i];
					arValues[i] = childName;
				}

				myPickListValuesList.AddRowsToList(arValues);
			}
		}

		myPickListValuesList.SetEmptyListMessages(m_asOwner.GetString(672, "ptmsgs_portaladminmsgs"), m_asOwner.GetString(671, "ptmsgs_portaladminmsgs"));
		myPickListValuesList.SetEnableMoveRowActions(true);

		// Add validation function
		m_strManagedListValidation = ((GrowableListModel) myPickListValuesList).SetClientValidationType(GrowableListModel.CLIENT_VALIDATION_TYPE_NON_EMPTY_STRING, m_asOwner.GetString(670, "ptmsgs_portaladminmsgs"));

		MainDP myListDP = (MainDP) m_asOwner.GetPage(MainDP.STR_MVC_CLASS_NAME);
		myListDP.AddToValidationFunction(MainDP.JAVASCRIPT_VALIDATE_MANAGEDPICKLIST);
	}

	/**
	 * Helper method to start the multi-column list for pick table column list.
	 *
	 * @param arPickTableColumns Array of pick table columns
	 */
	public void StartPickTableColList(Object[][] arPickTableColumns)
	{
		PickTreeColumnsListModel myModel = (PickTreeColumnsListModel) m_asOwner.GetModel(PropEditorAS.strPickTableColumnList);
		myModel.SetNumColumns(PT_PICKTABLE_COLUMNS.PT_PICKTABLE_COLUMN_INVALID);

		// Initialize the multi column list
		String[] arPickTableColumnNames = new String[PT_PICKTABLE_COLUMNS.PT_PICKTABLE_COLUMN_INVALID];

		arPickTableColumnNames[0]     = m_asOwner.GetString(676, "ptmsgs_portaladminmsgs");
		arPickTableColumnNames[1]     = m_asOwner.GetString(678, "ptmsgs_portaladminmsgs");

		myModel.InitializeMultiColumnList(m_asOwner.GetString(677, "ptmsgs_portaladminmsgs"), arPickTableColumnNames, PTImageType.ICON_URL, m_asOwner.GetString(669, "ptmsgs_portaladminmsgs"), m_nPickTableColListKey);

		// Use the method below to set the value to populate a newly created input text.
		// (By default it will be a null array, so it MUST be initialized.
		String[] arPickTableColumnDefaults = new String[PT_PICKTABLE_COLUMNS.PT_PICKTABLE_COLUMN_INVALID];
		arPickTableColumnDefaults[0]     = "";
		arPickTableColumnDefaults[1]     = "";
		myModel.SetDefaultValue(arPickTableColumnDefaults);
		myModel.SetEmptyListMessages(m_asOwner.GetString(675, "ptmsgs_portaladminmsgs"), m_asOwner.GetString(667, "ptmsgs_portaladminmsgs"));
		myModel.SetEnableMoveRowActions(true);

		myModel.SetPickType(m_nPickType);

		//Get the name value pairs from the model in edit mode and populate them in here
		if (EditorStartControl.EDITOR_START_FLAG_EDIT == GetEditorType())
		{
			// Add pick and sort columns
			if (null != arPickTableColumns)
			{
				myModel.AddRowsToList(arPickTableColumns);
			}
		}
		else
		{
			myModel.ClearColumns();
		}
	}

	/**
	 * Saves the data on the Main page.
	 *
	 * @param _htFormData
	 *
	 * @return
	 */
	protected int SaveMainPage(XPHashtable _htFormData)
	{
		int nRetVal = RepostControl.PAGE_STATUS_VALID;

		String[] data;

		// Value type
		data = (String[]) _htFormData.GetElement(SettingsView.HTMLSELECT_PROPTYPE);

		if ((null != data) && !(0 == data.length))
		{
			int nPropType = XPConvert.ToInteger(data[0]);

			// If the property type has been changed, then reset the pick list.
			if (m_nPropType != nPropType)
			{
				m_nPropType = nPropType;

				// Update pick list values list model
				((PickListValuesListModel) m_asOwner.GetModel(PropEditorAS.strPickListValuesList)).ResetPickList(m_nPropType);
			}
		}
		else
		{
			
				log.Warn("PropEditorModel::SaveMainPage Property Type was not in Query String.");
			
		}

		// URL
		if (PT_PROPERTY_TYPES.PT_PROPTYPE_STRING == m_nPropType)
		{
			data     = (String[]) _htFormData.GetElement(SettingsView.HTMLCHECKBOX_URL);

			m_bURL = ((null != data) && !(0 == data.length));
		}

		// Reference class
		if (PT_PROPERTY_TYPES.PT_PROPTYPE_REF == m_nPropType)
		{
			data = (String[]) _htFormData.GetElement(SettingsView.HTMLSELECT_REFCLASS);

			if ((null != data) && !(0 == data.length))
			{
				m_nReferenceClass = XPConvert.ToInteger(data[0]);
			}
		}

		// Card property
		if (m_bAlwaysCardProp)
		{
			m_bCardProp = true;
		}
		else
		{
			if (PT_PROPERTY_TYPES.PT_PROPTYPE_REF != m_nPropType)
			{
				data     = (String[]) _htFormData.GetElement(SettingsView.HTMLCHECKBOX_CARDPROP);

				m_bCardProp = ((null != data) && !(0 == data.length));
			}
			else
			{
				// Card prop must be set to false, because reference types cannot be used with cards
				m_bCardProp = false;
			}
		}

		// Visible
		data     = (String[]) _htFormData.GetElement(SettingsView.HTMLCHECKBOX_VISIBLE);

		m_bVisible     = ((null != data) && !(0 == data.length));

		// Read-only
		data     = (String[]) _htFormData.GetElement(SettingsView.HTMLCHECKBOX_READONLY);

		m_bReadOnly = ((null != data) && !(0 == data.length));

		// Mandatory
		if (m_bVisible && !m_bReadOnly)
		{
			data     = (String[]) _htFormData.GetElement(SettingsView.HTMLCHECKBOX_MANDATORY);

			m_bMandatory = ((null != data) && !(0 == data.length));
		}
		else
		{
			m_bMandatory = false;
		}

		// Searchable
		if (m_bVisible)
		{
			data     = (String[]) _htFormData.GetElement(SettingsView.HTMLCHECKBOX_SEARCHABLE);

			m_bSearchable = ((null != data) && !(0 == data.length));
		}

		// Multi-valued
		data     = (String[]) _htFormData.GetElement(SettingsView.HTMLCHECKBOX_MULTIVALUED);

		m_bMultiValued = ((null != data) && !(0 == data.length));

		// Pick type
		if ((PT_PROPERTY_TYPES.PT_PROPTYPE_REF != m_nPropType) && (PT_PROPERTY_TYPES.PT_PROPTYPE_DATE != m_nPropType))
		{
			data = (String[]) _htFormData.GetElement(SettingsView.HTMLSELECT_PICKTYPE);

			if ((null != data) && !(0 == data.length))
			{
				int nNewPickType = XPConvert.ToInteger(data[0]);

				// If pick type changed, then reset the columns
				if (nNewPickType != m_nPickType)
				{
					// PickTable columns
					PickTreeColumnsListModel myModel = (PickTreeColumnsListModel) m_asOwner.GetModel(PropEditorAS.strPickTableColumnList);
					myModel.SetPickType(nNewPickType);
					myModel.ClearColumns();
				}

				m_nPickType = nNewPickType;
			}
		}

		// PickTable
		switch (m_nPickType)
		{
		case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_UNMANAGED_PICKLIST:
		case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_PICKTREE:
		{
			data = (String[]) _htFormData.GetElement(PickTableView.HTMLINPUT_PICKTABLE);

			if ((null != data) && (0 != data.length))
			{
				if ((null != data[0]) && (0 != data[0].length()))
				{
					m_strPickTable = data[0];
				}
				else
				{
					// raise error to display to user
					UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER, m_asOwner.GetString(668, "ptmsgs_portaladminmsgs"), m_asOwner.GetString(668, "ptmsgs_portaladminmsgs"));
					UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());

					return RepostControl.PAGE_STATUS_INVALID;
				}
			}

			break;
		}

		default:
			m_strPickTable = null;

			break;
		}

		return nRetVal;
	}

	/**
	 * Save managed pick list values
	 */
	private void SaveManagedPickListValues()
	{
		IPTPickListEditor ptPickList = (IPTPickListEditor) m_ptProperty.GetPickList();
		ptPickList.RemoveAll();

		String[] aGrowableArrayValues = ((PickListValuesListModel) m_asOwner.GetModel(PropEditorAS.strPickListValuesList)).GetListValues();

		// Create array of 3 columns and maxRow rows
		int maxRow = aGrowableArrayValues.length;
		Object[][] picklistActionArray = new Object[3][maxRow];

		// Add new items to action array
		for (int currentRow = 0; currentRow < maxRow; currentRow++)
		{
			String picklistValue = aGrowableArrayValues[currentRow];
			Object picklistSortOrder = new Integer(currentRow);

			picklistActionArray[PT_EDIT_PICKLIST_COLS.PT_EDIT_PICKLIST_ACTION][currentRow]		 = new Integer(PT_EDIT_PICKLIST_ACTIONS.PT_EDIT_PICKLIST_ACTION_ADD);
			picklistActionArray[PT_EDIT_PICKLIST_COLS.PT_EDIT_PICKLIST_DATA][currentRow]		 = picklistValue;
			picklistActionArray[PT_EDIT_PICKLIST_COLS.PT_EDIT_PICKLIST_SORTDATA][currentRow]     = picklistSortOrder;
		}

		ptPickList.SetItems(picklistActionArray);
	}
}

