package com.plumtree.portalpages.admin.editors.remoteserver;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;

/**
 * @author adolfoc
 *
 */
public class RemoteServerModel extends ObjEditorModel
	implements IRemoteServerModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.editors.remoteserver.RemoteServerModel");


	public static final String STR_MVC_CLASS_NAME = "RemoteServer";

	/** Localized String: Error String: You must specify a user name and password in order to use Administrator basic authentication settings */


	/** Localized String: Error String: You must specify a valid base URL starting with http:// or https:// */


	/**
	 * The following statics are used by the Section as values returned by the
	 * Authentication Type radio buttons. The HTMLRADIO_BASE_AUTH_TYPE input
	 * must return one and only one of these. The value of these is irrelevant -
	 * they merely must be unique and URL/HTML safe.
	 */
	public static final int AUTHTYPE_NONE  = 0;


	public static final int AUTHTYPE_USER  = 1;


	public static final int AUTHTYPE_ADMIN = 2;

	/**
	 * This string is used in lieu of returning the actual password to the client.
	 * If this string is unchanged - we will assume that the client has not changed
	 * the password and not reset it.
	 */
	public static final String NULL_PASSWORD = "************";

	/**
	 * The following members are used to store data that will eventually be translated
	 * and stored in the Remote Server object.
	 */
	private String m_strBaseURL = "";
	private String m_strLoginName = "";
	private String m_strPublicKey = "";

	/**
	 * This member is used in determining whether the login name has changed (if the object
	 * was edited, say). We use this in order to determine when to blank the password when
	 * the object is edited.
	 */
	private String m_strOriginalLoginName = "";
	private String m_strPassword		  = "";
	private int m_intAuthTypeRadio;

	public RemoteServerModel()
	{
		super();
		m_intAuthTypeRadio = AUTHTYPE_NONE;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new RemoteServerModel();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskBeforeStore()
	 */
	public void DoTaskBeforeStore()
	{
		super.DoTaskBeforeStore();

		IPTGadgetContentServer ptRemoteServer = (IPTGadgetContentServer) super.m_ptObject;

		// Set the Base URL to what we collected.
		ptRemoteServer.SetURL(m_strBaseURL);
		ptRemoteServer.SetPublicKey(m_strPublicKey);

		// Set the Authentication Info
		switch (m_intAuthTypeRadio)
		{
		case AUTHTYPE_USER:
			try {
				ptRemoteServer.SetPassword(null);
				// AGC- Order seems to be important. Because of the possibility of corrupted passwords
				// causing the setlogin method to fail it is preferrable to call SetLogin immediately
				// AFTER calling SetPassword.
				ptRemoteServer.SetLogin(null);

			} catch (Exception e)
			{
				
					log.Error(e, "RemoteServerModel::DoTaskBeforeStore() AUTHTYPE_USER: Could not set login or password to null - the current password may be corrupt!");
				
			}
			ptRemoteServer.SetSendUserBasicAuthentication(true);

			break;

		case AUTHTYPE_ADMIN:


			ptRemoteServer.SetLogin(m_strLoginName);

			// Make sure we don't overwrite any existing password if it (and the Login Name)
			// wasn't changed.
			// AGC- This logic may reset the password to a bogus NULL_PASSWORD if
			// the login name is changed (but not the password). This is probably
			// a good idea - to avoid throwing passwords around that don't match
			// the username specified. If both are changed, this should store correctly.
			if ((0 != NULL_PASSWORD.compareTo(m_strPassword)) ||
					(0 != m_strLoginName.compareTo(m_strOriginalLoginName)))
			{
				ptRemoteServer.SetPassword(m_strPassword);
			}

			ptRemoteServer.SetSendUserBasicAuthentication(false);


			break;

		default: //AUTHTYPE_NONE
			try {

				ptRemoteServer.SetPassword(null);
				// AGC- Order seems to be important. Because of the possibility of corrupted passwords
				// causing the setlogin method to fail it is preferrable to call SetLogin immediately
				// AFTER calling SetPassword.
				ptRemoteServer.SetLogin(null);

			} catch (Exception e)
			{
				
					log.Error(e, "RemoteServerModel::DoTaskBeforeStore() AUTHTYPE_NONE: Could not set login or password to null - the current password may be corrupt!");
				
			}
			ptRemoteServer.SetSendUserBasicAuthentication(false);
		}

		// If this object is ever accessed again before this model goes away, we want to
		// make sure that the original login name is updated correctly.
		m_strOriginalLoginName = ptRemoteServer.GetLogin();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskOnStartEditor(_htFormData)
	 */
	public void DoTaskOnStartEditor(XPHashtable _htFormData)
	{
		super.DoTaskOnStartEditor(_htFormData);

		if (EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType())
		{
			m_strBaseURL			   = "";
			m_strLoginName			   = "";
			m_strOriginalLoginName     = "";
			m_strPassword			   = "";
			m_intAuthTypeRadio		   = AUTHTYPE_NONE;

			// AGC- In create mode, there will be no objects associated with the
			// newly created object (duh). So there's no reason to show the associated
			// objects page.
			((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
				AssociatedObjectsDP.STR_MVC_CLASS_NAME);

			// Hide migration status in Create mode
			((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
				MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);
		}
		else
		{
			IPTGadgetContentServer ptRemoteServer = (IPTGadgetContentServer) super.m_ptObject;
			m_strBaseURL     = ptRemoteServer.GetURL();

			m_strLoginName			   = ptRemoteServer.GetLogin();
			m_strOriginalLoginName     = m_strLoginName;
			m_strPassword			   = NULL_PASSWORD;
			m_strPublicKey = ptRemoteServer.GetPublicKey();

			if (ptRemoteServer.GetSendUserBasicAuthentication())
			{
				m_strLoginName			   = "";
				m_strOriginalLoginName     = "";
				m_intAuthTypeRadio		   = AUTHTYPE_USER;
			}
			else if (0 == m_strLoginName.length())
			{
				m_intAuthTypeRadio = AUTHTYPE_NONE;
			}
			else
			{
				m_intAuthTypeRadio = AUTHTYPE_ADMIN;
			}
		}
	}

	/**
	 *
	 *
	 * @return
	 */
	public IASQueryResult GetAssocWebServices()
	{
		if (EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType())
		{
			return null;
		}
		else
		{
			IPTSession ptSession			 = GetPTSession();
			IPTObjectManager ptWebServices   = (IPTObjectManager) ptSession.GetWebServices();
			PTObjectManagerQueryWrapper omqr = new PTObjectManagerQueryWrapper(ptWebServices);

			// Query filter
			Object[][] ptqf = new Object[3][1];
			ptqf[0][0]     = new Integer(PT_PROPIDS.PT_PROPID_WEBSERVICE_REMOTESERVERID);
			ptqf[1][0]     = new Integer(PT_FILTEROPS.PT_FILTEROP_EQ);
			ptqf[2][0]     = new Integer(m_ptObject.GetObjectID());
			omqr.Query(-1, -1, null, ptqf);

			return omqr;
		}
	}

	/**
	 * Returns the strAuthTypeRadio.
	 * @return String
	 */
	public int GetAuthType()
	{
		return m_intAuthTypeRadio;
	}

	/**
	 * Returns the strBaseURL.
	 * @return String
	 */
	public String GetBaseURL()
	{
		return m_strBaseURL;
	}

	/**
	 * Returns the strLoginName.
	 * Note that there is no getter for the Password (it is never
	 * sent back to the client).
	 * @return String
	 */
	public String GetLoginName()
	{
		return m_strLoginName;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostModel#SavePage(String, XPHashtable)
	 *
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		int nPageStatus = super.SavePage(_sPageName, _htFormData);

		if (_sPageName.equals(MainDP.STR_MVC_CLASS_NAME))
		{
			SetBaseURL(RSModelHelper.GetFormElementString(_htFormData,
					RemoteServerPropsView.HTMLINPUT_BASE_URL, m_strBaseURL));
			SetLoginName(RSModelHelper.GetFormElementString(_htFormData,
					RemoteServerPropsView.HTMLINPUT_LOGIN_NAME, m_strLoginName));
			SetAuthType(RSModelHelper.GetFormElementInt(_htFormData,
					RemoteServerPropsView.HTMLRADIO_BASE_AUTH_TYPE,
					m_intAuthTypeRadio));
			m_strPublicKey = RSModelHelper.GetFormElementString(_htFormData, RemoteServerPropsView.HTMLTEXTAREA_PUBLICKEY, m_strPublicKey);

			// Validation

			String password=RSModelHelper.GetFormElementString(_htFormData,RemoteServerPropsView.HTMLINPUT_PASSWORD, m_strPassword);
			String passconf=RSModelHelper.GetFormElementString(_htFormData,RemoteServerPropsView.HTMLINPUT_PASSWORD_CONFIRM, m_strPassword);
			if(password==null || passconf==null || !password.equals(passconf)) {
				nPageStatus=RemoteServerRepostControl.PAGE_STATUS_INVALID;
				if(log.IsWarnEnabled())
					log.Warn("WSModelHelper::ValidateNotEmptyString: Null or empty string found. ");
				String strError=m_asOwner.GetString(366, "ptmsgs_portaladminmsgs");
				UIError myError = new UIError(UIErrorLevel.FRIENDLY,UIErrorScope.CURRENT_USER, strError, strError);
				UIErrorHandler.RaiseError(myError,m_asOwner.GetSubSession(),m_asOwner.GetApplication());
			} else
				SetPassword(password);

			if (AUTHTYPE_ADMIN == m_intAuthTypeRadio)
			{
				nPageStatus = RSModelHelper.ValidateLoginPassword(m_strLoginName, m_strPassword, NULL_PASSWORD,
						m_asOwner,
						m_asOwner.GetString(732, "ptmsgs_portaladminmsgs"),
						nPageStatus);
			}

			nPageStatus = RSModelHelper.ValidateIsURLString(m_strBaseURL,
					m_asOwner,
					m_asOwner.GetString(733, "ptmsgs_portaladminmsgs"),
					nPageStatus);
		}

		return nPageStatus;
	}

	/**
	 * Sets the strAuthTypeRadio.
	 *
	 *
	 * @param strAuthTypeRadio The strAuthTypeRadio to set
	 */
	public void SetAuthType(int intAuthTypeRadio)
	{
		// We must receive one of the Radio value strings above (AUTHTYPE_NONE, ...USER or ...ADMIN).
		// These will determine what gets stored in the Remote Server object.
		switch (intAuthTypeRadio)
		{
		case AUTHTYPE_NONE:
		case AUTHTYPE_USER:
		case AUTHTYPE_ADMIN:
			this.m_intAuthTypeRadio = intAuthTypeRadio;

			break;

		default:

			
				log.Error("RemoteServerModel::SetAuthType(" + intAuthTypeRadio +
					") an invalid AuthType was received. The Authtype will be set to NONE.");
			
		}
	}

	/**
	 * Sets the strBaseURL.
	 * @param strBaseURL The strBaseURL to set
	 */
	public void SetBaseURL(String strBaseURL)
	{
		// Validate that the Base URL ends in a slash (silent correction).
		strBaseURL = strBaseURL.trim();

		if (strBaseURL.endsWith("\\"))
		{
			strBaseURL = strBaseURL.substring(0, strBaseURL.length() - 1) +
				"/";
		}
		else if (!strBaseURL.endsWith("/"))
		{
			strBaseURL += "/";
		}

		m_strBaseURL = strBaseURL;
	}

	/**
	 * Sets the strLoginName.
	 * @param strLoginName The strLoginName to set
	 */
	public void SetLoginName(String strLoginName)
	{
		m_strLoginName = strLoginName;
	}

	/**
	 * Sets the strPassword.
	 * Note that there is no getter for the Password.
	 * @param strPassword The strPassword to set
	 */
	public void SetPassword(String strPassword)
	{
		if (XPStringUtility.Compare(NULL_PASSWORD, strPassword) != 0)
		{
			m_strPassword = strPassword;
		}
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portalpages.admin.editors.remoteserver.IRemoteServerModelRO#GetPublicKey()
	 */
	public String GetPublicKey() {
		return m_strPublicKey;
	}

}

