package com.plumtree.portalpages.admin.editors.subportal;


// Framework Imports
import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.activityspace.*;

/**
 * This interface is implemented by SubPortalModel.
 * It contains all the read-only mothods of SubPortalModel.
 * The Views only have access to the interface.
 *
 * @author Julie Fournier
 */
public interface ISubPortalModelRO extends IObjEditorModelRO
{
	/**
	 * Returns the home Community ID.
	 * @return int
	 */
	public int GetHomeCommunityID();

	/**
	 * Returns the home Community Name.
	 * @return String
	 */
	public String GetHomeCommunityName();

	/**
	 * Returns the home Folder ID.
	 * @return int
	 */
	public int GetHomeFolderID();

	/**
	 * Returns the home Folder Name.
	 * @return String
	 */
	public String GetHomeFolderName();

	/**
	 * Return the ASURL object going to the Community Editor as a sub Editor.
	 *
	 * @param _sSubSpaceKey Key for the sub Editor.
	 * If we are going to a new sub Editor, the key can be null and
	 * we will just pass in the default key. If we are going back to an
	 * existing sub Editor, it should be the sub Editor key.
	 * @return ASURL
	 */
	public ASURL GetASURLToCommunity(String _sSubSpaceKey);

	/**
	 * Return the ASURL object going to the Folder Editor as a sub Editor.
	 *
	 * @param _sSubSpaceKey Key for the sub Editor.
	 * If we are going to a new sub Editor, the key can be null and
	 * we will just pass in the default key. If we are going back to an
	 * existing sub Editor, it should be the sub Editor key.
	 * @return ASURL
	 */
	public ASURL GetASURLToFolder(String _sSubSpaceKey);

	/**
	 * Returns the The type of subportal access (MyPages, Communities, Knowledge Directory) supported
	 * by this subportal.  Valid values are any mask combination of PT_SUBPORTAL_ACCESSTYPES
	 * Defaults to communities
	 * @return int
	 */
	public int GetAccessType();

	/**
	 * Returns the subspace ID used by the tree control.  This is used to ensure
	 * that trees that already exist in memory are reused instead of recreated.
	 *
	 * @return String
	 */
	public String GetCommSubSpaceID();

	/**
	 * Returns the first page ID based on the page type specifiede.  If the page type is
	 * PT_SUBPORTAL_COMMUNITIES_ACCESS, then the first page ID will be the Community ID that
	 * is set to appear as default.  An analogous result will occur with Directory.  In the
	 * case MyPages, this function will throw an error, because the Users will always be
	 * referred to their main MyPage
	 * @return int
	 */
	public int GetDefaultFirstPage();

	/**
	 * Returns which page the users will see upon logging in to the subportal: a MyPage, a
	 * Community, or a Directory page.  Valid values are enumerated in PT_SUBPORTAL_ACCESS_TYPES.
	 * @return int
	 */
	public int GetDefaultFirstPageType();

	/**
	 * Returns the subspace ID used by the Folder tree control.  This is used to ensure
	 * that trees that already exist in memory are reused instead of recreated.
	 *
	 * @return String
	 */
	public String GetFolderSubSpaceID();

	/**
	 * Returns the Footer ID.  If the specified Footer ID doesn't exist, the UI will revert to
	 * the default banner, which cannot be deleted.
	 *
	 * @return int
	 */
	public int GetFooterID();

	/**
	 * Returns the header ID.  If the specified Header ID doesn't exist, the UI will revert to
	 * the default banner, which cannot be deleted.
	 *
	 * @return int
	 */
	public int GetHeaderID();

	/**
	 * Returns the Stylesheet name.
	 *
	 * @return String
	 */
	public String GetStyleSheet();

	/**
	 * Returns the available Stylesheet names.
	 *
	 * @return IXPDictionary
	 */
	public IXPDictionary GetStyleSheetOptions();

	/**
	 * Returns true if the subportal has a PTGuestLogin associated to it, 
	 * or false otherwise.
	 * 
	 * @return boolean
	 */
	public boolean GetIsNewGuestLogin();
	
	/**
	 * Returns the Guest Username.
	 *
	 * @return String
	 */
	public String GetGuestUserName();

	/**
	 * Returns the Guest User ID.
	 *
	 * @return int
	 */
	public int GetGuestUserID();
	
	/**
	 * Returns the subspace ID used by the guest tree.  This is used to ensure
	 * that trees that already exist in memory are reused instead of recreated.
	 *
	 * @return String
	 */
	public String GetGuestTreeSubSpaceID();
	
	/**
	 * Returns the Guest Redirect To Login setting.
	 *
	 * @return boolean
	 */
	public boolean GetGuestRedirectToLogin();

	/**
	 * Returns the Redirect On Logout setting.
	 *
	 * @return boolean
	 */
	public boolean GetRedirectOnLogout();
	
	/**
	 * Returns the SSO Vendor setting of the subportal if it exists or the
	 * default SSO Vendor setting from PTConfig.xml if the subportal has 
	 * no SSO Vendor setting set.
	 *
	 * @return int
	 */
	public int GetSSOVendor();

	/**
	 * Return the default SSO Vendor setting from PTConfig.xml.
	 *
	 * @return int
	 */
	public int GetDefaultSSOVendor();
	
	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName();

	/**
	 * Returns the Navigation scheme for the subportal.  Valid values are enumerated in
	 * PT_NAVIGATION_SCHEMES.
	 *
	 * @return int
	 */
	public int GetNavigationScheme();

	/**
	 * Returns what related resources are accessible to users of this subportal.  The value
	 * is any mask combination of PT_RELATED_RESOURCES_ACCESS_TYPES.
	 * @return int
	 */
	public int GetRelatedResourcesAccessTypes();

	/**
	 * @return whether or not layouts are enabled for this subportal.
	 */
	public boolean GetLayoutEnabled();
	
	/**
	 * 
	 * @return The list of Layout Options selected for this subportal.
	 */
	public String GetLayoutOptions();

	/**
	 * Returns whether a newly created folder resides in the unclassified area
	 * of the object admin hierarchy
	 *
	 * @return boolean
	 */
	public boolean GetFolderUnclassified();
	
	/**
	 * Return the ASURL object going to the Admin Folder Editor as a sub Editor.
	 *
	 * @param _sSubSpaceKey Key for the sub Editor. If we are going to a new
	 *        sub Editor, the key can be null and  we will just pass in the
	 *        default key. If we are going back to an  existing sub Editor, it
	 *        should be the sub Editor key.
	 *
	 * @return ASURL
	 */
	public ASURL GetASURLToAdminFolder(String _sSubSpaceKey);
}