package com.plumtree.portalpages.admin.editors.webservice.plugins;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.portalpages.admin.editors.webservice.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.providerframework.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.form.*;

/**
 * Store the activity space for the intrinsic portlet preference pages.
 * Even though there is only one prefs plugin model and one prefs plugin
 * view, there are actually three instances of this model in existence.
 * Each of these models will contain information about one preference type,
 * whether it be admin portlet prefs, user portlet prefs, or community
 * portlet prefs.
 *
 * @author josephc
 */
public class IntrinsicUserPrefsPluginModel extends EditorHelperModel implements IIntrinsicUserPrefsPluginModelRO, IWSSubModel
{
	/** Class name for IntrinsicUserPrefsPluginModel */
	public static final String STR_MVC_CLASS_NAME = "IntrinsicUserPrefsPlugin";

	/** User portlet preferences */
	public static final int UPPM_PREFMODE_USERPREFS = 0;

	/** Community portlet preferences */
	public static final int UPPM_PREFMODE_COMMUNITYPREFS = 1;

	/** Admin portlet preferences */
	public static final int UPPM_PREFMODE_ADMINPREFS = 2;

	/** The Web Service object being edited by our parent Web Service editor */
	IPTWebService m_ptWebService = null;

	/** Prefs activity space */
	private String m_strPrefsSpace = "";

	/** Admin prefs template activity space */
	private String m_strTemplatePrefsSpace = "";

	/** Preference mode */
	private int m_nPrefMode = 0;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new IntrinsicUserPrefsPluginModel();
	}

	/**
	 * 
	 *
	 * @param nSupportsPersonalization 
	 * @param strPrefsSpace 
	 * @param nPortletPersonalizationRole 
	 *
	 * @return 
	 */
	private int UpdatePersonalizationFlag(int nSupportsPersonalization, String strPrefsSpace, int nPortletPersonalizationRole)
	{
		if ((null == strPrefsSpace) || (0 >= strPrefsSpace.trim().length()))
		{
			nSupportsPersonalization &= ~nPortletPersonalizationRole;
		}
		else
		{
			nSupportsPersonalization |= nPortletPersonalizationRole;
		}

		return nSupportsPersonalization;
	}

	/**
	 * This method updates the personalization flag of the web service. This model
	 * only represents one type of preference, either user prefs, community prefs,
	 * or admin prefs.  So we just need to check if the text box has a value, then
	 * OR the value so that the flag is set.  If the text box does not have a valid
	 * value, then AND the value so the flag is unset.
	 *
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskBeforeWSStore()
	 */
	public void DoTaskBeforeWSStore()
	{
		IXPPropertyBag ptProviderInfo = (IXPPropertyBag) m_ptWebService.GetProviderInfo();
		String strPrefsSpace = GetPrefsSpace();
		int nSupportsPersonalization = m_ptWebService.GetSupportsPersonalization();

		// Determine which type of preference our model is holding info about
		switch (m_nPrefMode)
		{
		case UPPM_PREFMODE_USERPREFS:
			nSupportsPersonalization = UpdatePersonalizationFlag(nSupportsPersonalization, strPrefsSpace, PT_GADGETPERSONALIZATION_ROLES.PT_GADGETPERSONALIZATION_ENDUSER);
			WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_PREFS_SPACE, strPrefsSpace);

			break;

		case UPPM_PREFMODE_COMMUNITYPREFS:
			nSupportsPersonalization = UpdatePersonalizationFlag(nSupportsPersonalization, strPrefsSpace, PT_GADGETPERSONALIZATION_ROLES.PT_GADGETPERSONALIZATION_COMMUNITYOWNER);
			WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_COMMUNITYPREFS_SPACE, strPrefsSpace);

			break;

		case UPPM_PREFMODE_ADMINPREFS:
			nSupportsPersonalization = UpdatePersonalizationFlag(nSupportsPersonalization, strPrefsSpace, PT_GADGETPERSONALIZATION_ROLES.PT_GADGETPERSONALIZATION_ADMINISTRATOR);
			WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_ADMINPREFS_SPACE, strPrefsSpace);

			nSupportsPersonalization = UpdatePersonalizationFlag(nSupportsPersonalization, GetTemplatePrefsSpace(), PT_GADGETPERSONALIZATION_ROLES.PT_GADGETPERSONALIZATION_ADMINTEMPLATE);
			WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_ADMINPREFS_TEMPLT_SPACE, GetTemplatePrefsSpace());

			break;
		}

		m_ptWebService.SetSupportsPersonalization(nSupportsPersonalization);
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskOnRemoteServerUpdate(String,
	 *      String, String)
	 */
	public void DoTaskOnRemoteServerUpdate(String strRemoteServerID, String strRemoteServerName, String strRemoteServerBaseURL)
	{
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskOnWSModeUpdate(int)
	 */
	public void DoTaskOnWSModeUpdate(int nNewWSMode)
	{
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * This method checks whether this model is used to set community
	 * preferences or user preferences. SetPrefMode can be used to set this
	 * when the model is initialized. GetPrefMode can be used to get which
	 * mode this model is in.
	 *
	 * @return Preference mode
	 */
	public int GetPrefMode()
	{
		return m_nPrefMode;
	}

	/**
	 * Gets the preference space value for the current type of preference.
	 *
	 * @return Preference space
	 *
	 * @see com.plumtree.portalpages.admin.editors.webservice.plugins.IIntrinsicUserPrefsPluginModelRO#GetPrefsSpace()
	 */
	public String GetPrefsSpace()
	{
		return m_strPrefsSpace;
	}

	/**
	 * Gets the preference space value for portlet template admin preferences.
	 *
	 * @return Portlet template admin preference space
	 *
	 * @see com.plumtree.portalpages.admin.editors.webservice.plugins.IIntrinsicUserPrefsPluginModelRO#GetTemplatePrefsSpace()
	 */
	public String GetTemplatePrefsSpace()
	{
		return m_strTemplatePrefsSpace;
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#GetValidationFunction()
	 */
	public String GetValidationFunction()
	{
		return null;
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#InitWSModel()
	 */
	public void InitWSModel(IPTWebService ptWebService)
	{
		m_ptWebService = ptWebService;

		IXPPropertyBag ptProviderInfo = (IXPPropertyBag) ptWebService.GetProviderInfo();

		switch (m_nPrefMode)
		{
		case UPPM_PREFMODE_USERPREFS:
			SetPrefsSpace(WSModelHelper.PBReadString(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_PREFS_SPACE, m_strPrefsSpace));

			break;

		case UPPM_PREFMODE_COMMUNITYPREFS:
			SetPrefsSpace(WSModelHelper.PBReadString(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_COMMUNITYPREFS_SPACE, m_strPrefsSpace));

			break;

		case UPPM_PREFMODE_ADMINPREFS:
			SetPrefsSpace(WSModelHelper.PBReadString(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_ADMINPREFS_SPACE, m_strPrefsSpace));
			SetTemplatePrefsSpace(WSModelHelper.PBReadString(ptProviderInfo, AExtensibilityConstants.PTC_I_GADGET_ADMINPREFS_TEMPLT_SPACE, m_strTemplatePrefsSpace));

			break;
		}
	}

	/**
	 * Verify that the specified activity space exists and has been loaded by
	 * dynamic discovery.
	 *
	 * @param strSpaceName Activity space name
	 * @param strErrorMsg Error message to display for invalid space
	 *
	 * @return Repost control flag
	 */
	private int ValidatePrefsSpace(String strSpaceName, String strErrorMsg)
	{
		if ((null == strSpaceName) || (0 == strSpaceName.length()))
		{
			// The space can be empty or null if the portlet doesn't support that type of pref.
			return RepostControl.PAGE_STATUS_VALID;
		}
		else
		{
			// If the space cannot be retrieved using the portlet class identifier, then
			// it must be invalid.
			if (!ASManager.INSTANCE.GetSpaceExists(strSpaceName))
			{
				UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER, strErrorMsg, strErrorMsg);
				UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());

				return RepostControl.PAGE_STATUS_INVALID;
			}

			return RepostControl.PAGE_STATUS_VALID;
		}
	}

	/**
	 * Saves the preference spaces.
	 *
	 * @see com.plumtree.uiinfrastructure.editor.EditorHelperModel#SaveSection(XPHashtable)
	 */
	public int SaveSection(XPHashtable _htFormData)
	{
		int nStatus = RepostControl.PAGE_STATUS_VALID;
		String strErrorMsg;

		switch (m_nPrefMode)
		{
		case UPPM_PREFMODE_USERPREFS:
			SetPrefsSpace(WSModelHelper.GetFormElementString(_htFormData, IntrinsicUserPrefsPluginView.HTMLINPUT_USER_PREFS_SPACE, GetPrefsSpace()));
			strErrorMsg = XPResourceManager.FormatMessage(m_asOwner.GetString(2386, "ptmsgs_portaladminmsgs"), GetPrefsSpace());
			nStatus = ValidatePrefsSpace(GetPrefsSpace(), strErrorMsg);

			break;

		case UPPM_PREFMODE_COMMUNITYPREFS:
			SetPrefsSpace(WSModelHelper.GetFormElementString(_htFormData, IntrinsicUserPrefsPluginView.HTMLINPUT_COMMUNITY_PREFS_SPACE, GetPrefsSpace()));
			strErrorMsg = XPResourceManager.FormatMessage(m_asOwner.GetString(2387, "ptmsgs_portaladminmsgs"), GetPrefsSpace());
			nStatus = ValidatePrefsSpace(GetPrefsSpace(), strErrorMsg);

			break;

		case UPPM_PREFMODE_ADMINPREFS:
			SetPrefsSpace(WSModelHelper.GetFormElementString(_htFormData, IntrinsicUserPrefsPluginView.HTMLINPUT_ADMIN_PREFS_SPACE, GetPrefsSpace()));
			SetTemplatePrefsSpace(WSModelHelper.GetFormElementString(_htFormData, IntrinsicUserPrefsPluginView.HTMLINPUT_TEMPLT_ADMIN_PREFS_SPACE, GetTemplatePrefsSpace()));

			strErrorMsg = XPResourceManager.FormatMessage(m_asOwner.GetString(2384, "ptmsgs_portaladminmsgs"), GetPrefsSpace());
			nStatus = ValidatePrefsSpace(GetPrefsSpace(), strErrorMsg);

			// Validate template space only if prefs space is valid.			
			if (nStatus == RepostControl.PAGE_STATUS_VALID)
			{
				strErrorMsg     = XPResourceManager.FormatMessage(m_asOwner.GetString(2385, "ptmsgs_portaladminmsgs"), GetTemplatePrefsSpace());
				nStatus		    = ValidatePrefsSpace(GetTemplatePrefsSpace(), strErrorMsg);
			}

			break;
		}

		return nStatus;
	}

	/**
	 * This method specifies whether this model is used to set community
	 * preferences or user preferences. SetPrefMode can be used to set this
	 * when the model is initialized. GetPrefMode can be used to get which
	 * mode this model is in.
	 *
	 * @param nPrefMode Preference mode
	 */
	public void SetPrefMode(int nPrefMode)
	{
		m_nPrefMode = nPrefMode;
	}

	/**
	 * Sets the prefs space
	 *
	 * @param strPrefsSpace Activity space for preferences
	 */
	public void SetPrefsSpace(String strPrefsSpace)
	{
		m_strPrefsSpace = strPrefsSpace;
	}

	/**
	 * Sets the Template Admin Prefs space
	 *
	 * @param strTemplatePrefsSpace Activity space for admin template
	 *        preferences
	 */
	public void SetTemplatePrefsSpace(String strTemplatePrefsSpace)
	{
		m_strTemplatePrefsSpace = strTemplatePrefsSpace;
	}
}

