package com.plumtree.portalpages.admin.editors.webservice.userinfotree;


import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.editors.webservice.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.treeiterator.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.tree.*;

/**
 * This version of the tree allows for browsing and
 * selecting of objects with no hierarchical structure
 *
 * @author AdolfoC
 */
public class UserInfoTreeModel extends ATreeModel implements ITreeModel,
	ITreeModelRO, IRepostModel
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.editors.webservice.userinfotree.UserInfoTreeModel");

	/** object name */
	public static final String STR_MVC_CLASS_NAME = "UserInfoTree";

	/** Tree iterator */
	protected IPTTreeIterator m_ptIterator = null;

	/**
	 * Storage for the list of all known user infos. This is used in order to
	 * get the UserInfo string based on the index into this array (the index is
	 * used in place of an object ID to uniquely identify each User Info).
	 */
	protected XPArrayList m_xpalKnownUserInfos;
	protected XPArrayList m_xpalSelected;

	public UserInfoTreeModel()
	{
		super();
		m_xpalKnownUserInfos = new XPArrayList();
		m_xpalSelected = new XPArrayList();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new UserInfoTreeModel();
	}

	/**
	 * This is called when a node is expanded.  Since User Info
	 * is a flat list, there will be no recursive thoughts here.  This simply
	 * retrieves all the User Info available to the user and puts it on
	 * the tree as leaf items.
	 *
	 * @param bExpand expands the branch
	 * @param nNodeID the uniqueid for this node
	 */
	public void ExpandNode(boolean bExpand, int nNodeID)
	{
		super.ExpandNode(bExpand, nNodeID);

		// Always expand the dummy title to show all rights
		TreeCompositeNode node = (TreeCompositeNode) m_Root.GetNode(nNodeID);
		node.m_bExpanded = true;

		// Populate with User Info
		try
		{
			IPTProfileManager ptProfileManager = (IPTProfileManager) ((IPTSession) m_asOwner.GetUserSession()).OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_PROFILE_MANAGER,
					false);

			Object[][] arrMap;
			arrMap = (Object[][]) ptProfileManager.GetUserInfoMap();

			TreeLeafNode myLeaf;
			int nNumUserInfos = 0;

			if (null != arrMap)
			{
				nNumUserInfos = arrMap[0].length;
			}

			// A stored array of User infos. We need this in order to be able to 
			// uniquely identify each UserInfo with an integer (rather than re-write the whole
			// damn Tree to support indexing by string).
			m_xpalKnownUserInfos = new XPArrayList();

			for (int nMapIndex = 0; nMapIndex < nNumUserInfos; nMapIndex++)
			{
				String strCurrentUserInfo = (String) arrMap[1][nMapIndex];
				
				if (!WSModelHelper.IsHardcodedUserInfo(strCurrentUserInfo))
				{
					// AGC- we need to keep track of the mapped items so we can do
					// selection properly.	        	
					int nCurrentUserInfoIndex = m_xpalKnownUserInfos.Add(strCurrentUserInfo);
					myLeaf				   = new TreeLeafNode(GetNextUniqueID());
					myLeaf.SetName(strCurrentUserInfo);
					myLeaf.m_nObjectID     = nCurrentUserInfoIndex; //AGC- the most unique ID we can find is the index into the stored array.
					myLeaf.m_nClassID	   = -1; //AGC- Not Used.

					// TODOCR: donh myLeaf.GetName() returns the name truncated to 50,
					// and myLeaf.GetEncodedName() returns it truncated and HTMLEncoded
					// The data is stored in the encoded format, so GetEncodedName
					// is faster than GetName().  The truncation may cause problems
					// here.
					if (m_xpalSelected.Contains(myLeaf.GetName()))
					{
						myLeaf.m_bSelected = true;
						m_xpmSelectedItems.PutElement(myLeaf.m_nClassID + "," +
								myLeaf.m_nObjectID, myLeaf);
					}
	
					node.AddLeafChild(myLeaf);
				}
			}
		}
		catch (Exception ex)
		{
			
				log.Error("UserInfoTreeModel::Expand Node Unable to load User Info tree.  The following exception occured: " +
					ex);
			

			throw new XPException(m_asOwner.GetString(1938, "ptmsgs_portaladminmsgs"), ex);
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the list of UserInfo strings that the Tree knows about (is
	 * displaying).
	 *
	 * @return XPArrayList
	 */
	public XPArrayList GetTreeUserInfos()
	{
		return m_xpalKnownUserInfos;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#Init(AActivitySpace)
	 */
	public void Init(AActivitySpace parent)
	{
		super.Init(parent);
	}

	/**
	 * Initializes and begins the loading of the tree.
	 */
	public void LoadUserInfo()
	{
		// Root is simply a dummy title
		m_Root				 = new TreeCompositeNode(GetNextUniqueID());
		m_Root.SetName(m_asOwner.GetString(1939, "ptmsgs_portaladminmsgs"));

		if (!m_bShowRoot)
		{
			ExpandNode(true, m_Root.m_nTreeNodeID);
		}
	}

	/**
	 * If there are multiple class ids, determine which
	 * ones to display.  For User Info, we only
	 * have 1 class id, so this method is unimplemented.
	 *
	 * @param nIDs determines which classids to display
	 */
	public void SetClassIDsToDisplay(int[] nIDs)
	{
		// User Info do not need this currently.
	}

	/**
	 * Determines what items in the tree are to be displayed.
	 *
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModel#SetObjectIDsToDisplay(int[])
	 *
	 * @param nIDs determines which nodes are not hidden.
	 */
	public void SetObjectIDsToDisplay(int[] nIDs)
	{
	}

	/**
	 * Determines what items in the tree are hidden.
	 *
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModel#SetObjectIDsToHide(int[])
	 *
	 * @param nIDs determines which nodes are hidden
	 */
	public void SetObjectIDsToHide(int[] nIDs)
	{
		// User Info do not need this currently.
	}

	/**
	 * Determines what items in the tree are selected.
	 *
	 * @param nIDs determines which nodes are selected
	 */
	public void SetObjectIDsToSelect(int[] nIDs)
	{
		// AGC- Not used by User Info
	}

	/**
	 * Sets the userinfos that are currently selected.
	 *
	 * @param arsUserInfos
	 */
	public void SetUserInfosToSelect(XPArrayList xpalUserInfos)
	{
		ClearSelectedItems();
		m_xpalSelected = xpalUserInfos;
	}

	/**
	 * Sets the userinfos that are currently selected.
	 *
	 * @param arsUserInfos
	 */
	public void SetUserInfosToSelect(String[] arsUserInfos)
	{
		SetUserInfosToSelect(WSModelHelper.StringArrayToXPArray(arsUserInfos));
	}

}

