package com.plumtree.portalpages.admin.objmgr;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.objmgr.popups.*;
import com.plumtree.portalpages.admin.ptutilities.*;
import com.plumtree.portalpages.admin.serversettings.*;
import com.plumtree.portalpages.common.createfolder.*;
import com.plumtree.portalpages.common.folder.*;
import com.plumtree.portalpages.common.objects.*;
import com.plumtree.portalpages.common.uiparts.*;
import com.plumtree.portaluiinfrastructure.application.varpacks.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.form.pagination.*;
import com.plumtree.uiinfrastructure.form.sort.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author Don Hayler
 *
 * This implements the admin directory.
 */
public class ObjMgrAS extends PlumtreeAS
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.objmgr.ObjMgrAS");

	/** sub-space constants */
	public static final String FOLDER_TREE_SUB_SPACE_KEY = "folder";

	/** object name */
	public static final String STR_MVC_CLASS_NAME = "ObjMgr";

	/** The maximum length of names of objects when displayed. */
	public static final int MAX_DISPLAYED_NAME_LENGTH = 50;

	
	public static final String JS_PARENT_SIMPLE_REPOST = "window.opener." +
		AFormDP.JAVASCRIPT_SUBMIT_FORM + "(" +
		RepostControl.POSTTOSELF_ACTION_SIMPLEREPOST + ");\n";

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.AActivitySpace#CheckBasicAccess(String, String, boolean)
	 */
	public boolean CheckBasicAccess(String strPage, String strControl,
		boolean bSameSpace)
	{
		// Do not check server configuration if user is getting an admin redirect,
		// otherwise users will not be able to redirect from a browsing server to an
		// admin server.
		if (!AdminRedirectControl.STR_MVC_CLASS_NAME.equals(strControl))
		{
			// Check if this server is configured to be used as an administration machine.
			// 0 means browsing, 1 means admin
			if (PTConfigHelper.GetConfigVariableAsInt(PTConfigConstants.ADMINISTRATION_SECTION,
				PTConfigConstants.IS_ADMIN_SITE_SUBSECTION, PTConfigConstants.VALUE_ATTRIBUTE, this) != 1)
			{
				
					log.Warn("Attempt to access administration on browsing only portal denied. Check the IsAdminSite " +
						"value in the Administration section of PTconfig.xml if this is believed to be a configuration issue.");
				
	
				return false;
			}
		}

		// DCA - call super to register nav views;
		boolean bAccessAllowed = super.CheckBasicAccess(strPage, strControl, bSameSpace);

		// Check for right to access administration
		bAccessAllowed = bAccessAllowed && ((IPTSession) GetUserSession()).GetSessionInfo()
			.CheckForActivityRight(
			PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_ACCESS_ADMINISTRATION);
		
		return bAccessAllowed;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new ObjMgrAS();
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.AActivitySpace#GetCurrentPage()
	 */
	public IDisplayPage GetCurrentPage()
	{
		boolean bRoot = ((IFolderModelRO) GetModel(ObjMgrModel.STR_MVC_CLASS_NAME)).InRootFolder();

		if (bRoot)
		{
			return GetPage(ObjMgrRootDP.STR_MVC_CLASS_NAME);
		}
		else
		{
			return GetPage(ObjMgrDP.STR_MVC_CLASS_NAME);
		}
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.AActivitySpace#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the name of the repost control class.
	 *
	 * @see com.plumtree.xpshared.form.IRepostAS#GetRepostControlName()
	 *
	 * @return String
	 */
	public String GetRepostControlName()
	{
		return ObjMgrRepostControl.STR_MVC_CLASS_NAME;
	}

	/**
	 * This method handles the ClosePopUpDP and passes everything else to the base
	 * implementation.  It adds javascript to the ClosePopUpDP that sets the selected
	 * items on the opener window (if it's a remote tree) and calls a javascript
	 * function on the opener window.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.AActivitySpace#GetPage(String)
	 */
	public IDisplayPage GetPage(String pageName)
	{
		if (ClosePopUpDP.STR_MVC_CLASS_NAME.equals(pageName))
		{
			// This is called when SimpleCardSubmitDP or CreateFolderDP is canceled.
			ClosePopUpDP result = (ClosePopUpDP) super.GetPage(pageName);

			try
			{
				HTMLScript myScript = new HTMLScript("text/javaScript");
				myScript.AddInnerHTMLString(JS_PARENT_SIMPLE_REPOST);
				result.SetScript(myScript);
			}
			catch (HTMLException he)
			{
				
					log.Error(he, "Error generating JS for ClosePopUpDP: " +
						he.getMessage());
				
			}

			return result;
		}


		return super.GetPage(pageName);
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.AActivitySpace#Init()
	 */
	public void Init()
	{
		super.Init();

		// ObjMgr DisplayPages
		PlumtreeDP myPage = new ObjMgrRootDP();
		myPage.SetAddMainForm(true);
		RegisterPage(myPage);

		myPage = new ObjMgrDP();
		myPage.SetAddMainForm(true);
		RegisterPage(myPage);

		// Obj Mgr popup Pages -- create folder
		RegisterPage(new CreateFolderDP());
		RegisterPage(new ClosePopUpDP());

		// Obj Mgr popup Pages -- delete warnings
		RegisterPage(new ObjMgrDeleteWarningsDP());
		RegisterPage(new ObjMgrABOLinksDP());

		// ObjMgr Model
		RegisterModel(ObjMgrModel.STR_MVC_CLASS_NAME);

		IModel myModel = GetModel(ObjMgrModel.STR_MVC_CLASS_NAME);

		//Plumtree Utilities Model
		RegisterModel(PlumtreeUtilitiesModel.STR_MVC_CLASS_NAME);

		IModel puModel = GetModel(PlumtreeUtilitiesModel.STR_MVC_CLASS_NAME);

		//Server Settings Model
		RegisterModel(ServerSettingsModel.STR_MVC_CLASS_NAME);

		IModel serverModel = GetModel(ServerSettingsModel.STR_MVC_CLASS_NAME);

		// ObjMgr Views
		RegisterView(ObjMgrBreadCrumbView.STR_MVC_CLASS_NAME, myModel);
		RegisterView(FolderView.STR_MVC_CLASS_NAME, myModel);
		RegisterView(ObjectView.STR_MVC_CLASS_NAME, myModel);
		RegisterView(ObjMgrSearchView.STR_MVC_CLASS_NAME, myModel);

		RegisterView(ObjMgrHeaderView.STR_MVC_CLASS_NAME, myModel);
		((ObjMgrHeaderView) GetView(ObjMgrHeaderView.STR_MVC_CLASS_NAME)).SetPUModel(puModel);
		((ObjMgrHeaderView) GetView(ObjMgrHeaderView.STR_MVC_CLASS_NAME)).SetServerModel(serverModel);

		// Obj Mgr popup Views
		RegisterView(ObjMgrCreateFolderView.STR_MVC_CLASS_NAME, CreateFolderView.STR_VIEW_STORAGE_NAME, myModel);
		RegisterView(ObjMgrDeleteWarningsView.STR_MVC_CLASS_NAME, myModel);
		RegisterView(ObjMgrABOLinksView.STR_MVC_CLASS_NAME, myModel);

		// ObjMgr Controls
		RegisterControl(ObjMgrRepostControl.STR_MVC_CLASS_NAME, myModel);
		RegisterControl(ObjMgrSettingsControl.STR_MVC_CLASS_NAME, myModel);

		// Obj Mgr popup Controls
		RegisterControl(CheckDeleteControl.STR_MVC_CLASS_NAME, myModel);

		// Sort Control
		RegisterControl(SortControl.STR_MVC_CLASS_NAME, myModel);

		// Pagination Control
		RegisterControl(PaginationControl.STR_MVC_CLASS_NAME, myModel);

		RegisterControl(AdminRedirectControl.STR_MVC_CLASS_NAME, myModel);
		RegisterControl(AdminLoginTokenControl.STR_MVC_CLASS_NAME, myModel);
	}

	/**
	 * This method returns a date time formatting object
	 * for the current user.
	 * 
	 * @return XPDateFormat the format for the current user
	 */
	public XPDateFormat GetDateFormat()
	{
		XPLocale xpLocale = PTPersonalSettingsHelper.GetPersonalSettingValue_Locale(this);
		XPTimeZone xpTimeZone = PTPersonalSettingsHelper.GetPersonalSettingValue_TimeZone(this);

		XPDateFormat result = XPDateFormat.GetDateTimeInstance(
			XPDateFormat.SHORT, XPDateFormat.SHORT, 
			xpLocale, xpTimeZone);

		return result;
	}
}

