package com.plumtree.portalpages.admin.objmgr;

import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;

/**
 * @author DonH
 *
 */
public class ObjectManagerHolder
{
	public final IPTObjectManager m_ptObjMgr;

	public int[] m_nSelectedObjectIDs;

	// pagination variables (-1 implies not yet set)

	private int m_nFirstObjectCount = -1;

	private int m_nLastObjectCount = -1;

	private int m_nSkippedRows = 0;

	private int[][] m_nSortOrder;
	
	/** Current pagination size */
	private int m_nPaginationSize = -1;   // (-1 implies not yet set)
	
	private int m_nDefaultPaginationSize = -1;
	
	/**
	 * Creates a new ObjectManagerHolder object.
	 *
	 * @param ptObjMgr 
	 */
	public ObjectManagerHolder(IPTObjectManager ptObjMgr)
	{
		m_ptObjMgr				 = ptObjMgr;
		m_nSelectedObjectIDs     = null;

		m_nSortOrder													   = new int[1][2];
		m_nSortOrder[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		   = PT_PROPIDS.PT_PROPID_NAME;
		m_nSortOrder[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
	}

	/**
	 * This resets the selected items and pagination for this object manager.
	 * This is usually due to changing folders.  Sort order does not get reset.
	 */
	public void Reset()
	{
		m_nFirstObjectCount = -1;
		m_nLastObjectCount = -1;
		m_nSkippedRows = 0;
		m_nSelectedObjectIDs = null;
		m_nPaginationSize = -1;
	}

	/**
	 * This gets the number of rows that are being
	 * skipped in the query.
	 *
	 * @return int the row to start on (default 0)
	 */
	public int GetSkippedRows()
	{
		return m_nSkippedRows;
	}

	/**
	 * This gets the first object count
	 *
	 * @return int the first object count
	 */
	public int GetFirstObjectCount()
	{
		return m_nFirstObjectCount;
	}

	/**
	 * This gets the last object count
	 *
	 * @return int the last object count
	 */
	public int GetLastObjectCount()
	{
		return m_nLastObjectCount;
	}

	/**
	 * This gets the sort property ID.
	 *
	 * @return int the property id to sort on
	 */
	public int GetSortDirection()
	{
		return m_nSortOrder[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION];
	}

	/**
	 * This gets the sort property ID.
	 *
	 * @return int the property id to sort on
	 */
	public int GetSortPropID()
	{
		return m_nSortOrder[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID];
	}

	/**
	 * This returns whether or not the object is selected
	 *
	 * @param int nID the object ID of the object to check
	 * @return boolean true implies selected, false implies not selected
	 */
	public boolean IsObjectSelected(int nID)
	{
		if (m_nSelectedObjectIDs != null)
		{
			for (int i = 0; i < m_nSelectedObjectIDs.length; i++)
			{
				if (nID == m_nSelectedObjectIDs[i])
				{
					return true;
				}
			}
		}

		return false;
	}

	/** This method returns a query of the objects with the correct sorting.
	 *
	 * @return IASQueryResult
	 */
	public IASQueryResult QueryObjects(int nFolderID)
	{
		// Bug 27566: Admin hierarchy displaying a large number of users.
		// Bug 21969 -- We should calculate how many items we actually
		// want to view, and call result.SetMaxRows() with that.  This requires
		// some changes in the ObjMgrModel as well.  This is causing us to
		// double or triple our temporary memory usage when viewing objects
		// where there are close to 1000 objects of that type.

		int nPropIDMask = PT_PROPIDS.PT_PROPID_ALL & ~PT_PROPIDS.PT_PROPID_LOCKED;
		PTObjectManagerQueryWrapper result = new PTObjectManagerQueryWrapper(this.m_ptObjMgr);
		
		int nMaxRows = m_nDefaultPaginationSize;
		if (-1 != m_nPaginationSize)
		{
			nMaxRows = m_nPaginationSize;
		} 		
	
		result.SetMaxRows(nMaxRows);
		result.Query(nPropIDMask, nFolderID, m_nSortOrder, m_nSkippedRows, null);

		return result;
	}
	
	/**
	 * This sets the pagination.
	 * 
	 * This assumes that the pagination bounds are not greater than the
	 * number of items we can query for.
	 * 
	 * @param int the first object count
	 * @param int the second object count
	 */
	public void SetPagination(int nFirstObjectCount, int nLastObjectCount)
	{
		m_nFirstObjectCount = nFirstObjectCount;
		m_nLastObjectCount = nLastObjectCount;

		m_nSkippedRows = m_nFirstObjectCount - 1;
		m_nPaginationSize = nLastObjectCount - m_nFirstObjectCount + 1;
	}
	
	public void SetDefaultPaginationSize(int nDefaultPaginationSize)
	{
		m_nDefaultPaginationSize = nDefaultPaginationSize;
	}
	
	/**
	 * This sets the sort order.
	 *
	 * @param int the property id to sort on
	 * @param int the sort direction (PTSortedArrayWrapper.PTSAW_SORT_*)
	 */
	public void SetSortOrder(int nPropID, int nSortDirection)
	{
		m_nSortOrder[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		   = nPropID;
		m_nSortOrder[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = nSortDirection;
	}
}



