package com.plumtree.portalpages.admin.ptutilities.diagnostic;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.common.constants.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.tips.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This Editor displays the plug in administration diagnostic pages.
 * 
 * @author DonH
 */
public class DiagnosticPageEditorAS extends PTEditorAS
{
	private static final String sClassName = "com.plumtree.portalpages.admin.ptutilities.diagnostic.DiagnosticPageEditorAS";

	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		sClassName);

	public static final String STR_MVC_CLASS_NAME = "diagnostic";

	/** Map of EditorDP names to IDiagnosticPages */
	protected XPHashtable m_DiagnosticPages = new XPHashtable();

	/** Tells whether or not we are switching pages and should reinitialize the page. */
	protected boolean m_bChangePage = true;

	/**
	 * Registers all the classes of the Activity Space.
	 *
	 */
	public void Init()
	{
		// Enable partial content display for in place refresh.
		SetIsInPlaceRefreshEnabled(true);

		m_strModelName = DiagnosticPageModel.STR_MVC_CLASS_NAME;

		// Warning: m_strModelName must be initialized before super.Init() is called.
		super.Init();

		IModel myModel = GetModel(DiagnosticPageModel.STR_MVC_CLASS_NAME);

		// Register our custom repost control
		RegisterControl(DiagnosticPageRepostControl.STR_MVC_CLASS_NAME, myModel);

		// Sets left nav menu
		// We will add the links to this menu as we iterate over the
		// IDiagnosticPages
		// Sub menu "Diagnostic Page"
		int nMenuOneID = m_mEditorMenu.StartNewSubMenu(GetString(3071, ResourceConstants.PORTALADMINMSGS));

		// Dynamic Discovery of Diagnostic Page Objects
		final Object[] oaDiagnosticPages = (Object[]) GetApplication().GetCachingManager().GetEntry("DisplayDiagnosticPages");

		for (int nPage = 0; nPage < oaDiagnosticPages.length; nPage++)
		{
			IDiagnosticPage oDiagnosticPage = (IDiagnosticPage) oaDiagnosticPages[nPage];

			EditorDP page = oDiagnosticPage.GetDiagnosticPage();

			// Map of display page to diagnostic page object
			// so we can reset tests and change hints/help.
			m_DiagnosticPages.PutElement(page.GetName(), oDiagnosticPage);

			// Register the Display Page
			RegisterPage(page);

			// Add the link for this Display Page to the left nav menu
			m_mEditorMenu.AddLinkToSubMenu(nMenuOneID, page.GetName());

			// This allows each page to register Models, Views, and extra
			// Display pages as necessary.
			oDiagnosticPage.Init(this);

			if (nPage == 0)
			{
				// Set the default page to the first page in the xml file.
				// This needs to happen after we call Init(), because this
				// will call GetPage(), which will cause the tests to be
				// run.
				SetCurrentPage(page.GetName());
			}
		}
	}

	/**
	 * Helper method to return the helper repost control
	 * for this page, if any.
	 * 
	 * @return The helper repost control, null if there isn't one.
	 */
	public EditorHelperRepostControl GetPageHelperRepostControl()
	{
		IDiagnosticPage oDiagnosticPage = (IDiagnosticPage)
			m_DiagnosticPages.GetElement(GetCurrentPage().GetName());

		String strControlName = oDiagnosticPage.GetHelperRepostControlName();

		if (null != strControlName)
		{
			return (EditorHelperRepostControl) GetControl(strControlName);
		}

		return null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.AActivitySpace#GetPage(String)
	 */
	public IDisplayPage GetPage(String pageName)
	{
		IDisplayPage result = super.GetPage(pageName);

		if (m_bChangePage)
		{
			IDiagnosticPage oDiagnosticPage = (IDiagnosticPage)
				m_DiagnosticPages.GetElement(pageName);

			// This can be null during initialization
			if (null != oDiagnosticPage)
			{
				m_bChangePage = false;

				ChangePage(oDiagnosticPage);
			}
		}

		return result;
	}

	/**
	 * This helper method handles the actions necessary
	 * when changing pages.  This includes running tests
	 * and setting the help and tips.
	 * 
	 * @param _oDiagnosticPage The new page.
	 */
	protected void ChangePage(IDiagnosticPage _oDiagnosticPage)
	{
		// Run the tests for the first page.
		_oDiagnosticPage.RunDiagnosticTest(this);

		// Help Topic
		SetHelpTopic(_oDiagnosticPage.GetHelpTopic());

		// TIPS
		TipsModel tm = (TipsModel) GetModel(TipsModel.STR_MVC_CLASS_NAME);

		if (tm != null)
		{
			try
			{
				if (_oDiagnosticPage.HasTip())
				{
					tm.SetGeneratingClass( sClassName );

					// Tag Libraries
					tm.SetTipTitle( GetString( _oDiagnosticPage.GetTipTitleID(), ResourceConstants.TIPS) );

					// Main tip
					tm.SetTip( new HTMLGenericElement( GetString( _oDiagnosticPage.GetTipID(), ResourceConstants.TIPS) ));

					// Secondary tip
					tm.SetSecondarTip( new HTMLGenericElement( GetString( _oDiagnosticPage.GetSecondaryTipID(), ResourceConstants.TIPS ) ));
				} else
				{
					// Disable tips for this page.
					tm.DisableTips();
				}
			}
			catch (Exception ex)
			{
				log.Error(ex, "Couldn't generate tip.");
			}
		}
	}

	/**
	 * This method tells the editor to (re)initialize
	 * the next page, since we are changing pages.
	 */
	public void SetChangingPage()
	{
		m_bChangePage = true;
	}

	/**
	 * Helper method to rerun the current test.
	 */
	public void RefreshCurrentTest()
	{
		IDiagnosticPage oDiagnosticPage = (IDiagnosticPage)
			m_DiagnosticPages.GetElement(GetCurrentPage().GetName());

		oDiagnosticPage.RunDiagnosticTest(this);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.editor.EditorAS#GetEditorTitle()
	 */
	public String GetEditorTitle()
	{
		return GetString(3071, ResourceConstants.PORTALADMINMSGS);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.editor.EditorAS#GetRepostControlName()
	 */
	public String GetRepostControlName()
	{
		return (DiagnosticPageRepostControl.STR_MVC_CLASS_NAME);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new DiagnosticPageEditorAS();
	}
}