package com.plumtree.portalpages.admin.ptutilities.diagnostic.relatedservers;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.util.XPHashtable;
import com.plumtree.openfoundation.util.IXPEnumerator;
import com.plumtree.openfoundation.util.XPConvert;
import com.plumtree.openfoundation.internationalization.XPResourceManager;
import com.plumtree.portaluiinfrastructure.statichelpers.PTDebugHelpers;
import com.plumtree.portaluiinfrastructure.statichelpers.AppDataObject;
import com.plumtree.server.*;
import com.plumtree.server.marshalers.PTException;
import com.plumtree.portalpages.admin.editors.webservice.WSModelHelper;
import com.plumtree.portalpages.admin.ptutilities.diagnostic.serverstatuslist.ServerStatusListView;
import com.plumtree.portalpages.common.constants.ResourceConstants;
import com.plumtree.portalpages.browsing.gateway.GatewayUtils;
import com.plumtree.uiinfrastructure.activityspace.AActivitySpace;
import com.plumtree.openlog.OpenLogger;
import com.plumtree.openlog.OpenLogService;

/**
 * Helper class for RemoteHostsModel and RelatedServersPage
 *
 * @author OkanA
 */
public class ServerStatusHelper {
    private static ServerStatusHelper instance = new ServerStatusHelper();

    private static final String sClassName = "com.plumtree.portalpages.admin.ptutilities.diagnostic.relatedservers.ServerStatusHelper";
    private static final String PROTOCOL_SEPARATOR = "://";
    private static OpenLogger log = OpenLogService.GetLogger(
        OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
        sClassName);

    private ServerStatusHelper() {}

    public static ServerStatusHelper getInstance() {
        return instance;
    }

    /**
     * This helper method retrieves the server status for given host->portlet id mappings
     *
     * @param m_asOwner
     * @param initialPortletStatus
     * @param hosts
     * @param objectData
     * @param nStatusTimeout
     * @return
     */
    public boolean GetServerStatus(AActivitySpace m_asOwner, XPHashtable initialPortletStatus, XPHashtable hosts, Object[][] objectData, int nStatusTimeout)
    {
    	if (hosts.GetSize() != objectData[ServerStatusListView.SERVER_STATUS_ERROR].length)
    	{
    		// Simply log this.  We make sure that our array handling below can deal with this.
    		log.Warn("GetServerStatus incoming hosts and output data arrays of different lengths.");
    	}

        int[] portletIDs = new int[hosts.GetSize()];
        boolean errorFound = false;

        IXPEnumerator hostNames = hosts.GetKeys();

        int i = 0;
        while (hostNames.MoveNext()) {
            portletIDs[i] = XPConvert.ToInteger(hosts.GetElement(hostNames.GetCurrent()));
            i++;
        }

        // Get the temporary page used just for the diagnostic page
        // to query for the portlet content
        IPTPage diagPage = (IPTPage) ((IPTSession) m_asOwner.GetUserSession()).
            GetPages().Open(PT_INTRINSICS.PT_PAGE_DIAGNOSTIC, false);

        // Add the portlets to the page
        diagPage.AssignPortlets(portletIDs, 0);

        // Query for the page content
        // Create Application Data Object
        // 0 means not a community page
        IPTState state = AppDataObject.GetAppDataObject(0, PT_INTRINSICS.PT_PAGE_DIAGNOSTIC, "uri", m_asOwner, false);

        state.Add(PlumtreeExtensibility.PT_PORTLET_TIMEOUT_KEY, nStatusTimeout, 0, true);
        state.Add(PlumtreeExtensibility.PT_RETURN_URI_KEY, m_asOwner.GetCurrentNormalizedBaseURL(), 0, true);

        // Use Diagnostic mode here instead of runtime or preview mode, because
        // runtime mode will get cached content if there are errors with the portlets.
        // That would mask host failures forever.  Preview mode does not use
        // the cached content if there is an error, but it does still use it normally.
        // Preview mode also ignores the portlet timeout.
        IPTMyPortalGadgetContent content = diagPage.QueryPortletContent(0, PT_INTRINSICS.PT_PAGE_DIAGNOSTIC, state,
            "", PT_GADGET_MODES.PT_GADGETMODES_DIAGNOSTIC, true);

        i = 0;
        hostNames = hosts.GetKeys();

        // Check the status of each portlet (making sure that we don't go beyond
		// the end of the objectData length).
        while ((hostNames.MoveNext()) &&
        	(i < objectData[ServerStatusListView.SERVER_STATUS_ERROR].length)) {
            // Initialize with null
            objectData[ServerStatusListView.SERVER_STATUS_MSG][i] = null;
            objectData[ServerStatusListView.SERVER_STATUS_ERROR][i] = null;

            int nPortletID = XPConvert.ToInteger(hosts.GetElement(hostNames.GetCurrent()));

            Object[][] objArrMarkup = null;

            try {
                objArrMarkup = content.GetContent(i);
            } catch (Exception e) {
                if( e instanceof PTException ) {
                    // TODO: This switch statement shouldn't be necessary, but
                    // some kinds of portlets throw exceptions (gadget_runtime_error)
                    // and some use the error reporting array below.
                    switch ( ((PTException) e).GetErrorCode() ) {
                        case PT_RESULTCODES.PT_E_GADGET_TIMEOUT: {
                            // Portlet timed out, might just be slow
                            objectData[ServerStatusListView.SERVER_STATUS][i] = XPConvert.ToString(ServerStatusListView.STATUS_RED);
                            // "Timeout"
                            objectData[ServerStatusListView.SERVER_STATUS_MSG][i] = m_asOwner.GetString(3104, ResourceConstants.PORTALADMINMSGS);
                            errorFound = true;
                            break;
                        }
                        case PT_RESULTCODES.PT_E_GADGET_RUNTIME_ERROR: {
                            // Portlet had a runtime error
                            // This could be because the machine is down
                            objectData[ServerStatusListView.SERVER_STATUS][i] = XPConvert.ToString(ServerStatusListView.STATUS_RED);
                            // This could be an intermittent problem.  Try again with a longer timeout
                            errorFound = true;
                            break;
                        }
                        case PT_RESULTCODES.PT_E_GADGET_HTTP_ERROR_CODE: {
                            // Portlet had an HTTP error
                            // This could be because the machine is down
                            objectData[ServerStatusListView.SERVER_STATUS][i] = XPConvert.ToString(ServerStatusListView.STATUS_RED);

                            // This could be an intermittent problem.  Try again with a longer timeout
                            errorFound = true;

                            break;
                        }

                        case PT_RESULTCODES.PT_E_MYPORTAL_GADGETDISABLED: {
                            // We display disabled WebServices for info purposes.
                            objectData[ServerStatusListView.SERVER_STATUS][i] = XPConvert.ToString(ServerStatusListView.STATUS_YELLOW);
                            // "Disabled"
                            objectData[ServerStatusListView.SERVER_STATUS_MSG][i] = m_asOwner.GetString(3105, ResourceConstants.PORTALADMINMSGS);
                            break;
                        }

                        default: {
                            // Inconclusive
                            objectData[ServerStatusListView.SERVER_STATUS][i] = XPConvert.ToString(ServerStatusListView.STATUS_YELLOW);
                            // Unknown
                            objectData[ServerStatusListView.SERVER_STATUS_MSG][i] = m_asOwner.GetString(3135, ResourceConstants.PORTALADMINMSGS);

                            errorFound = true;
                        }
                    }
                } else {
                    // Inconclusive
                    objectData[ServerStatusListView.SERVER_STATUS][i] = XPConvert.ToString(ServerStatusListView.STATUS_RED);
                    // Offline
                    objectData[ServerStatusListView.SERVER_STATUS_MSG][i] = m_asOwner.GetString(1443, ResourceConstants.PORTALADMINMSGS);

                    errorFound = true;

                    // TODO: Should we log this exception?
                }

                objArrMarkup = null;
            }

            if (objArrMarkup != null)
            {
                int nlength = objArrMarkup.length;
                if (objArrMarkup.length == PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_NUM_COLS)
                {
                    if (!GatewayUtils.IsErrorContentArray(objArrMarkup)) {
                        objectData[ServerStatusListView.SERVER_STATUS][i] = XPConvert.ToString(ServerStatusListView.STATUS_GREEN);
                        // Online
                        objectData[ServerStatusListView.SERVER_STATUS_MSG][i] = m_asOwner.GetString(1442, ResourceConstants.PORTALADMINMSGS);
                    } else {
                        int iErrorCode = GatewayUtils.GetErrorCode(objArrMarkup);

                        // TODO: Do we need to check this?
                        // if the response was received, need to make room in the array for response status, header, body
//						boolean bResponseReceived = GatewayUtils.ResponseReceivedForErrors(objArrMarkup);

                        switch (iErrorCode) {
                            case PT_RESULTCODES.PT_E_GADGET_TIMEOUT: {
                                // Portlet timed out, might just be slow
                                objectData[ServerStatusListView.SERVER_STATUS][i] = XPConvert.ToString(ServerStatusListView.STATUS_YELLOW);
                                // "Timeout"
                                objectData[ServerStatusListView.SERVER_STATUS_MSG][i] = m_asOwner.GetString(3104, ResourceConstants.PORTALADMINMSGS);

                                errorFound = true;

                                break;
                            }

                            case PT_RESULTCODES.PT_E_GADGET_RUNTIME_ERROR: {
                                // Portlet had a runtime error
                                // This could be because the machine is down
                                objectData[ServerStatusListView.SERVER_STATUS][i] = XPConvert.ToString(ServerStatusListView.STATUS_RED);

                                String strStatus = GatewayUtils.GetResponseStatus(objArrMarkup);
                                if (strStatus != null) {
                                    // "HTTP {0} error"
                                    objectData[ServerStatusListView.SERVER_STATUS_MSG][i] =
                                        XPResourceManager.FormatMessage(
                                            m_asOwner.GetString(3106, ResourceConstants.PORTALADMINMSGS),
                                            strStatus);
                                }
                                // We do not add the error message here, because it
                                // is available on the detailed web services page.

                                // This could be an intermittent problem.  Try again with a longer timeout
                                errorFound = true;

                                break;
                            }

                            case PT_RESULTCODES.PT_E_GADGET_HTTP_ERROR_CODE: {
                                // Portlet had an http error
                                // This could be because the machine is down
                                objectData[ServerStatusListView.SERVER_STATUS][i] = XPConvert.ToString(ServerStatusListView.STATUS_RED);

                                String strStatus = GatewayUtils.GetResponseStatus(objArrMarkup);
                                if (strStatus != null) {
                                    // "HTTP {0} error"
                                    objectData[ServerStatusListView.SERVER_STATUS_MSG][i] =
                                        XPResourceManager.FormatMessage(
                                            m_asOwner.GetString(3106, ResourceConstants.PORTALADMINMSGS),
                                            strStatus);
                                }
                                // We do not add the error message here, because it
                                // is available on the detailed web services page.

                                // This could be an intermittent problem.  Try again with a longer timeout
                                errorFound = true;

                                break;
                            }

                            case PT_RESULTCODES.PT_E_MYPORTAL_GADGETDISABLED: {
                                // We display disabled WebServices for info purposes.
                                objectData[ServerStatusListView.SERVER_STATUS][i] = XPConvert.ToString(ServerStatusListView.STATUS_YELLOW);
                                // "Disabled"
                                objectData[ServerStatusListView.SERVER_STATUS_MSG][i] = m_asOwner.GetString(3105, ResourceConstants.PORTALADMINMSGS);
                                break;
                            }
                            default: {
                                // Inconclusive
                                objectData[ServerStatusListView.SERVER_STATUS][i] = XPConvert.ToString(ServerStatusListView.STATUS_YELLOW);
                                // Unknown
                                objectData[ServerStatusListView.SERVER_STATUS_MSG][i] = m_asOwner.GetString(3135, ResourceConstants.PORTALADMINMSGS);
                                errorFound = true;
                                break;
                            }
                        }
                    }
                } else {
                    // Nothing in the markup - unclear what this means.
                    objectData[ServerStatusListView.SERVER_STATUS][i] = XPConvert.ToString(ServerStatusListView.STATUS_YELLOW);
                    // Unknown
                    objectData[ServerStatusListView.SERVER_STATUS_MSG][i] = m_asOwner.GetString(3135, ResourceConstants.PORTALADMINMSGS);

                    errorFound = true;
                }
            }

            Object oInitialStatus = initialPortletStatus.GetElement(new Integer(nPortletID));
            if (null != oInitialStatus) {
                if (XPConvert.ToInteger(oInitialStatus) == ServerStatusListView.STATUS_RED) {
                    if (XPConvert.ToInteger(objectData[ServerStatusListView.SERVER_STATUS][i]) ==
                        ServerStatusListView.STATUS_GREEN) {
                        // If we were red, but are now green, that still indicates a problem.
                        // Set to yellow
                        objectData[ServerStatusListView.SERVER_STATUS][i] = XPConvert.ToString(ServerStatusListView.STATUS_YELLOW);
                        // "Failed on first attempt."
                        objectData[ServerStatusListView.SERVER_STATUS_ERROR][i] = m_asOwner.GetString(3107, ResourceConstants.PORTALADMINMSGS);
                    }
                }
            } else {
                // Store the initial status for next time.
                initialPortletStatus.PutElement(new Integer(nPortletID),
                    objectData[ServerStatusListView.SERVER_STATUS][i]);
            }
            i++;
        }
        return errorFound;
    }

    /**
     * Returns a list of unique hosts and portlets for each host
     * @param m_asOwner
     * @param serverID object id of the server to query. If serverID is passed, the method will return a portlet
     *        for this server only. If -1 is passed, it will return all unique <host, portlet> tuples.
     * @return hashtable of <host, portletid>
     */
    public XPHashtable GetPortletIDs(AActivitySpace m_asOwner, int serverID) {
        XPHashtable result = new XPHashtable();
        Object[][] webServiceQueryFilter = null;

        if (serverID >= 0) {
            webServiceQueryFilter = new Object[3][2];
        } else {
            webServiceQueryFilter = new Object[3][1];
        }

        webServiceQueryFilter[0][0]     = new Integer(PT_PROPIDS.PT_PROPID_WEBSERVICE_PROVIDERCLSID);
        webServiceQueryFilter[1][0]     = new Integer(PT_FILTEROPS.PT_FILTEROP_EQ);
        webServiceQueryFilter[2][0]     = WSModelHelper.WSTYPE_HTTPGP;

        if (serverID > 0) {
            webServiceQueryFilter[0][1]     = new Integer(PT_PROPIDS.PT_PROPID_WEBSERVICE_REMOTESERVERID);
            webServiceQueryFilter[1][1]     = new Integer(PT_FILTEROPS.PT_FILTEROP_EQ);
            webServiceQueryFilter[2][1]     = new Integer(serverID);
        }

		IPTQueryResult webServiceResults = (IPTQueryResult) ((IPTSession) m_asOwner.GetUserSession()).GetWebServices().
			Query(PT_PROPIDS.PT_PROPID_OBJECTID | PT_PROPIDS.PT_PROPID_WEBSERVICE_ENABLED,
			-1, PT_PROPIDS.PT_PROPID_OBJECTID, 0, -1, webServiceQueryFilter);

        if (webServiceResults.RowCount() < 1) {
            return result;
        }

		IPTQueryResult portletResults = (IPTQueryResult) ((IPTSession) m_asOwner.GetUserSession()).GetGadgets().
			Query(PT_PROPIDS.PT_PROPID_OBJECTID | PT_PROPIDS.PT_PROPID_GADGET_WEBSERVICEID,
			-1, PT_PROPIDS.PT_PROPID_OBJECTID, 0, -1, null);

		for (int nWebService = 0; nWebService < webServiceResults.RowCount(); nWebService++) 		{
			int nEnabled = webServiceResults.ItemAsInt(nWebService, PT_PROPIDS.PT_PROPID_WEBSERVICE_ENABLED);
			int nObjectID = webServiceResults.ItemAsInt(nWebService, PT_PROPIDS.PT_PROPID_OBJECTID);

			IPTWebService ptWebService = (IPTWebService) ((IPTSession) m_asOwner.GetUserSession()).
				GetWebServices().Open(nObjectID, false);

			String strUrl;

			try {
				strUrl = ptWebService.GetAbsoluteURL(PlumtreeExtensibility.PT_PROPBAG_HTTPGADGET_URL);
			} catch (Exception e) {
				log.Error(e, "Unable to get absolute URL from Web Service " + nObjectID);
				strUrl = null;
			}

			if (null != strUrl) {
				String strHost = GetHostName(strUrl);

				// If there is no current match, store the webservice,
				// or, if there is a current match, and this webservice is
				// enabled (the stored one might not be), store this webservice.
				// We overwrite disabled webservices because we can't get content
				// back from them using a MyPage.  We should always use enabled
				// WebServices unless all WebServices for a host are disabled.
				// This results in the last webservice for each host being stored.
				if ((!result.ContainsKey(strHost)) || (nEnabled == 1)) {
					// Look for a portlet that matches this web service
					for (int nPortlet = 0; nPortlet < portletResults.RowCount(); nPortlet++) {
						if ( nObjectID == portletResults.ItemAsInt(nPortlet, PT_PROPIDS.PT_PROPID_GADGET_WEBSERVICEID)) {
							// Store the first portlet that matches
							result.PutElement(strHost, new Integer(portletResults.ItemAsInt(nPortlet, PT_PROPIDS.PT_PROPID_OBJECTID)));
						}
					}
				}
			}
		}

		return result;
	}


	/**
	 * This method gets the normalized host name from a URL.
	 * 
	 * @param _strURL
	 * @return The normalized host name.
	 */
    public static String GetHostName(String _strURL)
    {
        int nIndex = _strURL.indexOf(PROTOCOL_SEPARATOR);

        if (nIndex != -1) {
            _strURL = _strURL.substring(nIndex + PROTOCOL_SEPARATOR.length(), _strURL.length());
        }

        nIndex = _strURL.indexOf("/");

        if (nIndex != -1) {
            _strURL = _strURL.substring(0, nIndex);
        }

        return XPStringUtility.ToLowerCase(_strURL);
    }
}
