package com.plumtree.portalpages.admin.ptutilities.diagnostic.serverstatuslist;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.expandablelist.multicolumnlist.*;
import com.plumtree.uiinfrastructure.expandablelist.multicolumnlist.sortedmetadata.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This view displays a generic server status list.
 * 
 * @author DonH
 */
public class ServerStatusListView extends MultiColumnListView
{
	private static final String sClassName = "com.plumtree.portalpages.admin.ptutilities.diagnostic.serverstatuslist.ServerStatusListView";

	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		sClassName);

	/** Class name for the activity space */
	public static final String STR_MVC_CLASS_NAME = "ServerStatusList";

	/**
	 * The type column ID.  This usually contains
	 * a String representatio of an int that is
	 * different for every server type so that you
	 * can sort by server type.  The actual value
	 * is not used in this View.
	 */
	public static final int SERVER_TYPE = 0;

	/**
	 * The name column ID.
	 */
	public static final int SERVER_NAME = 1;

	/**
	 * The status column ID.
	 */
	public static final int SERVER_STATUS = 2;

	/**
	 * The server icon column ID.  This is a
	 * PTImageType object, and is not sortable.
	 */
	public static final int SERVER_ICON = 3;

	/**
	 * The server url ID.  This is an HTMLAnchor
	 * that includes the server name already.
	 * This may be null.
	 */
	public static final int SERVER_ANCHOR = 4;

	/**
	 * The server description ID.  This does not
	 * have it's own visible column.
	 * This may be null.
	 */
	public static final int SERVER_DESCRIPTION = 5;

	/**
	 * The server description ID.  This does not
	 * have it's own visible column.
	 * This may be null.
	 */
	public static final int SERVER_STATUS_MSG = 6;

	/**
	 * The server status error ID.  This does not
	 * have it's own visible column.
	 * This may be null.
	 */
	public static final int SERVER_STATUS_ERROR = 7;

	/**
	 * The number of columns.
	 */
	public static final int NUM_COLS = 8;

	// Server status codes
	public static final int STATUS_GREEN = 1;
	public static final int STATUS_YELLOW = 2;
	public static final int STATUS_RED = 3;


	public static final int MAX_ERROR_CHARS = 60;

	/**
	 * Returns the HTML for the row _nRowIndex. Each row contains just an icon
	 * and input text for each column.
	 *
	 * @param _saData
	 * @param _nRowIndex
	 *
	 * @return HTMLElement
	 *
	 */
	public HTMLElement GetHTMLForRow(IPTGrowableSortedArrayWrapperRO _saData,
		int _nRowIndex)
	{
		SortedMetaDataListModel model = (SortedMetaDataListModel) m_model;

		HTMLElementCollection result = new HTMLElementCollection();
		HTMLTableCell myCell;

		// Iterate through columns - this is not zero based
		for (int i = 1; i <= model.GetNumVisibleColumns(); i++)
		{
			myCell = new HTMLTableCell();
			myCell.SetWidth(model.GetColumnWidth(i));
			myCell.SetHeight("30");
			myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
			myCell.SetNoWrap(true);

			if (i == (SERVER_NAME + 1))
			{
				// We want to allow long names and descriptions to wrap.
				myCell.SetNoWrap(false);
			}

			myCell.AddInnerHTMLElement(GetColumnHTML(i, _nRowIndex, _saData, model));

			result.AddInnerHTMLElement(myCell);
		}

		// input text
		return result;
	}

	/**
	 * This helper method takes the row meta data and outputs
	 * the proper HTML for the column entry.
	 * 
	 * @param _nVisibleColumnIndex
	 * @param _nRowIndex
	 * @param _saData
	 * @param _model
	 * @return The HTML for the column entry.
	 */
	protected HTMLElement GetColumnHTML(int _nVisibleColumnIndex,
		int _nRowIndex, IPTGrowableSortedArrayWrapperRO _saData,
		SortedMetaDataListModel _model)
	{
		HTMLElementCollection result = new HTMLElementCollection();

		int nDataColumnIndex = _nVisibleColumnIndex - 1;

		switch (nDataColumnIndex)
		{
			case SERVER_TYPE :
				// The visible index stores the type integer for sorting.  We want the icon
				// Icon index is zero based, data index is not
				PTImageType icon = (PTImageType) _saData.GetItem(_nRowIndex, SERVER_ICON + 1);
				result.AddInnerHTMLElement(GetHTMLForIcon(icon));
				break;

			case SERVER_NAME :
				HTMLSpan mySpan = new HTMLSpan();
				result.AddInnerHTMLElement(mySpan);
				mySpan.SetStyleClass(PTStyleClass.LIST_SUBTITLE);

				HTMLAnchor nameAnchor = (HTMLAnchor) _saData.GetItem(_nRowIndex, SERVER_ANCHOR + 1);

				if (null != nameAnchor)
				{
					mySpan.AddInnerHTMLElement(nameAnchor);
				} else
				{
					String strName = (String) _saData.GetItem(_nRowIndex, nDataColumnIndex + 1);
					mySpan.AddInnerHTMLEncodedString(strName);
				}

				String strDescription = (String) _saData.GetItem(_nRowIndex, SERVER_DESCRIPTION + 1);
	
				if (strDescription != null)
				{
					result.AddInnerHTMLElement(new HTMLBr());

					mySpan = new HTMLSpan();
					result.AddInnerHTMLElement(mySpan);
					mySpan.SetStyleClass(PTStyleClass.LIST_TEXT);
					mySpan.AddInnerHTMLEncodedString(strDescription);
				}

				String strStatusError = (String) _saData.GetItem(_nRowIndex, SERVER_STATUS_ERROR + 1);
		
				if (strStatusError != null)
				{
					result.AddInnerHTMLElement(new HTMLBr());

					mySpan = new HTMLSpan();
					result.AddInnerHTMLElement(mySpan);
					mySpan.SetStyleClass(PTStyleClass.LIST_TEXT);

					// Since the error messages are often quite long, we need to
					// break them down so the display looks okay. 
					mySpan.AddInnerHTMLEncodedString(GeneralHelpers.BreakDownLongWordsInString
						(strStatusError, MAX_ERROR_CHARS));
				}
				break;

			case SERVER_STATUS :
				int nStatusCode = XPConvert.ToInteger(_saData.GetItem(_nRowIndex, nDataColumnIndex + 1));
				String strStatusMessage = (String) _saData.GetItem(_nRowIndex, SERVER_STATUS_MSG + 1);

				PTImageType statusLight = null;
	
				switch (nStatusCode)
				{
					case STATUS_GREEN :
						statusLight = PTImageType.ACTION_APPROVE;
						break;
	
					case STATUS_YELLOW :
						statusLight = PTImageType.ICON_HOLD;
						break;
	
					case STATUS_RED :
						statusLight = PTImageType.ACTION_UNAPPROVE;
						break;
	
					default :
						log.Warn("Invalid server status " + nStatusCode);
						break;
				}
	
				if (null != statusLight)
				{
					result.AddInnerHTMLElement(GetHTMLForIcon(statusLight));
					if (null != strStatusMessage)
					{
						// Disabled, timeout, etc...
						result.AddInnerHTMLString(strStatusMessage);
					}
				}
				break;

			default :
				break;
		}

		return result;
	}

	/**
	 * Returns the HTML for the delete cell in the action bar.
	 *
	 * @return HTMLTableCell
	 */
	protected HTMLTableCell GetHTMLForDeleteAction()
	{
		// To make sure the delete html doesn't get added.
		return new HTMLTableCell();
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new ServerStatusListView();
	}
}