package com.plumtree.portalpages.admin.ptutilities.localizedstrings;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openfoundation.xml.dom.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;

/**
 * @author MichaelS
 *
 * Controls process the events generated by users.  The Interpreter takes
 * the user events and fills out the fields in the control.  The control is then
 * executed and returns the next DisplayPage (usually the parent page, unless the
 * control is a go to url control).
 *
 * Controls should encapsulate the business logic of the AActivitySpace.  As an MVC
 * component, Views are subject to the strict separation between data, logic, and
 * presentation.
 *
 * Obviously, there should be a good way to refer to controls without having to
 * manually type in a text string.  There should be some sort of package identifier
 * used to get these.
 */
public class LocalizedStringsControl extends EditorRepostControl
	implements IHTTPControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.ptutilities.localizedstrings.LocalizedStringsControl");

	
	public static final String STR_MVC_CLASS_NAME = "LocalizedStrings";

	
	/** MES- Custom post actions that we post to ourself- we don't want to conflict
	 * with a superclass! */
	public static final int POSTTOSELF_ACTION_UPLOAD_LOCALIZED = 80;
	
	/* MES- The DTD to validate uploads, and to ignore whitespace */
	private static final String LOCALIZED_STRINGS_DTD = 
		"<!DOCTYPE localizationtable [" + 
  		"<!ELEMENT localizationtable  (languages,segments)>" +
  		"<!ATTLIST localizationtable  xml:space (default|preserve) 'default'>" +
  		"<!ELEMENT languages (language*)>" +
  		"<!ATTLIST languages count CDATA #IMPLIED>" +
  		"<!ELEMENT language  (#PCDATA)>" +
  		"<!ATTLIST language  xml:space (default|preserve) 'preserve'>" +
  		"<!ELEMENT segments  (segment*)>" +
  		"<!ATTLIST segments  count CDATA #IMPLIED>" +
  		"<!ELEMENT segment   (source,target*)>" +
  		"<!ATTLIST segment   classid CDATA #REQUIRED>" +
  		"<!ATTLIST segment   itemid CDATA #REQUIRED>" +
  		"<!ATTLIST segment   stringid CDATA #REQUIRED>" +
  		"<!ELEMENT source    (#PCDATA)>" +
  		"<!ATTLIST source    language CDATA #REQUIRED>" +
  		"<!ATTLIST source    xml:space (default|preserve) 'preserve'>" +
  		"<!ELEMENT target    (#PCDATA)>" +
  		"<!ATTLIST target    language CDATA #REQUIRED>" +
  		"<!ATTLIST target    xml:space (default|preserve) 'preserve'>" +
		"]>";

	/** these need to be nulled out after execution. */
	private IXPRequest m_request = null;
	private Object[][] m_arrData = null;
	private String[] m_saLangs   = null;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new LocalizedStringsControl();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IControl#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * MES- SetXPRequest is called by the system before calling
	 * PerformAction.
	 *
	 * @param request is the XPRequest object we're operating on
	 */
	public void SetHTTPItems(IXPRequest request, IWebData pageData)
	{
		//MES- Just store it to a member variable
		m_request = request;
	}

	/**
	 * Given the repost action mode, calls the associated method in the model.
	 *
	 * @param _nAction
	 */
	protected void PerformAction(int _nAction)
	{
		switch (_nAction)
		{
		case POSTTOSELF_ACTION_UPLOAD_LOCALIZED:
			PerformUploadAction();

			break;

		default:
			super.PerformAction(_nAction);

			break;

			// JF: Do not add a default case because sub classes will extend this one and add their own action modes.
		}

		// null out member variables so we don't leak them.
		m_request = null;
		m_arrData = null;
		m_saLangs = null;
	}

	/**
	 * MES- ParseUploadedXML takes in an XPDocument (XML DOM) and
	 * populates m_arrData and m_saLangs based on the XPDocument.
	 * On failure, it sets them to null.
	 *
	 * @param xpdocUploaded is the XPDocument to parse
	 * @return    a string describing the result suitable for displaying
	 * to the user (success or failure.)
	 */
	private String ParseUploadedXML(XPDocument xpdocUploaded)
	{
		//MES- Clear any existing data
		m_arrData     = null;
		m_saLangs     = null;

		try
		{
			//MES- Get the root node
			XPElement elemRoot    = xpdocUploaded.GetDocumentElement();
			//MES- Did we get it?
			if (null == elemRoot)
				return m_asOwner.GetString(2456, "ptmsgs_portaladminmsgs"); // "Unable to upload XML document: root node not found";
			
			XPElement elemLangs   = (XPElement) elemRoot.GetElementsByTagName(LocalizedStringsDownloadDP.LOC_XML_LANGS_NODE)
				.Item(0);
			//MES- Did we get it?
			if (null == elemLangs)
				return XPResourceManager.FormatMessage(m_asOwner.GetString(2457, "ptmsgs_portaladminmsgs"), LocalizedStringsDownloadDP.LOC_XML_LANGS_NODE); //"Unable to upload XML document: element '{0}' not found"
	 
				
			XPElement elemStrings = (XPElement) elemRoot.GetElementsByTagName(LocalizedStringsDownloadDP.LOC_XML_SEGMENTS_NODE)
				.Item(0);
			//MES- Did we get it?
			if (null == elemStrings)
				return XPResourceManager.FormatMessage(m_asOwner.GetString(2457, "ptmsgs_portaladminmsgs"), LocalizedStringsDownloadDP.LOC_XML_SEGMENTS_NODE); //"Unable to upload XML document: element '{0}' not found"

			//MES- Get the number of languages
			XPNodeList nodesLang = elemLangs.GetChildNodes();
			int iNumLangs = 0;
			if (nodesLang != null)
			{
				iNumLangs = nodesLang.GetLength();
			}

			String[] saLangs     = new String[iNumLangs];
			XPHashtable mapLangs = new XPHashtable();
			
			//MES- Did we get any langs?
			if (0 >= iNumLangs)
				return m_asOwner.GetString(2458, "ptmsgs_portaladminmsgs"); //"Unable to upload XML document: no languages found"

			for (int iLangIndex = 0; iLangIndex < iNumLangs; iLangIndex++)
			{
				XPNode nodeLang     = nodesLang.Item(iLangIndex);
				String sLang	    = nodeLang.GetFirstChild().GetNodeValue();
				saLangs[iLangIndex] = sLang;
				mapLangs.PutElement(sLang, new Integer(iLangIndex));
			}

			//MES- Now get the localized strings
			XPNodeList nodesStrings = elemStrings.GetChildNodes();
			int iNumStrings = 0;
			if (nodesStrings != null)
			{
				iNumStrings = nodesStrings.GetLength();
			}

			Object[][] arrData;

			if (0 < iNumStrings)
			{
				arrData = new Object[PT_SERIALIZED_LOCALIZED_STRINGS_COLS.PT_SLS_TARGET_STRING_START +
					iNumLangs][iNumStrings];

				for (int iStringIndex = 0; iStringIndex < iNumStrings; iStringIndex++)
				{
					//MES- Get the IDs for this string
					XPElement elemString = (XPElement) nodesStrings.Item(iStringIndex);
					String sItem = elemString.GetAttribute(LocalizedStringsDownloadDP.LOC_XML_CLASSID_ATT);
					if (null == sItem || 0 == sItem.length())
						return XPResourceManager.FormatMessage(m_asOwner.GetString(2459, "ptmsgs_portaladminmsgs"), LocalizedStringsDownloadDP.LOC_XML_CLASSID_ATT, XPConvert.ToString(iStringIndex)); //Unable to upload XML document: attribute '{0}' of segment {1} not found
					
					int intClassID   = XPConvert.ToInteger(sItem);
					
					
					sItem = elemString.GetAttribute(LocalizedStringsDownloadDP.LOC_XML_ITEMID_ATT);
					if (null == sItem || 0 == sItem.length())
						return XPResourceManager.FormatMessage(m_asOwner.GetString(2459, "ptmsgs_portaladminmsgs"), LocalizedStringsDownloadDP.LOC_XML_ITEMID_ATT, XPConvert.ToString(iStringIndex)); //Unable to upload XML document: attribute '{0}' of segment {1} not found
					
					int intItemID    = XPConvert.ToInteger(sItem);
					
					
					sItem = elemString.GetAttribute(LocalizedStringsDownloadDP.LOC_XML_STRINGID_ATT);
					if (null == sItem || 0 == sItem.length())
						return XPResourceManager.FormatMessage(m_asOwner.GetString(2459, "ptmsgs_portaladminmsgs"), LocalizedStringsDownloadDP.LOC_XML_STRINGID_ATT, XPConvert.ToString(iStringIndex)); //Unable to upload XML document: attribute '{0}' of segment {1} not found
					
					int intStringID  = XPConvert.ToInteger(sItem);

					arrData[PT_SERIALIZED_LOCALIZED_STRINGS_COLS.PT_SLS_CLASSID][iStringIndex] = new Integer(intClassID);
					arrData[PT_SERIALIZED_LOCALIZED_STRINGS_COLS.PT_SLS_ITEMID][iStringIndex] = new Integer(intItemID);
					arrData[PT_SERIALIZED_LOCALIZED_STRINGS_COLS.PT_SLS_STRINGID][iStringIndex] = new Integer(intStringID);

					//MES- Get the source string and lang.  Technically, the source string is not
					//	used, but let's get it for completeness.
					XPElement elemSource = (XPElement) elemString.GetElementsByTagName(LocalizedStringsDownloadDP.LOC_XML_SOURCE_NODE).Item(0);
					if (null == elemSource)
						return XPResourceManager.FormatMessage(m_asOwner.GetString(2460, "ptmsgs_portaladminmsgs"), LocalizedStringsDownloadDP.LOC_XML_SOURCE_NODE, XPConvert.ToString(iStringIndex)); //Unable to upload XML document: element '{0}' of segment {1} not found
					
					String sSourceLang   = elemSource.GetAttribute(LocalizedStringsDownloadDP.LOC_XML_LANG_ATT);
					if (null == sSourceLang || 0 == sSourceLang.length())
						return XPResourceManager.FormatMessage(m_asOwner.GetString(2461, "ptmsgs_portaladminmsgs"), LocalizedStringsDownloadDP.LOC_XML_LANG_ATT, LocalizedStringsDownloadDP.LOC_XML_SOURCE_NODE, XPConvert.ToString(iStringIndex)); //Unable to upload XML document: attribute '{0}' of element {1} of segment {2} not found

					//MES- The source string might be blank!
					String sSource = "";

					if (elemSource.HasChildNodes())
					{
						sSource = elemSource.GetFirstChild().GetNodeValue();
					}

					arrData[PT_SERIALIZED_LOCALIZED_STRINGS_COLS.PT_SLS_SOURCE_LANG][iStringIndex]		 = sSourceLang;
					arrData[PT_SERIALIZED_LOCALIZED_STRINGS_COLS.PT_SLS_SOURCE_STRING][iStringIndex]     = sSource;

					//MES- Finally, run through the localizations for this string
					XPNodeList nodesTargets = elemString.GetChildNodes();
					int iNumTargets		    = nodesTargets.GetLength();

					for (int iTargetIndex = 0; iTargetIndex < iNumTargets;
							++iTargetIndex)
					{
						XPElement elemTarget = (XPElement) nodesTargets.Item(iTargetIndex);
						if (null == elemTarget)
							return XPResourceManager.FormatMessage(m_asOwner.GetString(2462, "ptmsgs_portaladminmsgs"), XPConvert.ToString(iTargetIndex), XPConvert.ToString(iStringIndex)); //Unable to upload XML document: item {0} of segment {1} not found
						
						String sTargetLang   = elemTarget.GetAttribute(LocalizedStringsDownloadDP.LOC_XML_LANG_ATT);
						if (null == sTargetLang || 0 == sTargetLang.length())
							return XPResourceManager.FormatMessage(m_asOwner.GetString(2463, "ptmsgs_portaladminmsgs"), LocalizedStringsDownloadDP.LOC_XML_LANG_ATT, XPConvert.ToString(iTargetIndex), XPConvert.ToString(iStringIndex)); //Unable to upload XML document: attribute '{0}' of item {1} of segment {2} not found

						//MES- Do we have a child that contains a value and is this a recognized lang?  If not, forget about it.
						if (elemTarget.HasChildNodes() &&
								mapLangs.ContainsKey(sTargetLang))
						{
							String sTargetValue = elemTarget.GetFirstChild()
								.GetNodeValue();
							int iColumnID   = XPConvert.ToInteger(mapLangs.GetElement(sTargetLang));

							//MES- Put into the output
							arrData[PT_SERIALIZED_LOCALIZED_STRINGS_COLS.PT_SLS_TARGET_STRING_START +
							iColumnID][iStringIndex] = sTargetValue;
						}
					}
				}
			}
			else
			{
				arrData = null;
			}

			//MES- Copy the data to the member variables
			m_arrData     = arrData;
			m_saLangs     = saLangs;

			return m_asOwner.GetString(1512, "ptmsgs_portaladminmsgs");
		}
		catch (Exception e)
		{
			
				log.Error(e, "Exception occurred while parsing uploaded localized strings.");
			

			return m_asOwner.GetString(1513, "ptmsgs_portaladminmsgs") +
				e.getMessage();
		}
	}

	/**
	 * MES- PerformUploadAction uploads an XML file and stores it into
	 * Plumtree.
	 */
	private void PerformUploadAction()
	{
		ILocalizedStringsModel model = (ILocalizedStringsModel) m_model;

		try
		{
			//MES- Get the data from the post (i.e. the uploaded file)
			XPDocument docXML = XPBuilder.LoadDocumentFromByteArray(((XPRequest) m_request).GetBytes(), LOCALIZED_STRINGS_DTD, /*MES- Ignore white space*/true);

			//MES- Convert the XML to a 2D array
			model.SetUploadMessage(ParseUploadedXML(docXML));

			//MES- Store it into the model
			model.SetData(m_arrData);
			model.SetLangs(m_saLangs);
		}
		catch (Exception e)
		{
			
				log.Error(e, "Exception occurred in LocalizedStringsControl.PerformUploadAction: " +
					e.getMessage());
			

			model.SetUploadMessage(m_asOwner.GetString(1514,
					"ptmsgs_portaladminmsgs") + e.getMessage());
		}
	}
}

