package com.plumtree.portalpages.browsing.directory.rest;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.directory.*;
import com.plumtree.portalpages.rest.*;
import com.plumtree.portalpages.rest.utils.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.login.*;

/**
 * This control handles processing incoming requests for the KD
 * REST API.  It currently only handles JSON input.
 */
public class DirRESTControl implements IControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_REST),
		"com.plumtree.portalpages.browsing.directory.DirRESTControl");

	public static final String STR_MVC_CLASS_NAME = "DirREST";

	/**
	 * The owning space for this control.
	 */
	protected AActivitySpace m_asOwner;

	/**
	 * The instance of the directory model to use when processing data.
	 */
	protected IDirModel m_dirModel;

	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		try
		{
			if (LoginHelper.INSTANCE.GetIsGuestUser(m_asOwner.GetUserSession()))
			{
				log.Warn("User must be logged in to access the KD REST API.");

				// Must be logged in to use this API. 
				return ProcessError(CommonRestErrors.NOT_LOGGED_IN);
			}

			int nFolderID;
			try
			{
				// This originally comes from the friendly URL and is put into this format.
				String[] sValues = (String[]) arguments.GetElement(DirRepostControl.CARD_SUBMIT_FOLDER_OVERRIDE);
				nFolderID = XPConvert.ToInteger(sValues[0]);
			} catch (Exception e)
			{
				log.Warn("Unable to find folder ID in Rest API URL.");

				return ProcessError(DirRestErrors.NO_FOLDER_ID);
			}

			// TODO: Check security on folder - The folder submit will fail if the
			// user does not have access to the folder, but will only return a
			// generic message.  This could be improved by checking security here
			// and returning a more specific error (although this would not be
			// shown to the end-user anyways.

			// NOTE: This is currently fairly secure because it only accepts POSTs.
			// However, if in the future this accepts GETs and other verbs, then we
			// need to be more careful.  If a user is logged into the portal and
			// goes to a hacker site, that site can use a javascript src include tag
			// on their page to the portal JSON page.  This script request (a GET)
			// will be passed onto portal with the user's portal cookie, allowing them
			// to access the data.  We are safe from this right now because the GET
			// request will be ignored by the control.
			// One way to secure against this kind of attack is to require the JSON request
			// to generate a hash of the contents of the login cookie and add it to the
			// request as an argument of some sort.

			IXPRequest request = m_asOwner.GetCurrentHTTPRequest();
			String strMethod = request.GetMethod();

			if (XPStringUtility.CompareIgnoreCase(strMethod, "POST") != 0)
			{
				log.Warn("This KD REST API only supports the POST method.");

				// This KD REST API only supports the POST method.
				return ProcessError(CommonRestErrors.NOT_POST);
			}

			// Get the JSON input and parse it.
			// NOTE: We currently only support JSON POSTs, so we assume all requests
			// are the same.  We could eventually support GETs and XML as well.
			if (request.GetContentLength() == 0)
			{
				log.Warn("This KD REST API requires POST content, but none was found.");

				// This KD REST API requires POST content, but none was found.
				return ProcessError(CommonRestErrors.NO_POST_CONTENT);
			}

			// Get the POST body JSON string.
			String strJSON = new RESTHelper().GetPOSTBody(request);

			if ((strJSON == null) || (strJSON.length() == 0))
			{
				log.Warn("This KD REST API requires POST content, but none was found.");

				// This KD REST API requires POST content, but none was found.
				return ProcessError(CommonRestErrors.NO_JSON_POST_CONTENT);
			}

			IJSONParser parser = JSONParser.GetInstance();
			parser.ParseJSONObject(strJSON);

			if (XPStringUtility.CompareIgnoreCase(parser.GetObjectName(), DirRESTConstants.DOCUMENT_FORMAT_NAME) != 0)
			{
				log.Warn("This KD REST API only supports document objects, not " +
					parser.GetObjectName() + " objects.");

				// This KD REST API only supports document objects.
				return ProcessError(DirRestErrors.NO_JSON_DOCUMENT_OBJECT);
			}

			// Parse the JSON and execute the command.
			Redirect error = ProcessAction(parser, nFolderID);

			if (error != null)
			{
				return error;
			}
		} catch (Exception e)
		{
			log.Error(e, "Error processing KD REST API request.");

			// General error.
			return ProcessError(CommonRestErrors.GENERAL);
		}

		// Redirect to DirRESTDP.
		return GetDPRedirect();
	}

	/**
	 * This method handles parsing the JSON body and executing the
	 * appropriate command.
	 * 
	 * @param _parser The parser with the JSON object already loaded.
	 * @param _nFolderID The folder to create the object in.
	 * @return An error redirect, if necessary.  Null if there were no errors.
	 */
	public Redirect ProcessAction(IJSONParser _parser, int _nFolderID)
	{
		// Load the common data from the JSON parser and set it on the model.

		// If the name and description are not present, the model will generate
		// them automatically.
		String strName = _parser.GetMember(DirRESTConstants.DOCUMENT_NAME);
		if (strName != null)
		{
			m_dirModel.SetCardSubmitName(strName);			
		}

		String strDescription = _parser.GetMember(DirRESTConstants.DOCUMENT_DESCRIPTION);
		if (strDescription != null)
		{
			m_dirModel.SetCardSubmitDescription(strDescription);			
		}

		String strDataSourceID = _parser.GetMember(DirRESTConstants.DOCUMENT_DATA_SOURCE_ID);
		int nDataSourceID;
		try
		{
			nDataSourceID = XPConvert.ToInteger(strDataSourceID);
		} catch (Exception e)
		{
			log.Error(e, "Error retrieving data source ID from JSON data.");

			return ProcessError(DirRestErrors.NO_DATA_SOURCE_ID);
		}
		m_dirModel.SetCardSubmitDataSource(nDataSourceID);

		String strContentURL = _parser.GetMember(DirRESTConstants.DOCUMENT_CONTENT_URL);

		if ((strContentURL == null) || (strContentURL.length() == 0))
		{
			log.Error("Content URL not present in JSON data.");

			return ProcessError(DirRestErrors.NO_CONTENT_URL);
		}

		m_dirModel.SetCardSubmitURL(strContentURL);

		m_dirModel.SetCardSubmitFolder(_nFolderID);

		// There are two types of card creation - simple and web service.  They share
		// some data, but the web service version has additional data (starting with
		// the document type, which we use to tell the two versions apart).

		String strType = _parser.GetMember(DirRESTConstants.DOCUMENT_TYPE);
		if (strType == null)
		{
			// Simple Submit Mode
			m_dirModel.SubmitCard();			
		} else
		{
			// Web Service Submit Mode
			String strID = _parser.GetMember(DirRESTConstants.DOCUMENT_DOC_ID);
			String strServerID = _parser.GetMember(DirRESTConstants.DOCUMENT_SERVER_ID);

			m_dirModel.SetDocUploadParams(strID, strServerID);
			
			m_dirModel.SubmitCard(strContentURL, strType, strID);
			// This was commented out in the DirRepostControl - not sure why
			// m_dirModel.UpdatePropertiesForDocUpload(strID, strServerID);
		}

		m_dirModel.SetDefaultCardRefreshSettings(m_dirModel.GetCardID());

		// Reset the To and From (so that the new card will be displayed)
		// if we are on a directory page
		m_dirModel.ClearCardRange();

		// No errors
		return null;
	}

	/**
	 * This helper method sets the error on the model and redirects to the
	 * REST API display page.
	 * 
	 * @param _nErrorID the current error code.
	 * @return The redirect to the KD REST API display page.
	 */
	public Redirect ProcessError(IRESTError _error)
	{
		Redirect redirect = GetDPRedirect();

		m_dirModel.SetCardSubmitError(_error.GetCode());

		return redirect;
	}

	/**
	 * @return the redirect to the KD REST API display page.
	 */
	public Redirect GetDPRedirect()
	{
		Redirect redirect = new Redirect();
		redirect.SetLinkGetCachedSpace(m_asOwner);
		redirect.SetDisplayPage(DirRESTDP.STR_MVC_CLASS_NAME);

		return redirect;
	}

	public void Init(IModel model, AActivitySpace space)
	{
		m_asOwner = space;
		m_dirModel = (IDirModel) model;
	}

	public String GetName() {
		return STR_MVC_CLASS_NAME;
	}

	public Object Create() {
		return new DirRESTControl();
	}
}