package com.plumtree.portalpages.browsing.gateway;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.common.uiparts.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.displaytree.*;
import com.plumtree.portaluiinfrastructure.tags.xml.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author daniilk
 *
 */
public class GatewayAS extends PlumtreeAS
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_GATEWAY),
		"com.plumtree.portalpages.browsing.gateway.GatewayAS");

	public static final String STR_MVC_CLASS_NAME = "Gateway";

	// This is used to prevent possible infinite loops in tag processing.
	protected boolean m_bMarkupCalled = false;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new GatewayAS();
	}

	public boolean CheckIfControlWasCalled(GatewayModel gwModel)
	{
		if (!gwModel.WasControlCalled())
		{
			
				log.Debug("GetCurrentPage(): Trying to get a page before control was called. This is probably not correct.");
			

			GatewayUtils.SetRedirectToLastHostedPage(this, gwModel);
			return false;
		}
		return true;
	}

	private void ProcessMarkupAndUpdateModel(GatewayModel gwModel)
	{
		// In order to handle PT Tags that determine if the page is hosted or not, we 
		// need to get transformed body and process it here, before we decide whether
		// to call HostedDP or BinaryDP. However, we are not doing any extra work.

		// NOTE: We need to make sure this can only be called once so we can't
		// get into an infinite loop if a tag tries to call GetCurrentPage().
		if (m_bMarkupCalled)
		{
			return;
		} else
		{
			m_bMarkupCalled = true;
		}

		boolean bIsText = gwModel.IsText();
		
		if (bIsText)
		{
			/* we get the transformed content */
			Object[][] objArrMarkup = gwModel.GetTextBody();

			if (null != objArrMarkup)
			{
				try
				{
					TagEngineUtils utils = new TagEngineUtils();
					boolean bNeedsCSAPIInNonHostedMode = utils.NeedsCSAPIinNonHostedPage(objArrMarkup);

					if (bNeedsCSAPIInNonHostedMode)
					{
						FragmentListConverter converter = new FragmentListConverter();
						XPArrayList fragments = converter.ConvertMarkupToList(objArrMarkup);

						HeadPreprocessor preprocessor = new HeadPreprocessor();
						// This will ensure that we have a location to place the CSAPI &
						// JS Includes in the head element.
						preprocessor.PreprocessHead(fragments, GetSubSession(), this, true);

						// Use the new markup with additional tags.
						objArrMarkup = converter.ConvertListToMarkup(fragments);
					}

					HTMLElement result = new HTMLElementCollection();

					HTMLElement tagOutput = GatewayUtils.AddMarkupToElement(this,
						objArrMarkup);

					// NOTE: In hosted mode, shared javascript should be displayed
					// by the PlumtreeDP code.  In non-hosted mode, it should be
					// displayed in the headincludes tag.

					// Since hosted mode can include header/footer portlets, we need
					// to be careful not to convert the HTML Element tree to a text
					// string, since that means that the header/footer portlets will
					// break when they try to insert javascript/styles into the head,
					// since it will already have been displayed.

					result.AddInnerHTMLElement(tagOutput);

					gwModel.SetDisplayElement(result);
				}
				catch (HTMLException e)
				{
					gwModel.SetError(0, this.GetString(623, "ptmsgs_portalbrowsingmsgs"), e.getMessage());
				}
			}
			else // Empty body, since objArrMarkup is NULL
			{
				// Set it to an empty element, since null means it was
				// never set.
				gwModel.SetDisplayElement(new HTMLElementCollection());
			}			
		}
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IActivitySpace#GetCurrentPage()
	 */
	public IDisplayPage GetCurrentPage()
	{
		GatewayModel gwModel = (GatewayModel) GetModel(GatewayModel.strModelName);

		CheckIfControlWasCalled(gwModel);

		ProcessMarkupAndUpdateModel(gwModel);

		if (gwModel.IsReturnPage())
		{
			log.Debug("GetCurrentPage(): Return page was requested.");

			return GetPage(GatewayReturnDP.STR_MVC_CLASS_NAME);
		}

		// we can't show standard Error page in devices case
		if (gwModel.IsErrorPage() && !gwModel.IsDeviceUIMode())
		{
			log.Debug("GetCurrentPage(): Error page was requested.");
			
			//Refresh mode has special error page
			if (gwModel.GetRefreshMode() > PlumtreeExtensibility.PT_REFRESH_NONE)
			{
				return GetPage(GatewayRefreshErrorDP.STR_MVC_CLASS_NAME);
			}
			else
			{
				return GetPage(GatewayErrorDP.STR_MVC_CLASS_NAME);
			}
		}

		// Hosted mode is not supported for devices.
		if (!gwModel.IsDeviceUIMode())
		{
			boolean bHostedMode = gwModel.GetIsHostedMode();

			log.Debug("GetCurrentPage(): GetIsHostedMode is set to [" +
				bHostedMode + "]");

			if (bHostedMode && (gwModel.GetRefreshMode() == PlumtreeExtensibility.PT_REFRESH_NONE))
			{
				return GetPage(GatewayHostedDP.STR_MVC_CLASS_NAME);
			}
		}

		return GetPage(GatewayBinaryDP.STR_MVC_CLASS_NAME);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.AActivitySpace#GetStyleSheetLinks()
	 */
	public HTMLElementCollection GetStyleSheetLinks()
	{
		int nCommunityID = ((GatewayModel)this.GetModel(GatewayModel.strModelName)).GetCommunityID();
		
		if (0 < nCommunityID)
		{
			try
			{
				IPTSession ptSession = (IPTSession) this.GetUserSession();
				if (null != ptSession)
				{
					IPTObjectManager ptOM = ptSession.GetCommunities();
					if (null != ptOM)
					{
						IPTCommunityInfo ptCommInfo = ((IPTCommunityManager)ptOM.GetInterfaces("IPTCommunityManager"))
							.CachedOpenCommunityInfo(nCommunityID, false);
						if (null != ptCommInfo)
						{
							int nHeaderID = ptCommInfo.GetHeaderID();
							String strStyleSheet = "";
					
							if (0 < nHeaderID && GetIsAccessStyleStandard())
							{
								try
								{
									strStyleSheet = ptCommInfo.GetStyleSheet();
								}
								catch (Exception e)
								{
									UIError myError = new UIError(UIErrorLevel.FRIENDLY,
										UIErrorScope.CURRENT_USER, e,
										this.GetString(1522, "ptmsgs_portalbrowsingmsgs"),
										this.GetString(1523, "ptmsgs_portalbrowsingmsgs"));
				
									UIErrorHandler.RaiseError(myError, this.GetSubSession(),
										this.GetApplication());
								}
							}
					
							if ((null != strStyleSheet) && !("").equals(strStyleSheet))
							{
								String strSettingLanguage = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE,
										this);
				
								PTStyleSheetLink ptStyleSheetLink = new PTStyleSheetLink(strStyleSheet,
									strSettingLanguage);
										
								try
								{
									HTMLElementCollection pCollection = new HTMLElementCollection();
									pCollection.AddInnerHTMLElement(ptStyleSheetLink.GetLinkAsHTMLElement(ConfigHelper.GetStyleSheetsBaseURL(this)));
					
									return pCollection;
								}
								catch (HTMLException e)
								{
					
									log.Error(e, "Error occured while forming HTML for Community style sheet.");
						
								}
							}
						}
					}
				}
			}
			catch (Exception e)
			{					
				log.Error(e, "Error occured while getting Community style sheet.");
			}
		}

		// use the default subportal stylesheet
		return super.GetStyleSheetLinks();
	}

	/**
	 *
	 */
	public void Init()
	{
		super.Init();

		// Display Pages
		PlumtreeDP myDP = new GatewayHostedDP();
		RegisterPage(myDP);
		RegisterPage(new GatewayBinaryDP());
		RegisterPage(new GatewayReturnDP());
		RegisterPage(new GatewayErrorDP());
		RegisterPage(new GatewayRefreshErrorDP());

		// Gateway Model
		RegisterModel(GatewayModel.strModelName);

		IModel myModel = GetModel(GatewayModel.strModelName);

		// Gateway Control
		RegisterControl(GatewayControl.STR_MVC_CLASS_NAME, myModel);
	}
}