package com.plumtree.portalpages.browsing.myportal;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.statichelpers.*;

/**
 * Control to reassign portlets in a region(s) on a given community or my page
 * using the passed in portlet list(s).
 * 
 * This control is used by the Drag and Drop/Edit Page Flyout feature.
 * This control should handle these cases:
 * 
 * (1) User drags portlet from flyout to a region in the portlet content area.
 * (2) User drags a portlet in the portlet content area and reorders the portlets
 * in the same region.
 * (3) User drags a portlet to a different region in the portlet content area which 
 * causes reordering of both the source and target regions.
 * 
 * @author btse
 */
public class ReorderPortletsControl implements IControl 
{
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
			"com.plumtree.portalpages.browsing.myportal.ReorderPortletsControl");

	public static final String STR_MVC_CLASS_NAME = "ReorderPortlets";
		
	public static final String STR_SOURCE_REGION = "SourceRegion";
	
	public static final String STR_SOURCE_REGION_PORTLET_LIST = "SourcePortletList";

	public static final String STR_TARGET_REGION = "TargetRegion";
			
	public static final String STR_TARGET_REGION_PORTLET_LIST = "TargetPortletList";
	
	private static final String STR_PORTLET_SEPARATOR = "_"; 
		
	/** Space */
	private AActivitySpace m_asOwner;

	/** Model */
	private IMyPortalModel m_asModel;	

	/*
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#CheckActionSecurityAndExecute(com.plumtree.openfoundation.util.XPHashtable)
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments) 
	{
		if(MyPortalModel.IsGuestSession(m_asOwner))
		{
			return null;
		}
		
		// Get Page ID.
		String[] saPageID = (String[]) arguments.GetElement(SetPageControl.STR_PAGE_ID);
		if (saPageID == null)
		{
			log.Error("could not find page ID in query string.");

			return null;
		}
		int nPageID = XPConvert.ToInteger(saPageID[0]);
		
		// Get Community ID
		String[] saCommunityID = (String[]) arguments.GetElement(SetCommunityControl.STR_COMMUNITY_ID);
		int nCommunityID=0;
		if (saCommunityID == null)
		{
			log.Debug("could not find community ID in query string; will assume this is a mypage.");
		} else {
			nCommunityID = XPConvert.ToInteger(saCommunityID[0]);	
		}	
		
		// Get list of portlet IDs from original portlet column.				
		String[] saSourcePortletIDs = (String[]) arguments.GetElement(STR_SOURCE_REGION_PORTLET_LIST);

		if (saSourcePortletIDs == null)
		{
			log.Debug("could not find source portlet list for source region in query string. Assuming reordering of same column.");
		}

		// Get list of portlet IDs from new portlet column.				
		String[] saTargetPortletIDs = (String[]) arguments.GetElement(STR_TARGET_REGION_PORTLET_LIST);

		if (saTargetPortletIDs == null)
		{
			log.Error("could not find target portlet list for target region in query string.");
			return null;
		}
		
		// Get Source column number.
		String[] saSourceRegion = (String[]) arguments.GetElement(STR_SOURCE_REGION);
		int nSourceRegion = -1;

		if (saSourceRegion == null)
		{
			log.Debug("could not find source region number in query string. Assuming reordering of same column.");
		} else {
			nSourceRegion = XPConvert.ToInteger(saSourceRegion[0]);	
		}
		
		// Get Target column number.
		String[] saTargetRegion = (String[]) arguments.GetElement(STR_TARGET_REGION);
		int nTargetRegion = 0;

		if (saTargetRegion == null)
		{
			log.Error("could not find target region number in query string.");
			return null;
		} else {
			nTargetRegion = XPConvert.ToInteger(saTargetRegion[0]);	
		}
		
		// Parse the source and target portlet lists and update the source and target columns
		// to reflect the current display of the portlets on the page.
		XPStringTokenizer tok;
		int[] arrSourcePortletIDs = null;
		int[] arrTargetPortletIDs = null;
		int i;
		
		if (null != saSourcePortletIDs)
		{
			tok = new XPStringTokenizer(saSourcePortletIDs[0], STR_PORTLET_SEPARATOR);
			arrSourcePortletIDs = new int[tok.GetTokenCount()];
			i = 0;

			while (tok.HasMoreTokens()) 
			{
				String oneID = tok.GetNextToken();
				arrSourcePortletIDs[i++] = XPConvert.ToInteger(oneID);
				log.Debug("Source Portlet ID: " + oneID);
			}
		}
        
		if (null != saTargetPortletIDs)
		{
			tok = new XPStringTokenizer(saTargetPortletIDs[0], STR_PORTLET_SEPARATOR);
			arrTargetPortletIDs = new int[tok.GetTokenCount()];
			i = 0;
        
			while (tok.HasMoreTokens()) 
			{
				String oneID = tok.GetNextToken();
				arrTargetPortletIDs[i++] = XPConvert.ToInteger(oneID);
				log.Debug("Target Portlet ID: " + oneID);
			}
		}
        
        log.Debug("Source Region = " + nSourceRegion + ", Target Region = " + nTargetRegion);
		
        // Only reorder the target column if no source column was passed in. This
        // handles the case for user dragging portlet from flyout or reordering of
        // the same column.
        if (-1 == nSourceRegion || nSourceRegion == nTargetRegion)
        {
        	m_asModel.ReassignPortlets(arrTargetPortletIDs, nTargetRegion, nPageID);
        }
        else
        {
        	// Case for user dragging portlet between two different columns. 
        	m_asModel.ReassignPortlets(arrSourcePortletIDs, nSourceRegion, nPageID);
            m_asModel.ReassignPortlets(arrTargetPortletIDs, nTargetRegion, nPageID);
        }
        
		// Get the profile user id, if any. This is needed for the user profile community.
		int nProfileID = 0;

		if (nCommunityID == PT_INTRINSICS.PT_PROFILE_COMMUNITY)
		{
			// Default value of -1 indicates the user is looking at their own profile
			// value of 0 would mean this isn't a profile page.
			nProfileID = -1;
			String[] saProfileID = (String[]) arguments.GetElement(SetCommunityControl.PROFILEID);
			if (saProfileID != null)
			{
				nProfileID = XPConvert.ToInteger(saProfileID[0]);
			}
			
			// dap: force help on user profile community to appropriate page
			m_asOwner.SetHelpTopic("PT_SectionUserProfile");
		}
		// Set the profile ID.
		boolean bSetProfileSuccessful = m_asModel.SetProfileID(nProfileID);

		// Set Raw Headers
		String strHeaders = HttpUtils.GetRawHeaders( m_asOwner.GetCurrentHTTPRequest() );
		m_asModel.SetRawHeaders( strHeaders );
		
		Redirect redirect = null;
		
		if (0 != nCommunityID)
		{
			// Is a community.

			// GET THE REAL PAGEID AFTER GET COMMUNITY (handles the pageid=0 case)

			boolean bSetCommunitySuccessful = false;
			// Avoid useless work if we have to redirect anyway.
			if (bSetProfileSuccessful) 
			{
				bSetCommunitySuccessful = m_asModel.SetCommunity(nCommunityID, nPageID);
			}

			// If either SetPage or SetCommunity fails, we want to redirect here.
			if (!bSetProfileSuccessful || !bSetCommunitySuccessful)
			{
				redirect = new Redirect();
				redirect.SetIsErrorRedirect(true);
				redirect.SetLinkGetCachedSpace(m_asOwner);
				redirect.SetControl(SetCommunityControl.STR_MVC_CLASS_NAME);
				redirect.AddControlArgument(SetCommunityControl.STR_COMMUNITY_ID, nCommunityID);
				redirect.AddControlArgument(SetCommunityControl.STR_PAGE_ID, nPageID);
				if (nProfileID != -1)
				{
					redirect.AddControlArgument(SetCommunityControl.PROFILEID, nProfileID);
				}
			}
		}
		else
		{
			// Is a my page.
			
			// Setup the My Page.
			boolean bSuccessful = m_asModel.SetPage(nPageID);
			
			// If setting the my page was not successful, then redirect to an error page.
			if (!bSuccessful)
			{
				redirect = new Redirect();
				redirect.SetIsErrorRedirect(true);
				redirect.SetLinkGetCachedSpace(m_asOwner);
				redirect.SetControl(SetPageControl.STR_MVC_CLASS_NAME);
				redirect.AddControlArgument(SetPageControl.STR_PAGE_ID, nPageID);
			}
		}

		return redirect;
	}

	/*
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new ReorderPortletsControl();
	}

	/*
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName() 
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IControl#Init(IModel,
	 *      AActivitySpace)
	 */
	public void Init(IModel model, AActivitySpace space)
	{
		m_asModel = (IMyPortalModel) model;
		m_asOwner = space;
	}
}
