package com.plumtree.portalpages.browsing.portalsettings.localesettings;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.pei.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.interpreter.Interpreter;
import com.plumtree.uiinfrastructure.statichelpers.*;

/**
 *
 *
 */
public class LocaleSettingsModel extends EditorModel
	implements ILocaleSettingsModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.portalsettings.localesettings.LocaleSettingsModel");

	public static final String STR_MVC_CLASS_NAME = "Locale";

	/** Default timezone value (not display string) */	
	public static final String DEFAULT_TIMEZONE = "America/Los_Angeles";
	
	/** Web Data Object (needs to be nulled out after execution to prevent leaks) */
	private IWebData m_WebData;

	protected String m_strLocale   = "";
	protected String m_strTimeZone = "";

	/**
	 * Implements some actions to perform on start.
	 *
	 * @param _htFormData
	 *
	 * @return boolean Is true if the current user has the rights to access this Editor.
	 */
	public boolean CheckEditorAccessAndStart(XPHashtable _htFormData)
	{
		return true;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new LocaleSettingsModel();
	}
	
	/**
	 * Inits this Model with the Web Data object.
	 *
	 * @param _webdata
	 */
	public void SetWebData(IWebData _webdata)
	{
		m_WebData = _webdata;
	}

	/**
	 * Implement some code to execute when the user clicks on "Finish".
	 *
	 */
	public void Finish()
	{
		PTPersonalSettingsHelper.SetAndCachePersonalSetting(PersonalSettingNames.LOCALE,
			(String) m_strLocale, m_asOwner);

		// If the selected locale was "Use Browser" or "Use Default", convert this to the
		// appropriate locale and set it as a personal setting (but do not cache it to the DB.
		// This should override the original setting as a runtime setting, but not in the DB.
		// The setting will get overridden the next time the user logs on (for non-guest users).

		String strConvertedLocale = null;
		boolean bUseBrowserLocale = false;

		if ((m_strLocale == null) || (PTPersonalSettingsHelper.LOCALE_USE_DEFAULT_SETTING.equals(m_strLocale)))
		{
			// JF- Get the locale from config.xml
			String strDefaultLocale = ConfigHelper.GetConfigVariableAsString(ConfigConstants.INTERNATIONAL_SECTION,
				ConfigConstants.LOCALE_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE, m_asOwner.GetApplication());

			// Since we've already retrieved the default locale, treat default settings
			// as "use browser" so we can still get a decent locale.
			if ((null == strDefaultLocale) || ("".equals(strDefaultLocale)))
			{
				bUseBrowserLocale = true;
			} else
			{
				// Check whether or not this locale is actually supported by this instance of
				// the server (i.e. we have language files for that locale).  We will have to
				// check later if we are going to use the browser settings.
				if (!PTPersonalSettingsHelper.IsLocaleSupported
					(strDefaultLocale, m_asOwner.GetResourceManager()))
				{
					log.Info("Default locale " + strDefaultLocale +
						" not supported by this server, will use browser settings instead.");

					bUseBrowserLocale = true;
				}

				strConvertedLocale = strDefaultLocale;
			}
		} else if (PTPersonalSettingsHelper.LOCALE_USE_BROWSER_SETTING.equals(m_strLocale))
		{
			bUseBrowserLocale = true;
		}

		if (bUseBrowserLocale)
		{
			// Get the locale from the browser accepts header.
			// This should always return a valid locale for this server.
			strConvertedLocale = PTLoginActions.GetBrowserLocaleSetting(m_asOwner);
		}

		if (null != strConvertedLocale)
		{
			// JF- The Locale Settings page assumes that the locale is lower case...
			strConvertedLocale = XPStringUtility.ToLowerCase(strConvertedLocale);

			// This will cache the locale and language for the UI and as a
			// runtime setting, but not stored it in the DB.
			// This will override the previous runtime settings, which is the correct
			// thing to do for USE_BROWSER and USE_DEFAULT, which are not valid runtime
			// settings.
			PTPersonalSettingsHelper.SetPersonalSetting(PersonalSettingNames.LOCALE,
		 		strConvertedLocale, (IPTSession) m_asOwner.GetUserSession(), m_asOwner.GetSubSession(),
		 		m_asOwner.GetApplication(), m_asOwner.GetResourceManager());
		}

		 // JF- Set the content-language in the header so it can be used by braille devices
		 try
		 {
		 	if (null != m_WebData)
		 	{
				m_WebData.SetContentLanguage(m_strLocale);
		 	}
			
			// JF- Also set the content language on the Session so it can be set again 
			// in the header next request by the interpreter.
			m_asOwner.GetSubSession().SetAttribute(Interpreter.USERLOCALE, m_strLocale);
		 }
		 catch (Exception e)
		 {
			 
				 log.Error(e, "Unable to set the new locale: " + m_strLocale + " in the HTTP header.");
			 
		 }
		
		PTPersonalSettingsHelper.SetAndCachePersonalSetting(PersonalSettingNames.TIMEZONE,
			(String) m_strTimeZone, m_asOwner);

	}
	
	/**
	 *  Null out Web Data so we don't retain (leak) it.
	 */
	public void CleanupTempData()
	{
		super.CleanupTempData();
		m_WebData = null;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public String GetLocaleSetting()
	{
		// We retrieve this from the session info instead of the personal
		// settings, because personal settings could have been overridden
		// at login with the browser settings, but we want what's actually
		// stored in the DB.
		IPTSessionInfo sessionInfo = ((IPTSession)m_asOwner.GetUserSession()).GetSessionInfo();

		// Try to retrieve the setting from IPTSessionInfo.
		Object[][] personalSettings = (Object[][]) sessionInfo.LookupPreference(PersonalSettingNames.LOCALE, 0);

		String localeSetting = (String) PTPersonalSettingsHelper.FindPersonalSetting
			(personalSettings, PersonalSettingNames.LOCALE);

		// Treat null and empty string as use_default_locale
		if ((null == localeSetting) || (0 == XPStringUtility.Compare("", localeSetting)))
		{
			localeSetting = PTPersonalSettingsHelper.LOCALE_USE_DEFAULT_SETTING;
		}

		return localeSetting;
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public String GetTimeZoneSetting()
	{
		// We retrieve this from the session info instead of the personal
		// settings, because personal settings could have been overridden
		// at login with the config file setting, but we want what's actually
		// stored in the DB.
		IPTSessionInfo sessionInfo = ((IPTSession)m_asOwner.GetUserSession()).GetSessionInfo();

		// Try to retrieve the setting from IPTSessionInfo.
		Object[][] personalSettings = (Object[][]) sessionInfo.LookupPreference(PersonalSettingNames.TIMEZONE, 0);

		String timezoneSetting = (String) PTPersonalSettingsHelper.FindPersonalSetting
			(personalSettings, PersonalSettingNames.TIMEZONE);

		if ((null == timezoneSetting) || (timezoneSetting.length() == 0))
		{
			// TODOCR: this should probably use the j_config.xml value
			timezoneSetting = DEFAULT_TIMEZONE;
		}

		return timezoneSetting;
	}

	/**
	 * Calls the right save function according to the page name passed in argument.
	 *
	 * @param _sPageName
	 * @param _htFormData
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		if (_sPageName.equals(LocaleSettingsDP.STR_MVC_CLASS_NAME))
		{
			String[] data;
			data		    = (String[]) _htFormData.GetElement(LocaleSettingsView.HTMLINPUT_LANG);
			m_strLocale 	= data[0];


			data			  = (String[]) _htFormData.GetElement(LocaleSettingsView.HTMLINPUT_TZ);
			m_strTimeZone     = data[0];

			return RepostControl.PAGE_STATUS_VALID;
		}

		return RepostControl.PAGE_STATUS_INVALID;
	}
}

