package com.plumtree.portalpages.browsing.portalsettings.password;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.login.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.activityspace.utils.*;
import com.plumtree.uiinfrastructure.login.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * This control is responsible for caching the user session and replacing it
 * with a guest user session.
 */
public class ExpiredPasswordControl implements ILoginControl, IHTTPControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.portalsettings.password.ExpiredPasswordControl");

	/** MVC object name */
	public static final String STR_MVC_CLASS_NAME = "ExpiredPassword";

	// The cached "real" session
	public static final String SESSION = "ExpiredSession";

	// The cached login token cookie
	public static final String LOGIN_TOKEN_COOKIE = "CachedLoginCookie";

	protected AActivitySpace m_asOwner;

	// Be sure to clear this data when done.
	protected IXPRequest m_xpRequest;
	protected IWebData m_WebData;
	protected IPTSession m_userSession;

	// Whether or not the password has been changed and it's okay to
	// allow access to the "real" user.
	public boolean m_bPasswordChanged = false;

	/**
	 * @return whether or not we should replace the current user session
	 * with a new one.
	 */
	public boolean DoGetSession()
	{
		// If the password hasn't been changed, then we need to swap out the
		// real user session with a guest one.
		if (!m_bPasswordChanged)
		{
			// Check if we're able to login as guest.
			try
			{
				// Connect as the default guest user
				// NOTE: this may not always be correct if there are multiple guest users,
				// but people would only notice if they try to skip the change password page,
				// which they shouldn't be doing anyways.
				m_userSession = PortalObjectsFactory.CreateSession();
				m_userSession.Connect(PT_INTRINSICS.PT_USER_GUEST, null, null);
			}
			catch (Exception e)
			{
				log.Error(e, "Unable to connect as guest.");
				m_userSession = null;
				// Unable to connect to custom guest user, do not log in
				return false;
			}
		}

		return true;
	}

	/**
	 * @return The new user session to use.
	 */
	public Object GetSession()
	{
		// If the password hasn't been changed, then we need to swap out the
		// real user session with a guest one.
		if (!m_bPasswordChanged)
		{
			// Login the guest user.

			LoginResult rReturn = null;
			try
			{
				// Login the default guest user.  This calls the login PEIs.
				// NOTE: this may not always be correct if there are multiple guest users,
				// but people would only notice if they try to skip the change password page,
				// which they shouldn't be doing anyways.
				rReturn = LoginHelper.INSTANCE.AttemptLogin(m_userSession, m_asOwner, m_xpRequest, m_WebData);
			}
			catch (Exception e)
			{
				log.Error(e, "AttemptLogin() failed.  Ignoring.");
			}

			if (!rReturn.m_bSuccess)
			{
				log.Error("GuestLoginControl AttemptLogin() as guest failed: " + rReturn.m_strError + " (Ignoring)");
			}
	
			if (null != rReturn.m_Redirect)
			{
				log.Error("GuestLoginControl AttemptLogin() return redirect ignored.");
			}
		}

		IPTSession userSession = m_userSession;

		// Null out the IHTTPControl data so we don't retain the memory after the
		// request is done (i.e. leak the memory)
		m_xpRequest = null;
		m_WebData = null;
		m_userSession = null;

		// clean up, since the password has been changed and we're done now.
		if (m_bPasswordChanged)
		{
			ISessionManager perSession = m_asOwner.GetPersistentSubSession();
			perSession.RemoveAttribute(SESSION);
			perSession.RemoveAttribute(LOGIN_TOKEN_COOKIE);
			m_bPasswordChanged = false;
		}

		return userSession;
	}

	public void SetRedirectForUseAfterLogin(Redirect r) {
		// TODO Auto-generated method stub
	}

	public Redirect CheckActionSecurityAndExecute(XPHashtable _arguments)
	{
		// This value will be set directly on the control by the model to ensure
		// that the redirect URL cannot be hacked to log the user back in.
		if (m_bPasswordChanged)
		{
			// Since we're not going through the full login process, we need to
			// manually mark this session as valid (i.e. logged in)
			m_asOwner.GetSubSession().SetAttribute(com.plumtree.uiinfrastructure.interpreter.Interpreter.USERSESSIONVALID, Boolean.TRUE);

			// This will get returned by the GetSession() method and the Interpreter will log us in
			// as this user.
			m_userSession = (IPTSession) m_asOwner.GetPersistentSubSession().GetAttribute(SESSION);

			// Set the Login Occurred cookie back to it's normal value (after we removed it earlier).

			// Not setting an expires value on a cookie uses the default behaviour
			// in both Java and .NET (deleted when browser is closed).
			XPCookie cookie = new XPCookie(LoginHandlers.LOGIN_OCCURED_COOKIE_NAME,
				XPConvert.ToString(true));
			cookie.SetPath("/");
			m_WebData.AddCookie(cookie);

			XPCookie cachedLoginCookie = (XPCookie) m_asOwner.GetPersistentSubSession().GetAttribute(LOGIN_TOKEN_COOKIE);

			if (cachedLoginCookie != null)
			{
				// Restore the cached login token cookie.
				// NOTE: We have to rebuild this cookie, because
				// the cookies we receive from the browser are not
				// the same as the ones we want to set.
				XPCookie newCookie = new XPCookie(cachedLoginCookie.GetName(), cachedLoginCookie.GetValue());
				newCookie.SetPath(LoginHelper.LOGIN_TOKEN_COOKIE_PATH);

				int nExpires = cachedLoginCookie.GetExpires();
				// Leave it as default if the value is negative
				if (nExpires >= 0)
				{
					newCookie.SetExpireTime(nExpires);
				}
				
				m_WebData.AddCookie(newCookie);
			}

			// Redirect to the login control and have it redirect to where
			// we are supposed to go.  It will also handle making this a 302
			// redirect so the "new" user will be logged in correctly.
			Redirect myRedirect = new Redirect();  
			myRedirect.SetLinkGetSpaceIfCached(LoginAS.STR_MVC_CLASS_NAME, m_asOwner);
			myRedirect.SetControl(LoginControl.STR_MVC_CLASS_NAME);
			myRedirect.AddControlArgument(LoginHelper.PARAM_DONT_LOGIN_JUST_REDIRECT, true);

			return myRedirect;
		}

		// At this point we know the password has not been changed, so we need to
		// hide the real user session.
		IPTSession ptsession = (IPTSession) m_asOwner.GetUserSession();
		m_asOwner.GetPersistentSubSession().SetAttribute(SESSION, ptsession);

		// Clear the login occurred cookie, otherwise the Interpreter will think our
		// session has timed out and redirect us to the login page instead of allowing
		// access to the change password editor.

		// Not setting an expires value on a cookie uses the default behaviour in both
		// Java and .NET (deleted when browser is closed).
		// Perhaps this should delete the cookie (0 in Java, time in the past on .NET)
		XPCookie clearedCookie = new XPCookie(LoginHandlers.LOGIN_OCCURED_COOKIE_NAME,
			XPConvert.ToString(false));
		clearedCookie.SetPath("/");
		m_WebData.AddCookie( clearedCookie );

		// We also need to cache and clear the RMP login cookie, otherwise similar problems
		// will occur.
		XPCookie loginCookie = LoginHelper.INSTANCE.GetAutologinCookie(m_xpRequest);
		m_asOwner.GetPersistentSubSession().SetAttribute(LOGIN_TOKEN_COOKIE, loginCookie);

		if (loginCookie != null)
		{
			LoginHelper.INSTANCE.ClearAutologinToken(m_xpRequest.GetRequestURL(), m_WebData);
		}

		// Redirect to the Mark as Guest control which will finish hiding the user session.
		// We can't mark the user session as guest here, because after this control is finished,
		// the user will be changed, which would wipe out our attempts to mark it as guest.
		Redirect rReturn = new Redirect();
		rReturn.SetLinkGetCachedSpace(m_asOwner);
		rReturn.SetControl(ExpiredPasswordMarkAsGuestControl.STR_MVC_CLASS_NAME);

		// Do a full 302 redirect so the editor will be fully initialized for the new user.
		rReturn.SetIsHTTPRedirect(true);

		return rReturn;
	}

	/**
	 * Helper method that gets a boolean control argument.
	 * 
	 * @param _arguments The control arguments
	 * @param _strArgument The argument to retrieve
	 * @return the value of the argument.  False if it was not found or
	 * there was an error.
	 */
	public boolean GetArgument(String _strArgument, XPHashtable _arguments)
	{
		String[] sValues = (String[]) _arguments.GetElement(_strArgument);

		if (sValues != null)
		{
			try
			{
				return XPConvert.ToBoolean(sValues[0]);
			} catch (Exception e)
			{
				log.Error(e, "Unable to convert " + _strArgument + " argument to boolean. Ignoring: " + sValues);
			}
		}

		return false;
	}

	public void Init(IModel model, AActivitySpace space) {
		m_asOwner = space;
	}

	public String GetName() {
		return STR_MVC_CLASS_NAME;
	}

	public Object Create() {
		return new ExpiredPasswordControl();
	}

	public void SetHTTPItems(IXPRequest _request, IWebData _pageData) {
		// In a given request, this method is called first.
		m_xpRequest = _request;
		m_WebData = _pageData;
	}
}