package com.plumtree.portalpages.browsing.search.network;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.common.uiparts.*;
import com.plumtree.portalpages.pei.*;
import com.plumtree.portaluiinfrastructure.application.varpacks.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.*;
import com.plumtree.portaluiinfrastructure.helpers.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.opener.*;
import com.plumtree.portaluiinfrastructure.restconsumerframework.pathways.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.portaluiinfrastructure.search.pathways.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.impl.core.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 *
 * "network source" and "outgoing federated portal" are used interchangeably
 * in these comments.
 *
 * Upon initialization, the model queries for available outgoing federated
 * portals. They can be accessed through GetNumSources(), GetSourceID(), and
 * GetSourceName().<p>
 *
 *
 *
 * @author JosephSi
 *
 */
public class NetworkSearchModel extends AClearModel implements INetworkSearchModelRO,
	INetworkSearchModel
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.search.network.NetworkSearchModel");
	
	private static OpenLogger pathwayslog = OpenLogService.GetLogger(
			OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PATHWAYS),
		"com.plumtree.portalpages.browsing.search.network.NetworkSearchModel");

	public static final String STR_MVC_CLASS_NAME = "NetworkSearch";

    /**
     * key used for storing user pref for locations to be checked automatically
     * on the launch page
     **/
    public static final String NETSEARCHPREF_LOCATIONIDS = "netSearchLocationPrefs";


    /**
     * key for cache used to store prefs so we don't hit the db unnecessarily
     **/
    public static final String CACHEDPREFS_KEY = "netSearchLocationsKey";

    protected static final String LOCATIONPREF_DELIMITER = ",";

    public static final int MAX_SEARCHSTRING_LENGTH = 100;

	public static final String SPACE_COLON_SPACE = " : ";

	/**
	 * Outbound search SWS provider CLSID, stored in the search
	 * provider registry table PTFEDSEARCHREG. Objects of type
	 * PTFederatedPortal which have this value as their CLSID
	 * correspond to remote targets that we can search.
	 */
	public static final String OUTGOING_SWS_CLSID = "{4AB4F51D-FF6E-4157-8690-CD6BD2BDCF37}";

	/**
	 * Intrinsic search provider CLSID used to identify inbound
	 * federated portals, stored in the search provider registry
	 * table PTFEDSEARCHREG. Objects of type PTFederatedPortal
	 * which have this value as their CLSID correspond to portals
	 * providing SWS services to the rest of the world.
	 */
	public static final String INCOMING_SWS_CLSID = "{16153543-D12F-460F-9331-E7F1595F3A1E}";

	/**
	 * Integer id to distinguish local portal searching from searching
	 * a particular network source.
	 */
	public static final int THISPORTAL_ID = -2;

	/**
	 * TMP set this flag to true to make all searches run against
	 * the local portal. For debugging only.
	 */
	public boolean m_tmpAlwaysDoLocalSearch = false;

	/** Owning activity space */
	public AActivitySpace m_asOwner;

	public static class TempDataClass
	{
		/** ObjectManager for Federated Portals. Initialized by Init() */
		public  IPTFederatedPortalManager m_ptFedPortalMgr = null;

		/** Owning session */
		public  IPTSession m_ptSession;
	}

	//boolean value to distinguish if this is a pathways search
	protected boolean m_isPathwaysSearch = false;
	
	//boolean value to see if pathways is enabled
	protected boolean m_isPathwaysEnabled = false;

	// pathways format
	protected String m_pathwaysFormat = "XML";

	//pathways startAt value
	protected int m_pathwaysStartAt = 1;

	// pathways view id
	protected String m_pathwaysViewId = "ev";
	
	//pathways page size
	protected int m_numPathwaysResults = 5;

	//pathways order
	protected String m_pathwaysOrder = "DES";

	//pathways orderBy
	protected String m_pathwaysOrderBy = "RANK";

	// pathways totalCount via search
	protected int m_pathwaysTotalCount = 0;

	// pathways ObjectTypes via search
	protected XPArrayList m_pathwaysObjectTypes = null;

	// Pathways Query Arguments making public for unit test usability
	public PathwaysQueryArguments m_pathwaysQueryArgument = null;
	
	protected IPTSearchRequest m_searchRequest = null;
	
	// BannerHelperService
	IBannerSearchHelper m_bannerSearchHelper = null;
	
	// pathways rest api service
	IPathwaysRestAPIService m_pathwaysRESTAPIService;
	
	// pageSize
	protected int m_pageSize = 10;

	/** List of sources which will be searched. Initialized by Prepare() */
	private IXPList m_searchedSources;

	/** Search string. Initialized by Prepare() */
	private String m_strSearchString;

	/** default mode is parallel */
	private String m_viewMode = "parallel";

	/** Map of FedPortal structs. Initialized by Init() */
	private XPHashtable m_outboundFedPortals = null;

    /** List of object ids for the outbound fed portals. Initialized by Init() */
    private XPArrayList m_outboundFedPortalObjectIDs = null;


	/** Map of SearchResultSet structs. Initialized by Prepare() */
	private XPHashtable m_resultSets = null;

	/**
	 * The page number of results currently being displayed. Starts at 0.
	 * So e.g. if m_pageNum=1 and we're showing 5 results per page per source,
	 * then we're currently showing 6 through 10.
	 **/
	private int m_pageNum = 0;

	/** max number of results to display on a single results page from a source **/
	private int m_resultsPerPagePerSource = 5;

	public static final String NUM_RESULTS_PREFS_KEY = "NumResultsFedSearchPref";


    /**
     * if true, then the user entered a very long search string which
     * was truncted to MAX_SEARCHSTRING_LENGTH characters
     **/

    private boolean m_searchStringTruncated;

	private int m_nCurrentUserID = 819748;
	
	public NetworkSearchModel()
	{
		m_viewMode = NetworkSearchControl.VIEWMODE_PARALLEL;
		m_outboundFedPortals = new XPHashtable();
		m_outboundFedPortalObjectIDs = new XPArrayList();
		m_resultSets = new XPHashtable();
	}
	public void SetNumberResults(int num)
	{
		m_resultsPerPagePerSource = num;
	}

	public int GetNumberResults()
	{
		return m_resultsPerPagePerSource;
	}


	/** TMP A debugging method which will go away when the code is complete */
	public void SetAlwaysDoLocalSearch(boolean flag)
	{
		m_tmpAlwaysDoLocalSearch = flag;
	}


	/**
	 * Return true if the model holds a valid result set for a given source.
	 * False if there was an error in getting results from the source.
	 * Use String GetErrorFromSource(int) to get the error message in the
	 * event of invalid results.
	 */
	public boolean isResultSetValid(int sourceID)
	{
		SearchResultSet resultSet = (SearchResultSet) m_resultSets.GetElement(new Integer(
					sourceID));

		if (resultSet != null)
		{
			return resultSet.isValid();
		}

		return false;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean AreAllResultSetsValid()
	{
		IXPEnumerator resultSetEnum = m_resultSets.GetElements();

		while (resultSetEnum.MoveNext())
		{
			SearchResultSet resultSet = (SearchResultSet) resultSetEnum.GetCurrent();

			if (!resultSet.isValid())
			{
				return false;
			}
		}

		return true;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()
	 */
	public void CleanupTempData()
	{
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
	 */
	public void CleanupAllData()
	{
	}

	/**
	 *
	 */
	public void ClearSearchedSourceList()
	{
		m_searchedSources.Clear();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new NetworkSearchModel();
	}

	/**
	 * Returns null if we hold no result info for the given source or if
	 * an error occurred while searching the given source.
	 */
	public String GetDescriptionForResultFromSource(int sourceId, int resultId)
	{
		SearchResultSet resultSet = (SearchResultSet) m_resultSets.GetElement(new Integer(
					sourceId));

		if ((resultSet != null) && resultSet.isValid())
		{
			return resultSet.GetDescription(resultId);
		}

		return null;
	}

	/**
	 * Return any error message for a given source, used when the results
	 * for the source were invalid. !!! localization
	 * Can return null if there is no message.
	 * This error message was actually not localized in 4.5WS, just printed
	 * to the screen.
	 */
	public String GetErrorFromSource(int sourceID)
	{
		SearchResultSet resultSet = (SearchResultSet) m_resultSets.GetElement(new Integer(
					sourceID));

		if (resultSet != null)
		{
			return resultSet.GetErrorMessage();
		}

		return null;
	}

    public HTMLElement GetErrorLinkFromSource(int sourceID) {
	SearchResultSet resultSet = (SearchResultSet) m_resultSets.GetElement(new Integer(sourceID));

		if (resultSet != null)
		{
			return resultSet.GetErrorLink();
		}

		return null;
	}

	public String GetFormattedError(int sourceId)
	{
		String retval = XPResourceManager.FormatMessage(m_asOwner.GetString(1640, "ptmsgs_portalbrowsingmsgs"),
						GetSourceName(sourceId) + SPACE_COLON_SPACE +
						GetErrorFromSource(sourceId));
		return retval;
	}

	/**
	 *
	 *
	 * @param sourceId
	 *
	 * @return
	 */
	public int GetFirstDisplayedResultFromSource(int sourceId)
	{
		SearchResultSet resultSet = (SearchResultSet) m_resultSets.GetElement(new Integer(
					sourceId));

		if (resultSet._searchResponse != null)
		{
			return resultSet._searchResponse.GetSkipped() + 1;
		}

		return 0;
	}

	/**
	 * Overall rank of first result on page to be displayed. We
	 * calculate this by summing the number of skipped results from
	 * each source
	 **/
	public int GetFirstDisplayedResultOverall()
	{
		int overallTotal		    = 0;
		IXPEnumerator resultSetEnum = m_resultSets.GetElements();

		while (resultSetEnum.MoveNext())
		{
			SearchResultSet resultSet = (SearchResultSet) resultSetEnum.GetCurrent();

			if (resultSet._searchResponse != null)
			{
				// the way the search api works, if the first result
				// in the requested range is greater than the total number
				// of results, then GetSkipped() will return 0. So we have
				// to check for this case and adjust accordingly.

				/*
				if ( (resultSet._searchResponse.GetTotalMatches() > 0 ) &&
				(resultSet._searchResponse.GetSkipped()==0) &&
				(resultSet._searchResponse.GetResultsReturned()==0) ) {
				*/
				if ((resultSet._searchResponse.GetTotalMatches() < (m_pageNum * m_resultsPerPagePerSource)))
				{
					overallTotal += resultSet._searchResponse.GetTotalMatches();
				}
				else
				{
					overallTotal += resultSet._searchResponse.GetSkipped();
				}
			}
		}

		// add one since we're returning the rank of the first result on
		// the page, not the last result of the page before
		return overallTotal + 1;
	}

	/**
	 * Returns null if we hold no result info for the given source or if
	 * an error occurred while searching the given source.
	 */
	public HTMLImg GetImageForResultFromSource(int fedPortalID, int rank)
	{
		SearchResultSet resultSet = (SearchResultSet) m_resultSets.GetElement(new Integer(
					fedPortalID));

		HTMLImg img=null;
		if ((resultSet != null) && resultSet.isValid())
		{
			img=resultSet.GetImage(rank, m_asOwner);
		}

		// substitute the default icon if there isn't one
		if (img==null) {
		    img=new HTMLImg("/portal/public/img/icon_remote_source.gif","");
		}

		return img;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Number of results actually returned in the search response
	 * from this source, i.e., the number of results shown on the page
	 * at the moment. For instance, there might be a total of 8 results
	 * matching the query from the source. If we show 5 results per page,
	 * then this method will return 5 if we're on page 1 and 3 if we're
	 * on page 2.
	 **/
	public int GetNumDisplayedResultsFromSource(int sourceId)
	{
		SearchResultSet resultSet = (SearchResultSet) m_resultSets.GetElement(new Integer(
					sourceId));

		if (resultSet._searchResponse != null)
		{
			return resultSet._searchResponse.GetResultsReturned();
		}

		return 0;
	}

	/**
	 * Overall number of results on displayed page. We
	 * calculate this by summing the number of displayed results from
	 * each source.
	 **/
	public int GetNumDisplayedResultsOverall()
	{
		int overallTotal		    = 0;
		IXPEnumerator resultSetEnum = m_resultSets.GetElements();

		while (resultSetEnum.MoveNext())
		{
			SearchResultSet resultSet = (SearchResultSet) resultSetEnum.GetCurrent();

			if (resultSet._searchResponse != null)
			{
				overallTotal += resultSet._searchResponse.GetResultsReturned();
			}
		}

		return overallTotal;
	}

	/**
	 * Returns -1 if we hold no result info for the given source or if
	 * an error occurred while searching the given source.
	 */
	public int GetNumResultsFromSource(int sourceID)
	{
		SearchResultSet resultSet = (SearchResultSet) m_resultSets.GetElement(new Integer(
					sourceID));

		if ((resultSet != null) && resultSet.isValid())
		{
			return resultSet.GetNumTotalMatches();
		}

		return -1;
	}

	/**
	 * Overall number of results. Sum of the number of results from
	 * each source.
	 **/
	public int GetNumResultsOverall()
	{
		int overallTotal		    = 0;
		IXPEnumerator resultSetEnum = m_resultSets.GetElements();

		while (resultSetEnum.MoveNext())
		{
			SearchResultSet resultSet = (SearchResultSet) resultSetEnum.GetCurrent();

			if (resultSet._searchResponse != null)
			{
				overallTotal += resultSet._searchResponse.GetTotalMatches();
			}
		}

		return overallTotal;
	}

	/**
	 * Return the number of available outbound federated portals
	 * NOTE: total does *not* include the local portal (i.e, the first
	 * check box on the launch page)
	 **/
	public int GetNumSources()
	{
		return m_outboundFedPortals.GetSize();
	}

	/** Get the page number for the current results page being displayed */
	public int GetPageNum()
	{
		return m_pageNum;
	}

	/** Return the search string from the latest call to Prepare().
	 * Note: we return an html-encoded version, rather than the original
	 * string
	 **/
	public String GetSearchString()
	{
		return XPHttpUtility.HtmlEncode(m_strSearchString);
	}

    /**
     * Return the raw, unencoded search string from the latest call
     * to Prepare()
     **/
    public String GetUnencodedSearchString()
    {
	return m_strSearchString;
    }

	/**
	 * Return the object ids for the network sources which will be/were
	 * searched by this model.
	 */
	public IXPList GetSearchedSources()
	{
		return m_searchedSources;
	}

	/**
	 * Return the objectid of the ith available outbound federated portal.
	 */
	public int GetSourceID(int id)
	{
		if (id == THISPORTAL_ID)
		{
			// our local portal is not a true federated portal with a db
			// object id, so just return this made-up id
			return THISPORTAL_ID;
		}

		FedPortal fedPortal = (FedPortal) m_outboundFedPortals.GetElement( m_outboundFedPortalObjectIDs.GetElement(id) );

		if (fedPortal == null)
		{
			return -1;
		}

		return fedPortal.GetID();
	}

    /**
     * Return the object id of the fed portal's web service
     **/
    private int GetSourceWebServiceID(int id)
    {
		if (id==THISPORTAL_ID) {
			throw new XPException("Unable to retrieve web service ID because requested id is " + THISPORTAL_ID);
		    //return null;
		}

		FedPortal fedPortal = (FedPortal) m_outboundFedPortals.GetElement(new Integer(id));

		if (fedPortal==null) {
			throw new XPException("Failed to retrieve web service ID.");
		    //return null;
		}
		return fedPortal.GetWebServiceID();
    }

	/**
	 * Return the name of the ith available outbound federated portal.
	 */
	public String GetSourceName(int id)
	{
		if (id == THISPORTAL_ID)
		{
                  return m_asOwner.GetString(794, "ptmsgs_portalbrowsingmsgs");
		}

		FedPortal fedPortal = (FedPortal) m_outboundFedPortals.GetElement(new Integer(id));

		if (fedPortal == null)
		{
			return null;
		}
		return XPHttpUtility.HtmlEncode(fedPortal.GetName());
	}

	/**
	 * Returns null if we hold no result info for the given source or if
	 * an error occurred while searching the given source.
	 */
	public String GetTitleForResultFromSource(int sourceId, int resultId)
	{

	    SearchResultSet resultSet = (SearchResultSet) m_resultSets.GetElement(new Integer(
											     sourceId));

	    if ((resultSet != null) && resultSet.isValid())
		{
		    return resultSet.GetTitle(resultId);
		}

	    return null;
	}

	/**
	 * Returns null if we hold no result info for the given source or if
	 * an error occurred while searching the given source.
	 */
	public HTMLAnchor GetURLForResultFromSource(int fedPortalID, int rank, boolean popup)
	{
		SearchResultSet resultSet = (SearchResultSet) m_resultSets.GetElement(new Integer(
					fedPortalID));

		if ((resultSet != null) && resultSet.isValid())
		{
			return resultSet.GetLink(rank, popup, m_nCurrentUserID);
		}

		return null;
	}

	/** Get the mode in which we are displaying results (collated or parallel) */
	public String GetViewMode()
	{
		return m_viewMode;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IModel#Init(AActivitySpace)
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner	    = parent;
		TempDataClass tdc = (TempDataClass)GetTempData();
		tdc.m_ptSession     = (IPTSession) m_asOwner.GetUserSession();
		m_nCurrentUserID = tdc.m_ptSession.GetSessionInfo().GetCurrentUserID();

		m_searchedSources = new XPArrayList();

		IPTObjectManager ptOM = tdc.m_ptSession.GetFederatedPortals();

		tdc.m_ptFedPortalMgr = (IPTFederatedPortalManager) ptOM;

		// Initialize our internal list of available outbound federated portals
		GetAndSaveOutgoingFederatedPortals(ptOM);
	}

	/**
	 * Call this before using the model to retrieve search results
	 * for any results Section page. This method performs the necessary
	 * behind the scenes search requests needed to answer questions about
	 * results from the desired range and from the desired sources.
	 * !!! Needs language.
	 */
	public void Prepare(String searchString, IXPList sourceIDs, int pageNum)
	{
		// I'm not sure this is the right long-term solution
		m_resultSets.Clear();

		m_strSearchString     = searchString;
		m_searchedSources     = sourceIDs;
		m_pageNum			  = pageNum;

		// if the user failed to check anything, then bail out now- the
		// view will tell them they need to check at least one source. (Note
		// that this should only happen in 508 mode- in regular mode, the
		// javascript client-side validation should prevent us from getting
		// this far with nothing checked.
		if (m_searchedSources.GetSize() == 0)
		{
			return;
		}

		// if we're only searching one source, then we show
		// 20 results per page. Otherwise, we show 5 results per
		// page per source
		/*if (sourceIDs.GetSize() == 1)
		{
			m_resultsPerPagePerSource = 20;
		}
		else
		{
			m_resultsPerPagePerSource = 5;
		}*/

		int startIndex = pageNum * m_resultsPerPagePerSource;

		// subtract 1 since we start at 0
		int endIndex = ((pageNum + 1) * m_resultsPerPagePerSource) - 1;

		//PEI : NetworkSearchActions
		//BrianH - This set of code will process each of the available NetworkSearchActions
		//PEIs and return a object array with a possible modified search string
		//and IXPList of Source IDs.
		Object[] oa;
		Object o;
		INetworkSearchActions iActions;
		IApplication application = m_asOwner.GetApplication();
		NetworkSearchArguments returnNetworkObjects;

		oa = (Object[]) application.GetCachingManager().GetEntry("NetworkSearchActions");

		if (oa != null)
		{
			for (int x = 0; x < oa.length; x++)
			{
				o = oa[x];

				if(o != null)
				{
					iActions = (INetworkSearchActions) o;

					//Grab the NetworkSearchArguments object and use each item to determine if changes are made.
					returnNetworkObjects = iActions.OnBeforeNetworkSearchProcess(m_asOwner, 
						(IPTSession) m_asOwner.GetUserSession(), searchString, sourceIDs);
					if (returnNetworkObjects != null)
					{
						//Grab each of the individual item from the return object.
						String returnNetworkSearchString = returnNetworkObjects.GetNetworkSearch();
						IXPList returnList = returnNetworkObjects.GetNetworkSources();

						if (returnNetworkSearchString != null && !("").equals(returnNetworkSearchString))
						{
							//Set original SearchString to modified String
							searchString = returnNetworkSearchString;
						}
						if (returnList != null)
						{
							//Set original XPList to modified List.
							sourceIDs = returnList;
						}
					}
				 }
			}
		}
		//PEI : NetworkSearchActions : END

		// First cut, simple implementation is to always perform
		// the search indicated, no caching.
		IXPEnumerator idEnum = sourceIDs.GetEnumerator();

		while (idEnum.MoveNext())
		{
			int sourceID = XPConvert.ToInteger(idEnum.GetCurrent());

			// Perform the search for this source
			SearchResultSet resultSet = doSearch(sourceID,
					searchString, startIndex, endIndex);

			// Make sure errors appear at top of page as well.
			if(!resultSet.isValid())
			{
				String msg = XPResourceManager.FormatMessage(m_asOwner.GetString(1640, "ptmsgs_portalbrowsingmsgs"),
				GetSourceName(resultSet._sourceID) + SPACE_COLON_SPACE +
				resultSet.GetErrorMessage());

                                //!!! This should be fixed more cleanly later.  The resultSet.GetErrorMessage() call
                                //!!! sometimes returns an HTML comment embedded in the string with a technical error
                                //!!! message for administrators.  This is fine when the string is simply inserted in
                                //!!! the page (as NetworkSearchResultsView does), but when we use it as the text for
                                //!!! a UIError, the HTML comment tag gets HTML-encoded, so the whole error gets shown
                                //!!! on the page in giant red text.  To avoid this, just cut off any HTML comment at
                                //!!! the end of msg.
                                {
                                  int htmlCommentStart = XPStringUtility.IndexOf(msg, "<!--");
                                  if ((htmlCommentStart > 0) &&
                                      (XPStringUtility.EndsWith(msg, "-->"))) {
                                    msg = XPStringUtility.Substring(msg, 0, htmlCommentStart);
                                  }
                                }

				UIError myError = new UIError(UIErrorLevel.FRIENDLY,
						UIErrorScope.CURRENT_USER, null, msg, msg);
				UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
					m_asOwner.GetApplication());

			}
			m_resultSets.PutElement(new Integer(sourceID), resultSet);

		}
	}
	 // Prepare

	/** Set the page number for the current results page being displayed */
	public void SetPageNum(int pageNum)
	{
		m_pageNum = pageNum;
	}

	/** Set the mode in which we are displaying results (collated or parallel) */
	public void SetViewMode(String viewMode)
	{
		m_viewMode = viewMode;
	}


	/**
	 * Return true if the given portal is an outgoing portal. This logic
	 * was copied from the 4.5WS fedsearch.asp.
	 */
	protected boolean isOutgoingFedPortal(String clsid)
	{
		boolean result = false;

		if ((clsid != null) && clsid.equals(OUTGOING_SWS_CLSID))
		{
			// This fed portal uses the SWS search provider
			result = true;
		}
		else if ((clsid != null) && clsid.equals(INCOMING_SWS_CLSID))
		{
			// This fed portal object is incoming only and does not have a search provider
			result = false;
		}

		// else this is a malformed fedportal object and we don't know what
		// to do with it. So we return false
		return result;

		/**
		 // For posterity, here is the logic stolen from 4.5 WS
		if (clsid != null && clsid.equals(HTTP_SEARCH_PROVIDER_CLSID)) {
		    // This fed portal uses the http search provider
		    result = true;
		}
		else if (clsid != null && clsid.equals(INCOMING_ONLY_CLSID)) {
		    // This fed portal object is incoming only and does not have a search provider
		    result = false;
		}
		else if (identity != null && identity.trim().length() != 0) {
		    // This is an old-style fed portal object and its outgoing name is not blank
		    result = true;
		}
		else {
		    // This is an old-style fed portal object and its outgoing name is blank
		    result = false;
		}
		*/
	}
	 // isOutgoingFedPortal

	/**
	 * Get and save the id, name, providerclsid, and thisportalidentity
	 * of all available federated portals (inbound and outbound).
	 * Returns the number of portals found.
	 * Use GetFedPortalXXX(id) to get info from a particular fedportal
	 */
	protected int GetAndSaveOutgoingFederatedPortals(IPTObjectManager ptOM)
	{
		// Use the object manager to query for all available fed portals
		// Some are inbound, some are outbound
		IPTQueryResult ptQR = ptOM.Query(PT_PROPIDS.PT_PROPID_NAME | // propIDMask
				PT_PROPIDS.PT_PROPID_OBJECTID |
				PT_PROPIDS.PT_PROPID_FEDERATEDPORTAL_THISPORTALIDENTITY |
				PT_PROPIDS.PT_PROPID_FEDERATEDPORTAL_SEARCHPROVIDERCLSID |
PT_PROPIDS.PT_PROPID_FEDERATEDPORTAL_WEBSERVICEID, -1, // folderID, return all folders
				PT_PROPIDS.PT_PROPID_NAME, // orderBy
				0, // skipRows
				-1, // maxRows, return all rows
				null); // query filter

		for (int i = 0; i < ptQR.RowCount(); i++)
		{
			String strName     = ptQR.ItemAsString(i, PT_PROPIDS.PT_PROPID_NAME);
			int intgrID	   = ptQR.ItemAsInt(i,
					PT_PROPIDS.PT_PROPID_OBJECTID);
			String strCLSID    = ptQR.ItemAsString(i,
					PT_PROPIDS.PT_PROPID_FEDERATEDPORTAL_SEARCHPROVIDERCLSID);
			String strIdentity = ptQR.ItemAsString(i,
					PT_PROPIDS.PT_PROPID_FEDERATEDPORTAL_THISPORTALIDENTITY);
			int intgrWebServiceID = 0;
			//webserviceid is allowed to be null; this will throw
			//an exception if it is
			try {
				Object objWebServiceId = ptQR.ItemAsObject(i,PT_PROPIDS.PT_PROPID_FEDERATEDPORTAL_WEBSERVICEID);
				if (null != objWebServiceId) {
					intgrWebServiceID = XPConvert.ToInteger(objWebServiceId);
				}
			}
			catch (Throwable e) {
				// it was null or not a number;
			}

			if (isOutgoingFedPortal(strCLSID))
			{
			    boolean disabled=CheckIfDisabled(intgrWebServiceID);
			    //boolean disabled=false;
				FedPortal fedPortal =
				    new FedPortal(intgrID, strName,
						  strCLSID, strIdentity, intgrWebServiceID, disabled);

				m_outboundFedPortals.PutElement(new Integer(intgrID),fedPortal);
				m_outboundFedPortalObjectIDs.Add(new Integer(intgrID));
			}
		}

		return m_outboundFedPortals.GetSize();
	}
	 // GetFederatedPortals

    /**
     * returns true if the fed portal's associated web service
     * is currently disabled
     **/
    public boolean IsDisabled(int objectId) {
	FedPortal fedPortal = (FedPortal) m_outboundFedPortals.GetElement(new Integer(objectId));
	if (fedPortal==null) {
	    return false;
	}
	return fedPortal.IsDisabled();
    }

    /**
     * returns true if the outbound fed portals associated web
     * service is disabled
     **/
    protected boolean CheckIfDisabled(int objectIDintgr) {
	// should only be 0 if webserviceid was null, which shouldn't
	// happen if we're in this method-may want to log a warning
	// here.
	if (objectIDintgr == 0) {
	    return false;
	}
	TempDataClass tdc = (TempDataClass)GetTempData();
	IPTObjectManager wsManager=tdc.m_ptSession.GetWebServices();
	Object[][] filter = PlumtreeHelpers.GetQueryFilter(PT_PROPIDS.PT_PROPID_OBJECTID,
							   PT_FILTEROPS.PT_FILTEROP_EQ,
							   new Integer(objectIDintgr));

	// query the webservices table to find out if this web service
	// is disabled

	IPTQueryResult ptQR = wsManager.Query(PT_PROPIDS.PT_PROPID_WEBSERVICE_ENABLED,// really a boolean indicating whether the ws is enabled, 1 if it is, 0 if not
					 -1, // folderID, return all folders
					      (int[]) null,   // don't care about order-by
					 0, // skipRows
					 -1, // maxRows, return all rows
					 filter); // query filter

	// AGC- changed the following code because using Integer = null was blowing up
	// the jump.
	int intgrEnabled=0;
	try {
	    intgrEnabled	   =
	    ptQR.ItemAsInt(0,PT_PROPIDS.PT_PROPID_WEBSERVICE_ENABLED);
	}
	catch (Exception e) {

	    return false;
	}



	if (intgrEnabled==1) {

	    return false;
	}
	else {
	    return true;
	}

    }
    
    /**
     * Initialization code for constructing a pathways search request, much like in SearchResultsModel.PathwaysReinit()
     */
    protected void LocalPathwaysSearchInit()
    {
	    m_bannerSearchHelper = BannerSearchHelperService.GetInstance();
	    m_pathwaysObjectTypes = new XPArrayList();

	    //m_resultsPerPagePerSource is the value of the number of results to return for a federated search.
	    m_pathwaysQueryArgument = new PathwaysQueryArguments(m_pathwaysFormat, m_pathwaysOrder, m_pathwaysOrderBy, 
	    		m_pathwaysStartAt, m_pathwaysViewId, null, m_resultsPerPagePerSource, m_pathwaysObjectTypes);
	    
	    try {
			TempDataClass tdc = (TempDataClass)GetTempData();

			//we need to now instantiate a Pathways search request now that we have the IPTSearchRequest object.
	    	InternalSession m_InternalSession = ((IInternalPTSession)tdc.m_ptSession).GetInternalSession();
	    	m_searchRequest = (IPTSearchRequest)new PathwaysSearchRequest(m_InternalSession, m_pathwaysQueryArgument);
	    }
	    catch (Exception ex)
	    {
	    	pathwayslog.Error(ex, "NetworkSearchModel.LocalPathwaysSearchInit:: an error occurred while try to " +
	    			"initialize the pathways search request");
	    	SearchUIUtility.RethrowCOMExecutionException(ex, m_asOwner);
	    }
    }
    
    /**
     * This function handles the settings of the local portal search
     * @param searchString the search text value
     * @param startIndex the start of the search index value
     * @param endIndex the end of the search index value
     */
    protected void SetLocalSearchSettings(String searchString, int startIndex, int endIndex)
    {
		// The results range we want
    	m_searchRequest.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_SKIPRESULTS,
			new Integer(startIndex));
    	m_searchRequest.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_MAXRESULTS,
			new Integer((endIndex + 1) - startIndex));

		// Portal only
    	m_searchRequest.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_APPS,
			new Integer(PT_SEARCH_APPS.PT_SEARCH_APPS_PORTAL));

		// show kwics. Even though the results from the other sources
		// won't be keyword-highlighted, we may as well highlight
		// the ones from the portal
    	m_searchRequest.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_KWIC,
			Boolean.TRUE);

		// Cards only
    	m_searchRequest.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES,
				    new int[] { PT_CLASSIDS.PT_CATALOGCARD_ID } );    	
    }
    
    /**
     * This function sets up a pathways integrated federated search request
     * @param searchString the search term 
     * @param startIndex the start index number of the search
     * @param endIndex the end index number of the search
     * @return the SearchResultSet of the pathways search results
     */    
    protected SearchResultSet doLocalPathwaysSearch(String searchString, int startIndex, int endIndex)
    {    	
    	LocalPathwaysSearchInit();
    	
		SearchResultSet resultSet = new SearchResultSet(m_asOwner);
		resultSet._isValid		    = false;
		resultSet._searchString     = searchString;
		resultSet._startIndex	    = startIndex;
		resultSet._endIndex		    = endIndex;
		resultSet._sourceID		    = NetworkSearchModel.THISPORTAL_ID;
		resultSet._isLocal=true;
		
		PathwaysSearchResponse resp = null;
		PathwaysSearchRestData[] searchData = null;
		
		try
		{			
			//calling function to set the local portal search settings for this federated search
			SetLocalSearchSettings(searchString, startIndex, endIndex);

			//set the start index (not sure why it would be different than 0, but just in case, we will set the 
			//pathways argument to this value also
			m_pathwaysQueryArgument.SetStartAt(startIndex);
						
			int maxResults = (endIndex +1) - startIndex;
			//setting page size in pathways
			m_pathwaysQueryArgument.SetResultCount(maxResults);

			//setting documents as the object types to search for only
			m_pathwaysObjectTypes.Add(PT_CLASSIDS.PT_CATALOGCARD_ID);
			m_pathwaysQueryArgument.SetObjectTypes(m_pathwaysObjectTypes);
	
			//set the pathways search query string
			m_pathwaysQueryArgument.SetQuery(searchString);
			
			//execute the pathways search
			searchData = m_pathwaysRESTAPIService.DoSearch(m_pathwaysQueryArgument);

			//setting totalCount of search results
			m_pathwaysQueryArgument.SetTotalCount(m_pathwaysRESTAPIService.GetTotalCount());

			resp = new PathwaysSearchResponse(searchData, m_pathwaysQueryArgument);
			resultSet._searchResponse = (IPTSearchResponse)resp;

			if (resultSet._searchResponse != null)
			{
				resultSet._isValid = true;
			}
			else
			{
				// Is it possible to have a null SearchResponse but no
				// thrown exception? (CraigS says no)
				
				pathwayslog.Error("NetworkSearchModel :  Pathways search request returned null PathwaysSearchResponse");
				resultSet._errorMessage     = "NetworkSearchModel :  Pathways search request returned null PathwaysSearchResponse"; // !!!
				resultSet._isValid		    = false;
			}
		}
		catch (Exception e)
		{
			pathwayslog.Error(e, "NetworkSearchModel :  Pathways search error occurred.");
			resultSet._errorMessage     = SearchUIUtility.GetI18NException(e,m_asOwner).GetMessage();
			resultSet._isValid		    = false;
		}

		return resultSet;
    }

	/**
	 * Perform a search against the local portal.
	 */
	protected SearchResultSet doLocalSearch(String searchString,
		int startIndex, int endIndex)
	{
		SearchResultSet resultSet = new SearchResultSet(m_asOwner);
		resultSet._isValid		    = false;
		resultSet._searchString     = searchString;
		resultSet._startIndex	    = startIndex;
		resultSet._endIndex		    = endIndex;
		resultSet._sourceID		    = NetworkSearchModel.THISPORTAL_ID;
		resultSet._isLocal=true;

		try
		{
			TempDataClass tdc = (TempDataClass)GetTempData();
			m_searchRequest = tdc.m_ptSession.GetSearchRequest();

			//call function to set the local portal search settings
			SetLocalSearchSettings(searchString, startIndex, endIndex);
			
			IPTSearchQuery q = m_searchRequest.CreateBasicQuery(searchString, null);

			resultSet._searchResponse = m_searchRequest.Search(q);

			if (resultSet._searchResponse != null)
			{
				resultSet._isValid = true;
			}
			else
			{
				// Is it possible to have a null SearchResponse but no
				// thrown exception? (CraigS says no)
				log.Error("NetworkSearchModel :  SearchRequest.Search() returned null SearchResponse");
				resultSet._errorMessage     = "NetworkSearchModel :  SearchRequest.Search() returned null SearchResponse"; // !!!
				resultSet._isValid		    = false;
			}
		}
		catch (Exception e)
		{
			log.Error(e, "NetworkSearchModel :  SearchRequest.Search() error occurred.");
			resultSet._errorMessage     = SearchUIUtility.GetI18NException(e,m_asOwner).GetMessage(); //e.getMessage();
			resultSet._isValid		    = false;
		}

		return resultSet;
	}
	 // doLocalSearch

	/**
	 * Perform an actual search for the indicated source.
	 * !!! Needs to return some sort of error code?
	 */
	protected SearchResultSet doSearch(int sourceID, String searchString,
		int startIndex, int endIndex)
	{
		m_pathwaysRESTAPIService = PathwaysRestAPIService.GetInstance(m_asOwner);
    	m_isPathwaysEnabled = m_pathwaysRESTAPIService.IsPathwaysEnabled();
		
		// TMP For debugging, do a local search regardless of sourceID
		if (m_tmpAlwaysDoLocalSearch)
		{
			SearchResultSet result = null;
			if (m_isPathwaysEnabled) 
			{
				result = doLocalPathwaysSearch(searchString, startIndex,	endIndex);
			}
			else
			{
				result = doLocalSearch(searchString, startIndex,	endIndex);
			}
				
			result._sourceID = sourceID;
			return result;
		}

		// Handle the special case where sourceID indicates "this portal"
		if (sourceID == NetworkSearchModel.THISPORTAL_ID)
		{
			if (m_isPathwaysEnabled)
			{
				return doLocalPathwaysSearch(searchString, startIndex, endIndex);
			}
			else
			{
				return doLocalSearch(searchString, startIndex, endIndex);
			}
		}


		SearchResultSet resultSet = new SearchResultSet(m_asOwner);
		resultSet._isValid		  = false;
		resultSet._searchString   = searchString;
		resultSet._startIndex     = startIndex;
		resultSet._endIndex		  = endIndex;
		resultSet._sourceID		  = sourceID;

		try
		{
			// Open a FedSearch and add this source
			// !!! Here is an architectural issue. It is possible to Add() a
			// bunch of sourceIDs to a fedsearch object and then (I guess)
			// execute them in parallel. Right now because of the way
			// doSearch() is set up I'm doing a separate call for each. To
			// change this I would have to pop up a level and have Prepare()
			// add sourceIDs for any sources not
			// found in the cache, then do them all at once.
			IPTState ptState			   = (IPTState) (PortalObjectsFactory.CreateState());
			TempDataClass tdc = (TempDataClass)GetTempData();
			IPTFederatedSearch ptFedSearch = tdc.m_ptFedPortalMgr.OpenFederatedSearch(PT_FEDERATEDSEARCHMODES.PT_FEDERATEDSEARCHMODE_SIMPLE,
					ptState);
			ptFedSearch.Add(sourceID);

			// Perform the search
			ptFedSearch.ExecuteSimpleSearch(searchString, startIndex, // nSkip
				(endIndex + 1) - startIndex); // nMax

			IPTState objAppDataObject;
			objAppDataObject     = AppDataObject.GetAppDataObject(0, 0,
					ConfigHelper.GetApplicationBaseURL(m_asOwner), m_asOwner);

			resultSet._searchResponse = ptFedSearch.GetSearchResult(sourceID,
					objAppDataObject);

			if (resultSet._searchResponse != null)
			{
				resultSet._isValid = true;
			}
			else
			{



			    int errorCode=ptFedSearch.GetLastErrorCode(sourceID);


			    switch(errorCode) {

				// this error only applies to portal-to-portal
				// searches. It occurs when the Plumtree
				// search server is not contactable
			    case (PT_RESULTCODES.PT_E_SEARCH_SERVER_CONNECTION):
				resultSet._errorMessage=m_asOwner.GetString(1581, "ptmsgs_portalbrowsingmsgs");
				break;

				// this error occurs if the SWS could
				// not be contacted at all (e.g. because it
				// is down or because of some DNS problem).
				// We won't display the more specific error
				// (e.g., 404, 403, etc.) to the end user,
				// but we'll include it commented out for
				// an administrator to find when viewing
				// source
				//
			    case (PT_RESULTCODES.PT_E_FED_BADHTTPRESPONSE):

				String messageToBeCommentedOut="<!-- " + ptFedSearch.GetLastError(sourceID) + " -->";
				resultSet._errorMessage=m_asOwner.GetString(1580, "ptmsgs_portalbrowsingmsgs") + messageToBeCommentedOut;
 				break;
			    case (PT_RESULTCODES.PT_E_FED_BADADMINPREF):

				resultSet._errorLink=makeErrorLink(sourceID,
								   false,
								   ptFedSearch);

				break;
			    case (PT_RESULTCODES.PT_E_FED_BADUSERPREF):
				resultSet._errorLink=makeErrorLink(sourceID,
								   true,
								   ptFedSearch);

			   break;
			    case(PT_RESULTCODES.PT_E_FED_BLANKPORTALNAME):
				resultSet._errorMessage=m_asOwner.GetString(1362, "ptmsgs_portalbrowsingmsgs");
				break;
			    case(PT_RESULTCODES.PT_E_FED_BLANKUSERNAME):
				resultSet._errorMessage=m_asOwner.GetString(1363, "ptmsgs_portalbrowsingmsgs");
				break;
			    case(PT_RESULTCODES.PT_E_FED_BLANKSEARCHSTRING):
				resultSet._errorMessage=m_asOwner.GetString(1364, "ptmsgs_portalbrowsingmsgs");
				break;
			    case(PT_RESULTCODES.PT_E_FED_PORTALNOTRECOGNIZED):
				resultSet._errorMessage=m_asOwner.GetString(1365, "ptmsgs_portalbrowsingmsgs");
				break;
			    case(PT_RESULTCODES.PT_E_FED_USERNOTRECOGNIZED):
				resultSet._errorMessage=m_asOwner.GetString(1366, "ptmsgs_portalbrowsingmsgs");
				break;
			    case(PT_RESULTCODES.PT_E_FED_INVALIDPASSWORD):
				resultSet._errorMessage=m_asOwner.GetString(1367, "ptmsgs_portalbrowsingmsgs");
				break;
			    default:

				resultSet._errorMessage     = ptFedSearch.GetLastError(sourceID);
			    }
			    resultSet._isValid		    = false;
			}
		}
		catch (Exception e)
		{
			resultSet._errorMessage     = e.getMessage();
			resultSet._isValid		    = false;

			// tmp !!! non jumpable, debugging only
			//e.printStackTrace();
		}

		return resultSet;
	}
	 // doSearch

    /**
     * For certain errors, we want to provide a link leading
     * the user to a page where config info must be set up. This
     * method constructs that link
     **/

     /**
     * For certain errors, we want to provide a link leading
     * the user to a page where config info must be set up. This
     * method constructs that link
     **/
    protected HTMLElement makeErrorLink(int sourceID,
					boolean userConfig,
					IPTFederatedSearch ptFedSearch) {

	try {
	    int plugPage=0;
	    if (userConfig) {
		plugPage=GatewayHelpers.USERPLUGPAGE;
	    }
	    else {
		plugPage=GatewayHelpers.ADMINPLUGPAGE;
	    }

	    int webServiceID=GetSourceWebServiceID(sourceID);
	    String errorLinkString=
		GatewayHelpers.ConstructPrefPageLink(m_asOwner,
						     webServiceID,0,0,
						     plugPage,
						     PT_CLASSIDS.PT_WEBSERVICE_ID);


	    HTMLAnchor errorMessageLink=new HTMLAnchor(errorLinkString);

	    errorMessageLink.AddInnerHTMLString(ptFedSearch.GetLastError(sourceID));

	    return errorMessageLink;
	}
	catch (Exception e) {
	    log.Error(e, "Unable to provide error link for network search location corresponding to source id " + sourceID);
	    // log error?
	    return new HTMLGenericElement("");
	}

    }


	/**
	 *  Whether this model instance has been initialized. May be needed someday
	 */

	//private boolean _initialized = false;

	/**
	 * Inner helper class to hold info about outgoing federated portals
	 */
	public static class FedPortal
	{

		public String _clsid;


		public String _identity;


		public String _name;


		public int _id;

	    /** true if the associated web service is disabled **/
	    public boolean _disabled;

	    public int _intgrWebServiceID;
		/**
		 * Creates a new FedPortal object.
		 *
		 * @param id
		 * @param name
		 * @param clsid
		 * @param identity
		 */
		public FedPortal(int id, String name, String clsid, String identity, int wsID, boolean disabled)
		{
			_id			  = id;
			_name		  = name;
			_clsid		  = clsid;
			_identity     = identity;
			_intgrWebServiceID = wsID;
			_disabled=disabled;
		}

	    public int GetWebServiceID()
	    {
			return _intgrWebServiceID;
	    }

		/**
		 *
		 *
		 * @return
		 */
		public String GetClsid()
		{
			return _clsid;
		}

		/**
		 *
		 *
		 * @return
		 */
		public int GetID()
		{
			return _id;
		}

		/**
		 *
		 *
		 * @return
		 */
		public String GetIdentity()
		{
			return _identity;
		}

		/**
		 *
		 *
		 * @return
		 */
		public String GetName()
		{
			return _name;
		}

	    public boolean IsDisabled()
	    {
			return _disabled;
	    }
	}
	 // inner class FedPortal

	/**
	 * Inner helper class to hold info about search results
	 */
	public static class SearchResultSet
	{
	    public AActivitySpace m_asOwner;
	    
		/** Contains the actual search results. May be null. */
		public IPTSearchResponse _searchResponse;


		public String _errorMessage;


		public String _searchString;


		public boolean _isValid = false;


		public int _endIndex;


		public int _sourceID;


		public int _startIndex;

	    public HTMLElement _errorLink=null;

	    public boolean _isLocal=false;

	    public SearchResultSet(AActivitySpace _asOwner)
	    {
	        m_asOwner = _asOwner;
	    }
	    
	    /** may be null **/
	    public HTMLElement GetErrorLink() {
		return _errorLink;
	    }

		/** May be null. */
		public String GetDescription(int rank)
		{
			if (_searchResponse == null)
			{
				return null;
			}

			// if the result is from the local portal, then it might
			// be keyword highlighted, so first let's see if the keyword-
			// highlighted description is there.
			// temp workaround- getkwicfields is not working right now


			String desc= _searchResponse.GetKWICFieldsAsString(rank,
					PT_INTRINSICS.PT_PROPERTY_OBJECTSUMMARY);


			if (desc == null)
			{
			    if (_isLocal) {
				desc = XPHttpUtility.HtmlEncode( _searchResponse.GetFieldsAsString(rank,
						PT_INTRINSICS.PT_PROPERTY_OBJECTSUMMARY));
			    }
			    else {
				desc=  _searchResponse.GetFieldsAsString(rank,
						PT_INTRINSICS.PT_PROPERTY_OBJECTSUMMARY);
			    }
			}

			return desc;

		}

		/** May be null. */
		public String GetErrorMessage()
		{
			return _errorMessage;
		}

	    /**
	     * Only used by searches of "This Portal". Stealing code
	     * from banner search (i.e. SearchResultModel )
	     * for getting icons for cards
	     **/
	    public HTMLImg GetLocalResultIcon(int rank, String strLangID)
	    {

		HTMLImg icon = null;

		//Use the icon associated with the card's actual file type.
		String uuid = GetResultTypeUUID(rank);

		// If the image UUID is defined
		if ((null != uuid) && (0 < uuid.length()))
		    {
			// get the image for this UUID
			icon = PlumtreeHelpers.GetHTMLImage(uuid, /* smallImage= */
							    true);

                        if (null != icon) {
                          // See if we can get a tooltip based on the UUID.  Use the PTDocTypesVarPack.
                          IApplication application = ApplicationManager.GetInstance()
                            .GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
                          if (null != application) {
                            PTDocTypesVarPack varPack = (PTDocTypesVarPack) application.GetVarPackManager()
                              .GetVariablePackage(PTDocTypesVarPack.VARPACK_ID);
                            if (null != varPack) {
                              String altTag = varPack.GetLocalizedName(uuid, strLangID);

                              if ((null != altTag) && (altTag.length() > 0)) {
                                icon.SetAlt(varPack.GetLocalizedName(uuid, strLangID));
                              }
                            }
                          }
                        }
                    }


		// default to "sheet of paper" icon if we didn't get one
		if (icon==null) {
		    ClassTypeDescManager cdm = PTConfigHelper.GetPTClassTypeDescrManager();

		    if (null != cdm)
			{
			    icon = cdm.GetDefaultIconForClassType(PT_CLASSIDS.PT_CATALOGCARD_ID, strLangID);
			}
		}
		return icon;
	    }


	    public String GetResultTypeUUID(int rank)
	    {

		return _searchResponse.GetFieldsAsString(rank,
							 PT_INTRINSICS.PT_PROPERTY_OBJECTIMAGEUUID);
	    }



		/**
		 *
		 *
		 * @param rank
		 * @param asOwnder
		 *
		 * @return
		 */
		public HTMLImg GetImage(int rank, AActivitySpace asOwner)
		{ 
		  
		    if (_searchResponse == null)
			{
			    return null;
			}

		    if (_isLocal) {
			String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE, asOwner);
			return GetLocalResultIcon(rank, strLangID);
		    }
		    else
			{
			    try {

					String url =

					    _searchResponse.GetFieldsAsString(rank,
					  PT_INTRINSICS.PT_PROPERTY_OBJECTIMAGEURL);

					// if nothing returned, return null.
					// We'll plug in the default image
					// later.
					if (url==null || url.trim().equals("")) {
					    return null;
					}

				// figure out if this is a relative or
				// absolute url. Not sure if there's any
				// more principled strategy than checking
				// for http: or https: at the start
				if (url.indexOf("http:")!=0 &&
				    url.indexOf("https:")!=0 ) {

				    //relative, so we need to append the image
				    //server base url. We truly need the
				    //base folder itself. Simply calling
				    //ConfigHelper.GetImageBaseURL() will
				    //append /plumtree on the end of the base
				    //folder, which we don't want, since the
				    //network search result images
				    //don't necessarily live under the plumtree
				    //folder. That's why we have to do this
				    String urlroot=((ImgSvrURLHelper) ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString()).GetCachingManager().GetEntry("ImageServerHelper")).GetRootURL(asOwner.GetIsSecuredSpace());
				    url=urlroot + url;

				}

				HTMLImg resultImage= new HTMLImg(url,"network search result");
				resultImage.SetUseDisplayOptionsBaseUrl(false);
				
				return resultImage;
			    }
			    catch(Exception e) {
				return null;
			    }
			}

		}

		/**
		 *
		 *
		 * @return
		 */
		public int GetNumTotalMatches()
		{
			if (_searchResponse == null)
			{
				return -1;
			}

			return _searchResponse.GetTotalMatches();
		}

		/** May be null. */
		public String GetTitle(int rank)
		{
			if (_searchResponse == null)
			{
				return null;
			}

			String title = _searchResponse.GetFieldsAsString(rank,
					PT_INTRINSICS.PT_PROPERTY_OBJECTNAME);


			if (_isLocal) {
			    return XPHttpUtility.HtmlEncode(title);
			}

			return title;

		}

		/**
		 *
		 *
		 * @param rank
		 *
		 * @return
		 */
		public HTMLAnchor GetLink(int rank, boolean popup, int userID)
		{
			if (_searchResponse == null)
			{
				return null;
			}

			String url = _searchResponse.GetFieldsAsString(rank,
					PT_INTRINSICS.PT_PROPERTY_DOCUMENTURL);

			if (url == null)
			{
				url = GetPortalObjectOpenerURL(rank, userID);

				// mj: this url is a javascript function. if we're not popping up,
				// we need to make sure it gets in the OnClick. otherwise,
				// it will be handled by the window.open below.
				if (!popup)
				{
					try
					{
						return ConstructStaticHelpers.GetJavascriptAnchor(url);
					}
					catch(Exception e)
					{
						log.Error(e, "Unable to create javascript anchor for URL " + url);
						throw new XPException("Error: Unable to create javascript anchor for URL " + url, e);
					}
				}
			}

			if (popup)
			{
				try
				{
					HTMLAnchor anchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
					anchor.SetOnClick("window.open('" + NavigationCommonHelpers.EscapeSingleQuotes(url) + "','searchResultClickthru'); return false;");
					return anchor;
				}
				catch(Exception e)
				{
					log.Error(e, "Unable to create javascript anchor for URL " + url);
					throw new XPException("Error: Unable to create javascript anchor for URL " + url, e);
				}

			}

			return new HTMLAnchor(url);

		}

		/**
		 * If false, the contents of this result set are not valid and
		 * some kind of error occurred.
		 */
		public boolean isValid()
		{
			return _isValid;
		}

		/**
		 *
		 *
		 * @param rank
		 *
		 * @return
		 */
		protected String GetPortalObjectOpenerURL(int rank, int userID)
		{
			/** getting class id **/
			int classId = _searchResponse.GetFieldsAsInt(rank,
					PT_INTRINSICS.PT_PROPERTY_CLASSID);

			int objectId = _searchResponse.GetFieldsAsInt(rank,
					PT_INTRINSICS.PT_PROPERTY_OBJECTID);;

	        String strClickThroughURL = PTOpenerLinks.GetOpenerJSURLOpenObjID(classId, 
	            objectId, null, IPTOpenerConstants.OPENER_MODE_VIEW);			        

	        return PTOpenerLinks.GetJSOpenInSameWindow(strClickThroughURL);
		}
	}
	 // inner class SearchResultSet


    private static class LocationPreferences
    {
		public int _userID;
		public XPHashtable _locationsHash;
    }

    /**
     * if the user has checked "always check these locations for me"
     * on the launch page, then here we save the checked locations
     * as user preferences
     **/
    public void SaveLocationPreferences(String[] locationIDStrings) {

	XPStringBuilder sb = new XPStringBuilder();
	for (int i=0; i < locationIDStrings.length; i++)
	{
	    try {
			// I think it's lame that in order to store a list of
			// preference values associated with one key, you have
			// store it as a comma-delimted string, but after looking
			// at some other code, that seems to me to be the only
			// way to do it
			if (i > 0)
			{
			    sb.Append(LOCATIONPREF_DELIMITER);
			}
			sb.Append(locationIDStrings[i]);
	    }
	    catch (Exception e)
	    {
			// log this?? or just do nothing.. this shouldn't happen,
			// anyway
			return;
	    }
	}

	((IPTSession) m_asOwner.GetUserSession()).GetSessionInfo().AddPreference(NETSEARCHPREF_LOCATIONIDS, sb.ToString(), 0);

	// empty the cache so that we know we have to query the db next time
	m_asOwner.GetSubSession().SetAttribute(CACHEDPREFS_KEY,null);
    }

    private int GetUserID()
    {
	IPTSession session  = (IPTSession) m_asOwner.GetUserSession();
	IPTSessionInfo info = session.GetSessionInfo();

	return info.GetCurrentUserID();
    }

    /**
     * check to see if the user has any stored preferences regarding
     * which sources to check. Note: returns null if there are no such
     * preferences
     **/

    public XPHashtable GetLocationPreferencesForUser() {

	// first check to see if we can get it from the cache
	Object cachedPrefs=m_asOwner.GetSubSession().GetAttribute(CACHEDPREFS_KEY);
	if (cachedPrefs != null) {
	    LocationPreferences prefs=(LocationPreferences)cachedPrefs;
	    // make sure it's the same user
	    if (prefs._userID==GetUserID()) {
		return prefs._locationsHash;
	    }
	}

	// we are really using this as a hash set, e.g., we simply
	// want to store some ids and then quickly check later whether
	// one is in there
	XPHashtable locationsHash=new XPHashtable();

	Object[][] twoDimObjArray = (Object[][]) ((IPTSession) m_asOwner.GetUserSession()).GetSessionInfo().LookupPreference(NETSEARCHPREF_LOCATIONIDS, 0);
	if (twoDimObjArray !=null && twoDimObjArray[1][0] != null) {

	    String commaDelimitedLocations=(String)(twoDimObjArray[1][0]);
	    XPStringTokenizer st=new XPStringTokenizer(commaDelimitedLocations,
						       LOCATIONPREF_DELIMITER);
	    while (st.HasMoreTokens()) {
		String location=st.GetNextToken();
		try {
		    locationsHash.PutElement(new Integer(XPConvert.ToInteger(location)),
					     // again, we're just using this
					     // as a hash set- doesn't really
					     // matter what we put here
					     new Integer(1));
		}
		catch (Exception nfe) {
		    //??? log something
		    return null;
		}
	    }
	    // cache it so we don't have to hit the db next time
	    LocationPreferences newPrefs=new LocationPreferences();
	    newPrefs._userID=GetUserID();
	    newPrefs._locationsHash=locationsHash;
	    m_asOwner.GetSubSession().SetAttribute(CACHEDPREFS_KEY,newPrefs);
	    return locationsHash;
	}
	else {
	    return null;
	}

    }

    /**
     * Set a flag indicating whether we had to truncate the user's
     * search string to NetworkSearchModel.MAX_SEARCHSTRING_LENGTH
     * characters
     **/
    public void SetSearchStringTruncated(boolean flag) {
	m_searchStringTruncated=flag;
    }

     /**
     * A flag indicating whether we had to truncate the user's
     * search string to NetworkSearchModel.MAX_SEARCHSTRING_LENGTH
     * characters
     **/
    public boolean GetSearchStringTruncated() {
	return m_searchStringTruncated;
    }

	public void LoadNumberResults()
	{
		try
		{
			m_resultsPerPagePerSource = 5;

			Object[][] idArray	= ((IPTSession) m_asOwner.GetUserSession()).GetSessionInfo().LookupPreference(NUM_RESULTS_PREFS_KEY, 0);
			if(idArray.length > 0 && idArray[1] != null && idArray[1].length == 1 && idArray[1][0] != null)
			{
				m_resultsPerPagePerSource = XPConvert.ToInteger(idArray[1][0]);
			}
		}
		catch (Exception e)
		{
			throw XPException.GetInstance(e);
		}
	}
	
	//this function saves the number of results to return variable in a federated search.
	public void SaveNumberResults()
	{
		try
		{
			IPTSessionInfo sessionInfo  = ((IPTSession) m_asOwner.GetUserSession()).GetSessionInfo();

			String propVal = XPConvert.ToString(m_resultsPerPagePerSource);
			sessionInfo.RemovePreference(NUM_RESULTS_PREFS_KEY, 0);
			sessionInfo.AddPreference(NUM_RESULTS_PREFS_KEY, propVal, 0);
		}
		catch (Exception e)
		{
			throw XPException.GetInstance(e);
		}

	}
	public Object GetNewTempData()
	{
		return new TempDataClass();
	}
}

