package com.plumtree.portalpages.browsing.search.results;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.pei.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.portalpagesperf.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 *
 *
 * @author CraigS
 */
public class AdvancedStartSearchControl extends StartSearchControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.search.results.AdvancedStartSearchControl");

	/** Name of this control. */
	public static final String STR_MVC_CLASS_NAME = "advancedstart";

	/**
	 *  Creates an instance
	 *  @return new instance
	 */
	public Object Create()
	{
		return new AdvancedStartSearchControl();
	}

	/**
	 * Returns class name
	 * @return class name
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	// the user's original search string- we will log this for the sake
	// of search log reports
	private String _logString;

	/**
	 *  This method is called by StartSearchControl to allow customizations to
	 *  the query; it simply turns around and calls any advanced search PEIs that have been defined.
	 *  @param query the user's query
	 */
	protected Redirect CustomizeQuery(QueryArguments query)
	{
		// Bump up the advanced search counter
		PerfCounters.INSTANCE.IncrementCounter(PORTALPAGESPERF_COUNTERS.PORTALPAGESPERF_ADVANCED_SEARCHES);

		// Make sure PEIs are loaded
		LoadAdvancedSearchPEIs();

		// Call out to each PEI in turn
		if (null != _advancedSearchPEIs) {
			IXPEnumerator e = _advancedSearchPEIs.GetEnumerator();
			while (e.MoveNext()) {
				IAdvancedSearchActions pei = (IAdvancedSearchActions) e.GetCurrent();

				pei.CustomizeQueryOnBeforeSearch(_parent, (IPTSession) _parent.GetUserSession(), query);
			}
		}

		// Save the user's query for logging in the search server logs
		IPTFilter filter=query.advancedFilter;
		if (filter != null) {
			String searchString=filter.GetSearchString();
			if (searchString != null) {
				_logString = searchString;

                // Log this query to the Analytics Server
				if (_parent.GetModelExists(LoggingHelperModel.STR_MVC_CLASS_NAME)) {
					LoggingHelperModel logHelper = (LoggingHelperModel) _parent.GetModel(LoggingHelperModel.STR_MVC_CLASS_NAME);
					if (null != logHelper) {
						logHelper.LogStartSearchEvent(query.userQuery, LoggingHelperModel.SEARCH_TYPE_ADVANCED);
					}
				}

			}
		}

                // Return null to stay on the search results page
                return null;
	}

	/**
	 * This method is called just before the initial query is
	 * executed for advanced search... it simply calls out to any PEIs that have
	 * been defined to allow them to customize search settings.
	 */
	protected SearchSettingCollection GetCustomSearchSettings()
	{
		SearchSettingCollection settings = new SearchSettingCollection();

		// Make sure any PEIs are loaded
		LoadAdvancedSearchPEIs();

		if (null != _advancedSearchPEIs) {
			IXPEnumerator e = _advancedSearchPEIs.GetEnumerator();
			while (e.MoveNext()) {
				IAdvancedSearchActions pei = (IAdvancedSearchActions) e.GetCurrent();

				SearchSettingCollection customSettings =
					pei.GetCustomSettingsOnBeforeSearch(_parent, (IPTSession) _parent.GetUserSession());
				if (null != customSettings) {
					settings.merge(customSettings);
				}
			}
		}

		// All done
		return settings;
	}

	private IXPCollection _advancedSearchPEIs = null;

	private boolean _loadedPEIs = false;

	/**
	 * BrianH/CraigS : This function will pull in all of the available
	 * IAdvancedSearchActions PEIs.
	 */
	private void LoadAdvancedSearchPEIs()
	{
		// if already loaded
		if (_loadedPEIs) {
			// don't do it again
			return;
		}

		_loadedPEIs = true;

		IApplication application = _parent.GetApplication();
		Object[] actions = (Object[]) application.GetCachingManager().GetEntry("AdvancedSearchActions");

		if (null != actions) {

			_advancedSearchPEIs = new XPArrayList();

			for (int x = 0; x < actions.length; x++) {
				Object oAction = actions[x];

				if ((null != oAction) && (oAction instanceof IAdvancedSearchActions)) {
					_advancedSearchPEIs.Add(oAction);
				} else {
					log.Warn("AdvancedStartSearchControl.LoadAdvancedSearchPEIs: element " + XPConvert.ToString(x) + " is not an instance of IAdvancedSearchActions");
				}
			}
		}
	}

	/**
	 *  For advanced searches, the user's query is logged in the search server logs
	 */
	protected String GetLogString()
	{
		return _logString;
	}
}


