package com.plumtree.portalpages.browsing.search.results.assocobjects;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.server.*;

/**
 *  @author CraigS
 */
public class AssociatedObjectsModel implements IModel
{
  /** Name of this class */
  public static final String STR_MVC_CLASS_NAME = "SRAssociatedObjects";

  /** Owning session */
  private IPTSession _session;

  /** Owning activityspace */
  private AActivitySpace m_asOwner;

  /** Name of the item we are dealing with */
  private String _itemName;

  /** Names of portlets the item is associated with (and that the user can see)
   */
  private String[] _portletNames;

  /** Mapping of portlet names (from the _portletNames list) to IDs
   */
  private IXPDictionary _portletIDMap;

  /** Names of communities the item is associated with (and that the user can see)
   */
  private String[] _communityNames;

  /** Mapping of community names (from the _communityNames list) to IDs
   */
  private IXPDictionary _communityIDMap;

  public void Init(AActivitySpace parent)
  {
    m_asOwner = parent;
    _session = (IPTSession) m_asOwner.GetUserSession();

    _itemName = null;
    _portletNames = null;
    _portletIDMap = null;
    _communityNames = null;
    _communityIDMap = null;
  }

  public void CleanupTempData()
  {
  }

  public void CleanupAllData()
  {
    _itemName = null;
    _portletNames = null;
    _portletIDMap = null;
    _communityNames = null;
    _communityIDMap = null;
  }

  public Object Create()
  {
    return new AssociatedObjectsModel();
  }

  public String GetName()
  {
    return STR_MVC_CLASS_NAME;
  }

  public String GetItemName()
  {
    return _itemName;
  }

  public void SetItemName(String itemName)
  {
    _itemName = itemName;
  }

  public void SetPortletList(int[] portletList)
  {
    if ((null == portletList) || (0 == portletList.length)) {
      _portletNames = null;
      _portletIDMap = null;
      _communityNames = null;
      _communityIDMap = null;
      return;
    }

    XPArrayList pNames = new XPArrayList();
    XPArrayList cNames = new XPArrayList();

    _portletIDMap = new XPHashtable();
    _communityIDMap = new XPHashtable();

    IPTObjectManager pMgr = _session.GetGadgets();
    IPTCommunityManager cMgr = (IPTCommunityManager) (_session.GetCommunities());

    // For each portlet in list
    for (int j = 0; j < portletList.length; j++) {

      String portletName = null;

      // Check whether the user can see this portlet
      try {
        IPTObject ptObj = (IPTObject) (pMgr.Open(portletList[j], /* lock= */ false));
        portletName = ptObj.GetLocalizedName();
      } catch (Exception e) {
        portletName = null;
      }

      if ((null == portletName) || (0 == portletName.length())) {
        // user apparently can't see this portlet
        continue;
      }

      pNames.Add(portletName);
      _portletIDMap.PutElement(portletName, new Integer(portletList[j]));

      // Look up communities the portlet is associated with
      IPTQueryResult qr = cMgr.QueryCommunitiesForGadget(portletList[j]);

      int cCount = qr.RowCount();
      for (int k = 0; k < cCount; k++) {
        int communityID = qr.ItemAsInt(k, PT_PROPIDS.PT_PROPID_OBJECTID);
        String communityName = qr.ItemAsString(k, PT_PROPIDS.PT_PROPID_NAME);

        // See if the _communityIDMap already contains this community.  Have to use the community
        // name, since that is the key for the map, but it's unlikely that there will be
        // two communities with the same name.
        if (! _communityIDMap.ContainsKey(communityName)) {
          cNames.Add(communityName);
          _communityIDMap.PutElement(communityName, new Integer(communityID));
        }
      }  // end k-loop over communities for portlet j

    }  // end j-loop over portlets

    // If no portlets, couldn't possibly have gotten communities either, so null out everything
    if (0 == pNames.GetSize()) {
      _portletNames = null;
      _portletIDMap = null;
      _communityNames = null;
      _communityIDMap = null;
      return;
    }

    // At this point, we know we have something in the pNames list.  Call helper to sort it
    _portletNames = listToSortedStringArray(pNames);

    // It's possible that we got a list of portlets, but not communities
    if (0 == cNames.GetSize()) {
      _communityNames = null;
      _communityIDMap = null;
    } else {
      // Else, sort the community names list too
      _communityNames = listToSortedStringArray(cNames);
    }
  }

  /** Converts an unsorted IXPList of Strings into a sorted array of strings. */
  protected String[] listToSortedStringArray(IXPList list)
  {
    int n = list.GetSize();
    String[] sa = new String[n];
    for (int j = 0; j < n; j++) {
      sa[j] = (String) (list.GetElement(j));
    }

    XPArray.Sort(sa);
    return sa;
  }

  public int GetNumPortlets()
  {
    if (null == _portletNames) return 0;
    return _portletNames.length;
  }

  public int GetNumCommunities()
  {
    if (null == _communityNames) return 0;
    return _communityNames.length;
  }

  public String GetPortletName(int idx)
  {
    return _portletNames[idx];
  }

  public String GetCommunityName(int idx)
  {
    return _communityNames[idx];
  }

  public int GetPortletID(int idx)
  {
    return XPConvert.ToInteger(_portletIDMap.GetElement(_portletNames[idx]));
  }

  public int GetCommunityID(int idx)
  {
    return XPConvert.ToInteger(_communityIDMap.GetElement(_communityNames[idx]));
  }
}

