/**
 * 
 */
package com.plumtree.portalpages.browsing.sso.ensemble;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.login.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.interpreter.*;
import com.plumtree.uiinfrastructure.interpreter.Interpreter;
import com.plumtree.uiinfrastructure.interpreter.filter.*;

/**
 * 
 * 
 * This Interpreter Filter is used when Ensemble SSO is enabled. It handles all the requests that WCI gets
 * and sends a request to Ensemble to see if the user already logged in to Ensemble. Based on Ensemble response
 * this filter does the following
 * If Ensemble SSO is disabled then bypass this filter, else do the following
 * Send Request to Ensemble to see if user already logged into Ensemble, Wait for response from Ensemble
 * If request times out then bypass this filter, 
 * else if Ensemble sends response with Login Token then process the token
 * and create session for user and pass on the request for further processing. 
 * @author Ravi Tunuguntla
 *
 */
public class EnsembleHandlerFilter extends BaseInterpreterFilter {

	
	public static final String CSPLOGINTOKEN_REQUHEADER_NAME = "CSP-Session-Token"; 
	
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
			"com.plumtree.portalpages.browsing.sso.ensemble.EnsembleHandlerFilter");
			

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.interpreter.filter.IInterpreterFilter#PreFilter(com.plumtree.uiinfrastructure.interpreter.RequestData)
	 */
	public boolean PreFilter(RequestData requestData) {
		
		//RT - This custom sso feature between Ensemble and WCI is disabled and not supported in NEO release.
		// To achieve SSO between WCI and Ensemble or other WebCenter products like spaces it is suggested to use
		// a generic SSO setup which includes thirdparty SSO solution like OSSO, OAM etc to achieve SSO between WCI
		// and Ensemble. For setting up generic SSO it is suggested to follow WCI documentation.
		// If any one invokes this feature a UI Error message will be displayed saying this feature is disabled and suggesting
		// to use generic SSO solution.
		
		if(EnsembleHandlerControl.FEATURE_DISABLED)
		{
			log.Debug("custom SSO with Ensemble is disabled, We will not process current request for custom SSO. We are done with EnsembleHandlerFilter");
			return false;
		}
		
		int ensembleSSOTimeout = EnsembleHandlerTasks.GetSSOTimeout();
		if (ensembleSSOTimeout > 0) {
			
			IXPRequest request = requestData.m_xpRequest;
			
			// check if this request is a Ensemble response for the Request wci sent.
			String ssoStatus = request.GetParameterValue(EnsembleHandlerControl.SSO_STATUS);
			
			if(null != ssoStatus){
				log.Info("Ensemble sent response for WCI Request including sso status:" + request.GetRequestURL());
				// Process this request as Ensemble Response for wci request
				if(XPStringUtility.CompareIgnoreCase(ssoStatus, EnsembleHandlerControl.SSO_STATUS_FOUND) == 0){
					
					// Getting the token from Ensemble Response
					String loginToken = request.GetParameterValue(EnsembleHandlerControl.WCI_TOKEN);
					
					// We need to Base64 decode token before we pass it on to create session
					
					loginToken = XPBase64Coder.DecodeToString(loginToken);
					
					// Create a new session
					IPTSession currSession = PortalObjectsFactory.CreateSession();
					
					try {
						// Authenticate session and set it as current session
						currSession.Reconnect(loginToken);
						requestData.SetUserSession(currSession);
						
						// Set the flag so it shows user has logged in for further requests
						requestData.m_smSandbox.SetAttribute(Interpreter.USERSESSIONVALID, new Boolean(true));
						
					} catch (XPException e) {
						log.Warn(e, "Login authoriazation failed for token sent by Ensemble: "+loginToken);
					}
				} else {
					
					if (XPStringUtility.CompareIgnoreCase(ssoStatus, EnsembleHandlerControl.SSO_STATUS_ERR) == 0){
						log.Warn("Ensemble encountered error while processing sso request, current request will be redirected to login page"); 
					}				
				}
				
				String strFinalURL = requestData.GetRequestFullURL();

				// Removing SSO Status and login token from qs, since we are done using them.
				
				strFinalURL = EnsembleHandlerTasks.RemoveQueryStringParam(strFinalURL, EnsembleHandlerControl.SSO_STATUS);
				strFinalURL = EnsembleHandlerTasks.RemoveQueryStringParam(strFinalURL, EnsembleHandlerControl.WCI_TOKEN);
				
				requestData.SetRequestURL(strFinalURL);
				requestData.SetRequestFullURL(strFinalURL);
				log.Info("Final URL after processing Ensemble Response: " + strFinalURL );
				//Done with Pre-processing.
				return false;
			}
			
			
			// Before sending a request to Ensemble we check if the current request is initiated by Ensemble, then we 
			// ignore prefiltering of the request since it will be handled by Ensemble SSO Control.
			
			
			String strControlName = requestData.GetSpaceName();
			
			// If the current space is Login that means user is trying to Login with credentials and we don't need to
			// verify with Ensemble and we can pass through the request.
			
			if ((null != strControlName) 
					&& (XPStringUtility.Compare( LoginControl.STR_MVC_CLASS_NAME, strControlName) ==0)){
				log.Info("User is trying to login and we pass through the request for further processing.");
				return false;
			}
			boolean bEnsRequest = ((null != strControlName) 
					&& (XPStringUtility.Compare( EnsembleHandlerControl.STR_MVC_CLASS_NAME, strControlName) ==0));
			
			// If this is an Ensemble Request and not a normal request then it will be handled by Ensemble Handler Control.
			if(bEnsRequest){
				log.Info("Request directly came from Ensemble and it will be handled by Ensemble Handler Control:" + request.GetRequestURL());
				return false;
			}
			
			
			// There is no SSO Status in the request, so we process this request to check if user already logged into Ensemble.
			IPTSession session = (IPTSession) requestData.GetUserSession();
			
			boolean bLogged = (null != session && !session.GetUser().IsGuestUser());
			
			if( null != session){	
				log.Info("Current User detected by Filter is :" + session.GetUser().GetLoginName());
			}
			//Check if user already logged in, else invoke request to Ensemble if user logged in there. 
			if(!bLogged){
				log.Info("User did not login and not a guest user, and Ensemble SSO is enabled, so we check with Ensemble if user logged in ! ");
				// Get Ensemble URL, 
				
				String strURL = EnsembleHandlerTasks.GetEnsembleBaseUrl();
				if(null == strURL){				
					log.Error("Unable to get Ensemble SSO Endpoint from configuration, Request will not be redirected Ensemble!");
					return false;
				}
								
				XPStringBuilder strFinalURL = new XPStringBuilder();
				strFinalURL.Append(strURL);
				log.Info("Sending Redirect to Following URL to check if user already logged into Ensemble " + strURL);
				
				// Redirect URL with login request and original request url encrypted
				
				if(!XPStringUtility.EndsWith(strURL, EnsembleHandlerTasks.STR_CONTEXT_APPEND)){
					strFinalURL.Append(EnsembleHandlerTasks.STR_CONTEXT_APPEND);
				}
				
				strFinalURL.Append(EnsembleHandlerControl.EN_URL_SSO_SUFIX);
				strFinalURL.Append(EnsembleHandlerTasks.STR_QS_PREFIX);
				strFinalURL.Append(EnsembleHandlerControl.SSO_ACTION);
				strFinalURL.Append(EnsembleHandlerTasks.STR_QS_ASSIGN);
				
				strFinalURL.Append(EnsembleHandlerControl.LOGIN_VALUE);
				strFinalURL.Append(EnsembleHandlerTasks.STR_QS_APPEND);
				strFinalURL.Append(EnsembleHandlerControl.ES_POST_REDIRECT_URL);
				
				strFinalURL.Append(EnsembleHandlerTasks.STR_QS_ASSIGN);
				strFinalURL.Append(EnsembleHandlerTasks.EncryptUsingSeedToken(request.GetRequestURL()));
							 
				// Create a redirect request for Ensemble with original url encrypted.
				Redirect enRedirect = new Redirect();
				enRedirect.SetLinkToExternalURL(strFinalURL.ToString());
				enRedirect.SetIsHTTPRedirect(true);
				requestData.SetRedirect(enRedirect);
				return false;
			}
			return false;
		} else {
			// Ensemble SSO is disabled, we are done with this prefilter
			log.Debug("custom SSO with Ensemble is disabled, We will not process current request for custom SSO. We are done with EnsembleHandlerFilter");
			return false;
		}
	}
}