package com.plumtree.portalpages.common.helpers;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.form.*;

/**
 * @author adolfoc
 *
 * This class is for use in collecting and validating information sent to models.
 * Although designed for Editor models, most of these methods will work with any model.
 */
public class ModelHelper {
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
		"com.plumtree.portalpages.common.helpers.ModelHelper");

	/**
	 * Gets a boolean Form Element from a hashtable of form data - returns the
	 * default value if the form element was not found or is empty.
	 * NOTE: This assumes that booleans are defined in forms as follows.
	 *
	 * "0" == false
	 * Any non-zero integer (e.g. "1") == true
	 *
	 * @param _htFormData
	 * @param strFormElement
	 * @return boolean
	 */
	public static boolean GetFormCheckboxBool(XPHashtable _htFormData,
		String strFormElement)
	{
        String[] data = (String[]) _htFormData.GetElement(strFormElement);

        // Absence of a checkbox form element means it's not checked
        if (data == null || IsBlankStringArray(data))
        {
            return false;
        }

        int elementValue = 0;
        try
        {
            elementValue = XPConvert.ToInteger(data[0]);
        }
        catch (Exception e)
        {
            log.Debug(e, "ModelHelper::GetFormElementInt: Unknown error while retrieving form element (" + strFormElement + "). ");
        }

        // Non-zero value indicates the checkbox was checked.
        return elementValue != 0;
	}

	/**
	 * Gets a boolean Form Element from a hashtable of form data - returns the
	 * default value if the form element was not found or is empty. NOTE: Unlike
	 * the other GetForm... methods defined in this class, the default value
	 * cannot be null. ptdDefault is used to retrieve the default MASK value
	 * (among other things).
	 *
	 * @param _htFormData
	 * @param strFormElementDurationInput
	 * @param strFormElementDurationUnitsSelect
	 * @param ptdDefault
	 * @return PTDuration
	 */
	public static PTDuration GetFormElementDuration(XPHashtable _htFormData,
		String strFormElementDurationInput,
		String strFormElementDurationUnitsSelect, PTDuration ptdDefault)
	{
		// AGC- If we get one of the following two form elements and not the other, strange behavior will result. However,
		// this is unlikely since both of these elements are part of the same htmlconstruct.
		long lTimeout     = ModelHelper.GetFormElementLong(_htFormData,
				strFormElementDurationInput, -1L);
		int nTimeoutUnits = ModelHelper.GetFormElementInt(_htFormData,
				strFormElementDurationUnitsSelect, -1);

		if ((-1L == lTimeout) || (-1 == nTimeoutUnits))
		{
			
				log.Warn("ModelHelper::GetFormElementDuration: Null or empty form element found while retrieving form elements for Duration. Returning Default Duration.");
			

			return ptdDefault;
		}

		PTDuration ptdResult = new PTDuration(lTimeout, nTimeoutUnits,
				ptdDefault.GetSupportedUnitsMask());

		return ptdResult;
	}

	/**
	 * Checks to see if the string array passed in is null or
	 * of zero length
	 *
	 * @param data
	 * @return
	 */
	public static boolean IsEmptyStringArray(String[] arsData)
	{
		if (null == arsData)
		{
			return true;
		}

		if (arsData.length <= 0)
		{
			return true;
		}

		return false;
	}

	/**
	 * Checks to see if the string array passed in is null or
	 * of zero length OR the first element is a blank string.
	 *
	 * @param data
	 * @return
	 */
	public static boolean IsBlankStringArray(String[] arsData)
	{
		if (IsEmptyStringArray(arsData))
		{
			return true;
		}

		if (arsData[0].length() <= 0)
		{
			return true;
		}

		return false;
	}

	/**
	 * Checks to see if the string array passed in is null or
	 * of zero length
	 *
	 * @param data
	 * @return
	 */
	public static boolean IsBlankString(String sData)
	{
		if (null == sData)
		{
			return true;
		}

		if (sData.length() <= 0)
		{
			return true;
		}

		return false;
	}

	/**
	 * Gets an integer Form Element from a hashtable of form data - returns the
	 * default value if the form element was not found or is empty.
	 *
	 * @param _htFormData
	 * @param strFormElement
	 * @param nDefault
	 * @return String
	 */
	public static int GetFormElementInt(XPHashtable _htFormData,
		String strFormElement, int nDefault)
	{
		String[] data = (String[]) _htFormData.GetElement(strFormElement);

		if (!IsBlankStringArray(data))
		{
			try
			{
				return XPConvert.ToInteger(data[0]);
			}
			catch (Exception e)
			{
				
					log.Warn(e, "ModelHelper::GetFormElementInt: Unknown error while retrieving form element (" +
						strFormElement + "). ");
				

				return nDefault;
			}
		}
		else
		{
			
				log.Warn("ModelHelper::GetFormElementInt: Null or empty form element found while retrieving form element (" +
					strFormElement + "). ");
			

			return nDefault;
		}
	}

	/**
	 * Gets a long Form Element from a hashtable of form data - returns the
	 * default value if the form element was not found or is empty.
	 *
	 * @param _htFormData
	 * @param strFormElement
	 * @param lDefault
	 * @return String
	 */
	public static long GetFormElementLong(XPHashtable _htFormData,
		String strFormElement, long lDefault)
	{
		String[] data = (String[]) _htFormData.GetElement(strFormElement);

		if (!IsBlankStringArray(data))
		{
			try
			{
				return XPConvert.ToLong(data[0]);
			}
			catch (Exception e)
			{
				
					log.Warn(e, "ModelHelper::GetFormElementLong: Unknown error while retrieving form element (" +
						strFormElement + "). ");
				

				return lDefault;
			}
		}
		else
		{
			
				log.Warn("ModelHelper::GetFormElementLong: Null or empty form element found while retrieving form element (" +
					strFormElement + "). ");
			

			return lDefault;
		}
	}

	/**
	 * Gets a string Form Element from a hashtable of form data - returns the
	 * default value if the form element was not found or is empty.
	 *
	 * @param _htFormData
	 * @param strFormElement
	 * @param strDefault
	 * @return String
	 */
	public static String GetFormElementString(XPHashtable _htFormData,
		String strFormElement, String strDefault)
	{
		String[] data = (String[]) _htFormData.GetElement(strFormElement);

		if (!IsEmptyStringArray(data))
		{
			return data[0];
		}
		else
		{
			
				log.Warn("ModelHelper::GetFormElementString: Null or empty form element found while retrieving form element (" +
					strFormElement + "). ");
			

			return strDefault;
		}
	}

	/**
	 * Translates an Object array into an XPArrayList.
	 * Returns an empty XPArrayList if arObj is null.
	 *
	 * @param arObj
	 * @return XPArrayList
	 */
	public static XPArrayList ObjectArrayToXPArray(Object[] arObj)
	{
		XPArrayList xpalResult = new XPArrayList();

		if (null != arObj)
		{
			for (int nIndex = 0; nIndex < arObj.length; nIndex++)
			{
				xpalResult.Add(arObj[nIndex]);
			}
		}

		return xpalResult;
	}

	/**
	 * Returns the value of objValue or bDefault if objValue is null or
	 * the conversion fails.
	 */
	public static boolean ObjectToBool(Object objValue, boolean bDefault)
	{
		if (null == objValue)
		{
			return bDefault;
		}

		int nValue = bDefault ? 1 : 0;

		try
		{
			nValue = XPConvert.ToInteger(objValue);
		}
		catch (Exception e)
		{
			
				log.Debug(e, "ModelHelper::ObjectToBool: Error - Returning Default value.");
			

			nValue = bDefault ? 1 : 0;
		}

		if (0 == nValue)
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	/**
	 * Returns the value of objValue or nDefault if objValue is null or
	 * the conversion fails.
	 */
	public static int ObjectToInt(Object objValue, int nDefault)
	{
		if (null == objValue)
		{
			return nDefault;
		}

		int nValue = nDefault;

		try
		{
			nValue = XPConvert.ToInteger(objValue);
		}
		catch (Exception e)
		{
			
				log.Debug(e, "ModelHelper::ObjectToInt: Error - Returning Default value.");
			

			nValue = nDefault;
		}

		return nValue;
	}

	/**
	 * Reads a boolean value from a property bag. Returns bDefault if the
	 * value was not found or an error occurred.
	 *
	 * @param sPropID
	 * @param ptPropBag
	 * @param bDefault
	 * @return boolean
	 */
	public static boolean PBReadBool(IXPPropertyBag ptPropBag, String sPropID,
		boolean bDefault)
	{
		try
		{
			return ObjectToBool(ptPropBag.ReadAsObject(sPropID), bDefault);
		}
		catch (Exception e)
		{
			
				log.Warn(e, "ModelHelper::PBReadBool: Unable to Read Bool from property bag, returning default (" +
					sPropID + ").");
			

			return bDefault;
		}
	}

	/**
	 * Reads an int value from a property bag. Returns nDefault if the value
	 * was not found or an error occurred.
	 *
	 * @param sPropID
	 * @param ptPropBag
	 * @param nDefault
	 * @return int
	 */
	public static int PBReadInt(IXPPropertyBag ptPropBag, String sPropID,
		int nDefault)
	{
		try
		{
			return ObjectToInt(ptPropBag.ReadAsObject(sPropID), nDefault);
		}
		catch (Exception e)
		{
			
				log.Warn(e, "ModelHelper::PBReadInt: Unable to Read Int from property bag, returning default (" +
					sPropID + ").");
			

			return nDefault;
		}
	}

	/**
	 * Reads a String value from a property bag. Returns strDefault if the value
	 * was not found or an error occurred.
	 *
	 * @param sPropID
	 * @param ptPropBag
	 * @param strDefault
	 * @return String
	 */
	public static String PBReadString(IXPPropertyBag ptPropBag, String sPropID,
		String strDefault)
	{
		String strReturn = strDefault;
		try
		{
			 strReturn = ptPropBag.ReadAsString(sPropID);
			 if (null == strReturn)
			 {
				return strDefault;
			 }
			 return strReturn;

		}
		catch (Exception e)
		{
			
				log.Warn(e, "ModelHelper::PBReadString: Unable to Read String from property bag, returning default (" +
					sPropID + ").");
			

			return strDefault;
		}
	}

	/**
	 * Writes a value to a property bag.
	 *
	 * @param sPropID
	 * @param ptPropBag
	 * @param strValue
	 */
	public static void PBWrite(IXPPropertyBag ptPropBag, String sPropID,
		String strValue)
	{
		try
		{
			ptPropBag.Write(sPropID, strValue);
		}
		catch (Exception e)
		{
			
				log.Warn(e, "WSModeHelper::PBWrite (String): Unknown Error storing property bag value: (" +
					sPropID + ")(" + strValue + ")");
			
		}
	}

	/**
	 * Writes a value to a property bag.
	 *
	 * @param sPropID
	 * @param ptPropBag
	 * @param nValue
	 */
	public static void PBWrite(IXPPropertyBag ptPropBag, String sPropID,
		int nValue)
	{
		try
		{
			ptPropBag.Write(sPropID, nValue);
		}
		catch (Exception e)
		{
			
				log.Warn(e, "WSModeHelper::PBWrite (Int): Unknown Error storing property bag value: (" +
					sPropID + ")(" + XPConvert.ToString(nValue) + ")");
			
		}
	}

	/**
	 * Writes a value to a property bag.
	 *
	 * @param sPropID
	 * @param ptPropBag
	 * @param lValue
	 */
	public static void PBWrite(IXPPropertyBag ptPropBag, String sPropID,
		long lValue)
	{
		try
		{
			ptPropBag.Write(sPropID, (int)lValue);
		}
		catch (Exception e)
		{
			
				log.Warn(e, "WSModeHelper::PBWrite (Long): Unknown Error storing property bag value: (" +
					sPropID + ")(" + XPConvert.ToString(lValue) + ")");
			
		}
	}

	/**
	 * Writes a value to a property bag.
	 *
	 * @param sPropID
	 * @param ptPropBag
	 * @param bValue
	 */
	public static void PBWrite(IXPPropertyBag ptPropBag, String sPropID,
		boolean bValue)
	{
		if (bValue)
		{
			PBWrite(ptPropBag, sPropID, 1);
		}
		else
		{
			PBWrite(ptPropBag, sPropID, 0);
		}
	}

	/**
	 * Writes a value to a property bag.
	 *
	 * @param sPropID
	 * @param ptPropBag
	 * @param bValue
	 */
	public static void PBWrite(IXPPropertyBag ptPropBag, String sPropID,
		XPArrayList xpalValue)
	{
		try
		{
			ptPropBag.Write(sPropID, xpalValue.ToArray());
		}
		catch (Exception e)
		{
			
				log.Warn(e, "WSModeHelper::PBWrite (XPArrayList): Unknown Error storing property bag value: (" +
					sPropID + ")");
			
		}
	}

	/**
	 * Validates that a specified Duration falls within the specified duration
	 * range (specified in milliseconds).
	 *
	 * @param ptdIn
	 * @param asOwner
	 * @param strError
	 * @param nLastError
	 * @param lMinMilli
	 * @param lMaxMilli
	 * @return int
	 */
	public static int ValidateDurationRange(PTDuration ptdIn,
		AActivitySpace asOwner, String strError, int nLastError,
		long lMinMilli, long lMaxMilli)
	{
		long lMilliIn;
		boolean bInvalidDuration = false;
		
		if(ptdIn == null)
		{
			bInvalidDuration = true;
			lMilliIn = -1L;
		} else {
			lMilliIn = ptdIn.GetMilliseconds();
		}

		if (!bInvalidDuration &&
			(lMilliIn >= lMinMilli) && (lMilliIn <= lMaxMilli))
		{
			return nLastError;
		}
		else
		{
			
				log.Warn("ModelHelper::ValidateDurationRange: Input out of range. (" +
					XPConvert.ToString(lMinMilli) + ">" + XPConvert.ToString(lMilliIn) +
					">" + XPConvert.ToString(lMaxMilli));
			

			UIError myError = new UIError(UIErrorLevel.FRIENDLY,
					UIErrorScope.CURRENT_USER, strError, strError);
			UIErrorHandler.RaiseError(myError, asOwner.GetSubSession(),
				asOwner.GetApplication());

			return RepostControl.PAGE_STATUS_INVALID;
		}
	}

	/**
	 * Validates an arbitrary boolean. If bError is true it errors, else
	 * it does not.
	 *
	 * @param ptdIn
	 * @param asOwner
	 * @param strError
	 * @param nLastError
	 * @param lMinMilli
	 * @param lMaxMilli
	 * @return int
	 */
	public static int ValidateBool(boolean bValid,
		AActivitySpace asOwner, String strError, int nLastError)
	{

		if (bValid)
		{
			return nLastError;
		}
		else
		{
			
				log.Warn("ModelHelper::ValidateBool: Validate Failed.");
			

			UIError myError = new UIError(UIErrorLevel.FRIENDLY,
					UIErrorScope.CURRENT_USER, strError, strError);
			UIErrorHandler.RaiseError(myError, asOwner.GetSubSession(),
				asOwner.GetApplication());

			return RepostControl.PAGE_STATUS_INVALID;
		}
	}

	/**
	 * Validates that the provided string is an integer, if not
	 * returns PAGE_STATUS_INVALID. Returns nLastError otherwise.
	 *
	 * @param nIn
	 * @param asOwner
	 * @param strError
	 * @param nLastError
	 * @return int
	 */
	public static int ValidateInteger(String integer, AActivitySpace asOwner,
		String strError, int nLastError)
	{
		int result = RepostControl.PAGE_STATUS_INVALID; 
		try {
			XPConvert.ToInteger(integer);
			result = nLastError;
		} catch (XPException e) {
			UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER, strError, strError);
			UIErrorHandler.RaiseError(myError, asOwner.GetSubSession(),asOwner.GetApplication());
		}
		
		return result;
		

	}
	
	/**
	 * Validates that an integer is within the specified range (nMin to nMax).
	 * Raises a UIError with descriptiong strError if the integer is invalid
	 * (and returns PAGE_STATUS_INVALID). Returns nLastError otherwise.
	 *
	 * Note: If nMax is less than nMin, this function only validates that
	 * nIn is greater than nMin (nMax is ignored).
	 *
	 * Note: If nMax == nMin, then this function validates that nIn == nMax ==
	 * nMin.
	 *
	 * @param nIn
	 * @param asOwner
	 * @param strError
	 * @param nLastError
	 * @param nMin
	 * @param nMax
	 * @return int
	 */
	public static int ValidateIntegerRange(int nIn, AActivitySpace asOwner,
		String strError, int nLastError, int nMin, int nMax)
	{
		if (nMin <= nMax)
		{
			if ((nIn >= nMin) && (nIn <= nMax))
			{
				return nLastError;
			}
		} else
		{
			if (nIn >= nMin)
			{
				return nLastError;
			}
		}


		// AGC- If we haven't returned by now, there's an error.
		
			log.Warn("ModelHelper::ValidateIntegerRange: Input out of range. (" +
				XPConvert.ToString(nMin) + ">" + XPConvert.ToString(nIn) + ">" +
				XPConvert.ToString(nMax));
		

		UIError myError = new UIError(UIErrorLevel.FRIENDLY,
				UIErrorScope.CURRENT_USER, strError, strError);
		UIErrorHandler.RaiseError(myError, asOwner.GetSubSession(),
			asOwner.GetApplication());

		return RepostControl.PAGE_STATUS_INVALID;
	}

	/**
	 * Checks that a string starts with http:// or https://.
	 *
	 * @param strIn
	 * @return boolean
	 */
	public static boolean CheckIsURLString(String strIn)
	{
		String strToCheck = XPStringUtility.ToLowerCase(strIn).trim();
		strToCheck = TrimAllTrailingSlashes(strToCheck);

		if ((null != strToCheck) && !(0 == strToCheck.length()))
		{
			if ((XPStringUtility.StartsWith(strToCheck, "http://"))
				|| (XPStringUtility.StartsWith(strToCheck, "https://")))
			{
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * Validates that a string starts with http:// or https://. Raises a UIError
	 * with descriptiong strError if the string is invalid (and returns
	 * PAGE_STATUS_INVALID). Returns nLastError otherwise.
	 *
	 * @param strIn
	 * @param asOwner
	 * @param strError
	 * @param nLastError
	 * @return int
	 */
	public static int ValidateIsURLString(String strIn, AActivitySpace asOwner,
		String strError, int nLastError)
	{
		if (CheckIsURLString(strIn))
		{
			return nLastError;
		}

		log.Warn("ModelHelper::ValidateIsURLString: non-URL string found. ");
		

		UIError myError = new UIError(UIErrorLevel.FRIENDLY,
				UIErrorScope.CURRENT_USER, strError, strError);
		UIErrorHandler.RaiseError(myError, asOwner.GetSubSession(),
			asOwner.GetApplication());

		return RepostControl.PAGE_STATUS_INVALID;
	}

	public static boolean HasTrailingSlash(String strIn)
	{
		if ((XPStringUtility.EndsWith(strIn, "\\"))
			|| (XPStringUtility.EndsWith(strIn, "/")))
		{
			return true;
		} else
		{
			return false;
		}
	}

	public static String TrimTrailingSlash(String strIn)
	{
		String strResult = strIn;

		if (HasTrailingSlash(strResult))
		{
			strResult = XPStringUtility.Substring(strResult, 0,strResult.length() - 1);
		}

		return strResult;
	}

	public static String TrimAllTrailingSlashes(String strIn)
	{
		String strResult = strIn;
		// nMaxCount is just a safety variable. It doesn't strictly need to be here, but helps
		// just in case XPStringUtility.Substring is broken or misused (I have trust issues OK?).
		int nMaxCount = strIn.length();

		while ((strResult.length() > 0) && HasTrailingSlash(strResult) && (nMaxCount > 0))
		{
			strResult = XPStringUtility.Substring(strResult, 0,strResult.length() - 1);
			nMaxCount--;
		}

		return strResult;
	}

	/**
	 * Validates that a string is non-empty. Raises a UIError with descriptiong
	 * strError if the string is empty or null (and returns
	 * PAGE_STATUS_INVALID). Returns nLastError otherwise.
	 *
	 * @param strIn
	 * @param asOwner
	 * @param strError
	 * @param nLastError
	 * @return int
	 */
	public static int ValidateNonEmptyString(String strIn,
		AActivitySpace asOwner, String strError, int nLastError)
	{
		if ((null != strIn) && !(0 == strIn.length()))
		{
			return nLastError;
		}
		else
		{
			
				log.Warn("ModelHelper::ValidateNotEmptyString: Null or empty string found. ");
			

			UIError myError = new UIError(UIErrorLevel.FRIENDLY,
					UIErrorScope.CURRENT_USER, strError, strError);
			UIErrorHandler.RaiseError(myError, asOwner.GetSubSession(),
				asOwner.GetApplication());

			return RepostControl.PAGE_STATUS_INVALID;
		}
	}

	/**
	 * Validates that both a login and password were set.
	 *
	 * @param strLogin
	 * @param strPassword
	 * @param strNULLPASSWORD
	 * @param asOwner
	 * @param strError
	 * @param nLastError
	 * @return int
	 */
	public static int ValidateLoginPassword(String strLogin, String strPassword, String strNULLPASSWORD, AActivitySpace asOwner, String strError, int nLastError)
	{
		// AGC- Right now, the Password is not checked, but it may be in the future, if we decide to force
		// the password to be reset every time the editor is accessed.
		if (0 >= strLogin.trim().length())
		{
			
				log.Warn("ModelHelper::ValidateNotEmptyString: Null or empty string found. ");
			

			UIError myError = new UIError(UIErrorLevel.FRIENDLY,
					UIErrorScope.CURRENT_USER, strError, strError);
			UIErrorHandler.RaiseError(myError, asOwner.GetSubSession(),
				asOwner.GetApplication());

			return RepostControl.PAGE_STATUS_INVALID;
		} else {
			return nLastError;
		}

	}

	public static XPArrayList StringArrayToXPArray(String[] arsIn)
	{
		XPArrayList xpalResult = new XPArrayList();

		if (null != arsIn) {

			for (int nIndex = 0; nIndex < arsIn.length; nIndex++)
			{
				xpalResult.Add(arsIn[nIndex]);
			}
		}


		return xpalResult;
	}
}

