package com.plumtree.portalpages.common.login;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.common.constants.*;
import com.plumtree.portaluiinfrastructure.htmlconstructs.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author DonH
 *
 * This class contains common code to generate HTML for login
 * views, including the auth source dropdown.
 * 
 * This code was moved from PTLoginHelper during refactoring.
 */
public class LoginHTML
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
		"com.plumtree.portalpages.common.login.LoginHTML");

	/**
	 * Parameter that indicates that the login control should attempt a login,
	 * rather than just displaying the login page.
	 */
	public static final String PARAM_DOLOGIN				= "in_hi_dologin";
	
	public static final String PARAM_USERNAME		 		= "in_tx_username";
	public static final String PARAM_USERPASS		 		= "in_pw_userpass";
	public static final String PARAM_REMEMBER		 		= "in_cb_remember";
	public static final String PARAM_AUTHSOURCE		 		= "in_se_authsource";
	public static final String PARAM_HIDDEN_AUTHSOURCE		= "in_hi_authsource";
	public static final String PARAM_SHOWAUTHSOURCES		= "in_tx_showauthsources";
	public static final String PARAM_DOLOGIN_508			= "in_su_dologin508";
	public static final String HTMLBUTTON_LOGIN				= "in_bu_Login";
	public static final String STR_JS_FUNC_NAME_DO_LOGIN	= "doLogin";

	/** The maximum length of the auth source description */
	public static final int iMaxAuthSourceDescriptionLength = 25;	

	/**    This is the number of milliseconds for which the HTMLSelect presenting
	 *     the various authentication sources is cached on the application.  After
	 *     this many milliseconds, the server will be again querried, and a new
	 *     HTMLSelect will be generated.
	 */
	public static final long AUTHSOURCE_SELECT_CACHE_LIFE_MS = 900000;
	
	// TODO: Move this to either a configuration setting, or else refactor up higher into the view layer
	/** Whether or not to store information about last-login in a client cookie,
	 * to be restored at time of next login.  The trade-off is an nice increase
	 * in usability for a slight loss of security (i.e. hacker now knows a username and last auth source used) 
	 */
	public static final boolean bAllowLoginInfoPersistenceBetweenLogins = true;

	/**    Call this method to obtain an HTMLSelect presenting the available
	 *     authentication sources.  This HTMLSelect will be generated once, and
	 *     then cached on the application for AUTHSOURCE_SELECT_CACHE_LIFE_MS
	 *     milliseconds.  A subsequent call to this method will cause the server
	 *     to be querried, and a new HTMLSelect to be generated and cached.
	 *     <br><br>
	 *     This method caches auth source selects based on language and 508 value.
	 *     It is not entirely threadsafe.
	 *     <br><br>
	 *     NOTE: Since the cached HTMLSelect object is stored on the application
	 *     and shared between users, absolutely no changes can be made to it without
	 *     causing multi-threading problems.  The returned object is READ ONLY!
	 *     <br><br>
	 * 	   @param space the current Activity Space.
	 *     @param application Pass a reference to an IApplication object.
	 * 	   @param boolean whether or not we are displaying 508 compliant HTML.
	 */
	public static HTMLSelect GetAuthSourceSelect(AActivitySpace space,
		IApplication application, boolean b508, String sUniquifier )

	{
			log.Debug("GetAuthSourceSelect()");

		// mj: look up this user's language
		String sLanguage = XPConvert.ToString(PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE, space));
		
		HTMLSelect mySelect;
		
		// mj: Generate cache keys for the list for a specific language and 508 setting
		String sKeySelect     = "authSourceSelect_" + sLanguage + XPConvert.ToString(b508);
		String sKeyLastUpdate = "authSourceLastUpdate_" + sLanguage + XPConvert.ToString(b508);

		XPDateTime dLastUpdated = null;
		XPDateTime dNow		    = null;

		CachingManager cache = application.GetCachingManager();

		dLastUpdated = (XPDateTime) cache.GetEntry(sKeyLastUpdate);

		// NOTE: This is probably not entirely threadsafe.  It's fairly good, but
		// may still have some problems.
		if (dLastUpdated != null)
		{
			//   There is a last updated date
			
				log.Debug("GetAuthSourceSelect: Last updated for language " + sLanguage +
					" and 508 value " + XPConvert.ToString(b508) + ": " + dLastUpdated);
			

			dNow = new XPDateTime();

			if ((dNow.GetTimeInMilliSeconds() -
					dLastUpdated.GetTimeInMilliSeconds()) < AUTHSOURCE_SELECT_CACHE_LIFE_MS)
			{
				// If the last updated select is still valid,
				// attempt to get the select.
				mySelect = (HTMLSelect) cache.GetEntry(sKeySelect);

				if (mySelect != null)
				{
					
						log.Debug("GetAuthSourceSelect: Got a cached auth source select for language " + sLanguage +
							" and 508 value " + XPConvert.ToString(b508) + "!");
					

					//   if it's there, return it.
					return mySelect;
				}
			} else
			{
				
					log.Debug("GetAuthSourceSelect: Cached selected out of date for language " + sLanguage +
						" and 508 value " + XPConvert.ToString(b508) + ": " + dLastUpdated);
				
			}
		}

		//   Either there's no last updated date, the last updated date
		//   is too old, or there was no select cached on the application.
		//   we must make a new one.
		mySelect = CreateAuthSourceSelect(space, sLanguage, b508, sUniquifier);

		cache.SetEntry(sKeySelect, mySelect);

		dNow = new XPDateTime();

		cache.SetEntry(sKeyLastUpdate, dNow);

		return mySelect;
	}

	/**
	 * Helper method to create the auth source select for login & caching
	 */
	private static HTMLSelect CreateAuthSourceSelect(AActivitySpace asOwner, String sLanguage,
		boolean b508, String sUniquifier)
	{
		
			log.Debug("CreateAuthSourceSelect: Creating a new auth source select for language " + sLanguage +
				" and 508 value " + XPConvert.ToString(b508) + ".");
		

		HTMLSelect mySelect;
		HTMLOption myOption;
		String sPrefix;
		String sText;
		int nAuthSourceMode;

		// Get Auth Sources to display
		IPTObjectManager om = GetPTSession(asOwner).GetAuthSources();

		IPTQueryResult qr = om.Query(PT_PROPIDS.PT_PROPID_NAME | 
			PT_PROPIDS.PT_PROPID_DESCRIPTION |
			PT_PROPIDS.PT_PROPID_AUTHSOURCE_PREFIXSTRING |
			PT_PROPIDS.PT_PROPID_AUTHSOURCE_SUPPORTSSYNC, -1, 
			PT_PROPIDS.PT_PROPID_DESCRIPTION, 0, -1, null);

		int intAuthCount = qr.RowCount();
		boolean bSetSelected = false;

		XPArrayList htAuthSourceOptions = new XPArrayList();
		XPArrayList htAuthSourcePrefixes = new XPArrayList();

		// Create normal auth sources
		for (int i = 0; i < intAuthCount; i++)
		{
			nAuthSourceMode = qr.ItemAsInt( i, PT_PROPIDS.PT_PROPID_AUTHSOURCE_SUPPORTSSYNC );

			if( (nAuthSourceMode == -1) ||
				(nAuthSourceMode == PT_AUTHSOURCE_SUPPORTSSYNCMODES.PT_AUTHSOURCESUPPORTSSYNCMODE_AUTHONLY ) )
			{
				// Don't add options for auth sources that are servering as auth partners
				// They will be added through the sync + auth partner auth source that
				// is using them.
				continue;
			}

			sPrefix = (String) qr.ItemAsString(i, PT_PROPIDS.PT_PROPID_AUTHSOURCE_PREFIXSTRING);

			log.Debug("GetAuthSourceSelect() - sPrefix: " + sPrefix);

			myOption = new HTMLOption(sPrefix);

			if (sPrefix != null)
			{
				myOption.SetValue(sPrefix);

				if (sPrefix.equals(PTLoginHelper.GetDefaultPrefix()))
				{
					// Since we can have multiple auth sources with the same prefix,
					// only set one to be the default selected auth source.
					if (!bSetSelected)
					{
						bSetSelected = true;
						myOption.SetSelected(true);
					}
				}
			}
			else
			{
				if ((PTLoginHelper.GetDefaultPrefix() == null) ||
					("".equals(PTLoginHelper.GetDefaultPrefix())))
				{
					// Since we can have multiple auth sources with the same prefix,
					// only set one to be the default selected auth source.
					if (!bSetSelected)
					{
						bSetSelected = true;
						myOption.SetSelected(true);
					}
				}

				myOption.SetValue("");

				// Set this for use in the hashtable below
				// This is actually misleading, because not all Auth Sources
				// with "" prefixes are the Plumtree Auth Source.  Any time
				// you copy an Auth Source, it blanks the prefix.  However,
				// for the time being, it is OK to treat these auth sources
				// as equivalent to the Plumtree Auth Source, because we
				// have no reliable way to differentiate between them.
				// The behavior of copying an Auth Source will change in
				// the future (bug#34911)
				sPrefix = "Plumtree Auth Source";
			}
			
			// mj: note that the name and description are automatically localized 
			// for the current language
			sText = qr.ItemAsString(i, PT_PROPIDS.PT_PROPID_DESCRIPTION);

			if( sText == null )
			{
				sText = qr.ItemAsString(i, PT_PROPIDS.PT_PROPID_NAME);
			}

			if (sText == null)
			{
				sText = "";
			}

			sText = GeneralHelpers.TruncateLongString(sText, iMaxAuthSourceDescriptionLength, false);

			myOption.AddInnerHTMLString(XPHttpUtility.HtmlEncode(sText));

			htAuthSourceOptions.Add(myOption);
			htAuthSourcePrefixes.Add(sPrefix);
		}

		// Create Auth Source select list
		mySelect = new HTMLSelect(LoginHTML.PARAM_AUTHSOURCE, "authsourceID");
		mySelect.SetID(PTStyleID.PT_LOGIN_AUTH_SELECT);

		// Check for auth source ordering information in PTConfig.xml
		int nConfigAuthSourcePrefixIndex = 1;
		String strCurrentPrefix = ConfigHelper.GetConfigVariableAsString
			(ConfigConstants.AUTHENTICATION_SECTION, "AuthSourcePrefix" + nConfigAuthSourcePrefixIndex,
			ConfigConstants.VALUE_ATTRIBUTE, asOwner);

		// Add the pre-ordered auth source prefixes first
		while (!"".equals(strCurrentPrefix))
		{
			int nCurrentPrefixIndex = htAuthSourcePrefixes.IndexOf(strCurrentPrefix);
			while (nCurrentPrefixIndex != -1)
			{
				myOption = (HTMLOption) htAuthSourceOptions.GetElement(nCurrentPrefixIndex);
	
				if (myOption != null)
				{
					mySelect.AddInnerHTMLElement(myOption);
				} else
				{
					
						log.Error("The ordered auth source prefix " + strCurrentPrefix +
							" (AuthSourcePrefixn from PTConfig.xml) cannot be found.");
				}

				htAuthSourcePrefixes.RemoveAt(nCurrentPrefixIndex);
				htAuthSourceOptions.RemoveAt(nCurrentPrefixIndex);
				nCurrentPrefixIndex = htAuthSourcePrefixes.IndexOf(strCurrentPrefix);
			}

			nConfigAuthSourcePrefixIndex++;
			strCurrentPrefix = ConfigHelper.GetConfigVariableAsString(ConfigConstants.AUTHENTICATION_SECTION,
				"AuthSourcePrefix" + nConfigAuthSourcePrefixIndex, ConfigConstants.VALUE_ATTRIBUTE, asOwner);
		}

		// Add the remaining prefixes in whatever order they happen to be in
		IXPEnumerator options = htAuthSourceOptions.GetEnumerator();
		while (options.MoveNext())
		{
			myOption = (HTMLOption) options.GetCurrent();

			if (myOption != null)
			{
				mySelect.AddInnerHTMLElement(myOption);
			}
		}

		if (!b508)
		{
			mySelect.SetOnKeyPress("return " + PlumtreeHelpers.STR_JS_FUNC_NAME_EXECUTE_VIA_ENTER + "(event);");
		}

		mySelect.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
		
		// mj: handle default auth source prefix
		if( PTLoginHelper.GetDefaultPrefix() != null )
		{
			try
			{
				mySelect.SetValue( PTLoginHelper.GetDefaultPrefix() );
			}
			catch (HTMLException he)
			{
				// JF- If the default prefix was not found in the list of options,
				// just behave as if we had no default prefix.
				log.Error(he, "The default prefix value (DefaultAuthSourcePrefix) from PTConfig.xml is invalid: " 
					+ PTLoginHelper.GetDefaultPrefix() + ". It doesn't match any of the Authentication Source prefixes.");
			}
		}

		return mySelect;
	}

	/**
	 * This method creates the HTML that describes how to login.
	 */
	public static HTMLTable GetLoginDirections(AActivitySpace asOwner, String sWidth, boolean bShowAuthSource)
	{
		HTMLTable myTable;
		HTMLTableCell myCell;
		HTMLTableRow myRow;
		HTMLBr myBr		  = new HTMLBr();
		
		myTable = new HTMLTable();
		myTable.SetCellPadding("2" /*CommonHTMLStrings.TWO */);
		myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
			
		myTable.SetAlign(CommonHTMLStrings.CENTER);
		myTable.SetWidth(sWidth);

		myRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(myRow);

		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);

		myCell.SetStyleClass(PTStyleClass.LOGIN_TITLE);


		// Welcome to your portal
		myCell.AddInnerHTMLString(asOwner.GetString(631,
				"ptmsgs_portalbrowsingmsgs"));

		myRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(myRow);

		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);

		myCell.SetStyleClass(PTStyleClass.LOGIN_TEXT);


		// Log in to your personalized Portal account.
		myCell.AddInnerHTMLString(asOwner.GetString(632,
				"ptmsgs_portalbrowsingmsgs"));
		myCell.AddInnerHTMLElement(myBr);
		myCell.AddInnerHTMLElement(myBr);

		int nShowAuthSourceMode = 0;

		try
		{
			//	default access style for all users.
			nShowAuthSourceMode = PTConfigHelper.GetConfigVariableAsInt("Authentication",
					"AllowDefaultLoginPageAuthSource", "value", asOwner);
		}
		catch (Exception ex)
		{	
			log.Debug(ex, "Unable to get AllowDefaultLoginPageAuthSource config var as int.");

			nShowAuthSourceMode = 0;
		}

		// Determine which text string to show based on which options are enabled.
		if ( PTLoginHelper.ShowRememberMyPassword() &&
			( bShowAuthSource || PTLoginHelper.ShowAuthDropdown() ))
		{
			// All options turned on - full text

			// If you have an existing account, enter your Username and Password. If necessary, select
			// the appropriate Authentication Source. If you want to log in to the Portal automatically,
			// check &lt;b&gt;Remember my Password&lt;/b&gt;, and then click &lt;b&gt;Log In&lt;/b&gt;.
			myCell.AddInnerHTMLString(asOwner.GetString(638,
					"ptmsgs_portalbrowsingmsgs"));
		} else if ( PTLoginHelper.ShowRememberMyPassword() )
		{
			// Only remember my password

			if (nShowAuthSourceMode == 3)
			{
				// Mode 3 means never show auth source dropdown

				// If you have an existing account, enter your Username and Password.  If you want to log
				// in to the Portal automatically, check &lt;b&gt;Remember my Password&lt;/b&gt;, and
				// then click &lt;b&gt;Log In&lt;/b&gt;.
				myCell.AddInnerHTMLString(asOwner.GetString(1886,
					ResourceConstants.PORTALBROWSINGMSGS));
			} else
			{
				// If you have an existing account, enter your Username and Password. Select &lt;b&gt;Show
				// Auth Sources&lt;/b&gt; to use a non-default authentication source. If you want to log
				// in to the Portal automatically, check &lt;b&gt;Remember my Password&lt;/b&gt;, and
				// then click &lt;b&gt;Log In&lt;/b&gt;.
				myCell.AddInnerHTMLString(asOwner.GetString(1716,
						"ptmsgs_portalbrowsingmsgs"));
			}
		} else if ( bShowAuthSource || PTLoginHelper.ShowAuthDropdown() )
		{
			// Auth source dropdown currently displayed, no remember my password

			// If you have an existing account, enter your Username and Password. If necessary, select
			// the appropriate Authentication Source.
			myCell.AddInnerHTMLString(asOwner.GetString(1717,
					"ptmsgs_portalbrowsingmsgs"));
		} else
		{
			// Auth source dropdown not currently displayed, no remember my password

			if (nShowAuthSourceMode == 3)
			{
				// Mode 3 means never show auth source dropdown

				// If you have an existing account, enter your Username and Password.
				myCell.AddInnerHTMLString(asOwner.GetString(1885,
					ResourceConstants.PORTALBROWSINGMSGS));
			} else
			{
				// If you have an existing account, enter your Username and Password. Select &lt;b&gt;Show
				// Auth Sources&lt;/b&gt; to use a non-default authentication source.
				myCell.AddInnerHTMLString(asOwner.GetString(1718,
					ResourceConstants.PORTALBROWSINGMSGS));
			}
		}

		myCell.AddInnerHTMLElement(myBr);
		myCell.AddInnerHTMLElement(myBr);


		if( PTLoginHelper.GetAllowUserCreation( asOwner, asOwner.GetApplication() ) )
		{
			// To create an account ...
			myCell.AddInnerHTMLString(asOwner.GetString(639,
				"ptmsgs_portalbrowsingmsgs"));
		}
				
		return myTable;
	}

	/**
	 * This method creates the HTML for the login form.
	 */
	public static HTMLTableCell MakeLoginForm(HTMLForm myForm, String sFormName, 
												String sLoginASName, 
												String sSpaceID, String sControlName, 
												AActivitySpace asOwner, boolean b508, 
												String sUniquifier, HTMLElement myCreateLink,
												boolean bShowAuthSource, boolean bAllowAuthSourceDropdownDisplay)
	{
		HTMLTable myTable;
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLInput myInput;
		
		// mj: this code returns the right url for any security mode
		myForm.SetAction(ConfigHelper.GetApplicationBaseURLToAS(asOwner.GetCurrentNormalizedBaseURL(), sLoginASName, false));
		myForm.SetMethod(CommonHTMLStrings.POST);
		myForm.SetName(sFormName);

		// Sets the space name for the event to be passed to the server
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
				ASConstants.SPACENAME_INPUT, ASConstants.SPACENAME_INPUT,
				"This is an input");
		myInput.SetValue(sLoginASName);
		myForm.AddInnerHTMLElement(myInput);
		
		// Sets the space ID for the event to be passed to the server
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
				ASConstants.SPACEID_INPUT, ASConstants.SPACEID_INPUT,
				"This is an input");
		myInput.SetValue(sSpaceID);
		myForm.AddInnerHTMLElement(myInput);

		// Sets the control name for the event to be passed to the server
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
				ASConstants.CONTROL_INPUT, ASConstants.CONTROL_INPUT,
				"This is an input");
		myInput.SetValue(sControlName);
		myForm.AddInnerHTMLElement(myInput);

		// Sets the dologin parameter
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
				LoginHTML.PARAM_DOLOGIN, LoginHTML.PARAM_DOLOGIN,
				"This is an input");
		myInput.SetValue("false");
		myForm.AddInnerHTMLElement(myInput);
		
		// mj: do login text boxes
		myTable = new HTMLTable();

		myForm.AddInnerHTMLElement(myTable);

		myTable.SetCellPadding(CommonHTMLStrings.TWO);
		myTable.SetCellSpacing(CommonHTMLStrings.ZERO);

		myTable.SetAlign(CommonHTMLStrings.CENTER);
		myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);

		myRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(myRow);

		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);

		myCell.SetColSpan(CommonHTMLStrings.TWO);
		myCell.SetStyleClass(PTStyleClass.LOGIN_TEXT);

		myRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(myRow);

		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);

		myCell.SetWidth("40%");
		myCell.SetAlign(CommonHTMLStrings.RIGHT);
		myCell.SetStyleClass(PTStyleClass.LOGIN_SUBTITLE);

		String usernameText = asOwner.GetString(633, "ptmsgs_portalbrowsingmsgs");

		// Username:
		myCell.SetPreComment(usernameText);

		// Username:
		myCell.AddInnerHTMLString(usernameText);

		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);

		myCell.SetWidth("60%");
		myCell.SetAlign(CommonHTMLStrings.LEFT);
		myCell.SetStyleClass(PTStyleClass.LOGIN_SUBTITLE);

		myInput = new HTMLInput(HTMLInputTypes.TEXT,
				LoginHTML.PARAM_USERNAME, "id", usernameText);
		myInput.SetID(PTStyleID.PT_LOGIN_USERNAME_TEXTBOX);
		myInput.SetSize("30");
		myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);

		if (!b508)
		{
			myInput.SetOnKeyPress("return " + PlumtreeHelpers.STR_JS_FUNC_NAME_EXECUTE_VIA_ENTER + "(event);");
		}

		myCell.AddInnerHTMLElement(myInput);

		myRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(myRow);

		myCell = new HTMLTableCell();
		myCell.SetStyleClass(PTStyleClass.LOGIN_SUBTITLE);

		myRow.AddInnerHTMLElement(myCell);

		myCell.SetWidth("40%");
		myCell.SetAlign(CommonHTMLStrings.RIGHT);
		
		String passwordText = asOwner.GetString(634, "ptmsgs_portalbrowsingmsgs");

		// Password:
		myCell.AddInnerHTMLString(passwordText);

		// Password:
		myCell.SetPreComment(passwordText);

		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);

		myCell.SetWidth("60%");
		myCell.SetAlign(CommonHTMLStrings.LEFT);
		myCell.SetStyleClass(PTStyleClass.LOGIN_TEXT);
		myCell.SetPreComment("Password Input");

		myInput = new HTMLInput(HTMLInputTypes.PASSWORD,
				LoginHTML.PARAM_USERPASS, "id", passwordText);
		myInput.SetID(PTStyleID.PT_LOGIN_PASSWORD_FIELD);
		myInput.SetSize("30");
		myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);

		if (!b508)
		{
			myInput.SetOnKeyPress("return " + PlumtreeHelpers.STR_JS_FUNC_NAME_EXECUTE_VIA_ENTER + "(event);");
		}

		myCell.AddInnerHTMLElement(myInput);
		
		//	AUTH SOURCES

		if( (bAllowAuthSourceDropdownDisplay) &&
			(PTLoginHelper.ShowAuthDropdown() || bShowAuthSource) )
		{
			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);

			myCell = new HTMLTableCell();
			myCell.SetStyleClass(PTStyleClass.LOGIN_SUBTITLE);

			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("40%");
			myCell.SetAlign(CommonHTMLStrings.RIGHT);

			// Authentication Source:
			myCell.AddInnerHTMLString(asOwner.GetString(635, "ptmsgs_portalbrowsingmsgs"));

			myCell = new HTMLTableCell();
			myCell.SetStyleClass(PTStyleClass.LOGIN_TEXT);

			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("40%");
			myCell.SetAlign(CommonHTMLStrings.LEFT);

			// This select is cached on the application and shared between
			// users, so we can't edit it in any way.
			myCell.AddInnerHTMLElement(LoginHTML.GetAuthSourceSelect(asOwner,
					asOwner.GetApplication(), b508, sUniquifier));				
				
		}
		else if (bAllowAuthSourceDropdownDisplay)
		{
			// Create link to display auth source dropdown

			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);

			myCell = new HTMLTableCell();
			myCell.SetStyleClass(PTStyleClass.LOGIN_TEXT);

			myRow.AddInnerHTMLElement(myCell);

			myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);


			myCell = new HTMLTableCell();
			myCell.SetStyleClass(PTStyleClass.LOGIN_SUBTITLE);

			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("40%");
			myCell.SetAlign(CommonHTMLStrings.LEFT);
				
			//	Default with link to page presenting options
				
			ASURL myURL = new ASURL(asOwner);
			myURL.SetLinkGetSpaceIfCached( sLoginASName, asOwner );
			myURL.SetControl( sControlName );
			myURL.AddControlArgument( LoginHTML.PARAM_SHOWAUTHSOURCES, "true"  );
				
			HTMLElement eTemp = myURL.GetURLAsHTMLElement();

			// Show Auth Sources
			eTemp.AddInnerHTMLEncodedString( asOwner.GetString( 1566, "ptmsgs_portalbrowsingmsgs" ) );
			
			myCell.AddInnerHTMLElement( eTemp );
		}

		if( PTLoginHelper.ShowRememberMyPassword() )
		{
			//	REMEMBER MY PASSWORD
	
			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);
	
			myCell = new HTMLTableCell();
			myCell.SetStyleClass(PTStyleClass.LOGIN_TEXT);
	
			myRow.AddInnerHTMLElement(myCell);
	
			myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
	
			myCell = new HTMLTableCell();
			myCell.SetStyleClass(PTStyleClass.LOGIN_TEXT);
	
			myRow.AddInnerHTMLElement(myCell);
	
			myCell.SetAlign(CommonHTMLStrings.LEFT);
			
			String rememberPasswordText = asOwner.GetString(636, "ptmsgs_portalbrowsingmsgs");
	
			myInput = new HTMLInput(HTMLInputTypes.CHECKBOX,
					LoginHTML.PARAM_REMEMBER, rememberPasswordText);
			myInput.SetID(PTStyleID.PT_LOGIN_REMEMBER_CHECKBOX);
			myInput.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
	
			// Remember my Password
			myCell.SetPreComment(rememberPasswordText);
			myCell.AddInnerHTMLElement(myInput);
	
			// Remember my Password
			myCell.AddInnerHTMLString(rememberPasswordText);
		}

		myRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(myRow);
			
		myCell = new HTMLTableCell(); //empty cell
		myRow.AddInnerHTMLElement(myCell);

		myCell = new HTMLTableCell();
		myCell.SetStyleClass(PTStyleClass.LOGIN_TEXT);
//		myCell.SetColSpan(CommonHTMLStrings.TWO);
		myCell.SetAlign(CommonHTMLStrings.LEFT);

		myRow.AddInnerHTMLElement(myCell);

		// Note: Why do 508 and normal login use different login strings?
		if (!b508)
		{
			//	LOGIN BUTTON
			HTMLInput myLoginInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_LOGIN, asOwner.GetString(1460, "ptmsgs_portalbrowsingmsgs"));
			myLoginInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
			myLoginInput.SetOnClick(STR_JS_FUNC_NAME_DO_LOGIN + sUniquifier + "(); return false;");
			// Log In
			
			String loginText = asOwner.GetString(1460, "ptmsgs_portalbrowsingmsgs");
			myLoginInput.SetTitle(loginText);
			myLoginInput.SetValue(loginText);
				
			myCell.AddInnerHTMLElement(myLoginInput);
		}
		else
		{
			// "Login"
			String strLogin = asOwner.GetString(640, "ptmsgs_portalbrowsingmsgs");
				 
			//	508 LOGIN BUTTON
			myInput = new HTMLInput(HTMLInputTypes.SUBMIT_TYPE,
					LoginHTML.PARAM_DOLOGIN_508,
					strLogin);
			myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);

			myInput.SetValue(strLogin);

			myCell.AddInnerHTMLElement(myInput);
		}


		if( PTLoginHelper.GetAllowUserCreation( asOwner, asOwner.GetApplication() ) )
		{		
	
			if (myCreateLink != null)
			{					
				// CREATE ACCOUNT LINK
				//   This button can just be a redirect because the form
				//   needn't be submitted.
				HTMLSpan createSpan = new HTMLSpan();
				createSpan.SetID(PTStyleID.PT_LOGIN_CREATE_ACCOUNT);
				myCell.AddInnerHTMLElement(createSpan);

				createSpan.AddInnerHTMLString(CommonHTMLStrings.SPACE);
				createSpan.AddInnerHTMLElement(myCreateLink);
				createSpan.AddInnerHTMLString(CommonHTMLStrings.SPACE);
			}
			
		}

		
		return myCell;
	}

	/**
	 * This method returns the javascript for the login HTML.
	 */
	public static HTMLScript GetLoginJScript(boolean b508, String sFormName, String sUniquifier)
	{
		HTMLScript myScript = null;

		if (!b508)
		{
			String strJSUtilVersionPrefix = ConfigHelper.GetJSIncluderVersionPrefix(ConfigHelper.JSCOMPONENT_JSUTIL);
			
			myScript = PlumtreeHelpers.GetExecuteViaEnterJScript(STR_JS_FUNC_NAME_DO_LOGIN + sUniquifier);

			XPStringBuilder sb = new XPStringBuilder();
			
			sb.Append("\n\nfunction doOnLoad").Append(sUniquifier).Append("()\n");
			sb.Append("{\n");
			sb.Append("    restoreLoginInfo").Append(sUniquifier).Append("();\n");
			sb.Append("}\n\n");

			sb.Append("function getUserTextbox").Append(sUniquifier).Append("()\n");
			sb.Append("{\n");
			sb.Append("    return document.getElementById('" + PTStyleID.PT_LOGIN_USERNAME_TEXTBOX + "');\n");
			sb.Append("}\n\n");

			sb.Append("function getPasswordField").Append(sUniquifier).Append("()\n");
			sb.Append("{\n");
			sb.Append("    return document.getElementById('" + PTStyleID.PT_LOGIN_PASSWORD_FIELD + "');\n");
			sb.Append("}\n\n");

			sb.Append("function getAutoSourceSelect").Append(sUniquifier).Append("()\n");
			sb.Append("{\n");
			sb.Append("    return document.getElementById('" + PTStyleID.PT_LOGIN_AUTH_SELECT + "');\n");
			sb.Append("}\n");

			sb.Append("function getRememberCheckbox").Append(sUniquifier).Append("()\n");
			sb.Append("{\n");
			sb.Append("    return document.getElementById('" + PTStyleID.PT_LOGIN_REMEMBER_CHECKBOX + "');\n");
			sb.Append("}\n\n");

			sb.Append("function " + STR_JS_FUNC_NAME_DO_LOGIN + "").Append(sUniquifier).Append("()\n");
			sb.Append("{\n");
			sb.Append("    var persistInfo = " + XPConvert.ToString(bAllowLoginInfoPersistenceBetweenLogins) + ";\n");
			sb.Append("    var cb = getRememberCheckbox").Append(sUniquifier).Append("();\n");
			sb.Append("    if (!persistInfo || !cb || !cb.checked)\n");
			sb.Append("    {\n");
			sb.Append("        deleteLoginInfo").Append(sUniquifier).Append("();\n");
			sb.Append("    }\n");
			sb.Append("    else\n");
			sb.Append("    {\n");
			sb.Append("        var authSource = 0;\n");
			sb.Append("        var sel = getAutoSourceSelect").Append(sUniquifier).Append("();\n");
			sb.Append("        if (sel) { authSource = sel.selectedIndex; }\n");
			sb.Append("        storeLoginInfo").Append(sUniquifier).Append("(authSource);\n");
			sb.Append("    }\n");

			sb.Append("    var loginForm = document." + sFormName + ";\n");
			sb.Append("    var hiddenLogin = loginForm." + LoginHTML.PARAM_DOLOGIN + ";\n");
			sb.Append("    hiddenLogin.value = '" + XPConvert.ToString(true) + "';\n");
			
			// Fix for session fixation bug #7824904, the fix is different for java and dot
			// In dotnet portal we remove the asp.net session id cookie in javascript when login form is submitted. IIS will
			// generate a new session id on the next request. In java we invalidate the session on the server after the login
			// post request. This is done in LoginControl and XPPage
			if (XPSystem.IsRunningInDotNet()) {
				sb.Append("    document.cookie= \"ASP.NET_SessionId=; path=/; expires=Thu, 01-Jan-1970 00:00:01 GMT\" \n");
			}
			
			sb.Append("    loginForm.submit();\n");
			sb.Append("}\n\n");

			sb.Append("function deleteLoginInfo").Append(sUniquifier).Append("()\n");
			sb.Append("{\n");
			sb.Append("    writeLoginCookie").Append(sUniquifier).Append("('','');\n");
			sb.Append("}\n\n");

			sb.Append("function storeLoginInfo").Append(sUniquifier).Append("(authSource)\n");
			sb.Append("{\n");
			sb.Append("    var value = 'auth\\t' + authSource;\n");
			sb.Append("    var expires = "+strJSUtilVersionPrefix+"PTCookie.daysAway(60);\n");
			sb.Append("    writeLoginCookie").Append(sUniquifier).Append("(value,expires);\n");
			sb.Append("}\n\n");

			sb.Append("function restoreLoginInfo").Append(sUniquifier).Append("()\n");
			sb.Append("{\n");
			sb.Append("    var cookie = "+strJSUtilVersionPrefix+"PTCookie.get('ptLastLoginAuthSource');\n");
			sb.Append("    if (cookie)\n");
			sb.Append("    {\n");
			sb.Append("        var vals = cookie.split('\\t');\n");
			sb.Append("        var key = vals[0];\n");
			sb.Append("        var val = vals[1];\n");
			sb.Append("        if (key == 'auth')\n");
			sb.Append("        {\n");
			sb.Append("           var sel = getAutoSourceSelect").Append(sUniquifier).Append("();\n");
			sb.Append("           var src = parseInt(val);\n");
			sb.Append("           if (sel && !isNaN(src)) { sel.selectedIndex = src; }\n");
			sb.Append("        }\n");
			sb.Append("        var cb = getRememberCheckbox").Append(sUniquifier).Append("();\n");
			sb.Append("        if (cb) { cb.checked = true; }\n");
			sb.Append("        focusUserTextbox").Append(sUniquifier).Append("();\n");
			sb.Append("    }\n");
			sb.Append("}\n\n");

			sb.Append("function focusUserTextbox").Append(sUniquifier).Append("()\n");
			sb.Append("{\n");
			sb.Append("    var usr = getUserTextbox").Append(sUniquifier).Append("();\n");
			sb.Append("    if (usr) { usr.focus(); }\n");
			sb.Append("}\n\n");

			sb.Append("function focusPasswordField").Append(sUniquifier).Append("()\n");
			sb.Append("{\n");
			sb.Append("    var pw = getPasswordField").Append(sUniquifier).Append("();\n");
			sb.Append("    if (pw) { pw.focus(); }\n");
			sb.Append("}\n\n");

			sb.Append("function writeLoginCookie").Append(sUniquifier).Append("(value,expires)\n");
			sb.Append("{\n");
			sb.Append("    "+strJSUtilVersionPrefix+"PTCookie.set('ptLastLoginAuthSource',value,expires);\n");
			sb.Append("}\n\n");
			
			sb.Append("document.PCC.RegisterForWindowEvent('onload',restoreLoginInfo").Append(sUniquifier).Append(");\n\n");

			myScript.AddInnerHTMLString(sb.ToString());
		}

		return myScript;
	}

	/**
	 * This method creates the login text boxes.
	 */
	public static HTMLTable GetLoginTextBoxes(AActivitySpace asOwner, boolean b508, String sUniquifier) {
		HTMLTable myTable;
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLInput myInput;

		myTable = new HTMLTable();

		//myForm.AddInnerHTMLElement(myTable);

		myTable.SetCellPadding(CommonHTMLStrings.TWO);
		myTable.SetCellSpacing(CommonHTMLStrings.ZERO);

		myTable.SetAlign(CommonHTMLStrings.CENTER);
		myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);

		myRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(myRow);

		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);

		myCell.SetColSpan(CommonHTMLStrings.TWO);
		myCell.SetStyleClass(PTStyleClass.LOGIN_TEXT);

		myRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(myRow);

		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);

		myCell.SetWidth("40%");
		myCell.SetAlign(CommonHTMLStrings.RIGHT);
		myCell.SetStyleClass(PTStyleClass.LOGIN_SUBTITLE);


		myCell.SetPreComment(asOwner.GetString(633,
				"ptmsgs_portalbrowsingmsgs"));


		myCell.AddInnerHTMLString(asOwner.GetString(633,
				"ptmsgs_portalbrowsingmsgs"));

		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);

		myCell.SetWidth("60%");
		myCell.SetAlign(CommonHTMLStrings.LEFT);
		myCell.SetStyleClass(PTStyleClass.LOGIN_SUBTITLE);

		myInput = new HTMLInput(HTMLInputTypes.TEXT,
				LoginHTML.PARAM_USERNAME, "id", "alt");



		myInput.SetSize("30");
		myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);

		if (!b508)
		{
			myInput.SetOnKeyPress("return " + PlumtreeHelpers.STR_JS_FUNC_NAME_EXECUTE_VIA_ENTER + "(event);");
		}

		myCell.AddInnerHTMLElement(myInput);

		myRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(myRow);

		myCell = new HTMLTableCell();
		myCell.SetStyleClass(PTStyleClass.LOGIN_SUBTITLE);

		myRow.AddInnerHTMLElement(myCell);

		myCell.SetWidth("40%");
		myCell.SetAlign(CommonHTMLStrings.RIGHT);


		myCell.AddInnerHTMLString(asOwner.GetString(634,
				"ptmsgs_portalbrowsingmsgs"));


		myCell.SetPreComment(asOwner.GetString(634,
				"ptmsgs_portalbrowsingmsgs"));

		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);

		myCell.SetWidth("60%");
		myCell.SetAlign(CommonHTMLStrings.LEFT);
		myCell.SetStyleClass(PTStyleClass.LOGIN_TEXT);
		myCell.SetPreComment("Password Input");

		myInput = new HTMLInput(HTMLInputTypes.PASSWORD,
				LoginHTML.PARAM_USERPASS, "id", "alt");

		myInput.SetSize("30");
		myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);

		if (!b508)
		{
			myInput.SetOnKeyPress("return " + PlumtreeHelpers.STR_JS_FUNC_NAME_EXECUTE_VIA_ENTER + "(event);");
		}

		myCell.AddInnerHTMLElement(myInput);
		
		return myTable;
	}

	/**
	 * Helper method GetPTSession.  Gets IPTSession from AActivitySpace cleanly.
	 * @param as
	 * @return IPTSession
	 */
	public static IPTSession GetPTSession(AActivitySpace aSpace)
	{
		IPTSession ptSession;

		try
		{
			ptSession = (IPTSession) aSpace.GetUserSession();
		}
		catch (Exception ex)
		{
			log.Error(ex, "Couldn't cast UserSession to IPTSession.");

			throw new XPException("The user session must derive from IPTSession.",
				ex);
		}

		return ptSession;
	}
}

