package com.plumtree.portalpages.common.mediator;

import com.plumtree.portaluiinfrastructure.compoundlist.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;

/**
 * This class uses delegation to pass on link generation requests to the
 * appropriate link mediator.  If in 508 or low bandwidth access style, the
 * class delegates to CListURLFullMediator.  If not, it delegates to
 * CListURLTemplateLinkMediator.
 * 
 * This class is not a real CListURLMediator in itself. It dynamically decides 
 * which CListURLMediator to delegate to given the Display Option from the HTML
 * source.
 * 
 * From a practical point of view, use the Model methods to get an 
 * ICPListEntryIterator appropriate for the navigation you're constructing, 
 * and then use a mediator to quickly form HTMLAnchors for those links. 
 * 
 * 
 * @author DavidP
 *
 */
public class CListURLMediator implements ICPListIterator {
	/** copy of the AActivitySpace reference; used to determine access style */
	private AActivitySpace m_aspace; 
	private ICPListIterator m_linkDelegator;
	// Never gets used - private boolean m_bShowImages = true;

	/**
	 * Constructor for CompoundListASURLMediator. Get the iterator interface for the list
	 * @param aspace Activity space the links will be generated for
	 * @param list List of "link entries" used to create anchors
	 */
	public CListURLMediator(AActivitySpace aspace, ICPListEntryIterator list) {
		m_aspace = aspace;

		// instantiate the appropriate delegate
		if (m_aspace.GetAccessStyle() == AccessStyles.STANDARD) {
			// javascript-enabled template-based mediator
			m_linkDelegator =
				(ICPListIterator) (new CListURLTemplateLinkMediator(aspace,
					list));
		} else {
			// full URL, no link templating
			m_linkDelegator =
				(ICPListIterator) (new CListURLFullLinkMediator(aspace, list));
		}
	}

	/**
	 * @see com.plumtree.portalpages.common.mediator.ICPListIterator#GetEntry()
	 */
	public Object GetEntry() {
		//VerifyAccess();
		return m_linkDelegator.GetEntry();
	}

	/**
	 * 
	 * @see com.plumtree.portalpages.common.mediator.ICPListIterator#GetEntryAtIndex(int)
	 */
	public Object GetEntryAtIndex(int nIndex) {
		//VerifyAccess();
		return m_linkDelegator.GetEntryAtIndex(nIndex);
	}

	/**
	 * 
	 * @see com.plumtree.portalpages.common.mediator.ICPListIterator#GetList()
	 */
	public ICPListEntryIterator GetList() {
		return m_linkDelegator.GetList();
	}

	/**
	 * 
	 * @see com.plumtree.portalpages.common.mediator.ICPListIterator#Next()
	 */
	public boolean Next() {
		//VerifyAccess();
		return m_linkDelegator.Next();
	}

	
	/* (non-Javadoc)
	 * @see com.plumtree.portalpages.common.mediator.ICPListIterator#SetLabelMaxLength(int)
	 */
	public void SetLabelMaxLength(int nMax) {
		//VerifyAccess();
		m_linkDelegator.SetLabelMaxLength(nMax);
	}

	/** 
	 * @see com.plumtree.portalpages.common.mediator.ICPListIterator#SetShowImages(boolean)
	 **/
	public void SetShowImages(boolean bShow) {
		m_linkDelegator.SetShowImages(bShow);
	}

	//	/**
	//	 * Method VerifyAccess.  Swaps the underlying delegate as needed, sharing
	//	 * the original list.
	//	 * DEPRECATED: it's unlikely the mode will change during the lifetime of
	//	 * this helper instance.
	//	 */
	//	private void VerifyAccess()
	//	{
	//		// Switch delegates on the fly as needed
	//		if (b_is508 && (m_aspace.GetAccessStyle() == AccessStyles.STANDARD))
	//		{
	//			ICPListEntryIterator list = linkDelegator.GetList();
	//			linkDelegator     = null;
	//			linkDelegator     = (ICPListIterator) (new CListURLTemplateLinkMediator(m_aspace,
	//					list));
	//			b_is508			  = false;
	//		}
	//		else if (!b_is508 &&
	//				(m_aspace.GetAccessStyle() != AccessStyles.STANDARD))
	//		{
	//			ICPListEntryIterator list = linkDelegator.GetList();
	//			linkDelegator     = null;
	//			linkDelegator     = (ICPListIterator) (new CListURLFullLinkMediator(m_aspace,
	//					list));
	//			b_is508			  = true;
	//		}
	//	}
}

