package com.plumtree.portalpages.common.mediator;

import com.plumtree.openlog.*;
import com.plumtree.portalpages.common.uiparts.*;
import com.plumtree.portaluiinfrastructure.compoundlist.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This class extends CListURLTemplateMediator generating HTMLAnchors using URL Templates.
 * Clicking on the URL will construct the full URL and redirect to it. The downside with these
 * links is users can't open the link in a new window with "Open in New Window" option since
 * a javascript onclick event is used to trigger the redirect and not the HREF. See javadoc
 * for CListURLTemplateMediator for more information on URL Templates.
 * 
 * @see CListURLTemplateMediator  
 * 
 * Example use:
 * 
 * 		ICPListIterator mediator = new CListURLTemplateLinkMediator(m_asOwner, cpList1);
 *		mediator.SetLabelMaxLength(50);    //set the maximum label length to 50 characters
 *
 *		while (mediator.Next())
 *		{
 *			HTMLAnchor anchor = (HTMLAnchor) mediator.GetEntry();
 *			...
 *		}
 * 
 * @author robertz
 *
 */
public class CListURLTemplateLinkMediator extends CListURLTemplateMediator
	implements ICPListIterator
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
		"com.plumtree.portalpages.common.mediator.CListURLTemplateLinkMediator");

	/**
	 * Constructor for CListURLTemplateLinkMediator.
	 * @param aspace			The current Activityspace
	 * @param list				The ASCompoundlist used by this mediator
	 */
	public CListURLTemplateLinkMediator(AActivitySpace aspace,
		ICPListEntryIterator list)
	{
		super(aspace, list);
		// Set to top by default
		m_strImageAlign = CommonHTMLStrings.ABSMIDDLE;
	}

	/**
	 * @see com.plumtree.portalpages.common.mediator.ICPListIterator#GetEntry()
	 *
	 * @return 						An HTMLAnchor with the URL Template as onclick action
	 * 								and poundsign as href. Also includes image if one is
	 * 								set on the link.
	 *  
	 */
	public Object GetEntry()
	{
		HTMLAnchor anchor    = null;
		String[] arrTemplate = (String[]) super.GetEntry();

		try
		{
			// No URL, set the HREF to be null. An HTMLAnchor with null HREF will just output its innerHTML. 
			// Since we are adding just a string (and sometimes an image), the generated HTML will just be a string
			// and an image if one is set.
			if (1 == arrTemplate.length || null == arrTemplate[CListURLTemplateMediator.TEMPLATEARR_INDEX_URL] )
			{
				anchor = new HTMLAnchor(null);
			}
			else
			{
				anchor = new HTMLAnchor("#");
				anchor.SetOnClick(NavigationCommonHelpers.URLTEMPLATE_REDIRECT_JSFUNCNAME+"('" +	arrTemplate[CListURLTemplateMediator.TEMPLATEARR_INDEX_URL] + "'); return false;"); 
			}

			// Has image, add the html for the image.
			if ( IsImageEntry(arrTemplate) )
			{
				anchor.AddInnerHTMLString(arrTemplate[CListURLTemplateMediator.TEMPLATEARR_INDEX_IMGURL]);
			}

			String label = arrTemplate[CListURLTemplateMediator.TEMPLATEARR_INDEX_LABEL];
			// Add the label as innerHTML.
			anchor.AddInnerHTMLString(label);
			
			// Make a tooltip with the original label for truncated label strings.
			if ( label.endsWith("...") ) 
				anchor.SetTitle(super.GetNonEncodedEntryOrigLabel());
		}
		catch (HTMLException e)
		{
			
				log.Error("Error adding String to anchor tag, the String was: " +
					arrTemplate[0]);
			
		}

		return anchor;
	}

	/**
	 * @see com.plumtree.portalpages.common.mediator.ICPListIterator#GetEntry()
	 *
	 * @return 						An HTMLAnchor with the URL Template as onclick action
	 * 								and poundsign as href.
	 * 
	 */
	public Object GetEntryAtIndex(int nIndex)
	{
		HTMLAnchor anchor    = null;
		String[] arrTemplate = (String[]) super.GetEntryAtIndex(nIndex);

		try
		{
			// No URL, set the HREF to be null. An HTMLAnchor with null HREF will just output its innerHTML. 
			// Since we are adding just a string (and sometimes an image), the generated HTML will just be a string
			// and an image if one is set.
			if (1 == arrTemplate.length)
			{
				anchor = new HTMLAnchor(null);
			}
			else
			{
				// Set the onclick action to be a call to the templateURLRedirect which will construct the
				// full URL and redirect to it. The HREF is actually not used.
				anchor = new HTMLAnchor("#");
				anchor.SetOnClick(NavigationCommonHelpers.URLTEMPLATE_REDIRECT_JSFUNCNAME+"('" + arrTemplate[TEMPLATEARR_INDEX_URL] + "'); return false;");
			}

			// Has image, add the html for the image.
			if (arrTemplate.length == 3)
			{
				anchor.AddInnerHTMLString(arrTemplate[TEMPLATEARR_INDEX_IMGURL]);
			}
			// Add the label as innerHTML.
			anchor.AddInnerHTMLString(arrTemplate[TEMPLATEARR_INDEX_LABEL]);
		}
		catch (HTMLException e)
		{
			
				log.Error("Error adding String to anchor tag, the String was: " +
					arrTemplate[0]);
			
		}

		return anchor;
	}
}

