package com.plumtree.portalpages.common.mediator;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.directory.*;
import com.plumtree.portaluiinfrastructure.compoundlist.*;
import com.plumtree.portaluiinfrastructure.friendlyurl.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Created on Oct 29, 2002
 * 
 * Mediator for ASCompoundList, takes values from ASCompoundList and returns
 * URL Templates, comma separated String with the type of URL and its parameters.
 * A CommunityPage URL would look like 'C,201,-201' where C stands for Community
 * Template, 201 for the community id and -201 for page id.
 * 
 * URL Templates are used to lessen the data being sent down to the client. The full 
 * URL is constructed by calling the buildURLFromTemplate javascript function (use the
 * NavigationCommonHelpers.JSCRIPT_FUNC_BUILD_URL_FROM_TEMPL constant). 
 * The templateURLRedirect javascript function (use the NavigationCommonHelpers.URLTEMPLATE_REDIRECT_JSFUNCNAME)
 * to construct the full url and redirect to it.
 *
 * This mediator is for menus with their own onclick event handling. Use the CListURLTemplateLinkMediator 
 * to get an HTMLAnchor elements that redirects to the url in the template.
 * 
 * Example use:
 * 
 *  	ICPListIterator mediator = new CListURLTemplateMediator(m_asOwner, cpList1);
 *		mediator.SetLabelMaxLength(50);    //set the maximum label length to 50 characters
 *
 *		while (mediator.Next())
 *		{
 *			String[] arURLTempl = (String[]) mediator.GetEntry();
 * 
 * 			String strLabel 	= arURLTempl[CListURLTemplateMediator.TEMPLATEARR_INDEX_LABEL];
 * 			String strURL		= arURLTempl[CListURLTemplateMediator.TEMPLATEARR_INDEX_URL];
 * 			String strImageURL	= arURLTempl[CListURLTemplateMediator.TEMPLATEARR_INDEX_IMGURL];
 * 			...
 *		}
 *       
 *  
 * @author robertz
 */
public class CListURLTemplateMediator implements ICPListIterator
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
		"com.plumtree.portalpages.common.mediator.CListURLTemplateMediator");

	/** Index constants for the elements in the array generated by this mediator */
	public static final int TEMPLATEARR_INDEX_LABEL  = 0;
	public static final int TEMPLATEARR_INDEX_URL    = 1;
	public static final int TEMPLATEARR_INDEX_IMGURL = 2;
	
	/** 
	 * The constant amount of space in pixels images should take up.
	 * A spacer image will be added as padding if images doesn't fill
	 * up the entire space. 
	 **/
	public static final int IMAGE_SPACE_WIDTH			= 24;
	public static final int ACTIONIMAGE_SPACE_WIDTH		= 30;
	
	private AActivitySpace m_aspace;
	private ICPListEntryIterator m_list;
	
	// Default values
	private int m_nLabelMaxLength		 = 20;
	private boolean m_bShowImages = true;

	
	// Needs to be aligned to top to show up decent in dropdown menus in Netscape
	// Should be aigned to absmiddle in all other cases, set in child classes
	protected String m_strImageAlign = "";

	/**
	 * Constructor for CompoundListASURLMediator. Get the iterator interface for the list
	 * @param aspace 		Current Activityspace
	 * @param list 			ICPListEntryIterator to wrap
	 */
	public CListURLTemplateMediator(AActivitySpace aspace,
		ICPListEntryIterator list)
	{
		m_strImageAlign = CommonHTMLStrings.ABSMIDDLE;
		m_list		 = list;
		m_aspace     = aspace;
		list.GetIterator();
	}


	/**
	 * Late added method, get the untruncated label of the current entry.
	 * 
	 */
	public String GetEntryOrigLabel()
	{
		ListEntry entry = (ListEntry) m_list.GetEntry();
		return XPHttpUtility.HtmlEncode(entry.GetOrigLabel());	
	}
	
	/**
	 * Late added method, get the untruncated label of the current entry (non encoded).
	 * 
	 */
	public String GetNonEncodedEntryOrigLabel()
	{
		ListEntry entry = (ListEntry) m_list.GetEntry();
		return entry.GetOrigLabel();	
	}

	/**
	 * @see com.plumtree.portalpages.common.mediator.ICPListIterator#GetEntry()
	 *
	 * @return A String array. 
	 * First element with the label of the link.
	 * Second element with the URL Template String, a comma separated string
	 * with the link type and parameters for link.
	 * Third (optional), HTML string with image or images.
	 */
	public Object GetEntry()
	{
		ListEntry entry = (ListEntry) m_list.GetEntry();

		return GetURLFromListEntry(entry);
	}

	/**
	 * 
	 * @see com.plumtree.portalpages.common.mediator.ICPListIterator#GetEntryAtIndex(int)
	 */
	public Object GetEntryAtIndex(int nIndex)
	{
		ListEntry entry = (ListEntry) m_list.GetEntryAtIndex(nIndex);

		return GetURLFromListEntry(entry);
	}

	/**
	 * 
	 * @see com.plumtree.portalpages.common.mediator.ICPListIterator#GetList()
	 */
	public ICPListEntryIterator GetList()
	{
		return m_list;
	}

	/**
	 * @see com.plumtree.portalpages.common.mediator.ICPListIterator#Next()
	 */
	public boolean Next()
	{
		return m_list.Next();
	}


	/**
	 * 
	 * @see com.plumtree.portalpages.common.mediator.ICPListIterator#SetLabelMaxLength(int)
	 */
	public void SetLabelMaxLength(int nMax)
	{
		m_nLabelMaxLength = nMax;
	}
	
	/**
	 * 
	 * @see com.plumtree.portalpages.common.mediator.ICPListIterator#SetShowImages(boolean)
	 */
	public void SetShowImages(boolean bShow)
	{
		m_bShowImages = bShow;
	}
	
	

	/**
	 * Creates a Template URL for Community pages
	 *
	 * @param entry			Current listentry element 
	 * @return 				String array with string label, url template string and image string if assigned
	 */
	protected String[] CreateCommPageURLTemplate(ListEntry entry)
	{
		String label = entry.GetLabel();
		int nCommID		= entry.GetCommunityID();
		int nPageID		= entry.GetPageID();
		String imageString = SetListEntryImage(entry);
		boolean bUseFriendlyURL = ((-1 != entry.GetProfileID()) ? TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_USER_ID) : TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_COMMUNITY_ID));
		String strProfileID = ((-1 != entry.GetProfileID()) ? ","+entry.GetProfileID() : "" );
		
		if (bUseFriendlyURL) {
			FriendlyURL friendly;
			if (-1 != entry.GetProfileID()) {
				friendly = new FriendlyURL(PT_CLASSIDS.PT_USER_ID, entry.GetProfileID());
			} else {
				friendly = new FriendlyURL(PT_CLASSIDS.PT_COMMUNITY_ID, nCommID);
			}
			if (nPageID > 0) {
				friendly.SetChildId(nPageID);
			}
			if (imageString == null) {
				return new String[] { label, "IF," + friendly.GetFullFriendlyURL(m_aspace)};
			} else {
				return new String[] { label, "IF," + friendly.GetFullFriendlyURL(m_aspace), imageString};
			}
		} else {
			if (imageString == null) {
				return new String[] { label, "C," + nCommID + "," + nPageID + strProfileID};
			} else {
				return new String[] { label, "C," + nCommID + "," + nPageID + strProfileID, imageString };
			}
		}
		
	}

	/**
	 * Creates a Template URL for Community Knowledge Directory Root
	 *
	 * @param entry			Current listentry element 
	 * @return 				String array with string label, url template string and image string if assigned
	 */
	protected String[] CreateSiteMapRootURLTemplate(ListEntry entry)
	{
		String label = entry.GetLabel();
		int nCommID	= entry.GetCommunityID();
		int nMode = entry.GetMode();		
		boolean bIsPreview = entry.GetIsPreview();
		String imageString = SetListEntryImage(entry);
		if (imageString == null) {
			return new String[] { label, "SM," + nCommID + "," + nMode + "," + XPConvert.ToString(bIsPreview)};
		} else {
			return new String[] { label, "SM," + nCommID + "," + nMode + "," + XPConvert.ToString(bIsPreview), imageString};
		}
	}

	/**
	 * Create a Template URL for Directory pages
	 *
	 * @param entry			Current listentry element 
	 * @return 				String array with string label, url template string and image string if assigned
	 */
	protected String[] CreateDirPageURLTemplate(ListEntry entry)
	{
		String label 	= entry.GetLabel();
		int nMode 		= entry.GetMode();
		int nFolderID 	= entry.GetFolderID();
		String imageString = SetListEntryImage(entry);
		boolean bUseFriendlyURL = TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_CATALOGFOLDER_ID);
		
		if (bUseFriendlyURL) {
			FriendlyURL friendly = new FriendlyURL(PT_CLASSIDS.PT_CATALOGFOLDER_ID, nFolderID);
			String strDirMode = "";
			if (nMode > 1) {
				strDirMode = "?" + DirConstants.QS_DIRECTORY_MODE + "=" + nMode;
			}
			if (imageString == null) {
				return new String[] { label, "IF," + friendly.GetFullFriendlyURL(m_aspace) + strDirMode};
			} else {
				return new String[] { label, "IF," + friendly.GetFullFriendlyURL(m_aspace) + strDirMode, imageString};
			}
		} else {
			if (imageString == null) {
				return new String[] { label, "D," + nMode + "," + nFolderID };
			} else {
				return new String[] { label, "D," + nMode + "," + nFolderID, imageString };
			}
		}
	}

	/**
	 * Creates a Template URL for document pages
	 *
	 * @param entry			Current listentry element 
	 * @return 				String array with string label, url template string and image string if assigned
	 */
	protected String[] CreateDocURL(ListEntry entry)
	{
		String label 	= entry.GetLabel();
		int nDocID	= entry.GetPageID();
		
		
		String imageString = SetListEntryImage(entry);
		if (imageString == null) {
			return new String[] { label, "DO," + nDocID };
		} else {
			return new String[] { label, "DO," + nDocID, imageString };
		}
		
	}

	/**
	 * Create a Template URL for editor pages
	 *
	 * @param entry			Current listentry element 
	 * @return 				String array with string label, url template string and image string if assigned
	 */
	protected String[] CreateEditorURLTemplate(ListEntry entry)
	{
		String label		 = entry.GetLabel();
		String editorSpace   = entry.GetEditorAS();
		String control		 = entry.GetControl();
		String strPage		 = entry.GetPage();
		String strEditorType = entry.GetEditorType();
		String[] paramNames  = entry.GetParamNames();
		String[] paramValues = entry.GetParamValues();

		XPStringBuilder buf = new XPStringBuilder(40);
		buf.Append("E," + editorSpace);

		if (null != control)
		{
			buf.Append("," + ASConstants.CONTROL + "=" + control);
		}

		if (null != strPage)
		{
			buf.Append("," + ASConstants.PAGE_INPUT + "=" + strPage);
		}

		if (null != strEditorType)
		{
			buf.Append("," + EditorStartControl.QS_EDITOR_TYPE + "=" +
				strEditorType);
		}

		if (paramNames != null)
		{
			for (int i = 0; i < paramNames.length; i++)
			{
				buf.Append("," + paramNames[i] + "=" + paramValues[i]);
			}
		}

		String imageString = SetListEntryImage(entry);
		if (imageString == null) {
			return new String[] { label, buf.ToString() };
		} else {
			return new String[] { label, buf.ToString(), imageString };
		}
	}
	
	/**
	 * 
	 * @param entry
	 * @return
	 */
	protected String[] CreateFullInternalURL(ListEntry entry)
	{
		String label = entry.GetLabel();
		String url = entry.GetFullURL();
		String imageString = SetListEntryImage(entry);
		if (imageString == null) {
			return new String[] { label, "IF," + url};
		} else {
			return new String[] { label, "IF," + url, imageString };
		}
		
	}

	/**
	 * Creates a full url with associated label and image if one
	 *
	 * @param entry			Current listentry element 
	 * @return 				String array with string label, full url string and image string if assigned
	 */
	protected String[] CreateFullURL(ListEntry entry)
	{
		String label = entry.GetLabel();
		String url = entry.GetFullURL();
		String imageString = SetListEntryImage(entry);
		if (imageString == null) {
			return new String[] { label, "F," + url};
		} else {
			return new String[] { label, "F," + url, imageString };
		}
	}

	/**
	 * Creates a generic Plumtree Portal 50 URL
	 *
	 * @param entry			Current listentry element 
	 * @return 				String array with string label, url template string and image string if assigned
	 */
	protected String[] CreateGenericURL(ListEntry entry)
	{
		XPStringBuilder buf = new XPStringBuilder(40);
		String label 		= entry.GetLabel();
		String strAS		= entry.GetEditorAS();
		String[] paramNames = entry.GetParamNames();
		String[] paramValues =entry.GetParamValues();
		int nASType			= entry.GetNewASType();

		if (nASType == ASCompoundList.NEWASTYPE_CREATENEW)
		{
			buf.Append("G1," + strAS);
		}
		else
		{
			
				log.Error("This Activityspace creation type is currently not supported for this mediator.");
			
		}

		if (null != paramNames)
		{
			for (int i = 0; i < paramNames.length; i++)
			{
				buf.Append("&" + paramNames[i] + "=" + paramValues[i]);
			}
		}

		

		String imageString = SetListEntryImage(entry);
		if (imageString == null) {
			return new String[] { label, buf.ToString() };
		} else {
			return new String[] { label, buf.ToString(), imageString };
		}
		
	}

	/**
	 * Creates a Template URL for mypages
	 *
	 * @param entry			Current listentry element 
	 * @return 				String array with string label, url template string and image string if assigned
	 */
	protected String[] CreateMyPageURLTemplate(ListEntry entry)
	{
		String label	= entry.GetLabel();
		int nPageID		= entry.GetPageID();
		String imageString = SetListEntryImage(entry);
		boolean bUseFriendlyURL = TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_MYPAGESCONTAINER_ID);
		
		if (bUseFriendlyURL) {
			FriendlyURL friendly = new FriendlyURL(PT_CLASSIDS.PT_MYPAGESCONTAINER_ID, nPageID);
			if (imageString == null) {
				return new String[] { label, "IF," + friendly.GetFullFriendlyURL(m_aspace)};
			} else {
				return new String[] { label, "IF," + friendly.GetFullFriendlyURL(m_aspace), imageString};
			}
		} else {
			if (imageString == null) {
				return new String[] { label, "M," + nPageID };
			} else {
				return new String[] { label, "M," + nPageID, imageString };
			}
		}
	}

	/**
	 * Creates a Template URL for users preferences
	 *
	 * @param entry			Current listentry element 
	 * @return 				String array with string label, url template string and image string if assigned
	 */
	protected String[] CreatePrefsURL(ListEntry entry)
	{
		String label = entry.GetLabel();
		String strPage = entry.GetPage();
	
		String imageString = SetListEntryImage(entry);
		if (imageString == null) {
			return new String[] { label, "P," + strPage };
		} else {
			return new String[] { label, "P," + strPage, imageString };
		}
	}

	/**
	 * Creates an array with just a label without url but possible image
	 *
	 * @param entry			Current listentry element 
	 * @return 				String array with string label, url template string and image string if assigned
	 */
	protected String[] CreateStringLabel(ListEntry entry)
	{
		String label = entry.GetLabel();
		String imageString = SetListEntryImage(entry);
		if (imageString == null) {
			return new String[] { label };
		} else {
			return new String[] { label, null, imageString };
		}
	}

	/**
	 * Creates a Template URL for user profile pages
	 *
	 * @param entry			Current listentry element 
	 * @return 				String array with string label, url template string and image string if assigned
	 */
	protected String[] CreateUserURL(ListEntry entry)
	{
		String label = entry.GetLabel();
		int nUserID	= entry.GetPageID();
		String imageString = SetListEntryImage(entry);
		if (imageString == null) {
			return new String[] { label, "UP," + nUserID };
		} else {
			return new String[] { label, "UP," + nUserID, imageString };
		}
	}

	/**
	 * Creates a string containing the html for the assigned images
	 *
	 * @param entry			Current listentry element 
	 * @return 				String with the html for the assigned images
	 */
	protected String GetImageString(ListEntry entry)
	{
		PTImageType image     = entry.GetImage();
		PTImageType image2	  = entry.GetImage2();
		String strImageAlt    = entry.GetImageAltTextEncoded();
		int nTotalWidth		  = 0;

		HTMLImg htmlImg    = null;
		String imageString = "";

		if (null == image || !m_bShowImages)
		{
			return imageString;
		}

		try
		{
			htmlImg	= (HTMLImg) UIConstructFactory.GetConstruct(image, true, this.m_aspace).GetHTMLElement();
			nTotalWidth += XPConvert.ToInteger( htmlImg.GetWidth() );
			htmlImg.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_aspace));
			htmlImg.SetAlt(strImageAlt);
				
			htmlImg.SetAlign(m_strImageAlign);
			htmlImg.SetBorder(CommonHTMLStrings.ZERO);

			imageString = htmlImg.GetDisplayString();
			if ( null != image2) 
			{
				htmlImg	= (HTMLImg) UIConstructFactory.GetConstruct(image2, true, this.m_aspace).GetHTMLElement();
				nTotalWidth += XPConvert.ToInteger( htmlImg.GetWidth() );
				htmlImg.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_aspace));
				htmlImg.SetAlt(strImageAlt);
				htmlImg.SetAlign(m_strImageAlign);
				htmlImg.SetBorder(CommonHTMLStrings.ZERO);
	
				// HTMLEncode the string, might contain "
				//imageString = XPHttpUtility.HtmlEncode( htmlImg.GetDisplayString() );
				imageString += htmlImg.GetDisplayString();
			}
			
			if ( nTotalWidth < IMAGE_SPACE_WIDTH ) 
			{
				htmlImg = PlumtreeHelpers.GetSpacerImage(XPConvert.ToString(IMAGE_SPACE_WIDTH-nTotalWidth), "1");
				htmlImg.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_aspace));
				imageString += htmlImg.GetDisplayString();
			} else if ( nTotalWidth >= IMAGE_SPACE_WIDTH ) {
				htmlImg = PlumtreeHelpers.GetSpacerImage(XPConvert.ToString(ACTIONIMAGE_SPACE_WIDTH-nTotalWidth), "1");
				htmlImg.SetImgBaseUrl(ConfigHelper.GetImageServerBaseURL(m_aspace));
				imageString += htmlImg.GetDisplayString();
			}

			
		}
		catch (Exception e)
		{
		}

		return imageString;
	}
	
	/**
	 * Creates a string containing the image source for the assigned image
	 *
	 * @return 				String with the image source for the assigned images
	 */
	public String GetImageSource()
	{
		ListEntry entry = (ListEntry) m_list.GetEntry();
		PTImageType image     = entry.GetImage();
		String imageString = "";

		if (null == image || !m_bShowImages)
		{
			return imageString;
		}

		try
		{
			imageString = ConfigHelper.GetImageServerBaseURL(m_aspace) + image.GetFileName() + "', '" + image.GetWidth();
		}catch (Exception e)
		{
			// TODO CR DLH: Why do we ignore this?
		}
		return imageString;
	}

	/**
	 * Checks if an entry has image assigned.
	 * 
	 * @param entry				Template entry from this mediator.
	 * @return 					true if image(s) exist, false otherwise
	 */	
	public boolean IsImageEntry(String[] entry)
	{
		return ( entry.length == 3 );
	}
	
	/**
	 * Get the image string for the specified entry
	 *  
	 * @param 					entry
	 * @return 					Image HTML String 
	 */
	protected String SetListEntryImage(ListEntry entry)
	{
		if (!m_bShowImages) 
		{
			return null;
		}
		
		PTImageType image = entry.GetImage();
		String imageString = null;

		if (image != null)
		{
			imageString = GetImageString(entry);
		}

		return imageString;	
	}

	/**
	 * Creates the URL Template by delegating to the respective method
	 * based on the URL type.
	 *
	 * @param entry 				Current listentry
	 *
	 * @return 						An URL template
	 */
	private Object GetURLFromListEntry(ListEntry entry)
	{
		entry.SetTruncationLength(m_nLabelMaxLength);

		String[] template = null;

		switch (entry.GetType())
		{
		case ASCompoundList.ENTRYTYPE_MYPAGETEMPL:
			template = CreateMyPageURLTemplate(entry);

			break;

		case ASCompoundList.ENTRYTYPE_COMMPTEMPL:
			template = CreateCommPageURLTemplate(entry);
					

			break;

		case ASCompoundList.ENTRYTYPE_SITEMAPROOTTEMPL:
			template = CreateSiteMapRootURLTemplate(entry);
			
			break;

		case ASCompoundList.ENTRYTYPE_DIRTEMPL:
			template = CreateDirPageURLTemplate(entry);

			break;

		case ASCompoundList.ENTRYTYPE_EDITORTEMPL:
			template = CreateEditorURLTemplate(entry);

			break;

		case ASCompoundList.ENTRYTYPE_STRINGLABEL:
			template = CreateStringLabel(entry);

			break;

		case ASCompoundList.ENTRYTYPE_FULLINTERNALURL:
			template = CreateFullInternalURL(entry);

			break;	
			
		case ASCompoundList.ENTRYTYPE_FULLURL:
			template = CreateFullURL(entry);

			break;

		case ASCompoundList.ENTRYTYPE_PREFSTEMPL:
			template = CreatePrefsURL(entry);

			break;

		case ASCompoundList.ENTRYTYPE_DOCTEMPL:

			//DCA - GetPage returns EditorPage not PageID
			template = CreateDocURL(entry); // using PageID for DocID
			break;

		case ASCompoundList.ENTRYTYPE_USERTEMPL:

			//DCA - GetPage returns EditorPage not PageID
			template = CreateUserURL(entry); // using PageID from UserID
			break;

		case ASCompoundList.ENTRYTYPE_GENERICTEMPL:
			template = CreateGenericURL(entry);

			break;

		default:

			
				log.Error("Entry created with unsupported link type.");
			
		}

		return template;
	}
}

