
package com.plumtree.portalpages.common.mediator;

import com.plumtree.portaluiinfrastructure.compoundlist.*;

/**
 * Iterator interface for iterating through entries
 * 
 * This iterator interface uses the .net style iterator where the index of the iterator starts at -1. 
 * A call to Next() is required to advance to the first element. Example use:
 * 
 * while (iter.Next())
 * {
 * 	 Object entry = iter.GetEntry();
 * }
 * 
 * The interface is implemented by two different types of classes.
 *  
 * First, ASCompoundList, which is used by the NavigationModel when creating lists of links. These 
 * are the same lists that are returneed from methods on the different NavigationPages. They are populated
 * by ListEntry object, objects that store different parts of a link, such as page id, community id, label or image.
 * Cast the result from GetEntry or GetEntryAtIndex to ListEntry.
 * 
 * Example: Getting the Community ID from every community a user is member of
 * 
 * 		CommunityNavPage commP = (CommunityNavPage)m_model.GetPageSpecificMethods();
 * 		ICPListIterator cpList = commP.GetMyCommunities(false); // only communities, no actions
 * 
 *  	while ( cpList.Next() ) 
 * 		{
 *    		ListEntry entry  = (ListEntry)cpList.GetEntry();
 *    		int nCommunityID = entry.GetCommunityID()
 * 
 *    		...
 *  	}
 * 
 * The second type of classes are the mediators. Mediators are simply helper classes that formats an ListEntry objects
 * from an ICPListIterator list to a specific format. For example, CListURLFullLinkMediator creates HTMLAnchor objects
 * with image, label and url for each link.
 * 
 * Example:
 * 		CommunityNavPage commP = (CommunityNavPage)m_model.GetPageSpecificMethods();
 * 		ICPListIterator cpList = commP.GetMyCommunities(true); // both actions and communities
 * 
 * 		ICPListIterator mediator = new CListURLFullLinkMediator(m_asOwner, cpList);
 *		mediator.SetLabelMaxLength(50);    //set the maximum label length to 50 characters
 *
 *		while (mediator.Next())
 *		{
 *			HTMLAnchor anchor = (HTMLAnchor) mediator.GetEntry();
 *			...
 *		}
 *
 * 
 * @see ASCompoundList
 * @see NavigationPage
 * @see ListEntry
 * @see CListURLTemplateLinkMediator
 * @see CListURLFullLinkMediator
 * 
 * 
 * @author robertz
 */
public interface ICPListIterator
{
	/**
	 * Get the current entry in the list. Doesn't advance the iterator
	 *
	 * @return 					Current entry
	 */
	public Object GetEntry();

	/**
	 * Get the entry at a specific index
	 *
	 * @param nIndex 			Index of the entry
	 *
	 * @return 					Entry at specified index
	 */
	public Object GetEntryAtIndex(int nIndex);

	/**
	 * Get an iterator, the index is set to before the first
	 * element, call Next() to advance to first entry.
	 *
	 * @return 					Iterator for current list
	 */
	public ICPListEntryIterator GetList();

	/**
	 * Advances the iterator to next element.
	 *
	 * @return 					false if no more elements exist, true otherwise
	 */
	public boolean Next();

	/**
	 * Set the truncation width for all subsequent entries,
	 * can be altered between entries during an iteration
	 *
	 * @param nMaxLength 		Character position to truncate at
	 */
	public void SetLabelMaxLength(int nMaxLength);
	
	/**
	 * Set to display images for subsequent entries in the list,
	 * can be altered between entries during an iteration
	 * 
	 * @param bShow				true to show images (default), false to no show images in entries
	 */
	public void SetShowImages(boolean bShow);
}

