package com.plumtree.portalpages.pei;

import com.plumtree.portalpages.browsing.myportal.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;


/**
 * This interface was created to allow developers the ability to create
 * functions that can be dynamically discovered by the Plumtree application
 * for the purposes of performing some functionality when a particular
 * mypage based event occurs.  
 * <br><br>
 * NOTE: Should this interface be implemented more than once within the
 * Dynamic Discovery process, the implementations will be processed in the
 * order that they are displayed within the representative XML file. All
 * implementations will be called.
 * <br><br>
 * NOTE: Be careful if implementing this interface more than once,
 * as there could be very significant problems if multiple functions are
 * making changes to the same page name or page type. 
 * <br><br>
 * NOTE: In the case where multiple implementations return a String object, all 
 * the valid Strings (not null and not empty) will be appended together 
 * (separated by a blank space) and the final String will be the error message 
 * displayed on the page.
 */
public interface IMyPortalPageActions
{
    /**
     * This function will perform some functionality after a new MyPage has been
     * created.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _strPageName - Name of the new MyPage
     * @param _nPageTypeID - Type of the new MyPage
     */
    public void OnAfterAddMyPortalPage(AActivitySpace _asCurrentSpace,
                                       IPTSession _ptUserSession,
                                       String _strPageName, int _nPageTypeID);

    /**
     * This function will perform some functionality after a MyPage has been
     * edited.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _strPageName - Name of the MyPortal Page.
     * @param _nPageTypeID - Page Type of the MyPortal page.
     */
    public void OnAfterEditMyPortalPage(AActivitySpace _asCurrentSpace,
                                        IPTSession _ptUserSession,
                                        String _strPageName, int _nPageTypeID);

    /**
     * This function will perform some functionality after a user has deleted a
     * page.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _nPageID - ID of the MyPage that the user has removed.
     */
    public void OnAfterRemoveMyPortalPage(AActivitySpace _asCurrentSpace,
                                          IPTSession _ptUserSession,
                                          int _nPageID);

    /**
     * This function will perform some validation based on the page name and
     * page type when a user tries to add a new page to their available listing
     * of pages.  
     * <br><br>
     * NOTE: The int item of the MyPortalPageArguments needs to be
     * a valid PageTypeConstant or there will be errors when processed.  
     * <br><br>
     * NOTE: The String item of the MyPortalPageArguments needs to be non-null and
     * with a length greater than 0 but less than or equal to 20. Otherwise
     * the original _strPageName will superscede this value.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _strPageName - Name that the user is trying to assign to this new
     *        MyPortal Page.
     * @param _nPageTypeID - Page Type that the user wants to use for the new
     *        MyPortal page.
     *
     * @return MyPortalPageArguments - This object will contain a String
     *         pertaining to the  modified name being returned and a int
     *         pertaining to the modified page layout.
     */
    public MyPortalPageArguments OnBeforeAddMyPortalPage(AActivitySpace _asCurrentSpace,
                                                         IPTSession _ptUserSession,
                                                         String _strPageName,
                                                         int _nPageTypeID);

    /**
     * This function will perform some validation based on the page name and
     * page type when a user tries to edit a page.  
     * <br><br>
     * NOTE: The int item of the MyPortalPageArguments needs to be a valid 
     * PageTypeConstant or there will be errors when processed.  
     * <br><br>
     * NOTE: The String item of the MyPortalPageArguments needs to be non-null 
     * and with a length greater than 0 but less than or equal to 20. Otherwise 
     * the original _strPageName will superscede this value.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _strPageName - Name that the user is trying to assign to this
     *        MyPortal Page.
     * @param _nPageTypeID - Page Type that the user wants to use for the
     *        MyPortal page.
     *
     * @return MyPortalPageArguments - This object will contain a String
     *         pertaining to the  modified name being returned and a int
     *         pertaining to the modified page layout.
     */
    public MyPortalPageArguments OnBeforeEditMyPortalPage(AActivitySpace _asCurrentSpace,
                                                          IPTSession _ptUserSession,
                                                          String _strPageName,
                                                          int _nPageTypeID);

    /**
     * This function will perform some functionality when a user tries to remove
     * a page from their available listing of pages.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _nPageID - Page ID that the user is trying to remove.
     *
     * @return String - Error message if the deletion should not continue,
     *         empty string or null if it should.
     */
    public String OnBeforeRemoveMyPortalPage(AActivitySpace _asCurrentSpace,
                                             IPTSession _ptUserSession,
                                             int _nPageID);
}

