package com.plumtree.portalpages.pei;

import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;


/**
 * This interface was created to allow developers the ability to create
 * functions that can be dynamically discovered by the Plumtree application
 * for the purposes of performing some functionality when a particular
 * object management based event occurs.
 * <br><br>
 * NOTE: Should this interface be implemented more than once within the
 * Dynamic Discovery process, the implementations will be processed in the
 * order that they are displayed within the representative XML file. All
 * implementations will be called.  
 * <br><br>
 * NOTE: Developers should be very careful implementing either of these  
 * functions as these will be called every single time that a Move, Delete, 
 * Copy, or Migrate is performed and it could hurt performance.    
 * <br><br>
 * NOTE: In the case where multiple implementations return a String object, all 
 * the valid Strings (not null and not empty) will be appended together 
 * (separated by a blank space) and the final String will be the error message 
 * displayed on the page.
 */
public interface IObjectActions
{
    /**
     * This function will perform some functionality after an ABO Job is created 
     * to delete an Object.  
     * <br><br>
     * More info on ABO Jobs and when this PEI is called: <br>
     * When the user deletes an Admin Folder (including Community folders), since 
     * this action might  require a lot of processing time, he will have the choice
     * between deleting the folder now or  scheduling an ABO Job to do it
     * later and minimize the impact on the Portal's performance. If he
     * decides to do it now, the OnDeleteAdminFolder PEI is called, if he
     * decides to have it done later by a Job, the OnBeforeCreateABOJob and
     * OnAfterCreateABOJob PEIs are called.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _nJobID - ID of the ABO Job created
     * @param _nClassID - Class ID of the Object that the Job will delete
     * @param _nObjectID - ID of the Object that the Job will delete
     */
    public void OnAfterCreateABOJob(AActivitySpace _asCurrentSpace,
                                    IPTSession _ptUserSession, int _nJobID,
                                    int _nClassID, int _nObjectID);

    /**
     * This function will perform some functionality before an Admin Folder is copied.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _nCopiedFolderID - ID of the folder being copied
     * @param _nTargetFolderID - ID of the new folder
     *
     * @return String - Error message if the action should not continue,
     *         empty string or null if it should.
     */
    public String OnBeforeCopyAdminFolder(AActivitySpace _asCurrentSpace,
                                          IPTSession _ptUserSession,
                                          int _nCopiedFolderID,
                                          int _nTargetFolderID);

    /**
     * This function will perform some functionality before an Object is copied 
     * from a folder to another.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _nClassID - The type of object being copied
     * @param _nObjectID - The specific object being copied
     * @param _nFolderID - The Folder for which the objects will be moving to.
     *
     * @return String - Error message if the action should not continue,
     *         empty string or null if it should.
     */
    public String OnBeforeCopyObject(AActivitySpace _asCurrentSpace,
                                     IPTSession _ptUserSession, int _nClassID,
                                     int _nObjectID, int _nFolderID);

    /**
     * This function will perform some functionality before an ABO Job is created 
     * to delete an Object.  
     * <br><br>
     * More info on ABO Jobs and when this PEI is called: <br>
     * When the user deletes an Admin Folder (including Community folders), since 
     * this action might  require a lot of processing time, he will have the choice
     * between deleting the folder now or  scheduling an ABO Job to do it
     * later and minimize the impact on the Portal's performance. If he
     * decides to do it now, the OnDeleteAdminFolder PEI is called, if he
     * decides to have it done later by a Job, the OnBeforeCreateABOJob and
     * OnAfterCreateABOJob PEIs are called.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _nClassID - Class ID of the Object that the Job will delete
     * @param _nObjectID - ID of the Object that the Job will delete
     *
     * @return String - Error message if the creation of the ABO Job should not 
     * 			continue, empty string or null if it should.
     */
    public String OnBeforeCreateABOJob(AActivitySpace _asCurrentSpace,
                                       IPTSession _ptUserSession,
                                       int _nClassID, int _nObjectID);

    /**
     * This function will perform some functionality before the creation process of
     * a Admin Folder
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _strFolderName - The name for the specified Admin Folder
     * @param _strFolderDesc - Description of the specified Admin Folder
     *
     * @return String[] - String array containing new versions of the name and
     *         description. Name should be in the 0 position and Desc should
     *         be in the 1 position of the array. If nulls are returned for
     *         either or both positions, then the original values will take
     *         precedence.
     */
    public String[] OnBeforeCreateAdminFolder(AActivitySpace _asCurrentSpace,
                                              IPTSession _ptUserSession,
                                              String _strFolderName,
                                              String _strFolderDesc);

    /**
     * This function will perform some functionality before the deletion
     * process of a Admin Folder.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _nFolderID - The specific admin folder being deleted
     *
     * @return String - Error message if the deletion should not continue,
     *         empty string or null if it should.
     */
    public String OnBeforeDeleteAdminFolder(AActivitySpace _asCurrentSpace,
                                            IPTSession _ptUserSession,
                                            int _nFolderID);

    /**
     * This function will perform some functionality before the deletion
     * process of a set of objects.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _nClassID - The type of object being deleted
     * @param _nObjectID - The specific object being deleted
     *
     * @return String - Error message if the deletion should not continue,
     *         empty string or null if it should.
     */
    public String OnBeforeDeleteObject(AActivitySpace _asCurrentSpace,
                                       IPTSession _ptUserSession,
                                       int _nClassID, int _nObjectID);

    /**
     * This function will perform some functionality during the Migration
     * process of a set of objects.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _nClassID - The type of object being migrated
     * @param _arrObjectIDs - The specific set of objects being migrated.
     *
     * @return String - Error message if the migration should not continue,
     *         empty string or null if it should.
     */
    public String OnBeforeMigrateObject(AActivitySpace _asCurrentSpace,
                                        IPTSession _ptUserSession,
                                        int _nClassID, int[] _arrObjectIDs);

    /**
     * This function will perform some functionality or validation before the
     * move process of a object.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _nClassID - The type of object being moved.
     * @param _nObjectID - The specific objects being moved
     * @param _nFolderID - The Folder for which the objects will be moving to.
     *
     * @return String - Error message if the move should not continue, empty
     *         string or null if it should.
     */
    public String OnBeforeMoveObject(AActivitySpace _asCurrentSpace,
                                     IPTSession _ptUserSession, int _nClassID,
                                     int _nObjectID, int _nFolderID);
}

