package com.plumtree.portalpages.pei.util;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.serversettings.general.*;
import com.plumtree.portalpages.common.constants.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;

/**
 * This helper class handles password validation for native users.
 */
public class PasswordValidator
{
	/**
	 * Validate that the new password complies with the password management rules.
	 * 
	 * @param _strNewPassword The proposed new password.
	 * @param _strUserName The user name.
	 * @param _asCurrentSpace The current Activity Space.
	 * @param _log The logger to use for error logging.
	 * @return An i18nized error string.  Null if the password successfully validates.
	 */
	public String ValidatePassword(String _strNewPassword, String _strUserName,
		AActivitySpace _asCurrentSpace, OpenLogger _log)
	{
		IPTSession session = (IPTSession) _asCurrentSpace.GetUserSession();
		IPTServerConfigSettings config = (IPTServerConfigSettings) session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
			false);

		int nMinimumLength = config.GetSettingAsInt(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_LENGTH);

		if (_strNewPassword.length() < nMinimumLength)
		{
			_log.Info("New password is too short. Must be at least " + nMinimumLength + " characters.");
	
			return XPResourceManager.FormatMessage(_asCurrentSpace.GetString(1628,
				ResourceConstants.PORTALBROWSINGMSGS), nMinimumLength);
		}
	
		// Check new password length
		if (_strNewPassword.length() > SetServerSettingsModel.MAX_PASSWORD_LENGTH)
		{
			_log.Info("New password is too long.  Cannot be longer than " + SetServerSettingsModel.MAX_PASSWORD_LENGTH + ".");
	
			return _asCurrentSpace.GetString(1291, ResourceConstants.PORTALBROWSINGMSGS);
		}

		if (config.GetSettingAsBoolean(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_NO_USER_IDS))
		{
			if (XPStringUtility.IndexOf(XPStringUtility.ToLowerCase(_strNewPassword), 
					XPStringUtility.ToLowerCase(_strUserName)) >= 0)
			{
				_log.Info("New password cannot contain user ID.");

				return _asCurrentSpace.GetString(1982, ResourceConstants.PORTALBROWSINGMSGS);
			}
		}

		if (config.GetSettingAsBoolean(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_CASING))
		{
			boolean bUpperCase = false;
			boolean bLowerCase = false;
	
			for (int i = 0; i < _strNewPassword.length(); i++)
			{
				char c = _strNewPassword.charAt(i);
				// NOTE: This does not support i18nized character casing.
				// Character.isUpperCase(), etc... support this, but
				// OpenFoundation does not have isUpperCase.
				if ((c >= 'a') && (c <= 'z'))
				{
					bLowerCase = true;
				} else if ((c >= 'A') && (c <= 'Z'))
				{
					bUpperCase = true;
				}
			}
	
			if (!(bUpperCase && bLowerCase))
			{
				_log.Info("New password must contain both upper and lower case letters.");
	
				return _asCurrentSpace.GetString(1983, ResourceConstants.PORTALBROWSINGMSGS);
			}
		}

		if (config.GetSettingAsBoolean(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_NUMBERS))
		{
			boolean bNumber = false;
	
			for (int i = 0; i < _strNewPassword.length(); i++)
			{
				char c = _strNewPassword.charAt(i);
				if ((c >= '0') && (c <= '9'))
				{
					bNumber = true;
				}
			}
	
			if (!bNumber)
			{
				_log.Info("New password must contain a number.");
	
				return _asCurrentSpace.GetString(1984, ResourceConstants.PORTALBROWSINGMSGS);
			}
		}

		if (config.GetSettingAsBoolean(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_PUNCTUATION))
		{
			boolean bPunctuation = false;
	
			for (int i = 0; i < _strNewPassword.length(); i++)
			{
				char c = _strNewPassword.charAt(i);
				if ((c >= '!') && (c <= '/') ||
					(c >= '[') && (c <= '`') ||
					(c >= '{') && (c <= '~') ||
					(c >= ':') && (c <= '@'))
				{
					bPunctuation = true;
				}
			}
	
			if (!bPunctuation)
			{
				_log.Info("New password must contain punctuation.");
	
				return _asCurrentSpace.GetString(1985, ResourceConstants.PORTALBROWSINGMSGS);
			}
		}

		int nReuseHistory = config.GetSettingAsInt(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_REUSE_HISTORY);

		if (nReuseHistory > 0)
		{
			IPTUserManager users = (IPTUserManager) session.GetUsers();
			boolean bReused = users.IsPasswordInHistory(session.GetSessionInfo().GetCurrentUserID(),
				_strNewPassword, nReuseHistory);

			if (bReused)
			{
				_log.Info("New password was used previously.  Passwords cannot be reused for " + nReuseHistory + " times.");
		
				return XPResourceManager.FormatMessage(_asCurrentSpace.GetString(1998,
					ResourceConstants.PORTALBROWSINGMSGS), nReuseHistory);
			}
		}
		// The password appears to be valid.
		return null;
	}
}