package com.plumtree.portalpages.rest.ldap;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.rest.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;

/**
 * Model that looks up user object id from LDAP auth name
 * @author rozhang
 */
public class UserIdLookupModel extends ARESTModel implements IModel
{
	private static OpenLogger log = 
		OpenLogService.GetLogger(
				OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_REST),
				"com.plumtree.portalpages.rest.ldap.UserIdLookupModel");

	
	public static final String STR_MVC_CLASS_NAME = "UserIdLookupModel";
	
	private static boolean REMOTEAUTHSRC_QUERIED = false;
	
	// lock object for flag
	private static Object QUERY_LOCK = new Object();
	
	// fix for ensemble bug #10241080, return 2 instead of -1 so that ensemble will
	// authenticate as quest user for kdportlet request. -1 was causing 500 error
	// in ensemble rc6
	public static final int NO_USERID_FOUND = 2;
	
	protected AActivitySpace m_space;
	
	protected String m_authName = null;
	private static String m_defaultAuthenticationSourcePrefix = null;
	
	
	/**
	 * Main method to lookup user id from auth name set by control
	 * @return
	 */
	public int GetUserIdFromCurrentAuthName()
	{
		return LookupUserIdFromAuthName(GetAuthName());
	}
	
	/**
	 * 
	 * @param authName LDAP authentication name (without Portal Auth source prefix)
	 * @return
	 */
	protected int LookupUserIdFromAuthName(String authName)
	{
		IPTSession session = (IPTSession)m_space.GetUserSession();
		
		String defaultAuthSrcPrefix = GetSingleRemoteAuthSourceObjectId(session);
		
		// if there are more than one remote auth source
		if ( defaultAuthSrcPrefix == null ) {
			ConfigVarPack configVarPack = (ConfigVarPack) m_space.GetApplication().GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);
			defaultAuthSrcPrefix = configVarPack.GetConfigValueAsString(ConfigConstants.AUTHENTICATION_SECTION, "DefaultAuthSourcePrefix", ConfigConstants.VALUE_ATTRIBUTE);
			
			if ( defaultAuthSrcPrefix == null || "".equals(defaultAuthSrcPrefix )) {
				log.Error("More than one Authentication Sources were found but no default prefix was set in portalconfig.xml (DefaultAuthSourcePrefix setting). Reverting to Guest User.");
				return NO_USERID_FOUND;
			}
			
			if ( log.IsDebugEnabled() ) {
				log.Debug("Default Authentication Source prefix configured to "+defaultAuthSrcPrefix);
			}
		} else {
			if ( log.IsDebugEnabled() ) {
				log.Debug("Only a single Authentication Source with prefix "+defaultAuthSrcPrefix+" was found. ");
			}
		}
		String fullLoginName = defaultAuthSrcPrefix+"\\"+authName;
		IPTObjectManager userMgr = session.GetUsers();
		
		int[] userId = null;
		try {
			userId = userMgr.LookupObjectIdFromName(fullLoginName);
		} catch (Exception e) {
			// didn't find name, don't do anything
			log.Debug(e, "Not able to lookup user id for user with name: "+fullLoginName);
		}  
		if ( userId != null && userId.length == 1 ) {
			return userId[0];
		} else {
			log.Info("User id not found for user "+fullLoginName);
			return NO_USERID_FOUND;
			
		}
	}

	/**
	 * @param session
	 * @return null if more than one remote auth source was found
	 */
	private String GetSingleRemoteAuthSourceObjectId(IPTSession session)
	{
		if ( XPConvert.ToBoolean( REMOTEAUTHSRC_QUERIED ) ) {
			return m_defaultAuthenticationSourcePrefix;
		} else {
			int nPropIDsToQuery		= PT_PROPIDS.PT_PROPID_NAME |
	                                  PT_PROPIDS.PT_PROPID_AUTHSOURCE_PREFIXSTRING |
	                                  PT_PROPIDS.PT_PROPID_AUTHSOURCE_AUTHTYPE;
	        Object[][] ptqf			= new Object[3][1];
	        ptqf[0][0]				= new Integer(PT_PROPIDS.PT_PROPID_AUTHSOURCE_AUTHTYPE);
	        ptqf[1][0]				= new Integer(PT_FILTEROPS.PT_FILTEROP_EQ);
	        ptqf[2][0]				= new Integer(PT_AUTHTYPES.PT_AUTHTYPE_EXTENSIBLE);
			int[][] arrSort 		= new int[1][2];
	        arrSort[0][0/*PTSAW_SORT_ROW_PROPID*/]		 = PT_PROPIDS.PT_PROPID_OBJECTID;
	        arrSort[0][1/*PTSAW_SORT_ROW_DIRECTION*/]    = 1;
	        
			synchronized (QUERY_LOCK) {
				// check if someone initialized after we checked
				if ( XPConvert.ToBoolean( REMOTEAUTHSRC_QUERIED ) ) {
					return m_defaultAuthenticationSourcePrefix;
				}
				IPTObjectManager authSrcMan = session.GetAuthSources();
		        IPTQueryResult qr = authSrcMan.Query(nPropIDsToQuery, -1, arrSort, 0, -1, ptqf);
		        
		        if ( qr.RowCount() == 1 ) {
		        	m_defaultAuthenticationSourcePrefix = qr.ItemAsString(0, PT_PROPIDS.PT_PROPID_AUTHSOURCE_PREFIXSTRING );
		        }
		        REMOTEAUTHSRC_QUERIED = true;
			} // end of synchronized block
		    return m_defaultAuthenticationSourcePrefix;
		}
	}
	
	public void SetAuthName(String name)
	{
		m_authName = name;
	}
	
	public String GetAuthName()
	{
		return m_authName;
	}
	
	/*
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
	 */
	public void CleanupAllData()
	{

	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()
	 */
	public void CleanupTempData()
	{

	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#Init(com.plumtree.uiinfrastructure.activityspace.AActivitySpace)
	 */
	public void Init(AActivitySpace parent)
	{
		m_space = parent;

	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new UserIdLookupModel();
	}
}
