package com.plumtree.portalpages.rest.user;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.rest.*;
import com.plumtree.portalpages.rest.utils.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.login.*;

/**
 * This control handles processing the User REST API requests.
 */
public class UserRESTControl implements IControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_REST),
		"com.plumtree.portalpages.rest.user.UserRESTControl");

	public final static String STR_MVC_CLASS_NAME = "UserREST";

	/**
	 * The control argument for the user ID passed in from the friendly URL.
	 */
	public final static String USER_ID = "user";

	/**
	 * The default value - current user.
	 */
	public final static String CURRENT_USER = "current";

	/**
	 * The owning space for this control.
	 */
	protected AActivitySpace m_asOwner;

	/**
	 * The User REST model.
	 */
	protected IUserRestModel m_model;

	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		try
		{
			// Users must be logged in to access this API.  The only time guest
			// users can use the API is when an admin user is editing the guest
			// profile.
			if (LoginHelper.INSTANCE.GetIsGuestUser(m_asOwner.GetUserSession()) &&
				!TaskAPIUICommon.IsCurrentPageInEditDefaultProfileMode(m_asOwner))
			{
				log.Warn("User must be logged in to access the User REST API.");

				// Must be logged in to use this API. 
				return ProcessError(CommonRestErrors.NOT_LOGGED_IN);
			}

			// This originally comes from the friendly URL and is put into this format.
			String[] sValues = (String[]) arguments.GetElement(USER_ID);

			if ((sValues == null) || (sValues.length == 0))
			{
				log.Warn("Could not find user ID in User REST API URL.");
				return ProcessError(UserRestErrors.INVALID_USER_ID);				
			}

			String strUserID = sValues[0];

			// Current user is the only valid ID supported right now.
			if (XPStringUtility.CompareIgnoreCase(CURRENT_USER, strUserID) != 0)
			{				
				log.Warn("Found invalid user ID in User REST API URL.  Expecting " +
					CURRENT_USER + ", but found [" + strUserID + "].  This value is not " +
					"currently supported.");

				return ProcessError(UserRestErrors.INVALID_USER_ID);
			}

			// NOTE: This is currently fairly secure because it only accepts POSTs.
			// However, if in the future this accepts GETs and other verbs, then we
			// need to be more careful.  If a user is logged into the portal and
			// goes to a hacker site, that site can use a javascript src include tag
			// on their page to the portal JSON page.  This script request (a GET)
			// will be passed onto portal with the user's portal cookie, allowing them
			// to access the data.  We are safe from this right now because the GET
			// request will be ignored by the control.
			// One way to secure against this kind of attack is to require the JSON request
			// to generate a hash of the contents of the login cookie and add it to the
			// request as an argument of some sort.

			IXPRequest request = m_asOwner.GetCurrentHTTPRequest();
			String strMethod = request.GetMethod();

			if (XPStringUtility.CompareIgnoreCase(strMethod, "POST") != 0)
			{
				log.Warn("This User REST API only supports the POST method.");

				// This KD REST API only supports the POST method.
				return ProcessError(CommonRestErrors.NOT_POST);
			}

			// Get the JSON input and parse it.
			// NOTE: We currently only support JSON POSTs, so we assume all requests
			// are the same.  We could eventually support GETs and XML as well.
			if (request.GetContentLength() == 0)
			{
				log.Warn("This User REST API requires POST content, but none was found.");

				// This User REST API requires POST content, but none was found.
				return ProcessError(CommonRestErrors.NO_POST_CONTENT);
			}

			// Get the POST body JSON string.
			String strJSON = new RESTHelper().GetPOSTBody(request);

			if ((strJSON == null) || (strJSON.length() == 0))
			{
				log.Warn("This User REST API requires POST content, but none was found.");

				// This User REST API requires POST content, but none was found.
				return ProcessError(CommonRestErrors.NO_JSON_POST_CONTENT);
			}

			IJSONParser parser = JSONParser.GetInstance();
			parser.ParseJSONObject(strJSON);

			if (XPStringUtility.CompareIgnoreCase(parser.GetObjectName(), UserRESTConstants.USER_FORMAT_NAME) != 0)
			{
				log.Warn("This User REST API only supports user objects, not " +
					parser.GetObjectName() + " objects.");

				// This User REST API only supports document objects.
				return ProcessError(UserRestErrors.NO_JSON_USER_OBJECT);
			}

			// Parse the JSON and execute the command.
			Redirect error = ProcessAction(parser);

			if (error != null)
			{
				return error;
			}
		} catch (Exception e)
		{
			log.Error(e, "Error processing User REST API request.");

			// General error.
			return ProcessError(CommonRestErrors.GENERAL);
		}

		// Redirect to DirRESTDP.
		return GetDPRedirect();
	}

	/**
	 * This method handles parsing the JSON body and executing the
	 * appropriate command.
	 * 
	 * @param _parser The parser with the JSON object already loaded.
	 * @return An error redirect, if necessary.  Null if there were no errors.
	 */
	public Redirect ProcessAction(IJSONParser _parser)
	{
		// Assume this is for the current user.

		// Join communities

		String strJoinCommunityID = _parser.GetMember(UserRESTConstants.USER_JOIN_COMMUNITIES);

		if (strJoinCommunityID != null)
		{
			int nJoinCommunityID;
			try
			{
				nJoinCommunityID = XPConvert.ToInteger(strJoinCommunityID);
			} catch (Exception e)
			{
				log.Error(e, "Error retrieving ID of community to join from JSON data.");
	
				return ProcessError(UserRestErrors.NO_JOIN_COMMUNITY_ID);
			}

			try
			{
				m_model.JoinCommunity(nJoinCommunityID);
			} catch (Exception e)
			{
				log.Error(e, "Error joining community " + nJoinCommunityID + ".");
				
				return ProcessError(UserRestErrors.JOIN_COMMUNITY_ERROR);
			}
		}

		// Leave communities

		String strLeaveCommunityID = _parser.GetMember(UserRESTConstants.USER_LEAVE_COMMUNITIES);

		if (strLeaveCommunityID != null)
		{
			int nLeaveCommunityID;
			try
			{
				nLeaveCommunityID = XPConvert.ToInteger(strLeaveCommunityID);
			} catch (Exception e)
			{
				log.Error(e, "Error retrieving ID of community to leave from JSON data.");
	
				return ProcessError(UserRestErrors.NO_LEAVE_COMMUNITY_ID);
			}

			try
			{
				m_model.LeaveCommunity(nLeaveCommunityID);
			} catch (Exception e)
			{
				log.Error(e, "Error leaving community " + nLeaveCommunityID + ".");
				
				return ProcessError(UserRestErrors.LEAVE_COMMUNITY_ERROR);
			}
		}

		// No errors
		return null;
	}

	/**
	 * This helper method sets the error on the model and redirects to the
	 * REST API display page.
	 * 
	 * @param _nErrorID the current error code.
	 * @return The redirect to the User REST API display page.
	 */
	public Redirect ProcessError(IRESTError _error)
	{
		Redirect redirect = GetDPRedirect();

		m_model.SetError(_error);

		return redirect;
	}

	/**
	 * @return the redirect to the User REST API display page.
	 */
	public Redirect GetDPRedirect()
	{
		Redirect redirect = new Redirect();
		redirect.SetLinkGetCachedSpace(m_asOwner);
		redirect.SetDisplayPage(UserRESTDP.STR_MVC_CLASS_NAME);

		return redirect;
	}

	public void Init(IModel model, AActivitySpace space)
	{
		m_asOwner = space;
		m_model = (IUserRestModel) model;
	}

	public String GetName() {
		return STR_MVC_CLASS_NAME;
	}

	public Object Create() {
		return new UserRESTControl();
	}
}