/**
 * 
 */
package com.plumtree.portalpages.rest.utils;

import com.plumtree.openfoundation.util.*;
import com.plumtree.taskapi.portalui.*;

/**
 * @author rozhang
 * Serializes an XPHashtable or IXP list to JSON notation
 */
public class JSONSerializer
{
	
	//public static String[] INVALID_JSPROPNAME_CHARS = new String[] { "-", "&", "\\", "/", "#", "%, "", "!"};
	
	
	protected XPHashtable m_object = null; 
	
	public JSONSerializer(XPHashtable jsonObject)
	{
		m_object = jsonObject;
	}
	
	/**
	 * Main parsing function
	 * @param addPrefix 	Protects the JSON string from rewrites/pollution/hijacking hacks by prefixing
	 * 						the generated JSON with '{} && '. The prefix is skipped by the eval function
	 * 						when interpreting the JSON as data, i.e. transforming the JSON into an object
	 * 						with properties but turns the JSON into unexecutable javascript to protect against
	 * 						various CSRF vunerabilities
	 * @return
	 */
	public String GenerateStringRepresentation(boolean addPrefix)
	{
		XPStringBuilder buf = new XPStringBuilder();
		// add json hacking protection prefix 
		buf.Append( (addPrefix) ? "{} && " : "");
		buf.Append( ParseHashTable(m_object, 1) );
		return buf.ToString();
	}
	
	/**
	 * Generate JSON String representation for node with child nodes.
	 * The generated output will be in the following format:
	 * {
	 *   property1: value2,
	 *   property2: value2,
	 *   property3: value3,
	 *   ...
	 * }
	 * 
	 * @param map
	 * @param level	The depth from the top where top is 0 
	 * @return
	 */
	protected String ParseHashTable(XPHashtable map, int level)
	{
		XPStringBuilder buf = new XPStringBuilder();
		buf.Append(AddPadding(level-1)).Append("{\n");
		// iterate over the xphashtable
		IXPEnumerator keys = map.GetKeys();
		boolean isFirstEntry = true;
		
		while ( keys.MoveNext() ) 
		{
			if ( !isFirstEntry ) {
				buf.Append(",\n");
			}
			String curName = TaskAPIUIJavascript.EscapeJavaScriptString( (String)keys.GetCurrent() );
			Object val = map.GetElement(curName);
			
			if ( val instanceof XPHashtable ) 
			{
				buf.Append(AddPadding(level)).Append(curName).Append(":");
				buf.Append( ParseHashTable((XPHashtable)val, level+1) );
			} 
			else if ( val instanceof XPArrayList ) 
			{
				buf.Append(AddPadding(level)).Append(curName).Append(":\n");
				buf.Append(  ParseArray((XPArrayList)val, level+1 ));
			} 
			else 
			{
				buf.Append(AddPadding(level)).Append(curName).Append(":");
				buf.Append(" ").Append( GetStringValue(val, curName) );
			} 
			isFirstEntry = false;
		}
		buf.Append("\n").Append(AddPadding(level-1)).Append("}");
		return buf.ToString();
	}
	
	/**
	 * Generate JSON string representation for Array node
	 * The generated output will be in the following format:
	 * [
	 *   value1, value2, value3, ...
	 * ]
	 * 
	 * @param values
	 * @param level
	 * @return
	 */
	protected String ParseArray(IXPList values, int level)
	{
		XPStringBuilder buf = new XPStringBuilder();
		// want the brackets to display in parent's level
		buf.Append(AddPadding(level)).Append("[ ");
		boolean addIndent = false;
		for (int i = 0; i < values.GetSize(); i++ ) {
			if ( i != 0 ) {
				buf.Append( (addIndent) ?  AddPadding(level+1) : " ");
				buf.Append(",");
				addIndent = false;
			}
			Object val = values.GetElement(i);
			
			if ( val instanceof XPHashtable ) {
				buf.Append("\n");
				buf.Append( ParseHashTable((XPHashtable)val, level+1) );
			} 
			else if ( val instanceof XPArrayList ) 
			{
				buf.Append("\n");
				buf.Append(  ParseArray((XPArrayList)val, level+1 ) );
				buf.Append("\n");
				addIndent = true;
			} 
			// add support for arrays
//			else if ( val instanceof Object[] ) 
//			{
//				buf.Append("\n");
//				buf.Append(  ParseArray((Object[])val, level+1 ) );
//				buf.Append("\n");
//				addIndent = true;
//			} 
			else 
			{
				buf.Append( GetStringValue(val, null) );
			}
		}
		buf.Append(AddPadding(level-1)).Append(" ]");
		return buf.ToString();
	}
	
	/**
	 * Helper for adding the proper amount of indentation
	 * @param level
	 * @return
	 */
	protected String AddPadding(int level)
	{
		XPStringBuilder buf = new XPStringBuilder();
		for (int i = 0; i < level; i++ ) {
			buf.Append("  "); // 2 spaces
		}
		return buf.ToString();
	}
	
	/**
	 * Returns JSON string representation of a value. 
	 * Strings are escape encoded
	 *  
	 * @param val
	 * @param name
	 * @return
	 */
	protected String GetStringValue(Object val, String name)
	{
		String valString = "";
		if ( val instanceof String || val instanceof Character) {
			valString = "\""+TaskAPIUIJavascript.EscapeJavaScriptString( val.toString() )+"\"";
		} else if ( val instanceof Integer || val instanceof Float || val instanceof Double || val instanceof Long ) {
			valString += val;
		} else if ( val instanceof Boolean) {
			valString += val;
		} else if ( val instanceof XPDateTime) {
			// dates are converted to millisecs since utc sent as a number
			valString += XPConvert.ToString( ((XPDateTime)val).GetTimeInMilliSeconds() );
		} else if ( val == null ) {
			valString = "null";
		} else {
			if ( name == null ) {
				throw new XPIllegalArgumentException("Unsupported value type found for value of type: "+val);
			} else {
				throw new XPIllegalArgumentException("Unsupported value type found for value "+name+" of type: "+val);
			}
		}
		return valString;
	}
	
	/**
	 * Throws exception if validation fails
	 * 
	 * @param propName
	 * @return
	 */
//	protected boolean ValidateJSPropertyName(String propName)
//	{
//		// @TODO work in progress - add invalid variable/property name strings
//		
//		XPArrayList errors = new XPArrayList();
//		for ( int i = 0; i < INVALID_JSPROPNAME_CHARS.length; i++ ) {
//			if ( propName.indexOf(INVALID_JSPROPNAME_CHARS[i]) != -1 ) {
//				throw new XPIllegalArgumentException("Invalid Javascript property name: "+propName+". Contains following invalid characters: "+INVALID_JSPROPNAME_CHARS[i] );
//			}
//		}
//		return true;
//		
//	}
}
