package com.plumtree.taskapi.portalui;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portalpages.browsing.directory.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;

public interface ITaskAPIUIDirectory
{
	/**
	 * Checks if the Knowledge Directory feature is enabled in the current Experience Definition.
	 * 
	 * @param owner		Current Activityspace.
	 * @return	<code>true</code> Knowledge Directory feature is enabled, <code>false</code> otherwise.
	 */
	public  boolean IsDirectoryFeaturedEnabledInSubPortal(AActivitySpace owner);
	
	/**
	 * Gets the URL to the specified Knowledge Directory in Browse Mode.  
	 * The folder will not be accessible if the current user does not have sufficient access to it. 
	 * 
	 * @param owner		Current Activityspace.
	 * @param nFolderID	Object ID of a Knowledge Directory folder to create an URL to.
	 * @return		An <code>IPTURL</code> with the URL to the specified folder in Knowledge Directory Browse Mode,
	 * or <code>null</code> if the URL is not valid in the current context.
	 */
	public IPTURL GetURLForBrowseDirectory(AActivitySpace owner, int nFolderID);
	
	/**
	 * Gets the URL to the specified Knowledge Directory in Edit Mode.  
	 * The folder will not be accessible if the current user does not have sufficient access it.
	 * 
	 * @param owner		Current Activityspace.
	 * @param nFolderID	Object ID of a Knowledge Directory folder to create an URL to.
	 * @return		An <code>IPTURL</code> with the URL to the specified folder in Knowledge Directory Edit Mode,
	 * or <code>null</code> if the URL is not valid in the current context.
	 */
	public IPTURL GetURLForEditDirectory(AActivitySpace owner, int nFolderID);
	
	/**
	 * Gets an URL to open a specified Knowledge Directory folder in the selected mode.
	 * The folder will not be accessible if the current user does not have sufficient access it.
	 * 
	 * @param owner			Current Activityspace.
	 * @param strTitle		Title of the URL, normally the name of the folder to open.
	 * @param nDirMode	DirConstants.BROWSE_MODE or DirConstants.EDIT_MODE
	 * @param nFolderID	Object ID of the Knowledge Directory Folder to get an URL to.
	 * @return An <code>IPTURL</code> to a folder in the Knowledge Directory - no image is assigned to the URL.
	 * 
	 * @see DirConstants
	 */
	public  IPTURL CreateCommonOpenerDirectoryFolderURL(AActivitySpace owner, String strTitle, int nDirMode, int nFolderID);
	
	/**
	 * Gets an URL to open a specified Knowledge Directory folder in the selected mode.
	 * The folder will not be accessible if the current user does not have sufficient access it.
	 * 
	 * @param owner			Current Activityspace.
	 * @param strTitle		Title of the URL, normally the name of the folder to open.
	 * @param nDirMode	DirConstants.BROWSE_MODE or DirConstants.EDIT_MODE
	 * @param nFolderID	Object ID of the Knowledge Directory Folder to get an URL to.
	 * @return An <code>IPTURL</code> to a folder in the Knowledge Directory - no image is assigned to the URL.
	 * 
	 * @see DirConstants
	 */
	public IPTURL CreateCommonOpenerDirectoryFolderURL(AActivitySpace owner, String strTitle, int nDirMode, int nFolderID, String strFolderName);

	/**
	 * Gets an URL to open a specified Document in the Knowledge Directory.
	 * The document will not be accessible if the current user does not have sufficient access it.
	 * 
	 * @param owner		Current Activityspace.
	 * @param strTitle	Title of the URL, normally the name of the Document to open.
	 * @param nDocumentID	Object ID of the Document to get an URL to.
	 * @return	
	 */
	public IPTURL CreateCommonOpenerDocumentURL(AActivitySpace owner, String strTitle, int nDocumentID);

	/**
	 * Gets an URL to open a specified Document in the Knowledge Directory.
	 * The document will not be accessible if the current user does not have sufficient access it.
	 * 
	 * @param owner		Current Activityspace.
	 * @param strTitle	Title of the URL, normally the name of the Document to open.
	 * @param nDocumentID	Object ID of the Document to get an URL to.
	 * @param strDocumentName Name of the document.
	 * @return	The URL to the document
	 */
	public IPTURL CreateCommonOpenerDocumentURL(AActivitySpace owner, String strTitle, int nDocumentID, String strDocumentName);
	
	/**
	 * Checks if the current page is a Knowledge Directory page.
	 * 
	 * @param owner		Current Activityspace.
	 * @return	<code>true</code> if the current page us a Knowledge Directory page, <code>false</code> otherwise.
	 */
	public boolean IsCurrentPageDirectoryPage(AActivitySpace owner);

	/**
	 * Gets the Data Sources used for submitting documents to the Knowledge Directory.
	 * 
	 * @param owner		Current Activityspace.
	 * @return the list of Data Sources used for submitting documents to the Knowledge
	 * Directory.  Each entry in the list will be a HashTableDataObject with entries
	 * for "name" and "id", as described in the SubmitDataSourceConstants class.
	 */
	public IXPList GetSubmitDataSources(AActivitySpace owner);

	/**
	 * Adds the upload data for the Web Services associated with the
	 * Data Sources used for submitting documents to the Knowledge Directory.
	 * 
	 * This will also filter the list for invalid submit data sources.  Data sources
	 * should be either web submit, or remote file upload.  Any other kinds of data
	 * sources will be removed from the list.
	 * 
	 * @param owner		Current Activityspace.
	 * @param _sources	List of HashtableDataObjects of the Data Sources to lookup.
	 * This is of the same format as the result of the GetSubmitDataSources method.
	 * @return the original list of Data Sources with additional data added to each entry.
	 * The entries in the list will have additional entries for "type" (file or web),
	 * "url" (remote upload URL), and "wsid" (ID of the associated web service), as
	 * described in the SubmitDataSourceConstants class.  "url" will be null for
	 * data sources that don't support remote upload.
	 */
	public IXPList AddUploadData(AActivitySpace owner, IXPList _sources);

	/**
	 * Check if the provider for this data source supports web paths.
	 * 
	 * @param _dataSource The data source to check.
	 * @param _session the current user session
	 * @return whether or not this data source supports web paths.
	 */
	public boolean IsWebDataSource(IPTDataSource _dataSource, IPTSession _session);
}