package com.plumtree.taskapi.portalui;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.common.uiparts.*;
import com.plumtree.portaluiinfrastructure.common.constants.*;
import com.plumtree.portaluiinfrastructure.helpers.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * TaskAPI functions related to Portal Configuration, includes values in portalconfig.xml
 * 
 */
public class TaskAPIUIPortalConfig
{
	
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.taskapi.portalui.TaskAPIUIPortalConfig");

	/**
	 * Returns the Application URL according to the URL Mapping.
	 * Warning: In mode 1, it returns the non secure URL by default.
	 *
	 * @param <code>AActivitySpace</code> the current activity space.
	 *
	 * @return <code>String</code> with the Application Base URL
	 */
	public static final String GetApplicationBaseURL(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		return PTConfigService.getPTInstance().GetApplicationBaseURL(owner.GetCurrentNormalizedBaseURL());
	}
	
	/**
	 * Same as <code>TaskAPIUIPortalConfig.GetApplicationBaseURL</code> but without the
	 * trailing question mark.
	 * 
	 * @param <code>AActivitySpace</code> the current activity space.
	 * @return <code>String</code> with the Application Base URL and no trailing question mark 
	 */
	public static final String GetApplicationBaseURLForFriendlyURL(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		// remove the trailing question mark
		String baseURL = PTConfigService.getPTInstance().GetApplicationBaseURL(owner.GetCurrentNormalizedBaseURL());
		baseURL = GeneralHelpers.Replace(baseURL, "?", "");
		return baseURL;
	}
	
	/**
	 * Returns the Application URL according to the URL Mapping and
	 * determines the security according to the space name in argument.
	 *
	 * @param strNormalizedBaseURL Normalized Base URL from the AActivity Space.
	 * (or ConfigHelper.GetNormalizedBaseURL).
	 * @param strASName Name of the destination Activity Space.
	 * @param bIsSubSpace Whether the target Activityspace a sub space or not?
	 *
	 * @return <code>String</code> with the Application Base URL.
	 */
	public static final String GetApplicationBaseURLToAS(String strNormalizedBaseURL, String strASName, boolean bIsSubSpace)
	{
		return PTConfigService.getPTInstance().GetApplicationBaseURLToAS(strNormalizedBaseURL, strASName, bIsSubSpace);
	}

	/**
	 * This method to access an integer variable from portalconfig.xml.
	 *
	 * @param strSection One of the Section constants defined in <code>ConfigConstants</code>.
	 * @param strSubSection One of the Subsection constants defined in <code>ConfigConstants</code>.
	 * @param strAttribute One of the Attributes constants defined in <code>ConfigConstants</code>.
	 * @param owner current Activityspace.
	 *
	 * @return The requested attribute value as <code>int</code>  
	 * @throws <code>XPException</code> If the requested variable is not found or cannot be retrieved.
	 * 
	 * @see com.plumtree.uiinfrastructure.application.varpacks.ConfigConstants
	 */
	public static int GetPTConfigVariableAsInt(String strSection, String strSubSection, String strAttribute, AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		try {
			return PTConfigService.getPTInstance().GetConfigVariableAsInt(strSection, strSubSection, strAttribute, owner);
		} catch ( XPException e ) {
			throw new XPException("Unable to retrieve the requested attribute: "+strAttribute, e);
		}
	}

	/**
	 * This method to access a <code>String</code> variable from portalconfig.xml.
	 * 
	 * @param strSection One of the Section constants defined in <code>ConfigConstants</code>.
	 * @param strSubSection One of the Subsection constants defined in <code>ConfigConstants</code>.
	 * @param strAttribute One of the Attributes constants defined in <code>ConfigConstants</code>.
	 * @param owner current Activityspace.
	 * @return <code>String</code> with the requested variable value, or <code>null</code> if the variable is not found, or cannot be retrieved.
	 * 
	 * @see com.plumtree.uiinfrastructure.application.varpacks.ConfigConstants
	 */
	public static String GetPTConfigVariableAsString(String strSection, String strSubSection, String strAttribute, AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		try {
			return PTConfigService.getPTInstance().GetConfigVariableAsString(strSection, strSubSection, strAttribute, owner);
		} catch ( XPException e ) {
			log.Warn("Unable to retrieve the requested attribute: "+strAttribute+".", e);
			return null;
		}
	}

	/**
	 * This method to access a variable with multiple values from portalconfig.xml.
	 *
	 * @param strSection One of the Section constants defined in <code>ConfigConstants</code>.
	 * @param strSubSection One of the Subsection constants defined in <code>ConfigConstants</code>.
	 * @param strAttribute One of the Attributes constants defined in <code>ConfigConstants</code>.
	 * @param owner current Activityspace.
	 *
	 * @return Object Array with values of the requested attribute, or <code>null</code> if the attribute was not found or could 
	 * not be retrieved.
	 */
	public static Object[] GetPTConfigVariables(String strSection, String strSubSection, String strAttribute, AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		try {
			return PTConfigService.getPTInstance().GetConfigVariables(strSection, strSubSection, strAttribute, owner);
		} catch ( XPException e ) {
			log.Warn("Unable to retrieve the requested attribute: "+strAttribute+".", e);
			return null;
		}
	}

	/**
	 * This method returns the main application name.
	 *
	 * @return <code>String</code> with the main Portal Application name.
	 */
	public static final String GetMainApplicationName()
	{
		return ConfigHelper.GetMainApplicationName();
	}

	/**
	 * This method returns the Image Server Base URL.
	 *
	 * @param AActivitySpace Current Activityspace.
	 * @return <code>String</code> with the image server base URL.
	 */
	public static final String GetImageServerBaseURL(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		return ConfigHelper.GetImageServerBaseURL(owner);
	}

	/**
	 * This method returns the Portal Image Server JavaScript URL.
	 *
	 * @param AActivitySpace Current Activityspace.
	 * @return <code>String</code> with the Portal image server JavaScript URL.
	 */
	public static final String GetPortalImageServerJavaScriptURL(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		return ConfigHelper.GetPortalImageServerJavaScriptURL(owner.GetIsSecuredSpace());
	}

	/**
	 * Returns the base URL to the style sheets on the image server.
	 *
	 * @param AActivitySpace Current Activityspace.
	 * 
	 * @return <code>String</code> with the URL to the style sheets on the image server
	 */
	public static final String GetStyleSheetsBaseURL(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		return ConfigHelper.GetStyleSheetsBaseURL(owner);
	}

	/**
	 * This method returns the Portal Image Server Help URL.
	 *
	 * @param AActivitySpace Current Activityspace.
	 * @return <code>String</code> with the the Portal image server Help URL.
	 */
	public static final String GetPortalImageServerHelpURL(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		return ConfigHelper.GetPortalImageServerHelpURL(owner);
	
	}

	/**
	 * Returns the security mode value as defined in portalconfig.xml.
	 *
	 * @return integer with the current Portal Security mode.
	 */
	public static final int GetSecurityMode()
	{
		return ConfigHelper.GetSecurityMode(); 
	}

	
	
	/**
	 * Gets a Navigation settings from portalconfig.xml (what used to be NavigationSettings.xml in 5.x) as an integer value.
	 * Navigation Settings  are defined in 'portal:Navigation' the component and with the type defined in the 
	 * <code>PTNavSettingsVarPack.PORTAL_NAVIGATION_COMPONENT_TYPE</code> constant.
	 * 
	 * @param strKey	Navigation setting attribute.
	 * @param owner		Current Activityspace.
	 * @return	integer with the attribute value or, -1 if the key was not found or not an integer value.
	 */
	public static int GetNavSettingValueAsInt(String strKey, AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		return NavigationCommonHelpers.GetNavSettingsValueAsInt(strKey, owner);
	}
	
	/**
	 * Gets a Navigation specific settings from portalconfig.xml (what used to be NavigationSettings.xml in 5.x) as a <code>String</code> value.
	 * Navigation Settings  are defined in 'portal:Navigation' the component and with the type defined in the 
	 * <code>PTNavSettingsVarPack.PORTAL_NAVIGATION_COMPONENT_TYPE</code> constant.
	 * 
	 * @param strKey	Navigation setting attribute.
	 * @param owner		Current Activityspace.
	 * @return	<code>String</code> with the attribute value, or <code>null</code> if the attribute was not found.
	 */
	public static String GetNavSettingsValueAsString(String strKey, AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		return NavigationCommonHelpers.GetNavSettingsValueAsString(strKey, owner);
	}
	
	/**
	 * Gets a Navigation settings from portalconfig.xml (what used to be NavigationSettings.xml in 5.x) which has a comma separated
	 * strings value. Example:. <SomeKey value="val1,val2,val3"/>
	 * Navigation Settings  are defined in 'portal:Navigation' the component and with the type defined in the 
	 * <code>PTNavSettingsVarPack.PORTAL_NAVIGATION_COMPONENT_TYPE</code> constant.
	 * 
	 * @param strKey 	Key to a name/value pair in NavigationSettings.xml with a comma separated string values.
	 * 
	 * @param owner		Current Activityspace.
	 * @return					Array of <code>String</code>s with the comman separated values, or null if the attribute was not found, or 
	 * the comma separated values were in an invalid format.
	 * 
	 */
	public static String[] GetNavSettingsValueAsStringArray(String strKey, AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		
		String strVal =  NavigationCommonHelpers.GetNavSettingsValueAsString(strKey, owner);
		if ( null == strVal )
			return null;
		
		String[] result = null;
		try {
			XPStringTokenizer tok = new XPStringTokenizer(strVal, ",");
			result = new String[tok.GetTokenCount()];
			
			int i = 0;
			while ( tok.HasMoreTokens() ) {
				result[i++] = tok.GetNextToken(); 
			}
		} catch (Exception e) {
			log.Warn("Unable to parse the attribute value. Expecting a comma separated String, read value:"+strVal, e);
			return null;
		}
		
		return result;
		
	}
	
	/**
	 * Gets a Navigation settings from portalconfig.xml (what used to be NavigationSettings.xml in 5.x) which has a comma separated
	 * integer values. Example:. <SomeKey value="100,200,300"/>
	 * Navigation Settings  are defined in 'portal:Navigation' the component and with the type defined in the 
	 * <code>PTNavSettingsVarPack.PORTAL_NAVIGATION_COMPONENT_TYPE</code> constant.
	 * 
	 * @param strKey 	Key to a name/value pair in NavigationSettings.xml with a comma separated integer values.
	 * 
	 * @param owner		Current Activityspace.
	 * @return					Array of <code>int</code>s with the comman separated values, or null if the attribute was not found, or 
	 * the comma separated values were in an invalid format.
	 * 
	 */
	public static int[] GetNavSettingsValueAsIntArray(String strKey, AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		
		String strVal =  NavigationCommonHelpers.GetNavSettingsValueAsString(strKey, owner);
		if ( null == strVal )
			return null;
		
		int[] result = null;
		try {
			XPStringTokenizer tok = new XPStringTokenizer(strVal, ",");
			result = new int[tok.GetTokenCount()];
			
			int i = 0;
			while ( tok.HasMoreTokens() ) {
				result[i++] = XPConvert.ToInteger( tok.GetNextToken() ); 
			}
		} catch (Exception e) {
			throw new XPException("Error parsing NavigationSetting value - need to be a comma separated list of integers. Value: "+strVal);
		}
		return result;
		
	}
	
	/**
	 * Gets the URL to the Help Page for the current page.
	 * 
	 * @param owner		Current Activityspace.
	 * @return	<code>IPTURL</code> with URL to the Help Page for the current page. In standard display mode
	 * the url is just '#', instead a javascript call to open help in a pop-up is used. Get the javascript call with the
	 * <code>PTURLConstants.KEY_JAVASCRIPT_ACTION</code>.
	 * In 508 and low-bandwidth mode, a regular URL is used.
	 * 
	 */
	public static IPTURL GetURLForCurrentPageHelp(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		
		String strText = owner.GetString(337, ResourceConstants.PORTALCOMMONMSGS);
		
		HTMLAnchor helpAnchor = HTMLConstructHelpers.GetHelpAnchor( owner );
		String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
		
		FullURL url = new FullURL(strText, helpAnchor.GetHRef());
		// Get the onclick action, which is what is really used in standard display mode
		url.SetURLData(PTURLConstants.KEY_JAVASCRIPT_ACTION, helpAnchor.GetOnClick());
		
		url.SetURLData(PTURLConstants.KEY_ICONIMG, strImageServerURL+PTImageType.ICON_HELP.GetFileName());
		url.SetURLData(PTURLConstants.KEY_ICONIMGWIDTH, PTImageType.ICON_HELP.GetWidth());
		url.SetURLData(PTURLConstants.KEY_ICONIMGHEIGHT, PTImageType.ICON_HELP.GetHeight());
		
		return url;
	}

}
