/*
 */
package com.plumtree.taskapi.portalui;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.login.*;
import com.plumtree.portalpages.browsing.portalsettings.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.taskapi.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.login.*;
import com.plumtree.uiinfrastructure.web.*;
//Put explicit import for Interpreter (to avoid ambiguity of using the deprecated class)
import com.plumtree.uiinfrastructure.interpreter.Interpreter;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * TaskAPI functions for user related URLs and functions, such as getting personal preferences,
 * getting object ID of current user and checking user access.
 * 
 */
public class TaskAPIUIUser
{
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.taskapi.portalui.TaskAPIUIUser");
	
	/**
	 * Call this method to obtain the value of a personal setting.
	 * <p>
	 *  This value will be retrieved from the user's web session if possible,
	 *  otherwise the portal server will be queried.
	 * <p>
	 * 
	 * @param owner		Current Activityspace.
	 * @param sSettingName 		String name of personal setting as defined in 
	 * <code>com.plumtree.portaluiinfrastructure.login.PersonalSettingNames</code>, null value if the setting does not exist.
	 * @throws XPException This method will throw an <code>XPException</code> if you do not
	 *     pass an <code>ISessionManager</code> already containing the correct subSession.  The
	 *     <code>ISessionManager</code> obtained from your Activity Space will contain the correct
	 *     subsession.
	 *     
	 *     @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 *     
	 *   @see PersonalSettingNames
	 */
	public static Object GetPersonalSettingValue(AActivitySpace owner, String sSettingName)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		
		return PTPersonalSettingsHelper.GetPersonalSettingValue(sSettingName, owner);
	}
	
	/**
	 * Gets the locale set in user's My Account settings for the current user
	 * 
	 * @param owner Current Activityspace.
	 * @return <code>XPLocale</code> with the user's locale.  
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 */
	public static XPLocale GetPersonalSettingsLocale(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		
		String strLocaleID = (String)PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LOCALE, owner);
		return new XPLocale(strLocaleID);
	}
	
	/**
	 * Checks if the current user is a guest user.
	 * 
	 * @param owner 	Current Activityspace.
	 * 
	 * @return <code>true</code> if the current user is a guest user, <code>false</code> otherwise.
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 */
	public static boolean IsCurrentUserGuestUser(AActivitySpace owner)
    {
    	TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
    	
    	ISessionManager sessionMan = owner.GetSubSession();
		Object sesValid		   = sessionMan.GetAttribute(Interpreter.USERSESSIONVALID);

		if (sesValid == null)
		{
			return false;
		}
		
		boolean bIsGuestUser = !XPConvert.ToBoolean(sesValid); 

		return bIsGuestUser;
    	
    }
	
	/**
	 * Gets the object ID of the current user.
	 * 
	 * @param owner 	Current Activityspace.
	 * @return Object ID of the current user.
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 */
	public static int GetCurrentUserID(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);

		return TaskAPIServerUser.GetCurrentUserID(ptSession);		
	}

	/**
	 * Gets the name of the current user.
	 * 
	 * @param owner 	Current Activityspace.
	 * @return Name of the current user.
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 */
	public static String GetCurrentUserName(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);

		return TaskAPIServerUser.GetCurrentUserName(ptSession);
	}

	/**
	 * @see TaskAPIUISubPortal#GetURLForMyHome(AActivitySpace)
	 * 
	 */
	public static IPTURL GetURLForMyHome(AActivitySpace owner)
	{
		return TaskAPIUISubPortal.GetURLForMyHome(owner);
	}
	
	/**
	 * Gets the URL for the Login/Logoff link. Only one link is shown at any one time. 
	 * The login link appears if the user is not logged in, while the logoff link appears if the user
	 * is already logged in.
	 * 
	 * @param owner		Current Activityspace.
	 * @return	<code>IPTURL</code> with login or logoff URL, or null if the link should not be displayed, for example in
	 * Edit Default Profile or Preview Page mode.
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 */
	public static IPTURL GetURLForLoginLogoff(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		
		String strText = "";
		CommonOpenerOpenASURL url = null;
		
		String strApplBaseURL   		=  TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
		String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
		String strASName 				= LoginAS.STR_MVC_CLASS_NAME;
		int nUserID 							= TaskAPIUIUser.GetCurrentUserID(owner);
		
		
		if ( IsCurrentUserGuestUser( owner) )
		{
			// login
			strText = owner.GetString(336, ResourceConstants.PORTALCOMMONMSGS);
			
			url = new CommonOpenerOpenASURL(strText, strApplBaseURL, strASName, nUserID);
			url.AddControlParameter(ASConstants.CONTROL, LoginControl.STR_MVC_CLASS_NAME);
			url.AddControlParameter(LoginHelper.PARAM_LOGIN, "");
		}
		else
		{
			// Logoff
			if ( !TaskAPIUICommon.IsCurrentPageInEditDefaultProfileMode(owner) && !TaskAPIUICommon.IsCurrentPageInPreviewMode(owner) )
			{
				strText = owner.GetString(335, ResourceConstants.PORTALCOMMONMSGS);
				
				url = new CommonOpenerOpenASURL(strText, strApplBaseURL, strASName, nUserID);
				url.AddControlParameter(ASConstants.CONTROL, LoginControl.STR_MVC_CLASS_NAME);
				url.AddControlParameter(LoginHelper.PARAM_DOLOGOUT, "");
				url.AddControlParameter(ASConstants.CLEARSESSION, "true");

			}
		}
		if ( url != null ) {
			url.SetURLImageType(PTImageType.ICON_LOGIN, strImageServerURL);
		}
		
		return url;
		
	}
	
	/**
	 * Checks if the current user has access to the specified Activity Right.
	 * 
	 * @param owner		Current Activityspace.
	 * @param nActivityRightID	An Activity Rights ID defined in  <code>com.plumtree.server.PT_ACTIVITY_RIGHTS</code>
	 * @return <code>true</code> if the user has access to the Activity Right, <code>false</code> otherwise.
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 * 
	 * @see PT_ACTIVITY_RIGHTS
	 */
	public static boolean HasAccessToActivityRight(AActivitySpace owner, int nActivityRightID)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
		
		return TaskAPIServerUser.DoesUserHaveAccessToActivityRight(ptSession, nActivityRightID);
		
	}

	/**
	 * This method gets the user's personalized greeting, normally displayed in the Topbar. 
	 * E.g. "Hi Administrator" 
	 * 
	 * @param owner		Current Activityspace.
	 * @return	<code>String</code> with the user greeting, or if user greeting cannot be retrieved, a default greeting message.
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 */
	public static String GetWelcomeMessage(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
		String strGreeting = null;

		try
		{
			ISessionManager mgrSession = owner.GetSubSession();

			if (mgrSession == null)
			{
				// when a user first hits the login page, no subsession exists.  Use the default greeting
				// in this case.
				strGreeting = owner.GetString(315, ResourceConstants.PORTALCOMMONMSGS);
			}
			else
			{
				strGreeting = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.GREETING, owner);

				if ((strGreeting == null) || "".equals(strGreeting))
				{
					// use welcome
					strGreeting = XPResourceManager.FormatMessage(owner.GetString(546,ResourceConstants.PORTALCOMMONMSGS), TaskAPIServerUser.GetCurrentUserName(ptSession));
				}
			}
		}
		catch (Exception e)
		{
			// TODOCR DONH: This exception should probably get logged.
			// Just use the default greeting
			strGreeting = owner.GetString(315, ResourceConstants.PORTALCOMMONMSGS);
		}
		return strGreeting;
	}	
	
	/**
	 * Checks if the current user is a member of Administrators Group.
	 * 
	 * @param owner			Current Activityspace.
	 * @return	<code>true</code> if the current user is a member of the Administrators Group, <code>false</code> otherwise.
	 */
	static public boolean IsCurrentUserAdministrator(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
		
		return TaskAPIServerUser.DoesUserHaveAccessToActivityRight(ptSession, PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_ACCESS_ADMINISTRATION);
	}
	
	
	/**
	 * Checks if the link the Administration portal  should display or not. 
	 * The link is displayed in STANDARD mode and only on non-Preview pages.
	 *  
	 * @param owner		Current Activityspace.
	 * @return	<code>true</code> if the link to Administration Portal should be displayed, <code>false</code> otherwise.
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 */
	static public boolean HasAdminLinkAccess(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		
		return (	IsCurrentUserAdministrator(owner) && 
							!TaskAPIUICommon.IsCurrentPageInPreviewMode(owner) && 
							IsAccessStyleStandard(owner) 
						); 
	}
	
	/**
	 * Gets the Access Style for the current user. 
	 * The constants for the supported access styles, Standard, Low-bandwidth and Accessibility,
	 *  are defined in <code>AccessStyles</code>.
	 * 
	 * @param owner		Current Activityspace.
	 * @return An <code>AccessStyles</code> constant with the current Access Style mode.
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 * 
	 * @see AccessStyles
	 */
	public static AccessStyles GetAccessStyle(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		return owner.GetAccessStyle();
	}
	
	/**
	 * Checks if the Access Style mode for the current user is the Standard style.
	 * 
	 * @param owner		Current Activityspace.
	 * @return true if the Access Style for the current user is set to <code>AccessStyles.STANDARD</code>
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 * 
	 *  @see AccessStyles
	 */
	public static boolean IsAccessStyleStandard(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		AccessStyles style = GetAccessStyle(owner);
		return ( style == AccessStyles.STANDARD );
	}

	/**
	 * Checks if the Access Style mode for the current user is the Accessibility style.
	 * @param owner		Current Activityspace.
	 * @return true if the Access Style for the current user is set to <code>AccessStyles.SECTION_508</code>
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 * 
	 * @see AccessStyles
	 */
	public static boolean IsAccessStyleAccessibility(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		AccessStyles style = GetAccessStyle(owner);
		return ( style == AccessStyles.SECTION_508 );
	}
	
	/**
	 * Checks if the Access Style mode for the current user is the Low-bandwidth style.
	 * 
	 * @param owner		Current Activityspace.
	 * @return true if the Access Style for the current user is set to <code>AccessStyles.LOW_BANDWIDTH</code>
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 * 
	 * @see AccessStyles
	 */
	public static boolean IsAccessStyleLowBandwidth(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
		AccessStyles style = GetAccessStyle(owner);
		return ( style == AccessStyles.LOW_BANDWIDTH );
	}

	/**
	 * Gets the URL to the My Account settings page, where user's can edit their Portal setttings.
	 * 
	 * @param owner		Current Activity space.
	 * @return <code>IPTURL</code> with the URL to user's My Account settings page, or <code>null</code> value 
	 * if the URL is not applicable for the current user or page.
	 * 
	 * @throws <code>XPIllegalArgumentException</code> if the provided <code>AActivityspace</code> is <code>null</code> or not valid.
	 */
	public static IPTURL GetURLForMyAccountPage(AActivitySpace owner)
	{
		TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
	
		CommonOpenerOpenASURL url = null;
		if ( ! IsCurrentUserGuestUser(owner)  && !TaskAPIUICommon.IsCurrentPageInPreviewMode(owner) ) 
		{
			String strText 						= owner.GetString(296, "ptmsgs_portalcommonmsgs");
			String strApplBaseURL   =  TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
			String strASName 				= PortalSettingsAS.STR_MVC_CLASS_NAME;
			int nUserID 							= GetCurrentUserID(owner);
			
			url = new CommonOpenerOpenASURL(strText, strApplBaseURL, strASName, nUserID);
		}
		
		return url;
		
	}
	
	
}
