package com.plumtree.portalpages.test.browsing.directory.rest;

import junit.framework.*;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portalpages.browsing.directory.*;
import com.plumtree.portalpages.browsing.directory.rest.*;
import com.plumtree.portalpages.browsing.mock.directory.*;
import com.plumtree.portalpages.rest.*;
import com.plumtree.portalpages.rest.utils.*;
import com.plumtree.portaluiinfrastructure.mock.tags.*;
import com.plumtree.portaluiinfrastructure.mock.uiinfrastructure.*;

/**
 * This tests the formatting of the JSON output from the REST API.
 */
public class DirRESTDPTest extends TestCase
{
	public MockEnvironment m_environment = new MockEnvironment();
	public MockDirModel m_model = new MockDirModel();
	public DirRESTDP m_page = new DirRESTDP();

	public void setUp()
	{
		// This will probably log an error, but it will init what we want.
		m_environment.Init();
		m_environment.m_models.PutElement(DirModel.STR_MVC_CLASS_NAME, m_model);
		m_environment.RegisterPage(m_page);		
	}

	public void testSuccess()
	{
		m_model.m_nCardID = 534;
		m_model.m_strCardName = "my test";
		m_model.m_nCardSubmitFolder = 1212;
		m_model.SetFolderPath(m_model.m_nCardSubmitFolder, "test\\mytest\\test"); 

		MockWebData output = new MockWebData();
		m_page.Display(output);

		String strOutput = output.m_strData;

		assertNotNull("Page output should be non-null.", strOutput);

		assertTrue("Comment filtered JSON should start with /*.", XPStringUtility.StartsWith(strOutput, "/*"));
		assertTrue("Comment filtered JSON should start with /*.", XPStringUtility.EndsWith(strOutput, "*/"));

		// The parser can handle comment-filtered JSON.
		IJSONParser parser = JSONParser.GetInstance();
		parser.ParseJSONObject(strOutput);

		assertEquals("Unexpected JSON object name.", DirRESTConstants.DOCUMENT_FORMAT_NAME, parser.GetObjectName());

		assertEquals("Unexpected document name.", "my test", parser.GetMember(DirRESTConstants.DOCUMENT_NAME));
		assertEquals("Unexpected document id.", "534", parser.GetMember(DirRESTConstants.DOCUMENT_OBJECT_ID));

		assertEquals("Unexpected document folder ID.",
			"1212", parser.GetMember(DirRESTConstants.DOCUMENT_FOLDER_ID));
		assertEquals("Unexpected document folder path.",
			"test\\mytest\\test", parser.GetMember(DirRESTConstants.DOCUMENT_FOLDER_PATH));
	}

	public void testError()
	{
		m_model.m_nCardSubmitError = CommonRestErrors.GENERAL.GetCode();

		MockWebData output = new MockWebData();
		m_page.Display(output);

		String strOutput = output.m_strData;

		assertNotNull("Page output should be non-null.", strOutput);

		assertTrue("Comment filtered JSON should start with /*.", XPStringUtility.StartsWith(strOutput, "/*"));
		assertTrue("Comment filtered JSON should start with /*.", XPStringUtility.EndsWith(strOutput, "*/"));

		// The parser can handle comment-filtered JSON.
		IJSONParser parser = JSONParser.GetInstance();
		parser.ParseJSONObject(strOutput);

		assertEquals("Unexpected JSON object name.", RESTErrorConstants.ERROR_FORMAT_NAME, parser.GetObjectName());

		assertEquals("Unexpected error text.", CommonRestErrors.GENERAL.GetMessage(), parser.GetMember(RESTErrorConstants.ERROR_TEXT));
		assertEquals("Unexpected error code.", XPConvert.ToString(CommonRestErrors.GENERAL.GetCode()), parser.GetMember(RESTErrorConstants.ERROR_CODE));
	}

	public void testJSEscaping()
	{
		m_model.m_nCardID = 534;
		m_model.m_strCardName = "Card in folder test\\test\\test";
		m_model.m_nCardSubmitFolder = 13214;
		m_model.SetFolderPath(m_model.m_nCardSubmitFolder, "test\\othertest\\test"); 

		MockWebData output = new MockWebData();
		m_page.Display(output);

		String strOutput = output.m_strData;

		assertNotNull("Page output should be non-null.", strOutput);

		// NOTE: The parser will unescape \'s, so we should check for a single slash in the
		// parsed object, but a double slash in the actual string.
		assertTrue("String data should have escaped slashes in name.",
			strOutput.indexOf("Card in folder test\\\\test\\\\test") >= 0);
		assertTrue("String data should have escaped slashes in folder path.",
			strOutput.indexOf("test\\\\othertest\\\\test") >= 0);
		
		// The parser can handle comment-filtered JSON.
		IJSONParser parser = JSONParser.GetInstance();
		parser.ParseJSONObject(strOutput);

		assertEquals("Unexpected JSON object name.", DirRESTConstants.DOCUMENT_FORMAT_NAME, parser.GetObjectName());

		assertEquals("Document name should be JavaScript normal (i.e. escaped, then unescaped).",
			"Card in folder test\\test\\test", parser.GetMember(DirRESTConstants.DOCUMENT_NAME));

		assertEquals("Document folder path should be normal (i.e. escaped, then unescaped).",
			"test\\othertest\\test", parser.GetMember(DirRESTConstants.DOCUMENT_FOLDER_PATH));
	}

	public void testJSONAttack()
	{
		m_model.m_nCardID = 534;
		m_model.m_strCardName = "my test hack\"}} alert('hack');";

		MockWebData output = new MockWebData();
		m_page.Display(output);

		String strOutput = output.m_strData;

		assertNotNull("Page output should be non-null.", strOutput);

		assertTrue("Comment filtered JSON should start with /*.", XPStringUtility.StartsWith(strOutput, "/*"));
		assertTrue("Comment filtered JSON should start with /*.", XPStringUtility.EndsWith(strOutput, "*/"));

		// The parser can handle comment-filtered JSON.
		IJSONParser parser = JSONParser.GetInstance();
		parser.ParseJSONObject(strOutput);

		assertEquals("Unexpected JSON object name.", DirRESTConstants.DOCUMENT_FORMAT_NAME, parser.GetObjectName());

		assertEquals("Document name should be HTML encoded for safety.", "my test hack&quot;}} alert('hack');", parser.GetMember(DirRESTConstants.DOCUMENT_NAME));
		assertEquals("Unexpected document id.", "534", parser.GetMember(DirRESTConstants.DOCUMENT_OBJECT_ID));
		// TODO: folder ID and folder path
	}
}