package com.plumtree.portalpages.test.pei;

import junit.framework.*;

import com.plumtree.openfoundation.mockobjects.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.portalpages.browsing.portalsettings.password.*;
import com.plumtree.portalpages.common.constants.*;
import com.plumtree.portalpages.pei.*;
import com.plumtree.portaluiinfrastructure.mock.friendlyurl.*;
import com.plumtree.portaluiinfrastructure.mock.tags.*;
import com.plumtree.server.*;
import com.plumtree.server.mock.core.*;
import com.plumtree.server.mock.properties.*;
import com.plumtree.server.mock.users.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.login.*;
import com.plumtree.uiinfrastructure.web.*;

public class PTLoginActionsTest extends TestCase
{
	//  Test Data
	public static final String I18N_FILE = ResourceConstants.PORTALBROWSINGMSGS;
	public static final String USER_NAME = "testuser";

	protected static final long MS_ONE_DAY = 1000 * 60 * 60 * 24;

	protected IApplication m_application = null;
	protected AActivitySpace m_asSpace = null;
	protected MockEnvironment m_mockSpace = null;
	protected ISessionManager m_subSession = null;
	protected MockPTSession m_userSession = null;
	protected MockPTUser m_user = null;
	protected MockPTServerConfigSettings m_serverConfigSettings = null;

	protected PTLoginActions m_pei = null;

	/*
	 * @see junit.framework.TestCase#setUp()
	 */
	public void setUp() throws Exception 
	{
		m_application = new Application("","");
   		ApplicationManager.SetTestInstance(new MockApplicationManager());
		((MockApplicationManager) ApplicationManager.GetInstance()).SetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString(), m_application);

		// helper member that shadows the real Activity Space for convenience purposes.
   		m_mockSpace = new MockEnvironment();
   		m_asSpace = m_mockSpace;

   		m_subSession = SessionManagerFactory.GetSessionManager(new XPSessionSimulator());
   		m_asSpace.SetSubSession(m_subSession);

   		m_userSession = new MockPTSession();
   		m_asSpace.SetUserSession(m_userSession);

   		((MockPTSessionInfo) m_userSession.GetSessionInfo()).m_LoginName = USER_NAME;

   		// Set the user to be a native user
   		m_user = (MockPTUser) m_userSession.GetUser();
   		m_user.m_nAuthSourceID = PT_INTRINSICS.PT_AUTHSOURCE_PLUMTREE;

   		m_serverConfigSettings = (MockPTServerConfigSettings) m_userSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS, true);

   		// Default settings
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_EXPIRATION, 10);
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_EXPIRATION_WARNING, 5);

   		m_pei = new PTLoginActions();
	}

	/* 
	 * @see junit.framework.TestCase#tearDown()
	 */
	public void tearDown() throws Exception 
	{
		m_application = null;
		m_asSpace = null;
		m_mockSpace = null;
		m_subSession = null;
		m_userSession = null;
		m_serverConfigSettings = null;
		m_pei = null;
	}

	public void testExpirationDisabled()
	{
		//  Turn off password expiration
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_EXPIRATION, 0);

		// Set the user's last password change time to now - 20 days (in milliseconds)
		((MockPTUserInfo) m_userSession.GetUserInfo()).m_settings.PutElement(
			PTLoginActions.PASSWORD_LAST_CHANGED,
			XPConvert.ToString(XPSystem.GetCurrentTimeMilliSec() -
			MS_ONE_DAY * 20));

   		Redirect result = m_pei.CheckPasswordExpiration(m_userSession, null);

   		Assert.assertNull("A redirect should not be returned when expiration is disabled.", result);
	}

	public void testGuestUser()
	{
		// Set the user to be the guest user.
		((MockPTSessionInfo) m_userSession.GetSessionInfo()).m_bIsGuest = true;

		// Set the user's last password change time to now - 20 days (in milliseconds)
		((MockPTUserInfo) m_userSession.GetUserInfo()).m_settings.PutElement(
			PTLoginActions.PASSWORD_LAST_CHANGED,
			XPConvert.ToString(XPSystem.GetCurrentTimeMilliSec() -
			MS_ONE_DAY * 20));

		Redirect result = m_pei.CheckPasswordExpiration(m_userSession, null);

   		Assert.assertNull("A redirect should not be returned for guest users.", result);
	}

	public void testDefaultProfileUser()
	{
		// Set the user to be a default profile user.
		((MockPTSessionInfo) m_userSession.GetSessionInfo()).m_nAdminFolderID =
			PT_INTRINSICS.PT_ADMIN_FOLDER_DEFAULTPROFILES;

		// Set the user's last password change time to now - 20 days (in milliseconds)
		((MockPTUserInfo) m_userSession.GetUserInfo()).m_settings.PutElement(
			PTLoginActions.PASSWORD_LAST_CHANGED,
			XPConvert.ToString(XPSystem.GetCurrentTimeMilliSec() -
			MS_ONE_DAY * 20));

		Redirect result = m_pei.CheckPasswordExpiration(m_userSession, null);

   		Assert.assertNull("A redirect should not be returned for default profile users.", result);
	}

	public void testValidPassword()
	{
		// Set the user's last password change time to now (in milliseconds)
		((MockPTUserInfo) m_userSession.GetUserInfo()).m_settings.PutElement(
			PTLoginActions.PASSWORD_LAST_CHANGED, XPConvert.ToString(XPSystem.GetCurrentTimeMilliSec()));

		Redirect result = m_pei.CheckPasswordExpiration(m_userSession, null);

   		Assert.assertNull("A redirect should not be returned for valid passwrds.", result);
	}

	public void testExpiredPassword()
	{
		// Set the user's last password change time to now - 20 days (in milliseconds)
		((MockPTUserInfo) m_userSession.GetUserInfo()).m_settings.PutElement(
			PTLoginActions.PASSWORD_LAST_CHANGED,
			XPConvert.ToString(XPSystem.GetCurrentTimeMilliSec() -
			MS_ONE_DAY * 20));

		Redirect result = m_pei.CheckPasswordExpiration(m_userSession, new ApplicationData(null, null,
			m_application, m_subSession, null));

   		Assert.assertNotNull("The redirect should not be null.", result);

   		Assert.assertEquals("Incorrect redirect space.",
			PasswordAS.STR_MVC_CLASS_NAME, result.GetSpaceName());

		Assert.assertEquals("Incorrect redirect control.",
			ExpiredPasswordControl.STR_MVC_CLASS_NAME, result.GetControl());
	}

	public void testExpiredPasswordForNonNativeUser()
	{
		// Set the user to be non-native
		m_user.m_nAuthSourceID = PT_INTRINSICS.PT_AUTHSOURCE_SSO;

		// Set the user's last password change time to now - 20 days (in milliseconds)
		((MockPTUserInfo) m_userSession.GetUserInfo()).m_settings.PutElement(
			PTLoginActions.PASSWORD_LAST_CHANGED,
			XPConvert.ToString(XPSystem.GetCurrentTimeMilliSec() -
			MS_ONE_DAY * 20));

		Redirect result = m_pei.CheckPasswordExpiration(m_userSession, null);

   		Assert.assertNull("A redirect should not be returned for non-native users.", result);
	}

	public void testExpiredPasswordForAdminUser()
	{
		// Set the user to be the intrinsic admin user.
		MockPTSessionInfo info = (MockPTSessionInfo) m_userSession.GetSessionInfo();
		info.m_nUserID = PT_INTRINSICS.PT_USER_SYSADMIN;

		// Set the user's last password change time to now - 20 days (in milliseconds)
		((MockPTUserInfo) m_userSession.GetUserInfo()).m_settings.PutElement(
			PTLoginActions.PASSWORD_LAST_CHANGED,
			XPConvert.ToString(XPSystem.GetCurrentTimeMilliSec() -
			MS_ONE_DAY * 20));

		Redirect result = m_pei.CheckPasswordExpiration(m_userSession, null);

   		Assert.assertNull("A redirect should not be returned for the intrinsic admin user.", result);
	}

	public void testExpiredPasswordWarning()
	{
		// Set the user's last password change time to now - 7.5 days (in milliseconds)
		((MockPTUserInfo) m_userSession.GetUserInfo()).m_settings.PutElement(
			PTLoginActions.PASSWORD_LAST_CHANGED,
			XPConvert.ToString(XPSystem.GetCurrentTimeMilliSec() -
			((MS_ONE_DAY * 8) - (MS_ONE_DAY / 2))));

		Redirect result = m_pei.CheckPasswordExpiration(m_userSession, new ApplicationData(null, null,
			m_application, m_subSession, null));

   		Assert.assertNotNull("The redirect should not be null.", result);

   		Assert.assertEquals("Incorrect redirect space.",
			PasswordAS.STR_MVC_CLASS_NAME, result.GetSpaceName());

		Assert.assertEquals("Incorrect redirect control.",
			EditorStartControl.STR_MVC_CLASS_NAME, result.GetControl());

		Assert.assertEquals("Incorrect redirect control argument.",
			XPConvert.ToString(PasswordModel.EDITOR_START_FLAG_EXPIRATION_WARNING), result.GetControlArgument(EditorStartControl.QS_EDITOR_TYPE));

		// If password is 7.5 days old, and expires after 10 days, then there are
		// 2.5 days until expiration, which will be reported as 2 days.
		Assert.assertEquals("Incorrect redirect control argument.",
			"2", result.GetControlArgument(PasswordModel.PASSWORD_EXPIRATION));
	}

	public void testExpiredPasswordWarningForNonNativeUser()
	{
		// Set the user to be non-native
		m_user.m_nAuthSourceID = PT_INTRINSICS.PT_AUTHSOURCE_SSO;

		// Set the user's last password change time to now - 7 days (in milliseconds)
		((MockPTUserInfo) m_userSession.GetUserInfo()).m_settings.PutElement(
			PTLoginActions.PASSWORD_LAST_CHANGED,
			XPConvert.ToString(XPSystem.GetCurrentTimeMilliSec() -
			MS_ONE_DAY * 7));

		Redirect result = m_pei.CheckPasswordExpiration(m_userSession, null);

   		Assert.assertNull("A redirect should not be returned for non-native users.", result);
	}

	public void testUpgradeUser()
	{
		// Users that were created before the password policies were not in place
		// will not have the Password Last Changed property, so the PEI creates it for them.

		// Set the user's last password change to null
		((MockPTUserInfo) m_userSession.GetUserInfo()).m_settings.PutElement(
			PTLoginActions.PASSWORD_LAST_CHANGED, null);

		Redirect result = m_pei.CheckPasswordExpiration(m_userSession, null);

   		Assert.assertNull("A redirect should not be returned for users without the last password change property.", result);

   		// Check that the property exists now
   		MockPTObjectProperties props = m_user.m_properties;

   		Object oLastPasswordChange = props.m_properties.GetElement(new Integer(PT_INTRINSICS.PT_PASSWORD_LAST_CHANGED));
   		long lLastPasswordChange = XPConvert.ToLong(oLastPasswordChange);

   		Assert.assertTrue("New password change property should be positive.",
   			lLastPasswordChange > 0);
	}


	/**
	 * If you accidentally do integer arithmetic instead of long arithmetic
	 * when checking the times in milliseconds, you can cause the data to
	 * wrap.
	 */
	public void testLongArithmeticBug()
	{
   		// Set the password expiration to a large value that will trigger the
		// arithmetic error.
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_EXPIRATION, 180);

		// Set the user's last password change time to now - 20 days (in milliseconds)
		((MockPTUserInfo) m_userSession.GetUserInfo()).m_settings.PutElement(
			PTLoginActions.PASSWORD_LAST_CHANGED,
			XPConvert.ToString(XPSystem.GetCurrentTimeMilliSec() -
			MS_ONE_DAY * 20));

		Redirect result = m_pei.CheckPasswordExpiration(m_userSession, new ApplicationData(null, null,
			m_application, m_subSession, null));

   		Assert.assertNull("The redirect should be null.", result);
	}
}