package com.plumtree.portalpages.test.pei;

import junit.framework.*;

import com.plumtree.openfoundation.mockobjects.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.portalpages.browsing.portalsettings.password.*;
import com.plumtree.portalpages.common.constants.*;
import com.plumtree.portalpages.pei.*;
import com.plumtree.portaluiinfrastructure.mock.friendlyurl.*;
import com.plumtree.portaluiinfrastructure.mock.tags.*;
import com.plumtree.server.*;
import com.plumtree.server.mock.core.*;
import com.plumtree.server.mock.users.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * This class handles testing the PTPasswordActions and PTCreateAccountActions PEIs.
 * 
 * In reality, it is testing the underlying PasswordValidator that they both use,
 * and so is not really a unit test.
 * @author dhayler
 *
 */
public class PasswordPEIsTest extends TestCase
{
	//  Test Data
	public static final String I18N_FILE = ResourceConstants.PORTALBROWSINGMSGS;
	public static final String USER_NAME = "testuser";

	protected IApplication m_application = null;
	protected AActivitySpace m_asSpace = null;
	protected MockEnvironment m_mockSpace = null;
	protected ISessionManager m_subSession = null;
	protected MockPTSession m_userSession = null;
	protected MockPTServerConfigSettings m_serverConfigSettings = null;

	/*
	 * @see junit.framework.TestCase#setUp()
	 */
	public void setUp() throws Exception 
	{
		m_application = new Application("","");
   		ApplicationManager.SetTestInstance(new MockApplicationManager());
		((MockApplicationManager) ApplicationManager.GetInstance()).SetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString(), m_application);

		// helper member that shadows the real Activity Space for convenience purposes.
   		m_mockSpace = new MockEnvironment();
   		m_asSpace = m_mockSpace;

   		m_subSession = SessionManagerFactory.GetSessionManager(new XPSessionSimulator());
   		m_asSpace.SetSubSession(m_subSession);

   		m_userSession = (MockPTSession) m_asSpace.GetUserSession();

   		((MockPTSessionInfo) m_userSession.GetSessionInfo()).m_LoginName = USER_NAME;

   		m_serverConfigSettings = (MockPTServerConfigSettings) m_userSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS, true);

   		// Default settings
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_LENGTH, 7);
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_CASING, false);
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_NUMBERS, false);
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_PUNCTUATION, false);
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_NO_USER_IDS, true);
	}

	/* 
	 * @see junit.framework.TestCase#tearDown()
	 */
	public void tearDown() throws Exception 
	{
		m_application = null;
		m_asSpace = null;
		m_mockSpace = null;
		m_subSession = null;
		m_userSession = null;
		m_serverConfigSettings = null;
	}

	/**
	 * This method tests the various password rules implemented by the PTPasswordActions PEI.
	 */
	public void testBasicPasswordRulesInPasswordPEI()
	{
		internalTestBasicPasswordRules(new PasswordPEI(new PTPasswordActions()));
	}

	/**
	 * This method tests the various password rules implemented by the PTCreateAccountActions PEI.
	 */
	public void testBasicPasswordRulesInCreateAccountPEI()
	{
		internalTestBasicPasswordRules(new PasswordPEI(new PTCreateAccountActions()));
	}

	/**
	 * This helper method tests the various password rules implemented by the PEI.
	 * 
	 * @param _pei the PEI to test.
	 */
	private void internalTestBasicPasswordRules(PasswordPEI _pei)
	{
		/**
		 * Password too short
		 */
		String strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "123456");

		// Only check that it returns an i18nized string.  We will check the string IDs later.
		Assert.assertEquals("Too short password length not rejected correctly.",
			MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);

		Assert.assertEquals("Incorrect i18n string ID returned for too short password error.",
			1628, m_mockSpace.m_nLastI18NStringIndex);
		Assert.assertEquals("Incorrect i18n string File returned for too short password error.",
			I18N_FILE, m_mockSpace.m_strLastI18NStringFile);

		/**
		 * Password too long
		 */
		strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "12345678901234567890123");

		// Only check that it returns an i18nized string.  We will check the string IDs later.
		Assert.assertEquals("Too long password length not rejected correctly.",
			MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);

		Assert.assertEquals("Incorrect i18n string ID returned for too long password error.",
			1291, m_mockSpace.m_nLastI18NStringIndex);
		Assert.assertEquals("Incorrect i18n string File returned for too long password error.",
			I18N_FILE, m_mockSpace.m_strLastI18NStringFile);

		/**
		 * Password uses user ID
		 */
		strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, USER_NAME);

		// Only check that it returns an i18nized string.  We will check the string IDs later.
		Assert.assertEquals("Password using the user ID not rejected correctly.",
			MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);

		Assert.assertEquals("Incorrect i18n string ID returned for password using the user ID error.",
			1982, m_mockSpace.m_nLastI18NStringIndex);
		Assert.assertEquals("Incorrect i18n string File returned for password using the user ID error.",
			I18N_FILE, m_mockSpace.m_strLastI18NStringFile);

		/**
		 * Password just right
		 */
		strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "12345678");

		Assert.assertNull("Valid password rejected incorrectly.", strResult);
	}

	/**
	 * This method tests the password length rule implemented by the Password Actions PEI.
	 */
	public void testPasswordLengthRuleInPasswordPEI()
	{
		internalTestPasswordLengthRule(new PasswordPEI(new PTPasswordActions()));
	}

	/**
	 * This method tests the password length rule implemented by the Create Account Actions PEI.
	 */
	public void testPasswordLengthRuleInCreateAccountPEI()
	{
		internalTestPasswordLengthRule(new PasswordPEI(new PTCreateAccountActions()));
	}

	/**
	 * This method tests the password length rule implemented by the PEI.
	 */
	private void internalTestPasswordLengthRule(PasswordPEI _pei)
	{
		/**
		 * Password length
		 */
		// Set password length requirement
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_LENGTH, 8);

		String strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "1234567");

		// Only check that it returns an i18nized string.  We will check the string IDs later.
		Assert.assertEquals("Too short password length not rejected correctly.",
			MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);

		Assert.assertEquals("Incorrect i18n string ID returned for too short password error.",
			1628, m_mockSpace.m_nLastI18NStringIndex);
		Assert.assertEquals("Incorrect i18n string File returned for too short password error.",
			I18N_FILE, m_mockSpace.m_strLastI18NStringFile);

		strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "12345678");

		Assert.assertNull("Proper length password rejected incorrectly.", strResult);
	}

	/**
	 * This method tests the password casing rule implemented by the Password Actions PEI.
	 */
	public void testPasswordCasingRuleInPasswordPEI()
	{
		internalTestPasswordCasingRule(new PasswordPEI(new PTPasswordActions()));
	}

	/**
	 * This method tests the password casing rule implemented by the Create Account Actions PEI.
	 */
	public void testPasswordCasingRuleInCreateAccountPEI()
	{
		internalTestPasswordCasingRule(new PasswordPEI(new PTCreateAccountActions()));
	}

	/**
	 * This method tests the password casing rule implemented by the PEI.
	 */
	private void internalTestPasswordCasingRule(PasswordPEI _pei)
	{
		/**
		 * Password requires both upper case and lower case letters
		 */
		// Set password casing requirement to true
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_CASING, true);

		String strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "testcasing");

		// Only check that it returns an i18nized string.  We will check the string IDs later.
		Assert.assertEquals("All lower case password not rejected correctly.",
			MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);

		Assert.assertEquals("Incorrect i18n string ID returned for all lower case password error.",
			1983, m_mockSpace.m_nLastI18NStringIndex);
		Assert.assertEquals("Incorrect i18n string File returned for all lower case password error.",
			I18N_FILE, m_mockSpace.m_strLastI18NStringFile);

		strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "TESTCASING");

		// Only check that it returns an i18nized string.  We will check the string IDs later.
		Assert.assertEquals("All upper case password not rejected correctly.",
			MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);

		Assert.assertEquals("Incorrect i18n string ID returned for all upper case password error.",
			1983, m_mockSpace.m_nLastI18NStringIndex);
		Assert.assertEquals("Incorrect i18n string File returned for all upper case password error.",
			I18N_FILE, m_mockSpace.m_strLastI18NStringFile);

		strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "1234567");

		// Only check that it returns an i18nized string.  We will check the string IDs later.
		Assert.assertEquals("All number password not rejected correctly.",
			MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);

		Assert.assertEquals("Incorrect i18n string ID returned for all number password error.",
			1983, m_mockSpace.m_nLastI18NStringIndex);
		Assert.assertEquals("Incorrect i18n string File returned for all number password error.",
			I18N_FILE, m_mockSpace.m_strLastI18NStringFile);

		strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "!@#$%^&*");

		// Only check that it returns an i18nized string.  We will check the string IDs later.
		Assert.assertEquals("All punctuation password not rejected correctly.",
			MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);

		Assert.assertEquals("Incorrect i18n string ID returned for all punctuation password error.",
			1983, m_mockSpace.m_nLastI18NStringIndex);
		Assert.assertEquals("Incorrect i18n string File returned for all punctuation password error.",
			I18N_FILE, m_mockSpace.m_strLastI18NStringFile);

		strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "TestCasing");

		Assert.assertNull("Properly cased password rejected incorrectly.", strResult);
	}

	/**
	 * This method tests the password number rule implemented by the Password Actions PEI.
	 */
	public void testPasswordNumberRuleInPasswordPEI()
	{
		internalTestPasswordNumberRule(new PasswordPEI(new PTPasswordActions()));
	}

	/**
	 * This method tests the password number rule implemented by the Create Account Actions PEI.
	 */
	public void testPasswordNumberRuleInCreateAccountPEI()
	{
		internalTestPasswordNumberRule(new PasswordPEI(new PTCreateAccountActions()));
	}

	/**
	 * This method tests the password number rule implemented by the PEI.
	 */
	private void internalTestPasswordNumberRule(PasswordPEI _pei)
	{
		/**
		 * Password requires a number
		 */
		// Set password number requirement to true
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_NUMBERS, true);

		String strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "testnumbers");

		// Only check that it returns an i18nized string.  We will check the string IDs later.
		Assert.assertEquals("No number password not rejected correctly.",
			MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);

		Assert.assertEquals("Incorrect i18n string ID returned for no casing password error.",
			1984, m_mockSpace.m_nLastI18NStringIndex);
		Assert.assertEquals("Incorrect i18n string File returned for no casing password error.",
			I18N_FILE, m_mockSpace.m_strLastI18NStringFile);

		strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "test1Number");

		Assert.assertNull("Properly numberalized password rejected incorrectly.", strResult);
	}

	/**
	 * This method tests the password punctuation rule implemented by the Password Actions PEI.
	 */
	public void testPasswordPunctuationRuleInPasswordPEI()
	{
		internalTestPasswordPunctuationRule(new PasswordPEI(new PTPasswordActions()));
	}

	/**
	 * This method tests the password punctuation rule implemented by the CreateAccount Actions PEI.
	 */
	public void testPasswordPunctuationRuleInCreateAccountPEI()
	{
		internalTestPasswordPunctuationRule(new PasswordPEI(new PTCreateAccountActions()));
	}

	/**
	 * This method tests the password punctuation rule implemented by the PEI.
	 */
	private void internalTestPasswordPunctuationRule(PasswordPEI _pei)
	{
		/**
		 * Password requires punctuation
		 */
		// Set password number requirement to true
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_INCLUDE_PUNCTUATION, true);

		String strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "testpunctuation");

		// Only check that it returns an i18nized string.  We will check the string IDs later.
		Assert.assertEquals("No punctuation password not rejected correctly.",
			MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);

		Assert.assertEquals("Incorrect i18n string ID returned for no punctuation password error.",
			1985, m_mockSpace.m_nLastI18NStringIndex);
		Assert.assertEquals("Incorrect i18n string File returned for no punctuation password error.",
			I18N_FILE, m_mockSpace.m_strLastI18NStringFile);

		strResult = _pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "testpunctuat!on");

		Assert.assertNull("Properly punctuated password rejected incorrectly.", strResult);
	}

	/**
	 * This method tests the password reuse history rule implemented by the Create Account PEI.
	 */
	public void testPasswordReuseHistoryInCreateAccountPEI()
	{
		/**
		 * Password reuse history enabled
		 */
		// Set password reuse history to 12 (max)
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_REUSE_HISTORY, 12);

   		// We don't need to set any password history here because newly created accounts
   		// do not have a history.

   		PasswordPEI pei = new PasswordPEI(new PTCreateAccountActions());

		String strResult = pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "testpassword");

		Assert.assertNull("Newly created accounts should not fail due to password reuse.", strResult);
	}

	/**
	 * This method tests the password reuse history rule implemented by the Change Password PEI.
	 */
	public void testPasswordReuseHistoryInPasswordPEI()
	{
		/**
		 * Password reuse history enabled
		 */
		// Set password reuse history to 12 (max)
   		m_serverConfigSettings.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_PASSWORD_REUSE_HISTORY, 12);

   		String strPreviousPassword = "usedPass";

   		// Set password in mock history
   		((MockPTUserManager) m_userSession.GetUsers()).m_PasswordInHistory = strPreviousPassword;

   		PasswordPEI pei = new PasswordPEI(new PTPasswordActions());

		String strResult = pei.CallPEIPasswordMethod(m_asSpace, m_userSession, strPreviousPassword);

		// Only check that it returns an i18nized string.  We will check the string IDs later.
		Assert.assertEquals("Reused password not rejected correctly.",
			MockEnvironment.MOCK_I18N_PLACEHOLDER_STRING, strResult);

		Assert.assertEquals("Incorrect i18n string ID returned for reused password error.",
			1998, m_mockSpace.m_nLastI18NStringIndex);
		Assert.assertEquals("Incorrect i18n string File returned for reused password error.",
			I18N_FILE, m_mockSpace.m_strLastI18NStringFile);

		strResult = pei.CallPEIPasswordMethod(m_asSpace, m_userSession, "unusedPass");

		Assert.assertNull("Unused password rejected incorrectly.", strResult);
	}

	/**
	 * This method tests the behavior if the password is missing in the Password Actions PEI.
	 * 
	 * This is not relevant in the Create Account PEI, so it is not tested.
	 */
	public void testNoPasswordInPasswordPEI()
	{
		PasswordPEI pei = new PasswordPEI(new PTPasswordActions());

		String strResult = pei.CallPEIPasswordMethod(m_asSpace, m_userSession, null);

		// Test defaults
		Assert.assertNull("PEI should return null if password is missing.", strResult);

	}

	/**
	 * Private inner class to help us to run the same test on 2 PEIs with different
	 * method signatures.
	 */
	private class PasswordPEI
	{
		private IPasswordActions m_pwPEI = null;
		private ICreateAccountActions m_caPEI = null;

		public PasswordPEI(IPasswordActions _pei)
		{
			m_pwPEI = _pei;
		}

		public PasswordPEI(ICreateAccountActions _pei)
		{
			m_caPEI = _pei;
		}

		public String CallPEIPasswordMethod(AActivitySpace _asCurrentSpace,
			IPTSession _ptUserSession, String _strPassword)
		{
			if (m_pwPEI != null)
			{
				// Fill out required control arguments
		   		XPHashtable data = new XPHashtable();

		   		if (_strPassword != null)
		   		{
		   			data.PutElement(PasswordView.HTML_INPUT_NEW_PASSWORD,
		   					new String[] {_strPassword});
		   		}

				return m_pwPEI.OnBeforeChangePassword(_asCurrentSpace, _ptUserSession, data);
			}

			return m_caPEI.OnBeforeCreateAccount(_asCurrentSpace, _ptUserSession, USER_NAME, _strPassword);
		}
	}
}