using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portaluiinfrastructure.application.varpacks
{
  /// <summary>
  /// This VarPack contains the localized names for all the document types.
  /// </summary>
  /// <author>Jose Quezada</author>
  public class PTDocTypesVarPack : BaseVarPack
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.application.varpacks.PTDocTypesVarPack");
    
    /// <summary>
    /// The string ID of this variable package. 
    /// </summary>
    public const String VARPACK_ID = "PTDocTypesVarPack";
    
    /// <summary>
    /// JHQ- This XPHashtable contains a mapping of doc type UUIDs and a hashtable of  
    /// Localized Names that are stored in this varpack
    /// </summary>
    private XPHashtable m_xpUUIDtoLocalizedNames;
    
    /// <summary>
    /// Constructor for PTDocTypesVarPack.
    /// </summary>
    /// <param name="_strConfigFilePath">
    /// The path to the folder in which SearchLocales.xml
    /// resides.
    /// </param>
    public PTDocTypesVarPack() : base()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackID()"></seealso>
    public override String GetVarPackID()
    {
      return VARPACK_ID;
    }
    
    /// <summary>
    /// This method will return an enumeration of the UUIDs of doc types in this varpack.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackKeys()"></seealso>
    public override IXPEnumerator GetVarPackKeys()
    {
      return m_xpUUIDtoLocalizedNames.GetKeys();
    }
    
    /// <summary>
    /// You cannot expire the settings content. You can only reload it. This
    /// method does nothing.
    /// </summary>
    /// <seealso cref="com.plumtree.xpsupport.web.BaseVarPack#ExpireContent()"></seealso>
    protected override void ExpireContent()
    {
    }
    
    /// <summary>
    /// this will return an XPHashtable mapping of UUIDs & doc type localized names 
    /// for the locale code passed in.  It is best to use the helper methods to get 
    /// UUID to Name mappings.  
    /// </summary>
    /// <seealso cref="com.plumtree.xpsupport.web.BaseVarPack#GetValue(String)"></seealso>
    protected override Object GetValue(String _strUUID)
    {
      if (null == _strUUID)
      {
        return null;
      }
      return m_xpUUIDtoLocalizedNames.GetElement(XPStringUtility.ToUpperCase(_strUUID));
      }
    
    /// <summary>
    /// Helper method that retrieves the localized name for the given UUID.
    /// </summary>
    /// <param name="_UUID"></param>
    /// <param name="_langCode"></param>
    /// <returns>localized name if available, else, the 'en' lang name.</returns>
    public virtual String GetLocalizedName(String _UUID, String _langCode)
    {
      if (null == _UUID)
      {
        return "";
      }
      _UUID = XPStringUtility.ToUpperCase(_UUID);
      if (!m_xpUUIDtoLocalizedNames.ContainsKey(_UUID))
      {
        return "";
      }
      XPHashtable xpLocNames = (XPHashtable) m_xpUUIDtoLocalizedNames.GetElement(_UUID);
      if (null == _langCode || !xpLocNames.ContainsKey(XPStringUtility.ToUpperCase(_langCode)))
      {
        _langCode = "EN";
      } else
      {
        _langCode = XPStringUtility.ToUpperCase(_langCode);
      }
      return (String) xpLocNames.GetElement(_langCode);
    }
    
    /// <summary>
    /// This method loads the doc types uuid & localized names into memory.
    /// </summary>
    /// <seealso cref="com.plumtree.xpsupport.web.BaseVarPack#Load()"></seealso>
    protected override void Load()
    {
      // The PTDirPrefs VarPack is loaded before this one, so it
      // initializes the core server for us.  This call should not need
      // to do any initialization.
      IPTSession ptGuestSession = PortalObjectsFactory.CreateSession();
      // connect session
      // Guest login -- The guest user password check is bypassed,
      // so we don't need to pass in the password.
      ptGuestSession.Connect(PT_INTRINSICS.PT_USER_GUEST, null, null);
      // get the document type manager
      IPTObjectManager ptDTM = (IPTObjectManager) ptGuestSession.GetDocumentTypes();
      // query for all the document types
      IPTQueryResult ptqrDocTypes = ptDTM.SimpleQuery(-1, PT_PROPIDS.PT_PROPID_OBJECTID);
      // for each doc type, get it's localized names and map it to its UUID
      int rowcnt = ptqrDocTypes.RowCount();
      m_xpUUIDtoLocalizedNames = new XPHashtable(rowcnt);
      for (int idxOfDT = 0;idxOfDT < rowcnt;idxOfDT++)
      {
        int id = ptqrDocTypes.ItemAsInt(idxOfDT, PT_PROPIDS.PT_PROPID_OBJECTID);
        // ids 100-110 are the intrinsic doc types.  we will only support the intrinsic types
        // TODO: we should create a constants file under com.plumtree.server that holds
        // these UUIDs.
        /* get the image UUID */
        String UUID;
        switch (id){
        /* Microsoft Word */
        case 100:
        //PTOfficeAccessor.CLSID_WORD_IMAGE;
        UUID = "{0C35DD71-6453-11D2-88C3-006008168DE5}";
        break;
        /* Microsoft Excel */
        case 101:
        //PTOfficeAccessor.CLSID_EXCEL_IMAGE;
        UUID = "{0C35DD72-6453-11D2-88C3-006008168DE5}";
        break;
        /* Microsoft Powerpoint */
        case 102:
        //PTOfficeAccessor.CLSID_PPT_IMAGE;
        UUID = "{0C35DD73-6453-11D2-88C3-006008168DE5}";
        break;
        /* PDF Documents */
        case 103:
        //PTPDFAccessor.CLSID_PDF_IMAGE;
        UUID = "{DAA2F4D1-D037-11D2-8909-006008168DE5}";
        break;
        /* Text Documents */
        case 104:
        //PTFileAccessor.FILEIMAGE_CLSID;
        UUID = "{47C7EA11-7B19-11d2-88CB-006008168DE5}";
        break;
        /* Web Pages */
        case 106:
        //PTHTMLAccessor.HTMLDOCIMAGEID;
        UUID = "{52A5A5A0-84C9-11d2-A0C5-0060979C42D8}";
        break;
        /* Non-Indexed Files */
        case 107:
        //PTFileAccessor.FILEIMAGE_CLSID;
        UUID = "{47C7EA11-7B19-11d2-88CB-006008168DE5}";
        break;
        /* Microsoft Project */
        case 108:
        //DirectoryAccessorConstants.PT_MSPROJECT_DOCUMENT_IMAGEUUID;
        UUID = "{8D6D9F50-D512-11d3-8DB0-00C04FF44474}";
        break;
        /* Visio Documents */
        case 109:
        //DirectoryAccessorConstants.PT_VISIO_DOCUMENT_IMAGEUUID;
        UUID = "{2CEEC472-7CF0-11d3-BB3A-00105ACE365C}";
        break;
        /* Microsoft Exchange */
        case 110:
        //DirectoryAccessorConstants.PT_MSEXCHANGE_DOCUMENT_IMAGEUUID;
        UUID = "{e321a652-bc73-11d2-baca-00105ace365c}";
        break;
        default:
        continue;
        break;
        }
        /* get the localized names */
        IPTLocalizable ptDT = (IPTLocalizable) ptDTM.Open(id, false);
        Object[][] oLocNames = ptDT.GetLocalizedNames();
        XPHashtable xpLocNames = new XPHashtable();
        /* start with the primary language */
        xpLocNames.PutElement(XPStringUtility.ToUpperCase(ptDT.GetPrimaryLang()),
         ptDT.GetLocalizedName());
        /* localized names */
        if (null == oLocNames)
        {
          m_xpUUIDtoLocalizedNames.PutElement(XPStringUtility.ToUpperCase(UUID),
           xpLocNames);
          continue;
        }
        for (int idxOfNames = 0;idxOfNames < oLocNames[0].Length;idxOfNames++)
        {
          xpLocNames.PutElement(XPStringUtility.ToUpperCase((String) oLocNames[PT_LOCALIZED_STRINGS_INDICES.PT_LSINDEX_LANG][idxOfNames]),
           oLocNames[PT_LOCALIZED_STRINGS_INDICES.PT_LSINDEX_STRING][idxOfNames]);
          }
        m_xpUUIDtoLocalizedNames.PutElement(XPStringUtility.ToUpperCase(UUID),
         xpLocNames);
      }
    }
    
    /// <summary>
    /// This method reloads the doc types uuid & localized names . It delegates to Load().
    /// </summary>
    /// <seealso cref="com.plumtree.xpsupport.web.BaseVarPack#Reload()"></seealso>
    protected override void Reload()
    {
      Load();
    }
  }
}