using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.classtypedescriptors.providerframework;
using com.plumtree.portaluiinfrastructure.friendlyurl;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.opener.types;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.classtypedescriptors
{
  /// <summary>
  /// This class manages the class type descriptor objects.
  /// </summary>
  /// <author>John Osborne</author>
  /// <author>Ricky Wong</author>
  public class ClassTypeDescManager : ClassTypeManager,IClassTypeManager,
  IPTClassTypeDescManager,IFriendlyURLManager
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.classtypedescriptors.ClassTypeDescManager");
    //////////////////////////////
    // IPTClassTypeDesc Members //
    //////////////////////////////
    
    /// <summary>
    /// A map of ProvInfo[] objects available for a particular class. 
    /// </summary>
    private XPHashtable m_mapArrayProvInfoByClassID;
    
    /// <summary>
    /// A map that has ClassID arrays ordered alphabetically by name. 
    /// </summary>
    private XPHashtable m_mapClassIDArraysByLangID;
    
    /// <summary>
    /// A map of ProvInfo objects. The key is the English name for the provider. 
    /// </summary>
    private XPHashtable m_mapProvInfoByEnglishName;
    
    /// <summary>
    /// An array of all loaded ProvInfo objects. 
    /// </summary>
    private IProvInfo[] m_aProvInfo;
    
    /// <summary>
    /// Mapping of class ids to friendly URL keys 
    /// </summary>
    private XPHashtable m_htLookupTable;
    
    /// <param name="cmCachingManager"></param>
    public ClassTypeDescManager(CachingManager cmCachingManager) : 
    // load the dynamically discovered ClassTypeDesc objects.
    base(cmCachingManager)
    {
      /////////////////////////////////////
      // IPTClassTypeDesc Initialization //
      /////////////////////////////////////
      // load the dynamically discovered ProvInfo objects.
      Object[] oaProvInfo = (Object[]) cmCachingManager.GetEntry("ProvInfo");
      IProvInfo[] aProvInfo = new IProvInfo[oaProvInfo.Length];
      for (int i = 0;i < oaProvInfo.Length;i++)
      {
        aProvInfo[i] = (IProvInfo) oaProvInfo[i];
      }
      m_aProvInfo = aProvInfo;
      m_mapProvInfoByEnglishName = new XPHashtable();
      m_mapArrayProvInfoByClassID = new XPHashtable();
      m_mapClassIDArraysByLangID = new XPHashtable();
      ////////////////////////////
      // General Initialization //
      ////////////////////////////
      InitializeMapOfProvInfoByEnglishName();
      InitializeMapOfProvInfoArraysByClassID();
      InitializeMapOfListsOfClassIDsOrderedByName();
      ////////////////////////////////////////
      // IFriendlyURLManager Initialization //
      ////////////////////////////////////////
      m_htLookupTable = (XPHashtable) cmCachingManager.GetEntry("FriendlyURLKeyFromClassIdLookupTable");
      }
    private void InitializeMapOfProvInfoByEnglishName()
    {
      for (int i = 0;i < m_aProvInfo.Length;i++)
      {
        // Map by Provider English Name.
        m_mapProvInfoByEnglishName.PutElement(m_aProvInfo[i].GetProviderEnglishName(),
         m_aProvInfo[i]);
      }
    }
    private void InitializeMapOfProvInfoArraysByClassID()
    {
      for (int i = 0;i < m_aIClassTypes.Length;i++)
      {
        ClassTypeEnum myEnum = m_aIClassTypes[i].GetClassType();
        // set ProvInfo only for extensible PT classtypes.
        if (myEnum.Equals(ClassTypeEnum.PTCLASSTYPE) && ((APTClassTypeDesc) m_aIClassTypes[i]).IsExtensible())
        {
          int nClassID = ((APTClassTypeDesc) m_aIClassTypes[i]).GetClassID();
          XPArrayList alHolder = new XPArrayList();
          for (int j = 0;j < m_aProvInfo.Length;j++)
          {
            // Make map of arrays by class IDs
            int[] nClassIDs = m_aProvInfo[j].GetArrayOfClassIDsRelatedTo();
            for (int k = 0;k < nClassIDs.Length;k++)
            {
              if (nClassIDs[k] == nClassID)
              {
                alHolder.Add(m_aProvInfo[j]);
                k = nClassIDs.Length;
              }
            }
          }
          if (alHolder.GetSize() == 0)
          {
            return;
          }
          IProvInfo[] aProvInfos = new IProvInfo[alHolder.GetSize()];
          for (int j = 0;j < alHolder.GetSize();j++)
          {
            aProvInfos[j] = (IProvInfo) alHolder.GetElement(j);
          }
          m_mapArrayProvInfoByClassID.PutElement(nClassID, aProvInfos);
        }
      }
    }
    private void InitializeMapOfListsOfClassIDsOrderedByName()
    {
      IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
      XPResourceManager rm = (XPResourceManager) application.GetCachingManager().GetEntry("ResourceManager");
      String[] aKnownLangs = rm.GetSupportedLanguages();
      // loop through the languanges.
      for (int k = 0;k < aKnownLangs.Length;k++)
      {
        // make array list of PT classtypes.
        XPArrayList arrList = new XPArrayList();
        for (int i = 0;i < m_aIClassTypes.Length;i++)
        {
          ClassTypeEnum myEnum = m_aIClassTypes[i].GetClassType();
          if (myEnum.Equals(ClassTypeEnum.PTCLASSTYPE))
          {
            arrList.Add(m_aIClassTypes[i]);
          }
        }
        // sort the list alphabetically by name.
        XPCollections.Sort(arrList, new ClassTypeComparator(aKnownLangs[k]));
        int[] aIDs = new int[arrList.GetSize()];
        for (int i = 0;i < arrList.GetSize();i++)
        {
          aIDs[i] = ((APTClassTypeDesc) arrList.GetElement(i)).GetClassID();
          }
        m_mapClassIDArraysByLangID.PutElement(aKnownLangs[k], aIDs);
      }
    }
    
    /// <summary>
    /// Initializes an extensible Plumtree object.
    /// </summary>
    /// <param name="_ptObject">The Plumtree Object.</param>
    /// <param name="_sCLSID">The CLSID of the provider.</param>
    public static void InitializeExtensibleObject(IPTObject _ptObject, String _sCLSID)
    {
      int nClassID = _ptObject.GetClassID();
      if (nClassID == PT_CLASSIDS.PT_AUTHSOURCE_ID)
      {
        ((IPTAuthSource) _ptObject).SetProviderCLSID(_sCLSID);
      } else
      if (nClassID == PT_CLASSIDS.PT_DATASOURCE_ID)
      {
        ((IPTDataSource) _ptObject).Initialize(_sCLSID);
      } else
      if (nClassID == PT_CLASSIDS.PT_FEDERATEDPORTAL_ID)
      {
      } else
      // TODO: What does IPTFederatedPortal do now??
      // ((IPTFederatedPortal) _ptObject).SetProviderCLSID(_sCLSID);
      if (nClassID == PT_CLASSIDS.PT_GADGET_ID)
      {
        ((IPTGadget) _ptObject).Initialize(_sCLSID);
      }
    }
    
    /// <summary>
    /// Returns whether this class type appears in the create object dropdown.
    /// This method only applies to PTClassTypeDesc class types.
    /// </summary>
    /// <param name="_nClassID">ClassID of the object.</param>
    /// <returns>True if this class appears in the create object dropdown.</returns>
    public bool AppearsInMainCreateNewClassMenu(int _nClassID)
    {
      return ((APTClassTypeDesc) GetClassTypeDescItem(XPConvert.ToString(_nClassID))).AppearsInMainCreateNewClassMenu();
      }
    
    /// <summary>
    /// Returns whether this class type appears in the global object property 
    /// map utility.  This method only applies to PTClassTypeDesc class types.
    /// </summary>
    /// <param name="_nClassID">ClassID of the object.</param>
    /// <returns>
    /// True if this class appears in the global object property map 
    /// utility.
    /// </returns>
    public bool AppearsInGlobalObjPropMap(int _nClassID)
    {
      return ((APTClassTypeDesc) GetClassTypeDescItem(XPConvert.ToString(_nClassID))).AppearsInGlobalObjPropMap();
      }
    
    /// <summary>
    /// Returns all registered ProvInfo items.
    /// </summary>
    /// <returns>All registered ProvInfo items</returns>
    public IProvInfo[] GetAllRegisteredProvInfo()
    {
      return m_aProvInfo;
    }
    
    /// <summary>
    /// Returns the default icon to use for this PT class type.
    /// </summary>
    /// <param name="_nClassID">ClassID to get icon for.</param>
    /// <param name="_strLangID">If no icon, set alt in this language.</param>
    /// <returns>The default icon to use for this PT class type.</returns>
    public HTMLImg GetDefaultIconForClassType(int _nClassID, String _strLangID)
    {
      String strClassID = XPConvert.ToString(_nClassID);
      // if global boject, just return the job icon.
      if (_nClassID == PT_CLASSIDS.PT_GLOBALOBJECT_ID)
      {
        return ((APTClassTypeDesc) GetClassTypeDescItem(XPConvert.ToString(PT_CLASSIDS.PT_JOB_ID))).GetDefaultIcon();
        }
      HTMLImg htmlImg = ((APTClassTypeDesc) GetClassTypeDescItem(strClassID)).GetDefaultIcon();
      if (null != htmlImg)
      {
        htmlImg.SetAlt(((APTClassTypeDesc) GetClassTypeDescItem(strClassID)).GetClassTypeLocalizedName(_strLangID));
        }
      return htmlImg;
    }
    public HTMLImg GetDefaultIconForClassType(int _nClassID, String _strLangID,
     String _sProvEnglishName)
    {
      return GetDefaultIconForClassType(_nClassID, _strLangID);
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IPTClassTypeDescManager#GetProviderEnglishNameForExistingObject(int, int, com.plumtree.server.IPTSession)
     */
     
    public String GetProviderEnglishNameForExistingObject(int _nClassID, int _nObjectID,
     IPTSession _ptSession)
    {
      APTExtClassTypeDesc ptClassTypeDesc;
      try
      {
        ptClassTypeDesc = (APTExtClassTypeDesc) GetClassTypeDescItem(XPConvert.ToString(_nClassID));
        String strProvCLSID = ptClassTypeDesc.GetCLSIDForExistingObject(_nObjectID,
         _ptSession);
        return GetProviderEnglishNameGivenCLSID(strProvCLSID);
      }
      catch (Exception e)
      {
        throw new XPException("Error retrieving English Provider name for given object.",
         e);
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IPTClassTypeDescManager#GetProviderEnglishNameGivenCLSID(java.lang.String)
     */
     
    public String GetProviderEnglishNameGivenCLSID(String _sCLSID)
    {
      // retrieve the name from our discovered list of providers.
      for (int i = 0;i < m_aProvInfo.Length;i++)
      {
        if (XPStringUtility.CompareIgnoreCase(m_aProvInfo[i].GetProviderCLSID(),
         _sCLSID) == 0)
        {
          return m_aProvInfo[i].GetProviderEnglishName();
        }
      }
      // special case for plumtree auth source.
      if (_sCLSID.Equals("PLUMTREE"))
      {
        return _sCLSID;
      }
      // if the name is not on the list, throw an error.
      String errorMsg = "Error retrieving English Provider name for ClassID: " + _sCLSID;
      log.Error(errorMsg);
      throw new XPException(errorMsg);
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IPTClassTypeDescManager#GetRegisteredProvInfoForClassType(int)
     */
     
    public IProvInfo[] GetRegisteredProvInfoForClassType(int _nClassID)
    {
      if (m_mapArrayProvInfoByClassID.ContainsKey(_nClassID))
      {
        return (IProvInfo[]) m_mapArrayProvInfoByClassID.GetElement(_nClassID);
        }
      return null;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IPTClassTypeDescManager#GetRegisteredProvInfoGivenEnglishName(java.lang.String)
     */
     
    public IProvInfo GetRegisteredProvInfoGivenEnglishName(String _strProvInfoEngName)
    {
      if (m_mapProvInfoByEnglishName.ContainsKey(_strProvInfoEngName))
      {
        return (IProvInfo) m_mapProvInfoByEnglishName.GetElement(_strProvInfoEngName);
        }
      return null;
    }
    
    /// <summary>
    /// True if the Plumtree Object of this ClassID is extensible.
    /// </summary>
    /// <param name="_nClassID">The ClassID for this Plumtree Object.</param>
    /// <returns>True if the Plumtree Object of this ClassID is extensible.</returns>
    public bool IsObjectExtensible(int _nClassID)
    {
      return ((APTClassTypeDesc) GetClassTypeDescItem(XPConvert.ToString(_nClassID))).IsExtensible();
      }
    
    /// <summary>
    /// Returns the name of the activity space associated with the
    /// specified PT ClassID.
    /// </summary>
    /// <param name="_nClassID">The ClassID for this Plumtree Object.</param>
    /// <returns>The name of the associated activity space.</returns>
    public virtual String GetASNameForClassTypeEditor(int _nClassID)
    {
      APTClassTypeDesc baseType = (APTClassTypeDesc) GetClassTypeDescItem(XPConvert.ToString(_nClassID));
      if (baseType.IsExtensible())
      {
        String errorMsg = "Attempting to get non-extensible " + "editor name from extensible type: " + _nClassID;
        log.Error(errorMsg);
        throw new XPException(errorMsg);
      }
      APTNonExtClassTypeDesc realType = (APTNonExtClassTypeDesc) baseType;
      return realType.GetASNameForEditor();
    }
    public virtual String GetASNameForClassTypeEditor(int _nClassID, String _StrProvEnglishName)
    {
      APTClassTypeDesc baseType = (APTClassTypeDesc) GetClassTypeDescItem(XPConvert.ToString(_nClassID));
      if (!baseType.IsExtensible())
      {
        String errorMsg = "Attempting to get extensible editor name from " + "non-extensible type: " + _nClassID + " with provider name " + _StrProvEnglishName;
        log.Error(errorMsg);
        throw new XPException(errorMsg);
      }
      APTExtClassTypeDesc realType = (APTExtClassTypeDesc) baseType;
      return realType.GetASNameForEditor(_StrProvEnglishName);
    }
    
    /// <summary>
    /// Returns the localized name for a particular PT class.
    /// </summary>
    /// <param name="_nClassID">The ClassID for this Plumtree Object.</param>
    /// <returns>The localized name for a particular PT class.</returns>
    public virtual String GetClassTypeLocalizedName(int _nClassID, String _strLangID)
    {
      return ((APTClassTypeDesc) GetClassTypeDescItem(XPConvert.ToString(_nClassID))).GetClassTypeLocalizedName(_strLangID);
      }
    
    /// <summary>
    /// Returns the localized name for a particular PT class.
    /// </summary>
    /// <param name="_nClassID">The ClassID for this Plumtree Object.</param>
    /// <param name="_sProvEnglishName">The English Provider Name.</param>
    /// <returns>The localized name for a particular PT class.</returns>
    public virtual String GetClassTypeLocalizedName(int _nClassID, String _strLangID,
     String _sProvEnglishName)
    {
      IProvInfo ptProvInfo = (IProvInfo) m_mapProvInfoByEnglishName.GetElement(_sProvEnglishName);
      return ptProvInfo.GetProviderAndClassLocalizedName(_strLangID, _nClassID);
      }
    public int[] GetOrderedListOfClassIDs(String _strLangID)
    {
      return (int[]) m_mapClassIDArraysByLangID.GetElement(_strLangID);
    }
    public virtual int[] GetRelatedClassTypeIDs(int _nClassID)
    {
      return ((APTClassTypeDesc) GetClassTypeDescItem(XPConvert.ToString(_nClassID))).GetRelatedClassTypeIDs();
      }
    public virtual int[] GetDependentClassTypeIDs(int _nClassID)
    {
      return ((APTClassTypeDesc) GetClassTypeDescItem(XPConvert.ToString(_nClassID))).GetDependentClassTypeIDs();
      }
    public virtual bool HasGlobalClassTypeMap(int _nClassID)
    {
      return ((APTClassTypeDesc) GetClassTypeDescItem(XPConvert.ToString(_nClassID))).HasGlobalClassTypeMap();
      }
    public virtual bool HasCreateActivityRight(int _nClassID, IPTSession _ptSession)
    {
      return ((APTClassTypeDesc) GetClassTypeDescItem(XPConvert.ToString(_nClassID))).HasCreateActivityRight(_ptSession);
      }
    public virtual IPTClassTypeDesc[] GetLockManagerClassTypes(String _strLangID)
    {
      XPArrayList arrayList = new XPArrayList();
      for (int i = 0;i < m_aIClassTypes.Length;i++)
      {
        if ((m_aIClassTypes[i] is IPTClassTypeDesc) && ((IPTClassTypeDesc) (m_aIClassTypes[i])).HasObjectLockManager())
        {
          arrayList.Add(m_aIClassTypes[i]);
        }
      }
      XPCollections.Sort(arrayList, new LockManagerObjectComparator(_strLangID));
      return (IPTClassTypeDesc[]) (arrayList.ToArray(new IPTClassTypeDesc[]{}));
      }
    
    /// <summary>
    /// Creates a Redirect object by retrieving the correct class type descriptor to
    /// parse the friendly URL.
    /// </summary>
    /// <param name="_fURLParser">The friendly URL parser containing URL path</param>
    /// <param name="_asParent">The parent activity space.</param>
    /// <param name="_userSession">The current session.</param>
    /// <returns>Redirect object that tells the portal what page to go to.</returns>
    public virtual Redirect GetFriendlyViewOpenerRedirect(IFriendlyURLParser _fURLParser,
     AActivitySpace _asParent, Object _userSession)
    {
      // Get the class type descriptor
      IPTClassTypeDesc ptCLassTypeDesc = (IPTClassTypeDesc) GetClassTypeDescItem(_fURLParser.GetClassId());
      log.Debug("ClassTypeDescManager:GetFriendlyViewOpenerRedirect Retrieved class type descriptor: " + ptCLassTypeDesc.GetType());
      // Get Redirect object and return
      return ptCLassTypeDesc.GetRedirectViewObject(_fURLParser, _asParent,
       _userSession);
    }
    public virtual String ConstructFriendlyURL(FriendlyURL url, AActivitySpace space)
    {
      // Get the class type descriptor
      IPTClassTypeDesc ptClassTypeDesc = (IPTClassTypeDesc) GetClassTypeDescItem(url.GetClassIdAsString());
      log.Debug("ClassTypeDescManager:ConstructFriendlyURL Retrieved class type descriptor: " + ptClassTypeDesc.GetType());
      // Get Redirect object and return
      return ptClassTypeDesc.ConstructFriendlyURL(url, space);
    }
    public virtual String LookupFriendlyURLKeyFromClassId(String _strClassId)
    {
      // Community pages don't have a separate keyword, they are part of the community friendly URL,
      // so for the purpose of looking up keyword, return the same keyword as communities
      if ((PT_CLASSIDS.PT_PAGE_ID + "").Equals(_strClassId))
      {
        _strClassId = PT_CLASSIDS.PT_COMMUNITY_ID + "";
      }
      if (null == m_htLookupTable)
      {
        return null;
      }
      String strKeyword = (String) m_htLookupTable.GetElement(_strClassId);
      // DCA - we're encoding the keyword in case it has non-ASCII chars
      if (null != strKeyword)
      {
        strKeyword = XPHttpUtility.UrlEncode(strKeyword, LinkConstants.UTF_8_ENCODING);
        }
      return strKeyword;
    }
    public virtual void SetFriendlyURLLookupTable(XPHashtable htLookupTable)
    {
      m_htLookupTable = htLookupTable;
    }
  }
}