using System;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.portaluiinfrastructure.compoundlist
{
  /// <summary>
  /// List to store different types of URLs, primarily URLs to various
  /// portal areas.
  /// </summary>
  /// <author>robertz</author>
  public class ASCompoundList : ICPListEntryIterator
  {
    
    /// <summary>
    /// Constant values of the different types of URLs the list stores.
    /// </summary>
    public const int ENTRYTYPE_STRINGLABEL = 100;
    public const int ENTRYTYPE_FULLURL = 101;
    public const int ENTRYTYPE_COMMPTEMPL = 102;
    public const int ENTRYTYPE_MYPAGETEMPL = 103;
    public const int ENTRYTYPE_DIRTEMPL = 104;
    public const int ENTRYTYPE_EDITORTEMPL = 105;
    public const int ENTRYTYPE_PREFSTEMPL = 106;
    public const int ENTRYTYPE_GENERICTEMPL = 107;
    public const int ENTRYTYPE_DOCTEMPL = 108;
    public const int ENTRYTYPE_USERTEMPL = 109;
    public const int ENTRYTYPE_SITEMAPROOTTEMPL = 110;
    public const int ENTRYTYPE_FULLINTERNALURL = 111;
    
    /// <summary>
    /// Constant values of the different types of getting
    /// the Activity Space.
    /// </summary>
    public const int NEWASTYPE_CREATENEW = 0;
    public const int NEWASTYPE_GETCACHED = 1;
    public const int NEWASTYPE_GETIFCACHED = 2;
    private XPArrayList m_list;
    /* Iterator variables */
    private int m_curListItemIndex;
    private int m_nTotalCount;
    
    /// <summary>
    /// Creates a new ASCompoundList object.
    /// </summary>
    public ASCompoundList()
    {
      m_nTotalCount = 0;
      m_list = new XPArrayList();
    }
    
    /// <summary>
    /// Adds a Community page URL
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="nCommID">Community ID</param>
    /// <param name="nPageID">Community Page ID</param>
    public virtual void AddCommPageURL(String label, int nCommID, int nPageID)
    {
      if (nPageID < 0)
      {
        nPageID = 0;
      }
      AddCommPageURL(label, nCommID, nPageID, "", -1);
    }
    
    /// <summary>
    /// Adds a User Profile Community Page URL 
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="nCommID">Community ID</param>
    /// <param name="nPageID">Community Page ID</param>
    /// <param name="strDescription">Description for the user profile</param>
    /// <param name="nProfileID">Profile User ID</param>
    public virtual void AddCommPageURL(String label, int nCommID, int nPageID,
     String strDescription, int nProfileID)
    {
      ListEntry entry = new ListEntry(ENTRYTYPE_COMMPTEMPL);
      entry.SetLabel(label);
      entry.SetCommunityID(nCommID);
      entry.SetDescription(strDescription);
      entry.SetPageID(nPageID);
      entry.SetProfileID(nProfileID);
      m_list.Add(entry);
      m_nTotalCount++;
    }
    
    /// <summary>
    /// Adds a Community Preview URL
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="nCommID">Community ID</param>
    /// <param name="nPageID">Communiyt Page ID</param>
    /// <param name="strDescription">Description for the Community</param>
    public virtual void AddCommPreviewPageURL(String label, int nCommID, int nPageID,
     String strDescription)
    {
      ListEntry entry = new ListEntry(ENTRYTYPE_COMMPTEMPL);
      String[] saParams = new String[1];
      String[] values = new String[1];
      entry.SetLabel(label);
      entry.SetCommunityID(nCommID);
      entry.SetPageID(nPageID);
      entry.SetDescription(strDescription);
      saParams[0] = "preview";
      values[0] = "y";
      entry.SetExtraParams(saParams, values);
      m_list.Add(entry);
      m_nTotalCount++;
    }
    
    /// <summary>
    /// Adds a Community Knowledge Directory Root URL
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="nCommID">Community ID</param>
    /// <param name="nMode">Browse or Edit Mode</param>
    /// <param name="bIsPreview">Is this page in preview mode</param>
    public virtual void AddSiteMapRootURL(String label, int nCommID, int nMode,
     bool bIsPreview)
    {
      ListEntry entry = new ListEntry(ENTRYTYPE_SITEMAPROOTTEMPL);
      entry.SetLabel(label);
      entry.SetCommunityID(nCommID);
      entry.SetMode(nMode);
      entry.SetIsPreview(bIsPreview);
      m_list.Add(entry);
      m_nTotalCount++;
    }
    
    /// <summary>
    /// Adds the content of another ASCompoundList
    /// </summary>
    /// <param name="list">ASCompoundList, does nothing if null</param>
    public virtual void AddCompoundList(ICPListEntryIterator list)
    {
      if (null != list)
      {
        list.GetIterator();
        while (list.Next())
        {
          m_list.Add(list.GetEntry());
          m_nTotalCount++;
        }
      }
    }
    
    /// <summary>
    /// Adds a Directory URL
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="nMode">Browse or Edit Mode</param>
    /// <param name="nFolderID">ID of Folder to display</param>
    public virtual void AddDirectoryURL(String label, int nMode, int nFolderID)
    {
      AddDirectoryURL(label, nMode, nFolderID, "");
    }
    
    /// <summary>
    /// Adds a Directory URL with description
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="nMode">Browse or Edit Mode</param>
    /// <param name="nFolderID">ID of Folder to display</param>
    /// <param name="strDescription">Description of folder</param>
    public virtual void AddDirectoryURL(String label, int nMode, int nFolderID,
     String strDescription)
    {
      ListEntry entry = new ListEntry(ENTRYTYPE_DIRTEMPL);
      entry.SetLabel(label);
      entry.SetMode(nMode);
      entry.SetFolderID(nFolderID);
      entry.SetDescription(strDescription);
      m_list.Add(entry);
      m_nTotalCount++;
    }
    
    /// <summary>
    /// Adds an URL to a directory document
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="nDocID">ID of document</param>
    public virtual void AddDocumentURL(String label, int nDocID)
    {
      AddDocumentURL(label, nDocID, "");
    }
    
    /// <summary>
    /// Adds an URL to a directory document with description
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="nDocID">ID of document</param>
    /// <param name="strDescription">Document description</param>
    public virtual void AddDocumentURL(String label, int nDocID, String strDescription)
    {
      ListEntry entry = new ListEntry(ENTRYTYPE_DOCTEMPL);
      entry.SetLabel(label);
      entry.SetPageID(nDocID);
      entry.SetDescription(strDescription);
      m_list.Add(entry);
      m_nTotalCount++;
    }
    
    /// <summary>
    /// Adds an URL to an editor
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="editor">Editor Activityspace ID</param>
    /// <param name="control">Editor Control ID</param>
    /// <param name="nEditorType">Editor type constant</param>
    public virtual void AddEditorPageURL(String label, String editor, String control,
     int nEditorType)
    {
      ListEntry entry = new ListEntry(ENTRYTYPE_EDITORTEMPL);
      entry.SetLabel(label);
      entry.SetControl(control);
      entry.SetEditorAS(editor);
      entry.SetEditorType(nEditorType);
      m_list.Add(entry);
      m_nTotalCount++;
    }
    
    /// <summary>
    /// Adds an URL to an editor with additional parameters.
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="editor">Editor Activityspace ID</param>
    /// <param name="control">Editor Control ID</param>
    /// <param name="page">Editor Page ID</param>
    /// <param name="paramNames">
    /// String array of parameter names, each name element must
    /// have a corresponding value element in paramValues
    /// </param>
    /// <param name="paramValues">
    /// String array of parameter values, each value element must
    /// have a corresponding name element in paramNames
    /// </param>
    public virtual void AddEditorPageURL(String label, String editor, String control,
     String page, String[] paramNames, String[] paramValues)
    {
      ListEntry entry = new ListEntry(ENTRYTYPE_EDITORTEMPL);
      entry.SetLabel(label);
      entry.SetEditorAS(editor);
      entry.SetControl(control);
      entry.SetPage(page);
      entry.SetExtraParams(paramNames, paramValues);
      m_list.Add(entry);
      m_nTotalCount++;
    }
    
    /// <summary>
    /// Adds a full URL for a portal page
    /// Use this URL type instead of AddFullURL for portal pages since ENTRYTYPE_FULLURL 
    /// URL type will open in a popup if displayoptions specify external URL to open in a pop.  
    /// </summary>
    /// <param name="label"></param>
    /// <param name="url"></param>
    public virtual void AddInternalFullURL(String label, String url)
    {
      ListEntry entry = new ListEntry(ENTRYTYPE_FULLINTERNALURL);
      entry.SetLabel(label);
      entry.SetFullURL(url);
      m_list.Add(entry);
      m_nTotalCount++;
    }
    
    /// <summary>
    /// Adds an external URL
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="url">The full url to an external site (including http://)</param>
    public virtual void AddFullURL(String label, String url)
    {
      AddFullURL(label, url, "");
    }
    
    /// <summary>
    /// Adds an external URL with description
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="url">The full url to an external site (including http://)</param>
    /// <param name="strDescription">Description of external URL</param>
    public virtual void AddFullURL(String label, String url, String strDescription)
    {
      ListEntry entry = new ListEntry(ENTRYTYPE_FULLURL);
      entry.SetLabel(label);
      entry.SetFullURL(url);
      entry.SetDescription(strDescription);
      m_list.Add(entry);
      m_nTotalCount++;
    }
    
    /// <summary>
    /// Adds a generic portal URL
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="strAS">Activityspace ID</param>
    /// <param name="paramNames">
    /// String array of parameter names, each name element must
    /// have a corresponding value element in paramValues
    /// </param>
    /// <param name="paramValues">
    /// String array of parameter values, each value element must
    /// have a corresponding name element in paramNames
    /// </param>
    /// <param name="nNewASType">How to retrieve the activityspace, use NEWSASTYPE_ constant</param>
    public virtual void AddGenericURL(String label, String strAS, String[] paramNames,
     String[] paramValues, int nNewASType)
    {
      ListEntry entry = new ListEntry(ENTRYTYPE_GENERICTEMPL);
      entry.SetLabel(label);
      entry.SetEditorAS(strAS);
      entry.SetExtraParams(paramNames, paramValues);
      entry.SetNewASType(nNewASType);
      m_list.Add(entry);
      m_nTotalCount++;
    }
    
    /// <summary>
    /// Adds just a label
    /// </summary>
    /// <param name="label">String to display</param>
    public virtual void AddLabel(String label)
    {
      ListEntry entry = new ListEntry(ENTRYTYPE_STRINGLABEL);
      entry.SetLabel(label);
      m_list.Add(entry);
      m_nTotalCount++;
    }
    
    /// <summary>
    /// Adds a Mypage URL
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="nPageID">Mypage ID</param>
    public virtual void AddMyPageURL(String label, int nPageID)
    {
      ListEntry entry = new ListEntry(ENTRYTYPE_MYPAGETEMPL);
      entry.SetLabel(label);
      entry.SetPageID(nPageID);
      m_list.Add(entry);
      m_nTotalCount++;
    }
    
    /// <summary>
    /// Adds a User Preference Page URL 
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="strPrefsPage">Preference Page ID</param>
    public virtual void AddPreferencesURL(String label, String strPrefsPage)
    {
      ListEntry entry = new ListEntry(ENTRYTYPE_PREFSTEMPL);
      entry.SetLabel(label);
      entry.SetPage(strPrefsPage);
      m_list.Add(entry);
      m_nTotalCount++;
    }
    
    /// <summary>
    /// Adds a User Profile URL
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="nUserID">The ID of the user</param>
    public virtual void AddUserProfileURL(String label, int nUserID)
    {
      AddUserProfileURL(label, nUserID, "");
    }
    
    /// <summary>
    /// Adds a User Profile URL with description
    /// </summary>
    /// <param name="label">String to display</param>
    /// <param name="nUserID">The ID of the user</param>
    /// <param name="strDescription">Description of the user</param>
    public virtual void AddUserProfileURL(String label, int nUserID, String strDescription)
    {
      ListEntry entry = new ListEntry(ENTRYTYPE_USERTEMPL);
      entry.SetLabel(label);
      entry.SetPageID(nUserID);
      entry.SetDescription(strDescription);
      m_list.Add(entry);
      m_nTotalCount++;
    }
    
    /// <summary>
    /// Gets the current entry
    /// ResetIterator and GetNextEntry will get the
    /// first entry in the list.
    /// </summary>
    /// <returns>					The current entry in the list</returns>
    public virtual ListEntry GetEntry()
    {
      return (ListEntry) m_list.GetElement(m_curListItemIndex);
    }
    
    /// <summary>
    /// Gets an entry at a specific entry
    /// </summary>
    /// <param name="nIndex">Index of the entry</param>
    /// <returns>					Entry at the specified entry</returns>
    public virtual ListEntry GetEntryAtIndex(int nIndex)
    {
      return (ListEntry) m_list.GetElement(nIndex);
    }
    
    /// <summary>
    /// Reset the Iterator
    /// </summary>
    public virtual void GetIterator()
    {
      m_curListItemIndex = -1;
    }
    
    /// <summary>
    /// Get the number of entries in the list
    /// </summary>
    /// <returns>					Count of entries in the list</returns>
    public virtual int GetTotalCount()
    {
      return m_nTotalCount;
    }
    
    /// <summary>
    /// Advances the iterator to next entry
    /// </summary>
    /// <returns>					False if no more exist, otherwise true</returns>
    public virtual bool Next()
    {
      // TODO CR DONH: Why doesn't this behave the same as normal Java &
      // C# iterators?  Normally you say while movenext, getcurrent, but
      // here that will miss the first item.  Why is this non-standard?
      m_curListItemIndex++;
      return (m_curListItemIndex < (m_list.GetSize()));
    }
    // bjf -- need a version that doesn't require passing in icon height & width, too hard
    // to keep it in sync with what is hard-coded in html constructs.
    
    /// <summary>
    /// Assigns an image to the entry last added to the list
    /// </summary>
    /// <param name="image">Image construct for the image to add</param>
    /// <param name="strImageAltText">Alt String for the image</param>
    public virtual void SetImageForLastAddedURL(PTImageType image, String strImageAltText)
    {
      ListEntry entry = (ListEntry) m_list.GetElement(m_nTotalCount - 1);
      entry.SetImage(image);
      entry.SetImageAltText(strImageAltText);
    }
    
    /// <summary>
    /// Assigns a two part image to the entry last added to the list
    /// </summary>
    /// <param name="image">Image construct for the first image</param>
    /// <param name="image2">Image construct for the second image</param>
    /// <param name="strImageAltText">Alt string for both images</param>
    public virtual void SetImageForLastAddedURL(PTImageType image, PTImageType image2,
     String strImageAltText)
    {
      ListEntry entry = (ListEntry) m_list.GetElement(m_nTotalCount - 1);
      entry.SetImage(image);
      entry.SetImage2(image2);
      entry.SetImageAltText(strImageAltText);
    }
    
    /// <summary>
    /// Not for general use
    /// Only added for converting IPTURLs to ListEntry URLs in NavigationModel. 
    /// Only supports icons for Community Actions, My Page actions and Mandatory link entry types
    /// </summary>
    /// <param name="strImagePath"></param>
    /// <param name="strImageAltText"></param>
    public virtual void SetImageForLastAddedURL(String strImagePath, String strImageAltText)
    {
      ListEntry entry = (ListEntry) m_list.GetElement(m_nTotalCount - 1);
      entry.SetImageAltText(strImageAltText);
      // This is a poor way to do this but there is no way to map image names or paths to PTImageTypes 
      // Community Actions
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ACTION_COMMUNITIES_JOIN.GetFileName()))
      {
        entry.SetImage(PTImageType.ACTION_COMMUNITIES_JOIN);
      } else
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ACTION_COMMUNITIES_UNSUB.GetFileName()))
      {
        entry.SetImage(PTImageType.ACTION_COMMUNITIES_UNSUB);
      } else
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ACTION_MYPAGE_NEW.GetFileName()))
      {
        entry.SetImage(PTImageType.ACTION_MYPAGE_NEW);
      } else
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ACTION_COMMUNITIES_EDIT.GetFileName()))
      {
        entry.SetImage(PTImageType.ACTION_COMMUNITIES_EDIT);
      } else
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ACTION_PORTLET_ADD.GetFileName()))
      {
        entry.SetImage(PTImageType.ACTION_PORTLET_ADD);
      } else
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ACTION_PORTLET_PREF.GetFileName()))
      {
        entry.SetImage(PTImageType.ACTION_PORTLET_PREF);
      } else
      // MyPage Actions
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ACTION_MYPAGE_NEW.GetFileName()))
      {
        entry.SetImage(PTImageType.ACTION_MYPAGE_NEW);
      } else
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ICON_EDIT_MENU.GetFileName()))
      {
        entry.SetImage(PTImageType.ICON_EDIT_MENU);
      } else
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ACTION_PORTLET_ADD.GetFileName()))
      {
        entry.SetImage(PTImageType.ACTION_PORTLET_ADD);
      } else
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ACTION_DELETE_MENU.GetFileName()))
      {
        entry.SetImage(PTImageType.ACTION_DELETE_MENU);
      } else
      // Mandatory Links entries
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ICON_URL.GetFileName()))
      {
        entry.SetImage(PTImageType.ICON_URL);
      } else
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ICON_COMMUNITY.GetFileName()))
      {
        entry.SetImage(PTImageType.ICON_COMMUNITY);
      } else
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ICON_USER.GetFileName()))
      {
        entry.SetImage(PTImageType.ICON_USER);
      } else
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ICON_FOLDER.GetFileName()))
      {
        entry.SetImage(PTImageType.ICON_FOLDER);
      } else
      if (XPStringUtility.EndsWith(strImagePath, PTImageType.ICON_DOCUMENT.GetFileName()))
      {
        entry.SetImage(PTImageType.ICON_DOCUMENT);
      }
    }
  }
}