using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.login;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.server.condition;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.interpreter;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portaluiinfrastructure.condition
{
  /// <author>dustina</author>
  public class ConditionTypeManager : IExpressionEvaluator
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.condition.ConditionTypeManager");
    private static readonly OpenLogger ruleDebugger = OpenLogService.GetLogger(OpenLogService.GetComponent("Condition"),
     "com.plumtree.portaluiinfrastructure.condition.ConditionTypeManager");
    
    /// <summary>
    /// Use this class static to access the one and only instance of
    /// ActivitySpaceManager.
    /// </summary>
    private static ConditionTypeManager INSTANCE;
    public const String RULES_DEBUG_MSG = "RULES_DEBUG_MSG";
    
    /// <summary>
    /// Whether or not this object has been initialized 
    /// </summary>
    private bool m_bInitialized = false;
    
    /// <summary>
    /// list of condition types discovered 
    /// </summary>
    private XPHashtable m_htConditionTypes;
    
    /// <summary>
    /// list of condition types discovered 
    /// </summary>
    private XPArrayList m_alConditionTypes;
    public static ConditionTypeManager GetInstance()
    {
      if (null == INSTANCE)
      {
        INSTANCE = new ConditionTypeManager();
      }
      return INSTANCE;
    }
    
    /// <summary>
    /// Private constructor to ensure there is only one instance.
    /// </summary>
    private ConditionTypeManager()
    {
      m_htConditionTypes = new XPHashtable();
      m_alConditionTypes = new XPArrayList();
    }
    
    /// <param name="requestData"></param>
    public virtual void Init(CachingManager cmCachingManager)
    {
      if (m_bInitialized)
      {
        log.Error("ConditionTypeManager Init called twice.");
        return;
      } else
      {
        try
        {
          Object[] arCondtionTypes = (Object[]) cmCachingManager.GetEntry("ConditionTypes");
          if (arCondtionTypes != null)
          {
            AConditionType ct = null;
            for (int x = 0;x < arCondtionTypes.Length;x++)
            {
              ct = (AConditionType) arCondtionTypes[x];
              if (ct != null)
              {
                m_htConditionTypes.PutElement(ct.GetTypeID(), ct);
                m_alConditionTypes.Add(ct);
              }
            }
            if (m_htConditionTypes.GetSize() == 0)
            {
              log.Error("Init: Dynamic Discovery found no Condition Types. The Expression Engine will not work properly.");
              }
            PortalObjectsFactory.GetExpressionEngineInstance().LoadConditionTypes(m_htConditionTypes);
            } else
          {
            log.Error("Init: Dynamic Discovery found no Condition Types. The Expression Engine will not work properly.");
            }
        }
        catch (Exception ex)
        {
          log.Error(ex, "Error in initializing the Condition Manager. The Expression Engine will not work properly.");
          throw XPException.GetInstance(ex);
        }
        m_bInitialized = true;
      }
    }
    
    /// <returns></returns>
    public virtual XPHashtable GetHashedConditionTypes()
    {
      if (m_bInitialized)
      {
        return m_htConditionTypes;
      } else
      {
        log.Error("GetHashedConditionTypes: ConditionTypeManager needs to be initialized first.");
        return null;
      }
    }
    
    /// <returns></returns>
    public virtual XPArrayList GetArrayConditionTypes()
    {
      if (m_bInitialized)
      {
        return m_alConditionTypes;
      } else
      {
        log.Error("GetArrayConditionTypes: ConditionTypeManager needs to be initialized first.");
        return null;
      }
    }
    
    /// <param name="requestData"></param>
    /// <returns></returns>
    public virtual XPHashtable GetUserEnvironment(AActivitySpace asCurrentSpace,
     IApplication application, IXPRequest xpRequest, IPTSession ptUserSession,
     bool bAtGuestPhase)
    {
      if (!m_bInitialized)
      {
        log.Error("GetUserEnvironment: ConditionTypeManager needs to be initialized first.");
        return null;
      }
      XPHashtable htUserEnvironment = new XPHashtable();
      IXPEnumerator condEnumerator = m_alConditionTypes.GetEnumerator();
      while (condEnumerator.MoveNext())
      {
        AConditionType cond = (AConditionType) condEnumerator.GetCurrent();
        try
        {
          if (!bAtGuestPhase)
          {
            IXPRequest currentRequest = xpRequest;
            if (cond.GetPhaseType() == AConditionType.PHASE_TYPE_REGULAR)
            {
              ((ARegularConditionType) cond).GetCurrentValue(asCurrentSpace,
               currentRequest, ptUserSession, htUserEnvironment);
            }
          }
          XPLimitedRequest limitedRequest = new XPLimitedRequest(xpRequest);
          if (cond.GetPhaseType() == AConditionType.PHASE_TYPE_GUEST)
          {
            ((AGuestConditionType) cond).GetCurrentValue(limitedRequest, PortalObjectsFactory.GetGuestReadOnlySession(),
             htUserEnvironment);
          }
        }
        catch (Exception e)
        {
          log.Error("ConditionTypeManager::GetUserEnvironment An error occured while calling GetCurrentValue() on " + cond.GetType().FullName + "The following exception occured: " + e);
          }
      }
      return htUserEnvironment;
    }
    public virtual int EvaluateExpressionForGuestLoginPhase(AActivitySpace asCurrentSpace,
     IApplication application, IXPRequest xpRequest, Object ptUserSession)
    {
      try
      {
        // DCA - we don't want the expression engine to run if we're trying to access the debug output page
        // RulesDebugMSGAS.STR_MVC_CLASS_NAME = "RulesDebugMSGAS"
        if (asCurrentSpace == null || !asCurrentSpace.GetName().Equals("RulesDebugMSGAS"))
        {
          // init debug text
          XPStringBuilder sbDebugText = new XPStringBuilder();
          // DCA TODO - change to adding to session
          xpRequest.SetAttribute(RULES_DEBUG_MSG, sbDebugText);
          int result = EvaluateExpressions(ActionTypeEnum.SUBPORTAL, asCurrentSpace,
           application, xpRequest, (IPTSession) ptUserSession, true, "Guest Login Phase",
           sbDebugText);
          return result;
        } else
        {
          return -1;
        }
      }
      catch (Exception ex)
      {
        log.Error(ex, "The Expression Engine failed to evaluate the expression for the Guest Login Phase.");
        return -1;
      }
    }
    public virtual void EvaluateExpressionForLogInDestination(AActivitySpace asCurrentSpace,
     IApplication application, IXPRequest xpRequest, Object ptUserSession)
    {
      try
      {
        if (null == ptUserSession)
        {
          log.Warn("EvaluateExpressionForLogInDestination: Rules cannot be evaluated without a valid user session.");
          return;
        }
        // DCA - we don't want the expression engine to run if we're trying to access the debug output page
        // RulesDebugMSGAS.STR_MVC_CLASS_NAME = "RulesDebugMSGAS"
        if (asCurrentSpace == null || !asCurrentSpace.GetName().Equals("RulesDebugMSGAS"))
        {
          // init debug text
          XPStringBuilder sbDebugText = (XPStringBuilder) xpRequest.GetAttribute(RULES_DEBUG_MSG);
          if (null == sbDebugText)
          {
            sbDebugText = new XPStringBuilder();
            xpRequest.SetAttribute(RULES_DEBUG_MSG, sbDebugText);
          }
          int result = EvaluateExpressions(ActionTypeEnum.SUBPORTAL, asCurrentSpace,
           application, xpRequest, (IPTSession) ptUserSession, false, "Log In Destination Phase",
           sbDebugText);
          ((IPTSession) ptUserSession).GetSessionInfo().SetOverrideSubportal(result);
          }
      }
      catch (Exception ex)
      {
        log.Error(ex, "The Expression Engine failed to evaluate the expression for the Login Destination.");
        }
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.interpreter.IExpressionEvaluator#EvaluateExpressionForSubportalAction(com.plumtree.uiinfrastructure.activityspace.AActivitySpace, com.plumtree.uiinfrastructure.web.IApplication, com.plumtree.openfoundation.web.IXPRequest, java.lang.Object)
     */
     
    public virtual void EvaluateExpressionForSubportalAction(AActivitySpace asCurrentSpace,
     IApplication application, IXPRequest xpRequest, Object ptUserSession)
    {
      try
      {
        if (null == ptUserSession)
        {
          log.Warn("EvaluateExpressionForSubportalAction: Rules cannot be evaluated without a valid user session.");
          return;
        }
        // DCA - we don't want the expression engine to run if we're trying to access the debug output page
        // RulesDebugMSGAS.STR_MVC_CLASS_NAME = "RulesDebugMSGAS"
        if (asCurrentSpace == null || !asCurrentSpace.GetName().Equals("RulesDebugMSGAS"))
        {
          // init debug text
          XPStringBuilder sbDebugText = (XPStringBuilder) xpRequest.GetAttribute(RULES_DEBUG_MSG);
          if (null == sbDebugText)
          {
            sbDebugText = new XPStringBuilder();
          }
          int result = EvaluateExpressions(ActionTypeEnum.SUBPORTAL, asCurrentSpace,
           application, xpRequest, (IPTSession) ptUserSession, false, "Display Page Phase",
           sbDebugText);
          ((IPTSession) ptUserSession).GetSessionInfo().SetOverrideSubportal(result);
          // check to see if rules debugging is enabled
          IPTServerConfigSettings config = (IPTServerConfigSettings) ((IPTSession) ptUserSession).OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
           false);
          bool bRulesDebugEnabled = false;
          try
          {
            bRulesDebugEnabled = config.GetSettingAsInt(PT_SERVER_CONFIG_SETTINGNAME.PT_SETTINGNAME_ENABLE_DEBUG_SUBPORTAL_RULES_MODE) == 1;
            }
          catch (Exception e)
          {
          }
          // DCA - setting not found in the database; assume debug mode off
          if (bRulesDebugEnabled && asCurrentSpace != null)
          {
            Object ret = PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.ENABLE_RULES_DEBUG_MODE,
             asCurrentSpace);
            bRulesDebugEnabled = (null != ret) && (XPConvert.ToInteger(ret) == 1);
            }
          if (bRulesDebugEnabled && asCurrentSpace != null)
          {
            asCurrentSpace.GetSubSession().SetAttribute(RULES_DEBUG_MSG, sbDebugText.ToString());
            }
        } else
        //xpRequest.GetSession().RemoveAttribute(RULES_DEBUG_MSG);
        {
        }
      }
      //xpRequest.GetSession().RemoveAttribute(RULES_DEBUG_MSG);
      catch (Exception ex)
      {
        log.Error(ex, "The Expression Engine failed to evaluate the expression for a Subportal Action.");
        }
    }
    public virtual int EvaluateExpressions(int nActionType, AActivitySpace asCurrentSpace,
     IApplication application, IXPRequest xpRequest, IPTSession ptUserSession,
     bool bAtGuestLogin, String strDebugPhaseTitle, XPStringBuilder sbDebugText)
    {
      XPDateTime dtStart = new XPDateTime();
      XPHashtable htUserEnvironment = GetUserEnvironment(asCurrentSpace, application,
       xpRequest, ptUserSession, bAtGuestLogin);
      int result = -1;
      sbDebugText.Append("<br><b>").Append(strDebugPhaseTitle).Append(" Begin</b>");
      ruleDebugger.Debug("Experience Rules - " + strDebugPhaseTitle + " Begin");
      result = PortalObjectsFactory.GetExpressionEngineInstance().Execute(nActionType,
       htUserEnvironment, sbDebugText);
      sbDebugText.Append("<b>").Append(strDebugPhaseTitle).Append(" Ends</b><br>");
      ruleDebugger.Debug("Experience Rules - " + strDebugPhaseTitle + " Ends");
      XPDateTime dtEnd = new XPDateTime();
      sbDebugText.Append("<b>Rules Execution Time: " + XPConvert.ToString(dtEnd.GetTimeInMilliSeconds() - dtStart.GetTimeInMilliSeconds()) + " milliseconds</b><br>");
      ruleDebugger.Debug("Experience Rules - Rules Execution Time: " + XPConvert.ToString(dtEnd.GetTimeInMilliSeconds() - dtStart.GetTimeInMilliSeconds()) + " milliseconds");
      return result;
    }
  }
}