using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.statichelpers;
namespace com.plumtree.portaluiinfrastructure.editor
{
  /// <summary>
  /// This is the Model class for Global Objects.
  /// This class contains the Global Object as a member variable.
  /// It initializes it so the Object is ready to be used.
  /// Model classes for Global Objects should extend this class.
  /// </summary>
  /// <author>Julie Fournier</author>
  public abstract class GbObjEditorModel : EditorModel
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.editor.GbObjEditorModel");
    
    /// <summary>
    /// Query String constant param name for Global Object ID 
    /// </summary>
    public const String EDITOR_QS_INT_QS_GLOBAL_OBJECT_ID = "globalObjectID";
    
    /// <summary>
    /// The Plumtree Session. 
    /// </summary>
    protected IPTSession m_Session;
    
    /// <summary>
    /// JF- Server objects
    /// The Global Object being edited.
    /// </summary>
    protected Object m_ptGlobalObject;
    
    /// <summary>
    /// JF- This is to identify the global object. It has to be a PT_GLOBALOBJECTS constant.
    /// </summary>
    protected int m_nGlobalObjectID = -1;
    
    /// <summary>
    /// That method is called by the EditorStartControl.
    /// It creates the global object and puts it in m_ptGlobalObject.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>boolean Is true if the current user has the rights to access this Editor.</returns>
    public override bool CheckEditorAccessAndStart(XPHashtable _htFormData)
    {
      // JF- First we check the user access rights
      if (!m_Session.GetSessionInfo().CheckForActivityRight(PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_ACCESS_UTILITIES))
      {
        UIError myError = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER,
         null, m_asOwner.GetString(429, "ptmsgs_portalinfrastructure"), null);
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        return false;
      }
      // Set global object ID with values from QS.
      SetMemberVariablesFromQS(_htFormData);
      try
      {
        // JF- We open the Global Obejct with the ID provided in the QS.
        m_ptGlobalObject = m_Session.OpenGlobalObject(m_nGlobalObjectID, false);
        }
      catch (Exception e)
      {
        GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_UI_Infrastructure"),
         "GbObjEditorModel::CheckEditorAccessAndStart", e, true, UIErrorLevel.SERIOUS,
         m_asOwner.GetString(152, "ptmsgs_portalinfrastructure"));
      }
      StartMigrationHistory();
      DoTaskOnStartEditor(_htFormData);
      return true;
    }
    
    /// <summary>
    /// Returns the the Global Object being edited.
    /// </summary>
    /// <returns>IPTObject</returns>
    public virtual Object GetGlobalObject()
    {
      return (m_ptGlobalObject);
    }
    
    /// <summary>
    /// Returns the IPTSession for the current user.
    /// </summary>
    /// <returns>IPTSession</returns>
    public virtual IPTSession GetPTSession()
    {
      return (m_Session);
    }
    
    /// <summary>
    /// Inits the Model with the IPTSession.
    /// </summary>
    /// <param name="parent"></param>
    public override void Init(AActivitySpace parent)
    {
      base.Init(parent);
      m_Session = (IPTSession) parent.GetUserSession();
    }
    
    /// <summary>
    /// Sets the global object ID.
    /// </summary>
    /// <param name="IPTObject"></param>
    public virtual void SetGlobalObjectID(int _nGbObjectID)
    {
      m_nGlobalObjectID = _nGbObjectID;
    }
    
    /// <summary>
    /// Helper method for the StartEditor method.
    /// Fill out the member variables with data from QS.
    /// </summary>
    /// <param name="_htFormData"></param>
    public virtual void SetMemberVariablesFromQS(XPHashtable _htFormData)
    {
      // JF- Get the Global Object ID form the QS and 
      // set it in the associated member variable.
      String[] data = (String[]) _htFormData.GetElement(EDITOR_QS_INT_QS_GLOBAL_OBJECT_ID);
      if ((null != data) && (0 != data.Length))
      {
        SetGlobalObjectID(XPConvert.ToInteger(data[0]));
        log.Debug("GbObjEditorModel::SetMemberVariablesFromQS Global Object ID: " + data[0]);
        } else
      {
        log.Warn("GbObjEditorModel::SetMemberVariablesFromQS Global Object ID was not in Query String.");
        }
    }
    
    /// <summary>
    /// Saves the Migration history poge.
    /// </summary>
    protected virtual void StartMigrationHistory()
    {
      ((MigrationHistoryPluginModel) m_asOwner.GetModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME)).InitializeHistory(PT_CLASSIDS.PT_GLOBALOBJECT_ID,
       m_nGlobalObjectID);
    }
  }
}