using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.editor.commonpages.keywords;
using com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames;
using com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory;
using com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties;
using com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity;
using com.plumtree.portaluiinfrastructure.editor.saveobject;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.expandablelist;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.editor
{
  /// <summary>
  /// This is the Activity Space class for the ObjEditor Framework.
  /// Editor Activity Space classes should extend this class.
  /// </summary>
  /// <author>Julie Fournier</author>
  public abstract class ObjEditorAS : PTEditorAS
  {
    
    /// <summary>
    /// MJD- Storage name for Keywords classes
    /// </summary>
    public const String STR_MVC_STORAGE_NAME_KEYWORDS = "ObjectKeywordsObjEditor";
    
    /// <summary>
    /// JF- Storage name for Security classes
    /// </summary>
    public const String STR_MVC_STORAGE_NAME_SECURITY_TAB = "ObjectSecurityObjEditor";
    
    /// <summary>
    /// JF- Storage name for Priority drop-down classes
    /// </summary>
    public const String STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN = "ObjectSecurityPriorityDropDown";
    
    /// <summary>
    /// Javascript function name 
    /// </summary>
    public const String JAVASCRIPT_OBJEDITOR_OPEN = "ObjEditor_OpenEditor";
    
    /// <summary>
    /// Key for the AS to store the save object pop-up as a sub space.
    /// </summary>
    public const String SAVEOBJECT_SUBSPACE_KEY = "saveObjectPopUp";
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.activityspace.AActivitySpace#CheckBasicAccess(String,String,boolean)"></seealso>
    public override bool CheckBasicAccess(String strPage, String strControl,
     bool bSameSpace)
    {
      bool bAccess = base.CheckBasicAccess(strPage, strControl, bSameSpace);
      // JF- The save pop-up is going to redirect to one of the Editor pages 
      // (Security Page or RedirectDP or ObjDependenciesDP).
      // We know it's OK because the save pop-up is a sub AS of this one 
      // and we know the access has been checked already.
      if ((ObjectSecurityDP.STR_MVC_CLASS_NAME.Equals(strPage)) || (SaveObjectRedirectDP.STR_MVC_CLASS_NAME.Equals(strPage)) || (ObjDependenciesDP.STR_MVC_CLASS_NAME.Equals(strPage)))
      {
        bAccess = true;
      }
      return bAccess;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.IFormAS#DisplayCommonJavaScript()"></seealso>
    public override HTMLScriptCollection DisplayCommonJavaScript()
    {
      HTMLScript myScript = new HTMLScript("text/javaScript");
      XPStringBuilder sb = new XPStringBuilder(200);
      // ObjEditor_OpenEditor
      sb.Append("function ").Append(JAVASCRIPT_OBJEDITOR_OPEN).Append("(nClassID, nObjectID, nFolderID, nOpenerMode, sParentName, sParentID, strMiscQSArgs)\n");
      sb.Append("{\n");
      sb.Append("\tvar strURL = \"" + PTConfigHelper.GetApplicationBaseURL(this) + "space=Opener&cached=true&\";\n");
      sb.Append("\tstrURL = strURL + \"parentname=\" + sParentName + \"&\"\n");
      sb.Append("\tstrURL = strURL + \"parentid=\" + sParentID + \"&\"\n");
      sb.Append("\tstrURL = strURL + \"in_hi_ClassID=\" + nClassID + \"&\"\n");
      sb.Append("\tstrURL = strURL + \"in_hi_ObjectID=\" + nObjectID + \"&\"\n");
      sb.Append("\tstrURL = strURL + \"in_hi_FolderID=\" + nFolderID + \"&\"\n");
      sb.Append("\tstrURL = strURL + \"in_hi_OpenerMode=\" + nOpenerMode + \"&\"\n");
      sb.Append("\tif (null != strMiscQSArgs)\n");
      sb.Append("\t{\n");
      sb.Append("\t  strURL = strURL + strMiscQSArgs; \n");
      sb.Append("\t}\n");
      // TODO CR - MJD : document.location was deprecated in Javascript 1.1, use window.location instead		
      sb.Append("\tdocument.location = strURL;\n");
      sb.Append("}\n");
      myScript.AddInnerHTMLString(sb.ToString());
      HTMLScriptCollection myScriptCollection = base.DisplayCommonJavaScript();
      myScriptCollection.AddInnerHTMLElement(myScript);
      return myScriptCollection;
    }
    
    /// <summary>
    /// Returns the class name of the Dependencies Section.
    /// This page will be displayed at the end of the save pop-up.
    /// It will inform the user that the Object was successfully created
    /// and can also include some intructions on what to do next.
    /// Object Editors should override this method if they have their own
    /// implementation of ObjDependenciesView.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetDependenciesViewName()
    {
      return (ObjDependenciesView.STR_MVC_CLASS_NAME);
    }
    
    /// <summary>
    /// Returns the truncated name of the object being created or edited.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetObjectName()
    {
      String sObjectName = ((ObjEditorModel) GetModel(m_strModelName)).GetObjectName();
      sObjectName = GeneralHelpers.TruncateLongString(sObjectName, ObjEditorModel.OBJECT_NAME_TRUNCATE,
       false);
      return sObjectName;
    }
    
    /// <summary>
    /// Returns the class name of the Editor Repost Control.
    /// Editors should also implement that method and return
    /// the name of their own repost  control, unless they
    /// don't have any. In that case they don't need to implement
    /// the method and EditorRepostControl will be used by default.
    /// </summary>
    /// <returns>String</returns>
    public override String GetRepostControlName()
    {
      return (ObjEditorRepostControl.STR_MVC_CLASS_NAME);
    }
    
    /// <summary>
    /// Registers Views, Controls and Models used by the Editor Framework.
    /// Warning: m_myModel must be set before calling Init().
    /// </summary>
    public override void Init()
    {
      base.Init();
      // JF- Register the Repost Control
      RegisterControl(ObjEditorRepostControl.STR_MVC_CLASS_NAME, GetModel(m_strModelName));
      // JF- Register classes for the localize names and descriptions page
      // JF- Note that we don't need a storage name for this page because, nobody will extend those classes.
      // Whereas for the Security page, some people might extend the classes and use the name 
      // of the super class when they register it.
      RegisterModel(LocNamesModel.STR_MVC_CLASS_NAME);
      RegisterView(LocNamesView.STR_MVC_CLASS_NAME, GetModel(LocNamesModel.STR_MVC_CLASS_NAME));
      RegisterView(LocNamesPrimaryNameView.STR_MVC_CLASS_NAME, GetModel(LocNamesModel.STR_MVC_CLASS_NAME));
      RegisterView(LocNamesPopUpView.STR_MVC_CLASS_NAME, GetModel(LocNamesModel.STR_MVC_CLASS_NAME));
      RegisterControl(LocNamesRepostControl.STR_MVC_CLASS_NAME, GetModel(LocNamesModel.STR_MVC_CLASS_NAME));
      // Register Plug-In: Object Properties
      RegisterModel(ObjPropModel.STR_MVC_CLASS_NAME);
      RegisterView(ObjPropEditView.STR_MVC_CLASS_NAME, GetModel(ObjPropModel.STR_MVC_CLASS_NAME));
      RegisterControl(ExpListControl.STR_MVC_CLASS_NAME, GetModel(ObjPropModel.STR_MVC_CLASS_NAME));
      // Register Plug-In: Keywords
      RegisterModel(KeywordsModel.STR_MVC_CLASS_NAME, STR_MVC_STORAGE_NAME_KEYWORDS);
      RegisterView(KeywordsView.STR_MVC_CLASS_NAME, STR_MVC_STORAGE_NAME_KEYWORDS,
       GetModel(STR_MVC_STORAGE_NAME_KEYWORDS));
      RegisterControl(ExpListControl.STR_MVC_CLASS_NAME, STR_MVC_STORAGE_NAME_KEYWORDS,
       GetModel(STR_MVC_STORAGE_NAME_KEYWORDS));
      LocNamesDP dpLocNames = new LocNamesDP();
      dpLocNames.AddToValidationFunction(LocNamesPrimaryNameView.VALIDATION_FUNC_NAME);
      RegisterPage(dpLocNames);
      RegisterPage(new LocNamesPopUpDP());
      // JF- Register classes for the security page
      RegisterModel(ObjectSecurityModel.STR_MVC_CLASS_NAME, STR_MVC_STORAGE_NAME_SECURITY_TAB);
      RegisterView(ObjectSecurityView.STR_MVC_CLASS_NAME, STR_MVC_STORAGE_NAME_SECURITY_TAB,
       GetModel(STR_MVC_STORAGE_NAME_SECURITY_TAB));
      RegisterControl(ExpListControl.STR_MVC_CLASS_NAME, STR_MVC_STORAGE_NAME_SECURITY_TAB,
       GetModel(STR_MVC_STORAGE_NAME_SECURITY_TAB));
      RegisterPage(new ObjectSecurityDP());
      // JF- Register classes for the priority drop-down just below the Security tab
      RegisterModel(PriorityDropDownModel.STR_MVC_CLASS_NAME, STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN);
      RegisterView(PriorityDropDownView.STR_MVC_CLASS_NAME, STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN,
       GetModel(STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN));
      // Register Migration History Plugin
      RegisterModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME);
      RegisterView(MigrationHistoryView.STR_MVC_CLASS_NAME, GetModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME));
      RegisterPage(new MigrationHistoryPluginDP());
      // Register Dependencies class
      RegisterPage(new ObjDependenciesDP());
      RegisterView(ObjDependenciesView.STR_MVC_CLASS_NAME, GetModel(m_strModelName));
      }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.AActivitySpace#SetCurrentPage()"></seealso>
    /// <param name="IDisplayPage">the current Display Page</param>
    public override void SetCurrentPage(IDisplayPage page)
    {
      base.SetCurrentPage(page);
      String sPageName = page.GetName();
      if (MigrationHistoryPluginDP.STR_MVC_CLASS_NAME.Equals(sPageName))
      {
        this.SetHelpTopic("PT_MigrationHistoryandStatus");
      } else
      if (ObjPropBrowseDP.STR_MVC_CLASS_NAME.Equals(sPageName))
      {
        this.SetHelpTopic("PT_PropertiesandNames");
      } else
      if (ObjectSecurityDP.STR_MVC_CLASS_NAME.Equals(sPageName))
      {
        this.SetHelpTopic("PT_Security");
      } else
      if (LocNamesDP.STR_MVC_CLASS_NAME.Equals(sPageName))
      {
        this.SetHelpTopic("PT_PropertiesandNames");
      }
    }
  }
}