using System;
// Plumtree Server Imports
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames;
using com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity;
using com.plumtree.portaluiinfrastructure.editor.saveobject;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.expandablelist;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.editor
{
  /// <summary>
  /// This class has the responsability to call the PerformRepostAction method on
  /// common pages such as the Security page or the Localize Names and
  /// Descriptions page.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class ObjEditorRepostControl : EditorRepostControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.editor.ObjEditorRepostControl");
    
    /// <summary>
    /// Class name for the Activity Space. 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "ObjEditorRepost";
    
    /// <summary>
    /// Redirects the main window when you are done with the pop-up. 
    /// </summary>
    public const int POSTTOSELF_ACTION_REDIRECT_AFTER_SAVEPOPUP = 7;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new ObjEditorRepostControl();
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// For the Object Editor we don't want to store the object yet.  We need to
    /// pop-up the save Object window so the user can select the folder, set
    /// the security and and set the name and description. To do this we add a
    /// JavaScript to the current DP to open the pop-up window.
    /// </summary>
    public override void PerformFinishAction()
    {
      // JF- Go to the save pop-up in create mode, unless the save pop up is disabled
      if (!DisableSavePopUp() && (PAGE_STATUS_VALID == m_nPageStatus) && (EditorStartControl.EDITOR_START_FLAG_CREATE == ((ObjEditorModel) m_model).GetEditorType()))
      {
        base.CheckMandatoryPages();
        if (PAGE_STATUS_VALID == m_nPageStatus)
        {
          OpenSavePopUp();
        }
      } else
      {
        base.PerformFinishAction();
      }
    }
    
    /// <summary>
    /// Override this method to disable the save pop up.
    /// </summary>
    /// <returns>false</returns>
    protected virtual bool DisableSavePopUp()
    {
      // Bug 41946 fix:
      // Put this default class id check here (as some of the Editors for the
      // following class ids reside in prov* library, e.g., provUIWWW)
      int nClassID = ((ObjEditorModel) m_model).GetClassID();
      if ((PT_CLASSIDS.PT_CATALOGCARD_ID == nClassID) || (PT_CLASSIDS.PT_CATALOGFOLDER_ID == nClassID))
      {
        return true;
      } else
      {
        return false;
      }
    }
    
    /// <summary>
    /// Helper nmethod to build the URL to the save pop-up and  add the
    /// Javascript to open it to the curent page.
    /// </summary>
    /// <throws>XPException</throws>
    protected virtual void OpenSavePopUp()
    {
      HTMLScript myScript = new HTMLScript("text/javaScript");
      ASURL saveObjectPopUp = new ASURL(m_asOwner);
      XPStringBuilder sb;
      // JF- Build the URL to the open the save pop-up	
      if (m_asOwner.GetSubSpaceExists(ObjEditorAS.SAVEOBJECT_SUBSPACE_KEY))
      {
        saveObjectPopUp.SetLinkGetCachedSpace(m_asOwner.GetSubSpace(ObjEditorAS.SAVEOBJECT_SUBSPACE_KEY));
        } else
      {
        saveObjectPopUp.SetLinkCreateNewSpace(SaveObjectAS.STR_MVC_CLASS_NAME,
         m_asOwner);
        saveObjectPopUp.SetSubSpaceKey(ObjEditorAS.SAVEOBJECT_SUBSPACE_KEY);
        saveObjectPopUp.SetControl(SaveObjectStartControl.STR_MVC_CLASS_NAME);
        }
      // JF- Add a javascript to the page to open a pop-up going to that URL
      try
      {
        sb = new XPStringBuilder(200);
        sb.Append("var winSaveObject = window.open(\'").Append(saveObjectPopUp.GetURLAsString()).Append("\', \'SaveObject\', \'scrollbars=yes,location=no,menubar=no,height=575,width=600\');\n");
        myScript.AddInnerHTMLString(sb.ToString());
        EditorAS myEditorAS = (EditorAS) m_asOwner;
        IDisplayPage myTempDP = null;
        EditorDP myEditorDP = null;
        // JF- Get the current page
        myTempDP = myEditorAS.GetCurrentPage();
        // JF- Make sure this is a valid Editor page 
        // (and not the Security page because in create mode that page is in the save pop-up)
        if ((myTempDP is EditorDP) && !ObjectSecurityDP.STR_MVC_CLASS_NAME.Equals(myTempDP.GetName()))
        {
          myEditorDP = (EditorDP) myTempDP;
        } else
        {
          // JF- If the current page is not valid, try with the previous page
          myEditorDP = (EditorDP) myEditorAS.GetPreviousPage();
        }
        // JF- Add the script that opens the save pop-up to the current page
        myEditorDP.AddScriptToHead(myScript);
      }
      catch (HTMLException he)
      {
        log.Error(he, "ObjEditorRepostControl::PerformFinishAction An error occured" + " when trying to generate the JavaScript for the save object pop-up window.");
        throw new XPException(m_asOwner.GetString(159, "ptmsgs_portalinfrastructure"),
         he);
      }
    }
    
    /// <summary>
    /// Switch between the action modes and call  the associated method for each
    /// action.
    /// </summary>
    /// <param name="_nAction"></param>
    protected override void PerformAction(int _nAction)
    {
      base.PerformAction(_nAction);
      // JF- If the Editor is read-only we need to raise the warning saying that the changes will not be saved on each repost.
      // (Otherwise it gets deleted after the 1st repost)
      if ((POSTTOSELF_ACTION_FINISH != _nAction) && (POSTTOSELF_ACTION_CANCEL != _nAction) && ((ObjEditorModel) m_model).GetIsEditorReadOnly())
      {
        ((ObjEditorModel) m_model).SetEditorToReadOnly();
      }
      if (POSTTOSELF_ACTION_REDIRECT_AFTER_SAVEPOPUP == _nAction)
      {
        if (m_asOwner.IsSubSpace())
        {
          SetRedirectToCloseSubEditor();
        } else
        {
          // JF- Redirect (default implementation of GetRedirectAfterFinish 
          // redirects to the previous AS).
          Redirect myRedirect = m_model.GetRedirectAfterFinish();
          // JF- In mode 1, we need an HTTP redirect because the Editor might not have 
          // the same security as the AS where it's redirecting to.
          if (ConfigVarPack.SECURITY_MODE_SSL_CRITICAL_AS_ONLY == ConfigHelper.GetSecurityMode())
          {
            myRedirect.SetIsHTTPRedirect(true);
          }
          bool bCloseOnDone = false;
          try
          {
            bCloseOnDone = ((EditorModel) m_model).GetCloseOnDone();
          }
          catch (Exception e)
          {
            log.Info("Unable to retrieve the CloseOnCancel flag from the EditorModel.");
            }
          if (XPStringUtility.EqualsIgnoreCase("Gateway", myRedirect.GetSpaceName()) || bCloseOnDone)
          {
            // JF- If the parent space was the Gateway then just close the window
            myRedirect = new Redirect();
            myRedirect.SetLinkGetCachedSpace(m_asOwner);
            myRedirect.SetDisplayPage(ClosePopUpDP.STR_MVC_CLASS_NAME);
          }
          SetRedirect(myRedirect);
        }
      }
      ((LocNamesRepostControl) m_asOwner.GetControl(LocNamesRepostControl.STR_MVC_CLASS_NAME)).PerformRepostAction(_nAction,
       m_htFormData);
      if (!((ObjEditorModel) m_model).DisableSecurity())
      {
        ((ExpListControl) m_asOwner.GetControl(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB)).PerformRepostAction(_nAction,
         m_htFormData);
      }
      ((ExpListControl) m_asOwner.GetControl(ExpListControl.STR_MVC_CLASS_NAME)).PerformRepostAction(_nAction,
       m_htFormData);
    }
    
    /// <summary>
    /// Get the current page name and call the save method on the model.
    /// </summary>
    protected override void SaveCurrentPage()
    {
      // JF- Do not save the page if this is the redirect after save pop-up 
      // (in this case we are done with the Editor, the object has been stored already). 
      if (POSTTOSELF_ACTION_REDIRECT_AFTER_SAVEPOPUP != m_nAction)
      {
        base.SaveCurrentPage();
      }
    }
  }
}