using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.editor;
namespace com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory
{
  /// <summary>
  /// Model for the migration history of a single PTObject.
  /// </summary>
  /// <author>ArvindS</author>
  public class MigrationHistoryPluginModel : EditorHelperModel,IMigrationHistoryModel,
  IMigrationHistoryModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory.MigrationHistoryPluginModel");
    public const String STR_MVC_CLASS_NAME = "MigrationHistoryPlugin";
    private XPArrayList m_arrList = null;
    private int m_nCurrentMigrationStatus;
    private bool m_bApprovedForMigration;
    public MigrationHistoryPluginModel() : base()
    {
      m_arrList = new XPArrayList();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()"></seealso>
    public override void CleanupTempData()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new MigrationHistoryPluginModel();
    }
    
    /// <returns></returns>
    public virtual int GetCurrentMigrationStatus()
    {
      return m_nCurrentMigrationStatus;
    }
    
    /// <param name="_nIndex"></param>
    /// <returns></returns>
    public virtual MigrationHistoryItem GetHistoryItem(int _nIndex)
    {
      return (MigrationHistoryItem) m_arrList.GetElement(_nIndex);
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.activityspace.IModelRO#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <returns></returns>
    public virtual int GetNumHistoryItems()
    {
      return m_arrList.GetSize();
    }
    
    /// <summary>
    /// Implements some actions to perform on start.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>boolean Is true if the current user has the rights to access this Editor.</returns>
    public virtual bool InitializeHistory(int nClassID, int nObjectID)
    {
      try
      {
        IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
        IPTMigrationManager ptMM = (IPTMigrationManager) ptSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_MIGRATION_MANAGER,
         false);
        m_nCurrentMigrationStatus = ptMM.QueryMigrationStatus(nClassID, nObjectID);
        IPTQueryResult ptQRHistory = ptMM.QueryMigrationHistory(nClassID,
         nObjectID);
        int nCount = ptQRHistory.RowCount();
        for (int i = 0;i < nCount;i++)
        {
          String strMigrationComment = ptQRHistory.ItemAsString(i, PT_PROPIDS.PT_PROPID_MIGRATION_COMMENT);
          XPDateTime xpdtMigration = ptQRHistory.ItemAsXPDateTime(i, PT_PROPIDS.PT_PROPID_MIGRATION_DATE);
          String strMigrationSourcePortalUUID = ptQRHistory.ItemAsString(i,
           PT_PROPIDS.PT_PROPID_MIGRATION_SOURCEPORTALUUID);
          MigrationHistoryItem mhi = new MigrationHistoryItem(xpdtMigration,
           strMigrationComment, strMigrationSourcePortalUUID, m_asOwner);
          m_arrList.Add(mhi);
        }
        return true;
      }
      catch (Exception e)
      {
        log.Error(e, "MigrationHistoryPluginModel: Unable to initialize history.");
        return false;
      }
    }
    
    /// <summary>
    /// Retrieves the flag indicating whether or not we are being
    /// hosted by another activity space, or our own.
    /// </summary>
    public virtual bool IsStandalone()
    {
      return false;
    }
    
    /// <summary>
    /// Retrieves the flag indicating whether or not to display
    /// the approval checkbox (currently only for full sysadmins)
    /// </summary>
    public virtual bool ShouldDisplayApproval()
    {
      if (PT_MIGRATION_STATUS_TYPES.PT_MIGRATION_STATUS_APPROVED == m_nCurrentMigrationStatus)
      {
        return false;
      }
      IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
      // Return true if the current user is a system administrator
      return PlumtreeHelpers.CheckCurrentUserIsAdmin(ptSession);
    }
    
    /// <summary>
    /// Helper method to Store the migration status on the object.
    /// </summary>
    public virtual void StoreMigrationStatus(int nClassID, int nObjectID)
    {
      IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
      if (PlumtreeHelpers.CheckCurrentUserIsAdmin(ptSession))
      {
        if (((MigrationHistoryPluginModel) m_asOwner.GetModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME)).IsMigrationCheckboxChecked())
        {
          try
          {
            // open the migration manager
            IPTMigrationManager ptMM = (IPTMigrationManager) ptSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_MIGRATION_MANAGER,
             false);
            // simulate approval process:
            // first set the request
            ptMM.BulkSetMigrationStatus(nClassID, new int[]{nObjectID}, PT_MIGRATION_STATUS_TYPES.PT_MIGRATION_STATUS_APPROVAL_REQUESTED,
             "");
            // then set the migration status to approved for the object
            ptMM.BulkSetMigrationStatus(nClassID, new int[]{nObjectID}, PT_MIGRATION_STATUS_TYPES.PT_MIGRATION_STATUS_APPROVED,
             "");
          }
          catch (Exception e)
          {
            log.Error("MigrationHistoryPluginModel::SaveMigrationStatus Unable to save the migration status." + " The following exception happened: " + e);
            }
        }
      }
    }
    
    /// <summary>
    /// Sets the flag indicating this object has been approved for migration
    /// </summary>
    public virtual void SetApprovedForMigration(bool bApproved)
    {
      m_bApprovedForMigration = bApproved;
    }
    
    /// <summary>
    /// Retrieves the flag indicating whether the checkbox has
    /// been checked in this editor session.
    /// </summary>
    public virtual bool IsMigrationCheckboxChecked()
    {
      return m_bApprovedForMigration;
    }
    
    /// <summary>
    /// Calls the right save function according to the page name passed in argument.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>int Page status for server side validation</returns>
    public override int SaveSection(XPHashtable _htFormData)
    {
      if (_htFormData.ContainsKey(MigrationHistoryView.HTMLCHECKBOX_MIGRATION))
      {
        m_bApprovedForMigration = true;
      } else
      {
        m_bApprovedForMigration = false;
      }
      return EditorRepostControl.PAGE_STATUS_VALID;
    }
  }
}