using System;
// Plumtree Server
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.expandablelist.selectptobject;
using com.plumtree.portaluiinfrastructure.expandablelist.selectptobject.selectptobjectsbytype;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tree;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.tree;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity
{
  /// <summary>
  /// This is the implementation of the Expandable List
  /// framework for the the Select PTObject list.
  /// The Select PTObject is a list of objects.
  /// New objects can be added to the list by selecting
  /// them from the associated Object tree.
  /// The list is by default sorted by name ascending.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class ObjectSecurityModel : SelectPTObjectsByTypeModel,IObjectSecurityModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity.ObjectSecurityModel");
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "ObjectSecurity";
    
    /// <summary>
    /// Key name used by the Activity Space to store the read-only groups in groups tree. 
    /// </summary>
    public const String SUBSPACE_KEY_GROUPTREE = "GroupTreeSecurity";
    
    /// <summary>
    /// Key name used by the Activity Space to store the read-only everyone tree. 
    /// </summary>
    public const String SUBSPACE_KEY_EVERYONETREE = "EveryoneTreeSecurity";
    
    /// <summary>
    /// Number total of columns 
    /// </summary>
    public const int EXPLIST_COL_MAX = 9;
    
    /// <summary>
    /// This column is used to store the access level ID of each row.
    /// </summary>
    public const int EXPLIST_COL_ACCESSLEVEL_ID = 4;
    
    /// <summary>
    /// This column is used to store the access level String of each row.
    /// </summary>
    public const int EXPLIST_COL_ACCESSLEVEL_STRING = 5;
    
    /// <summary>
    /// If the row is read only, we won't deisplay the checkbox to delete it.
    /// </summary>
    public const int EXPLIST_COL_ROWACCESS = 6;
    
    /// <summary>
    /// If the Object being created or edited is mandatory for this user or group.
    /// Note: not use for all objects.
    /// Used for both mandatory column index for portlets and communities.
    /// CONSTRAINT: Cannot two of these mandatory cols in the same table for now.
    /// </summary>
    public const int EXPLIST_COL_MANDATORY = 7;
    
    /// <summary>
    /// On each group on the list there is a link to get the tree of groups in the users in that group.
    /// This column is used the sotre the tree AS IDs.
    /// </summary>
    public const int EXPLIST_COL_GROUPTREEASID = 8;
    
    /// <summary>
    /// Mandatory column value: the Object is mandatory for this user of group
    /// </summary>
    public const int MANDATORY_TRUE = 1;
    
    /// <summary>
    /// Mandatory column value: the Object is  NOT mandatory for this user of group
    /// </summary>
    public const int MANDATORY_FALSE = 0;
    
    /// <summary>
    /// Default initial size for the sorted array.
    /// </summary>
    public const int EXPLIST_DEFAULT_INITIAL_SIZE = 50;
    
    /// <summary>
    /// Value for m_nSupportMandatoryColumn: none
    /// </summary>
    public const int MANDCOL_NONE = 0;
    
    /// <summary>
    /// Value for m_nSupportMandatoryColumn: regular
    /// </summary>
    public const int MANDCOL_REGULAR = 1;
    
    /// <summary>
    /// Value for m_nSupportMandatoryColumn: community
    /// </summary>
    public const int MANDCOL_COMMUNITY = 2;
    
    /// <summary>
    /// Read only flag value for the EXPLIST_COL_ROWACCESS column.
    /// Read only means that the user won't be able to delete that row.
    /// </summary>
    public const bool PT_ACCESS_LIST_ENTRY_READONLY = true;
    
    /// <summary>
    /// Read/Write flag value for the EXPLIST_COL_ROWACCESS column.
    /// Read/Write means that the user will be able to delete that row.
    /// </summary>
    public const bool PT_ACCESS_LIST_ENTRY_READWRITE = false;
    
    /// <summary>
    /// Object being created or edited.
    /// We use it on start to prepopulate the list and on finish to store the ACL.
    /// </summary>
    protected IPTObject m_ptObject;
    
    /// <summary>
    /// Current User Session
    /// </summary>
    protected IPTSession m_ptSession;
    
    /// <summary>
    /// String to display on the left navigation menu.
    /// The default value is "Security"
    /// </summary>
    protected String m_sLeftNavString;
    
    /// <summary>
    /// Array of supported Access Levels (constants from PT_ACCESS_LEVELS).
    /// </summary>
    protected int[] m_arrAccessLevels;
    
    /// <summary>
    /// Special case for Crawler Editor where it's possible to Edit the imported cards ACL.
    /// </summary>
    protected bool m_bCardACL = false;
    
    /// <summary>
    /// Special case for ACLs that the current user doesn't have at least select access to.
    /// These ACLs can be removed but not re-added. 
    /// </summary>
    protected XPArrayList m_arrLowAccessObjects;
    
    /// <summary>
    /// You can click on a group only if the current user has at least read access to it. 
    /// </summary>
    protected XPArrayList m_arrNoContentTree;
    
    /// <summary>
    /// We don't want to display the header and the left navigation in
    /// create mode when the Security table is displayed in the save object pop-up.
    /// </summary>
    protected bool m_bShowHeaderAndLeftNav = true;
    
    /// <summary>
    /// Flag to know if we should display the mandatory column for this Object.
    /// </summary>
    protected int m_nSupportMandatoryColumn = 0;
    public ObjectSecurityModel() : base()
    {
      m_nSupportMandatoryColumn = MANDCOL_NONE;
    }
    
    /// <summary>
    /// This method adds the group and users selected in the tree to the list.
    /// It deletes the unselected values and adds the newly selected values.
    /// It doesn't change the values that were already in the list (this way
    /// we don't loose the rights information).
    /// </summary>
    /// <param name="_htFormData">Form data</param>
    public virtual void AddACLFromTree(XPHashtable _htFormData)
    {
      TreeAS asTree;
      asTree = (TreeAS) m_asOwner.GetSubSpace(GetUniqueString(SUBSPACE_KEY_SELECTTREE));
      if (null != asTree)
      {
        ITreeModelRO myTree = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
        XPArrayList mySelected = (XPArrayList) myTree.GetSelectedItems().Clone();
        int nObjectClassID;
        int nObjectID;
        int[] arrRowsToRemove = new int[0];
        bool bRemoveRow;
        TreeLeafNode tLeafNode;
        XPArrayList alRowsToRemove = new XPArrayList();
        // JF- We need to add the newly selected items, and delete the unselected ones
        // Note that we can't clear the list and re add all the values because we also store 
        // the access right for each row. We don't want to loose the values for that column.
        // JF- First we loop through m_saData and for each row we check if the same object 
        // was also selected in the tree.
        for (int i = 0;i < m_saData.GetCount();i++)
        {
          bRemoveRow = true;
          for (int k = 0;k < mySelected.GetSize();k++)
          {
            tLeafNode = (TreeLeafNode) mySelected.GetElement(k);
            nObjectClassID = tLeafNode.m_nClassID;
            nObjectID = tLeafNode.m_nObjectID;
            // JF- if 2 objects have the same Object ID and the same class ID, they are equal.
            if ((nObjectID == XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_OBJECTID))) && (nObjectClassID == XPConvert.ToInteger(m_saData.GetItem(i,
             EXPLIST_COL_CLASSID))))
            {
              // JF- This row was also selected in the tree, we can remove it from our local 
              // copy of the tree selected items, we already have it in m_saData
              mySelected.RemoveAt(k);
              bRemoveRow = false;
              break;
            }
          }
          if (bRemoveRow)
          {
            // JF- Just make sure it was not a read-only row
            if (!XPConvert.ToBoolean(m_saData.GetItem(i, EXPLIST_COL_ROWACCESS)) && !m_arrLowAccessObjects.Contains(m_saData.GetItem(i,
             EXPLIST_COL_CLASSID) + "" + m_saData.GetItem(i, EXPLIST_COL_OBJECTID)))
            {
              // JF- Then we add the index of the row to delete						
              alRowsToRemove.Add(i);
            }
          }
        }
        if (null != arrRowsToRemove)
        {
          arrRowsToRemove = new int[alRowsToRemove.GetSize()];
          for (int k = 0;k < alRowsToRemove.GetSize();k++)
          {
            arrRowsToRemove[k] = XPConvert.ToInteger(alRowsToRemove.GetElement(k));
            }
          // JF- We remove all the rows that have been unselected from the tree
          m_saData.RemoveRows(arrRowsToRemove);
        }
        // JF- Finally in mySelected we only have the newly selected objects. We add them to m_saData.
        for (int i = 0;i < mySelected.GetSize();i++)
        {
          tLeafNode = (TreeLeafNode) mySelected.GetElement(i);
          AddACLToList(tLeafNode.m_nClassID, tLeafNode.m_nObjectID, tLeafNode.GetName());
          }
        // sorting growablelist after adding individual entries
        // Updated to fix bug #51045:
        base.GetItems().SortArray();
      }
    }
    
    /// <summary>
    /// Helper method that adds a row to the list.
    /// NOTE: Items added to the growable sorted list is not resorted after adding an entry due to
    /// scalability reasons. Resort the growable sort array manually by calling:
    /// super.GetItems().SortArray() 
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_nObjectID"></param>
    /// <param name="_sName"></param>
    protected virtual void AddACLToList(int _nClassID, int _nObjectID, String _sName)
    {
      Object[] oNewRow = new Object[EXPLIST_COL_MAX];
      oNewRow[EXPLIST_SORTEDARRAY_PROPID_INDEX] = m_nNextIndex;
      m_nNextIndex++;
      oNewRow[EXPLIST_COL_CLASSID] = _nClassID;
      oNewRow[EXPLIST_COL_OBJECTID] = _nObjectID;
      oNewRow[EXPLIST_COL_STRING] = _sName;
      // JF- Defaut selected access level: select
      oNewRow[EXPLIST_COL_ACCESSLEVEL_ID] = PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT;
      oNewRow[EXPLIST_COL_ACCESSLEVEL_STRING] = GetAccessLevelString(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT);
      // JF- By default the user has read/write access to the row
      oNewRow[EXPLIST_COL_ROWACCESS] = PT_ACCESS_LIST_ENTRY_READWRITE;
      // JF- By default the object is not mandatory
      oNewRow[EXPLIST_COL_MANDATORY] = MANDATORY_FALSE;
      // JF- By default there is no tree ID
      oNewRow[EXPLIST_COL_GROUPTREEASID] = "";
      // the growablesortedlist is sorted outside this method
      // Updated to fix bug #51045:
      base.GetItems().AddRowWithoutSort(oNewRow);
    }
    
    /// <summary>
    /// Calls the right AddRow method based on the Object type.
    /// </summary>
    /// <param name="_htFormData">Form data</param>
    public override void AddRowsOnRepost(XPHashtable _htFormData)
    {
      SaveTreeID(_htFormData);
      AddACLFromTree(_htFormData);
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new ObjectSecurityModel();
    }
    
    /// <summary>
    /// Saves the security settings into the IPTObject.
    /// </summary>
    public virtual void Finish()
    {
      IPTServerContext ptscObject = (IPTServerContext) m_ptObject;
      int nClassID;
      int nObjectID;
      int nAccessLevel;
      bool bRowAccess;
      int nMandatoryLevel;
      String sObjectName;
      IPTAccessList ptACL;
      if (m_bCardACL && (PT_CLASSIDS.PT_CRAWLER_ID == m_ptObject.GetClassID()))
      {
        ptACL = ((IPTCrawler) m_ptObject).GetCardACL();
      } else
      {
        ptACL = ptscObject.GetACL();
      }
      // JF- First we remove all the users and groups from the Object ACL except the read-only ones.
      // (We loop through the ACL entry list in reverse order because we are removing items from 
      // the list inside the loop.)
      for (int i = ptACL.GetCount() - 1;i >= 0;i--)
      {
        if (PT_ACCESS_LIST_ENTRY_READWRITE == ptACL.GetItem(i).GetReadOnly())
        {
          try
          {
            ptACL.Remove(i);
          }
          catch (Exception e)
          {
            log.Error("ObjectSecurityModel::Finish Error when trying to remove an ACL entry. " + "The following exception was thrown: " + e);
            }
        }
      }
      if (MANDCOL_REGULAR == m_nSupportMandatoryColumn)
      {
        // AGC- Once again, we find that the IPTGadget here might actually be a template (and not 
        // capable of answering the QueryMandatoryGroups call. Fortunately, we are guaranteed that
        // this little bugger is initialized by now - so, we'll explicitly check that we're not a 
        // template before moving on.
        IPTGadget ptPortlet = (IPTGadget) m_ptObject;
        if (PT_GADGET_TYPES.PT_GADGETTYPE_TEMPLATE != ptPortlet.GetGadgetType())
        {
          // JF- We loop through the list of mandatory groups for this gadget and remove the read-write ones.
          IPTQueryResult qrMandatoryGroups = ptPortlet.QueryMandatoryGroups();
          int nGroupID;
          for (int i = 0;i < qrMandatoryGroups.RowCount();i++)
          {
            // JF- Get back the group id for the current row to check the access
            nGroupID = qrMandatoryGroups.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
            if (HasEditAccessToGroup(nGroupID))
            {
              try
              {
                ((IPTGadget) m_ptObject).RemoveMandatoryGroup(nGroupID);
              }
              catch (Exception e)
              {
                log.Error("ObjectSecurityModel::Finish Error when trying to remove the mandatory group with ID: " + nGroupID + ". The following exception was thrown:" + e);
                }
            }
          }
        }
      }
      if (PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID())
      {
        IPTQueryResult qrMandatoryGroups = ((IPTCommunity) m_ptObject).QueryMembers(PT_CLASSIDS.PT_USERGROUP_ID);
        int nGroupID;
        for (int i = 0;i < qrMandatoryGroups.RowCount();i++)
        {
          // JF- Get back the group id for the current row to check the access
          nGroupID = qrMandatoryGroups.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
          if (HasEditAccessToGroup(nGroupID))
          {
            try
            {
              ((IPTCommunity) m_ptObject).RemoveMandatoryGroupMember(nGroupID);
              }
            catch (Exception e)
            {
              log.Error("ObjectSecurityModel::Finish Error when trying to remove the mandatory group with ID: " + nGroupID + ". The following exception was thrown:" + e);
              }
          }
        }
      }
      // JF- Then we go through all the users and groups from the list and we add 
      // them to the Object ACL (except the read only ones.)		
      for (int i = 0;i < m_saData.GetCount();i++)
      {
        bRowAccess = XPConvert.ToBoolean(m_saData.GetItem(i, EXPLIST_COL_ROWACCESS));
        nMandatoryLevel = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_MANDATORY));
        nClassID = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_CLASSID));
        nObjectID = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_OBJECTID));
        nAccessLevel = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_ACCESSLEVEL_ID));
        // JF- The everyone group can be a special case.
        // You always want to be able to change the access level, even when it's read-only (i.e. the user cannot delete it).
        // So if we are in this special case, we need to look for it in the ACL and modify the access level. 	
        if (IsEveryoneGroup(i) && (PT_ACCESS_LIST_ENTRY_READWRITE != bRowAccess))
        {
          for (int k = 0;k < ptACL.GetCount();k++)
          {
            IPTAccessListEntry ptListEntry = ptACL.GetItem(k);
            // JF- Is this the Eveyrone group?
            if ((PT_CLASSIDS.PT_USERGROUP_ID == ptListEntry.GetClassID()) && (PT_INTRINSICS.PT_USERGROUP_EVERYONE == ptListEntry.GetObjectID()))
            {
              // JF- Update the access level
              ptListEntry.SetAccessLevel(nAccessLevel);
              break;
            }
          }
        }
        if (PT_ACCESS_LIST_ENTRY_READWRITE == bRowAccess)
        {
          sObjectName = (String) m_saData.GetItem(i, EXPLIST_COL_STRING);
          try
          {
            ptACL.Add(nClassID, nObjectID, nAccessLevel, sObjectName, -1,
             bRowAccess);
          }
          catch (Exception e)
          {
            log.Error("ObjectSecurityModel::Finish Error when trying to add a new ACL entry. " + "The following exception was thrown:" + e);
            }
        }
        // JF- Then add the mandatory groups if supported
        if (PT_CLASSIDS.PT_USERGROUP_ID == nClassID)
        {
          if (HasEditAccessToGroup(nObjectID))
          {
            if ((MANDCOL_REGULAR == m_nSupportMandatoryColumn) && (MANDATORY_TRUE == nMandatoryLevel))
            {
              // JF- If it's a mandatory group we add it to the gadget.
              try
              {
                ((IPTGadget) m_ptObject).AddMandatoryGroup(nObjectID);
              }
              catch (Exception e)
              {
                log.Error("ObjectSecurityModel::Finish Error when trying to add a new ACL entry. " + "The following exception was thrown:" + e);
                }
            }
            if ((PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID()) && ((PT_COMMUNITY_MEMBERSHIPTYPE.PT_COMMUNITYMEMBER_MANDATORY == nMandatoryLevel) || (PT_COMMUNITY_MEMBERSHIPTYPE.PT_COMMUNITYMEMBER_TAB == nMandatoryLevel)))
            {
              // JF- If it's a mandatory group we add it to the gadget.
              try
              {
                log.Info("Adding group with ID: " + nObjectID + " as a member of type: " + nMandatoryLevel);
                ((IPTCommunity) m_ptObject).AddMandatoryGroupMember(nObjectID,
                 nMandatoryLevel);
              }
              catch (Exception e)
              {
                log.Error("ObjectSecurityModel::Finish Error when adding a new mandatory group member. " + "The following exception was thrown:" + e);
                }
            }
          }
        }
      }
    }
    
    /// <summary>
    /// Returns the String associated with the Access Level ID in argument.
    /// The ID has to be a PT_ACCESS_LEVELS constant.
    /// </summary>
    /// <param name="_nAccessLevelID"></param>
    /// <returns>String</returns>
    public virtual String GetAccessLevelString(int _nAccessLevelID)
    {
      switch (_nAccessLevelID){
      case PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ:
      return m_asOwner.GetString(352, "ptmsgs_portalinfrastructure");
      goto case PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT;
      case PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT:
      return m_asOwner.GetString(353, "ptmsgs_portalinfrastructure");
      goto case PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT;
      case PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT:
      return m_asOwner.GetString(351, "ptmsgs_portalinfrastructure");
      goto case PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_ADMIN;
      case PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_ADMIN:
      return m_asOwner.GetString(350, "ptmsgs_portalinfrastructure");
      break;
      }
      return "";
    }
    
    /// <summary>
    /// Returns the array of supported Access Levels.
    /// Those levels will be printed out on each row in a drop-down list.
    /// </summary>
    /// <returns>int[]</returns>
    public virtual int[] GetAccessLevels()
    {
      return m_arrAccessLevels;
    }
    
    /// <summary>
    /// Returns the String to display on the left navigation menu.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetLeftNavString()
    {
      return m_sLeftNavString;
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Tells whether or not the DP should display the
    /// header and the left navigation.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetShowHeaderAndLeftNav()
    {
      return m_bShowHeaderAndLeftNav;
    }
    
    /// <summary>
    /// Returns the value of m_nSupportMandatoryColumn.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetSupportMandatoryColumn()
    {
      return m_nSupportMandatoryColumn;
    }
    
    /// <summary>
    /// Returns a String of Class IDs and Object IDs of the Objects
    /// to select in the tree.
    /// E.g.: "classID1,ObjID1,classID2,objID2"
    /// This is the format used by the tree.
    /// </summary>
    /// <returns>String</returns>
    public override String GetSelectedItems()
    {
      String sSelectedItems = "";
      int nClassID;
      int nObjectID;
      if (null == m_arrLowAccessObjects)
      {
        m_arrLowAccessObjects = new XPArrayList();
      }
      // JF- We loop through the array of items and for each one, 
      // add the class ID and the object ID to the String.
      for (int i = 0;i < m_saData.GetCount();i++)
      {
        nClassID = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_CLASSID));
        nObjectID = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_OBJECTID));
        if ((!XPConvert.ToBoolean(m_saData.GetItem(i, EXPLIST_COL_ROWACCESS))) && (!m_arrLowAccessObjects.Contains(nClassID + "" + nObjectID)))
        {
          // JS- Add the separator
          if (0 != sSelectedItems.Length)
          {
            sSelectedItems+=",";
          }
          sSelectedItems+=(nClassID + "," + nObjectID);
        }
      }
      return sSelectedItems;
    }
    
    /// <summary>
    /// Helper function that returns true if the current user has edit
    /// access on the group identified by the ID in argument.
    /// </summary>
    /// <param name="_nGroupID"></param>
    /// <returns>boolean</returns>
    public virtual bool HasEditAccessToGroup(int _nGroupID)
    {
      return m_ptSession.GetUserGroups().IsObjectAccessible(_nGroupID, m_ptSession.GetSessionInfo().GetCurrentUserID(),
       PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT);
    }
    
    /// <summary>
    /// Returns a flag to know whether the security has changed compared
    /// to what we have in the DB or no.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool HasSecurityChanged()
    {
      IPTServerContext ptscObject = (IPTServerContext) m_ptObject;
      IPTAccessListEntry ptACLEntry;
      IPTAccessList ptACL = ptscObject.GetACL();
      // JF- Take the ACLs from the Object and make sure each of them is in the list.
      for (int i = 0;i < ptACL.GetCount();i++)
      {
        ptACLEntry = ptACL.GetItem(i);
        if (!IsACLInArray(ptACLEntry.GetClassID(), ptACLEntry.GetObjectID(),
         ptACLEntry.GetAccessLevel()))
        {
          return true;
        }
      }
      int nClassID;
      int nObjectID;
      int nAccessLevel;
      // JF- Take the ACLs from the list and make sure each of them is in the Object
      for (int i = 0;i < m_saData.GetCount();i++)
      {
        nClassID = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_CLASSID));
        nObjectID = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_OBJECTID));
        nAccessLevel = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_ACCESSLEVEL_ID));
        if (!IsRowInObjectACL(nClassID, nObjectID, nAccessLevel))
        {
          return true;
        }
      }
      return false;
    }
    public virtual bool CanPropagateSecurity()
    {
      return ((IPTSession) m_asOwner.GetUserSession()).GetSessionInfo().CheckForActivityRight(PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_PROPAGATE_SECURITY);
      }
    
    /// <summary>
    /// Initializes the expandable list.
    /// </summary>
    /// <param name="_ptObject">Object being created or edited</param>
    /// <param name="_ptSession">User session</param>
    public virtual void InitializeSecurityPage(IPTObject _ptObject, IPTSession _ptSession)
    {
      m_ptObject = _ptObject;
      m_ptSession = _ptSession;
      m_sLeftNavString = m_asOwner.GetString(355, "ptmsgs_portalinfrastructure");
      // JF- Class ID of Objects to display on the tree
      m_arrObjectClassIDs = new int[]{PT_CLASSIDS.PT_USER_ID, PT_CLASSIDS.PT_USERGROUP_ID};
      // JF- We initialize the array of access levels with the contants from PT_ACCESS_LEVELS
      // and the associated Strings. This will be used by the Section to display the drop-down 
      // list on each row.
      InitAccessLevels();
      //InitArrayOfObjectsToDisplayInTree();
      // JF- Special case for Communities: we display the mandatory column
      if (PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID())
      {
        m_nSupportMandatoryColumn = MANDCOL_COMMUNITY;
      }
      // JF- Set the icon to display on each row.
      //SetDefaultRowIcon(PTImageType.ICON_USER);
      // RWW - Set the icon for users
      SetRowIconByClassID(PT_CLASSIDS.PT_USER_ID, PTImageType.ICON_USER);
      // RWW - Set the icon for groups
      SetRowIconByClassID(PT_CLASSIDS.PT_USERGROUP_ID, PTImageType.ICON_GROUP);
      // JF- We initialize the m_bIsReadOnly flag (Only users with "Edit Object ACL" activity rights can edit this page).
      InitReadOnlyAccessFlag(_ptSession);
      if (GetIsCardACL())
      {
        SetTableInstructions(m_asOwner.GetString(359, "ptmsgs_portalinfrastructure"));
        }
      // JF- Set Tree title and sub title.
      SetTreeTitle(m_asOwner.GetString(358, "ptmsgs_portalinfrastructure"));
      SetTreeSubTitle(m_asOwner.GetString(357, "ptmsgs_portalinfrastructure"));
      base.InitializeExpandableList(m_asOwner.GetString(356, "ptmsgs_portalinfrastructure"),
       GetColumnInfos(), PTImageType.ICON_GROUP, m_asOwner.GetString(354,
       "ptmsgs_portalinfrastructure"), 11);
      InitListWithACLFromObject(_ptObject, _ptSession);
    }
    
    /// <summary>
    /// Tells whether the Object ACL identified by the class ID, the object ID
    /// and the Access Level ID in argument is in the list or no.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_nObjectID"></param>
    /// <param name="_nAccessLevel"></param>
    /// <returns>boolean</returns>
    public virtual bool IsACLInArray(int _nClassID, int _nObjectID, int _nAccessLevel)
    {
      for (int i = 0;i < m_saData.GetCount();i++)
      {
        if ((_nClassID == XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_CLASSID))) && (_nObjectID == XPConvert.ToInteger(m_saData.GetItem(i,
         EXPLIST_COL_OBJECTID))) && (_nAccessLevel == XPConvert.ToInteger(m_saData.GetItem(i,
         EXPLIST_COL_ACCESSLEVEL_ID))))
        {
          return true;
        }
      }
      return false;
    }
    
    /// <summary>
    /// Returns true if this group should have a link to view its members. 
    /// </summary>
    /// <param name="_nGroupID"></param>
    /// <returns>boolean</returns>
    public virtual bool DisplayGroupMembersTree(int _nGroupID)
    {
      if ((null != m_arrNoContentTree) && m_arrNoContentTree.Contains(XPConvert.ToString(_nGroupID)))
      {
        return false;
      }
      return true;
    }
    
    /// <summary>
    /// Special case for Crawler Editor where it's possible to Edit the
    /// ACL for cards imported by this crawler.
    /// </summary>
    /// <param name="_bCardACL"></param>
    public virtual void SetCardACL(bool _bCardACL)
    {
      m_bCardACL = _bCardACL;
    }
    
    /// <summary>
    /// Returns the value of m_bCardACL.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetIsCardACL()
    {
      return m_bCardACL;
    }
    
    /// <summary>
    /// Sets the String to display on the left navigation menu.
    /// </summary>
    /// <param name="_sLeftNavString"></param>
    public virtual void SetLeftNavString(String _sLeftNavString)
    {
      m_sLeftNavString = _sLeftNavString;
    }
    
    /// <summary>
    /// Changes the value of the flag m_bShowHeaderAndLeftNav
    /// that decides if the DP should display the header
    /// and the left navigation.
    /// </summary>
    /// <param name="_bShowHeaderAndLeftNav"></param>
    public virtual void SetShowHeaderAndLeftNav(bool _bShowHeaderAndLeftNav)
    {
      m_bShowHeaderAndLeftNav = _bShowHeaderAndLeftNav;
    }
    
    /// <summary>
    /// Sets a new value for m_nSupportMandatoryColumn.
    /// </summary>
    /// <param name="_nSupportMandatoryColumn"></param>
    public virtual void SetSupportMandatoryColumn(int _nSupportMandatoryColumn)
    {
      m_nSupportMandatoryColumn = _nSupportMandatoryColumn;
      // JF- We also need to reinitialize the array of column info 
      // to add or remove the mandatory column.
      m_arrExpListColInfo = GetColumnInfos();
    }
    
    /// <summary>
    /// Helper method to get the sort.
    /// </summary>
    /// <returns>int[]</returns>
    protected override int[] GetArrayOfColumns()
    {
      // JF- Column mapping
      int[] arrColumns;
      arrColumns = new int[]{EXPLIST_SORTEDARRAY_PROPID_INDEX, EXPLIST_COL_CLASSID,
       EXPLIST_COL_OBJECTID, EXPLIST_COL_STRING, EXPLIST_COL_ACCESSLEVEL_ID,
       EXPLIST_COL_ACCESSLEVEL_STRING, EXPLIST_COL_ROWACCESS, EXPLIST_COL_MANDATORY,
       EXPLIST_COL_GROUPTREEASID};
      return arrColumns;
    }
    
    /// <summary>
    /// Helper method to get the array of column info.
    /// </summary>
    /// <returns>Object[][]</returns>
    protected virtual Object[][] GetColumnInfos()
    {
      Object[][] arrExpListColInfo;
      if (m_nSupportMandatoryColumn > 0)
      {
        // JF- Create the the 2D array to give clolumn information to the super class.
        arrExpListColInfo = new Object[3][];
        for (int j2c_index = 0;j2c_index < 3;j2c_index++)
        {
          arrExpListColInfo[j2c_index] = new Object[5];
        }
        // First column: object name
        arrExpListColInfo[0][EXPLIST_COLUMN_ID] = EXPLIST_COL_STRING;
        arrExpListColInfo[0][EXPLIST_COLUMN_TITLE] = m_asOwner.GetString(347,
         "ptmsgs_portalinfrastructure");
        arrExpListColInfo[0][EXPLIST_COLUMN_WIDTH] = "60%";
        arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN] = "1";
        arrExpListColInfo[0][EXPLIST_COLUMN_ENABLESORT] = true;
        // Second column: mandatory
        arrExpListColInfo[1][EXPLIST_COLUMN_ID] = EXPLIST_COL_MANDATORY;
        arrExpListColInfo[1][EXPLIST_COLUMN_TITLE] = m_asOwner.GetString(348,
         "ptmsgs_portalinfrastructure");
        arrExpListColInfo[1][EXPLIST_COLUMN_WIDTH] = "20%";
        arrExpListColInfo[1][EXPLIST_COLUMN_COLSPAN] = "1";
        arrExpListColInfo[1][EXPLIST_COLUMN_ENABLESORT] = true;
        // Third column: rights
        arrExpListColInfo[2][EXPLIST_COLUMN_ID] = EXPLIST_COL_ACCESSLEVEL_STRING;
        arrExpListColInfo[2][EXPLIST_COLUMN_TITLE] = m_asOwner.GetString(349,
         "ptmsgs_portalinfrastructure");
        arrExpListColInfo[2][EXPLIST_COLUMN_WIDTH] = "20%";
        arrExpListColInfo[2][EXPLIST_COLUMN_COLSPAN] = "1";
        arrExpListColInfo[2][EXPLIST_COLUMN_ENABLESORT] = true;
      } else
      {
        // JF- Create the the 2D array to give clolumn information to the super class.
        arrExpListColInfo = new Object[2][];
        for (int j2c_index = 0;j2c_index < 2;j2c_index++)
        {
          arrExpListColInfo[j2c_index] = new Object[5];
        }
        // First column: object name
        arrExpListColInfo[0][EXPLIST_COLUMN_ID] = EXPLIST_COL_STRING;
        arrExpListColInfo[0][EXPLIST_COLUMN_TITLE] = m_asOwner.GetString(347,
         "ptmsgs_portalinfrastructure");
        arrExpListColInfo[0][EXPLIST_COLUMN_WIDTH] = "80%";
        arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN] = "1";
        arrExpListColInfo[0][EXPLIST_COLUMN_ENABLESORT] = true;
        // Second column: rights
        arrExpListColInfo[1][EXPLIST_COLUMN_ID] = EXPLIST_COL_ACCESSLEVEL_STRING;
        arrExpListColInfo[1][EXPLIST_COLUMN_TITLE] = m_asOwner.GetString(349,
         "ptmsgs_portalinfrastructure");
        arrExpListColInfo[1][EXPLIST_COLUMN_WIDTH] = "20%";
        arrExpListColInfo[1][EXPLIST_COLUMN_COLSPAN] = "1";
        arrExpListColInfo[1][EXPLIST_COLUMN_ENABLESORT] = true;
      }
      return arrExpListColInfo;
    }
    
    /// <summary>
    /// Inits the array of supported Access Levels.
    /// </summary>
    protected virtual void InitAccessLevels()
    {
      m_arrAccessLevels = new int[4];
      m_arrAccessLevels[0] = PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ;
      m_arrAccessLevels[1] = PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT;
      m_arrAccessLevels[2] = PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT;
      m_arrAccessLevels[3] = PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_ADMIN;
    }
    
    /// <summary>
    /// Initializes the list with the ACL from the PTObject.
    /// </summary>
    /// <param name="_ptObject">Object being created or edited</param>
    protected virtual void InitListWithACLFromObject(IPTObject _ptObject,
     IPTSession _ptSession)
    {
      IPTServerContext ptscObject = (IPTServerContext) _ptObject;
      IPTAccessListEntry ptACLEntry;
      Object[][] oNewRow;
      IPTAccessList ptACL;
      m_arrLowAccessObjects = new XPArrayList();
      m_arrNoContentTree = new XPArrayList();
      if (m_bCardACL && (PT_CLASSIDS.PT_CRAWLER_ID == m_ptObject.GetClassID()))
      {
        ptACL = ((IPTCrawler) m_ptObject).GetCardACL();
      } else
      {
        ptACL = ptscObject.GetACL();
      }
      // For communities need the list of mandatory members
      XPHashtable htMandMembers = new XPHashtable();
      if (PT_CLASSIDS.PT_COMMUNITY_ID == _ptObject.GetClassID())
      {
        IPTQueryResult qrMandMembers = ((IPTCommunity) _ptObject).QueryMembers(PT_CLASSIDS.PT_USERGROUP_ID);
        for (int i = 0;i < qrMandMembers.RowCount();i++)
        {
          int iObjectID = qrMandMembers.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
          int iMandLevel = qrMandMembers.ItemAsInt(i, PT_PROPIDS.PT_PROPID_MEMBERSHIP_TYPE);
          htMandMembers.PutElement(iObjectID, iMandLevel);
        }
      }
      int nACLClassID;
      int nACLObjectID;
      oNewRow = new Object[ptACL.GetCount()][];
      for (int j2c_index = 0;j2c_index < ptACL.GetCount();j2c_index++)
      {
        oNewRow[j2c_index] = new Object[EXPLIST_COL_MAX];
      }
      for (int i = 0;i < ptACL.GetCount();i++)
      {
        ptACLEntry = ptACL.GetItem(i);
        nACLClassID = ptACLEntry.GetClassID();
        nACLObjectID = ptACLEntry.GetObjectID();
        oNewRow[i][EXPLIST_SORTEDARRAY_PROPID_INDEX] = m_nNextIndex;
        m_nNextIndex++;
        oNewRow[i][EXPLIST_COL_CLASSID] = nACLClassID;
        oNewRow[i][EXPLIST_COL_OBJECTID] = nACLObjectID;
        oNewRow[i][EXPLIST_COL_STRING] = ptACLEntry.GetName();
        oNewRow[i][EXPLIST_COL_ACCESSLEVEL_ID] = ptACLEntry.GetAccessLevel();
        oNewRow[i][EXPLIST_COL_ACCESSLEVEL_STRING] = GetAccessLevelString(ptACLEntry.GetAccessLevel());
        oNewRow[i][EXPLIST_COL_ROWACCESS] = ptACLEntry.GetReadOnly();
        oNewRow[i][EXPLIST_COL_GROUPTREEASID] = "";
        if (PT_ACCESS_LIST_ENTRY_READONLY == ptACLEntry.GetReadOnly())
        {
          // JF- If this is a read only row, we also add it to the array of hidden 
          // objects, this way it won't be displayed in the tree.
          // (Otherwise it would be possible to remove it from the list by unchecking it in the tree)
          int[][] arrReadOnlyObjects = new int[1][];
          for (int j2c_index = 0;j2c_index < 1;j2c_index++)
          {
            arrReadOnlyObjects[j2c_index] = new int[2];
          }
          arrReadOnlyObjects[0][SelectPTObjectModel.READ_ONLY_ROW_CLASSID] = nACLClassID;
          arrReadOnlyObjects[0][SelectPTObjectModel.READ_ONLY_ROW_OBJECTID] = nACLObjectID;
          SetHiddenObjects(arrReadOnlyObjects);
        } else
        {
          if (!HasAccessRights(_ptSession, nACLClassID, nACLObjectID, PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT))
          {
            m_arrLowAccessObjects.Add(nACLClassID + "" + nACLObjectID);
          }
        }
        if (PT_CLASSIDS.PT_GADGET_ID == m_ptObject.GetClassID())
        {
          if ((PT_CLASSIDS.PT_USERGROUP_ID == nACLClassID) && IsMandatoryGroup(nACLObjectID))
          {
            oNewRow[i][EXPLIST_COL_MANDATORY] = MANDATORY_TRUE;
          } else
          {
            oNewRow[i][EXPLIST_COL_MANDATORY] = MANDATORY_FALSE;
          }
        } else
        if ((PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID()) && (PT_CLASSIDS.PT_USERGROUP_ID == nACLClassID))
        {
          Object iMandLevel = htMandMembers.GetElement(ptACLEntry.GetObjectID());
          // For all the groups not in the membership query.
          if (null == iMandLevel)
          {
            iMandLevel = PT_COMMUNITY_MEMBERSHIPTYPE.PT_COMMUNITYMEMBER_FREE;
            }
          oNewRow[i][EXPLIST_COL_MANDATORY] = iMandLevel;
        } else
        {
          // This is a hack, since this column is used for both gadgets and communities they share the same
          // default value, might not work if MANDATORY_FALSE and PT_COMMUNITY_MEMBERSHIPTYPE.PT_COMMUNITYMEMBER_FREE were different.
          oNewRow[i][EXPLIST_COL_MANDATORY] = MANDATORY_FALSE;
        }
        if ((PT_CLASSIDS.PT_USERGROUP_ID == nACLClassID) && !HasAccessRights(_ptSession,
         nACLClassID, nACLObjectID, PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ))
        {
          m_arrNoContentTree.Add(XPConvert.ToString(nACLObjectID));
        }
      }
      base.GetItems().AddRows(oNewRow);
    }
    
    /// <summary>
    /// Helper method to know whether or not the current user has at 
    /// least select access to the object in argument.
    /// </summary>
    /// <param name="_ptSession"></param>
    /// <param name="_nClassID"></param>
    /// <param name="_nObjectID"></param>
    /// <param name="_nAccessLevel"></param>
    /// <returns>boolean</returns>
    protected virtual bool HasAccessRights(IPTSession _ptSession, int _nClassID,
     int _nObjectID, int _nAccessLevel)
    {
      bool bHasAccess = false;
      try
      {
        switch (_nClassID){
        case PT_CLASSIDS.PT_CATALOGFOLDER_ID:
        case PT_CLASSIDS.PT_CATALOGCARD_ID:
        bHasAccess = _ptSession.GetCatalog().IsCatalogObjectAccessible(_nClassID,
         _nObjectID, _nAccessLevel);
        break;
        case PT_CLASSIDS.PT_ADMIN_FOLDER_ID:
        bHasAccess = _ptSession.GetAdminCatalog().IsAdminFolderAccessible(_nObjectID,
         _nAccessLevel);
        break;
        default:
        bHasAccess = _ptSession.GetObjectManagers(_nClassID).IsObjectAccessible(_nObjectID,
         _ptSession.GetSessionInfo().GetCurrentUserID(), _nAccessLevel);
        break;
        }
      }
      catch (Exception e)
      {
        log.Error("Unable to determine the access level on object with class ID " + _nClassID + " and object ID " + _nObjectID);
        }
      return bHasAccess;
    }
    
    /// <summary>
    /// Initializes the m_bIsReadOnly flag to know if the user can edit
    /// the page or not.
    /// Only users with admin access on the object can edit this page.
    /// </summary>
    /// <param name="_ptSession">User session</param>
    protected virtual void InitReadOnlyAccessFlag(IPTSession _ptSession)
    {
      // JIC- To know if the user has read/write access to the security page, 
      // we make sure that the user has admin access on the object. 
      IPTServerContext ptscObject = (IPTServerContext) m_ptObject;
      if (ptscObject.GetAccessLevel() < PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_ADMIN)
      {
        m_bIsReadOnly = true;
      } else
      {
        m_bIsReadOnly = false;
      }
    }
    
    /// <summary>
    /// Returns true if the object on the row in argument corresponds to the Everyone Group.
    /// </summary>
    /// <param name="_nRowIndex"></param>
    /// <returns>boolean</returns>
    public virtual bool IsEveryoneGroup(int _nRowIndex)
    {
      bool bIsEveryoneGroup = false;
      try
      {
        int nClassID = XPConvert.ToInteger(m_saData.GetItem(_nRowIndex, ObjectSecurityModel.EXPLIST_COL_CLASSID));
        int nObjectID = XPConvert.ToInteger(m_saData.GetItem(_nRowIndex, ObjectSecurityModel.EXPLIST_COL_OBJECTID));
        if ((PT_CLASSIDS.PT_USERGROUP_ID == nClassID) && (PT_INTRINSICS.PT_USERGROUP_EVERYONE == nObjectID))
        {
          bIsEveryoneGroup = true;
        }
      }
      catch (Exception e)
      {
        log.Error("Unable to determine if the item on row " + _nRowIndex + "is the Everyone Group or no");
        }
      return bIsEveryoneGroup;
    }
    
    /// <summary>
    /// Saves the access level for the row _nRowIndex.
    /// </summary>
    /// <param name="_saData">List to update the data on row _nRowIndex</param>
    /// <param name="_nRowIndex">Index of the row to save.</param>
    /// <param name="_htFormData">Data posted by the form.</param>
    /// <returns>int Status for server side validation.</returns>
    protected override int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex,
     XPHashtable _htFormData)
    {
      // JF- We need to save the privilege drop-down and the mandatory column if it's supported by the object.
      if (!m_bIsReadOnly)
      {
        // JF- Save the privilege only if the user had rights to Edit this row.
        bool bRowAccess = XPConvert.ToBoolean(_saData.GetItem(_nRowIndex,
         EXPLIST_COL_ROWACCESS));
        // JF- We only save it if the user has write access to that particular row 
        // or if this is the Everyone group (special case)
        if ((PT_ACCESS_LIST_ENTRY_READWRITE == bRowAccess) || IsEveryoneGroup(_nRowIndex))
        {
          // JF- We are building the select input name the same way we did 
          // in ObjectSecurityView when we displayed it.
          String sInputSelectName = GetUniqueString(ObjectSecurityView.HTMLSELECT_ACCESS_LEVEL) + _nRowIndex;
          String[] data = (String[]) _htFormData.GetElement(sInputSelectName);
          if ((data != null) && (0 != data.Length) && (null != data[0]))
          {
            // JF- we set the new value in the access level ID and String columns.
            _saData.SetItem(_nRowIndex, EXPLIST_COL_ACCESSLEVEL_ID, XPConvert.ToInteger(data[0]));
            _saData.SetItem(_nRowIndex, EXPLIST_COL_ACCESSLEVEL_STRING, GetAccessLevelString(XPConvert.ToInteger(data[0])));
            } else
          {
            log.Error("ObjectSecurityModel::SaveRow Error when trying to save the access level for row " + _nRowIndex + ". The access level value posted by the form is: " + data);
            }
        }
        // JF- Save the mandatory column only if the Object supports it
        if (m_nSupportMandatoryColumn > MANDCOL_NONE)
        {
          // JF- Get back the class id for the current row
          int nClassID = XPConvert.ToInteger(_saData.GetItem(_nRowIndex, EXPLIST_COL_CLASSID));
          // Only groups have mandatory settings
          if (PT_CLASSIDS.PT_USERGROUP_ID == nClassID)
          {
            // JF- Get back the group id for the current row to check the access
            int nGroupID = XPConvert.ToInteger(_saData.GetItem(_nRowIndex,
             EXPLIST_COL_OBJECTID));
            if (HasEditAccessToGroup(nGroupID))
            {
              // JF- We are building the select input name the same way we did 
              // in ObjectSecurityView when we displayed it.
              String sInputSelectName = GetUniqueString(ObjectSecurityView.HTMLSELECT_MANDATORY) + _nRowIndex;
              String[] data = (String[]) _htFormData.GetElement(sInputSelectName);
              if ((data != null) && (0 != data.Length) && (null != data[0]))
              {
                // JF- we set the new value in the mandatory column
                _saData.SetItem(_nRowIndex, EXPLIST_COL_MANDATORY, XPConvert.ToInteger(data[0]));
                } else
              {
                log.Error("ObjectSecurityModel::SaveRow Error when trying to save the mandatory setting for row " + _nRowIndex + ". The mandatory value posted by the form is: " + data);
                }
            }
          }
        }
      }
      int nObjectID = XPConvert.ToInteger(_saData.GetItem(_nRowIndex, EXPLIST_COL_OBJECTID));
      String sGroupTreeASID = (String) _saData.GetItem(_nRowIndex, EXPLIST_COL_GROUPTREEASID);
      // JF- We save the Group tree ID only if we don't already have it.
      if (("").Equals(sGroupTreeASID) && _htFormData.ContainsKey(GetUniqueString(ObjectSecurityView.HTMLINPUT_GROUPTREEASID) + nObjectID))
      {
        String[] data = (String[]) _htFormData.GetElement(GetUniqueString(ObjectSecurityView.HTMLINPUT_GROUPTREEASID) + nObjectID);
        if ((null != data) && !(0 == data.Length))
        {
          _saData.SetItem(_nRowIndex, EXPLIST_COL_GROUPTREEASID, data[0]);
          }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Returns true if the Group id passed in argument is mandatory for the
    /// Object being created or editor.
    /// If the current Object is not a Portlet, will return false.
    /// </summary>
    /// <param name="_nGroupID"></param>
    /// <returns>boolean</returns>
    private bool IsMandatoryGroup(int _nGroupID)
    {
      bool bIsMandatory = false;
      if (PT_CLASSIDS.PT_GADGET_ID == m_ptObject.GetClassID())
      {
        // AGC- I've Added a minor change - namely this try - catch block.
        // There are a few reasons:
        // #1 This gadget might be a template - and thus the queryMandatoryGroups will fail.
        // #2 Because this is called before the gadget is initialized, I can't tell whether this
        //    is a template or not (the call to getgadgettype fails before initialization).
        // #3 This call is made in StartEditor instead of DoTaskOnStartEditor (where it probably
        // 	  should be damnit) so I can't force initialization to happen before this call is made
        //    without recoding all of start editor (which is a bad idea).
        // #4 Because of certain weirdnesses of the gadget editor (namely the whole template thing)
        //    this call will be made again AFTER we are properly initialized - at which point it will
        //    do the right thing.
        try
        {
          // JF- The server returns a query result of the mandatory group for the current gadget.
          // We loop through the list and check if the Group ID passed in argument is in that list.
          IPTQueryResult qrMandatoryGroups = ((IPTGadget) m_ptObject).QueryMandatoryGroups();
          for (int i = 0;i < qrMandatoryGroups.RowCount();i++)
          {
            if (_nGroupID == qrMandatoryGroups.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID))
            {
              bIsMandatory = true;
              break;
            }
          }
        }
        catch (Exception e)
        {
          // AGC- This is only an info message because we expect this sucker to throw the first time it is called.
          log.Info(e, "ObjectSecurityModel::IsMandatoryGroup: Expected error querying gadget mandatory groups. Portlet may be a template or is not yet initialized. Continuing. e: " + e.Message);
          }
      }
      return bIsMandatory;
    }
    
    /// <summary>
    /// Tells whether the row identified by the class ID, the object ID
    /// and the Access Level ID in argument is in the Object ACL or no.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_nObjectID"></param>
    /// <param name="_nAccessLevel"></param>
    /// <returns>boolean</returns>
    private bool IsRowInObjectACL(int _nClassID, int _nObjectID, int _nAccessLevel)
    {
      IPTServerContext ptscObject = (IPTServerContext) m_ptObject;
      IPTAccessListEntry ptACLEntry;
      IPTAccessList ptACL = ptscObject.GetACL();
      for (int i = 0;i < ptACL.GetCount();i++)
      {
        ptACLEntry = ptACL.GetItem(i);
        if ((_nClassID == ptACLEntry.GetClassID()) && (_nObjectID == ptACLEntry.GetObjectID()) && (_nAccessLevel == ptACLEntry.GetAccessLevel()))
        {
          return true;
        }
      }
      return false;
    }
  }
}