using System;
// Plumtree Server
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.form;
namespace com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity
{
  /// <summary>
  /// Model class for the Priority drop-down.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class PriorityDropDownModel : EditorHelperModel,IPriorityDropDownModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity.PriorityDropDownModel");
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "PriorityDropDown";
    
    /// <summary>
    /// Object being created or edited 
    /// </summary>
    private IPTObject m_ptObject;
    
    /// <summary>
    /// Flag to know whether the current Object supports the mandatory column or no.
    /// We display the Rpiority drop-down only if this flag is true.
    /// </summary>
    private bool m_bSupportMandatoryColumn = false;
    
    /// <summary>
    /// Member variable to store the current value of the Mandatory tab priority 
    /// </summary>
    private int m_nMandatoryPriority;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new PriorityDropDownModel();
    }
    
    /// <summary>
    /// Sets the mandatory tab priority value on the Object.
    /// </summary>
    public virtual void Finish()
    {
      if (m_bSupportMandatoryColumn)
      {
        if (PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID())
        {
          ((IPTCommunity) m_ptObject).SetMandatoryTabOrder(m_nMandatoryPriority);
          } else
        if (PT_CLASSIDS.PT_GADGET_ID == m_ptObject.GetClassID())
        {
          ((IPTGadget) m_ptObject).SetMandatoryGadgetOrder(m_nMandatoryPriority);
          }
      }
    }
    
    /// <summary>
    /// Returns the current value of the Mandatory tab priority drop-down.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetMandatoryPriority()
    {
      return m_nMandatoryPriority;
    }
    
    /// <summary>
    /// Get the string to display to the left of the mandatory drop-down list.
    /// </summary>
    /// <returns>int</returns>
    public virtual String GetStringForUI()
    {
      if (PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID())
      {
        // "Mandatory Tab Priority"
        return m_asOwner.GetString(118, "ptmsgs_portalinfrastructure");
      } else
      if (PT_CLASSIDS.PT_GADGET_ID == m_ptObject.GetClassID())
      {
        // "Mandatory Portlet Priority"
        return m_asOwner.GetString(368, "ptmsgs_portalinfrastructure");
      } else
      {
        // default to the community case, "Mandatory Tab Priority".
        return m_asOwner.GetString(118, "ptmsgs_portalinfrastructure");
      }
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Tells whether the priority drop-down should be displayed
    /// for the current object or no.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetSupportMandatoryColumn()
    {
      return m_bSupportMandatoryColumn;
    }
    
    /// <summary>
    /// Initializes this plug-in section.
    /// </summary>
    /// <param name="_ptObject"></param>
    public virtual void InitializePriorityDropDown(IPTObject _ptObject)
    {
      m_ptObject = _ptObject;
      // JF- We already know that all communities should have it.
      // (For Portlets it different, only certain types of Portlets support it.)
      if (PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID())
      {
        m_bSupportMandatoryColumn = true;
        m_nMandatoryPriority = ((IPTCommunity) m_ptObject).GetMandatoryTabOrder();
        }
    }
    
    /// <summary>
    /// Saves the selected value in the priority drop-down.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>int Status for server side validation</returns>
    public override int SaveSection(XPHashtable _htFormData)
    {
      if (m_bSupportMandatoryColumn)
      {
        String[] data = (String[]) _htFormData.GetElement(PriorityDropDownView.HTMLSELECT_PRIORITY);
        if ((null != data) && !(0 == data.Length))
        {
          m_nMandatoryPriority = XPConvert.ToInteger(data[0]);
        } else
        {
          log.Warn("PriorityDropDownModel::SaveSection Mandatory Tab Priority value was not found in the form params.");
          }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Changes the value of the m_bSupportMandatoryTab member variable.
    /// </summary>
    /// <param name="_bSupportMandatoryTab"></param>
    public virtual void SetSupportMandatoryColumn(bool _bSupportMandatoryColumn)
    {
      m_bSupportMandatoryColumn = _bSupportMandatoryColumn;
      if (m_bSupportMandatoryColumn)
      {
        if (PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID())
        {
          m_nMandatoryPriority = ((IPTCommunity) m_ptObject).GetMandatoryTabOrder();
          } else
        if (PT_CLASSIDS.PT_GADGET_ID == m_ptObject.GetClassID())
        {
          m_nMandatoryPriority = ((IPTGadget) m_ptObject).GetMandatoryGadgetOrder();
          }
      }
    }
  }
}