using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.editor;
using com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory;
using com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tree;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.tree;
namespace com.plumtree.portaluiinfrastructure.editor.saveobject
{
  /// <summary>
  /// This is the Model class for the Save object pop-up.
  /// We use it to store the state of the save object pop-up,
  /// except for the security page and the folder tree.
  /// The Security page is part of the Object Editor AS which is the
  /// parent space of this one.
  /// The Folder Tree is a sub space.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class SaveObjectModel : IRepostModel,ISaveObjectModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.editor.saveobject.SaveObjectModel");
    
    /// <summary>
    /// Class Name for the Activity Space.
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "SaveObjectPopUp";
    
    /// <summary>
    /// Class Name for the Activity Space.
    /// </summary>
    public const String RECENT_FOLDERS_COOKIE_NAME = "ptrecentfolders";
    
    /// <summary>
    /// Key to store the folder tree as a sub space.
    /// </summary>
    public const String SAVEOBJECT_FOLDERTREE_SUBSPACEKEY = "SaveObjectPopUpFolderTree";
    
    /// <summary>
    /// Constant to identify the folder tree page in the body frame.
    /// </summary>
    public const int SAVEOBJECT_TAB_FOLDERTREE = 0;
    
    /// <summary>
    /// Constant to identify the security page in the body frame.
    /// </summary>
    public const int SAVEOBJECT_TAB_SECURITY = 1;
    
    /// <summary>
    /// Object name max length.
    /// </summary>
    private const int OBJECT_NAME_MAX_LENGTH = 252;
    
    /// <summary>
    /// Parent Activity Space.
    /// </summary>
    protected SaveObjectAS m_asOwner;
    
    /// <summary>
    /// Folder Name.
    /// </summary>
    protected String m_sFolderName;
    
    /// <summary>
    /// For some objects, you can't set/change the description from the Editor.
    /// </summary>
    protected bool m_bDisableDescription;
    
    /// <summary>
    /// Flag to know on which tab we are: folder tree or security.
    /// </summary>
    protected int m_nCurrentTab;
    
    /// <summary>
    /// Object being created.
    /// </summary>
    private IPTObject m_ptObject;
    
    /// <summary>
    /// User session.
    /// </summary>
    private IPTSession m_ptSession;
    
    /// <summary>
    /// JF- Array of class id and name of the 4 most rcently used folders.
    /// The IDs are stored in a Cookie on the user's machine.
    /// </summary>
    private Object[][] m_arrRecentlyUsedFolders;
    
    /// <summary>
    /// Forces the object to be saved in a particular folder, the save in folder tree is
    /// disabled when this variable is set. Retrieved from the parent model.
    /// </summary>
    private bool m_bForceSaveInFolder = false;
    
    /// <summary>
    /// Some Objects don't support ACLs. We don't display the Security page for those Objects.
    /// </summary>
    private bool m_bHideSecurityTab = false;
    
    /// <summary>
    /// See m_bRepostBodyFromTopFrame.
    /// </summary>
    private bool m_bRepostBodyFromBottomFrame = false;
    
    /// <summary>
    /// This flag is used to know if the Top frame should repost the body frame.
    /// The problem is that each frame is a different form, but when we change tab
    /// for example, we need to repost the top frame and change the selected tab,
    /// and then we need to repost the body frame, save the content and redirect it
    /// to the other tab. That's why when we repost the Top frame we also set this
    /// flag to true, this way a JavaScript will take care of reposting the body tab.
    /// </summary>
    private bool m_bRepostBodyFromTopFrame = false;
    
    /// <summary>
    /// See m_bRepostBodyFromTopFrame.
    /// </summary>
    private bool m_bRepostTreeAndSave = false;
    
    /// <summary>
    /// See m_bRepostBodyFromTopFrame.
    /// </summary>
    private bool m_bRepostTreeRecentFoldersFrame = false;
    
    /// <summary>
    /// Admin Folder ID.
    /// </summary>
    private int m_nFolderID;
    
    /// <summary>
    /// Adds the selected admin folder ID to Cookie.
    /// </summary>
    /// <param name="_xpRequest"></param>
    /// <param name="_webdata"></param>
    public virtual void AddAdminFolderIDToCookie(IXPRequest _xpRequest, IWebData _webdata)
    {
      XPCookie[] arrCookies;
      XPCookie myCookie;
      bool bFoundCookie = false;
      if (null == _xpRequest)
      {
        return;
      }
      if (IsAdminFolderInRecentFolders(m_nFolderID))
      {
        return;
      }
      String sCookieValue = "";
      int nMax = 4;
      if (PT_INTRINSICS.PT_ADMIN_FOLDER_UNCLASSIFIEDOBJECTS != m_nFolderID)
      {
        sCookieValue = XPConvert.ToString(m_nFolderID) + "-";
        nMax--;
      }
      int k = 0;
      while ((k < nMax) && (k < m_arrRecentlyUsedFolders.Length))
      {
        if (null != m_arrRecentlyUsedFolders[k][0])
        {
          sCookieValue = sCookieValue + m_arrRecentlyUsedFolders[k][0] + "-";
          }
        k++;
      }
      arrCookies = _xpRequest.GetCookies();
      if (null != arrCookies)
      {
        for (int i = 0;i < arrCookies.Length;i++)
        {
          myCookie = arrCookies[i];
          if (RECENT_FOLDERS_COOKIE_NAME.Equals(myCookie.GetName()))
          {
            myCookie.SetValue(sCookieValue);
            // JF- The Cookie will expire in 90 days.
            myCookie.SetExpireTime(60 * 60 * 24 * 90);
            myCookie.SetPath("");
            bFoundCookie = true;
            _webdata.AddCookie(myCookie);
            break;
          }
        }
      }
      if ((!bFoundCookie) && (PT_INTRINSICS.PT_ADMIN_FOLDER_UNCLASSIFIEDOBJECTS != m_nFolderID))
      {
        // Create cookie
        myCookie = new XPCookie(RECENT_FOLDERS_COOKIE_NAME, "" + m_nFolderID + "-");
        // JF- The Cookie will expire in 90 days.
        myCookie.SetExpireTime(60 * 60 * 24 * 90);
        myCookie.SetPath("");
        _webdata.AddCookie(myCookie);
      }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()"></seealso>
    public virtual void CleanupTempData()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()"></seealso>
    public virtual void CleanupAllData()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new SaveObjectModel();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.IRepostModel#Finish()"></seealso>
    public virtual void Finish()
    {
      // JF- There is nothing to do on Finish for regular Objects.
      // The control will take care of calling Finish on the parent AS which is the Editor.
      // JF- However for communities, we need to set the admin folder.
      // The community is a special case. The Object is being stored before the 
      // end of the Editor in the Unclassified Folder.
      // We want to set the right admin folder only at the very end when we know the user 
      // cannot cancel the Editor anymore.
      if (PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID())
      {
        m_ptObject.SetAdminFolderID(m_nFolderID);
      }
      if (PT_CLASSIDS.PT_SITEMAPFOLDER_ID == m_ptObject.GetClassID())
      {
        m_ptObject.SetAdminFolderID(m_nFolderID);
      }
    }
    
    /// <summary>
    /// Returns true if the current object is a community. 
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsCurrentObjectCommunity()
    {
      if (PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID())
      {
        return true;
      }
      return false;
    }
    
    /// <summary>
    /// Returns the Folder ID in which the Object will be saved.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool ForceSaveInFolder()
    {
      return m_bForceSaveInFolder;
    }
    
    /// <summary>
    /// Returns the admin folder ID.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetAdminFolderID()
    {
      return m_nFolderID;
    }
    
    /// <summary>
    /// Returns the ID of the current selected tab.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetCurrentTab()
    {
      return m_nCurrentTab;
    }
    
    /// <summary>
    /// Returns the folder name.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetFolderName()
    {
      return m_sFolderName;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Returns the description of the Object being created.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetObjectDescription()
    {
      //MES- This should be the PRIMARY description, since the
      //	user will be editing this description and saving it
      //	as the primary description.
      return m_ptObject.GetDescription();
    }
    
    /// <summary>
    /// Returns the name of the Object being created.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetObjectName()
    {
      //MES- This should be the PRIMARY name, since the
      //	user will be editing this name and saving it
      //	as the primary name.
      return m_ptObject.GetName();
    }
    
    /// <summary>
    /// Returns the array of most recently used folder IDs and names.
    /// </summary>
    /// <returns>Object[][]</returns>
    public virtual Object[][] GetRecentlyUsedFolders()
    {
      return m_arrRecentlyUsedFolders;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.IRepostModel#GetRedirectAfterFinish()"></seealso>
    public virtual Redirect GetRedirectAfterFinish()
    {
      return null;
    }
    
    /// <summary>
    /// Returns true if the body frame should be reposted from the
    /// bottom frame.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetRepostBodyFromBottomFrame()
    {
      return m_bRepostBodyFromBottomFrame;
    }
    
    /// <summary>
    /// Returns true if the body frame should be reposted from the
    /// top frame.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetRepostBodyFromTopFrame()
    {
      return m_bRepostBodyFromTopFrame;
    }
    
    /// <summary>
    /// Returns true if the tree frame should be reposted from
    /// the recent folders frame.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetRepostTreeAndSave()
    {
      return m_bRepostTreeAndSave;
    }
    
    /// <summary>
    /// Returns true if the tree frame should be reposted from
    /// the recent folders frame.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetRepostTreeRecentFoldersFrame()
    {
      return m_bRepostTreeRecentFoldersFrame;
    }
    
    /// <summary>
    /// Returns the title for the browser.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetTitle()
    {
      return m_asOwner.GetString(138, "ptmsgs_portalinfrastructure");
    }
    
    /// <summary>
    /// Returns true if the current Object doesn't support ACLs and
    /// the security tab should not be displayed.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool HideSecurityTab()
    {
      return m_bHideSecurityTab;
    }
    
    /// <summary>
    /// Returns true if the object description cannot be set/changed from the Editor.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsDescriptionDisabled()
    {
      return m_bDisableDescription;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#Init(AActivitySpace)"></seealso>
    public virtual void Init(AActivitySpace parent)
    {
      m_asOwner = (SaveObjectAS) parent;
    }
    
    /// <summary>
    /// Adds the selected admin folder ID to Cookie.
    /// </summary>
    /// <param name="_xpRequest"></param>
    /// <param name="_webdata"></param>
    public virtual void InitRecentlyUsedFolders(IXPRequest _xpRequest)
    {
      XPCookie[] arrCookies;
      XPCookie myCookie;
      if (null == _xpRequest)
      {
        return;
      }
      arrCookies = _xpRequest.GetCookies();
      if (null != arrCookies)
      {
        for (int i = 0;i < arrCookies.Length;i++)
        {
          myCookie = arrCookies[i];
          if (RECENT_FOLDERS_COOKIE_NAME.Equals(myCookie.GetName()))
          {
            String sFolderIDs = myCookie.GetValue();
            int nFolderID = -1;
            int k = -1;
            Object[][] arrFolders = new Object[2][];
            for (int j2c_index = 0;j2c_index < 2;j2c_index++)
            {
              arrFolders[j2c_index] = new Object[4];
            }
            IPTAdminFolder myFolder = null;
            while ((-1 != XPStringUtility.IndexOf(sFolderIDs, "-")) && (k < 3))
            {
              try
              {
                nFolderID = XPConvert.ToInteger(XPStringUtility.Substring(sFolderIDs,
                 0, XPStringUtility.IndexOf(sFolderIDs, "-")));
                if (!IsAdminFolderInRecentFolders(nFolderID) && (PT_INTRINSICS.PT_ADMIN_FOLDER_UNCLASSIFIEDOBJECTS != nFolderID) && (-1 != nFolderID))
                {
                  try
                  {
                    // JF- The current user must have at least edit access to the folder
                    bool bHasEditAccess = m_ptSession.GetAdminCatalog().IsAdminFolderAccessible(nFolderID,
                     PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT);
                    if (bHasEditAccess)
                    {
                      myFolder = m_ptSession.GetAdminCatalog().OpenAdminFolder(nFolderID,
                       false);
                    }
                  }
                  catch (Exception e)
                  {
                    log.Error("InitRecentlyUsedFolders Unable to open admin foler with ID: " + nFolderID + ". The exception was: " + e);
                    }
                  if (null != myFolder)
                  {
                    k++;
                    arrFolders[0][k] = nFolderID;
                    arrFolders[1][k] = myFolder;
                  }
                  myFolder = null;
                }
              }
              catch (Exception e)
              {
                log.Error("InitRecentlyUsedFolders Unable to parse the array of recently used folders." + "The exception was: " + e);
                }
              sFolderIDs = XPStringUtility.Substring(sFolderIDs, XPStringUtility.IndexOf(sFolderIDs,
               "-") + 1);
              nFolderID = -1;
            }
            if (k != -1)
            {
              m_arrRecentlyUsedFolders = new Object[k + 1][];
              for (int j2c_index = 0;j2c_index < k + 1;j2c_index++)
              {
                m_arrRecentlyUsedFolders[j2c_index] = new Object[2];
              }
              for (int l = 0;l < k + 1;l++)
              {
                try
                {
                  myFolder = (IPTAdminFolder) arrFolders[1][l];
                  m_arrRecentlyUsedFolders[l][0] = arrFolders[0][l];
                  m_arrRecentlyUsedFolders[l][1] = myFolder.GetLocalizedName();
                  }
                catch (Exception e)
                {
                  log.Error("InitRecentlyUsedFolders Unable to open admin foler with ID: " + nFolderID + ". The exception was: " + e);
                  }
              }
            }
            break;
          }
        }
      }
    }
    
    /// <summary>
    /// This methods should be called when the pop-up is displayed for the 1st time.
    /// </summary>
    public virtual void InitializeSaveObjectPopUp(IXPRequest _xpRequest)
    {
      // JF- First we need the Object being created and the Session. 
      // We get them from the parent AS (which is the Editor AS).
      EditorAS parentSpace = (EditorAS) m_asOwner.GetParentSpace();
      ObjEditorModel parentModel = (ObjEditorModel) parentSpace.GetModel(parentSpace.m_strModelName);
      m_ptObject = parentModel.GetObject();
      m_ptSession = parentModel.GetPTSession();
      m_nFolderID = parentModel.GetFolderID();
      m_bDisableDescription = parentModel.IsDescriptionDisabled();
      m_arrRecentlyUsedFolders = new Object[0][];
      for (int j2c_index = 0;j2c_index < 0;j2c_index++)
      {
        m_arrRecentlyUsedFolders[j2c_index] = new Object[0];
      }
      if (parentModel.ForceSaveInFolder())
      {
        m_sFolderName = parentModel.GetFolderName();
        m_bForceSaveInFolder = true;
      } else
      {
        InitRecentlyUsedFolders(_xpRequest);
      }
      if (null == m_ptObject)
      {
        throw new XPException("SaveObjectModel::InitializeSaveObjectPopUp The IPTObject from the parent" + " Activity Space is null");
        }
      // JF- We always start with the folder tree tab.
      m_nCurrentTab = SAVEOBJECT_TAB_FOLDERTREE;
      // JF- Users don't support ACLs
      if (PT_CLASSIDS.PT_USER_ID == m_ptObject.GetClassID())
      {
        m_bHideSecurityTab = true;
      }
    }
    
    /// <summary>
    /// Returns true if the current admin folder is one of the recently used folders.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsAdminFolderInRecentFolders(int _nFolderID)
    {
      bool bIsAdminFolderInRecentFolders = false;
      for (int i = 0;i < m_arrRecentlyUsedFolders.Length;i++)
      {
        if (null != m_arrRecentlyUsedFolders[i][0])
        {
          if (_nFolderID == XPConvert.ToInteger(m_arrRecentlyUsedFolders[i][0]))
          {
            bIsAdminFolderInRecentFolders = true;
            break;
          }
        }
      }
      return bIsAdminFolderInRecentFolders;
    }
    
    /// <summary>
    /// Returns true if the migration has already been approved for the current object.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsMigrationApproved()
    {
      // JF- Retrieve the parent Model
      EditorAS myParentAS = (EditorAS) m_asOwner.GetParentSpace();
      ObjEditorModel myObjEditorModel = (ObjEditorModel) myParentAS.GetModel(myParentAS.m_strModelName);
      return myObjEditorModel.IsMigrationApproved();
    }
    
    /// <summary>
    /// Returns true if the current user has system administrator activity rights.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsSysAdmin()
    {
      // JF- Retrieve the parent Model
      EditorAS myParentAS = (EditorAS) m_asOwner.GetParentSpace();
      bool bIsSysAdmin = ((ObjEditorModel) myParentAS.GetModel(myParentAS.m_strModelName)).GetIsSysAdmin();
      return bIsSysAdmin;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.IRepostModel#SavePage(String,XPHashtable)"></seealso>
    /// <returns>int Page status for server side validation</returns>
    public virtual int SavePage(String _sPageName, XPHashtable _htFormData)
    {
      // JF- Those falgs are for "one time" JavaScripts. So we need to set them back 
      // to false after they have been used. 
      m_bRepostBodyFromTopFrame = false;
      m_bRepostBodyFromBottomFrame = false;
      m_bRepostTreeAndSave = false;
      m_bRepostTreeRecentFoldersFrame = false;
      if (null == m_ptObject)
      {
        throw new XPException("SaveObjectModel::SavePage The save object pop-up has not been initialized.");
        }
      // JF- Then we switch between the pages to know which one we need to save.
      if (_sPageName.Equals(SaveObjectBottomFrameDP.STR_MVC_CLASS_NAME))
      {
        SaveNameDesc(_htFormData);
      } else
      if (!m_bHideSecurityTab && _sPageName.Equals(SaveObjectTopFrameDP.STR_MVC_CLASS_NAME))
      {
        SaveTab(_htFormData);
      } else
      if (!m_bHideSecurityTab && _sPageName.Equals(ObjectSecurityDP.STR_MVC_CLASS_NAME))
      {
        // JF- We need to call the save method on the security page.
        AActivitySpace mySecurityAS = m_asOwner.GetParentSpace();
        ObjectSecurityModel mySecurityModel = (ObjectSecurityModel) mySecurityAS.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB);
        mySecurityModel.SaveSection(_htFormData);
        PriorityDropDownModel myPriorityDropDownModel = (PriorityDropDownModel) mySecurityAS.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN);
        myPriorityDropDownModel.SaveSection(_htFormData);
      } else
      if (_sPageName.Equals(TreeWithSearchDP.STR_MVC_CLASS_NAME))
      //else if (_sPageName.equals(SaveObjectMainFrameDP.STR_MVC_CLASS_NAME)) 
      {
        // JF- We need to call the save method on the tree.
        AActivitySpace myTreeAS = m_asOwner.GetSubSpace(SAVEOBJECT_FOLDERTREE_SUBSPACEKEY);
        ATreeModel myTreeModel = (ATreeModel) myTreeAS.GetModel(ATreeAS.TREEMODEL);
        //_htFormData.PutElement(TreeView.HTMLBUTTON_RADIO, new String[]{XPConvert.ToString(m_nFolderID)});
        //_htFormData.PutElement(TreeWithSearchDP.HTMLINPUT_DISPLAYING_SEARCH, new String[]{XPConvert.ToString(false)});
        //_htFormData.PutElement(TreeView.HTMLHIDDEN_SELECTEDIDS, new String[]{XPConvert.ToString(m_nFolderID)});
        myTreeModel.SavePage(_sPageName, _htFormData);
        SaveAdminFolderID(_htFormData);
      } else
      if (_sPageName.Equals(SaveObjectRecentFoldersDP.STR_MVC_CLASS_NAME))
      {
        if (!m_bForceSaveInFolder)
        {
          String[] data = (String[]) _htFormData.GetElement(SaveObjectRecentFoldersView.HTMLINPUT_RECENT_FOLDERS);
          if ((null != data) && (0 != data.Length) && (null != data[0]) && (0 != data[0].Length))
          {
            try
            {
              m_nFolderID = XPConvert.ToInteger(data[0]);
            }
            catch (Exception e)
            {
              log.Error("SaveObjectModel::SaveAdminFolderID The Folder ID posted by the form was invalid: " + data[0]);
              }
          }
        }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Sets a flag to know if the body frame should be reposted from
    /// the bottom frame.
    /// </summary>
    /// <param name="_bRepostBodyFromBottomFrame"></param>
    public virtual void SetRepostBodyFromBottomFrame(bool _bRepostBodyFromBottomFrame)
    {
      m_bRepostBodyFromBottomFrame = _bRepostBodyFromBottomFrame;
    }
    
    /// <summary>
    /// Sets a flag to know if the body frame should be reposted from
    /// the top frame.
    /// </summary>
    /// <param name="_bRepostBodyFromTopFrame"></param>
    public virtual void SetRepostBodyFromTopFrame(bool _bRepostBodyFromTopFrame)
    {
      m_bRepostBodyFromTopFrame = _bRepostBodyFromTopFrame;
    }
    
    /// <summary>
    /// Sets a flag to know if the tree frame should be reposted from
    /// the recent folders frame.
    /// </summary>
    /// <param name="_bRepostTreeAndSave"></param>
    public virtual void SetRepostTreeAndSave(bool _bRepostTreeAndSave)
    {
      m_bRepostTreeAndSave = _bRepostTreeAndSave;
    }
    
    /// <summary>
    /// Sets a flag to know if the tree frame should be reposted from
    /// the recent folders frame.
    /// </summary>
    /// <param name="_bRepostTreeRecentFoldersFrame"></param>
    public virtual void SetRepostTreeRecentFoldersFrame(bool _bRepostTreeRecentFoldersFrame)
    {
      m_bRepostTreeRecentFoldersFrame = _bRepostTreeRecentFoldersFrame;
    }
    
    /// <summary>
    /// Saves the new folder ID and set it in the Object.
    /// Also if the folder ID has changed, we re-initialize
    /// the security page as the security is based on the folder security.
    /// </summary>
    protected virtual void SaveAdminFolderID(XPHashtable _htFormData)
    {
      if (!m_bForceSaveInFolder)
      {
        /*String[] data = (String[]) _htFormData.GetElement(SaveObjectRecentFoldersView.HTMLHIDDEN_SELECTED_FOLDER);
         
         if ((null != data) && (0 != data.length) && (null != data[0]) && (0 != data[0].length()))
         {
         try
         {
         m_nFolderID = XPConvert.ToInteger(data[0]);
         } catch (Exception e) {
         if ( log.IsErrorEnabled() ) {
         log.Error("SaveObjectModel::SaveAdminFolderID The Folder ID posted by the form was invalid: " + data[0]);
         }
         }
         
         if ((m_nFolderID != m_ptObject.GetAdminFolderID()) && (PT_CLASSIDS.PT_COMMUNITY_ID != m_ptObject.GetClassID()))
         {
         // JF- We update the Object
         m_ptObject.SetAdminFolderID(m_nFolderID);
         
         // JF- And also we re-initialize the security page
         ((ObjectSecurityModel) m_asOwner.GetParentSpace().GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB)).
         InitializeSecurityPage(m_ptObject, m_ptSession);
         }
         }*/
         
        TreeAS asTree = (TreeAS) m_asOwner.GetSubSpace(SAVEOBJECT_FOLDERTREE_SUBSPACEKEY);
        if (null != asTree)
        {
          // JF- We retrieve the tree
          ITreeModelRO myTree = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
          XPArrayList mySelected = myTree.GetSelectedItems();
          if (0 != mySelected.GetSize())
          {
            int nFolderID = ((TreeLeafNode) mySelected.GetElement(0)).m_nObjectID;
            if (PT_INTRINSICS.PT_ADMIN_FOLDER_UNCLASSIFIEDOBJECTS != nFolderID)
            {
              // JF- We get the selected folder ID
              m_nFolderID = nFolderID;
            }
          } else
          {
            log.Debug("SaveObjectModel::SaveAdminFolderID No folder ID was returned by the Tree.");
            }
        }
        if ((m_nFolderID != m_ptObject.GetAdminFolderID()) && (PT_CLASSIDS.PT_COMMUNITY_ID != m_ptObject.GetClassID()))
        {
          // JF- We update the Object
          m_ptObject.SetAdminFolderID(m_nFolderID);
          // JF- And also we re-initialize the security page
          ((ObjectSecurityModel) m_asOwner.GetParentSpace().GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB)).InitializeSecurityPage(m_ptObject,
           m_ptSession);
        }
      }
    }
    
    /// <summary>
    /// Saves name and description..
    /// </summary>
    /// <param name="_htFormData"></param>
    protected virtual void SaveNameDesc(XPHashtable _htFormData)
    {
      String[] data;
      // JF- Save the name
      data = (String[]) _htFormData.GetElement(SaveObjectNameDescView.HTMLINPUT_NAME);
      if ((null != data) && (0 != data.Length) && (null != data[0]) && (0 != data[0].Length))
      {
        // JF- Somehow if the name is longer than 251 char, the SetName will work 
        // but Store will throw an exception
        if (OBJECT_NAME_MAX_LENGTH > data[0].Length)
        {
          try
          {
            m_ptObject.SetName(data[0]);
          }
          catch (Exception e)
          {
            log.Error("SaveObjectModel::SaveNameDesc The Object Name posted by the form was invalid: " + data[0]);
            }
        } else
        {
          log.Error("SaveObjectModel::SaveNameDesc The Object Name is too long and " + "it will not be saved. The length is: " + data[0].Length);
          }
      } else
      {
        log.Warn("SaveObjectModel::SaveNameDesc The Object Name has not been posted by the form.");
        }
      if (!IsDescriptionDisabled())
      {
        // JF- Save the description
        data = (String[]) _htFormData.GetElement(SaveObjectNameDescView.HTMLINPUT_DESCRIPTION);
        if ((null != data) && !(0 == data.Length) && null != data[0])
        {
          try
          {
            m_ptObject.SetDescription(data[0]);
          }
          catch (Exception e)
          {
            log.Error("SaveObjectModel::SaveNameDesc The Object Description posted by the form was invalid." + data[0]);
            }
        } else
        {
          log.Warn("SaveObjectModel::SaveNameDesc The Object Description has not been posted by the form.");
          }
      }
      // JF- save the object migration checkbox
      if (_htFormData.ContainsKey(SaveObjectNameDescView.HTMLCHECKBOX_MIGRATION))
      {
        // JF- Retrieve the parent Model
        EditorAS myParentAS = (EditorAS) m_asOwner.GetParentSpace();
        // AS- Mark that the object has been approved for migration
        ((IMigrationHistoryModel) myParentAS.GetModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME)).SetApprovedForMigration(true);
        }
    }
    
    /// <summary>
    /// Saves the new tab ID.
    /// </summary>
    /// <param name="_htFormData"></param>
    protected virtual void SaveTab(XPHashtable _htFormData)
    {
      String[] data;
      data = (String[]) _htFormData.GetElement(SaveObjectMenuView.HTMLHIDDEN_NEWTAB);
      if ((null != data) && (0 != data.Length) && (null != data[0]) && (0 != data[0].Length))
      {
        try
        {
          m_nCurrentTab = XPConvert.ToInteger(data[0]);
        }
        catch (Exception e)
        {
          log.Error("SaveObjectModel::SaveTab The new tab ID posted by the form was invalid." + data[0]);
          }
      } else
      {
        log.Warn("SaveObjectModel::SaveTab The new tab name has not been posted by the form.");
        }
    }
  }
}