using System;
// Plumtree Server
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.editor;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tree;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.expandablelist;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.tree;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.expandablelist.selectptobject
{
  /// <summary>
  /// This is the implementation of the Expandable List
  /// framework for the the Select PTObject list.
  /// The Select PTObject is a list of objects.
  /// New objects can be added to the list by selecting
  /// them from the associated Object tree.
  /// The list is by default sorted by name ascending.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class SelectPTObjectModel : ExpListModel,ISelectPTObjectModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.expandablelist.selectptobject.SelectPTObjectModel");
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "SelectPTObject";
    
    /// <summary>
    /// Key name used by the Activity Space to store the tree. 
    /// </summary>
    public const String SUBSPACE_KEY_SELECTTREE = "SelectPTObjectTree";
    
    /// <summary>
    /// Number total of columns
    /// </summary>
    public const int EXPLIST_COL_MAX = 4;
    
    /// <summary>
    /// This column contains the Object Class IDs.
    /// </summary>
    public const int EXPLIST_COL_CLASSID = 1;
    
    /// <summary>
    /// This column contains the Object IDs.
    /// </summary>
    public const int EXPLIST_COL_OBJECTID = 2;
    
    /// <summary>
    /// This is the String that we doisplay in the list.
    /// E.g.: Object name or folder path.
    /// </summary>
    public const int EXPLIST_COL_STRING = 3;
    
    /// <summary>
    /// Default initial size for the sorted array.
    /// </summary>
    public const int EXPLIST_DEFAULT_INITIAL_SIZE = 50;
    
    /// <summary>
    /// Number total of columns for hide objects array.
    /// </summary>
    public const int HIDE_COL_MAX = 2;
    
    /// <summary>
    /// This column of the passed Objects to Hide contains the class IDs.
    /// </summary>
    public const int HIDE_COL_CLASSID = 0;
    
    /// <summary>
    /// This column of the passed Objects to Hide contains the object IDs.
    /// </summary>
    public const int HIDE_COL_OBJECTID = 1;
    
    /// <summary>
    /// Create new array column: class ID
    /// </summary>
    public const int CREATE_NEW_COL_CLASSID = 0;
    
    /// <summary>
    /// Create new array column: AS Name
    /// </summary>
    public const int CREATE_NEW_COL_ASNAME = 1;
    
    /// <summary>
    /// Create new array column: text to display with the create new link
    /// </summary>
    public const int CREATE_NEW_COL_LABEL = 2;
    
    /// <summary>
    /// Create new array column: icon to display for the create new link
    /// </summary>
    public const int CREATE_NEW_COL_ICON = 3;
    
    /// <summary>
    /// Create new array column: extra control arguments for the create action.
    /// Use CREATE_NEW_COL_MAXWITHPARAMS to set the size of the array when using
    /// this element.
    /// </summary>
    public const int CREATE_NEW_COL_PARAMS = 4;
    
    /// <summary>
    /// Size of each m_arrCreateNewInfo row
    /// </summary>
    public const int CREATE_NEW_COL_MAX = 4;
    
    /// <summary>
    /// Size of each m_arrCreateNewInfo row with optional control parameters.
    /// </summary>
    public const int CREATE_NEW_COL_MAXWITHPARAMS = 5;
    
    /// <summary>
    /// Optional control parameter index for the parameter name. Used in the
    /// create new array column: icon to display for the create new link
    /// </summary>
    public const int PARAM_COL_PARAMNAME = 0;
    
    /// <summary>
    /// Optional control parameter index for the parameter value. Used in the
    /// create new array column: icon to display for the create new link
    /// </summary>
    public const int PARAM_COL_PARAMVALUE = 1;
    
    /// <summary>
    /// Icon to display on each row, just before the input text.
    /// RWW: This will now be the "default". See m_xphtRowIcons 
    /// for precedence in icon loading.
    /// </summary>
    protected PTImageType m_imgDefaultRowIcon;
    
    /// <summary>
    /// RWW: Stores multiple icons - allows us to store a different 
    /// icon for each class id.  When retrieving icons internally, 
    /// this table will be checked first.  If no icons appear here,
    /// then we will check the default row icon.  If that is null,
    /// then we will show no image.  Having an icon for a specific
    /// class id in this table will take precedence.
    /// </summary>
    protected XPHashtable m_xphtRowIcons;
    
    /// <summary>
    /// JH: Set to true if no row icons are to be displayed.  This will override
    /// m_xphtRowIcons, m_imgDefaultRowIcon, and the default ClassDescriptor icon
    /// retrieval mechanism.
    /// </summary>
    protected bool m_bNoRowIcons = false;
    
    /// <summary>
    /// Tree Space ID. The first time the ID is empty and a new AS is opened,
    /// but after that, we save the ID and the same tree AS is reopened each time. 
    /// </summary>
    protected String m_sTreeSpaceID = "";
    
    /// <summary>
    /// Tree sub title
    /// </summary>
    protected String m_sTreeSubTitle;
    
    /// <summary>
    /// Tree title
    /// </summary>
    protected String m_sTreeTitle;
    
    /// <summary>
    /// Class IDs and Object IDs of objects to display in the tree.
    /// </summary>
    protected String m_sDisplayObjects;
    
    /// <summary>
    /// Objects to hide for the object tree.
    /// </summary>
    protected String m_strObjectsToHide = "";
    
    /// <summary>
    /// Query Filter for the object tree.
    /// </summary>
    protected String m_strTreeQueryFilter = "";
    
    /// <summary>
    /// Information about the create new links to display
    /// </summary>
    protected Object[][] m_arrCreateNewInfo;
    
    /// <summary>
    /// Class ID of the Objects displayed on the list.
    /// </summary>
    protected int[] m_arrObjectClassIDs;
    
    /// <summary>
    /// Selection type for the object tree.
    /// </summary>
    protected bool m_bTreeMultiSelect = true;
    
    /// <summary>
    /// This value determines the security access requirements for the objects to
    /// be added to the list... (a filter for the tree control).
    /// </summary>
    protected int m_nAccessLevel = 0;
    
    /// <summary>
    /// Community Mode to display objects.
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tree.StartTreeControl.COMMUNITY_MODE"></seealso>
    protected int m_nCommunityMode = 0;
    
    /// <summary>
    /// Community Folder ID to display objects from.
    /// </summary>
    protected int m_nCommunityFolderID = 0;
    
    /// <summary>
    /// Leaf type for the object tree.
    /// </summary>
    protected int m_treeLeafType = 0;
    
    /// <summary>
    /// JF- Array of classID/objectID that cannot be deleted from the list.
    /// </summary>
    protected int[][] m_arrReadOnlyObjects;
    
    /// <summary>
    /// JF- Array of read-only objects: column index for classID.
    /// </summary>
    public const int READ_ONLY_ROW_CLASSID = 0;
    
    /// <summary>
    /// JF- Array of read-only objects: column index for objectID.
    /// </summary>
    public const int READ_ONLY_ROW_OBJECTID = 1;
    public SelectPTObjectModel() : base()
    {
      m_treeLeafType = TreeConstants.N_LEAFSELECT;
      m_nAccessLevel = PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT;
    }
    
    /// <summary>
    /// Helper function for extracting and adding extra control parameters from
    /// the create object string.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="myURL"></param>
    public virtual void AddControlParamsToURL(int _nClassID, ASURL myURL)
    {
      Object oParams = null;
      String[][] arParams = null;
      bool bFoundFolderID = false;
      //SE: logging
      log.FunctionBegin("SelectPTObjectModel::AddControlParamsToURL");
      for (int i = 0;i < m_arrCreateNewInfo.Length;i++)
      {
        int nClassIDFromArray = XPConvert.ToInteger(m_arrCreateNewInfo[i][CREATE_NEW_COL_CLASSID]);
        if (nClassIDFromArray == _nClassID)
        {
          if (m_arrCreateNewInfo[i].Length >= CREATE_NEW_COL_MAXWITHPARAMS)
          {
            oParams = m_arrCreateNewInfo[i][CREATE_NEW_COL_PARAMS];
            break;
          }
        }
      }
      if (null != oParams)
      {
        arParams = (String[][]) oParams;
        for (int j = 0;j < arParams.Length;j++)
        {
          if ((null != arParams[j]) && (2 == arParams[j].Length))
          {
            String strName = arParams[j][PARAM_COL_PARAMNAME];
            if ((null != strName) && (0 != strName.Length))
            {
              String strValue = arParams[j][PARAM_COL_PARAMVALUE];
              // Add the parameter to the url.
              myURL.AddControlArgument(strName, strValue);
              if (XPStringUtility.EqualsIgnoreCase(strName, ObjEditorModel.EDITOR_QS_INT_QS_FOLDER_ID))
              {
                bFoundFolderID = true;
              }
            }
          }
        }
      } else
      {
        log.Debug("AddControlParamsToURL: Unable to retrieve control info.");
        }
      if (!bFoundFolderID)
      {
        try
        {
          ObjEditorModel myModel = (ObjEditorModel) m_asOwner.GetModel(((ObjEditorAS) m_asOwner).m_strModelName);
          int nFolderID = myModel.GetFolderID();
          myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_QS_FOLDER_ID,
           XPConvert.ToString(nFolderID));
        }
        catch (Exception e)
        {
          log.Warn("Could not retrieve the Admin Folder ID from the current Editor." + " No Admin Folder will be pre-selected in the save-as pop-up of sub Editors.");
          }
      }
      //SE: logging
      log.FunctionEnd("SelectPTObjectModel::AddControlParamsToURL");
    }
    
    /// <summary>
    /// Returns a unique String for the name of the pop-up.
    /// </summary>
    /// <param name="_sName"></param>
    public virtual String GetUniquePopUpName(String _sName)
    {
      return GetUniqueString(_sName) + m_asOwner.GetSpaceID();
    }
    
    /// <summary>
    /// Clear the list and add all the values from the tree.
    /// </summary>
    /// <param name="_htFormData">Form data</param>
    public override void AddRowsOnRepost(XPHashtable _htFormData)
    {
      //SE: logging
      log.FunctionBegin("SelectPTObjectModel::AddRowsOnRepost");
      if (!m_bIsReadOnly)
      {
        SaveTreeID(_htFormData);
        TreeAS asTree = (TreeAS) m_asOwner.GetSubSpace(GetUniqueString(SUBSPACE_KEY_SELECTTREE));
        if (null != asTree)
        {
          ITreeModelRO myTree = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
          XPArrayList mySelected = myTree.GetSelectedItems();
          int nClassID;
          int nObjectID;
          int nIndex;
          // JF- Before we clear the items and get the brand new list from the tree, we want to keep a copy of the indexes.
          // If the move row actions are enabled, we need to keep the same indexes for Objects already in the list. 
          // We need to keep the order for existing Objects and add the new ones at the end.
          Object[][] arrIndexMapping = CreateIndexMapping(GetItems());
          // JF- We clear the array before we start adding the selected items from the tree.
          base.GetItems().ClearArray();
          // Add rows in bulk to avoid resorting the entire list on each add.
          // This becomes expensive for groups with 4000 users.
          Object[][] arrBulkNewRows = new Object[mySelected.GetSize()][];
          for (int j2c_index = 0;j2c_index < mySelected.GetSize();j2c_index++)
          {
            arrBulkNewRows[j2c_index] = new Object[EXPLIST_COL_MAX];
          }
          for (int i = 0;i < mySelected.GetSize();i++)
          {
            nClassID = ((TreeLeafNode) mySelected.GetElement(i)).m_nClassID;
            nObjectID = ((TreeLeafNode) mySelected.GetElement(i)).m_nObjectID;
            nIndex = GetIndex(arrIndexMapping, nClassID, nObjectID);
            if (-1 == nIndex)
            {
              nIndex = m_nNextIndex;
              m_nNextIndex++;
            }
            arrBulkNewRows[i][EXPLIST_SORTEDARRAY_PROPID_INDEX] = nIndex;
            arrBulkNewRows[i][EXPLIST_COL_CLASSID] = nClassID;
            arrBulkNewRows[i][EXPLIST_COL_OBJECTID] = nObjectID;
            if (PT_CLASSIDS.PT_CATALOGFOLDER_ID == nClassID)
            {
              arrBulkNewRows[i][EXPLIST_COL_STRING] = ((TreeLeafNode) mySelected.GetElement(i)).m_strLocation;
              } else
            {
              arrBulkNewRows[i][EXPLIST_COL_STRING] = ((TreeLeafNode) mySelected.GetElement(i)).GetName();
              }
          }
          base.GetItems().AddRows(arrBulkNewRows);
        }
      }
      //SE: logging
      log.FunctionEnd("SelectPTObjectModel::AddRowsOnRepost");
    }
    
    /// <summary>
    /// Tells whether the Object identified by the class ID
    /// and the object ID in argument is in the list or no.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_nObjectID"></param>
    /// <returns>boolean</returns>
    public virtual bool ContainsObject(int _nClassID, int _nObjectID)
    {
      //SE: logging
      log.FunctionBegin("SelectPTObjectModel::ContainsObject");
      for (int i = 0;i < m_saData.GetCount();i++)
      {
        if ((_nClassID == XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_CLASSID))) && (_nObjectID == XPConvert.ToInteger(m_saData.GetItem(i,
         EXPLIST_COL_OBJECTID))))
        {
          //SE: logging
          log.FunctionEnd("SelectPTObjectModel::ContainsObject");
          return true;
        }
      }
      //SE: logging
      log.FunctionEnd("SelectPTObjectModel::ContainsObject");
      return false;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new SelectPTObjectModel();
    }
    
    /// <summary>
    /// This method will be called when you come back to the main Editor from a sub Editor.
    /// </summary>
    public virtual void DoTaskOnComeBackFromSubEditor()
    {
      String sEditorASName;
      EditorAS myEditorAS;
      ObjEditorModel myObjEditorModel;
      Object[] oNewRow;
      int nClassID;
      int nObjectID;
      //SE: logging
      log.FunctionBegin("SelectPTObjectModel::DoTaskOnComeBackFromSubEditor");
      // JF- Check if they are any newly created objects
      for (int i = 0;i < m_arrCreateNewInfo.Length;i++)
      {
        sEditorASName = (String) m_arrCreateNewInfo[i][CREATE_NEW_COL_ASNAME];
        if (m_asOwner.GetSubSpaceExists(sEditorASName))
        {
          myEditorAS = (EditorAS) m_asOwner.GetSubSpace(sEditorASName);
          myObjEditorModel = (ObjEditorModel) myEditorAS.GetModel(myEditorAS.m_strModelName);
          oNewRow = new Object[EXPLIST_COL_MAX];
          oNewRow[EXPLIST_SORTEDARRAY_PROPID_INDEX] = m_nNextIndex;
          m_nNextIndex++;
          nClassID = myObjEditorModel.GetClassID();
          oNewRow[EXPLIST_COL_CLASSID] = nClassID;
          oNewRow[EXPLIST_COL_OBJECTID] = myObjEditorModel.GetObjectID();
          if (PT_CLASSIDS.PT_CATALOGFOLDER_ID == nClassID)
          {
            oNewRow[EXPLIST_COL_STRING] = ((IPTFolder) myObjEditorModel.GetObject()).GetPath();
            } else
          {
            oNewRow[EXPLIST_COL_STRING] = myObjEditorModel.GetObjectName();
            }
          // Wait with resorting until we finisehd adding all entries rather than
          // resorting after adding each individual entry
          // Updated to fix bug #51045:
          base.GetItems().AddRowWithoutSort(oNewRow);
          m_asOwner.DeleteSubSpace(sEditorASName);
        }
      }
      // done adding all the items, resort array
      // Updated to fix bug #51045:
      base.GetItems().SortArray();
      // SE: Bug 57861 - use StringBuilder to avoid CPU spike
      XPStringBuilder sbTemp = new XPStringBuilder(100);
      String sSubSpace;
      // JF- Go through all the existing sub spaces and update the Object name.
      for (int i = 0;i < m_saData.GetCount();i++)
      {
        nClassID = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_CLASSID));
        if (PT_CLASSIDS.PT_CATALOGFOLDER_ID != nClassID)
        {
          nObjectID = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_OBJECTID));
          // SE: Bug 57861 - use StringBuilder to avoid CPU spike
          sbTemp.SetLength(0);
          sbTemp.Append(STR_MVC_CLASS_NAME);
          sbTemp.Append(nClassID);
          sbTemp.Append(nObjectID);
          sSubSpace = sbTemp.ToString();
          if (m_asOwner.GetSubSpaceExists(sSubSpace))
          {
            try
            {
              EditorAS mySpace = (EditorAS) m_asOwner.GetSubSpace(sSubSpace);
              ObjEditorModel myModel = (ObjEditorModel) mySpace.GetModel(mySpace.m_strModelName);
              m_saData.SetItem(i, EXPLIST_COL_STRING, myModel.GetObjectName());
              }
            catch (Exception e)
            {
              log.Warn("Could not retrieve the sub Space with key: " + STR_MVC_CLASS_NAME + nClassID + nObjectID);
              }
          }
        }
      }
      //SE: logging
      log.FunctionEnd("SelectPTObjectModel::DoTaskOnComeBackFromSubEditor");
      }
    
    /// <summary>
    /// Returns the nAccessLevel. The accesslevel of the objects for the tree control to display.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetAccessLevel()
    {
      return m_nAccessLevel;
    }
    
    /// <summary>
    /// Returns the community mode for the tree.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetCommunityMode()
    {
      return m_nCommunityMode;
    }
    
    /// <summary>
    /// Returns the community id for the tree.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetCommunityFolderID()
    {
      return m_nCommunityFolderID;
    }
    
    /// <summary>
    /// Returns the create new action informations.
    /// </summary>
    /// <returns>Object[][]</returns>
    public virtual Object[][] GetCreateNewInfo()
    {
      return m_arrCreateNewInfo;
    }
    
    /// <summary>
    /// Returns the URL to the Editor in create mode (new object).
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_sEditorASName"></param>
    /// <returns>ASURL URL to the Editor</returns>
    public virtual ASURL GetCreateNewURL(int _nClassID, String _sEditorASName)
    {
      ASURL myURL = new ASURL(m_asOwner);
      myURL.SetLinkCreateNewSpace(_sEditorASName, m_asOwner);
      myURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
      myURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE, XPConvert.ToString(EditorStartControl.EDITOR_START_FLAG_CREATE));
      myURL.AddControlArgument(EditorStartControl.QS_DELETE_ON_CANCEL, XPConvert.ToString(true));
      myURL.SetSubSpaceKey(_sEditorASName);
      myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_CLASS_ID, XPConvert.ToString(_nClassID));
      // Get the extra control arguments
      AddControlParamsToURL(_nClassID, myURL);
      return myURL;
    }
    
    /// <summary>
    /// Returns the URL to edit the object.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_nObjectID"></param>
    /// <param name="_sEditorASName"></param>
    /// <returns>ASURL URL to the Editor</returns>
    public virtual ASURL GetEditObjectURL(int _nClassID, int _nObjectID)
    {
      ASURL myURL = new ASURL(m_asOwner);
      // JF- For any Object in the list, the sub space key will be 
      // this Model name with the object class id and the object id.
      String sSubEditorKey = STR_MVC_CLASS_NAME + _nClassID + _nObjectID;
      if (m_asOwner.GetSubSpaceExists(sSubEditorKey))
      {
        EditorAS mySubEditor = (EditorAS) m_asOwner.GetSubSpace(sSubEditorKey);
        myURL.SetLinkGetCachedSpace(mySubEditor);
      } else
      {
        String sEditorASName = GetEditorASName(_nClassID);
        myURL.SetLinkCreateNewSpace(sEditorASName, m_asOwner);
        myURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
        myURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE, XPConvert.ToString(EditorStartControl.EDITOR_START_FLAG_EDIT));
        myURL.SetSubSpaceKey(sSubEditorKey);
        myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_CLASS_ID, XPConvert.ToString(_nClassID));
        myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_QS_OBJECT_ID,
         XPConvert.ToString(_nObjectID));
      }
      AddControlParamsToURL(_nClassID, myURL);
      return myURL;
    }
    
    /// <summary>
    /// Returns the hidden objects for the tree.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetHiddenObjects()
    {
      return m_strObjectsToHide;
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Returns String of Class ID of the Objects
    /// displayed in the tree.
    /// E.g.: "2,45,3"
    /// This is the format used by the tree.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetObjectClassIDs()
    {
      //SE: logging
      log.FunctionBegin("SelectPTObjectModel::GetObjectClassIDs");
      // SE: Bug 57861 - use StringBuilder to avoid CPU spike
      XPStringBuilder sbClassIDs = new XPStringBuilder(3 * m_arrObjectClassIDs.Length);
      // JF- There is no object manager for admin folders so the tree has to be started differently.
      if (PT_CLASSIDS.PT_ADMIN_FOLDER_ID != m_arrObjectClassIDs[0])
      {
        // JF- We loop through the array of IDs and add each ID 
        // to the String.
        for (int i = 0;i < m_arrObjectClassIDs.Length;i++)
        {
          if (i > 0)
          {
            // SE: Bug 57861 - use StringBuilder to avoid CPU spike
            sbClassIDs.Append(",");
          }
          // SE: Bug 57861 - use StringBuilder to avoid CPU spike
          sbClassIDs.Append(XPConvert.ToString(m_arrObjectClassIDs[i]));
        }
      }
      //SE: logging
      log.FunctionEnd("SelectPTObjectModel::GetObjectClassIDs");
      // SE: Bug 57861 - use StringBuilder to avoid CPU spike
      return sbClassIDs.ToString();
    }
    
    /// <summary>
    /// Returns the icon to display on each row before the
    /// input text.
    /// If the value is null, no icon is displayed.
    /// </summary>
    /// <returns>PTImageType</returns>
    public virtual PTImageType GetDefaultRowIcon()
    {
      return m_imgDefaultRowIcon;
    }
    
    /// <summary>
    /// Returns the icon to display on each row before the
    /// input text, given a class id.  If the value is null
    /// then we get the default row icon.
    /// </summary>
    /// <returns>PTImageType</returns>
    public virtual PTImageType GetRowIconByClassID(int _nClassID)
    {
      // RWW: since we don't use this all the time
      // let's do lazy set/gets, save memory in init.
      if (null == m_xphtRowIcons)
      {
        return m_imgDefaultRowIcon;
      }
      // RWW: if we didn't specify a specific image
      // for this class id, then just return the default
      // SE: Bug 57861 - use XPConvert for speed
      if (!m_xphtRowIcons.ContainsKey(XPConvert.ToString(_nClassID)))
      {
        return m_imgDefaultRowIcon;
      }
      // RWW: ah, we have a specific image, use it.
      // SE: Bug 57861 - use XPConvert for speed
      return (PTImageType) m_xphtRowIcons.GetElement(XPConvert.ToString(_nClassID));
      }
    
    /// <summary>
    /// Returns a String of Class IDs and Object IDs of the Objects
    /// to select in the tree.
    /// E.g.: "classID1,ObjID1,classID2,objID2"
    /// This is the format used by the tree.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetSelectedItems()
    {
      //SE: logging
      log.FunctionBegin("SelectPTObjectModel::GetSelectedItems");
      // SE: Bug 57861 - use StringBuilder to avoid CPU spike
      XPStringBuilder sbSelectedItems = new XPStringBuilder(6 * m_saData.GetCount());
      // JF- We loop through the array of items and for each one, 
      // add the class ID and the object ID to the String.
      for (int i = 0;i < m_saData.GetCount();i++)
      {
        // JS- Add the separator
        if (i > 0)
        {
          // SE: Bug 57861 - use StringBuilder to avoid CPU spike
          sbSelectedItems.Append(",");
        }
        // SE: Bug 57861 - use StringBuilder to avoid CPU spike
        sbSelectedItems.Append(m_saData.GetItem(i, EXPLIST_COL_CLASSID));
        sbSelectedItems.Append(",");
        sbSelectedItems.Append(m_saData.GetItem(i, EXPLIST_COL_OBJECTID));
        }
      //SE: logging
      log.FunctionEnd("SelectPTObjectModel::GetSelectedItems");
      // SE: Bug 57861 - use StringBuilder to avoid CPU spike
      return sbSelectedItems.ToString();
    }
    
    /// <summary>
    /// Helper method to get a String with the selected Object names.
    /// E.g.: User1, User2, Group1
    /// Will return an empty String if no Object was selected.
    /// </summary>
    /// <returns>HTMLElementCollection</returns>
    public virtual HTMLElementCollection GetSelectedNamesAsHTML()
    {
      //SE: logging
      log.FunctionBegin("SelectPTObjectModel::GetSelectedNamesAsHTML");
      HTMLElementCollection result = new HTMLElementCollection();
      HTMLB myBold;
      String sName;
      for (int i = 0;i < m_saData.GetCount();i++)
      {
        sName = (String) m_saData.GetItem(i, EXPLIST_COL_STRING);
        if ((null != sName) && (0 != sName.Length))
        {
          myBold = new HTMLB();
          myBold.AddInnerHTMLString(XPHttpUtility.HtmlEncode(sName));
          result.AddInnerHTMLElement(myBold);
          result.AddInnerHTMLElement(new HTMLBr());
        }
      }
      //SE: logging
      log.FunctionEnd("SelectPTObjectModel::GetSelectedNamesAsHTML");
      return result;
    }
    
    /// <summary>
    /// Returns the leaf type of the tree.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetTreeLeafType()
    {
      return m_treeLeafType;
    }
    
    /// <summary>
    /// Returns the selection type of the tree.
    /// Converts the boolean value to a string so that it can be
    /// put in the query string for the tree.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetTreeMultiSelect()
    {
      return XPConvert.ToString(m_bTreeMultiSelect);
    }
    
    /// <summary>
    /// Returns the query filter for the tree.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetTreeQueryFilter()
    {
      return m_strTreeQueryFilter;
    }
    
    /// <summary>
    /// Returns tree AS ID.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetTreeSpaceID()
    {
      return m_sTreeSpaceID;
    }
    
    /// <summary>
    /// Resets the tree ID, this way a new tree will be opened. 
    /// This can be used to switch from a multi select tree to a single select tree for example.
    /// </summary>
    public virtual void ResetTreeID()
    {
      m_sTreeSpaceID = "";
    }
    
    /// <summary>
    /// Returns the Tree sub title.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetTreeSubTitle()
    {
      return m_sTreeSubTitle;
    }
    
    /// <summary>
    /// Returns the Tree title.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetTreeTitle()
    {
      return m_sTreeTitle;
    }
    
    /// <summary>
    /// Initializes the read only expandable list.
    /// </summary>
    /// <param name="_sSectionTitle">Main title.</param>
    /// <param name="_sSectionSubTitle">Sub title</param>
    /// <param name="_arrObjectClassIDs">Class ID of the Objects displayed on the list.</param>
    /// <param name="_nListKey">
    /// In case you have multiple Expandable Lists on
    /// the same DP, they should have a different key (otherwise, just set it to 1).
    /// </param>
    public virtual void InitializeReadOnlySelectPTObject(String _sSectionTitle,
     String _sSectionSubTitle, int[] _arrObjectClassIDs, int _nListKey)
    {
      Object[][] arrExpListColInfo = new Object[1][];
      for (int j2c_index = 0;j2c_index < 1;j2c_index++)
      {
        arrExpListColInfo[j2c_index] = new Object[5];
      }
      arrExpListColInfo[0][EXPLIST_COLUMN_ID] = EXPLIST_COL_STRING;
      arrExpListColInfo[0][EXPLIST_COLUMN_TITLE] = _sSectionSubTitle;
      arrExpListColInfo[0][EXPLIST_COLUMN_WIDTH] = "100%";
      arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN] = "1";
      arrExpListColInfo[0][EXPLIST_COLUMN_ENABLESORT] = true;
      m_arrObjectClassIDs = _arrObjectClassIDs;
      base.InitializeReadOnlyExpandableList(_sSectionTitle, arrExpListColInfo,
       _nListKey);
    }
    
    /// <summary>
    /// Initializes the expandable list.
    /// </summary>
    /// <param name="_sSectionTitle">Main title.</param>
    /// <param name="_sSectionSubTitle">Sub title</param>
    /// <param name="_imgAddIcon">Icon for the add action.</param>
    /// <param name="_sAddLabel">Text to display next to the add icon.</param>
    /// <param name="_arrObjectClassIDs">Class ID of the Objects displayed on the list.</param>
    /// <param name="_nListKey">
    /// In case you have multiple Expandable Lists on
    /// the same DP, they should have a different key (otherwise, just set it to 1).
    /// </param>
    public virtual void InitializeSelectPTObject(String _sSectionTitle, String _sSectionSubTitle,
     PTImageType _imgAddIcon, String _sAddLabel, int[] _arrObjectClassIDs,
     int _nListKey)
    {
      Object[][] arrExpListColInfo = new Object[1][];
      for (int j2c_index = 0;j2c_index < 1;j2c_index++)
      {
        arrExpListColInfo[j2c_index] = new Object[5];
      }
      arrExpListColInfo[0][EXPLIST_COLUMN_ID] = EXPLIST_COL_STRING;
      arrExpListColInfo[0][EXPLIST_COLUMN_TITLE] = _sSectionSubTitle;
      arrExpListColInfo[0][EXPLIST_COLUMN_WIDTH] = "100%";
      arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN] = "1";
      arrExpListColInfo[0][EXPLIST_COLUMN_ENABLESORT] = true;
      m_arrObjectClassIDs = _arrObjectClassIDs;
      base.InitializeExpandableList(_sSectionTitle, arrExpListColInfo, _imgAddIcon,
       _sAddLabel, _nListKey);
    }
    
    /// <summary>
    /// Partialally initializes the expandable list. InitializeExpandableList must also be called when this method is used.
    /// </summary>
    /// <param name="_arrObjectClassIDs">Class ID of the Objects displayed on the list.</param>
    public virtual void InitializeListClassIDs(int[] _arrObjectClassIDs)
    {
      m_arrObjectClassIDs = _arrObjectClassIDs;
    }
    
    /// <summary>
    /// Initalizes the list (PTGrowableSortedArrayWrapper)
    /// with a column mapping and an array of sort.
    /// The growable array will always be sorted by the 1st
    /// column ascending.
    /// </summary>
    /// <param name="_saData"></param>
    public override void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData)
    {
      //SE: logging
      log.FunctionBegin("SelectPTObjectModel::InitializeSortedArray");
      // JF- Column mapping
      int[] arrColumns = new int[]{EXPLIST_SORTEDARRAY_PROPID_INDEX, EXPLIST_COL_CLASSID,
       EXPLIST_COL_OBJECTID, EXPLIST_COL_STRING};
      // JF- Sort array
      // The growable array will always be sorted by the String column (the one displayed).
      int[][] arrSort = new int[1][];
      for (int j2c_index = 0;j2c_index < 1;j2c_index++)
      {
        arrSort[j2c_index] = new int[2];
      }
      arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID] = EXPLIST_COL_STRING;
      arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
      // JF- We don't initalize the list itself (null below)
      // At first it's empty and we will be initalized if needed using the 
      // AddRowsToList method.
      _saData.Initialize(arrColumns, null, arrSort, true, EXPLIST_DEFAULT_INITIAL_SIZE);
      //SE: logging
      log.FunctionEnd("SelectPTObjectModel::InitializeSortedArray");
    }
    
    /// <summary>
    /// Returns true if it should be possible to edit Objects from this class ID.
    /// </summary>
    /// <param name="_nClassID"></param>
    public virtual bool IsEditLinkEnabled(int _nClassID)
    {
      if (null != m_arrCreateNewInfo)
      {
        int nClassIDFromArray;
        for (int i = 0;i < m_arrCreateNewInfo.Length;i++)
        {
          nClassIDFromArray = XPConvert.ToInteger(m_arrCreateNewInfo[i][CREATE_NEW_COL_CLASSID]);
          if (nClassIDFromArray == _nClassID)
          {
            return true;
          }
        }
      }
      return false;
    }
    
    /// <summary>
    /// Sets the nAccessLevel. The accesslevel of the objects for the tree control to display.
    /// </summary>
    /// <param name="nAccessLevel">The nAccessLevel to set</param>
    public virtual void SetAccessLevel(int nAccessLevel)
    {
      m_nAccessLevel = nAccessLevel;
    }
    
    /// <summary>
    /// Sets the community mode for the tree.
    /// By default there is no community mode.
    /// </summary>
    /// <param name="_nCommunityMode"></param>
    public virtual void SetCommunityMode(int _nCommunityMode)
    {
      m_nCommunityMode = _nCommunityMode;
    }
    
    /// <summary>
    /// Sets the community folder id for the tree.
    /// By default there is no community folder id.
    /// </summary>
    /// <param name="_nCommunityFolderID"></param>
    public virtual void SetCommunityFolderID(int _nCommunityFolderID)
    {
      m_nCommunityFolderID = _nCommunityFolderID;
    }
    
    /// <summary>
    /// Add the array passed in argument to the list.
    /// The array should be created using the EXPLIST_COL
    /// constants defined in this class.
    /// </summary>
    /// <param name="_arrRows">Array of rows to add.</param>
    public virtual void SetDefaultObjects(Object[][] _arrRows)
    {
      //SE: logging
      log.FunctionBegin("SelectPTObjectModel::SetDefaultObjects");
      base.GetItems().AddRows(_arrRows);
      //SE: logging
      log.FunctionEnd("SelectPTObjectModel::SetDefaultObjects");
    }
    
    /// <summary>
    /// Enables the create new links for the Class ID in argument.
    /// Doing this will also enbale the links to Edit the Objects in the list.
    /// </summary>
    /// <param name="_arrCreateNewInfo">Information for each create new link</param>
    public virtual void SetEnableCreateNew(Object[][] _arrCreateNewInfo)
    {
      if (_arrCreateNewInfo[0].Length < CREATE_NEW_COL_MAX)
      {
        throw new XPException("SelectPTObjectModel::SetEnableCreateNew The create new information array" + " should contain " + CREATE_NEW_COL_MAX + " columns.");
        }
      m_arrCreateNewInfo = _arrCreateNewInfo;
    }
    
    /// <summary>
    /// Because this select ptobject list needs to support both move
    /// row actions enable and disable, we need to change the
    /// colspan of the header row when this happens.
    /// </summary>
    /// <param name="_bEnableMoveRowActions"></param>
    public override void SetEnableMoveRowActions(bool _bEnableMoveRowActions)
    {
      base.SetEnableMoveRowActions(_bEnableMoveRowActions);
      if (m_bEnableMoveRowActions)
      {
        // JF- If the move row actions are enabled, this will 
        // create one more cell at the end of each row and 
        // we need to reflect that change on the header row.
        m_arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN] = "2";
      } else
      {
        m_arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN] = "1";
      }
    }
    
    /// <summary>
    /// Clears the hidden objects for the tree.
    /// </summary>
    public virtual void ClearHiddenObjects()
    {
      m_strObjectsToHide = "";
    }
    
    /// <summary>
    /// Sets the hidden objects for the tree.
    /// By default there are no hidden objects.
    /// </summary>
    /// <param name="_anClassIDObjectID"></param>
    public virtual void SetHiddenObjects(int[][] _anClassIDObjectID)
    {
      //SE: logging
      log.FunctionBegin("SelectPTObjectModel::SetHiddenObjects");
      XPStringBuilder sb = new XPStringBuilder(6 * _anClassIDObjectID.Length);
      for (int i = 0;i < _anClassIDObjectID.Length;i++)
      {
        if (i > 0)
        {
          sb.Append(",");
        }
        sb.Append(_anClassIDObjectID[i][HIDE_COL_CLASSID]).Append(",").Append(_anClassIDObjectID[i][HIDE_COL_OBJECTID]);
        }
      if (0 != m_strObjectsToHide.Length)
      {
        m_strObjectsToHide = m_strObjectsToHide + "," + sb.ToString();
      } else
      {
        m_strObjectsToHide = sb.ToString();
      }
      //SE: logging
      log.FunctionEnd("SelectPTObjectModel::SetHiddenObjects");
    }
    
    /// <summary>
    /// Sets the read only objects.
    /// Those object won't appear in the tree and the user will not be able to delete them.
    /// </summary>
    /// <param name="_arrReadOnlyObjects"></param>
    public virtual void SetReadOnlyObjects(int[][] _arrReadOnlyObjects)
    {
      m_arrReadOnlyObjects = _arrReadOnlyObjects;
      SetHiddenObjects(m_arrReadOnlyObjects);
    }
    
    /// <summary>
    /// Returns true if the object identified by the class id 
    /// and the object id in argument is read only.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_nObjectID"></param>
    /// <returns>boolean</returns>
    public virtual bool IsObjectReadOnly(int _nClassID, int _nObjectID)
    {
      bool bIsReadOnly = false;
      if (null != m_arrReadOnlyObjects)
      {
        for (int i = 0;i < m_arrReadOnlyObjects.Length;i++)
        {
          if ((_nClassID == m_arrReadOnlyObjects[i][READ_ONLY_ROW_CLASSID]) && (_nObjectID == m_arrReadOnlyObjects[i][READ_ONLY_ROW_OBJECTID]))
          {
            bIsReadOnly = true;
            break;
          }
        }
      }
      return bIsReadOnly;
    }
    
    /// <summary>
    /// Sets the array of objects to display in the tree. 
    /// (Only those objects will be display in the tree.)
    /// The array in argument should be in the following format:
    /// classid1,objectid1,classid2,objectid2....
    /// </summary>
    /// <param name="_sDisplayObjects"></param>
    public virtual void SetDisplayObjects(String _sDisplayObjects)
    {
      m_sDisplayObjects = _sDisplayObjects;
    }
    
    /// <summary>
    /// Returns the array of objects to display in the tree.
    /// </summary>
    /// <param name="String"></param>
    public virtual String GetDisplayObjects()
    {
      return m_sDisplayObjects;
    }
    
    /// <summary>
    /// Returns the value of the m_bNoRowIcons member variable.  True if no
    /// icons are to be displayed on each selected row.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetNoRowIcons()
    {
      return m_bNoRowIcons;
    }
    
    /// <summary>
    /// Sets the m_bNoRowIcons member variable.  True if no icons are to be
    /// displayed on each selected row.
    /// </summary>
    /// <param name="m_bNoRowIcons"></param>
    public virtual void SetNoRowIcons(bool m_bNoRowIcons)
    {
      this.m_bNoRowIcons = m_bNoRowIcons;
    }
    
    /// <summary>
    /// Sets the icon to display on each row before the
    /// input text.
    /// By default, no icon is displayed.
    /// </summary>
    /// <param name="_imgRowIcon"></param>
    public virtual void SetDefaultRowIcon(PTImageType _imgRowIcon)
    {
      m_imgDefaultRowIcon = _imgRowIcon;
    }
    
    /// <summary>
    /// Sets the icon to display on each row before the
    /// input text, given a class id.  By default, all
    /// class ids are associated with empty images. Call
    /// SetDefaultRowIcon to set a universal default icon.
    /// </summary>
    /// <param name="_nClassID">the class id to be associated with this icon</param>
    /// <param name="_imgRowIcon">the icon you want to be displayed</param>
    public virtual void SetRowIconByClassID(int _nClassID, PTImageType _imgRowIcon)
    {
      // RWW: since we do lazy get/sets to save memory
      // in init, we need to make sure we have an object
      if (null == m_xphtRowIcons)
      {
        m_xphtRowIcons = new XPHashtable();
      }
      // SE: Bug 57861 - use XPConvert for speed
      m_xphtRowIcons.PutElement(XPConvert.ToString(_nClassID), _imgRowIcon);
      }
    
    /// <summary>
    /// Sets the leaf type for the tree.
    /// Is set to N_LEAFSELECT by default.
    /// </summary>
    /// <param name="_leafType"></param>
    public virtual void SetTreeLeafType(int _leafType)
    {
      m_treeLeafType = _leafType;
    }
    
    /// <summary>
    /// Sets the selection type for the tree.
    /// Multiselect is set to true by default.
    /// </summary>
    /// <param name="_selectType"></param>
    public virtual void SetTreeMultiSelect(bool _selectType)
    {
      m_bTreeMultiSelect = _selectType;
    }
    
    /// <summary>
    /// Sets the query filter for the tree.
    /// By default there is no query filter.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_nPropID"></param>
    /// <param name="_anValues"></param>
    public virtual void SetTreeQueryFilter(int _nClassID, int _nPropID, int[] _anValues)
    {
      XPStringBuilder sb = new XPStringBuilder(20);
      sb.Append(_nClassID).Append(",").Append(_nPropID);
      for (int i = 0;i < _anValues.Length;i++)
      {
        sb.Append(",").Append(_anValues[i]);
      }
      m_strTreeQueryFilter = sb.ToString();
    }
    
    /// <summary>
    /// Sets the Tree sub title.
    /// </summary>
    /// <param name="_sTreeSubTitle"></param>
    public virtual void SetTreeSubTitle(String _sTreeSubTitle)
    {
      m_sTreeSubTitle = _sTreeSubTitle;
    }
    
    /// <summary>
    /// Sets the Tree title.
    /// </summary>
    /// <param name="_sTreeTitle"></param>
    public virtual void SetTreeTitle(String _sTreeTitle)
    {
      m_sTreeTitle = _sTreeTitle;
    }
    
    /// <summary>
    /// Saves the data on the row _nRowIndex.
    /// </summary>
    /// <param name="_saData">List to update the data on row _nRowIndex</param>
    /// <param name="_nRowIndex">Index of the row to save.</param>
    /// <param name="_htFormData">Data posted by the form.</param>
    /// <returns>int Status for server side validation.</returns>
    protected override int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex,
     XPHashtable _htFormData)
    {
      // JF- Nothing to save on repost.
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Helper method to save the Tree ID on repost.
    /// </summary>
    /// <param name="_htFormData">Form data</param>
    protected virtual void SaveTreeID(XPHashtable _htFormData)
    {
      // JF- We save the tree ID only if we don't already have it.
      if (("").Equals(m_sTreeSpaceID) && _htFormData.ContainsKey(GetUniqueString(SelectPTObjectView.HTMLINPUT_TREEASID)))
      {
        String[] data = (String[]) _htFormData.GetElement(GetUniqueString(SelectPTObjectView.HTMLINPUT_TREEASID));
        if ((null != data) && !(0 == data.Length))
        {
          m_sTreeSpaceID = data[0];
        }
      }
    }
    
    /// <summary>
    /// Helper method to create a temporary array of index mapping.
    /// On each row we have the following info: class ID - Object ID - index
    /// This method is used to not loose the order when the move row actions are enabled.
    /// </summary>
    /// <param name="_saItems">List items</param>
    /// <returns>Object[][]</returns>
    protected virtual Object[][] CreateIndexMapping(PTGrowableSortedArrayWrapper _saItems)
    {
      Object[][] arrIndexMapping = new Object[_saItems.GetCount()][];
      for (int j2c_index = 0;j2c_index < _saItems.GetCount();j2c_index++)
      {
        arrIndexMapping[j2c_index] = new Object[3];
      }
      for (int i = 0;i < _saItems.GetCount();i++)
      {
        if (null != _saItems.GetItem(i, EXPLIST_SORTEDARRAY_PROPID_INDEX))
        {
          arrIndexMapping[i][0] = _saItems.GetItem(i, EXPLIST_COL_CLASSID);
          arrIndexMapping[i][1] = _saItems.GetItem(i, EXPLIST_COL_OBJECTID);
          arrIndexMapping[i][2] = _saItems.GetItem(i, EXPLIST_SORTEDARRAY_PROPID_INDEX);
          }
      }
      return arrIndexMapping;
    }
    
    /// <summary>
    /// Helper method to get the EditorAS object name from the class ID.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <returns>String</returns>
    private String GetEditorASName(int _nClassID)
    {
      String sASName = null;
      int nClassIDFromArray;
      for (int i = 0;i < m_arrCreateNewInfo.Length;i++)
      {
        nClassIDFromArray = XPConvert.ToInteger(m_arrCreateNewInfo[i][CREATE_NEW_COL_CLASSID]);
        if (nClassIDFromArray == _nClassID)
        {
          sASName = (String) m_arrCreateNewInfo[i][CREATE_NEW_COL_ASNAME];
          break;
        }
      }
      return sASName;
    }
    
    /// <summary>
    /// Helper method to get the value of the index for the object identified
    /// by the object ID and class ID in argument.
    /// Returns -1 if the object was not in the array.
    /// </summary>
    /// <param name="_arrIndexMapping">Index mapping</param>
    /// <param name="_nClassID">Object class ID</param>
    /// <param name="_nObjectID">Object ID</param>
    /// <returns>int Value of the index</returns>
    protected virtual int GetIndex(Object[][] _arrIndexMapping, int _nClassID,
     int _nObjectID)
    {
      int nIndex = -1;
      int nRowClassID;
      int nRowObjectID;
      for (int i = 0;i < _arrIndexMapping.Length;i++)
      {
        if ((null != _arrIndexMapping[i][0]) && (null != _arrIndexMapping[i][1]) && (null != _arrIndexMapping[i][2]))
        {
          nRowClassID = XPConvert.ToInteger(_arrIndexMapping[i][0]);
          nRowObjectID = XPConvert.ToInteger(_arrIndexMapping[i][1]);
          if ((_nClassID == nRowClassID) && (_nObjectID == nRowObjectID))
          {
            nIndex = XPConvert.ToInteger(_arrIndexMapping[i][2]);
            break;
          }
        }
      }
      return nIndex;
    }
  }
}