using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.classtypedescriptors;
using com.plumtree.portaluiinfrastructure.helpers;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.friendlyurl
{
  /// <summary>
  /// This class represents a friendly Url and provides an interface to generate 
  /// friendly Url string based on the provided information. 
  /// </summary>
  /// <author>rozhang</author>
  public class FriendlyURL
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_FRIENDLYURL),
     "com.plumtree.portaluiinfrastructure.friendlyurl.FriendlyURL");
    
    /// <summary>
    /// Type of destination page the url links too
    /// </summary>
    protected int m_classId = 0;
    
    /// <summary>
    /// objectName Name of the object to generate friendly name to (either one of name or id must be provided)
    /// </summary>
    protected String m_objectName = null;
    
    /// <summary>
    /// Id of the object to generate friendly name to (either one of name or id must be provided)
    /// </summary>
    protected int m_objectId = 0;
    
    /// <summary>
    /// String of query parameters that need to be attached at the end of the friendly URL
    /// </summary>
    protected String m_queryParams = null;
    
    /// <summary>
    /// Additional name specifier, used for example for Urls to community pages which also needs the community name. Optional and not used by most class types
    /// </summary>
    protected String m_childName = null;
    
    /// <summary>
    /// Additional id specifier, used for example for Urls to community pages which also needs the community name. Optional and not used by most class types
    /// </summary>
    protected int m_childId = 0;
    
    /// <summary>
    /// Whether to generate the object id in the URL if the name is also available
    /// </summary>
    protected bool m_useIdIfAvailable = false;
    
    /// <summary>
    /// Constructor with object name as argument
    /// </summary>
    /// <param name="classId"></param>
    public FriendlyURL(int classId, String objectName) : this(classId, objectName,
     0, null)
    {
    }
    public FriendlyURL(int classId, String objectName, int objectId) : this(classId,
     objectName, objectId, null)
    {
    }
    
    /// <summary>
    /// Constructor with object id as argument
    /// </summary>
    /// <param name="classId"></param>
    public FriendlyURL(int classId, int objectId) : this(classId, null, objectId,
     null)
    {
    }
    public FriendlyURL(int classId, String objectName, int objectId, String queryParams)
    {
      if (log.IsDebugEnabled())
      {
        log.Debug("Creating friendly Url object with classid: " + classId + " objectid: " + objectId + "and name: " + objectName);
        }
      m_classId = classId;
      m_objectName = objectName;
      m_objectId = objectId;
      if (null != queryParams && XPStringUtility.StartsWith(queryParams, "&"))
      {
        m_queryParams = XPStringUtility.Substring(queryParams, 1);
      } else
      {
        m_queryParams = queryParams;
      }
    }
    
    /// <summary>
    /// Constructs the Friendly URL and returns it as a string. Only the friendly
    /// url part is returned, no the application base url part or server.pt/
    /// Note that any application base url used with friendly url cannot have a 
    /// trailing question mark.
    /// </summary>
    /// <param name="space">Current ActivitySpace</param>
    /// <returns>For example /community/engineeringcommunity/page2</returns>
    public virtual String GetFriendlyURL(AActivitySpace space)
    {
      String friendlyURL = ((IFriendlyURLManager) PTConfigService.getPTInstance().GetPTClassTypeDescrManager()).ConstructFriendlyURL(this,
       space);
      if (null != friendlyURL && null != m_queryParams && !m_queryParams.Equals(""))
      {
        friendlyURL = friendlyURL + "?" + m_queryParams;
      }
      return friendlyURL;
    }
    
    /// <summary>
    /// Constructs the Friendly URL and returns it as a string. The application base url 
    /// (without trailing question mark) is included.
    /// </summary>
    /// <param name="space">Current ActivitySpace</param>
    /// <returns>For example http://portalserver:8080/portal/server.pt/community/engineeringcommunity/page2</returns>
    public virtual String GetFullFriendlyURL(AActivitySpace space)
    {
      String baseURL = PTConfigService.getPTInstance().GetApplicationBaseURL(space.GetCurrentNormalizedBaseURL());
      baseURL = GeneralHelpers.Replace(baseURL, "?", "");
      String friendlyURL = GetFriendlyURL(space);
      if (null != friendlyURL)
      {
        return baseURL + friendlyURL;
      } else
      {
        return null;
      }
    }
    
    /// <summary>
    /// Convert PTObject name into a format that is suitable for friendly Urls.
    /// Use <c>FriendlyURL#DeNormalizeObjectName</c> to convert the value back.
    /// The casing of the object name is not preserved through the conversion    
    /// NOTE: This method is duplicated on ObjectIdFromNameCache class in the server. Make sure to update
    /// both methods.
    /// </summary>
    public static String NormalizeObjectName(String objectName)
    {
      // TODO CR DLH: Shouldn't we just do this using GeneralHelpers.Replace,
      // since it's cleaner?  Then we can worry about optimizations if a code
      // profiler shows that this is slow?
      // ROB what's wrong with writing optimized code in the first place?
      // creating extra String are always wasteful and this is scales if 
      // we need additional replacements 
      // DLH: In this case it's fairly okay, although often optimized code is harder
      // to read and maintain.  Any optimized code should have a comment describing
      // what it's doing, since it's almost always harder to read than just
      // String.replace(' ', '_');  At any rate, please add a descriptive comment.
      XPStringBuilder sb = new XPStringBuilder();
      char[] chars = XPStringUtility.GetChars(objectName);
      for (int i = 0;i < chars.Length;i++)
      {
        char c = chars[i];
        switch (c){
        case ' ':
        case '?':
        case '<':
        case '>':
        case '&':
        case '^':
        case '#':
        case '%':
        case '\"':
        case '\\':
        case '*':
        case '/':
        case ':':
        case '.':
        sb.Append('_');
        break;
        default:
        sb.Append(c);
        break;
        }
      }
      String ret = XPStringUtility.ToLowerCase(sb.ToString());
      return ret;
    }
    
    /// <summary>
    /// Reverses any character replacement performed in <c>FriendlyURL#NormalizeObjectName</c>,
    /// except for character casing. All characters will be lower case.
    /// NOTE: This method is duplicated on ObjectIdFromNameCache class in the server. Make sure to update
    /// both methods.
    /// </summary>
    public static String DeNormalizeObjectName(String objectName)
    {
      // CR DLH: Shouldn't we just do this using GeneralHelpers.Replace,
      // since it's cleaner?  Then we can worry about optimizations if a code
      // profiler shows that this is slow?
      // ROB what's wrong with writing optimized code in the first place?
      // creating extra String are always wasteful and this is scales if 
      // we need additional replacements
      XPStringBuilder sb = new XPStringBuilder();
      char[] chars = XPStringUtility.GetChars(objectName);
      for (int i = 0;i < chars.Length;i++)
      {
        char c = chars[i];
        switch (c){
        case '_':
        case '?':
        case '<':
        case '>':
        case '&':
        case '^':
        case '#':
        case '%':
        case '\"':
        case '\\':
        case '*':
        case '/':
        case ':':
        case '.':
        sb.Append(' ');
        break;
        /*case '/':
         sb.Append('\\');
         break;*/
         
        default:
        sb.Append(c);
        break;
        }
      }
      return sb.ToString();
    }
    //return GeneralHelpers.Replace(objectName, "_", " ");
    public virtual String GetChildName()
    {
      return m_childName;
    }
    public virtual void SetChildName(String name)
    {
      m_childName = name;
    }
    public virtual int GetClassId()
    {
      return m_classId;
    }
    public virtual String GetClassIdAsString()
    {
      return m_classId + "";
    }
    public virtual void SetClassId(int id)
    {
      m_classId = id;
    }
    public virtual int GetObjectId()
    {
      return m_objectId;
    }
    public virtual void SetObjectId(int id)
    {
      m_objectId = id;
    }
    public virtual String GetOjectName()
    {
      return m_objectName;
    }
    public virtual void SetObjectName(String name)
    {
      m_objectName = name;
    }
    public virtual bool GetUseIdIfAvailable()
    {
      return m_useIdIfAvailable;
    }
    public virtual void SetUseIdIfAvailable(bool useidIfAvailable)
    {
      m_useIdIfAvailable = useidIfAvailable;
    }
    public virtual int GetChildId()
    {
      return m_childId;
    }
    public virtual void SetChildId(int id)
    {
      m_childId = id;
    }
  }
}