using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.portaluiinfrastructure.friendlyurl
{
  public class FriendlyURLParserManager : IFriendlyURLParserManager
  {
    // DCA - the only instance to be used
    private static IFriendlyURLParserManager INSTANCE;
    // DCA - boolean to indicate whether we are in production mode or test mode
    private static bool m_bIsManagerProduction = false;
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_FRIENDLYURL),
     "com.plumtree.portaluiinfrastructure.friendlyurl.FriendlyURLParserManager");
    // DCA - Map of keywords to class ids taken from FriendlyURLS.xml
    protected XPHashtable m_htKeyToClassIdLookup = null;
    
    /// <returns>The singleton manager for retrieving parsers.</returns>
    public static IFriendlyURLParserManager getInstance()
    {
      if (INSTANCE == null)
      {
        m_bIsManagerProduction = true;
        INSTANCE = new FriendlyURLParserManager();
      }
      return INSTANCE;
    }
    
    /// <summary>
    /// Make constructor private so that no other class can create an instance of this class.
    /// </summary>
    private FriendlyURLParserManager()
    {
    }
    /*
     * (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.friendlyurl.IFriendlyURLParserManager#SetKeyToClassIdLookupTable(com.plumtree.openfoundation.util.XPHashtable)
     */
     
    public virtual void SetKeyToClassIdLookupTable(XPHashtable htKeyToClassIdLookup)
    {
      m_htKeyToClassIdLookup = htKeyToClassIdLookup;
    }
    /*
     * (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.friendlyurl.IFriendlyURLParserManager#GetParser(com.plumtree.openfoundation.util.XPStringTokenizer)
     */
     
    public virtual IFriendlyURLParser GetParser(XPStringTokenizer stURLTokens)
    {
      // DCA - basic friendly URL structure will have /keyword/object name/
      String strKeyword = stURLTokens.GetNextToken();
      //  DCA - make sure to decode since it was part of the URL
      try
      {
        strKeyword = XPHttpUtility.UrlDecode(strKeyword, LinkConstants.UTF_8_ENCODING);
        }
      catch (XPUnsupportedEncodingException e)
      {
        // DCA - object name must've not been encoded. Log exception and move on.
        log.Debug(e, GetType().FullName + ":GetParser The string: " + strKeyword + " could not be decoded.");
        }
      String strClassId = FindClassID(strKeyword);
      // NOTE: If there gete to be too many cases in this switch statement, we should
      // probably make the class type descriptor return the appropriate URL parser.
      FriendlyURLParser parser;
      if (null == strClassId)
      {
        parser = new FriendlyURLParser(strKeyword, null, stURLTokens);
        // DCA - check if non pt object friendly URL
        // Certain non pt object friendly URLs have the entire path as the keyword so that customers
        // can change the any part of the path.
        if (null != parser.GetObjectName(false))
        {
          strKeyword+="/" + parser.GetObjectName(false);
          parser.SetClassId(FindClassID(strKeyword));
        }
        return parser;
      }
      // DCA - the FriendlyURLParser will parse the rest
      switch (XPConvert.ToInteger(strClassId)){
      case PT_CLASSIDS.PT_COMMUNITY_ID:
      log.Debug("FriendlyURLParserManager:GetParser returning CommunityFriendlyURLParser");
      return new CommunityFriendlyURLParser(strKeyword, strClassId, stURLTokens);
      goto case PT_CLASSIDS.PT_CATALOGFOLDER_ID;
      case PT_CLASSIDS.PT_CATALOGFOLDER_ID:
      log.Debug("FriendlyURLParserManager:GetParser returning KDFriendlyURLParser");
      return new KDFriendlyURLParser(strKeyword, strClassId, stURLTokens);
      goto case PT_CLASSIDS.PT_MYPAGESCONTAINER_ID;
      case PT_CLASSIDS.PT_MYPAGESCONTAINER_ID:
      log.Debug("FriendlyURLParserManager:GetParser returning MyPageFriendlyURLParser");
      return new MyPageFriendlyURLParser(strKeyword, strClassId, stURLTokens);
      goto case PT_CLASSIDS.PT_CATALOGCARD_ID;
      case PT_CLASSIDS.PT_CATALOGCARD_ID:
      log.Debug("FriendlyURLParserManager:GetParser returning DocumentFriendlyURLParser");
      return new DocumentFriendlyURLParser(strKeyword, strClassId, stURLTokens);
      goto case PT_CLASSIDS.PT_USER_ID;
      case PT_CLASSIDS.PT_USER_ID:
      log.Debug("FriendlyURLParserManager:GetParser returning UserFriendlyURLParser");
      return new UserFriendlyURLParser(strKeyword, strClassId, stURLTokens);
      goto case FriendlyURLConstants.REST_API_ID;
      case FriendlyURLConstants.REST_API_ID:
      log.Debug("FriendlyURLParserManager:GetParser returning APIFriendlyURLParser");
      return new APIFriendlyURLParser(strKeyword, strClassId, stURLTokens);
      goto case FriendlyURLConstants.FRIENDLY_SEARCH_ID;
      case FriendlyURLConstants.FRIENDLY_SEARCH_ID:
      log.Debug("FriendlyURLParserManager:GetParser returning SearchFriendlyURLParser");
      return new SearchFriendlyURLParser(strKeyword, strClassId, stURLTokens);
      goto case FriendlyURLConstants.FRIENDLY_RSS_ID;
      case FriendlyURLConstants.FRIENDLY_RSS_ID:
      log.Debug("FriendlyURLParserManager:GetParser returning RSSFriendlyURLParser");
      return new RSSFriendlyURLParser(strKeyword, strClassId, stURLTokens);
      goto default;
      default:
      log.Debug("FriendlyURLParserManager:GetParser returning FriendlyURLParser");
      parser = new FriendlyURLParser(strKeyword, strClassId, stURLTokens);
      // DCA - check if non object friendly URL
      if (null != parser.GetObjectName(false))
      {
        strKeyword+="/" + parser.GetObjectName(false);
        parser.SetClassId(FindClassID(strKeyword));
      }
      return parser;
      break;
      }
    }
    
    /// <summary>
    /// Returns the correct class Id based on the keyword passed in.
    /// </summary>
    /// <param name="strClassTypeKeyword"></param>
    /// <returns></returns>
    private String FindClassID(String strClassTypeKeyword)
    {
      log.Debug("FriendlyURLPlugin:FindClassID  Using key: " + strClassTypeKeyword);
      if (null == m_htKeyToClassIdLookup)
      {
        throw new XPException("The FriendlyURLParserManager class was not initialized properly.");
        }
      String strClassId = "";
      try
      {
        strClassId = (String) m_htKeyToClassIdLookup.GetElement(strClassTypeKeyword);
        log.Debug("FriendlyURLParserManager:FindClassID  Found: " + strClassId);
        }
      catch (XPException e)
      {
        log.Error(e, "FriendlyURLParserManager:FindClassID Could not find class Id from keyword: " + strClassTypeKeyword);
        strClassId = null;
      }
      return strClassId;
    }
    
    /// <param name="typeId">Class id or type id</param>
    /// <returns></returns>
    public virtual String FindKeywordForType(String typeId)
    {
      if (typeId == null)
      {
        return null;
      }
      if (null == m_htKeyToClassIdLookup)
      {
        throw new XPException("The FriendlyURLParserManager class was not initialized properly.");
        }
      if (m_htKeyToClassIdLookup.ContainsValue(typeId))
      {
        Object[] keys = m_htKeyToClassIdLookup.GetArray(XPHashtable.KEYS);
        Object[] values = m_htKeyToClassIdLookup.GetArray(XPHashtable.VALUES);
        // look up the corresponding key for the specified value
        for (int i = 0;i < values.Length;i++)
        {
          if (typeId.Equals(values[i]))
          {
            return (String) keys[i];
          }
        }
      }
      return null;
    }
    
    /// <summary>
    /// Unit tests should use this method to overwrite/teardown existing manager singleton.
    /// DO NOT USE for production code. 
    /// </summary>
    public static void setManagerTestInstance(IFriendlyURLParserManager testManager)
    {
      if (m_bIsManagerProduction)
      {
        String strMessage = "Can\'t override Layout Manager instance for testing... LayoutManager has already been created.";
        log.Error(strMessage);
        throw new XPException(strMessage);
      }
      INSTANCE = testManager;
    }
  }
}