using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using PT_CLASSIDS=com.plumtree.server.PT_CLASSIDS;
namespace com.plumtree.portaluiinfrastructure.friendlyurl
{
  /// <summary>
  /// This class handles friendly url requests for RSS feeds that are generated
  /// in KD Browse and Search Results layout mode.
  /// </summary>
  /// <author>BobbyT</author>
  public class RSSFriendlyURLParser : FriendlyURLParser
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.friendlyurl.RSSFriendlyURLParser");
    
    /// <summary>
    /// The key used to specify the KD RSS feed.
    /// </summary>
    public const String RSS_TYPE_KD = "kd";
    
    /// <summary>
    /// The key used to specify the Search RSS feed.
    /// </summary>
    public const String RSS_TYPE_SEARCH = "searchresults";
    
    /// <summary>
    /// The key used to specify basic auth in the RSS feed.
    /// </summary>
    public const String RSS_KEY_KD = "directory";
    
    /// <summary>
    /// The key used to specify basic auth in the RSS feed.
    /// </summary>
    public const String RSS_KEY_SEARCH = "search";
    
    /// <summary>
    /// The RSS friendly URL keyword
    /// </summary>
    public const String RSS_FRIENDLY_URL_KEYWORD = "rss";
    
    /// <summary>
    /// Public constructor that is initialized to parse friendly URLs with rss keyword.
    /// </summary>
    /// <param name="strKeyword">This is friendly URL for rss keyword defined in friendlyURLs.xml</param>
    /// <param name="strClassId">This is the classid of friendly URL for rss defined in FriendlyURLs.xml and FriendlyURLConstants.</param>
    /// <param name="stURLTokens">
    /// The tokens here are resource type, resource name to identify in which resource the search has 
    /// to take place. This is to support rest style url syntax. If no resources are defined in url searching will be done 
    /// in all the resources.
    /// </param>
    public RSSFriendlyURLParser(String strKeyword, String strClassId, XPStringTokenizer stURLTokens) : 
    // This should either be /rss/kd or /rss/searchresults. We don't really care about the 
    // object name at this level (i.e. "kd" or "searchresults") since it's only needed for the
    // URL pattern in the Experience Definition rules.
    base(strKeyword, strClassId, stURLTokens)
    {
      // BT - This is where we check for the "real" KD or Search Friendly URL. We parse for the resource
      // type and resource name in the URL here.
      try
      {
        // Check whether this is a kd or search results friendly URL and parse
        // to get the object id if any (i.e. /rss/kd/directory or /rss/searchresults/search)
        if (stURLTokens.HasMoreTokens())
        {
          String strObjName = URLDecode(stURLTokens.GetNextToken());
          if (RSS_KEY_KD.Equals(strObjName))
          {
            // If this is a KD Friendly URL, we want to set the parser up as 
            // if it was a KDFriendlyURLParser so that it will create the redirect
            // URL correctly later when ptFriendlyURLMgr.GetFriendlyViewOpenerRedirect
            // is called in FriendlyURLPlugin.ProcessPlugIn
            // KD friendly URL keyword, resource name and object id:
            // /directory/...
            m_strClassId = XPConvert.ToString(PT_CLASSIDS.PT_CATALOGFOLDER_ID);
            // /directory/directory_name/...
            if (stURLTokens.HasMoreTokens())
            {
              String folderName = URLDecode(stURLTokens.GetNextToken());
              m_strObjectName = folderName;
              // /directory/directory_name/object_id
              if (stURLTokens.HasMoreTokens())
              {
                String strId = stURLTokens.GetNextToken();
                m_nObjectId = XPConvert.ToInteger(strId);
              }
            }
          } else
          if (RSS_KEY_SEARCH.Equals(strObjName))
          {
            // This is a Search Friendly URL. Get the object id if any and we're done
            // /search/
            // /search/object_id/
            if (stURLTokens.HasMoreTokens())
            {
              String strId = stURLTokens.GetNextToken();
              m_nObjectId = XPConvert.ToInteger(strId);
            }
          }
        }
      }
      catch (Exception e)
      {
        log.Warn(e, "RSSFriendlyURLParser:Init The resource name was not available in the URL.");
        }
    }
  }
}