using System;
using XPException=com.plumtree.openfoundation.util.XPException;
using XPSystem=com.plumtree.openfoundation.util.XPSystem;
using OpenLogService=com.plumtree.openlog.OpenLogService;
using OpenLogger=com.plumtree.openlog.OpenLogger;
using PTDebugHelpers=com.plumtree.portaluiinfrastructure.statichelpers.PTDebugHelpers;
using IPTQueryResult=com.plumtree.server.IPTQueryResult;
using IPTSession=com.plumtree.server.IPTSession;
using PT_FILTEROPS=com.plumtree.server.PT_FILTEROPS;
using PT_PROPIDS=com.plumtree.server.PT_PROPIDS;
namespace com.plumtree.portaluiinfrastructure.helpers
{
  /// <author>Administrator</author>
  public class CollaborationHelperService : ICollaborationHelper
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.helpers.CollaborationHelperService");
    
    /// <summary>
    /// This is used to lookup the Collab web service instance.  "Notification Gateway Entry" is the actual
    /// name of web service I grabbed this UUID from, when you install the collaboration.pte.
    /// </summary>
    public const String COLLAB_WEB_SERVICE_UUID = "{AC0A5760-B0AC-40DA-A977-67F6E2F3949C}";
    
    /// <summary>
    /// The timeout for caching the web service data, in seconds.
    /// </summary>
    protected const int TIMEOUT = 300;
    //the singleton instance
    private static ICollaborationHelper m_helperInstance;
    // This stores whether or not the factory is in test mode.
    private static bool m_bIsHelperProduction = false;
    
    /// <summary>
    /// Whether or not the cache has been initialized.
    /// </summary>
    private bool m_bInitialized = false;
    
    /// <summary>
    /// The last time we queried the DB.  This must be accessed
    /// in a threadsafe way, since assignments to long are not
    /// atomic/threadsafe.
    /// </summary>
    private long m_nLastChecked = 0;
    
    /// <summary>
    /// Lock to guarantee atomic access to the cache update/lazy
    /// init code segment and/or timestamp.
    /// </summary>
    private readonly Object m_lock = new Object();
    
    /// <summary>
    /// The cached ID of the collab web service, if installed.
    /// </summary>
    private int m_nServerID = -1;
    
    /// <summary>
    /// Private constructor to ensure controlled access to the singleton.
    /// </summary>
    private CollaborationHelperService() : base()
    {
    }
    
    /// <returns>The singleton PT config helper.</returns>
    public static ICollaborationHelper getInstance()
    {
      // Technically this should be synchronized to ensure
      // threadsafety, but we don't care if an extra helper service
      // is created on startup and then overwritten.
      if (m_helperInstance == null)
      {
        m_bIsHelperProduction = true;
        m_helperInstance = new CollaborationHelperService();
      }
      return m_helperInstance;
    }
    
    /// <summary>
    /// Unit tests should use this method to overwrite/teardown existing helper singleton.
    /// DO NOT USE for production code.
    /// NOTE: When finished with a unit test, this should be set back to null so as not
    /// to interfere with other tests.
    /// </summary>
    public static void setTestInstance(ICollaborationHelper testHelper)
    {
      // Technically this should be synchronized to ensure
      // threadsafety, but we don't care about threadsafety
      // during unit tests.
      if (m_bIsHelperProduction)
      {
        String strMessage = "Can\'t override CollaborationHelperService instance for testing... CollaborationHelperService has already been initialized.";
        log.Error(strMessage);
        throw new XPException(strMessage);
      }
      m_helperInstance = testHelper;
    }
    
    /// <summary>
    /// Load the collab web service, if installed.
    /// This handles cache timeouts and lazy initialization.
    /// </summary>
    protected virtual void LoadCollaborationWebService(IPTSession _session)
    {
      // NOTE: We don't really care that much about stale data,
      // just safe data.
      // No need for this to be synchronized, because we don't care
      // if the cache is updated at exactly the timeout time, or if
      // we have to wait for the next time someone calls this method.
      long nCurrentTime = XPSystem.GetCurrentTimeMilliSec();
      long nTimeout = TIMEOUT * 1000;
      lock (m_lock) {
        // Only run the query if we haven't run it yet,
        // or if our cached version is timed out.
        if (m_bInitialized)
        {
          if (nCurrentTime < (m_nLastChecked + nTimeout))
          {
            // The cache is still valid.
            return;
          }
        }
        int nNewServerID = -1;
        try
        {
          // NOTE: It's okay to run this query with various user's sessions because the
          // query doesn't run a security check, so every user should get the same results.
          // Otherwise we would have to do something like use the
          // dangerousimpersonateadminuserlight method.
          // look for collab web service
          Object[][] filter = new Object[3][];
          for (int j2c_index = 0;j2c_index < 3;j2c_index++)
          {
            filter[j2c_index] = new Object[1];
          }
          filter[0][0] = PT_PROPIDS.PT_PROPID_UUID;
          filter[1][0] = PT_FILTEROPS.PT_FILTEROP_EQ;
          filter[2][0] = COLLAB_WEB_SERVICE_UUID;
          IPTQueryResult ptqrServer = (IPTQueryResult) _session.GetWebServices().Query(PT_PROPIDS.PT_PROPID_OBJECTID,
           -1, PT_PROPIDS.PT_PROPID_OBJECTID, 0, -1, filter);
          if (ptqrServer.RowCount() > 1)
          {
            // this should never happen.
            log.Warn("More than one Collaboration web service returned by query when initializing Collaboration Helper.  Ignoring.");
            }
          if (ptqrServer.RowCount() > 0)
          {
            nNewServerID = ptqrServer.ItemAsInt(0, PT_PROPIDS.PT_PROPID_OBJECTID);
            } else
          {
            //Collaboration Remote Server not installed.
            nNewServerID = -1;
          }
        }
        catch (Exception e)
        {
          log.Error(e, "Unable to determine if the Collaboration web service is installed.");
          m_nServerID = -1;
        }
        // Set the new server ID.
        m_nServerID = nNewServerID;
        // Set timestamp to now.
        m_nLastChecked = XPSystem.GetCurrentTimeMilliSec();
        m_bInitialized = true;
      }
    }
    public virtual bool IsCollaborationInstalled(IPTSession _session)
    {
      // This handles cache timeouts and lazy initialization.
      LoadCollaborationWebService(_session);
      return (m_nServerID > 0);
    }
    public virtual int GetWebServiceID(IPTSession _session)
    {
      // This handles cache timeouts and lazy initialization.
      LoadCollaborationWebService(_session);
      return m_nServerID;
    }
  }
}