using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using PTDebugHelpers=com.plumtree.portaluiinfrastructure.statichelpers.PTDebugHelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portaluiinfrastructure.helpers
{
  /// <summary>
  /// This class handles common functionality related to Pathways integrations.
  /// NOTE: This class is threadsafe.
  /// </summary>
  public class PathwaysHelperService : IPathwaysHelper
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.helpers.PathwaysHelperService");
    
    /// <summary>
    /// This is used to lookup the Pathways API web service instance for making calls to Pathways.
    /// </summary>
    public const String PATHWAYS_API_WEBSERVICE_UUID = "{8C5D58E2-6205-A2BB-9EAD-093DBBCFA000}";
    
    /// <summary>
    /// The timeout for caching the Web Service data, in seconds.
    /// </summary>
    protected const int TIMEOUT = 300;
    //the singleton instance
    private static IPathwaysHelper m_helperInstance;
    // This stores whether or not the factory is in test mode.
    private static bool m_bIsHelperProduction = false;
    
    /// <summary>
    /// Whether or not the cache has been initialized.
    /// </summary>
    private bool m_bInitialized = false;
    
    /// <summary>
    /// The last time we queried the DB.  This must be accessed
    /// in a threadsafe way, since assignments to long are not
    /// atomic/threadsafe.
    /// </summary>
    private long m_nLastChecked = 0;
    //Setting for a Pathways search error that occurred
    private bool m_isPathwaysSearchError = false;
    //start time of timeout value
    private long m_errorStartTime = 0;
    //setting for if we're in fall back to legacy mode in our search request
    private bool m_isLegacySearchMode = false;
    
    /// <summary>
    /// Lock to guarantee atomic access to the cache update/lazy
    /// init code segment and/or timestamp.
    /// </summary>
    private readonly Object m_lock = new Object();
    
    /// <summary>
    /// The cached ID of the Pathways API Web Service, if installed.
    /// </summary>
    private int m_nWebServiceID = -1;
    
    /// <summary>
    /// Private constructor to ensure controlled access to the singleton.
    /// </summary>
    private PathwaysHelperService() : base()
    {
    }
    
    /// <returns>The singleton PT config helper.</returns>
    public static IPathwaysHelper GetInstance()
    {
      // Technically this should be synchronized to ensure
      // threadsafety, but we don't care if an extra helper service
      // is created on startup and then overwritten.
      if (m_helperInstance == null)
      {
        m_bIsHelperProduction = true;
        m_helperInstance = new PathwaysHelperService();
      }
      return m_helperInstance;
    }
    
    /// <summary>
    /// Unit tests should use this method to overwrite/teardown existing helper singleton.
    /// DO NOT USE for production code.
    /// NOTE: When finished with a unit test, this should be set back to null so as not
    /// to interfere with other tests.
    /// </summary>
    public static void SetTestInstance(IPathwaysHelper testHelper)
    {
      // Technically this should be synchronized to ensure
      // threadsafety, but we don't care about threadsafety
      // during unit tests.
      if (m_bIsHelperProduction)
      {
        String strMessage = "Can\'t override PathwaysHelperService instance for testing... PathwaysHelperService has already been initialized.";
        log.Error(strMessage);
        throw new XPException(strMessage);
      }
      m_helperInstance = testHelper;
    }
    
    /// <summary>
    /// Load the Pathways API Web Service, if installed.
    /// This handles cache timeouts and lazy initialization.
    /// </summary>
    protected virtual void LoadPathwaysAPIWebService(IPTSession _session)
    {
      // NOTE: We don't really care that much about stale data,
      // just safe data.
      // No need for this to be synchronized, because we don't care
      // if the cache is updated at exactly the timeout time, or if
      // we have to wait for the next time someone calls this method.
      long nCurrentTime = XPSystem.GetCurrentTimeMilliSec();
      // Get the timeout config value.
      int nTimeout = GetTimeout();
      lock (m_lock) {
        // Only run the query if we haven't run it yet,
        // or if our cached version is timed out.
        if (m_bInitialized)
        {
          if (nCurrentTime < (m_nLastChecked + nTimeout))
          {
            // The cache is still valid.
            return;
          }
        }
        int nNewWebServiceID = -1;
        try
        {
          // NOTE: It's okay to run this query with various user's sessions because the
          // query doesn't run a security check, so every user should get the same results.
          // Otherwise we would have to do something like use the
          // dangerousimpersonateadminuserlight method.
          // look for Pathways API Web Service
          Object[][] filter = new Object[3][];
          for (int j2c_index = 0;j2c_index < 3;j2c_index++)
          {
            filter[j2c_index] = new Object[1];
          }
          filter[0][0] = PT_PROPIDS.PT_PROPID_UUID;
          filter[1][0] = PT_FILTEROPS.PT_FILTEROP_EQ;
          filter[2][0] = PATHWAYS_API_WEBSERVICE_UUID;
          IPTQueryResult ptqrWebService = (IPTQueryResult) _session.GetWebServices().Query(PT_PROPIDS.PT_PROPID_OBJECTID,
           -1, PT_PROPIDS.PT_PROPID_OBJECTID, 0, -1, filter);
          if (ptqrWebService.RowCount() > 1)
          {
            // this should never happen.
            log.Warn("More than one Pathways API Web Service returned by query when initializing Pathways Helper.  Ignoring.");
            }
          if (ptqrWebService.RowCount() > 0)
          {
            nNewWebServiceID = ptqrWebService.ItemAsInt(0, PT_PROPIDS.PT_PROPID_OBJECTID);
            } else
          {
            // Pathways API Web Service not installed.
            nNewWebServiceID = -1;
          }
        }
        catch (Exception e)
        {
          log.Error(e, "Unable to determine if the Pathways API Web Service is installed.");
          m_nWebServiceID = -1;
        }
        // Set the new Web Service ID.
        m_nWebServiceID = nNewWebServiceID;
        // Set timestamp to now.
        m_nLastChecked = XPSystem.GetCurrentTimeMilliSec();
        m_bInitialized = true;
      }
    }
    
    /// <returns>the timeout value from the config file.</returns>
    protected virtual int GetTimeout()
    {
      int nTimeout;
      try
      {
        // Get the timeout config value.
        IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
        ConfigVarPack vpConfig = (ConfigVarPack) application.GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);
        nTimeout = vpConfig.GetConfigValueAsInt(ConfigConstants.SERVICE_URLS_SECTION,
         ConfigConstants.PATHWAYS_CACHE_TIMEOUT, ConfigConstants.VALUE_ATTRIBUTE);
        }
      catch (Exception e)
      {
        log.Warn(e, "Unable to load Pathways Server Cache Timeout.  Using default value of 5.");
        nTimeout = 5;
      }
      // Config timeout is in minutes, we need it in milliseconds
      return nTimeout * 60 * 1000;
    }
    public virtual bool IsPathwaysInstalled(IPTSession _session)
    {
      // This handles cache timeouts and lazy initialization.
      LoadPathwaysAPIWebService(_session);
      return (m_nWebServiceID > 0);
    }
    public virtual int GetAPIWebServiceID(IPTSession _session)
    {
      // This handles cache timeouts and lazy initialization.
      LoadPathwaysAPIWebService(_session);
      return m_nWebServiceID;
    }
    
    /// <summary>
    /// When a Pathways search server issue occurs, this sets timeout value for when to use Pathways again
    /// </summary>
    public virtual void StartErrorState()
    {
      //now we can set the timeout value
      m_isPathwaysSearchError = true;
      m_errorStartTime = XPSystem.GetCurrentTimeMilliSec();
    }
    
    /// <returns>
    /// whether or not Pathways timeout has expired.  Returns true if the time out still occurs and false if 
    /// the timeout expired and we can try Pathways again.
    /// </returns>
    public virtual bool IsErrorState()
    {
      //getting current time
      long currentTime = XPSystem.GetCurrentTimeMilliSec();
      //getting timeout setting from config manager
      int timeOut = GetTimeout();
      if (currentTime < (m_errorStartTime + timeOut))
      {
        //Pathways timeout hasn't expired yet, we're still in timeout for not using pathways
        m_isPathwaysSearchError = true;
        //setting the fall back search mode so the Pathways layout can handle the display properly when Pathways
        //search errors out and we default to legacy
        return m_isPathwaysSearchError;
      } else
      {
        //pathways timeout has expired, will set flag back to false
        m_isPathwaysSearchError = false;
        //turning off fall back mode so we can display the proper info in the layout
        return m_isPathwaysSearchError;
      }
    }
    
    /// <param name="value">the boolean value representing if the search is in fall back to legacy mode</param>
    public virtual void SetLegacySearchMode(bool value)
    {
      m_isLegacySearchMode = value;
    }
    
    /// <summary>
    /// This is for advanced search, since it will fall back to legacy search if a search request is not valid for a 
    /// pathways search even though it's enabled and without error
    /// </summary>
    /// <returns>the boolean value if the search is currently defaulting to legacy mode</returns>
    public virtual bool GetLegacySearchMode()
    {
      return m_isLegacySearchMode;
    }
  }
}