using System;
using com.plumtree.openfoundation.util;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.portaluiinfrastructure.intrinsicportlets
{
  /// <summary>
  /// This is the base intrinsic portlet model.  All intrinsic portlet models
  /// should extend this base class.
  /// </summary>
  /// <author>josephc</author>
  public abstract class IntrinsicPortletModel : AClearModel,IIntrinsicPortletModelRO,
  IModel
  {
    
    /// <summary>
    /// Setting type for user portlet preferences 
    /// </summary>
    public const int SETTINGSTYPE_PORTLET = 1;
    
    /// <summary>
    /// Setting type for admin preferences 
    /// </summary>
    public const int SETTINGSTYPE_ADMIN = 2;
    
    /// <summary>
    /// Setting type for community preferences 
    /// </summary>
    public const int SETTINGSTYPE_COMMUNITY = 3;
    
    /// <summary>
    /// Setting type for community portlet preferences 
    /// </summary>
    public const int SETTINGSTYPE_COMMUNITYPORTLET = 4;
    
    /// <summary>
    /// Setting type for user preferences 
    /// </summary>
    public const int SETTINGSTYPE_USER = 5;
    
    /// <summary>
    /// Preference type for admin preferences 
    /// </summary>
    public const int PREF_TYPE_ADMIN = 1;
    
    /// <summary>
    /// Preference type for personal preferences 
    /// </summary>
    public const int PREF_TYPE_PERSONAL = 2;
    
    /// <summary>
    /// Preference type for community preferences 
    /// </summary>
    public const int PREF_TYPE_COMMUNITY = 3;
    
    /// <summary>
    /// Parent Activity Space 
    /// </summary>
    protected AActivitySpace m_asOwner;
    
    /// <summary>
    /// Portlet object 
    /// </summary>
    protected IPTGadget m_ptPortlet;
    
    /// <summary>
    /// IPTSession 
    /// </summary>
    protected IPTSession m_ptSession;
    
    /// <summary>
    /// AppDataState object 
    /// </summary>
    protected IPTState m_ptAppDataState;
    
    /// <summary>
    /// ContainerClassID 
    /// </summary>
    protected int m_nContainerClassID;
    
    /// <summary>
    /// ContainerObjectID 
    /// </summary>
    protected int m_nContainerObjectID;
    
    /// <summary>
    /// PageID 
    /// </summary>
    protected int m_nPageID;
    
    /// <summary>
    /// Portlet ID 
    /// </summary>
    protected int m_nPortletID;
    
    /// <summary>
    /// Preference type 
    /// </summary>
    protected int m_nPrefType;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()"></seealso>
    public override void CleanupTempData()
    {
      base.CleanupTempData();
      m_ptAppDataState = null;
      m_ptPortlet = null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()"></seealso>
    public override void CleanupAllData()
    {
      base.CleanupAllData();
      m_ptAppDataState = null;
      m_ptPortlet = null;
    }
    
    /// <summary>
    /// Returns true if the portlet has a control associated with it.  This
    /// function should be implemented by the child class.
    /// </summary>
    /// <returns>Returns true if portlet has control. Otherwise return false.</returns>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.intrinsicportlets.IIntrinsicPortletModelRO#ControlExists()"></seealso>
    public abstract bool ControlExists();
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public abstract override String GetName();
    
    /// <summary>
    /// Returns portlet ID
    /// </summary>
    /// <returns>Portlet ID</returns>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.intrinsicportlets.IIntrinsicPortletModelRO#GetPortletID()"></seealso>
    public virtual int GetPortletID()
    {
      return m_nPortletID;
    }
    
    /// <summary>
    /// Inits the Model with the Activity Space. Initialize the IPTSession from
    /// the Activity Space
    /// </summary>
    /// <param name="parent"></param>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#Init(AActivitySpace)"></seealso>
    public override void Init(AActivitySpace parent)
    {
      m_asOwner = parent;
      // Initialize the IPTSession from the parent activity space
      m_ptSession = (IPTSession) m_asOwner.GetUserSession();
    }
    
    /// <summary>
    /// Initializes the member variables of the portlet.
    /// </summary>
    /// <param name="__ptPageInfo">IPTPageInfo</param>
    /// <param name="_ptAppDataState">AppDataState object</param>
    /// <param name="_ptPortlet">Portlet object</param>
    /// <param name="_nPortletID">Portlet ID</param>
    public virtual void InitPortlet(IPTPageInfo _ptPageInfo, IPTState _ptAppDataState,
     IPTGadget _ptPortlet, int _nPortletID)
    {
      if (null != _ptPageInfo)
      {
        m_nContainerClassID = _ptPageInfo.GetContainerClassID();
        m_nContainerObjectID = _ptPageInfo.GetContainerObjectID();
        m_nPageID = _ptPageInfo.GetObjectID();
      } else
      {
        m_nContainerClassID = 0;
        m_nContainerObjectID = 0;
        m_nPageID = 0;
      }
      m_nPortletID = _nPortletID;
      m_ptAppDataState = _ptAppDataState;
      m_ptPortlet = _ptPortlet;
      if (PT_CLASSIDS.PT_COMMUNITY_ID == m_nContainerClassID)
      {
        m_nPrefType = PREF_TYPE_COMMUNITY;
      } else
      {
        m_nPrefType = PREF_TYPE_PERSONAL;
      }
    }
    
    /// <summary>
    /// Checks the markup array to see whether or not the portlet is intrinsic
    /// </summary>
    /// <param name="objArrMarkup">Array retrieved from IPTGadget::GetHTMLText()</param>
    /// <returns>True if the portlet is intrinsic.  False otherwise.</returns>
    public static bool IsIntrinsicPortlet(Object[][] objArrMarkup)
    {
      int nPortletContentType = XPConvert.ToInteger(objArrMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_TYPE][0]);
      if (PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_IPP == nPortletContentType)
      {
        return true;
      } else
      {
        return false;
      }
    }
    
    /// <summary>
    /// Returns a single setting value, based on the setting type specified.
    /// </summary>
    /// <param name="nSettingType">Setting type</param>
    /// <param name="strName">Setting name</param>
    /// <returns>Setting value</returns>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.IIntrinsicPortletPrefsModelRO#GetSettingValue(int,String)"></seealso>
    public virtual Object GetSettingValue(int nSettingType, String strName)
    {
      String[] arFilter = new String[]{strName};
      Object[][] arValues;
      Object oValue = null;
      arValues = GetSettingValues(nSettingType, arFilter);
      if ((null != arValues) && (arValues.Length > 0))
      {
        oValue = arValues[1][0];
      }
      return oValue;
    }
    //JD- TODOCR 1/16/03
    //There are helper functions to get user preferences in LoginHelper.  I'm not sure whether
    //or not they should be used here... but either way there should be a comment specifying 
    //what the decision is, and why was it made.
    //JD- TODOCR 1/16/03
    // Perhaps it's worth doing a sanity check that if community or community-gadget prefs
    // are set, it's happenning on a community.  This applies to all functions that set,
    // read, or remove a pref.
    //JD- TODOCR 1/16/03
    // How are you handling security for admin prefs? What happens if someone who does not
    // have edit rights on the Portlet object tries to set admin prefs?
    
    /// <summary>
    /// Returns all settings, based on the setting type and filter specified.
    /// </summary>
    /// <param name="nSettingType">Setting type</param>
    /// <param name="arFilter">Filter for settings</param>
    /// <returns>All settings</returns>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.IIntrinsicPortletPrefsModelRO#GetSettingValues(int,Object[])"></seealso>
    public virtual Object[][] GetSettingValues(int nSettingType, String[] arFilter)
    {
      Object[][] arValues;
      switch (nSettingType){
      case SETTINGSTYPE_PORTLET:
      if (PREF_TYPE_COMMUNITY == m_nPrefType)
      {
        arValues = (Object[][]) ((IPTCommunityManager) m_ptSession.GetCommunities()).CachedOpenCommunityInfo(m_nContainerObjectID,
         false).LookupPreferences(arFilter, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_CURRENT_USER);
        } else
      {
        arValues = (Object[][]) m_ptSession.GetSessionInfo().LookupPreferences(arFilter,
         m_nPortletID);
      }
      break;
      case SETTINGSTYPE_ADMIN:
      {
        IPTAdminSettings ptAdminSettings = (IPTAdminSettings) m_ptPortlet.GetInterfaces("IPTAdminSettings");
        arValues = (Object[][]) ptAdminSettings.LookupAdminGadgetSettings(arFilter);
        break;
      }
      case SETTINGSTYPE_COMMUNITY:
      arValues = (Object[][]) ((IPTCommunityManager) m_ptSession.GetCommunities()).CachedOpenCommunityInfo(m_nContainerObjectID,
       true).LookupPreferences(arFilter, 0, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);
      break;
      case SETTINGSTYPE_COMMUNITYPORTLET:
      arValues = (Object[][]) ((IPTCommunityManager) m_ptSession.GetCommunities()).CachedOpenCommunityInfo(m_nContainerObjectID,
       true).LookupPreferences(arFilter, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);
      break;
      case SETTINGSTYPE_USER:
      arValues = (Object[][]) m_ptSession.GetSessionInfo().LookupPreferences(arFilter,
       0);
      break;
      default:
      return null;
      break;
      }
      return arValues;
    }
  }
}