using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.classtypedescriptors.providerframework;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.opener.types.classtypes;
namespace com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs
{
  /// <summary>
  /// Contains utility functions that deal with intrinsic portlets.
  /// </summary>
  /// <author>josephc</author>
  public class IntrinsicPortletPrefsUtils
  {
    
    /// <summary>
    /// The prefs suffix is appended to the portlet class ID to form the name of
    /// prefs activity space.
    /// </summary>
    public const String PREFS_SUFFIX = "Prefs";
    
    /// <summary>
    /// Subspace key suffix for intrinsic portlets pref pages 
    /// </summary>
    public const String SUBSPACE_KEY_SUFFIX = "PrefsSubSpace";
    
    /// <summary>
    /// Query string constant for CommunityID 
    /// </summary>
    public const String COMMUNITYID = "CommunityID";
    
    /// <summary>
    /// Query string constant for PageID 
    /// </summary>
    public const String PAGEID = "PageID";
    
    /// <summary>
    /// Query string constant for PortletID 
    /// </summary>
    public const String PORTLETID = "PortletID";
    
    /// <summary>
    /// Query string constant for Admin type, 1 for true, 0 for false 
    /// </summary>
    public const String ISADMINPREF = "IsAdminPref";
    
    /// <summary>
    /// Gets the portlet class identifier
    /// </summary>
    /// <param name="_ptPortlet">Portlet object</param>
    /// <param name="_objAppData">AppDataState object</param>
    /// <returns>
    /// Portlet class identifier (which is also the activity space of
    /// portlet)
    /// </returns>
    public static String GetClassIdentifier(IPTGadget _ptPortlet, IPTState _objAppData)
    {
      // Get the portlet class identifier by calling GetHTMLText().
      Object[][] objArrMarkup = (Object[][]) _ptPortlet.GetHTMLText(_objAppData);
      String strPortletClass = (String) objArrMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT][0];
      return strPortletClass;
    }
    
    /// <summary>
    /// Constructs URL for accessing portlet preferences activity space.
    /// </summary>
    /// <param name="_ptWebService">Web service object</param>
    /// <param name="_nPageID">Page ID</param>
    /// <param name="_nCommunityID">Community ID</param>
    /// <param name="_nPortletID">Portlet ID</param>
    /// <param name="_nAssociatedContentID">Associated content ID</param>
    /// <param name="asOwner">Activity space owner</param>
    /// <returns>URL to preference page for intrinsic portlets</returns>
    public static ASURL GetPrefsURL(IPTWebService _ptWebService, int _nPageID,
     int _nCommunityID, int _nPortletID, bool isAdminPref, int _nAssociatedContentID,
     AActivitySpace asOwner)
    {
      // Construct the URL to the prefs page
      ASURL asURL = new ASURL(asOwner);
      IXPPropertyBag ptProviderInfo = (IXPPropertyBag) _ptWebService.GetProviderInfo();
      String strPrefsSpace;
      bool isCommunityPrefs = PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_COMMUNITYPREFSPAGE == _nAssociatedContentID;
      if (isCommunityPrefs)
      {
        strPrefsSpace = ptProviderInfo.ReadAsString(AExtensibilityConstants.PTC_I_GADGET_COMMUNITYPREFS_SPACE);
        } else
      if (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_ADMINPREFSPAGE == _nAssociatedContentID)
      {
        strPrefsSpace = ptProviderInfo.ReadAsString(AExtensibilityConstants.PTC_I_GADGET_ADMINPREFS_SPACE);
        } else
      {
        strPrefsSpace = ptProviderInfo.ReadAsString(AExtensibilityConstants.PTC_I_GADGET_PREFS_SPACE);
        }
      asURL.SetLinkCreateNewSpace(strPrefsSpace, asOwner);
      // Set the control
      asURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
      // Set editor type
      asURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE, XPConvert.ToString(EditorStartControl.EDITOR_START_FLAG_MISC));
      // Set pageID
      asURL.AddControlArgument(PAGEID, XPConvert.ToString(_nPageID));
      if (isCommunityPrefs)
      {
        // Set communityID
        asURL.AddControlArgument(COMMUNITYID, XPConvert.ToString(_nCommunityID));
        }
      if (isAdminPref)
      {
        asURL.AddControlArgument(ISADMINPREF, XPConvert.ToString(1));
      }
      // Set portletID
      asURL.AddControlArgument(PORTLETID, XPConvert.ToString(_nPortletID));
      // Make URL a subspace so that URL is correctly constructed no matter what security mode we're in. -jic
      asURL.SetSubSpaceKey(asOwner.GetName());
      return asURL;
    }
    
    /// <summary>
    /// Constructs URL for accessing portlet help.
    /// </summary>
    /// <param name="_ptWebService">Web service object</param>
    /// <returns>URL to help page for intrinsic portlets</returns>
    public static String GetHelpURL(IPTWebService _ptWebService)
    {
      IXPPropertyBag ptProviderInfo = (IXPPropertyBag) _ptWebService.GetProviderInfo();
      return ptProviderInfo.ReadAsString(AExtensibilityConstants.PTC_I_GADGET_HELP_URL);
      }
    
    /// <summary>
    /// Given a URL String remove psname & subspace control arguments from the query string.
    /// Use this method only when generating a link to a portlet prefs page from an intrinsic
    /// portlet which may be in 508 mode where these arguemnts confuse the control flow.
    /// </summary>
    /// <param name="_strURL">a URL string to a portlet pref page</param>
    /// <param name="_asOwner">the current activity space</param>
    /// <returns>a new URL String without the psname and subspace control arguments</returns>
    public static String GetAccessStyleComplientPortletPrefURL(String _strURL,
     AActivitySpace _asOwner)
    {
      String myReturnValue = _strURL;
      if (AccessStyles.STANDARD == _asOwner.GetAccessStyle())
      {
        return myReturnValue;
      }
      myReturnValue = RemoveFromQueryString(myReturnValue, UIActivitySpaces.QS_PARENT_SPACE_NAME);
      myReturnValue = RemoveFromQueryString(myReturnValue, ASConstants.SUBSPACE);
      return myReturnValue;
    }
    
    /// <summary>
    /// Given a query string remove a gateway control parameter. For example with an 
    /// '_strControlToRemove' value of 'psname' remove the string '&psname=Gateway' from the 
    /// query string. If the query string is null return null, if the query string doesn't
    /// contain the control name return the query string unchanged.
    /// </summary>
    /// <param name="_strQueryString">a query string ie '?name=foo&blah=bar&psname=Gateway'</param>
    /// <param name="_strControlToRemove">a gateway control to remove</param>
    /// <returns>the modified query string.</returns>
    private static String RemoveFromQueryString(String _strQueryString, String _strControlToRemove)
    {
      String myPSNameString = "&" + _strControlToRemove + "=Gateway";
      int myPSNameIndex = XPStringUtility.IndexOf(_strQueryString, myPSNameString);
      int myPSNameLength = myPSNameString.Length;
      if (myPSNameIndex != -1)
      {
        String start = XPStringUtility.Substring(_strQueryString, 0, myPSNameIndex);
        String end = XPStringUtility.Substring(_strQueryString, myPSNameIndex + myPSNameLength);
        _strQueryString = start + end;
      }
      return _strQueryString;
    }
  }
}