using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.login;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portaluiinfrastructure.login
{
  /// <summary>
  /// This class handles getting and setting personal settings using the plumtree
  /// session info object.
  /// </summary>
  public class PTPersonalSettingsHelper
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.login.PTPersonalSettingsHelper");
    
    /// <summary>
    /// helper constant 
    /// </summary>
    private static readonly NoValue NO_VALUE = new NoValue();
    
    /// <summary>
    /// Access Style Cookie name 
    /// </summary>
    public const String ACCESS_STYLE_COOKIE_NAME = "ptaccessstyle";
    
    /// <summary>
    /// Use Browser Setting ID (Locale)
    /// </summary>
    public const String LOCALE_USE_BROWSER_SETTING = "UseBrowser";
    
    /// <summary>
    /// Use Default Setting ID (Locale)
    /// </summary>
    public const String LOCALE_USE_DEFAULT_SETTING = "";
    
    /// <summary>
    /// Call this method to cache all personal settings on the session.
    /// <br /><br />
    /// This method will also set the session language on the Plumtree server.
    /// <br /><br />
    /// </summary>
    /// <param name="AActivitySpace"></param>
    /// <param name="IApplication"></param>
    /// <param name="XPResourceManager"></param>
    public static void CachePersonalSettings(AActivitySpace aSpace, IApplication application,
     XPResourceManager rm, ApplicationData appData)
    {
      // This helper method simply insures you're using the right HTTPSession "sandbox" off of the AActivitySpace
      IPTSession ptSession = GetPTSession(aSpace);
      ISessionManager sessionManager = aSpace.GetSubSession();
      CachePersonalSettings(sessionManager, application, ptSession, rm, appData);
      }
    
    /// <summary>
    /// Caches all settings to the IPTSession as well as the HTTPSession.
    /// <br /><br />
    /// This method will also set the session language on the Plumtree server.
    /// <br /><br />
    /// </summary>
    /// <param name="ISessionManager"></param>
    /// <param name="IApplication"></param>
    /// <param name="IPTSession"></param>
    /// <param name="XPResourceManager"></param>
    public static void CachePersonalSettings(ISessionManager sessionManager,
     IApplication application, IPTSession ptSession, XPResourceManager rm,
     ApplicationData appData)
    {
      // This function caches all of the user's personal settings on the IPTSession, and
      // also caches a limited subset of settings on the lower-level HTTPSession.
      // The subset cached on the lower level is specified in the <PersonalSettings>
      // node in x_config.xml 
      // Some general rules/guidelines regarding locale settings
      //
      // 1) "Use Default", the zero length string, and not set should all act equivalently.
      // 2) If a non-guest/non-default profile user logs in with USE_DEFAULT or USE_BROWSER
      // set for it's locale, an appropriate locale will be determined, and then promoted
      // and stored in the DB as the user's new locale setting.
      // 3) There is a strict hierarchy of settings (i.e. it uses the user setting, but if
      // that's default, then it uses the XML setting, but if that's empty it uses the browser.
      // 4) guest user and default profile user locale settings never get promoted.
      log.Info("Entering CachePersonalSettings()");
      if (!PersonalSettingsHelper.DoesPrefsSubSessionExist(sessionManager))
      {
        throw new XPException("SubSession for prefs cache does not exist, please make sure you\'re passing the top-level ISessionManager object retrieved from the ActivitySpace.");
        }
      // Cache all settings on IPTSessionInfo.
      // Verify there's a locale, access style, and timezone.
      // If those are not available, use defaults.
      // See SetAndCachePersonalSetting to understand how some prefs get set on lower-level 
      // HTTPSession
      bool bFoundLocale = false;
      bool bFoundAccessStyle = false;
      bool bFoundTimezone = false;
      // keep track of which settings to cache have values
      XPHashtable xpmSettingsWithValues = new XPHashtable();
      IPTSessionInfo sessionInfo = ptSession.GetSessionInfo();
      // NOTE: The access style value is more complicated than the rest of the values
      // to cache.  It is relied on by many parts of the server, and therefore should
      // always be cached.
      // If the user has previously logged on as a 508 user, they will have an access
      // style cookie, and that value should be used above all others.
      // If they are the guest user, there is a config value that tells whether or not
      // to override the guest user access setting.  If this override is set to true,
      // then the accessibility value from the config file should be used for the
      // guest user.
      // In all other cases, the accessibility value should be retrieved from the
      // SessionInfo object.
      // If this fails, the accessibility value from the config file should be used.
      if (LoginHelper.INSTANCE.GetIsGuestUser(ptSession))
      {
        // JF- If this is the Guest user, we first check if there is an access style cookie
        String sStyleFromCookie = appData.GetCookie(ACCESS_STYLE_COOKIE_NAME);
        bool bIsValidAccessID = false;
        if (null != sStyleFromCookie)
        {
          if (!"".Equals(sStyleFromCookie))
          {
            if (AccessStyles.IsValidAccessID(XPConvert.ToInteger(sStyleFromCookie)))
            {
              bIsValidAccessID = true;
            }
          }
        }
        if (bIsValidAccessID)
        {
          log.Debug("CachePersonalSettings() - Using access style from cookie for guest user (" + sStyleFromCookie + ").");
          bFoundAccessStyle = true;
          // JF- If we found a valid cookie, we take this value
          PersonalSettingsHelper.SetPersonalSettingValue(PersonalSettingNames.ACCESS_STYLE,
           sStyleFromCookie, sessionManager);
          // remember that we've cached this setting		
          xpmSettingsWithValues.PutElement(PersonalSettingNames.ACCESS_STYLE,
           sStyleFromCookie);
        } else
        {
          try
          {
            // Check x_config.xml to see if we should override the guest user access style
            // with the value from the config file.
            int nOverride = ConfigHelper.GetConfigVariableAsInt(ConfigConstants.PERSONAL_SETTINGS_SECTION,
             ConfigConstants.GUEST_ACCESS_STYLE_OVERRIDE, ConfigConstants.VALUE_ATTRIBUTE,
             (Application) application);
            if (nOverride == 1)
            {
              // Get the default accessibility value from the x_config.xml file and store it
              // on the session.
              String strAccessStyle = GetAccessibilityValueFromConfig(application);
              log.Debug("CachePersonalSettings() - Using guest override access style from x_config.xml (" + strAccessStyle + ").");
              PersonalSettingsHelper.SetPersonalSettingValue(PersonalSettingNames.ACCESS_STYLE,
               strAccessStyle, sessionManager);
              // remember that we've cached this setting		
              xpmSettingsWithValues.PutElement(PersonalSettingNames.ACCESS_STYLE,
               strAccessStyle);
              bFoundAccessStyle = true;
            }
          }
          catch (Exception e)
          {
            log.Warn(e, "CachePersonalSettings() - Error retrieving OverrideGuestAccessStyle or Accessibility value from x_config.xml file.");
            // Do nothing other than log the exception.  We will get the access style
            // from the guest user below.
            bFoundAccessStyle = false;
          }
        }
      }
      // Retrieve all the user's cached preferences, or load them all from the server if needed,
      // via IPTSessionInfo 
      Object[][] oaSettings = (Object[][]) sessionInfo.LookupPreferences((String[]) null,
       0);
      Object oVal = null;
      String sSettingName;
      //	iterate all of the users personal settings from SessionInfo (DB).
      for (int iSettingIndex = 0;iSettingIndex < oaSettings[0].Length;++iSettingIndex)
      {
        sSettingName = (String) oaSettings[0][iSettingIndex];
        oVal = oaSettings[1][iSettingIndex];
        // Only cache the access style if we did not find it and cache it from the cookie above.
        if (!bFoundAccessStyle || !sSettingName.Equals(PersonalSettingNames.ACCESS_STYLE))
        {
          // If this setting needs to be cached on the lower-level HTTPSession, do so		
          if (PTPersonalSettingsHelper.IsThisASettingToCache(application,
           sSettingName))
          {
            PersonalSettingsHelper.SetPersonalSettingValue(sSettingName, oVal,
             sessionManager);
            //	remember that we've cached this setting		
            xpmSettingsWithValues.PutElement(sSettingName, oVal);
          }
        }
        if (sSettingName.Equals(PersonalSettingNames.LOCALE))
        {
          if (null == oVal)
          {
            // Treat null and the empty string as "Use Default"
            oVal = "";
          }
          // JAO -- BEGIN -- We want to make sure that locale is always stored in the DB
          // as lowercase (i.e. en-us not en-US). It is easier to make sure that it is
          // lowercase here than to make sure people are always using a case insensitive
          // comparison. It is not too expensive to do here either.
          String strSettingValue = (String) oVal;
          String strSettingValueLowerCase = XPStringUtility.ToLowerCase(strSettingValue);
          // Both of the following Set*PersonalSettings methods save the lang/locale info
          // as runtime settings on the ptsession as well.  This is because the server
          // doesn't use the lang/locale info from the sessioninfo object, but only from
          // the ptSession object.
          if (!strSettingValue.Equals(strSettingValueLowerCase))
          {
            oVal = strSettingValueLowerCase;
            // This will cache the locale and language for the UI and as a
            // runtime setting, as well as store it in the DB.
            // We put the locale from the session info (which gets it
            // from the DB) back in the DB after we have made it lower case.
            SetAndCachePersonalSetting(PersonalSettingNames.LOCALE, strSettingValueLowerCase,
             ptSession, sessionManager, application, rm);
          } else
          {
            // This will cache the locale for the UI and as a runtime setting, but not
            // save it to the DB (because it is the same as the value in the DB).
            SetPersonalSetting(PersonalSettingNames.LOCALE, strSettingValueLowerCase,
             ptSession, sessionManager, application, rm);
          }
          bFoundLocale = true;
        } else
        // JAO -- END
        if (sSettingName.Equals(PersonalSettingNames.ACCESS_STYLE))
        {
          bFoundAccessStyle = true;
        } else
        if (sSettingName.Equals(PersonalSettingNames.TIMEZONE))
        {
          bFoundTimezone = true;
        }
      }
      //	get an enumeration of settings to cache
      XPArrayList xpmSettingsToCache = GetSettingsToCache(application);
      //	now iterate all of the settings to cache
      for (int i = 0;i < xpmSettingsToCache.GetSize();i++)
      {
        sSettingName = (String) xpmSettingsToCache.GetElement(i);
        if (!xpmSettingsWithValues.ContainsKey(sSettingName))
        {
          //	if it hasn't already been cached, cache it with the NO_VALUE
          //	value.  This will prevent unnecessary db queries in the future
          PersonalSettingsHelper.SetPersonalSettingValue(sSettingName, PTPersonalSettingsHelper.NO_VALUE,
           sessionManager);
        }
      }
      String strLocale = (String) GetPersonalSettingValue(PersonalSettingNames.LOCALE,
       sessionManager, application, ptSession);
      // We don't check if the use default or use browser cases are supported locales here
      // because they will be converted into a real locale elsewhere.
      if ((null != strLocale) && (0 != XPStringUtility.CompareIgnoreCase(LOCALE_USE_DEFAULT_SETTING,
       strLocale)) && (0 != XPStringUtility.CompareIgnoreCase(LOCALE_USE_BROWSER_SETTING,
       strLocale)))
      {
        // Check whether or not this locale is actually supported by this instance of
        // the server (i.e. we have language files for that locale).  We will have to
        // check later if we are going to use the browser settings.
        if (!IsLocaleSupported(strLocale, rm))
        {
          log.Info("CachePersonalSettings() - Locale " + strLocale + " not supported by this server, will use default settings instead.");
          // This will force us to use the default settings below.
          bFoundLocale = false;
        }
      }
      // If we didn't find the locale setting, or we are
      // supposed to use the default, use the default.
      if ((!bFoundLocale) || (null == strLocale) || (0 == XPStringUtility.CompareIgnoreCase(LOCALE_USE_DEFAULT_SETTING,
       strLocale)))
      {
        log.Debug("CachePersonalSettings() - No locale found in personal settings or supposed to use default, using default locale from x_config.xml.");
        String sDefaultLocale = "";
        try
        {
          // JF- Get the locale from config.xml - this will return the empty string if the value
          // is not set.
          sDefaultLocale = ConfigHelper.GetConfigVariableAsString(ConfigConstants.INTERNATIONAL_SECTION,
           ConfigConstants.LOCALE_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE,
           application);
          if ((null != sDefaultLocale) && (!("").Equals(sDefaultLocale)) && (!XPStringUtility.EqualsIgnoreCase(LOCALE_USE_BROWSER_SETTING,
           sDefaultLocale)))
          {
            // JF- The Locale Settings page assumes that the locale is lower case...
            sDefaultLocale = XPStringUtility.ToLowerCase(sDefaultLocale);
            // Check whether or not this locale is actually supported by this instance of
            // the server (i.e. we have language files for that locale).  We will have to
            // check later if we are going to use the browser settings.
            if (!IsLocaleSupported(sDefaultLocale, rm))
            {
              log.Info("CachePersonalSettings() - Default locale " + sDefaultLocale + " not supported by this server, will use browser settings instead.");
              sDefaultLocale = LOCALE_USE_BROWSER_SETTING;
            }
            // Do not promote the locale setting to the DB for the guest user or any
            // default profile users.
            if (!LoginHelper.INSTANCE.GetIsGuestUser(ptSession) && !PlumtreeHelpers.IsEditingDefaultProfile(appData.GetPersistentSubSession()))
            {
              // Promote the default locale to be the users new locale.
              // This will cache the locale and language for the UI and as a
              // runtime setting, as well as store it in the DB.
              SetAndCachePersonalSetting(PersonalSettingNames.LOCALE, sDefaultLocale,
               ptSession, sessionManager, application, rm);
            }
          }
        }
        catch (Exception e)
        {
          log.Info(e, "CachePersonalSettings() - Unable to get default locale from x_config.xml," + " will use browser settings instead.");
          sDefaultLocale = LOCALE_USE_BROWSER_SETTING;
        }
        // Since we've already retrieved the default locale, treat default settings
        // as "use browser" so we can still get a decent locale.
        if ((null == sDefaultLocale) || (0 == XPStringUtility.Compare(LOCALE_USE_DEFAULT_SETTING,
         sDefaultLocale)))
        {
          sDefaultLocale = LOCALE_USE_BROWSER_SETTING;
        }
        // JF- The Locale Settings page assumes that the locale is lower case...
        sDefaultLocale = XPStringUtility.ToLowerCase(sDefaultLocale);
        log.Debug("CachePersonalSettings() - Using the following default locale: " + sDefaultLocale);
        // This will cache the locale for the UI and as a runtime setting, but not
        // save it to the DB.
        SetPersonalSetting(PersonalSettingNames.LOCALE, sDefaultLocale, ptSession,
         sessionManager, application, rm);
      }
      if (!bFoundAccessStyle)
      {
        // Get the default accessibility value from the x_config.xml file and store it
        // on the session.
        String strAccessStyle = GetAccessibilityValueFromConfig(application);
        SetAndCachePersonalSetting(PersonalSettingNames.ACCESS_STYLE, strAccessStyle,
         ptSession, sessionManager, application, rm);
        log.Debug("CachePersonalSettings() - No access style found in personal settings, using default access style from x_config.xml (" + strAccessStyle + ").");
        }
      if (!bFoundTimezone)
      {
        log.Debug("CachePersonalSettings() - No timezone found in personal settings, using default timezone from x_config.xml.");
        String sDefaultTimeZone = "-1";
        try
        {
          sDefaultTimeZone = PTConfigHelper.GetConfigVariableAsString(ConfigConstants.INTERNATIONAL_SECTION,
           ConfigConstants.TIMEZONE_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE,
           application);
        }
        catch (Exception e)
        {
          log.Debug(e, "CachePersonalSettings() - Unable to get default timezone from x_config.xml," + " will use the internal default timezone instead.");
          }
        // If the default timezone is set to -1, set to XPTimeZone.GetCurrentTimeZone()
        if (sDefaultTimeZone.Equals("-1") || (0 == sDefaultTimeZone.Length))
        {
          log.Debug("CachePersonalSettings() - The default time zone from x_config.xml " + "is invalid, using the default internal time zone instead.");
          sDefaultTimeZone = XPTimeZone.GetCurrentTimeZone().GetTZName();
          }
        log.Debug("CachePersonalSettings() - Using the following default time zone: " + sDefaultTimeZone);
        SetPersonalSetting(PersonalSettingNames.TIMEZONE, sDefaultTimeZone,
         ptSession, sessionManager, application, rm);
      }
      // Calculate the user's strBaseURL preference for multiple domains.
      // The preference will be stored in the form of http://<host>:<port>
      // or https://<host>:<port>.  The preference will be used to to calculate the
      // imageserver base url when it is configured as relative in portalconfig.xml.
      // This imageserver base url gets passed in the AppDataObject to the portlet stack.
      String appBaseURL = ConfigHelper.GetApplicationBaseURL(ConfigHelper.GetNormalizedBaseURL(appData.GetRequestURL()));
      log.Debug("CachePersonalSettings() - RequestURL is: " + appData.GetRequestURL());
      log.Debug("CachePersonalSettings() - AppBaseURL is: " + appBaseURL);
      int indProtocol = XPStringUtility.IndexOf(appBaseURL, "://") + 3;
      appBaseURL = XPStringUtility.Substring(appBaseURL, 0, XPStringUtility.IndexOf(appBaseURL,
       "/", indProtocol));
      log.Debug("CachePersonalSettings() - strBaseURL is: " + appBaseURL);
      // Cache on http session if configured; always store as portal user pref.
      // SE: Bug 64401 - Fix for DFAS. Only store to base URL preference to DB 
      // if it is non-guest user. Otherwise, just store on session.
      if (!ptSession.GetUser().IsGuestUser())
      {
        String strBaseURL = XPConvert.ToString(GetPersonalSettingValue(PersonalSettingNames.BASE_URL,
         sessionManager, application, ptSession));
        // Only store preference to DB if there is a change.
        if (null == strBaseURL || 0 != XPStringUtility.CompareIgnoreCase(appBaseURL,
         strBaseURL))
        {
          SetAndCachePersonalSetting(PersonalSettingNames.BASE_URL, appBaseURL,
           ptSession, sessionManager, application, rm);
        }
      } else
      {
        // This is a guest, only store to http session.
        SetPersonalSetting(PersonalSettingNames.BASE_URL, appBaseURL, ptSession,
         sessionManager, application, rm);
      }
    }
    // TODO: this get-default approach should be extended to all other entries on
    // IsThisASettingToCache()
    
    /// <summary>
    /// Helper method to get access style (i.e. 508) from x_config file and cache it on
    /// the subsession.
    /// </summary>
    private static String GetAccessibilityValueFromConfig(IApplication application)
    {
      int nDefaultAccessStyle = -1;
      try
      {
        // JF- Get the default Access style from x_config.xml
        nDefaultAccessStyle = ConfigHelper.GetConfigVariableAsInt(ConfigConstants.PERSONAL_SETTINGS_SECTION,
         ConfigConstants.ACCESSIBILITY_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE,
         (Application) application);
      }
      catch (Exception e)
      {
        log.Debug(e, "CachePersonalSettings() - Unable to get default access style from x_config.xml," + " will use the internal default access style instead.");
        }
      // JF- Make sure it's valid, if not, get the default one from AccessStyles
      if (!AccessStyles.IsValidAccessID(nDefaultAccessStyle))
      {
        log.Debug("CachePersonalSettings() - The default access style from x_config.xml " + "is invalid, using the default internal access style instead.");
        nDefaultAccessStyle = AccessStyles.GetDefaultAccessID();
      }
      log.Debug("CachePersonalSettings() - Using the following default access style: " + nDefaultAccessStyle);
      return XPConvert.ToString(nDefaultAccessStyle);
    }
    
    /// <summary>
    /// Call this method to determine if the subsession for caching personal
    /// settings exists in the specified ISessionManager.
    /// <br /><br />
    /// </summary>
    /// <param name="ISessionManager"></param>
    public static bool DoesPrefsSubSessionExist(AActivitySpace asSpace)
    {
      return PersonalSettingsHelper.DoesPrefsSubSessionExist(asSpace);
    }
    
    /// <summary>
    /// Call this method to find the value associated with given setting name.
    /// <br /><br />
    /// </summary>
    /// <param name="oaSettings">Pass an Object[][] obtained from LookupPersonalSettings</param>
    /// <param name="String">name of setting</param>
    public static Object FindPersonalSetting(Object[][] oaSettings, String sName)
    {
      return PersonalSettingsHelper.FindPersonalSetting(oaSettings, sName);
      }
    
    /// <summary>
    /// Call this method to find the remove the specified setting from the DB
    /// <br /><br />
    /// </summary>
    /// <param name="oaSettings">Pass an Object[][] obtained from LookupPersonalSettings</param>
    /// <param name="String">name of setting</param>
    public static void RemovePersonalSetting(String sSettingName, AActivitySpace asSpace)
    {
      //		This version of this method simply makes sure you're using the right sandbox off of HTTPSession
      // derived from AActivitySpace
      IPTSession ptSession = GetPTSession(asSpace);
      IPTSessionInfo sessionInfo = ptSession.GetSessionInfo();
      ISessionManager sessionManager = asSpace.GetSubSession();
      sessionInfo.RemovePreference(sSettingName, 0);
      PersonalSettingsHelper.RemovePersonalSetting(sSettingName, sessionManager);
      }
    
    /// <summary>
    /// Call this method to obtain the value of a personal setting.
    /// <br /><br />
    /// This value will be retrieved from the user's web session if possible,
    /// otherwise the portal server will be queried.
    /// <br /><br />
    /// </summary>
    /// <param name="sSettingName">String name of personal setting</param>
    /// <param name="as">AActivitySpace</param>
    /// <throws>
    /// XPException This method will throw an XPException if you do not
    /// pass an ISessionManager already containing the correct subSession.  The
    /// ISessionManager obtained from your Activity Space will contain the correct
    /// subsession.
    /// </throws>
    public static Object GetPersonalSettingValue(String sSettingName, AActivitySpace asSpace)
    {
      // This version of this method simply makes sure you're using the right sandbox off of HTTPSession
      // derived from AActivitySpace
      IPTSession ptSession = GetPTSession(asSpace);
      IApplication app = asSpace.GetApplication();
      ISessionManager sessionManager = asSpace.GetSubSession();
      return GetPersonalSettingValue(sSettingName, sessionManager, app, ptSession);
      }
    
    /// <summary>
    /// Only use this version of this function if you do not have an
    /// AActivitySpace.
    /// <br /><br />
    /// This value will be retrieved from the user's web session if possible,
    /// otherwise the portal server will be queried.
    /// <br /><br />
    /// </summary>
    /// <param name="sSettingName">String name of personal setting</param>
    /// <param name="sessionManager">ISessionManager</param>
    /// <param name="ptSession">IPTSession</param>
    /// <throws>
    /// XPException This method will throw an XPException if you do not
    /// pass an ISessionManager already containing the correct subSession.  The
    /// ISessionManager obtained from your Activity Space will contain the correct
    /// subsession.
    /// </throws>
    public static Object GetPersonalSettingValue(String sSettingName, ISessionManager sessionManager,
     IApplication application, IPTSession ptSession)
    {
      // This version of this function is the raw low-level individual settings getter
      log.Debug("Entering GetPersonalSettingValue() - sSettingsName: " + sSettingName);
      Object oVal = null;
      if (!PersonalSettingsHelper.DoesPrefsSubSessionExist(sessionManager))
      {
        throw new XPException("SubSession for prefs cache does not exist, please make sure you\'re passing the top-level ISessionManager object retrieved from the ActivitySpace.");
        }
      // Get the setting out of the low level UI settings cache (i.e. HTTP Session)
      oVal = PersonalSettingsHelper.GetPersonalSettingValue(sSettingName,
       sessionManager, application);
      if (oVal == null)
      {
        // no value was cached on the low-level HTTP session
        log.Debug("GetPersonalSettingValue() - Couldn\'t get from the session cache, about to get from server.");
        IPTSessionInfo sessionInfo = ptSession.GetSessionInfo();
        // Try to retrieve the setting from IPTSessionInfo.
        Object[][] personalSettings = (Object[][]) sessionInfo.LookupPreference(sSettingName,
         0);
        oVal = PTPersonalSettingsHelper.FindPersonalSetting(personalSettings,
         sSettingName);
        if ((oVal == null) && (PersonalSettingNames.LANGUAGE.Equals(sSettingName)))
        {
          XPResourceManager rm = (XPResourceManager) application.GetCachingManager().GetEntry("ResourceManager");
          // get the locale from the x_config.xml file and convert it to the language
          String strLocale = ConfigHelper.GetConfigVariableAsString(ConfigConstants.INTERNATIONAL_SECTION,
           ConfigConstants.LOCALE_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE,
           application);
          // This will return the supported language based on this locale, or the default
          // if we do not support the language in this locale.
          oVal = rm.MapLocaleToSupportedUILangID(strLocale);
        }
        // Check to see if this specific setting should be also stored on the low-level HTTPSession
        // List of values to store lower found in x_config.xml under the <PersonalSettings> node 
        if (PTPersonalSettingsHelper.IsThisASettingToCache(application, sSettingName))
        {
          if (null == oVal)
          {
            //	nothing found in db, we cache this NoValue constant to
            //	prevent useless db lookups in the future.
            oVal = NO_VALUE;
          }
          //	save on the http session.
          PersonalSettingsHelper.SetPersonalSettingValue(sSettingName, oVal,
           sessionManager);
        }
      } else
      {
        log.Debug("GetPersonalSettingValue() - Got setting from the session cache, oVal: " + oVal);
        }
      log.Debug("GetPersonalSettingValue() - oVal: " + oVal);
      if (oVal == NO_VALUE)
      {
        //	if the value we got is not a true value, but a novalue,
        //	then we return null, because that's what people expect.
        oVal = null;
      }
      return oVal;
    }
    
    /// <summary>
    /// This method returns an XPLocale object created from the current user's
    /// locale. If that cannot be done, then it returns an XPLocale made from
    /// the default locale.
    /// </summary>
    /// <param name="_asSpace">The current Activity Space.</param>
    /// <returns>The XPLocale.</returns>
    public static XPLocale GetPersonalSettingValue_Locale(AActivitySpace _asSpace)
    {
      String strLocaleID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LOCALE,
       _asSpace);
      XPLocale retValue = null;
      try
      {
        retValue = new XPLocale(strLocaleID);
        // If we could not get a valid locale
        if (null == retValue)
        {
          String sDefaultLocale = "";
          try
          {
            // JF- Get the locale from config.xml
            sDefaultLocale = ConfigHelper.GetConfigVariableAsString(ConfigConstants.INTERNATIONAL_SECTION,
             ConfigConstants.LOCALE_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE,
             _asSpace.GetApplication());
          }
          catch (Exception e)
          {
            log.Debug(e, "CachePersonalSettings() - Unable to get default locale from x_config.xml," + " will use the internal default locale instead.");
            }
          // JF- Get the default locale from XPLocale if needed
          if ((0 == sDefaultLocale.Length) || (-1 == XPStringUtility.IndexOf(sDefaultLocale,
           "-")))
          {
            sDefaultLocale = XPLocale.GetDefault().GetLanguage() + "-" + XPLocale.GetDefault().GetCountry();
            }
          // JF- The Locale Settings page assumes that the locale is lower case...
          sDefaultLocale = XPStringUtility.ToLowerCase(sDefaultLocale);
          retValue = new XPLocale(sDefaultLocale);
          if (null == retValue)
          {
            retValue = XPLocale.GetDefault();
          }
          log.Debug("GetPersonalSettingValue_Locale() - Using the following default locale: " + retValue.GetLanguage() + "-" + retValue.GetCountry());
          }
      }
      catch (Exception e)
      {
        retValue = XPLocale.GetDefault();
      }
      return retValue;
    }
    
    /// <summary>
    /// This method returns an XPTimeZone object created from the current user's
    /// time zone. If that cannot be done, then it returns an XPTimeZone made from
    /// the default time zone.
    /// </summary>
    /// <param name="_asSpace">The current Activity Space.</param>
    /// <returns>The XPTimeZone.</returns>
    public static XPTimeZone GetPersonalSettingValue_TimeZone(AActivitySpace _asSpace)
    {
      String strTimeZoneID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.TIMEZONE,
       _asSpace);
      XPTimeZone retValue = null;
      try
      {
        retValue = XPTimeZone.GetTimeZone(strTimeZoneID);
        if (null == retValue)
        {
          retValue = XPTimeZone.GetCurrentTimeZone();
        }
      }
      catch (Exception e)
      {
        retValue = XPTimeZone.GetCurrentTimeZone();
      }
      return retValue;
    }
    
    /// <summary>
    /// This method returns an XPDateFormat in the user's current locale and timezone.
    /// It's basically a wrapper around GetPersonalSettingValue_Locale and
    /// GetPersonalSettingValue_TimeZone.
    /// </summary>
    public static XPDateFormat GetPersonalDateTimeFormat(AActivitySpace _asSpace,
     bool _localizedToUsersTimeZone)
    {
      XPLocale xpLocale = GetPersonalSettingValue_Locale(_asSpace);
      XPTimeZone xpTimeZone = null;
      if (_localizedToUsersTimeZone)
      {
        xpTimeZone = GetPersonalSettingValue_TimeZone(_asSpace);
      } else
      {
        xpTimeZone = XPTimeZone.GetCurrentTimeZone();
      }
      return XPDateFormat.GetDateTimeInstance(XPDateFormat.SHORT, XPDateFormat.SHORT,
       xpLocale, xpTimeZone);
    }
    
    /// <summary>
    /// Retrieve the individual settings to cache in the lower-level HTTPSession as opposed to
    /// the higher-level IPTSessionInfo.  Defined in x_config.xml
    /// </summary>
    /// <param name="app">Application object</param>
    /// <returns>Hashmap of settings to be cached </returns>
    public static XPArrayList GetSettingsToCache(IApplication app)
    {
      // NOTE: we originally cached a hashtable, but this can cause
      // problems in certain weird situations (related to the gateway)
      // where multiple threads attempt to login at the same time.  One
      // thread creates the hash table and starts filling it it, and
      // then the second thread starts iterating it before the first
      // thread is finished initializing it.  The second thread then
      // gets an Illegal State Exception when the first thread adds items
      // to the hash table that it is iterating over.  The simple
      // solution is to not cache this list, thereby not having
      // any shared memory.  There is no good reason to cache this
      // anyways.
      //	all settings to cache are listed in x_config.xml
      Object[] oaSettingsToCache = null;
      try
      {
        oaSettingsToCache = PTConfigHelper.GetConfigVariables("CachedSettings",
         "SettingToCache", "value", app);
      }
      catch (Exception ex)
      {
        log.Error(ex, "GetSettingsToCache() - Exception: ");
      }
      if (oaSettingsToCache != null)
      {
        XPArrayList xpmSettingsToCache = new XPArrayList();
        for (int x = 0;x < oaSettingsToCache.Length;x++)
        {
          xpmSettingsToCache.Add(oaSettingsToCache[x]);
        }
        return xpmSettingsToCache;
      }
      return null;
    }
    
    /// <summary>
    /// Call this method to set a personal setting name value pair on the
    /// portal server, and update the session cache of personal settings.
    /// <br /><br />
    /// </summary>
    /// <param name="sSettingName">String name of personal setting</param>
    /// <param name="oSettingVal">Object value of personal setting</param>
    /// <param name="as">
    /// AActivitySpace Pass an AActivitySpace implementing
    /// IXPActivitySpace.
    /// </param>
    /// <throws>XPException This method may throw an XPException.</throws>
    public static void SetAndCachePersonalSetting(String sSettingName, Object oSettingVal,
     AActivitySpace asSpace)
    {
      IPTSession ptSession = GetPTSession(asSpace);
      IApplication app = asSpace.GetApplication();
      ISessionManager sessionManager = asSpace.GetSubSession();
      XPResourceManager rm = asSpace.GetResourceManager();
      SetAndCachePersonalSetting(sSettingName, oSettingVal, ptSession, sessionManager,
       app, rm);
    }
    
    /// <summary>
    /// Simply checks if this specific setting needs to be cached in the lower-level HTTPSession
    /// </summary>
    /// <param name="app"></param>
    /// <param name="sSettingName"></param>
    /// <returns></returns>
    private static bool IsThisASettingToCache(IApplication app, String sSettingName)
    {
      // TODO: We should probably also always cache language.
      if ((XPStringUtility.EqualsIgnoreCase(sSettingName, PersonalSettingNames.LOCALE)) || (XPStringUtility.EqualsIgnoreCase(sSettingName,
       PersonalSettingNames.ACCESS_STYLE)) || (XPStringUtility.EqualsIgnoreCase(sSettingName,
       PersonalSettingNames.TIMEZONE)))
      {
        // Always cache these settings, because we need them.
        return true;
      }
      //	Not all personal settings are to be cached.
      //	only those specified in x_config.xml are cached.
      XPArrayList xpmSettingsToCache = PTPersonalSettingsHelper.GetSettingsToCache(app);
      if (xpmSettingsToCache != null)
      {
        return xpmSettingsToCache.Contains(sSettingName);
      }
      return true;
    }
    
    /// <summary>
    /// This method stores a personal setting on the HTTP
    /// Session, but not in the DB.
    /// It also handles setting the runtime settings in
    /// the server for language and locale if you try to
    /// set the locale.  You should always set the Locale,
    /// which will automatically set the language, rather
    /// than just setting the language, which will allow
    /// the language and locale to get out of sync.
    /// </summary>
    /// <param name="sSettingName">String name of personal setting</param>
    /// <param name="oSettingVal">Object value of personal setting</param>
    /// <param name="as">
    /// AActivitySpace Pass an AActivitySpace implementing
    /// IXPActivitySpace.
    /// </param>
    /// <throws>XPException This method may throw an XPException.</throws>
    public static void SetPersonalSetting(String sSettingName, Object oSettingVal,
     IPTSession ptSession, ISessionManager sessionManager, IApplication app,
     XPResourceManager rm)
    {
      log.Debug("Entering SetPersonalSetting() - sSettingsName: " + sSettingName);
      InternalSetAndCachePersonalSetting(sSettingName, oSettingVal, ptSession,
       false, sessionManager, app, rm);
    }
    
    /// <summary>
    /// This method stores a personal setting on the HTTP
    /// Session and in the DB.
    /// It also handles setting the runtime settings in
    /// the server for language and locale if you try to
    /// set the locale.  You should always set the Locale,
    /// which will automatically set the language, rather
    /// than just setting the language, which will allow
    /// the language and locale to get out of sync.
    /// Only call this version of this function if you don't have an
    /// AActivitySpace.
    /// <br /><br />
    /// </summary>
    /// <param name="sSettingName">String name of personal setting</param>
    /// <param name="oSettingVal">Object value of personal setting</param>
    /// <param name="as">
    /// AActivitySpace Pass an AActivitySpace implementing
    /// IXPActivitySpace.
    /// </param>
    /// <throws>XPException This method may throw an XPException.</throws>
    public static void SetAndCachePersonalSetting(String sSettingName, Object oSettingVal,
     IPTSession ptSession, ISessionManager sessionManager, IApplication app,
     XPResourceManager rm)
    {
      log.Debug("Entering SetAndCachePersonalSetting() - sSettingsName: " + sSettingName);
      InternalSetAndCachePersonalSetting(sSettingName, oSettingVal, ptSession,
       true, sessionManager, app, rm);
    }
    private static void InternalSetAndCachePersonalSetting(String sSettingName,
     Object oSettingVal, IPTSession ptSession, bool bStoreToDB, ISessionManager sessionManager,
     IApplication app, XPResourceManager rm)
    {
      // If this setting needs to be cached on the lower-level HTTPSession, do so		
      if (PTPersonalSettingsHelper.IsThisASettingToCache(app, sSettingName))
      {
        PersonalSettingsHelper.SetPersonalSettingValue(sSettingName, oSettingVal,
         sessionManager);
      }
      if (bStoreToDB)
      {
        // Add this setting to the higher-level IPTSessionInfo
        IPTSessionInfo sessionInfo = ptSession.GetSessionInfo();
        // This gets persisted to the DB
        sessionInfo.AddPreference(sSettingName, oSettingVal, 0);
      }
      // If changing locale, ensure language is changed also
      if (PersonalSettingNames.LOCALE.Equals(sSettingName))
      {
        // if the client is attempting to set the locale personal setting,
        // then we must also update the language.
        String sLocale = null;
        try
        {
          sLocale = (String) oSettingVal;
        }
        catch (Exception ex)
        {
          log.Error("The value of the locale setting must be a String: " + oSettingVal);
          }
        // Verify if the Locale is custom mapped to a different locale in
        // LocaleMap.xml file, then we use the custom locale
        PTLocaleMap lmap = PTLocaleMap.GetInstance();
        if (lmap.IsLocaleMapped(sLocale))
        {
          sLocale = lmap.GetMappedLocale(sLocale);
        }
        String sLangID = rm.MapLocaleToSupportedUILangID(sLocale);
        // Set the runtime settings in the server (these are not persisted in the DB).
        // These need to be set separately, because the server doesn't actually use
        // the sessionInfo PersonalSettings lang/locale settings.  It only uses the
        // ptSession settings.
        ptSession.SetSettings(PT_SESSION_SETTINGS.PT_SESSION_LOCALE, sLocale);
        ptSession.SetSettings(PT_SESSION_SETTINGS.PT_SESSION_LANGUAGE, sLangID);
        // And record the language into the UI settings cache.
        PersonalSettingsHelper.SetPersonalSettingValue(PersonalSettingsHelper.LOCALE,
         sLocale, sessionManager);
        PersonalSettingsHelper.SetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE,
         sLangID, sessionManager);
      }
    }
    
    /// <summary>
    /// Method GetPTSession.  Gets IPTSession from AActivitySpace cleanly.
    /// </summary>
    /// <param name="as"></param>
    /// <returns>IPTSession</returns>
    public static IPTSession GetPTSession(AActivitySpace asSpace)
    {
      IPTSession ptSession;
      try
      {
        ptSession = (IPTSession) asSpace.GetUserSession();
      }
      catch (Exception ex)
      {
        log.Error(ex, "Couldn\'t cast UserSession to IPTSession.");
        throw new XPException("The user session must derive from IPTSession.",
         ex);
      }
      return ptSession;
    }
    
    /// <summary>
    /// This helper method determines the fall back default locale,
    /// assuming that the locale could not be determined by the
    /// browser settings.
    /// This method will not return UseBrowser as a locale.
    /// </summary>
    /// <param name="application"></param>
    /// <returns>the default locale</returns>
    public static String GetDefaultLocale(IApplication application)
    {
      XPResourceManager rm = (XPResourceManager) application.GetCachingManager().GetEntry("ResourceManager");
      String sDefaultLocale = "";
      try
      {
        // JF- Get the locale from config.xml
        sDefaultLocale = ConfigHelper.GetConfigVariableAsString(ConfigConstants.INTERNATIONAL_SECTION,
         ConfigConstants.LOCALE_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE,
         application);
      }
      catch (Exception e)
      {
        log.Debug(e, "CachePersonalSettings() - Unable to get default locale from x_config.xml," + " will use the internal default locale instead.");
        sDefaultLocale = "";
      }
      if (0 == XPStringUtility.CompareIgnoreCase(sDefaultLocale, LOCALE_USE_BROWSER_SETTING))
      {
        // We don't want to return UseBrowser.  Setting this to empty string will cause
        // the internal default locale to be calculated below.
        sDefaultLocale = "";
      }
      if (!IsLocaleSupported(sDefaultLocale, rm))
      {
        sDefaultLocale = "";
      }
      // JF- Get the default locale from XPLocale if needed
      if ((0 == sDefaultLocale.Length) || (-1 == XPStringUtility.IndexOf(sDefaultLocale,
       "-")))
      {
        // The default locale may not be a supported locale.  We need to make
        // sure that it is, or use a supported locale if it is not.
        sDefaultLocale = XPLocale.GetDefault().GetLanguage() + "-" + XPLocale.GetDefault().GetCountry();
        if (!IsLocaleSupported(sDefaultLocale, rm))
        {
          // Bug#32140 -- when this bug is fixed, we shouldn't have to do this.
          // Map the first supported language (randomly chosen) to it's most
          // common supported locale and, if we can't come up with anything
          // better, just use english.
          sDefaultLocale = rm.MapLocaleToUISupportedLocaleID(rm.GetSupportedLanguages()[0],
           "en-us");
        }
      }
      // JF- The Locale Settings page assumes that the locale is lower case...
      sDefaultLocale = XPStringUtility.ToLowerCase(sDefaultLocale);
      return sDefaultLocale;
    }
    
    /// <summary>
    /// This method determines whether or not the supplied locale
    /// matches one of the languages that this server can actually
    /// display.
    /// </summary>
    /// <param name="strLocale"></param>
    /// <param name="rm"></param>
    /// <returns>true implies the locale is supported</returns>
    public static bool IsLocaleSupported(String strLocale, XPResourceManager rm)
    {
      String[] strSupportedLanguages = rm.GetSupportedLanguages();
      int nSeparatorIndex = XPStringUtility.IndexOf(strLocale, "-");
      bool bSupported = false;
      if (nSeparatorIndex > 0)
      {
        String strLanguageName = XPStringUtility.Substring(strLocale, 0, nSeparatorIndex);
        for (int i = 0;i < strSupportedLanguages.Length;i++)
        {
          if (0 == XPStringUtility.CompareIgnoreCase(strLanguageName, strSupportedLanguages[i]))
          {
            bSupported = true;
          }
        }
      }
      return bSupported;
    }
    
    /// <summary>
    /// internal helper class 
    /// </summary>
    private class NoValue
    {
      internal NoValue()
      {
      }
    }
  }
}