/*
 * Created on Mar 7, 2003
 */
 
using System;
using com.plumtree.openfoundation.util;
namespace com.plumtree.portaluiinfrastructure.navtype
{
  /// <summary>
  /// Enumerates the view names, by area, of the page where you can supply navigation.
  /// View names must be unique and be defined in GetName() in the view implementation 
  /// See dynamic loading, MVC names examples; must implement IView or extend of it
  /// The areas and their meanings are:
  /// ABOVEBODY: Return the name of the view for the area below the banner, or null if not implemented.
  /// RIGHTOFBODY: Return the name of the view for the area right of the page, or null if not implemented.
  /// LEFTOFBODY: Return the name of the view for the area left of the page, or null if not implemented. 
  /// BELOWBODY: Return the name of the view for the area above the footer, or null if not implemented.
  /// BELOWBANNER: Return the name of the view for the area below the Header, or null if not implemented. 
  /// ABOVEBANNER: Return the name of the view for the area above the Header, or null if not implemented. 
  /// BELOWFOOTER: Return the name of the view for the area below the footer, or null if not implemented. 
  /// ABOVEFOOTER: Return the name of the view for the area above the footer, or null if not implemented.
  /// TOPBAR: Return the name of the view for the topbar area, or null if not implemented.  
  /// </summary>
  /// <author>DavidP</author>
  /// <author>michaeld</author>
  public class NavAreaEnum
  {
    private int m_navAreaID;
    private String m_navAreaName;
    // When adding new Page Area, create a new NavAreaEnum and also add it to array below. 
    // enumeration of page areas
    public static readonly NavAreaEnum ABOVEBANNER = new NavAreaEnum(0, "ABOVEHEADER");
    public static readonly NavAreaEnum BELOWBANNER = new NavAreaEnum(1, "BELOWHEADER");
    public static readonly NavAreaEnum ABOVEBODY = new NavAreaEnum(2, "ABOVEBODY");
    public static readonly NavAreaEnum LEFTOFBODY = new NavAreaEnum(3, "LEFTOFBODY");
    public static readonly NavAreaEnum RIGHTOFBODY = new NavAreaEnum(4, "RIGHTOFBODY");
    public static readonly NavAreaEnum BELOWBODY = new NavAreaEnum(5, "BELOWBODY");
    public static readonly NavAreaEnum ABOVEFOOTER = new NavAreaEnum(6, "ABOVEFOOTER");
    public static readonly NavAreaEnum BELOWFOOTER = new NavAreaEnum(7, "BELOWFOOTER");
    public static readonly NavAreaEnum TOPBAR = new NavAreaEnum(8, "TOPBAR");
    // array of page types
    public static readonly NavAreaEnum[] m_navAreaEnums = {ABOVEBANNER, BELOWBANNER,
     ABOVEBODY, LEFTOFBODY, RIGHTOFBODY, BELOWBODY, ABOVEFOOTER, BELOWFOOTER,
     TOPBAR};
    
    /// <summary>
    /// Private constructor
    /// </summary>
    /// <param name="navModel"></param>
    private NavAreaEnum(int navTypeID, String navName)
    {
      m_navAreaID = navTypeID;
      m_navAreaName = navName;
    }
    
    /// <seealso cref="java.lang.Object#toString()"></seealso>
    public override String ToString()
    {
      String areaName = "UNDEFINED";
      if (m_navAreaID < m_navAreaEnums.Length)
      {
        areaName = m_navAreaEnums[m_navAreaID].m_navAreaName;
      }
      return areaName;
    }
    
    /// <summary>
    /// Converts the NavArea Name provied as string to the integer id.
    /// </summary>
    /// <returns>areaId valid navarea id if exists or -1 if no NavArea name matched.</returns>
    public static NavAreaEnum GetNavArea(String aName)
    {
      NavAreaEnum area = null;
      for (int i = 0;i < m_navAreaEnums.Length;i++)
      {
        if (XPStringUtility.EqualsIgnoreCase(aName, m_navAreaEnums[i].m_navAreaName))
        {
          area = m_navAreaEnums[i];
        }
      }
      return area;
    }
    
    /// <summary>
    /// Retrieves the integer ID for this navigation page area.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetID()
    {
      return m_navAreaID;
    }
    
    /// <summary>
    /// Retrieves the String Name for this navigation page area.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetName()
    {
      return m_navAreaName;
    }
    
    /// <seealso cref="java.lang.Object#equals(java.lang.Object)"></seealso>
    public sealed override bool Equals(Object o)
    {
      bool retval = false;
      try
      {
        retval = (((NavAreaEnum) o).GetID() == m_navAreaID);
      }
      catch (Exception e)
      {
      }
      return retval;
    }
  }
}