using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.friendlyurl;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.taskapi.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.helpers;
using com.plumtree.uiinfrastructure.opener;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.opener
{
  /// <summary>
  /// This helper object enables client programmers to
  /// easily and generically open objects throughout the
  /// Plumtree Portal system.
  /// </summary>
  public class PTOpenerLinks : OpenerLinks
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.opener.PTOpenerLinks");
    private static OpenLogger friendlyURLlog = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_FRIENDLYURL),
     "com.plumtree.portaluiinfrastructure.opener.PTOpenerLinks");
    
    /// <summary>
    /// Returns the Javascript needed by the Common Opener.  We add both dynamic 
    /// script and static script.  To provide a set of simpler API methods, some
    /// methods in the static script will call methods from the dynamic script.
    /// However, the static API will also include methods that don't depend on 
    /// the dynamic API.
    /// </summary>
    /// <returns>This method returns the HTMLScript for the Common Opener.</returns>
    public static HTMLScriptCollection GetOpenerJavascript(AActivitySpace _asOwner)
    {
      HTMLScriptCollection myScriptCollection = new HTMLScriptCollection();
      HTMLScript myScript;
      // add our dynamic script.
      myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      XPStringBuilder sb = new XPStringBuilder(200);
      sb.Append("\n");
      sb.Append("function ").Append(IPTOpenerConstants.JS_METHOD_GETAPPLICATIONBASEURL).Append("()\n");
      sb.Append("{\n");
      sb.Append("\treturn \"" + ConfigService.getInstance().GetApplicationBaseURL(_asOwner) + "\";\n");
      sb.Append("}\n");
      sb.Append("function ").Append(IPTOpenerConstants.JS_METHOD_GETCURRENTUSERID).Append("()\n");
      sb.Append("{\n");
      sb.Append("\treturn \"" + ((IPTSession) _asOwner.GetUserSession()).GetSessionInfo().GetCurrentUserID() + "\";\n");
      sb.Append("}\n");
      sb.Append("function ").Append(IPTOpenerConstants.JS_METHOD_PARENTSPACE_NAME).Append("()\n");
      sb.Append("{\n");
      if (null != _asOwner.GetParentSpaceName())
      {
        sb.Append("\treturn \"" + _asOwner.GetParentSpaceName() + "\";\n");
        } else
      {
        sb.Append("\treturn \"\";\n");
      }
      sb.Append("}\n");
      sb.Append("function ").Append(IPTOpenerConstants.JS_METHOD_PARENTSPACE_ID).Append("()\n");
      sb.Append("{\n");
      if (null != _asOwner.GetParentSpaceID())
      {
        sb.Append("\treturn \"" + _asOwner.GetParentSpaceID() + "\";\n");
        } else
      {
        sb.Append("\treturn \"\";\n");
      }
      sb.Append("}\n");
      myScript.AddInnerHTMLString(sb.ToString());
      myScriptCollection.AddInnerHTMLElement(myScript);
      // common opener needs access to PTWindowUtil calls.
      // Add the callback function for ptcommonopener.js to return the jsutil version number prefix
      String strJSUTILPrefixString = ConfigHelper.GetJSIncluderVersionPrefix(ConfigHelper.JSCOMPONENT_JSUTIL);
      if (XPStringUtility.EndsWith(strJSUTILPrefixString, "."))
      {
        strJSUTILPrefixString = XPStringUtility.Substring(strJSUTILPrefixString,
         0, strJSUTILPrefixString.Length - 1);
      }
      HTMLScript preScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      preScript.AddInnerHTMLString("function getJSUtilVersionObject() { return \'" + strJSUTILPrefixString + "\';}");
      myScriptCollection.AddInnerHTMLElement(preScript);
      // add our static script.
      myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      myScript.SetSrc(ConfigHelper.GetPortalImageServerJavaScriptURL(_asOwner.GetIsSecuredSpace()) + IPTOpenerConstants.FILENAME_JS_COMMONOPENER);
      myScriptCollection.AddInnerHTMLElement(myScript);
      return myScriptCollection;
    }
    
    /// <summary>
    /// Returns an URL that brings you to the location for creating an object of 
    /// the specified class type.  The object will be saved in specified folder.
    /// By default, the Application Base URL and User ID will be retrieved from 
    /// the AS owner.  If you want to use alternate values, or remove the User
    /// ID from the URL, you need to use the base GetOpenerURL method.
    /// Additionally, the ParentSpace Name and ID will also be placed onto the 
    /// URL and these too will be retrieved from the AS.  If you want to change 
    /// the location of the Parent or remove it from the URL, you can use the 
    /// overloaded method of the same name.
    /// </summary>
    /// <param name="_nClassID">The ClassID of the object to create.</param>
    /// <param name="_nFolderID">The folder to save the object in.</param>
    /// <param name="_asOwner">AS to retrieve information from.</param>
    /// <returns>URL that brings you to the location for creating an object.</returns>
    public static String GetOpenerURLCreate(int _nClassID, int _nFolderID,
     AActivitySpace _asOwner)
    {
      return GetOpenerURLCreate(_nClassID, _nFolderID, _asOwner, _asOwner.GetName(),
       _asOwner.GetSpaceID(), null);
    }
    
    /// <summary>
    /// Returns an URL that brings you to the location for creating an object of 
    /// the specified class type.  The object will be saved in specified folder.
    /// By default, the Application Base URL and User ID will be retrieved from 
    /// the AS owner.  If you want to use alternate values, or remove the User
    /// ID from the URL, you need to use the base GetOpenerURL method.
    /// </summary>
    /// <param name="_nClassID">The ClassID of the object to create.</param>
    /// <param name="_nFolderID">The folder to save the object in.</param>
    /// <param name="_asOwner">AS to retrieve information from.</param>
    /// <param name="_strOverrideParentSpaceName">Alternative ParentSpaceName to use, NULL to exclude.</param>
    /// <param name="_strOverrideParentSpaceID">Alternative ParentSpaceID to use, NULL to exclude.</param>
    /// <param name="_strOptQSArgs">Additional arguments to add to the URL, NULL to exclude.</param>
    /// <returns>URL that brings you to the location for creating an object.</returns>
    public static String GetOpenerURLCreate(int _nClassID, int _nFolderID,
     AActivitySpace _asOwner, String _strOverrideParentSpaceName, String _strOverrideParentSpaceID,
     String _strOptQSArgs)
    {
      XPStringBuilder sbQSArgs = new XPStringBuilder(256);
      HTTPHelpers.AppendQSArgument(sbQSArgs, IPTOpenerConstants.QS_OPEN, _nClassID,
       true);
      HTTPHelpers.AppendQSArgument(sbQSArgs, IPTOpenerConstants.QS_FOLDERID,
       _nFolderID);
      if (null != _strOptQSArgs)
      {
        sbQSArgs.Append(_strOptQSArgs);
      }
      if ((null != _strOverrideParentSpaceName) && (null != _strOverrideParentSpaceID))
      {
        // only add space and id if both of them are valid.
        HTTPHelpers.AppendQSArgument(sbQSArgs, ASConstants.PARENTSPACENAME,
         _strOverrideParentSpaceName);
        HTTPHelpers.AppendQSArgument(sbQSArgs, ASConstants.PARENTSPACEID,
         _strOverrideParentSpaceID);
      }
      return GetOpenerURL(sbQSArgs.ToString(), IPTOpenerConstants.OPENER_MODE_CREATE,
       _asOwner);
    }
    
    /// <summary>
    /// Returns an URL that brings you to the location for creating an object of 
    /// the specified class type.  The object will be saved in specified folder.
    /// By default, the Application Base URL and User ID will be retrieved from 
    /// the AS owner.  If you want to use alternate values, or remove the User
    /// ID from the URL, you need to use the base GetOpenerURL method.
    /// Additionally, the ParentSpace Name and ID will also be placed onto the 
    /// URL and these too will be retrieved from the AS.  If you want to change 
    /// the location of the Parent or remove it from the URL, you can use the 
    /// overloaded method of the same name.
    /// </summary>
    /// <param name="_nClassID">The ClassID of the object to create.</param>
    /// <param name="_nFolderID">The folder to save the object in.</param>
    /// <param name="_strProvEngName">English Provider Name, NULL to exclude.</param>
    /// <param name="_nDataSourceID">Datasource ID for this object, use OpenerLinks.INT_PARAM_EXCLUDEVALUE constan to exclude.</param>
    /// <param name="_nWebServiceID">WebService ID for this object, use OpenerLinks.INT_PARAM_EXCLUDEVALUE constan to exclude.</param>
    /// <param name="_asOwner">AS to retrieve information from.</param>
    /// <returns>URL that brings you to the location for creating an extensible object.</returns>
    public static String GetOpenerURLCreateExt(int _nClassID, int _nFolderID,
     String _strProvEngName, int _nDataSourceID, int _nWebServiceID, AActivitySpace _asOwner)
    {
      return GetOpenerURLCreateExt(_nClassID, _nFolderID, _strProvEngName,
       _nDataSourceID, _nWebServiceID, _asOwner, _asOwner.GetName(), _asOwner.GetSpaceID(),
       null);
    }
    
    /// <summary>
    /// Returns an URL that brings you to the location for creating an object of 
    /// the specified class type.  The object will be saved in specified folder.
    /// By default, the Application Base URL and User ID will be retrieved from 
    /// the AS owner.  If you want to use alternate values, or remove the User
    /// ID from the URL, you need to use the base GetOpenerURL method.
    /// </summary>
    /// <param name="_nClassID">The ClassID of the object to create.</param>
    /// <param name="_nFolderID">The folder to save the object in.</param>
    /// <param name="_strProvEngName">English Provider Name, NULL to exclude.</param>
    /// <param name="_nDataSourceID">Datasource ID for this object, -1 to exclude.</param>
    /// <param name="_nWebServiceID">WebService ID for this object, -1 to exclude.</param>
    /// <param name="_asOwner">AS to retrieve information from.</param>
    /// <param name="_strOverrideParentSpaceName">Alternative ParentSpaceName to use, NULL to exclude.</param>
    /// <param name="_strOverrideParentSpaceID">Alternative ParentSpaceID to use, NULL to exclude.</param>
    /// <param name="_strOptQSArgs">Additional arguments to add to the URL, NULL to exclude.</param>
    /// <returns>URL that brings you to the location for creating an extensible object.</returns>
    public static String GetOpenerURLCreateExt(int _nClassID, int _nFolderID,
     String _strProvEngName, int _nDataSourceID, int _nWebServiceID, AActivitySpace _asOwner,
     String _strOverrideParentSpaceName, String _strOverrideParentSpaceID,
     String _strOptQSArgs)
    {
      XPStringBuilder sbQSArgs = new XPStringBuilder(256);
      HTTPHelpers.AppendQSArgument(sbQSArgs, IPTOpenerConstants.QS_OPEN, _nClassID,
       true);
      HTTPHelpers.AppendQSArgument(sbQSArgs, IPTOpenerConstants.QS_FOLDERID,
       _nFolderID);
      HTTPHelpers.AppendQSArgument(sbQSArgs, IPTOpenerConstants.QS_PROV_ENG_NAME,
       _strProvEngName);
      HTTPHelpers.AppendQSArgument(sbQSArgs, XUIHelperMethods.QS_XUI_DATASOURCE_ID,
       _nDataSourceID);
      HTTPHelpers.AppendQSArgument(sbQSArgs, XUIHelperMethods.QS_XUI_WEBSERVICE_ID,
       _nWebServiceID);
      if (null != _strOptQSArgs)
      {
        sbQSArgs.Append(_strOptQSArgs);
      }
      if ((null != _strOverrideParentSpaceName) && (null != _strOverrideParentSpaceID))
      {
        // only add space and id if both of them are valid.
        HTTPHelpers.AppendQSArgument(sbQSArgs, ASConstants.PARENTSPACENAME,
         _strOverrideParentSpaceName);
        HTTPHelpers.AppendQSArgument(sbQSArgs, ASConstants.PARENTSPACEID,
         _strOverrideParentSpaceID);
      }
      return GetOpenerURL(sbQSArgs.ToString(), IPTOpenerConstants.OPENER_MODE_CREATE,
       _asOwner);
    }
    
    /// <summary>
    /// Returns an URL that opens the specified object.  If you do not specify
    /// the mode to open the object in, the default mode for that class will
    /// be used.  Note that the UUID will be broken down by the Opener based 
    /// on the UUID plugin.
    /// By default, the Application Base URL and User ID will be retrieved from 
    /// the AS owner.  If you want to use alternate values, or remove the User
    /// ID from the URL, you need to use the base GetOpenerURL method.
    /// Additionally, the ParentSpace Name and ID will also be placed onto the 
    /// URL and these too will be retrieved from the AS.  If you want to change 
    /// the location of the Parent or remove it from the URL, you can use the 
    /// overloaded method of the same name.
    /// </summary>
    /// <param name="_strUUID">UUID of the object to open.</param>
    /// <param name="_nOpenerMode">Mode to open object in, use OpenerLinks.INT_PARAM_EXCLUDEVALUE constan to exclude.</param>
    /// <param name="_asOwner">AS to retrieve information from.</param>
    /// <returns>URL that opens the specified object.</returns>
    public static String GetOpenerURLOpenUUID(String _strUUID, int _nOpenerMode,
     AActivitySpace _asOwner)
    {
      return GetOpenerURLOpenUUID(_strUUID, _nOpenerMode, _asOwner, _asOwner.GetName(),
       _asOwner.GetSpaceID(), null);
    }
    
    /// <summary>
    /// Returns an URL that opens the specified object.  If you do not specify
    /// the mode to open the object in, the default mode for that class will
    /// be used.  Note that the UUID will be broken down by the Opener based 
    /// on the UUID plugin.
    /// By default, the Application Base URL and User ID will be retrieved from 
    /// the AS owner.  If you want to use alternate values, or remove the User
    /// ID from the URL, you need to use the base GetOpenerURL method.
    /// </summary>
    /// <param name="_strUUID">UUID of the object to open.</param>
    /// <param name="_nOpenerMode">Mode to open object in, use OpenerLinks.INT_PARAM_EXCLUDEVALUE constan to exclude.</param>
    /// <param name="_asOwner">AS to retrieve information from.</param>
    /// <param name="_strOverrideParentSpaceName">Alternative ParentSpaceName to use, NULL to exclude.</param>
    /// <param name="_strOverrideParentSpaceID">Alternative ParentSpaceID to use, NULL to exclude.</param>
    /// <param name="_strOptQSArgs">Additional arguments to add to the URL, NULL to exclude.</param>
    /// <returns>URL that opens the specified object.</returns>
    public static String GetOpenerURLOpenUUID(String _strUUID, int _nOpenerMode,
     AActivitySpace _asOwner, String _strOverrideParentSpaceName, String _strOverrideParentSpaceID,
     String _strOptQSArgs)
    {
      XPStringBuilder sbQSArgs = new XPStringBuilder(256);
      HTTPHelpers.AppendQSArgument(sbQSArgs, IPTOpenerConstants.QS_UUID, _strUUID);
      if (null != _strOptQSArgs)
      {
        sbQSArgs.Append(_strOptQSArgs);
      }
      if ((null != _strOverrideParentSpaceName) && (null != _strOverrideParentSpaceID))
      {
        // only add space and id if both of them are valid.
        HTTPHelpers.AppendQSArgument(sbQSArgs, ASConstants.PARENTSPACENAME,
         _strOverrideParentSpaceName);
        HTTPHelpers.AppendQSArgument(sbQSArgs, ASConstants.PARENTSPACEID,
         _strOverrideParentSpaceID);
      }
      return GetOpenerURL(sbQSArgs.ToString(), _nOpenerMode, _asOwner);
    }
    
    /// <summary>
    /// Returns an URL that opens the specified object.  If you do not specify
    /// the mode to open the object in, the default mode for that class will
    /// be used.  Opening by ClassID/ObjectID pair is the simplest way.
    /// By default, the Application Base URL and User ID will be retrieved from 
    /// the AS owner.  If you want to use alternate values, or remove the User
    /// ID from the URL, you need to use the base GetOpenerURL method.
    /// Additionally, the ParentSpace Name and ID will also be placed onto the 
    /// URL and these too will be retrieved from the A aS.  If you want to change 
    /// the location of the Parent or remove it from the URL, you can use the 
    /// overloaded method of the same name.
    /// </summary>
    /// <param name="_nClassID">ClassID of the object to open.</param>
    /// <param name="_nObjectID">ObjectID of the object to open.</param>
    /// <param name="_nOpenerMode">Mode to open object in, use OpenerLinks.INT_PARAM_EXCLUDEVALUE constan to exclude.</param>
    /// <param name="_asOwner">AS to retrieve information from.</param>
    /// <returns>URL that opens the specified object.</returns>
    public static String GetOpenerURLOpenObjID(int _nClassID, int _nObjectID,
     int _nOpenerMode, AActivitySpace _asOwner)
    {
      return GetOpenerURLOpenObjID(_nClassID, _nObjectID, _nOpenerMode, _asOwner,
       _asOwner.GetName(), _asOwner.GetSpaceID(), null);
    }
    
    /// <summary>
    /// Returns an URL that opens the specified object.  If you do not specify
    /// the mode to open the object in, the default mode for that class will
    /// be used.  Opening by ClassID/ObjectID pair is the simplest way.
    /// By default, the Application Base URL and User ID will be retrieved from 
    /// the AS owner.  If you want to use alternate values, or remove the User
    /// ID from the URL, you need to use the base GetOpenerURL method.
    /// Additionally, the ParentSpace Name and ID will also be placed onto the 
    /// URL and these too will be retrieved from the A aS.  If you want to change 
    /// the location of the Parent or remove it from the URL, you can use the 
    /// overloaded method of the same name.
    /// </summary>
    /// <param name="_nClassID">ClassID of the object to open.</param>
    /// <param name="_nObjectID">ObjectID of the object to open.</param>
    /// <param name="_strObjectName">Name of the object to open.  Should be null if not used, but should always be passed in if available.</param>
    /// <param name="_nOpenerMode">Mode to open object in, use OpenerLinks.INT_PARAM_EXCLUDEVALUE constan to exclude.</param>
    /// <param name="_asOwner">AS to retrieve information from.</param>
    /// <returns>URL that opens the specified object.</returns>
    public static String GetOpenerURLOpenObjID(int _nClassID, int _nObjectID,
     String _strObjectName, int _nOpenerMode, AActivitySpace _asOwner)
    {
      return GetOpenerURLOpenObjID(_nClassID, _nObjectID, _strObjectName,
       _nOpenerMode, _asOwner, _asOwner.GetName(), _asOwner.GetSpaceID(),
       null);
    }
    
    /// <summary>
    /// Returns an URL that opens the specified object.  If you do not specify
    /// the mode to open the object in, the default mode for that class will
    /// be used.  Opening by ClassID/ObjectID pair is the simplest way.
    /// By default, the Application Base URL and User ID will be retrieved from 
    /// the AS owner.  If you want to use alternate values, or remove the User
    /// ID from the URL, you need to use the base GetOpenerURL method.
    /// </summary>
    /// <param name="_nClassID">ClassID of the object to open.</param>
    /// <param name="_nObjectID">ObjectID of the object to open.</param>
    /// <param name="_nOpenerMode">Mode to open object in, use OpenerLinks.INT_PARAM_EXCLUDEVALUE constan to exclude.</param>
    /// <param name="_asOwner">AS to retrieve information from.</param>
    /// <param name="_strOverrideParentSpaceName">Alternative ParentSpaceName to use, NULL to exclude.</param>
    /// <param name="_strOverrideParentSpaceID">Alternative ParentSpaceID to use, NULL to exclude.</param>
    /// <param name="_strOptQSArgs">Additional arguments to add to the URL, NULL to exclude.</param>
    /// <returns>URL that opens the specified object.</returns>
    public static String GetOpenerURLOpenObjID(int _nClassID, int _nObjectID,
     int _nOpenerMode, AActivitySpace _asOwner, String _strOverrideParentSpaceName,
     String _strOverrideParentSpaceID, String _strOptQSArgs)
    {
      return PTOpenerLinks.GetOpenerURLOpenObjID(_nClassID, _nObjectID, null,
       _nOpenerMode, _asOwner, _strOverrideParentSpaceName, _strOverrideParentSpaceID,
       _strOptQSArgs);
    }
    
    /// <summary>
    /// Returns an URL that opens the specified object.  If you do not specify
    /// the mode to open the object in, the default mode for that class will
    /// be used.  Opening by ClassID/ObjectID pair is the simplest way.
    /// By default, the Application Base URL and User ID will be retrieved from 
    /// the AS owner.  If you want to use alternate values, or remove the User
    /// ID from the URL, you need to use the base GetOpenerURL method.
    /// </summary>
    /// <param name="_nClassID">ClassID of the object to open.</param>
    /// <param name="_nObjectID">ObjectID of the object to open.</param>
    /// <param name="_strObjectName">Name of the object to open.  Should be null if not used, but should always be passed in if available.</param>
    /// <param name="_nOpenerMode">Mode to open object in, use OpenerLinks.INT_PARAM_EXCLUDEVALUE constan to exclude.</param>
    /// <param name="_asOwner">AS to retrieve information from.</param>
    /// <param name="_strOverrideParentSpaceName">Alternative ParentSpaceName to use, NULL to exclude.</param>
    /// <param name="_strOverrideParentSpaceID">Alternative ParentSpaceID to use, NULL to exclude.</param>
    /// <param name="_strOptQSArgs">Additional arguments to add to the URL, NULL to exclude.</param>
    /// <returns>URL that opens the specified object.</returns>
    public static String GetOpenerURLOpenObjID(int _nClassID, int _nObjectID,
     String _strObjectName, int _nOpenerMode, AActivitySpace _asOwner, String _strOverrideParentSpaceName,
     String _strOverrideParentSpaceID, String _strOptQSArgs)
    {
      // DCA - check for user profile community
      if (PT_CLASSIDS.PT_COMMUNITY_ID == _nClassID && PT_INTRINSICS.PT_PROFILE_COMMUNITY == _nObjectID && _nOpenerMode == IPTOpenerConstants.OPENER_MODE_VIEW)
      {
        _nObjectID = TaskAPIServerUser.GetCurrentUserID((IPTSession) _asOwner.GetUserSession());
        _strObjectName = TaskAPIServerUser.GetCurrentUserName((IPTSession) _asOwner.GetUserSession());
        _nClassID = PT_CLASSIDS.PT_USER_ID;
      }
      // community pages, need to lookup community id from community page
      String strPageName = null;
      int nPageId = 0;
      if (PT_CLASSIDS.PT_PAGE_ID == _nClassID && _nOpenerMode == IPTOpenerConstants.OPENER_MODE_VIEW)
      {
        IPTSession session = (IPTSession) _asOwner.GetUserSession();
        IPTPageInfo curPage = ((IPTPageManager) session.GetPages()).CachedOpenPageInfo(_nObjectID,
         false, false);
        int commId = curPage.GetContainerObjectID();
        // DCA - also check for user profile pages
        if (PT_INTRINSICS.PT_PROFILE_COMMUNITY == commId)
        {
          nPageId = _nObjectID;
          strPageName = curPage.GetName();
          _nObjectID = TaskAPIServerUser.GetCurrentUserID((IPTSession) _asOwner.GetUserSession());
          _strObjectName = TaskAPIServerUser.GetCurrentUserName((IPTSession) _asOwner.GetUserSession());
          _nClassID = PT_CLASSIDS.PT_USER_ID;
        }
      }
      // Check if we should create a friendly URL
      if (useFriendlyURLS(_nOpenerMode, _nClassID, _strOptQSArgs))
      {
        if (null == _strObjectName)
        {
          friendlyURLlog.Info("Object name was not passed in. Object Id: " + _nObjectID + " Class Id: " + _nClassID);
          }
        if (null == strPageName && nPageId > 0)
        {
          friendlyURLlog.Info("Object name was not passed in. Object Id: " + nPageId + " Class Id: " + PT_CLASSIDS.PT_PAGE_ID);
          }
        try
        {
          FriendlyURL friendly = new FriendlyURL(_nClassID, _strObjectName,
           _nObjectID, _strOptQSArgs);
          if (nPageId > 0)
          {
            friendly.SetChildId(nPageId);
            friendly.SetChildName(strPageName);
          }
          String strFriendlyURL = friendly.GetFullFriendlyURL(_asOwner);
          // DCA - return if we found a friendly URL, otherwise create a non-friendly one below
          if (null != strFriendlyURL)
          {
            return strFriendlyURL;
          }
        }
        catch (Exception e)
        {
          // DCA - could not create friendly URL, return non-friendly URL instead
          log.Error(e, "PTOpenerLinks:GetOpenerURLOpenObjID Could not construct friendlyURL. Displaying non-friendly mode.");
          }
      }
      XPStringBuilder sbQSArgs = new XPStringBuilder(256);
      HTTPHelpers.AppendQSArgument(sbQSArgs, IPTOpenerConstants.QS_OPEN, _nClassID,
       true);
      HTTPHelpers.AppendQSArgument(sbQSArgs, IPTOpenerConstants.QS_OBJECTID,
       _nObjectID);
      if (null != _strOptQSArgs)
      {
        sbQSArgs.Append(_strOptQSArgs);
      }
      if ((null != _strOverrideParentSpaceName) && (null != _strOverrideParentSpaceID))
      {
        // only add space and id if both of them are valid.
        HTTPHelpers.AppendQSArgument(sbQSArgs, ASConstants.PARENTSPACENAME,
         _strOverrideParentSpaceName);
        HTTPHelpers.AppendQSArgument(sbQSArgs, ASConstants.PARENTSPACEID,
         _strOverrideParentSpaceID);
      }
      return GetOpenerURL(sbQSArgs.ToString(), _nOpenerMode, _asOwner);
    }
    
    /// <summary>
    /// Returns an URL that opens the specified object.  If you do not specify
    /// the mode to open the object in, the default mode for that class will
    /// be used.  Opening by ClassID/ObjectID pair is the simplest way.
    /// This method takes in optional QS arguments.  Therefore no arguments
    /// (i.e. ParentSpaceName/ParentSpaceID), etc will be set for you.  You must
    /// specify all arguments using the QSArgs argument.
    /// </summary>
    /// <param name="_nClassID">ClassID of the object to open.</param>
    /// <param name="_nObjectID">ObjectID of the object to open.</param>
    /// <param name="_strOptQSArgs">Arguments to use for opening the object, NULL to exclude.</param>
    /// <param name="_nOpenerMode">Mode to open object in, use OpenerLinks.INT_PARAM_EXCLUDEVALUE constan to exclude.</param>
    /// <param name="_strApplicationBaseURL">The Application Base URL.</param>
    /// <param name="_nUserID">User ID to append to the URL, use OpenerLinks.INT_PARAM_EXCLUDEVALUE constan to exclude.</param>
    /// <returns>URL that opens the specified object, generically.</returns>
    public static String GetOpenerURLOpenObjID(int _nClassID, int _nObjectID,
     String _strOptQSArgs, int _nOpenerMode, String _strApplicationBaseURL,
     int _nUserID)
    {
      XPStringBuilder sbQSArgs = new XPStringBuilder(256);
      HTTPHelpers.AppendQSArgument(sbQSArgs, IPTOpenerConstants.QS_OPEN, _nClassID,
       true);
      HTTPHelpers.AppendQSArgument(sbQSArgs, IPTOpenerConstants.QS_OBJECTID,
       _nObjectID);
      if (null != _strOptQSArgs)
      {
        sbQSArgs.Append("&" + _strOptQSArgs);
      }
      return GetOpenerURL(sbQSArgs.ToString(), _nOpenerMode, _strApplicationBaseURL,
       XPConvert.ToString(_nUserID));
    }
    
    /// <summary>
    /// Returns the JS that will create an URL that opens the specified activity
    /// space.  If you choose to exclude the opener mode (using NULL), the 
    /// default mode for that class will be used.  Note that you need to provide 
    /// both the QS name and value pairs in the OtpQSArgs argument (the extended 
    /// GetOpenerJSXXX methods do this for you automatically).
    /// Note that this JS just generates the URL.  You will need to wrap the 
    /// URL with more JS (such as the GetJSOpenInXXX methods) if you want to do 
    /// something useful with the URL.  Note that you can generate URLS using 
    /// the GetOpenerURL server methods, and then wrapping those URLS - these
    /// GetOpenerJSURL methods are useful primarly for dynamic JS work, or if 
    /// you don't want to repeat values on the page.
    /// Note that the ApplicationBaseURL and CurrentUserID has already been 
    /// placed on the page and thus do not need to be passed in.
    /// </summary>
    /// <param name="_strASName">Name of AS to open.</param>
    /// <param name="_strOptQSArgs">Additional arguments to add to the URL, NULL to exclude.</param>
    /// <param name="_strOpenerMode">Mode to open object in, NULL to exclude.</param>
    /// <returns>The JS that will generate an URL to open the specified object.</returns>
    public static String GetOpenerJSURLOpenAS(String _strASName, String _strOptQSArgs,
     String _strOpenerMode)
    {
      XPStringBuilder sbOpenerJS = new XPStringBuilder(300);
      sbOpenerJS.Append(IPTOpenerConstants.JS_METHOD_GETOPENERURL_OPENAS);
      sbOpenerJS.Append("(\'");
      sbOpenerJS.Append(_strASName).Append("\', \'");
      sbOpenerJS.Append(_strOptQSArgs).Append("\', ");
      sbOpenerJS.Append(_strOpenerMode);
      sbOpenerJS.Append(")");
      return sbOpenerJS.ToString();
    }
    
    /// <summary>
    /// Overloaded method for easy access.  Refer to base class for JavaDoc.
    /// </summary>
    /// <seealso cref="PTOpenerLinks#GetOpenerJSURLOpenAS(String,String,String,String)"></seealso>
    public static String GetOpenerJSURLOpenAS(String _strASName, String _strOptQSArgs,
     int _nOpenerMode)
    {
      return GetOpenerJSURLOpenAS(_strASName, _strOptQSArgs, XPConvert.ToString(_nOpenerMode));
      }
    
    /// <summary>
    /// Returns the JS that will create an URL that opens the specified object.
    /// If you choose to exclude the opener mode (using NULL), the default mode
    /// for that class will be used.  Note that you need to provide both the QS
    /// name and value pairs in the OtpQSArgs argument (the extended 
    /// GetOpenerJSXXX methods do this for you automatically).
    /// Note that this JS just generates the URL.  You will need to wrap the 
    /// URL with more JS (such as the GetJSOpenInXXX methods) if you want to do 
    /// something useful with the URL.  Note that you can generate URLS using 
    /// the GetOpenerURL server methods, and then wrapping those URLS - these
    /// GetOpenerJSURL methods are useful primarly for dynamic JS work, or if 
    /// you don't want to repeat values on the page.
    /// Note that the ApplicationBaseURL and CurrentUserID has already been 
    /// placed on the page and thus do not need to be passed in.
    /// </summary>
    /// <param name="_strUUID">UUID of the object to open.</param>
    /// <param name="_strOptQSArgs">Additional arguments to add to the URL, NULL to exclude.</param>
    /// <param name="_strOpenerMode">Mode to open object in, NULL to exclude.</param>
    /// <returns>The JS that will generate an URL to open the specified object.</returns>
    public static String GetOpenerJSURLOpenUUID(String _strUUID, String _strOptQSArgs,
     String _strOpenerMode)
    {
      XPStringBuilder sbOpenerJS = new XPStringBuilder(300);
      sbOpenerJS.Append(IPTOpenerConstants.JS_METHOD_GETOPENERURL_OPENUUID);
      sbOpenerJS.Append("(");
      sbOpenerJS.Append(_strUUID).Append(", \'");
      sbOpenerJS.Append(_strOptQSArgs).Append("\', ");
      sbOpenerJS.Append(_strOpenerMode);
      sbOpenerJS.Append(")");
      return sbOpenerJS.ToString();
    }
    
    /// <summary>
    /// Overloaded method for easy access.  Refer to base class for JavaDoc.
    /// </summary>
    /// <seealso cref="PTOpenerLinks#GetOpenerJSURLOpenUUID(String,String,String)"></seealso>
    public static String GetOpenerJSURLOpenUUID(String _strUUID, String _strOptQSArgs,
     int _nOpenerMode)
    {
      return GetOpenerJSURLOpenUUID(_strUUID, _strOptQSArgs, XPConvert.ToString(_nOpenerMode));
      }
    
    /// <summary>
    /// Returns the JS that will create an URL that opens the specified object.
    /// If you choose to exclude the opener mode (using NULL), the default mode
    /// for that class will be used.  Note that you need to provide both the QS
    /// name and value pairs in the OtpQSArgs argument (the extended 
    /// GetOpenerJSXXX methods do this for you automatically).
    /// Note that this JS just generates the URL.  You will need to wrap the 
    /// URL with more JS (such as the GetJSOpenInXXX methods) if you want to do 
    /// something useful with the URL.  Note that you can generate URLS using 
    /// the GetOpenerURL server methods, and then wrapping those URLS - these
    /// GetOpenerJSURL methods are useful primarly for dynamic JS work, or if 
    /// you don't want to repeat values on the page.
    /// Note that the ApplicationBaseURL and CurrentUserID has already been 
    /// placed on the page and thus do not need to be passed in.
    /// </summary>
    /// <param name="_strClassID">ClassID of the object to open.</param>
    /// <param name="_strObjectID">ObjectID of the object to open.</param>
    /// <param name="_strOptQSArgs">Additional arguments to add to the URL, NULL to exclude.</param>
    /// <param name="_strOpenerMode">Mode to open object in, NULL to exclude.</param>
    /// <returns>The JS that will generate an URL to open the specified object.</returns>
    public static String GetOpenerJSURLOpenObjID(String _strClassID, String _strObjectID,
     String _strOptQSArgs, String _strOpenerMode)
    {
      XPStringBuilder sbOpenerJS = new XPStringBuilder(300);
      sbOpenerJS.Append(IPTOpenerConstants.JS_METHOD_GETOPENERURL_OPENOBJID);
      sbOpenerJS.Append("(");
      sbOpenerJS.Append(_strClassID).Append(", ");
      sbOpenerJS.Append(_strObjectID).Append(", \'");
      sbOpenerJS.Append(_strOptQSArgs).Append("\', ");
      sbOpenerJS.Append(_strOpenerMode);
      sbOpenerJS.Append(")");
      return sbOpenerJS.ToString();
    }
    
    /// <summary>
    /// Overloaded method for easy access.  Refer to base class for JavaDoc.
    /// </summary>
    /// <seealso cref="PTOpenerLinks#GetOpenerJSURLOpenObjID(String,String,String,String)"></seealso>
    public static String GetOpenerJSURLOpenObjID(int _nClassID, int _nObjectID,
     String _strOptQSArgs, int _nOpenerMode)
    {
      return GetOpenerJSURLOpenObjID(XPConvert.ToString(_nClassID), XPConvert.ToString(_nObjectID),
       _strOptQSArgs, XPConvert.ToString(_nOpenerMode));
    }
    
    /// <summary>
    /// Returns the JS that will create an URL that opens the specified object.
    /// If you choose to exclude the opener mode (using NULL), the default mode
    /// for that class will be used.  Note that you need to provide both the QS
    /// name and value pairs in the OtpQSArgs argument (the extended 
    /// GetOpenerJSXXX methods do this for you automatically).
    /// This is the base GetOpenerJSURL method.  Besides the ApplicationBaseURL,
    /// the CurrentUserID and the OpenerMode, you will need to provide all of 
    /// the other information you need to open your object.  You must specify
    /// all of the other arguments using the QSArgs argument.
    /// Note that this JS just generates the URL.  You will need to wrap the 
    /// URL with more JS (such as the GetJSOpenInXXX methods) if you want to do 
    /// something useful with the URL.  Note that you can generate URLS using 
    /// the GetOpenerURL server methods, and then wrapping those URLS - these
    /// GetOpenerJSURL methods are useful primarly for dynamic JS work, or if 
    /// you don't want to repeat values on the page.
    /// </summary>
    /// <param name="_strQSArgs">Arguments to use for opening the object.</param>
    /// <param name="_strOpenerMode">Mode to open object in, NULL to exclude.</param>
    /// <returns>The JS that will generate an URL to open the specified object.</returns>
    public static String GetOpenerJSURL(String _strQSArgs, String _strOpenerMode)
    {
      XPStringBuilder sbOpenerJS = new XPStringBuilder(300);
      sbOpenerJS.Append(IPTOpenerConstants.JS_METHOD_GETOPENERURL);
      sbOpenerJS.Append("(\'");
      sbOpenerJS.Append(_strQSArgs).Append("\', ");
      sbOpenerJS.Append(_strOpenerMode);
      sbOpenerJS.Append(")");
      return sbOpenerJS.ToString();
    }
    
    /// <summary>
    /// Overloaded method for easy access.  Refer to base class for JavaDoc.
    /// </summary>
    /// <seealso cref="PTOpenerLinks#GetOpenerJSURL(String,String)"></seealso>
    public static String GetOpenerJSURL(String _strOptQSArgs, int _nOpenerMode)
    {
      return GetOpenerJSURL(_strOptQSArgs, XPConvert.ToString(_nOpenerMode));
      }
    
    /// <summary>
    /// Returns the JS method to open the specified URL in a new window.
    /// The common use case is to use this in the onclick attribute of an
    /// HTML element.
    /// </summary>
    /// <param name="_strURL">The URL to navigate to.</param>
    /// <param name="_strWindowName">Name of the new window.</param>
    /// <param name="_strWindowWidth">Width of the new window.</param>
    /// <param name="_strWindowHeight">Height of the new window.</param>
    /// <param name="_bIsWindowFancy">Windows can be fancy or not.</param>
    /// <returns>The JS method for opening the URL in a new window.</returns>
    public static String GetJSOpenInNewWindow(String _strURL, String _strWindowName,
     String _strWindowWidth, String _strWindowHeight, bool _bIsWindowFancy)
    {
      XPStringBuilder sbOpenerJS = new XPStringBuilder(300);
      sbOpenerJS.Append("var currentWin = ");
      sbOpenerJS.Append(IPTOpenerConstants.JS_METHOD_OPEN_NEW_WINDOW);
      sbOpenerJS.Append("(\'\', \'");
      sbOpenerJS.Append(_strWindowName).Append("\', ");
      sbOpenerJS.Append(_strWindowWidth).Append(", ");
      sbOpenerJS.Append(_strWindowHeight).Append(", ");
      sbOpenerJS.Append(_bIsWindowFancy);
      sbOpenerJS.Append(");");
      sbOpenerJS.Append("currentWin.location=").Append(_strURL);
      return sbOpenerJS.ToString();
    }
    
    /// <summary>
    /// Returns the JS method to open the specified URL in the same window.
    /// The common use case is to use this in the onclick attribute of an
    /// HTML element.
    /// </summary>
    /// <param name="_strURL">The URL to navigate to.</param>
    /// <returns>The JS method for opening the URL in the same window.</returns>
    public static String GetJSOpenInSameWindow(String _strURL)
    {
      XPStringBuilder sbOpenerJS = new XPStringBuilder(300);
      sbOpenerJS.Append(IPTOpenerConstants.JS_METHOD_OPEN_SAME_WINDOW);
      sbOpenerJS.Append("(");
      sbOpenerJS.Append(_strURL);
      sbOpenerJS.Append(")");
      return sbOpenerJS.ToString();
    }
    
    /// <summary>
    /// Checks whether to use friendly URLs or not from the provided arguments. 
    /// </summary>
    /// <param name="openerMode">Opener Mode of the URL to check for</param>
    /// <param name="classId">Class id to the object type</param>
    /// <param name="queryStringArgs">Any additional query string arguments to add</param>
    /// <returns></returns>
    protected static bool useFriendlyURLS(int openerMode, int classId, String queryStringArgs)
    {
      bool useFriendly = true;
      if (openerMode != IPTOpenerConstants.OPENER_MODE_VIEW)
      {
        useFriendly = false;
      }
      if (useFriendly)
      {
        useFriendly = PTConfigHelper.UseFriendlyURLS(classId, queryStringArgs);
        }
      return useFriendly;
    }
  }
}