using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.editor;
using com.plumtree.portaluiinfrastructure.tree;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.expandablelist;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.tree;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.portaluiinfrastructure.ptmulticolumnlist
{
  /// <summary>
  /// This is the implementation of the Expandable List
  /// framework for the the Select PTObject list.
  /// The Select PTObject is a list of objects.
  /// New objects can be added to the list by selecting
  /// them from the associated Object tree.
  /// The list is by default sorted by name ascending.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class PTMultiColumnListModel : ExpListModel,IPTMultiColumnListModelRO
  {
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "PTMultiColumnListModel";
    
    /// <summary>
    /// Key name used by the Activity Space to store the tree. 
    /// </summary>
    public const String SUBSPACE_KEY_SELECTTREE = "PTSelectTree";
    
    /// <summary>
    /// This column contains the Object Class IDs.
    /// </summary>
    public const int EXPLIST_COL_CLASSID = 1;
    
    /// <summary>
    /// This column contains the Object IDs.
    /// </summary>
    public const int EXPLIST_COL_OBJECTID = 2;
    
    /// <summary>
    /// This is the String that we doisplay in the list.
    /// E.g.: Object name or folder path.
    /// </summary>
    public const int EXPLIST_COL_STRING = 3;
    
    /// <summary>
    /// Default initial size for the sorted array.
    /// </summary>
    public const int EXPLIST_DEFAULT_INITIAL_SIZE = 50;
    
    /// <summary>
    /// Number total of columns for hide objects array.
    /// </summary>
    public const int HIDE_COL_MAX = 2;
    
    /// <summary>
    /// This column of the passed Objects to Hide contains the class IDs.
    /// </summary>
    public const int HIDE_COL_CLASSID = 0;
    
    /// <summary>
    /// This column of the passed Objects to Hide contains the object IDs.
    /// </summary>
    public const int HIDE_COL_OBJECTID = 1;
    
    /// <summary>
    /// Create new array column: class ID
    /// </summary>
    public const int CREATE_NEW_COL_CLASSID = 0;
    
    /// <summary>
    /// Create new array column: AS Name
    /// </summary>
    public const int CREATE_NEW_COL_ASNAME = 1;
    
    /// <summary>
    /// Create new array column: text to display with the create new link
    /// </summary>
    public const int CREATE_NEW_COL_LABEL = 2;
    
    /// <summary>
    /// Create new array column: icon to display for the create new link
    /// </summary>
    public const int CREATE_NEW_COL_ICON = 3;
    
    /// <summary>
    /// Create new array column: extra control arguments for the create action.
    /// Use CREATE_NEW_COL_MAXWITHPARAMS to set the size of the array when using
    /// this element.
    /// </summary>
    public const int CREATE_NEW_COL_PARAMS = 4;
    
    /// <summary>
    /// Size of each m_arrCreateNewInfo row
    /// </summary>
    public const int CREATE_NEW_COL_MAX = 4;
    
    /// <summary>
    /// Size of each m_arrCreateNewInfo row with optional control parameters.
    /// </summary>
    public const int CREATE_NEW_COL_MAXWITHPARAMS = 5;
    
    /// <summary>
    /// Optional control parameter index for the parameter name. Used in the
    /// create new array column: icon to display for the create new link
    /// </summary>
    public const int PARAM_COL_PARAMNAME = 0;
    
    /// <summary>
    /// Optional control parameter index for the parameter value. Used in the
    /// create new array column: icon to display for the create new link
    /// </summary>
    public const int PARAM_COL_PARAMVALUE = 1;
    
    /// <summary>
    /// Icon to display on each row, just before the input text. 
    /// </summary>
    protected PTImageType m_imgRowIcon;
    
    /// <summary>
    /// Tree Space ID. The first time the ID is empty and a new AS is opened,
    /// but after that, we save the ID and the same tree AS is reopened each time. 
    /// </summary>
    protected String m_sTreeSpaceID = "";
    
    /// <summary>
    /// Tree sub title
    /// </summary>
    protected String m_sTreeSubTitle;
    
    /// <summary>
    /// Tree title
    /// </summary>
    protected String m_sTreeTitle;
    
    /// <summary>
    /// Objects to hide for the object tree.
    /// </summary>
    protected String m_strObjectsToHide = "";
    
    /// <summary>
    /// Query Filter for the object tree.
    /// </summary>
    protected String m_strTreeQueryFilter = "";
    
    /// <summary>
    /// Information about the create new links to display
    /// </summary>
    protected Object[][] m_arrCreateNewInfo;
    
    /// <summary>
    /// This is used on add to populate the non plumtree object columns with
    /// default values in the newly added row. Note, this array MUST be
    /// initialized explicitly by the client - it can't be null if it is used.
    /// </summary>
    protected String[] m_arrDefaultValue;
    
    /// <summary>
    /// Class ID of the Objects displayed on the list.
    /// </summary>
    protected int[] m_arrObjectClassIDs;
    
    /// <summary>
    /// Indicates whether to include a Plumtree object picker in this list
    /// control.  If true, 2 hidden columns and a visible name column will be
    /// added to the list, along with the complete facility for picking Plumtree
    /// objects.  If false, a simple text-entry multicolumn list will be
    /// produced.
    /// </summary>
    protected bool m_bIncludePTSelect = true;
    
    /// <summary>
    /// Selection type for the object tree.  True to enable multiselect, false to
    /// disable multiselect.
    /// </summary>
    protected bool m_bTreeMultiSelect = true;
    
    /// <summary>
    /// The index of the HTML row to display in edit mode.  This is set when the
    /// user clicks on the Edit icon for a given row.
    /// </summary>
    protected int m_intEditRow = -1;
    
    /// <summary>
    /// Total number of columns in addition to the plumtree select columns.  For
    /// each one of these columns, a text box will be displayed in EDIT mode,
    /// and a string in DISPLAY mode.  
    /// </summary>
    protected int m_intNumColumns = 2;
    
    /// <summary>
    /// The zero-based position of the Plumtree object name column.  Ignored if
    /// b_bIncludePTSelect is false, and automatically set to zero if
    /// it is set to a number greater than intNumColumns.
    /// </summary>
    protected int m_intPTObjectPosition = 0;
    
    /// <summary>
    /// This value determines the security access requirements for the objects to
    /// be added to the list... (a filter for the tree control).
    /// </summary>
    protected int m_nAccessLevel = 0;
    
    /// <summary>
    /// Class IDs and Object IDs of objects to display in the tree.
    /// </summary>
    protected String m_sDisplayObjects;
    
    /// <summary>
    /// Leaf type for the object tree.
    /// </summary>
    protected int m_treeLeafType = 0;
    public PTMultiColumnListModel() : base()
    {
      m_treeLeafType = TreeConstants.N_LEAFSELECT;
      m_nAccessLevel = PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT;
    }
    
    /// <summary>
    /// Parses the string of mapped fields and returns a two dimensional array
    /// in the format expected by the Plumtree server
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskOnStartEditor(XPHashtable)"></seealso>
    public static Object[][] ParseFields(String strMappedFields)
    {
      Object[][] myArray = null;
      int intTokenIndex = 0;
      XPStringTokenizer myTokenizer = new XPStringTokenizer(strMappedFields,
       ",");
      myArray = new Object[1][];
      for (int j2c_index = 0;j2c_index < 1;j2c_index++)
      {
        myArray[j2c_index] = new Object[myTokenizer.GetTokenCount()];
      }
      for (XPStringTokenizer i = myTokenizer;i.HasMoreTokens();)
      {
        String strToken = i.GetNextToken();
        myArray[0][intTokenIndex] = XPStringUtility.Trim(strToken);
        intTokenIndex++;
      }
      return myArray;
    }
    
    /// <summary>
    /// Helper function for extracting and adding extra control parameters from
    /// the create object string.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="myURL"></param>
    public virtual void AddControlParamsToURL(int _nClassID, ASURL myURL)
    {
      String[][] arParams = null;
      for (int i = 0;i < m_arrCreateNewInfo.Length;i++)
      {
        int nClassIDFromArray = XPConvert.ToInteger(m_arrCreateNewInfo[i][CREATE_NEW_COL_CLASSID]);
        if (nClassIDFromArray == _nClassID)
        {
          if (m_arrCreateNewInfo[i].Length >= CREATE_NEW_COL_MAXWITHPARAMS)
          {
            arParams = (String[][]) m_arrCreateNewInfo[i][CREATE_NEW_COL_PARAMS];
            break;
          }
        }
      }
      if (null != arParams)
      {
        for (int j = 0;j < arParams.Length;j++)
        {
          if ((null != arParams[j]) && (2 == arParams[j].Length))
          {
            String strName = arParams[j][PARAM_COL_PARAMNAME];
            String strValue = arParams[j][PARAM_COL_PARAMVALUE];
            // Add the parameter to the url.
            myURL.AddControlArgument(strName, strValue);
          }
        }
      } else
      {
      }
    }
    // Warning
    
    /// <summary>
    /// Clear the list and add all the values selected from the tree, as well as
    /// values or defaults for the other columns in the control.
    /// </summary>
    /// <param name="_htFormData">Form data</param>
    public override void AddRowsOnRepost(XPHashtable _htFormData)
    {
      if (m_bIncludePTSelect)
      {
        //Validate that the proposed position of the selected Plumtree
        //object is not greater than the number of other columns
        if (m_intPTObjectPosition > m_intNumColumns)
        {
          m_intPTObjectPosition = 0;
        }
        SaveTreeID(_htFormData);
        TreeAS asTree = (TreeAS) m_asOwner.GetSubSpace(GetUniqueString(SUBSPACE_KEY_SELECTTREE));
        if (null != asTree)
        {
          ITreeModelRO myTree = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
          XPArrayList mySelected = myTree.GetSelectedItems();
          Object[] oNewRow;
          int nClassID;
          //First, save the data so we can re-populate the non-tree items
          XPHashtable[] arrHashtables = new XPHashtable[m_intNumColumns + 3];
          for (int i = 0;i < (m_intNumColumns + 3);i++)
          {
            arrHashtables[i] = new XPHashtable();
          }
          for (int i = 0;i < m_saData.GetCount();i++)
          {
            //skip the index column
            for (int j = 1;j < (m_intNumColumns + 4);j++)
            {
              arrHashtables[j - 1].PutElement(m_saData.GetItem(i, EXPLIST_COL_STRING + m_intPTObjectPosition),
               m_saData.GetItem(i, j));
            }
          }
          // JF- We clear the array before we start adding the selected items from the tree.
          base.GetItems().ClearArray();
          for (int i = 0;i < mySelected.GetSize();i++)
          {
            oNewRow = new Object[m_intNumColumns + 4];
            oNewRow[EXPLIST_SORTEDARRAY_PROPID_INDEX] = m_nNextIndex;
            m_nNextIndex++;
            nClassID = ((TreeLeafNode) mySelected.GetElement(i)).m_nClassID;
            oNewRow[EXPLIST_COL_CLASSID + m_intPTObjectPosition] = nClassID;
            oNewRow[EXPLIST_COL_OBJECTID + m_intPTObjectPosition] = ((TreeLeafNode) mySelected.GetElement(i)).m_nObjectID;
            oNewRow[EXPLIST_COL_STRING + m_intPTObjectPosition] = ((TreeLeafNode) mySelected.GetElement(i)).GetName();
            //Add the other empty columns
            String strValue;
            int intColIndex;
            for (int j = 0;j < m_intNumColumns;j++)
            {
              if (j < m_intPTObjectPosition)
              {
                intColIndex = j + 1;
              } else
              {
                intColIndex = j + 4;
              }
              //TODO - get the right hashtable - only j when m_intPTObjectPosition is 1
              strValue = (String) arrHashtables[intColIndex - 1].GetElement(oNewRow[EXPLIST_COL_STRING + m_intPTObjectPosition]);
              if ((null == strValue) || strValue.Equals(""))
              {
                strValue = m_arrDefaultValue[j];
              }
              oNewRow[intColIndex] = strValue;
            }
            base.GetItems().AddRow(oNewRow);
          }
        }
      } else
      {
        String[][] arrDefault = new String[m_intNumColumns][];
        for (int j2c_index = 0;j2c_index < m_intNumColumns;j2c_index++)
        {
          arrDefault[j2c_index] = new String[1];
        }
        for (int i = 0;i < m_intNumColumns;i++)
        {
          arrDefault[i][0] = m_arrDefaultValue[i];
        }
        AddRowsToList(arrDefault);
      }
    }
    
    /// <summary>
    /// Adds an array of new rows to the list.  This is only called when
    /// m_bIncludePTSelect is FALSE.
    /// </summary>
    /// <param name="_aData"></param>
    public virtual void AddRowsToList(Object[][] _aData)
    {
      Object[] oRow;
      for (int i = 0;i < _aData[0].Length;i++)
      {
        oRow = new Object[m_intNumColumns + 1];
        oRow[EXPLIST_SORTEDARRAY_PROPID_INDEX] = m_nNextIndex;
        m_nNextIndex++;
        for (int j = 0;j < m_intNumColumns;j++)
        {
          oRow[j + 1] = _aData[j][i];
        }
        base.GetItems().AddRow(oRow);
      }
    }
    
    /// <summary>
    /// Tells whether the Object identified by the class ID
    /// and the object ID in argument is in the list or no.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_nObjectID"></param>
    /// <returns>boolean</returns>
    public virtual bool ContainsObject(int _nClassID, int _nObjectID)
    {
      for (int i = 0;i < m_saData.GetCount();i++)
      {
        if ((_nClassID == XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_CLASSID + m_intPTObjectPosition))) && (_nObjectID == XPConvert.ToInteger(m_saData.GetItem(i,
         EXPLIST_COL_OBJECTID + m_intPTObjectPosition))))
        {
          return true;
        }
      }
      return false;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new PTMultiColumnListModel();
    }
    
    /// <summary>
    /// This method will be called when you come back to the main Editor from a sub Editor.
    /// </summary>
    public virtual void DoTaskOnComeBackFromSubEditor()
    {
      String sEditorASName;
      EditorAS myEditorAS;
      ObjEditorModel myObjEditorModel;
      Object[] oNewRow;
      int nClassID;
      // JF- Check if they are any newly created objects
      for (int i = 0;i < m_arrCreateNewInfo.Length;i++)
      {
        sEditorASName = (String) m_arrCreateNewInfo[i][CREATE_NEW_COL_ASNAME];
        if (m_asOwner.GetSubSpaceExists(sEditorASName))
        {
          myEditorAS = (EditorAS) m_asOwner.GetSubSpace(sEditorASName);
          myObjEditorModel = (ObjEditorModel) myEditorAS.GetModel(myEditorAS.m_strModelName);
          oNewRow = new Object[m_intNumColumns];
          oNewRow[EXPLIST_SORTEDARRAY_PROPID_INDEX] = m_nNextIndex;
          m_nNextIndex++;
          nClassID = myObjEditorModel.GetClassID();
          oNewRow[EXPLIST_COL_CLASSID + m_intPTObjectPosition] = nClassID;
          oNewRow[EXPLIST_COL_OBJECTID + m_intPTObjectPosition] = myObjEditorModel.GetObjectID();
          if (PT_CLASSIDS.PT_CATALOGFOLDER_ID == nClassID)
          {
            oNewRow[EXPLIST_COL_STRING + m_intPTObjectPosition] = ((IPTFolder) myObjEditorModel.GetObject()).GetPath();
            } else
          {
            oNewRow[EXPLIST_COL_STRING + m_intPTObjectPosition] = myObjEditorModel.GetObjectName();
            }
          base.GetItems().AddRow(oNewRow);
          m_asOwner.DeleteSubSpace(sEditorASName);
        }
      }
    }
    
    /// <summary>
    /// Returns the accesslevel of the objects for the tree control to display.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetAccessLevel()
    {
      return m_nAccessLevel;
    }
    
    /// <summary>
    /// Returns the create new actions information.
    /// </summary>
    /// <returns>Object[][]</returns>
    public virtual Object[][] GetCreateNewInfo()
    {
      return m_arrCreateNewInfo;
    }
    
    /// <summary>
    /// Returns the URL to the Editor in create mode (new object).
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_sEditorASName"></param>
    /// <returns>ASURL URL to the Editor</returns>
    public virtual ASURL GetCreateNewURL(int _nClassID, String _sEditorASName)
    {
      ASURL myURL = new ASURL(m_asOwner);
      myURL.SetLinkCreateNewSpace(_sEditorASName, m_asOwner);
      myURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
      myURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE, XPConvert.ToString(EditorStartControl.EDITOR_START_FLAG_CREATE));
      myURL.AddControlArgument(EditorStartControl.QS_DELETE_ON_CANCEL, XPConvert.ToString(true));
      myURL.SetSubSpaceKey(_sEditorASName);
      myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_CLASS_ID, XPConvert.ToString(_nClassID));
      // Get the extra control arguments
      AddControlParamsToURL(_nClassID, myURL);
      return myURL;
    }
    
    /// <summary>
    /// Returns the URL to edit the object.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_nObjectID"></param>
    /// <param name="_sEditorASName"></param>
    /// <returns>ASURL URL to the Editor</returns>
    public virtual ASURL GetEditObjectURL(int _nClassID, int _nObjectID)
    {
      ASURL myURL = new ASURL(m_asOwner);
      // JF- For any Object in the list, the sub space key will be
      // this Model name with the object class id and the object id.
      String sSubEditorKey = STR_MVC_CLASS_NAME + _nClassID + _nObjectID;
      if (m_asOwner.GetSubSpaceExists(sSubEditorKey))
      {
        EditorAS mySubEditor = (EditorAS) m_asOwner.GetSubSpace(sSubEditorKey);
        myURL.SetLinkGetCachedSpace(mySubEditor);
      } else
      {
        String sEditorASName = GetEditorASName(_nClassID);
        myURL.SetLinkCreateNewSpace(sEditorASName, m_asOwner);
        myURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
        myURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE, XPConvert.ToString(EditorStartControl.EDITOR_START_FLAG_EDIT));
        myURL.SetSubSpaceKey(sSubEditorKey);
        myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_CLASS_ID, XPConvert.ToString(_nClassID));
        myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_QS_OBJECT_ID,
         XPConvert.ToString(_nObjectID));
      }
      AddControlParamsToURL(_nClassID, myURL);
      return myURL;
    }
    
    /// <summary>
    /// Returns the row currently being edited.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetEditRow()
    {
      return m_intEditRow;
    }
    
    /// <summary>
    /// Returns the comma-delimited string of classids and objectids for objects
    /// that are to be hidden by the tree.  The format of the string is as
    /// follows: ClassID1,ObjectID1, ClassID2,ObjectID2, etc..
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetHiddenObjects()
    {
      return m_strObjectsToHide;
    }
    
    /// <summary>
    /// Returns true if this list includes a Plumtree object selected from the
    /// tree object selector.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetIncludePTSelect()
    {
      return m_bIncludePTSelect;
    }
    
    /// <summary>
    /// This method should ge used by the client to get back the array of values
    /// other than the 3 columns used for the Plumtree selected object (ObjectID,
    /// ClassID, and Name) at the end of the Editor and store them. It should
    /// only be called in cases where there are columns other than the Plumtree
    /// object columns (i.e. m_intNumColumns > 0)
    /// </summary>
    /// <returns>Object[][]</returns>
    public virtual Object[][] GetListValues()
    {
      Object[][] aListValues = new Object[m_intNumColumns][];
      for (int j2c_index = 0;j2c_index < m_intNumColumns;j2c_index++)
      {
        aListValues[j2c_index] = new Object[m_saData.GetCount()];
      }
      for (int i = 0;i < m_saData.GetCount();i++)
      {
        for (int j = 0;j < m_intNumColumns;j++)
        {
          if (j < m_intPTObjectPosition)
          {
            aListValues[j][i] = (String) m_saData.GetItem(i, j + 1);
          } else
          {
            aListValues[j][i] = (String) m_saData.GetItem(i, j + 4);
          }
        }
      }
      return aListValues;
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Returns the number of columns other than the columns used by the
    /// Plumtree selected object.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetNumColumns()
    {
      return m_intNumColumns;
    }
    
    /// <summary>
    /// Returns String of Class ID of the Objects
    /// displayed in the tree.
    /// E.g.: "2,45,3"
    /// This is the format used by the tree.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetObjectClassIDs()
    {
      String sClassIDs = "";
      // JF- We loop through the array of IDs and add each ID
      // to the String.
      for (int i = 0;i < m_arrObjectClassIDs.Length;i++)
      {
        if (0 != sClassIDs.Length)
        {
          sClassIDs+=",";
        }
        sClassIDs+=(XPConvert.ToString(m_arrObjectClassIDs[i]));
      }
      return sClassIDs;
    }
    
    /// <summary>
    /// Returns the zero-based display position of the selected Plumtree object,
    /// if applicable.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetPTObjectPosition()
    {
      return m_intPTObjectPosition;
    }
    
    /// <summary>
    /// Returns the icon to display on each row before the
    /// input text. If the value is null, no icon is displayed.
    /// </summary>
    /// <returns>PTImageType</returns>
    public virtual PTImageType GetRowIcon()
    {
      return m_imgRowIcon;
    }
    
    /// <summary>
    /// Returns a String of Class IDs and Object IDs of the Objects
    /// to select in the tree.
    /// E.g.: "classID1,ObjID1,classID2,objID2"
    /// This is the format used by the tree.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetSelectedItems()
    {
      String sSelectedItems = "";
      // JF- We loop through the array of items and for each one,
      // add the class ID and the object ID to the String.
      for (int i = 0;i < m_saData.GetCount();i++)
      {
        // JS- Add the separator
        if (0 != sSelectedItems.Length)
        {
          sSelectedItems+=",";
        }
        sSelectedItems+=(m_saData.GetItem(i, EXPLIST_COL_CLASSID + m_intPTObjectPosition) + "," + m_saData.GetItem(i,
         EXPLIST_COL_OBJECTID + m_intPTObjectPosition));
      }
      return sSelectedItems;
    }
    
    /// <summary>
    /// Returns the leaf type of the tree.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetTreeLeafType()
    {
      return m_treeLeafType;
    }
    
    /// <summary>
    /// Returns the selection type of the tree.
    /// Converts the boolean value to a string so that it can be
    /// put in the query string for the tree.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetTreeMultiSelect()
    {
      return XPConvert.ToString(m_bTreeMultiSelect);
    }
    
    /// <summary>
    /// Returns the query filter for the tree.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetTreeQueryFilter()
    {
      return m_strTreeQueryFilter;
    }
    
    /// <summary>
    /// Returns tree AS ID.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetTreeSpaceID()
    {
      return m_sTreeSpaceID;
    }
    
    /// <summary>
    /// Initializes the expandable list.
    /// </summary>
    /// <param name="_sSectionTitle">- Main title of the view.</param>
    /// <param name="_sFirstColSubTitle">
    /// - The subtitle for the selected Plumtree
    /// Object, if appropriate.  If _bIncludePTSelect is FALSE, this shoudl be
    /// set to null.
    /// </param>
    /// <param name="_sSectionSubTitle">
    /// - An array of subtitles for non-plumtree object
    /// columns
    /// </param>
    /// <param name="_imgAddIcon">- Icon for the add action.</param>
    /// <param name="_sAddLabel">- Text to display next to the add icon.</param>
    /// <param name="_arrObjectClassIDs">
    /// - The Class ID of the objects displayed on the
    /// list.  This should be null if _bIncludePTSelect is FALSE.
    /// </param>
    /// <param name="_nListKey">
    /// - In case you have multiple Expandable Lists on the same
    /// DP, they should have a different key (otherwise, just set it to 1).
    /// </param>
    public virtual void InitializePTMultiColumnList(String _sSectionTitle,
     String _sFirstColSubTitle, String[] _sSectionSubTitle, PTImageType _imgAddIcon,
     String _sAddLabel, int[] _arrObjectClassIDs, int _nListKey)
    {
      int intNumColumns;
      if (m_bIncludePTSelect)
      {
        intNumColumns = m_intNumColumns + 1;
      } else
      {
        intNumColumns = m_intNumColumns;
      }
      //Set the width based on the number of columns
      int intColWidth = 100 / (intNumColumns);
      XPStringBuilder sb = new XPStringBuilder(5);
      sb.Append(intColWidth).Append("%");
      Object[][] arrExpListColInfo = new Object[intNumColumns][];
      for (int j2c_index = 0;j2c_index < intNumColumns;j2c_index++)
      {
        arrExpListColInfo[j2c_index] = new Object[5];
      }
      if (m_bIncludePTSelect)
      {
        //First initialize the name column
        arrExpListColInfo[m_intPTObjectPosition][EXPLIST_COLUMN_ID] = EXPLIST_COL_STRING + m_intPTObjectPosition;
        arrExpListColInfo[m_intPTObjectPosition][EXPLIST_COLUMN_TITLE] = _sFirstColSubTitle;
        arrExpListColInfo[m_intPTObjectPosition][EXPLIST_COLUMN_WIDTH] = sb.ToString();
        arrExpListColInfo[m_intPTObjectPosition][EXPLIST_COLUMN_COLSPAN] = "1";
        arrExpListColInfo[m_intPTObjectPosition][EXPLIST_COLUMN_ENABLESORT] = true;
        }
      //now initialize the subsequent columns
      int intColumnID;
      int intColIndex;
      for (int i = 0;i < m_intNumColumns;i++)
      {
        if (i < m_intPTObjectPosition)
        {
          intColIndex = i;
          intColumnID = i + 1;
        } else
        {
          intColIndex = i + 1;
          intColumnID = i + 4;
        }
        arrExpListColInfo[intColIndex][EXPLIST_COLUMN_ID] = intColumnID;
        arrExpListColInfo[intColIndex][EXPLIST_COLUMN_TITLE] = _sSectionSubTitle[i];
        arrExpListColInfo[intColIndex][EXPLIST_COLUMN_WIDTH] = sb.ToString();
        arrExpListColInfo[intColIndex][EXPLIST_COLUMN_COLSPAN] = "1";
        arrExpListColInfo[intColIndex][EXPLIST_COLUMN_ENABLESORT] = false;
        }
      m_arrObjectClassIDs = _arrObjectClassIDs;
      base.InitializeExpandableList(_sSectionTitle, arrExpListColInfo, _imgAddIcon,
       _sAddLabel, _nListKey);
    }
    
    /// <summary>
    /// Initalizes the list (PTGrowableSortedArrayWrapper)
    /// with a column mapping and an array of sort.
    /// The growable array will always be sorted by the 1st
    /// column ascending.
    /// </summary>
    /// <param name="_saData"></param>
    public override void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData)
    {
      // JH- Column mapping
      int[] arrColumns;
      if (m_bIncludePTSelect)
      {
        arrColumns = new int[m_intNumColumns + 4];
      } else
      {
        arrColumns = new int[m_intNumColumns + 1];
      }
      arrColumns[0] = EXPLIST_SORTEDARRAY_PROPID_INDEX;
      arrColumns[m_intPTObjectPosition + 1] = EXPLIST_COL_CLASSID + m_intPTObjectPosition;
      arrColumns[m_intPTObjectPosition + 2] = EXPLIST_COL_OBJECTID + m_intPTObjectPosition;
      arrColumns[m_intPTObjectPosition + 3] = EXPLIST_COL_STRING + m_intPTObjectPosition;
      for (int i = 0;i < m_intNumColumns;i++)
      {
        if (i < m_intPTObjectPosition)
        {
          arrColumns[i + 1] = i + 1;
        } else
        {
          arrColumns[i + 4] = i + 4;
        }
      }
      // JF- Sort array
      // The growable array will always be sorted by the String column (the one displayed).
      int[][] arrSort = new int[1][];
      for (int j2c_index = 0;j2c_index < 1;j2c_index++)
      {
        arrSort[j2c_index] = new int[2];
      }
      arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID] = EXPLIST_COL_STRING + m_intPTObjectPosition;
      arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
      // JF- We don't initalize the list itself (null below)
      // At first it's empty and we will be initalized if needed using the
      // AddRowsToList method.
      _saData.Initialize(arrColumns, null, arrSort, true, EXPLIST_DEFAULT_INITIAL_SIZE);
      }
    
    /// <summary>
    /// Returns true if it should be possible to edit Objects from this class ID.
    /// </summary>
    /// <param name="_nClassID"></param>
    public virtual bool IsEditLinkEnabled(int _nClassID)
    {
      if (null != m_arrCreateNewInfo)
      {
        int nClassIDFromArray;
        for (int i = 0;i < m_arrCreateNewInfo.Length;i++)
        {
          nClassIDFromArray = XPConvert.ToInteger(m_arrCreateNewInfo[i][CREATE_NEW_COL_CLASSID]);
          if (nClassIDFromArray == _nClassID)
          {
            return true;
          }
        }
      }
      return false;
    }
    
    /// <summary>
    /// Action for modifying a row
    /// </summary>
    /// <param name="_htFormData"></param>
    public override void ModifyRow(XPHashtable _htFormData)
    {
      String[] data = (String[]) _htFormData.GetElement(PTMultiColumnListView.HTMLHIDDEN_EDITROW);
      if (null != data)
      {
        this.SetEditRow(XPConvert.ToInteger(data[0]));
      }
    }
    
    /// <summary>
    /// Sets the accesslevel of the objects for the tree control to display.
    /// </summary>
    /// <param name="nAccessLevel">The AccessLevel to set</param>
    public virtual void SetAccessLevel(int nAccessLevel)
    {
      m_nAccessLevel = nAccessLevel;
    }
    
    /// <summary>
    /// Add the array passed in argument to the list.
    /// The array should be created using the EXPLIST_COL
    /// constants defined in this class.
    /// </summary>
    /// <param name="_arrRows">Array of rows to add.</param>
    public virtual void SetDefaultObjects(Object[][] _arrRows)
    {
      base.GetItems().AddRows(_arrRows);
    }
    
    /// <summary>
    /// Sets a new default value.
    /// By default this is an array of empty strings. This value is used on add
    /// to populate the input text of the newly added row.
    /// </summary>
    /// <param name="_sDefaultValue">New value</param>
    public virtual void SetDefaultValue(String[] _sDefaultValue)
    {
      m_arrDefaultValue = _sDefaultValue;
    }
    
    /// <summary>
    /// Sets the row currently being edited.
    /// </summary>
    /// <param name="editRow">The row to set</param>
    public virtual void SetEditRow(int editRow)
    {
      m_intEditRow = editRow;
    }
    
    /// <summary>
    /// Enables the create new links for the Class ID in argument.
    /// Doing this will also enbale the links to Edit the Objects in the list.
    /// </summary>
    /// <param name="_arrCreateNewInfo">Information for each create new link</param>
    public virtual void SetEnableCreateNew(Object[][] _arrCreateNewInfo)
    {
      if (_arrCreateNewInfo[0].Length < CREATE_NEW_COL_MAX)
      {
        throw new XPException("UserAliasModel::SetEnableCreateNew The create new information array" + " should contain " + CREATE_NEW_COL_MAX + " columns.");
        }
      m_arrCreateNewInfo = _arrCreateNewInfo;
    }
    
    /// <summary>
    /// Because this select ptobject list needs to support both move
    /// row actions enable and disable, we need to change the
    /// colspan of the header row when this happens.
    /// </summary>
    /// <param name="_bEnableMoveRowActions"></param>
    public override void SetEnableMoveRowActions(bool _bEnableMoveRowActions)
    {
      base.SetEnableMoveRowActions(_bEnableMoveRowActions);
      if (m_bEnableMoveRowActions)
      {
        // JF- If the move row actions are enabled, this will
        // create one more cell at the end of each row and
        // we need to reflect that change on the header row.
        m_arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN] = "2";
      } else
      {
        m_arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN] = "1";
      }
    }
    
    /// <summary>
    /// Populates m_strObjectsToHide, a comma-delimited list of classid/objectid
    /// pairs.  The format of the string is as follows: ClassID1,ObjectID1,
    /// ClassID2,ObjectID2, etc..
    /// </summary>
    /// <param name="_anClassIDObjectID">
    /// - a two dimensional array of classID's and
    /// ObjectID's.  Objects specified will be hidden from display in the tree's
    /// output.
    /// </param>
    public virtual void SetHiddenObjects(int[][] _anClassIDObjectID)
    {
      XPStringBuilder sb = new XPStringBuilder(6 * _anClassIDObjectID.Length);
      for (int i = 0;i < _anClassIDObjectID.Length;i++)
      {
        if (i > 0)
        {
          sb.Append(",");
        }
        sb.Append(_anClassIDObjectID[i][HIDE_COL_CLASSID]).Append(",").Append(_anClassIDObjectID[i][HIDE_COL_OBJECTID]);
        }
      m_strObjectsToHide = sb.ToString();
    }
    
    /// <summary>
    /// Sets the value for m_bIncludePTSelect, indicating whether a Plumtree
    /// object selector should be included with this list.
    /// </summary>
    /// <param name="m_bIncludePTSelect">- true or false</param>
    public virtual void SetIncludePTSelect(bool m_bIncludePTSelect)
    {
      this.m_bIncludePTSelect = m_bIncludePTSelect;
    }
    
    /// <summary>
    /// Returns the number of columns other than the columns used by the
    /// Plumtree selected object.
    /// </summary>
    /// <param name="numColumns">The numColumns to set</param>
    public virtual void SetNumColumns(int numColumns)
    {
      m_intNumColumns = numColumns;
    }
    
    /// <summary>
    /// Sets the the zero-based display position of the selected Plumtree object,
    /// if applicable.
    /// </summary>
    /// <param name="pTObjectPosition">The pTObjectPosition to set</param>
    public virtual void SetPTObjectPosition(int pTObjectPosition)
    {
      m_intPTObjectPosition = pTObjectPosition;
    }
    
    /// <summary>
    /// Sets the icon to display on each row before the
    /// input text.
    /// By default, no icon is displayed.
    /// </summary>
    /// <param name="_imgRowIcon"></param>
    public virtual void SetRowIcon(PTImageType _imgRowIcon)
    {
      m_imgRowIcon = _imgRowIcon;
    }
    
    /// <summary>
    /// Sets the leaf type for the tree.
    /// Is set to N_LEAFSELECT by default.
    /// </summary>
    /// <param name="_leafType"></param>
    public virtual void SetTreeLeafType(int _leafType)
    {
      m_treeLeafType = _leafType;
    }
    
    /// <summary>
    /// Sets the selection type for the tree.
    /// Multiselect is set to true by default.
    /// </summary>
    /// <param name="_selectType"></param>
    public virtual void SetTreeMultiSelect(bool _selectType)
    {
      m_bTreeMultiSelect = _selectType;
    }
    
    /// <summary>
    /// Sets the query filter for the tree.
    /// By default there is no query filter.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_nPropID"></param>
    /// <param name="_anValues"></param>
    public virtual void SetTreeQueryFilter(int _nClassID, int _nPropID, int[] _anValues)
    {
      XPStringBuilder sb = new XPStringBuilder(20);
      sb.Append(_nClassID).Append(",").Append(_nPropID);
      for (int i = 0;i < _anValues.Length;i++)
      {
        sb.Append(",").Append(_anValues[i]);
      }
      m_strTreeQueryFilter = sb.ToString();
    }
    
    /// <summary>
    /// Sets the Tree sub title.
    /// </summary>
    /// <param name="_sTreeSubTitle"></param>
    public virtual void SetTreeSubTitle(String _sTreeSubTitle)
    {
      m_sTreeSubTitle = _sTreeSubTitle;
    }
    
    /// <summary>
    /// Sets the Tree title.
    /// </summary>
    /// <param name="_sTreeTitle"></param>
    public virtual void SetTreeTitle(String _sTreeTitle)
    {
      m_sTreeTitle = _sTreeTitle;
    }
    
    /// <summary>
    /// Saves the data for columns other than the plumtree object select column
    /// on the row _nRowIndex.
    /// </summary>
    /// <param name="_saData">List to update the data on row _nRowIndex</param>
    /// <param name="_nRowIndex">Index of the row to save.</param>
    /// <param name="_htFormData">Data posted by the form.</param>
    /// <returns>int Status for server side validation.</returns>
    protected override int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex,
     XPHashtable _htFormData)
    {
      XPStringBuilder sb = new XPStringBuilder(100);
      sb.Append(PTMultiColumnListView.HTMLINPUT_ITEM);
      int intIndex;
      for (int i = 0;i < m_intNumColumns;i++)
      {
        if (i < m_intPTObjectPosition)
        {
          intIndex = i + 1;
        } else
        {
          intIndex = i + 4;
        }
        String sDataKey = GetUniqueString(sb.Append(intIndex).ToString()) + _nRowIndex;
        String[] aDataValue;
        // JF- For each input text posted by the form, we get the value back
        // and set the new value in the array.
        if (_htFormData.ContainsKey(sDataKey))
        {
          aDataValue = (String[]) _htFormData.GetElement(sDataKey);
          if ((null != aDataValue) && !(0 == aDataValue.Length))
          {
            _saData.SetItem(_nRowIndex, intIndex, aDataValue[0]);
          }
        }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Returns the Tree sub title.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetTreeSubTitle()
    {
      return m_sTreeSubTitle;
    }
    
    /// <summary>
    /// Returns the Tree title.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetTreeTitle()
    {
      return m_sTreeTitle;
    }
    
    /// <summary>
    /// Sets the array of objects to display in the tree. 
    /// (Only those objects will be display in the tree.)
    /// The array in argument should be in the following format:
    /// classid1,objectid1,classid2,objectid2....
    /// </summary>
    /// <param name="_sDisplayObjects"></param>
    public virtual void SetDisplayObjects(String _sDisplayObjects)
    {
      m_sDisplayObjects = _sDisplayObjects;
    }
    
    /// <summary>
    /// Returns the array of objects to display in the tree.
    /// </summary>
    /// <param name="String"></param>
    public virtual String GetDisplayObjects()
    {
      return m_sDisplayObjects;
    }
    
    /// <summary>
    /// Helper method to save the Tree ID on repost.
    /// </summary>
    /// <param name="_htFormData">Form data</param>
    protected virtual void SaveTreeID(XPHashtable _htFormData)
    {
      // JF- We save the tree ID only if we don't already have it.
      if (("").Equals(m_sTreeSpaceID) && _htFormData.ContainsKey(GetUniqueString(PTMultiColumnListView.HTMLINPUT_TREEASID)))
      {
        String[] data = (String[]) _htFormData.GetElement(GetUniqueString(PTMultiColumnListView.HTMLINPUT_TREEASID));
        if ((null != data) && !(0 == data.Length))
        {
          m_sTreeSpaceID = data[0];
        }
      }
    }
    
    /// <summary>
    /// Helper method to get the EditorAS object name from the class ID.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <returns>String</returns>
    private String GetEditorASName(int _nClassID)
    {
      String sASName = null;
      int nClassIDFromArray;
      for (int i = 0;i < m_arrCreateNewInfo.Length;i++)
      {
        nClassIDFromArray = XPConvert.ToInteger(m_arrCreateNewInfo[i][CREATE_NEW_COL_CLASSID]);
        if (nClassIDFromArray == _nClassID)
        {
          sASName = (String) m_arrCreateNewInfo[i][CREATE_NEW_COL_ASNAME];
          break;
        }
      }
      return sASName;
    }
  }
}