using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.helpers;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.statichelpers;
namespace com.plumtree.portaluiinfrastructure.restconsumerframework
{
  public class EnsembleRestAPIService
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_REST),
     typeof(EnsembleRestAPIService));
    
    /// <summary>
    /// Ensemble Constants *
    /// </summary>
    public const int ENSEMBLE_PAGELETPARAMETER_MAXLENGTH = 1024;
    
    /// <summary>
    /// Rest API URL Parameters *
    /// </summary>
    protected const String BASEURI = "api/v2/ensemble/";
    protected const String GETPAGELETS = "pagelets/";
    protected const String LIMITRESULTS = "?resultCount=";
    protected const String SEARCHNAME = "?name=";
    protected const String SEARCHID = "?id=";
    
    /// <summary>
    /// Element names in Rest API Response *
    /// </summary>
    protected const String ROOTNAME_PAGELETLIST = "ns2:PageletList";
    protected const String ROOTNAME_SINGLEPAGELET = "ns2:Pagelet";
    protected const String DATAENTRY_PAGELET = "Pagelet";
    
    /// <summary>
    /// Expected Pagelets data node names *
    /// </summary>
    public const String NAME = "name";
    public const String DESCRIPTION = "desc";
    public const String ID = "id";
    public const String LASTMODIFIED = "lastModified";
    public const String CREATED = "created";
    public const String LIBRARYNAME = "m_strLibraryName";
    public const String PARENT_RESOURCE_NAME = "parent_resource_name";
    public const String PARENT_RESOURCE_DESCRIPTION = "parent_resource_desc";
    public const String PARENT_RESOURCE_ID = "parent_resource_id";
    public const String PARENT_RESOURCE_EXTURL_LIST = "parent_resource_external_url_list";
    public const String CODESAMPLE = "m_strCodeSample";
    public const String PUBLISHDOCS = "m_bPublishDocs";
    public const String ADD_INLINEREFRESH = "m_addInlineRefreshToAllUrls";
    public const String REFRESH_INTERVAL = "m_refreshInterval";
    public const String URL_SUFFIX = "url_suffix";
    public const String PAGELET_EXTURL = "pagelet_external_url";
    public const String PARAMETERS_TRANSPORT = "parametersTransport";
    public const String ALLOW_ALL = "m_bAllowAll";
    public const String PARAMETERS = "m_Parameters";
    
    /// <summary>
    /// Pagelet Parameter node names *
    /// </summary>
    public const String PARAM_NAME = "name";
    public const String PARAM_DESCRIPTION = "desc";
    //public static final String PARAM_TYPE 			= "type";
    public const String PARAM_ISMANDATORY = "isMandatory";
    
    /// <summary>
    /// Gets the ensemble base url from the ensemble remote server object
    /// </summary>
    /// <param name="session"></param>
    /// <returns></returns>
    protected virtual String GetEnsembleBaseUrl(IPTSession session)
    {
      int ensembleServiceId = EnsembleHelperService.getInstance().GetRemoteServerID(session);
      IPTObjectManager spServers = session.GetGadgetContentServers();
      IPTGadgetContentServer server = (IPTGadgetContentServer) spServers.Open(ensembleServiceId,
       false);
      String serverUrl = server.GetURL();
      return serverUrl;
    }
    
    /// <summary>
    /// Get a list of all pagelets from Ensemble Rest API by sending a Rest Request to Ensemble service  
    /// </summary>
    /// <param name="session"></param>
    /// <returns></returns>
    public virtual EnsemblePageletRestData[] GetAllPageletsFromRest(IPTSession session)
    {
      return GetAllPageletsFromRest(-1, session);
    }
    
    /// <param name="maxResults">Max results to return in the response</param>
    /// <param name="session"></param>
    /// <returns>	null if response is not valid and no data was retrieved</returns>
    public virtual EnsemblePageletRestData[] GetAllPageletsFromRest(int maxResults,
     IPTSession session)
    {
      XPStringBuilder url = new XPStringBuilder();
      url.Append(GetEnsembleBaseUrl(session)).Append(BASEURI + GETPAGELETS);
      if (maxResults > 0)
      {
        url.Append(LIMITRESULTS).Append(maxResults);
      }
      if (log.IsInfoEnabled())
      {
        log.Info("Sending Rest Request to: " + url.ToString());
      }
      RestRequest req = RestRequest.GetInstance(url.ToString(), OpenHTTPServiceHelper.GetOpenHTTPService(),
       OpenHTTPServiceHelper.GetOpenHTTPSession());
      EnsembleAuthenticate(req, session);
      IRestResponse response;
      try
      {
        response = req.Submit();
      }
      catch (Exception e)
      {
        throw new RestConnectFailedException(e);
      }
      bool valid = VerifyEnsemblePageletListResponse(response);
      if (!valid)
      {
        throw new XPIllegalStateException("Invalid Rest API response received from Ensemble. ");
        }
      //return null;
      EnsemblePageletRestData[] pagelets = GetPageletListData(response);
      return pagelets;
    }
    
    /// <summary>
    /// Returns a single pagelet entry specified by its name
    /// ?? Pagelet with same name? Partial search on name?
    /// </summary>
    /// <param name="session"></param>
    /// <returns>m_ensembleServerValid = false;</returns>
    public virtual EnsemblePageletRestData[] GetPageletWithNameFromRest(String pageletName,
     IPTSession session)
    {
      if (null == pageletName || "".Equals(pageletName))
      {
        throw new XPIllegalArgumentException("Provided pagelet name cannot be empty");
        }
      if (log.IsInfoEnabled())
      {
        log.Info("Sending Rest Request to: " + GetEnsembleBaseUrl(session) + BASEURI + GETPAGELETS + SEARCHNAME + pageletName);
        }
      RestRequest req = RestRequest.GetInstance(GetEnsembleBaseUrl(session) + BASEURI + GETPAGELETS + SEARCHNAME + pageletName,
       OpenHTTPServiceHelper.GetOpenHTTPService(), OpenHTTPServiceHelper.GetOpenHTTPSession());
      EnsembleAuthenticate(req, session);
      IRestResponse response = req.Submit();
      EnsemblePageletRestData[] result = null;
      if (VerifyEnsemblePageletListResponse(response))
      {
        result = GetPageletListData(response);
      } else
      if (VerifyEnsemblePageletEntryResponse(response))
      {
        EnsemblePageletRestData data = GetSinglePageletData(response);
        result = new EnsemblePageletRestData[]{data};
      } else
      {
        // invalid result type
        throw new XPIllegalStateException("Retrieved invalid REST data response from Ensemble REST API!");
        }
      return result;
    }
    
    /// <param name="id"></param>
    /// <param name="session"></param>
    /// <returns></returns>
    public virtual EnsemblePageletRestData GetPageletWithIdFromRest(int id,
     IPTSession session)
    {
      if (id < 0)
      {
        throw new XPIllegalArgumentException("Provided pagelet id cannot be empty");
        }
      if (log.IsInfoEnabled())
      {
        log.Info("Sending Rest Request to: " + GetEnsembleBaseUrl(session) + BASEURI + GETPAGELETS + SEARCHID + id);
        }
      RestRequest req = RestRequest.GetInstance(GetEnsembleBaseUrl(session) + BASEURI + GETPAGELETS + SEARCHID + id,
       OpenHTTPServiceHelper.GetOpenHTTPService(), OpenHTTPServiceHelper.GetOpenHTTPSession());
      EnsembleAuthenticate(req, session);
      IRestResponse response = req.Submit();
      EnsemblePageletRestData result = null;
      if (VerifyEnsemblePageletEntryResponse(response))
      {
        result = GetSinglePageletData(response);
      } else
      {
        // invalid result type
        throw new XPIllegalStateException("Retrieved invalid REST data response from Ensemble REST API!");
        }
      return result;
    }
    
    /// <summary>
    /// NOTE: Ensemble moving to use basic auth instead of csp login tokens
    /// Set authentication 
    /// </summary>
    /// <param name="request"></param>
    /// <param name="session"></param>
    protected virtual void EnsembleAuthenticate(RestRequest request, IPTSession session)
    {
    }
    //
    //String token = session.GetToken(3);
    //request.SetCSPLoginToken(token);
    
    /// <summary>
    /// Helper method to verify Ensemble Rest API response with a list of pagelet entries 
    /// </summary>
    /// <param name="response"></param>
    /// <returns></returns>
    protected virtual bool VerifyEnsemblePageletListResponse(IRestResponse response)
    {
      if (response == null)
      {
        log.Error("No response received from Ensemble REST API call.");
        return false;
      }
      String rootname = response.GetRootElementName();
      if (!ROOTNAME_PAGELETLIST.Equals(rootname) && !ROOTNAME_SINGLEPAGELET.Equals(rootname))
      {
        log.Error("Found unexpected root element in Rest API response: " + rootname + ", expected root element with name: " + ROOTNAME_SINGLEPAGELET + " or " + ROOTNAME_PAGELETLIST);
        return false;
      }
      // Doesnt work for single pagelet response
      //		String listnodename = response.GetListName();
      //		if ( !DATAENTRY_PAGELET.equals(listnodename) ) {
      //			log.Error("Expected pagelet list element with name: "+DATAENTRY_PAGELET+", instead found element with name: "+listnodename+" in response." );
      //			return false;
      //		}
      return true;
    }
    
    /// <summary>
    /// Helper method to verify Ensemble Rest API response with a single pagelet entry
    /// Calling method should perform validation that the response contains a single pagelet data entry should  
    /// </summary>
    /// <param name="response"></param>
    /// <returns>false if response is not valid </returns>
    protected virtual bool VerifyEnsemblePageletEntryResponse(IRestResponse response)
    {
      if (response == null)
      {
        return false;
      }
      String rootname = response.GetRootElementName();
      if (!ROOTNAME_SINGLEPAGELET.Equals(rootname))
      {
        return false;
      }
      return true;
    }
    
    /// <summary>
    /// Internal helper method to get pagelet data in Ensemble Rest API response with single pagelet
    /// Calling method should perform validation that the response contains a list of pagelet data entries
    /// </summary>
    /// <param name="response"></param>
    /// <returns>null if response is not valid and no data was retrieved</returns>
    protected virtual EnsemblePageletRestData GetSinglePageletData(IRestResponse response)
    {
      if (response == null)
      {
        return null;
      }
      XPHashtable data = response.GetSingleDataItem();
      EnsemblePageletRestData pagelet = new EnsemblePageletRestData();
      if (pagelet.Init(data))
      {
        return pagelet;
      } else
      {
        return null;
      }
    }
    
    /// <summary>
    /// Internal helper method to get pagelet data in Ensemble Rest API response with a list of pagelets 
    /// </summary>
    /// <param name="response"></param>
    /// <returns>null if response is not valid and no data was retrieved</returns>
    protected virtual EnsemblePageletRestData[] GetPageletListData(IRestResponse response)
    {
      if (response == null)
      {
        return null;
      }
      String rootEleName = response.GetRootElementName();
      bool pageletList = ROOTNAME_PAGELETLIST.Equals(rootEleName);
      bool singlePagelet = ROOTNAME_SINGLEPAGELET.Equals(rootEleName);
      EnsemblePageletRestData[] pagelets = null;
      if (pageletList)
      {
        XPArrayList list = response.GetResponseList();
        pagelets = new EnsemblePageletRestData[list.GetSize()];
        for (int i = 0;i < list.GetSize();i++)
        {
          XPHashtable xmldata = (XPHashtable) list.GetElement(i);
          EnsemblePageletRestData data = new EnsemblePageletRestData();
          if (data.Init(xmldata))
          {
            pagelets[i] = data;
          }
        }
      } else
      // Single pagelet
      if (singlePagelet)
      {
        XPHashtable item = response.GetSingleDataItem();
        pagelets = new EnsemblePageletRestData[1];
        EnsemblePageletRestData data = new EnsemblePageletRestData();
        if (data.Init(item))
        {
          pagelets[0] = data;
        }
      }
      if (!singlePagelet && !pageletList)
      {
        throw new XPIllegalStateException("Unknown root element name encountered while parsing response to get all Pagelets: " + rootEleName);
        }
      return pagelets;
    }
  }
}