using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openkernel.exceptions;
using com.plumtree.openkernel.factory;
using com.plumtree.openkernel.http;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.portaluiinfrastructure.restconsumerframework
{
  /// <summary>
  /// Helper class to perform REST API Requests
  /// </summary>
  /// <author>rozhang</author>
  public class RestRequest
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_REST),
     typeof(RestRequest));
    public const int AUTHENTICATION_METHOD_NONE = 0;
    public const int AUTHENTICATION_METHOD_BASICAUTH = 1;
    public const int AUTHENTICATION_METHOD_HEADERTOKEN = 2;
    public const int AUTHENTICATION_METHOD_CSPLOGINTOKEN = 10;
    protected const String BASICAUTH_USERNAME = "basicauth_user";
    protected const String BASICAUTH_PASSWORD = "basicauth_password";
    protected const String HEADERTOKEN_NAME = "token_name";
    protected const String HEADERTOKEN_VALUE = "token_value";
    protected const String CSPLOGINTOKEN_VALUE = "csplogintoken";
    protected const String CSPLOGINTOKEN_NAME = "CSP-Session-Token";
    protected const String ACCEPT_LANGUAGE = "Accept-Language";
    
    /// <summary>
    /// HTTP Request method *
    /// </summary>
    protected String m_reqMethod = "GET";
    
    /// <summary>
    /// HTTP Request url *
    /// </summary>
    protected IOKUrl m_url = null;
    
    /// <summary>
    /// Authentication method *
    /// </summary>
    protected int m_authMethod = AUTHENTICATION_METHOD_NONE;
    
    /// <summary>
    /// Container to store authentication data *
    /// </summary>
    protected XPHashtable m_authData = new XPHashtable();
    
    /// <TODO>: Add ability to change timeout? Timeout in milli seconds *</TODO>
    protected int m_msTimeout = 30000;
    
    /// <summary>
    /// Request Post body *
    /// </summary>
    protected byte[] m_postBody = null;
    
    /// <summary>
    /// Content-Type header value *
    /// </summary>
    protected String m_contentType = null;
    private IOKHttpService m_okService = null;
    private IOKHttpSession m_okSession = null;
    
    /// <summary>
    /// Gets an instance of a RestRequest object
    /// </summary>
    /// <param name="url">Url to the Rest endpoint</param>
    /// <param name="service">Same service object should be used for all requests. Available from OpenHTTPServiceHelper</param>
    /// <param name="session">Same session object should be used for all requests. Available from OpenHTTPServiceHelper</param>
    public static RestRequest GetInstance(String url, IOKHttpService service,
     IOKHttpSession session)
    {
      return new RestRequest(url, service, session);
    }
    
    /// <summary>
    /// Internal constructor
    /// </summary>
    /// <param name="url"></param>
    /// <deprecated></deprecated>
    private RestRequest(String url)
    {
      if (url == null)
      {
        throw new XPIllegalArgumentException("Empty url provided.");
      }
      IOKUrl okUrl = null;
      IOKHttpService service = OKHttpFactory.getInstance();
      try
      {
        okUrl = service.CreateUrl(url);
      }
      catch (OKException e)
      {
        throw new XPIllegalArgumentException("Unable to create URL object from: " + url);
        }
      m_url = okUrl;
    }
    
    /// <summary>
    /// Internal constructor
    /// </summary>
    /// <param name="url"></param>
    /// <param name="service"></param>
    /// <param name="session"></param>
    private RestRequest(String url, IOKHttpService service, IOKHttpSession session)
    {
      m_okService = service;
      m_okSession = session;
      if (service == null)
      {
        throw new XPIllegalArgumentException("IOKHTTPService was null.");
        }
      if (session == null)
      {
        throw new XPIllegalArgumentException("IOKHttpSession was null.");
        }
      IOKUrl okUrl = null;
      try
      {
        okUrl = service.CreateUrl(url);
      }
      catch (OKException e)
      {
        throw new XPIllegalArgumentException("Unable to create URL object from: " + url,
         e);
      }
      m_url = okUrl;
    }
    
    /// <summary>
    /// Used by MockRestRequest
    /// </summary>
    protected RestRequest()
    {
    }
    
    /// <summary>
    /// Submits the Rest request
    /// </summary>
    /// <returns></returns>
    public virtual IRestResponse Submit()
    {
      if (m_okService == null || m_okSession == null)
      {
        throw new XPIllegalStateException("Either OpenHttpService or OpenHttpSession is missing. RestRequest not initialized properly!");
        }
      IOKHttpRequest req = m_okSession.CreateRequest(m_url);
      IOKHttpResponse rawResponse = null;
      IOKRequestWaiter waiter = m_okService.GetWaiterObject();
      SetAuthenticationOnRequest(req);
      // set the post body if needed
      if (m_postBody != null)
      {
        req.SetBody(m_postBody);
      }
      //could do timeout here...
      req.SetTimeout(m_msTimeout);
      req.SetHttpMethod(m_reqMethod);
      req.GetRequestHeaders().SetHeader(ACCEPT_LANGUAGE, XPLocale.GetDefault().GetShortName());
      // If Content-Type has been set, we will add it to the header
      if (null != m_contentType)
      {
        req.GetRequestHeaders().SetHeader(HTTPHeaderConstants.CONTENT_TYPE,
         m_contentType);
      }
      waiter.AddRequest(req);
      bool result = req.Submit();
      if (!result)
      {
        OKHttpException submitError = req.GetErrorInfo();
        throw new XPIllegalStateException("Rest Request failed!", submitError);
        }
      waiter.Wait(m_msTimeout);
      if (OKHttpRequestState.DONE == req.GetRequestState())
      {
        rawResponse = req.GetResponse();
      } else
      if (OKHttpRequestState.CANCELED == req.GetRequestState())
      {
        if (log.IsInfoEnabled())
        {
          log.Info("Rest request to: " + m_url.GetFullPath() + " timed out after " + (m_msTimeout / 1000));
          }
      } else
      if (OKHttpRequestState.ERROR == req.GetRequestState())
      {
        OKHttpException e = req.GetErrorInfo();
        if (e != null)
        {
          throw e;
        }
      } else
      {
        log.Warn("Unhandled request state encountered: " + req.GetRequestState());
        }
      // received a response, now check the result of it
      IRestResponse response = null;
      if (null == rawResponse)
      {
        throw new RestResponseFailedException("An error occurred while waiting for the REST Response.  This could be due to the REST Request timing out or an " + "unknown connection issue with the REST host server.");
        }
      int httpResponseCode = rawResponse.GetResponseCode();
      if (httpResponseCode == OKHttpResponseCode.OK || httpResponseCode == OKHttpResponseCode.ACCEPTED || httpResponseCode == OKHttpResponseCode.CREATED)
      {
        // figure out whether the response is of xml or json format
        String responseContent = rawResponse.GetBodyAsString();
        if (XPStringUtility.StartsWith(responseContent, "<?xml"))
        {
          response = new RestXMLResponse(rawResponse);
        } else
        if (XPStringUtility.StartsWith(responseContent, "{"))
        {
          // @TODO once we have a jumpable JSON parser
          throw new XPNotImplementedException("Rest JSON Responses are not supported by this framework at this time.");
          } else
        {
          throw new XPIllegalStateException("Unknown REST response received, unable to parse as either XML or JSON: " + responseContent);
          }
      } else
      // error responses, log the error and return null
      if (httpResponseCode == OKHttpResponseCode.NOT_FOUND)
      {
        log.Error("REST API endpoint was not reachable (HTTP Response 404). REST API URL: " + m_url + ". Verify that the url is correct and the REST service is running properly.");
        } else
      if (httpResponseCode == OKHttpResponseCode.SERVER_ERROR)
      {
        log.Error("REST API endpoint returned a Server Error (HTTP Response 500). REST API URL: " + m_url + ". Verify that REST endpoint is configured and running properly.");
        } else
      if (httpResponseCode == OKHttpResponseCode.AUTH_REQUIRED)
      {
        log.Error("REST API endpoint did not accept authentication provided (HTTP Response 401). REST API URL: " + m_url + ". Verify proper authentication is configured and set.");
        } else
      {
        log.Error("REST API endpoint returned unhandled HTTP Response Code: " + httpResponseCode + " from url: " + m_url);
        }
      return response;
    }
    //	public IRestResponse Submit()
    //	{
    //		IOKHttpService service = OKHttpFactory.getInstance();
    //		IOKHttpSession session = service.CreateHttpSession();
    //		IOKHttpRequest req = session.CreateRequest(m_url);
    //		IOKHttpResponse rawResponse = null;
    //		IOKRequestWaiter waiter = service.GetWaiterObject();
    //			
    //		SetAuthenticationOnRequest(req);
    //		
    //		// set the post body if needed
    //		if ( m_postBody != null ) {
    //			req.SetBody(m_postBody);
    //		}
    //		
    //		//could do timeout here...
    //		req.SetTimeout( m_msTimeout );
    //		req.SetHttpMethod(m_reqMethod);
    //		req.GetRequestHeaders().SetHeader(ACCEPT_LANGUAGE, XPLocale.GetDefault().GetShortName());
    //		waiter.AddRequest( req );
    //		boolean result = req.Submit();
    //		
    //		if ( !result ) {
    //			OKHttpException submitError = req.GetErrorInfo();
    //			throw new XPIllegalStateException("Rest Request failed!", submitError);
    //		}
    //		
    //		waiter.Wait( m_msTimeout );
    //
    //		if ( OKHttpRequestState.DONE == req.GetRequestState() )
    //		{
    //			rawResponse = req.GetResponse();
    //		}
    //		else if (OKHttpRequestState.CANCELED == req.GetRequestState())
    //		{
    //			if ( log.IsInfoEnabled() ) {
    //				log.Info("Rest request to: "+m_url.GetFullPath()+" timed out after "+(m_msTimeout/1000));
    //			}
    //		}
    //		else if (OKHttpRequestState.ERROR == req.GetRequestState())
    //		{
    //			OKHttpException e = req.GetErrorInfo();
    //			if (e != null) { throw e; }
    //		}
    //		else {
    //			log.Warn("Unhandled request state encountered: "+req.GetRequestState());
    //		}
    //		
    //		// received a response, now check the result of it
    //		
    //		IRestResponse response = null;
    //		if (null == rawResponse)
    //		{
    //			throw new RestResponseFailedException("An error occurred while waiting for the REST Response.  This could be due to the REST Request timing out or an " +
    //					"unknown connection issue with the REST host server.");
    //
    //		}
    //		int httpResponseCode = rawResponse.GetResponseCode();
    //
    //		if ( httpResponseCode == OKHttpResponseCode.OK 
    //				|| httpResponseCode == OKHttpResponseCode.ACCEPTED 
    //				|| httpResponseCode == OKHttpResponseCode.CREATED) {
    //			// figure out whether the response is of xml or json format
    //			String responseContent = rawResponse.GetBodyAsString();
    //			if ( responseContent.startsWith("<?xml") ) {
    //				response = new RestXMLResponse(rawResponse);
    //			} else if ( responseContent.startsWith("{") ) {
    //				// @TODO once we have a jumpable JSON parser
    //				throw new XPNotImplementedException("Rest JSON Responses are not supported by this framework at this time.");
    //			} else  {
    //				throw new XPIllegalStateException("Unknown REST response received, unable to parse as either XML or JSON: "+responseContent);
    //			}
    //			// error responses, log the error and return null
    //		} else if ( httpResponseCode == OKHttpResponseCode.NOT_FOUND  ) {
    //			log.Error("REST API endpoint was not reachable (HTTP Response 404). REST API URL: "+m_url+". Verify that the url is correct and the REST service is running properly.");
    //		} else if ( httpResponseCode == OKHttpResponseCode.SERVER_ERROR ) {
    //			log.Error("REST API endpoint returned a Server Error (HTTP Response 500). REST API URL: "+m_url+". Verify that REST endpoint is configured and running properly.");
    //		}  else if ( httpResponseCode == OKHttpResponseCode.AUTH_REQUIRED ) {
    //			log.Error("REST API endpoint did not accept authentication provided (HTTP Response 401). REST API URL: "+m_url+". Verify proper authentication is configured and set.");
    //		} else {
    //			log.Error("REST API endpoint returned unhandled HTTP Response Code: "+httpResponseCode+" from url: "+m_url);
    //		}
    //
    //		return response;
    //
    //	}
    //	
    
    /// <summary>
    /// Applies the authentication on the request
    /// Extend this method to add additional authentication methods
    /// </summary>
    /// <param name="req"></param>
    protected virtual void SetAuthenticationOnRequest(IOKHttpRequest req)
    {
      switch (m_authMethod){
      case AUTHENTICATION_METHOD_BASICAUTH:
      String username = (String) m_authData.GetElement(BASICAUTH_USERNAME);
      String password = (String) m_authData.GetElement(BASICAUTH_PASSWORD);
      req.SetHttpAuthentication(username, password);
      break;
      case AUTHENTICATION_METHOD_HEADERTOKEN:
      String tokenName = (String) m_authData.GetElement(HEADERTOKEN_NAME);
      String tokenValue = (String) m_authData.GetElement(HEADERTOKEN_VALUE);
      req.GetRequestHeaders().SetHeader(tokenName, tokenValue);
      break;
      case AUTHENTICATION_METHOD_CSPLOGINTOKEN:
      String csploginTokenVal = (String) m_authData.GetElement(CSPLOGINTOKEN_NAME);
      req.GetRequestHeaders().SetHeader(CSPLOGINTOKEN_NAME, csploginTokenVal);
      break;
      case AUTHENTICATION_METHOD_NONE:
      // none needed, handled by remote server object
      break;
      default:
      throw new XPIllegalStateException("Invalid Authentication method specified: " + m_authMethod);
      break;
      }
    }
    
    /// <summary>
    /// Sets basic http authentication name and passsword
    /// </summary>
    /// <param name="userName"></param>
    /// <param name="password"></param>
    public virtual void SetHttpAuthentication(String userName, String password)
    {
      if (userName == null)
      {
        throw new XPIllegalArgumentException("Username must contain a value");
        }
      if (password == null)
      {
        throw new XPIllegalArgumentException("Password must contain a value");
        }
      m_authMethod = AUTHENTICATION_METHOD_BASICAUTH;
      m_authData.PutElement(BASICAUTH_USERNAME, userName);
      m_authData.PutElement(BASICAUTH_PASSWORD, password);
    }
    
    /// <summary>
    /// Sets a generic request header name value pair
    /// </summary>
    /// <param name="tokenName"></param>
    /// <param name="tokenValue"></param>
    public virtual void SetRequestHeaderTokenAuthentication(String tokenName,
     String tokenValue)
    {
      if (tokenName == null)
      {
        throw new XPIllegalArgumentException("Token name must contain a value");
        }
      if (tokenValue == null)
      {
        throw new XPIllegalArgumentException("Token value must contain a value");
        }
      m_authMethod = AUTHENTICATION_METHOD_HEADERTOKEN;
      m_authData.PutElement(HEADERTOKEN_NAME, tokenName);
      m_authData.PutElement(HEADERTOKEN_VALUE, tokenValue);
    }
    
    /// <summary>
    /// Set a WCI issued login token recognized by portlets  
    /// </summary>
    /// <param name="tokenValue"></param>
    public virtual void SetCSPLoginToken(String tokenValue)
    {
      if (tokenValue == null)
      {
        throw new XPIllegalArgumentException("Token value must contain a value");
        }
      m_authMethod = AUTHENTICATION_METHOD_CSPLOGINTOKEN;
      m_authData.PutElement(CSPLOGINTOKEN_NAME, tokenValue);
    }
    
    /// <summary>
    /// Setter
    /// </summary>
    /// <param name="method"></param>
    public virtual void SetRequestMethod(String method)
    {
      if (method == null)
      {
        throw new XPIllegalArgumentException("Request method must contain a value");
        }
      m_reqMethod = method;
    }
    public virtual void SetPostBody(String bodyContent)
    {
      if (bodyContent == null)
      {
        throw new XPIllegalArgumentException("Post body content cannot be null!");
        }
      m_postBody = XPConvert.ToByteArray(bodyContent, "UTF-8");
    }
    
    /// <summary>
    /// Getter
    /// </summary>
    /// <returns></returns>
    public virtual String getRequestMethod()
    {
      return m_reqMethod;
    }
    
    /// <summary>
    /// Getter
    /// </summary>
    /// <returns></returns>
    public virtual IOKUrl getRequestUrl()
    {
      return m_url;
    }
    
    /// <summary>
    /// This method can be used to set Content-Type header on the
    /// rest request.
    /// </summary>
    /// <param name="contentType"></param>
    public virtual void SetContentTypeHeader(String contentType)
    {
      m_contentType = contentType;
    }
    
    /// <summary>
    /// Returns the Content-Type header value being set on this request.
    /// </summary>
    /// <returns></returns>
    public virtual String GetContentTypeHeader()
    {
      return m_contentType;
    }
  }
}