using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.xml.dom;
using com.plumtree.openkernel.http;
namespace com.plumtree.portaluiinfrastructure.restconsumerframework
{
  /// <summary>
  /// Helper class to process responses containing XML
  /// </summary>
  /// <author>rozhang</author>
  public class RestXMLResponse : IRestResponse
  {
    protected XPDocument m_doc = null;
    protected String m_toplevelnodenames = null;
    protected bool m_notoplevelnodes = false;
    
    /// <param name="response"></param>
    public RestXMLResponse(IOKHttpResponse response)
    {
      if (response == null)
      {
        throw new XPIllegalArgumentException("Valid response required, cannot be null.");
        }
      XPDocument doc = XPBuilder.LoadDocumentFromByteArray(response.GetBodyAsBinary());
      if (doc != null)
      {
        m_doc = doc;
      }
    }
    
    /// <summary>
    /// Mainly for unit tests 
    /// </summary>
    /// <param name="response"></param>
    public RestXMLResponse(String responseContent)
    {
      if (responseContent == null)
      {
        throw new XPIllegalArgumentException("Valid response content required, cannot be null.");
        }
      XPDocument doc = XPBuilder.LoadDocumentFromString(responseContent);
      if (doc != null)
      {
        m_doc = doc;
      }
    }
    
    /// <summary>
    /// Gets the name of the top level element
    /// </summary>
    /// <returns></returns>
    public virtual String GetRootElementName()
    {
      if (m_doc == null)
      {
        throw new XPIllegalStateException("This object has not been properly initialized. No valid response was provided");
        }
      XPElement root = m_doc.GetDocumentElement();
      String name = root.GetNodeName();
      return name;
    }
    
    /// <summary>
    /// Returns the name of a node list under the root node. All the nodes
    /// under the root node has to share the same name to be considered a valid
    /// node list. 
    /// </summary>
    /// <returns>null if the response content doesn't contain a list </returns>
    public virtual String GetListName()
    {
      if (m_notoplevelnodes || m_toplevelnodenames != null)
      {
        return m_toplevelnodenames;
      }
      XPElement root = m_doc.GetDocumentElement();
      XPNodeList list = root.GetChildNodes();
      String listnodenames = null;
      for (int i = 0;i < list.GetLength();i++)
      {
        XPNode node = list.Item(i);
        if (node.IsTextNode() || node.IsCommentNode())
        {
          continue;
        }
        if (node.GetNodeType() == XPNode.ELEMENT_NODE)
        {
          // first node encountered, store the name
          if (listnodenames == null)
          {
            listnodenames = node.GetNodeName();
          } else
          // we have node name stored, check if it is the same
          if (!listnodenames.Equals(node.GetNodeName()))
          {
            m_notoplevelnodes = true;
            m_toplevelnodenames = null;
          } else
          {
            m_notoplevelnodes = false;
            m_toplevelnodenames = listnodenames;
          }
        }
      }
      return listnodenames;
    }
    
    /// <summary>
    /// Exclusive from GetResponseList method, only one is valid in any one response
    /// </summary>
    /// <returns>null if the response is actually a list of items</returns>
    public virtual XPHashtable GetSingleDataItem()
    {
      XPElement root = m_doc.GetDocumentElement();
      XPHashtable datamap = ExtractNodeChildData(root);
      return datamap;
    }
    
    /// <summary>
    /// Gets the element name of the root element.
    /// </summary>
    /// <returns></returns>
    public virtual String GetRootNodeName()
    {
      XPElement root = m_doc.GetDocumentElement();
      return root.GetNodeName();
    }
    
    /// <summary>
    /// Exclusive from GetSingleDataItem method, only one is valid in any one response
    /// </summary>
    /// <returns>
    /// null if the current response does not contain a top level list, XPArrayList contains 
    /// XPHashtables
    /// </returns>
    public virtual XPArrayList GetResponseList()
    {
      // first check that we have a top level list in the response (compared to single item)
      if (GetListName() == null)
      {
        // log
        return null;
      }
      XPArrayList outVal = new XPArrayList();
      XPElement root = m_doc.GetDocumentElement();
      XPNodeList list = root.GetChildNodes();
      for (int i = 0;i < list.GetLength();i++)
      {
        XPNode node = list.Item(i);
        if (node.IsTextNode() || node.IsCommentNode())
        {
          continue;
        }
        if (node.GetNodeType() == XPNode.ELEMENT_NODE)
        {
          XPHashtable datamap = ExtractNodeChildData(node);
          outVal.Add(datamap);
        }
      }
      return outVal;
    }
    
    /// <summary>
    /// INTERNAL METHODS ****
    /// </summary>
    protected virtual XPElement GetRootNode()
    {
      if (m_doc == null)
      {
        throw new XPIllegalStateException("This object has not been properly initialized. No valid response was provided");
        }
      XPElement root = m_doc.GetDocumentElement();
      return root;
    }
    
    /// <summary>
    /// Helper method to get the text value in a node
    /// </summary>
    /// <param name="node"></param>
    /// <returns></returns>
    protected virtual String GetNodeTextBody(XPNode node)
    {
      if (node == null)
      {
        throw new XPIllegalArgumentException("Provided XML node element is not valid");
        }
      // empty node, just treat it as empty string 
      if (node.GetFirstChild() == null)
      {
        return "";
      }
      String textValue = node.GetFirstChild().GetNodeValue();
      if ("".Equals(textValue))
      {
        return "";
      } else
      {
        return textValue;
      }
    }
    
    /// <summary>
    /// Fills a hashtable with the child node names and node body text of the provided node.
    /// Multiple child nodes with the same names are inserted into the same hash key, and the multiple
    /// values are added to an XPArrayList
    /// Nested child nodes are inserted as nested XPHashTable    
    /// </summary>
    /// <param name="topnode"></param>
    /// <returns>null if the provided node is null or does not have any child nodes.</returns>
    protected virtual XPHashtable ExtractNodeChildData(XPNode topnode)
    {
      XPNodeList list = topnode.GetChildNodes();
      if ((list == null) || (list.GetLength() == 0))
      {
        return null;
      }
      XPHashtable hashtable = new XPHashtable();
      // iterate over all the child nodes and add the values for the nodes in there
      for (int i = 0;i < list.GetLength();i++)
      {
        XPNode node = list.Item(i);
        if (node.IsTextNode() || node.IsCommentNode())
        {
          continue;
        }
        if (node.GetNodeType() == XPNode.ELEMENT_NODE)
        {
          String name = node.GetNodeName();
          XPNodeList children = node.GetChildNodes();
          // value will be empty string for nodes with no body since we need to put in a non-null
          // value into the hashtable
          // @TODO should we trim the string here?
          Object value = GetNodeTextBody(node);
          //P2/portalui/main/ptwebui/portaluiinfrastructure/java/prod/src/com/plumtree/portaluiinfrastructure/restconsumerframework/RestXMLResponse.java
          /*  bug fix 9410440 - IF ONLY ONE TAG IS AVAILABLE FOR A PORTAL OBJECT, IT IS NOT VISIBLE
           *  It seems in the .net Node, node.GetFirstChild().GetNodeValue(), results in a null value and that's a key difference with the java
           *  node.  B/c of this, a single item in the <tags> node will result in a null value.  This is the same result for multiple items in the
           *  <tags> node, except that we traverse the children of the child nodes in below if statement:  if (children.GetLength() >1) so the tag
           *  items are actually getting extracted.  In a single tag value, we skip the extraction of the node.
           *  To fix this bug, I will check if the system is running in .net, we're traversing the <tags> node, and there is an item length of 1.
           */
           
          String TAGS_NODE = "tags";
          if (XPSystem.IsRunningInDotNet() && name.Equals(TAGS_NODE) && (children.GetLength() == 1))
          {
            value = ExtractNodeChildData(node);
          }
          // we got a complex node with child elements, populate a hashtable with the child nodes
          if (children.GetLength() > 1)
          {
            value = ExtractNodeChildData(node);
          }
          // check if that value already exists, if so add values to a list
          if (hashtable.ContainsKey(name))
          {
            Object o = hashtable.GetElement(name);
            // just add to existing list if one has already been created
            if (o is XPArrayList)
            {
              ((XPArrayList) o).Add(value);
            } else
            {
              // create a list and put the old value in there before the current value
              XPArrayList bucket = new XPArrayList();
              bucket.Add(o);
              bucket.Add(value);
              hashtable.PutElement(name, bucket);
            }
          } else
          {
            hashtable.PutElement(name, value);
          }
        }
      }
      return hashtable;
    }
  }
}