using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.search
{
  /// <author>CraigS</author>
  public class PaginationControl : SearchResultControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.search.PaginationControl");
    public const String STR_MVC_CLASS_NAME = "page";
    public const String FIRST_RESULT_DATA = "first";
    public const String PAGE_SIZE_DATA = "num";
    public const String ORDER_BY_DATA = "order";
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.activityspace.IControl#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Factory method to make a pagination link for some search results.
    /// </summary>
    /// <param name="model">Current batch of search results</param>
    /// <param name="parent">ActivitySpace we're living in</param>
    /// <param name="urlText">Desired text within URL</param>
    /// <param name="orderBy">Sort order within which newFirst is to be interpreted</param>
    /// <param name="newFirst">Desired first result on new page</param>
    /// <param name="pageSize">Desired number of results on page</param>
    public static String makeURLAsString(IBaseSearchResultModelRO model, AActivitySpace parent,
     String urlText, String orderBy, int newFirst, int pageSize)
    {
      ASURL url = (ASURL) makeURL(model, parent, urlText, orderBy, newFirst,
       false);
      url.AddControlArgument(PaginationControl.PAGE_SIZE_DATA, pageSize);
      return url.GetURLAsString();
    }
    
    /// <summary>
    /// Factory method to make a pagination link for some search results.
    /// </summary>
    /// <param name="model">Current batch of search results</param>
    /// <param name="parent">ActivitySpace we're living in</param>
    /// <param name="urlText">Desired text within URL</param>
    /// <param name="orderBy">Sort order within which newFirst is to be interpreted</param>
    /// <param name="newFirst">Desired first result on new page</param>
    /// <param name="pageSize">Desired number of results on page</param>
    public static HTMLElement makeURL(IBaseSearchResultModelRO model, AActivitySpace parent,
     String urlText, String orderBy, int newFirst, int pageSize)
    {
      ILinkElement url = makeURL(model, parent, urlText, orderBy, newFirst);
      url.AddControlArgument(PaginationControl.PAGE_SIZE_DATA, pageSize);
      return url.GetURLAsHTMLElement();
    }
    
    /// <summary>
    /// Factory method to make a pagination link for some search results.
    /// </summary>
    /// <param name="model">Current batch of search results</param>
    /// <param name="parent">ActivitySpace we're living in</param>
    /// <param name="urlText">Desired text within URL</param>
    /// <param name="orderBy">Sort order within which newFirst is to be interpreted</param>
    /// <param name="newFirst">Desired first result on new page</param>
    public static ILinkElement makeURL(IBaseSearchResultModelRO model, AActivitySpace parent,
     String urlText, String orderBy, int newFirst)
    {
      bool embedInForm = SpaceNeedsPassthruURL(parent);
      return makeURL(model, parent, urlText, orderBy, newFirst, embedInForm);
      }
    
    /// <summary>
    /// Factory method to make a pagination link for some search results.
    /// </summary>
    /// <param name="model">Current batch of search results</param>
    /// <param name="parent">ActivitySpace we're living in</param>
    /// <param name="urlText">Desired text within URL</param>
    /// <param name="orderBy">Sort order within which newFirst is to be interpreted</param>
    /// <param name="newFirst">Desired first result on new page</param>
    /// <param name="bEmbed">Whether or not this URL should be embedded in a form</param>
    public static ILinkElement makeURL(IBaseSearchResultModelRO model, AActivitySpace parent,
     String urlText, String orderBy, int newFirst, bool bEmbed)
    {
      // Create a URL that directs into the current Activity Space
      ILinkElement url = null;
      if (bEmbed)
      {
        url = new PassthruURL();
      } else
      {
        url = new ASURL(parent);
        url.SetLinkGetCachedSpace(parent);
        url.SetDisplayPage(parent.GetCurrentPage().GetName());
      }
      // Create a PaginationControl within the search space
      url.SetControl(PaginationControl.STR_MVC_CLASS_NAME);
      // Set superclass data
      // NOTE: We know we're always passed a fullblown
      // ISearchResultModelRO, since there's only one
      // implementation of either interface (SearchResultModel).
      addCommonData((ISearchResultModelRO) model, url);
      // Set link text
      url.AddInnerHTMLString(urlText);
      // Set other data
      url.AddControlArgument(PaginationControl.ORDER_BY_DATA, orderBy);
      url.AddControlArgument(PaginationControl.FIRST_RESULT_DATA, XPConvert.ToString(newFirst));
      return url;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new PaginationControl();
    }
    
    /// <seealso cref="com.plumtree.portalpages.search.SearchResultControl#ExecuteSearchControl()"></seealso>
    protected override Redirect ExecuteSearchControl(XPHashtable args, String key)
    {
      int newFirstResult;
      int newPageSize;
      String orderBy;
      // First result
      String[] values = (String[]) args.GetElement(FIRST_RESULT_DATA);
      if (null == values)
      {
        log.Error("could not find first result data in query string.");
        return null;
      }
      try
      {
        newFirstResult = XPConvert.ToInteger(values[0]);
      }
      catch (Exception e)
      {
        log.Error(e, "could not parse first result data in query string: " + values[0]);
        return null;
      }
      // Page size
      values = (String[]) args.GetElement(PAGE_SIZE_DATA);
      if (null == values)
      {
        log.Error("could not find page size in query string.");
        return null;
      }
      try
      {
        newPageSize = XPConvert.ToInteger(values[0]);
      }
      catch (Exception e)
      {
        log.Error(e, "could not parse new page size data in query string: " + values[0]);
        return null;
      }
      // Sort order
      values = (String[]) args.GetElement(ORDER_BY_DATA);
      if (null != values)
      {
        orderBy = values[0];
      } else
      {
        log.Error("could not find order by data in query string");
        return null;
      }
      // Tell the model to go to the new page
      _model.GoToPage(orderBy, newFirstResult, newPageSize, key);
      // Don't need to redirect to a different page or space; just
      // redisplay the page
      return null;
    }
  }
}