using System;
using com.plumtree.openfoundation.util;
using SearchFriendlyURLParser=com.plumtree.portaluiinfrastructure.friendlyurl.SearchFriendlyURLParser;
using BannerSearchHelperService=com.plumtree.portaluiinfrastructure.helpers.BannerSearchHelperService;
using com.plumtree.portaluiinfrastructure.search.exceptions;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
namespace com.plumtree.portaluiinfrastructure.search
{
  /// <author>CraigS</author>
  public class StartSearchControl : SearchResultControl
  {
    
    /// <summary>
    /// Name of this control.
    /// When the UI framework sees a URL like
    /// http://blah/HTTPServlet&space=search&control=searchstart& ...
    /// the in_hi_control argument tells it to create a StartSearchControl.
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "searchstart";
    //
    //  Names of URL arguments governing PTSearchQuery creation
    //
    //  .. All of these are now defined in SearchResultControl.
    // CONSTRAINTS (aka SEARCH SETTINGS)
    //
    // These use the String-to-PT_SEARCH_SETTING equivalents defined in
    // class SearchSettingCollection, but with the following prefixes:
    // Prefixes for required constraints
    // (we need multiple versions because htmlelements enforces that the prefix
    //  on the <input ..> tag match the input type..)
    public const String REQUIRED_CONSTRAINT_PREFIX = "in_hi_req_";
    public const String REQUIRED_CONSTRAINT_PFX_CHECKBOXES = "in_cb_req_";
    public const String REQUIRED_CONSTRAINT_PFX_RADIO = "in_ra_req_";
    // Prefix for optional constraints
    public const String OPTIONAL_CONSTRAINT_PREFIX = "in_hi_opt_";
    public const String NON_INPUT_PREFIX = "non_input_";
    // Which options have been selected via pulldown menus
    public const String SELECTED_PREFIX = "in_se_sel_";
    
    /// <summary>
    /// Search filter string parameter name.
    /// </summary>
    public const String FILTER_QS = "filter";
    private SearchSettingCollection _removableSettings = null;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new StartSearchControl();
    }
    
    /// <summary>
    /// Public method to let other objects see what types banner search returns.
    /// </summary>
    public static int[] GetBannerSearchObjectTypes()
    {
      return new int[]{PT_CLASSIDS.PT_CATALOGCARD_ID, PT_CLASSIDS.PT_CATALOGFOLDER_ID,
       PT_CLASSIDS.PT_USER_ID, PT_CLASSIDS.PT_COMMUNITY_ID, PT_CLASSIDS.PT_GADGET_ID,
       PT_CLASSIDS.PT_PAGE_ID};
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.activityspace.IControl#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.portalpages.search.SearchResultControl#ExecuteSearchControl()"></seealso>
    protected sealed override Redirect ExecuteSearchControl(XPHashtable args,
     String key)
    {
      // Call superclass method to parse query arguments
      /* prefix= */
      QueryArguments query = parseQueryArgs(args, "");
      if (null == query)
      {
        throw new EmptyQueryException(_parent);
      }
      // Parse search setting arguments, and combine optional settings
      // and required settings into the set to be used for the search
      SearchSettingCollection settings = parseSettingArgs(args);
      // Check which mode the page will be displayed in
      String[] modeVals = (String[]) args.GetElement(SearchResultModel.MODEL_MODE_INPUT);
      if (modeVals != null && modeVals.Length != 0)
      {
        ((ISearchResultModel) _model).SetPageMode(modeVals[0]);
      }
      // Check page size
      String[] pageSize = (String[]) args.GetElement(PaginationControl.PAGE_SIZE_DATA);
      if (null != pageSize && pageSize.Length != 0)
      {
        settings.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_MAXRESULTS, pageSize[0]);
        }
      // Check to See Filter QS is enabled, if this is enabled then only filtered type of objects
      // are shown in search results.
      IPTSessionInfo sessionInfo = ((IPTSession) _parent.GetUserSession()).GetSessionInfo();
      String[] filter = (String[]) args.GetElement(FILTER_QS);
      if (null != filter && filter.Length != 0)
      {
        // we need to override the ObjectTypes that Search gets from the Search Preferences. 
        // Set a session pref so that we can figure out if we need to override when processing 
        // the search preferences later in StartSearchControl.
        // Note this particular use case is for tag query click-throughs from KD Browse layout.
        sessionInfo.AddSessionPreference(BannerSearchHelperService.SEARCHPREF_OBJECTTYPES_OVERRIDE,
         filter[0], false);
      } else
      {
        // Clear the filter preference if it exists from previous filter searches.
        sessionInfo.RemoveSessionPreference(BannerSearchHelperService.SEARCHPREF_OBJECTTYPES_OVERRIDE);
        }
      // Call "hook" to allow customization of the query
      Redirect optionalRedirect = CustomizeQuery(query);
      // If the subclass decided we should go to a different space, return the Redirect
      if (null != optionalRedirect)
      {
        return optionalRedirect;
      }
      // See what the subclass wants to set as the "userQuery" logged in
      // the search server logs
      String logString = GetLogString();
      if (null != logString)
      {
        settings.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_INITIALSEARCHSTRING,
         logString);
      }
      // Now, some special case behavior for the search result page (only) -
      // the following should *not* happen in other ActivitySpaces that use
      // the classes in portaluiinfrastructure.search.
      //
      // This really deserves a refactoring.  Right now, this control class
      // (StartSearchControl) handles initialization for:
      //  (1) searches run in ActivitySpaces that integrate search (such as Add Portlets)
      //  (2) any search ending up on the search result page OTHER than
      //      banner searches or the advanced search page (such as clickthroughs
      //      from snapshot queries or custom search portlets).
      // We should really only cover case (1) in this class, and create
      // a subclass in portalpages.browsing.search.result to
      // handle case (2), since case (2) needs a bit of special-case behavior
      // that we should NOT force on a space like Add Portlets.
      // 
      // However, adding a subclass would change the URL format a bit -
      // the "control" argument would need to be different for one of the
      // two classes.  This would require a lot of little changes in
      // various parts of the portal that I don't want to touch, and/or
      // might break a lot of custom search portlets out in the world.
      // Instead, I'm just going to hack it here.
      // 
      // Note to future maintainers of this package - in the past a LOT
      // of bugs have resulted from special-case behavior like this getting
      // put into various places in portaluiinfrastructure.search.
      // Then it gets picked up by any other part of the portal that
      // uses these common utility classes.
      // Any changes that should only affect the search result page
      // should really go into some class in
      // portalpages.browsing.search.result, NOT HERE!!!
      if (_parent.GetName().Equals("SearchResult"))
      {
        // If we didn't already specify a group-by setting
        if (null == settings.GetSettingValue(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP))
        {
          // if we're in 508/low-bandwidth mode
          if (_parent.GetAccessStyle() != AccessStyles.STANDARD)
          {
            // then default to object-type categorization
            settings.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_GROUPBYPROP, SearchResultModel.GROUPBY_OBJECTTYPE);
            }
        }
      }
      // Initialize the model by running a search
      _model.Search(query.userQuery, query.basicFields, query.advancedFilter,
       settings, _removableSettings);
      // Don't need to redirect to a different page or space; just
      // display the page
      return null;
    }
    
    /// <summary>
    /// Overrides superclass method; indicates we are initializing the
    /// model and that SearchResultControl.Execute() shouldn't try to
    /// deal with an uninitialized model.
    /// </summary>
    protected sealed override bool expectModelInitialized()
    {
      return false;
    }
    
    /// <summary>
    /// This is a "hook" that subclasses can override to make special-
    /// purpose modifications to a search query just before it runs, or
    /// to redirect specific searches to a different page.
    /// See
    /// portalui.browsing.search.results.BannerStartSearchControl.CustomizeQuery()
    /// to customize banner search.
    /// The provided QueryArguments object contains the query as specified
    /// in the URL (see the javadocs or n-docs for that class).
    /// The subclass can do either of two things with this object:
    /// (1) return a Redirect that forwards the user on to some other
    /// page - for example, if the query has special syntax like
    /// a ">" in front of the query string
    /// (2) replace the members of the QueryArguments to substitute
    /// a different search - for example, you can replace the simple
    /// string userQuery with an IPTFilter that runs a complex search.
    /// Then return null to stay on the search page instead of redirecting.
    /// </summary>
    protected virtual Redirect CustomizeQuery(QueryArguments query)
    {
      // Default implementation does nothing
      return null;
    }
    
    /// <summary>
    /// This is a "hook" that subclasses can override to add additional
    /// settings to a partcular class of search (such as banner search).
    /// It's an alternative to encoding the settings in every search
    /// form that initiates such a search.
    /// See portalui.browsing.search.results.BannerStartSearchControl
    /// for a usage example.
    /// </summary>
    protected virtual SearchSettingCollection GetCustomSearchSettings()
    {
      // Default implementation does nothing
      return null;
    }
    
    /// <summary>
    /// Subclasses may override this method to log a string as the "userQuery"
    /// in the search server log files.  This method will always be called last,
    /// after GetCustomSearchSettings() and CustomizeQuery().
    /// </summary>
    protected virtual String GetLogString()
    {
      // Default implementation logs nothing
      return null;
    }
    
    /// <summary>
    /// Parse all required and optional search settings in the provided
    /// set of arguments, determine which sets of options were actually
    /// selected, and combine all required and selected settings into
    /// a single (returned) SearchSettingCollection.  Also populates
    /// _removableSettings with those settings that could be removed
    /// on a followup search.
    /// </summary>
    private SearchSettingCollection parseSettingArgs(XPHashtable args)
    {
      // Required settings
      SearchSettingCollection requiredSettings = new SearchSettingCollection();
      // Possible collections of settings that could be selected; this is a
      // map from String keys to SearchSettingCollection objects
      XPHashtable optionalCollections = new XPHashtable();
      // Which SearchSettingCollection objects are actually being used;
      // this is a list of String keys into optionalCollections
      IXPList selectedCollections = new XPArrayList();
      IXPEnumerator e = args.GetKeys();
      while (e.MoveNext())
      {
        String name = (String) e.GetCurrent();
        String[] data = (String[]) args.GetElement(name);
        if (XPStringUtility.StartsWith(name, SELECTED_PREFIX))
        {
          //
          // OPTIONAL SETTINGS
          //
          // A collection of settings selected from a pulldown menu
          selectedCollections.Add(data[0]);
        } else
        //
        //  ONE SETTING WITHIN ONE SET
        //
        {
          SearchSettingCollection coll = null;
          String constraintName = null;
          // First, figure out which collection this one belongs to.
          // It's either the required collection, with a name like req_objtype,
          // or an optional collection, in which case the name is something like
          // opt_foobar_objtype, and will be used only if the "foobar"
          // option is selected.
          if (XPStringUtility.StartsWith(name, REQUIRED_CONSTRAINT_PREFIX) || XPStringUtility.StartsWith(name,
           REQUIRED_CONSTRAINT_PFX_CHECKBOXES) || XPStringUtility.StartsWith(name,
           REQUIRED_CONSTRAINT_PFX_RADIO))
          {
            // Required set
            coll = requiredSettings;
            constraintName = XPStringUtility.Substring(name, REQUIRED_CONSTRAINT_PREFIX.Length);
            } else
          if (XPStringUtility.StartsWith(name, OPTIONAL_CONSTRAINT_PREFIX))
          {
            // extract the name of the set
            String postPrefix = XPStringUtility.Substring(name, OPTIONAL_CONSTRAINT_PREFIX.Length);
            int underscore = XPStringUtility.IndexOf(postPrefix, '_');
            // if postPrefix contains an underscore
            if (-1 != underscore)
            {
              // and does not start or end with an underscore
              if ((underscore > 0) && (underscore < (postPrefix.Length - 1)))
              {
                // then the set name is the part of postPrefix up to the underscore;
                // the constraint name is everything afterward
                constraintName = XPStringUtility.Substring(postPrefix, underscore + 1);
                String setName = XPStringUtility.Substring(postPrefix, 0,
                 underscore);
                // If we've seen the set before
                if (optionalCollections.ContainsKey(setName))
                {
                  // use the collection already created
                  coll = (SearchSettingCollection) optionalCollections.GetElement(setName);
                  } else
                {
                  // create the collection and store it
                  coll = new SearchSettingCollection();
                  optionalCollections.PutElement(setName, coll);
                }
              }
            }
          }
          // end optional constraint
          if ((null != coll) && (null != constraintName))
          {
            // Change the appropriate setting in whichever SearchSettingCollection object
            int settingId = SearchSettingCollection.getSettingIdFromString(constraintName);
            if (0 != settingId)
            {
              try
              {
                Object settingValue = SearchSettingCollection.parseURLString(settingId,
                 data);
                coll.add(settingId, settingValue);
              }
              catch (Exception nfe)
              {
              }
            }
          }
        }
      }
      // Begin to prepare final set of settings
      SearchSettingCollection settings = requiredSettings;
      // If our subclass requires any settings, merge these in
      SearchSettingCollection subclassSettings = GetCustomSearchSettings();
      if (null != subclassSettings)
      {
        settings.merge(subclassSettings);
      }
      // Now iterate over the selected collections, merging their settings into
      // one unified set of removable settings
      if (selectedCollections.GetSize() > 0)
      {
        SearchSettingCollection removableSettings = new SearchSettingCollection();
        IXPEnumerator selections = selectedCollections.GetEnumerator();
        while (selections.MoveNext())
        {
          String selectionName = (String) selections.GetCurrent();
          if (optionalCollections.ContainsKey(selectionName))
          {
            SearchSettingCollection selectedColl = (SearchSettingCollection) optionalCollections.GetElement(selectionName);
            // Merge the selected set in with the combined set of removable settings
            removableSettings.merge(selectedColl);
          }
        }
        if (removableSettings.GetSize() > 0)
        {
          // Finally, merge removable settings with actual set being used
          settings.merge(removableSettings);
          _removableSettings = removableSettings;
        }
      }
      return settings;
    }
  }
}