using System;
// Framework Imports
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.common.constants;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.form;
namespace com.plumtree.portaluiinfrastructure.securitymatrix
{
  /// <summary>
  /// This class keeps an up-to-date state of all the settings.
  /// This class is used by the Section and by the Repost Control.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class SecurityMatrixModel : EditorModel,ISecurityMatrixModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.securitymatrix.SecurityMatrixModel");
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "SecurityMatrix";
    public const String BR_ELEMENT = "<BR>";
    
    /// <summary>
    /// job id for ABO security change 
    /// </summary>
    private int m_nABOJobID;
    
    /// <summary>
    /// job name for ABO security change 
    /// </summary>
    private String m_sABOJobName;
    
    /// <summary>
    /// folder name job created in 
    /// </summary>
    private String m_sABOFolderJobCreatedInName;
    
    /// <summary>
    /// Implements some actions to perform on start.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>boolean Is true if the current user has the rights to access this Editor.</returns>
    public override bool CheckEditorAccessAndStart(XPHashtable _htFormData)
    {
      return ((MultiObjectsSecurityModel) m_asOwner.GetModel(SecurityMatrixEditorAS.STR_MVC_STORAGE_NAME_MULTI_SECURITY)).InitializeMultiObjsSecurityPage(_htFormData,
       (IPTSession) m_asOwner.GetUserSession());
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new SecurityMatrixModel();
    }
    
    /// <summary>
    /// Implement some code to execute when the user clicks on "Finish".
    /// </summary>
    public override void Finish()
    {
      ((MultiObjectsSecurityModel) m_asOwner.GetModel(SecurityMatrixEditorAS.STR_MVC_STORAGE_NAME_MULTI_SECURITY)).Finish();
      }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Calls the right save function according to the page name passed in argument.
    /// </summary>
    /// <param name="_sPageName"></param>
    /// <param name="_htFormData"></param>
    /// <returns>int Page status for server side validation</returns>
    public override int SavePage(String _sPageName, XPHashtable _htFormData)
    {
      if (_sPageName.Equals(SecurityMatrixDP.STR_MVC_CLASS_NAME))
      {
        ((MultiObjectsSecurityModel) m_asOwner.GetModel(SecurityMatrixEditorAS.STR_MVC_STORAGE_NAME_MULTI_SECURITY)).SaveSection(_htFormData);
        }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <returns>the current IPTAccessList of the folder whose security is being edited</returns>
    public virtual IPTAccessList GetCurrentFolderACL()
    {
      return ((MultiObjectsSecurityModel) m_asOwner.GetModel(SecurityMatrixEditorAS.STR_MVC_STORAGE_NAME_MULTI_SECURITY)).GetCurrentFolderACL();
      }
    
    /// <param name="_ptOldACL">- the IPTAccessList from before the ACL was changed</param>
    /// <returns>true, if an admin acl was added or an acl was changed to admin.</returns>
    public virtual bool IsAdminModifiedOnFolder(IPTAccessList _ptOldACL)
    {
      return ((MultiObjectsSecurityModel) m_asOwner.GetModel(SecurityMatrixEditorAS.STR_MVC_STORAGE_NAME_MULTI_SECURITY)).IsAdminModifiedOnFolder(_ptOldACL);
      }
    
    /// <summary>
    /// Creates a job to propagate security.
    /// </summary>
    /// <param name="bPropagateAllACLs">
    /// - If true propagate all acls to child objects
    /// and subfolders, if false propagate only admin acls.
    /// </param>
    public virtual void PropagateSecurityUsingABO(bool bPropagateAllACLs)
    {
      IPTServerContext ptSC;
      IPTAdminFolder ptFolder;
      IPTScheduledOperation ptOp;
      // get our session
      IPTSession mySession = (IPTSession) m_asOwner.GetUserSession();
      try
      {
        // can only edit one admin folder at a time so the first
        // element of the object array will be the folder we want 
        ptFolder = (IPTAdminFolder) ((MultiObjectsSecurityModel) m_asOwner.GetModel(SecurityMatrixEditorAS.STR_MVC_STORAGE_NAME_MULTI_SECURITY)).GetObjectArray().GetElement(0);
        // lock the folder for edit								
        ptSC = (IPTServerContext) ptFolder;
        ptSC.LockObject();
        int nAdminFolderJobType;
        if (bPropagateAllACLs)
        {
          nAdminFolderJobType = PT_ADMIN_FOLDER_JOB_TYPES.PT_ADMIN_FOLDER_JOB_PROPAGATEACL_REPLACE;
          } else
        {
          nAdminFolderJobType = PT_ADMIN_FOLDER_JOB_TYPES.PT_ADMIN_FOLDER_JOB_PROPAGATEACL_ADMINSONLY;
          }
        // set the job type
        ptFolder.SetJobType(nAdminFolderJobType);
        // save and unlock
        ptSC.Store();
        ptSC.UnlockObject();
        // create job to propagate security
        ptOp = (IPTScheduledOperation) ptFolder;
        m_nABOJobID = ptOp.ScheduleImmediateJob();
        // get the job's localized name
        IPTJob myTempJob = (IPTJob) mySession.GetObjectManagers(PT_CLASSIDS.PT_JOB_ID).Open(m_nABOJobID,
         false);
        m_sABOJobName = myTempJob.GetLocalizedName();
      }
      catch (Exception ex)
      {
        m_nABOJobID = -1;
        log.Error(m_asOwner.GetString(1701, ResourceConstants.PORTALADMINMSGS) + PT_CLASSIDS.PT_ADMIN_FOLDER_ID);
        // display error
        RaiseError(m_asOwner.GetString(1701, ResourceConstants.PORTALADMINMSGS) + BR_ELEMENT);
        }
      try
      {
        // get the folder that the job was created in's localized name
        IPTAdminFolder myTempAdminFolder = (IPTAdminFolder) mySession.GetAdminCatalog().OpenAdminFolder(PT_INTRINSICS.PT_ADMIN_FOLDER_INTRINSICOPS,
         false);
        m_sABOFolderJobCreatedInName = myTempAdminFolder.GetLocalizedName();
        }
      catch (Exception ex)
      {
        // DCA - could not get folder name, user must not have access.
        m_sABOFolderJobCreatedInName = "";
        log.Error(ex, "SecurityMatrixModel:PropagateSecurityUsingABO Could not get localized name of folder with Id: " + PT_INTRINSICS.PT_ADMIN_FOLDER_INTRINSICOPS);
        }
    }
    
    /// <summary>
    /// Retrieves the id of the job created by ABO.
    /// </summary>
    public virtual int GetABOJobID()
    {
      return m_nABOJobID;
    }
    
    /// <summary>
    /// Retrieves the name of the job created by ABO.
    /// </summary>
    public virtual String GetCreatedJobName()
    {
      return m_sABOJobName;
    }
    
    /// <summary>
    /// Retrieves the name of the folder that ABO created the job in.
    /// </summary>
    public virtual String GetFolderNameForCreatedJobs()
    {
      return m_sABOFolderJobCreatedInName;
    }
    // helper classes
    public virtual void RaiseError(String strError)
    {
      UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
       strError, strError);
      myError.SetActivitySpace(m_asOwner.GetName());
      myError.SetDisplayOnSeparatePage(false);
      myError.SetClearWithActivitySpaceChange(true);
      myError.SetImmediateClear(true);
      UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
      }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.editor.IObjEditorModelRO#GetClassID()
     */
     
    public virtual int GetClassID()
    {
      // XXX Auto-generated method stub
      return 0;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.editor.IObjEditorModelRO#GetObject()
     */
     
    public virtual IPTObject GetObject()
    {
      // XXX Auto-generated method stub
      return null;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.editor.IObjEditorModelRO#GetObjectDescription()
     */
     
    public virtual String GetObjectDescription()
    {
      // XXX Auto-generated method stub
      return null;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.editor.IObjEditorModelRO#GetObjectID()
     */
     
    public virtual int GetObjectID()
    {
      // XXX Auto-generated method stub
      return 0;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.editor.IObjEditorModelRO#GetObjectName()
     */
     
    public virtual String GetObjectName()
    {
      // XXX Auto-generated method stub
      return null;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.editor.IObjEditorModelRO#GetParentFolderPath()
     */
     
    public virtual String GetParentFolderPath()
    {
      // XXX Auto-generated method stub
      return null;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.editor.IObjEditorModelRO#GetPTSession()
     */
     
    public virtual IPTSession GetPTSession()
    {
      // XXX Auto-generated method stub
      return null;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.securitymatrix.ISecurityMatrixModelRO#ABOJobIDToDisplay()
     */
     
    public virtual bool ABOJobIDToDisplay()
    {
      // XXX Auto-generated method stub
      return false;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.securitymatrix.ISecurityMatrixModelRO#GetEditObjectURL(int, int)
     */
     
    public virtual ASURL GetEditObjectURL(int _nClassID, int _nObjectID)
    {
      // XXX Auto-generated method stub
      return null;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.securitymatrix.ISecurityMatrixModelRO#GetNumSubPortals()
     */
     
    public virtual int GetNumSubPortals()
    {
      // XXX Auto-generated method stub
      return 0;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.securitymatrix.ISecurityMatrixModelRO#GetRegularFolderID()
     */
     
    public virtual String GetRegularFolderID()
    {
      // XXX Auto-generated method stub
      return null;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.securitymatrix.ISecurityMatrixModelRO#GetSelectedSubPortalID()
     */
     
    public virtual String GetSelectedSubPortalID()
    {
      // XXX Auto-generated method stub
      return null;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.securitymatrix.ISecurityMatrixModelRO#GetSubPortalID(int)
     */
     
    public virtual String GetSubPortalID(int pos)
    {
      // XXX Auto-generated method stub
      return null;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.securitymatrix.ISecurityMatrixModelRO#GetSubPortalName(int)
     */
     
    public virtual String GetSubPortalName(int pos)
    {
      // XXX Auto-generated method stub
      return null;
    }
  }
}