using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.portaluiinfrastructure.sso
{
  /// <summary>
  /// Interface for classes which integrate Plumtree with SSO products.  Classes
  /// implementing this interface must include a default constructor in order for
  /// Plumtree to instantiate them.
  /// Methods can return <c>null</c> if they are unimplemented, encounter an
  /// error, or have no return value.
  /// Implementations are used in the following manner: first, a single static
  /// instance is instantiated using reflection.  <c>Initialize</c> is called
  /// on this instance, passing in settings from the <c>SSOSettings</c>
  /// section of <c>x_config.xml</c>.  <c>GetSecureHeaders</c>,
  /// <c>GetSecureCookies</c>, and <c>GetSSOProductName</c> are each
  /// called once on the static instance, and the results cached.
  /// Each time a user logs in, <c>GetLoginInfo</c> is called on the static
  /// instance to produce a <c>SSOLoginInfo</c> instance with information
  /// about the current user. <c>GetLoginInfo</c> may be called concurrently
  /// by several threads, so it must be thread safe.
  /// When a user clicks logout, <c>OnLogout</c> is called to end the SSO
  /// session.  Note that this method is not called every time a session ends, for
  /// instance if a user closes the browser window or a session times out.  This
  /// method may also be called concurrently by various threads, so it must also be
  /// thread safe.
  /// </summary>
  /// <author>Mike Jahr</author>
  /// <seealso cref="com.plumtree.portaluiinfrastructure.sso.SSOLoginInfo"></seealso>
  public interface ISSOIntegration
  {
    
    /// <summary>
    /// Initializes the object with settings from the <c>sso.xml</c> config
    /// file.  The map is from <c>String</c> to <c>XPArrayList</c>.
    /// </summary>
    /// <param name="mapSettings">an <c>XPHashtable</c> of settings</param>
    /// <returns>a <c>boolean</c> indicating success</returns>
    bool Initialize(XPHashtable mapSettings);
    
    /// <summary>
    /// Returns a brief description of the SSO product.  This method will only be
    /// called once, and the result cached.
    /// </summary>
    /// <returns>a description</returns>
    String GetSSOProductName();
    
    /// <summary>
    /// Returns the names of secure cookies which should be forwarded to portlets
    /// for authentication.  This method will only be called once, and the result
    /// cached.
    /// </summary>
    /// <returns>a <c>String[]</c> array of cookie names</returns>
    String[] GetSecureCookies();
    
    /// <summary>
    /// Returns an array of header prefixes which should not be forwarded in
    /// requests to portlets.  For example, if "PT_" is in the array, no headers
    /// beginning with "PT_" will be included in HTTP requests to portlets.  This
    /// list is case-sensitive.  This method will only be called once, and the
    /// result cached.
    /// </summary>
    /// <returns>a <c>String[]</c> array of header names</returns>
    String[] GetSecureHeaders();
    
    /// <summary>
    /// Returns login information extracted from the headers of the given
    /// request.  This method is called whenever a user logs in to Plumtree.
    /// This method must be thread safe.
    /// </summary>
    /// <param name="request">an <c>IXPRequest</c> value</param>
    /// <returns>a <c>SSOLoginInfo</c> value</returns>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.sso.SSOLoginInfo"></seealso>
    SSOLoginInfo GetLoginInfo(IXPRequest request);
    
    /// <summary>
    /// Performs any logout actions required by the SSO product.  This can
    /// include clearing cookies or redirecting to another web page that performs
    /// logout actions.  If a redirect is performed, the other web page should
    /// redirect back to <c>sReturnURL</c> after performing the SSO logout.
    /// Note that this function is only called if the user clicks the logout link
    /// in the portal, not if their session times out or they close the browser
    /// window.
    /// This method must be thread safe.
    /// </summary>
    /// <param name="response">an <c>IXPResponse</c> value</param>
    /// <returns><c>true</c> if a redirect is required, <c>false</c> otherwise</returns>
    bool OnLogout(IXPResponse response, String sReturnURL);
  }
}