using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.sso;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.server.impl.core;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portaluiinfrastructure.sso.integrations
{
  /// <summary>
  /// Handles SSO integration with Oracle Application Server Single-Sign On (OSSO)
  /// </summary>
  /// <author>Ravi Tunuguntla</author>
  public class OSSO : ISSOIntegration
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_SSO),
     "com.plumtree.portaluiinfrastructure.sso.integrations.OSSO");
    private static readonly String[] HEADERS = null;
    private static readonly String[] COOKIES = null;
    
    /// <summary>
    /// User distinguished name sent by OSSO in the request header 
    /// </summary>
    private const String USER_DN = "osso-user-dn";
    
    /// <summary>
    /// DefaultAuthSourcePrefix setting in portalconfig.xml 
    /// </summary>
    public const String DEFAULT_AUTH_PREFIX = "DefaultAuthSourcePrefix";
    private const String LOGOUT_URL = "LogoutURL";
    protected String m_sLogoutURL;
    private String m_defaultAuthenticationSourcePrefix = null;
    private bool bRemoteAuthSourceQueried;
    // lock object for flag
    private static Object QUERY_LOCK = new Object();
    /*
     * Creates a new OSSO object.
     */
     
    public OSSO()
    {
      log.Debug("Creating new OSSO Object");
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.sso.ISSOIntegration#GetLoginInfo(com.plumtree.openfoundation.web.IXPRequest)
     */
     
    public virtual SSOLoginInfo GetLoginInfo(IXPRequest request)
    {
      String strUserDN = request.GetHeader(USER_DN);
      String fullLoginName;
      if (null != strUserDN)
      {
        IPTSession session = PTSession.GetGuestReadOnlySession();
        String defaultAuthSrcPrefix = GetSingleRemoteAuthSourceObjectId(session);
        // if there are more than one remote auth source
        if (defaultAuthSrcPrefix == null)
        {
          log.Debug("Found more than one remote auth sources, so checking configuration to get default prefix");
          IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
          ConfigVarPack vpConfig = (ConfigVarPack) application.GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);
          defaultAuthSrcPrefix = vpConfig.GetConfigValueAsString(ConfigConstants.AUTHENTICATION_SECTION,
           DEFAULT_AUTH_PREFIX, ConfigConstants.VALUE_ATTRIBUTE);
          if (defaultAuthSrcPrefix == null || "".Equals(defaultAuthSrcPrefix))
          {
            log.Error("More than one Authentication Sources were found and no default prefix was set in portalconfig.xml (DefaultAuthSourcePrefix setting)");
            }
          if (log.IsDebugEnabled())
          {
            log.Debug("Default Authentication Source prefix configured to " + defaultAuthSrcPrefix);
            }
        } else
        {
          if (log.IsDebugEnabled())
          {
            log.Debug("Only a single Authentication Source with prefix " + defaultAuthSrcPrefix + " was found. ");
            }
        }
        log.Debug("Got " + USER_DN + " from OSSO :" + strUserDN);
        if (XPStringUtility.StartsWith(strUserDN, "cn"))
        {
          // We have to lookup the user id from common name
          String strName = XPStringUtility.Substring(strUserDN, "cn=".Length,
           XPStringUtility.IndexOf(strUserDN, ","));
          log.Debug("Got the user name from " + USER_DN + " as follows " + strName);
          fullLoginName = defaultAuthSrcPrefix + "\\" + strName;
        } else
        if (XPStringUtility.StartsWith(strUserDN, "uid"))
        {
          // We have user ID in LDAP FDN for the user so we don't need to lookup
          String strName = XPStringUtility.Substring(strUserDN, "uid=".Length,
           XPStringUtility.IndexOf(strUserDN, ","));
          fullLoginName = defaultAuthSrcPrefix + "\\" + strName;
          log.Debug(USER_DN + " contains uid and not cn, so directly got name :" + strName);
          } else
        {
          throw new XPParseException("Failed to get uid from " + USER_DN + ":" + strUserDN + ", Since it does not start with \'cn\' or \'uid\'.");
          }
      } else
      {
        throw new XPNullPointerException("Failed to get " + USER_DN + "from header unable to process the request!");
        }
      log.Debug("Successfully returning SSOLoginInfo with full login name: " + fullLoginName);
      return new SSOLoginInfo(fullLoginName);
    }
    private String GetSingleRemoteAuthSourceObjectId(IPTSession session)
    {
      if (bRemoteAuthSourceQueried)
      {
        return m_defaultAuthenticationSourcePrefix;
      } else
      {
        int nPropIDsToQuery = PT_PROPIDS.PT_PROPID_NAME | PT_PROPIDS.PT_PROPID_AUTHSOURCE_PREFIXSTRING | PT_PROPIDS.PT_PROPID_AUTHSOURCE_AUTHTYPE;
        Object[][] ptqf = new Object[3][];
        for (int j2c_index = 0;j2c_index < 3;j2c_index++)
        {
          ptqf[j2c_index] = new Object[1];
        }
        ptqf[0][0] = PT_PROPIDS.PT_PROPID_AUTHSOURCE_AUTHTYPE;
        ptqf[1][0] = PT_FILTEROPS.PT_FILTEROP_EQ;
        ptqf[2][0] = PT_AUTHTYPES.PT_AUTHTYPE_EXTENSIBLE;
        int[][] arrSort = new int[1][];
        for (int j2c_index = 0;j2c_index < 1;j2c_index++)
        {
          arrSort[j2c_index] = new int[2];
        }
        /*PTSAW_SORT_ROW_PROPID*/
        arrSort[0][0] = PT_PROPIDS.PT_PROPID_OBJECTID;
        /*PTSAW_SORT_ROW_DIRECTION*/
        arrSort[0][1] = 1;
        lock (QUERY_LOCK) {
          // check if someone initialized after we checked
          if (bRemoteAuthSourceQueried)
          {
            return m_defaultAuthenticationSourcePrefix;
          }
          IPTObjectManager authSrcMan = session.GetAuthSources();
          IPTQueryResult qr = authSrcMan.Query(nPropIDsToQuery, -1, arrSort,
           0, -1, ptqf);
          if (qr.RowCount() == 1)
          {
            m_defaultAuthenticationSourcePrefix = qr.ItemAsString(0, PT_PROPIDS.PT_PROPID_AUTHSOURCE_PREFIXSTRING);
            }
          bRemoteAuthSourceQueried = true;
        }
        // end of synchronized block
        return m_defaultAuthenticationSourcePrefix;
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.sso.ISSOIntegration#GetSSOProductName()
     */
     
    public virtual String GetSSOProductName()
    {
      return "OSSO";
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.sso.ISSOIntegration#GetSecureCookies()
     */
     
    public virtual String[] GetSecureCookies()
    {
      return COOKIES;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.sso.ISSOIntegration#GetSecureHeaders()
     */
     
    public virtual String[] GetSecureHeaders()
    {
      return HEADERS;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.sso.ISSOIntegration#Initialize(com.plumtree.openfoundation.util.XPHashtable)
     */
     
    public virtual bool Initialize(XPHashtable mapSettings)
    {
      XPArrayList list = (XPArrayList) mapSettings.GetElement(LOGOUT_URL);
      if (list != null && list.GetSize() > 0)
      {
        m_sLogoutURL = (String) list.GetElement(0);
      }
      log.Info("OSSO Setting LogoutURL: " + m_sLogoutURL);
      return true;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.sso.ISSOIntegration#OnLogout(com.plumtree.openfoundation.web.IXPResponse, java.lang.String)
     */
     
    public virtual bool OnLogout(IXPResponse response, String sReturnURL)
    {
      log.Debug("WCI Return URL from the Input Arguments: " + sReturnURL);
      if (null != m_sLogoutURL && m_sLogoutURL.Length > 0)
      {
        String sURL = m_sLogoutURL;
        // Set the return URL
        log.Info("OSSO Adding Return URl \'Osso-Return-Url\' to header " + sReturnURL);
        response.AddHeader("Osso-Return-Url", sReturnURL);
        // Send Dynamic Directive for logout
        log.Info("OSSO Setting the dynamic directive for Logout: " + 470);
        response.SetStatus(470, "Oracle SSO");
        LoginHandlers.ClearLoginOccurredCookiePresent(response);
        log.Debug("OSSO is setting ptloginoccurred cookie to false");
        if (XPStringUtility.EndsWith(m_sLogoutURL, "="))
        {
          sURL+=XPHttpUtility.UrlEncode(sReturnURL, "UTF-8");
        }
        response.Redirect(sURL);
        log.Info("OSSO is redirecting to " + sURL);
        return true;
      }
      return false;
    }
  }
}