using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.classtypedescriptors;
using com.plumtree.portaluiinfrastructure.friendlyurl;
using com.plumtree.portaluiinfrastructure.opener;
using com.plumtree.server;
using com.plumtree.server.impl.core;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.application;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.helpers;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portaluiinfrastructure.statichelpers
{
  /// <summary>
  /// <p />This class provides a set of static helpers for constucting valid gateway URLs.
  /// Gateway URLs follow a well-defined format to communicate information about the request
  /// to the gateway control. This section describes the format of the gateway URLs. For simplicity
  /// portal url is assumed to be http://machinename/portal/server.pt
  /// <p />The gateway URLs are identified by a special substring "gateway/PTARGS_" that should follow
  /// the portal URL. So, a general gateway URL looks like:
  /// <br /><br />
  /// <B>&nbsp;&nbsp;&nbsp;http://machinename/portal/server.pt/gateway/PTARGS_1_2_3_4_5_6/http://remotemachine/url_to_gateway/somefile.html</B>
  /// <br /><br />
  /// where 1,2,3,4,5,6 are numeric arguments described below.
  /// <OL>
  /// <LI>The first argument in an integer that contains a bitmask of gateway flags and type of associated gadget content
  /// that is being requested.
  /// <UL>
  /// <LI>
  /// <B>POPUPFLAG</B>\u0009-\u0009notifies the gateway that requested page will be displayed in the popup window and therefore
  /// gateway should set the RETURN-URI to "close window" javascript.</LI>
  /// <LI>
  /// <B>RETURNFLAG</B>  - \u0009notifies the gateway that return URI is requested. The gateway will check that POPUPFLAG and
  /// either redirect to the last visited community or mypage or close the window.</LI>
  /// <LI>
  /// <B>REDIRECTGUEST</B> -\u0009notifies the gateway that it should redirect guest users to the login page before displaying
  /// the request resource. (This is mostly used by Collaboration server)</LI>
  /// </UL>
  /// </LI>
  /// <LI>The second argument is a numeric id of the user requesting the page. This argument is not used by the gateway
  /// and is intended for PPC PTTracker tool that analyzes the logs from the webserver. If user id is not known you can
  /// set it to any value, but 0 is prefered.
  /// </LI>
  /// <LI>The third argument is a numeric id of the object that is being requested. For example, if you are requesting
  /// a page through gadget 201, then this argument should be set to 201.
  /// </LI>
  /// <LI>The fourth argument is a community id.</LI>
  /// <LI>The fifth arguemnt is a page id.</LI>
  /// <LI>The sixth argument is a resource type id. It defines what kind of object is identified by the object id provided
  /// in the third argument. Refer to Plumtree.idl to see what are numeric values for various resource types. Gateway supports
  /// the following resource types: Gadget, WebService, Card, DataSource.</LI>
  /// </OL>
  /// </summary>
  /// <author>daniilk</author>
  public class GatewayHelpers
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_GATEWAY),
     "com.plumtree.portaluiinfrastructure.statichelpers.GatewayHelpers");
    public const String SPECIAL_HEADER_NAME = "PTParams";
    public const String GATEWAY_URL_PREFIX = "/gateway/PTARGS_";
    
    /// <summary>
    /// this variable is set in the static intializer for this class *
    /// </summary>
    public static String GATEWAY_FRIENDLYURL_KEYWORD = null;
    public const String GATEWAY_ARG_DELIMITER = "_";
    public const String PORTLET_SAVED_ON_SESSION = "PortletBeingEdited";
    public const String LAST_VISITED_HOSTED_PAGE = "LastVisitedHostGatewayPage";
    public const String GATEWAY_HOSTED_MODE_HEADER = "csp-display-mode";
    public const String GATEWAY_SPACE_NAME = "Gateway";
    public const int NOFLAGS = 0;
    public const int POPUPFLAG = 8;
    public const int RETURNFLAG = 16;
    public const int REDIRECTGUEST = 32;
    public const int REFRESHFLAG = 64;
    public static readonly int POPUPREFRESHFLAG = REFRESHFLAG | POPUPFLAG;
    public static readonly int ADMINPREFS_POPUP = (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_ADMINPREFSPAGE + 1) | POPUPFLAG;
    public static readonly int ADMINPREFS = PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_ADMINPREFSPAGE + 1;
    public static readonly int GADGETPREFS = (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_PREFSPAGE + 1);
    public static readonly int GADGETPREFS_POPUP = (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_PREFSPAGE + 1) | POPUPREFRESHFLAG;
    public static readonly int COMMUNITYPREFS = (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_COMMUNITYPREFSPAGE + 1);
    public static readonly int COMMUNITYPREFS_POPUP = (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_COMMUNITYPREFSPAGE + 1) | POPUPFLAG;
    public static readonly int HELP = (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_HELPPAGE + 1);
    public static readonly int HELP_POPUP = (PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_HELPPAGE + 1) | POPUPFLAG;
    public static readonly int USERPLUGPAGE = PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_PERSONAL_PLUG + 1;
    public static readonly int ADMINPLUGPAGE = PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_ADMIN_PLUG + 1;
    public static readonly int PORTLETCONTENT = PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_PORTLETCONTENT + 1;
    public const String PT_REQUEST_TYPE_HEADER = "PT-HTTPRequest-Type";
    public const String PT_CLIENT_SIDE_HEADER_VALUE = "CLIENT_SIDE";
    public const String PT_PORTLET_REFRESH_HEADER_VALUE = "PORTLET_REFRESH";
    public const String PT_REFERER_HEADER = "Referer";
    public const String CLOSEPOPUPJAVASCRIPT = "<HTML><BODY onload=\"window.close();\"></BODY></HTML>";
    public const String CLOSEPOPUPANDREFRESHJAVASCRIPT = "<HTML><BODY onload=\"parent.opener.location.reload();window.close();\"></BODY></HTML>";
    public const String UI_STD = "";
    public const String UI_508 = "http://www.plumtree.com/uis/section508";
    public static readonly String UI_LOW = UI_508;
    public const String UI_WML = "http://www.plumtree.com/uis/wml";
    public const String UI_PDA = "http://www.plumtree.com/uis/pda";
    public const String DEV_SUFFIX_PDA = "_HTML32";
    public const String DEV_SUFFIX_WML = "_WML";
    public const String DEFAULT_FRIENDLYGATEWAYURL_KEYWORD = "gw";
    public const String URL_SEPARATOR = "/";
    public static readonly String GWARG_PORTLET_PREFIX = Global.GWARG_PORTLET_PREFIX;
    public static readonly String GWARG_WEBSRV_PREFIX = Global.GWARG_WEBSRV_PREFIX;
    public static readonly String GWARG_COMMUNITY_PREFIX = Global.GWARG_COMMUNITY_PREFIX;
    public static readonly String GWARG_PAGE_PREFIX = Global.GWARG_PAGE_PREFIX;
    public static readonly String GWARG_PREFTYPE_PREFIX = Global.GWARG_PREFTYPE_PREFIX;
    public static readonly String GWARG_USER_PREFIX = Global.GWARG_USER_PREFIX;
    public const String OPENCONFIG_GATEWAY_COMPONENT = "Gateway";
    public const String OPENCONFIG_ALLOW_EXTGWREDIRECTS = "AllowExternalGatewayRedirect";
    static GatewayHelpers()
    {
      // This code block allows us to tell the Interpreter not to parse the POST body of gateway
      // requests.  Otherwise we won't be able to access the Input Stream.
      // lookup gateway friendly url keyword
      IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
      IFriendlyURLManager ptFriendlyURLMgr = (IFriendlyURLManager) application.GetCachingManager().GetEntry(AppWarmUp.KEY_CACHEMAN_CLASS_TYPE_MANAGER);
      GATEWAY_FRIENDLYURL_KEYWORD = ptFriendlyURLMgr.LookupFriendlyURLKeyFromClassId(FriendlyURLConstants.FRIENDLY_GATEWAY_URL);
      // if gateway friendly url is enabled
      if (GATEWAY_FRIENDLYURL_KEYWORD != null)
      {
        com.plumtree.uiinfrastructure.interpreter.Interpreter.AddNonFormDataRequestPattern(URL_SEPARATOR + GATEWAY_FRIENDLYURL_KEYWORD + URL_SEPARATOR);
        }
      // always add the regular gateway pattern, it is always supported even if friendly gateway urls are enabled
      com.plumtree.uiinfrastructure.interpreter.Interpreter.AddNonFormDataRequestPattern(GATEWAY_URL_PREFIX);
      }
    
    /// <summary>
    /// This method is used to determine the current UI mode of the portal. UI mode depends both
    /// on the user setting (Standard, 508, Low Bandwidth) and type of browser (PDA, Phone or full-featured
    /// browser like IE or Mozilla). 
    /// </summary>
    /// <param name="xpRequest">- current request object</param>
    /// <param name="asOwner">- current activity space</param>
    /// <returns>This method returns one of the following string constants: UI_STD, UI_508, UI_LOW, UI_WML, UI_PDA</returns>
    public static String GetUIMode(IXPRequest xpRequest, AActivitySpace asOwner)
    {
      try
      {
        int iStyleID = asOwner.GetAccessStyle().GetAccessID();
        if (iStyleID == AccessStyles.SECTION_508.GetAccessID())
        {
          return UI_508;
        } else
        if (iStyleID == AccessStyles.LOW_BANDWIDTH.GetAccessID())
        {
          return UI_LOW;
        } else
        {
          String strAccepts = xpRequest.GetHeader("Accept");
          String strUserHeader = xpRequest.GetHeader("User-Agent");
          DevicesVarPack vpPTD = (DevicesVarPack) asOwner.GetApplication().GetVarPackManager().GetVariablePackage(DevicesVarPack.VARPACK_ID);
          if (vpPTD != null)
          {
            String sDevSuffix = vpPTD.GetDevice(strAccepts, strUserHeader);
            if (sDevSuffix != null && DEV_SUFFIX_PDA.CompareTo(sDevSuffix) == 0)
            {
              return UI_PDA;
            } else
            if (sDevSuffix != null && DEV_SUFFIX_WML.CompareTo(sDevSuffix) == 0)
            {
              return UI_WML;
            }
          }
        }
      }
      catch (Exception e)
      {
        log.Warn(e, "Exception trying to get AccessStyle.");
      }
      return UI_STD;
    }
    
    /// <summary>
    /// This function returns absolute URL to the admin prefs page.
    /// </summary>
    /// <param name="bPopup">
    /// -     boolean flag to indicate if the admin page will be displayed
    /// in a separate popup window or not. All this flag does is tells
    /// gateway to set Return-URI action to "close window".
    /// </param>
    public static String ConstructAdminPrefsLink(AActivitySpace asOwner, bool bPopup)
    {
      return ConstructPrefPageLink(asOwner, 0, 0, 0, bPopup ? ADMINPREFS_POPUP : ADMINPREFS);
      }
    
    /// <summary>
    /// This function returns absolute URL to the admin prefs page.
    /// </summary>
    /// <param name="bPopup">
    /// -     boolean flag to indicate if the admin page will be displayed
    /// in a separate popup window or not. All this flag does is tells
    /// gateway to set Return-URI action to "close window".
    /// </param>
    public static String ConstructTemplateAdminPrefsLink(AActivitySpace asOwner,
     bool bPopup)
    {
      return ConstructPrefPageLink(asOwner, 0, 0, 0, bPopup ? ADMINPREFS_POPUP : ADMINPREFS,
       PT_CLASSIDS.PT_GADGETTEMPLATE_ID);
    }
    
    /// <summary>
    /// This function returns absolute URL to the admin prefs page and treats it as popup.
    /// </summary>
    public static String ConstructAdminPrefsLink(AActivitySpace asOwner)
    {
      return ConstructPrefPageLink(asOwner, 0, 0, 0, ADMINPREFS_POPUP);
    }
    
    /// <summary>
    /// This function returns absolute URL to the community prefs page.
    /// </summary>
    /// <param name="iPortletID">- ID of the portlet for which the gadget community preferences are set</param>
    /// <param name="iCommunityID">- ID of the community for which the gadget community preferences are set</param>
    /// <param name="bPopup">
    /// -     boolean flag to indicate if the pref page will be displayed
    /// in a separate popup window or not. All this flag does is tells
    /// gateway to set Return-URI action to "close window".
    /// </param>
    public static String ConstructCommunityPrefsLink(AActivitySpace asOwner,
     int iPortletID, int iCommunityID, bool bPopup)
    {
      return ConstructPrefPageLink(asOwner, iPortletID, iCommunityID, 0, bPopup ? COMMUNITYPREFS_POPUP : COMMUNITYPREFS);
      }
    
    /// <summary>
    /// This function returns absolute URL to the community prefs page.
    /// </summary>
    /// <param name="iPortletID">- ID of the portlet for which the gadget community preferences are set</param>
    /// <param name="iCommunityID">- ID of the community for which the gadget community preferences are set</param>
    /// <param name="iPageID">- ID of the community page which the gadget community preferences are set</param>
    /// <param name="bPopup">
    /// -     boolean flag to indicate if the pref page will be displayed
    /// in a separate popup window or not. All this flag does is tells
    /// gateway to set Return-URI action to "close window".
    /// </param>
    public static String ConstructCommunityPrefsLink(AActivitySpace asOwner,
     int iPortletID, int iCommunityID, int iPageID, bool bPopup)
    {
      return ConstructPrefPageLink(asOwner, iPortletID, iCommunityID, iPageID,
       bPopup ? COMMUNITYPREFS_POPUP : COMMUNITYPREFS);
    }
    
    /// <summary>
    /// This function returns absolute URL to the gadget prefs page.
    /// </summary>
    /// <param name="iPortletID">- ID of the portlet for which the gadget preferences are set</param>
    /// <param name="bPopup">
    /// -     boolean flag to indicate if the pref page will be displayed
    /// in a separate popup window or not. All this flag does is tells
    /// gateway to set Return-URI action to "close window".
    /// </param>
    public static String ConstructPortletPrefsLink(AActivitySpace asOwner,
     int iPortletID, bool bPopup)
    {
      return ConstructPrefPageLink(asOwner, iPortletID, 0, 0, bPopup ? GADGETPREFS_POPUP : GADGETPREFS);
      }
    
    /// <summary>
    /// This function returns absolute URL to the gadget prefs page.
    /// </summary>
    /// <param name="iPortletID">- ID of the portlet for which the gadget preferences are set</param>
    /// <param name="iPageID">- ID of the community page which the gadget preferences are set</param>
    /// <param name="bPopup">
    /// -     boolean flag to indicate if the pref page will be displayed
    /// in a separate popup window or not. All this flag does is tells
    /// gateway to set Return-URI action to "close window".
    /// </param>
    public static String ConstructPortletPrefsLink(AActivitySpace asOwner,
     int iPortletID, int iPageID, bool bPopup)
    {
      return ConstructPrefPageLink(asOwner, iPortletID, 0, iPageID, bPopup ? GADGETPREFS_POPUP : GADGETPREFS);
      }
    
    /// <summary>
    /// This function returns absolute URL to the community user prefs page.
    /// </summary>
    /// <param name="iPortletID">- ID of the portlet for which the gadget community user preferences are set</param>
    /// <param name="iCommunityID">- ID of the community for which the gadget community user preferences are set</param>
    /// <param name="iPageID">- ID of the community page which the gadget community user preferences are set</param>
    /// <param name="bPopup">
    /// -     boolean flag to indicate if the prefs page will be displayed
    /// in a separate popup window or not. All this flag does is tells
    /// gateway to set Return-URI action to "close window".
    /// </param>
    public static String ConstructPortletUserCommunityPrefsLink(AActivitySpace asOwner,
     int iPortletID, int iCommunityID, bool bPopup)
    {
      return ConstructPrefPageLink(asOwner, iPortletID, iCommunityID, 0, bPopup ? GADGETPREFS_POPUP : GADGETPREFS);
      }
    
    /// <summary>
    /// This function returns absolute URL to the help page.
    /// </summary>
    /// <param name="iPortletID">- ID of the portlet for which the help should be displayed</param>
    /// <param name="iCommunityID">- ID of the community for which the help should be displayed</param>
    /// <param name="iPageID">- ID of the community page which the help should be displayed</param>
    /// <param name="bPopup">
    /// -     boolean flag to indicate if the help page will be displayed
    /// in a separate popup window or not. All this flag does is tells
    /// gateway to set Return-URI action to "close window".
    /// </param>
    public static String ConstructHelpLink(AActivitySpace asOwner, int iPortletID,
     int iCommunityID, int iPageID, bool bPopup)
    {
      return ConstructPrefPageLink(asOwner, iPortletID, iCommunityID, iPageID,
       bPopup ? HELP_POPUP : HELP);
    }
    
    /// <summary>
    /// This method constructs gateway URL that points to a specific associated resource (preference page, help page).
    /// If this method is called with iPrefType parameter set to 0, the result is a gateway prefix that should be 
    /// followed by the actual absolute URL to the gatewayed resource.
    /// </summary>
    /// <param name="asOwner">- current activity space object</param>
    /// <param name="iPortletID">- portlet id</param>
    /// <param name="iCommunityID">- community ID (0 if not applicable)</param>
    /// <param name="iPageID">- page ID (0 if not applicable)</param>
    /// <param name="iPrefType">- preference type</param>
    /// <returns>		string that is a gateway URL.</returns>
    public static String ConstructPrefPageLink(AActivitySpace asOwner, int iPortletID,
     int iCommunityID, int iPageID, int iPrefType)
    {
      return ConstructPrefPageLink(asOwner, iPortletID, iCommunityID, iPageID,
       iPrefType, PT_CLASSIDS.PT_GADGET_ID);
    }
    
    /// <summary>
    /// This method constructs gateway URL that points to a specific associated resource (preference page, help page).
    /// If this method is called with iPrefType parameter set to 0, the result is a gateway prefix that should be 
    /// followed by the actual absolute URL to the gatewayed resource.
    /// </summary>
    /// <param name="asOwner">- current activity space object</param>
    /// <param name="iPortletID">- portlet or web service id</param>
    /// <param name="iCommunityID">- community ID (0 if not applicable)</param>
    /// <param name="iPageID">- page ID (0 if not applicable)</param>
    /// <param name="iPrefType">- preference type</param>
    /// <param name="iClassID">- class id that describles iPortletID parameter.</param>
    /// <returns>		string that is a gateway URL.</returns>
    public static String ConstructPrefPageLink(AActivitySpace asOwner, int iPortletID,
     int iCommunityID, int iPageID, int iPrefType, int iClassID)
    {
      return ConstructGatewayURL(asOwner.GetCurrentRequestURL(), iPortletID,
       iCommunityID, iPageID, iPrefType, iClassID, "");
    }
    
    /// <summary>
    /// This method generates a gateway URL pointing to a resource specified by strURLToGateway parameter.
    /// </summary>
    /// <param name="strCurrentRequestURL">
    /// - the URL of the current request 
    /// (normally, the result of AActivitySpace.GetCurrentRequestURL())
    /// </param>
    /// <param name="iPortletID">- numeric ID of the portlet object or webservice object</param>
    /// <param name="iCommunityID">- community ID (0 if not applicable)</param>
    /// <param name="iPageID">- page ID (0 if not applicable)</param>
    /// <param name="iPrefType">- preference type. For normal portlet click-throughs it should be 0.</param>
    /// <param name="iClassID">
    /// - class ID. This arguments tells whether second argument of this method is 
    /// portlet ID or webservice ID.
    /// </param>
    /// <param name="strURLToGateway">- absolute URL to the resource that should be gatewayed (i.e. http://www.google.com)</param>
    /// <returns>						- a gateway URL string that points to the resource specified.</returns>
    public static String ConstructGatewayURL(String strCurrentRequestURL,
     int iPortletID, int iCommunityID, int iPageID, int iPrefType, int iClassID,
     String strURLToGateway)
    {
      return ConstructGatewayURL(strCurrentRequestURL, iPortletID, iCommunityID,
       iPageID, iPrefType, iClassID, 0, strURLToGateway);
    }
    
    /// <summary>
    /// This method generates a gateway URL pointing to a resource specified by strURLToGateway parameter.
    /// </summary>
    /// <param name="curReqUrl">- the URL of the current request (normally, the result of AActivitySpace.GetCurrentRequestURL())</param>
    /// <param name="objId">- numeric ID of the portlet object or webservice object</param>
    /// <param name="commId">- community ID (0 if not applicable)</param>
    /// <param name="pageId">- page ID (0 if not applicable)</param>
    /// <param name="prefType">- preference type. For normal portlet click-throughs it should be 0.</param>
    /// <param name="classId">- This arguments tells whether second argument of this method is portlet ID or webservice ID.</param>
    /// <param name="userId">- user id, or 0 not to use one</param>
    /// <param name="gatewayPath">- absolute URL to the resource that should be gatewayed (i.e. http://www.google.com)</param>
    /// <returns>			- a gateway URL string that points to the resource specified.</returns>
    public static String ConstructGatewayURL(String curReqUrl, int objId,
     int commId, int pageId, int prefType, int classId, int userId, String gatewayPath)
    {
      int iSecurityMode = PTConfigHelper.GetSecurityMode();
      bool bSecureOnlyMode = iSecurityMode > 1;
      String strAppBaseURL = "";
      /* if we got a valid request URL */
      if (null != curReqUrl && curReqUrl.Length > 0)
      {
        strAppBaseURL = ConfigService.getInstance().GetApplicationBaseURL(ConfigHelper.GetNormalizedBaseURL(curReqUrl),
         bSecureOnlyMode || XPStringUtility.StartsWith(curReqUrl, ConfigHelper.HTTPS_SCHEME));
        } else
      /* it was messed up */
      {
        strAppBaseURL = ConfigHelper.GetMainServerApplicationURL(false);
      }
      /* GetMainServerApplicationURL returns URL with "?" appended. Since it is too late
       * to change it and bugging Julie is not part of my plans for the nearest future i
       * will just fix the URL here. In case someone will ever decide to fix it, we will
       * take an intelligent approach and search for "?" first, instead of blindly removing
       * the last character.
       * 		-- Moose
       */
       
      if (null != strAppBaseURL)
      {
        int iQuestionMarkLocation = XPStringUtility.IndexOf(strAppBaseURL,
         "?");
        if (-1 != iQuestionMarkLocation)
        {
          strAppBaseURL = XPStringUtility.Substring(strAppBaseURL, 0, iQuestionMarkLocation);
          }
      }
      String frGWkeyword = GetFriendlyGatewayURLKeyword();
      if (null == frGWkeyword)
      {
        return ConstructRegularGatewayURL(strAppBaseURL, objId, commId, pageId,
         prefType, classId, userId, gatewayPath);
      } else
      {
        return ConstructFriendlyGatewayURL(strAppBaseURL, objId, commId, pageId,
         prefType, classId, userId, gatewayPath);
      }
    }
    
    /// <summary>
    /// This method generates a gateway URL pointing to a resource specified by strURLToGateway parameter.
    /// </summary>
    /// <param name="curReqUrl">- the URL of the current request (normally, the result of AActivitySpace.GetCurrentRequestURL())</param>
    /// <param name="objId">- numeric ID of the portlet object or webservice object</param>
    /// <param name="commId">- community ID (0 if not applicable)</param>
    /// <param name="pageId">- page ID (0 if not applicable)</param>
    /// <param name="prefType">- preference type. For normal portlet click-throughs it should be 0.</param>
    /// <param name="classId">- This arguments tells whether second argument of this method is portlet ID or webservice ID.</param>
    /// <param name="userId">- user id, or 0 not to use one</param>
    /// <param name="gatewayPath">- absolute URL to the resource that should be gatewayed (i.e. http://www.google.com)</param>
    /// <returns>			- a gateway URL string that points to the resource specified.</returns>
    public static String ConstructRegularGatewayURL(String strAppBaseURL,
     int objId, int commId, int pageId, int prefType, int classId, int userId,
     String gatewayPath)
    {
      return strAppBaseURL + GATEWAY_URL_PREFIX + prefType + "_" + userId + "_" + objId + "_" + commId + "_" + pageId + "_" + classId + "/" + CreateGatewayFriendlyURL(gatewayPath);
      }
    
    /// <returns>null if friendly gateway urls is disabled</returns>
    public static String GetFriendlyGatewayURLKeyword()
    {
      return GATEWAY_FRIENDLYURL_KEYWORD;
    }
    
    /// <summary>
    /// This method generates a gateway URL pointing to a resource specified by strURLToGateway parameter.
    /// </summary>
    /// <param name="appBaseUrl-"></param>
    /// <param name="objId">- numeric ID of the portlet object or webservice object</param>
    /// <param name="commId">- community ID (0 if not applicable)</param>
    /// <param name="pageId">- page ID (0 if not applicable)</param>
    /// <param name="prefType">- preference type. For normal portlet click-throughs it should be 0.</param>
    /// <param name="classId">- This arguments tells whether second argument of this method is portlet ID or webservice ID.</param>
    /// <param name="userId">- user id, or 0 not to use one</param>
    /// <param name="gatewayPath">- absolute URL to the resource that should be gatewayed (i.e. http://www.google.com)</param>
    /// <returns>			- a gateway URL string that points to the resource specified.</returns>
    public static String ConstructFriendlyGatewayURL(String appBaseUrl, int objId,
     int commId, int pageId, int prefType, int classId, int userId, String gatewayPath)
    {
      return Global.ConstructFriendlyGatewayURI(GetFriendlyGatewayURLKeyword(),
       appBaseUrl, objId, commId, pageId, prefType, classId, userId, gatewayPath);
      }
    
    /// <summary>
    /// Used by AppDataObject to creates a partial gateway url with just the preference type and user id to send
    /// to the server, which will complete out the second half of the gateway url. 
    /// </summary>
    /// <returns></returns>
    public static String ConstructPartialGatewayURL(String appBaseUrl, int prefType,
     int userId)
    {
      return appBaseUrl + GATEWAY_URL_PREFIX + prefType + "_" + userId + "_";
      }
    
    /// <summary>
    /// Used by AppDataObject to creates a partial gateway url with just the preference type and user id to send
    /// to the server, which will complete out the second half of the gateway url.
    /// This method creates a friendly gateway url
    /// </summary>
    /// <param name="appBaseUrl"></param>
    /// <param name="prefType"></param>
    /// <param name="userId"></param>
    /// <returns>null if friendly gateway url is disabled</returns>
    public static String ConstructPartialFriendlyGatewayURLKeyword(String appBaseUrl,
     int prefType, int userId)
    {
      String frGWkw = GetFriendlyGatewayURLKeyword();
      if (null != frGWkw)
      {
        return appBaseUrl + URL_SEPARATOR + frGWkw + URL_SEPARATOR + 
        // only add in arguments if they have values
        ((prefType == 0) ? "" : GWARG_PREFTYPE_PREFIX + prefType) + 
        // add a separator if both arguments have values
        ((prefType > 0 && userId > 0) ? "_" : "") + ((userId == 0) ? "" : GWARG_USER_PREFIX + userId);
        }
      return null;
    }
    
    /// <summary>
    /// procedure:\u0009ConstructReturnURI
    /// purpose: 
    /// </summary>
    /// <param name="asOwner"></param>
    /// <param name="iPageID"></param>
    /// <param name="iCommunityID"></param>
    /// <param name="bPopup"></param>
    /// <param name="bRefreshPopup"></param>
    /// <returns></returns>
    public static String ConstructReturnURI(AActivitySpace asOwner, int iPageID,
     int iCommunityID, bool bPopup, bool bRefreshPopup)
    {
      // SE: Bug 55717 - Fix for Unicredit. Backport of DavidP's fix (CL 238479) to get rid of extra 302 redirect
      // in cases where a GET or POST is done on a portlet in normal view (Non-popups and non-gatewayed pages).
      // DAP: This redirect refers to a portal page *only*.
      String retURL = "";
      // If we are not redirecting to a MyPage or Community Page, then
      // we need to redirect the "old way" using ConstructPrefPageLink.
      // DLH Updated fix for bug #58040: use the "old way" 302 redirect for 
      // utilities that return to admin UI.
      bool bIsPTPageRedirect = ((iCommunityID != 0) || (iPageID != 0));
      // DAP: For popups, we do require a (second) 302 redirect processed 
      // by the gateway (this is "old way") as a page request
      if (bPopup || bRefreshPopup || !bIsPTPageRedirect)
      {
        int iFlags = RETURNFLAG;
        iFlags|=bPopup ? POPUPFLAG : 0;
        iFlags|=bRefreshPopup ? POPUPREFRESHFLAG : 0;
        retURL = ConstructPrefPageLink(asOwner, 0, iCommunityID, iPageID,
         iFlags);
        log.Debug("GatewayHelpers.ConstructReturnURI - Constructing return URL for popup: [" + retURL + "]");
        } else
      // DAP: Normal flat view mode refresh can use opener directly to avoid excess 302 redirect
      {
        if (iCommunityID > 0)
        {
          if (iPageID == 0)
          {
            // If we have a community, but not page ID, then try to open the community itself.
            retURL = PTOpenerLinks.GetOpenerURLOpenObjID(PT_CLASSIDS.PT_COMMUNITY_ID,
             iCommunityID, IPTOpenerConstants.OPENER_MODE_VIEW, asOwner);
            } else
          {
            retURL = PTOpenerLinks.GetOpenerURLOpenObjID(PT_CLASSIDS.PT_PAGE_ID,
             iPageID, IPTOpenerConstants.OPENER_MODE_VIEW, asOwner);
          }
          log.Debug("GatewayHelpers.ConstructReturnURI - Constructing return URL for Community Page: [" + retURL + "]");
          } else
        {
          retURL = PTOpenerLinks.GetOpenerURLOpenObjID(PT_CLASSIDS.PT_MYPAGESCONTAINER_ID,
           iPageID, IPTOpenerConstants.OPENER_MODE_VIEW, asOwner);
          log.Debug("GatewayHelpers.ConstructReturnURI - Constructing return URL for normal view page: [" + retURL + "]");
          }
      }
      return retURL;
    }
    
    /// <summary>
    /// Helper function to encode
    /// </summary>
    /// <param name="strURL"></param>
    /// <returns></returns>
    public static String CreateGatewayFriendlyURL(String strURL)
    {
      int iPos = XPStringUtility.IndexOf(strURL, '?');
      String strResult = "";
      // NOTE: I'm not sure why this doesn't use the Global.PTGatewayEncode method.
      // Converting pre-existing ;'s to their % encoded form (%3b) seems problematic,
      // because the decode method in globals converts all %3Bs into ;'s, which means they
      // could be converted back to colons incorrectly.  However, this method also encodes
      // the % as %25 in the following line, leaving you with %253b, which may allow
      // this to work.  This code really needs a comment explaining why it's doing this.
      if (-1 == iPos)
      {
        strResult = GeneralHelpers.Replace(strURL, ";", "%3B");
        strResult = GeneralHelpers.Replace(strResult, "%", "%25");
        strResult = GeneralHelpers.Replace(strResult, "://", ";/");
        strResult = GeneralHelpers.Replace(strResult, ":", ";");
        return strResult;
      } else
      {
        String strURLPath = XPStringUtility.Substring(strURL, 0, iPos);
        strResult = GeneralHelpers.Replace(strURLPath, ";", "%3B");
        strResult = GeneralHelpers.Replace(strResult, "%", "%25");
        strResult = GeneralHelpers.Replace(strResult, "://", ";/");
        strResult = GeneralHelpers.Replace(strResult, ":", ";");
        return strResult + XPStringUtility.Substring(strURL, iPos);
      }
    }
    
    /// <summary>
    /// Converts a friendly gateway url to regular.
    /// Legend: {} = required, [] = optional, | = or, <> = value
    /// Friendly format: /gw/{pt|ws<obj id>}[_p<page id>][_c<community id>][_u<user id>][_r<preference type>]/[<gateway path>]
    /// Regular format:  /gateway/PTARGS_<preference type>_<user id>_<object id>_<community id>_<page id>_<object type>/[<gateway path>]
    /// </summary>
    /// <param name="friendlyUrl"></param>
    /// <param name="keyword"></param>
    /// <returns></returns>
    public static String ConvertToRegularGatewayURL(String friendlyUrl, String keyword)
    {
      // First we need to separate the server base url, gateway keyword, gateway arguments and gateway path
      // Then, parse the gateway arguments
      // and finally construct the gateway url 
      // Tokenize the string based 
      XPStringTokenizer urlSeg = new XPStringTokenizer(friendlyUrl, URL_SEPARATOR,
       true);
      // This string buffer for the server name of the url, e.g. http://myserver:8080
      XPStringBuilder baseUrl = new XPStringBuilder();
      // To store the token with the gateway arguments, e.g. /ws3420_c2340_p22342_u2342/
      String argsSeg = null;
      // This string buffer for the portlet/webservice path e.g. /path1/path2/file1
      XPStringBuilder gwPathSeg = new XPStringBuilder();
      // flag to signal if we got to the friendly gateway keyword, -1 for before, 0 for at, and 1 for after
      int postKeyUrlSegment = -1;
      // Iterate through the tokens. Add tokens to the base url buffer if we haven't encountered the keyword,
      // and add tokens to the path segment buffer if we already encountered the keyword
      while (urlSeg.HasMoreTokens())
      {
        String cur = urlSeg.GetNextToken();
        if (postKeyUrlSegment == -1)
        {
          if (keyword.Equals(cur))
          {
            // found the keyword, skip it and handle the arguments the next iteration
            postKeyUrlSegment = 0;
            // remove the last slash before the gateway keyword
            baseUrl.RemoveCharAt(baseUrl.GetLength() - 1);
          } else
          // don't do anything with the key token
          // any token before the keyword is part of the baseurl
          {
            baseUrl.Append(cur);
          }
        } else
        if (URL_SEPARATOR.Equals(cur))
        {
          // after the base url, ignore the slashes
          continue;
        } else
        if (postKeyUrlSegment == 0)
        {
          // keep the argument token to parse later
          argsSeg = cur;
          postKeyUrlSegment++;
        } else
        if (postKeyUrlSegment >= 1)
        {
          // there is only the path after the keyword and arguments 
          if (postKeyUrlSegment != 1)
          {
            gwPathSeg.Append(URL_SEPARATOR);
          }
          gwPathSeg.Append(cur);
          postKeyUrlSegment++;
        }
      }
      // parse the args, in this format: {pt|ws<obj id>}[_p<page id>][_c<community id>][_u<user id>][_r<preference type>]/[<gateway path>]
      XPStringTokenizer args = new XPStringTokenizer(argsSeg, "_");
      int objectType = 0;
      int objectId = 0;
      int commId = 0;
      int pageId = 0;
      int prefType = 0;
      int userId = 0;
      // go through the gw arguments, make sure a portlet or webservice id is included
      // if there are errors processing ids for arguments, just set that value to 0
      bool kwFound = false;
      while (args.HasMoreTokens())
      {
        String arg = args.GetNextToken();
        if (XPStringUtility.StartsWith(arg, GWARG_WEBSRV_PREFIX) || XPStringUtility.StartsWith(arg,
         GWARG_PORTLET_PREFIX))
        {
          objectType = (XPStringUtility.StartsWith(arg, GWARG_WEBSRV_PREFIX) ? PT_CLASSIDS.PT_WEBSERVICE_ID : PT_CLASSIDS.PT_GADGET_ID);
          objectId = GetId(XPStringUtility.Substring(arg, 2));
          kwFound = true;
        } else
        if (XPStringUtility.StartsWith(arg, GWARG_COMMUNITY_PREFIX))
        {
          commId = GetId(XPStringUtility.Substring(arg, 1));
        } else
        if (XPStringUtility.StartsWith(arg, GWARG_PAGE_PREFIX))
        {
          pageId = GetId(XPStringUtility.Substring(arg, 1));
        } else
        if (XPStringUtility.StartsWith(arg, GWARG_PREFTYPE_PREFIX))
        {
          prefType = GetId(XPStringUtility.Substring(arg, 1));
        } else
        if (XPStringUtility.StartsWith(arg, GWARG_USER_PREFIX))
        {
          userId = GetId(XPStringUtility.Substring(arg, 1));
        } else
        {
        }
      }
      if (!kwFound)
      {
        objectId = 0;
      }
      // construct regular url
      String regGWUrl = GatewayHelpers.ConstructRegularGatewayURL(baseUrl.ToString(),
       objectId, commId, pageId, prefType, objectType, userId, "");
      // don't want to encode the gateway path again so don't pass it in to the convert function 
      return regGWUrl + gwPathSeg.ToString();
    }
    
    /// <param name="url"></param>
    /// <param name="keyword"></param>
    /// <returns></returns>
    public static bool IsFriendlyGatewayURL(String url, String keyword)
    {
      // sanity checks
      if (keyword == null || "".Equals(keyword) || url == null)
      {
        return false;
      }
      return (-1 != XPStringUtility.IndexOf(url, URL_SEPARATOR + keyword + URL_SEPARATOR));
      }
    
    /// <summary>
    /// Return 0 if the excpected id is not numeric
    /// </summary>
    /// <param name="stringId"></param>
    /// <returns></returns>
    public static int GetId(String stringId)
    {
      try
      {
        return XPConvert.ToInteger(stringId);
      }
      catch (XPException e)
      {
        // no a valid number, just return 0
        return 0;
      }
    }
  }
}