using System;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
namespace com.plumtree.portaluiinfrastructure.statichelpers
{
  /// <summary>
  /// Provide security related configuration information.
  /// This class contains helper methods to get variables from the ConfigVarPack.
  /// This class accompanies ConfigHelper and PTConfigHelper.
  /// </summary>
  public class SecurityConfigHelper
  {
    private const String OUR_NAME = "SecurityConfigHelper";
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.statichelpers.SecurityConfigHelper");
    private String ssoAbsoluteURL = null;
    private SecurityConfigHelper()
    {
      InitializeSSOAbsoluteURL();
      log.Debug(OUR_NAME + ".ctor completed.");
    }
    // mel: Avoid the double checked locking problem by lazy initalization of a static instance.
    // The JVM spec (and the .NET runtime) make guarantees that a static member
    // will be initialized only once upon first active use of the class.
    // See http://www.cs.umd.edu/~pugh/java/memoryModel/DoubleCheckedLocking.html
    // http://java.sun.com/docs/books/vmspec/html/Concepts.doc.html#19075
    private class SecurityConfigHelperLatch
    {
      public static SecurityConfigHelper singleton = new SecurityConfigHelper();
      }
    // Since SecurityConfigHelper uses lazy initialization, this should only be called
    // when an absolute URL is needed by calling GetInstance().GetSSOAbsoluteURL().
    private void InitializeSSOAbsoluteURL()
    {
      String ssoBaseURL = PTConfigHelper.GetSSOBaseURL();
      try
      {
        // ssoBaseURL=/portal/SSOServlet
        bool useSecureURL = false;
        if (ConfigHelper.GetSecurityMode() == 1 || ConfigHelper.GetSecurityMode() == 2 || ConfigHelper.GetSecurityMode() == 3)
        {
          useSecureURL = true;
        }
        String portalMainServerURL = ConfigHelper.GetMainServerApplicationURL(useSecureURL);
        if (portalMainServerURL == null || XPStringUtility.IndexOf(portalMainServerURL,
         "*") == 0)
        {
          log.Warn(OUR_NAME + ".InitializeSSOAbsoluteURL failed to determine an absolute URL. The main server application URL (secure=" + XPConvert.ToString(useSecureURL) + ") returned was: " + portalMainServerURL);
          } else
        {
          // portalMainServerURL=https://hostname:8080/portal/server.pt
          IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
          ConfigVarPack vpConfig = (ConfigVarPack) application.GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);
          String portalVirtualDirectoryPath = vpConfig.GetConfigValueAsString(ConfigConstants.SYSTEM_PROPERTIES_SECTION,
           ConfigConstants.VIRTUAL_DIRECTORY_PATH_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE);
          // portalVirtualDirectoryPath=/portal/
          ssoAbsoluteURL = portalMainServerURL;
          int indexOfPortalVirtualDir = XPStringUtility.IndexOf(ssoAbsoluteURL,
           portalVirtualDirectoryPath);
          if (indexOfPortalVirtualDir >= 0)
          {
            ssoAbsoluteURL = XPStringUtility.Substring(ssoAbsoluteURL, 0,
             indexOfPortalVirtualDir);
          }
          // ssoAbsoluteURL=https://hostname:8080
          if (!XPStringUtility.EndsWith(ssoAbsoluteURL, "/") && !XPStringUtility.StartsWith(ssoBaseURL,
           "/"))
          {
            ssoAbsoluteURL+="/";
          }
          ssoAbsoluteURL+=ssoBaseURL;
        }
      }
      // ssoAbsoluteURL=https://hostname:8080/portal/SSOServlet
      catch (Exception anyException)
      {
        ssoAbsoluteURL = ssoBaseURL;
        log.Error(anyException, OUR_NAME + ".InitializeSSOAbsoluteURL failed. SSO login redirects in Security Mode 1 and 2 may use http instead of https.");
        }
      log.Debug(OUR_NAME + ".InitializeSSOAbsoluteURL assigned ssoAbsoluteURL: " + ssoAbsoluteURL);
      }
    
    /// <summary>
    /// GetInstance must be called after the ConfigVarPack has loaded.
    /// Calling GetInstance prior to warmup or prior to loading ConfigVarPack
    /// will result in an error.
    /// Any errors will be logged and a best attempt is made to continue initialization.
    /// </summary>
    /// <returns>the singleton instance of SecurityConfigHelper.</returns>
    public static SecurityConfigHelper GetInstance()
    {
      return SecurityConfigHelperLatch.singleton;
    }
    
    /// <summary>
    /// Concatenate GetSSOBaseURL with the application or secure application URL.
    /// The additional virtual directory and endpoint (e.g. /portal/server.pt)
    /// that would normally be present is removed.
    /// For security mode 1, 2, and 3 the secure URL is used.
    /// For security mode 0 the application URL is used.
    /// </summary>
    /// <returns>
    /// the SSO base URL in absolute form, e.g.
    /// https://hostname:8080/ssoVirtualDir/SSOServlet
    /// </returns>
    public virtual String GetSSOAbsoluteURL()
    {
      return ssoAbsoluteURL;
    }
  }
}