using System;
using com.plumtree.openfoundation.reflect;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.tags.helper;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.uitasks;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.tags
{
  /// <summary>
  /// This is the base class that developers will use to write
  /// their own custom tags.
  /// <p />
  /// In order to implement a Tag, developers need to follow
  /// the steps below:
  /// <ol>
  /// <li> Implement the DisplayTag() abstract method.  If you want
  /// this tag to display the HTML and tags inside the tag, you
  /// must call ProcessTagBody() and return the resulting HTML.
  /// </li>
  /// <li> Implement the Create() abstract method to return a new
  /// instance of this Tag.
  /// </li>
  /// <li> Create exactly one public static final
  /// ITagMetaData member variable that provides the name and
  /// description of the Tag and initialize it in the static
  /// initializer.
  /// <p />
  /// E.G. &lt;pt:thistag/&gt; would need <br />
  /// public static final ITagMetaData TAG;<br />
  /// static ThisTag() <I>(or just static in c#)</I><br />
  /// {<br />
  /// TAG = new TagMetaData("thistag", "This tag displays some HTML ...");<br />
  /// }<br />
  /// <p />
  /// Note that the TagMetaData class automatically converts the
  /// tag name to lowercase, which means that all references to the
  /// tag in HTML need to use the lowercase name.
  /// </li>
  /// <li> Create one public static final RequiredTagAttribute or
  /// OptionalTagAttribute member variable for every attribute
  /// that this tag supports and initialize them in the static
  /// initializer.
  /// <p />
  /// E.G. &lt;pt:thistag pt:firstattribute="foo" pt:secondattribute="bar"/&gt;
  /// would need <br />
  /// public static final RequiredTagAttribute FIRST_ATTRIBUTE;<br />
  /// public static final OptionalTagAttribute SECOND_ATTRIBUTE;<br />
  /// static ThisTag() <I>(or just static in c#)</I><br />
  /// {<br />
  /// FIRST_ATTRIBUTE = new RequiredTagAttribute(
  /// "firstattribute", "This attribute is used to ...");
  /// <br />
  /// SECOND_ATTRIBUTE = new OptionalTagAttribute(
  /// "secondattribute", "This optional attribute is used to ...",
  /// AttributeType.STRING, "default value");
  /// <br />
  /// }
  /// </li>
  /// </ol>
  /// The ITagMetaData, RequiredTagAttribute, and OptionalTagAttribute
  /// objects are used for programmatic access to tag meta data, as well
  /// as to pre-process tag attributes (presence, correct type, and
  /// default values).  If the required attributes are not correct, an
  /// error will be logged and the tag and its children will be skipped
  /// and not displayed.  An HTML Comment describing the tag and error
  /// will be displayed instead.
  /// <p />
  /// A description of the valid formats for attributes can be found
  /// in the AttributeType class.
  /// <p />
  /// Optionally, a Tag developer can also:
  /// <ol>
  /// <li> Override the DisplaySharedJavascript() method to include
  /// javascript once per page, rather than once per tag.
  /// </li>
  /// <li> Override the GetTagType() method to show that this tag is
  /// a different type of tag; e.g. LOOPING or NO_BODY.
  /// </li>
  /// <li> Override the SupportsAccessStyle() method to display this
  /// tag in non-standard access styles, such as 508 compliant.  If
  /// a tag does not use javascript, this method should be overridden
  /// to always return true so the tag can be used in 508 and
  /// low-bandwidth modes.
  /// </li>
  /// <li> Override the ReleaseTag() method to release data stored
  /// on the tag.  All tags that store custom data as member
  /// variables should override this method.
  /// </li>
  /// <li> Create a public static final RequiredParentTag member
  /// variable to require that this tag be used inside a particular tag
  /// and initialize it in the static initializer. If there are multiple
  /// RequiredParentTag members, then at least one parent tag must be
  /// present in order for the tag to function (but not all parent tags
  /// are required to be present).
  /// </li>
  /// <li> Create one public static final RequiredChildTag member
  /// variable for every tag that is required to be used inside
  /// this tag and initialize it in the static initializer.
  /// </li>
  /// <li> Create one public static final RelatedChildTag member
  /// variable for every tag that is designed to be used inside
  /// this tag, but is not required to be used inside this tag.
  /// Initialize the member variable in the static initializer.
  /// </li>
  /// </ol>
  /// All public static final metadata objects should be initialized
  /// together in a single static initializer so that the order of
  /// initialization can be properly controlled.
  /// <p />
  /// The ITagMetaData TAG member variable needs to be initialized
  /// first since it can be used in other tags (i.e. as a
  /// RequiredParentTag) that are referenced by member variables
  /// of this tag (i.e. as a RelatedChildTag).  This is necessary
  /// for tags that have circular references such as a parent /
  /// child tag relationship.
  /// <p />
  /// The ATag methods are grouped into several different categories
  /// of methods.
  /// <ul>
  /// <li><b>Abstract Tag Methods:</b> These methods need to implemented by the
  /// Tag developer.
  /// </li>
  /// <li><b>Default Tag Methods:</b> These methods can be overridden by the
  /// Tag developer.
  /// </li>
  /// <li><b>Tag Utility Methods:</b> These methods are used by the Tag developer
  /// in their\u0009Tag and should not be overridden.  This includes the
  /// IEnvironment class, which contains information about the current
  /// request and user, including locale information.
  /// </li>
  /// <li><b>Tag State Variable methods:</b> These methods are used by the Tag
  /// developer to get and set Tag State Variables in their Tags and
  /// should not be overridden.  Tag State Variables are used for
  /// inter-tag communication and are similar to storing data on the
  /// HTTP Session.
  /// </li>
  /// <li><b>PT Tag Attribute Methods:</b> These methods are used by the Tag
  /// developer to access PT Tag attributes in their Tags and should
  /// not be overridden.<p />
  /// E.G. &lt;pt:tag pt:attribute="false"/&gt;
  /// </li>
  /// <li><b>XML Tag Attribute Methods:</b> These methods are used by the Tag
  /// developer to access XML Tag attributes in their Tags and should
  /// not be overridden.<p />
  /// E.G. &lt;pt:tag attribute="false"/&gt;
  /// </li>
  /// <li><b>Framework Helper Methods:</b> These methods are used by the
  /// framework and should\u0009not be called or overridden by the Tag
  /// developer.
  /// </li>
  /// </ul>
  /// Thread Safety
  /// <p />
  /// Each tag instance is guaranteed to be accessed by only a single thread
  /// at a time.  However, there may be multiple threads accessing different
  /// instances of the same tag class at the same time, either from the same
  /// user or a different user.  This means that any static fields will need
  /// to be accessed using synchronized methods.  It is a best practice not
  /// to use static fields for data storage in tags.
  /// <p />
  /// Since there can be multiple instances of the same tag running at the
  /// same time, state variables set in shared Scopes (E.G. Scope.SESSION,
  /// Scope.PERSISTENT_SESSION, and Scope.APPLICATION) may change values
  /// during the execution of a single tag.
  /// </summary>
  /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.metadata.AttributeType"></seealso>
  /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.metadata.ITagMetaData"></seealso>
  /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.metadata.TagMetaData"></seealso>
  /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.metadata.RequiredTagAttribute"></seealso>
  /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.metadata.OptionalTagAttribute"></seealso>
  /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.metadata.RequiredParentTag"></seealso>
  /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.metadata.RequiredChildTag"></seealso>
  /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.metadata.RelatedChildTag"></seealso>
  /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ITagState"></seealso>
  public abstract class ATag : ATagInfrastructure
  {
    
    /// <summary>
    /// This is the main display method of the Tag.  This
    /// method is responsible for displaying all of the tag
    /// specific HTML, as well as displaying the inner HTML
    /// of the tag.
    /// <p />
    /// E.G.
    /// &lt;pt:thistag&gt;&lt;span&gt;&lt;pt:innertag/&gt;&lt;/span&gt;&lt;/pt:thistag&gt;
    /// This method is responsible for displaying the thistag
    /// UI and displaying the inner HTML
    /// (&lt;span&gt;&lt;pt:innertag/&gt;&lt;/span&gt;) by calling ProcessTagBody
    /// and including the results in the HTMLElement returned
    /// by this method.
    /// <p />
    /// This method is also responsible for displaying any
    /// instances of JavaScript need by this tag through the
    /// AddJavascript method.  The DisplaySharedJavascript
    /// method will be called automatically by the framework
    /// and does not need to be called by this method.
    /// <p />
    /// If there are errors in the tag and it cannot be displayed
    /// properly, the tag should throw an XPException with the
    /// error message, and the tag framework will log the error
    /// and add the message and stack trace to the HTML as an
    /// HTML comment.  The message contents will be HTML encoded
    /// before being added to the comment.
    /// <p />
    /// Abstract Tag Method
    /// </summary>
    /// <returns>
    /// HTMLElement The result of displaying this
    /// Tag and all the HTML and Transformer Tags within it.
    /// May be null.
    /// </returns>
    /// <throws>XPException if there are any errors in the tag.</throws>
    public abstract HTMLElement DisplayTag();
    
    /// <summary>
    /// This method is used to create new instance of managed
    /// (i.e. dynamically discovered) tags.
    /// <p />
    /// Tag methods will not be functional during execution of
    /// this method (E.G. tag.SetStateVariable, etc... will not work).
    /// </summary>
    /// <returns>
    /// ATag A new instance of the tag class
    /// (e.g. return new FooTag();). 
    /// </returns>
    public abstract ATag Create();
    
    /// <summary>
    /// This method is used to display JavaScript that only
    /// needs to included on a page once, regardless of how
    /// many times a particular tag is used on that page
    /// (such as including JavaScript files).
    /// <p />
    /// Tags that need to include javascript once per page should
    /// override this method and include the javascript here.
    /// <p />
    /// This method is called automatically by the framework
    /// and does not need to be called in the DisplayTag()
    /// method.
    /// <p />
    /// This method will not be called in Section 508 compliance
    /// mode, as 508 screen readers do not allow JavaScript.
    /// <p />
    /// If there are errors in the tag and the JavaScript
    /// cannot be displayed properly, the tag should throw an
    /// XPException with the error message, and the tag
    /// framework will log the error and add the message and
    /// stack trace to the HTML as an HTML comment.  The message
    /// contents will be HTML encoded before being added to the
    /// comment.
    /// <p />
    /// Default Tag Method
    /// </summary>
    /// <returns>
    /// HTMLScriptCollection The JavaScript to be displayed.
    /// May be null.
    /// </returns>
    public virtual HTMLScriptCollection DisplaySharedJavascript()
    {
      return null;
    }
    
    /// <summary>
    /// This method describes what kind of tag this is.  The default
    /// is a simple tag (i.e. displays some HTML, displays the body,
    /// and then displays some more HTML).  Tags that display the
    /// tag body more than once (e.g. looping tags) should override
    /// this method and return TagType.LOOPING.  Tags that will
    /// never display the tag body should override this method and
    /// return TagType.NO_BODY to allow for performance optimizations.
    /// <p />
    /// The TagType data is used for performance optimizations.
    /// <p />
    /// Default Tag Method
    /// </summary>
    /// <returns>TagType The type of this tag</returns>
    public virtual TagType GetTagType()
    {
      return TagType.SIMPLE;
    }
    
    /// <summary>
    /// This method is used to verify whether or not this tag
    /// supports the passed in access style.
    /// <br />
    /// By default we only support the standard access style.
    /// To support 508 access, the tag must be certified as
    /// not containing any javascript (onClick, etc...)
    /// <br />
    /// If a tag does not support the current access style, an
    /// HTML comment saying that will be displayed in its place.
    /// <p />
    /// Default Tag Method
    /// </summary>
    /// <param name="_style">The access style to check.</param>
    /// <returns>
    /// boolean True implies the passed in access style is
    /// supported.
    /// </returns>
    public virtual bool SupportsAccessStyle(AccessStyles _style)
    {
      // By default we only support the standard access style.
      // To support 508 access, the tag must be certified as
      // not containing any javascript (onClick, etc...)
      if (_style == AccessStyles.STANDARD)
      {
        return true;
      }
      return false;
    }
    
    /// <summary>
    /// This method releases all data stored in this tag
    /// to prevent excess memory retention and to prepare
    /// the tag for reuse.  This method should return the
    /// tag to the state it was in immediately after the
    /// Create() method was called. After this method is
    /// called, the tag should be able to be re-initialized
    /// and used again.
    /// <p />
    /// All tags that store custom data as member variables
    /// should override this method and release the data
    /// here.  If a tag does not store custom data as
    /// member variables, then there is no need to override
    /// this method.
    /// <p /> 
    /// Tag methods will not be functional during execution of
    /// this method (E.G. tag.GetEnvironment, etc... will not work).
    /// Default Tag Method
    /// </summary>
    public override void ReleaseTag()
    {
    }
    
    /// <summary>
    /// Gets the environment for the current tag and request.
    /// <p />
    /// Tag Utility Method
    /// </summary>
    /// <returns>IEnvironment The current environment</returns>
    public virtual IEnvironment GetEnvironment()
    {
      return GetTagHelper().GetEnvironment();
    }
    
    /// <summary>
    /// Gets the variable state helper object.  This object
    /// allows you to get and set variables for various
    /// contexts (session, request, tag, etc...).
    /// <p />
    /// Tag Utility Method
    /// </summary>
    /// <returns>ITagState The tag state helper object</returns>
    public virtual ITagState GetState()
    {
      return GetTagHelper().GetState();
    }
    
    /// <summary>
    /// This method displays the HTML and Transformer
    /// Tags that are inside this tag.
    /// <p />
    /// This method may be called zero times, once, or
    /// multiple times during the lifecycle of a tag.
    /// Tags that call this method multiple times should
    /// return TagType.LOOPING from the GetTagType
    /// method.  Tags that never call this method should
    /// return TagType.NO_BODY from the GetTagType method.
    /// <p />
    /// Tag Utility Method
    /// </summary>
    /// <returns>
    /// HTMLElement The result of displaying
    /// the HTML and Transformer Tags inside this tag.
    /// </returns>
    /// <throws>XPException if tags of type NO_BODY call this method.</throws>
    public virtual HTMLElement ProcessTagBody()
    {
      if (TagType.NO_BODY == GetTagType())
      {
        throw new XPException("Tags of type NOBODY cannot call ProcessTagBody.");
        }
      return GetTagHelper().ProcessTagBody();
    }
    
    /// <summary>
    /// This method is used to add JavaScript for this
    /// tag.
    /// <p />
    /// This method should not be used for common JavaScript
    /// for a tag (such as JavaScript includes) that only
    /// needs to be displayed once per page, regardless of
    /// how many instances of this tag there are on the page.
    /// DisplaySharedJavaScript() should be used instead.
    /// <p />
    /// JavaScript added using this method will not be displayed in
    /// Section 508 compliance mode, as 508 screen readers do not
    /// allow JavaScript.
    /// <p />
    /// Tag Utility Method
    /// </summary>
    /// <param name="HTMLScriptCollection">
    /// _script The JavaScript to
    /// be displayed
    /// </param>
    public virtual void AddJavascript(HTMLScriptCollection _script)
    {
      GetTagHelper().AddJavascript(_script);
    }
    
    /// <summary>
    /// Check whether or not the current Tag is nested
    /// inside a given parent Tag.  The parent tag must
    /// be in the same library as the current tag.
    /// <p />
    /// E.G.
    /// &lt;pt:parenttag&gt;&lt;pt:thistag/&gt;&lt;/pt:parenttag&gt;
    /// would return true for "parenttag".
    /// <p />
    /// Tag Utility Method
    /// </summary>
    /// <param name="ITagMetaData">
    /// _parentTag The Tag Meta Data
    /// of the requested parent tag
    /// </param>
    /// <returns>boolean True implies that the parent tag was found.</returns>
    public virtual bool HasParentTag(ITagMetaData _parentTag)
    {
      return (GetTagHelper().HasParentTag(_parentTag) > 0);
    }
    
    /// <summary>
    /// Check whether or not a particular Transformer
    /// Tag is nested inside the current Tag.  The
    /// child tag must be in the same library as the
    /// current tag.
    /// <p />
    /// E.G.
    /// &lt;pt:thistag&gt;&lt;span&gt;&lt;pt:firstchildtag&gt;&lt;pt:subsubchildtag/&gt;&lt;/pt:firstchildtag&gt;
    /// &lt;pt:secondchildtag/&gt;&lt;span&gt;&lt;/pt:thistag&gt;
    /// would return true for "firstchildtag", "secondchildtag",
    /// and "subsubchildtag", but not "span", since it is
    /// an HTML Tag, not a Transformer Tag.
    /// <p />
    /// Tag Utility Method
    /// </summary>
    /// <param name="ITagMetaData">
    /// _childTag A Tag Meta
    /// Data objects of the requested child tag
    /// </param>
    /// <returns>boolean True implies that the child tag was found.</returns>
    public virtual bool HasChildTag(ITagMetaData _childTag)
    {
      return (GetTagHelper().HasChildTag(_childTag) > 0);
    }
    
    /// <summary>
    /// Gets a unique ID for this instance of this tag.
    /// <p />
    /// This ID is unique across all tags on all portlets
    /// on a given page.
    /// <p />
    /// This ID cannot be re-used across multiple pages and
    /// needs to be regenerated.
    /// <p />
    /// Tag Utility Method
    /// </summary>
    /// <returns>String A unique ID for this instance of this tag.</returns>
    public virtual String GetUniqueID()
    {
      return GetTagHelper().GetUniqueID();
    }
    
    /// <summary>
    /// Sets a variable in memory in the appropriate scope.
    /// <p />
    /// The scope determines who can see this variable and
    /// how long it stays in memory.
    /// <p />
    /// For instance, a variable stored in Tag Scope can only
    /// be seen by children of the current tag and will be
    /// removed from memory when the tag is finished.
    /// <p />
    /// A variable stored in Portlet Request Scope will
    /// be visible to all tags in the same portlet as the
    /// current tag, and will be removed from memory when
    /// the portlet is finished displaying.  Tags in other
    /// portlets on the same page will not be able to see
    /// the variable.
    /// <p />
    /// Standard variables (as opposed to shared variables)
    /// can only be accessed by tags from the same tag
    /// library as the tag that originally stored the
    /// variable.
    /// <p />
    /// Note: Displaying an HTMLElement in a tag and then
    /// caching it so that another tag can add more HTML
    /// to the original tag later is not supported.
    /// HTMLElement trees can be generated and stored for
    /// later use, as long as they are self-contained
    /// trees and used in a read only way.  It is safest
    /// to make clones of a cached HTMLElement tree before
    /// trying to display it again to make sure there are
    /// no threading problems.
    /// <p />
    /// Tag State Variable Method
    /// </summary>
    /// <param name="String">
    /// _strKey The key used to store the data.
    /// The key cannot contain the reserved character '.'.
    /// </param>
    /// <param name="Object">
    /// _oValue The data to be stored. Note: setting
    /// the value to null is the same as never having set the variable.
    /// </param>
    /// <param name="Scope">_scope The scope to store the data in</param>
    /// <throws>
    /// XPIllegalArgumentException if the key is null or
    /// contains the reserved character '.', or if the scope
    /// is null or unknown.
    /// </throws>
    public virtual void SetStateVariable(String _strKey, Object _oValue, Scope _scope)
    {
      GetState().SetVariable(_strKey, _oValue, _scope);
    }
    
    /// <summary>
    /// Sets a variable in memory in the appropriate scope.
    /// <p />
    /// The scope determines who can see this variable and
    /// how long it stays in memory.
    /// <p />
    /// For instance, a variable stored in Tag Scope can only
    /// be seen by children of the current tag and will be
    /// removed from memory when the tag is finished.
    /// <p />
    /// A variable stored in Portlet Request Scope will
    /// be visible to all tags in the same portlet as the
    /// current tag, and will be removed from memory when
    /// the portlet is finished displaying.  Tags in other
    /// portlets on the same page will not be able to see
    /// the variable.
    /// <p />
    /// Shared variables (as opposed to standard variables)
    /// can be accessed by tags from any library.  If
    /// bOwnerEditOnly is set to true when a shared variable
    /// is first stored, then any tag can read the variable,
    /// but only tags from the same library as the tag that
    /// originally stored the variable can edit it (and
    /// replace the value in memory).
    /// <p />
    /// Note: Displaying an HTMLElement in a tag and then
    /// caching it so that another tag can add more HTML
    /// to the original tag later is not supported.
    /// HTMLElement trees can be generated and stored for
    /// later use, as long as they are self-contained
    /// trees and used in a read only way.  It is safest
    /// to make clones of a cached HTMLElement tree before
    /// trying to display it again to make sure there are
    /// no threading problems.
    /// <p />
    /// Tag State Variable Method
    /// </summary>
    /// <param name="String">
    /// _strKey The key used to store the data.
    /// The key cannot contain the reserved character '.'.
    /// </param>
    /// <param name="Object">
    /// _oValue The data to be stored. Note: setting
    /// the value to null is the same as never having set the variable.
    /// </param>
    /// <param name="Scope">_scope The scope to store the data in.</param>
    /// <param name="bOwnerEditOnly">
    /// True implies that only tags from
    /// the same tag library should be able to store the value
    /// for this tag.  If this variable has already been saved,
    /// then the original bOwnerEditOnly value will be retained
    /// and this argument will be ignored.
    /// </param>
    /// <throws>
    /// XPIllegalArgumentException if the key is null or
    /// contains the reserved character '.', or if the scope
    /// is null or unknown.
    /// </throws>
    /// <throws>
    /// XPIllegalAccessException if the variable is being
    /// set by a tag from a different library than the original
    /// tag that stored the variable with bOwnerEditOnly set to
    /// true.
    /// </throws>
    public virtual void SetStateSharedVariable(String _strKey, Object _oValue,
     Scope _scope, bool bOwnerEditOnly)
    {
      GetState().SetSharedVariable(_strKey, _oValue, _scope, bOwnerEditOnly);
      }
    
    /// <summary>
    /// Gets a variable from memory in the appropriate scope.
    /// <p />
    /// The scope determines who can see this variable and
    /// how long it stays in memory.
    /// <p />
    /// For instance, a variable can only be retrieved from
    /// Tag Scope if a tag that includes the current tag
    /// added the variable to Tag Scope.
    /// <p />
    /// A variable can only be retrieved from Portlet
    /// Request Scope if another tag in the portlet stored
    /// the variable in memory.
    /// <p />
    /// Standard variables (as opposed to shared variables)
    /// can only be accessed by tags from the same tag
    /// library as the tag that originally stored the
    /// variable.
    /// <p />
    /// Note: Retrieving an HTMLElement from memory that
    /// has already been displayed in another tag and then
    /// adding more HTML to the original tag is not
    /// supported.  HTMLElement trees can be retrieved from
    /// memory and re-used, as long as they are not modified.
    /// The safest way to do this is to make a clone of the
    /// cached HTMLElement tree before trying to display it
    /// again to make sure there are no threading problems.
    /// <p />
    /// Tag State Variable Method
    /// </summary>
    /// <param name="String">
    /// _strKey The key used to store the data.
    /// The key cannot contain the reserved character '.'.
    /// </param>
    /// <param name="Scope">_scope The scope used to store the data.</param>
    /// <returns>
    /// Object The requested data. Null if the variable
    /// has never been set.
    /// </returns>
    /// <throws>
    /// XPIllegalArgumentException if the key is null or
    /// contains the reserved character '.', or if the scope
    /// is null or unknown.
    /// </throws>
    public virtual Object GetStateVariable(String _strKey, Scope _scope)
    {
      return GetState().GetVariable(_strKey, _scope);
    }
    
    /// <summary>
    /// Gets a variable from memory from all scopes.
    /// <p />
    /// The scope determines who can see this variable and
    /// how long it stays in memory.
    /// <p />
    /// For instance, a variable can only be retrieved from
    /// Tag Scope if a tag that includes the current tag
    /// added the variable to Tag Scope.
    /// <p />
    /// A variable can only be retrieved from Portlet
    /// Request Scope if another tag in the portlet stored
    /// the variable in memory.
    /// <p />
    /// Standard variables (as opposed to shared variables)
    /// can only be accessed by tags from the same tag
    /// library as the tag that originally stored the
    /// variable.
    /// <p />
    /// This method will first look for standard variables,
    /// and if not found will then look for shared variables.
    /// <p />
    /// Note: Retrieving an HTMLElement from memory that
    /// has already been displayed in another tag and then
    /// adding more HTML to the original tag is not
    /// supported.  HTMLElement trees can be retrieved from
    /// memory and re-used, as long as they are not modified.
    /// The safest way to do this is to make a clone of the
    /// cached HTMLElement tree before trying to display it
    /// again to make sure there are no threading problems.
    /// <p />
    /// Tag State Variable Method
    /// </summary>
    /// <param name="String">_strKey The key used to store the data.</param>
    /// <returns>
    /// Object The requested data. Null if the variable
    /// has never been set.
    /// </returns>
    /// <throws>XPIllegalArgumentException if the key is null.</throws>
    public virtual Object GetAnyStateVariable(String _strKey)
    {
      return GetState().GetAnyVariable(_strKey);
    }
    
    /// <summary>
    /// Gets a shared variable from memory in the appropriate
    /// scope.
    /// <p />
    /// The scope determines who can see this variable and
    /// how long it stays in memory.
    /// <p />
    /// For instance, a variable can only be retrieved from
    /// Tag Scope if a tag that includes the current tag
    /// added the variable to Tag Scope.
    /// <p />
    /// A variable can only be retrieved from Portlet
    /// Request Scope if another tag in the portlet stored
    /// the variable in memory.
    /// <p />
    /// Shared variables can be accessed by tags from any
    /// tag library (as opposed to standard variables, which
    /// can only be accessed by tags from the same tag
    /// library).
    /// <p />
    /// Note: Retrieving an HTMLElement from memory that
    /// has already been displayed in another tag and then
    /// adding more HTML to the original tag is not
    /// supported.  HTMLElement trees can be retrieved from
    /// memory and re-used, as long as they are not modified.
    /// The safest way to do this is to make a clone of the
    /// cached HTMLElement tree before trying to display it
    /// again to make sure there are no threading problems.
    /// <p />
    /// Tag State Variable Method
    /// </summary>
    /// <param name="String">
    /// _strKey The key used to store the data.
    /// The key cannot contain the reserved character '.'.
    /// </param>
    /// <param name="Scope">_scope The scope used to store the data.</param>
    /// <returns>
    /// Object The requested data. Null if the variable
    /// has never been set.
    /// </returns>
    /// <throws>
    /// XPIllegalArgumentException if the key is null or
    /// contains the reserved character '.', or if the scope
    /// is null or unknown.
    /// </throws>
    public virtual Object GetStateSharedVariable(String _strKey, Scope _scope)
    {
      return GetState().GetSharedVariable(_strKey, _scope);
    }
    
    /// <summary>
    /// This method gets the requested attribute if it was present in the tag.  Otherwise it returns
    /// the default value for the attribute, if there is one.
    /// <p />
    /// This is a helper method to get retrieve tag variable from tag variable id 
    /// <p />
    /// PT Tag Attribute Method
    /// </summary>
    /// <param name="ATagAttribute">The requested attribute.</param>
    /// <param name="Scope">The scope to retrieve the tag variable from</param>
    /// <returns>Value of the tag attribute or the default value if none was found.</returns>
    public virtual Object GetTagAttributeAsTagVariable(ATagAttribute _attribute,
     Scope scope)
    {
      return GetTagHelper().GetAttributeAsTagVariable(_attribute, scope);
      }
    
    /// <summary>
    /// This method gets the requested attribute if it was present in the tag.  Otherwise it returns
    /// the default value for the attribute, if there is one.
    /// <p />
    /// This is a helper method to get retrieve the Scope object from a attribute that takes a scope name 
    /// <p />
    /// PT Tag Attribute Method
    /// </summary>
    /// <param name="ATagAttribute">The requested attribute.</param>
    /// <returns>Value of the tag attribute or the default value if none was found.</returns>
    public virtual Scope GetTagAttributeAsScope(ATagAttribute _attribute)
    {
      return GetTagHelper().GetAttributeAsScope(_attribute);
    }
    
    /// <summary>
    /// This method gets the requested attribute if it
    /// was present in the tag.  Otherwise it returns
    /// the default value for the attribute, if there
    /// is one.
    /// <p />
    /// E.G.
    /// &lt;pt:tag pt:ptattr="test" width="10"/&gt;
    /// would get the ptattr attribute, but not the width attribute.
    /// <p />
    /// A description of the valid formats for attributes can be found
    /// in the AttributeType class.
    /// <p />
    /// PT Tag Attribute Method
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.metadata.AttributeType#STRING"></seealso>
    /// <param name="ATagAttribute">The requested attribute.</param>
    /// <returns>String The value of the attribute.</returns>
    public virtual String GetTagAttributeAsString(ATagAttribute _attribute)
    {
      return GetTagHelper().GetAttributeAsString(_attribute);
    }
    
    /// <summary>
    /// This method gets the requested attribute if it
    /// was present in the tag.  Otherwise it returns
    /// the default value for the attribute.
    /// <p />
    /// E.G.
    /// &lt;pt:tag pt:ptattr="12" width="10"/&gt;
    /// would get the ptattr attribute, but not the width attribute.
    /// <p />
    /// A description of the valid formats for attributes can be found
    /// in the AttributeType class.
    /// <p />
    /// PT Tag Attribute Method
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.metadata.AttributeType#INT"></seealso>
    /// <param name="ATagAttribute">The requested attribute</param>
    /// <returns>int The value of the attribute.</returns>
    /// <throws>
    /// XPFormatException if the attribute value
    /// cannot be converted to an int
    /// </throws>
    public virtual int GetTagAttributeAsInt(ATagAttribute _attribute)
    {
      return GetTagHelper().GetAttributeAsInt(_attribute);
    }
    
    /// <summary>
    /// This method gets the requested attribute if it
    /// was present in the tag.  Otherwise it returns
    /// the default value for the attribute.
    /// <p />
    /// E.G.
    /// &lt;pt:tag pt:ptattr="false" width="10"/&gt;
    /// would get the ptattr attribute, but not the width attribute.
    /// <p />
    /// A description of the valid formats for attributes can be found
    /// in the AttributeType class.
    /// <p />
    /// PT Tag Attribute Method
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.metadata.AttributeType#BOOLEAN"></seealso>
    /// <param name="ATagAttribute">The requested attribute</param>
    /// <returns>boolean The value of the attribute.</returns>
    /// <throws>
    /// XPFormatException if the attribute value
    /// cannot be converted to a boolean
    /// </throws>
    public virtual bool GetTagAttributeAsBoolean(ATagAttribute _attribute)
    {
      return GetTagHelper().GetAttributeAsBoolean(_attribute);
    }
    
    /// <summary>
    /// This method gets the requested attribute if it
    /// was present in the tag.  Otherwise it returns
    /// the default value for the attribute.
    /// <p />
    /// E.G.
    /// &lt;pt:tag pt:ptattr="t" width="10"/&gt;
    /// would get the ptattr attribute, but not the width attribute.
    /// <p />
    /// A description of the valid formats for attributes can be found
    /// in the AttributeType class.
    /// <p />
    /// PT Tag Attribute Method
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.metadata.AttributeType#CHAR"></seealso>
    /// <param name="ATagAttribute">The requested attribute</param>
    /// <returns>char The value of the attribute.</returns>
    /// <throws>
    /// XPFormatException if the attribute value
    /// cannot be converted to a char
    /// </throws>
    public virtual char GetTagAttributeAsChar(ATagAttribute _attribute)
    {
      return GetTagHelper().GetAttributeAsChar(_attribute);
    }
    
    /// <summary>
    /// This method gets the requested attribute if it
    /// was present in the tag.  Otherwise it returns
    /// the default value for the attribute.
    /// <p />
    /// E.G.
    /// &lt;pt:tag pt:ptattr="1.5" width="10"/&gt;
    /// would get the ptattr attribute, but not the width attribute.
    /// <p />
    /// A description of the valid formats for attributes can be found
    /// in the AttributeType class.
    /// <p />
    /// PT Tag Attribute Method
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.metadata.AttributeType#DOUBLE"></seealso>
    /// <param name="ATagAttribute">The requested attribute</param>
    /// <returns>double The value of the attribute.</returns>
    /// <throws>
    /// XPFormatException if the attribute value
    /// cannot be converted to a double
    /// </throws>
    public virtual double GetTagAttributeAsDouble(ATagAttribute _attribute)
    {
      return GetTagHelper().GetAttributeAsDouble(_attribute);
    }
    
    /// <summary>
    /// This method gets the requested attribute if it
    /// was present in the tag.  Otherwise it returns
    /// the default value for the attribute.
    /// <p />
    /// E.G.
    /// &lt;pt:tag pt:ptattr="100" width="10"/&gt;
    /// would get the ptattr attribute, but not the width attribute.
    /// <p />
    /// A description of the valid formats for attributes can be found
    /// in the AttributeType class.
    /// <p />
    /// PT Tag Attribute Method
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.metadata.AttributeType#LONG"></seealso>
    /// <param name="ATagAttribute">The requested attribute</param>
    /// <returns>long The value of the attribute.</returns>
    /// <throws>
    /// XPFormatException if the attribute value
    /// cannot be converted to a long
    /// </throws>
    public virtual long GetTagAttributeAsLong(ATagAttribute _attribute)
    {
      return GetTagHelper().GetAttributeAsLong(_attribute);
    }
    
    /// <summary>
    /// Check to see whether or not a specified attribute is present
    /// in the HTML for this tag.  This will always return true for
    /// required tags.
    /// <p />
    /// E.G.
    /// &lt;pt:tag pt:ptattr="test" width="10"/&gt;
    /// would return true for ptattr, but not for width.
    /// <p />
    /// Note: Even though the tag may be present, it may produce
    /// an error when trying to process the tag.
    /// <p />
    /// PT Tag Attribute Method
    /// </summary>
    /// <param name="_attribute">ATagAttribute The attribute to check</param>
    /// <returns>boolean True if the attribute is present in the HTML</returns>
    public virtual bool HasTagAttribute(ATagAttribute _attribute)
    {
      return GetTagHelper().HasAttribute(_attribute);
    }
    
    /// <summary>
    /// Gets an enumeration of the names of all the pt attributes
    /// in this tag.
    /// <p />
    /// E.G.
    /// &lt;pt:tag pt:ptattr="test" width="10"/&gt;
    /// would return the ptattr attribute, but not the width attribute.
    /// <p />
    /// PT attributes are designed to specify the logic of the tag,
    /// while non-pt attributes specify the behavior of the resulting
    /// HTML tag.
    /// <p />
    /// In general, PT Attributes should have a corresponding
    /// ATagAttribute class and should be retrieved using the
    /// GetTagAttributeAs* methods.  These methods handle
    /// default attribute values and attribute type conversion.
    /// <p />
    /// This method is present for backwards compatibility with
    /// the standard Transformer tags and should not be used if
    /// at all possible.
    /// <p />
    /// PT Tag Attribute Method
    /// </summary>
    /// <returns>IXPEnumerator All of the pt attribute names of this tag</returns>
    public virtual IXPEnumerator GetTagAttributeNames()
    {
      return GetTagHelper().GetAttributeNames();
    }
    
    /// <summary>
    /// This method gets the value of the requested pt
    /// attribute.
    /// <p />
    /// E.G.
    /// &lt;pt:tag pt:ptattr="test" width="10"/&gt;
    /// would get the ptattr attribute, but not the width attribute.
    /// <p />
    /// PT attributes are designed to specify the logic of the tag,
    /// while non-pt attributes specify the behavior of the resulting
    /// HTML tag.
    /// <p />
    /// In general, PT Attributes should have a corresponding
    /// ATagAttribute class and should be retrieved using the
    /// GetTagAttributeAs* methods.  These methods handle
    /// default attribute values and attribute type conversion.
    /// <p />
    /// This method does not handle default attribute values if
    /// the attribute is not present, or if there is an error
    /// processing the attribute.
    /// <p />
    /// This method is present for backwards compatibility with
    /// the standard Transformer tags and should not be used if
    /// at all possible.
    /// <p />
    /// PT Tag Attribute Method
    /// </summary>
    /// <param name="_strAttribute">The requested attribute name</param>
    /// <returns>
    /// The value of the requested attribute, or null
    /// if it is not present in the tag or there was an error
    /// processing the attribute.
    /// </returns>
    public virtual String GetTagAttribute(String _strAttribute)
    {
      return GetTagHelper().GetAttribute(_strAttribute);
    }
    
    /// <summary>
    /// This method gets the value of the requested non-pt
    /// attribute.
    /// <p />
    /// E.G.
    /// &lt;pt:tag pt:ptattr="test" width="10"/&gt;
    /// would get the width attribute, but not the ptattr attribute.
    /// <p />
    /// XML attributes are designed to let a web designer specify
    /// standard HTML attributes for a tag without the tag author
    /// having to handle them individually.  PT attributes specify
    /// the logic of the tag, while non-pt attributes specify
    /// the behavior of the resulting HTML tag.
    /// <p />
    /// This is only appropriate for tags that output a simple
    /// HTML tag.
    /// <p />
    /// This method does not handle default attribute values.
    /// <p />
    /// XML Tag Attribute Method
    /// </summary>
    /// <param name="_strAttribute">The requested attribute name</param>
    /// <returns>
    /// The value of the requested attribute, or null
    /// if it is not present in the tag.  The empty string can
    /// be returned for various reasons, including if there was
    /// an error processing the attribute.
    /// </returns>
    public virtual String GetXMLTagAttribute(String _strAttribute)
    {
      return GetTagHelper().GetXMLAttribute(_strAttribute);
    }
    
    /// <summary>
    /// Check to see whether or not a specified non-pt attribute is present
    /// in the HTML for this tag.  This will always return true for
    /// required tags.
    /// <p />
    /// E.G.
    /// &lt;pt:tag pt:ptattr="test" width="10"/&gt;
    /// would return true for the width attribute, but not for the ptattr
    /// attribute.
    /// <p />
    /// XML attributes are designed to let a web designer specify
    /// standard HTML attributes for a tag without the tag author
    /// having to handle them individually.  PT attributes specify
    /// the logic of the tag, while non-pt attributes specify
    /// the behavior of the resulting HTML tag.
    /// <p />
    /// This is only appropriate for tags that output a simple
    /// HTML tag.
    /// <p />
    /// Note: Even though the tag may be present, it may produce
    /// an error when trying to process the tag.
    /// <p />
    /// XML Tag Attribute Method
    /// </summary>
    /// <param name="String">_strAttribute The name of the non-pt attribute to check</param>
    /// <returns>boolean True if the attribute is present in the HTML</returns>
    public virtual bool HasXMLTagAttribute(String _strAttribute)
    {
      return GetTagHelper().HasXMLAttribute(_strAttribute);
    }
    
    /// <summary>
    /// Gets an enumeration of the names of all the non-pt attributes
    /// in this tag.
    /// <p />
    /// E.G.
    /// &lt;pt:tag pt:ptattr="test" width="10"/&gt;
    /// would return the width attribute, but not the ptattr attribute.
    /// <p />
    /// XML attributes are designed to let a web designer specify
    /// standard HTML attributes for a tag without the tag author
    /// having to handle them individually.  PT attributes specify
    /// the logic of the tag, while non-pt attributes specify
    /// the behavior of the resulting HTML tag.
    /// <p />
    /// XML Tag Attribute Method
    /// <p />
    /// This is only appropriate for tags that output a simple
    /// HTML tag.
    /// </summary>
    /// <returns>IXPEnumerator All of the non-pt attribute names of this tag</returns>
    public virtual IXPEnumerator GetXMLTagAttributeNames()
    {
      return GetTagHelper().GetXMLAttributeNames();
    }
    
    /// <summary>
    /// Calculates a string containing all of the non-pt attributes
    /// that can be inserted inside an XML tag to add all of the
    /// attributes to the tag.
    /// <p />
    /// E.G.
    /// &lt;pt:tag pt:ptattr="test" width="10" length="20"/&gt;
    /// would return the following string:
    /// width="10" length="20"
    /// <p />
    /// XML attributes are designed to let a web designer specify
    /// standard HTML attributes for a tag without the tag author
    /// having to handle them individually.  PT attributes specify
    /// the logic of the tag, while non-pt attributes specify
    /// the behavior of the resulting HTML tag.
    /// <p />
    /// XML Tag Attribute Method
    /// <p />
    /// This is only appropriate for tags that output a simple
    /// HTML tag.
    /// </summary>
    /// <returns>String All of the non-pt attributes of this tag.</returns>
    public virtual String GetXMLTagAttributesAsString()
    {
      return GetTagHelper().GetXMLTagAttributesAsString();
    }
    
    /// <summary>
    /// Helper function to format a string with HTML, tags to properly display in Tag Docs.
    /// The main use case is for example strings where any special HTML characters (<, >, &) need
    /// to be surrounded with CDATA tags and special XML characters encoded, in effect,
    /// double encode tags to display properly in the tag docs.
    /// To create a line-break in the tag docs, add the <br /> tag in the string. 
    /// Example: String s = "This is line 1 <br />This is line 2";
    /// To show code in a fixed width font in the tag docs, add the <c> tag in the string.
    /// Example: String s = "Use the <c>EncodeForTagDoc()</c> call to encode your documentation";
    /// Lists of items can be shown in the tag docs using the <ul>, </ul> and <li> tags.
    /// Example: String s = "The output contains 3 items: <ul> <li> One <li> Two, and <li> 3 </ul>";
    /// NOTE: This only encodes certain HTML elements (br, ul, li, and code), and only if
    /// they are lower case.
    /// </summary>
    /// <param name="tagdocString"></param>
    /// <returns>The HTML string properly escaped for use in tag docs.</returns>
    public static String EncodeForTagDoc(String tagdocString)
    {
      // TODO CR DLH: This doesn't seem like the right way to do this.  What about
      // other tags like <p>?  What if someone uses a capital-case tag like <BR>?
      // This seems kind of hacky.
      // first replace the XML characters with temp markers
      tagdocString = GeneralHelpers.Replace(tagdocString, "<br>", "!!!br!!!");
      tagdocString = GeneralHelpers.Replace(tagdocString, "<br/>", "!!!br/!!!");
      tagdocString = GeneralHelpers.Replace(tagdocString, "<ul>", "!!!ul!!!");
      tagdocString = GeneralHelpers.Replace(tagdocString, "</ul>", "!!!/ul!!!");
      tagdocString = GeneralHelpers.Replace(tagdocString, "<li>", "!!!li!!!");
      tagdocString = GeneralHelpers.Replace(tagdocString, "</li>", "!!!/li!!!");
      tagdocString = GeneralHelpers.Replace(tagdocString, "<code>", "!!!code!!!");
      tagdocString = GeneralHelpers.Replace(tagdocString, "</code>", "!!!/code!!!");
      tagdocString = GeneralHelpers.Replace(tagdocString, "<p>", "!!!p!!!");
      tagdocString = GeneralHelpers.Replace(tagdocString, "</p>", "!!!/p!!!");
      tagdocString = GeneralHelpers.Replace(tagdocString, "&nbsp;", "!!!nbsp!!!");
      tagdocString = GeneralHelpers.Replace(tagdocString, "&", "&amp;");
      tagdocString = GeneralHelpers.Replace(tagdocString, "<", "&lt;");
      tagdocString = GeneralHelpers.Replace(tagdocString, ">", "&gt;");
      // Then replace the temp markers with CDATA encoded HTML
      tagdocString = GeneralHelpers.Replace(tagdocString, "!!!br!!!", "]]><![CDATA[<br>]]><![CDATA[");
      tagdocString = GeneralHelpers.Replace(tagdocString, "!!!br/!!!", "]]><![CDATA[<br/>]]><![CDATA[");
      tagdocString = GeneralHelpers.Replace(tagdocString, "!!!ul!!!", "]]><![CDATA[<ul>]]><![CDATA[");
      tagdocString = GeneralHelpers.Replace(tagdocString, "!!!/ul!!!", "]]><![CDATA[</ul>]]><![CDATA[");
      tagdocString = GeneralHelpers.Replace(tagdocString, "!!!li!!!", "]]><![CDATA[<li>]]><![CDATA[");
      tagdocString = GeneralHelpers.Replace(tagdocString, "!!!/li!!!", "]]><![CDATA[</li>]]><![CDATA[");
      tagdocString = GeneralHelpers.Replace(tagdocString, "!!!code!!!", "]]><![CDATA[<code>]]><![CDATA[");
      tagdocString = GeneralHelpers.Replace(tagdocString, "!!!/code!!!", "]]><![CDATA[</code>]]><![CDATA[");
      tagdocString = GeneralHelpers.Replace(tagdocString, "!!!p!!!", "]]><![CDATA[<p>]]><![CDATA[");
      tagdocString = GeneralHelpers.Replace(tagdocString, "!!!/p!!!", "]]><![CDATA[</p>]]><![CDATA[");
      tagdocString = GeneralHelpers.Replace(tagdocString, "!!!nbsp!!!", "]]><![CDATA[&nbsp;]]><![CDATA[");
      tagdocString = "<![CDATA[" + tagdocString + "]]>";
      return tagdocString;
    }
  }
}