using System;
using com.plumtree.openfoundation.util;
namespace com.plumtree.portaluiinfrastructure.tags
{
  /// <summary>
  /// This enumeration specifies the various scopes
  /// that can be used to store data in memory.
  /// <p />
  /// The Duration comments specify how long the data
  /// stored in that scope will be maintained, and when
  /// the data will be removed.
  /// <p />
  /// The Visibility comments specify which tags can
  /// access the data stored in that scope.
  /// <p />
  /// Note: Displaying an HTMLElement in a tag and then
  /// caching it so that another tag can add more HTML
  /// to the original tag later is not supported.
  /// HTMLElement trees can be generated and stored for
  /// later use, as long as they are self-contained
  /// trees and used in a read only way.  It is safest
  /// to make clones of a cached HTMLElement tree before
  /// trying to display it again to make sure there are
  /// no threading problems.
  /// </summary>
  public class Scope
  {
    
    /// <summary>
    /// Printable name of the scope.
    /// </summary>
    private String m_strName;
    
    /// <summary>
    /// Tag Scope
    /// <p />
    /// A variable stored in Tag Scope can only
    /// be seen by children of the current tag and will be
    /// removed from memory when the tag is finished.
    /// <p />
    /// I.E. &lt;pt:atag&gt;&lt;pt:btag/&gt;&lt;/pt:atag&gt;&lt;pt:ctag/&gt;
    /// Data stored in Tag Scope by "atag" will be visible
    /// to "btag" but not "ctag".
    /// <p />
    /// Duration: The DisplayTag() method call in
    /// this tag (i.e. The life of the tag).
    /// <br />
    /// Visibility: This tag and all child tags.
    /// </summary>
    public static readonly Scope TAG = new Scope("tag");
    
    /// <summary>
    /// Portlet Request Scope
    /// <p />
    /// A variable stored in Portlet Request Scope will
    /// be visible to all tags in the same portlet as the
    /// current tag, and will be removed from memory when
    /// the portlet is finished displaying.  Tags in other
    /// portlets on the same page will not be able to see
    /// the variable.
    /// <p />
    /// Duration: The portlet request.
    /// <br />
    /// Visibility: All tags in the same portlet.
    /// </summary>
    public static readonly Scope PORTLET_REQUEST = new Scope("portlet request");
    
    /// <summary>
    /// HTTP Request Scope
    /// <p />
    /// A variable stored in HTTP Request Scope will
    /// be visible to all tags in the same HTTP request as the
    /// current tag, and will be removed from memory when
    /// the HTTP request is finished displaying.
    /// <p />
    /// Duration: The HTTP request.
    /// <br />
    /// Visibility: All tags in the same HTTP request.
    /// </summary>
    public static readonly Scope HTTP_REQUEST = new Scope("http request");
    
    /// <summary>
    /// Session Scope
    /// <p />
    /// This data on this scope is cleared when a user logs out and
    /// logs in again.
    /// <p />
    /// If the browser session times out, a new browser is opened, or the HTTP session
    /// is abandoned, this information will not be maintained.
    /// <p />
    /// Duration: The session login (not over session timeouts or session abandonment).
    /// <br />
    /// Visibility: All tags for this user / HTTP Session (unless a user is logged in
    /// multiple times with different HTTP Sessions).
    /// </summary>
    public static readonly Scope SESSION = new Scope("session");
    
    /// <summary>
    /// Persistent Session Scope
    /// <p />
    /// This scope is not cleared of its data on user logout, so be sure not to cache
    /// anything on this scope that could be considered a security risk if it was
    /// leaked to another user.  In practice, however, since HTTP sessions are abandoned
    /// on portal logout, this Scope behaves almost exactly the same as the normal
    /// Session Scope.
    /// <p />
    /// If the browser session times out, a new browser is opened, or the HTTP session
    /// is abandoned, this information will not be maintained.
    /// <p />
    /// NOTE: Clicking logout on the portal will automatically abandon this session.
    /// The only time when sessions are not abandoned is for internal session changes
    /// (E.G. Editing the default profile or multiple guest user customizations).
    /// <p />
    /// Most developers should use the Session Scope for HTTP
    /// Session data storage.
    /// <p />
    /// Duration: The HTTP session (over multiple logins, but not over session timeouts
    /// or session abandonment).
    /// <br />
    /// Visibility: All tags for this user / HTTP Session (unless a user is logged in
    /// multiple times with different HTTP Sessions).
    /// <br />
    /// </summary>
    /// <deprecated>The Persistent Session is no longer available in tags.  The normal Session scope should be used instead.</deprecated>
    public static readonly Scope PERSISTENT_SESSION = new Scope("persistent session");
    
    /// <summary>
    /// Application Scope
    /// <p />
    /// Note: Data stored in this scope will only be removed
    /// when the application is restarted, so care should be
    /// used when storing data on the application to make sure
    /// it does not become cluttered with large amounts of
    /// data.
    /// <p />
    /// Duration: The lifetime of the application.
    /// <br />
    /// Visibility: All tags for all users.
    /// </summary>
    public static readonly Scope APPLICATION = new Scope("application");
    
    /// <summary>
    /// The order in which Scopes are searched during attribute value
    /// replacement.  Variables are searched first, and then shared
    /// variables.
    /// </summary>
    private static readonly Scope[] SCOPE_SEARCH_ORDER = new Scope[]{Scope.TAG,
     Scope.PORTLET_REQUEST, Scope.HTTP_REQUEST, Scope.SESSION, Scope.PERSISTENT_SESSION,
     Scope.APPLICATION};
    
    /// <summary>
    /// This helper method converts from a String representation
    /// of a scope to a Scope object.  This is not case-sensitive.
    /// <p />
    /// </summary>
    /// <param name="_strName">The name of the scope.</param>
    /// <returns>
    /// Scope The requested Scope object.  Null if the
    /// name is not a valid scope.
    /// </returns>
    public static Scope GetScope(String _strName)
    {
      for (int i = 0;i < SCOPE_SEARCH_ORDER.Length;i++)
      {
        if (0 == XPStringUtility.CompareIgnoreCase(_strName, SCOPE_SEARCH_ORDER[i].m_strName))
        {
          return SCOPE_SEARCH_ORDER[i];
        }
      }
      return null;
    }
    
    /// <summary>
    /// Private constructor to ensure that instances of
    /// this class form a specific enumeration of values.
    /// </summary>
    private Scope(String _strName)
    {
      m_strName = _strName;
    }
    
    /// <summary>
    /// Human readable name of the Scope.
    /// </summary>
    /// <returns>String Human readable name of the Scope</returns>
    public override String ToString()
    {
      return m_strName;
    }
    
    /// <summary>
    /// Returns the order in which Scopes are searched during attribute
    /// value replacement.  Variables are searched first, and then shared
    /// variables.
    /// </summary>
    /// <returns>The Scope search order.</returns>
    public static Scope[] GetScopeSearchOrder()
    {
      return SCOPE_SEARCH_ORDER;
    }
  }
}