using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.tags.helper;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.tags.displaytree
{
  /// <summary>
  /// This class contains multiple Display Elements.
  /// This is a framework only class and should not be
  /// used by external developers.
  /// </summary>
  /// <author>donh</author>
  public class CompositeDisplayElement : IDisplayElement
  {
    
    /// <summary>
    /// The child display elements of this element.
    /// </summary>
    protected XPArrayList m_DisplayElements = null;
    
    /// <summary>
    /// The immediate parent tag of this element.
    /// </summary>
    protected TagDisplayElement m_ParentTag = null;
    
    /// <summary>
    /// The container that this element is stored in.  This
    /// may be an internal child container of the parent tag
    /// element.
    /// </summary>
    protected CompositeDisplayElement m_ParentContainer = null;
    
    /// <summary>
    /// Constructor for CompositeDisplayElement.  This should
    /// only be called by the tag framework.
    /// </summary>
    /// <param name="_parentTag">The immediate parent tag of this element.</param>
    /// <param name="_parentContainer">
    /// The container that this element is
    /// stored in.  This may be an internal child container of the
    /// parent tag element.
    /// </param>
    public CompositeDisplayElement(TagDisplayElement _parentTag, CompositeDisplayElement _parentContainer)
    {
      // Note: We only need to initialize this array if this
      // tag will contain child Display Elements.  We could
      // do a lazy initialization of this array if necessary.
      m_DisplayElements = new XPArrayList();
      m_ParentTag = _parentTag;
      m_ParentContainer = _parentContainer;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.displaytree.IDisplayElement#Display(TagVisitor)"></seealso>
    public virtual HTMLElement Display(TagVisitor _tagVisitor)
    {
      HTMLElementCollection result = new HTMLElementCollection();
      IXPEnumerator elements = m_DisplayElements.GetEnumerator();
      while (elements.MoveNext())
      {
        result.AddInnerHTMLElement(((IDisplayElement) elements.GetCurrent()).Display(_tagVisitor));
        }
      return result;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.displaytree.IDisplayElement#Release()"></seealso>
    public virtual void Release()
    {
      IXPEnumerator elements = m_DisplayElements.GetEnumerator();
      while (elements.MoveNext())
      {
        ((IDisplayElement) elements.GetCurrent()).Release();
      }
    }
    
    /// <summary>
    /// Add a child display element to this container.
    /// </summary>
    /// <param name="IDisplayElement">_child The child to add</param>
    public virtual void AddChildElement(IDisplayElement _child)
    {
      m_DisplayElements.Add(_child);
    }
    
    /// <summary>
    /// Get the parent tag of this container.
    /// The parent tag is the next TagDisplayElement
    /// above this element in the display tree.
    /// </summary>
    /// <returns>CompositeDisplayElement The parent tag.</returns>
    public virtual TagDisplayElement GetParentTag()
    {
      return m_ParentTag;
    }
    
    /// <summary>
    /// Get the parent container of this container.
    /// The parent container is the next CompositeDisplayElement
    /// above this element in the display tree, not the next
    /// element (which will often be a TagDisplayElement), but
    /// rather the composite element above that one.
    /// </summary>
    /// <returns>CompositeDisplayElement The parent container.</returns>
    public virtual CompositeDisplayElement GetParentContainer()
    {
      return m_ParentContainer;
    }
    
    /// <summary>
    /// Returns the unprocessed display elements
    /// Only used for Tag definitions.  
    /// </summary>
    /// <returns></returns>
    public virtual XPArrayList GetDisplayElements()
    {
      return m_DisplayElements;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.displaytree.IDisplayElement#HasChildElements(ITagMetaData[],Object[],TagLibraryMetaData,int)"></seealso>
    public virtual bool HasChildElements(ITagMetaData[] _childTags, Object[] _results,
     TagLibraryMetaData _library, int _nCurrentDepth)
    {
      IXPEnumerator children = m_DisplayElements.GetEnumerator();
      bool bCompleted = false;
      while (children.MoveNext() && (!bCompleted))
      {
        IDisplayElement child = (IDisplayElement) children.GetCurrent();
        // This allows us to short-circuit the search if we have found
        // all of the child tags.
        bCompleted = child.HasChildElements(_childTags, _results, _library,
         _nCurrentDepth);
      }
      return bCompleted;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.displaytree.IDisplayElement#HasParentElement(ITagMetaData,TagLibraryMetaData)"></seealso>
    public virtual int HasParentElement(ITagMetaData _parentTag, TagLibraryMetaData _library)
    {
      if (null == m_ParentTag)
      {
        return 0;
      }
      // Don't change the depth information here.  We only count PT Tags.
      return m_ParentTag.HasParentElement(_parentTag, _library);
    }
  }
}