using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags.helper;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.tags.displaytree
{
  /// <summary>
  /// This class is used to process the portlet data
  /// and transform the tags.
  /// This is a framework only class and should not be
  /// used by external developers.
  /// </summary>
  /// <author>donh</author>
  public class DisplayTree
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
     "com.plumtree.portaluiinfrastructure.tags.displaytree.DisplayTree");
    
    /// <summary>
    /// Processes the portlet data and transforms
    /// the tags.
    /// </summary>
    /// <param name="AActivitySpace">aSpace The current space</param>
    /// <param name="Object[][]">objArrMarkup The portlet data</param>
    /// <returns>HTMLElement The transformed portlet HTML</returns>
    public static HTMLElement ProcessTagMarkup(AActivitySpace aSpace, Object[][] objArrMarkup)
    {
      IDisplayElement tagTree = BuildDisplayTree(objArrMarkup);
      HTMLElement result = ExecuteTags(tagTree, aSpace);
      return result;
    }
    
    /// <summary>
    /// Processes the portlet data and transforms
    /// the tags.
    /// </summary>
    /// <param name="AActivitySpace">aSpace The current space</param>
    /// <param name="Object[][]">objArrMarkup The portlet data</param>
    /// <returns>HTMLElement The transformed portlet HTML</returns>
    public static HTMLElement GetSharedJavaScriptFromMarkup(AActivitySpace aSpace)
    {
      HTMLElement result = TagState.GetSharedJavascriptCollection(aSpace);
      // This will throw an exception if we try to display this more than once.
      TagState.SetSharedJavascriptCollectionAsDisplayed(aSpace);
      return result;
    }
    
    /// <summary>
    /// Builds the display tree from the portlet data.
    /// This is public for unit testing purposes, and should
    /// never be called by methods outside of this class.
    /// </summary>
    /// <param name="Object[][]">objArrMarkup The portlet data</param>
    /// <returns>IDisplayElement The portlet display tree</returns>
    public static IDisplayElement BuildDisplayTree(Object[][] objArrMarkup)
    {
      // There is no parent, because this is the root Display Element.
      CompositeDisplayElement result = new CompositeDisplayElement(null, null);
      CompositeDisplayElement currentContainer = result;
      // Convert each fragment into a display element
      for (int nFragment = 0;nFragment < objArrMarkup[0].Length;nFragment++)
      {
        int nType = XPConvert.ToInteger(objArrMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_TYPE][nFragment]);
        if (PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_FINISHED == nType)
        {
          // HTML text fragment
          // Create a new display element for the HTML fragment
          // and add it to the display tree.
          currentContainer.AddChildElement(new HTMLDisplayElement(objArrMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT][nFragment]));
          } else
        if (PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_PT_XML == nType)
        {
          // PT XML tag
          Object[][] objXML = (Object[][]) objArrMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT][nFragment];
          int nXMLType = XPConvert.ToInteger(objXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_TYPE][0]);
          if (PT_XML_TYPES.PT_XML_TYPES_PTUI == nXMLType)
          {
            // Opening PT XML tag
            bool bIsSingletonTag = false;
            for (int i = 0;i < objXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_TYPE].Length;
            i++)
            {
              if (PT_XML_TYPES.PT_XML_TYPES_PTUI_CLOSE == XPConvert.ToInteger(objXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_TYPE][i]))
              {
                bIsSingletonTag = true;
              }
            }
            // Make a display element for this tag
            TagDisplayElement currentTag = new TagDisplayElement(objXML, currentContainer);
            currentContainer.AddChildElement(currentTag);
            if (!bIsSingletonTag)
            {
              // If this tag can have HTML inside the tag, set it's child
              // container as the current container so that all new
              // display elements will go inside it.
              // Possible Optimization: The child container is only necessary
              // if there actually are child display elements, so we only need
              // to call this method if there are child elements.
              currentContainer = currentTag.GetChildElements(currentContainer);
              }
          } else
          if (PT_XML_TYPES.PT_XML_TYPES_PTUI_CLOSE == nXMLType)
          {
            // Closing PT XML tag
            CompositeDisplayElement parent = currentContainer.GetParentContainer();
            if (null == parent)
            {
              // This should not happened with a well formed objArrMarkup
              // This means we have more closed tags than open tags and
              // we have a mismatch.
              String strName = (String) objXML[PT_XML_ARRAY_COLS.PT_XML_ARRAY_COLS_NAME][0];
              String strError = "Tag HTML had an extra closed PT Transformer tag (Library.Tag: " + strName + " or an earlier close tag).  Ignoring error.";
              // Add an HTML comment describing the error to the unclosed tag.
              // This will get added to the HTML before the problem tag so it will
              // be easier to find in the HTML source.
              // Create a new display element and add an HTML comment
              // describing the problem.
              currentContainer.AddChildElement(new HTMLDisplayElement("<!-- " + strError + "-->"));
              log.Warn(strError);
            } else
            // Do not set current container to null.  Just ignore this
            // and continue processing.
            {
              // Since we are closing this tag, set the current container as
              // the parent of this one, so all new display elements will
              // go into the parent tag instead of this one.
              currentContainer = parent;
            }
          }
        }
      }
      if (currentContainer != result)
      {
        // We have an unclosed pt tag.
        // Log a warning, but don't fail.  Tags need to be robust.
        // Tag with missing close tag hasn't been processed yet, so we need
        // to calculate the tag name before we can include it in the error.
        String strError;
        TagDisplayElement parentTag = currentContainer.GetParentTag();
        if (null != parentTag)
        {
          try
          {
            strError = "Tag HTML had an unclosed PT Transformer tag (Tag: " + parentTag.GetTagName() + ", Library: " + parentTag.GetTagLibraryName() + ").  Ignoring error.";
            }
          catch (Exception e)
          {
            log.Error(e, "Unable to calculate tag name for unclosed tag.");
            XPException xpe = XPException.GetInstance(e);
            strError = "Tag HTML had an unclosed PT Transformer tag.  Could not determine tag name. Ignoring error. " + xpe.GetMessage() + ": " + xpe.GetStackTrace();
            }
          // Add an HTML comment describing the error to the unclosed tag.
          // This will get added to the HTML before the problem tag so it will
          // be easier to find in the HTML source.
          parentTag.SetErrorMessage(strError);
        } else
        {
          // I don't think this should ever be null, but just in case.
          strError = "Tag HTML had an unclosed PT Transformer tag.  Could not determine tag name. Ignoring error.";
          }
        log.Warn(strError);
      }
      return result;
    }
    
    /// <summary>
    /// Transforms the tags in the display tree.
    /// This is public for unit testing purposes, and should
    /// never be called by methods outside of this class.
    /// </summary>
    /// <param name="IDisplayElement">tagTree The portlet display tree</param>
    /// <param name="AActivitySpace">aSpace The current space</param>
    /// <returns>HTMLElement The transformed portlet HTML</returns>
    public static HTMLElement ExecuteTags(IDisplayElement tagTree, AActivitySpace aSpace)
    {
      // If we reuse the TagState object between portlet requests,
      // we need to reset the portlet request data.
      TagState tagState = new TagState(aSpace.GetApplication(), aSpace.GetSubSession(),
       aSpace.GetPersistentSubSession(), TagState.GetHTTPRequestData(aSpace));
      TagVisitor tagVisitor = new TagVisitor(tagState, aSpace, TagState.GetSharedJavascriptCollection(aSpace));
      HTMLElement result = tagTree.Display(tagVisitor);
      // Allow tag authors to execute cleanup code.
      tagTree.Release();
      return result;
    }
  }
}