using System;
using com.plumtree.openfoundation.util;
namespace com.plumtree.portaluiinfrastructure.tags.helper
{
  /// <summary>
  /// This class separates out the infrastructure classes used
  /// in a tag so that they are not documented with the ATag class.
  /// </summary>
  /// <author>DonH</author>
  public abstract class ATagInfrastructure
  {
    
    /// <summary>
    /// The TagHelper encapsulates the internal Tag
    /// implementation details.  This is private so it doesn't
    /// confuse Tag developers.
    /// </summary>
    private TagHelper m_TagHelper;
    
    /// <summary>
    /// The TagMetaDataHelper encapsulates the internal Tag
    /// Meta Data implementation details.  This is private
    /// so it doesn't confuse Tag developers.  This variable
    /// needs to be in the Tag class itself, not TagHelper
    /// because it is mimicing a static variable and needs
    /// to be set on each instance of this class when it is
    /// constructed.
    /// </summary>
    private TagMetaDataHelper m_TagMetaDataHelper;
    
    /// <summary>
    /// This method releases all data stored in this tag
    /// to prevent excess memory retention and to prepare
    /// the tag for reuse.  This method should return the
    /// tag to the state it was in immediately after the
    /// Create() method was called. After this method is
    /// called, the tag should be able to be re-initialized
    /// and used again.
    /// All tags that store custom data as member variables
    /// should override this method and release the data
    /// here.  If a tag does not store custom data as
    /// member variables, then there is no need to override
    /// this method.
    /// Default Tag Method
    /// </summary>
    public abstract void ReleaseTag();
    
    /// <summary>
    /// The Tag Container uses this method to initialize the
    /// Tag with all of the data it needs for display.  This
    /// is called before the tag is used the first time, and
    /// before it is used after every Release call.
    /// Framework Helper Method
    /// </summary>
    /// <param name="TagHelper">_helper The tag helper for this tag</param>
    public virtual void InitForDisplay(TagHelper _helper)
    {
      m_TagHelper = _helper;
    }
    
    /// <summary>
    /// The Tag Container uses this method to get the
    /// name of this tag. If m_TagMetaDataHelper is not yet
    /// initialized it is initialized by this method.
    /// Framework Helper Method
    /// </summary>
    /// <returns>String The name of this tag.</returns>
    public virtual String GetName()
    {
      return GetTagMetaDataHelper().GetTagMetaData().GetName();
    }
    
    /// <summary>
    /// The Tag Container uses this method to get the
    /// meta data about this tag perform error checking.
    /// Framework Helper Method
    /// </summary>
    /// <returns>TagMetaDataHelper The meta data for this tag.</returns>
    /// <throws>
    /// XPException if an attempt is made to access the 
    /// tag meta data helper before it has been set.  This should
    /// not happen.
    /// </throws>
    public virtual TagMetaDataHelper GetTagMetaDataHelper()
    {
      if (null == m_TagMetaDataHelper)
      {
        throw new XPException("Attempting to access tag meta data helper before it has been set.");
        }
      return m_TagMetaDataHelper;
    }
    
    /// <summary>
    /// This method is used to store the meta data about this tag
    /// and is only called once per tag class when the class is loaded.
    /// Framework Helper Method
    /// </summary>
    /// <param name="TagMetaDataHelper">_tagMetaDataHelper The meta data for this tag.</param>
    public virtual void SetTagMetaDataHelper(TagMetaDataHelper _tagMetaDataHelper)
    {
      // NOTE: This should be stored as a static variable on ATag, 
      // since we are trying to make this simpler for the tag developer,
      // but we cannot do that.  If we put the static class variable in
      // the abstract ATag class, there will only be one instance for
      // all tags, rather than one instance per type of tag.
      m_TagMetaDataHelper = _tagMetaDataHelper;
    }
    
    /// <summary>
    /// The Tag Container uses this method to get the
    /// tag helper class to perform various framework
    /// functions.
    /// Framework Helper Method
    /// </summary>
    /// <returns>TagHelper The helper class for this tag.</returns>
    /// <throws>
    /// XPException if an attempt is made to access the 
    /// tag helper before it has been set.  This should
    /// not happen.
    /// </throws>
    public virtual TagHelper GetTagHelper()
    {
      if (null == m_TagHelper)
      {
        throw new XPException("Attempting to access tag helper before it has been set.");
        }
      return m_TagHelper;
    }
    
    /// <summary>
    /// The Tag Container uses this method to release all
    /// data stored in this tag to prevent excess memory
    /// retention and to prepare the tag for reuse.  After
    /// this method is called, the tag should be able to
    /// be re-initialized and used again.
    /// This method should not be used by Tag developers.
    /// ReleaseTag() should be used instead.
    /// Framework Helper Method
    /// </summary>
    public virtual void Release()
    {
      ReleaseTag();
      m_TagHelper.ReleaseTagVisitor();
      m_TagHelper = null;
    }
  }
}