using System;
using com.plumtree.openfoundation.reflect;
using com.plumtree.openfoundation.util;
namespace com.plumtree.portaluiinfrastructure.tags.helper
{
  /// <summary>
  /// This class wraps shared state variables and
  /// tracking information to provide edit access
  /// checks.
  /// </summary>
  /// <author>DonH</author>
  public class SharedStateVariable
  {
    
    /// <summary>
    /// The actual value to be stored.
    /// </summary>
    protected Object m_oValue;
    
    /// <summary>
    /// The name of the library that has edit access
    /// to this object.  If this is null, it means
    /// all libraries have edit access.
    /// </summary>
    protected String m_strOwningLibraryName;
    
    /// <summary>
    /// Construct a new SharedStateVariable for an
    /// object with no edit access restrictions.
    /// </summary>
    /// <param name="_oValue">The actual value to be stored.</param>
    public SharedStateVariable(Object _oValue)
    {
      m_strOwningLibraryName = null;
      m_oValue = _oValue;
    }
    
    /// <summary>
    /// Construct a new SharedStateVariable for an
    /// object with edit access restricted to the
    /// owning library only.
    /// </summary>
    /// <param name="__strOwningLibraryName">
    /// The name of the
    /// owning library with edit access.
    /// </param>
    /// <param name="_oValue">The actual value to be stored.</param>
    public SharedStateVariable(String _strOwningLibraryName, Object _oValue)
    {
      m_strOwningLibraryName = _strOwningLibraryName;
      m_oValue = _oValue;
    }
    
    /// <summary>
    /// Check whether or not the specified library
    /// has edit access to this object.
    /// </summary>
    /// <param name="_strCurrentLibraryName">
    /// The name of the
    /// library to check security for.
    /// </param>
    /// <param name="_strKey">
    /// The key used to access this variable.
    /// This is used for error reporting purposes only.
    /// </param>
    /// <param name="__strCurrentTagName">
    /// The name of the current tag.
    /// This is used for error reporting purposes only.
    /// </param>
    /// <returns>true implies that the library has edit access</returns>
    /// <throws>
    /// XPIllegalAccessException if the specified library
    /// does not have edit access to this variable.
    /// </throws>
    public virtual void CheckHasEditAccess(String _strCurrentLibraryName,
     String _strKey, String _strCurrentTagName)
    {
      if ((null != m_strOwningLibraryName) && (0 != XPStringUtility.Compare(m_strOwningLibraryName,
       _strCurrentLibraryName)))
      {
        // Does not have edit access to this variable
        throw new XPIllegalAccessException("Illegal attempt to replace the shared state variable " + _strKey + " by tag " + _strCurrentTagName + ".  The library " + _strCurrentLibraryName + " does not have edit access.  Access restricted to the creating library: " + m_strOwningLibraryName + ".");
        }
    }
    
    /// <summary>
    /// Check whether or not access is restricted
    /// to a particular library in this shared state
    /// variable.
    /// </summary>
    /// <returns>boolean Whether or not access is restricted.</returns>
    public virtual bool IsAccessRestricted()
    {
      return (null != m_strOwningLibraryName);
    }
    
    /// <summary>
    /// Gets the value of the stored object.
    /// </summary>
    /// <returns>The value of the stored object.</returns>
    public virtual Object GetValue()
    {
      return m_oValue;
    }
    
    /// <summary>
    /// Updates the value of the stored object.
    /// </summary>
    /// <param name="The">new value for the stored object.</param>
    public virtual void SetValue(Object _value)
    {
      m_oValue = _value;
    }
  }
}