using System;
using com.plumtree.openfoundation.reflect;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.xml.dom;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.helper;
using com.plumtree.portaluiinfrastructure.tags.metadata;
namespace com.plumtree.portaluiinfrastructure.tags.manager
{
  /// <summary>
  /// The TagLibManager is a singleton class that returns tags.
  /// This uses dynamic discovery to import all of the ATag and
  /// ITagLibrary objects in certain locations, and then maintain
  /// a hashmap of clones, with possible resource pooling.
  /// </summary>
  /// <author>donh</author>
  /// <author>caiusv</author>
  public class TagLibManager : ITagLibManager
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.tags.manager.TagLibManager");
    
    /// <summary>
    /// ATag abstract class
    /// </summary>
    private const String ATAG = "com.plumtree.portaluiinfrastructure.tags.ATag";
    
    /// <summary>
    /// ITagLibrary Interface
    /// </summary>
    private const String ITAGLIBRARY = "com.plumtree.portaluiinfrastructure.tags.metadata.ITagLibrary";
    
    /// <summary>
    /// name of the ITagLibraryMetaData interface used by dynamic discovery
    /// </summary>
    private const String ITAGLIBRARYDATA = "com.plumtree.portaluiinfrastructure.tags.metadata.ITagLibraryMetaData";
    
    /// <summary>
    /// Constant for portaluiinfrastructure package name
    /// Used to find ATag abstract class and ITagLibrary interface
    /// </summary>
    private const String PORTALUIINFRASTRUCTURE = "portaluiinfrastructure";
    private const char LIB_VERSION_SEPARATOR = '.';
    
    /// <summary>
    /// Use this class static to access the one and only instance of
    /// TagLibManager.
    /// </summary>
    private static ITagLibManager INSTANCE;
    
    /// <summary>
    /// This stores whether or not we are running in test mode.
    /// </summary>
    private static bool m_bIsProduction = false;
    
    /// <summary>
    /// Map of the Tags
    /// </summary>
    private XPHashtable m_xphTags = new XPHashtable();
    
    /// <summary>
    /// Map of Libraries and version for quick library & version lookup.
    /// Key is the library name and value is an arraylist of versions.
    /// </summary>
    private XPHashtable m_xphLibraries = new XPHashtable();
    
    /// <summary>
    /// Map of the lib files used to load each individual library.  Used
    /// to reload libraries.  Key is the library name, the LIB_VERSION_SEPARATOR,
    /// and the version.  The value is the name of the lib file.
    /// </summary>
    private XPHashtable m_xphLibraryLibFileNames = new XPHashtable();
    
    /// <summary>
    /// The directory used to load normal tag libraries.
    /// </summary>
    private String m_strLibDir = null;
    
    /// <summary>
    /// The config file used to load normal tag libraries.
    /// </summary>
    private String m_strConfigFile = null;
    
    /// <summary>
    /// The directory used to load custom tag libraries.
    /// </summary>
    private String m_strCustomLibDir = null;
    
    /// <summary>
    /// The config file used to load custom tag libraries.
    /// </summary>
    private String m_strCustomConfigFile = null;
    
    /// <summary>
    /// Whether or not the TagLibManager has been initialized.
    /// </summary>
    private bool m_bInitialized = false;
    
    /// <summary>
    /// Counters used in the diagnostic page.
    /// </summary>
    private int m_nNumCustomTagLibraries = 0;
    private int m_nNumCustomTags = 0;
    
    /// <summary>
    /// Tag Library Meta Data type
    /// </summary>
    private XPType m_TagLibraryMetaDataType = new XPType(typeof(TagLibraryMetaData));
    
    /// <summary>
    /// Empty string
    /// </summary>
    private const String EMPTY = "";
    
    /// <summary>
    /// Private constructor to ensure there is only one instance.
    /// </summary>
    private TagLibManager()
    {
    }
    
    /// <returns>the singleton TagLibManager instance.</returns>
    public static ITagLibManager getInstance()
    {
      if (INSTANCE == null)
      {
        m_bIsProduction = true;
        INSTANCE = new TagLibManager();
      }
      return INSTANCE;
    }
    
    /// <summary>
    /// Unit tests should use this method to overwrite/teardown existing singleton.
    /// DO NOT USE for production code. 
    /// </summary>
    public static void setTestInstance(ITagLibManager testManager)
    {
      if (m_bIsProduction)
      {
        String strMessage = "Can\'t override TaglibManager instance for testing... TagLibManager is running in production mode.";
        log.Error(strMessage);
        throw new XPException(strMessage);
      }
      INSTANCE = testManager;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#GetTag(java.lang.String, java.lang.String, double)
     */
     
    public virtual ATag GetTag(String _strTagName, String _strLibraryName,
     double _dLibraryVersion)
    {
      ATag storedTag = null;
      ATag result = null;
      LibKey myLibKey = new LibKey(_strLibraryName, _dLibraryVersion, _strTagName);
      try
      {
        try
        {
          Object temp = m_xphTags.GetElement(myLibKey);
          storedTag = (ATag) temp;
        }
        catch (Exception ex)
        {
          throw XPException.GetInstance(ex);
        }
      }
      catch (XPClassCastException ex)
      {
        log.Error(ex, "GetTag: XPClassCastException: The tag " + _strTagName + " in library " + _strLibraryName + " is being miscast.");
        }
      catch (XPException e)
      {
        log.Error(e, "Couldn\'t retrieve tag " + _strTagName + " from library " + _strLibraryName + ".");
        }
      if (null == storedTag)
      {
        log.Error("Couldn\'t retrieve tag " + _strTagName + " from library " + _strLibraryName + ". Tag and Library names are case-sensitive.  Many IDEs automatically change the casing of tag names.");
        return null;
      }
      try
      {
        // Note: This could possibly be implemented differently
        // so that Tag developers wouldn't need to write a Create()
        // method for every tag.  This would solve some problems
        // people have when overriding tags.  However, there are
        // definite performance issues to the most obvious solution,
        // and since this is called many times per page, we need to
        // be very careful about performance.
        // If we do the following:
        // XPType tagType = new XPType(this.getClass());
        // ATag result = (ATag) tagType.CreateInstance();
        // Tag authors don't need a Create() method, but it's 6 times slower
        // on my machine.
        // Another possibility is to do Tag pooling in the manager
        // so that the reflection would only happen when the pool
        // size increased.
        result = storedTag.Create();
        result.SetTagMetaDataHelper(storedTag.GetTagMetaDataHelper());
      }
      catch (Exception e)
      {
        log.Error(e, "Couldn\'t create tag " + _strTagName + " from library " + _strLibraryName + ".");
        result = null;
      }
      return result;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#GetTag(java.lang.String, java.lang.String)
     */
     
    public virtual ATag GetTag(String _strTagName, String _strLibraryName)
    {
      // look in m_xphLibraries to see if there are libraries with this name
      Object myLibMetadataObject = m_xphLibraries.GetElement(_strLibraryName);
      // if there aren't any then log that
      if (null == myLibMetadataObject)
      {
        log.Error("No such library " + _strLibraryName + "  found.  Library names are case-sensitive.  Many IDEs automatically change the casing of tag names.");
        return null;
      } else
      {
        // get the arraylist of versions
        XPArrayList myLibMetadataArray = (XPArrayList) myLibMetadataObject;
        if (myLibMetadataArray.GetSize() == 1)
        {
          // if there's one then use that one
          double myLibVersion = ((TagLibraryMetaData) myLibMetadataArray.GetElement(0)).GetVersion();
          return GetTag(_strTagName, _strLibraryName, myLibVersion);
        } else
        {
          // if there's more than one then use the highest
          double highestLibVersion = 0.0;
          for (int i = 1;i < myLibMetadataArray.GetSize();i++)
          {
            double myLibVersion = ((TagLibraryMetaData) myLibMetadataArray.GetElement(i)).GetVersion();
            if (myLibVersion > highestLibVersion)
            {
              highestLibVersion = myLibVersion;
            }
          }
          return GetTag(_strTagName, _strLibraryName, highestLibVersion);
          }
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#HasTag(java.lang.String, java.lang.String, double)
     */
     
    public virtual bool HasTag(String _strTagName, String _strLibraryName,
     double _dLibraryVersion)
    {
      ATag myTag = GetTag(_strTagName, _strLibraryName, _dLibraryVersion);
      if (null == myTag)
      {
        return false;
      }
      return true;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#HasTag(java.lang.String, java.lang.String)
     */
     
    public virtual bool HasTag(String _strTagName, String _strLibraryName)
    {
      ATag myTag = GetTag(_strTagName, _strLibraryName);
      if (null == myTag)
      {
        return false;
      }
      return true;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#LoadClasses(java.lang.String, java.lang.String)
     */
     
    public virtual void LoadClasses(String strLibDir, String strConfigFile)
    {
      if (m_bInitialized)
      {
        log.Error("TagLibManager LoadClasses called twice.");
        return;
      }
      m_bInitialized = true;
      log.Info("Loading from library: " + strLibDir);
      log.Info("Loading config file: " + strConfigFile);
      XPDocument document = XPBuilder.LoadDocument(strConfigFile);
      if (document != null)
      {
        InternalLoadClasses(strLibDir, document, false);
      } else
      {
        log.Info("No TagLib xml file found at " + strLibDir);
      }
      m_strLibDir = strLibDir;
      m_strConfigFile = strConfigFile;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#LoadCustomClasses(java.lang.String, java.lang.String)
     */
     
    public virtual void LoadCustomClasses(String strLibDir, String strConfigFile)
    {
      log.Info("Loading from library: " + strLibDir);
      log.Info("Loading config file: " + strConfigFile);
      XPDocument document = XPBuilder.LoadDocument(strConfigFile);
      if (document != null)
      {
        // Reset custom object counters
        m_nNumCustomTagLibraries = 0;
        m_nNumCustomTags = 0;
        InternalLoadClasses(strLibDir, document, true);
      } else
      {
        log.Info("No custom TagLib xml file found at " + strLibDir);
      }
      m_strCustomLibDir = strLibDir;
      m_strCustomConfigFile = strConfigFile;
    }
    
    /// <summary>
    /// This method loads the lib files from an XML document into the
    /// TagLibManager.
    /// </summary>
    /// <param name="strLibDir">The file directory where jars and dlls can be found</param>
    /// <param name="XPDocument">the config file to load from</param>
    /// <param name="boolean">
    /// bOverWrite false implies error if a tag overwrite is attempted
    /// true implies that the Custom Tag Counter will be incremented for these classes.
    /// </param>
    private void InternalLoadClasses(String strLibDir, XPDocument document,
     bool bOverwrite)
    {
      int i;
      int length;
      XPNodeList libFiles = document.GetElementsByTagName("libfile");
      if (libFiles == null)
      {
        // No library files were found
        log.Warn("No tag library files were found.");
        return;
      }
      if (bOverwrite)
      {
        // Reset the custom object counters
        m_nNumCustomTagLibraries = 0;
        m_nNumCustomTags = 0;
      }
      for (int j = 0;j < libFiles.GetLength();j++)
      {
        String strLibFileName = ((XPElement) libFiles.Item(j)).GetAttributeNode("name").GetNodeValue();
        InternalLoadLibFile(strLibDir, strLibFileName, bOverwrite);
      }
    }
    
    /// <summary>
    /// This method loads the lib files from an XML document into the
    /// TagLibManager.
    /// </summary>
    /// <param name="strLibDir">The file directory where jars and dlls can be found</param>
    /// <param name="strLibFileName">the name of the file to load</param>
    /// <param name="boolean">
    /// bOverWrite false implies error if a tag overwrite is attempted
    /// true implies that the Custom Tag Counter will be incremented for these classes.
    /// </param>
    private void InternalLoadLibFile(String strLibDir, String strLibFileName,
     bool bOverwrite)
    {
      TagLibraryMetaData tagLibraryMetaData = null;
      log.Info("Loading lib file: " + strLibFileName);
      Object[] tags = null;
      bool bTagLibCollisionFailure = false;
      try
      {
        tagLibraryMetaData = GetLibMetaData(strLibDir, strLibFileName);
        if (null == tagLibraryMetaData)
        {
          log.Fatal("Unable to load lib file: " + strLibFileName + " from lib dir: " + strLibDir);
          throw new XPException("Fatal error loading lib file " + strLibFileName + ": Unable to locate Tag Library meta data.");
          }
        if (!CacheLibName(tagLibraryMetaData, bOverwrite))
        {
          bTagLibCollisionFailure = true;
        } else
        {
          CacheLibFileName(tagLibraryMetaData, strLibFileName);
          tags = XPDynamicDiscovery.GetInstancesFromLib(strLibDir, PORTALUIINFRASTRUCTURE,
           strLibFileName, ATAG);
          if (bOverwrite)
          {
            // Increment the number of custom tag libraries loaded.
            m_nNumCustomTagLibraries++;
          }
        }
      }
      catch (Exception ex)
      {
        log.Fatal(ex, "Fatal Exception loading lib file: " + strLibFileName);
        throw new XPException("Fatal error loading lib file " + strLibFileName + ": Unable to locate Tag Library meta data.",
         ex);
      }
      // If we are unable to load this tag library because of a collision
      if (bTagLibCollisionFailure)
      {
        log.Fatal("Attempt to load duplicate tag library from xml file " + "Tags.xml: " + strLibFileName);
        throw new XPException("Attempt to load duplicate tag library from xml file " + "Tags.xml: " + strLibFileName);
        }
      if (null == tagLibraryMetaData)
      {
        log.Fatal("Fatal error loading lib file " + strLibFileName + ": Unable to locate Tag Library meta data.");
        throw new XPException("Fatal error loading lib file " + strLibFileName + ": Unable to locate Tag Library meta data.");
        }
      log.Debug("Putting Tags from lib files into hash tables");
      if (tags != null)
      {
        // Load Tags
        ATag tag;
        int length = tags.Length;
        for (int i = 0;i < length;i++)
        {
          tag = (ATag) tags[i];
          AddTag(tag, tagLibraryMetaData, bOverwrite);
        }
      }
    }
    
    /// <summary>
    /// Adds the given TagLibraryMetaData to a hashtable of TagLibraryMetaData
    /// objects, m_xphLibraries. m_xphLibraries uses the library name as a key.
    /// The value is an array of TagLibraryMetaData. So potentially more
    /// than one library version number can be held for each library name.
    /// </summary>
    /// <param name="_tlmd">TagLibraryMetaData to cache</param>
    /// <param name="bOverwrite">
    /// boolean overwrite existing tag or not
    /// false implies that if a duplicate tag library is passed in,
    /// this method will return a failure value.
    /// </param>
    /// <returns>
    /// boolean true implies success, false implies that a duplicate
    /// tag library was passed in while _bOverwrite was false.
    /// </returns>
    private bool CacheLibName(TagLibraryMetaData _tlmd, bool _bOverwrite)
    {
      // see if there's a value in the hashtable with this key
      if (null != m_xphLibraries.GetElement(_tlmd.GetName()))
      {
        XPArrayList myArrayList = (XPArrayList) m_xphLibraries.GetElement(_tlmd.GetName());
        // but first make sure no library with this same name and version have been
        // added already.
        for (int i = 0;i < myArrayList.GetSize();i++)
        {
          TagLibraryMetaData myTagLibraryMetaData = (TagLibraryMetaData) myArrayList.GetElement(i);
          if (myTagLibraryMetaData.Equals(_tlmd) && !_bOverwrite)
          {
            log.Fatal("Tag library " + _tlmd.GetDisplayName() + " " + _tlmd.GetVersion() + " loaded previously.");
            // failure
            return false;
          }
        }
        //then add this lib version to the arraylist
        myArrayList.Add(_tlmd);
        m_xphLibraries.PutElement(_tlmd.GetName(), myArrayList);
      } else
      {
        // if there isn't then create an arraylist and add it
        XPArrayList myArrayList = new XPArrayList(1);
        myArrayList.Add(_tlmd);
        m_xphLibraries.PutElement(_tlmd.GetName(), myArrayList);
      }
      // success
      return true;
    }
    
    /// <param name="_strLibName"></param>
    /// <param name="_dVersion"></param>
    private void ClearLibName(String _strLibName, double _dVersion)
    {
      // see if there's a value in the hashtable with this key
      if (null != m_xphLibraries.GetElement(_strLibName))
      {
        XPArrayList myArrayList = (XPArrayList) m_xphLibraries.GetElement(_strLibName);
        // but first make sure no library with this same name and version have been
        // added already.
        bool bAdded = false;
        for (int i = 0;i < myArrayList.GetSize();i++)
        {
          TagLibraryMetaData myTagLibraryMetaData = (TagLibraryMetaData) myArrayList.GetElement(i);
          if (myTagLibraryMetaData.GetVersion() == _dVersion)
          {
            myArrayList.RemoveAt(i);
            return;
          }
        }
      }
    }
    
    /// <summary>
    /// Caches the lib file name (jar/dll) for the library and version.
    /// </summary>
    /// <param name="_tlmd">TagLibraryMetaData to cache</param>
    /// <param name="_strLibFileName">The name of the library file used to load the tag lib.</param>
    private void CacheLibFileName(TagLibraryMetaData _tlmd, String _strLibFileName)
    {
      m_xphLibraryLibFileNames.PutElement(_tlmd.GetName() + LIB_VERSION_SEPARATOR + _tlmd.GetVersion(),
       _strLibFileName);
    }
    
    /// <summary>
    /// Adds a tag to the tag hashtable. The key to the tag is a LibKey
    /// object made from the library name, tag name and version. 
    /// </summary>
    /// <param name="_tag">Atag, tag to add</param>
    /// <param name="_tagLibMetaDAta,">meta data for the current library</param>
    /// <param name="bOverwrite">
    /// boolean overwrite existing tag or not
    /// true implies that the Custom Tag Counter will be incremented for these classes.
    /// </param>
    private void AddTag(ATag _tag, TagLibraryMetaData _tagLibMetaData, bool bOverwrite)
    {
      double dLibVersion = GetLibVersion(_tagLibMetaData);
      String strLibName = GetLibName(_tagLibMetaData);
      if (dLibVersion <= 0.0)
      {
        // Tag library has an invalid version.
        log.Error("Tag library" + _tagLibMetaData.GetDisplayName() + " does not have a valid version.  Ignoring.");
        return;
      }
      if (null == strLibName || XPStringUtility.Compare(strLibName, "") == 0)
      {
        // Tag library has an invalid name.
        log.Error("Tag library" + _tagLibMetaData.GetDisplayName() + " does not have a valid name.  Ignoring.");
        return;
      }
      Object oldObject = null;
      String strObjectName;
      if (null != _tag)
      {
        try
        {
          // The meta data must be discovered before we
          // can query for the name of the tag.
          TagMetaDataHelper tagMetaDataHelper = new TagMetaDataHelper();
          if (tagMetaDataHelper.Init(_tag, _tagLibMetaData))
          {
            // Store the newly initialized tag meta data helper on the tag.
            _tag.SetTagMetaDataHelper(tagMetaDataHelper);
            strObjectName = _tag.GetName();
            if ((strObjectName != null) && (!"".Equals(strObjectName)))
            {
              LibKey myLibKey = new LibKey(strLibName, dLibVersion, strObjectName);
              oldObject = m_xphTags.PutElement(myLibKey, _tag);
              if (!bOverwrite && (oldObject != null))
              {
                log.Warn("Loaded tag " + strObjectName + " twice.  This is usually caused by two Tags in the same library sharing the same tag name.");
                }
              if (bOverwrite)
              {
                // Increment the number of custom tags loaded.
                m_nNumCustomTags++;
              }
            } else
            {
              // Tag has an invalid Tag name.
              log.Error("Tag " + _tag + " does not have a valid tag name.  Ignoring.");
              }
          } else
          {
            // Unable to load Tag meta data (and therefore cannot
            // access tag name)
            log.Error("Unable to load Tag meta data for tag " + _tag + ".  Ignoring.");
            }
        }
        catch (Exception ex)
        {
          log.Error(ex, "Invalid Tag. Ignoring.");
        }
      }
    }
    
    /// <summary>
    /// Attempt to dervive the version number from the TagLibraryMetaData
    /// from the library. If there's a problem or it doesn't exist, log an
    /// error and return 1.0
    /// </summary>
    /// <param name="_tagLibMetaData">The meta data for the current Tag Library</param>
    /// <returns>double The version of the tag library or 1.0</returns>
    private double GetLibVersion(TagLibraryMetaData _tagLibMetaData)
    {
      try
      {
        return _tagLibMetaData.GetVersion();
      }
      catch (Exception ex)
      {
        log.Error(ex, "couldn\'t find version number in library " + _tagLibMetaData.GetDisplayName() + ".  Using 1.0");
        }
      return 1.0;
    }
    
    /// <summary>
    /// Attempt to dervive the name from the TagLibraryMetaData from the
    /// library. If there's a problem or it doesn't exist, log an error and return null
    /// </summary>
    /// <param name="_tagLibMetaData">The meta data for the current Tag Library</param>
    /// <returns>String the name of the tag library or null</returns>
    private String GetLibName(TagLibraryMetaData _tagLibMetaData)
    {
      try
      {
        return _tagLibMetaData.GetName();
      }
      catch (Exception ex)
      {
        log.Error(ex, "couldn\'t find version number in library " + _tagLibMetaData.GetDisplayName());
        }
      return null;
    }
    
    /// <summary>
    /// Return a TagLibraryMetaData object found in _strLibDir with the name _strLibName.
    /// </summary>
    /// <param name="_strLibDir">
    /// the name of the directory or file (ie .war file) where the library
    /// is found.
    /// </param>
    /// <param name="_strLibName">the name of the library whose metadata is returned.</param>
    /// <returns>
    /// the LibraryMetaData object found in the specified library or null if 
    /// it is not found
    /// </returns>
    private TagLibraryMetaData GetLibMetaData(String _strLibDir, String _strLibName)
    {
      XPType myTagLibraryMetaDataType = new XPType(typeof(TagLibraryMetaData));
      TagLibraryMetaData myTagLibraryMetaData = null;
      try
      {
        Object[] libObjectArray = XPDynamicDiscovery.GetInstancesFromLib(_strLibDir,
         PORTALUIINFRASTRUCTURE, _strLibName, ITAGLIBRARYDATA);
        if ((null == libObjectArray) || (0 == libObjectArray.Length))
        {
          log.Error(_strLibDir + "/" + _strLibName + " must have exactly one class with tag library meta data.  Could not find meta data.");
          return null;
        } else
        if (1 != libObjectArray.Length)
        {
          log.Error(_strLibDir + "/" + _strLibName + " must have exactly one class with tag library meta data.  Too many meta data classes found.");
          return null;
        }
        Object libObject = libObjectArray[0];
        XPType myType = new XPType(libObject.GetType());
        XPField[] fieldlist = myType.GetPublicFields();
        for (int i = 0;i < fieldlist.Length;i++)
        {
          XPField fld = fieldlist[i];
          // if this is a library data object get the taglibrarymetadata
          if (fld.GetFieldType().Equals(myTagLibraryMetaDataType))
          {
            myTagLibraryMetaData = (TagLibraryMetaData) fld.GetValue(new TagLibraryMetaData(EMPTY,
             EMPTY, EMPTY, 0.0));
          }
        }
        return myTagLibraryMetaData;
      }
      catch (Exception ex)
      {
        log.Error(ex, "couldn\'t find library meta data " + _strLibName);
        }
      return null;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#GetLibraries()
     */
     
    public virtual XPHashtable GetLibraries()
    {
      return m_xphLibraries;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#GetTagLibMetadata(java.lang.String, double)
     */
     
    public virtual TagLibraryMetaData GetTagLibMetadata(String _sLibname,
     double _dVersion)
    {
      XPArrayList myLibMetadatList = (XPArrayList) m_xphLibraries.GetElement(_sLibname);
      if (null == myLibMetadatList)
      {
        log.Error("no such library found " + _sLibname);
        return null;
      } else
      {
        for (int i = 0;i < myLibMetadatList.GetSize();i++)
        {
          TagLibraryMetaData myTagLibraryMetaData = (TagLibraryMetaData) myLibMetadatList.GetElement(i);
          if (myTagLibraryMetaData.GetVersion() == _dVersion)
          {
            return myTagLibraryMetaData;
          }
        }
        log.Error("no such library found " + _sLibname + " with version " + _dVersion);
        return null;
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#GetLibraryTagList(java.lang.String, double)
     */
     
    public virtual XPArrayList GetLibraryTagList(String _sLibname, double _dVersion)
    {
      if (null == _sLibname || _sLibname.Equals(""))
      {
        log.Error("Invalid library name " + _sLibname + ".");
        return null;
      } else
      if (_dVersion <= 0.0)
      {
        log.Error("Invalid library version " + _dVersion + ".");
        return null;
      } else
      {
        XPArrayList myTagArray = new XPArrayList();
        IXPEnumerator xpEnumerator = m_xphTags.GetKeys();
        while (xpEnumerator.MoveNext())
        {
          LibKey myLibKey = (LibKey) xpEnumerator.GetCurrent();
          if (myLibKey.GetLibName().Equals(_sLibname) && myLibKey.GetVersion() == _dVersion)
          {
            myTagArray.Add(myLibKey.GetTagName());
          }
        }
        if (myTagArray.GetSize() > 0)
        {
          return myTagArray;
        } else
        {
          log.Error(" No tags found for " + _sLibname + " version " + _dVersion);
          return null;
        }
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#GetLibraryTags(java.lang.String, double)
     */
     
    public virtual XPArrayList GetLibraryTags(String _sLibname, double _dVersion)
    {
      if (null == _sLibname || _sLibname.Equals(""))
      {
        log.Error("Invalid library name " + _sLibname + ".");
        return null;
      } else
      if (_dVersion <= 0.0)
      {
        log.Error("Invalid library version " + _dVersion + ".");
        return null;
      } else
      {
        XPArrayList myTagArray = new XPArrayList();
        IXPEnumerator xpEnumerator = m_xphTags.GetKeys();
        while (xpEnumerator.MoveNext())
        {
          LibKey myLibKey = (LibKey) xpEnumerator.GetCurrent();
          if (myLibKey.GetLibName().Equals(_sLibname) && myLibKey.GetVersion() == _dVersion)
          {
            myTagArray.Add((ATag) m_xphTags.GetElement(myLibKey));
          }
        }
        if (myTagArray.GetSize() > 0)
        {
          return myTagArray;
        } else
        {
          log.Error(" No tags found for " + _sLibname + " version " + _dVersion);
          return null;
        }
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#GetTagMetadata(java.lang.String, double, java.lang.String)
     */
     
    public virtual void GetTagMetadata(String _sLibname, double _dVersion,
     String _sTagname)
    {
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#ReloadTagLibrary(java.lang.String, double)
     */
     
    public virtual void ReloadTagLibrary(String _sLibname, double _dVersion)
    {
      // Clear the library name from the cache
      ClearLibName(_sLibname, _dVersion);
      String strLibFileName = (String) m_xphLibraryLibFileNames.GetElement(_sLibname + LIB_VERSION_SEPARATOR + _dVersion);
      // For now we assume that all tag libraries came from the
      // same lib dir, which might not be true in the future.
      InternalLoadLibFile(m_strLibDir, strLibFileName, true);
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#ReloadAllTagLibraries()
     */
     
    public virtual void ReloadAllTagLibraries()
    {
      log.Info("Reloading all tag libraries.");
      // TODO: This should be done in a safer way.  Right now,
      // adding tags is fairly safe, because it just replaces them one
      // by one in the hashtable (although then you could get old and
      // new tags mixed together), but the meta data about libraries
      // is too entertwined to be changed safely and easily.
      // The entire tag library data should be reset, in case a tag
      // is removed when the libraries are reloaded.
      m_xphLibraries = new XPHashtable();
      // Load the normal classes again.  We can't use LoadClasses()
      // because it can only be run once.
      XPDocument document = XPBuilder.LoadDocument(m_strConfigFile);
      InternalLoadClasses(m_strLibDir, document, true);
      // Reset custom object counters
      m_nNumCustomTagLibraries = 0;
      m_nNumCustomTags = 0;
      // Load the custom classes
      document = XPBuilder.LoadDocument(m_strCustomConfigFile);
      InternalLoadClasses(m_strCustomLibDir, document, true);
    }
    
    /// <summary>
    /// An object to hold a Tag Library name, version number and and Tag name
    /// used as a key in a hashtable of tags.
    /// </summary>
    /// <author>caiusv</author>
    private class LibKey : Object
    {
      // the tag library version
      private double m_dLibVersion;
      // the tag libraray name
      private String m_strLibName;
      // the tag name
      private String m_strTagName;
      
      /// <summary>
      /// Construct a new LibKey with the supplied name, tag name and version number.
      /// </summary>
      /// <param name="_strLibName">name of the tag library</param>
      /// <param name="_dLibVersion">tag library version</param>
      /// <param name="_strTagName">tag name</param>
      internal LibKey(String _strLibName, double _dLibVersion, String _strTagName)
      {
        this.m_strLibName = _strLibName;
        this.m_strTagName = _strTagName;
        this.m_dLibVersion = _dLibVersion;
      }
      
      /// <summary>
      /// LibKey acts as a hashtable key so must override Object's hashCode
      /// method. Returns a hashcode based on m_strLibName's hashcode, 
      /// m_strTagName's hashcode and m_dLibVersion. 
      /// </summary>
      /// <returns>int This object's hashcode</returns>
      public override int GetHashCode()
      {
        int code = m_strTagName.GetHashCode() + m_strLibName.GetHashCode() + (XPConvert.ToInteger(m_dLibVersion * 1000));
        return code;
      }
      
      /// <summary>
      /// Returns the library version.
      /// </summary>
      /// <returns>version or the library</returns>
      public virtual double GetVersion()
      {
        return m_dLibVersion;
      }
      
      /// <summary>
      /// Returns the library name.
      /// </summary>
      /// <returns>Returns the library name.</returns>
      public virtual String GetLibName()
      {
        return m_strLibName;
      }
      
      /// <summary>
      /// Returns the tag name.
      /// </summary>
      /// <returns>Returns the tag name.</returns>
      public virtual String GetTagName()
      {
        return m_strTagName;
      }
      
      /// <summary>
      /// Returns true if this tag name library name and library version 
      /// matches this objects attributes. Otherwise returns false.
      /// </summary>
      /// <param name="_object">a LibKey to compare to this object.</param>
      /// <returns>true if the object is equal to this, otherwise false.</returns>
      public override bool Equals(Object _object)
      {
        if ((_object != null) && (_object is LibKey))
        {
          LibKey myLibKey = (LibKey) _object;
          if (this.m_dLibVersion == myLibKey.GetVersion() && this.m_strLibName.Equals(myLibKey.GetLibName()) && this.m_strTagName.Equals(myLibKey.GetTagName()))
          {
            return true;
          }
          return false;
        }
        return false;
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#GetNumCustomTagLibraries()
     */
     
    public virtual int GetNumCustomTagLibraries()
    {
      return m_nNumCustomTagLibraries;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.manager.ITagLibManager#GetNumCustomTags()
     */
     
    public virtual int GetNumCustomTags()
    {
      return m_nNumCustomTags;
    }
  }
}