using System;
using com.plumtree.openfoundation.util;
namespace com.plumtree.portaluiinfrastructure.tags.metadata
{
  /// <summary>
  /// This base class provides programmatic access to the
  /// tag meta data for attributes, including name,
  /// description, type, requirement, and default values.
  /// <p />
  /// Adding public static final member variables of
  /// the various ATagAttribute classes turns on
  /// validation that the tag attributes are being used
  /// correctly, as well as providing useful documentation
  /// information.
  /// </summary>
  public abstract class ATagAttribute
  {
    // TODO: Should this be converted to lowercase?
    protected String m_strName;
    protected String m_strDescription;
    protected readonly AttributeType m_Type;
    
    /// <summary>
    /// Creates a new tag attribute.
    /// </summary>
    /// <param name="_strName">The attribute name.</param>
    /// <param name="_strDescription">The attribute description.</param>
    /// <param name="_type">The attribute type.</param>
    /// <throws>
    /// XPFormatException if the name contains any
    /// whitespace.
    /// </throws>
    /// <throws>
    /// XPIllegalArgumentException if the AttributeType
    /// is null.
    /// </throws>
    protected ATagAttribute(String _strName, String _strDescription, AttributeType _type)
    {
      m_strName = _strName;
      m_strDescription = _strDescription;
      m_Type = _type;
      if (null == m_Type)
      {
        throw new XPIllegalArgumentException("Cannot create Tag Attribute with null AttributeType.  Tag: " + m_strName + " Description: " + m_strDescription + ".");
        }
      CheckNameSafety();
    }
    
    /// <summary>
    /// Gets whether or not this attribute is required to be
    /// present every time the tag is used.  If it is
    /// required, the tag will fail if it is not present.
    /// </summary>
    /// <returns>boolean True means that this attribute is required</returns>
    public abstract bool GetIsRequired();
    
    /// <summary>
    /// The default value for an optional attribute
    /// if the attribute is not present or is of the
    /// incorrect type.  This will return null for
    /// required attributes.
    /// </summary>
    /// <returns>
    /// String The default value for this
    /// attribute.  Null implies there is no default
    /// value.
    /// </returns>
    public abstract String GetDefaultValue();
    
    /// <summary>
    /// The name of the attribute.
    /// </summary>
    /// <returns>String The attribute name</returns>
    public virtual String GetName()
    {
      return m_strName;
    }
    
    /// <summary>
    /// The description of the attribute and
    /// it's behavior.
    /// </summary>
    /// <returns>String The attribute description</returns>
    public virtual String GetDescription()
    {
      return m_strDescription;
    }
    
    /// <summary>
    /// The data type of the attribute.
    /// E.G. int, string, etc...
    /// <p />
    /// Specifying the attribute type will
    /// force the conversion of the attribute
    /// to that type and will cause an error
    /// if it cannot be converted.
    /// <p />
    /// Required attributes will be converted
    /// before the tag is displayed and will
    /// cause the tag to fail if they cannot
    /// be converted correctly.
    /// </summary>
    /// <returns>AttributeType The type of the attribute</returns>
    public virtual AttributeType GetType()
    {
      return m_Type;
    }
    
    /// <summary>
    /// This helper method checks whether or not the
    /// attribute name contains any illegal characters,
    /// such as whitespace.
    /// </summary>
    /// <throws>
    /// XPFormatException if the name contains any
    /// whitespace.
    /// </throws>
    protected virtual void CheckNameSafety()
    {
      // Not the most efficient way to do this, but this
      // method should only be called at startup.  This
      // can be optimized if need be.
      if ((XPStringUtility.IndexOf(m_strName, "\r") >= 0) || (XPStringUtility.IndexOf(m_strName,
       "\n") >= 0) || (XPStringUtility.IndexOf(m_strName, "\t") >= 0) || (XPStringUtility.IndexOf(m_strName,
       " ") >= 0))
      {
        throw new XPFormatException("TagAttribute name cannot contain whitespace.  Tag: " + m_strName + " Description: " + m_strDescription + ".");
        }
    }
  }
}