using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
namespace com.plumtree.portaluiinfrastructure.tags.metadata
{
  /// <summary>
  /// This class provides programmatic access to the
  /// tag library meta data for name and description.
  /// <p />
  /// This class also enforces all lower case for the
  /// names of libraries.
  /// </summary>
  public class TagLibraryMetaData
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.tags.metadata.TagLibraryMetaData");
    protected String m_strDisplayName;
    protected String m_strLibraryName;
    protected String m_strDescription;
    protected double m_nVersion;
    
    /// <summary>
    /// This constructor takes the name, description,
    /// and version of the tag.  It also converts the
    /// library name to all lower case for standards
    /// compliance. 
    /// </summary>
    /// <param name="String">
    /// _strDisplayName The human readable name
    /// of this library for documentation purposes
    /// </param>
    /// <param name="String">
    /// _strLibraryName The library name
    /// for use in tags.  This will be converted to lowercase
    /// </param>
    /// <param name="String">
    /// _strDescription The library description
    /// for documentation.
    /// </param>
    /// <param name="double">_nVersion The version of this library.</param>
    public TagLibraryMetaData(String _strDisplayName, String _strLibraryName,
     String _strDescription, double _nVersion)
    {
      m_strDisplayName = _strDisplayName;
      m_strLibraryName = XPStringUtility.ToLowerCase(_strLibraryName);
      m_strDescription = _strDescription;
      m_nVersion = _nVersion;
      if (!m_strLibraryName.Equals(_strLibraryName))
      {
        log.Warn("Tag Library " + _strLibraryName + " uses uppercase " + "letters in its name, which are automatically converted to " + "lowercase.  This can cause confusion when using " + "tags in HTML.");
        }
    }
    
    /// <summary>
    /// Gets the display name of this tag library.
    /// <p />
    /// This is the human readable name for documentation
    /// purposes, rather than the name used in tags.
    /// <p />
    /// E.G. "The Plumtree Core Tag Library"
    /// </summary>
    /// <returns>String The name of this tag library.</returns>
    public virtual String GetDisplayName()
    {
      return m_strDisplayName;
    }
    
    /// <summary>
    /// Gets the name of this tag library.
    /// <p />
    /// This is the name used to access a particular
    /// tag.
    /// <p />
    /// E.G.
    /// &lt;pt:taglibraryname.tagname/&gt;
    /// </summary>
    /// <returns>String The name of this tag library.</returns>
    public virtual String GetName()
    {
      return m_strLibraryName;
    }
    
    /// <summary>
    /// Gets the description of this tag library.
    /// </summary>
    /// <returns>String The description of this tag library.</returns>
    public virtual String GetDescription()
    {
      return m_strDescription;
    }
    
    /// <summary>
    /// Gets the version of this tag library.
    /// </summary>
    /// <returns>double The version of this tag library</returns>
    public virtual double GetVersion()
    {
      return m_nVersion;
    }
    
    /// <summary>
    /// Return true if the Object _object is a TagLibraryMetaData object and it's name
    /// and version are the same as this TagLibraryMetaData object's name and version.
    /// Otherwise return false.
    /// </summary>
    /// <param name="_object">An object whose equality is being tested.</param>
    /// <returns>true if the objects are equal, otherwise false.</returns>
    public virtual bool Equals(Object _object)
    {
      try
      {
        TagLibraryMetaData myTagLibraryMetaData = (TagLibraryMetaData) _object;
        if (myTagLibraryMetaData.GetName().Equals(GetName()) && myTagLibraryMetaData.GetVersion() == GetVersion())
        {
          return true;
        }
      }
      catch (Exception e)
      {
        return false;
      }
      return false;
    }
  }
}