using System;
namespace com.plumtree.portaluiinfrastructure.tree
{
  /// <author>Don Hayler</author>
  public interface IObjectTreeModel : IObjectTreeModelRO
  {
    
    /// <summary>
    /// This method sets the access level that the user needs to have
    /// to view items in the tree (default is SELECT).
    /// This must be called before SetQueryFilter().
    /// </summary>
    /// <param name="int">
    /// the instance of PT_ACCESS_LEVELS to use
    /// This should be either ADMIN, EDIT, SELECT, or READ otherwise
    /// it will log an error
    /// </param>
    void SetAccessLevel(int nLevel);
    
    /// <summary>
    /// This method sets the community mode for this tree.
    /// </summary>
    /// <seealso cref="IObjectTreeModelRO"></seealso>
    /// <param name="int">nMode the community mode</param>
    /// <param name="int">nID the community folder ID (for THISCOMMUNITY mode)</param>
    void SetCommunityMode(int nMode, int nID);
    
    /// <summary>
    /// This method sets a filter on a specific class.  This is used for
    /// things like distinguishing gadgets from gadget templates (which
    /// share the same classID).  Hiding objects should be done with the
    /// ClassIDsToHide QS argument.
    /// This can only be called for filtering on gadget alignments or widths.
    /// This will create a Query Filter as follows:
    /// [0][0] will be equal to nPropID
    /// [1][0] will be equal to PT_FILTEROPS.PT_FILTEROP_IN
    /// [2][0] will be equal to nValues
    /// Filters will only be applied to objects of type nClassID.
    /// This must be called after SetAccessLevel() and SetClassIDsToDisplay().
    /// </summary>
    /// <param name="int">
    /// nClassID the classID of the class of objects to apply
    /// this filter to
    /// </param>
    /// <param name="Integer">the property ID to filter on</param>
    /// <param name="int[]">the values to filter with.</param>
    void SetQueryFilter(int nClassID, int nPropID, int[] nValues);
    
    /// <summary>
    /// This method sets a filter on a specific class.  This is used for
    /// things like distinguishing gadgets from gadget templates (which
    /// share the same classID).  Hiding objects should be done with the
    /// ClassIDsToHide QS argument.
    /// This can only be called for filtering on data source or web service provider clsids.
    /// This will create a Query Filter as follows:
    /// [0][0] will be equal to nPropID
    /// [1][0] will be equal to PT_FILTEROPS.PT_FILTEROP_IN
    /// [2][0] will be equal to nValues
    /// Filters will only be applied to objects of type nClassID.
    /// This must be called after SetAccessLevel() and SetClassIDsToDisplay().
    /// </summary>
    /// <param name="int">
    /// nClassID the classID of the class of objects to apply
    /// this filter to
    /// </param>
    /// <param name="Integer">the property ID to filter on</param>
    /// <param name="String[]">the values to filter with.</param>
    void SetQueryFilter(int nClassID, int nPropID, String[] sValues);
    
    /// <summary>
    /// This method de/selects the specified tree node
    /// </summary>
    /// <param name="nID">The ID of the node to de/select</param>
    /// <param name="bSelect">true implies select, false deselect</param>
    void SelectNode(int nID, bool bSelect);
    
    /// <summary>
    /// This method resets the tree view to display the browsing tree
    /// rather than the search tree.
    /// </summary>
    void ResetToBrowsingMode();
    
    /// <summary>
    /// This method sets the root id for object type trees
    /// </summary>
    /// <param name="nID">The ID of the root folder/catalog/etc...</param>
    void SetRootFolderID(int nID);
    
    /// <summary>
    /// This method returns the class ID to be filtered.
    /// </summary>
    /// <returns>the class ID to apply the above filter to</returns>
    int GetFilterClassID();
  }
}