using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.server.treeiterator;
namespace com.plumtree.portaluiinfrastructure.tree.sitemap
{
  /// <summary>
  /// This class allows you to iterate through a Site Map Folder hierarchy.
  /// </summary>
  /// <author>Bridget Frey</author>
  public class SiteMapTreeIterator : IPTTreeIterator
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.tree.sitemap.SiteMapTreeIterator");
    
    /// <summary>
    /// Site Map Folder object manager 
    /// </summary>
    protected IPTObjectManager m_objOM;
    
    /// <summary>
    /// User's session. 
    /// </summary>
    protected IPTSession m_objSession;
    
    /// <summary>
    /// Current folder; can be NULL if we move to a folder that is inaccessible. 
    /// </summary>
    protected IPTSiteMapFolder m_objFolder;
    
    /// <summary>
    /// Root of the iteration. 
    /// </summary>
    protected IPTSiteMapFolder m_objRoot;
    
    /// <summary>
    /// Has the class been initialized. 
    /// </summary>
    protected bool m_bInitialized = false;
    
    /// <summary>
    /// Only 0 is recognized. 
    /// </summary>
    protected int m_nStyle = 0;
    
    /// <summary>
    /// Constructor for SiteMapTreeIterator.
    /// </summary>
    public SiteMapTreeIterator() : base()
    {
    }
    
    /// <seealso cref="com.plumtree.server.treeiterator.IPTTreeIterator#Children(int,int)"></seealso>
    public virtual Object[][] Children(int arg0, int arg1)
    {
      if (!CheckInitialized())
      {
        return null;
      }
      if (m_objFolder == null)
      {
        return null;
      }
      IPTQueryResult qrFolders = m_objFolder.QuerySubfolders(false);
      int nNumChildren = qrFolders.RowCount();
      Object[][] objChildren = new Object[10][];
      for (int j2c_index = 0;j2c_index < 10;j2c_index++)
      {
        objChildren[j2c_index] = new Object[nNumChildren];
      }
      for (int i = 0;i < nNumChildren;i++)
      {
        String strName = qrFolders.ItemAsString(i, PT_PROPIDS.PT_PROPID_NAME);
        int lObjectID = qrFolders.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
        String strObjectID = XPConvert.ToString(lObjectID);
        objChildren[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_NAME][i] = strName;
        objChildren[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_IDENTIFIER][i] = strObjectID;
        objChildren[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_TYPE][i] = PT_TREE_ITERATOR_CHILD_TYPES.PT_TREE_ITERATOR_CHILD_TYPE_CONTAINER;
        // -1 signifies that we don't know how many children we have	
        objChildren[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_CHILDCOUNT][i] = -1;
        }
      return objChildren;
    }
    
    /// <seealso cref="com.plumtree.server.treeiterator.IPTTreeIterator#CurrentLocInfo()"></seealso>
    public virtual Object[][] CurrentLocInfo()
    {
      if (!CheckInitialized())
      {
        return null;
      }
      if (m_objFolder == null)
      {
        return null;
      }
      IPTServerContext objSC = (IPTServerContext) m_objFolder.GetInterfaces("IPTServerContext");
      String strObjectID = XPConvert.ToString(objSC.GetObjectID());
      Object[][] objCurrentLocInfo = new Object[10][];
      for (int j2c_index = 0;j2c_index < 10;j2c_index++)
      {
        objCurrentLocInfo[j2c_index] = new Object[1];
      }
      objCurrentLocInfo[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_NAME][0] = m_objFolder.GetLocalizedName();
      objCurrentLocInfo[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_IDENTIFIER][0] = strObjectID;
      objCurrentLocInfo[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_TYPE][0] = PT_TREE_ITERATOR_CHILD_TYPES.PT_TREE_ITERATOR_CHILD_TYPE_CONTAINER;
      objCurrentLocInfo[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_CHILDCOUNT][0] = GetChildCount();
      return objCurrentLocInfo;
    }
    
    /// <seealso cref="com.plumtree.server.treeiterator.IPTTreeIterator#GetAtRoot()"></seealso>
    public virtual bool GetAtRoot()
    {
      if (!CheckInitialized())
      {
        return false;
      }
      if (m_objFolder == null)
      {
        return false;
      }
      IPTServerContext scFolder = (IPTServerContext) m_objFolder.GetInterfaces("IPTServerContext");
      IPTServerContext scRoot = (IPTServerContext) m_objRoot.GetInterfaces("IPTServerContext");
      return scFolder.GetObjectID() == scRoot.GetObjectID();
    }
    
    /// <seealso cref="com.plumtree.server.treeiterator.IPTTreeIterator#GetChildCount()"></seealso>
    public virtual int GetChildCount()
    {
      if (!CheckInitialized())
      {
        return 0;
      }
      if (m_objFolder == null)
      {
        return 0;
      }
      IPTQueryResult qrFolders = m_objFolder.QuerySubfolders(false);
      return qrFolders.RowCount();
    }
    
    /// <seealso cref="com.plumtree.server.treeiterator.IPTTreeIterator#GetLocation()"></seealso>
    public virtual String GetLocation()
    {
      if (!CheckInitialized())
      {
        return "";
      }
      if (m_objFolder == null)
      {
        return "0";
      }
      IPTServerContext objSC = (IPTServerContext) m_objFolder.GetInterfaces("IPTServerContext");
      String strObjectID = XPConvert.ToString(objSC.GetObjectID());
      return strObjectID;
    }
    
    /// <seealso cref="com.plumtree.server.treeiterator.IPTTreeIterator#GetStyle()"></seealso>
    public virtual int GetStyle()
    {
      return m_nStyle;
    }
    
    /// <seealso cref="com.plumtree.server.treeiterator.IPTTreeIterator#InitializeIterator(java.lang.Object)"></seealso>
    public virtual void InitializeIterator(Object arg0)
    {
    }
    
    /// <seealso cref="com.plumtree.server.treeiterator.IPTTreeIterator#MoveTo(java.lang.String)"></seealso>
    public virtual void MoveTo(String arg0)
    {
      if (!CheckInitialized())
      {
        return;
      }
      int nMoveToID = XPConvert.ToInteger(arg0);
      m_objFolder = (IPTSiteMapFolder) m_objOM.Open(nMoveToID, false);
      if (m_objFolder == null)
      {
        log.Warn("SiteMapTreeIterator: Attempted to MoveTo a folder that does not exist, or this user does not have access to the folder.  Will set folder to null.");
        m_objFolder = null;
        return;
      }
    }
    
    /// <seealso cref="com.plumtree.server.treeiterator.IPTTreeIterator#MoveToParent()"></seealso>
    public virtual void MoveToParent()
    {
      if (!CheckInitialized())
      {
        return;
      }
      if (m_objFolder == null)
      {
        // this can happen if someone moved us to a folder we don't have access to, or that
        // failed to open.  so in this case, just send them to the root.
        m_objFolder = m_objRoot;
        return;
      }
      m_objFolder = (IPTSiteMapFolder) m_objOM.Open(m_objFolder.GetParentFolderID(),
       false);
    }
    
    /// <seealso cref="com.plumtree.server.treeiterator.IPTTreeIterator#MoveToRoot()"></seealso>
    public virtual void MoveToRoot()
    {
      if (!CheckInitialized())
      {
        return;
      }
      m_objFolder = m_objRoot;
    }
    
    /// <summary>
    /// Use this function to initialize the Iterator.  Do not use
    /// InitializeIterator, which is on IPTTreeIterator, but does not meet the
    /// needs of this class.
    /// </summary>
    /// <param name="nSiteMapFolderID:">ID of SiteMapFolder to iterate.</param>
    /// <param name="objSession:">User's session.</param>
    public virtual void RealInitializeIterator(int nSiteMapFolderID, IPTSession objSession)
    {
      m_objSession = objSession;
      m_objOM = objSession.GetSiteMapFolders();
      m_objRoot = (IPTSiteMapFolder) m_objOM.Open(nSiteMapFolderID, false);
      // set the current folder equal to the root
      m_objFolder = m_objRoot;
      if ((m_objRoot == null) || (m_objSession == null) || (m_objOM == null))
      {
        log.Warn("SiteMapTreeIterator: Iterator was initialized with null values.");
        return;
      }
      m_bInitialized = true;
    }
    
    /// <seealso cref="com.plumtree.server.treeiterator.IPTTreeIterator#SetStyle(int)"></seealso>
    public virtual void SetStyle(int nStyle)
    {
      m_nStyle = nStyle;
    }
    
    /// <summary>
    /// Test function.
    /// </summary>
    /// <param name="ptTreeIterator:">IPTTreeIterator to iterate.</param>
    /// <param name="strIndent:">
    /// This function is called recursively, and adds spaces to
    /// strIndent for every level of recursion.  Pass in "" for the first call to
    /// the function.
    /// </param>
    public static void printTree(IPTTreeIterator ptTreeIterator, String strIndent)
    {
      Object[][] currentLocation = (Object[][]) ptTreeIterator.CurrentLocInfo();
      String parentName = (String) currentLocation[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_NAME][0];
      log.Info(strIndent + parentName);
      Object[][] childNodes = (Object[][]) ptTreeIterator.Children(0, -1);
      if (null == childNodes)
      {
        return;
      }
      int numChildren = childNodes[0].Length;
      int parentType = XPConvert.ToInteger(currentLocation[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_TYPE][0]);
      if (PT_TREE_ITERATOR_CHILD_TYPES.PT_TREE_ITERATOR_CHILD_TYPE_CONTAINER == parentType)
      {
        for (int i = 0;i < numChildren;i++)
        {
          String childLocation = (String) childNodes[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_IDENTIFIER][i];
          ptTreeIterator.MoveTo(childLocation);
          printTree(ptTreeIterator, strIndent + "  ");
        }
      }
    }
    
    /// <summary>
    /// Check whether Initialized was called.
    /// </summary>
    /// <returns>True if initialized, False if not initialized.</returns>
    private bool CheckInitialized()
    {
      if (m_bInitialized == false)
      {
        log.Warn("SiteMapTreeIterator: Iterator has not been initialized.");
        return false;
      } else
      {
        return true;
      }
    }
  }
}