using System;
using com.plumtree.openfoundation.util;
namespace com.plumtree.portaluiinfrastructure.mock.restapi
{
  /// <summary>
  /// Takes the XPHashtable generated by RestXMLResponse and converts it to a 
  /// human readable string representation of the hashtable content. 
  /// Use the string representation to validate the expected output from the parsed
  /// xml.
  /// Format of the string representation:
  /// Some rules:
  /// 1. Each level adds 2 spaces (not tabs) as indentation
  /// 2. Arrays and nested child nodes has '+' instead of the '-' used by simple text data
  /// 3. Array elements are denoted by '[n]' where n is the array index of the element 
  /// ROOT
  /// - name1: value2 \u0009\u0009\u0009\u0009"Simple name/value pairs"
  /// - name2: value2\u0009\u0009\u0009\u0009"Simple name/value pairs"
  /// + name3:\u0009\u0009\u0009\u0009\u0009\u0009"Nested node "
  /// - name4: value4
  /// - name5: value5
  /// + name6:\u0009\u0009\u0009\u0009"Nested nested node"
  /// - name6: value7
  /// + name8: \u0009\u0009\u0009\u0009\u0009"List node (nodes with the same name)
  /// [0] value10
  /// [1] value11
  /// [2] value12
  /// + name9\u0009\u0009\u0009\u0009\u0009\u0009"List node with nested nodes and nested list nodes
  /// - name20: value20
  /// - name21: value21\u0009
  /// + name22:\u0009\u0009\u0009\u0009\u0009"List of nested child nodes"
  /// [0]:\u0009\u0009\u0009\u0009\u0009\u0009
  /// [0]: value22A\u0009\u0009\u0009"List in a List
  /// [1]: value22B
  /// [2]: value22C
  /// [1]: \u0009\u0009\u0009\u0009\u0009\u0009"Nested child node in array"
  /// - name30 : value30A
  /// - name31 : value30B
  /// - name32 : value30C
  /// </summary>
  /// <author>rozhang</author>
  public class XMLParseResultVisualizer
  {
    protected XPHashtable m_result = null;
    public XMLParseResultVisualizer(XPHashtable result)
    {
      m_result = result;
    }
    
    /// <summary>
    /// Main parsing function 
    /// </summary>
    /// <returns></returns>
    public virtual String GenerateStringRepresentation()
    {
      XPStringBuilder buf = new XPStringBuilder();
      buf.Append("ROOT\n");
      buf.Append(ParseHashTable(m_result, 0));
      return buf.ToString();
    }
    
    /// <summary>
    /// Generate String representation for node with child nodes
    /// </summary>
    /// <param name="map"></param>
    /// <param name="level">The depth from the top where top is 0</param>
    /// <returns></returns>
    protected virtual String ParseHashTable(XPHashtable map, int level)
    {
      XPStringBuilder buf = new XPStringBuilder();
      // iterate over the xphashtable
      IXPEnumerator keys = map.GetKeys();
      while (keys.MoveNext())
      {
        String curName = (String) keys.GetCurrent();
        Object val = map.GetElement(curName);
        if (val is XPHashtable)
        {
          buf.Append(AddPadding(level)).Append("+ ").Append(curName).Append(":");
          buf.Append("\n");
          buf.Append(ParseHashTable((XPHashtable) val, level + 1));
        } else
        if (val is XPArrayList)
        {
          buf.Append(AddPadding(level)).Append("+ ").Append(curName).Append(":");
          buf.Append("\n");
          buf.Append(ParseArray((XPArrayList) val, level));
        } else
        {
          buf.Append(AddPadding(level)).Append("- ").Append(curName).Append(":");
          buf.Append(" ").Append(val).Append("\n");
        }
      }
      return buf.ToString();
    }
    
    /// <summary>
    /// Generate String representation for Array node
    /// </summary>
    /// <param name="values"></param>
    /// <param name="level"></param>
    /// <returns></returns>
    protected virtual String ParseArray(XPArrayList values, int level)
    {
      XPStringBuilder buf = new XPStringBuilder();
      for (int i = 0;i < values.GetSize();i++)
      {
        Object val = values.GetElement(i);
        if (val is XPHashtable)
        {
          buf.Append(AddPadding(level)).Append("[" + i + "]:\n");
          buf.Append(ParseHashTable((XPHashtable) val, level + 1));
        } else
        if (val is XPArrayList)
        {
          buf.Append("\n");
          buf.Append(ParseArray((XPArrayList) val, level + 1));
        } else
        {
          buf.Append(AddPadding(level)).Append("  [" + i + "]: ").Append(val).Append("\n");
          }
      }
      return buf.ToString();
    }
    
    /// <summary>
    /// Helper for adding the proper amount of indentation
    /// </summary>
    /// <param name="level"></param>
    /// <returns></returns>
    protected virtual String AddPadding(int level)
    {
      XPStringBuilder buf = new XPStringBuilder();
      for (int i = 0;i < level;i++)
      {
        // 2 spaces
        buf.Append("  ");
      }
      return buf.ToString();
    }
  }
}