using System;
using com.plumtree.openfoundation.util;
using com.plumtree.server;
namespace com.plumtree.portaluiinfrastructure.mock.tags
{
  /// <summary>
  /// Helper class that provides an object oriented way to generate markup fragment.
  /// This helper only supports adaptive tag and pure content (strings or html) markup elements. 
  /// </summary>
  /// <author>rozhang</author>
  public class XMLFragmentTestHelper
  {
    protected const String ATTRIB_HTML = "h";
    protected const String ATTRIB_PT = "pt";
    
    /// <summary>
    /// negative values so they aren't confused with the markup types *
    /// </summary>
    public const int TAGTYPE_HTML = -100;
    public const int TAGTYPE_ADAPTIVETAG = -200;
    public static readonly int[] MARKUP_TYPE_MAP = new int[]{PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_FINISHED,
     PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_PT_XML};
    
    /// <summary>
    /// shorter, simpler and clearerconstant definitions *
    /// </summary>
    protected static readonly int TYPECOL = PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_TYPE;
    protected static readonly int NAMECOL = PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT;
    protected static readonly int VALUECOL = PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_NUM_COLS;
    protected int m_tagType = 0;
    protected String m_libName = null;
    protected String m_tagName = null;
    
    /// <summary>
    /// Array list to store content items (nested tag inside a tag body)
    /// It is also used to store a list of processed fragments when creating sequential elements
    /// with a list of fragmenttesthelpers.  
    /// This could get a little confusing since it used to store both processed fragments and
    /// raw fragments and they will have different types (either the internal tag type or the
    /// xml fragment type).
    /// </summary>
    protected XPArrayList m_content = new XPArrayList();
    protected XPArrayList m_attribs = new XPArrayList();
    
    /// <summary>
    /// Constructor for Adaptive Tags
    /// </summary>
    /// <param name="libName"></param>
    /// <param name="tagName"></param>
    public XMLFragmentTestHelper(String libName, String tagName)
    {
      m_tagType = TAGTYPE_ADAPTIVETAG;
      m_libName = libName;
      m_tagName = tagName;
    }
    
    /// <summary>
    /// Constructor for HTML markup entry
    /// </summary>
    /// <param name="fragment"></param>
    public XMLFragmentTestHelper(XMLFragmentTestHelper fragment)
    {
      // for html fragement, the html is stored in the content array list
      m_tagType = TAGTYPE_HTML;
      // store the processed fragment into the content array, more might get added later
      m_content.Add(fragment.ToArrayEntireFragment());
    }
    
    /// <summary>
    /// Use this helper as a container for a list of tag fragments created with the
    /// helper
    /// </summary>
    /// <param name="fragments"></param>
    public XMLFragmentTestHelper(XMLFragmentTestHelper[] fragments)
    {
      // html type since this is just a container for other tags
      m_tagType = TAGTYPE_HTML;
      for (int i = 0;i < fragments.Length;i++)
      {
        // store the internal tag type (not the xml tag type) in the first element
        // and the processed fragment in the second element
        m_content.Add(new Object[]{fragments[i].GetTagType(), fragments[i].ToArrayEntireFragment()});
        }
    }
    
    /// <param name="libName"></param>
    /// <param name="tagName"></param>
    /// <param name="attribs">Map of attribute name/value pairs. Attribute names prefixed with '-' are treated as XML attributes (not prefixed with pt:)</param>
    /// <returns></returns>
    public static XMLFragmentTestHelper CreateTag(String libName, String tagName,
     XPHashtable attribs)
    {
      XMLFragmentTestHelper helper = new XMLFragmentTestHelper(libName, tagName);
      // get all keys in attribute map
      IXPEnumerator enumKeys = attribs.GetKeys();
      while (enumKeys.MoveNext())
      {
        Object value = enumKeys.GetCurrent();
        if (value is String)
        {
          String key = (String) value;
          // dont bother adding attributes with null value
          if (null != attribs.GetElement(key))
          {
            // XML attribute
            if (XPStringUtility.StartsWith(key, "-"))
            {
              // snip off the starting '-'
              helper.AddAttribute(XPStringUtility.Substring(key, 1, key.Length),
               (String) attribs.GetElement(key));
            } else
            // regular adaptive tag attribute
            {
              helper.AddPTAttribute(key, (String) attribs.GetElement(key));
              }
          }
        } else
        {
          throw new XPIllegalStateException("Unexpected Tag Attribute type: " + value);
          }
      }
      return helper;
    }
    //	public XMLFragmentTestHelper(Object[][] fragment)
    //	{
    //		m_tagType = TAGTYPE_ADAPTIVETAG;
    //		m_content.Add(fragment);
    //	}
    
    /// <summary>
    /// Add tag body content
    /// </summary>
    /// <param name="fragment"></param>
    public virtual void AddContent(XMLFragmentTestHelper fragment)
    {
      m_content.Add(new Object[]{fragment.GetTagType(), fragment.ToArrayEntireFragment()});
      }
    
    /// <summary>
    /// Add tag body content
    /// </summary>
    /// <param name="fragmentType"></param>
    /// <param name="bodyFragment">Markup fragment (with markup type in the first column)</param>
    public virtual void AddContent(int fragmentType, Object[][] bodyFragment)
    {
      m_content.Add(new Object[]{fragmentType, bodyFragment});
    }
    
    /// <summary>
    /// Add HTML tag body content 
    /// </summary>
    /// <param name="content"></param>
    public virtual void AddContent(String content)
    {
      m_content.Add(new Object[]{PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_FINISHED,
       content});
    }
    
    /// <summary>
    /// Add opener HTML tag attribute
    /// </summary>
    /// <param name="attribName"></param>
    /// <param name="attribValue"></param>
    public virtual void AddAttribute(String attribName, String attribValue)
    {
      m_attribs.Add(new String[]{ATTRIB_HTML, attribName, attribValue});
    }
    
    /// <summary>
    /// Add opener PT tag attribute (for adaptive tags)
    /// </summary>
    /// <param name="attribName"></param>
    /// <param name="attribValue"></param>
    public virtual void AddPTAttribute(String attribName, String attribValue)
    {
      m_attribs.Add(new String[]{ATTRIB_PT, attribName, attribValue});
    }
    
    /// <summary>
    /// Generate the markup fragment value opener or begin tag.
    /// Only the markup fragment value is returned, not the full markup row with the type. 
    /// Note that this is only applicable for adaptive tag fragments, not fragment with
    /// just html content.  
    /// </summary>
    /// <returns></returns>
    public virtual Object[][] ToArrayOpenTag()
    {
      // for the adaptive tag attribute array we need elements for all the attributes + 1 for the tag name
      // and possible one to close the tag if the tag doesnt have a body
      int size = 1 + m_attribs.GetSize() + ((ClosedTag()) ? 1 : 0);
      // tag data columns for type, name and value 
      Object[][] markup = new Object[3][];
      for (int j2c_index = 0;j2c_index < 3;j2c_index++)
      {
        markup[j2c_index] = new Object[size];
      }
      switch (m_tagType){
      case TAGTYPE_ADAPTIVETAG:
      // add tag name to markup array
      markup[TYPECOL][0] = PT_XML_TYPES.PT_XML_TYPES_PTUI;
      markup[NAMECOL][0] = m_libName + "." + m_tagName;
      markup[VALUECOL][0] = null;
      // add arguments, iterate over the argument array and add array data for each argument
      for (int args = 0;args < m_attribs.GetSize();args++)
      {
        String[] attribArray = ((String[]) m_attribs.GetElement(args));
        // add one to the markup index since the name entry occupies the first element
        int markupIndex = args + 1;
        // figure out which type to set here, different for tag attribute with pt: and regular attributes
        if (ATTRIB_PT.Equals(attribArray[0]))
        {
          markup[TYPECOL][markupIndex] = PT_XML_TYPES.PT_XML_TYPES_PTUI_ATT;
          } else
        if (ATTRIB_HTML.Equals(attribArray[0]))
        {
          markup[TYPECOL][markupIndex] = PT_XML_TYPES.PT_XML_TYPES_XHTML_ATT;
          } else
        {
          throw new XPIllegalArgumentException("Invalid attribute type passed in: " + attribArray[0] + ". Refer to the ATTRIB_x constants for valid attribute types.");
          }
        // set the name and value for the attribute 
        markup[NAMECOL][markupIndex] = attribArray[1];
        markup[VALUECOL][markupIndex] = attribArray[2];
      }
      // add a closed attribute if there is no content in the tag
      if (ClosedTag())
      {
        markup[TYPECOL][size - 1] = PT_XML_TYPES.PT_XML_TYPES_PTUI_CLOSE;
        markup[NAMECOL][size - 1] = null;
        markup[VALUECOL][size - 1] = null;
      }
      break;
      case TAGTYPE_HTML:
      throw new XPIllegalStateException("Cannot generate an opener tag for HTML Fragment");
      break;
      }
      return markup;
    }
    
    /// <summary>
    /// Convert the body to an markup array format (where the first column contains the value for the fragment type). 
    /// For HTML fragments, the body is the entire fragment while adaptive tags contain an 2d object array with
    /// info about the tag.
    /// </summary>
    /// <returns></returns>
    public virtual Object[][] ToArrayTagBody()
    {
      return ToArrayTagBody(false);
    }
    
    /// <param name="withOpenCloseEntries">
    /// If this is used to generate the markup fragment for the entire
    /// tag we want to leave space in the array for the open and close tags
    /// </param>
    /// <returns></returns>
    protected virtual Object[][] ToArrayTagBody(bool withOpenCloseEntries)
    {
      // the markup array needs to have 2 additional elements, one for open and one for close tags
      int additionalElements = (withOpenCloseEntries) ? 2 : 0;
      // don't need a close markup entry if we already closed the open tag (no content exists)
      if (ClosedTag())
      {
        additionalElements--;
      }
      // to leave the first element open for the open tag, add one to the array index 
      int indexOffset = (withOpenCloseEntries) ? 1 : 0;
      // keep two array list to store all the markup entries from the content
      // assembly them into an array later. we need to go through all the content
      // entries first since we don't know how many markup entries each
      // content entry contains
      XPArrayList markupTypeList = new XPArrayList();
      XPArrayList markupFragList = new XPArrayList();
      Object[][] markup = null;
      // don't bother go through the content if there is no body set
      if (!ClosedTag())
      {
        for (int i = 0;i < m_content.GetSize();i++)
        {
          Object[] contentEntry = (Object[]) m_content.GetElement(i);
          // check what type of content, could be a tag attribute array or just a string
          if (PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_FINISHED == ((int) contentEntry[0]))
          {
            markupTypeList.Add(PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_FINISHED);
            markupFragList.Add(contentEntry[1]);
          } else
          {
            Object[][] contentMarkup = (Object[][]) contentEntry[1];
            // iterate over all the markup entries and add them into the list
            for (int j = 0;j < contentMarkup[0].Length;j++)
            {
              // a list for the markup type and the markup fragment
              markupTypeList.Add(contentMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_TYPE][j]);
              markupFragList.Add(contentMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT][j]);
              }
          }
        }
        // ok done with creating the temporary array, now put it all in a nice big 2d array
        markup = new Object[2][];
        for (int j2c_index = 0;j2c_index < 2;j2c_index++)
        {
          markup[j2c_index] = new Object[markupTypeList.GetSize() + additionalElements];
          }
        for (int k = 0;k < markupTypeList.GetSize();k++)
        {
          markup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_TYPE][k + indexOffset] = markupTypeList.GetElement(k);
          markup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT][k + indexOffset] = markupFragList.GetElement(k);
          }
      } else
      {
        // still need to create a markup array for the caller
        markup = new Object[2][];
        for (int j2c_index = 0;j2c_index < 2;j2c_index++)
        {
          markup[j2c_index] = new Object[additionalElements];
        }
      }
      return markup;
    }
    
    /// <summary>
    /// Generate the markup fragment value opener or begin tag.
    /// Only the markup fragment value is returned, not the full markup row with the type. 
    /// Note that this is only applicable for adaptive tag fragments, not fragment with
    /// just html content.
    /// </summary>
    /// <returns></returns>
    public virtual Object[][] ToArrayTagClose()
    {
      // return a symbolic empty 2d array, this should get used though
      if (ClosedTag())
      {
        return null;
      }
      // close markup can only have one entry
      Object[][] markup = new Object[3][];
      for (int j2c_index = 0;j2c_index < 3;j2c_index++)
      {
        markup[j2c_index] = new Object[1];
      }
      switch (m_tagType){
      case TAGTYPE_ADAPTIVETAG:
      markup[TYPECOL][0] = PT_XML_TYPES.PT_XML_TYPES_PTUI_CLOSE;
      markup[NAMECOL][0] = m_libName + "." + m_tagName;
      markup[VALUECOL][0] = null;
      break;
      case TAGTYPE_HTML:
      throw new XPIllegalStateException("Cannot generate a close tag for HTML markup");
      break;
      }
      return markup;
    }
    
    /// <summary>
    /// Helper to see if this should be a singleton tag
    /// (tags that are closed in the opener, eg <tag ... />)
    /// </summary>
    /// <returns></returns>
    protected virtual bool ClosedTag()
    {
      bool closedTagEntry = (m_content.GetSize() == 0);
      return closedTagEntry;
    }
    
    /// <summary>
    /// Generate full markup fragment for this tag with markup fragement type and value columns
    /// Internally calls ToArrayTagOpen, ToArrayTagBody and ToArrayTagClose
    /// </summary>
    /// <returns></returns>
    public virtual Object[][] ToArrayEntireFragment()
    {
      Object[][] markup = null;
      switch (m_tagType){
      case TAGTYPE_ADAPTIVETAG:
      // this creates a markup array with slots for open and close tags at the
      // beginning and end of the array
      markup = ToArrayTagBody(true);
      // now set the open 
      markup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_TYPE][0] = PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_PT_XML;
      markup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT][0] = ToArrayOpenTag();
      // don't set a close tag if the begin tag is already closed
      if (!ClosedTag())
      {
        markup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_TYPE][markup[0].Length - 1] = PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_PT_XML;
        markup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT][markup[0].Length - 1] = ToArrayTagClose();
        }
      break;
      case TAGTYPE_HTML:
      // no begin or close tag for html content, just the body
      markup = ToArrayTagBody();
      break;
      }
      return markup;
    }
    
    /// <summary>
    /// Tag type
    /// </summary>
    /// <seealso cref="TAGTYPE_HTML,">TAGTYPE_ADAPTIVETAG</seealso>
    /// <returns></returns>
    public virtual int GetTagType()
    {
      return m_tagType;
    }
  }
}