using System;
using junit.framework;
using com.plumtree.openfoundation.reflect;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.helper;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.portaluiinfrastructure.test.tags;
namespace com.plumtree.portaluiinfrastructure.test.tags.helper
{
  /// <summary>
  /// Tests for the Tag State class.
  /// </summary>
  /// <author>donh</author>
  public class TagStateTest : TestCase
  {
    public const String EDITABLE_KEY = "testeditable";
    public const String UNEDITABLE_KEY = "testuneditable";
    protected TagState m_TagState;
    protected TestTag m_TestTag;
    protected SecondTestTag m_SecondTestTag;
    
    /// <summary>
    /// Constructor.
    /// </summary>
    public TagStateTest(String name) : base(name)
    {
      // Initialize the meta data for the test tags.
      // This is normally done by the TagLibManager.
      m_TestTag = new TestTag();
      TagMetaDataHelper testMetaDataHelper = new TagMetaDataHelper();
      testMetaDataHelper.Init(m_TestTag, new TagLibraryMetaData("Test Tag Library",
       "testtaglibrary", "Test tag library description", 1.0));
      m_TestTag.SetTagMetaDataHelper(testMetaDataHelper);
      m_SecondTestTag = new SecondTestTag();
      TagMetaDataHelper secondTestMetaDataHelper = new TagMetaDataHelper();
      secondTestMetaDataHelper.Init(m_SecondTestTag, new TagLibraryMetaData("Second Test Tag Library",
       "secondtesttaglibrary", "Second test tag library description", 1.0));
      m_SecondTestTag.SetTagMetaDataHelper(secondTestMetaDataHelper);
    }
    
    /// <summary>
    /// Setup if any is required.
    /// </summary>
    public override void setUp()
    {
      base.setUp();
      m_TagState = new TagState(null, null, null, new XPHashtable());
    }
    public override void tearDown()
    {
      base.tearDown();
      m_TagState.ResetPortletRequestData();
      m_TagState = null;
    }
    
    /// <summary>
    /// This tests resetting the Tag State object.
    /// </summary>
    public virtual void testResetPortletRequestData()
    {
      testTagScope();
      m_TagState.ResetPortletRequestData();
      testPortletScope();
      m_TagState.ResetPortletRequestData();
      testTagScope();
      m_TagState.ResetPortletRequestData();
      testPortletScope();
      m_TagState.ResetPortletRequestData();
    }
    
    /// <summary>
    /// This tests the argument checking for
    /// accessing variables.
    /// </summary>
    public virtual void testIllegalArguments()
    {
      // Tag State cannot be used without entering a tag first.
      m_TagState.EnterTag(GetNewTestTagInstance());
      bool bFailed = false;
      try
      {
        m_TagState.SetVariable(null, "test", Scope.TAG);
      }
      catch (XPIllegalArgumentException xpiae)
      {
        // Expected result
        bFailed = true;
      }
      if (!bFailed)
      {
        fail("Setting Variable with null key argument did not fail.");
      }
      bFailed = false;
      try
      {
        m_TagState.SetVariable("test.test", "test", Scope.SESSION);
      }
      catch (XPIllegalArgumentException xpiae)
      {
        // Expected result
        bFailed = true;
      }
      if (!bFailed)
      {
        fail("Setting Variable with illegal character (\".\") in key did not fail.");
        }
      bFailed = false;
      try
      {
        m_TagState.SetVariable("test", "test", null);
      }
      catch (XPIllegalArgumentException xpiae)
      {
        // Expected result
        bFailed = true;
      }
      if (!bFailed)
      {
        fail("Setting Variable with null scope argument did not fail.");
      }
      bFailed = false;
      try
      {
        m_TagState.GetVariable(null, Scope.SESSION);
      }
      catch (XPIllegalArgumentException xpiae)
      {
        // Expected result
        bFailed = true;
      }
      if (!bFailed)
      {
        fail("Getting Variable with null key argument did not fail.");
      }
      bFailed = false;
      try
      {
        m_TagState.GetVariable("test", null);
      }
      catch (XPIllegalArgumentException xpiae)
      {
        // Expected result
        bFailed = true;
      }
      if (!bFailed)
      {
        fail("Getting Variable with null scope argument did not fail.");
      }
      m_TagState.ExitTag();
    }
    
    /// <summary>
    /// This tests Tag Scope variables.
    /// </summary>
    public virtual void testTagScope()
    {
      bool bTest;
      Object oTest;
      // tag 1
      m_TagState.EnterTag(GetNewTestTagInstance());
      // tag 2
      m_TagState.EnterTag(GetNewTestTagInstance());
      // Use an object, since this is null, and booleans
      // are not objects in C#.
      oTest = m_TagState.GetVariable("test", Scope.TAG);
      Assert.assertNull("Getting empty variable test before it has been set.",
       oTest);
      m_TagState.SetVariable("test", true, Scope.TAG);
      bTest = (bool) m_TagState.GetVariable("test", Scope.TAG);
      Assert.assertTrue("Getting variable test that has been set to true in this tag.",
       XPConvert.ToBoolean(bTest));
      // tag 3
      m_TagState.EnterTag(GetNewTestTagInstance());
      bTest = (bool) m_TagState.GetVariable("test", Scope.TAG);
      Assert.assertTrue("Getting variable test that has been set to true in a parent tag.",
       XPConvert.ToBoolean(bTest));
      m_TagState.SetVariable("test", false, Scope.TAG);
      bTest = (bool) m_TagState.GetVariable("test", Scope.TAG);
      Assert.assertFalse("Getting variable test that has been set to false in this tag.",
       XPConvert.ToBoolean(bTest));
      // tag 3
      m_TagState.ExitTag();
      bTest = (bool) m_TagState.GetVariable("test", Scope.TAG);
      Assert.assertFalse("Getting variable test that has been set to false in another tag.",
       XPConvert.ToBoolean(bTest));
      m_TagState.SetVariable("test", true, Scope.TAG);
      bTest = (bool) m_TagState.GetVariable("test", Scope.TAG);
      Assert.assertTrue("Getting variable test that has been set to true in this tag.",
       XPConvert.ToBoolean(bTest));
      // tag 2
      m_TagState.ExitTag();
      // Use an object, since we have just set this to null, and booleans
      // are not objects in C#.
      oTest = m_TagState.GetVariable("test", Scope.TAG);
      Assert.assertNull("Getting empty variable test before it has been set.",
       oTest);
      // tag 1
      m_TagState.ExitTag();
    }
    
    /// <summary>
    /// This tests removing Tag Scope variables.
    /// </summary>
    public virtual void testTagScopeRemoval()
    {
      bool bTest;
      Object oTest;
      // Test setting tags to null
      // Setting a tag to null should be the same as never having set the 
      // tag in the first place.
      // tag 1
      m_TagState.EnterTag(GetNewTestTagInstance());
      m_TagState.SetVariable("test", true, Scope.TAG);
      // tag 2
      m_TagState.EnterTag(GetNewTestTagInstance());
      // Make sure setting it to null and resetting it cause the
      // tag association to be changed (i.e. which tag is it in scope for)
      m_TagState.SetVariable("test", null, Scope.TAG);
      // Replace the reset tag using the current tag scope.
      m_TagState.SetVariable("test", true, Scope.TAG);
      // tag 3
      m_TagState.ExitTag();
      // Use an object, since this variable has just passed out of tag scope,
      // and booleans are not objects in C#.
      oTest = m_TagState.GetVariable("test", Scope.TAG);
      Assert.assertNull("Getting empty variable test after it has passed out of tag scope.",
       oTest);
    }
    
    /// <summary>
    /// This tests Portlet Scope variables.
    /// </summary>
    public virtual void testPortletScope()
    {
      // This method does not enter a tag so that it can be called from
      // within other test methods.
      Object oTest;
      bool bTest;
      m_TagState.ResetPortletRequestData();
      // Tag State cannot be used without entering a tag first.
      m_TagState.EnterTag(GetNewTestTagInstance());
      // Use an object, since this is null, and booleans
      // are not objects in C#.
      oTest = m_TagState.GetVariable("test", Scope.PORTLET_REQUEST);
      Assert.assertNull("Getting empty variable test before it has been set.",
       oTest);
      m_TagState.SetVariable("test", true, Scope.PORTLET_REQUEST);
      bTest = (bool) m_TagState.GetVariable("test", Scope.PORTLET_REQUEST);
      Assert.assertTrue("Getting variable test that has been set to true in this tag.",
       XPConvert.ToBoolean(bTest));
      m_TagState.ExitTag();
      m_TagState.ResetPortletRequestData();
      // Tag State cannot be used without entering a tag first.
      m_TagState.EnterTag(GetNewTestTagInstance());
      // Use an object, since this is null, and booleans
      // are not objects in C#.
      oTest = m_TagState.GetVariable("test", Scope.PORTLET_REQUEST);
      Assert.assertNull("Getting empty variable test after portlet request has been cleared.",
       oTest);
      m_TagState.ExitTag();
      m_TagState.ResetPortletRequestData();
    }
    
    /// <summary>
    /// This tests HTTP Request Scope variables.
    /// </summary>
    public virtual void testHTTPRequestScope()
    {
      // This method does not enter a tag so that it can be called from
      // within other test methods.
      Object oTest;
      bool bTest;
      m_TagState.ResetPortletRequestData();
      // Tag State cannot be used without entering a tag first.
      m_TagState.EnterTag(GetNewTestTagInstance());
      // Use an object, since this is null, and booleans
      // are not objects in C#.
      oTest = m_TagState.GetVariable("test", Scope.HTTP_REQUEST);
      Assert.assertNull("Getting empty variable test before it has been set.",
       oTest);
      m_TagState.SetVariable("test", true, Scope.HTTP_REQUEST);
      bTest = (bool) m_TagState.GetVariable("test", Scope.HTTP_REQUEST);
      Assert.assertTrue("Getting variable test that has been set to true in this tag.",
       XPConvert.ToBoolean(bTest));
      m_TagState.ExitTag();
      m_TagState.ResetPortletRequestData();
      // Tag State cannot be used without entering a tag first.
      m_TagState.EnterTag(GetNewTestTagInstance());
      bTest = (bool) m_TagState.GetVariable("test", Scope.HTTP_REQUEST);
      Assert.assertTrue("Getting variable test that has been set to true in previous portlet.",
       XPConvert.ToBoolean(bTest));
      m_TagState.ExitTag();
      m_TagState.ResetPortletRequestData();
    }
    
    /// <summary>
    /// This tests shared variable edit access protection.
    /// </summary>
    public virtual void testSharedVariableProtection()
    {
      m_TagState.EnterTag(GetNewTestTagInstance());
      m_TagState.SetSharedVariable(UNEDITABLE_KEY, true, Scope.PORTLET_REQUEST,
       true);
      bool bTest = (bool) m_TagState.GetSharedVariable(UNEDITABLE_KEY, Scope.PORTLET_REQUEST);
      Assert.assertTrue("Getting shared variable test that has been set to true in this tag.",
       XPConvert.ToBoolean(bTest));
      m_TagState.SetSharedVariable(EDITABLE_KEY, true, Scope.PORTLET_REQUEST,
       false);
      bTest = (bool) m_TagState.GetSharedVariable(EDITABLE_KEY, Scope.PORTLET_REQUEST);
      Assert.assertTrue("Getting shared variable test that has been set to true in this tag.",
       XPConvert.ToBoolean(bTest));
      m_TagState.ExitTag();
      m_TagState.EnterTag(GetNewSecondTestTagInstance());
      bTest = (bool) m_TagState.GetSharedVariable(UNEDITABLE_KEY, Scope.PORTLET_REQUEST);
      Assert.assertTrue("Getting uneditable shared variable test that has been set to true in another tag.",
       XPConvert.ToBoolean(bTest));
      bool bFailed = false;
      try
      {
        m_TagState.SetSharedVariable(UNEDITABLE_KEY, false, Scope.PORTLET_REQUEST,
         false);
      }
      catch (XPIllegalAccessException xpiae)
      {
        // Expected result
        bFailed = true;
      }
      if (!bFailed)
      {
        fail("Setting Shared Variable as editable without proper edit access did not fail.");
        }
      try
      {
        m_TagState.SetSharedVariable(UNEDITABLE_KEY, false, Scope.PORTLET_REQUEST,
         true);
      }
      catch (XPIllegalAccessException xpiae)
      {
        // Expected result
        bFailed = true;
      }
      if (!bFailed)
      {
        fail("Setting Shared Variable as uneditable without proper edit access did not fail.");
        }
      bTest = (bool) m_TagState.GetSharedVariable(EDITABLE_KEY, Scope.PORTLET_REQUEST);
      Assert.assertTrue("Getting editable shared variable test that has been set to true in another tag.",
       XPConvert.ToBoolean(bTest));
      m_TagState.SetSharedVariable(EDITABLE_KEY, false, Scope.PORTLET_REQUEST,
       false);
      bTest = (bool) m_TagState.GetSharedVariable(EDITABLE_KEY, Scope.PORTLET_REQUEST);
      Assert.assertFalse("Getting editable shared variable test that has been set to false in this tag.",
       XPConvert.ToBoolean(bTest));
      m_TagState.ExitTag();
    }
    
    /// <summary>
    /// This helper method creates and initializes a new TestTag instance.
    /// This would normally be handled by the TagLibManager.
    /// </summary>
    /// <returns> A new TestTag instance.</returns>
    protected virtual TestTag GetNewTestTagInstance()
    {
      TestTag result = new TestTag();
      result.SetTagMetaDataHelper(m_TestTag.GetTagMetaDataHelper());
      return result;
    }
    
    /// <summary>
    /// This helper method creates and initializes a new TestSecondTestTagTag
    /// instance.  This would normally be handled by the TagLibManager.
    /// </summary>
    /// <returns> A new SecondTestTag instance.</returns>
    protected virtual SecondTestTag GetNewSecondTestTagInstance()
    {
      SecondTestTag result = new SecondTestTag();
      result.SetTagMetaDataHelper(m_SecondTestTag.GetTagMetaDataHelper());
      return result;
    }
  }
}