using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.mock.tags;
using com.plumtree.portaluiinfrastructure.tags.helper;
namespace com.plumtree.portaluiinfrastructure.utils
{
  /// <summary>
  /// Helper classes to access tag variables from unit tests
  /// </summary>
  /// <author>rozhang</author>
  public class TagStateTestHelper
  {
    protected XPHashtable m_portletRequestMap = new XPHashtable();
    public TagStateTestHelper()
    {
      // Seed the test variable in Tag state
      TagState.TEST_PORTLET_REQUEST_DATA = m_portletRequestMap;
    }
    
    /// <summary>
    /// Get tag variable from Portlet Request Scope
    /// Portlet Request Scope only exists within one execution of a DisplayTree.ProcessTagMarkup call
    /// </summary>
    /// <param name="varName">
    /// Tag variable name, if the tag variable name contains a '.' this function assumes that the taglibrary name is
    /// already included in the name
    /// </param>
    /// <param name="env"></param>
    /// <returns>null if the variable is not found</returns>
    public virtual Object GetTagPortletRequestVar(String varName)
    {
      // tag variable names are prefixed with their tag library in the scopedata map, but most tags store in the shared scope
      // so they are prefixed with a '.'. In case the variable name contains a dot, assume that the caller already included
      // it so don't add it again
      String adjustedTagVarName = (XPStringUtility.IndexOf(varName, ".") < 0) ? "." + varName : varName;
      SharedStateVariable stateVar = (SharedStateVariable) m_portletRequestMap.GetElement(adjustedTagVarName);
      if (stateVar == null)
      {
        return null;
      } else
      {
        return stateVar.GetValue();
      }
    }
    
    /// <summary>
    /// Set tag variable from Portlet Request Scope
    /// Portlet Request Scope only exists within one execution of a DisplayTree.ProcessTagMarkup call
    /// </summary>
    /// <param name="varLibName"></param>
    /// <param name="varName"></param>
    /// <param name="varValue"></param>
    /// <returns></returns>
    public virtual Object SetTagPortletRequestVar(String varLibName, String varName,
     Object varValue)
    {
      String finalVarName = (varLibName == null || "".Equals(varLibName)) ? "." + varName : varLibName + "." + varName;
      SharedStateVariable stateVar = new SharedStateVariable(varValue);
      return m_portletRequestMap.PutElement(finalVarName, stateVar);
    }
    
    /// <summary>
    /// Get tag variable in the HTTP Request Scope
    /// HTTP Request Scope variables persists between portlets runs (Portlet Request Scope)
    /// </summary>
    /// <param name="varName"></param>
    /// <param name="env"></param>
    /// <returns></returns>
    public virtual Object GetTagHTTPRequestVar(String varName, MockEnvironment env)
    {
      XPHashtable scopeData = TagState.GetHTTPRequestData(env);
      // tag variable names are prefixed with their tag library in the scopedata map, but most tags store in the shared scope
      // so they are prefixed with a '.'. In case the variable name contains a dot, assume that the caller already included
      // it so don't add it again
      String adjustedTagVarName = (XPStringUtility.IndexOf(varName, ".") < 0) ? "." + varName : varName;
      SharedStateVariable stateVar = (SharedStateVariable) scopeData.GetElement(adjustedTagVarName);
      if (stateVar == null)
      {
        return null;
      } else
      {
        return stateVar.GetValue();
      }
    }
    
    /// <summary>
    /// Set tag variable in the HTTP Request Scope
    /// HTTP Request Scope variables persists between portlets runs (Portlet Request Scope)
    /// </summary>
    /// <param name="varLibName"></param>
    /// <param name="varName"></param>
    /// <param name="varValue"></param>
    /// <param name="env"></param>
    /// <returns></returns>
    public virtual Object SetTagHTTPRequestVar(String varLibName, String varName,
     Object varValue, MockEnvironment env)
    {
      String finalVarName = (varLibName == null || "".Equals(varLibName)) ? "." + varName : varLibName + "." + varName;
      XPHashtable httpReqScopeData = TagState.GetHTTPRequestData(env);
      SharedStateVariable tagVarValue = new SharedStateVariable(varValue);
      return httpReqScopeData.PutElement(finalVarName, tagVarValue);
    }
    
    /// <summary>
    /// Get Tag variable on session
    /// Session variables aren't stored as SharedStateVariable type like HTTP and PORTLET REQUEST tag variables
    /// </summary>
    /// <param name="varName"></param>
    /// <param name="env"></param>
    /// <returns></returns>
    public virtual Object GetTagSessionVar(String varName, MockEnvironment env)
    {
      String adjustedTagVarName = (XPStringUtility.IndexOf(varName, ".") < 0) ? "." + varName : varName;
      Object varValue = env.GetSubSession().GetAttribute(adjustedTagVarName);
      return varValue;
    }
    
    /// <summary>
    /// Set Tag variable on session
    /// Session variables aren't stored as SharedStateVariable type like HTTP and PORTLET REQUEST tag variables
    /// </summary>
    /// <param name="varLibName"></param>
    /// <param name="varName"></param>
    /// <param name="varValue"></param>
    /// <param name="env"></param>
    public virtual void SetTagSessionVar(String varLibName, String varName,
     Object varValue, MockEnvironment env)
    {
      String finalVarName = (varLibName == null || "".Equals(varLibName)) ? "." + varName : varLibName + "." + varName;
      env.GetSubSession().SetAttribute(finalVarName, varValue);
    }
    
    /// <summary>
    /// Get Tag variable from Application
    /// Session variables aren't stored as SharedStateVariable type like HTTP and PORTLET REQUEST tag variables
    /// </summary>
    /// <param name="varName"></param>
    /// <param name="env"></param>
    /// <returns></returns>
    public virtual Object GetTagApplicationVar(String varName, MockEnvironment env)
    {
      String adjustedTagVarName = (XPStringUtility.IndexOf(varName, ".") < 0) ? "." + varName : varName;
      Object varValue = env.GetApplication().GetCachingManager().GetEntry(adjustedTagVarName);
      return varValue;
    }
    
    /// <summary>
    /// Set Tag variable from Application
    /// Session variables aren't stored as SharedStateVariable type like HTTP and PORTLET REQUEST tag variables
    /// </summary>
    /// <param name="varLibName"></param>
    /// <param name="varName"></param>
    /// <param name="varValue"></param>
    /// <param name="env"></param>
    public virtual void SetTagApplicationVar(String varLibName, String varName,
     Object varValue, MockEnvironment env)
    {
      String finalVarName = (varLibName == null || "".Equals(varLibName)) ? "." + varName : varLibName + "." + varName;
      env.GetApplication().GetCachingManager().SetEntry(finalVarName, varValue);
      }
  }
}