/*
 * Garbage collector thread for the portal.
 * 
 * Created on Jan 29, 2004
 * @author DavidP
 */
package com.plumtree.portaluiinfrastructure.application;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;


/**
 * This class calls the garbage collector at regular intervals.
 * 
 * This is wrapped in a thread so it can be interrupted.
 * 
 * Thread priority is controlled by the thread class itself and 
 * can be set.  Interval in milliseconds between gc's can be set.
 * Teh shutdown boolean is used by the app server to terminate
 * the thread. 
 * 
 * @author DavidP
 * @see PTGarbageCollectorThread
 */
public class PTGarbageCollectorRunnable implements IXPRunnable 
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.application.PTGarbageCollectorRunnable");

	// Milliseconds between tests for the shutdown boolean
	private final int m_checkShutDown = 500;
	
	// Initial start time in milliseconds
	private long m_startTime = 0;
	
	// Interval in milliseconds between garbage collections 
	private long m_garbageCollectInterval = 0;
	
	// Boolean to support remot shutdown
	private boolean m_shutDown = false;
	
	/**
	 * Code executed in a thread to call garbage 
	 * collector at regular intervals. 
	 */
	public PTGarbageCollectorRunnable() 
	{
		super();
	}

	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() 
	{
		long intervalElapsed = 0;
		
		// Initialize for run
		m_startTime = XPSystem.GetCurrentTimeMilliSec();
		intervalElapsed = m_startTime; 
		
		// Loop until commanded to close
		while( !this.isShutDown() )
		{
			// Check every m_checkShutDown msec on the shutdown boolean
			XPThread.Sleep(m_checkShutDown);
			
			// If the interval has elapsed then call the garbage collector
			if ((XPSystem.GetCurrentTimeMilliSec() - intervalElapsed) >= m_garbageCollectInterval)
			{
				// Garbage collect
				XPSystem.GarbageCollect();
				
				
					log.Debug("Garbage collector called.");
								
				
				// Start counting down yet again
				intervalElapsed = XPSystem.GetCurrentTimeMilliSec(); 
			}
		}
	}
	
	/**
	 * Gets the interval in milliseconds between garbage 
	 * collections.
	 *  
	 * @return
	 */
	public long getGarbageCollectInterval() 
	{
		return m_garbageCollectInterval;
	}

	/**
	 * Tests the value of the shutDown boolean.
	 * If true, the thread will shut itself
	 * down.
	 * 
	 * @return
	 */
	public boolean isShutDown() 
	{
		return m_shutDown;
	}

	/**
	 * Sets the interval for garbage collection in 
	 * milliseconds.  Maximum is 23 days.
	 * 
	 * @param i
	 */
	public void setGarbageCollectInterval(long i) 
	{
		m_garbageCollectInterval = i;
	}

	/**
	 * Sets the shut down boolean. When true, the thread 
	 * terminates itself.
	 * 
	 * @param b
	 */
	public void setShutDown(boolean b) 
	{
		m_shutDown = b;
	}
}

