package com.plumtree.portaluiinfrastructure.application.varpacks;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.io.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.xml.dom.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.openkernel.config.*;

/**
 * This VarPack contains the localized names for all the installed locales
 *
 * @author Stephen Chang
 */
public class PTSearchLocaleVarPack extends BaseVarPack
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.application.varpacks.PTSearchLocaleVarPack");

	/** The string ID of this variable package. */
	public static final String VARPACK_ID = "PTSearchLocaleVarPack";

	// OpenConfig component type
	public static final String PORTAL_SEARCHLOCALES_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/searchlocales";

	// private static final String POPULAR_YES = "yes";
	private static final String POPULAR_SUFFIX = ".popular";

	private XPHashtable m_htAllLocales;
	private XPHashtable m_htPopularLocales;
	private XPArrayList m_xpAllLocaleCodes;
	private XPArrayList m_xpPopularLocaleCodes;

	/**
	 * This method will return the All the Language Codes for a particular locale
	 */
	public String[] GetAllLanguageCodes()
	{
		int nNumCodes = m_xpAllLocaleCodes.GetSize();
		String[] languageCodes = new String[nNumCodes];
		for(int i=0; i<nNumCodes; i++)
		{
			languageCodes[i] = (String) m_xpAllLocaleCodes.GetElement(i);
		}

		return languageCodes;
	}

	/**
	 * This method will return the All the Localized Language names for a particular locale
	 */
	public String[] GetAllLanguageNames(String strLocale)
	{
		return (String[]) m_htAllLocales.GetElement(strLocale);
	}

	/**
	 * This method will return the localized language name given its code.
	 */
	public String GetLanguageNameForCode(String strCode, String strLanguage)
	{
		String[] codes = GetAllLanguageCodes();
		String[] names = GetAllLanguageNames(strLanguage);

		if (null == names)
		{
			return null;
		}

		for (int j = 0; j < codes.length; j++)
		{
			if (codes[j].equals(strCode))
			{
				return names[j];
			}
		}

		return null;
	}

	/**
	 * This method will return the Popular Localized Language Codes for a particular locale
	 */
	public String[] GetPopularLanguageCodes()
	{
		int nNumCodes = m_xpPopularLocaleCodes.GetSize();
		String[] languageCodes = new String[nNumCodes];
		for(int i=0; i<nNumCodes; i++)
		{
			languageCodes[i] = (String) m_xpPopularLocaleCodes.GetElement(i);
		}

		return languageCodes;
	}

	/**
	 * This method will return the Popular Localized Language names for a particular locale
	 */
	public String[] GetPopularLanguageNames(String strLocale)
	{
		return (String[]) m_htPopularLocales.GetElement(strLocale);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackID()
	 */
	public String GetVarPackID()
	{
		return VARPACK_ID;
	}

	/**
	 * This method will return an enumeration of the top level nodes in the XML
	 * tree that is SearchLocales.xml.
	 *
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackKeys()
	 */
	public IXPEnumerator GetVarPackKeys()
	{
		return m_htAllLocales.GetKeys();
	}

	/**
	 * This method check whether this language appears in the popular list
	 */
	public boolean IsPopularLanguage(String strLocaleCode)
	{
		if(null == strLocaleCode)
			return false;

		int nNumPopularLocaleCodes = m_xpPopularLocaleCodes.GetSize();

		for (int i = 0; i < nNumPopularLocaleCodes; i++)
		{
			if (0 == XPStringUtility.CompareIgnoreCase(strLocaleCode,
					(String)m_xpPopularLocaleCodes.GetElement(i)))
			{
				return true;
			}
		}

		return false;
	}

	/**
	 * This method check whether this language appears in the ALL list
	 */
	public boolean IsValidLanguage(String strLocaleCode)
	{
		if(null == strLocaleCode)
			return false;
		
		int nNumAllLocaleCodes = m_xpAllLocaleCodes.GetSize();

		for (int i = 0; i < nNumAllLocaleCodes; i++)
		{
			if (0 == XPStringUtility.CompareIgnoreCase(strLocaleCode,
						(String) m_xpAllLocaleCodes.GetElement(i)))
			{
				return true;
			}
		}

		return false;
	}

	/**
	 * You cannot expire the SearchLocales.xml content. You can only reload it. This
	 * method does nothing.
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#ExpireContent()
	 */
	protected void ExpireContent()
	{
	}

	/**
	 * This method will return the Localized Language Codes for a particular locale
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#GetValue(String)
	 */
	protected Object GetValue(String strLocale)
	{
		return m_htAllLocales.GetElement(strLocale);
	}

	/**
	 * This method loads the SearchLocales.xml file into memory.
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#Load()
	 */
	protected void Load()
	{
		// Load Search Locale Codes
		try
		{
			m_xpAllLocaleCodes = new XPArrayList();
			m_xpPopularLocaleCodes = new XPArrayList();

			// Use OpenConfig to read the settings
			IOKContext configContext = AppWarmUp.GetOKConfigContext();
			IOKComponent component = null;
		
			component = configContext.getComponentByType(PORTAL_SEARCHLOCALES_COMPONENT_TYPE);

			if (null == component)
			{
				log.Warn("Cannot retrieve component type: {0}", PORTAL_SEARCHLOCALES_COMPONENT_TYPE);
				return;
			}			
			
			IOKSetting[] settings = component.querySettings();
			for(int j=0;j<settings.length;j++)
			{
				String strName = settings[j].getName();

				if (null == strName) 
				{
					log.Warn("PTSearchLocaleVarPack::Load() Found setting with null name");
					continue;
				} 
				
				boolean bValue = false;

				int suffixNdx = XPStringUtility.IndexOf(strName, POPULAR_SUFFIX);
				if (suffixNdx <=0)
				{
					log.Warn("PTSearchLocaleVarPack::Load() Found invalid setting name: {0}", strName);
					continue;
				}
				
				String strCode = XPStringUtility.Substring(strName, 0, suffixNdx);

				m_xpAllLocaleCodes.Add(strCode);
									        
				int nSettingType = settings[j].getDataType();
				switch(nSettingType)
				{
					case OKConfigDataTypes.TYPE_BOOLEAN:
						bValue = settings[j].getBooleanValue();
						if (bValue)	// if popular
						{
							m_xpPopularLocaleCodes.Add(strCode);
						}
						break;
					default:
						log.Warn("Unknown data type for setting name {0}: {1} ", strName, nSettingType);
				}

				//log.Info("Search locale loaded:{0}, popular: {1}", strCode,
				//	(bValue?"yes":"no"));		        
			}

			int nNumAllLanguages = m_xpAllLocaleCodes.GetSize();
			int nNumPopularLanguages = m_xpPopularLocaleCodes.GetSize();

			// Retrieve the Localized Names for the languages
			IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
			XPResourceManager rm     = (XPResourceManager) application.GetCachingManager()
				.GetEntry("ResourceManager");
			String[] asKnownLangs    = rm.GetSupportedLanguages();

			int nNumKnownLocales = asKnownLangs.length;

			m_htAllLocales		   = new XPHashtable();
			m_htPopularLocales     = new XPHashtable();

			for (int i = 0; i < nNumKnownLocales; i++)
			{
				String strLanguageName = asKnownLangs[i];

				String[] arAllLocalizedLanguageNames     = new String[nNumAllLanguages];
				String[] arPopularLocalizedLanguageNames = new String[nNumPopularLanguages];

				String strLocaleCode;
				String strLocaleName;

				// Retrieve names for ALL Inxight languages
				for (int j = 0; j < nNumAllLanguages; j++)
				{
					strLocaleCode	    = (String) m_xpAllLocaleCodes.GetElement(j);
					strLocaleName     = rm.GetDisplayLocaleFromLocaleID(XPStringUtility.ToLowerCase(strLocaleCode),
							strLanguageName);
					
					arAllLocalizedLanguageNames[j] = strLocaleName;
				}

				// Retrieve names for POPULAR Inxight languages
				for (int j = 0; j < nNumPopularLanguages; j++)
				{
					strLocaleCode	    = (String) m_xpPopularLocaleCodes.GetElement(j);
					strLocaleName     = rm.GetDisplayLocaleFromLocaleID(strLocaleCode,
							strLanguageName);

					arPopularLocalizedLanguageNames[j] = strLocaleName;
				}

				// Store the lists in the appropriate hash table
				m_htAllLocales.PutElement(strLanguageName,
					arAllLocalizedLanguageNames);
				m_htPopularLocales.PutElement(strLanguageName,
					arPopularLocalizedLanguageNames);
			}
		}
		catch (Exception e)
		{
			
				log.Error(e, "Unable to load data.");
			
		}
	}

	/**
	 * This method reloads the SearchLocales.xml file into memory. It clears the
	 * current data and then delegates to Load().
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#Reload()
	 */
	protected void Reload()
	{
		// We don't really need to null these out, but we do anyways.
		m_htAllLocales			   = null;
		m_htPopularLocales		   = null;
		m_xpAllLocaleCodes		   = null;
		m_xpPopularLocaleCodes     = null;

		Load();
	}
}

