package com.plumtree.portaluiinfrastructure.classtypedescriptors;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.providerframework.*;
import com.plumtree.portaluiinfrastructure.friendlyurl.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.opener.types.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This class manages the class type descriptor objects.
 * 
 * @author John Osborne
 * @author Ricky Wong
 */
public class ClassTypeDescManager extends ClassTypeManager implements
	IClassTypeManager, IPTClassTypeDescManager, IFriendlyURLManager 
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.classtypedescriptors.ClassTypeDescManager");

	//////////////////////////////
	// IPTClassTypeDesc Members //
	//////////////////////////////
	
	/** A map of ProvInfo[] objects available for a particular class. */
	private XPHashtable m_mapArrayProvInfoByClassID; 

	/** A map that has ClassID arrays ordered alphabetically by name. */
	private XPHashtable m_mapClassIDArraysByLangID;

	/** A map of ProvInfo objects. The key is the English name for the provider. */
	private XPHashtable m_mapProvInfoByEnglishName;

	/** An array of all loaded ProvInfo objects. */
	private IProvInfo[] m_aProvInfo;
	
	/** Mapping of class ids to friendly URL keys */
	private XPHashtable m_htLookupTable;
		
	/**
	 * @param cmCachingManager
	 */
	public ClassTypeDescManager(CachingManager cmCachingManager)
	{
		// load the dynamically discovered ClassTypeDesc objects.
		super(cmCachingManager);

	    /////////////////////////////////////
	    // IPTClassTypeDesc Initialization //
	    /////////////////////////////////////

		// load the dynamically discovered ProvInfo objects.
		Object[] oaProvInfo   = (Object[]) cmCachingManager.GetEntry("ProvInfo");
		IProvInfo[] aProvInfo = new IProvInfo[oaProvInfo.length];

		for (int i = 0; i < oaProvInfo.length; i++)
		{
			aProvInfo[i] = (IProvInfo) oaProvInfo[i];
		}

		m_aProvInfo					    = aProvInfo;
		m_mapProvInfoByEnglishName	    = new XPHashtable();
		m_mapArrayProvInfoByClassID     = new XPHashtable();
		m_mapClassIDArraysByLangID	    = new XPHashtable();

		////////////////////////////
		// General Initialization //
		////////////////////////////

		InitializeMapOfProvInfoByEnglishName();
		InitializeMapOfProvInfoArraysByClassID();
		InitializeMapOfListsOfClassIDsOrderedByName();
		
		////////////////////////////////////////
		// IFriendlyURLManager Initialization //
		////////////////////////////////////////
		m_htLookupTable = (XPHashtable) cmCachingManager.GetEntry("FriendlyURLKeyFromClassIdLookupTable");
	}

	private final void InitializeMapOfProvInfoByEnglishName()
	{
		for (int i = 0; i < m_aProvInfo.length; i++)
		{
			// Map by Provider English Name.
			m_mapProvInfoByEnglishName.PutElement(m_aProvInfo[i].
			    GetProviderEnglishName(), m_aProvInfo[i]);
		}
	}

	private final void InitializeMapOfProvInfoArraysByClassID()
	{
		for (int i = 0; i < m_aIClassTypes.length; i++)
		{
		    ClassTypeEnum myEnum = m_aIClassTypes[i].GetClassType();

		    // set ProvInfo only for extensible PT classtypes.
		    if (myEnum.equals(ClassTypeEnum.PTCLASSTYPE) &&
		        ((APTClassTypeDesc) m_aIClassTypes[i]).IsExtensible())
		    {
		        int nClassID = ((APTClassTypeDesc) m_aIClassTypes[i]).GetClassID();
					
				XPArrayList alHolder = new XPArrayList();

				for (int j = 0; j < m_aProvInfo.length; j++)
				{
					// Make map of arrays by class IDs
					int[] nClassIDs = m_aProvInfo[j].GetArrayOfClassIDsRelatedTo();

					for (int k = 0; k < nClassIDs.length; k++)
					{
						if (nClassIDs[k] == nClassID)
						{
							alHolder.Add(m_aProvInfo[j]);
							k = nClassIDs.length;
						}
					}
				}

				if (alHolder.GetSize() == 0)
				{
					return;
				}

				IProvInfo[] aProvInfos = new IProvInfo[alHolder.GetSize()];

				for (int j = 0; j < alHolder.GetSize(); j++)
				{
					aProvInfos[j] = (IProvInfo) alHolder.GetElement(j);
				}

				m_mapArrayProvInfoByClassID.PutElement(new Integer(nClassID),
					aProvInfos);
		    }
		}
	}
	
	private final void InitializeMapOfListsOfClassIDsOrderedByName()
	{
		IApplication application = ApplicationManager.GetInstance()
			.GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
		XPResourceManager rm = (XPResourceManager) application
			.GetCachingManager().GetEntry("ResourceManager");
		String[] aKnownLangs = rm.GetSupportedLanguages();

		// loop through the languanges.
		for (int k = 0; k < aKnownLangs.length; k++)
		{
			// make array list of PT classtypes.
			XPArrayList arrList = new XPArrayList();

			for (int i = 0; i < m_aIClassTypes.length; i++)
			{
			    ClassTypeEnum myEnum = m_aIClassTypes[i].GetClassType();

			    if (myEnum.equals(ClassTypeEnum.PTCLASSTYPE))
			    {
			        arrList.Add(m_aIClassTypes[i]);
			    }
			}

			// sort the list alphabetically by name.
			XPCollections.Sort(arrList, new ClassTypeComparator(aKnownLangs[k]));

			int[] aIDs = new int[arrList.GetSize()];

			for (int i = 0; i < arrList.GetSize(); i++)
			{
				aIDs[i] = ((APTClassTypeDesc) arrList.GetElement(i)).GetClassID();
			}

			m_mapClassIDArraysByLangID.PutElement(aKnownLangs[k], aIDs);
		}
	}

	/**
	 * Initializes an extensible Plumtree object.
	 *
	 * @param _ptObject The Plumtree Object.
	 * @param _sCLSID The CLSID of the provider.
	 */
	public static final void InitializeExtensibleObject(IPTObject _ptObject,
		String _sCLSID)
	{
		int nClassID = _ptObject.GetClassID();

		if (nClassID == PT_CLASSIDS.PT_AUTHSOURCE_ID)
		{
			((IPTAuthSource) _ptObject).SetProviderCLSID(_sCLSID);
		}
		else if (nClassID == PT_CLASSIDS.PT_DATASOURCE_ID)
		{
			((IPTDataSource) _ptObject).Initialize(_sCLSID);
		}
		else if (nClassID == PT_CLASSIDS.PT_FEDERATEDPORTAL_ID)
		{
			// TODO: What does IPTFederatedPortal do now??
			// ((IPTFederatedPortal) _ptObject).SetProviderCLSID(_sCLSID);
		}
		else if (nClassID == PT_CLASSIDS.PT_GADGET_ID)
		{
			((IPTGadget) _ptObject).Initialize(_sCLSID);
		}
	}

	/**
	 * Returns whether this class type appears in the create object dropdown.
	 * This method only applies to PTClassTypeDesc class types.
	 * 
	 * @param _nClassID ClassID of the object.
	 * 
	 * @return True if this class appears in the create object dropdown.
	 */
	public final boolean AppearsInMainCreateNewClassMenu(int _nClassID)
	{
	    return ((APTClassTypeDesc) GetClassTypeDescItem(
	        XPConvert.ToString(_nClassID))).AppearsInMainCreateNewClassMenu(); 
	}

	/**
	 * Returns whether this class type appears in the global object property 
	 * map utility.  This method only applies to PTClassTypeDesc class types.
	 * 
	 * @param _nClassID ClassID of the object.
	 * 
	 * @return True if this class appears in the global object property map 
	 * 		   utility.
	 */
	public final boolean AppearsInGlobalObjPropMap(int _nClassID)
	{
	    return ((APTClassTypeDesc) GetClassTypeDescItem(
	        XPConvert.ToString(_nClassID))).AppearsInGlobalObjPropMap(); 
	}

	/**
	 * Returns all registered ProvInfo items.
	 *
	 * @return All registered ProvInfo items
	 */
	public final IProvInfo[] GetAllRegisteredProvInfo()
	{
		return m_aProvInfo;
	}

	/**
	 * Returns the default icon to use for this PT class type.
	 *   
	 * @param _nClassID ClassID to get icon for.
	 * @param _strLangID If no icon, set alt in this language.
	 * 
	 * @return The default icon to use for this PT class type.
	 */
	public final HTMLImg GetDefaultIconForClassType(int _nClassID, String _strLangID)
	{
	    String strClassID = XPConvert.ToString(_nClassID);
	    
		// if global boject, just return the job icon.
		if (_nClassID == PT_CLASSIDS.PT_GLOBALOBJECT_ID)
		{
			return ((APTClassTypeDesc) GetClassTypeDescItem(XPConvert.ToString(PT_CLASSIDS.PT_JOB_ID)))
				.GetDefaultIcon();
		}

		HTMLImg htmlImg = ((APTClassTypeDesc) GetClassTypeDescItem(strClassID))
				.GetDefaultIcon();

		if (null != htmlImg)
		{
		    htmlImg.SetAlt(((APTClassTypeDesc) GetClassTypeDescItem(strClassID))
		    	.GetClassTypeLocalizedName(_strLangID));
		}
		
		return htmlImg;
	}

	public final HTMLImg GetDefaultIconForClassType(int _nClassID, String _strLangID, 
	    String _sProvEnglishName)
	{
		return GetDefaultIconForClassType(_nClassID, _strLangID);
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IPTClassTypeDescManager#GetProviderEnglishNameForExistingObject(int, int, com.plumtree.server.IPTSession)
	 */
	public final String GetProviderEnglishNameForExistingObject(int _nClassID,
		int _nObjectID, IPTSession _ptSession)
	{
		APTExtClassTypeDesc ptClassTypeDesc;

		try
		{
			ptClassTypeDesc = (APTExtClassTypeDesc) GetClassTypeDescItem(
			    XPConvert.ToString(_nClassID));

			String strProvCLSID = ptClassTypeDesc.GetCLSIDForExistingObject(
			    _nObjectID, _ptSession);

			return GetProviderEnglishNameGivenCLSID(strProvCLSID);
		}
		catch (Exception e)
		{			
			throw new XPException("Error retrieving English Provider name for given object.", e);
		}
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IPTClassTypeDescManager#GetProviderEnglishNameGivenCLSID(java.lang.String)
	 */
	public final String GetProviderEnglishNameGivenCLSID(String _sCLSID)
	{
		// retrieve the name from our discovered list of providers.
		for (int i = 0; i < m_aProvInfo.length; i++)
		{
			if (XPStringUtility.CompareIgnoreCase(
			    	m_aProvInfo[i].GetProviderCLSID(), _sCLSID) == 0)
			{
				return m_aProvInfo[i].GetProviderEnglishName();
			}
		}
		
		// special case for plumtree auth source.
		if (_sCLSID.equals("PLUMTREE"))
		{
			return _sCLSID;
		}
				
		// if the name is not on the list, throw an error.
		String errorMsg = "Error retrieving English Provider name for ClassID: " + _sCLSID;		
		
		log.Error(errorMsg);				
		throw new XPException(errorMsg);
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IPTClassTypeDescManager#GetRegisteredProvInfoForClassType(int)
	 */
	public final IProvInfo[] GetRegisteredProvInfoForClassType(int _nClassID)
	{
		if (m_mapArrayProvInfoByClassID.ContainsKey(new Integer(_nClassID)))
		{
			return (IProvInfo[]) m_mapArrayProvInfoByClassID.GetElement(new Integer(
					_nClassID));
		}

		return null;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IPTClassTypeDescManager#GetRegisteredProvInfoGivenEnglishName(java.lang.String)
	 */
	public final IProvInfo GetRegisteredProvInfoGivenEnglishName(
		String _strProvInfoEngName)
	{
		if (m_mapProvInfoByEnglishName.ContainsKey(_strProvInfoEngName))
		{
			return (IProvInfo) m_mapProvInfoByEnglishName.GetElement(_strProvInfoEngName);
		}

		return null;
	}

	/**
	 * True if the Plumtree Object of this ClassID is extensible.
	 *
	 * @param _nClassID The ClassID for this Plumtree Object.
	 *
	 * @return True if the Plumtree Object of this ClassID is extensible.
	 */
	public final boolean IsObjectExtensible(int _nClassID)
	{
		return ((APTClassTypeDesc) GetClassTypeDescItem(
		    XPConvert.ToString(_nClassID))).IsExtensible();
	}

	/**
	 * Returns the name of the activity space associated with the
	 * specified PT ClassID.
	 *
	 * @param _nClassID The ClassID for this Plumtree Object.
	 *
	 * @return The name of the associated activity space.
	 */
	public String GetASNameForClassTypeEditor(int _nClassID)
	{
		APTClassTypeDesc baseType = (APTClassTypeDesc) GetClassTypeDescItem(
		    XPConvert.ToString(_nClassID));

		if (baseType.IsExtensible())
		{
			String errorMsg = "Attempting to get non-extensible " +
				"editor name from extensible type: " + _nClassID;

			log.Error(errorMsg);
			throw new XPException(errorMsg);
		}

		APTNonExtClassTypeDesc realType = (APTNonExtClassTypeDesc) baseType;
		return realType.GetASNameForEditor();
	}

	public String GetASNameForClassTypeEditor(int _nClassID,
		String _StrProvEnglishName)
	{
		APTClassTypeDesc baseType = (APTClassTypeDesc) GetClassTypeDescItem(
		    XPConvert.ToString(_nClassID));

		if (!baseType.IsExtensible())
		{
			String errorMsg = "Attempting to get extensible editor name from " +
				"non-extensible type: " + _nClassID + " with provider name " + 
				_StrProvEnglishName;
			 
			log.Error(errorMsg);
			throw new XPException(errorMsg);
		}

		APTExtClassTypeDesc realType = (APTExtClassTypeDesc) baseType;
		return realType.GetASNameForEditor(_StrProvEnglishName);
	}

	/**
	 * Returns the localized name for a particular PT class.
	 *
	 * @param _nClassID The ClassID for this Plumtree Object.
	 *
	 * @return The localized name for a particular PT class.
	 */
	public String GetClassTypeLocalizedName(int _nClassID, String _strLangID)
	{
		return ((APTClassTypeDesc) GetClassTypeDescItem(XPConvert.ToString(_nClassID)))
			.GetClassTypeLocalizedName(_strLangID);
	}

	/**
	 * Returns the localized name for a particular PT class.
	 *
	 * @param _nClassID The ClassID for this Plumtree Object.
	 * @param _sProvEnglishName The English Provider Name.
	 *
	 * @return The localized name for a particular PT class.
	 */
	public String GetClassTypeLocalizedName(int _nClassID, String _strLangID, String _sProvEnglishName)
	{
		IProvInfo ptProvInfo = (IProvInfo) m_mapProvInfoByEnglishName.GetElement(_sProvEnglishName);

		return ptProvInfo.GetProviderAndClassLocalizedName(_strLangID, _nClassID);
	}

	public final int[] GetOrderedListOfClassIDs(String _strLangID)
	{
		return (int[]) m_mapClassIDArraysByLangID.GetElement(_strLangID);
	}

	public int[] GetRelatedClassTypeIDs(int _nClassID)
	{
		return ((APTClassTypeDesc) GetClassTypeDescItem(
			XPConvert.ToString(_nClassID))).GetRelatedClassTypeIDs();
	}

	public int[] GetDependentClassTypeIDs(int _nClassID)
	{
		return ((APTClassTypeDesc) GetClassTypeDescItem(
			XPConvert.ToString(_nClassID))).GetDependentClassTypeIDs();
	}

	public boolean HasGlobalClassTypeMap(int _nClassID)
	{
		return ((APTClassTypeDesc) GetClassTypeDescItem(
		    XPConvert.ToString(_nClassID))).HasGlobalClassTypeMap();
	}

	public boolean HasCreateActivityRight(int _nClassID, IPTSession _ptSession)
	{
		return ((APTClassTypeDesc) GetClassTypeDescItem(
		    XPConvert.ToString(_nClassID))).HasCreateActivityRight(_ptSession);
	}
    
    public IPTClassTypeDesc[] GetLockManagerClassTypes(String _strLangID) { 
        XPArrayList arrayList = new XPArrayList();
        for (int i = 0; i < m_aIClassTypes.length; i++) {
            if ((m_aIClassTypes[i] instanceof IPTClassTypeDesc) && 
                ((IPTClassTypeDesc)(m_aIClassTypes[i])).HasObjectLockManager()) {
                arrayList.Add(m_aIClassTypes[i]);
            }
        }
        XPCollections.Sort(arrayList, new LockManagerObjectComparator(_strLangID));
        
        return (IPTClassTypeDesc[])(arrayList.ToArray(new IPTClassTypeDesc[]{}));
    }
    
    /**
     * Creates a Redirect object by retrieving the correct class type descriptor to
     * parse the friendly URL.
     * 
     * @param _fURLParser The friendly URL parser containing URL path
	 * @param _asParent The parent activity space.
	 * @param _userSession The current session.
     * @return Redirect object that tells the portal what page to go to.
     */
    public Redirect GetFriendlyViewOpenerRedirect(
    	IFriendlyURLParser _fURLParser,
		AActivitySpace _asParent,
		Object _userSession)
	{
    	// Get the class type descriptor
    	IPTClassTypeDesc ptCLassTypeDesc = (IPTClassTypeDesc) GetClassTypeDescItem(_fURLParser.GetClassId());
    	log.Debug("ClassTypeDescManager:GetFriendlyViewOpenerRedirect Retrieved class type descriptor: " + ptCLassTypeDesc.getClass());
    	// Get Redirect object and return
		return ptCLassTypeDesc.GetRedirectViewObject(_fURLParser, _asParent, _userSession);		
	}
    
    public String ConstructFriendlyURL(FriendlyURL url, AActivitySpace space)
    {
    	// Get the class type descriptor
    	IPTClassTypeDesc ptClassTypeDesc = (IPTClassTypeDesc) GetClassTypeDescItem(url.GetClassIdAsString());
    	log.Debug("ClassTypeDescManager:ConstructFriendlyURL Retrieved class type descriptor: " + ptClassTypeDesc.getClass());
    	// Get Redirect object and return
		return ptClassTypeDesc.ConstructFriendlyURL(url, space);
    	
    }

	public String LookupFriendlyURLKeyFromClassId(String _strClassId) {
		
		// Community pages don't have a separate keyword, they are part of the community friendly URL,
		// so for the purpose of looking up keyword, return the same keyword as communities
		if ( (PT_CLASSIDS.PT_PAGE_ID+"").equals(_strClassId) ) {
			_strClassId = PT_CLASSIDS.PT_COMMUNITY_ID+""; 
		}
		if (null == m_htLookupTable) {
			return null;
		}
		String strKeyword = (String) m_htLookupTable.GetElement(_strClassId);
		// DCA - we're encoding the keyword in case it has non-ASCII chars
		if (null != strKeyword) {
			strKeyword = XPHttpUtility.UrlEncode(strKeyword, LinkConstants.UTF_8_ENCODING);
		}
		return strKeyword;
	}

	public void SetFriendlyURLLookupTable(XPHashtable htLookupTable) {
		m_htLookupTable = htLookupTable;
	}
}
