package com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.providerframework.*;
import com.plumtree.portaluiinfrastructure.opener.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.opener.types.*;
import com.plumtree.uiinfrastructure.statichelpers.*;

/**
 * This psuedo class type represents the ID to use to tell the
 * Common Opener to open Gateway objects.  This type is completely
 * generic.  Given the UUID of the web service, we will construct
 * the Gateway URL based on the remote server and portlet URLs.  
 * Querystring arguments are passed on into the destination URL,
 * thus allowing for control at the destination.
 * 
 * @author RickyW
 */
public class UIGateway extends AUIClassTypeDesc
{
    /** Name that identifies this UIClassTypeDesc */
    public static final String UICLASSTYPEDESC_CLASSNAME = "gateway";

    /** Querystring name to use for UUID.  */
    public static final String QS_WEBSERVICE_UUID = "wsUUID";
    
    /** Key to use for caching Gateway URLs in the Application. */
    public static final String CACHE_KEY_GATEWAY_URLS = "OpenerGatewayURLs";
    
	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IUIClassTypeDesc#GetClassName()
	 */
	public String GetClassName()
	{
	    return UICLASSTYPEDESC_CLASSNAME;
	}
	
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IClassTypeDesc#GetClassType()
     */
    public ClassTypeEnum GetClassType()
	{
	    return ClassTypeEnum.UICLASSTYPE;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IPTClassTypeDesc#GetRedirectViewObject(XPHashtable, AActivitySpace, Object)
	 */
	public Redirect GetRedirectViewObject(XPHashtable _htArguments, AActivitySpace _asParent, Object _userSession)
	{		    
		// retrieve the web service UUID from the URL.
		String strWebServiceUUID = HTTPHelpers.GetArgumentAsString(
		    _htArguments, UIGateway.QS_WEBSERVICE_UUID, null);
		
		if (null == strWebServiceUUID)
		{
			UIError myError = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER,  
				null, _asParent.GetString(497, "ptmsgs_portalinfrastructure"), null);
			UIErrorHandler.RaiseError(myError, _asParent.GetSubSession(), 
				_asParent.GetApplication());
						
			throw new XPException("A web service UUID argument is required in the " +
			    "Common Opener URL to open gateway objects.");			
		}

		// the ClassID and WebServiceUUID should not be passed.
		_htArguments.RemoveElement(IPTOpenerConstants.QS_CLASSID);
		_htArguments.RemoveElement(UIGateway.QS_WEBSERVICE_UUID);
		
		// other infrastructure arguments that should not be passed.
		_htArguments.RemoveElement(HttpUtils.HASHTABLE_CONTROL_KEY_URL);
		_htArguments.RemoveElement(HttpUtils.HASHTABLE_CONTROL_KEY_HTTP_HEADERS);
		
		// create the gateway URL to redirect to.  
		String strRedirectURL = GetGatewayURL(strWebServiceUUID, _asParent); 
		String strQSArguments = HTTPHelpers.GetQueryString(_htArguments);

		if ((null != strQSArguments) && 
		    (!strQSArguments.equalsIgnoreCase("")))
		{
		    // if there are QS arguments pass them through.
		    strRedirectURL = strRedirectURL + "?" + strQSArguments;
		}

		// set the redirect to be a true 302.
		Redirect resultingRedirect = new Redirect();
		resultingRedirect.SetLinkToExternalURL(strRedirectURL);

		return resultingRedirect;
	}

	/**
	 * Given the WebService UUID, we can construct the gateway URL that
	 * we need.  From the UUID, we get access to our web service and 
	 * the remote server for the web service.  From the remote server
	 * we can get the remote URL.  From the web service, we by default
	 * get the Portlet URL.  Given that external URL, we can generate
	 * our gateway URL.  The gateway URLs will be cached by UUID.  
	 * 
	 * @param _strWebServiceUUID The UUID of the Web Service to use.
	 * @param _asOwner The activity space gives us session and more.
	 * 
	 * @return The gateway URL given the specified web service UUID.
	 */
	public static String GetGatewayURL(String _strWebServiceUUID, AActivitySpace _asOwner)
	{
	    // retrieve the cached Gateway URLs hashtable.  we will
	    // be storing and retrieving from the persistent sub
	    // session as opposed to the application.  this handles
	    // the case where someone changes the web service.
	    XPHashtable htGatewayURLs = (XPHashtable) _asOwner.GetPersistentSubSession().
	    	GetAttribute(UIGateway.CACHE_KEY_GATEWAY_URLS);
	    
	    if (null == htGatewayURLs)
	    {
	        // if the Gateway URLs hashtable doesn't exist
	        // in the cache yet, we need to create it.
	        htGatewayURLs = new XPHashtable();
	    }	   

	    if (htGatewayURLs.ContainsKey(_strWebServiceUUID))
	    {
	        // the Gateway URL is in the cache, so use it.  what
	        // happens if someone changes the web service though?
	        return XPConvert.ToString(htGatewayURLs.GetElement(
	            _strWebServiceUUID));
	    }
	    else
	    {
	        XPStringBuilder sbPartialGatewayURL = new XPStringBuilder(300);

			// create the query for transforming our UUID.
			IPTSession ptSession = (IPTSession) _asOwner.GetUserSession();
			Object[][] ptQueryFilter = PlumtreeHelpers.GetQueryFilter(PT_PROPIDS.PT_PROPID_UUID, 
			    PT_FILTEROPS.PT_FILTEROP_EQ, _strWebServiceUUID);
			IPTQueryResult ptQueryResultWebServiceUUID = ptSession.GetWebServices().Query(
			    PT_PROPIDS.PT_PROPID_OBJECTID | PT_PROPIDS.PT_PROPID_WEBSERVICE_REMOTESERVERID, 
			    -1, PT_PROPIDS.PT_PROPID_OBJECTID, 0, -1, ptQueryFilter);

			// get the web service ID from the query.		
			int nWebServiceID = ptQueryResultWebServiceUUID.ItemAsInt(
			    0, PT_PROPIDS.PT_PROPID_OBJECTID);
			
			// get the remote server ID from the query.
			int nRemoteServerID = ptQueryResultWebServiceUUID.ItemAsInt(
			    0, PT_PROPIDS.PT_PROPID_WEBSERVICE_REMOTESERVERID);

			// open the remote server and get the remote URL.
			IPTGadgetContentServer ptRemoteServer = (IPTGadgetContentServer) ptSession.
				GetGadgetContentServers().Open(nRemoteServerID, false);
			sbPartialGatewayURL.Append(ptRemoteServer.GetURL());
			
			// retrieve and tack on the portlet URL, if it exists.
			IPTWebService ptWebService = (IPTWebService) ptSession.GetWebServices().
				Open(nWebServiceID, false);
			IXPPropertyBag ptProviderInfo = (IXPPropertyBag) ptWebService.GetProviderInfo();
			sbPartialGatewayURL.Append(PlumtreeHelpers.PBReadString(ptProviderInfo, 
			    AExtensibilityConstants.PT_PROPBAG_HTTPGADGET_URL, ""));
			
			// generate the Gateway URL from our information.
	        String strFullGatewayURL = GatewayHelpers.ConstructGatewayURL(
	            _asOwner.GetCurrentRequestURL(), nWebServiceID, 0, 0, 0, 
	            PT_CLASSIDS.PT_WEBSERVICE_ID, sbPartialGatewayURL.ToString());

	        // now that we have generated the Gateway URL, let's put
	        // it back into the Application cache for next time. 
	        htGatewayURLs.PutElement(_strWebServiceUUID, strFullGatewayURL);
	        _asOwner.GetPersistentSubSession().SetAttribute(
	            UIGateway.CACHE_KEY_GATEWAY_URLS, htGatewayURLs);
	            
	        return strFullGatewayURL;
	    }
	}
}
