package com.plumtree.portaluiinfrastructure.condition;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.condition.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.expandablelist.*;
import com.plumtree.uiinfrastructure.expandablelist.growablelist.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * Condition Type interface. Condition Types are needed for two reasons:
 * 1) To determine whether whether a condition of its type is met. This 
 * involves finding the current value and comparing it to the condition's
 * value. For example: Checking to see whether a condition the user's 
 * IP address involves getting the current user's IP Address from the http
 * request and checking to see if it matches the IP Address stored in the
 * expression. 
 * 2) Display the conditions of its type that have been created. Each expression
 * contains a list of conditions, one for each condition type. The list for
 * conditions of a certain type could have a display as simple as a textbox with
 * the condition value inside, or more elaborate involving a tree picker and 
 * row icons.
 * 
 * @author dustina
 */
public abstract class AConditionType implements IConditionServerType {
    
    protected static OpenLogger log = OpenLogService.GetLogger(
    		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
    		"com.plumtree.portaluiinfrastructure.condition.AConditionType");
    protected final static OpenLogger ruleDebugger = OpenLogService.GetLogger(OpenLogService.GetComponent("Condition"), 
            "com.plumtree.portaluiinfrastructure.condition.AConditionType");

	public static final String CONDITION_LIST_CONSTANT = "ConditionList";
	public static final int PHASE_TYPE_GUEST = 1;
	public static final int PHASE_TYPE_REGULAR = 2;
	
	/**
	 * Method to retrieve the phase type of this condition type. There are currently two phases:
	 * guest phase and regular phase.
	 * 
	 * @see com.plumtree.portaluiinfrastructure.condition.AGuestConditionType
	 * @see com.plumtree.portaluiinfrastructure.condition.ARegularConditionType
	 */
	public abstract int GetPhaseType();
	
	/**
	 * This method combines the CONDITION_LIST_CONSTANT variable with it's type ID
	 * to come up with a unique string that can be used to store this condition type's
	 * list of conditions in the activityspace.
	 * 
	 * @return A String the uniquely identifies this condition type
	 */
	public String GetConditionTypeIdentifier() {
		return CONDITION_LIST_CONSTANT + XPConvert.ToString(GetTypeID());
	}
	
	/**
	 * This method will register the type of Expandable List that this condition type will
	 * use to display its list of conditions. The expandable list is registered using the
	 * condition type's unique identifier.
	 * The Expandable List itself is merely an abstract class that needs to be extended. There
	 * are two list objects that have extended the Expandable List:
	 * 1) Growable List under package com.plumtree.uiinfrastructure.expandablelist.growablelist.
	 * This is a vary simple, but useful extenstion of the Expandable List consisting of an 
	 * input box per row. Adding a new condition is as easy as inputting the condition value in
	 * the text box.
	 * Check out com.plumtree.portalpages.condition.conditiontypes.ConditionTypeURLDomain as an
	 * example of how to use the Growable List.
	 * 2) Select PTObject List under package com.plumtree.portaluiinfrastructure.expandablelist.selectptobject.
	 * The Select PTObject List allows users to select ptobjects easily. It comes set up with a 
	 * pop up tree picker where you can select more than one object at a time, and it displays 
	 * the name of the objects selected in the rows.
	 * Check out com.plumtree.portalpages.condition.conditiontypes.ConditionTypeCommunityID as an
	 * example of how to use the Select PTObject List.
	 * If neither of the two lists above suits your need, another option would be to create a new
	 * list by extending either of the two lists mentioned above, or extending Expandable List
	 * itself. 
	 * 
	 * Registering the chosen list involves registering a list model, a list view, and a list control.
	 * To register the model, use the RegiserModel() method of the ActivitySpace:
	 * asCurrentSpace.RegisterModel(MyListModel.STR_MVC_CLASS_NAME, GetConditionTypeIdentifier());
	 * The registered list model is needed to register the view and the control, therefore, we need
	 * to get back the Model we just registered using this condition type's identifier:
	 * IModel myListModel = asCurrentSpace.GetModel(GetConditionTypeIdentifier());
	 * Now that we have the model, we can continue to register the list view and the list control:
	 * asCurrentSpace.RegisterView(MyListListView.STR_MVC_CLASS_NAME, GetConditionTypeIdentifier(), myListModel);
	 * asCurrentSpace.RegisterControl(ExpListControl.STR_MVC_CLASS_NAME, GetConditionTypeIdentifier(), myListModel);
	 * It is very likely that the ExpListControl is sufficient to use and will not need to be extended.
	 * In such a case, simply register the ExpListControl as your list control. 
	 * 
	 * @param asCurrentSpace - the activityspace the expandable list will be registered to
	 */
	public void RegisterMyConditionsList(AActivitySpace asCurrentSpace) {
		// Register the model.
		asCurrentSpace.RegisterModel(GrowableListModel.STR_MVC_CLASS_NAME, GetConditionTypeIdentifier());

		// Get back the Model we just registered because we need it to register the Section and the Control
		IModel myGrowableListModel = asCurrentSpace.GetModel(GetConditionTypeIdentifier());
		asCurrentSpace.RegisterView(GrowableListView.STR_MVC_CLASS_NAME, GetConditionTypeIdentifier(), myGrowableListModel);
		asCurrentSpace.RegisterControl(ExpListControl.STR_MVC_CLASS_NAME, GetConditionTypeIdentifier(), myGrowableListModel);
	}
	
	/**
	 * This method specifies what text will be displayed on the section title. Use the
	 * Resource Manager and the string language to localize.
	 * NOTE: The string returned by this method is not HTMLEncoded
	 * 
	 * Examples:
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeURLDomain
	 * "The base URL is ANY of the following URLs:"
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeClientIPAddress
	 * "The user's IP is ANY of the following IPs:"
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeCommunityID
	 * "The user is in ANY of the following Communities:"
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeGroupID
	 * "The user is in ANY of the following Groups:"
	 * 
	 * @param xpRM - the resource manager which can be used to localize strings.
	 * @param strLanguage - current user's language locale
	 */
	public abstract String GetMyConditionsListSectionTitle(XPResourceManager xpRM, String strLanguage);
	
	/**
	 * This method specifies what text will be displayed on the section sub title. Use the
	 * Resource Manager and the string language to localize.
	 * NOTE: The string returned by this method is not HTMLEncoded
	 * 
	 * Examples:
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeURLDomain
	 * "URL Domains"
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeClientIPAddress
	 * "Client IP Addresses"
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeCommunityID
	 * "Community Name"
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeGroupID
	 * "Group Name"
	 * 
	 * @param xpRM - the resource manager which can be used to localize strings.
	 * @param strLanguage - current user's language locale
	 */
	public abstract String GetMyConditionsListSectionSubTitle(XPResourceManager xpRM, String strLanguage);
	
	/**
	 * The Add label specifies what text will be displayed on the Add button. Use the
	 * Resource Manager and the string language to localize.
	 * NOTE: The string returned by this method is not HTMLEncoded
	 * 
	 * Examples:
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeURLDomain
	 * "Add URL"
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeClientIPAddress
	 * "Add an IP Address"
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeCommunityID
	 * "Add Communities"
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeGroupID
	 * "Add Groups"
	 * 
	 * @param xpRM - the resource manager which can be used to localize strings.
	 * @param strLanguage - current user's language locale
	 */
	public abstract String GetMyConditionsListAddLabel(XPResourceManager xpRM, String strLanguage);
	
	/**
	 * This method specifies what icon will be displayed on the add button.
	 * Use the PTImageType class to select an icon.
	 * 
	 */
	public abstract PTImageType  GetMyConditionsListIcon();
	
	/**
	 * This method initializes the expandable list registered by the RegisterMyConditionsList
	 * method. This method simply calls the expandable list's initialize method located in 
	 * the list's model. The name of the initialize method depends on which type of expandable
	 * list was registered.
	 * Initializing the expandable list simply involves specifying display items like the list's
	 * title, or the add label. Use the methods above: GetMyConditionsListSectionTitle(), 
	 * GetMyConditionsListSectionSubTitle(), GetMyConditionsListAddLabel(), GetMyConditionsListIcon() 
	 * to help initialize the list.
	 * 
	 * @param myListModel - the model of the expandable list registered with this condition type.
	 * The list's initialization method resides in the model. To be able to call the correct
	 * initialization method, casting the ExpListModel to the correct type may be needed.
	 * @param strLanguage - this is the laguange of the current user that is trying to create
	 * or edit an expression.
	 */
	public void InitializeMyConditionsList(ExpListModel myListModel, String strLanguage) {
		
		// Get the resource manager for localization
		IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
		XPResourceManager xpRM     = (XPResourceManager) application.GetCachingManager().GetEntry("ResourceManager");
		
		GrowableListModel myGrowableListModel = (GrowableListModel) myListModel;
		// Initialize the Growable list.
		//      _sSectionTitle: Main title.
	 	//      _sSectionSubTitle: Sub title.
		//      _imgAddIcon: Icon for the add action.
		//      _sAddLabel: Text to display next to the add icon.
		//      _nListKey: In case you have multiple Expandable Lists on
		//                 the same DP, they should have a different key
		//                 (otherwise, just set it to 1).
		myGrowableListModel.InitializeGrowableList(
				GetMyConditionsListSectionTitle(xpRM, strLanguage),
				GetMyConditionsListSectionSubTitle(xpRM, strLanguage),
				GetMyConditionsListIcon(),
				GetMyConditionsListAddLabel(xpRM, strLanguage),
				GetTypeID());
	}
	
	/**
	 * Each condition type has an expectation of what type of value they are to receive. A
	 * condition on url domain most likey will expect its value type to be of string, while
	 * a condition on community ID would have a value of type integer.
	 * 
	 * @return the condition value type that is expected by this condition type. The condition value
	 * types are specified in the ValueTypeEnum class.
	 */
	public abstract int GetConditionValueType();
	
	/**
	 * To return the condition value, we have to retrieve the data from the list using the nRow
	 * and saData variables. Then it needs to be converted to the value type that is expected.
	 * The default list used is the Growable List, which is what the URL condition type uses.
	 * To retrieve the data from the Growable List, use the GrowableListModel.EXPLIST_SORTEDARRAY_PROPID_INPUTTEXT
	 * constant: Object result = saData.GetItem(nRow, GrowableListModel.EXPLIST_SORTEDARRAY_PROPID_INPUTTEXT);
	 * At this point, the result has been retrieved as a String. The next step would be to convert
	 * the string object to the type returned by GetConditionValueType().
	 * This method is also where validation of the value occurs. If the value is suppose to be an integer,
	 * validate that it is and throw a ValidationFailedException if it's not.  
	 * 
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeURLDomain
	 * 
	 * @param nRow - the row index of the object wanted
	 * @param saData - the list containing the object
	 * 
	 * @return - the object at row nRow of saData formatted to the condition type's value type
	 */
	public abstract Object GetConditionValue(int nRow, IPTGrowableSortedArrayWrapperRO saData);
	
	/**
	 * If a ValidationFailedException occurs, the RulesEditorModel will call this method
	 * to retrieve the appropriete error message.
	 * NOTE: The string returned by this method is not HTMLEncoded
	 * 
	 * @param xpRM - the resource manager which can be used to localize strings.
	 * @param strLanguage - current user's language locale
	 * @return the string that will be displayed as the error message
	 */
	public abstract String GetValidationFailedString(XPResourceManager xpRM, String strLanguage);
	
	/**
	 * This method is almost like the opposite of the GetConditionValue() method. It takes the
	 * objItem, which is the server version of the condition value, and converts it to the UI
	 * version. The variable objItem is currently of the type returned by GetConditionValueType(). 
	 * It now needs to be converted to the type that the ExpListModel understands. If you are using 
	 * the default Growable List, then the objItem simply needs to be converted to a string. Then, 
	 * add it to a String[], and call the AddRowsToList(String[]) method.
	 * 
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeURLDomain
	 * 
	 * @param objItem - the condition value that needs to be added to the list.
	 * @param myListModel - the list model which holds the data of items in the
	 * list
	 * @param ptSession - the current user's session is provided in case it is needed
	 */
	public abstract void AddItemToMyConditionsList(Object objItem, ExpListModel myListModel, IPTSession ptSession);
	
	/**
	 * Constructs the string to be set in an expression's description. Simply return
	 * a string describing the current condition.
	 * NOTE: The string returned by this method will be HTMLEncoded
	 * 
	 * Example:
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeURLDomain
	 * "URL Domain contains xxx"
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeClientIPAddress
	 * "Client IP Address matches xxx.xxx.xxx.xxx"
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeCommunityID
	 * "User is in Community: xxx"
	 * @see com.plumtree.portalpages.condition.conditiontypes.ConditionTypeGroupID
	 * "User part of Group: xxx"
	 * 
	 * @param ConditionValue - the condition value for this condition type
	 * @param ptSession - the current user's session is provided in case it is needed
	 * 
	 * @return string containing the condition type's name and condition value name
	 */
	public abstract String GetDescriptionString(IValue ConditionValue, IPTSession ptSession);
}
