package com.plumtree.portaluiinfrastructure.condition;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.server.*;


/**
 * There are two things to consider before deciding whether to extend the AGuestConditionType or the ARegularConditionType: 
 * 1) Will conditions of this type be used to determine which subportal with a particular guest to use when 
 * there is no valid session available? 
 * The reason is that certain condition types does not make sense during the Guest/Login Phase. For example, a
 * condition on the user's group has no meaning during the Guest/Login Phase since no user is logged in yet. A 
 * condition on the client's IP address would be valid since this has meaning on bothe the Guest/Login and the
 * Regular Phase. 
 * 2) Is the Activity Space NOT needed to get the current value of the condition type? 
 * A condition the current community a user is on needs the activity space to determine the current value. However,
 * during the Guest/Login Phase, the activity space has not been initialized. Therefore, there is no way to 
 * determine what community the user is in. Since the current value cannot be determined during the Guest/Login 
 * Phase, a condition on community is only valid during the regular Phase. A condition on the URL Domain, on the 
 * other hand, only needs the XPRequest to figure out its current value. Therefore, it is safe to have a condition 
 * on the user's URL domain during the Guest/Login Phase.
 * 
 * @author DustinA
 */
public abstract class AGuestConditionType extends AConditionType {

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.condition.AConditionType#GetPhaseType()
	 */
	public int GetPhaseType() {
		return AConditionType.PHASE_TYPE_GUEST;
	}
	
	/**
	 * GetCurrentValue determines the condition type's current value, and adds it to the htUserEnvironment. This is important
	 * since the Compare method is relying for the correct current value to be in the htUserEnvironment. If the current
	 * value is missing or incorrect, then the execution of the rules might return erroneous values. 
	 * NOTE: Any exception thrown from this method will be caught and discarded.
	 * 
	 * @param xpRequest - http request's wrapper. This is actually another wrapper that conceals the real XPRequest object.
	 * The reason is that at the Guest/Login Phase, the XPRequest's body cannot be parsed. Therefore, any attempt to get
	 * the XPRequest's parameter names or values will result in an exception being thrown.
	 * @param userSession - a guest read only ptSession that can be used to access server objects
	 * @param htUserEnvironment - the user's environment of current values for condition types
	 */
	public abstract void GetCurrentValue(XPLimitedRequest xpRequest, IPTSession guestReadOnlySession, XPHashtable htUserEnvironment);
	
	
	
}
