package com.plumtree.portaluiinfrastructure.condition;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;

/**
 * Wrapper class for the XPRequest in the openfoundation jar. This class does everything
 * except for certain methods which reads the body of the request and causes the
 * gateway to fail.
 * 
 */
public class XPLimitedRequest {

	private XPRequest m_xpRealRequest;
	
	/**
	 * Constructor - Creates an XPLimitedRequest wrapper object around xpRealRequest which
	 * must implement the IXPRequest Interface.
	 * 
	 * @param xpRealRequest
	 */
	public XPLimitedRequest(IXPRequest xpRealRequest) {
		m_xpRealRequest = (XPRequest) xpRealRequest;
	}
	
	/**
	 * Returns the named attribute
	 * 
	 * @param name     Name of the attribute
	 * @return         Object value of the attribute
	 *
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetAttribute(java.lang.String)
	 */
	public Object GetAttribute(String name) {
		return m_xpRealRequest.GetAttribute(name);
	}

	/**
	 * Returns the length of the content sent by the client in bytes
	 * 
	 * @return      Length of the content sent by the client in bytes
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetContentLength()
	 */
	public int GetContentLength() {
		return m_xpRealRequest.GetContentLength();
	}

	/**
	 * Returns the name of the authentication scheme used.
	 * 
	 * @return      string indicating the authentication scheme,
	 * or null if no authentication is being used.	
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetAuthType()
	 */
	public String GetAuthType() {
		return m_xpRealRequest.GetAuthType();
	}

	/**
	 * Returns the user name being used for basic authentication.
	 * 
	 * @return      The user name being used for basic authentication,
	 * empty string if basic authentication is not being used.	
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetBasicAuthUserName()
	 */
	public String GetBasicAuthUserName() {
		return m_xpRealRequest.GetBasicAuthUserName();
	}

	/**
	 * Returns the password being used for basic authentication.	
	 * 
	 * @return      The password being used for basic authentication,
	 * empty string if basic authentication is not being used.	
	 *
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetBasicAuthPassword()
	 */
	public String GetBasicAuthPassword() {
		return m_xpRealRequest.GetBasicAuthPassword();
	}

	/**
	 * Returns the IP address of the client as a String in dotted quad form.
	 * 
	 * @return      the IP address of the client in dotted quad form.
	 *
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetClientAddress()
	 */
	public String GetClientAddress() {
		return m_xpRealRequest.GetClientAddress();
	}

	/**
	 * Returns the MIME type of the content sent by the client
	 * 
	 * @return     MIME type of the content sent by the client. For e.g. text/html
	 * null if type is not known.
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetContentType()
	 */
	public String GetContentType() {
		return m_xpRealRequest.GetContentType();
	}

	/**
	 * Returns the value of the given HTTP header 
	 *  
	 * @param  name 	Name of the HTTP header. For e.g. HTTP_USER_AGENT
	 * @return 			The given HTTP header as a String, null if not found.
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetHeader(java.lang.String)
	 */
	public String GetHeader(String name) {
		return m_xpRealRequest.GetHeader(name);
	}

	/**
	 * Returns an Enumeration of names of all HTTP headers in the HTTP request
	 * 
	 * @return      Enumeration of names of all HTTP headers in the HTTP request
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetHeaderNames()
	 */
	public IXPEnumerator GetHeaderNames() {
		return m_xpRealRequest.GetHeaderNames();
	}

	/**
	 * Returns an Enumeration of XPLocale that the client will accept content in, 
	 * based on the Accept-Language header. 
	 * 
	 * @return      Enumeration of XPLocale objects.
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetLocales()
	 */
	public IXPEnumerator GetLocales() {
		return m_xpRealRequest.GetLocales();
	}

	/**
	 * Returns the HTTP data transfer method used by the client
	 *  
	 * @return      String value that specifies the HTTP data transfer method used
	 * by the client. For e.g., GET, POST, HEAD etc.
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetMethod()
	 */
	public String GetMethod() {
		return m_xpRealRequest.GetMethod();
	}

	/**
	 * Returns a boolean indicating whether this request was
	 * made using a secure channel, such as HTTPS.
	 *  
	 * @return      a boolean indicating whether this request was
	 * made using a secure channel, such as HTTPS.
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#IsSecureConnection()
	 */
	public boolean IsSecureConnection() {
		return m_xpRealRequest.IsSecureConnection();
	}

	/**
	 * Returns the query string
	 * 
	 * @return      a String that is the query string, null if no
	 * query string present.
	 *  
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetQueryString()
	 */
	public String GetQueryString() {
		return m_xpRealRequest.GetQueryString();
	}

	/**
	 * Returns the session ID specified by the client
	 * 
	 * @return      a String that is the session ID, null if not specified.
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetRequestedSessionId()
	 */
	public String GetRequestedSessionId() {
		return m_xpRealRequest.GetRequestedSessionId();
	}

	/**
	 * Returns the section of the URL between the protocol and the query string.
	 * 
	 * @return      A String for the URI.
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetRequestURI()
	 */
	public String GetRequestURI() {
		return m_xpRealRequest.GetRequestURI();
	}

	/**
	 * Returns the full URL, without the query string.
	 * 
	 * @return      A String for the full URL, without query string.
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetRequestURL()
	 */
	public String GetRequestURL() {
		return m_xpRealRequest.GetRequestURL();
	}

	/**
	 * Returns the name of the remote user, if present; otherwise empty string.
	 * 
	 * @return      The remote user name.
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetRemoteUser()
	 */
	public String GetRemoteUser() {
		return m_xpRealRequest.GetRemoteUser();
	}

	/**
	 * Returns a String containing the name of the scheme used to make this request
	 * (such as http, https, or ftp).
	 * 
	 * @return      a String containing the name of the scheme used to make this request
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetScheme()
	 */
	public String GetScheme() {
		return m_xpRealRequest.GetScheme();
	}

	/**
	 * Returns the host name of the server.
	 * 
	 * @return      A String containing the host name of the server
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetServerName()
	 */
	public String GetServerName() {
		return m_xpRealRequest.GetServerName();
	}

	/**
	 * Returns the server port.
	 * 
	 * @return      The integer value of the server port.
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetServerPort()
	 */
	public int GetServerPort() {
		return m_xpRealRequest.GetServerPort();
	}

	/**
	 * Returns the XPSession for this request.
	 *  
	 * @return      the XPSession for this request.
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetSession()
	 */
	public XPSession GetSession() {
		return m_xpRealRequest.GetSession();
	}

	/**
	 * Returns the complete path to the uploaded file.  This method is only
	 * valid after XPRequest.Save() has been called.  It will return null
	 * until then.
	 *  
	 * @return      a String containing the complete path to the uploaded file,
	 * null if no file was uploaded.
	 * 
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetCompletePath()
	 */
	public String GetCompletePath() {
		return m_xpRealRequest.GetCompletePath();
	}

	/**
	 * Returns the path to the remote file to be uploaded.  This is the last
	 * file to be uploaded in a multi-part MIME request.
	 *  
	 * @return      a String containing the path to the remote file to be uploaded,
	 * null if no file was uploaded.
	 *  
	 * @see com.plumtree.openfoundation.web.IXPRequest#GetRemoteFilePath()
	 */
	public String GetRemoteFilePath() {
		return m_xpRealRequest.GetRemoteFilePath();
	}

}
