package com.plumtree.portaluiinfrastructure.editor;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.statichelpers.*;

/**
 * This is the Model class for Global Objects.
 *
 * This class contains the Global Object as a member variable.
 * It initializes it so the Object is ready to be used.
 * Model classes for Global Objects should extend this class.
 *
 * @author Julie Fournier
 */
public abstract class GbObjEditorModel extends EditorModel
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.editor.GbObjEditorModel");


	/** Query String constant param name for Global Object ID */
	public static final String EDITOR_QS_INT_QS_GLOBAL_OBJECT_ID = "globalObjectID";

	/**  The Plumtree Session. */
	protected IPTSession m_Session;

	/** 
	 *  JF- Server objects
	 * The Global Object being edited.
	 */
	protected Object m_ptGlobalObject;

	/**
	 * JF- This is to identify the global object. It has to be a PT_GLOBALOBJECTS constant.
	 */
	protected int m_nGlobalObjectID = -1;

	/**
	 * That method is called by the EditorStartControl.
	 * It creates the global object and puts it in m_ptGlobalObject.
	 *
	 * @param _htFormData
	 *
	 * @return boolean Is true if the current user has the rights to access this Editor.
	 */
	public boolean CheckEditorAccessAndStart(XPHashtable _htFormData)
	{
		// JF- First we check the user access rights
		if (!m_Session.GetSessionInfo().CheckForActivityRight(
						PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_ACCESS_UTILITIES))
		{
			UIError myError = new UIError(UIErrorLevel.SERIOUS,
				UIErrorScope.CURRENT_USER, null, m_asOwner.GetString(429, "ptmsgs_portalinfrastructure"),
				null);
			UIErrorHandler.RaiseError(myError,
				m_asOwner.GetSubSession(), m_asOwner.GetApplication());
			return false;
		}

		// Set global object ID with values from QS.
		SetMemberVariablesFromQS(_htFormData);

		try
		{
			// JF- We open the Global Obejct with the ID provided in the QS.
			m_ptGlobalObject = m_Session.OpenGlobalObject(m_nGlobalObjectID, false);
		}
		catch (Exception e)
		{
			GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_UI_Infrastructure"), 
					"GbObjEditorModel::CheckEditorAccessAndStart", 
					e, true, UIErrorLevel.SERIOUS, 
					m_asOwner.GetString(152, "ptmsgs_portalinfrastructure"));
		}

		StartMigrationHistory();

		DoTaskOnStartEditor(_htFormData);

		return true;
	}

	/**
	 * Returns the the Global Object being edited.
	 *
	 * @return IPTObject
	 */
	public Object GetGlobalObject()
	{
		return (m_ptGlobalObject);
	}

	/**
	 * Returns the IPTSession for the current user.
	 *
	 * @return IPTSession
	 */
	public IPTSession GetPTSession()
	{
		return (m_Session);
	}

	/**
	 * Inits the Model with the IPTSession.
	 *
	 * @param parent
	 */
	public void Init(AActivitySpace parent)
	{
		super.Init(parent);
		m_Session = (IPTSession) parent.GetUserSession();
	}

	/**
	 * Sets the global object ID.
	 *
	 * @param IPTObject
	 */
	public void SetGlobalObjectID(int _nGbObjectID)
	{
		m_nGlobalObjectID = _nGbObjectID;
	}

	/**
	 * Helper method for the StartEditor method.
	 * Fill out the member variables with data from QS.
	 *
	 * @param _htFormData
	 */
	public void SetMemberVariablesFromQS(XPHashtable _htFormData)
	{
		// JF- Get the Global Object ID form the QS and 
		// set it in the associated member variable.
		String[] data = (String[]) _htFormData.GetElement(EDITOR_QS_INT_QS_GLOBAL_OBJECT_ID);

		if ((null != data) && (0 != data.length))
		{
			SetGlobalObjectID(XPConvert.ToInteger(data[0]));

			
				log.Debug("GbObjEditorModel::SetMemberVariablesFromQS Global Object ID: " +
					data[0]);
			
		}
		else
		{
			
				log.Warn("GbObjEditorModel::SetMemberVariablesFromQS Global Object ID was not in Query String.");
			
		}
	}

	/**
	 * Saves the Migration history poge.
	 */
	protected void StartMigrationHistory()
	{
		((MigrationHistoryPluginModel) m_asOwner.GetModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME)).InitializeHistory(PT_CLASSIDS.PT_GLOBALOBJECT_ID,
			m_nGlobalObjectID);
	}
}


