package com.plumtree.portaluiinfrastructure.editor;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.keywords.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity.*;
import com.plumtree.portaluiinfrastructure.editor.saveobject.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.expandablelist.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This is the Activity Space class for the ObjEditor Framework.
 * Editor Activity Space classes should extend this class.
 *
 * @author Julie Fournier
 */
public abstract class ObjEditorAS extends PTEditorAS
{
	/**
	 * MJD- Storage name for Keywords classes
	 */
	public static final String STR_MVC_STORAGE_NAME_KEYWORDS = "ObjectKeywordsObjEditor";
	
	/**
	 * JF- Storage name for Security classes
	 */
	public static final String STR_MVC_STORAGE_NAME_SECURITY_TAB = "ObjectSecurityObjEditor";

	/**
	 * JF- Storage name for Priority drop-down classes
	 */
	public static final String STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN = "ObjectSecurityPriorityDropDown";

	/** Javascript function name */
	public static final String JAVASCRIPT_OBJEDITOR_OPEN = "ObjEditor_OpenEditor";

	/**
	 * Key for the AS to store the save object pop-up as a sub space.
	 */
	public static final String SAVEOBJECT_SUBSPACE_KEY = "saveObjectPopUp";

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.AActivitySpace#CheckBasicAccess(String, String, boolean)
	 */
	public boolean CheckBasicAccess(String strPage, String strControl,
		boolean bSameSpace)
	{
		boolean bAccess = super.CheckBasicAccess(strPage, strControl, bSameSpace);

		// JF- The save pop-up is going to redirect to one of the Editor pages 
		// (Security Page or RedirectDP or ObjDependenciesDP).
		// We know it's OK because the save pop-up is a sub AS of this one 
		// and we know the access has been checked already.
		if ((ObjectSecurityDP.STR_MVC_CLASS_NAME.equals(strPage))
			|| (SaveObjectRedirectDP.STR_MVC_CLASS_NAME.equals(strPage))
			|| (ObjDependenciesDP.STR_MVC_CLASS_NAME.equals(strPage)))
		{
			bAccess = true;
		}

		return bAccess;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.IFormAS#DisplayCommonJavaScript()
	 */
	public HTMLScriptCollection DisplayCommonJavaScript()

	{
		HTMLScript myScript = new HTMLScript("text/javaScript");

		XPStringBuilder sb = new XPStringBuilder(200);

		// ObjEditor_OpenEditor
		sb.Append("function ").Append(JAVASCRIPT_OBJEDITOR_OPEN).Append("(nClassID, nObjectID, nFolderID, nOpenerMode, sParentName, sParentID, strMiscQSArgs)\n");
		sb.Append("{\n");
		sb.Append("	var strURL = \"" + PTConfigHelper.GetApplicationBaseURL(this) +
			"space=Opener&cached=true&\";\n");
		sb.Append("	strURL = strURL + \"parentname=\" + sParentName + \"&\"\n");
		sb.Append("	strURL = strURL + \"parentid=\" + sParentID + \"&\"\n");
		sb.Append("	strURL = strURL + \"in_hi_ClassID=\" + nClassID + \"&\"\n");
		sb.Append(
			"	strURL = strURL + \"in_hi_ObjectID=\" + nObjectID + \"&\"\n");
		sb.Append(
			"	strURL = strURL + \"in_hi_FolderID=\" + nFolderID + \"&\"\n");
		sb.Append(
			"	strURL = strURL + \"in_hi_OpenerMode=\" + nOpenerMode + \"&\"\n");
			
		sb.Append("	if (null != strMiscQSArgs)\n");	
		sb.Append("	{\n");	
		sb.Append("	  strURL = strURL + strMiscQSArgs; \n");	
		sb.Append("	}\n");
		// TODO CR - MJD : document.location was deprecated in Javascript 1.1, use window.location instead		
		sb.Append("	document.location = strURL;\n");
		sb.Append("}\n");

		myScript.AddInnerHTMLString(sb.ToString());

		HTMLScriptCollection myScriptCollection = super.DisplayCommonJavaScript();
		myScriptCollection.AddInnerHTMLElement(myScript);

		return myScriptCollection;
	}

	/**
	 * Returns the class name of the Dependencies Section.
	 *
	 * This page will be displayed at the end of the save pop-up.
	 * It will inform the user that the Object was successfully created
	 * and can also include some intructions on what to do next.
	 *
	 * Object Editors should override this method if they have their own
	 * implementation of ObjDependenciesView.
	 *
	 * @return String
	 */
	public String GetDependenciesViewName()
	{
		return (ObjDependenciesView.STR_MVC_CLASS_NAME);
	}
	
	/**
	 * Returns the truncated name of the object being created or edited.
	 *
	 * @return String
	 */
	public String GetObjectName()
	{
		String sObjectName = ((ObjEditorModel) GetModel(m_strModelName)).GetObjectName();
		sObjectName = GeneralHelpers.TruncateLongString(sObjectName, ObjEditorModel.OBJECT_NAME_TRUNCATE, false);
		
		return sObjectName;
	}

	/**
	 * Returns the class name of the Editor Repost Control.
	 *
	 * Editors should also implement that method and return
	 * the name of their own repost  control, unless they
	 * don't have any. In that case they don't need to implement
	 * the method and EditorRepostControl will be used by default.
	 *
	 * @return String
	 */
	public String GetRepostControlName()
	{
		return (ObjEditorRepostControl.STR_MVC_CLASS_NAME);
	}

	/**
	 * Registers Views, Controls and Models used by the Editor Framework.
	 *
	 * Warning: m_myModel must be set before calling Init().
	 */
	public void Init()
	{
		super.Init();

		// JF- Register the Repost Control
		RegisterControl(ObjEditorRepostControl.STR_MVC_CLASS_NAME,
			GetModel(m_strModelName));

		// JF- Register classes for the localize names and descriptions page
		// JF- Note that we don't need a storage name for this page because, nobody will extend those classes.
		// Whereas for the Security page, some people might extend the classes and use the name 
		// of the super class when they register it.
		RegisterModel(LocNamesModel.STR_MVC_CLASS_NAME);
		RegisterView(LocNamesView.STR_MVC_CLASS_NAME,
			GetModel(LocNamesModel.STR_MVC_CLASS_NAME));
		RegisterView(LocNamesPrimaryNameView.STR_MVC_CLASS_NAME,
			GetModel(LocNamesModel.STR_MVC_CLASS_NAME));
		RegisterView(LocNamesPopUpView.STR_MVC_CLASS_NAME,
			GetModel(LocNamesModel.STR_MVC_CLASS_NAME));
		RegisterControl(LocNamesRepostControl.STR_MVC_CLASS_NAME,
			GetModel(LocNamesModel.STR_MVC_CLASS_NAME));

		// Register Plug-In: Object Properties
		RegisterModel(ObjPropModel.STR_MVC_CLASS_NAME);
		RegisterView(ObjPropEditView.STR_MVC_CLASS_NAME,
			GetModel(ObjPropModel.STR_MVC_CLASS_NAME));
		RegisterControl(ExpListControl.STR_MVC_CLASS_NAME,
			GetModel(ObjPropModel.STR_MVC_CLASS_NAME));
			
		// Register Plug-In: Keywords
		RegisterModel(KeywordsModel.STR_MVC_CLASS_NAME,
			STR_MVC_STORAGE_NAME_KEYWORDS);
		RegisterView(KeywordsView.STR_MVC_CLASS_NAME,
			STR_MVC_STORAGE_NAME_KEYWORDS,
			GetModel(STR_MVC_STORAGE_NAME_KEYWORDS));
		RegisterControl(ExpListControl.STR_MVC_CLASS_NAME,
			STR_MVC_STORAGE_NAME_KEYWORDS,
			GetModel(STR_MVC_STORAGE_NAME_KEYWORDS));

		LocNamesDP dpLocNames = new LocNamesDP();
		dpLocNames.AddToValidationFunction(LocNamesPrimaryNameView.VALIDATION_FUNC_NAME);
		RegisterPage(dpLocNames);
		RegisterPage(new LocNamesPopUpDP());

		// JF- Register classes for the security page
		RegisterModel(ObjectSecurityModel.STR_MVC_CLASS_NAME,
			STR_MVC_STORAGE_NAME_SECURITY_TAB);
		RegisterView(ObjectSecurityView.STR_MVC_CLASS_NAME,
			STR_MVC_STORAGE_NAME_SECURITY_TAB,
			GetModel(STR_MVC_STORAGE_NAME_SECURITY_TAB));
		RegisterControl(ExpListControl.STR_MVC_CLASS_NAME,
			STR_MVC_STORAGE_NAME_SECURITY_TAB,
			GetModel(STR_MVC_STORAGE_NAME_SECURITY_TAB));
		RegisterPage(new ObjectSecurityDP());

		// JF- Register classes for the priority drop-down just below the Security tab
		RegisterModel(PriorityDropDownModel.STR_MVC_CLASS_NAME,
			STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN);
		RegisterView(PriorityDropDownView.STR_MVC_CLASS_NAME,
			STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN,
			GetModel(STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN));

		// Register Migration History Plugin
		RegisterModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME);
		RegisterView(MigrationHistoryView.STR_MVC_CLASS_NAME,
			GetModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME));
		RegisterPage(new MigrationHistoryPluginDP());

		// Register Dependencies class
		RegisterPage(new ObjDependenciesDP());
		RegisterView(ObjDependenciesView.STR_MVC_CLASS_NAME,
			GetModel(m_strModelName));
	}
	
	
	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.AActivitySpace#SetCurrentPage()
	 *
	 * @param IDisplayPage the current Display Page
	 */
	public void SetCurrentPage(IDisplayPage page)
	{
		super.SetCurrentPage( page );
		
		String sPageName = page.GetName();
		
		if( MigrationHistoryPluginDP.STR_MVC_CLASS_NAME.equals( sPageName ))
		{
			this.SetHelpTopic( "PT_MigrationHistoryandStatus" );			
		}
		else if( ObjPropBrowseDP.STR_MVC_CLASS_NAME.equals( sPageName ))
		{
			this.SetHelpTopic( "PT_PropertiesandNames" );
		}
		else if( ObjectSecurityDP.STR_MVC_CLASS_NAME.equals( sPageName ))
		{
			this.SetHelpTopic( "PT_Security" );
		}
		else if( LocNamesDP.STR_MVC_CLASS_NAME.equals( sPageName ))
		{
			this.SetHelpTopic( "PT_PropertiesandNames" );
		}
		
		
	}
	
	
}

