package com.plumtree.portaluiinfrastructure.editor;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.common.constants.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.keywords.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity.*;
import com.plumtree.portaluiinfrastructure.pei.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.marshalers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.web.*;


/**
 * This is the Model class for the Editor Framework.  This class assumes the
 * framework is based on an IPTObject.  Editors based on other Objects like
 * for example Global Objects  need to extend EditorModel directly.  This
 * class contains the IPTObject as a member variable.  It initializes
 * IPTObject when the editor starts and stores it in the DB when the user is
 * done. Model classes should extend this class.
 *
 * @author Julie Fournier
 */
public abstract class ObjEditorModel extends EditorModel
	implements IObjEditorModelRO, IMigrationModelRO, IDescriptionModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.editor.ObjEditorModel");

	/** Query String param name: Class ID */
	public static final String EDITOR_QS_INT_CLASS_ID = "classID";

	/** Query String param name: Object ID */
	public static final String EDITOR_QS_INT_QS_OBJECT_ID = "objectID";

	/** Query String param name: Folder ID */
	public static final String EDITOR_QS_INT_QS_FOLDER_ID = "folderID";

	/** Query String param name: Create In Parent ID */
	public static final String EDITOR_QS_BOL_QS_FORCESAVEINFOLDER = "bForceSaveInFolder";

	/** Query String param default value: unknown class ID */
	public static final int EDITOR_CLASSID_UNKNOWN = -1;

	/** Query String param default value: unknown object ID */
	public static final int EDITOR_OBJECTID_UNKNOWN = -1;

	/** Query String param default value: unknown folder ID */
	public static final int EDITOR_FOLDERID_UNKNOWN = -1;

	/** Server object: Object being created or edited */
	protected IPTObject m_ptObject;

	/** Server object: Plumtree Session for the current user */
	protected IPTSession m_Session;
	
	/**
	 * Truncate object name after this number of chars.
	 */
	public static final int OBJECT_NAME_TRUNCATE = 50;
	
	/**
	 * Folder Name.
	 */
	protected String m_sFolderName;

	/**
	 * Some Objects don't support ACLs. We don't display the Security page for those Objects.
	 */
	protected boolean m_bDisableSecurity;

	/**
	 * This var is set if this editor is opened by a parent editor (although not
	 * being a sub editor), and the requires the edited object to be stored in
	 * a certain folder. E.g is when creating a subcommunity within a community,
	 * always want the subcommonity to be created in the parent community's
	 * folder. The parent editor passes in the ForceSaveIn parameter to set
	 * this.
	 */
	protected boolean m_bForceSavedInSuppliedFolder = false;

	/**
	 * Flag to know if the current user has system administration activity rights.
	 * We need it to know when to display the migration checkbox.
	 */
	protected boolean m_bIsSysAdmin;

	/** Object being created or edited class ID */
	protected int m_nClassID = 0;

	/**
	 * Folder ID where this object should be saved (has value only in create
	 * mode)
	 */
	protected int m_nFolderID = 0;

	/**
	 * Folder type.
	 */
	protected int m_nFolderType = 0;
	
	/**
	 * For some objects, you can't set/change the description from the Editor.
	 */
	protected boolean m_bDisableDescription;

	/** Object being created or edited ID (has value only in edit mode) */
	protected int m_nObjectID = 0;
	
	/** Object array of all NewEditObjectActions PEIs */
	private Object[] m_oObjectActions = null;

	/**
	 * Object migration status */
	private int m_nMigrationStatus = 0;

	public ObjEditorModel()
	{
		super();
		m_nClassID = EDITOR_CLASSID_UNKNOWN;
		m_nFolderID = PT_INTRINSICS.PT_ADMIN_FOLDER_UNCLASSIFIEDOBJECTS;
		m_nFolderType = PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_REGULARFOLDER;
		m_nObjectID = EDITOR_OBJECTID_UNKNOWN;
		m_nMigrationStatus = PT_MIGRATION_STATUS_TYPES.PT_MIGRATION_STATUS_NONE;		
		m_bDisableDescription = false;		
	}

	/**
	 * That method is called by the EditorStartControl.  It creates a new
	 * IPTObject in create mode and  retrieves the existing IPTObject in edit
	 * mode.
	 *
	 * @param _htFormData
	 *
	 * @return boolean Is true if the current user has the rights to access this Editor.
	 *
	 * @throws XPException
	 */
	public boolean CheckEditorAccessAndStart(XPHashtable _htFormData)
	{
		// Set class ID and object ID with values from QS.
		SetMemberVariablesFromQS(_htFormData);

		if (!CheckEditorAccess(_htFormData))
		{
			return false;
		}

		if (PT_CLASSIDS.PT_USER_ID == m_nClassID || ( PT_CLASSIDS.PT_PAGE_ID == m_nClassID && 0 > m_nFolderID ))
		{
			m_bDisableSecurity = true;
		}

		InitIsSysAdminFlag();
		
		if (EditorStartControl.EDITOR_START_FLAG_CREATE == m_nEditorType)
		{
			//PEI : NewEditObjectActions
			//BrianH - This set of code will pull in all of the available NewEditObjectActions PEIs
			//and process through each of the OnCreateObject functions to determine if the object
			//that is trying to be created will be allowed.
			Object oObjectAction;
			INewEditObjectActions iAction;
			
			if (m_oObjectActions != null)
			{
				String sErrorMsgs = "";
				
				for (int x = 0; x < m_oObjectActions.length; x++)
				{
					oObjectAction = m_oObjectActions[x];
	
					
						log.Debug("CheckEditorAccessAndStart() - About to call OnCreateObject on an: " + oObjectAction);
					
	
					if(oObjectAction != null)
					{
						String sError;
	
						iAction = (INewEditObjectActions) oObjectAction;
						sError = iAction.OnCreateObject(m_asOwner, m_Session, m_nFolderID, m_nClassID);
						
						if ((null != sError) && (sError.length() > 0))
						{
							sErrorMsgs += sError + " ";
						}
					 }
				}
				
				if ((null != sErrorMsgs) && (sErrorMsgs.length() > 0))
				{
					UIError myError = new UIError(UIErrorLevel.SERIOUS,
						UIErrorScope.CURRENT_USER, sErrorMsgs, sErrorMsgs);
					UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
						m_asOwner.GetApplication());
					return false;
				}
			}
			//PEI : OnCreateObject : END
		}
		else
		{
			//PEI : NewEditObjectActions
			//BrianH - This set of code will pull in all of the available NewEditObjectActions PEIs
			//and process through each of the OnEditObject functions to determine if the object
			//that is trying to be edited will be allowed.
			Object oObjectAction;
			INewEditObjectActions iAction;
		
			if (m_oObjectActions != null)
			{
				String sErrorMsgs = "";
				
				for (int x = 0; x < m_oObjectActions.length; x++)
				{
					oObjectAction = m_oObjectActions[x];

					
						log.Debug("CheckEditorAccessAndStart() - About to call OnEditObject on an: " + oObjectAction);
					

					if(oObjectAction != null)
					{
						String sError;

						iAction = (INewEditObjectActions) oObjectAction;
						sError = iAction.OnEditObject(m_asOwner, m_Session, m_nClassID, m_nObjectID);
						
						if ((null != sError) && (sError.length() > 0))
						{
							sErrorMsgs += sError + " ";
						}
				  	}
				}
				
				if ((null != sErrorMsgs) && (sErrorMsgs.length() > 0))
				{
					UIError myError = new UIError(UIErrorLevel.SERIOUS,
						UIErrorScope.CURRENT_USER, sErrorMsgs, sErrorMsgs);
					UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
						m_asOwner.GetApplication());
					return false;
				}
			}
			//PEI : OnEditObject : END
		}
		
		// Initialize m_ptObject
		InitPTObject(_htFormData);
		CheckFolderType();

		// Init common pages
		StartLocNamesList();

		if (!m_bDisableSecurity)
		{
			StartSecurityList();
			
			if ((EditorStartControl.EDITOR_START_FLAG_CREATE == m_nEditorType) &&
				(PT_CLASSIDS.PT_CATALOGCARD_ID != m_nClassID) &&
				(PT_CLASSIDS.PT_ADMIN_FOLDER_ID != m_nClassID) &&
				(PT_CLASSIDS.PT_CATALOGFOLDER_ID != m_nClassID) &&
				(PT_CLASSIDS.PT_SUBPORTAL_ID != m_nClassID) &&
				(PT_CLASSIDS.PT_SITEMAPFOLDER_ID != m_nClassID))
			{	
				((EditorAS) m_asOwner).GetLeftMenu().HideLink(true, ObjectSecurityDP.STR_MVC_CLASS_NAME);
			}
		}

		StartObjectProp();
		StartKeywords();

		// Only do this initialization in edit mode.
		if (EditorStartControl.EDITOR_START_FLAG_EDIT == GetEditorType() && !( PT_CLASSIDS.PT_PAGE_ID == m_nClassID && 0 > m_nFolderID ))
		{
			StartMigrationHistory();
		}
		
		// Perform Editor specific tasks
		DoTaskOnStartEditor(_htFormData);

		return true;
	}

	/**
	  * Returns true if the current Object doesn't support ACLs.
	  *
	  * @return boolean
	  */
	public boolean DisableSecurity()
	{
		return m_bDisableSecurity;
	}

	/**
	 * That method is called by the EditorFinishControl.  It takes care of
	 * storing the IPTObject in the DB  when the user finishes the Editor.
	 *
	 * @throws XPException
	 */
	public void Finish()
	{
		DoTaskBeforeStore();

		((LocNamesModel) m_asOwner.GetModel(LocNamesModel.STR_MVC_CLASS_NAME)).Finish();

		if ((!m_bDisableSecurity) 
			&& ((PT_CLASSIDS.PT_COMMUNITY_ID != m_nClassID) || (EditorStartControl.EDITOR_START_FLAG_EDIT == m_nEditorType)))
		{
			((ObjectSecurityModel) m_asOwner.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB)).Finish();
		}

		((PriorityDropDownModel) m_asOwner.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN)).Finish();

		((ObjPropModel) m_asOwner.GetModel(ObjPropModel.STR_MVC_CLASS_NAME)).Finish();

		StoreObject();

		SaveMigrationStatus();
		
		DoTaskAfterStore();

		if ((!m_bDisableSecurity)
			&& (PT_CLASSIDS.PT_COMMUNITY_ID == m_nClassID) && (EditorStartControl.EDITOR_START_FLAG_CREATE == m_nEditorType))
		{
			((ObjectSecurityModel) m_asOwner.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB)).Finish();
			StoreObject();
		}

		// JF- The Object has been stored. We are now in Edit mode.
		if (EditorStartControl.EDITOR_START_FLAG_CREATE == m_nEditorType)
		{
			m_nEditorType = EditorStartControl.EDITOR_START_FLAG_EDIT;
		}
	}

	/**
	 * Returns the Folder ID in which the Object will be saved.
	 *
	 * @return boolean
	 */
	public boolean ForceSaveInFolder()
	{
		return m_bForceSavedInSuppliedFolder;
	}

	/**
	 * Returns the Class ID corresponding to the object being created or
	 * edited.
	 *
	 * @return int
	 */
	public int GetClassID()
	{
		return m_nClassID;
	}

	/**
	 * Returns the Folder ID in which the Object will be saved.
	 *
	 * @return int
	 */
	public int GetFolderID()
	{
		return m_nFolderID;
	}
	
	/**
	 * Returns true if the object description cannot be set/changed from the Editor.
	 *
	 * @return boolean
	 */
	public boolean IsDescriptionDisabled()
	{
		return m_bDisableDescription;
	}
	
	/**
	 * Sets the value of the m_bDisableDescription member variable.
	 *
	 * @return _bDisableDescription
	 */
	public void SetDisableDescription(boolean _bDisableDescription)
	{
		m_bDisableDescription = _bDisableDescription;
	}

	/**
	 * Returns the folder name.
	 *
	 * @return String
	 */
	public String GetFolderName()
	{
		return m_sFolderName;
	}

	/**
	 * Returns the value of the m_bIsSysAdmin member variable.
	 *
	 * @return boolean
	 */
	public boolean GetIsSysAdmin()
	{
		return m_bIsSysAdmin;
	}

	/**
	 * Returns the current Object's migration status.
	 *
	 * @return int
	 */
	public int GetMigrationStatus()
	{
		return m_nMigrationStatus;
	}

	/**
	 * Returns the the IPTObject being created or edited.
	 *
	 * @return IPTObject
	 */
	public IPTObject GetObject()
	{
		return m_ptObject;
	}

	/**
	 * Returns the description of the IPTObject being created or edited.
	 *
	 * @return String
	 */
	public String GetObjectDescription()
	{
		String sDescription = "";

		try
		{
			sDescription = m_ptObject.GetLocalizedDescription();
		}
		catch (Exception e)
		{
			log.Error(e, "ObjEditorModel::GetObjectDescription Unable to get the Object Description.");	
		}

		return sDescription;
	}

	/**
	 * Returns the ID of the IPTObject being created or edited.
	 *
	 * @return int
	 */
	public int GetObjectID()
	{
		return m_nObjectID;
	}

	/**
	 * Returns the name of the IPTObject being created or edited.
	 *
	 * @return String
	 */
	public String GetObjectName()
	{
		String sName = "";

		try
		{
			sName = m_ptObject.GetLocalizedName();
		}
		catch (Exception e)
		{
			
				log.Error(e, "ObjEditorModel::GetObjectName Unable to get the Object Name.");
			
		}

		return sName;
	}

	/**
	 * Returns the IPTSession for the current user.
	 *
	 * @return IPTSession
	 */
	public IPTSession GetPTSession()
	{
		return m_Session;
	}

	/**
	 * Returns the path to the folder where the object has been saved.
	 *
	 * @return String
	 */
	public String GetParentFolderPath()
	{
		String sPath = "";

		try
		{
			IPTAdminFolder ptFolder = m_Session.GetAdminCatalog()
				.OpenAdminFolder(m_ptObject.GetAdminFolderID(), false);
			sPath = ptFolder.GetPath();
		}
		catch (XPException e)
		{
			
				log.Error("GetParentFolderPath - Unable to retrieve the Admin Folder path.");
			
		}

		return sPath;
	}

	/**
	 * Inits the Model with the IPTSession.
	 *
	 * @param parent
	 */
	public void Init(AActivitySpace parent)
	{
		super.Init(parent);

		// JF- Get the Session object from the associated AS.
		m_Session = (IPTSession) parent.GetUserSession();
		
		//PEI : NewEditObjectActions
		//BrianH: Initialization of all NewEditObjectActions PEIs
		IApplication application = m_asOwner.GetApplication();
		m_oObjectActions = (Object[]) application.GetCachingManager().GetEntry("NewEditObjectActions");
	}

	/**
	  * Returns true if the migration has already been approved for the current object.
	  *
	  * @return boolean
	  */
	public boolean IsMigrationApproved()
	{
		if (PT_MIGRATION_STATUS_TYPES.PT_MIGRATION_STATUS_APPROVED == m_nMigrationStatus)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Calls the right save function according to the page name passed in
	 * argument.
	 *
	 * @param _sPageName
	 * @param _htFormData
	 *
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		int nPageStatus = RepostControl.PAGE_STATUS_VALID;

		if (_sPageName.equals(LocNamesDP.STR_MVC_CLASS_NAME))
		{
			nPageStatus = ((LocNamesModel) m_asOwner.GetModel(LocNamesModel.STR_MVC_CLASS_NAME)).SaveSection(_htFormData);
			nPageStatus = ((ObjPropModel) m_asOwner.GetModel(ObjPropModel.STR_MVC_CLASS_NAME)).SaveSection(_htFormData);
		}
		else if (!m_bDisableSecurity &&
				_sPageName.equals(ObjectSecurityDP.STR_MVC_CLASS_NAME))
		{
			nPageStatus     = ((ObjectSecurityModel) m_asOwner.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB)).SaveSection(_htFormData);
			nPageStatus     = ((PriorityDropDownModel) m_asOwner.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN)).SaveSection(_htFormData);
		}
		else if (_sPageName.equals(MigrationHistoryPluginDP.STR_MVC_CLASS_NAME))
		{
			nPageStatus		= ((MigrationHistoryPluginModel) m_asOwner.GetModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME)).SaveSection(_htFormData);
		}		
		return nPageStatus;
	}

	/**
	 * Helper method for the StartEditor method. Fill out the member variables
	 * with data from QS.
	 *
	 * @param _htFormData
	 */
	public void SetMemberVariablesFromQS(XPHashtable _htFormData)
	{
		// Class ID
		String[] data = (String[]) _htFormData.GetElement(EDITOR_QS_INT_CLASS_ID);

		if ((null != data) && (0 != data.length))
		{
			m_nClassID = XPConvert.ToInteger(data[0]);

			
				log.Debug("ObjEditorModel::SetMemberVariablesFromQS class ID: " +
					data[0]);
			
		}
		else
		{
			// JF- This is ok for sub editors when it's possible 
			// to open multiple times the same Editor in crete mode.
			
				log.Info("ObjEditorModel::SetMemberVariablesFromQS Class ID was not in Query String.");
			
		}

		if (EditorStartControl.EDITOR_START_FLAG_EDIT == GetEditorType())
		{
			// Object ID
			data = (String[]) _htFormData.GetElement(EDITOR_QS_INT_QS_OBJECT_ID);

			if ((null != data) && (0 != data.length) && (data[0] != null))
			{
				m_nObjectID = XPConvert.ToInteger(data[0]);

				
					log.Debug("ObjEditorModel::SetMemberVariablesFromQS Object ID: " +
						data[0]);
				
			}
			else
			{
				
					// JF- We need the Object ID in Edit mode.
					log.Error("ObjEditorModel::SetMemberVariablesFromQS Object ID was not in Query String.");
					throw new XPException("Unable to start the Editor.");
				
			}
		}

		// Folder ID
		data = (String[]) _htFormData.GetElement(EDITOR_QS_INT_QS_FOLDER_ID);

		if ((null != data) && (0 != data.length))
		{
			// If the root folder was selected, create the object in the unclassified folder. 
			// (At the end of the Editor, in the save pop up, the user will have to select a valid folder).
			
			// JH: added exception for folders - OK to create in the root folder
			if ( (PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER != XPConvert.ToInteger(data[0])) || (PT_CLASSIDS.PT_CATALOGFOLDER_ID == m_nClassID) || (PT_CLASSIDS.PT_ADMIN_FOLDER_ID == m_nClassID) )
			{
				m_nFolderID = XPConvert.ToInteger(data[0]);

				
					log.Debug("ObjEditorModel::SetMemberVariablesFromQS Folder ID: " +
						data[0]);
				
			}
			else
			{
				
					log.Warn("ObjEditorModel::SetMemberVariablesFromQS Objects can not be created in the root folder," +
						" the unclassified folder will be used instead.");
				
			}
		}
		else
		{
			
				// JF- If there is no folder ID, the object will be created in the unclassified folder.
				log.Info("ObjEditorModel::SetMemberVariablesFromQS Folder ID was not in Query String.");
			
		}

		// Create in Folder ID, optional parameter
		data = (String[]) _htFormData.GetElement(EDITOR_QS_BOL_QS_FORCESAVEINFOLDER);

		if ((null != data) && (0 != data.length))
		{
			m_bForceSavedInSuppliedFolder = XPConvert.ToBoolean(data[0]);
		}
	}

	/**
	* Sets the current Object's migration status.
	*
	* @param _nMigrationStatus
	*/
	public void SetMigrationStatus(int _nMigrationStatus)
	{
		m_nMigrationStatus = _nMigrationStatus;
	}

	/**
	 * Helper method to start the localized names  and description list.
	 */
	public void StartLocNamesList()
	{
		((LocNamesModel) m_asOwner.GetModel(LocNamesModel.STR_MVC_CLASS_NAME)).InitializeNameDescEditor(m_ptObject, 12);
	}

	/**
	 * Helper method to start the migration history page.
	 */
	public void StartMigrationHistory()
	{
		((MigrationHistoryPluginModel) m_asOwner.GetModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME)).InitializeHistory(m_nClassID,
			m_nObjectID);
	}

	/**
	 * Helper method to start the object property page.
	 */
	public void StartObjectProp()
	{
		// Attempt to initialize object properties.  If object doesn't support EOD, then flag will be set to false.
		((ObjPropModel) m_asOwner.GetModel(ObjPropModel.STR_MVC_CLASS_NAME)).InitializeObjectProperties(m_Session,
				m_ptObject,
				m_asOwner.GetString(156, "ptmsgs_portalinfrastructure"), 13);
	}
	
	/**
	 * Helper method to start the keywords section.
	 */
	public void StartKeywords()
	{
		((KeywordsModel) m_asOwner.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_KEYWORDS)).InitializeKeywords(m_ptObject, 71);
	}

	/**
	 * Helper method to start the security page.
	 */
	public void StartSecurityList()
	{
		((ObjectSecurityModel) m_asOwner.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB)).InitializeSecurityPage(m_ptObject,
			m_Session);
		((PriorityDropDownModel) m_asOwner.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN)).InitializePriorityDropDown(m_ptObject);

		if (PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER == m_nFolderType)
		{
			int nFolderID			   = m_nFolderID;
			int nLastCommunityFolderID = m_nFolderID;
			IPTAdminFolder ptFolder;

			while (PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER == nFolderID)
			{
				nLastCommunityFolderID     = nFolderID;
				ptFolder				   = m_Session.GetAdminCatalog()
					.OpenAdminFolder(nFolderID, false);
				nFolderID				   = ptFolder.GetParentFolderID();
			}

			((ObjectSecurityModel) m_asOwner.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB)).SetCommunityFolderID(nLastCommunityFolderID);
		}
	}

	/**
	 * Helper method to check the current user rights to this Object Editor.
	 *
	 * @param _htFormData Form data
	 * @return boolean
	 */
	protected boolean CheckEditorAccess(XPHashtable _htFormData)
	{
		// JF- In create mode we check the activity rights.
		if (EditorStartControl.EDITOR_START_FLAG_CREATE == m_nEditorType)
		{
			int nActivityID;

			// JIC- If we are creating an object within a community folder, then allow creation without
			// requiring activity right.
			
			// MJD - If we are creating a page, and the folder is negative, it is a my page and we return true
			if ( PT_CLASSIDS.PT_PAGE_ID == m_nClassID && 0 > m_nFolderID )
			{
				return true;
			}
			
			// JF- Make sure it's not a card or a catalog folder (if it is, the parent folder is not an admin folder) 
			if ((PT_CLASSIDS.PT_CATALOGCARD_ID != m_nClassID) && (PT_CLASSIDS.PT_CATALOGFOLDER_ID != m_nClassID))
			{
				try{
					IPTAdminFolder ptParentFolder = m_Session.GetAdminCatalog().OpenAdminFolder(m_nFolderID, false);
					
					int nParentFolderType = ptParentFolder.GetFolderType();
		
					if (PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER == nParentFolderType)
					{
						return true;
					}
				}
				catch (Exception e)
				{
					if( e instanceof PTException )
					{
						PTException ptex = (PTException) e;
						
						int iCode = ptex.GetErrorCode();
						
						if((iCode == PT_RESULTCODES.PT_E_INSUFFICIENTROLEACCESS) 
							|| (iCode == PT_RESULTCODES.PT_E_ACCESSDENIED))
						{
							UIError myError = new UIError(UIErrorLevel.SERIOUS,
							UIErrorScope.CURRENT_USER, null, 
								m_asOwner.GetString(444, "ptmsgs_portalinfrastructure"), null);
							UIErrorHandler.RaiseError(myError,
								m_asOwner.GetSubSession(), m_asOwner.GetApplication());
						}
											
						
					}
	
					throw XPException.GetInstance(e);
				}
			}

			switch (m_nClassID)
			{
			case PT_CLASSIDS.PT_ADMIN_FOLDER_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_ADMINFOLDERS;

				break;

			case PT_CLASSIDS.PT_AUTHSOURCE_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_AUTHSOURCES;

				break;

			case PT_CLASSIDS.PT_COMMUNITY_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_COMMUNITIES;

				break;

			case PT_CLASSIDS.PT_COMMUNITYTEMPLATE_ID:
			case PT_CLASSIDS.PT_PAGETEMPLATE_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_COMMUNITY_INFRASTRUCTURE;

				break;

			case PT_CLASSIDS.PT_CRAWLER_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_CRAWLERS;

				break;

			case PT_CLASSIDS.PT_DATASOURCE_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_DATASOURCES;

				break;

			case PT_CLASSIDS.PT_DOCUMENTTYPE_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_DOCTYPES;

				break;

			case PT_CLASSIDS.PT_FILTER_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_FILTERS;

				break;

			case PT_CLASSIDS.PT_CATALOGCARD_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_USE_ADVANCEDDOCSUBMISSION;

				break;

			case PT_CLASSIDS.PT_CATALOGFOLDER_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_FOLDERS;

				break;

			case PT_CLASSIDS.PT_USERGROUP_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_GROUPS;

				break;

			case PT_CLASSIDS.PT_INVITATION_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_INVITATIONS;

				break;

			case PT_CLASSIDS.PT_JOB_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_JOBS;

				break;

			case PT_CLASSIDS.PT_FEDERATEDPORTAL_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_NETWORKSEARCHES;

				break;

			case PT_CLASSIDS.PT_GADGET_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_PORTLETS;

				break;

			case PT_CLASSIDS.PT_WEBSERVICE_ID:
			case PT_CLASSIDS.PT_GADGETCONTENTSERVER_ID:
			case PT_CLASSIDS.PT_GADGETBUNDLE_ID:
			case PT_CLASSIDS.PT_GADGETTEMPLATE_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_WEBSERVICE_INFRASTRUCTURE;

				break;

			case PT_CLASSIDS.PT_PROFILESOURCE_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_PROFILESOURCES;

				break;

			case PT_CLASSIDS.PT_PROPERTY_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_PROPERTIES;

				break;

			case PT_CLASSIDS.PT_SAVEDSEARCH_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_SAVEDSEARCHES;

				break;

			case PT_CLASSIDS.PT_SUBPORTAL_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_SUBPORTALS;

				break;

			case PT_CLASSIDS.PT_USER_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_USERS;

				break;

			case PT_CLASSIDS.PT_SITEMAPFOLDER_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_SUBPORTALS;

				break;

			case PT_CLASSIDS.PT_TAXONOMIST_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_TAXONOMISTS;

				break;

			case PT_CLASSIDS.PT_EXTOPERATION_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_EXTOPERATIONS;

				break;
				
			case PT_CLASSIDS.PT_EXPRESSION_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_ACCESS_SUBPORTAL_RULES_MANAGER;

				break;
				
			case PT_CLASSIDS.PT_PROFILESECTION_ID:
				nActivityID = PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_ACCESS_UTILITIES;

				break;
				
			default:
				return false;
			}

			if (m_Session.GetSessionInfo().CheckForActivityRight(nActivityID))
			{
				return true;
			}
			else
			{
				UIError myError = new UIError(UIErrorLevel.SERIOUS,
					UIErrorScope.CURRENT_USER, null, 
					m_asOwner.GetString(438, "ptmsgs_portalinfrastructure"), null);
				UIErrorHandler.RaiseError(myError,
					m_asOwner.GetSubSession(), m_asOwner.GetApplication());
				return false;	
			}
		}

		// JF- In edit mode, we need to check the object ACL.
		else if (EditorStartControl.EDITOR_START_FLAG_EDIT == m_nEditorType)
		{
			boolean bObjectIsAccessible = false;
			
			// JF- First we check the edit access
			switch (m_nClassID)
			{
			case PT_CLASSIDS.PT_CATALOGFOLDER_ID:
			case PT_CLASSIDS.PT_CATALOGCARD_ID:
				bObjectIsAccessible = m_Session.GetCatalog().IsCatalogObjectAccessible(m_nClassID,
						m_nObjectID, PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT);
				break;
				
			case PT_CLASSIDS.PT_ADMIN_FOLDER_ID:
				bObjectIsAccessible = m_Session.GetAdminCatalog()
					.IsAdminFolderAccessible(m_nObjectID,
						PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT);
				break;
				
			case PT_CLASSIDS.PT_PAGE_ID:
				if ( 0 > m_nFolderID )
				{
					bObjectIsAccessible = true;
					break;
				}

			default:
				bObjectIsAccessible = m_Session.GetObjectManagers(m_nClassID)
					.IsObjectAccessible(m_nObjectID,
						m_Session.GetSessionInfo().GetCurrentUserID(),
						PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT);
				
				break;
				}
				
			/* Bug fix 9046977 - NEW SETTING NEEDED TO TOGGLE READ/NO READ ACCESS TO EDITORS FOR USERS WITH READ
			 * Adding check to see if the READ access is enabled on the portal for users without Edit access to objects they are trying
			 * to look at.
			 */
			boolean isReadAccessAllowed = GetAccessToEditorPriviledges();
			
			// Rob: This section was commented out to fix bugs #63667 and 63632
			// Everyone user was able to open editors with only read access. No users should be able to open an editor with just
			// read access.
			// Any evetual special cases needs to be specifically added below with a description of why it is needed 
			// Rob 09/15/2009: Bug 8450987 - Boeing wants to give users with read access ability to open editor in read-only mode
			// re-enabling this code again with an added check that guest users cannot open editors in read only mode
			if (bObjectIsAccessible)
			{
				return true;
			}
			// if READ access is enabled for all objects regardless of Edit permissions, we'll check for proper permissions and that the user
			// is not guest
			else if (isReadAccessAllowed) 
			{
				if ( !((IPTSession)m_asOwner.GetUserSession()).GetSessionInfo().CheckCurrentUserIsGuest() )
				{
					// JF- If it didn't succeed, we check read-only access.
					switch (this.m_nClassID)
					{
					case PT_CLASSIDS.PT_CATALOGFOLDER_ID:
					case PT_CLASSIDS.PT_CATALOGCARD_ID:
						bObjectIsAccessible = m_Session.GetCatalog().IsCatalogObjectAccessible(this.m_nClassID, m_nObjectID, PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ);
						break;

					case PT_CLASSIDS.PT_ADMIN_FOLDER_ID:
						bObjectIsAccessible = m_Session.GetAdminCatalog().IsAdminFolderAccessible(m_nObjectID, PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ);
						break;

					default:
						bObjectIsAccessible = m_Session.GetObjectManagers(m_nClassID).IsObjectAccessible(m_nObjectID, m_Session.GetSessionInfo().GetCurrentUserID(), PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ);
					break;
					}
					if (bObjectIsAccessible)
					{
						// JF- The user has access but only in read-only mode.
						SetEditorToReadOnly();
						return true;
					}
				}
			}
			// display error if the user does not have neither edit or read access to editor
			UIError myError = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER, null, m_asOwner.GetString(439, "ptmsgs_portalinfrastructure"), null);
			UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
		}
		return false;
		
	}

	/**
	 * Helper method to check retrieve the folder type.
	 *
	 */
	protected void CheckFolderType()
	{
		// JF- We need to check the folder type for admin objects only.
		if ((PT_CLASSIDS.PT_CATALOGCARD_ID == m_nClassID) ||
			(PT_CLASSIDS.PT_CATALOGFOLDER_ID == m_nClassID))
		{
			return;
		}
		// JF- In Edit mode we get the admin folder ID from the Object directly.
		// JF- The method GetAdminFolderID is not implemented for Admin Folders, 
		// Directory Folders and Cards and we know that those Objects cannot be 
		// created inside a Community Folder.
		if ((EditorStartControl.EDITOR_START_FLAG_EDIT == m_nEditorType) &&
				(PT_CLASSIDS.PT_ADMIN_FOLDER_ID != m_nClassID))
		{
			m_nFolderID = m_ptObject.GetAdminFolderID();
		}

		if ( PT_CLASSIDS.PT_PAGE_ID != m_nClassID || 0 <= m_nFolderID )
		{
			IPTAdminFolder ptFolder = m_Session.GetAdminCatalog().OpenAdminFolder(m_nFolderID, false);
			m_nFolderType     = ptFolder.GetFolderType();
			m_sFolderName     = ptFolder.GetLocalizedName();
		}
		else
		{
			m_bForceSavedInSuppliedFolder = true;
		}

		if (PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER == m_nFolderType)
		{
			m_bForceSavedInSuppliedFolder = true;
		}
		
		if ((EditorStartControl.EDITOR_START_FLAG_MISC != m_nEditorType) && ! m_bForceSavedInSuppliedFolder)
		{
			// JF- If this is a sub Editor, it should be the same as the parent.
			if (m_asOwner.IsSubSpace())
			{
				try 
				{
					EditorAS asParentEditor = (EditorAS) m_asOwner.GetParentSpace();
					EditorModel asParentModel = (EditorModel) asParentEditor.GetModel(asParentEditor.m_strModelName);
				
					if (EditorStartControl.EDITOR_START_FLAG_MISC != asParentModel.GetEditorType())
					{
						if ( ((ObjEditorModel) asParentModel).ForceSaveInFolder())
						{
							m_bForceSavedInSuppliedFolder = true;
							m_nFolderID = ((ObjEditorModel) asParentModel).GetFolderID();
							m_sFolderName = ((ObjEditorModel) asParentModel).GetFolderName();
						}
					}
				}
				catch (Exception e)
				{
						log.Warn(e, "ObjEditorModel::CheckFolderType Unable to retrieve the parent Activity Space and parent Model.");
				}
			}
		}

	}

	/**
	 * Helper method to init the isSysAdmin flag.
	 */
	protected void InitIsSysAdminFlag()
	{
		m_bIsSysAdmin = PlumtreeHelpers.CheckCurrentUserIsAdmin(m_Session);
	}

	/**
	 * Helper method to initialize the PTObject being created or edited.
	 *
	 * @param _htFormData
	 */
	protected void InitPTObject(XPHashtable _htFormData)
	{
		// JF- We are creating a new object
		if ((EDITOR_OBJECTID_UNKNOWN == m_nObjectID) &&
				(EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType()))
		{
			try
			{
				switch (this.m_nClassID)
				{
				case PT_CLASSIDS.PT_CATALOGCARD_ID:
					m_ptObject = (IPTObject) m_Session.GetCatalog().CreateCard();

					break;

				case PT_CLASSIDS.PT_CATALOGFOLDER_ID:
					m_ptObject = (IPTObject) m_Session.GetCatalog()
						.CreateFolder(m_nFolderID);

					break;

				case PT_CLASSIDS.PT_ADMIN_FOLDER_ID:
					m_ptObject = (IPTObject) m_Session.GetAdminCatalog()
						.CreateAdminFolder(m_nFolderID, PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_REGULARFOLDER);

					break;

				case PT_CLASSIDS.PT_SUBPORTAL_ID:
					m_ptObject = (IPTObject) m_Session.GetObjectManagers(m_nClassID)
						.Create(m_nFolderID);

					break;

				default:
					m_ptObject = (IPTObject) m_Session.GetObjectManagers(m_nClassID)
						.Create(m_nFolderID);
				}
				
				m_ptObject.SetName(PlumtreeHelpers.GetNewObjectName(m_nClassID ,m_asOwner));
			}
			catch (Exception e)
			{
				
					log.Error(e, "ObjEditorModel::InitPTObject Unable to create new Object with classID " +
						m_nClassID + ", in folderID: " + m_nFolderID +
						".");
				

				String sMessage = m_asOwner.GetString(153, "ptmsgs_portalinfrastructure" );
				// JF- The message in ptspy shouldn't be extracted
				String sExceptionMessage = "The Portal was unable to create the Object.";
				
				if( e instanceof PTException )
				{
					PTException ptex = (PTException) e;
					
					int iCode = ptex.GetErrorCode();
					
					if ( iCode == PT_RESULTCODES.PT_E_INSUFFICIENTROLEACCESS )
					{
						sMessage = m_asOwner.GetString(438, "ptmsgs_portalinfrastructure");
						sExceptionMessage = "The current user does not have sufficient rights to create this object.";
					}
					else if (iCode == PT_RESULTCODES.PT_E_ACCESSDENIED)
					{
						sMessage = m_asOwner.GetString(447, "ptmsgs_portalinfrastructure");
						sMessage += " " + m_asOwner.GetString(448, "ptmsgs_portalinfrastructure");
						sMessage += " " + m_asOwner.GetString(449, "ptmsgs_portalinfrastructure");
						sExceptionMessage = "The current user does not have sufficient rights to create this object in this folder.";
					}
					
					UIError myError = new UIError(UIErrorLevel.SERIOUS,
						UIErrorScope.CURRENT_USER, null, 
						sMessage, null);
					UIErrorHandler.RaiseError(myError,
						m_asOwner.GetSubSession(), m_asOwner.GetApplication());
				}

				throw new XPException(sExceptionMessage, e);				
			}
		}

		// JF- We are editing an existing object	 
		else if (EditorStartControl.EDITOR_START_FLAG_EDIT == GetEditorType())
		{
			try
			{
				switch (this.m_nClassID)
				{
				case PT_CLASSIDS.PT_CATALOGCARD_ID:
					m_ptObject = (IPTObject) m_Session.GetCatalog().OpenCard(m_nObjectID, false);

					break;

				case PT_CLASSIDS.PT_CATALOGFOLDER_ID:
					m_ptObject = (IPTObject) m_Session.GetCatalog().OpenFolder(m_nObjectID, false);

					break;

				case PT_CLASSIDS.PT_ADMIN_FOLDER_ID:
					m_ptObject = (IPTObject) m_Session.GetAdminCatalog()
						.OpenAdminFolder(m_nObjectID, false);

					break;

				default:
					m_ptObject = (IPTObject) m_Session.GetObjectManagers(m_nClassID)
						.Open(m_nObjectID, false);
				}
				
				IPTServerContext ptSC = (IPTServerContext) m_ptObject.GetInterfaces(
					"IPTServerContext");
					
				if (PT_LOCKSTATES.PT_LOCKED == ptSC.GetLockState())
				{
					UIError myError = new UIError(UIErrorLevel.WARNING,
				
					UIErrorScope.CURRENT_USER, null,
						m_asOwner.GetString(440, "ptmsgs_portalinfrastructure"), "");
			
					myError.SetActivitySpace(m_asOwner.GetName());
			
					UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
						m_asOwner.GetApplication());
			
					m_bIsEditorReadOnly = true;	
				}
			
			}
			catch (Exception e)
			{
				
					log.Error(e, "ObjEditorModel::InitPTObject Unable to edit the Object with ID " +
						m_nObjectID + ".");
				

				String sMessage = m_asOwner.GetString(154, "ptmsgs_portalinfrastructure" );

				
				if( e instanceof PTException )
				{
					PTException ptex = (PTException) e;
					
					int iCode = ptex.GetErrorCode();
					
					if((iCode == PT_RESULTCODES.PT_E_INSUFFICIENTROLEACCESS) 
						|| (iCode == PT_RESULTCODES.PT_E_ACCESSDENIED))
					{
						sMessage = m_asOwner.GetString(439, "ptmsgs_portalinfrastructure");
					}
										
					UIError myError = new UIError(UIErrorLevel.SERIOUS,
						UIErrorScope.CURRENT_USER, null, 
						sMessage, null);
					UIErrorHandler.RaiseError(myError,
						m_asOwner.GetSubSession(), m_asOwner.GetApplication());
				}

				throw new XPException("The current user does not have sufficient rights to edit this Object.", e);
				
			}
		}
	}

	/**
	 * Helper method to Save the migration status on the object.
	 *
	 */
	protected void SaveMigrationStatus()
	{
		((IMigrationHistoryModel) m_asOwner.GetModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME)).StoreMigrationStatus(m_nClassID, m_nObjectID);
	}

	/**
	 * Helper method to Store the Object.
	 *
	 * @throws XPException
	 */
	protected void StoreObject()
	{
		//PEI : NewEditObjectActions
		//BrianH - This set of code will pull in all of the available NewEditObjectActions PEIs
		//and process through each of the OnBeforeStoreObject functions to determine if the object
		//and all of its properties should be stored in its current format. The function will allow for
		//modifications to be made to the object before actually being stored.
		Object oObjectAction;
		INewEditObjectActions iAction;
	
		if (m_oObjectActions != null)
		{
			for (int x = 0; x < m_oObjectActions.length; x++)
			{
				oObjectAction = m_oObjectActions[x];
	
				
					log.Debug("StoreObject() - About to call OnBeforeStoreObject on an: " + oObjectAction);
				
	
				if(oObjectAction != null)
				{
					IPTObject returnPTObject;
	
					iAction = (INewEditObjectActions) oObjectAction;
					returnPTObject = iAction.OnBeforeStoreObject(m_asOwner, m_Session, m_ptObject);
					if (returnPTObject != null)
					{
						m_ptObject = returnPTObject;
					}
				 }
			}
		}
		//PEI : OnBeforeStoreObject : END
		
		// JF- Store the Object in the DB.
		IPTServerContext ptSC = (IPTServerContext) m_ptObject.GetInterfaces(
			"IPTServerContext");
					
		try
		{
			// JF- Shouldn't lock the Object if this is a new one
			if (EDITOR_OBJECTID_UNKNOWN != m_nObjectID)
			{
				ptSC.LockObject();
			}

			ptSC.Store();
			
			m_nObjectID = ptSC.GetObjectID();
			
			//PEI : OnAfterStoreObject : BEGIN
			if (m_oObjectActions != null)
			{
				for (int x = 0; x < m_oObjectActions.length; x++)
				{
					oObjectAction = m_oObjectActions[x];
	
					
						log.Debug("StoreObject() - About to call OnAfterStoreObject on an: " + oObjectAction);
					
	
					if(oObjectAction != null)
					{
						iAction = (INewEditObjectActions) oObjectAction;
						iAction.OnAfterStoreObject(m_asOwner, m_Session, m_ptObject);
					 }
				}
			}
			//PEI : OnAfterStoreObject : END
		}
		catch (Exception e)
		{
			log.Error(e, "ObjEditorModel::StoreObject Unable to store the Object.");
			
			String sMessage = m_asOwner.GetString(158, "ptmsgs_portalinfrastructure" );
			// JF- The exception message should not be extracted.
			String sExceptionMessage = "The Portal was unable to store the Object.";
				
			if( e instanceof PTException )
			{
				PTException ptex = (PTException) e;
				
				int iCode = ptex.GetErrorCode();
					
				if((iCode == PT_RESULTCODES.PT_E_INSUFFICIENTROLEACCESS) 
					|| (iCode == PT_RESULTCODES.PT_E_ACCESSDENIED)
					|| (iCode == PT_RESULTCODES.PT_E_CANNOTEDITREADONLY))
				{
					sMessage = m_asOwner.GetString(441, "ptmsgs_portalinfrastructure");
					sExceptionMessage = "The current user does not have sufficient rights to store this Object";
				}
				else if (iCode == PT_RESULTCODES.PT_E_OBJECTLOCKED)
				{
					sMessage = m_asOwner.GetString(442, "ptmsgs_portalinfrastructure");
					sExceptionMessage = "This Object is already locked. It cannot be stored.";
				}
				else if (iCode == PT_RESULTCODES.PT_E_OBJECTNOTLOCKED)
				{
					sMessage = m_asOwner.GetString(443, "ptmsgs_portalinfrastructure");
					sExceptionMessage = "Unable to lock this Object. It cannot be stored.";
				} else if (iCode == PT_RESULTCODES.PT_E_CIRCULAR_GROUP_STRUCTURE)
				{
					sMessage = m_asOwner.GetString(539, ResourceConstants.PORTALINFRASTRUCTURE);
					sExceptionMessage = "Unable to store this group because it creates a circular group dependency.";
				} else if (iCode == PT_RESULTCODES.PT_E_DYNAMICRULE_INVALID_OPERATOR)
				{
					sMessage = m_asOwner.GetString(540, ResourceConstants.PORTALINFRASTRUCTURE);
					sExceptionMessage = "Unable to store this group because the dynamic membership rules use an invalid operator.";
				} else if (iCode == PT_RESULTCODES.PT_E_DYNAMICRULE_INVALID_PROP_TYPE)
				{
					sMessage = m_asOwner.GetString(541, ResourceConstants.PORTALINFRASTRUCTURE);
					sExceptionMessage = "Unable to store this group because the dynamic membership rules use an invalid property type.";
				} else if ( iCode == PT_RESULTCODES.PT_E_DYNAMICRULE_INVALID_COMPARISON_VALUE)
				{
					sMessage = m_asOwner.GetString(543, ResourceConstants.PORTALINFRASTRUCTURE);
					sExceptionMessage = "Unable to store this group because an invalid comparison value has been used for one or more dynamic membership rules.";

				}
										
				UIError myError = new UIError(UIErrorLevel.SERIOUS,
					UIErrorScope.CURRENT_USER, null, 
					sMessage, null);
				UIErrorHandler.RaiseError(myError,
					m_asOwner.GetSubSession(), m_asOwner.GetApplication());
			}

			throw new XPException(sExceptionMessage, e);
		}
		finally
		{
			ptSC.UnlockObject();
		}
	}

	/**
	 * Helper method to set the read-only flag to true and display
	 * a warning on every page to remind the user his changes will not be saved.
	 *
	 */
	public void SetEditorToReadOnly()
	{
		UIError myError = new UIError(UIErrorLevel.WARNING,		
			UIErrorScope.CURRENT_USER, null,
			m_asOwner.GetString(157, "ptmsgs_portalinfrastructure"), "");

		myError.SetActivitySpace(m_asOwner.GetName());

		UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
			m_asOwner.GetApplication());

		m_bIsEditorReadOnly = true;
	}
	
	/**
	 * This method will be called when you come back to the main Editor from a sub Editor.
	 *
	 */
	public void DoTaskOnComeBackFromSubEditor()
	{
		((ObjPropModel) m_asOwner.GetModel(ObjPropModel.STR_MVC_CLASS_NAME)).DoTaskOnComeBackFromSubEditor();
	}
}

