package com.plumtree.portaluiinfrastructure.editor;


// Plumtree Server Imports
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity.*;
import com.plumtree.portaluiinfrastructure.editor.saveobject.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.expandablelist.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This class has the responsability to call the PerformRepostAction method on
 * common pages such as the Security page or the Localize Names and
 * Descriptions page.
 *
 * @author Julie Fournier
 */
public class ObjEditorRepostControl extends EditorRepostControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.editor.ObjEditorRepostControl");

	/** Class name for the Activity Space. */
	public static final String STR_MVC_CLASS_NAME = "ObjEditorRepost";

	/** Redirects the main window when you are done with the pop-up. */
	public final static int POSTTOSELF_ACTION_REDIRECT_AFTER_SAVEPOPUP = 7;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new ObjEditorRepostControl();
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * For the Object Editor we don't want to store the object yet.  We need to
	 * pop-up the save Object window so the user can select the folder, set
	 * the security and and set the name and description. To do this we add a
	 * JavaScript to the current DP to open the pop-up window.
	 */
	public void PerformFinishAction()
	{
		// JF- Go to the save pop-up in create mode, unless the save pop up is disabled
		if (!DisableSavePopUp() &&
			(PAGE_STATUS_VALID == m_nPageStatus) &&
			(EditorStartControl.EDITOR_START_FLAG_CREATE == ((ObjEditorModel) m_model).GetEditorType()))
		{
			super.CheckMandatoryPages();
			
			if (PAGE_STATUS_VALID == m_nPageStatus)
			{
				OpenSavePopUp();
			}
		}
		else
		{
			super.PerformFinishAction();
		}
	}
	
	/**
	 * Override this method to disable the save pop up.
	 *
	 * @return false
	 */
	protected boolean DisableSavePopUp()
	{
		// Bug 41946 fix:
		// Put this default class id check here (as some of the Editors for the
		// following class ids reside in prov* library, e.g., provUIWWW)
		int nClassID = ((ObjEditorModel) m_model).GetClassID();
		
		if ((PT_CLASSIDS.PT_CATALOGCARD_ID == nClassID) ||
			(PT_CLASSIDS.PT_CATALOGFOLDER_ID == nClassID))
		{
			return true;
		} 
		else
		{			
			return false;
		}
	}

	/**
	 * Helper nmethod to build the URL to the save pop-up and  add the
	 * Javascript to open it to the curent page.
	 *
	 * @throws XPException
	 */
	protected void OpenSavePopUp()
	{
		HTMLScript myScript   = new HTMLScript("text/javaScript");
		ASURL saveObjectPopUp = new ASURL(m_asOwner);
		XPStringBuilder sb;

		// JF- Build the URL to the open the save pop-up	
		if (m_asOwner.GetSubSpaceExists(ObjEditorAS.SAVEOBJECT_SUBSPACE_KEY))
		{
			saveObjectPopUp.SetLinkGetCachedSpace(m_asOwner.GetSubSpace(
					ObjEditorAS.SAVEOBJECT_SUBSPACE_KEY));
		}
		else
		{
			saveObjectPopUp.SetLinkCreateNewSpace(SaveObjectAS.STR_MVC_CLASS_NAME,
				m_asOwner);
			saveObjectPopUp.SetSubSpaceKey(ObjEditorAS.SAVEOBJECT_SUBSPACE_KEY);
			saveObjectPopUp.SetControl(SaveObjectStartControl.STR_MVC_CLASS_NAME);
		}

		// JF- Add a javascript to the page to open a pop-up going to that URL
		try
		{
			sb = new XPStringBuilder(200);
			
			sb.Append("var winSaveObject = window.open('").Append(saveObjectPopUp.GetURLAsString())
			.Append("', 'SaveObject', 'scrollbars=yes,location=no,menubar=no,height=575,width=600');\n");
			myScript.AddInnerHTMLString(sb.ToString());
			
			EditorAS myEditorAS = (EditorAS) m_asOwner;
			IDisplayPage myTempDP = null;
			EditorDP myEditorDP = null;
			
			// JF- Get the current page
			myTempDP = myEditorAS.GetCurrentPage();
				
			// JF- Make sure this is a valid Editor page 
			// (and not the Security page because in create mode that page is in the save pop-up)
			if ((myTempDP instanceof EditorDP) 
				&& !ObjectSecurityDP.STR_MVC_CLASS_NAME.equals(myTempDP.GetName()))
			{
				myEditorDP = (EditorDP) myTempDP;
			}
			else
			{
				// JF- If the current page is not valid, try with the previous page
				myEditorDP = (EditorDP) myEditorAS.GetPreviousPage();
			}
			
			// JF- Add the script that opens the save pop-up to the current page
			myEditorDP.AddScriptToHead(myScript);
		}
		catch (HTMLException he)
		{
			log.Error(he, "ObjEditorRepostControl::PerformFinishAction An error occured" +
				" when trying to generate the JavaScript for the save object pop-up window.");

			throw new XPException(m_asOwner.GetString(159,
					"ptmsgs_portalinfrastructure"), he);
		}
	}

	/**
	 * Switch between the action modes and call  the associated method for each
	 * action.
	 *
	 * @param _nAction
	 */
	protected void PerformAction(int _nAction)
	{
		super.PerformAction(_nAction);
		
		// JF- If the Editor is read-only we need to raise the warning saying that the changes will not be saved on each repost.
		// (Otherwise it gets deleted after the 1st repost)
		if ((POSTTOSELF_ACTION_FINISH != _nAction) 
			&& (POSTTOSELF_ACTION_CANCEL != _nAction) 
			&& ((ObjEditorModel) m_model).GetIsEditorReadOnly())
		{
			((ObjEditorModel) m_model).SetEditorToReadOnly();
		}

		if (POSTTOSELF_ACTION_REDIRECT_AFTER_SAVEPOPUP == _nAction)
		{
			if (m_asOwner.IsSubSpace())
			{
				SetRedirectToCloseSubEditor();
			}
			else
			{
				
				// JF- Redirect (default implementation of GetRedirectAfterFinish 
				// redirects to the previous AS).
				Redirect myRedirect = m_model.GetRedirectAfterFinish();
				
				// JF- In mode 1, we need an HTTP redirect because the Editor might not have 
				// the same security as the AS where it's redirecting to.
				if (ConfigVarPack.SECURITY_MODE_SSL_CRITICAL_AS_ONLY == ConfigHelper.GetSecurityMode())
				{
					myRedirect.SetIsHTTPRedirect(true);
				}
				
				boolean bCloseOnDone = false;
				try
				{
					bCloseOnDone = ((EditorModel) m_model).GetCloseOnDone();
				}
				catch (Exception e)
				{
					
						log.Info("Unable to retrieve the CloseOnCancel flag from the EditorModel.");
						
				}
						
				if ("Gateway".equalsIgnoreCase(myRedirect.GetSpaceName()) || bCloseOnDone)
				{
					// JF- If the parent space was the Gateway then just close the window
					myRedirect = new Redirect();
					myRedirect.SetLinkGetCachedSpace(m_asOwner);
					myRedirect.SetDisplayPage(ClosePopUpDP.STR_MVC_CLASS_NAME);		
				}
				
				SetRedirect(myRedirect);
			}
		}

		((LocNamesRepostControl) m_asOwner.GetControl(LocNamesRepostControl.STR_MVC_CLASS_NAME)).PerformRepostAction(_nAction,
			m_htFormData);

		if (!((ObjEditorModel) m_model).DisableSecurity())
		{
			((ExpListControl) m_asOwner.GetControl(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB)).PerformRepostAction(_nAction,
				m_htFormData);
		}

		((ExpListControl) m_asOwner.GetControl(ExpListControl.STR_MVC_CLASS_NAME)).PerformRepostAction(_nAction,
			m_htFormData);
	}

	/**
	 * Get the current page name and call the save method on the model.
	 */
	protected void SaveCurrentPage()
	{
		// JF- Do not save the page if this is the redirect after save pop-up 
		// (in this case we are done with the Editor, the object has been stored already). 
		if (POSTTOSELF_ACTION_REDIRECT_AFTER_SAVEPOPUP != m_nAction)
		{
			super.SaveCurrentPage();
		}
	}
}

