package com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.expandablelist.properties.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.marshalers.*;
import com.plumtree.server.treeiterator.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.expandablelist.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.tree.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * This is the implementation of the Expandable List for object properties.
 *
 * @author josephc
 */
public class ObjPropModel extends ExpListPropModel implements IObjPropModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties.ObjPropModel");

	/** Class name for the ObjPropModel */
	public static final String STR_MVC_CLASS_NAME = "ObjProp";

	/** Number total of columns */
	public static final int EXPLIST_COL_MAX = 9;

	/** Expandable list column for query result containing EOD data */
	public static final int EXPLIST_COL_DATATABLE = 5;

	/** Expandable list column for index into query result containing EOD data */
	public static final int EXPLIST_COL_DATATABLE_INDEX = 6;

	/** Expandable list column for reference value name */
	public static final int EXPLIST_COL_REFVALUE_NAME = 7;

	/** Sort index for value */
	public static final int EXPLIST_COL_SORT_INDEX = 8;

	/** Status of deleted row */
	public static final int ROW_STATUS_DELETED = 10;

	/** Subspace key for pick tree */
	public static final String SUBSPACE_KEY_PICKTREE = "PickTreeSubSpace";

	/** Subspace key for reference tree */
	public static final String SUBSPACE_KEY_REFTREE = "RefTreeSubSpace";

	/** Object ID for name property */
	public static final int PROP_OBJECTID_NAME = 1;
	
	/** Object ID for description property */
	public static final int PROP_OBJECTID_DESCRIPTION = 2;

	/** IPTObject */
	protected IPTObject m_ptObject;

	/** IPTObjectProperties */
	protected IPTObjectProperties m_ptObjProp;

	/** IPTSession */
	protected IPTSession m_ptSession;

	/** This is used on add to populate the input text of the newly added row. */
	protected Object m_oDefaultValue;

	/** Icon to display on each row, just before the input text. */
	protected PTImageType m_imgRowIcon = null;

	/** Query filter for non read-only, multi-select properties */
	protected String m_strTreeQueryFilter;

	/** Does object support EOD? */
	protected boolean m_bSupportsEOD;

	/** Flag determines if display page containing object properties is mandatory. */
	protected boolean m_bDisplayPageIsMandatory;

	/** A table of the sort order for property values */
	protected XPHashtable m_propValueSortingOrder;

	/**
	 * Creates a new ObjPropModel object.
	 */
	public ObjPropModel()
	{
		super();
		m_imgRowIcon = PTImageType.ACTION_ADD;
	}

	/**
	 * Adds a new row to the list of object property values
	 *
	 * @param _htFormData Hash table of form data
	 *
	 * @see com.plumtree.uiinfrastructure.expandablelist.ExpListModel#AddRowsOnRepost(XPHashtable)
	 */
	public void AddRowsOnRepost(XPHashtable _htFormData)
	{
		// Get the hidden input with the row to modify
		String sDataKey = GetUniqueString(ExpListView.HTMLHIDDEN_ROWINDEX);

		if (_htFormData.ContainsKey(sDataKey))
		{
			String[] aDataValue = (String[]) _htFormData.GetElement(sDataKey);

			if ((null != aDataValue) && (aDataValue.length > 0))
			{
				int nRowIndex = XPConvert.ToInteger(aDataValue[0]);

				Object nPropertyID = GetRowPropertyInfo(nRowIndex, PT_PROPIDS.PT_PROPID_OBJECTID);

				int nSortIndex = UpdateMaxSortIndex(nPropertyID);

				// Create new row to add to list
				Object[] oRow = new Object[EXPLIST_COL_MAX];
				oRow[EXPLIST_COL_PROPID]			  = nPropertyID;
				oRow[EXPLIST_COL_VALUE]				  = m_oDefaultValue;
				oRow[EXPLIST_COL_NAME]				  = GetRowName(nRowIndex);
				oRow[EXPLIST_COL_TYPE]				  = new Integer(GetRowType(nRowIndex));
				oRow[EXPLIST_COL_STATUS]			  = new Integer(ROW_STATUS_ADDED);
				oRow[EXPLIST_COL_DATATABLE]			  = m_saData.GetItem(nRowIndex, EXPLIST_COL_DATATABLE);
				oRow[EXPLIST_COL_DATATABLE_INDEX]     = m_saData.GetItem(nRowIndex, EXPLIST_COL_DATATABLE_INDEX);
				oRow[EXPLIST_COL_SORT_INDEX]		  = new Integer(nSortIndex);

				m_saData.AddRow(oRow);

				// Update unique datetime keys whenever expandable list is modified.
				UpdateDateTimeUniqueKeys();

				// Update mandatory page status whenever expandable list is modified.
				UpdateMandatoryPageStatus();
			}
		}
	}

	/**
	 * Override method to save pick list data
	 *
	 * @param _nRowIndex Row index
	 * @param _htFormData Form data
	 *
	 * @return Return key for pick list if row has picklist. Otherwise return regular key.
	 */
	protected String GetDataKey(int _nRowIndex, XPHashtable _htFormData)
	{
		int nPickType = XPConvert.ToInteger(GetRowPropertyInfo(_nRowIndex, PT_PROPIDS.PT_PROPID_PROP_PICKTYPE));

		if ((PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_MANAGED_PICKLIST == nPickType) || (PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_UNMANAGED_PICKLIST == nPickType))
		{
			return GetUniqueString(ObjPropEditView.HTMLINPUT_SELECT_PICKLIST_PREFIX) + _nRowIndex;
		}
		else
		{
			return super.GetDataKey(_nRowIndex, _htFormData);
		}
	}

	/**
	 * Checks to see if the object supports EOD
	 *
	 * @return Returns true if object supports EOD.  Otherwise returns false.
	 *
	 * @see com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties.IObjPropModelRO#CheckObjectSupportsEOD()
	 */
	public boolean CheckObjectSupportsEOD()
	{
		return m_bSupportsEOD;
	}

	/**
	 * Checks to see if the property is an HREF
	 *
	 * @param _nRowIndex Current row index
	 *
	 * @return Returns true if HREF. Otherwise returns false.
	 *
	 * @see com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties.IObjPropModelRO#CheckRowIsHREF(int)
	 */
	public boolean CheckRowIsHREF(int _nRowIndex)
	{
		return XPConvert.ToBoolean(GetRowPropertyInfo(_nRowIndex, PT_PROPIDS.PT_PROPID_PROP_HREF));
	}

	/**
	 * Checks to see if the property is mandatory
	 *
	 * @param _nRowIndex Current row index
	 *
	 * @return Returns true if mandatory. Otherwise returns false.
	 *
	 * @see com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties.IObjPropModelRO#CheckRowIsMandatory(int)
	 */
	public boolean CheckRowIsMandatory(int _nRowIndex)
	{
		return XPConvert.ToBoolean(GetRowPropertyInfo(_nRowIndex, PT_PROPIDS.PT_PROPID_PROP_MANDATORY));
	}

	/**
	 * Checks to see if the property is multivalued
	 *
	 * @param _nRowIndex Current row index
	 *
	 * @return Returns true if multivalued. Otherwise returns false.
	 *
	 * @see com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties.IObjPropModelRO#CheckRowIsMultiValued(int)
	 */
	public boolean CheckRowIsMultiValued(int _nRowIndex)
	{
		return XPConvert.ToBoolean(GetRowPropertyInfo(_nRowIndex, PT_PROPIDS.PT_PROPID_PROP_MULTIVALUED));
	}

	/**
	 * Checks to see if the property is read-only
	 *
	 * @param _nRowIndex Current row index
	 *
	 * @return Returns true if read-only. Otherwise returns false.
	 *
	 * @see com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties.IObjPropModelRO#CheckRowIsReadOnly(int)
	 */
	public boolean CheckRowIsReadOnly(int _nRowIndex)
	{
		return XPConvert.ToBoolean(GetRowPropertyInfo(_nRowIndex, PT_PROPIDS.PT_PROPID_PROP_READONLYVALUE));
	}

	/**
	 * Checks to see if the property is visible
	 *
	 * @param _nRowIndex Current row index
	 *
	 * @return Returns true if visible. Otherwise returns false.
	 *
	 * @see com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties.IObjPropModelRO#CheckRowIsVisible(int)
	 */
	public boolean CheckRowIsVisible(int _nRowIndex)
	{
		return XPConvert.ToBoolean(GetRowPropertyInfo(_nRowIndex, PT_PROPIDS.PT_PROPID_PROP_VISIBLE));
	}

	/**
	 * Returns true if the display page is mandatory because it contains
	 * mandatory properties.
	 *
	 * @return True if page contains mandatory properties that user must enter values for.
	 */
	public boolean CheckDisplayPageIsMandatory()
	{
		return m_bDisplayPageIsMandatory;
	}

	/**
	 * Determines if the page containing object properties must be a mandatory page.
	 * Iterate through expandable list looking for visible, not read-only, mandatory
	 * rows that are also not deleted.
	 *
	 * @return True if list contains mandatory value that the user must enter. False otherwise.
	 */
	public void UpdateMandatoryPageStatus()
	{
		m_bDisplayPageIsMandatory = false;

		for (int nRowIndex = 0; nRowIndex < m_saData.GetCount(); nRowIndex++)
		{
			// Look for a row that's visible, not read-only, mandatory, and not deleted 
			if (CheckRowIsVisible(nRowIndex) && !CheckRowIsReadOnly(nRowIndex) && CheckRowIsMandatory(nRowIndex) && (ObjPropModel.ROW_STATUS_DELETED != GetRowStatus(nRowIndex)))
			{
				// Special case for name and description properties.  We don't want to validate
				// these property values.
				if (ObjPropModel.PROP_OBJECTID_NAME != GetRowPropertyID(nRowIndex) &&
					ObjPropModel.PROP_OBJECTID_DESCRIPTION != GetRowPropertyID(nRowIndex))
				{
					// If the property is mandatory but the value is null, then page must be mandatory.
					Object oValue = GetRowValue(nRowIndex);
					if (null == oValue)
					{
						m_bDisplayPageIsMandatory = true;
						break;
					}
				}
			}
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new ObjPropModel();
	}

	/**
	 * Deletes am existing row from the list of object property values.
	 *
	 * @param _htFormData Hash table of form data
	 *
	 * @see com.plumtree.uiinfrastructure.expandablelist.ExpListModel#DeleteRowsOnRepost(XPHashtable)
	 */
	public void DeleteRowsOnRepost(XPHashtable _htFormData)
	{
		String sDataKey = GetUniqueString(ExpListView.HTMLHIDDEN_ROWINDEX);

		if (_htFormData.ContainsKey(sDataKey))
		{
			String[] aDataValue = (String[]) _htFormData.GetElement(sDataKey);

			if ((null != aDataValue) && (aDataValue.length > 0))
			{
				int nRowIndex = XPConvert.ToInteger(aDataValue[0]);

				if (ROW_STATUS_ADDED == GetRowStatus(nRowIndex))
				{
					m_saData.RemoveRow(nRowIndex);

					// Update unique datetime keys whenever expandable list is modified.
					UpdateDateTimeUniqueKeys();

					// Update mandatory page status whenever expandable list is modified.
					UpdateMandatoryPageStatus();
				}
				else
				{
					SetRowStatus(nRowIndex, ROW_STATUS_DELETED);
				}
			}
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Recursively iterates through a pick tree and validates that all the nodes
	 * are valid.
	 *
	 * @param ptTreeIterator Tree to validate
	 * @param nLevel The current level to validate, which should initially be 0.
	 */
	public static void ValidateTree(IPTTreeIterator ptTreeIterator, int nLevel)
	{
		Object[][] currentLocation = (Object[][]) ptTreeIterator.CurrentLocInfo();
		int parentType = XPConvert.ToInteger(currentLocation[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_TYPE][0]);
		Object[][] childNodes = (Object[][]) ptTreeIterator.Children(0, -1);

		switch (parentType)
		{
		case PT_TREE_ITERATOR_CHILD_TYPES.PT_TREE_ITERATOR_CHILD_TYPE_CONTAINER:
		{
			// Containers may or may not have children.  If there are no sub-containers, then just return.
			if (null == childNodes)
			{
				break;
			}

			int numChildren = childNodes[0].length;

			for (int i = 0; i < numChildren; i++)
			{
				String childLocation = (String) childNodes[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_IDENTIFIER][i];
				ptTreeIterator.MoveTo(childLocation);

				ValidateTree(ptTreeIterator, nLevel++);
			}

			break;
		}

		default:
			break;
		}
	}

	/**
	 * If a property has a pick tree or pick list, check that the pick values
	 * can be successfully retrieved.
	 * 
	 * @param _nRowIndex Row index
	 * 
	 * @return True if pick values can be successfully retrieved.
	 */
	public boolean CheckPickValuesAreValid(int _nRowIndex)
	{
		try
		{
			int nPropertyID = GetRowPropertyID(_nRowIndex);
			
			// Open the property
			IPTObjectManager ptOM = m_ptSession.GetProperties();
			boolean bLockInitially = false;

			IPTProperty ptProperty = (IPTProperty) ptOM.Open(nPropertyID, bLockInitially);
			
			if (null != ptProperty)
			{
				// Get pick list
				IPTPickListEditor ptPickList = (IPTPickListEditor) ptProperty.GetPickList();
				IPTTreeIterator ptTreeIterator = (IPTTreeIterator) ptPickList.GetInterfaces("IPTTreeIterator");
				ObjPropModel.ValidateTree(ptTreeIterator, 0);
				
				return true;
			}
			else
			{
				return false;
			}
		}
		catch(PTException e)
		{
			return false;
		}
	}

	/**
	 * Return pick list values for the property
	 *
	 * @param _nPropertyID Property ID
	 *
	 * @return Pick list values
	 *
	 * @see com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties.IObjPropModelRO#GetPickListValues(int)
	 */
	public Object[][] GetPickListValues(int _nPropertyID)
	{
		try
		{
			// Open the property
			IPTObjectManager ptOM = m_ptSession.GetProperties();
			boolean bLockInitially = false;
	
			IPTProperty ptProperty = (IPTProperty) ptOM.Open(_nPropertyID, bLockInitially);
	
			Object[][] aPickListValues = null;
	
			if (null != ptProperty)
			{
				// Get pick list
				IPTPickListEditor ptPickList = (IPTPickListEditor) ptProperty.GetPickList();
	
				if (null != ptPickList)
				{
					// Get tree iterator from pick list
					IPTTreeIterator ptTreeIterator = (IPTTreeIterator) ptPickList.GetInterfaces("IPTTreeIterator");
					aPickListValues = (Object[][]) ptTreeIterator.Children(0, -1);
				}
			}
	
			return aPickListValues;
		}
		catch(PTException e)
		{
			return null;
		}
	}

	/**
	 * Returns the value of the requested PROPID from the EOD query result.
	 *
	 * @param _nRowIndex Row index
	 * @param nPropID PT_PROPID
	 *
	 * @return Value of requested PROPID
	 *
	 * @see com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties.IObjPropModelRO#GetRowPropertyInfo(int,
	 *      int)
	 */
	public Object GetRowPropertyInfo(int _nRowIndex, int nPropID)
	{
		IASQueryResult qrData = (IASQueryResult) m_saData.GetItem(_nRowIndex, EXPLIST_COL_DATATABLE);
		int nTableIndex = XPConvert.ToInteger(m_saData.GetItem(_nRowIndex, EXPLIST_COL_DATATABLE_INDEX));

		return qrData.GetFields(nTableIndex, nPropID);
	}

	/**
	 * Returns the reference value name
	 *
	 * @param _nRowIndex Row index
	 *
	 * @return Reference value name
	 *
	 * @see com.plumtree.portaluiinfrastructure.editor.commonpages.objectproperties.IObjPropModelRO#GetRowRefValueName(int)
	 */
	public String GetRowRefValueName(int _nRowIndex)
	{
		return (String) m_saData.GetItem(_nRowIndex, EXPLIST_COL_REFVALUE_NAME);
	}

	/**
	 * Returns string representation of property value
	 *
	 * @param oValue Property value as Object
	 * @param nPropertyType Property type
	 *
	 * @return Property value as string
	 */
	public String GetStringFromObjectValue(Object oValue, int nPropertyType)
	{
		String strValue;

		if (null == oValue)
		{
			strValue = "";
		}
		else
		{
			switch (nPropertyType)
			{
			case PT_PROPERTY_TYPES.PT_PROPTYPE_DATE:
			{
				// Format date value using XPDateFormat
				XPDateTime dtValue = (XPDateTime) oValue;
				XPDateFormat myDateTimeFormat = XPDateFormat.GetDateTimeInstance(XPDateFormat.MEDIUM, XPDateFormat.SHORT, XPLocale.GetDefault(), XPTimeZone.GetCurrentTimeZone());
				strValue = myDateTimeFormat.Format(dtValue);
			}

			break;

			case PT_PROPERTY_TYPES.PT_PROPTYPE_STRING:
			case PT_PROPERTY_TYPES.PT_PROPTYPE_ENCRYPTED:
				strValue = (String) oValue;

				break;

			case PT_PROPERTY_TYPES.PT_PROPTYPE_DOUBLE:
			case PT_PROPERTY_TYPES.PT_PROPTYPE_LONG:
			case PT_PROPERTY_TYPES.PT_PROPTYPE_REF:default:
				strValue = XPConvert.ToString(oValue);

				break;
			}
		}

		return strValue;
	}

	/**
	 * Returns a String of Class IDs and Object IDs of the Objects  to select
	 * in the tree. E.g.: "refClassID,ObjID1".
	 *
	 * @param _nRowIndex Row index
	 *
	 * @return String
	 */
	public String GetSelectedReference(int _nRowIndex)
	{
		XPStringBuilder sb = new XPStringBuilder(100);

		Object oValue = GetRowValue(_nRowIndex);

		if (null != oValue)
		{
			// Get reference class ID
			int nReferenceClassID = XPConvert.ToInteger(GetRowPropertyInfo(_nRowIndex, PT_PROPIDS.PT_PROPID_PROP_REFCLASS));

			// Get object ID
			int nObjectID = XPConvert.ToInteger(oValue);

			sb.Append(nReferenceClassID).Append(",").Append(nObjectID);
		}

		return sb.ToString();
	}

	/**
	 * Returns true if current user has access to global object property map.
	 * In order to access this editor, the user must be a member of the administrators
	 * group and must have the right to access the Plumtree utilities.
	 *
	 * @return True if user has access. False otherwise.
	 */
	public boolean HasAccessToGlobalObjectPropMapEditor()
	{
		return (m_ptSession.GetSessionInfo().CheckForActivityRight(PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_ACCESS_UTILITIES) && m_ptSession.GetSessionInfo().CheckCurrentUserIsAdmin());
	}

	/**
	 * Initialize Object Properties
	 *
	 * @param _ptSession IPTSession
	 * @param _ptObject IPTObject
	 * @param _sSectionTitle Title
	 * @param _nListKey List key
	 */
	public void InitializeObjectProperties(IPTSession _ptSession, IPTObject _ptObject, String _sSectionTitle, int _nListKey)
	{
		m_ptSession     = _ptSession;
		m_ptObject	    = _ptObject;

		m_ptObjProp = _ptObject.GetObjectProperties();

		if (null == m_ptObjProp)
		{
			m_bSupportsEOD = false;
		}
		else
		{
			m_bSupportsEOD = true;

			Object[][] arrExpListColInfo = new Object[2][EXPLIST_COLUMN_INVALID];

			// The first column will be the property name and will be sortable
			arrExpListColInfo[0][EXPLIST_COLUMN_ID]			    = new Integer(EXPLIST_COL_NAME);
			arrExpListColInfo[0][EXPLIST_COLUMN_TITLE]		    = m_asOwner.GetString(100, "ptmsgs_portalinfrastructure");
			arrExpListColInfo[0][EXPLIST_COLUMN_WIDTH]		    = "20%";
			arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN]	    = "1";
			
			if (m_asOwner.GetAccessStyle() == AccessStyles.STANDARD)
			{
				arrExpListColInfo[0][EXPLIST_COLUMN_ENABLESORT]     = Boolean.TRUE;
			}
			else
			{
				arrExpListColInfo[0][EXPLIST_COLUMN_ENABLESORT]     = Boolean.FALSE;
			}

			// The second column will be the property value and will not be sortable
			arrExpListColInfo[1][EXPLIST_COLUMN_ID]			    = new Integer(EXPLIST_COL_VALUE);
			arrExpListColInfo[1][EXPLIST_COLUMN_TITLE]		    = m_asOwner.GetString(101, "ptmsgs_portalinfrastructure");
			arrExpListColInfo[1][EXPLIST_COLUMN_WIDTH]		    = "80%";
			arrExpListColInfo[1][EXPLIST_COLUMN_COLSPAN]	    = "2";
			arrExpListColInfo[1][EXPLIST_COLUMN_ENABLESORT]     = Boolean.FALSE;

			// Initialize as read-only list because we don't want checkboxes or action bar
			super.InitializeReadOnlyExpandableList(_sSectionTitle, arrExpListColInfo, _nListKey);

			// Load the properties only after expandable list is initialized.
			LoadProperties();

			// Update datetime keys
			UpdateDateTimeUniqueKeys();

			// Update mandatory page status whenever expandable list is modified.
			UpdateMandatoryPageStatus();

			SetEmptyListMessages(m_asOwner.GetString(96, "ptmsgs_portalinfrastructure"), m_asOwner.GetString(95, "ptmsgs_portalinfrastructure"));
		}
	}

	/**
	 * Initalizes the list (PTGrowableSortedArrayWrapper)  with a column
	 * mapping and an array of sort. The growable array will be sorted by the
	 * 2 columns, first the name and second propID, ascending
	 *
	 * @param _saData
	 *
	 * @see com.plumtree.uiinfrastructure.expandablelist.ExpListModel#InitializeSortedArray(PTGrowableSortedArrayWrapper)
	 */
	public void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData)
	{
		// JF- Column mapping
		int[] arrColumns = new int[EXPLIST_COL_MAX];

		arrColumns[EXPLIST_COL_PROPID]			    = EXPLIST_COL_PROPID;
		arrColumns[EXPLIST_COL_VALUE]			    = EXPLIST_COL_VALUE;
		arrColumns[EXPLIST_COL_NAME]			    = EXPLIST_COL_NAME;
		arrColumns[EXPLIST_COL_TYPE]			    = EXPLIST_COL_TYPE;
		arrColumns[EXPLIST_COL_STATUS]			    = EXPLIST_COL_STATUS;
		arrColumns[EXPLIST_COL_DATATABLE]		    = EXPLIST_COL_DATATABLE;
		arrColumns[EXPLIST_COL_DATATABLE_INDEX]     = EXPLIST_COL_DATATABLE_INDEX;
		arrColumns[EXPLIST_COL_REFVALUE_NAME]	    = EXPLIST_COL_REFVALUE_NAME;
		arrColumns[EXPLIST_COL_SORT_INDEX]	   		= EXPLIST_COL_SORT_INDEX;

		// The growable array will be sorted by 3 columns, first the name, second the propID, third the sort index 
		int[][] arrSort = new int[3][2];
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		  = EXPLIST_COL_NAME;
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
		arrSort[1][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		  = EXPLIST_COL_PROPID;
		arrSort[1][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
		arrSort[1][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		  = EXPLIST_COL_SORT_INDEX;
		arrSort[1][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;

		// JF- We don't initalize the list itself (null bellow)
		_saData.Initialize(arrColumns, null, arrSort, true);
	}

	/**
	 * Action for modifying a row. Override the default behavior of this method
	 * so that pick tree or reference tree can modify the row.
	 *
	 * @param _htFormData Form data
	 *
	 * @see com.plumtree.uiinfrastructure.expandablelist.ExpListModel#ModifyRow(XPHashtable)
	 */
	public void ModifyRow(XPHashtable _htFormData)
	{
		// Get the hidden input with the row to modify
		String sDataKey = GetUniqueString(ExpListView.HTMLHIDDEN_ROWINDEX);

		if (_htFormData.ContainsKey(sDataKey))
		{
			String[] aDataValue = (String[]) _htFormData.GetElement(sDataKey);

			if ((null != aDataValue) && (aDataValue.length > 0))
			{
				int nRowIndex = XPConvert.ToInteger(aDataValue[0]);
				int nPropertyType = GetRowType(nRowIndex);

				ATreeAS asTree;

				switch (nPropertyType)
				{
					case PT_PROPERTY_TYPES.PT_PROPTYPE_REF:
					{
						boolean bClearValue = false;

						// Get the hidden input that controls clearing the value
						String sClearValueKey = GetUniqueString(ObjPropEditView.HTMLHIDDEN_CLEARVALUE);

						if (_htFormData.ContainsKey(sClearValueKey))
						{
							String[] aClearDataValue = (String[]) _htFormData.GetElement(sClearValueKey);

							if ((null != aClearDataValue) && (aClearDataValue.length > 0))
							{
								bClearValue = XPConvert.ToBoolean(aClearDataValue[0]);
							}
						}

						if (bClearValue)
						{
							// We are trying to remove the value

							// Reset the reference object's name
							SetRowRefValueName(nRowIndex, null);

							// Remove the value from the list
							UpdateRowInList(nRowIndex, null);
						} else if (m_asOwner.GetSubSpaceExists(GetUniqueString(SUBSPACE_KEY_REFTREE)))
						{
							// Get the new value from the tree

							asTree = (ATreeAS) m_asOwner.GetSubSpace(GetUniqueString(SUBSPACE_KEY_REFTREE));

							ITreeModelRO myTree = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
							XPArrayList mySelectedItems = myTree.GetSelectedItems();

							Object oPickValue = null;

							if (0 < mySelectedItems.GetSize())
							{
								// Get the first selected itme
								String strRefObjectName = ((TreeLeafNode) mySelectedItems.GetElement(0)).GetName();
								int nRefObjectID = ((TreeLeafNode) mySelectedItems.GetElement(0)).m_nObjectID;

								// Convert string value to object
								oPickValue = new Integer(nRefObjectID);

								// Store the reference object's name
								SetRowRefValueName(nRowIndex, strRefObjectName);
							}

							UpdateRowInList(nRowIndex, oPickValue);
						}
						break;
					}
					
					default:
					{
						if (m_asOwner.GetSubSpaceExists(GetUniqueString(SUBSPACE_KEY_PICKTREE)))
						{
							asTree = (ATreeAS) m_asOwner.GetSubSpace(GetUniqueString(SUBSPACE_KEY_PICKTREE));

							ITreeModelRO myTree = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
							XPArrayList mySelectedItems = myTree.GetSelectedItems();

							Object oPickValue = null;

							if (0 < mySelectedItems.GetSize())
							{
								// Get the first selected itme
								String strPickValue = ((TreeLeafNode) mySelectedItems.GetElement(0)).GetName();

								// Convert string value to object
								try
								{
									oPickValue = GetPropValueFromString(strPickValue, nPropertyType);
								}
								catch (Exception e)
								{
									String strErrorMsg = GetRowErrorMessage(nRowIndex, m_asOwner.GetString(99, "ptmsgs_portalinfrastructure"));
									UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER, strErrorMsg, strErrorMsg);
									UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());

									return;
								}
							}

							UpdateRowInList(nRowIndex, oPickValue);
						}
						
						break;
					}
				}
			}
		}
	}

	/**
	 * Stores object properties
	 */
	public void Finish()
	{
		if (m_bSupportsEOD)
		{
			XPArrayList alistAddInfo = new XPArrayList();
			XPArrayList alistAddRefInfo = new XPArrayList();
			XPArrayList alistDeleteInfo = new XPArrayList();
			XPArrayList alistDeleteRefInfo = new XPArrayList();

			for (int nRowIndex = 0; nRowIndex < m_saData.GetCount();
					nRowIndex++)
			{
				int nValueType = GetRowType(nRowIndex);

				if (PT_PROPERTY_TYPES.PT_PROPTYPE_REF == nValueType)
				{
					UpdateRowArrayListForDeletions(nRowIndex, alistDeleteRefInfo);
				}
				else
				{
					UpdateRowArrayListForDeletions(nRowIndex, alistDeleteInfo);
				}
			}

			for (int nRowIndex = 0; nRowIndex < m_saData.GetCount();
					nRowIndex++)
			{
				int nValueType = GetRowType(nRowIndex);

				if (PT_PROPERTY_TYPES.PT_PROPTYPE_REF == nValueType)
				{
					UpdateRowArrayListForAdditions(nRowIndex, alistAddRefInfo, alistDeleteRefInfo);
				}
				else
				{
					UpdateRowArrayListForAdditions(nRowIndex, alistAddInfo, alistDeleteInfo);
				}
			}

			Object[][] aEditPropInfo = CreateArrayForEdit(alistAddInfo, alistDeleteInfo);
			Object[][] aEditPropRefInfo = CreateArrayForEdit(alistAddRefInfo, alistDeleteRefInfo);

			if(0 != aEditPropInfo[0].length)
			{
				m_ptObjProp.SetPropertyData(aEditPropInfo);
			}

			if(0 != aEditPropRefInfo[0].length)
			{
				m_ptObjProp.SetPropertyRefData(aEditPropRefInfo);
			}
		}
	}

	/**
	 * Saves the data when a repost occurs. Editors using this framework need
	 * to call that method from the SavePage method of the main Model.  If the
	 * object doesn't support object properties, then don't save the rows.
	 *
	 * @param _htFormData
	 *
	 * @return int Status for server side validation
	 *
	 * @see com.plumtree.uiinfrastructure.editor.EditorHelperModel#SaveSection(com.plumtree.openfoundation.util.XPHashtable)
	 */
	public int SaveSection(XPHashtable _htFormData)
	{
		int nSaveStatus = RepostControl.PAGE_STATUS_VALID;

		if (m_bSupportsEOD)
		{
			nSaveStatus = super.SaveSection(_htFormData);
		}

		return nSaveStatus;
	}

	/**
	 * Creates an array for editing the object property data for an object.
	 *
	 * @param _alistAddInfo List of values to add
	 * @param _alistDeleteInfo List of values to remove
	 *
	 * @return Returns a 2D array which can be passed into SetPropertyData and
	 *         SetPropertyRefData
	 */
	private Object[][] CreateArrayForEdit(XPArrayList _alistAddInfo, XPArrayList _alistDeleteInfo)
	{
		Object[][] aEditPropInfo = new Object[PT_EDIT_PROPDATA_COLUMNS.PT_EDIT_PROPDATA_INVALID][_alistAddInfo.GetSize() + _alistDeleteInfo.GetSize()];

		int nIndex = 0;

		// Add the deletion values to the array first
		IXPEnumerator enumPropsToDelete = _alistDeleteInfo.GetEnumerator();

		while (enumPropsToDelete.MoveNext())
		{
			Object nPropID = enumPropsToDelete.GetCurrent();
			aEditPropInfo[PT_EDIT_PROPDATA_COLUMNS.PT_EDIT_PROPDATA_PROPERTYID][nIndex] = nPropID;
			nIndex++;
		}

		// Add the edit and new values to the array
		IXPEnumerator enumPropsToAdd = _alistAddInfo.GetEnumerator();

		while (enumPropsToAdd.MoveNext())
		{
			Object[] aPropEditEntry = (Object[]) enumPropsToAdd.GetCurrent();
			aEditPropInfo[PT_EDIT_PROPDATA_COLUMNS.PT_EDIT_PROPDATA_PROPERTYID][nIndex]     = aPropEditEntry[PT_EDIT_PROPDATA_COLUMNS.PT_EDIT_PROPDATA_PROPERTYID];
			aEditPropInfo[PT_EDIT_PROPDATA_COLUMNS.PT_EDIT_PROPDATA_VALUE][nIndex]		    = aPropEditEntry[PT_EDIT_PROPDATA_COLUMNS.PT_EDIT_PROPDATA_VALUE];
			nIndex++;
		}

		return aEditPropInfo;
	}

	/**
	 * Returns the name for an object
	 *
	 * @param nClassID Class ID of object
	 * @param nObjectID Object ID of object
	 *
	 * @return Name of object.
	 */
	protected String GetNameFromObjectID(int nClassID, int nObjectID)
	{
		// Get the name of the reference object
		switch (nClassID)
		{
		case PT_CLASSIDS.PT_ADMIN_FOLDER_ID:
		{
			IPTAdminCatalog ptAC = m_ptSession.GetAdminCatalog();

			if (ptAC.IsAdminFolderAccessible(nObjectID, PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ))
			{
				IPTAdminFolder ptAdminFolder = ptAC.OpenAdminFolder(nObjectID, false);

				return ptAdminFolder.GetLocalizedName();
			}

			break;
		}

		case PT_CLASSIDS.PT_CATALOGFOLDER_ID:
		{
			IPTCatalog ptCatalog = m_ptSession.GetCatalog();

			if (ptCatalog.IsCatalogObjectAccessible(PT_CLASSIDS.PT_CATALOGFOLDER_ID, nObjectID, PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ))
			{
				IPTFolder ptFolder = ptCatalog.OpenFolder(nObjectID, false);

				return ptFolder.GetLocalizedName();
			}

			break;
		}

		default:
		{
			// Get the object manager for the object and query for the name
			IPTAdminCatalog ptAC = m_ptSession.GetAdminCatalog();
			IPTObjectManager ptOM = ptAC.GetObjectManager(nClassID);
			IPTQueryResult ptQR = ptOM.QuerySingleObject(nObjectID);

			if (0 < ptQR.RowCount())
			{
				return ptQR.ItemAsString(0, PT_PROPIDS.PT_PROPID_NAME);
			}

			break;
		}
		}

		return null;
	}

	/**
	 * Return object property non-reference data
	 *
	 * @param _ptObjProp IPTObjectProperties
	 *
	 * @return Data in query result
	 *
	 * @throws XPException
	 */
	protected IASQueryResult GetObjectPropData(IPTObjectProperties _ptObjProp)
	{
		IASQueryResult asqrObjectPropData = null;

		try
		{
			if (null != _ptObjProp)
			{
				IPTQueryResult qrObjectPropData = _ptObjProp.GetPropertyData(PT_PROPIDS.PT_PROPID_ALL);
				asqrObjectPropData = new ASQueryResultWrapper(qrObjectPropData);
			}
		}
		catch (Exception e)
		{
			
				log.Error("ObjPropModel::GetObjectPropData " + m_asOwner.GetString(97, "ptmsgs_portalinfrastructure") + ". The following exception occured: " + e);
			

			throw new XPException(m_asOwner.GetString(97, "ptmsgs_portalinfrastructure"), e);
		}

		return asqrObjectPropData;
	}

	/**
	 * Return object property reference data
	 *
	 * @param _ptObjProp IPTObjectProperties
	 *
	 * @return Data in query result
	 *
	 * @throws XPException
	 */
	protected IASQueryResult GetObjectPropRefData(IPTObjectProperties _ptObjProp)
	{
		IASQueryResult asqrObjectPropRefData = null;

		try
		{
			if (null != _ptObjProp)
			{
				IPTQueryResult qrObjectPropRefData = _ptObjProp.GetPropertyRefData(PT_PROPIDS.PT_PROPID_ALL);
				asqrObjectPropRefData = new ASQueryResultWrapper(qrObjectPropRefData);
			}
		}
		catch (Exception e)
		{
			
				log.Error("ObjPropModel::GetObjectPropRefData " + m_asOwner.GetString(97, "ptmsgs_portalinfrastructure") + ". The following exception occured: " + e);
			

			throw new XPException(m_asOwner.GetString(97, "ptmsgs_portalinfrastructure"), e);
		}

		return asqrObjectPropRefData;
	}


	/**
	 * Looks up the max sort index from table using property ID.
	 * 
	 * @param nPropertyID Property ID
	 * 
	 * @return Updated sort index for the property
	 */
	private int UpdateMaxSortIndex(Object nPropertyID)
	{
		int nSortIndex;
					
		if (!m_propValueSortingOrder.ContainsKey(nPropertyID))
		{
			// Set sort index to 0
			nSortIndex = 0; 
		}
		else
		{
			// Increment sort index by 1
			nSortIndex = XPConvert.ToInteger(m_propValueSortingOrder.GetElement(nPropertyID)) + 1;
		}

		// Update sort index
		m_propValueSortingOrder.PutElement(nPropertyID, new Integer(nSortIndex));
		
		return nSortIndex;
	}
	
	/**
	 * Loads object properties for object
	 */
	protected void LoadProperties()
	{
		int nStatus = ObjPropModel.ROW_STATUS_NO_CHANGE;

		// Add non-reference data
		IASQueryResult qrData = GetObjectPropData(m_ptObjProp);
		// Add reference data
		IASQueryResult qrRefData = GetObjectPropRefData(m_ptObjProp);

		m_propValueSortingOrder = new XPHashtable(qrData.GetCount() + qrRefData.GetCount());

		{
			Object[][] aRowsToAdd = new Object[qrData.GetCount()][EXPLIST_COL_MAX];
	
			for (int i = 0; i < qrData.GetCount(); i++)
			{
				Object nPropertyID = qrData.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID);
				Object nPropertyType = qrData.GetFields(i, PT_PROPIDS.PT_PROPID_PROP_VALUETYPE);
				Object oValue = qrData.GetFields(i, PT_PROPIDS.PT_PROPID_PROP_VALUE);
				String strName = (String) qrData.GetFields(i, PT_PROPIDS.PT_PROPID_NAME);
			
				int nSortIndex = UpdateMaxSortIndex(nPropertyID);
	
				aRowsToAdd[i][EXPLIST_COL_PROPID]			   = nPropertyID;
				aRowsToAdd[i][EXPLIST_COL_VALUE]			   = oValue;
				aRowsToAdd[i][EXPLIST_COL_NAME]				   = strName;
				aRowsToAdd[i][EXPLIST_COL_TYPE]				   = nPropertyType;
				aRowsToAdd[i][EXPLIST_COL_STATUS]			   = new Integer(nStatus);
				aRowsToAdd[i][EXPLIST_COL_DATATABLE]		   = qrData;
				aRowsToAdd[i][EXPLIST_COL_DATATABLE_INDEX]     = new Integer(i);
				aRowsToAdd[i][EXPLIST_COL_SORT_INDEX]  		   = new Integer(nSortIndex);
			}
	
			m_saData.AddRows(aRowsToAdd);
		}

		{
			Object[][] aRefRowsToAdd = new Object[qrRefData.GetCount()][EXPLIST_COL_MAX];
	
			for (int i = 0; i < qrRefData.GetCount(); i++)
			{
				Object nPropertyID = qrRefData.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID);
				Object nPropertyType = qrRefData.GetFields(i, PT_PROPIDS.PT_PROPID_PROP_VALUETYPE);
				String strName = (String) qrRefData.GetFields(i, PT_PROPIDS.PT_PROPID_NAME);
				Object nRefObjectID = qrRefData.GetFields(i, PT_PROPIDS.PT_PROPID_PROP_VALUE);
				int nRefClassID = XPConvert.ToInteger(qrRefData.GetFields(i, PT_PROPIDS.PT_PROPID_PROP_REFCLASS));
	
				int nSortIndex = UpdateMaxSortIndex(nPropertyID);
	
				aRefRowsToAdd[i][EXPLIST_COL_PROPID]			  = nPropertyID;
				aRefRowsToAdd[i][EXPLIST_COL_VALUE]				  = nRefObjectID;
				aRefRowsToAdd[i][EXPLIST_COL_NAME]				  = strName;
				aRefRowsToAdd[i][EXPLIST_COL_TYPE]				  = nPropertyType;
				aRefRowsToAdd[i][EXPLIST_COL_STATUS]			  = new Integer(nStatus);
				aRefRowsToAdd[i][EXPLIST_COL_DATATABLE]			  = qrRefData;
				aRefRowsToAdd[i][EXPLIST_COL_DATATABLE_INDEX]     = new Integer(i);
				aRefRowsToAdd[i][EXPLIST_COL_SORT_INDEX]   		  = new Integer(nSortIndex);
	
				if (null != nRefObjectID)
				{
					aRefRowsToAdd[i][EXPLIST_COL_REFVALUE_NAME] = GetNameFromObjectID(nRefClassID, XPConvert.ToInteger(nRefObjectID));
				}
			}
	
			m_saData.AddRows(aRefRowsToAdd);
		}
	}

	/**
	 * Add newly mapped properties to our expandable list.  First construct an array
	 * list of all of the properties from the new query result of object property
	 * data.  Next iterate through our existing expandable list and remove the properties
	 * found from the array list.  This should leave us with just the new properties in
	 * the array list.
	 *
	 */
	private void UpdateObjectProperties()
	{
		IASQueryResult qrData = GetObjectPropData(m_ptObjProp);
		IASQueryResult qrRefData = GetObjectPropRefData(m_ptObjProp);

		// Add all property ID's from new query results to lists
		XPArrayList alistProperties = new XPArrayList(qrData.GetCount());
		XPArrayList alistRefProperties = new XPArrayList(qrRefData.GetCount());
		XPArrayList alistRemoveProperties = new XPArrayList();

		for (int i = 0; i < qrData.GetCount(); i++)
		{
			Object nPropertyID = qrData.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID);

			if (!alistProperties.Contains(nPropertyID))
			{
				alistProperties.Add(nPropertyID);
			}
		}

		for (int i = 0; i < qrRefData.GetCount(); i++)
		{
			Object nPropertyID = qrRefData.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID);

			if (!alistRefProperties.Contains(nPropertyID))
			{
				alistRefProperties.Add(nPropertyID);
			}
		}

		// Remove existing property ID's from lists
		int nLastRowPropertyID = -1;

		for (int nRowIndex = 0; nRowIndex < m_saData.GetCount(); nRowIndex++)
		{
			Object nPropertyID = m_saData.GetItem(nRowIndex, EXPLIST_COL_PROPID);

			// Skip the current row if we saw the property alraedy in the previous row, because this means the property is multi-valued.
			if (XPConvert.ToInteger(nPropertyID) != nLastRowPropertyID)
			{
				if (alistProperties.Contains(nPropertyID))
				{
					alistProperties.Remove(nPropertyID);
				}
				else if (alistRefProperties.Contains(nPropertyID))
				{
					alistRefProperties.Remove(nPropertyID);
				}
				else
				{
					if (!alistRemoveProperties.Contains(nPropertyID))
					{
						alistRemoveProperties.Add(nPropertyID);
					}
				}
			}

			nLastRowPropertyID = XPConvert.ToInteger(nPropertyID);
		}

		int nStatus = ObjPropModel.ROW_STATUS_NO_CHANGE;

		// If there are any items left in the lists, then they are the new property's.
		if (!alistProperties.IsEmpty())
		{
			// We will have at least alistProperties.GetSize() number of rows to add (possibly more for multi-valued propertis).
			XPArrayList alistRowsToAdd = new XPArrayList(alistProperties.GetSize());

			for (int i = 0; i < qrData.GetCount(); i++)
			{
				Object nPropertyID = qrData.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID);

				if (alistProperties.Contains(nPropertyID))
				{
					Object oValue = qrData.GetFields(i, PT_PROPIDS.PT_PROPID_PROP_VALUE);
					Object nPropertyType = qrData.GetFields(i, PT_PROPIDS.PT_PROPID_PROP_VALUETYPE);
					String strName = (String) qrData.GetFields(i, PT_PROPIDS.PT_PROPID_NAME);

					int nSortIndex = UpdateMaxSortIndex(nPropertyID);

					Object[] arRowToAdd = new Object[EXPLIST_COL_MAX];

					arRowToAdd[EXPLIST_COL_PROPID]			    = nPropertyID;
					arRowToAdd[EXPLIST_COL_VALUE]			    = oValue;
					arRowToAdd[EXPLIST_COL_NAME]			    = strName;
					arRowToAdd[EXPLIST_COL_TYPE]			    = nPropertyType;
					arRowToAdd[EXPLIST_COL_STATUS]			    = new Integer(nStatus);
					arRowToAdd[EXPLIST_COL_DATATABLE]		    = qrData;
					arRowToAdd[EXPLIST_COL_DATATABLE_INDEX]     = new Integer(i);
					arRowToAdd[EXPLIST_COL_SORT_INDEX]  		= new Integer(nSortIndex);

					alistRowsToAdd.Add(arRowToAdd);
				}
			}

			for (int i = 0; i < alistRowsToAdd.GetSize(); i++)
			{
				m_saData.AddRow((Object[]) alistRowsToAdd.GetElement(i));
			}
		}

		// If there are any items left in the lists, then they are the new property's.
		if (!alistRefProperties.IsEmpty())
		{
			// We will have at least alistRefProperties.GetSize() number of rows to add (possibly more for multi-valued propertis).
			XPArrayList alistRowsToAdd = new XPArrayList(alistRefProperties.GetSize());

			for (int i = 0; i < qrRefData.GetCount(); i++)
			{
				Object nPropertyID = qrRefData.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID);

				if (alistRefProperties.Contains(nPropertyID))
				{
					Object oValue = qrRefData.GetFields(i, PT_PROPIDS.PT_PROPID_PROP_VALUE);
					Object nPropertyType = qrRefData.GetFields(i, PT_PROPIDS.PT_PROPID_PROP_VALUETYPE);
					String strName = (String) qrRefData.GetFields(i, PT_PROPIDS.PT_PROPID_NAME);

					int nSortIndex = UpdateMaxSortIndex(nPropertyID);
					
					Object[] arRowToAdd = new Object[EXPLIST_COL_MAX];

					arRowToAdd[EXPLIST_COL_PROPID]			    = nPropertyID;
					arRowToAdd[EXPLIST_COL_VALUE]			    = oValue;
					arRowToAdd[EXPLIST_COL_NAME]			    = strName;
					arRowToAdd[EXPLIST_COL_TYPE]			    = nPropertyType;
					arRowToAdd[EXPLIST_COL_STATUS]			    = new Integer(nStatus);
					arRowToAdd[EXPLIST_COL_DATATABLE]		    = qrRefData;
					arRowToAdd[EXPLIST_COL_DATATABLE_INDEX]     = new Integer(i);
					arRowToAdd[EXPLIST_COL_SORT_INDEX] 			= new Integer(nSortIndex);

					alistRowsToAdd.Add(arRowToAdd);
				}
			}

			for (int i = 0; i < alistRowsToAdd.GetSize(); i++)
			{
				m_saData.AddRow((Object[]) alistRowsToAdd.GetElement(i));
			}
		}

		if (!alistRemoveProperties.IsEmpty())
		{
			XPArrayList alistRowIndexesToRemove = new XPArrayList();

			for (int nRowIndex = 0; nRowIndex < m_saData.GetCount();
					nRowIndex++)
			{
				Object nPropertyID = m_saData.GetItem(nRowIndex, EXPLIST_COL_PROPID);

				if (alistRemoveProperties.Contains(nPropertyID))
				{
					alistRowIndexesToRemove.Add(new Integer(nRowIndex));
				}
			}

			int[] arRowIndexesToRemove = new int[alistRowIndexesToRemove.GetSize()];

			for (int i = 0; i < arRowIndexesToRemove.length; i++)
			{
				arRowIndexesToRemove[i] = XPConvert.ToInteger(alistRowIndexesToRemove.GetElement(i));
			}

			m_saData.RemoveRows(arRowIndexesToRemove);
		}
	}

	/**
	 * Set the row reference value name
	 *
	 * @param _nRowIndex Row index
	 * @param _strName Name of reference value
	 */
	private void SetRowRefValueName(int _nRowIndex, Object _strName)
	{
		m_saData.SetItem(_nRowIndex, EXPLIST_COL_REFVALUE_NAME, _strName);
	}

	/**
	 * Helper method to add rows for addition to array list.
	 *
	 * @param _nRowIndex Row index
	 * @param _alistAddInfo Array list of items to add
	 * @param _alistDeleteInfo Array list of items to delete
	 */
	private void UpdateRowArrayListForAdditions(int _nRowIndex, XPArrayList _alistAddInfo, XPArrayList _alistDeleteInfo)
	{
		int nStatus = GetRowStatus(_nRowIndex);

		Object nPropID = new Integer(GetRowPropertyID(_nRowIndex));
		Object oValue = GetRowValue(_nRowIndex);

		Object[] aPropEditEntry = new Object[PT_EDIT_PROPDATA_COLUMNS.PT_EDIT_PROPDATA_INVALID];
		aPropEditEntry[PT_EDIT_PROPDATA_COLUMNS.PT_EDIT_PROPDATA_PROPERTYID]     = nPropID;
		aPropEditEntry[PT_EDIT_PROPDATA_COLUMNS.PT_EDIT_PROPDATA_VALUE]			 = oValue;

		if (!CheckRowIsMultiValued(_nRowIndex))
		{
			// Single-valued
			switch (nStatus)
			{
			case ROW_STATUS_DELETED:
			case ROW_STATUS_MODIFIED:

				// For single-valued properties, simply adding a value will overwrite the old value
				// so we just need to add the value to the addition list.
				_alistAddInfo.Add(aPropEditEntry);

				break;

			default:

				// Do nothing
				break;
			}
		}
		else
		{
			// Multi-valued
			switch (nStatus)
			{
			case ROW_STATUS_ADDED:
			case ROW_STATUS_MODIFIED:

				if (null != oValue)
				{
					_alistAddInfo.Add(aPropEditEntry);
				}

				break;

			case ROW_STATUS_NO_CHANGE:

				// If a property value was deleted, we need to re-add all the values, including 
				// values that haven't changed.
				if (_alistDeleteInfo.Contains(nPropID))
				{
					_alistAddInfo.Add(aPropEditEntry);
				}

				break;

			default:

				// Do nothing
				break;
			}
		}
	}

	/**
	 * If the property is multi-valued, then if there were any changes to the
	 * row (additions, deletions, or edits) then the row will be deleted.
	 *
	 * @param _nRowIndex Row index
	 * @param _alistDeleteInfo Array list of items to delete
	 */
	private void UpdateRowArrayListForDeletions(int _nRowIndex, XPArrayList _alistDeleteInfo)
	{
		if (CheckRowIsMultiValued(_nRowIndex))
		{
			// Multi-valued
			int nStatus = GetRowStatus(_nRowIndex);

			switch (nStatus)
			{
			case ROW_STATUS_ADDED:
			case ROW_STATUS_DELETED:
			case ROW_STATUS_MODIFIED:
			{
				Object nPropID = new Integer(GetRowPropertyID(_nRowIndex));

				if (!_alistDeleteInfo.Contains(nPropID))
				{
					_alistDeleteInfo.Add(nPropID);
				}
			}

			break;

			default:
				break;
			}
		}
	}

	/**
	 * This method will be called when you come back to the main Editor from a sub Editor.
	 *
	 */
	public void DoTaskOnComeBackFromSubEditor()
	{
		if (m_asOwner.GetSubSpaceExists("ObjPropMapEditor"))
		{
			m_asOwner.DeleteSubSpace("ObjPropMapEditor");

			// Update expandable list when coming back from global object property map.
			UpdateObjectProperties();

			// Update unique datetime keys whenever expandable list is modified.
			UpdateDateTimeUniqueKeys();

			// Update mandatory page status whenever expandable list is modified.
			UpdateMandatoryPageStatus();
		}
	}

	/**
	 * Saves the data on the row only if the row is visible and is not read-only.
	 *
	 * @param _saData List to update the data on row _nRowIndex
	 * @param _nRowIndex Index of the row to save.
	 * @param _htFormData Data posted by the form.
	 *
	 * @return int Status for server side validation.
	 */
	protected int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex, XPHashtable _htFormData)
	{
		if (CheckRowIsVisible(_nRowIndex) && !CheckRowIsReadOnly(_nRowIndex))
		{
			return super.SaveRow(_saData, _nRowIndex, _htFormData);
		}
		else
		{
			return RepostControl.PAGE_STATUS_VALID;
		}
	}
}

