package com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity;

// Plumtree Server
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.form.*;

/**
 * Model class for the Priority drop-down.
 *
 * @author Julie Fournier
 */
public class PriorityDropDownModel extends EditorHelperModel
	implements IPriorityDropDownModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity.PriorityDropDownModel");

	/**
	 * Class name for the activity space */
	public static final String STR_MVC_CLASS_NAME = "PriorityDropDown";

	/**
	 * Object being created or edited */
	private IPTObject m_ptObject;

	/**
	 * Flag to know whether the current Object supports the mandatory column or no.
	 * We display the Rpiority drop-down only if this flag is true.
	 */
	private boolean m_bSupportMandatoryColumn = false;

	/**
	 * Member variable to store the current value of the Mandatory tab priority */
	private int m_nMandatoryPriority;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new PriorityDropDownModel();
	}

	/**
	 * Sets the mandatory tab priority value on the Object.
	 *
	 */
	public void Finish()
	{
		if (m_bSupportMandatoryColumn)
		{
			if (PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID())
			{
				((IPTCommunity) m_ptObject).SetMandatoryTabOrder(m_nMandatoryPriority);
			}
			else if (PT_CLASSIDS.PT_GADGET_ID == m_ptObject.GetClassID())
			{
				((IPTGadget) m_ptObject).SetMandatoryGadgetOrder(m_nMandatoryPriority);
			}
		}
	}

	/**
	 * Returns the current value of the Mandatory tab priority drop-down.
	 *
	 * @return int
	 */
	public int GetMandatoryPriority()
	{
		return m_nMandatoryPriority;
	}

	/**
	 * Get the string to display to the left of the mandatory drop-down list.
	 *
	 * @return int
	 */
	public String GetStringForUI()
	{	
		if (PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID())
		{
			// "Mandatory Tab Priority"
			return m_asOwner.GetString(118, "ptmsgs_portalinfrastructure");
		}
		else if (PT_CLASSIDS.PT_GADGET_ID == m_ptObject.GetClassID())
		{
			// "Mandatory Portlet Priority"
			return m_asOwner.GetString(368, "ptmsgs_portalinfrastructure");
		} else {
			// default to the community case, "Mandatory Tab Priority".
			return m_asOwner.GetString(118, "ptmsgs_portalinfrastructure");
		}	
	}


	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Tells whether the priority drop-down should be displayed
	 * for the current object or no.
	 *
	 * @return boolean
	 */
	public boolean GetSupportMandatoryColumn()
	{
		return m_bSupportMandatoryColumn;
	}

	/**
	 * Initializes this plug-in section.
	 *
	 * @param _ptObject
	 */
	public void InitializePriorityDropDown(IPTObject _ptObject)
	{
		m_ptObject = _ptObject;

		// JF- We already know that all communities should have it.
		// (For Portlets it different, only certain types of Portlets support it.)
		if (PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID())
		{
			m_bSupportMandatoryColumn     = true;
			m_nMandatoryPriority		  = ((IPTCommunity) m_ptObject).GetMandatoryTabOrder();
		}
	}

	/**
	 * Saves the selected value in the priority drop-down.
	 *
	 * @param _htFormData
	 *
	 * @return int Status for server side validation
	 */
	public int SaveSection(XPHashtable _htFormData)
	{
		if (m_bSupportMandatoryColumn)
		{
			String[] data = (String[]) _htFormData.GetElement(PriorityDropDownView.HTMLSELECT_PRIORITY);

			if ((null != data) && !(0 == data.length))
			{
				m_nMandatoryPriority = XPConvert.ToInteger(data[0]);
			}
			else
			{
				
					log.Warn("PriorityDropDownModel::SaveSection Mandatory Tab Priority value was not found in the form params.");
				
			}
		}

		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * Changes the value of the m_bSupportMandatoryTab member variable.
	 *
	 * @param _bSupportMandatoryTab
	 */
	public void SetSupportMandatoryColumn(boolean _bSupportMandatoryColumn)
	{
		m_bSupportMandatoryColumn = _bSupportMandatoryColumn;

		if (m_bSupportMandatoryColumn)
		{
			if (PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID())
			{
				m_nMandatoryPriority = ((IPTCommunity) m_ptObject).GetMandatoryTabOrder();
			}
			else if (PT_CLASSIDS.PT_GADGET_ID == m_ptObject.GetClassID())
			{
				m_nMandatoryPriority = ((IPTGadget) m_ptObject).GetMandatoryGadgetOrder();
			}
		}
	}
}

