package com.plumtree.portaluiinfrastructure.editor.saveobject;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tree.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.tree.*;

/**
 * This is the Model class for the Save object pop-up.
 * We use it to store the state of the save object pop-up,
 * except for the security page and the folder tree.
 *
 * The Security page is part of the Object Editor AS which is the
 * parent space of this one.
 * The Folder Tree is a sub space.
 *
 * @author Julie Fournier
 */
public class SaveObjectModel implements IRepostModel, ISaveObjectModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.editor.saveobject.SaveObjectModel");

	/**
	  * Class Name for the Activity Space.
	  */
	public static final String STR_MVC_CLASS_NAME = "SaveObjectPopUp";

	/**
	  * Class Name for the Activity Space.
	  */
	public static final String RECENT_FOLDERS_COOKIE_NAME = "ptrecentfolders";

	/**
	  * Key to store the folder tree as a sub space.
	  */
	public static final String SAVEOBJECT_FOLDERTREE_SUBSPACEKEY = "SaveObjectPopUpFolderTree";

	/**
	  * Constant to identify the folder tree page in the body frame.
	  */
	public static final int SAVEOBJECT_TAB_FOLDERTREE = 0;

	/**
	  * Constant to identify the security page in the body frame.
	  */
	public static final int SAVEOBJECT_TAB_SECURITY = 1;
	
	/**
	 * Object name max length.
	 */
	private static final int OBJECT_NAME_MAX_LENGTH = 252;

	/**
	  * Parent Activity Space.
	  */
	protected SaveObjectAS m_asOwner;

	/**
	 * Folder Name.
	 */
	protected String m_sFolderName;
	
	/**
	 * For some objects, you can't set/change the description from the Editor.
	 */
	protected boolean m_bDisableDescription;

	/**
	  * Flag to know on which tab we are: folder tree or security.
	  */
	protected int m_nCurrentTab;

	/**
	  * Object being created.
	  */
	private IPTObject m_ptObject;

	/**
	  * User session.
	  */
	private IPTSession m_ptSession;
	
	/**
	 * JF- Array of class id and name of the 4 most rcently used folders.
	 * The IDs are stored in a Cookie on the user's machine.
	 */
	private Object[][] m_arrRecentlyUsedFolders;

	/**
	 * Forces the object to be saved in a particular folder, the save in folder tree is
	 * disabled when this variable is set. Retrieved from the parent model.
	 */
	private boolean m_bForceSaveInFolder = false;

	/**
	  * Some Objects don't support ACLs. We don't display the Security page for those Objects.
	  */
	private boolean m_bHideSecurityTab = false;

	/**
	  * See m_bRepostBodyFromTopFrame.
	  */
	private boolean m_bRepostBodyFromBottomFrame = false;

	/**
	  * This flag is used to know if the Top frame should repost the body frame.
	  * The problem is that each frame is a different form, but when we change tab
	  * for example, we need to repost the top frame and change the selected tab,
	  * and then we need to repost the body frame, save the content and redirect it
	  * to the other tab. That's why when we repost the Top frame we also set this
	  * flag to true, this way a JavaScript will take care of reposting the body tab.
	  */
	private boolean m_bRepostBodyFromTopFrame = false;

	/**
	  * See m_bRepostBodyFromTopFrame.
	  */
	private boolean m_bRepostTreeAndSave = false;

	/**
	  * See m_bRepostBodyFromTopFrame.
	  */
	private boolean m_bRepostTreeRecentFoldersFrame = false;

	/**
	  * Admin Folder ID.
	  */
	private int m_nFolderID;

	/**
	 * Adds the selected admin folder ID to Cookie.
	 *
	 * @param _xpRequest
	 * @param _webdata
	 */
	public void AddAdminFolderIDToCookie(IXPRequest _xpRequest,
		IWebData _webdata)
	{
		XPCookie[] arrCookies;
		XPCookie myCookie;
		boolean bFoundCookie = false;

		if (null == _xpRequest)
		{
			return;
		}

		if (IsAdminFolderInRecentFolders(m_nFolderID))
		{
			return;
		}

		String sCookieValue = "";
		int nMax = 4;
		
		if (PT_INTRINSICS.PT_ADMIN_FOLDER_UNCLASSIFIEDOBJECTS != m_nFolderID)
		{
			sCookieValue = XPConvert.ToString(m_nFolderID) + "-";
			nMax --;
		}

		int k = 0;
		
		while ((k < nMax) && (k < m_arrRecentlyUsedFolders.length))
		{
			if (null != m_arrRecentlyUsedFolders[k][0])
			{
				sCookieValue = sCookieValue + m_arrRecentlyUsedFolders[k][0] +
					"-";
			}
			k ++;
		}

		arrCookies = _xpRequest.GetCookies();

		if (null != arrCookies)
		{
			for (int i = 0; i < arrCookies.length; i++)
			{
				myCookie = arrCookies[i];

				if (RECENT_FOLDERS_COOKIE_NAME.equals(myCookie.GetName()))
				{
					myCookie.SetValue(sCookieValue);
					// JF- The Cookie will expire in 90 days.
					myCookie.SetExpireTime(60*60*24*90);
					myCookie.SetPath("");
					bFoundCookie = true;
					_webdata.AddCookie(myCookie);

					break;
				}
			}
		}

		if ((!bFoundCookie) && (PT_INTRINSICS.PT_ADMIN_FOLDER_UNCLASSIFIEDOBJECTS != m_nFolderID))
		{
			// Create cookie
			myCookie = new XPCookie(RECENT_FOLDERS_COOKIE_NAME,
					"" + m_nFolderID + "-");
			// JF- The Cookie will expire in 90 days.
			myCookie.SetExpireTime(60*60*24*90);
			myCookie.SetPath("");
			_webdata.AddCookie(myCookie);
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()
	 */
	public void CleanupTempData()
	{
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
	 */
	public void CleanupAllData()
	{
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new SaveObjectModel();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostModel#Finish()
	 */
	public void Finish()
	{
		// JF- There is nothing to do on Finish for regular Objects.
		// The control will take care of calling Finish on the parent AS which is the Editor.
		// JF- However for communities, we need to set the admin folder.
		// The community is a special case. The Object is being stored before the 
		// end of the Editor in the Unclassified Folder.
		// We want to set the right admin folder only at the very end when we know the user 
		// cannot cancel the Editor anymore.
		if (PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID())
		{
			m_ptObject.SetAdminFolderID(m_nFolderID);
		}
		
		if (PT_CLASSIDS.PT_SITEMAPFOLDER_ID == m_ptObject.GetClassID())
		{
			m_ptObject.SetAdminFolderID(m_nFolderID);	
		}
	}
	
	/**
	 * Returns true if the current object is a community. 
	 * 
	 * @return boolean
	 */
	public boolean IsCurrentObjectCommunity()
	{
		if (PT_CLASSIDS.PT_COMMUNITY_ID == m_ptObject.GetClassID())
		{
			return true;
		}
		
		return false;
	}	

	/**
	 * Returns the Folder ID in which the Object will be saved.
	 *
	 * @return boolean
	 */
	public boolean ForceSaveInFolder()
	{
		return m_bForceSaveInFolder;
	}

	/**
	  * Returns the admin folder ID.
	  *
	  * @return int
	  */
	public int GetAdminFolderID()
	{
		return m_nFolderID;
	}

	/**
	  * Returns the ID of the current selected tab.
	  *
	  * @return int
	  */
	public int GetCurrentTab()
	{
		return m_nCurrentTab;
	}

	/**
	 * Returns the folder name.
	 *
	 * @return String
	 */
	public String GetFolderName()
	{
		return m_sFolderName;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	  * Returns the description of the Object being created.
	  *
	  * @return String
	  */
	public String GetObjectDescription()
	{
		//MES- This should be the PRIMARY description, since the
		//	user will be editing this description and saving it
		//	as the primary description.
		return m_ptObject.GetDescription();
	}

	/**
	  * Returns the name of the Object being created.
	  *
	  * @return String
	  */
	public String GetObjectName()
	{
		//MES- This should be the PRIMARY name, since the
		//	user will be editing this name and saving it
		//	as the primary name.
		return m_ptObject.GetName();
	}

	/**
	 * Returns the array of most recently used folder IDs and names.
	 *
	 * @return Object[][]
	 */
	public Object[][] GetRecentlyUsedFolders()
	{
		return m_arrRecentlyUsedFolders;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostModel#GetRedirectAfterFinish()
	 */
	public Redirect GetRedirectAfterFinish()
	{
		return null;
	}

	/**
	  * Returns true if the body frame should be reposted from the
	  * bottom frame.
	  *
	  * @return boolean
	  */
	public boolean GetRepostBodyFromBottomFrame()
	{
		return m_bRepostBodyFromBottomFrame;
	}

	/**
	  * Returns true if the body frame should be reposted from the
	  * top frame.
	  *
	  * @return boolean
	  */
	public boolean GetRepostBodyFromTopFrame()
	{
		return m_bRepostBodyFromTopFrame;
	}

	/**
	  * Returns true if the tree frame should be reposted from
	  * the recent folders frame.
	  *
	  * @return boolean
	  */
	public boolean GetRepostTreeAndSave()
	{
		return m_bRepostTreeAndSave;
	}

	/**
	  * Returns true if the tree frame should be reposted from
	  * the recent folders frame.
	  *
	  * @return boolean
	  */
	public boolean GetRepostTreeRecentFoldersFrame()
	{
		return m_bRepostTreeRecentFoldersFrame;
	}

	/**
	  * Returns the title for the browser.
	  *
	  * @return String
	  */
	public String GetTitle()
	{
		return m_asOwner.GetString(138, "ptmsgs_portalinfrastructure");
	}

	/**
	  * Returns true if the current Object doesn't support ACLs and
	  * the security tab should not be displayed.
	  *
	  * @return boolean
	  */
	public boolean HideSecurityTab()
	{
		return m_bHideSecurityTab;
	}
	
	/**
	 * Returns true if the object description cannot be set/changed from the Editor.
	 *
	 * @return boolean
	 */
	public boolean IsDescriptionDisabled()
	{
		return m_bDisableDescription;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#Init(AActivitySpace)
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner = (SaveObjectAS) parent;
	}

	/**
	 * Adds the selected admin folder ID to Cookie.
	 *
	 * @param _xpRequest
	 * @param _webdata
	 */
	public void InitRecentlyUsedFolders(IXPRequest _xpRequest)
	{
		XPCookie[] arrCookies;
		XPCookie myCookie;

		if (null == _xpRequest)
		{
			return;
		}

		arrCookies = _xpRequest.GetCookies();

		if (null != arrCookies)
		{	
			
			for (int i = 0; i < arrCookies.length; i++)
			{
				myCookie = arrCookies[i];

				if (RECENT_FOLDERS_COOKIE_NAME.equals(myCookie.GetName()))
				{
					String sFolderIDs  = myCookie.GetValue();
					int nFolderID	   = -1;
					int k			   = -1;
					Object[][] arrFolders   = new Object[2][4];
					IPTAdminFolder myFolder = null;

					while ((-1 != sFolderIDs.indexOf("-")) && (k < 3))
					{
						try
						{
							nFolderID = XPConvert.ToInteger(sFolderIDs.substring(
										0, sFolderIDs.indexOf("-")));

							if (!IsAdminFolderInRecentFolders(nFolderID) && 
								(PT_INTRINSICS.PT_ADMIN_FOLDER_UNCLASSIFIEDOBJECTS != nFolderID) &&
								(-1 != nFolderID))
							{
								
								try
								{
									// JF- The current user must have at least edit access to the folder
									boolean bHasEditAccess = m_ptSession.GetAdminCatalog()
										.IsAdminFolderAccessible(nFolderID,
										PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT);
										
									if (bHasEditAccess)
									{								 
										myFolder = m_ptSession.GetAdminCatalog().OpenAdminFolder(nFolderID, false);
									}
								}
								catch (Exception e)
								{
									
										log.Error("InitRecentlyUsedFolders Unable to open admin foler with ID: " +
											nFolderID + ". The exception was: " + e);
										
								}
								
								if (null != myFolder)
								{
									k ++;
									arrFolders[0][k] = new Integer(nFolderID);
									arrFolders[1][k] = myFolder;
								}
								myFolder = null;
							}
						}
						catch (Exception e)
						{
							
								log.Error("InitRecentlyUsedFolders Unable to parse the array of recently used folders." + 
									"The exception was: " + e);
								
						}

						sFolderIDs     = sFolderIDs.substring(sFolderIDs.indexOf(
									"-") + 1);
						nFolderID	   = -1;
					}
					
					if (k != -1)
					{
						m_arrRecentlyUsedFolders = new Object[k + 1][2];
						
						for (int l = 0; l < k + 1; l ++)
						{
							try
							{
								myFolder = (IPTAdminFolder) arrFolders[1][l];
								m_arrRecentlyUsedFolders[l][0] = arrFolders[0][l];
								m_arrRecentlyUsedFolders[l][1] = myFolder.GetLocalizedName();
							}
							catch (Exception e)
							{
								
									log.Error("InitRecentlyUsedFolders Unable to open admin foler with ID: " +
										nFolderID + ". The exception was: " + e);
									
							}
						}
						}

					break;
				}
			}
		}
	}

	/**
	 * This methods should be called when the pop-up is displayed for the 1st time.
	 *
	 */
	public void InitializeSaveObjectPopUp(IXPRequest _xpRequest)
	{
		// JF- First we need the Object being created and the Session. 
		// We get them from the parent AS (which is the Editor AS).
		EditorAS parentSpace	   = (EditorAS) m_asOwner.GetParentSpace();
		ObjEditorModel parentModel = (ObjEditorModel) parentSpace.GetModel(parentSpace.m_strModelName);
		m_ptObject				   = parentModel.GetObject();
		m_ptSession				   = parentModel.GetPTSession();
		m_nFolderID				   = parentModel.GetFolderID();
		m_bDisableDescription	   = parentModel.IsDescriptionDisabled();
		m_arrRecentlyUsedFolders   = new Object[0][0];

		if (parentModel.ForceSaveInFolder())
		{
			m_sFolderName			 = parentModel.GetFolderName();
			m_bForceSaveInFolder     = true;
		}
		else
		{
			InitRecentlyUsedFolders(_xpRequest);
		}

		if (null == m_ptObject)
		{
			throw new XPException(
				"SaveObjectModel::InitializeSaveObjectPopUp The IPTObject from the parent" +
				" Activity Space is null");
		}

		// JF- We always start with the folder tree tab.
		m_nCurrentTab = SAVEOBJECT_TAB_FOLDERTREE;

		// JF- Users don't support ACLs
		if (PT_CLASSIDS.PT_USER_ID == m_ptObject.GetClassID())
		{
			m_bHideSecurityTab = true;
		}
	}

	/**
	  * Returns true if the current admin folder is one of the recently used folders.
	  *
	  * @return boolean
	  */
	public boolean IsAdminFolderInRecentFolders(int _nFolderID)
	{
		boolean bIsAdminFolderInRecentFolders = false;

		for (int i = 0; i < m_arrRecentlyUsedFolders.length; i++)
		{
			if (null != m_arrRecentlyUsedFolders[i][0])
			{
				if (_nFolderID == XPConvert.ToInteger(m_arrRecentlyUsedFolders[i][0]))
				{
					bIsAdminFolderInRecentFolders = true;

					break;
				}
			}
		}

		return bIsAdminFolderInRecentFolders;
	}

	/**
	  * Returns true if the migration has already been approved for the current object.
	  *
	  * @return boolean
	  */
	public boolean IsMigrationApproved()
	{
		// JF- Retrieve the parent Model
		EditorAS myParentAS			    = (EditorAS) m_asOwner.GetParentSpace();
		ObjEditorModel myObjEditorModel = (ObjEditorModel) myParentAS.GetModel(myParentAS.m_strModelName);

		return myObjEditorModel.IsMigrationApproved();
	}

	/**
	  * Returns true if the current user has system administrator activity rights.
	  *
	  * @return boolean
	  */
	public boolean IsSysAdmin()
	{
		// JF- Retrieve the parent Model
		EditorAS myParentAS = (EditorAS) m_asOwner.GetParentSpace();
		boolean bIsSysAdmin = ((ObjEditorModel) myParentAS.GetModel(myParentAS.m_strModelName)).GetIsSysAdmin();

		return bIsSysAdmin;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostModel#SavePage(String, XPHashtable)
	 *
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		// JF- Those falgs are for "one time" JavaScripts. So we need to set them back 
		// to false after they have been used. 
		m_bRepostBodyFromTopFrame		    = false;
		m_bRepostBodyFromBottomFrame	    = false;
		m_bRepostTreeAndSave			    = false;
		m_bRepostTreeRecentFoldersFrame     = false;

		if (null == m_ptObject)
		{
			throw new XPException(
				"SaveObjectModel::SavePage The save object pop-up has not been initialized.");
		}

		// JF- Then we switch between the pages to know which one we need to save.
		if (_sPageName.equals(SaveObjectBottomFrameDP.STR_MVC_CLASS_NAME))
		{
			SaveNameDesc(_htFormData);
		}
		else if (!m_bHideSecurityTab &&
				_sPageName.equals(SaveObjectTopFrameDP.STR_MVC_CLASS_NAME))
		{
			SaveTab(_htFormData);
		}
		else if (!m_bHideSecurityTab &&
				_sPageName.equals(ObjectSecurityDP.STR_MVC_CLASS_NAME))
		{
			// JF- We need to call the save method on the security page.
			AActivitySpace mySecurityAS		    = m_asOwner.GetParentSpace();
			ObjectSecurityModel mySecurityModel = (ObjectSecurityModel) mySecurityAS.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB);
			mySecurityModel.SaveSection(_htFormData);

			PriorityDropDownModel myPriorityDropDownModel = (PriorityDropDownModel) mySecurityAS.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN);
			myPriorityDropDownModel.SaveSection(_htFormData);
		}
		else if (_sPageName.equals(TreeWithSearchDP.STR_MVC_CLASS_NAME))
		//else if (_sPageName.equals(SaveObjectMainFrameDP.STR_MVC_CLASS_NAME)) 
		{
			// JF- We need to call the save method on the tree.
			AActivitySpace myTreeAS = m_asOwner.GetSubSpace(SAVEOBJECT_FOLDERTREE_SUBSPACEKEY);
			ATreeModel myTreeModel  = (ATreeModel) myTreeAS.GetModel(ATreeAS.TREEMODEL);

			//_htFormData.PutElement(TreeView.HTMLBUTTON_RADIO, new String[]{XPConvert.ToString(m_nFolderID)});
			//_htFormData.PutElement(TreeWithSearchDP.HTMLINPUT_DISPLAYING_SEARCH, new String[]{XPConvert.ToString(false)});
			//_htFormData.PutElement(TreeView.HTMLHIDDEN_SELECTEDIDS, new String[]{XPConvert.ToString(m_nFolderID)});
			myTreeModel.SavePage(_sPageName, _htFormData);

			SaveAdminFolderID(_htFormData);
		}
		else if (_sPageName.equals(SaveObjectRecentFoldersDP.STR_MVC_CLASS_NAME))
		{
			if (!m_bForceSaveInFolder)
			{
				String[] data = (String[]) _htFormData.GetElement(SaveObjectRecentFoldersView.HTMLINPUT_RECENT_FOLDERS);

				if ((null != data) && (0 != data.length) && (null != data[0]) &&
						(0 != data[0].length()))
				{
					try
					{
						m_nFolderID = XPConvert.ToInteger(data[0]);
					}
					catch (Exception e)
					{
						
							log.Error("SaveObjectModel::SaveAdminFolderID The Folder ID posted by the form was invalid: " +
								data[0]);
						
					}
				}
			}
		}

		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	  * Sets a flag to know if the body frame should be reposted from
	  * the bottom frame.
	  *
	  * @param _bRepostBodyFromBottomFrame
	  */
	public void SetRepostBodyFromBottomFrame(
		boolean _bRepostBodyFromBottomFrame)
	{
		m_bRepostBodyFromBottomFrame = _bRepostBodyFromBottomFrame;
	}

	/**
	  * Sets a flag to know if the body frame should be reposted from
	  * the top frame.
	  *
	  * @param _bRepostBodyFromTopFrame
	  */
	public void SetRepostBodyFromTopFrame(boolean _bRepostBodyFromTopFrame)
	{
		m_bRepostBodyFromTopFrame = _bRepostBodyFromTopFrame;
	}

	/**
	  * Sets a flag to know if the tree frame should be reposted from
	  * the recent folders frame.
	  *
	  * @param _bRepostTreeAndSave
	  */
	public void SetRepostTreeAndSave(boolean _bRepostTreeAndSave)
	{
		m_bRepostTreeAndSave = _bRepostTreeAndSave;
	}

	/**
	  * Sets a flag to know if the tree frame should be reposted from
	  * the recent folders frame.
	  *
	  * @param _bRepostTreeRecentFoldersFrame
	  */
	public void SetRepostTreeRecentFoldersFrame(
		boolean _bRepostTreeRecentFoldersFrame)
	{
		m_bRepostTreeRecentFoldersFrame = _bRepostTreeRecentFoldersFrame;
	}

	/**
	 * Saves the new folder ID and set it in the Object.
	 * Also if the folder ID has changed, we re-initialize
	 * the security page as the security is based on the folder security.
	 *
	 */
	protected void SaveAdminFolderID(XPHashtable _htFormData)
	{
		if (!m_bForceSaveInFolder)
		{
			/*String[] data = (String[]) _htFormData.GetElement(SaveObjectRecentFoldersView.HTMLHIDDEN_SELECTED_FOLDER);

			if ((null != data) && (0 != data.length) && (null != data[0]) && (0 != data[0].length()))
			{
			    try
			    {
			        m_nFolderID = XPConvert.ToInteger(data[0]);
			    } catch (Exception e) {
			        if ( log.IsErrorEnabled() ) {
			            log.Error("SaveObjectModel::SaveAdminFolderID The Folder ID posted by the form was invalid: " + data[0]);
			        }
			    }

			    if ((m_nFolderID != m_ptObject.GetAdminFolderID()) && (PT_CLASSIDS.PT_COMMUNITY_ID != m_ptObject.GetClassID()))
			    {
			        // JF- We update the Object
			        m_ptObject.SetAdminFolderID(m_nFolderID);

			        // JF- And also we re-initialize the security page
			        ((ObjectSecurityModel) m_asOwner.GetParentSpace().GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB)).
			            InitializeSecurityPage(m_ptObject, m_ptSession);
			    }
			}*/
			TreeAS asTree = (TreeAS) m_asOwner.GetSubSpace(SAVEOBJECT_FOLDERTREE_SUBSPACEKEY);

			if (null != asTree)
			{
				// JF- We retrieve the tree
				ITreeModelRO myTree    = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
				XPArrayList mySelected = myTree.GetSelectedItems();

				if (0 != mySelected.GetSize())
				{
					int nFolderID = ((TreeLeafNode) mySelected.GetElement(0)).m_nObjectID;
					
					if (PT_INTRINSICS.PT_ADMIN_FOLDER_UNCLASSIFIEDOBJECTS != nFolderID)
					{
						// JF- We get the selected folder ID
						m_nFolderID = nFolderID;
					}
				}
				else
				{
					
						log.Debug("SaveObjectModel::SaveAdminFolderID No folder ID was returned by the Tree.");
					
				}
			}

			if ((m_nFolderID != m_ptObject.GetAdminFolderID()) &&
					(PT_CLASSIDS.PT_COMMUNITY_ID != m_ptObject.GetClassID()))
			{
				// JF- We update the Object
				m_ptObject.SetAdminFolderID(m_nFolderID);

				// JF- And also we re-initialize the security page
				((ObjectSecurityModel) m_asOwner.GetParentSpace().GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB)).InitializeSecurityPage(m_ptObject,
					m_ptSession);
			}
		}
	}

	/**
	 * Saves name and description..
	 *
	 * @param _htFormData
	 */
	protected void SaveNameDesc(XPHashtable _htFormData)
	{
		String[] data;

		// JF- Save the name
		data = (String[]) _htFormData.GetElement(SaveObjectNameDescView.HTMLINPUT_NAME);

		if ((null != data) && (0 != data.length) && (null != data[0]) &&
				(0 != data[0].length()))
		{
			// JF- Somehow if the name is longer than 251 char, the SetName will work 
			// but Store will throw an exception
			if (OBJECT_NAME_MAX_LENGTH > data[0].length())
			{
				try
				{
					m_ptObject.SetName(data[0]);
				}
				catch (Exception e)
				{
					
						log.Error("SaveObjectModel::SaveNameDesc The Object Name posted by the form was invalid: " +
							data[0]);
					
				}
			}
			else
			{
				
					log.Error("SaveObjectModel::SaveNameDesc The Object Name is too long and " + 
						"it will not be saved. The length is: " + data[0].length());
				
			}
		}
		else
		{
			
				log.Warn("SaveObjectModel::SaveNameDesc The Object Name has not been posted by the form.");
			
		}
		
		if (! IsDescriptionDisabled())
		{
			// JF- Save the description
			data = (String[]) _htFormData.GetElement(SaveObjectNameDescView.HTMLINPUT_DESCRIPTION);
	
			if ((null != data) && !(0 == data.length) && null != data[0])
			{
				try
				{
					m_ptObject.SetDescription(data[0]);
				}
				catch (Exception e)
				{
					
						log.Error("SaveObjectModel::SaveNameDesc The Object Description posted by the form was invalid." +
							data[0]);
					
				}
			}
			else
			{
				
					log.Warn("SaveObjectModel::SaveNameDesc The Object Description has not been posted by the form.");
				
			}
		}

		// JF- save the object migration checkbox
		if (_htFormData.ContainsKey(
					SaveObjectNameDescView.HTMLCHECKBOX_MIGRATION))
		{
			// JF- Retrieve the parent Model
			EditorAS myParentAS = (EditorAS) m_asOwner.GetParentSpace();

			// AS- Mark that the object has been approved for migration
			((IMigrationHistoryModel) myParentAS.GetModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME)).SetApprovedForMigration(true);
		}
	}

	/**
	 * Saves the new tab ID.
	 *
	 * @param _htFormData
	 */
	protected void SaveTab(XPHashtable _htFormData)
	{
		String[] data;

		data = (String[]) _htFormData.GetElement(SaveObjectMenuView.HTMLHIDDEN_NEWTAB);

		if ((null != data) && (0 != data.length) && (null != data[0]) &&
				(0 != data[0].length()))
		{
			try
			{
				m_nCurrentTab = XPConvert.ToInteger(data[0]);
			}
			catch (Exception e)
			{
				
					log.Error("SaveObjectModel::SaveTab The new tab ID posted by the form was invalid." +
						data[0]);
				
			}
		}
		else
		{
			
				log.Warn("SaveObjectModel::SaveTab The new tab name has not been posted by the form.");
			
		}
	}
}

