package com.plumtree.portaluiinfrastructure.expandablelist.selectptobject;


// Plumtree Server
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tree.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.expandablelist.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.tree.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This is the implementation of the Expandable List
 * framework for the the Select PTObject list.
 * The Select PTObject is a list of objects.
 * New objects can be added to the list by selecting
 * them from the associated Object tree.
 * The list is by default sorted by name ascending.
 *
 * @author Julie Fournier
 */
public class SelectPTObjectModel extends ExpListModel
	implements ISelectPTObjectModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.expandablelist.selectptobject.SelectPTObjectModel");

	/**
	 * Class name for the activity space */
	public static final String STR_MVC_CLASS_NAME = "SelectPTObject";

	/**
	 * Key name used by the Activity Space to store the tree. */
	public static final String SUBSPACE_KEY_SELECTTREE = "SelectPTObjectTree";

	/**
	 * Number total of columns
	 */
	public static final int EXPLIST_COL_MAX = 4;

	/**
	 * This column contains the Object Class IDs.
	 */
	public static final int EXPLIST_COL_CLASSID = 1;

	/**
	 * This column contains the Object IDs.
	 */
	public static final int EXPLIST_COL_OBJECTID = 2;

	/**
	 * This is the String that we doisplay in the list.
	 * E.g.: Object name or folder path.
	 */
	public static final int EXPLIST_COL_STRING = 3;

	/**
	 * Default initial size for the sorted array.
	 */
	public static final int EXPLIST_DEFAULT_INITIAL_SIZE = 50;

	/**
	 * Number total of columns for hide objects array.
	 */
	public static final int HIDE_COL_MAX = 2;

	/**
	 * This column of the passed Objects to Hide contains the class IDs.
	 */
	public static final int HIDE_COL_CLASSID = 0;

	/**
	 * This column of the passed Objects to Hide contains the object IDs.
	 */
	public static final int HIDE_COL_OBJECTID = 1;

	/**
	 * Create new array column: class ID
	 */
	public static final int CREATE_NEW_COL_CLASSID = 0;

	/**
	 * Create new array column: AS Name
	 */
	public static final int CREATE_NEW_COL_ASNAME = 1;

	/**
	 * Create new array column: text to display with the create new link
	 */
	public static final int CREATE_NEW_COL_LABEL = 2;

	/**
	 * Create new array column: icon to display for the create new link
	 */
	public static final int CREATE_NEW_COL_ICON = 3;

	/**
	 * Create new array column: extra control arguments for the create action.
	 * Use CREATE_NEW_COL_MAXWITHPARAMS to set the size of the array when using
	 * this element.
	 */
	public static final int CREATE_NEW_COL_PARAMS = 4;

	/**
	 * Size of each m_arrCreateNewInfo row
	 */
	public static final int CREATE_NEW_COL_MAX = 4;

	/**
	 * Size of each m_arrCreateNewInfo row with optional control parameters.
	 */
	public static final int CREATE_NEW_COL_MAXWITHPARAMS = 5;

	/**
	 * Optional control parameter index for the parameter name. Used in the
	 * create new array column: icon to display for the create new link
	 */
	public static final int PARAM_COL_PARAMNAME = 0;

	/**
	 * Optional control parameter index for the parameter value. Used in the
	 * create new array column: icon to display for the create new link
	 */
	public static final int PARAM_COL_PARAMVALUE = 1;

	/**
	 * Icon to display on each row, just before the input text.
	 * RWW: This will now be the "default". See m_xphtRowIcons 
	 * for precedence in icon loading.
	 */
	protected PTImageType m_imgDefaultRowIcon;

	/**
	 * RWW: Stores multiple icons - allows us to store a different 
	 * icon for each class id.  When retrieving icons internally, 
	 * this table will be checked first.  If no icons appear here,
	 * then we will check the default row icon.  If that is null,
	 * then we will show no image.  Having an icon for a specific
	 * class id in this table will take precedence.
	 */
	protected XPHashtable m_xphtRowIcons;

	/**
	 * JH: Set to true if no row icons are to be displayed.  This will override
	 * m_xphtRowIcons, m_imgDefaultRowIcon, and the default ClassDescriptor icon
	 * retrieval mechanism.
	 */
	protected boolean m_bNoRowIcons = false;
	
	/**
	 * Tree Space ID. The first time the ID is empty and a new AS is opened,
	 * but after that, we save the ID and the same tree AS is reopened each time. */
	protected String m_sTreeSpaceID = "";

	/**
	 * Tree sub title
	 */
	protected String m_sTreeSubTitle;

	/**
	 * Tree title
	 */
	protected String m_sTreeTitle;
	
	/**
	 * Class IDs and Object IDs of objects to display in the tree.
	 */
	protected String m_sDisplayObjects;

	/**
	 * Objects to hide for the object tree.
	 */
	protected String m_strObjectsToHide = "";

	/**
	 * Query Filter for the object tree.
	 */
	protected String m_strTreeQueryFilter = "";

	/**
	 * Information about the create new links to display
	 */
	protected Object[][] m_arrCreateNewInfo;

	/**
	 * Class ID of the Objects displayed on the list.
	 */
	protected int[] m_arrObjectClassIDs;

	/**
	 * Selection type for the object tree.
	 */
	protected boolean m_bTreeMultiSelect = true;

	/**
	 * This value determines the security access requirements for the objects to
	 * be added to the list... (a filter for the tree control).
	 */
	protected int m_nAccessLevel = 0;

	/**
	 * Community Mode to display objects.
	 * 
	 * @see com.plumtree.portaluiinfrastructure.tree.StartTreeControl.COMMUNITY_MODE
	 */
	protected int m_nCommunityMode = 0;

	/**
	 * Community Folder ID to display objects from.
	 */
	protected int m_nCommunityFolderID = 0;

	/**
	 * Leaf type for the object tree.
	 */
	protected int m_treeLeafType = 0;
	
	/**
	 * JF- Array of classID/objectID that cannot be deleted from the list.
	 */
	protected int[][] m_arrReadOnlyObjects;
	
	/**
	 * JF- Array of read-only objects: column index for classID.
	 */
	public static final int READ_ONLY_ROW_CLASSID = 0;
	
	/**
	 * JF- Array of read-only objects: column index for objectID.
	 */
	public static final int READ_ONLY_ROW_OBJECTID = 1;

	public SelectPTObjectModel()
	{
		super();
		m_treeLeafType = TreeConstants.N_LEAFSELECT;
		m_nAccessLevel = PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT;
	}

	/**
	 * Helper function for extracting and adding extra control parameters from
	 * the create object string.
	 *
	 * @param _nClassID
	 * @param myURL
	 */
	public void AddControlParamsToURL(int _nClassID, ASURL myURL)
	{
		Object oParams = null;
		String[][] arParams = null;
		boolean bFoundFolderID = false;

		//SE: logging
		log.FunctionBegin("SelectPTObjectModel::AddControlParamsToURL");
		
		for (int i = 0; i < m_arrCreateNewInfo.length; i++)
		{
			int nClassIDFromArray = XPConvert.ToInteger(m_arrCreateNewInfo[i][CREATE_NEW_COL_CLASSID]);

			if (nClassIDFromArray == _nClassID)
			{
				if (m_arrCreateNewInfo[i].length >= CREATE_NEW_COL_MAXWITHPARAMS)
				{
					oParams = m_arrCreateNewInfo[i][CREATE_NEW_COL_PARAMS];

					break;
				}
			}
		}

		if (null != oParams)
		{
			arParams = (String[][]) oParams;
			
			for (int j = 0; j < arParams.length; j++)
			{
				if ((null != arParams[j]) && (2 == arParams[j].length))
				{
					String strName  = arParams[j][PARAM_COL_PARAMNAME];
					
					if ((null != strName) && (0 != strName.length()))
					{
						String strValue = arParams[j][PARAM_COL_PARAMVALUE];
						
						// Add the parameter to the url.
						myURL.AddControlArgument(strName, strValue);
						
						if (strName.equalsIgnoreCase(ObjEditorModel.EDITOR_QS_INT_QS_FOLDER_ID))
						{
							bFoundFolderID = true;	
						}
					}
				}
			}
		}
		else
		{
			
				log.Debug("AddControlParamsToURL: Unable to retrieve control info.");
				
		}
		
		if (! bFoundFolderID)
		{
			
			try
			{	
				ObjEditorModel myModel = (ObjEditorModel) m_asOwner.GetModel(((ObjEditorAS)m_asOwner).
					m_strModelName);
				int nFolderID = myModel.GetFolderID();
				myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_QS_FOLDER_ID,
					XPConvert.ToString(nFolderID));
			}
			catch (Exception e)
			{
				
					log.Warn("Could not retrieve the Admin Folder ID from the current Editor." 
						+ " No Admin Folder will be pre-selected in the save-as pop-up of sub Editors.");
				
			}	
		}
		//SE: logging
		log.FunctionEnd("SelectPTObjectModel::AddControlParamsToURL");
	}
	
	/**
	 * Returns a unique String for the name of the pop-up.
	 *
	 * @param _sName
	 */
	public String GetUniquePopUpName(String _sName)
	{
		return GetUniqueString(_sName) + m_asOwner.GetSpaceID();
	}

	/**
	 * Clear the list and add all the values from the tree.
	 *
	 * @param _htFormData Form data
	 */
	public void AddRowsOnRepost(XPHashtable _htFormData)
	{
		//SE: logging
		log.FunctionBegin("SelectPTObjectModel::AddRowsOnRepost");
		
		if (!m_bIsReadOnly)
		{
			SaveTreeID(_htFormData);

			TreeAS asTree = (TreeAS) m_asOwner.GetSubSpace(GetUniqueString(
						SUBSPACE_KEY_SELECTTREE));

			if (null != asTree)
			{
				ITreeModelRO myTree    = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
				XPArrayList mySelected = myTree.GetSelectedItems();
				int nClassID;
				int nObjectID;
				int nIndex;

				// JF- Before we clear the items and get the brand new list from the tree, we want to keep a copy of the indexes.
				// If the move row actions are enabled, we need to keep the same indexes for Objects already in the list. 
				// We need to keep the order for existing Objects and add the new ones at the end.
				Object[][] arrIndexMapping = CreateIndexMapping(GetItems());

				// JF- We clear the array before we start adding the selected items from the tree.
				super.GetItems().ClearArray();

				// Add rows in bulk to avoid resorting the entire list on each add.
				// This becomes expensive for groups with 4000 users.
				Object[][] arrBulkNewRows = new Object[mySelected.GetSize()][EXPLIST_COL_MAX];

				for (int i = 0; i < mySelected.GetSize(); i++)
				{
					nClassID	  = ((TreeLeafNode) mySelected.GetElement(i)).m_nClassID;
					nObjectID     = ((TreeLeafNode) mySelected.GetElement(i)).m_nObjectID;
					nIndex		  = GetIndex(arrIndexMapping, nClassID,
							nObjectID);

					if (-1 == nIndex)
					{
						nIndex = m_nNextIndex;
						m_nNextIndex++;
					}

					arrBulkNewRows[i][EXPLIST_SORTEDARRAY_PROPID_INDEX]   = new Integer(nIndex);
					arrBulkNewRows[i][EXPLIST_COL_CLASSID]				  = new Integer(nClassID);
					arrBulkNewRows[i][EXPLIST_COL_OBJECTID]				  = new Integer(nObjectID);

					if (PT_CLASSIDS.PT_CATALOGFOLDER_ID == nClassID)
					{
						arrBulkNewRows[i][EXPLIST_COL_STRING] = ((TreeLeafNode) mySelected.GetElement(i)).m_strLocation;
					}
					else
					{
						arrBulkNewRows[i][EXPLIST_COL_STRING] = ((TreeLeafNode) mySelected.GetElement(i)).GetName();
					}
				}

				super.GetItems().AddRows(arrBulkNewRows);
			}
		}
		
		//SE: logging
		log.FunctionEnd("SelectPTObjectModel::AddRowsOnRepost");
	}

	/**
	 * Tells whether the Object identified by the class ID
	 * and the object ID in argument is in the list or no.
	 *
	 * @param _nClassID
	 * @param _nObjectID
	 *
	 * @return boolean
	 */
	public boolean ContainsObject(int _nClassID, int _nObjectID)
	{
		//SE: logging
		log.FunctionBegin("SelectPTObjectModel::ContainsObject");
		
		for (int i = 0; i < m_saData.GetCount(); i++)
		{
			if ((_nClassID == XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_CLASSID))) &&
					(_nObjectID == XPConvert.ToInteger(m_saData.GetItem(i,
						EXPLIST_COL_OBJECTID))))
			{
				//SE: logging
				log.FunctionEnd("SelectPTObjectModel::ContainsObject");
				return true;
			}
		}

		//SE: logging
		log.FunctionEnd("SelectPTObjectModel::ContainsObject");
		return false;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new SelectPTObjectModel();
	}

	/**
	 * This method will be called when you come back to the main Editor from a sub Editor.
	 *
	 */
	public void DoTaskOnComeBackFromSubEditor()
	{
		String sEditorASName;
		EditorAS myEditorAS;
		ObjEditorModel myObjEditorModel;
		Object[] oNewRow;
		int nClassID, nObjectID;

		//SE: logging
		log.FunctionBegin("SelectPTObjectModel::DoTaskOnComeBackFromSubEditor");
		
		// JF- Check if they are any newly created objects
		for (int i = 0; i < m_arrCreateNewInfo.length; i++)
		{
			sEditorASName = (String) m_arrCreateNewInfo[i][CREATE_NEW_COL_ASNAME];

			if (m_asOwner.GetSubSpaceExists(sEditorASName))
			{
				myEditorAS			 = (EditorAS) m_asOwner.GetSubSpace(sEditorASName);
				myObjEditorModel     = (ObjEditorModel) myEditorAS.GetModel(myEditorAS.m_strModelName);

				oNewRow										  = new Object[EXPLIST_COL_MAX];
				oNewRow[EXPLIST_SORTEDARRAY_PROPID_INDEX]     = new Integer(m_nNextIndex);
				m_nNextIndex++;
				nClassID						  = myObjEditorModel.GetClassID();
				oNewRow[EXPLIST_COL_CLASSID]	  = new Integer(nClassID);
				oNewRow[EXPLIST_COL_OBJECTID]     = new Integer(myObjEditorModel.GetObjectID());

				if (PT_CLASSIDS.PT_CATALOGFOLDER_ID == nClassID)
				{
					oNewRow[EXPLIST_COL_STRING] = ((IPTFolder) myObjEditorModel.GetObject()).GetPath();
				}
				else
				{
					oNewRow[EXPLIST_COL_STRING] = myObjEditorModel.GetObjectName();
				}
				// Wait with resorting until we finisehd adding all entries rather than
				// resorting after adding each individual entry
				// Updated to fix bug #51045:
				super.GetItems().AddRowWithoutSort(oNewRow); 

				m_asOwner.DeleteSubSpace(sEditorASName);
			}
		}
		// done adding all the items, resort array
		// Updated to fix bug #51045:
		super.GetItems().SortArray();
		
		
		// SE: Bug 57861 - use StringBuilder to avoid CPU spike
		XPStringBuilder sbTemp = new XPStringBuilder(100);
		String sSubSpace;

		// JF- Go through all the existing sub spaces and update the Object name.
		for (int i = 0; i < m_saData.GetCount(); i ++)
		{
			nClassID = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_CLASSID));

			if (PT_CLASSIDS.PT_CATALOGFOLDER_ID != nClassID)
			{
				nObjectID = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_OBJECTID));

				// SE: Bug 57861 - use StringBuilder to avoid CPU spike
				sbTemp.SetLength(0);
				sbTemp.Append(STR_MVC_CLASS_NAME);
				sbTemp.Append(nClassID);
				sbTemp.Append(nObjectID);
				sSubSpace = sbTemp.ToString();

				if (m_asOwner.GetSubSpaceExists(sSubSpace))
				{
					try
					{
						EditorAS mySpace = (EditorAS) m_asOwner.GetSubSpace(sSubSpace);
						ObjEditorModel myModel = (ObjEditorModel) mySpace.GetModel(mySpace.m_strModelName);
						m_saData.SetItem(i, EXPLIST_COL_STRING, myModel.GetObjectName());

					}
					catch (Exception e)
					{

							log.Warn("Could not retrieve the sub Space with key: "
								+ STR_MVC_CLASS_NAME + nClassID + nObjectID);

					}
				}
			}
		}
		//SE: logging
		log.FunctionEnd("SelectPTObjectModel::DoTaskOnComeBackFromSubEditor");
	}

	/**
	 * Returns the nAccessLevel. The accesslevel of the objects for the tree control to display.
	 * @return int
	 */
	public int GetAccessLevel()
	{
		return m_nAccessLevel;
	}

	/**
	 * Returns the community mode for the tree.
	 *
	 * @return int
	 */
	public int GetCommunityMode()
	{
		return m_nCommunityMode;
	}

	/**
	 * Returns the community id for the tree.
	 *
	 * @return int
	 */
	public int GetCommunityFolderID()
	{
		return m_nCommunityFolderID;
	}

	/**
	 * Returns the create new action informations.
	 *
	 * @return Object[][]
	 */
	public Object[][] GetCreateNewInfo()
	{
		return m_arrCreateNewInfo;
	}

	/**
	 * Returns the URL to the Editor in create mode (new object).
	 *
	 * @param _nClassID
	 * @param _sEditorASName
	 *
	 * @return ASURL URL to the Editor
	 */
	public ASURL GetCreateNewURL(int _nClassID, String _sEditorASName)
	{
		ASURL myURL = new ASURL(m_asOwner);

		myURL.SetLinkCreateNewSpace(_sEditorASName, m_asOwner);
		myURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
		myURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE,
			XPConvert.ToString(EditorStartControl.EDITOR_START_FLAG_CREATE));
		myURL.AddControlArgument(EditorStartControl.QS_DELETE_ON_CANCEL,
			XPConvert.ToString(true));
		myURL.SetSubSpaceKey(_sEditorASName);
		myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_CLASS_ID,
			XPConvert.ToString(_nClassID));
		
		
		
		// Get the extra control arguments
		AddControlParamsToURL(_nClassID, myURL);

		return myURL;
	}

	/**
	 * Returns the URL to edit the object.
	 *
	 * @param _nClassID
	 * @param _nObjectID
	 * @param _sEditorASName
	 *
	 * @return ASURL URL to the Editor
	 */
	public ASURL GetEditObjectURL(int _nClassID, int _nObjectID)
	{
		ASURL myURL = new ASURL(m_asOwner);

		// JF- For any Object in the list, the sub space key will be 
		// this Model name with the object class id and the object id.
		String sSubEditorKey = STR_MVC_CLASS_NAME + _nClassID + _nObjectID;

		if (m_asOwner.GetSubSpaceExists(sSubEditorKey))
		{
			EditorAS mySubEditor = (EditorAS) m_asOwner.GetSubSpace(sSubEditorKey);
			myURL.SetLinkGetCachedSpace(mySubEditor);
		}
		else
		{
			String sEditorASName = GetEditorASName(_nClassID);

			myURL.SetLinkCreateNewSpace(sEditorASName, m_asOwner);
			myURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
			myURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE,
				XPConvert.ToString(EditorStartControl.EDITOR_START_FLAG_EDIT));
			myURL.SetSubSpaceKey(sSubEditorKey);
			myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_CLASS_ID,
				XPConvert.ToString(_nClassID));
			myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_QS_OBJECT_ID,
				XPConvert.ToString(_nObjectID));
		}

		AddControlParamsToURL(_nClassID, myURL);

		return myURL;
	}

	/**
	 * Returns the hidden objects for the tree.
	 *
	 * @return String
	 */
	public String GetHiddenObjects()
	{
		return m_strObjectsToHide;
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns String of Class ID of the Objects
	 * displayed in the tree.
	 * E.g.: "2,45,3"
	 *
	 * This is the format used by the tree.
	 *
	 * @return String
	 */
	public String GetObjectClassIDs()
	{
		//SE: logging
		log.FunctionBegin("SelectPTObjectModel::GetObjectClassIDs");
		
		// SE: Bug 57861 - use StringBuilder to avoid CPU spike
		XPStringBuilder sbClassIDs = new XPStringBuilder(3 * m_arrObjectClassIDs.length);

		// JF- There is no object manager for admin folders so the tree has to be started differently.
		if (PT_CLASSIDS.PT_ADMIN_FOLDER_ID != m_arrObjectClassIDs[0])
		{
			// JF- We loop through the array of IDs and add each ID 
			// to the String.
			for (int i = 0; i < m_arrObjectClassIDs.length; i++)
			{
				if (i > 0)
				{
					// SE: Bug 57861 - use StringBuilder to avoid CPU spike
					sbClassIDs.Append(",");
				}
				// SE: Bug 57861 - use StringBuilder to avoid CPU spike
				sbClassIDs.Append(XPConvert.ToString(m_arrObjectClassIDs[i]));
			}
		}

		//SE: logging
		log.FunctionEnd("SelectPTObjectModel::GetObjectClassIDs");
		// SE: Bug 57861 - use StringBuilder to avoid CPU spike
		return sbClassIDs.ToString();
	}

	/**
	 * Returns the icon to display on each row before the
	 * input text.
	 * If the value is null, no icon is displayed.
	 *
	 * @return PTImageType
	 */
	public PTImageType GetDefaultRowIcon()
	{
		return m_imgDefaultRowIcon;
	}

	/**
	 * Returns the icon to display on each row before the
	 * input text, given a class id.  If the value is null
	 * then we get the default row icon.
	 *
	 * @return PTImageType
	 */
	public PTImageType GetRowIconByClassID(int _nClassID)
	{
		// RWW: since we don't use this all the time
		// let's do lazy set/gets, save memory in init.
		if (null == m_xphtRowIcons)
			return m_imgDefaultRowIcon;

		// RWW: if we didn't specify a specific image
		// for this class id, then just return the default
		// SE: Bug 57861 - use XPConvert for speed
		if (!m_xphtRowIcons.ContainsKey(XPConvert.ToString(_nClassID)))
			return m_imgDefaultRowIcon;

		// RWW: ah, we have a specific image, use it.
		// SE: Bug 57861 - use XPConvert for speed
		return (PTImageType)m_xphtRowIcons.GetElement(XPConvert.ToString(_nClassID));
	}

	/**
	 * Returns a String of Class IDs and Object IDs of the Objects
	 * to select in the tree.
	 * E.g.: "classID1,ObjID1,classID2,objID2"
	 *
	 * This is the format used by the tree.
	 *
	 * @return String
	 */
	public String GetSelectedItems()
	{
		//SE: logging
		log.FunctionBegin("SelectPTObjectModel::GetSelectedItems");
		
		// SE: Bug 57861 - use StringBuilder to avoid CPU spike
		XPStringBuilder sbSelectedItems = new XPStringBuilder(6 * m_saData.GetCount());

		// JF- We loop through the array of items and for each one, 
		// add the class ID and the object ID to the String.
		for (int i = 0; i < m_saData.GetCount(); i++)
		{
			// JS- Add the separator
			if (i > 0)
			{
				// SE: Bug 57861 - use StringBuilder to avoid CPU spike
				sbSelectedItems.Append(",");
			}

			// SE: Bug 57861 - use StringBuilder to avoid CPU spike
			sbSelectedItems.Append(m_saData.GetItem(i, EXPLIST_COL_CLASSID));
			sbSelectedItems.Append(",");
			sbSelectedItems.Append(m_saData.GetItem(i, EXPLIST_COL_OBJECTID));
		}

		//SE: logging
		log.FunctionEnd("SelectPTObjectModel::GetSelectedItems");
		// SE: Bug 57861 - use StringBuilder to avoid CPU spike
		return sbSelectedItems.ToString();
	}

	/**
	 * Helper method to get a String with the selected Object names.
	 * E.g.: User1, User2, Group1
	 * Will return an empty String if no Object was selected.
	 *
	 * @return HTMLElementCollection
	 */
	public HTMLElementCollection GetSelectedNamesAsHTML()

	{
		//SE: logging
		log.FunctionBegin("SelectPTObjectModel::GetSelectedNamesAsHTML");
		
		HTMLElementCollection result = new HTMLElementCollection();
		HTMLB myBold;

		String sName;

		for (int i = 0; i < m_saData.GetCount(); i++)
		{
			sName = (String) m_saData.GetItem(i, EXPLIST_COL_STRING);

			if ((null != sName) && (0 != sName.length()))
			{
				myBold = new HTMLB();
				myBold.AddInnerHTMLString(XPHttpUtility.HtmlEncode(sName));
				result.AddInnerHTMLElement(myBold);
				result.AddInnerHTMLElement(new HTMLBr());
			}
		}

		//SE: logging
		log.FunctionEnd("SelectPTObjectModel::GetSelectedNamesAsHTML");
		return result;
	}

	/**
	 * Returns the leaf type of the tree.
	 *
	 * @return int
	 */
	public int GetTreeLeafType()
	{
		return m_treeLeafType;
	}

	/**
	 * Returns the selection type of the tree.
	 * Converts the boolean value to a string so that it can be
	 * put in the query string for the tree.
	 *
	 * @return String
	 */
	public String GetTreeMultiSelect()
	{
		return XPConvert.ToString(m_bTreeMultiSelect);
	}

	/**
	 * Returns the query filter for the tree.
	 *
	 * @return String
	 */
	public String GetTreeQueryFilter()
	{
		return m_strTreeQueryFilter;
	}

	/**
	 * Returns tree AS ID.
	 *
	 * @return String
	 */
	public String GetTreeSpaceID()
	{
		return m_sTreeSpaceID;
	}
	
	/**
	 * Resets the tree ID, this way a new tree will be opened. 
	 * This can be used to switch from a multi select tree to a single select tree for example.
	 *
	 */
	public void ResetTreeID()
	{
		m_sTreeSpaceID = "";
	}

	/**
	 * Returns the Tree sub title.
	 *
	 * @return String
	 */
	public String GetTreeSubTitle()
	{
		return m_sTreeSubTitle;
	}

	/**
	 * Returns the Tree title.
	 *
	 * @return String
	 */
	public String GetTreeTitle()
	{
		return m_sTreeTitle;
	}

	/**
	 * Initializes the read only expandable list.
	 *
	 * @param _sSectionTitle Main title.
	 * @param _sSectionSubTitle Sub title
	 * @param _arrObjectClassIDs Class ID of the Objects displayed on the list.
	 * @param _nListKey In case you have multiple Expandable Lists on
	 * the same DP, they should have a different key (otherwise, just set it to 1).
	 */
	public void InitializeReadOnlySelectPTObject(String _sSectionTitle,
		String _sSectionSubTitle, int[] _arrObjectClassIDs, int _nListKey)
	{
		Object[][] arrExpListColInfo = new Object[1][5];
		arrExpListColInfo[0][EXPLIST_COLUMN_ID]			    = new Integer(EXPLIST_COL_STRING);
		arrExpListColInfo[0][EXPLIST_COLUMN_TITLE]		    = _sSectionSubTitle;
		arrExpListColInfo[0][EXPLIST_COLUMN_WIDTH]		    = "100%";
		arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN]	    = "1";
		arrExpListColInfo[0][EXPLIST_COLUMN_ENABLESORT]     = Boolean.TRUE;

		m_arrObjectClassIDs = _arrObjectClassIDs;

		super.InitializeReadOnlyExpandableList(_sSectionTitle,
			arrExpListColInfo, _nListKey);
	}

	/**
	 * Initializes the expandable list.
	 *
	 * @param _sSectionTitle Main title.
	 * @param _sSectionSubTitle Sub title
	 * @param _imgAddIcon Icon for the add action.
	 * @param _sAddLabel Text to display next to the add icon.
	 * @param _arrObjectClassIDs Class ID of the Objects displayed on the list.
	 * @param _nListKey In case you have multiple Expandable Lists on
	 * the same DP, they should have a different key (otherwise, just set it to 1).
	 */
	public void InitializeSelectPTObject(String _sSectionTitle,
		String _sSectionSubTitle, PTImageType _imgAddIcon, String _sAddLabel,
		int[] _arrObjectClassIDs, int _nListKey)
	{
		Object[][] arrExpListColInfo = new Object[1][5];
		arrExpListColInfo[0][EXPLIST_COLUMN_ID]			    = new Integer(EXPLIST_COL_STRING);
		arrExpListColInfo[0][EXPLIST_COLUMN_TITLE]		    = _sSectionSubTitle;
		arrExpListColInfo[0][EXPLIST_COLUMN_WIDTH]		    = "100%";
		arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN]	    = "1";
		arrExpListColInfo[0][EXPLIST_COLUMN_ENABLESORT]     = Boolean.TRUE;

		m_arrObjectClassIDs = _arrObjectClassIDs;

		super.InitializeExpandableList(_sSectionTitle, arrExpListColInfo,
			_imgAddIcon, _sAddLabel, _nListKey);
	}
	
	/**
	 * Partialally initializes the expandable list. InitializeExpandableList must also be called when this method is used.
	 * 
	 * @param _arrObjectClassIDs Class ID of the Objects displayed on the list.
	 */
	public void InitializeListClassIDs(int[] _arrObjectClassIDs)
	{
		m_arrObjectClassIDs = _arrObjectClassIDs;
	}

	/**
	 * Initalizes the list (PTGrowableSortedArrayWrapper)
	 * with a column mapping and an array of sort.
	 * The growable array will always be sorted by the 1st
	 * column ascending.
	 *
	 * @param _saData
	 */
	public void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData)
	{
		//SE: logging
		log.FunctionBegin("SelectPTObjectModel::InitializeSortedArray");
		
		// JF- Column mapping
		int[] arrColumns = new int[]
			{
				EXPLIST_SORTEDARRAY_PROPID_INDEX, EXPLIST_COL_CLASSID,
				EXPLIST_COL_OBJECTID, EXPLIST_COL_STRING
			};

		// JF- Sort array
		// The growable array will always be sorted by the String column (the one displayed).
		int[][] arrSort = new int[1][2];
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		  = EXPLIST_COL_STRING;
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;

		// JF- We don't initalize the list itself (null below)
		// At first it's empty and we will be initalized if needed using the 
		// AddRowsToList method.
		_saData.Initialize(arrColumns, null, arrSort, true,
			EXPLIST_DEFAULT_INITIAL_SIZE);
		//SE: logging
		log.FunctionEnd("SelectPTObjectModel::InitializeSortedArray");
	}

	/**
	 * Returns true if it should be possible to edit Objects from this class ID.
	 *
	 * @param _nClassID
	 */
	public boolean IsEditLinkEnabled(int _nClassID)
	{
		if (null != m_arrCreateNewInfo)
		{
			int nClassIDFromArray;

			for (int i = 0; i < m_arrCreateNewInfo.length; i++)
			{
				nClassIDFromArray = XPConvert.ToInteger(m_arrCreateNewInfo[i][CREATE_NEW_COL_CLASSID]);

				if (nClassIDFromArray == _nClassID)
				{
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * Sets the nAccessLevel. The accesslevel of the objects for the tree control to display.
	 * @param nAccessLevel The nAccessLevel to set
	 */
	public void SetAccessLevel(int nAccessLevel)
	{
		m_nAccessLevel = nAccessLevel;
	}

	/**
	 * Sets the community mode for the tree.
	 * By default there is no community mode.
	 *
	 * @param _nCommunityMode
	 */
	public void SetCommunityMode(int _nCommunityMode)
	{
		m_nCommunityMode = _nCommunityMode;
	}

	/**
	 * Sets the community folder id for the tree.
	 * By default there is no community folder id.
	 *
	 * @param _nCommunityFolderID
	 */
	public void SetCommunityFolderID(int _nCommunityFolderID)
	{
		m_nCommunityFolderID = _nCommunityFolderID;
	}

	/**
	 * Add the array passed in argument to the list.
	 * The array should be created using the EXPLIST_COL
	 * constants defined in this class.
	 *
	 * @param _arrRows Array of rows to add.
	 */
	public void SetDefaultObjects(Object[][] _arrRows)
	{
		//SE: logging
		log.FunctionBegin("SelectPTObjectModel::SetDefaultObjects");
		
		super.GetItems().AddRows(_arrRows);
		
		//SE: logging
		log.FunctionEnd("SelectPTObjectModel::SetDefaultObjects");
	}

	/**
	 * Enables the create new links for the Class ID in argument.
	 * Doing this will also enbale the links to Edit the Objects in the list.
	 *
	 * @param _arrCreateNewInfo Information for each create new link
	 */
	public void SetEnableCreateNew(Object[][] _arrCreateNewInfo)
	{
		if (_arrCreateNewInfo[0].length < CREATE_NEW_COL_MAX)
		{
			throw new XPException(
				"SelectPTObjectModel::SetEnableCreateNew The create new information array" +
				" should contain " + CREATE_NEW_COL_MAX + " columns.");
		}

		m_arrCreateNewInfo = _arrCreateNewInfo;
	}

	/**
	 * Because this select ptobject list needs to support both move
	 * row actions enable and disable, we need to change the
	 * colspan of the header row when this happens.
	 *
	 * @param _bEnableMoveRowActions
	 */
	public void SetEnableMoveRowActions(boolean _bEnableMoveRowActions)
	{
		super.SetEnableMoveRowActions(_bEnableMoveRowActions);

		if (m_bEnableMoveRowActions)
		{
			// JF- If the move row actions are enabled, this will 
			// create one more cell at the end of each row and 
			// we need to reflect that change on the header row.
			m_arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN] = "2";
		}
		else
		{
			m_arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN] = "1";
		}
	}

	/**
	 * Clears the hidden objects for the tree.
	 *
	 */
	public void ClearHiddenObjects()
	{
		m_strObjectsToHide = "";
	}
	
	/**
	 * Sets the hidden objects for the tree.
	 * By default there are no hidden objects.
	 *
	 * @param _anClassIDObjectID
	 */
	public void SetHiddenObjects(int[][] _anClassIDObjectID)
	{
		//SE: logging
		log.FunctionBegin("SelectPTObjectModel::SetHiddenObjects");
		
		XPStringBuilder sb = new XPStringBuilder(6 * _anClassIDObjectID.length);

		for (int i = 0; i < _anClassIDObjectID.length; i++)
		{
			if (i > 0)
			{
				sb.Append(",");
			}

			sb.Append(_anClassIDObjectID[i][HIDE_COL_CLASSID]).Append(",")
			.Append(_anClassIDObjectID[i][HIDE_COL_OBJECTID]);
		}
		
		if (0 != m_strObjectsToHide.length())
		{
			m_strObjectsToHide = m_strObjectsToHide + "," + sb.ToString();
		}
		else
		{
			m_strObjectsToHide = sb.ToString();
		}
		
		//SE: logging
		log.FunctionEnd("SelectPTObjectModel::SetHiddenObjects");
	}
	
	/**
	 * Sets the read only objects.
	 * Those object won't appear in the tree and the user will not be able to delete them.
	 *
	 * @param _arrReadOnlyObjects
	 */
	public void SetReadOnlyObjects(int[][] _arrReadOnlyObjects)
	{
		m_arrReadOnlyObjects = _arrReadOnlyObjects;
		SetHiddenObjects(m_arrReadOnlyObjects);
	}
	
	/**
	 * Returns true if the object identified by the class id 
	 * and the object id in argument is read only.
	 *
	 * @param _nClassID
	 * @param _nObjectID
	 * 
	 * @return boolean
	 */
	public boolean IsObjectReadOnly(int _nClassID, int _nObjectID)
	{
		boolean bIsReadOnly = false;
		
		if (null != m_arrReadOnlyObjects)
		{
		
			for (int i = 0; i < m_arrReadOnlyObjects.length; i ++)
			{
				if ((_nClassID == m_arrReadOnlyObjects[i][READ_ONLY_ROW_CLASSID]) 
					&& (_nObjectID == m_arrReadOnlyObjects[i][READ_ONLY_ROW_OBJECTID]))
				{
					bIsReadOnly = true;	
					break;
				}	
			}
		}
		
		return bIsReadOnly;
	}
	
	/**
	 * Sets the array of objects to display in the tree. 
	 * (Only those objects will be display in the tree.)
	 * The array in argument should be in the following format:
	 * classid1,objectid1,classid2,objectid2....
	 * 
	 * @param _sDisplayObjects
	 */
	public void SetDisplayObjects(String _sDisplayObjects)
	{
		m_sDisplayObjects = _sDisplayObjects;
	}
	
	/**
	 * Returns the array of objects to display in the tree.
	 * 
	 * @param String
	 */
	public String GetDisplayObjects()
	{
		return m_sDisplayObjects;
	}

	/**
	 * Returns the value of the m_bNoRowIcons member variable.  True if no
	 * icons are to be displayed on each selected row.
	 *
	 * @return boolean
	 */
	public boolean GetNoRowIcons() {
		return m_bNoRowIcons;
	}

	/**
	 * Sets the m_bNoRowIcons member variable.  True if no icons are to be
	 * displayed on each selected row.
	 *
	 * @param m_bNoRowIcons
	 */
	public void SetNoRowIcons(boolean m_bNoRowIcons) {
		this.m_bNoRowIcons = m_bNoRowIcons;
	}

	/**
	 * Sets the icon to display on each row before the
	 * input text.
	 * By default, no icon is displayed.
	 *
	 * @param _imgRowIcon
	 */
	public void SetDefaultRowIcon(PTImageType _imgRowIcon)
	{
		m_imgDefaultRowIcon = _imgRowIcon;
	}

	/**
	 * Sets the icon to display on each row before the
	 * input text, given a class id.  By default, all
	 * class ids are associated with empty images. Call
	 * SetDefaultRowIcon to set a universal default icon.
	 *
	 * @param _nClassID the class id to be associated with this icon
	 * @param _imgRowIcon the icon you want to be displayed
	 */
	public void SetRowIconByClassID(int _nClassID, PTImageType _imgRowIcon)
	{
		// RWW: since we do lazy get/sets to save memory
		// in init, we need to make sure we have an object
		if (null == m_xphtRowIcons)
			m_xphtRowIcons = new XPHashtable();
			
		// SE: Bug 57861 - use XPConvert for speed
		m_xphtRowIcons.PutElement(XPConvert.ToString(_nClassID), _imgRowIcon);
	}

	/**
	 * Sets the leaf type for the tree.
	 * Is set to N_LEAFSELECT by default.
	 *
	 * @param _leafType
	 */
	public void SetTreeLeafType(int _leafType)
	{
		m_treeLeafType = _leafType;
	}

	/**
	 * Sets the selection type for the tree.
	 * Multiselect is set to true by default.
	 *
	 * @param _selectType
	 */
	public void SetTreeMultiSelect(boolean _selectType)
	{
		m_bTreeMultiSelect = _selectType;
	}

	/**
	 * Sets the query filter for the tree.
	 * By default there is no query filter.
	 *
	 * @param _nClassID
	 * @param _nPropID
	 * @param _anValues
	 */
	public void SetTreeQueryFilter(int _nClassID, int _nPropID, int[] _anValues)
	{
		XPStringBuilder sb = new XPStringBuilder(20);
		sb.Append(_nClassID).Append(",").Append(_nPropID);

		for (int i = 0; i < _anValues.length; i++)
		{
			sb.Append(",").Append(_anValues[i]);
		}

		m_strTreeQueryFilter = sb.ToString();
	}

	/**
	 * Sets the Tree sub title.
	 *
	 * @param _sTreeSubTitle
	 */
	public void SetTreeSubTitle(String _sTreeSubTitle)
	{
		m_sTreeSubTitle = _sTreeSubTitle;
	}

	/**
	 * Sets the Tree title.
	 *
	 * @param _sTreeTitle
	 */
	public void SetTreeTitle(String _sTreeTitle)
	{
		m_sTreeTitle = _sTreeTitle;
	}

	/**
	 * Saves the data on the row _nRowIndex.
	 *
	 * @param _saData List to update the data on row _nRowIndex
	 * @param _nRowIndex Index of the row to save.
	 * @param _htFormData Data posted by the form.
	 * @return int Status for server side validation.
	 */
	protected int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex,
		XPHashtable _htFormData)
	{
		// JF- Nothing to save on repost.
		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * Helper method to save the Tree ID on repost.
	 *
	 * @param _htFormData Form data
	 */
	protected void SaveTreeID(XPHashtable _htFormData)
	{
		// JF- We save the tree ID only if we don't already have it.
		if (("").equals(m_sTreeSpaceID) &&
				_htFormData.ContainsKey(GetUniqueString(
						SelectPTObjectView.HTMLINPUT_TREEASID)))
		{
			String[] data = (String[]) _htFormData.GetElement(GetUniqueString(
						SelectPTObjectView.HTMLINPUT_TREEASID));

			if ((null != data) && !(0 == data.length))
			{
				m_sTreeSpaceID = data[0];
			}
		}
	}

	/**
	 * Helper method to create a temporary array of index mapping.
	 * On each row we have the following info: class ID - Object ID - index
	 * This method is used to not loose the order when the move row actions are enabled.
	 *
	 * @param _saItems List items
	 *
	 * @return Object[][]
	 */
	protected Object[][] CreateIndexMapping(PTGrowableSortedArrayWrapper _saItems)
	{
		Object[][] arrIndexMapping = new Object[_saItems.GetCount()][3];

		for (int i = 0; i < _saItems.GetCount(); i++)
		{
			if (null != _saItems.GetItem(i, EXPLIST_SORTEDARRAY_PROPID_INDEX))
			{
				arrIndexMapping[i][0]     = _saItems.GetItem(i,
						EXPLIST_COL_CLASSID);
				arrIndexMapping[i][1]     = _saItems.GetItem(i,
						EXPLIST_COL_OBJECTID);
				arrIndexMapping[i][2]     = _saItems.GetItem(i,
						EXPLIST_SORTEDARRAY_PROPID_INDEX);
			}
		}

		return arrIndexMapping;
	}

	/**
	 * Helper method to get the EditorAS object name from the class ID.
	 *
	 * @param _nClassID
	 *
	 * @return String
	 */
	private String GetEditorASName(int _nClassID)
	{
		String sASName		  = null;
		int nClassIDFromArray;

		for (int i = 0; i < m_arrCreateNewInfo.length; i++)
		{
			nClassIDFromArray = XPConvert.ToInteger(m_arrCreateNewInfo[i][CREATE_NEW_COL_CLASSID]);

			if (nClassIDFromArray == _nClassID)
			{
				sASName = (String) m_arrCreateNewInfo[i][CREATE_NEW_COL_ASNAME];

				break;
			}
		}

		return sASName;
	}

	/**
	 * Helper method to get the value of the index for the object identified
	 * by the object ID and class ID in argument.
	 * Returns -1 if the object was not in the array.
	 *
	 * @param _arrIndexMapping Index mapping
	 * @param _nClassID Object class ID
	 * @param _nObjectID Object ID
	 *
	 * @return int Value of the index
	 */
	protected int GetIndex(Object[][] _arrIndexMapping, int _nClassID,
		int _nObjectID)
	{
		int nIndex		 = -1;
		int nRowClassID;
		int nRowObjectID;

		for (int i = 0; i < _arrIndexMapping.length; i++)
		{
			if ((null != _arrIndexMapping[i][0]) &&
				(null != _arrIndexMapping[i][1]) &&
				(null != _arrIndexMapping[i][2]))
			{
				nRowClassID		 = XPConvert.ToInteger(_arrIndexMapping[i][0]);
				nRowObjectID     = XPConvert.ToInteger(_arrIndexMapping[i][1]);

				if ((_nClassID == nRowClassID) && (_nObjectID == nRowObjectID))
				{
					nIndex = XPConvert.ToInteger(_arrIndexMapping[i][2]);

					break;
				}
			}
		}

		return nIndex;
	}
	
}

