package com.plumtree.portaluiinfrastructure.filter;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.server.*;
import com.plumtree.server.impl.users.*;
import com.plumtree.uiinfrastructure.activityspace.*;

/**
 * @author CraigS, based on MikeS code
 */
public class FilterIterator implements IFilterIteratorRO
{
	
	public static final String STR_MVC_CLASS_NAME = "FilterIterator";
	private AActivitySpace m_asOwner;
	private FlattenedFilter _flattenedFilter;
	private IASQueryResult _propertyList = null;
	private IPTFilter _filter;
	private IPTSession _ptSession;
	private QueryCondition _queryCondition = null;

	private boolean _currentClauseUsesAND;
	private int _currentClauseDepth;
	private int _currentIndexPosition;
	
	// aj: this ht defines the properties that a user can choose from,
	//     irrespective of ACLs, etc. (that gets checked later).  it 
	//     is primarily used by the Dynamic Membership Rule chooser.  it
	//     is null by default and whenever in a "show all" mode...
	private XPHashtable m_htAllowablePropertyIDs = null;
	
	// aj: this boolean toggles including group memberships as a property,
	//     should be false by default and manually turned on by client code
	boolean m_bIncludeGroupMembership = false;

	/**
	 * Creates a new FilterIterator object.
	 *
	 * @param session 
	 * @param filter 
	 * @param owner 
	 */
	public FilterIterator(IPTSession session, IPTFilter filter,
		AActivitySpace owner)
	{
		_ptSession     = session;
		m_asOwner     = owner;
		InitFromFilter(filter);
	}

	/**
	 * Manufactures a QueryCondition
	 * @return a QueryCondition
	 */
	public QueryCondition MakeQueryCondition()
	{
		if (null == _queryCondition)
		{
			// Create a query condition
			_queryCondition = new QueryCondition(m_asOwner, _ptSession);

			// Get all properties the user can see
			IASQueryResult properties = GetPropertiesVisibleToCurrentUser();

			// Seed the condition 
			for (int i = 0; i < properties.GetCount(); i++)
			{
				// Hide the OBJECTIMAGEUUID (property 7) from the user
				int ObjID;
				
				// aj: we need to lookup all properties the user has access to in the
				//     HT of allowable properties.  If that HT is null, then just go 
				//     ahead and add the property to the chooser...
				if(m_htAllowablePropertyIDs != null)
				{
					Object oPropID = m_htAllowablePropertyIDs.GetElement(
							properties.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID));
					
					// aj: continue if not in HT
					if(oPropID == null){
						continue;
					} else {
						ObjID = XPConvert.ToInteger(oPropID);
					}
				} else {
					ObjID = XPConvert.ToInteger(properties.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID));
				}
				
				// aj: disallow OBJECTIMAGEUUID...
				if (7 != ObjID) {
					_queryCondition.AddProperty(
							XPConvert.ToString(properties.GetFields(i, PT_PROPIDS.PT_PROPID_NAME)),
							ObjID,
							XPConvert.ToInteger(properties.GetFields(i, PT_PROPIDS.PT_PROPID_PROP_VALUETYPE)));
				}
			}
			
			// aj: check group membership special case
			if(m_bIncludeGroupMembership)
			{
				_queryCondition.AddProperty(
						m_asOwner.GetString(500, "ptmsgs_portalinfrastructure"), 
						PTUserGroup.MEMBER_OF_PROP_ID, 
						PT_PROPERTY_TYPES.PT_PROPTYPE_MEMBERSHIP);
				
				// aj: bug 44237, also make sure the proper operators are shown
				//     for dynamic groups vs. search query conditions...
				_queryCondition.SetIsDynamicGroupView(true);
			}
			
		}

		return _queryCondition;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public int GetCurrentClauseDepth()
	{
		return _currentClauseDepth;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public boolean GetCurrentClauseOperator()
	{
		return _currentClauseUsesAND;
	}

	/**
	 * Gets the filter's top-level search string.
	 */
	public String GetTextSearchString()
	{
		return _filter.GetSearchString();
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public boolean GetNextClausePosition()
	{
		if (_currentIndexPosition >= _flattenedFilter.GetNumItems())
		{
			return false;
		}

		IPTPropertyFilterItem currItem = _flattenedFilter.GetItem(_currentIndexPosition);

		// this shouldn't really happen.  If it does we've messed up.
		if (PT_FILTER_ITEM_TYPES.PT_FILTER_ITEM_STATEMENT == currItem.GetType())
		{
			throw new XPException(
				"in FilterIterator.GetNextClausePosition, we should not be positioned on a statement");
		}

		IPTPropertyFilterClauses clause = (IPTPropertyFilterClauses) currItem;

		_currentClauseUsesAND     = (PT_BOOLOPS.PT_BOOLOP_OR != clause.GetOperator());
		_currentClauseDepth		 = _flattenedFilter.GetItemDepth(_currentIndexPosition);
		_currentIndexPosition++;

		return true;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public IPTPropertyFilterStatement GetNextStatement()
	{
		// if we've gone beyond the end
		if (_currentIndexPosition >= _flattenedFilter.GetNumItems())
		{
			return null;
		}

		// get current item
		IPTPropertyFilterItem currItem = _flattenedFilter.GetItem(_currentIndexPosition);

		// if clause
		if (PT_FILTER_ITEM_TYPES.PT_FILTER_ITEM_CLAUSES == currItem.GetType())
		{
			// end of current clause; return null
			return null;
		}

		// next position
		_currentIndexPosition++;

		// cast to its underlying type and return it
		return (IPTPropertyFilterStatement) currItem;
	}

	/**
	 * 
	 *
	 * @param qc 
	 *
	 * @return 
	 */
	public boolean GetNextStatementAndSetQC(QueryCondition qc)
	{
		// get the statement itself
		IPTPropertyFilterStatement statement = GetNextStatement();

		// if something went wrong
		if (null == statement)
		{
			return false;
		}
		
		if (null == qc)
		{
			qc = MakeQueryCondition();
		}

		qc.SetVisible(true);

	    int propID = statement.GetOperand();

	    if (PT_PROPERTY_TYPES.PT_PROPTYPE_DATE == qc.GetTypeExpectedForProperty(propID)) 
	    {
	        int opID = statement.GetOperator();
	        if ( (PT_FILTEROPS.PT_FILTEROP_RELATIVE_LT == opID) || (PT_FILTEROPS.PT_FILTEROP_RELATIVE_LTE == opID) ||
                 (PT_FILTEROPS.PT_FILTEROP_RELATIVE_GT == opID) || (PT_FILTEROPS.PT_FILTEROP_RELATIVE_GTE == opID) )
	        {
	  	      // could be String or Integer.. just pass in the value as a string
	  	      qc.SetSelections(propID, statement.GetOperator(), XPConvert.ToString(statement.GetValueAsObject()));
	        }
	        else
	        {
		      // Expect date type.. check whether we've been given a string or a XPDateTime
		
		      // Have to initialize this to something non-null for jumpability
		      XPDateTime dtValue = new XPDateTime();
		      boolean gotDate = false;
		      try{
		        dtValue = statement.GetValueAsXPDateTime();
		        gotDate = true;
		      }catch (Exception e){
		        gotDate = false;
		      }
		
		      if (gotDate) {
		        // It's a date/time.. pass it into QueryCondition directly
		        qc.SetSelections(propID,
		                         statement.GetOperator(),
		                         dtValue);
		      } else {
		        // Not a XPDateTime.  Try getting it as a String and assuming it's
		        // a number of milliseconds since 1970, which is what
		        // FilterForm.EncodeFilterAsControlArgs does, then converting it
		        // to XPDateTime.
		
		        String strValue = statement.GetValueAsString();
		
		        if ((null == strValue) ||
		            (0 == strValue.length())) {
		          // No value, just pass it in.
		          qc.SetSelections(propID,
		                           statement.GetOperator(),
		                           strValue);
		        }
		
		        long ticks = 0;
		        boolean gotTicks = false;
		        try {
		          ticks = XPConvert.ToLong(strValue);
		          gotTicks = true;
		        } catch (Exception e) {
		          gotTicks = false;
		        }
		
		        if (gotTicks) {
		          qc.SetSelections(propID,
		                           statement.GetOperator(),
		                           new XPDateTime(ticks));
		        } else {
		          // Oh well, we tried.  Just use the string.
		          qc.SetSelections(propID,
		                           statement.GetOperator(),
		                           strValue);
		        }
		      }
	        }	
	    } 
	    else if(PT_PROPERTY_TYPES.PT_PROPTYPE_MEMBERSHIP == qc.GetTypeExpectedForProperty(propID))
	    {
	    	String sGroupID = XPConvert.ToString(statement.GetValueAsObject());
	    	qc.SetSelections(propID, statement.GetOperator(), sGroupID);
		} 
	    else 
	    {
	      // Expect string.. just pass in the value as a string directly
	      qc.SetSelections(propID, statement.GetOperator(), statement.GetValueAsString());
	    }

		return true;
	}

	public boolean AddSetProperty(QueryCondition qc)
	{
	    /* user is expected to have access to this property via the filter, but doesn't actually have
	     * access to it.  in this case, we show the property read-only. */
	    int iStmtPropID = qc.GetStatementProperty();
	    if (!qc.ContainsProperty(iStmtPropID))
	    {
	        IPTQueryResult qrPropInfo = _ptSession.GetProperties().QuerySingleObject(iStmtPropID);

	        if (qrPropInfo.RowCount()>0)
	        {
		        qc.AddProperty(
		                qrPropInfo.ItemAsString(0,PT_PROPIDS.PT_PROPID_NAME), 
		                iStmtPropID,
		                qrPropInfo.ItemAsInt(0,PT_PROPIDS.PT_PROPID_PROP_VALUETYPE));
		        
		        return true;
	        }
	    }
	    
	    return false;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public int GetNumItemsInFirstClause()
	{
		try
		{
			IPTPropertyFilterClauses clause = (IPTPropertyFilterClauses) _filter.GetPropertyFilter();

			if (null != clause)
			{
				return clause.GetCount();
			}
			else
			{
				return 0;
			}
		}
		catch (Exception e)
		{
			return 0;
		}
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public IASQueryResult GetPropertiesVisibleToCurrentUser()
	{
		if (null == _propertyList)
		{
			int nFields = PT_PROPIDS.PT_PROPID_OBJECTID |
				PT_PROPIDS.PT_PROPID_NAME |
				PT_PROPIDS.PT_PROPID_PROP_VALUETYPE;

			PTObjectManagerQueryWrapper PropMgr = new PTObjectManagerQueryWrapper(_ptSession.GetProperties());

			Object[][] ptqf;
			
			ptqf = new Object[3][2];
			ptqf[0][0]     = new Integer(PT_PROPIDS.PT_PROPID_PROP_SEARCHABLE);
			ptqf[1][0]     = new Integer(PT_FILTEROPS.PT_FILTEROP_GT);
			ptqf[2][0]     = new Integer(0);
			ptqf[0][1]     = new Integer(PT_PROPIDS.PT_PROPID_ACCESSLEVEL);
			ptqf[1][1]     = new Integer(PT_FILTEROPS.PT_FILTEROP_GTE);
			ptqf[2][1]     = new Integer(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT);

			PropMgr.Query(nFields, -1, null, ptqf);
			_propertyList = (IASQueryResult) PropMgr;
		}

		return _propertyList;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public boolean GetTopLevelAllMustPass()
	{
		return (PT_BOOLOPS.PT_BOOLOP_AND == _filter.GetOperator());
	}

	/**
	 * 
	 *
	 * @param filter 
	 */
	public void InitFromFilter(IPTFilter filter)
	{
		_filter = filter;
		_flattenedFilter = new FlattenedFilter(filter);

		SetIteratorToStart();
	}

	/**
	 * 
	 */
	public void SetIteratorToStart()
	{
		_currentIndexPosition		 = 0;
		_currentClauseUsesAND     = false;
		_currentClauseDepth		 = 0;
	}
	
	/**
	 * Allows the Filter Iterator to only show those properties
	 * specified.  Default behavior is to show all properties the
	 * current user has access to.
	 * 
	 * @param iArAllowablePropertyIDs - an array of allowed property 
	 * 		ids as Integers.  pass in a null array to allow all 
	 * 		properties (default)
	 */
	public void SetAllowablePropertyIDs(Object[] iArAllowablePropertyIDs)
	{
		// aj: check input, if null or empty set HT to null
		if(iArAllowablePropertyIDs == null || iArAllowablePropertyIDs.length == 0)
		{
			// aj: show all properties
			m_htAllowablePropertyIDs = null;
		} else {
			// aj: create the HT if none existed
			if(m_htAllowablePropertyIDs == null)
			{
				m_htAllowablePropertyIDs = new XPHashtable(iArAllowablePropertyIDs.length);
			} else {
				// aj: clear the existing HT
				m_htAllowablePropertyIDs.Clear();
			}
			 
			for(int i=0; i<iArAllowablePropertyIDs.length; i++)
			{
				m_htAllowablePropertyIDs.PutElement(iArAllowablePropertyIDs[i], iArAllowablePropertyIDs[i]);
			}
		}
		
		// aj: clear this prop list, or else it will be stale
		_propertyList = null;
	}
	
	public void SetIncludeGroupMembership(boolean bIncludeMembership)
	{
		m_bIncludeGroupMembership = bIncludeMembership;
	}
}

