package com.plumtree.portaluiinfrastructure.friendlyurl;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.*;
import com.plumtree.portaluiinfrastructure.helpers.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This class represents a friendly Url and provides an interface to generate 
 * friendly Url string based on the provided information. 
 * 
 * @author rozhang
 */
public class FriendlyURL 
{
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_FRIENDLYURL),
			"com.plumtree.portaluiinfrastructure.friendlyurl.FriendlyURL"); 
	
	/**
	 * Type of destination page the url links too
	 */
	protected int m_classId		= 0;
	
	/**
	 * objectName Name of the object to generate friendly name to (either one of name or id must be provided)
	 */
	protected String m_objectName = null;
	
	/**
	 * Id of the object to generate friendly name to (either one of name or id must be provided)
	 */
	protected int m_objectId		= 0;
	
	/**
	 * String of query parameters that need to be attached at the end of the friendly URL
	 */
	protected String m_queryParams = null;
	
	/**
	 * Additional name specifier, used for example for Urls to community pages which also needs the community name. Optional and not used by most class types
	 */
	protected String m_childName	= null;
	
	/**
	 * Additional id specifier, used for example for Urls to community pages which also needs the community name. Optional and not used by most class types
	 */
	protected int m_childId 		= 0; 
	
	/**
	 * Whether to generate the object id in the URL if the name is also available
	 */
	protected boolean m_useIdIfAvailable = false;
	
	
	
	/**
	 * Constructor with object name as argument
	 * 
	 * @param classId
	 */
	public FriendlyURL(int classId, String objectName) 
	{
		this(classId, objectName, 0, null);
	}
	
	public FriendlyURL(int classId, String objectName, int objectId) 
	{
		this(classId, objectName, objectId, null);
	}
	
	/**
	 * Constructor with object id as argument
	 * 
	 * @param classId
	 */
	public FriendlyURL(int classId, int objectId) 
	{
		this(classId, null, objectId, null);
	}
	
	public FriendlyURL(int classId, String objectName, int objectId, String queryParams) 
	{
		if ( log.IsDebugEnabled() ) { log.Debug("Creating friendly Url object with classid: "+classId+" objectid: "+objectId+"and name: "+objectName); }
		m_classId = classId;
		m_objectName = objectName;
		m_objectId = objectId;
		if (null != queryParams && XPStringUtility.StartsWith(queryParams, "&")) {
			m_queryParams = XPStringUtility.Substring(queryParams, 1);
		} else {
			m_queryParams = queryParams;
		}
	}
	
	/**
	 * Constructs the Friendly URL and returns it as a string. Only the friendly
	 * url part is returned, no the application base url part or server.pt/
	 * Note that any application base url used with friendly url cannot have a 
	 * trailing question mark.
	 * 
	 * @param space	Current ActivitySpace
	 * @return For example /community/engineeringcommunity/page2
	 */
	public String GetFriendlyURL(AActivitySpace space)
	{
		String friendlyURL = ((IFriendlyURLManager) PTConfigService.getPTInstance().GetPTClassTypeDescrManager()).ConstructFriendlyURL(this, space);
		if (null != friendlyURL && null != m_queryParams && !m_queryParams.equals("")) {
			friendlyURL = friendlyURL + "?" + m_queryParams;
		}
		return friendlyURL;
	}
	
	/**
	 * Constructs the Friendly URL and returns it as a string. The application base url 
	 * (without trailing question mark) is included.
	 * 
	 * @param space	Current ActivitySpace
	 * @return For example http://portalserver:8080/portal/server.pt/community/engineeringcommunity/page2
	 */
	public String GetFullFriendlyURL(AActivitySpace space)
	{
		String baseURL = PTConfigService.getPTInstance().GetApplicationBaseURL(space.GetCurrentNormalizedBaseURL());
		baseURL = GeneralHelpers.Replace(baseURL, "?", "");
		String friendlyURL = GetFriendlyURL(space);
		if (null != friendlyURL) {
			return baseURL+friendlyURL;
		} else {
			return null;
		}
	}

	/**
	 * Convert PTObject name into a format that is suitable for friendly Urls.
	 * 
	 * Use <code>FriendlyURL#DeNormalizeObjectName</code> to convert the value back.
	 * The casing of the object name is not preserved through the conversion    
	 * 
	 * NOTE: This method is duplicated on ObjectIdFromNameCache class in the server. Make sure to update
	 * both methods.
	 */
	public static String NormalizeObjectName(String objectName)
	{
		// TODO CR DLH: Shouldn't we just do this using GeneralHelpers.Replace,
		// since it's cleaner?  Then we can worry about optimizations if a code
		// profiler shows that this is slow?
		// ROB what's wrong with writing optimized code in the first place?
		// creating extra String are always wasteful and this is scales if 
		// we need additional replacements 
		// DLH: In this case it's fairly okay, although often optimized code is harder
		// to read and maintain.  Any optimized code should have a comment describing
		// what it's doing, since it's almost always harder to read than just
		// String.replace(' ', '_');  At any rate, please add a descriptive comment.
		XPStringBuilder sb = new XPStringBuilder();
		char[] chars = XPStringUtility.GetChars(objectName);
		for (int i=0; i<chars.length; i++) {
			char c = chars[i];
			switch (c) {
				case ' ':
				case '?':
				case '<':
				case '>':
				case '&':
				case '^':
				case '#':
				case '%':
				case '\"':
				case '\\':
				case '*':
				case '/':
				case ':':
				case '.':
					sb.Append('_');
					break;
				default:
					sb.Append(c);
			}
		}
		String ret = XPStringUtility.ToLowerCase(sb.ToString());
		return ret;
	}

	/**
	 * Reverses any character replacement performed in <code>FriendlyURL#NormalizeObjectName</code>,
	 * except for character casing. All characters will be lower case.
	 * 
	 * NOTE: This method is duplicated on ObjectIdFromNameCache class in the server. Make sure to update
	 * both methods.
	 */
	public static String DeNormalizeObjectName(String objectName) 
	{
		// CR DLH: Shouldn't we just do this using GeneralHelpers.Replace,
		// since it's cleaner?  Then we can worry about optimizations if a code
		// profiler shows that this is slow?
		// ROB what's wrong with writing optimized code in the first place?
		// creating extra String are always wasteful and this is scales if 
		// we need additional replacements
		XPStringBuilder sb = new XPStringBuilder();
		char[] chars = XPStringUtility.GetChars(objectName);
		for (int i=0; i<chars.length; i++) {
			char c = chars[i];
			switch (c) {
			case '_':
			case '?':
			case '<':
			case '>':
			case '&':
			case '^':
			case '#':
			case '%':
			case '\"':
			case '\\':
			case '*':
			case '/':
			case ':':
			case '.':
				sb.Append(' ');
				break;
			/*case '/':
				sb.Append('\\');
				break;*/
			default:
				sb.Append(c);
			}
		}
		return sb.ToString();
		
		//return GeneralHelpers.Replace(objectName, "_", " ");
	}

	public String GetChildName() 
	{
		return m_childName;
	}

	public void SetChildName(String name) 
	{
		m_childName = name;
	}

	public int GetClassId() 
	{
		return m_classId;
	}
	
	public String GetClassIdAsString() 
	{
		return m_classId+"";
	}
	

	public void SetClassId(int id) 
	{
		m_classId = id;
	}

	public int GetObjectId() 
	{
		return m_objectId;
	}

	public void SetObjectId(int id) 
	{
		m_objectId = id;
	}

	public String GetOjectName() 
	{
		return m_objectName;
	}

	public void SetObjectName(String name) 
	{
		m_objectName = name;
	}

	public boolean GetUseIdIfAvailable() {
		return m_useIdIfAvailable;
	}

	public void SetUseIdIfAvailable(boolean useidIfAvailable) {
		m_useIdIfAvailable = useidIfAvailable;
	}
	

	public int GetChildId() {
		return m_childId;
	}

	public void SetChildId(int id) {
		m_childId = id;
	}
}