package com.plumtree.portaluiinfrastructure.friendlyurl;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;

public class FriendlyURLParserManager implements IFriendlyURLParserManager 
{
	// DCA - the only instance to be used
	private static IFriendlyURLParserManager INSTANCE;
	// DCA - boolean to indicate whether we are in production mode or test mode
	private static boolean m_bIsManagerProduction = false;
	
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_FRIENDLYURL),
			"com.plumtree.portaluiinfrastructure.friendlyurl.FriendlyURLParserManager");
	
	// DCA - Map of keywords to class ids taken from FriendlyURLS.xml
	protected XPHashtable m_htKeyToClassIdLookup = null;

	/**
	 * @return The singleton manager for retrieving parsers.
	 */
	public static IFriendlyURLParserManager getInstance()
    {
		if (INSTANCE == null)
		{
			m_bIsManagerProduction = true;
			INSTANCE = new FriendlyURLParserManager();
		}
		return INSTANCE;
	}
	
	/**
	 * Make constructor private so that no other class can create an instance of this class.
	 *
	 */
	private FriendlyURLParserManager() {
		
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.friendlyurl.IFriendlyURLParserManager#SetKeyToClassIdLookupTable(com.plumtree.openfoundation.util.XPHashtable)
	 */
	public void SetKeyToClassIdLookupTable(XPHashtable htKeyToClassIdLookup) {
		m_htKeyToClassIdLookup = htKeyToClassIdLookup;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.friendlyurl.IFriendlyURLParserManager#GetParser(com.plumtree.openfoundation.util.XPStringTokenizer)
	 */
	public IFriendlyURLParser GetParser(XPStringTokenizer stURLTokens) {
		// DCA - basic friendly URL structure will have /keyword/object name/
		String strKeyword = stURLTokens.GetNextToken();
		//  DCA - make sure to decode since it was part of the URL
		try {
			strKeyword = XPHttpUtility.UrlDecode(strKeyword, LinkConstants.UTF_8_ENCODING);
		} catch (XPUnsupportedEncodingException e) {
			// DCA - object name must've not been encoded. Log exception and move on.
			log.Debug(e, getClass().getName() + ":GetParser The string: " + strKeyword + " could not be decoded.");
		}
		String strClassId = FindClassID(strKeyword);

		// NOTE: If there gete to be too many cases in this switch statement, we should
		// probably make the class type descriptor return the appropriate URL parser.
		FriendlyURLParser parser;
		if (null == strClassId) {
			parser = new FriendlyURLParser(strKeyword, null, stURLTokens);
			// DCA - check if non pt object friendly URL
			// Certain non pt object friendly URLs have the entire path as the keyword so that customers
			// can change the any part of the path.
			if (null != parser.GetObjectName(false)) {
				strKeyword += "/"+parser.GetObjectName(false);
				parser.SetClassId(FindClassID(strKeyword));
			}
			return parser;
		}
		// DCA - the FriendlyURLParser will parse the rest
		switch (XPConvert.ToInteger(strClassId)) {
			case PT_CLASSIDS.PT_COMMUNITY_ID:
				log.Debug("FriendlyURLParserManager:GetParser returning CommunityFriendlyURLParser");
				return new CommunityFriendlyURLParser(strKeyword, strClassId, stURLTokens);
			case PT_CLASSIDS.PT_CATALOGFOLDER_ID:
				log.Debug("FriendlyURLParserManager:GetParser returning KDFriendlyURLParser");
				return new KDFriendlyURLParser(strKeyword, strClassId, stURLTokens);
			case PT_CLASSIDS.PT_MYPAGESCONTAINER_ID:
				log.Debug("FriendlyURLParserManager:GetParser returning MyPageFriendlyURLParser");
				return new MyPageFriendlyURLParser(strKeyword, strClassId, stURLTokens);
			case PT_CLASSIDS.PT_CATALOGCARD_ID:
				log.Debug("FriendlyURLParserManager:GetParser returning DocumentFriendlyURLParser");
				return new DocumentFriendlyURLParser(strKeyword, strClassId, stURLTokens);
			case PT_CLASSIDS.PT_USER_ID:
				log.Debug("FriendlyURLParserManager:GetParser returning UserFriendlyURLParser");
				return new UserFriendlyURLParser(strKeyword, strClassId, stURLTokens);
			case FriendlyURLConstants.REST_API_ID:
				log.Debug("FriendlyURLParserManager:GetParser returning APIFriendlyURLParser");
				return new APIFriendlyURLParser(strKeyword, strClassId, stURLTokens);
			case FriendlyURLConstants.FRIENDLY_SEARCH_ID:
				log.Debug("FriendlyURLParserManager:GetParser returning SearchFriendlyURLParser");
				return new SearchFriendlyURLParser(strKeyword, strClassId, stURLTokens);
			case FriendlyURLConstants.FRIENDLY_RSS_ID:
				log.Debug("FriendlyURLParserManager:GetParser returning RSSFriendlyURLParser");
				return new RSSFriendlyURLParser(strKeyword, strClassId, stURLTokens);
			default:
				log.Debug("FriendlyURLParserManager:GetParser returning FriendlyURLParser");
				parser = new FriendlyURLParser(strKeyword, strClassId, stURLTokens);
				// DCA - check if non object friendly URL
				if (null != parser.GetObjectName(false)) {
					strKeyword += "/"+parser.GetObjectName(false);
					parser.SetClassId(FindClassID(strKeyword));
				}
				return parser;
		}
	}
	
	/**
	 * Returns the correct class Id based on the keyword passed in.
	 * 
	 * @param strClassTypeKeyword
	 * @return
	 */
	private String FindClassID(String strClassTypeKeyword) {
		log.Debug("FriendlyURLPlugin:FindClassID  Using key: " + strClassTypeKeyword);
		if (null == m_htKeyToClassIdLookup) {
			throw new XPException("The FriendlyURLParserManager class was not initialized properly.");
		}
		String strClassId = "";
		try {
			strClassId = (String) m_htKeyToClassIdLookup.GetElement(strClassTypeKeyword);
			log.Debug("FriendlyURLParserManager:FindClassID  Found: " + strClassId);
		} catch (XPException e) {
			log.Error(e, "FriendlyURLParserManager:FindClassID Could not find class Id from keyword: " + strClassTypeKeyword);
			strClassId = null;
		}
		return strClassId;
	}
	
	 /** 
	 * @param typeId Class id or type id
	 * @return
	 */
	public String FindKeywordForType(String typeId)
	{
		if ( typeId == null ) {
			return null;
		}
		
		if (null == m_htKeyToClassIdLookup) {
			throw new XPException("The FriendlyURLParserManager class was not initialized properly.");
		}
		
		if ( m_htKeyToClassIdLookup.ContainsValue(typeId) ) {
			Object[] keys = m_htKeyToClassIdLookup.GetArray(XPHashtable.KEYS);
			Object[] values = m_htKeyToClassIdLookup.GetArray(XPHashtable.VALUES);
			// look up the corresponding key for the specified value
			for ( int i = 0; i < values.length; i++ ) {
				if (typeId.equals(values[i]) ) {
					return (String)keys[i];
				}
			}
		}
		return null;
		
	}

	
	
	/**
     * Unit tests should use this method to overwrite/teardown existing manager singleton.
     * DO NOT USE for production code. 
     */
	public static void setManagerTestInstance(IFriendlyURLParserManager testManager)
	{
		if (m_bIsManagerProduction)
		{
			String strMessage = "Can't override Layout Manager instance for testing... LayoutManager has already been created.";
			log.Error(strMessage);
			throw new XPException(strMessage);
		}

		INSTANCE = testManager;
	}
}
