package com.plumtree.portaluiinfrastructure.helpers;

import com.plumtree.openfoundation.util.XPArrayList;
import com.plumtree.openfoundation.util.XPConvert;
import com.plumtree.openfoundation.util.XPException;
import com.plumtree.openlog.OpenLogService;
import com.plumtree.openlog.OpenLogger;
import com.plumtree.portaluiinfrastructure.restconsumerframework.pathways.PathwaysConstants;
import com.plumtree.portaluiinfrastructure.search.IBaseSearchResultModelRO;
import com.plumtree.portaluiinfrastructure.search.SearchResultModel;
import com.plumtree.portaluiinfrastructure.statichelpers.PTDebugHelpers;
import com.plumtree.server.IPTSession;
import com.plumtree.server.IPTSessionInfo;
import com.plumtree.server.PT_CLASSIDS;
import com.plumtree.server.PT_SEARCH_SETTING;
import com.plumtree.uiinfrastructure.activityspace.AActivitySpace;

public class BannerSearchHelperService implements IBannerSearchHelper 
{
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
			"com.plumtree.portaluiinfrastructure.helpers.BannerSearchControlHelper");

	//the singleton instance
	private static IBannerSearchHelper m_helperInstance;

	// This stores whether or not the factory is in test mode.
	private static boolean m_bIsHelperProduction = false;
	
	protected AActivitySpace _parentAS;
	
	/** Name of Object Type override search preference.
	 *  Used when performing tag searches from KD Browse
	 */
	public static final String SEARCHPREF_OBJECTTYPES_OVERRIDE = "searchObjectTypesOverride";

/*
	// Names of search preferences in the Plumtree database.
	public static final String SEARCHPREF_LIMITBYLOCALE = "searchLimitByLocale";	
	public static final String SEARCHPREF_PAGESIZE = "searchPageSize";
    public static final String SEARCHPREF_GROUPBY = "searchGroupBy";
    public static final String SEARCHPREF_OBSOLETE_GROUPBY = "GroupBySearchPrefs";
    //Name under which we cache banner search preferences in our subsession. 
    public static final String CACHED_SEARCHPREFS_KEY = "BannerSearchPreferences";
*/
	public static final String SEARCHPREF_GROUPBY = "searchGroupBy";
	public static final String SEARCHPREF_OBJECTTYPES = "searchObjectTypes";
	public static final String SPECIAL_SORT_BY_LASTMODIFIED_VALUE = "lastmodified";
    public static final String SPECIAL_SORT_BY_NAME_VALUE = "name";

	// NOTE: These bitflags MUST stay in sync with those in
	//  com.plumtree.portalpages.browsing.portalsettings.search; I don't want to import
	//  symbols in the "ext" package here
    public static final int BIT_CARD = 1;
    public static final int BIT_FOLDER = 2;
    public static final int BIT_USER = 4;
    public static final int BIT_COMMUNITY = 8;
    public static final int BIT_PORTLET = 16;
    public static final int BIT_COLLAB = 32;
    public static final int BIT_CONTENT = 64;
    public static final int BIT_PAGE = 128;
    
	/**
	 * Private constructor to ensure controlled access to the singleton.
	 */
	private BannerSearchHelperService() { super(); }

	/**
	 * @return The singleton PT config helper.
	 */
	public static IBannerSearchHelper GetInstance()
    {
		if (m_helperInstance == null)
		{
			m_bIsHelperProduction = true;
			m_helperInstance = new BannerSearchHelperService();
		}
		return m_helperInstance;
    }
	
    /**
     * Unit tests should use this method to overwrite/teardown existing helper singleton.
     * DO NOT USE for production code.
     * 
     * NOTE: When finished with a unit test, this should be set back to null so as not
     * to interfere with other tests.
     */
/*	public static void setTestPTInstance(IPTConfigHelper testHelper)
	{
		if (m_bIsHelperProduction)
		{
			String strMessage = "Can't override PTConfigHelper instance for testing... PTConfigService has already been initialized.";
			log.Error(strMessage);
			throw new XPException(strMessage);
		}

		m_PTHelperInstance = testHelper;
	}
*/
	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.helpers.IBannerSearchHelper#GetBannerSearchPreferenceSortOrder(com.plumtree.uiinfrastructure.activityspace.AActivitySpace)
	 */
	public String GetBannerSearchPreferenceSortOrder(AActivitySpace _parent)
	{
		IPTSession session = (IPTSession) _parent.GetUserSession();
		IPTSessionInfo sessionInfo = session.GetSessionInfo();
		String sortByValue = GetStringSetting(sessionInfo, SEARCHPREF_GROUPBY, null);
		
		//if null, means pref doesn't exist, so I will set to "relevance" by default
		if (null == sortByValue)
		{
			//0 stands for relevance ranking
            sortByValue = SearchResultModel.SORTORDER_RELEVANCE;
		}
		
		//TODO check for the other 2 options that Search Preference Editor allows
        if (sortByValue.equals(SPECIAL_SORT_BY_LASTMODIFIED_VALUE)) {
            // Special case: relevance-rank, but sort by date
            sortByValue = SearchResultModel.SORTORDER_MODIFIED_DESC;
        } else if (sortByValue.equals(SPECIAL_SORT_BY_NAME_VALUE)) {
            sortByValue = SearchResultModel.SORTORDER_NAME_ASC;
        } else {
        	//0 stands for relevance ranking
            sortByValue = SearchResultModel.SORTORDER_RELEVANCE;
        }
		
		return sortByValue;
	}

	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.helpers.IBannerSearchHelper#GetSearchConstraints(com.plumtree.uiinfrastructure.activityspace.AActivitySpace)
	 */
	public XPArrayList GetSearchConstraints(AActivitySpace _parent)
	{
		_parentAS = _parent;
		boolean returnCollabObjects;
		boolean returnContentObjects;
		
		/**
		 * in pathways, collab contains 6 search terms 
		 * in pathways, content contains 1 search term
		 * I will add 7 to the size to potentially hold that amount. 
		 */
		XPArrayList searchConstraints = new XPArrayList();
		
		IPTSession session = (IPTSession) _parent.GetUserSession();
		IPTSessionInfo sessionInfo = session.GetSessionInfo();
		
		int objectTypeBitmask = GetIntSetting(sessionInfo, SEARCHPREF_OBJECTTYPES, 0);
		XPArrayList classIDs = GetPortalObjectTypes(sessionInfo, objectTypeBitmask);

		// We need to convert the classIDs to String so that it can be fed into the
		// Pathways Rest API.
		if (0 < classIDs.GetSize())
		{
			for (int j = 0; j < classIDs.GetSize(); j++)
			{
				String value = GetPathwaysClassIdStringValue(XPConvert.ToInteger(classIDs.GetElement(j)));
				searchConstraints.Add(value);					
			}
		}
		
		returnCollabObjects	   = (0 == (objectTypeBitmask & BIT_COLLAB));
		returnContentObjects     = (0 == (objectTypeBitmask & BIT_CONTENT));
		
		if (returnCollabObjects)
		{
			searchConstraints = setCollabItems(searchConstraints);
		}
		if (returnContentObjects)
		{
			searchConstraints = setContentItems(searchConstraints);
		}
		
		return searchConstraints;
	}
	
	/*(non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.helpers.IBannerSearchHelper#LookupSearchBitMask(int)
	 * 
	 * The contents of this method were moved from BannerStartSearchControl.GetPreferences() to figure 
	 * out the bitmask mapping for search preferences.
	 * 
	 * The reason for moving this out is because Pathways requires these classIDs in string format 
	 * for the calls to the Pathways Rest API.
	 */
	public XPArrayList GetPortalObjectTypes(IPTSessionInfo sessionInfo, int objectTypeBitmask)
	{
		int nObjectTypeOverride = GetIntSessionSetting(sessionInfo, 
				BannerSearchHelperService.SEARCHPREF_OBJECTTYPES_OVERRIDE, -1);
		XPArrayList classIDs = new XPArrayList(5);

		if (nObjectTypeOverride != -1)
		{
			classIDs.Add(new Integer(nObjectTypeOverride));
		}
		else
		{
			if (0 == (objectTypeBitmask & BIT_CARD))
			{
				classIDs.Add(new Integer(PT_CLASSIDS.PT_CATALOGCARD_ID));
			}
	
			if (0 == (objectTypeBitmask & BIT_FOLDER))
			{
				classIDs.Add(new Integer(PT_CLASSIDS.PT_CATALOGFOLDER_ID));
			}
	
			if (0 == (objectTypeBitmask & BIT_USER))
			{
				classIDs.Add(new Integer(PT_CLASSIDS.PT_USER_ID));
			}
	
			if (0 == (objectTypeBitmask & BIT_COMMUNITY))
			{
				classIDs.Add(new Integer(PT_CLASSIDS.PT_COMMUNITY_ID));
			}
	
			if (0 == (objectTypeBitmask & BIT_PORTLET))
			{
				classIDs.Add(new Integer(PT_CLASSIDS.PT_GADGET_ID));
			}
	
			if (0 == (objectTypeBitmask & BIT_PAGE))
			{
				classIDs.Add(new Integer(PT_CLASSIDS.PT_PAGE_ID));
			}
		}

		return classIDs;
	}

	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.helpers.IBannerSearchHelper#GetPathwaysClassIdStringValue(int)
	 */
	public String GetPathwaysClassIdStringValue(int value)
	{
		String strClassId = null;
		switch (value) {
			case 18:
				strClassId = PathwaysConstants.PORTAL_DOCUMENTS;
				break;
			case 17:
				strClassId = PathwaysConstants.DOCUMENT_FOLDERS;
				break;
			case 1:
				strClassId = PathwaysConstants.USERS;
				break;
			case 512:
				strClassId = PathwaysConstants.COMMUNITIES;
				break;
			case 43:
				strClassId = PathwaysConstants.PORTLETS;
				break;
			case 514:
				strClassId = PathwaysConstants.COMMUNITY_PAGES;
				break;
			case 20:
				strClassId = PathwaysConstants.ADMIN_FOLDERS;
				break;
			case 3:
				strClassId = PathwaysConstants.AUTHENTICATION_SOURCES;
				break;
			case 54:
				strClassId = PathwaysConstants.COMMUNITY_TEMPLATES;
				break;
			case 35:
				strClassId = PathwaysConstants.CONTENT_SOURCES;
				break;
			case 37:
				strClassId = PathwaysConstants.CONTENT_TYPES;
				break;
			case 38:
				strClassId = PathwaysConstants.CRAWLERS;
				break;
			case 8:
				strClassId = PathwaysConstants.EXPERIENCE_DEFINITIONS;
				break;
			case 58:
				strClassId = PathwaysConstants.EXTERNAL_OPERATIONS;
				break;
			case 46:
				strClassId = PathwaysConstants.FEDERATED_SEARCHES;
				break;
			case 32:
				strClassId = PathwaysConstants.FILTERS;
				break;
			case 2:
				strClassId = PathwaysConstants.GROUPS;
				break;
			case 44:
				strClassId = PathwaysConstants.INVITATIONS;
				break;
			case 256:
				strClassId = PathwaysConstants.JOBS;
				break;
			case 56:
				strClassId = PathwaysConstants.PAGE_TEMPLATES;
				break;
			case 61:
				strClassId = PathwaysConstants.PORTLET_TEMPLATES;
				break;
			case 7:
				strClassId = PathwaysConstants.PROFILE_SOURCES;
				break;
			case 36:
				strClassId = PathwaysConstants.PROPERTIES;
				break;
			case 48:
				strClassId = PathwaysConstants.REMOTE_SERVERS;
				break;
			case 33:
				strClassId = PathwaysConstants.SNAPSHOT_QUERIES;
				break;
			case 47:
				strClassId = PathwaysConstants.WEB_SERVICES;
				break;
			default:
				//the filter type has been reset to "All Types"
				break;
		}
				
		return strClassId;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.helpers.IBannerSearchHelper#setCollabItems(com.plumtree.openfoundation.util.XPArrayList)
	 */
	public XPArrayList setCollabItems(XPArrayList constraints)
	{
		String [] collabSearchTerms = PathwaysConstants.COLLAB_ITEMS;
		for (int i = 0; i < collabSearchTerms.length; i++)
		{
			constraints.Add(collabSearchTerms[i]); 
		}
		return constraints;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.helpers.IBannerSearchHelper#setContentItems(com.plumtree.openfoundation.util.XPArrayList)
	 */
	public XPArrayList setContentItems(XPArrayList constraints)
	{
		constraints.Add(PathwaysConstants.PUBLISHER_ITEMS);
		return constraints;
	}

	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.helpers.IBannerSearchHelper#GetIntSetting(com.plumtree.server.IPTSessionInfo, java.lang.String, int)
	 * 
	 *  This method was copied by BannerStartSearchControl 
	 *  In the long run we probably want to refactor this so that BannerStartSearchControl class calls into these functions too instead of having duplicate code
	 */
	public int GetIntSetting(IPTSessionInfo sessionInfo, String settingName, int def)
	{
		int r = def;

		Object[][] setting = (Object[][]) sessionInfo.LookupPreference(settingName, 0);

		if (null != setting)
		{
			Object o = setting[1][0];

			if (null != o)
			{
				r = XPConvert.ToInteger(o);
			}
		}

		return r;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.helpers.IBannerSearchHelper#GetIntSessionSetting(com.plumtree.server.IPTSessionInfo, java.lang.String, int)
	 * 
	 *  This method was copied by BannerStartSearchControl 
	 *  In the long run we probably want to refactor this so that BannerStartSearchControl class calls into these functions too instead of having duplicate code
	 */
	public int GetIntSessionSetting(IPTSessionInfo sessionInfo, String settingName, int def)
	{
		int r = def;

		Object[][] setting = (Object[][]) sessionInfo.LookupSessionPreference(settingName);

		if (null != setting)
		{
			Object o = setting[1][0];

			if (null != o)
			{
				r = XPConvert.ToInteger(o);
			}
		}

		return r;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.helpers.IBannerSearchHelper#GetStringSetting(com.plumtree.server.IPTSessionInfo, java.lang.String, java.lang.String)
	 * 
	 *  This method was copied by BannerStartSearchControl.GetPreferences() to figure out the bitmask mapping for search preferences
	 * In the long run we probably want to refactor this so that BannerStartSearchControl class calls into these functions too instead of having duplicate code
	 */
	public String GetStringSetting(IPTSessionInfo sessionInfo, String settingName, String def)
	{
		String r = def;

		Object[][] setting = (Object[][]) sessionInfo.LookupPreference(settingName, 0);

		if (null != setting)
		{
			Object o = setting[1][0];

			if (null != o)
			{
				r = XPConvert.ToString(o);
			}
		}

		return r;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.helpers.IBannerSearchHelper#SetPathwaysSearchWithinResults(java.lang.String, java.lang.String)
	 */
	public String SetPathwaysSearchHistory(Object[] queryArray)
	{	    
	    String pathwaysSearchString = "";
	    String ancestorString = "";

	    int length = queryArray.length;

	    //we get the first search string b/c we need to make sure that the initial string gets added properly
	    pathwaysSearchString = XPConvert.ToString(queryArray[0]);

	    //grabbing the additionalQuery from the last entry of the array
	    String newSearch = XPConvert.ToString(queryArray[length-1]);
	    	    
	    //queryArray is never null as it can never search within a result without having a previous query that was executed.
	    //doing length-1 b/c we already took out the last entry in the array above, so we don't count it again.	    
	    for (int i=1; i < length-1; i++)
	    {
	    	ancestorString = XPConvert.ToString(queryArray[i]);
	    	pathwaysSearchString = "(" + pathwaysSearchString + ") and " + ancestorString;
	    }
	    
	    //now we add the new search string
	    pathwaysSearchString = "(" + pathwaysSearchString + ") and " + newSearch;
	    return pathwaysSearchString;
	}
}
