package com.plumtree.portaluiinfrastructure.helpers;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * This class handles common functionality related to Ensemble integrations.
 * 
 * NOTE: This class is threadsafe.
 */
public class EnsembleHelperService implements IEnsembleHelper
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.helpers.EnsembleHelperService");

	/**
	 * This is used to lookup the Ensemble remote server instance for pagelet web services.
	 */
	public static final String ENSEMBLE_REMOTE_SERVER_UUID = "{4051EE46-9E4E-5D57-31C3-CD398FF42000}"; 

	/**
	 * The timeout for caching the Remote Server data, in seconds.
	 */
	protected static final int TIMEOUT = 300;

	//the singleton instance
	private static IEnsembleHelper m_helperInstance;

	// This stores whether or not the factory is in test mode.
	private static boolean m_bIsHelperProduction = false;

	/**
	 * Whether or not the cache has been initialized.
	 */
	private boolean m_bInitialized = false;

	/**
	 * The last time we queried the DB.  This must be accessed
	 * in a threadsafe way, since assignments to long are not
	 * atomic/threadsafe.
	 */
	private long m_nLastChecked = 0;

	/**
	 * Lock to guarantee atomic access to the cache update/lazy
	 * init code segment and/or timestamp.
	 */
	private final Object m_lock = new Object();

	/**
	 * The cached ID of the Ensemble Remote Server, if installed.
	 */
	private int m_nServerID = -1;

	/**
	 * Private constructor to ensure controlled access to the singleton.
	 */
	private EnsembleHelperService() { super(); }

	/**
	 * @return The singleton PT config helper.
	 */
	public static IEnsembleHelper getInstance()
    {
		// Technically this should be synchronized to ensure
		// threadsafety, but we don't care if an extra helper service
		// is created on startup and then overwritten.
		if (m_helperInstance == null)
		{
			m_bIsHelperProduction = true;
			m_helperInstance = new EnsembleHelperService();
		}
		return m_helperInstance;
    }

    /**
     * Unit tests should use this method to overwrite/teardown existing helper singleton.
     * DO NOT USE for production code.
     * 
     * NOTE: When finished with a unit test, this should be set back to null so as not
     * to interfere with other tests.
     */
	public static void setTestInstance(IEnsembleHelper testHelper)
	{
		// Technically this should be synchronized to ensure
		// threadsafety, but we don't care about threadsafety
		// during unit tests.
		if (m_bIsHelperProduction)
		{
			String strMessage = "Can't override EnsembleHelperService instance for testing... EnsembleHelperService has already been initialized.";
			log.Error(strMessage);
			throw new XPException(strMessage);
		}

		m_helperInstance = testHelper;
	}

	/** 
	 * Load the Ensemble Remote Server, if installed.
	 *
	 * This handles cache timeouts and lazy initialization.
	 */
	protected void LoadEnsembleRemoteServer(IPTSession _session)
	{
		// NOTE: We don't really care that much about stale data,
		// just safe data.

		// No need for this to be synchronized, because we don't care
		// if the cache is updated at exactly the timeout time, or if
		// we have to wait for the next time someone calls this method.
		long nCurrentTime = XPSystem.GetCurrentTimeMilliSec();

		// Get the timeout config value.
		int nTimeout = GetTimeout();

		synchronized (m_lock)
		{
			// Only run the query if we haven't run it yet,
			// or if our cached version is timed out.
			if (m_bInitialized)
			{
				if (nCurrentTime < (m_nLastChecked + nTimeout))
				{
					// The cache is still valid.
					return;
				}
			}
	
			int nNewServerID = -1;

			try
			{
				// NOTE: It's okay to run this query with various user's sessions because the
				// query doesn't run a security check, so every user should get the same results.
				// Otherwise we would have to do something like use the
				// dangerousimpersonateadminuserlight method.
	
				// look for Ensemble remoteserver
				Object[][] filter = new Object[3][1];
				filter[0][0]      = new Integer(PT_PROPIDS.PT_PROPID_UUID);
				filter[1][0]      = new Integer(PT_FILTEROPS.PT_FILTEROP_EQ);
				filter[2][0]      = ENSEMBLE_REMOTE_SERVER_UUID;
		
				IPTQueryResult ptqrServer = (IPTQueryResult) _session.GetGadgetContentServers().
					Query(PT_PROPIDS.PT_PROPID_OBJECTID | PT_PROPIDS.PT_PROPID_CONTENTSERVER_URL,
						-1, PT_PROPIDS.PT_PROPID_OBJECTID, 0, -1, filter);
		
				if (ptqrServer.RowCount() > 1)
				{
					// this should never happen.
					log.Warn("More than one Ensemble remote server returned by query when initializing Ensemble Helper.  Ignoring.");
				}
		
				if (ptqrServer.RowCount() > 0)
				{
					nNewServerID = ptqrServer.ItemAsInt(0, PT_PROPIDS.PT_PROPID_OBJECTID);
				} else
				{
					// Ensemble Remote Server not installed.
					nNewServerID = -1;
				}
	
			} catch (Exception e)
			{
				log.Error(e, "Unable to determine if the Ensemble Remote Server is installed.");
	
				m_nServerID = -1;
			}
	
			// Set the new server ID.
			m_nServerID = nNewServerID;
	
			// Set timestamp to now.
			m_nLastChecked = XPSystem.GetCurrentTimeMilliSec();
	
			m_bInitialized = true;
		}
	}

	/**
	 * @return the timeout value from the config file.
	 */
	protected int GetTimeout()
	{
		int nTimeout;
		try
		{
			// Get the timeout config value.
			IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
			ConfigVarPack vpConfig = (ConfigVarPack) application.GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);
			nTimeout = vpConfig.GetConfigValueAsInt(ConfigConstants.SERVICE_URLS_SECTION, ConfigConstants.ENSEMBLE_CACHE_TIMEOUT, ConfigConstants.VALUE_ATTRIBUTE);
		} catch (Exception e) {
			log.Warn(e, "Unable to load Ensemble Server Cache Timeout.  Using default value of 5.");
			nTimeout = 5;
		}

		// Config timeout is in minutes, we need it in milliseconds
		return nTimeout * 60 * 1000;
	}

	public boolean IsEnsembleInstalled(IPTSession _session)
	{
		// This handles cache timeouts and lazy initialization.
		LoadEnsembleRemoteServer(_session);

		return (m_nServerID > 0);
	}

	public int GetRemoteServerID(IPTSession _session)
	{
		// This handles cache timeouts and lazy initialization.
		LoadEnsembleRemoteServer(_session);

		return m_nServerID;
	}
}