package com.plumtree.portaluiinfrastructure.helpers;

import com.plumtree.openfoundation.util.XPException;
import com.plumtree.openfoundation.util.XPSystem;
import com.plumtree.openlog.OpenLogService;
import com.plumtree.openlog.OpenLogger;
import com.plumtree.portaluiinfrastructure.statichelpers.PTDebugHelpers;
import com.plumtree.server.IPTQueryResult;
import com.plumtree.server.IPTSession;
import com.plumtree.server.PT_FILTEROPS;
import com.plumtree.server.PT_PROPIDS;


public class PublisherHelperService implements IPublisherHelper {

	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
			"com.plumtree.portaluiinfrastructure.helpers.PublisherHelperService");

	/**
	 * This is used to lookup the Collab web service instance.  "Notification Gateway Entry" is the actual
	 * name of web service I grabbed this UUID from, when you install the collaboration.pte.
	 */
	public static final String PUBLISHER_WEB_SERVICE_UUID = "{CE43C34B-B98F-4FA4-A540-A547EF937D81}"; 

	/**
	 * The timeout for caching the web service data, in seconds.
	 */
	protected static final int TIMEOUT = 300;

	//the singleton instance
	private static IPublisherHelper m_helperInstance;

	// This stores whether or not the factory is in test mode.
	private static boolean m_bIsHelperProduction = false;

	/**
	 * Whether or not the cache has been initialized.
	 */
	private boolean m_bInitialized = false;

	/**
	 * The last time we queried the DB.  This must be accessed
	 * in a threadsafe way, since assignments to long are not
	 * atomic/threadsafe.
	 */
	private long m_nLastChecked = 0;

	/**
	 * Lock to guarantee atomic access to the cache update/lazy
	 * init code segment and/or timestamp.
	 */
	private final Object m_lock = new Object();

	/**
	 * The cached ID of the collab web service, if installed.
	 */
	private int m_nServerID = -1;

	/**
	 * Private constructor to ensure controlled access to the singleton.
	 */
	private PublisherHelperService() { super(); }

	/**
	 * @return The singleton PT config helper.
	 */
	public static IPublisherHelper getInstance()
    {
		// Technically this should be synchronized to ensure
		// threadsafety, but we don't care if an extra helper service
		// is created on startup and then overwritten.
		if (m_helperInstance == null)
		{
			m_bIsHelperProduction = true;
			m_helperInstance = new PublisherHelperService();
		}
		return m_helperInstance;
    }

    /**
     * Unit tests should use this method to overwrite/teardown existing helper singleton.
     * DO NOT USE for production code.
     * 
     * NOTE: When finished with a unit test, this should be set back to null so as not
     * to interfere with other tests.
     */
	public static void setTestInstance(IPublisherHelper testHelper)
	{
		// Technically this should be synchronized to ensure
		// threadsafety, but we don't care about threadsafety
		// during unit tests.
		if (m_bIsHelperProduction)
		{
			String strMessage = "Can't override PublisherHelperService instance for testing... PublisherHelperService has already been initialized.";
			log.Error(strMessage);
			throw new XPException(strMessage);
		}

		m_helperInstance = testHelper;
	}

	/** 
	 * Load the Publisher web service, if installed.
	 *
	 * This handles cache timeouts and lazy initialization.
	 */
	protected void LoadPublisherWebService(IPTSession _session)
	{
		// NOTE: We don't really care that much about stale data,
		// just safe data.

		// No need for this to be synchronized, because we don't care
		// if the cache is updated at exactly the timeout time, or if
		// we have to wait for the next time someone calls this method.
		long nCurrentTime = XPSystem.GetCurrentTimeMilliSec();
		long nTimeout = TIMEOUT * 1000;

		synchronized (m_lock)
		{
			// Only run the query if we haven't run it yet,
			// or if our cached version is timed out.
			if (m_bInitialized)
			{
				if (nCurrentTime < (m_nLastChecked + nTimeout))
				{
					// The cache is still valid.
					return;
				}
			}
	
			int nNewServerID = -1;

			try
			{
				// NOTE: It's okay to run this query with various user's sessions because the
				// query doesn't run a security check, so every user should get the same results.
				// Otherwise we would have to do something like use the
				// dangerousimpersonateadminuserlight method.
	
				// look for collab web service
				Object[][] filter = new Object[3][1];
				filter[0][0]      = new Integer(PT_PROPIDS.PT_PROPID_UUID);
				filter[1][0]      = new Integer(PT_FILTEROPS.PT_FILTEROP_EQ);
				filter[2][0]      = PUBLISHER_WEB_SERVICE_UUID;
		
				IPTQueryResult ptqrServer = (IPTQueryResult) _session.GetWebServices().
					Query(PT_PROPIDS.PT_PROPID_OBJECTID,
						-1, PT_PROPIDS.PT_PROPID_OBJECTID, 0, -1, filter);
		
				if (ptqrServer.RowCount() > 1)
				{
					// this should never happen.
					log.Warn("More than one Publisher web service returned by query when initializing Publisher Helper.  Ignoring.");
				}
		
				if (ptqrServer.RowCount() > 0)
				{
					nNewServerID = ptqrServer.ItemAsInt(0, PT_PROPIDS.PT_PROPID_OBJECTID);
				} else
				{
					//Collaboration Remote Server not installed.
					nNewServerID = -1;
				}
	
			} catch (Exception e)
			{
				log.Error(e, "Unable to determine if the Publisher web service is installed.");
	
				m_nServerID = -1;
			}
	
			// Set the new server ID.
			m_nServerID = nNewServerID;
	
			// Set timestamp to now.
			m_nLastChecked = XPSystem.GetCurrentTimeMilliSec();
	
			m_bInitialized = true;
		}
	}

	public boolean IsPublisherInstalled(IPTSession _session)
	{
		// This handles cache timeouts and lazy initialization.
		LoadPublisherWebService(_session);

		return (m_nServerID > 0);
	}

	public int GetWebServiceID(IPTSession _session)
	{
		// This handles cache timeouts and lazy initialization.
		LoadPublisherWebService(_session);

		return m_nServerID;
	}
}
