package com.plumtree.portaluiinfrastructure.intrinsicportlets;

import com.plumtree.openfoundation.util.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;

/**
 * This is the base intrinsic portlet model.  All intrinsic portlet models
 * should extend this base class.
 *
 * @author josephc
 */
public abstract class IntrinsicPortletModel extends AClearModel
	implements IIntrinsicPortletModelRO, IModel
{
	/** Setting type for user portlet preferences */
	public static final int SETTINGSTYPE_PORTLET = 1;

	/** Setting type for admin preferences */
	public static final int SETTINGSTYPE_ADMIN = 2;

	/** Setting type for community preferences */
	public static final int SETTINGSTYPE_COMMUNITY = 3;

	/** Setting type for community portlet preferences */
	public static final int SETTINGSTYPE_COMMUNITYPORTLET = 4;

	/** Setting type for user preferences */
	public static final int SETTINGSTYPE_USER = 5;

	/** Preference type for admin preferences */
	public static final int PREF_TYPE_ADMIN = 1;

	/** Preference type for personal preferences */
	public static final int PREF_TYPE_PERSONAL = 2;

	/** Preference type for community preferences */
	public static final int PREF_TYPE_COMMUNITY = 3;

	/** Parent Activity Space */
	protected AActivitySpace m_asOwner;

	/** Portlet object */
	protected IPTGadget m_ptPortlet;

	/** IPTSession */
	protected IPTSession m_ptSession;

	/** AppDataState object */
	protected IPTState m_ptAppDataState;

	/** ContainerClassID */
	protected int m_nContainerClassID;

	/** ContainerObjectID */
	protected int m_nContainerObjectID;

	/** PageID */
	protected int m_nPageID;

	/** Portlet ID */
	protected int m_nPortletID;

	/** Preference type */
	protected int m_nPrefType;


	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()
	 */
	public void CleanupTempData()
	{
		super.CleanupTempData();
		
		m_ptAppDataState     = null;
		m_ptPortlet			 = null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
	 */
	public void CleanupAllData()
	{
		super.CleanupAllData();
		
		m_ptAppDataState     = null;
		m_ptPortlet			 = null;
	}

	/**
	 * Returns true if the portlet has a control associated with it.  This
	 * function should be implemented by the child class.
	 *
	 * @return Returns true if portlet has control. Otherwise return false.
	 *
	 * @see com.plumtree.portaluiinfrastructure.intrinsicportlets.IIntrinsicPortletModelRO#ControlExists()
	 */
	public abstract boolean ControlExists();

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public abstract String GetName();

	/**
	 * Returns portlet ID
	 *
	 * @return Portlet ID
	 *
	 * @see com.plumtree.portaluiinfrastructure.intrinsicportlets.IIntrinsicPortletModelRO#GetPortletID()
	 */
	public int GetPortletID()
	{
		return m_nPortletID;
	}

	/**
	 * Inits the Model with the Activity Space. Initialize the IPTSession from
	 * the Activity Space
	 *
	 * @param parent
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#Init(AActivitySpace)
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner     = parent;

		// Initialize the IPTSession from the parent activity space
		m_ptSession = (IPTSession) m_asOwner.GetUserSession();
	}

	/**
	 * Initializes the member variables of the portlet.
	 *
	 * @param __ptPageInfo IPTPageInfo
	 * @param _ptAppDataState AppDataState object
	 * @param _ptPortlet Portlet object
	 * @param _nPortletID Portlet ID
	 */
	public void InitPortlet(IPTPageInfo _ptPageInfo, IPTState _ptAppDataState,
		IPTGadget _ptPortlet, int _nPortletID)
	{
		if (null != _ptPageInfo)
		{
			m_nContainerClassID  = _ptPageInfo.GetContainerClassID();
			m_nContainerObjectID = _ptPageInfo.GetContainerObjectID();
			m_nPageID			 = _ptPageInfo.GetObjectID();
		}
		else
		{
			m_nContainerClassID  = 0;
			m_nContainerObjectID = 0;
			m_nPageID			 = 0;
		}
		m_nPortletID		 = _nPortletID;
		m_ptAppDataState     = _ptAppDataState;
		m_ptPortlet			 = _ptPortlet;

		if (PT_CLASSIDS.PT_COMMUNITY_ID == m_nContainerClassID)
		{
			m_nPrefType = PREF_TYPE_COMMUNITY;
		}
		else
		{
			m_nPrefType = PREF_TYPE_PERSONAL;
		}
	}

	/**
	 * Checks the markup array to see whether or not the portlet is intrinsic
	 *
	 * @param objArrMarkup Array retrieved from IPTGadget::GetHTMLText()
	 *
	 * @return True if the portlet is intrinsic.  False otherwise.
	 */
	public static boolean IsIntrinsicPortlet(Object[][] objArrMarkup)
	{
		int nPortletContentType = XPConvert.ToInteger(objArrMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_TYPE][0]);

		if (PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_IPP == nPortletContentType)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Returns a single setting value, based on the setting type specified.
	 *
	 * @param nSettingType Setting type
	 * @param strName Setting name
	 *
	 * @return Setting value
	 *
	 * @see com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.IIntrinsicPortletPrefsModelRO#GetSettingValue(int,
	 *      String)
	 */
	public Object GetSettingValue(int nSettingType, String strName)
	{
		String[] arFilter = new String[] { strName };
		Object[][] arValues;
		Object oValue = null;

		arValues = GetSettingValues(nSettingType, arFilter);

		if ((null != arValues) && (arValues.length > 0))
		{
			oValue = arValues[1][0];
		}

		return oValue;
	}

	//JD- TODOCR 1/16/03
	//There are helper functions to get user preferences in LoginHelper.  I'm not sure whether
	//or not they should be used here... but either way there should be a comment specifying 
	//what the decision is, and why was it made.
	//JD- TODOCR 1/16/03
	// Perhaps it's worth doing a sanity check that if community or community-gadget prefs
	// are set, it's happenning on a community.  This applies to all functions that set,
	// read, or remove a pref.
	//JD- TODOCR 1/16/03
	// How are you handling security for admin prefs? What happens if someone who does not
	// have edit rights on the Portlet object tries to set admin prefs?

	/**
	 * Returns all settings, based on the setting type and filter specified.
	 *
	 * @param nSettingType Setting type
	 * @param arFilter Filter for settings
	 *
	 * @return All settings
	 *
	 * @see com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.IIntrinsicPortletPrefsModelRO#GetSettingValues(int,
	 *      Object[])
	 */
	public Object[][] GetSettingValues(int nSettingType, String[] arFilter)
	{
		Object[][] arValues;

		switch (nSettingType)
		{
		case SETTINGSTYPE_PORTLET:
			if (PREF_TYPE_COMMUNITY == m_nPrefType)
			{
				arValues = (Object[][]) ((IPTCommunityManager)m_ptSession.GetCommunities()).CachedOpenCommunityInfo(m_nContainerObjectID, false)
					.LookupPreferences(arFilter, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_CURRENT_USER);
			}
			else
			{
				arValues = (Object[][]) m_ptSession.GetSessionInfo().LookupPreferences(arFilter, m_nPortletID);
			}

			break;

		case SETTINGSTYPE_ADMIN:
		{
			IPTAdminSettings ptAdminSettings = (IPTAdminSettings) m_ptPortlet.GetInterfaces("IPTAdminSettings");
			arValues = (Object[][]) ptAdminSettings.LookupAdminGadgetSettings(arFilter);
		}

		break;

		case SETTINGSTYPE_COMMUNITY:
			
			arValues = (Object[][]) ((IPTCommunityManager)m_ptSession.GetCommunities()).CachedOpenCommunityInfo(m_nContainerObjectID, true)
				.LookupPreferences(arFilter, 0, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);

			break;

		case SETTINGSTYPE_COMMUNITYPORTLET:
			arValues = (Object[][]) ((IPTCommunityManager)m_ptSession.GetCommunities()).CachedOpenCommunityInfo(m_nContainerObjectID, true)
				.LookupPreferences(arFilter, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);

			break;

		case SETTINGSTYPE_USER:
			arValues = (Object[][]) m_ptSession.GetSessionInfo().LookupPreferences(arFilter, 0);

			break;

		default:
			return null;
		}

		return arValues;
	}
}

