package com.plumtree.portaluiinfrastructure.intrinsicportlets;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This is the base intrinsic portlet view.  All intrinsic portlet views should
 * extend this base class.
 *
 * @author josephc
 */
public abstract class IntrinsicPortletView implements IView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.intrinsicportlets.IntrinsicPortletView");

	/**
	 * The name of the intrinsic portlet form prefix (for use in Javascript,
	 * etc...). The portlet ID will be appended to the prefix.
	 */
	public static final String INTRINSIC_PORTLET_HTMLFORM_NAME = "IntrinsicPortletForm";

	/**
	 * The name of the JavaScript used to submit the form. The portlet ID will
	 * be appended to the prefix.
	 */
	public static final String INTRINSIC_PORTLET_JAVASCRIPT_SUBMIT_FORM = "IntrinsicPortletSubmitForm";

	/** Parent Activity Space */
	protected AActivitySpace m_asOwner; // Associated Activity Space

	/** RO interface to model */
	protected IModelRO m_model; // Associated read-only model

	/**
	 * Set to true if you want your portlet to have a form and JavaScript for
	 * repost.
	 */
	protected boolean m_bAddFormToPortlet = false;

	/** Set to true if you want your portlet to have a multi-part form */
	protected boolean m_bMultiPart = false;

	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	abstract public Object Create();

	/**
	 * This is called by the MyPortalContentView, which aggregates all the
	 * intrinsic portlet views. This method calls DisplayPortlet() in order to
	 * generate the HTML specific to the portlet. If a form is required, then
	 * the HTML returned from DisplayPortlet() is put into a form.
	 *
	 * @return HTMLElement
	 *
	 * @throws XPException Exception thrown if portlet was uanble to display.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Display()
	 */
	public HTMLElement Display()
	{
		if (m_bAddFormToPortlet)
		{
			HTMLForm myForm;
			HTMLInput myInput;

			// wrapper table avoids extra white space from form
			HTMLTable wrapperTable;

			IIntrinsicPortletModelRO iModelRO = (IIntrinsicPortletModelRO) m_model;

			String strPortletFormName = INTRINSIC_PORTLET_HTMLFORM_NAME + iModelRO.GetPortletID();

			try
			{
				// Form and hidden inputs
				myForm = new HTMLForm("");

				myForm.AddInnerHTMLElement(new HTMLComment("Begin: Display form and hidden inputs"));

				if (m_bMultiPart)
				{
					myForm.SetEncType(AFormDP.MULTIPART_ENCODING);
				}

				myForm.SetAction(ConfigHelper.GetApplicationBaseURLToAS(m_asOwner.GetCurrentNormalizedBaseURL(), m_asOwner.GetName(), m_asOwner.IsSubSpace()));
				myForm.SetMethod(AFormDP.POST);
				myForm.SetName(strPortletFormName);

				// Hidden Inputs
				// The action for the repost control to perform (repost, finish, cancel, etc...)
				myInput = new HTMLInput(HTMLInputTypes.HIDDEN, RepostControl.HTMLINPUT_POSTTOSELF, "");
				myForm.AddInnerHTMLElement(myInput);
				myInput.SetValue(XPConvert.ToString(RepostControl.POSTTOSELF_ACTION_UNDEFINED));

				// Name of space to POST this form to.
				myInput = new HTMLInput(HTMLInputTypes.HIDDEN, ASConstants.SPACENAME_INPUT, "");
				myForm.AddInnerHTMLElement(myInput);
				myInput.SetValue(m_asOwner.GetName());

				// ID of space to POST this form to.
				myInput = new HTMLInput(HTMLInputTypes.HIDDEN, ASConstants.SPACEID_INPUT, "");
				myForm.AddInnerHTMLElement(myInput);
				myInput.SetValue(m_asOwner.GetSpaceID());

				// Name of control to POST this form to.
				myInput = new HTMLInput(HTMLInputTypes.HIDDEN, ASConstants.CONTROL_INPUT, "");
				myForm.AddInnerHTMLElement(myInput);
				myInput.SetValue(((IFormAS) m_asOwner).GetRepostControlName());

				// Name of display page to display after POST.
				myInput = new HTMLInput(HTMLInputTypes.HIDDEN, ASConstants.PAGE_INPUT, "");
				myForm.AddInnerHTMLElement(myInput);
				myInput.SetValue(m_asOwner.GetCurrentPage().GetName());

				myForm.AddInnerHTMLElement(new HTMLComment("End: Display form and hidden inputs"));

				// the following block of code wraps our form
				// in an outer empty table, thus preventing empty
				// white space from being displayed after the form
				wrapperTable = new HTMLTable();
				wrapperTable.SetCellSpacing(CommonHTMLStrings.ZERO);
				wrapperTable.SetCellPadding(CommonHTMLStrings.ZERO);
				wrapperTable.SetBorder(CommonHTMLStrings.ZERO);
				wrapperTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);

				HTMLTableRow rowHack = new HTMLTableRow();
				HTMLTableCell cellHack = new HTMLTableCell();
				cellHack.AddInnerHTMLElement(DisplayPortlet());
				rowHack.AddInnerHTMLElement(cellHack);
				wrapperTable.AddInnerHTMLElement(myForm);
				myForm.AddInnerHTMLElement(rowHack);
			}
			catch (HTMLException e)
			{
				log.Error(e, m_asOwner.GetString(163, "ptmsgs_portalinfrastructure"));

				throw new XPException(m_asOwner.GetString(163, "ptmsgs_portalinfrastructure"), e);
			}

			return wrapperTable;
		}
		else
		{
			return DisplayPortlet();
		}
	}

	/**
	 * This is called by the ??, which aggregates all the
	 * intrinsic portlet WML views. This method calls DisplayPortletWML() in order to
	 * generate the WML specific to the portlet.
	 *
	 * @return HTMLElement representing WML for portlet
	 */
	public String DisplayWML()
	{
		return DisplayPortletWML();
	}

	/**
	 * Portlet Javascript is aggregated by PlumtreeDP.  Note that you may need
	 * to take into account the portletID in your Javascript.
	 *
	 * @return HTMLScript
	 *
	 * @throws XPException Exception thrown if portlet was uanble to generate
	 *         JavaScript.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 */
	public HTMLScript DisplayJavascript()
	{
		if (m_bAddFormToPortlet)
		{
			HTMLScript myScript;

			IIntrinsicPortletModelRO iModelRO = (IIntrinsicPortletModelRO) m_model;

			String strPortletFormName = IntrinsicPortletView.INTRINSIC_PORTLET_HTMLFORM_NAME + iModelRO.GetPortletID();
			String strPortletJavaScriptSubmitForm = IntrinsicPortletView.INTRINSIC_PORTLET_JAVASCRIPT_SUBMIT_FORM + iModelRO.GetPortletID();

			try
			{
				// This javascript function reposts the page based upon hidden HTML inputs in
				// the form.
				XPStringBuilder sb = new XPStringBuilder(300);
				sb.Append("\n");

				sb.Append("function ").Append(strPortletJavaScriptSubmitForm).Append("(varPostToSelf)\n");
				sb.Append("{\n");
				sb.Append("  if ((varPostToSelf != ").Append(RepostControl.POSTTOSELF_ACTION_FINISH).Append("  ) && (varPostToSelf != ").Append(RepostControl.POSTTOSELF_ACTION_GOTOPAGE).Append(")) {\n");
				sb.Append("    document.").Append(strPortletFormName).Append(".").Append(RepostControl.HTMLINPUT_POSTTOSELF).Append(".value= varPostToSelf;\n");
				sb.Append("    document.").Append(strPortletFormName).Append(".submit();\n");
				sb.Append("  }\n");
				sb.Append("}\n");

				myScript = new HTMLScript("text/javaScript");
				myScript.AddInnerHTMLString(sb.ToString());
			}
			catch (HTMLException e)
			{
				log.Error(e, m_asOwner.GetString(164, "ptmsgs_portalinfrastructure"));

				throw new XPException(m_asOwner.GetString(164, "ptmsgs_portalinfrastructure"), e);
			}

			return myScript;
		}
		else
		{
			return null;
		}
	}

	/**
	 * Displays the HTML for the portlet. The child classes must implement this
	 * function. This function is called by Display().
	 *
	 * @return HTML for portlet
	 */
	public abstract HTMLElement DisplayPortlet();

	/**
	 * Displays the WML for the portlet. The child classes must implement this
	 * function.  This function is called by DisplayWML().
	 *
	 * @return WML for portlet
	 */
	public abstract String DisplayPortletWML();

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public abstract String GetName();

	/**
	 * Inits the Section with the read-only Model and the Activity Space.
	 *
	 * @param model
	 * @param parent
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Init(IModelRO,
	 *      AActivitySpace)
	 */
	public void Init(IModelRO model, AActivitySpace parent)
	{
		m_model		  = model;
		m_asOwner     = parent;
	}
}

