package com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.providerframework.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.opener.types.classtypes.*;

/**
 * Contains utility functions that deal with intrinsic portlets.
 *
 * @author josephc
 */
public class IntrinsicPortletPrefsUtils
{
	/**
	 * The prefs suffix is appended to the portlet class ID to form the name of
	 * prefs activity space.
	 */
	public static final String PREFS_SUFFIX = "Prefs";

	/** Subspace key suffix for intrinsic portlets pref pages */
	public static final String SUBSPACE_KEY_SUFFIX = "PrefsSubSpace";

	/** Query string constant for CommunityID */
	public static final String COMMUNITYID = "CommunityID";

	/** Query string constant for PageID */
	public static final String PAGEID = "PageID";

	/** Query string constant for PortletID */
	public static final String PORTLETID = "PortletID";
	
	/** Query string constant for Admin type, 1 for true, 0 for false */
	public static final String ISADMINPREF = "IsAdminPref";

	/**
	 * Gets the portlet class identifier
	 *
	 * @param _ptPortlet Portlet object
	 * @param _objAppData AppDataState object
	 *
	 * @return Portlet class identifier (which is also the activity space of
	 *         portlet)
	 */
	public static String GetClassIdentifier(IPTGadget _ptPortlet,
		IPTState _objAppData)
	{
		// Get the portlet class identifier by calling GetHTMLText().
		Object[][] objArrMarkup = (Object[][]) _ptPortlet.GetHTMLText(_objAppData);

		String strPortletClass = (String) objArrMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT][0];

		return strPortletClass;
	}

	/**
	 * Constructs URL for accessing portlet preferences activity space.
	 *
	 * @param _ptWebService Web service object
	 * @param _nPageID Page ID
	 * @param _nCommunityID Community ID
	 * @param _nPortletID Portlet ID
	 * @param _nAssociatedContentID Associated content ID
	 * @param asOwner Activity space owner
	 *
	 * @return URL to preference page for intrinsic portlets
	 */
	public static ASURL GetPrefsURL(IPTWebService _ptWebService, int _nPageID,
		int _nCommunityID, int _nPortletID, boolean isAdminPref, int _nAssociatedContentID, AActivitySpace asOwner)
	{
		// Construct the URL to the prefs page
		ASURL asURL = new ASURL(asOwner);

		IXPPropertyBag ptProviderInfo = (IXPPropertyBag) _ptWebService.GetProviderInfo();
		String strPrefsSpace;

		boolean isCommunityPrefs = PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_COMMUNITYPREFSPAGE == _nAssociatedContentID;
		
		if (isCommunityPrefs)
		{
			strPrefsSpace = ptProviderInfo.ReadAsString(AExtensibilityConstants.PTC_I_GADGET_COMMUNITYPREFS_SPACE);
		}
		else if ( PT_ASSOCIATEDCONTENT.PT_ASSOCIATEDCONTENT_ADMINPREFSPAGE == _nAssociatedContentID )
		{
			strPrefsSpace = ptProviderInfo.ReadAsString(AExtensibilityConstants.PTC_I_GADGET_ADMINPREFS_SPACE);
		} 
		else 
		{
			strPrefsSpace = ptProviderInfo.ReadAsString(AExtensibilityConstants.PTC_I_GADGET_PREFS_SPACE);
		}

		asURL.SetLinkCreateNewSpace(strPrefsSpace, asOwner);

		// Set the control
		asURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);

		// Set editor type
		asURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE, XPConvert.ToString(EditorStartControl.EDITOR_START_FLAG_MISC));

		// Set pageID
		asURL.AddControlArgument(PAGEID, XPConvert.ToString(_nPageID));

		if (isCommunityPrefs) {
			// Set communityID
			asURL.AddControlArgument(COMMUNITYID, XPConvert.ToString(_nCommunityID));
		}
		if (isAdminPref) {
			asURL.AddControlArgument(ISADMINPREF, XPConvert.ToString(1));
		}

		// Set portletID
		asURL.AddControlArgument(PORTLETID, XPConvert.ToString(_nPortletID));

		// Make URL a subspace so that URL is correctly constructed no matter what security mode we're in. -jic
		asURL.SetSubSpaceKey(asOwner.GetName());
		
		return asURL;
	}
	
	/**
	 * Constructs URL for accessing portlet help.
	 *
	 * @param _ptWebService Web service object
	 *
	 * @return URL to help page for intrinsic portlets
	 */
	public static String GetHelpURL(IPTWebService _ptWebService)
	{
		IXPPropertyBag ptProviderInfo = (IXPPropertyBag) _ptWebService.GetProviderInfo();
		return ptProviderInfo.ReadAsString(AExtensibilityConstants.PTC_I_GADGET_HELP_URL);
	}
	
	/**
	 * Given a URL String remove psname & subspace control arguments from the query string.
	 * Use this method only when generating a link to a portlet prefs page from an intrinsic
	 * portlet which may be in 508 mode where these arguemnts confuse the control flow.
	 * 
	 * @param _strURL a URL string to a portlet pref page
	 * @param _asOwner the current activity space
	 * @return a new URL String without the psname and subspace control arguments
	 */
	public static String GetAccessStyleComplientPortletPrefURL(String _strURL, AActivitySpace _asOwner)
	{
	    String myReturnValue = _strURL;
	    if (AccessStyles.STANDARD == _asOwner.GetAccessStyle())
		{
		    return myReturnValue;
		}
	    myReturnValue = RemoveFromQueryString(myReturnValue, UIActivitySpaces.QS_PARENT_SPACE_NAME);
	    myReturnValue = RemoveFromQueryString(myReturnValue, ASConstants.SUBSPACE);
	    return myReturnValue;
	}
	
	/**
	 * Given a query string remove a gateway control parameter. For example with an 
	 * '_strControlToRemove' value of 'psname' remove the string '&psname=Gateway' from the 
	 * query string. If the query string is null return null, if the query string doesn't
	 * contain the control name return the query string unchanged.
	 * 
	 * @param _strQueryString a query string ie '?name=foo&blah=bar&psname=Gateway'
	 * @param _strControlToRemove a gateway control to remove
	 * @return the modified query string.
	 */
	private static String RemoveFromQueryString(String _strQueryString, String _strControlToRemove)
	{
	    String myPSNameString = "&" + _strControlToRemove + "=Gateway";
	    int myPSNameIndex = XPStringUtility.IndexOf(_strQueryString, myPSNameString);
	    int myPSNameLength = myPSNameString.length();
	    if (myPSNameIndex != -1)
	    {
	        String start = XPStringUtility.Substring(_strQueryString, 0, myPSNameIndex);
	        String end   = XPStringUtility.Substring(_strQueryString, myPSNameIndex + myPSNameLength);
	        _strQueryString = start + end;
	    }
	    return _strQueryString;
	}
}

