package com.plumtree.portaluiinfrastructure.layout;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.layout.impl.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;

/**
 * The LayoutService provides access to the factory for creating
 * layout related classes.
 */
public class LayoutService
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.layout.LayoutService");

	//the singleton instance
	private static ILayoutFactory m_FactoryInstance;

	// This stores whether or not the factory is in test mode.
	private static boolean m_bIsFactoryProduction = false;

	//the singleton instance
	private static ILayoutManager m_ManagerInstance;

	// This stores whether or not the manager is in test mode.
	private static boolean m_bIsManagerProduction = false;

	//the singleton instance
	private static ILayoutChooser m_ChooserInstance;

	// This stores whether or not the chooser is in test mode.
	private static boolean m_bIsChooserProduction = false;

	// Private key used to ensure that only the service can create sub-objects.
	private static Object m_oKey = new Object();

	/**
	 * @return The singleton factory for creating layout related objects.
	 */
	public static ILayoutFactory getFactoryInstance()
    {
		if (m_FactoryInstance == null)
		{
			m_bIsFactoryProduction = true;
			m_FactoryInstance = new LayoutFactory(m_oKey);
		}
		return m_FactoryInstance;
	}

    /**
     * Unit tests should use this method to overwrite/teardown existing factory singleton.
     * DO NOT USE for production code. 
     * 
     * @param testFactory - if this is null, a real LayoutFactory will be used.
     */
	public static void setFactoryTestInstance(ILayoutFactory testFactory)
	{
		if (m_bIsFactoryProduction)
		{
			String strMessage = "Can't override Layout Factory instance for testing... LayoutService has already been initialized.";
			log.Error(strMessage);
			throw new XPException(strMessage);
		}

		m_FactoryInstance = testFactory;

		if (m_FactoryInstance == null)
		{
			m_FactoryInstance = new LayoutFactory(m_oKey);
		}
	}

	/**
	 * @return The singleton manager for retrieving layouts.
	 */
	public static ILayoutManager getManagerInstance()
    {
		if (m_ManagerInstance == null)
		{
			m_bIsManagerProduction = true;
			m_ManagerInstance = new LayoutManager(m_oKey);
		}
		return m_ManagerInstance;
	}

    /**
     * Unit tests should use this method to overwrite/teardown existing manager singleton.
     * DO NOT USE for production code. 
     * 
     * @param testManager - if this is null, a real LayoutManager will be used.
     */
	public static void setManagerTestInstance(ILayoutManager testManager)
	{
		if (m_bIsManagerProduction)
		{
			String strMessage = "Can't override Layout Manager instance for testing... LayoutManager has already been created.";
			log.Error(strMessage);
			throw new XPException(strMessage);
		}

		m_ManagerInstance = testManager;

		if (m_ManagerInstance == null)
		{
			m_ManagerInstance = new LayoutManager(m_oKey);
		}
	}

	/**
	 * @return The singleton object for calculating which layouts to use.
	 */
	public static ILayoutChooser getChooserInstance()
	{
		if (m_ChooserInstance == null)
		{
			m_bIsChooserProduction = true;
			m_ChooserInstance = new LayoutChooser(m_oKey);
		}
		return m_ChooserInstance;
	}

    /**
     * Unit tests should use this method to overwrite/teardown existing chooser singleton.
     * DO NOT USE for production code. 
     * 
     * @param testChooser - if this is null, a real LayoutChooser will be used.
     */
	public static void setChooserTestInstance(ILayoutChooser testChooser)
	{
		if (m_bIsChooserProduction)
		{
			String strMessage = "Can't override Layout Chooser instance for testing... LayoutChooser has already been created.";
			log.Error(strMessage);
			throw new XPException(strMessage);
		}

		m_ChooserInstance = testChooser;

		if (m_ChooserInstance == null)
		{
			m_ChooserInstance = new LayoutChooser(m_oKey);
		}
	}

	/**
	 * This method is used by sub-objects to validate creation requests.
	 * 
	 * @param _oKey The service key to validate.
	 * @return true means the key is valid.
	 */
	public static boolean isValid(Object _oKey)
	{
		return (_oKey == m_oKey);
	}
}