package com.plumtree.portaluiinfrastructure.layout.impl;

import com.plumtree.openfoundation.reflect.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.application.varpacks.*;
import com.plumtree.portaluiinfrastructure.helpers.*;
import com.plumtree.portaluiinfrastructure.layout.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;

public class LayoutChooser implements ILayoutChooser
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.layout.impl.LayoutChooser");

	public static final int ALL_LAYOUTS_DISABLED = 0;
	public static final int ALL_LAYOUTS_ENABLED = 1;
	public static final int PAGE_LAYOUTS_ENABLED = 2;
	public static final int PORTLET_LAYOUTS_ENABLED = 3;

	protected int m_nLayoutMode = -1; 

	/**
	 * To make sure people don't create LayoutChoosers directly, we require
	 * a special key that only the LayoutService has.  Call LayoutService.getChooserInstance()
	 * instead to get the Layout Chooser.
	 * @param _oKey
	 */
	public LayoutChooser(Object _oKey)
	{
		if (!LayoutService.isValid(_oKey))
		{
			throw new XPIllegalAccessException("Layout Chooser can only be created by the Layout Service. " +
				"Call LayoutService.getChooserInstance() to access the Layout Chooser.");
		}
	}

	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.layout.ILayoutChooser#GetLayoutWebServiceBackUp(com.plumtree.portaluiinfrastructure.layout.Layouts, com.plumtree.uiinfrastructure.activityspace.AActivitySpace)
	 */
	public int GetLayoutWebServiceBackUp(Layouts _layout, AActivitySpace _asOwner)
	{
		return GetLayoutIDForExperience(_layout, _asOwner);
	}

	
	public int GetLayoutWebService(Layouts _layout, AActivitySpace _asOwner)
	{
		if (_layout.GetLayoutType() == Layouts.LAYOUT_TYPE.PORTLET_TYPE) {
			if (!GetPortletLayoutsEnabled(_asOwner))
			{
				// Layout mode disabled
				return -1;
			}
		} else {
			if (!GetPageLayoutsEnabled(_asOwner))
			{
				// Layout mode disabled
				return -1;
			}
		}

		if (!GetLayoutsEnabledForExperience(_asOwner))
		{
			// Layout mode disabled for this Experience
			return -1;
		}

		return GetLayoutIDForExperience(_layout, _asOwner);
	}

	public boolean GetPageLayoutsEnabled(AActivitySpace _asOwner)
	{
		CacheLayoutMode(_asOwner);

		return ((m_nLayoutMode == ALL_LAYOUTS_ENABLED) ||
				(m_nLayoutMode == PAGE_LAYOUTS_ENABLED));
	}

	public boolean GetPortletLayoutsEnabled(AActivitySpace _asOwner)
	{
		CacheLayoutMode(_asOwner);

		return ((m_nLayoutMode == ALL_LAYOUTS_ENABLED) ||
				(m_nLayoutMode == PORTLET_LAYOUTS_ENABLED));
	}

	/**
	 * This method caches the layout mode if it isn't already cached.
	 * 
	 * @param _asOwner
	 */
	protected void CacheLayoutMode(AActivitySpace _asOwner)
	{
		// If we haven't looked this up yet
		if (m_nLayoutMode <= -1)
		{
			// Disabled by default if we can't load the setting.
			int nMode = ALL_LAYOUTS_DISABLED;
	
			String strMode = PTConfigService.getPTInstance().GetConfigVariableAsString(
				PTConfigConstants.SYSTEM_PROPERTIES_SECTION, PTConfigConstants.LAYOUT_MODE_SUBSECTION,
				PTConfigConstants.VALUE_ATTRIBUTE, _asOwner);
	
			if ((strMode != null) && (strMode.length() > 0))
			{
				// NOTE: No exception catching here because OpenConfig won't allow
				// the setting to be non-integer, if it's there.
				nMode = XPConvert.ToInteger(strMode);
			}

			// Cache the setting so we don't need to look it up again.
			m_nLayoutMode = nMode;
		}
	}

	protected int GetLayoutIDForExperience(Layouts _layout, AActivitySpace _space)
	{
		try
		{
			IPTSubPortalInfo subPortal = (((IPTSession)_space.GetUserSession()).GetSessionInfo()
				.GetCurrentUserCachedSubPortalInfo());
			int currLayoutFolder = _layout.GetWebServiceFolderID();
			String strLayoutOptions = subPortal.GetLayoutOptions();

			// make sure layout options are selected
			if ((null != strLayoutOptions) && (!"".equals(strLayoutOptions)))
			{			
				// tokenize the string of selected layouts for this experience definition
				XPStringTokenizer st = new XPStringTokenizer(strLayoutOptions, ",");
				int numLayouts = st.GetTokenCount();
				String layoutArray[][] = new String[numLayouts][2];
				int i=0;
				
				// populate 2D array
				while(st.HasMoreTokens())
				{
					XPStringTokenizer st2 = new XPStringTokenizer(st.GetNextToken(), ":");
					if (st2.HasMoreTokens())
						layoutArray[i][0] = st2.GetNextToken(); // folderID
					if (st2.HasMoreTokens())
						layoutArray[i][1] = st2.GetNextToken(); // web service ID
					i++;
				}
	
				// check array for web service match in current layout folder
				for (int j=0; j<numLayouts; j++)
				{
					if (currLayoutFolder == XPConvert.ToInteger(layoutArray[j][0]))
						return XPConvert.ToInteger(layoutArray[j][1]);
				}
			}
		}
		catch (Exception e)
		{
			log.Error(e, "Unable to get the layout information to load layout.");
			throw XPException.GetInstance(e);
		}
		
		// DCA - we didn't find any a lyout option from the XD, so let's return the default ones.
		// DCA - if layout type is portlet and no override was found in the XD, then return -1 as default;
		if (_layout.GetLayoutType() == Layouts.LAYOUT_TYPE.PORTLET_TYPE) {
			return -1;
		} else {
			return _layout.GetDefaultWebServiceID();
		}
	}

	protected boolean GetLayoutsEnabledForExperience(AActivitySpace _space)
	{
		try
		{
			IPTSubPortalInfo subPortal = (((IPTSession)_space.GetUserSession()).GetSessionInfo()
				.GetCurrentUserCachedSubPortalInfo());
			boolean bLayoutsEnabled = subPortal.GetLayoutEnabled();

			return bLayoutsEnabled;
		}
		catch (Exception e)
		{
			log.Error(e, "Unable to get the SubPortalInfo object to check for layout mode.");

			throw XPException.GetInstance(e);
			// NOTE: This CR comment is copied from other code.  Not sure if it's relevant.
			// TODOCR: This method fails sometimes while checking ACLS in the PT object cache
			// Should we handle this error?
//			return -1;
		}
	}

	public IPTQueryResult GetAvailableWebServices(Layouts _layout, AActivitySpace _space)
	{
		IPTAdminCatalog catalog = ((IPTSession) _space.GetUserSession()).GetAdminCatalog();

		IPTObjectManager webServiceManager = catalog.GetObjectManager(PT_CLASSIDS.PT_WEBSERVICE_ID);

		// NOTE: We are currently querying for all web services in the folder.
		// This could be large and could cause a performance issue.
		return webServiceManager.Query(PT_PROPIDS.PT_PROPID_NAME | PT_PROPIDS.PT_PROPID_DESCRIPTION | PT_PROPIDS.PT_PROPID_OBJECTID,
			_layout.GetWebServiceFolderID(), PT_PROPIDS.PT_PROPID_NAME, 0, -1, null);
	}
}