/**
 * 
 */
package com.plumtree.portaluiinfrastructure.login;


import com.plumtree.openfoundation.io.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.xml.dom.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.uiinfrastructure.statichelpers.*;



/**
 * PTLocaleMap is used to facilitate custom mapping of one locale
 * to another locale by user. This mappings can be set in LocaleMap.xml
 * in i18n directory of WCI installation. When user specifies a custom mapping
 * from a particular locale to a different locale. This class stores that mapping
 * and it can be used by WCI to get the mapped locale to be set for the user.
 *  
 * @author Ravi Tunuguntla
 *
 */
public class PTLocaleMap {
	
	// This is single ton instance that points to PTLocleMap
	private static PTLocaleMap m_Instance;
	
	// Hashtable that contains all the locale Mappings
	private XPHashtable m_htKeytoValueLookup;
	
	private static final String ATTRIB_FROM = "from";
	private static final String ATTRIB_TO = "to";
	
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
			"com.plumtree.portaluiinfrastructure.login.PTLocaleMap");
	
	
	private PTLocaleMap() {
		
		String strMapPath = AppWarmUp.GetSettingValueAsString(AppWarmUp.KEY_PATH_PORTAL_I18N);
		String strMapXMLFile = GeneralHelpers.FormatPath(strMapPath +
				"LocaleMap.xml", AppWarmUp.PATH_SEPARATOR, false);
		log.Debug("PTLocaleMap:PTLocaleMap Retrieving configuration from " + strMapXMLFile);
		
		try {
			
			XPDocument document = XPBuilder.LoadDocument(strMapXMLFile);
			SetUpConfiguration(document);

		}catch (XPFileNotFoundException ex){
			log.Warn(ex, "Failed to Load " + strMapXMLFile + 
					" Since it is not found, Locale Mapping will be turned off!");

		}catch(XPException ex) {
			log.Warn(ex, "Failed to Load " + strMapXMLFile + ", So Locale Mapping will be turned off");
		}
	}
	
	/**
	 * This method takes LocaleMap.xml document object and loads all the locale
	 * mappings. 
	 * @param document
	 */
	private void SetUpConfiguration(XPDocument document) {
		if(null != document) {
		XPNode node;
		m_htKeytoValueLookup = new XPHashtable();
		
		XPNodeList nlKeys = document.GetElementsByTagName(ATTRIB_FROM);
		XPNodeList nlValues = document.GetElementsByTagName(ATTRIB_TO);
			for (int i=0; i< nlKeys.GetLength(); i++) {
				node = nlKeys.Item(i);
				String key = node.GetFirstChild().GetNodeValue();
				node = nlValues.Item(i);
				String value = node.GetFirstChild().GetNodeValue();
				m_htKeytoValueLookup.PutElement(key, value);			
				log.Info("PTLocaleMap:PTLocaleMap Locale mapping found: " + key + " -> " + value);
			}
		}
	}
	
	/**
	 * This static method returns singleton instance of PTLocaleMap.
	 * @return Singleton instance of PTLocaleMap.
	 */
	public static PTLocaleMap GetInstance() {
		if(null == m_Instance) {
			m_Instance = new PTLocaleMap();			
		}
		return m_Instance;
	}
	
	/**
	 * This method takes input a locale string and returns
	 * mapped locale for the input locale. IsLocaleMapped method
	 * should be used before using this method to check if a mapping
	 * exists, only if a mapping exists this method can be used to retrieve
	 * the mapping
	 * @param locale from locale for which there is a mapping
	 * @return locale mapped locale for a given from locale.
	 */
	public String GetMappedLocale(String locale) {
		// If locale map is not initialized then we return mapped locale as the one
		// input for this method to avoid Null value.
		if(null == m_htKeytoValueLookup) {
			return locale;
		}
		String mappedLocale = (String)m_htKeytoValueLookup.GetElement(locale);

		// If the locale is not mapped i.e., some one is calling this method without
		// checking IsLocaleMapped, we return input locale		
		if(null == mappedLocale) {
			return locale;
		}
		// Mapped locale is there so we return it.
		return mappedLocale;
	}
	
	/**
	 * This method takes input a locale and returns true if that locale
	 * has mapping, if not or if there is no LocaleMapping.xml file in 
	 * WCI i18n directory then it returns false
	 * @param locale from locale for which there is a mapping
	 * @return true if there is a mapping, else false.
	 */
	public boolean IsLocaleMapped(String locale) {

		if(null == m_htKeytoValueLookup) {
			return false;
		}
		return m_htKeytoValueLookup.ContainsKey(locale);
	}

}
