package com.plumtree.portaluiinfrastructure.navtype;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.xml.dom.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;

/**
 * Singleton navigation types loader and manager for pluggable navigation.  To create a pluggable 
 * navigation type, implement INavTypes, implement multiple views and specify them in your INavTypes 
 * implementation, and add the JAR that contains these views and INavTypes implementation to 
 * CustomActivitySpaces.xml.  The new nav will then show up in the interface.  Please insure your 
 * navigation ID does not conflict with an existing ID.
 * ````````````````````````````````````````````````````````````````````	``````````````````````````````````````````````
 * @author DavidP
 * @author AdolfoC
 */
public class NavManager 
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.navtype.NavManager");

	/** constants for interfaces to look for */
	private static final String INAVTYPE = "com.plumtree.portaluiinfrastructure.navtype.INavTypes";

	/**
	 * Use this class static to access the one and only instance of
	 * ActivitySpaceManager.
	 */
	public static final NavManager INSTANCE = new NavManager();

	/** whether or not the system was able to start up correctly. */
	public boolean m_bSystemStartupSuccesful = true;

	/** List of navigation types: hash and array */
	private XPHashtable m_xphNavTypes = null;
	private XPArrayList m_xpaNavTypes = null;
 
	/** Whether or not this object has been initialized */
	private boolean m_bInitialized = false;

	/** Counters used in the diagnostic page */
	private int m_nNumCustomNavTypes = 0;

	/** constant for portaluiinfrastructure package name */
	private static final String PORTALUIINFRASTRUCTURE = "portaluiinfrastructure";
	

	/**
	 * Private constructor to ensure there is only one instance.
	 */
	private NavManager()
	{
		m_xphNavTypes = new XPHashtable();
		m_xpaNavTypes = new XPArrayList();
	}

	/**
	 * Return the requested NavTypes.
	 * @param navID the unique identifier for this navigation scheme.
	 *
	 * @return INavTypes
	 */
	public INavTypes GetNavType(int navID)
	{
		INavTypes myNavType = null;

		try
		{
			try
			{
				Object temp = m_xphNavTypes.GetElement(new Integer(navID));
				myNavType = (INavTypes) temp;
			}
			catch (Exception ex)
			{
				throw XPException.GetInstance(ex);
			}
		}
		catch (XPClassCastException e)
		{
			
				log.Error(e, "GetNavType: XPClassCastException: The navtype " +
					navID + " is being miscast.");
			
		}
		catch (XPException e)
		{
			
				log.Error(e, "Couldn't retrieve navtype " + navID + ".");
			
		}

		return myNavType;
	}
	
	/**
	 * Gets the navigation types loaded.  UNRELATED to NavPageType or NavCategoryType.
	 * This simply gets the pluggable navigation elements, each of which have an ID
	 * that uniquely identifies them.
	 * 
	 * @return IXPEnumerator
	 */
	public IXPEnumerator GetNavTypes()
	{
		if ((log.IsErrorEnabled()) && (m_xphNavTypes.GetSize() == 0))
		{
			log.Error("GetNavTypes: No navigation types have been loaded" );
		}

		return m_xphNavTypes.GetElements();
	}

	
	/**
	 * Gets the navigation types loaded as sorted array list
	 * 
	 * @return XPArrayList
	 */
	public XPArrayList GetNavTypesAsSortedArray()
	{
		if ((log.IsErrorEnabled()) && (m_xphNavTypes.GetSize() == 0))
		{
			log.Error("GetNavTypes: No navigation types have been loaded" );
		}

		return m_xpaNavTypes;
	}

	/**
	 * This method initializes the NavManager and should only be called by the
	 * warmup servlet.
	 * 
	 * This method can only be called once.
	 * 
	 * @param strLibDir 
	 * @param strConfigFile 
	 */
	public void LoadClasses(String strLibDir, String strConfigFile)
	{
		if (m_bInitialized)
		{
			log.Error("NavManager LoadClasses called twice.");

			return;
		}

		m_bInitialized = true;

		// TODO: Bug 41706  -- bIsCustom should be rolled together
		// with bOverwrite, the same as in ASManager and TagLibManager.
		// We can't now, because standard and custom classes both set this
		// to true.  Standard classes should set it to false.

		// Second to last argument: *allow* navigation overrides.
		InternalLoadClasses(strLibDir, strConfigFile, true, false);
	}

	/**
	 * This method initializes the NavManager and should only be called by the
	 * warmup servlet.
	 * 
	 * @param strLibDir 
	 * @param strConfigFile 
	 */
	public void LoadCustomClasses(String strLibDir, String strConfigFile)
	{
		// Reset custom object counters.
		m_nNumCustomNavTypes = 0;

		InternalLoadClasses(strLibDir, strConfigFile, true, true);
	}

	/**
	 * This method initializes the NavManager and should only be called by the
	 * warmup servlet.
	 * 
	 * @param strLibDir 
	 * @param strConfigFile
	 * @param boolean bOverWrite false implies log a warning if a nav scheme overwrite is attempted
	 * @param boolean bIsCustom true implies that the Custom Object Counters will be incremented for these classes.
	 */
	private void InternalLoadClasses(String strLibDir, String strConfigFile,
		boolean bOverwrite, boolean bIsCustom)
	{
		// TODO: Bug 41706  -- bIsCustom should be rolled together
		// with bOverwrite, the same as in ASManager and TagLibManager.
		// We can't now, because standard and custom classes both set this
		// to true.  Standard classes should set it to false.

		// Currently we store names to full package names and re-reflect.
		// We should store objects and clone off them.
		log.Info("Loading from library: " + strLibDir);
		log.Info("Loading config file: " + strConfigFile);

		int length = 0;

		XPDocument document = XPBuilder.LoadDocument(strConfigFile);
		XPNodeList libFiles = document.GetElementsByTagName("libfile");

		INavTypes discoveredNavType;
		Object overwrittenNavType;

		if (libFiles != null)
		{
			for (int nLibFile = 0; nLibFile < libFiles.GetLength(); nLibFile++)
			{
				String strLibFileName = ((XPElement) libFiles.Item(nLibFile)).GetAttributeNode(
						"name").GetNodeValue();
	
				log.Info("Loading lib file: " + strLibFileName);
	
				// Load the instances of the nav types into an array to be consumed by navigation
				Object[] navtypes = XPDynamicDiscovery.GetInstancesFromLib(strLibDir,
					PORTALUIINFRASTRUCTURE, strLibFileName, INAVTYPE);
				
				// Handle the case where no navigation is present
				if (navtypes == null)
				{				
					String msg = "*** SEVERE FATAL ERROR: You specified '" +strLibFileName+"' in " + strConfigFile + ", but it is not present in your classpath (or IIS src\\web\\bin directory).  You need to correct this before the portal will start. ***";
					m_bInitialized = false;
					throw new XPException(msg);
				} 
					
				// Proceed normally
				log.Debug("Putting infrastructure objects from lib files into hash tables");
				length = 0;
				if (navtypes != null)
				{
					length = navtypes.length;
				}
	
				for (int nNavType = 0; nNavType < length; nNavType++)
				{
					try {
					
						discoveredNavType = (INavTypes) navtypes[nNavType];
						overwrittenNavType = null;
		
						if (discoveredNavType != null)
						{
							overwrittenNavType = m_xphNavTypes.PutElement(new Integer(discoveredNavType.GetID()), discoveredNavType);
							
							// Use en as the language. No locale set during startup.
							log.Info("  Loading navigation type with navID of " + discoveredNavType.GetID() + "." );
	
							if ((overwrittenNavType != null) && !bOverwrite)
							{
								log.Warn("Navigation Override: Navigation " + discoveredNavType.GetName("en-us") 
								        + " overrides a previous navigation with an ID of " 
								        + discoveredNavType.GetID() 
								        + ".");
							}

							if (bIsCustom)
							{
								// Increment custom object counters.
								m_nNumCustomNavTypes++;
							}
						}
					} catch (Exception e) {
						log.Error(e, "Error loading navigation scheme class: "+navtypes+", skipping.");
					}
				}
	
			}
		}
		
		// you must have navigation present
		if (m_xphNavTypes.GetSize() == 0)
		{
			String msg = "*** SEVERE FATAL ERROR: You have no navigation JARs (or DLLs) present on your classpath (or IIS \\src\\web\\bin directory).  Please check: (1) portalnavigation.jar (or .dll) is listed in ActivitySpaces.xml (2) portalnavigation.jar (or .dll) is present on the classpath or WEB-INF directory (or IIS \\src\\web\\bin).  You must manually add a pluggable navigation package to ActivitySpaces.xml or CustomActivitySpaces.xml for the portal to start. ***";
			m_bInitialized = false;
			throw new XPException(msg);
		}

		int minNavID = 9999;
		int maxNavID = -9999;

		// Calculate max and min nav ids.
		IXPEnumerator navTypes = m_xphNavTypes.GetElements();
		INavTypes currentNavType;
		while (navTypes.MoveNext())
		{
			currentNavType = (INavTypes) navTypes.GetCurrent();

			if (null != currentNavType)
			{
				if (minNavID > currentNavType.GetID())
					minNavID = currentNavType.GetID();
				if (maxNavID < currentNavType.GetID())
					maxNavID = currentNavType.GetID();
			}
		}

		// m_xphNavTypes gets added to and maintained while adding
		// custom classes, so we re-use it as is.  m_xpaNavTypes is
		// just an ordered list version of m_xphNavTypes, so we
		// clear it every time.
		m_xpaNavTypes.Clear();

		// Sort by navigation ID
		INavTypes orderedNavType;
		if (minNavID != 9999 && maxNavID != -9999)
		{
			// Sort them and re-add them to the nav types hash in order
			// This assumes order is preserved on our internal hashmap
			for (int nNavTypeID = maxNavID; nNavTypeID >= minNavID; --nNavTypeID)
			{
				orderedNavType = (INavTypes)m_xphNavTypes.GetElement(new Integer(nNavTypeID));
				if (orderedNavType != null)
					m_xpaNavTypes.Add(orderedNavType);				
			}
		}
	}

	/**
	 * This method returns the number of custom NavTypes
	 * currently loaded.  This number is reset every time
	 * custom objects are loaded.
	 *  
	 * @return The number of custom NavTypes currently loaded.
	 */
	public int GetNumCustomNavTypes()
	{
		return m_nNumCustomNavTypes;
	}
}