package com.plumtree.portaluiinfrastructure.pei;

import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;


/**
 * This interface was created to allow developers the ability to create
 * functions that can be dynamically discovered by the Plumtree application
 * for the purposes of performing some functionality when a particular
 * object creation or modification based event occurs.
 * <br><br>
 * NOTE: Should this interface be implemented more than once within the
 * Dynamic Discovery process, the implementations will be processed in the
 * order that they are displayed within the representative XML file. All
 * implementations will be called.
 * <br><br>
 * NOTE: The implementation of these particular functions
 * should be held to a minimum and done very carefully as these functions will
 * be called every single time that a Create, Edit and Store of a object is called
 * and this could seriously hurt performance on the Administration or Portal
 * Servers.  
 * <br><br>
 * NOTE: In the case where multiple implementations return a String object, all 
 * the valid Strings (not null and not empty) will be appended together 
 * (separated by a blank space) and the final String will be the error message 
 * displayed on the page.
 */
public interface INewEditObjectActions
{
    /**
     * This will perform some functionality just after the object has been
     * stored in the database.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _ptObject - The actual object that has been stored along with all
     *        of its property information.
     */
    public void OnAfterStoreObject(AActivitySpace _asCurrentSpace,
                                   IPTSession _ptUserSession,
                                   IPTObject _ptObject);

    /**
     * This will perform some functionality just before the object is stored to
     * the database. The function will return a valid IPTObject that has
     * possible been modified to then be stored.  
     * <br><br>
     * NOTE: Be very careful in implementing this so that invalid properties 
     * are not stored with the object.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _ptObject - The actual object that is trying to be stored along
     *        with all of its property information.
     *
     * @return IPTObject - The returned object after possible modifications
     *         have been made.
     */
    public IPTObject OnBeforeStoreObject(AActivitySpace _asCurrentSpace,
                                         IPTSession _ptUserSession,
                                         IPTObject _ptObject);

    /**
     * This function will perform some functionality just as we launch an Editor 
     * to create a new PTObject. The function will return a String containing an error 
     * message if this should not occur.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _nFolderID - Admin Folder ID of where the object will be created.
     * @param _nClassID - Class ID of the object that is being created.
     *
     * @return String - Error message if the creation should not continue,
     *         empty string or null if it should.
     */
    public String OnCreateObject(AActivitySpace _asCurrentSpace,
                                 IPTSession _ptUserSession, int _nFolderID,
                                 int _nClassID);

    /**
     * This function will perform some functionality just as we launch an Editor 
     * to edit and an existing PTObject. The function will return a String containing 
     * a error message if this should not occur.
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _nClassID - Class ID of the object that is being edited.
     * @param _nObjectID - Object ID of the object that is being edited.
     *
     * @return String - Error message if the action should not continue,
     *         empty string or null if it should.
     */
    public String OnEditObject(AActivitySpace _asCurrentSpace,
                               IPTSession _ptUserSession, int _nClassID,
                               int _nObjectID);
}

