package com.plumtree.portaluiinfrastructure.ptmulticolumnlist;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.tree.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.expandablelist.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.tree.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * This is the implementation of the Expandable List
 * framework for the the Select PTObject list.
 * The Select PTObject is a list of objects.
 * New objects can be added to the list by selecting
 * them from the associated Object tree.
 * The list is by default sorted by name ascending.
 *
 * @author Julie Fournier
 */
public class PTMultiColumnListModel extends ExpListModel
	implements IPTMultiColumnListModelRO
{
	/**
	 * Class name for the activity space */
	public static final String STR_MVC_CLASS_NAME = "PTMultiColumnListModel";

	/**
	 * Key name used by the Activity Space to store the tree. */
	public static final String SUBSPACE_KEY_SELECTTREE = "PTSelectTree";

	/**
	 * This column contains the Object Class IDs.
	 */
	public static final int EXPLIST_COL_CLASSID = 1;

	/**
	 * This column contains the Object IDs.
	 */
	public static final int EXPLIST_COL_OBJECTID = 2;

	/**
	 * This is the String that we doisplay in the list.
	 * E.g.: Object name or folder path.
	 */
	public static final int EXPLIST_COL_STRING = 3;

	/**
	 * Default initial size for the sorted array.
	 */
	public static final int EXPLIST_DEFAULT_INITIAL_SIZE = 50;

	/**
	 * Number total of columns for hide objects array.
	 */
	public static final int HIDE_COL_MAX = 2;

	/**
	 * This column of the passed Objects to Hide contains the class IDs.
	 */
	public static final int HIDE_COL_CLASSID = 0;

	/**
	 * This column of the passed Objects to Hide contains the object IDs.
	 */
	public static final int HIDE_COL_OBJECTID = 1;

	/**
	 * Create new array column: class ID
	 */
	public static final int CREATE_NEW_COL_CLASSID = 0;

	/**
	 * Create new array column: AS Name
	 */
	public static final int CREATE_NEW_COL_ASNAME = 1;

	/**
	 * Create new array column: text to display with the create new link
	 */
	public static final int CREATE_NEW_COL_LABEL = 2;

	/**
	 * Create new array column: icon to display for the create new link
	 */
	public static final int CREATE_NEW_COL_ICON = 3;

	/**
	 * Create new array column: extra control arguments for the create action.
	 * Use CREATE_NEW_COL_MAXWITHPARAMS to set the size of the array when using
	 * this element.
	 */
	public static final int CREATE_NEW_COL_PARAMS = 4;

	/**
	 * Size of each m_arrCreateNewInfo row
	 */
	public static final int CREATE_NEW_COL_MAX = 4;

	/**
	 * Size of each m_arrCreateNewInfo row with optional control parameters.
	 */
	public static final int CREATE_NEW_COL_MAXWITHPARAMS = 5;

	/**
	 * Optional control parameter index for the parameter name. Used in the
	 * create new array column: icon to display for the create new link
	 */
	public static final int PARAM_COL_PARAMNAME = 0;

	/**
	 * Optional control parameter index for the parameter value. Used in the
	 * create new array column: icon to display for the create new link
	 */
	public static final int PARAM_COL_PARAMVALUE = 1;

	/**
	 * Icon to display on each row, just before the input text. */
	protected PTImageType m_imgRowIcon;

	/**
	 * Tree Space ID. The first time the ID is empty and a new AS is opened,
	 * but after that, we save the ID and the same tree AS is reopened each time. */
	protected String m_sTreeSpaceID = "";

	/**
	 * Tree sub title
	 */
	protected String m_sTreeSubTitle;

	/**
	 * Tree title
	 */
	protected String m_sTreeTitle;

	/**
	 * Objects to hide for the object tree.
	 */
	protected String m_strObjectsToHide = "";

	/**
	 * Query Filter for the object tree.
	 */
	protected String m_strTreeQueryFilter = "";

	/**
	 * Information about the create new links to display
	 */
	protected Object[][] m_arrCreateNewInfo;

	/**
	 * This is used on add to populate the non plumtree object columns with
	 * default values in the newly added row. Note, this array MUST be
	 * initialized explicitly by the client - it can't be null if it is used.
	 */
	protected String[] m_arrDefaultValue;

	/**
	 * Class ID of the Objects displayed on the list.
	 */
	protected int[] m_arrObjectClassIDs;

	/**
	 * Indicates whether to include a Plumtree object picker in this list
	 * control.  If true, 2 hidden columns and a visible name column will be
	 * added to the list, along with the complete facility for picking Plumtree
	 * objects.  If false, a simple text-entry multicolumn list will be
	 * produced.
	 */
	protected boolean m_bIncludePTSelect = true;

	/**
	 * Selection type for the object tree.  True to enable multiselect, false to
	 * disable multiselect.
	 */
	protected boolean m_bTreeMultiSelect = true;

	/**
	 * The index of the HTML row to display in edit mode.  This is set when the
	 * user clicks on the Edit icon for a given row.
	 */
	protected int m_intEditRow = -1;

	/**
	 * Total number of columns in addition to the plumtree select columns.  For
	 * each one of these columns, a text box will be displayed in EDIT mode,
	 * and a string in DISPLAY mode.  */
	protected int m_intNumColumns = 2;

	/**
	 * The zero-based position of the Plumtree object name column.  Ignored if
	 * b_bIncludePTSelect is false, and automatically set to zero if
	 * it is set to a number greater than intNumColumns.
	 */
	protected int m_intPTObjectPosition = 0;

	/**
	 * This value determines the security access requirements for the objects to
	 * be added to the list... (a filter for the tree control).
	 */
	protected int m_nAccessLevel = 0;

	/**
	 * Class IDs and Object IDs of objects to display in the tree.
	 */
	protected String m_sDisplayObjects;

	/**
	 * Leaf type for the object tree.
	 */
	protected int m_treeLeafType = 0;

	public PTMultiColumnListModel()
	{
		super();
		m_treeLeafType = TreeConstants.N_LEAFSELECT;
		m_nAccessLevel = PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT;
	}

	/**
	 * Parses the string of mapped fields and returns a two dimensional array
	 * in the format expected by the Plumtree server
	 *
	 * @see com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskOnStartEditor(XPHashtable)
	 */
	public static Object[][] ParseFields(String strMappedFields)
	{
		Object[][] myArray			  = null;
		int intTokenIndex			  = 0;
		XPStringTokenizer myTokenizer = new XPStringTokenizer(strMappedFields,
				",");
		myArray						  = new Object[1][myTokenizer.GetTokenCount()];

		for (XPStringTokenizer i = myTokenizer; i.HasMoreTokens();)
		{
			String strToken = i.GetNextToken();
			myArray[0][intTokenIndex] = XPStringUtility.Trim(strToken);
			intTokenIndex++;
		}

		return myArray;
	}

	/**
	 * Helper function for extracting and adding extra control parameters from
	 * the create object string.
	 *
	 * @param _nClassID
	 * @param myURL
	 */
	public void AddControlParamsToURL(int _nClassID, ASURL myURL)
	{
		String[][] arParams = null;

		for (int i = 0; i < m_arrCreateNewInfo.length; i++)
		{
			int nClassIDFromArray = XPConvert.ToInteger(m_arrCreateNewInfo[i][CREATE_NEW_COL_CLASSID]);

			if (nClassIDFromArray == _nClassID)
			{
				if (m_arrCreateNewInfo[i].length >= CREATE_NEW_COL_MAXWITHPARAMS)
				{
					arParams = (String[][]) m_arrCreateNewInfo[i][CREATE_NEW_COL_PARAMS];

					break;
				}
			}
		}

		if (null != arParams)
		{
			for (int j = 0; j < arParams.length; j++)
			{
				if ((null != arParams[j]) && (2 == arParams[j].length))
				{
					String strName  = arParams[j][PARAM_COL_PARAMNAME];
					String strValue = arParams[j][PARAM_COL_PARAMVALUE];

					// Add the parameter to the url.
					myURL.AddControlArgument(strName, strValue);
				}
			}
		}
		else
		{
			// Warning
		}
	}

	/**
	 * Clear the list and add all the values selected from the tree, as well as
	 * values or defaults for the other columns in the control.
	 *
	 * @param _htFormData Form data
	 */
	public void AddRowsOnRepost(XPHashtable _htFormData)
	{
		if (m_bIncludePTSelect)
		{
			//Validate that the proposed position of the selected Plumtree
			//object is not greater than the number of other columns
			if (m_intPTObjectPosition > m_intNumColumns)
			{
				m_intPTObjectPosition = 0;
			}

			SaveTreeID(_htFormData);

			TreeAS asTree = (TreeAS) m_asOwner.GetSubSpace(GetUniqueString(
						SUBSPACE_KEY_SELECTTREE));

			if (null != asTree)
			{
				ITreeModelRO myTree    = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
				XPArrayList mySelected = myTree.GetSelectedItems();
				Object[] oNewRow;
				int nClassID;

				//First, save the data so we can re-populate the non-tree items
				XPHashtable[] arrHashtables = new XPHashtable[m_intNumColumns +
					3];

				for (int i = 0; i < (m_intNumColumns + 3); i++)
				{
					arrHashtables[i] = new XPHashtable();
				}

				for (int i = 0; i < m_saData.GetCount(); i++)
				{
					for (int j = 1; j < (m_intNumColumns + 4); j++) //skip the index column
					{
						arrHashtables[j - 1].PutElement(m_saData.GetItem(i,
								EXPLIST_COL_STRING + m_intPTObjectPosition),
							m_saData.GetItem(i, j));
					}
				}

				// JF- We clear the array before we start adding the selected items from the tree.
				super.GetItems().ClearArray();

				for (int i = 0; i < mySelected.GetSize(); i++)
				{
					oNewRow										  = new Object[m_intNumColumns +
						4];
					oNewRow[EXPLIST_SORTEDARRAY_PROPID_INDEX]     = new Integer(m_nNextIndex);
					m_nNextIndex++;
					nClassID												  = ((TreeLeafNode) mySelected.GetElement(i)).m_nClassID;
					oNewRow[EXPLIST_COL_CLASSID + m_intPTObjectPosition]	  = new Integer(nClassID);
					oNewRow[EXPLIST_COL_OBJECTID + m_intPTObjectPosition]     = new Integer(((TreeLeafNode) mySelected.GetElement(
								i)).m_nObjectID);
					oNewRow[EXPLIST_COL_STRING + m_intPTObjectPosition]		  = ((TreeLeafNode) mySelected.GetElement(i)).GetName();

					//Add the other empty columns
					String strValue;
					int intColIndex;

					for (int j = 0; j < m_intNumColumns; j++)
					{
						if (j < m_intPTObjectPosition)
						{
							intColIndex = j + 1;
						}
						else
						{
							intColIndex = j + 4;
						}

						//TODO - get the right hashtable - only j when m_intPTObjectPosition is 1
						strValue = (String) arrHashtables[intColIndex - 1].GetElement(oNewRow[EXPLIST_COL_STRING +
								m_intPTObjectPosition]);

						if ((null == strValue) || strValue.equals(""))
						{
							strValue = m_arrDefaultValue[j];
						}

						oNewRow[intColIndex] = strValue;
					}

					super.GetItems().AddRow(oNewRow);
				}
			}
		}
		else
		{
			String[][] arrDefault = new String[m_intNumColumns][1];

			for (int i = 0; i < m_intNumColumns; i++)
			{
				arrDefault[i][0] = m_arrDefaultValue[i];
			}

			AddRowsToList(arrDefault);
		}
	}

	/**
	 * Adds an array of new rows to the list.  This is only called when
	 * m_bIncludePTSelect is FALSE.
	 *
	 * @param _aData
	 */
	public void AddRowsToList(Object[][] _aData)
	{
		Object[] oRow;

		for (int i = 0; i < _aData[0].length; i++)
		{
			oRow									   = new Object[m_intNumColumns +
				1];
			oRow[EXPLIST_SORTEDARRAY_PROPID_INDEX]     = new Integer(m_nNextIndex);
			m_nNextIndex++;

			for (int j = 0; j < m_intNumColumns; j++)
			{
				oRow[j + 1] = _aData[j][i];
			}

			super.GetItems().AddRow(oRow);
		}
	}

	/**
	 * Tells whether the Object identified by the class ID
	 * and the object ID in argument is in the list or no.
	 *
	 * @param _nClassID
	 * @param _nObjectID
	 *
	 * @return boolean
	 */
	public boolean ContainsObject(int _nClassID, int _nObjectID)
	{
		for (int i = 0; i < m_saData.GetCount(); i++)
		{
			if ((_nClassID == XPConvert.ToInteger(m_saData.GetItem(i,
						EXPLIST_COL_CLASSID + m_intPTObjectPosition))) &&
					(_nObjectID == XPConvert.ToInteger(m_saData.GetItem(i,
						EXPLIST_COL_OBJECTID + m_intPTObjectPosition))))
			{
				return true;
			}
		}

		return false;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new PTMultiColumnListModel();
	}

	/**
	 * This method will be called when you come back to the main Editor from a sub Editor.
	 *
	 */
	public void DoTaskOnComeBackFromSubEditor()
	{
		String sEditorASName;
		EditorAS myEditorAS;
		ObjEditorModel myObjEditorModel;
		Object[] oNewRow;
		int nClassID;

		// JF- Check if they are any newly created objects
		for (int i = 0; i < m_arrCreateNewInfo.length; i++)
		{
			sEditorASName = (String) m_arrCreateNewInfo[i][CREATE_NEW_COL_ASNAME];

			if (m_asOwner.GetSubSpaceExists(sEditorASName))
			{
				myEditorAS			 = (EditorAS) m_asOwner.GetSubSpace(sEditorASName);
				myObjEditorModel     = (ObjEditorModel) myEditorAS.GetModel(myEditorAS.m_strModelName);

				oNewRow										  = new Object[m_intNumColumns];
				oNewRow[EXPLIST_SORTEDARRAY_PROPID_INDEX]     = new Integer(m_nNextIndex);
				m_nNextIndex++;
				nClassID												  = myObjEditorModel.GetClassID();
				oNewRow[EXPLIST_COL_CLASSID + m_intPTObjectPosition]	  = new Integer(nClassID);
				oNewRow[EXPLIST_COL_OBJECTID + m_intPTObjectPosition]     = new Integer(myObjEditorModel.GetObjectID());

				if (PT_CLASSIDS.PT_CATALOGFOLDER_ID == nClassID)
				{
					oNewRow[EXPLIST_COL_STRING + m_intPTObjectPosition] = ((IPTFolder) myObjEditorModel.GetObject()).GetPath();
				}
				else
				{
					oNewRow[EXPLIST_COL_STRING + m_intPTObjectPosition] = myObjEditorModel.GetObjectName();
				}

				super.GetItems().AddRow(oNewRow);

				m_asOwner.DeleteSubSpace(sEditorASName);
			}
		}
	}

	/**
	 * Returns the accesslevel of the objects for the tree control to display.
	 *
	 * @return int
	 */
	public int GetAccessLevel()
	{
		return m_nAccessLevel;
	}

	/**
	 * Returns the create new actions information.
	 *
	 * @return Object[][]
	 */
	public Object[][] GetCreateNewInfo()
	{
		return m_arrCreateNewInfo;
	}

	/**
	 * Returns the URL to the Editor in create mode (new object).
	 *
	 * @param _nClassID
	 * @param _sEditorASName
	 *
	 * @return ASURL URL to the Editor
	 */
	public ASURL GetCreateNewURL(int _nClassID, String _sEditorASName)
	{
		ASURL myURL = new ASURL(m_asOwner);

		myURL.SetLinkCreateNewSpace(_sEditorASName, m_asOwner);
		myURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
		myURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE,
			XPConvert.ToString(EditorStartControl.EDITOR_START_FLAG_CREATE));
		myURL.AddControlArgument(EditorStartControl.QS_DELETE_ON_CANCEL,
			XPConvert.ToString(true));
		myURL.SetSubSpaceKey(_sEditorASName);
		myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_CLASS_ID,
			XPConvert.ToString(_nClassID));

		// Get the extra control arguments
		AddControlParamsToURL(_nClassID, myURL);

		return myURL;
	}

	/**
	 * Returns the URL to edit the object.
	 *
	 * @param _nClassID
	 * @param _nObjectID
	 * @param _sEditorASName
	 *
	 * @return ASURL URL to the Editor
	 */
	public ASURL GetEditObjectURL(int _nClassID, int _nObjectID)
	{
		ASURL myURL = new ASURL(m_asOwner);

		// JF- For any Object in the list, the sub space key will be
		// this Model name with the object class id and the object id.
		String sSubEditorKey = STR_MVC_CLASS_NAME + _nClassID + _nObjectID;

		if (m_asOwner.GetSubSpaceExists(sSubEditorKey))
		{
			EditorAS mySubEditor = (EditorAS) m_asOwner.GetSubSpace(sSubEditorKey);
			myURL.SetLinkGetCachedSpace(mySubEditor);
		}
		else
		{
			String sEditorASName = GetEditorASName(_nClassID);

			myURL.SetLinkCreateNewSpace(sEditorASName, m_asOwner);
			myURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
			myURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE,
				XPConvert.ToString(EditorStartControl.EDITOR_START_FLAG_EDIT));
			myURL.SetSubSpaceKey(sSubEditorKey);
			myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_CLASS_ID,
				XPConvert.ToString(_nClassID));
			myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_QS_OBJECT_ID,
				XPConvert.ToString(_nObjectID));
		}

		AddControlParamsToURL(_nClassID, myURL);

		return myURL;
	}

	/**
	 * Returns the row currently being edited.
	 * @return int
	 */
	public int GetEditRow()
	{
		return m_intEditRow;
	}

	/**
	 * Returns the comma-delimited string of classids and objectids for objects
	 * that are to be hidden by the tree.  The format of the string is as
	 * follows: ClassID1,ObjectID1, ClassID2,ObjectID2, etc..
	 *
	 * @return String
	 */
	public String GetHiddenObjects()
	{
		return m_strObjectsToHide;
	}

	/**
	 * Returns true if this list includes a Plumtree object selected from the
	 * tree object selector
	 * .
	 * @return boolean
	 */
	public boolean GetIncludePTSelect()
	{
		return m_bIncludePTSelect;
	}

	/**
	 * This method should ge used by the client to get back the array of values
	 * other than the 3 columns used for the Plumtree selected object (ObjectID,
	 * ClassID, and Name) at the end of the Editor and store them. It should
	 * only be called in cases where there are columns other than the Plumtree
	 * object columns (i.e. m_intNumColumns > 0)
	 *
	 * @return Object[][]
	 */
	public Object[][] GetListValues()
	{
		Object[][] aListValues = new Object[m_intNumColumns][m_saData.GetCount()];

		for (int i = 0; i < m_saData.GetCount(); i++)
		{
			for (int j = 0; j < m_intNumColumns; j++)
			{
				if (j < m_intPTObjectPosition)
				{
					aListValues[j][i] = (String) m_saData.GetItem(i, j + 1);
				}
				else
				{
					aListValues[j][i] = (String) m_saData.GetItem(i, j + 4);
				}
			}
		}

		return aListValues;
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the number of columns other than the columns used by the
	 * Plumtree selected object.
	 * @return int
	 */
	public int GetNumColumns()
	{
		return m_intNumColumns;
	}

	/**
	 * Returns String of Class ID of the Objects
	 * displayed in the tree.
	 * E.g.: "2,45,3"
	 *
	 * This is the format used by the tree.
	 *
	 * @return String
	 */
	public String GetObjectClassIDs()
	{
		String sClassIDs = "";

		// JF- We loop through the array of IDs and add each ID
		// to the String.
		for (int i = 0; i < m_arrObjectClassIDs.length; i++)
		{
			if (0 != sClassIDs.length())
			{
				sClassIDs += ",";
			}

			sClassIDs += (XPConvert.ToString(m_arrObjectClassIDs[i]));
		}

		return sClassIDs;
	}

	/**
	 * Returns the zero-based display position of the selected Plumtree object,
	 * if applicable.
	 *
	 * @return int
	 */
	public int GetPTObjectPosition()
	{
		return m_intPTObjectPosition;
	}

	/**
	 * Returns the icon to display on each row before the
	 * input text. If the value is null, no icon is displayed.
	 *
	 * @return PTImageType
	 */
	public PTImageType GetRowIcon()
	{
		return m_imgRowIcon;
	}

	/**
	 * Returns a String of Class IDs and Object IDs of the Objects
	 * to select in the tree.
	 * E.g.: "classID1,ObjID1,classID2,objID2"
	 *
	 * This is the format used by the tree.
	 *
	 * @return String
	 */
	public String GetSelectedItems()
	{
		String sSelectedItems = "";

		// JF- We loop through the array of items and for each one,
		// add the class ID and the object ID to the String.
		for (int i = 0; i < m_saData.GetCount(); i++)
		{
			// JS- Add the separator
			if (0 != sSelectedItems.length())
			{
				sSelectedItems += ",";
			}

			sSelectedItems += (m_saData.GetItem(i,
				EXPLIST_COL_CLASSID + m_intPTObjectPosition) + "," +
			m_saData.GetItem(i, EXPLIST_COL_OBJECTID + m_intPTObjectPosition));
		}

		return sSelectedItems;
	}

	/**
	 * Returns the leaf type of the tree.
	 *
	 * @return int
	 */
	public int GetTreeLeafType()
	{
		return m_treeLeafType;
	}

	/**
	 * Returns the selection type of the tree.
	 * Converts the boolean value to a string so that it can be
	 * put in the query string for the tree.
	 *
	 * @return String
	 */
	public String GetTreeMultiSelect()
	{
		return XPConvert.ToString(m_bTreeMultiSelect);
	}

	/**
	 * Returns the query filter for the tree.
	 *
	 * @return String
	 */
	public String GetTreeQueryFilter()
	{
		return m_strTreeQueryFilter;
	}

	/**
	 * Returns tree AS ID.
	 *
	 * @return String
	 */
	public String GetTreeSpaceID()
	{
		return m_sTreeSpaceID;
	}

	/**
	 * Initializes the expandable list.
	 *
	 * @param _sSectionTitle  - Main title of the view.
	 * @param _sFirstColSubTitle - The subtitle for the selected Plumtree
	 * Object, if appropriate.  If _bIncludePTSelect is FALSE, this shoudl be
	 * set to null.
	 * @param _sSectionSubTitle - An array of subtitles for non-plumtree object
	 * columns
	 * @param _imgAddIcon - Icon for the add action.
	 * @param _sAddLabel - Text to display next to the add icon.
	 * @param _arrObjectClassIDs - The Class ID of the objects displayed on the
	 * list.  This should be null if _bIncludePTSelect is FALSE.
	 * @param _nListKey - In case you have multiple Expandable Lists on the same
	 * DP, they should have a different key (otherwise, just set it to 1).
	 */
	public void InitializePTMultiColumnList(String _sSectionTitle,
		String _sFirstColSubTitle, String[] _sSectionSubTitle,
		PTImageType _imgAddIcon, String _sAddLabel, int[] _arrObjectClassIDs,
		int _nListKey)
	{
		int intNumColumns;

		if (m_bIncludePTSelect)
		{
			intNumColumns = m_intNumColumns + 1;
		}
		else
		{
			intNumColumns = m_intNumColumns;
		}

		//Set the width based on the number of columns
		int intColWidth    = 100 / (intNumColumns);
		XPStringBuilder sb = new XPStringBuilder(5);
		sb.Append(intColWidth).Append("%");

		Object[][] arrExpListColInfo = new Object[intNumColumns][5];

		if (m_bIncludePTSelect)
		{
			//First initialize the name column
			arrExpListColInfo[m_intPTObjectPosition][EXPLIST_COLUMN_ID]			    = new Integer(EXPLIST_COL_STRING +
					m_intPTObjectPosition);
			arrExpListColInfo[m_intPTObjectPosition][EXPLIST_COLUMN_TITLE]		    = _sFirstColSubTitle;
			arrExpListColInfo[m_intPTObjectPosition][EXPLIST_COLUMN_WIDTH]		    = sb.ToString();
			arrExpListColInfo[m_intPTObjectPosition][EXPLIST_COLUMN_COLSPAN]	    = "1";
			arrExpListColInfo[m_intPTObjectPosition][EXPLIST_COLUMN_ENABLESORT]     = Boolean.TRUE;
		}

		//now initialize the subsequent columns
		int intColumnID;
		int intColIndex;

		for (int i = 0; i < m_intNumColumns; i++)
		{
			if (i < m_intPTObjectPosition)
			{
				intColIndex     = i;
				intColumnID     = i + 1;
			}
			else
			{
				intColIndex     = i + 1;
				intColumnID     = i + 4;
			}

			arrExpListColInfo[intColIndex][EXPLIST_COLUMN_ID]			  = new Integer(intColumnID);
			arrExpListColInfo[intColIndex][EXPLIST_COLUMN_TITLE]		  = _sSectionSubTitle[i];
			arrExpListColInfo[intColIndex][EXPLIST_COLUMN_WIDTH]		  = sb.ToString();
			arrExpListColInfo[intColIndex][EXPLIST_COLUMN_COLSPAN]		  = "1";
			arrExpListColInfo[intColIndex][EXPLIST_COLUMN_ENABLESORT]     = Boolean.FALSE;
		}

		m_arrObjectClassIDs = _arrObjectClassIDs;

		super.InitializeExpandableList(_sSectionTitle, arrExpListColInfo,
			_imgAddIcon, _sAddLabel, _nListKey);
	}

	/**
	 * Initalizes the list (PTGrowableSortedArrayWrapper)
	 * with a column mapping and an array of sort.
	 * The growable array will always be sorted by the 1st
	 * column ascending.
	 *
	 * @param _saData
	 */
	public void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData)
	{
		// JH- Column mapping
		int[] arrColumns;

		if (m_bIncludePTSelect)
		{
			arrColumns = new int[m_intNumColumns + 4];
		}
		else
		{
			arrColumns = new int[m_intNumColumns + 1];
		}

		arrColumns[0]							  = EXPLIST_SORTEDARRAY_PROPID_INDEX;
		arrColumns[m_intPTObjectPosition + 1]     = EXPLIST_COL_CLASSID +
			m_intPTObjectPosition;
		arrColumns[m_intPTObjectPosition + 2]     = EXPLIST_COL_OBJECTID +
			m_intPTObjectPosition;
		arrColumns[m_intPTObjectPosition + 3]     = EXPLIST_COL_STRING +
			m_intPTObjectPosition;

		for (int i = 0; i < m_intNumColumns; i++)
		{
			if (i < m_intPTObjectPosition)
			{
				arrColumns[i + 1] = i + 1;
			}
			else
			{
				arrColumns[i + 4] = i + 4;
			}
		}

		// JF- Sort array
		// The growable array will always be sorted by the String column (the one displayed).
		int[][] arrSort = new int[1][2];
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		  = EXPLIST_COL_STRING +
			m_intPTObjectPosition;
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;

		// JF- We don't initalize the list itself (null below)
		// At first it's empty and we will be initalized if needed using the
		// AddRowsToList method.
		_saData.Initialize(arrColumns, null, arrSort, true,
			EXPLIST_DEFAULT_INITIAL_SIZE);
	}

	/**
	 * Returns true if it should be possible to edit Objects from this class ID.
	 *
	 * @param _nClassID
	 */
	public boolean IsEditLinkEnabled(int _nClassID)
	{
		if (null != m_arrCreateNewInfo)
		{
			int nClassIDFromArray;

			for (int i = 0; i < m_arrCreateNewInfo.length; i++)
			{
				nClassIDFromArray = XPConvert.ToInteger(m_arrCreateNewInfo[i][CREATE_NEW_COL_CLASSID]);

				if (nClassIDFromArray == _nClassID)
				{
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * Action for modifying a row
	 *
	 * @param _htFormData 
	 */
	public void ModifyRow(XPHashtable _htFormData)
	{
		String[] data = (String[]) _htFormData.GetElement(PTMultiColumnListView.HTMLHIDDEN_EDITROW);

		if (null != data)
		{
			this.SetEditRow(XPConvert.ToInteger(data[0]));
		}
	}

	/**
	 * Sets the accesslevel of the objects for the tree control to display.
	 *
	 * @param nAccessLevel The AccessLevel to set
	 */
	public void SetAccessLevel(int nAccessLevel)
	{
		m_nAccessLevel = nAccessLevel;
	}

	/**
	 * Add the array passed in argument to the list.
	 * The array should be created using the EXPLIST_COL
	 * constants defined in this class.
	 *
	 * @param _arrRows Array of rows to add.
	 */
	public void SetDefaultObjects(Object[][] _arrRows)
	{
		super.GetItems().AddRows(_arrRows);
	}

	/**
	 * Sets a new default value.
	 * By default this is an array of empty strings. This value is used on add
	 * to populate the input text of the newly added row.
	 *
	 * @param _sDefaultValue New value
	 */
	public void SetDefaultValue(String[] _sDefaultValue)
	{
		m_arrDefaultValue = _sDefaultValue;
	}

	/**
	 * Sets the row currently being edited.
	 * @param editRow The row to set
	 */
	public void SetEditRow(int editRow)
	{
		m_intEditRow = editRow;
	}

	/**
	 * Enables the create new links for the Class ID in argument.
	 * Doing this will also enbale the links to Edit the Objects in the list.
	 *
	 * @param _arrCreateNewInfo Information for each create new link
	 */
	public void SetEnableCreateNew(Object[][] _arrCreateNewInfo)
	{
		if (_arrCreateNewInfo[0].length < CREATE_NEW_COL_MAX)
		{
			throw new XPException(
				"UserAliasModel::SetEnableCreateNew The create new information array" +
				" should contain " + CREATE_NEW_COL_MAX + " columns.");
		}

		m_arrCreateNewInfo = _arrCreateNewInfo;
	}

	/**
	 * Because this select ptobject list needs to support both move
	 * row actions enable and disable, we need to change the
	 * colspan of the header row when this happens.
	 *
	 * @param _bEnableMoveRowActions
	 */
	public void SetEnableMoveRowActions(boolean _bEnableMoveRowActions)
	{
		super.SetEnableMoveRowActions(_bEnableMoveRowActions);

		if (m_bEnableMoveRowActions)
		{
			// JF- If the move row actions are enabled, this will
			// create one more cell at the end of each row and
			// we need to reflect that change on the header row.
			m_arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN] = "2";
		}
		else
		{
			m_arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN] = "1";
		}
	}

	/**
	 * Populates m_strObjectsToHide, a comma-delimited list of classid/objectid
	 * pairs.  The format of the string is as follows: ClassID1,ObjectID1,
	 * ClassID2,ObjectID2, etc..
	 *
	 * @param _anClassIDObjectID - a two dimensional array of classID's and
	 * ObjectID's.  Objects specified will be hidden from display in the tree's
	 * output.
	 */
	public void SetHiddenObjects(int[][] _anClassIDObjectID)
	{
		XPStringBuilder sb = new XPStringBuilder(6 * _anClassIDObjectID.length);

		for (int i = 0; i < _anClassIDObjectID.length; i++)
		{
			if (i > 0)
			{
				sb.Append(",");
			}

			sb.Append(_anClassIDObjectID[i][HIDE_COL_CLASSID]).Append(",")
			.Append(_anClassIDObjectID[i][HIDE_COL_OBJECTID]);
		}

		m_strObjectsToHide = sb.ToString();
	}

	/**
	 * Sets the value for m_bIncludePTSelect, indicating whether a Plumtree
	 * object selector should be included with this list.
	 *
	 * @param m_bIncludePTSelect - true or false
	 */
	public void SetIncludePTSelect(boolean m_bIncludePTSelect)
	{
		this.m_bIncludePTSelect = m_bIncludePTSelect;
	}

	/**
	 * Returns the number of columns other than the columns used by the
	 * Plumtree selected object.
	 *
	 * @param numColumns The numColumns to set
	 */
	public void SetNumColumns(int numColumns)
	{
		m_intNumColumns = numColumns;
	}

	/**
	 * Sets the the zero-based display position of the selected Plumtree object,
	 * if applicable.
	 *
	 * @param pTObjectPosition The pTObjectPosition to set
	 */
	public void SetPTObjectPosition(int pTObjectPosition)
	{
		m_intPTObjectPosition = pTObjectPosition;
	}

	/**
	 * Sets the icon to display on each row before the
	 * input text.
	 * By default, no icon is displayed.
	 *
	 * @param _imgRowIcon
	 */
	public void SetRowIcon(PTImageType _imgRowIcon)
	{
		m_imgRowIcon = _imgRowIcon;
	}

	/**
	 * Sets the leaf type for the tree.
	 * Is set to N_LEAFSELECT by default.
	 *
	 * @param _leafType
	 */
	public void SetTreeLeafType(int _leafType)
	{
		m_treeLeafType = _leafType;
	}

	/**
	 * Sets the selection type for the tree.
	 * Multiselect is set to true by default.
	 *
	 * @param _selectType
	 */
	public void SetTreeMultiSelect(boolean _selectType)
	{
		m_bTreeMultiSelect = _selectType;
	}

	/**
	 * Sets the query filter for the tree.
	 * By default there is no query filter.
	 *
	 * @param _nClassID
	 * @param _nPropID
	 * @param _anValues
	 */
	public void SetTreeQueryFilter(int _nClassID, int _nPropID, int[] _anValues)
	{
		XPStringBuilder sb = new XPStringBuilder(20);
		sb.Append(_nClassID).Append(",").Append(_nPropID);

		for (int i = 0; i < _anValues.length; i++)
		{
			sb.Append(",").Append(_anValues[i]);
		}

		m_strTreeQueryFilter = sb.ToString();
	}

	/**
	 * Sets the Tree sub title.
	 *
	 * @param _sTreeSubTitle
	 */
	public void SetTreeSubTitle(String _sTreeSubTitle)
	{
		m_sTreeSubTitle = _sTreeSubTitle;
	}

	/**
	 * Sets the Tree title.
	 *
	 * @param _sTreeTitle
	 */
	public void SetTreeTitle(String _sTreeTitle)
	{
		m_sTreeTitle = _sTreeTitle;
	}

	/**
	 * Saves the data for columns other than the plumtree object select column
	 * on the row _nRowIndex.
	 *
	 * @param _saData List to update the data on row _nRowIndex
	 * @param _nRowIndex Index of the row to save.
	 * @param _htFormData Data posted by the form.
	 * @return int Status for server side validation.
	 */
	protected int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex,
		XPHashtable _htFormData)
	{
		XPStringBuilder sb = new XPStringBuilder(100);
		sb.Append(PTMultiColumnListView.HTMLINPUT_ITEM);

		int intIndex;

		for (int i = 0; i < m_intNumColumns; i++)
		{
			if (i < m_intPTObjectPosition)
			{
				intIndex = i + 1;
			}
			else
			{
				intIndex = i + 4;
			}

			String sDataKey     = GetUniqueString(sb.Append(intIndex).ToString()) +
				_nRowIndex;
			String[] aDataValue;

			// JF- For each input text posted by the form, we get the value back
			// and set the new value in the array.
			if (_htFormData.ContainsKey(sDataKey))
			{
				aDataValue = (String[]) _htFormData.GetElement(sDataKey);

				if ((null != aDataValue) && !(0 == aDataValue.length))
				{
					_saData.SetItem(_nRowIndex, intIndex, aDataValue[0]);
				}
			}
		}

		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * Returns the Tree sub title.
	 *
	 * @return String
	 */
	public String GetTreeSubTitle()
	{
		return m_sTreeSubTitle;
	}

	/**
	 * Returns the Tree title.
	 *
	 * @return String
	 */
	public String GetTreeTitle()
	{
		return m_sTreeTitle;
	}

	/**
	 * Sets the array of objects to display in the tree. 
	 * (Only those objects will be display in the tree.)
	 * The array in argument should be in the following format:
	 * classid1,objectid1,classid2,objectid2....
	 * 
	 * @param _sDisplayObjects
	 */
	public void SetDisplayObjects(String _sDisplayObjects)
	{
		m_sDisplayObjects = _sDisplayObjects;
	}
	
	/**
	 * Returns the array of objects to display in the tree.
	 * 
	 * @param String
	 */
	public String GetDisplayObjects()
	{
		return m_sDisplayObjects;
	}

	/**
	 * Helper method to save the Tree ID on repost.
	 *
	 * @param _htFormData Form data
	 */
	protected void SaveTreeID(XPHashtable _htFormData)
	{
		// JF- We save the tree ID only if we don't already have it.
		if (("").equals(m_sTreeSpaceID) &&
				_htFormData.ContainsKey(GetUniqueString(
						PTMultiColumnListView.HTMLINPUT_TREEASID)))
		{
			String[] data = (String[]) _htFormData.GetElement(GetUniqueString(
						PTMultiColumnListView.HTMLINPUT_TREEASID));

			if ((null != data) && !(0 == data.length))
			{
				m_sTreeSpaceID = data[0];
			}
		}
	}

	/**
	 * Helper method to get the EditorAS object name from the class ID.
	 *
	 * @param _nClassID
	 *
	 * @return String
	 */
	private String GetEditorASName(int _nClassID)
	{
		String sASName		  = null;
		int nClassIDFromArray;

		for (int i = 0; i < m_arrCreateNewInfo.length; i++)
		{
			nClassIDFromArray = XPConvert.ToInteger(m_arrCreateNewInfo[i][CREATE_NEW_COL_CLASSID]);

			if (nClassIDFromArray == _nClassID)
			{
				sASName = (String) m_arrCreateNewInfo[i][CREATE_NEW_COL_ASNAME];

				break;
			}
		}

		return sASName;
	}
}

