package com.plumtree.portaluiinfrastructure.restconsumerframework;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.helpers.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.statichelpers.*;

public class EnsembleRestAPIService 
{
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_REST), EnsembleRestAPIService.class);
	
	/** Ensemble Constants **/
	public static final int ENSEMBLE_PAGELETPARAMETER_MAXLENGTH = 1024; 
	
	/** Rest API URL Parameters **/ 
	protected static final String BASEURI 		= "api/v2/ensemble/";
	protected static final String GETPAGELETS 	= "pagelets/";
	protected static final String LIMITRESULTS	= "?resultCount=";
	protected static final String SEARCHNAME	= "?name=";
	protected static final String SEARCHID		= "?id=";

	/** Element names in Rest API Response **/
	protected static final String ROOTNAME_PAGELETLIST		= "ns2:PageletList";
	protected static final String ROOTNAME_SINGLEPAGELET	= "ns2:Pagelet";
	protected static final String DATAENTRY_PAGELET			= "Pagelet";

	/** Expected Pagelets data node names **/ 
	public static final String NAME 				= "name";
	public static final String DESCRIPTION 			= "desc";
	public static final String ID 					= "id";
	public static final String LASTMODIFIED 		= "lastModified";
	public static final String CREATED 				= "created";
	public static final String LIBRARYNAME 			= "m_strLibraryName";
	public static final String PARENT_RESOURCE_NAME = "parent_resource_name";
	public static final String PARENT_RESOURCE_DESCRIPTION = "parent_resource_desc";
	public static final String PARENT_RESOURCE_ID 	= "parent_resource_id";
	public static final String PARENT_RESOURCE_EXTURL_LIST = "parent_resource_external_url_list";
	public static final String CODESAMPLE 			= "m_strCodeSample";
	public static final String PUBLISHDOCS 			= "m_bPublishDocs";
	public static final String ADD_INLINEREFRESH 	= "m_addInlineRefreshToAllUrls"; 
	public static final String REFRESH_INTERVAL 	= "m_refreshInterval";
	public static final String URL_SUFFIX 			= "url_suffix";
	public static final String PAGELET_EXTURL 		= "pagelet_external_url";
	public static final String PARAMETERS_TRANSPORT = "parametersTransport";
	public static final String ALLOW_ALL 			= "m_bAllowAll";

	public static final String PARAMETERS 			= "m_Parameters";
	/** Pagelet Parameter node names **/
	public static final String PARAM_NAME 			= "name";
	public static final String PARAM_DESCRIPTION 	= "desc";
	//public static final String PARAM_TYPE 			= "type";
	public static final String PARAM_ISMANDATORY 	= "isMandatory";	
	
	/**
	 * Gets the ensemble base url from the ensemble remote server object
	 * 
	 * @param session
	 * @return
	 */
	protected String GetEnsembleBaseUrl(IPTSession session)
	{
		int ensembleServiceId = EnsembleHelperService.getInstance().GetRemoteServerID(session);
		IPTObjectManager spServers = session.GetGadgetContentServers();
		IPTGadgetContentServer server = (IPTGadgetContentServer)spServers.Open(ensembleServiceId, false);
		String serverUrl = server.GetURL();
		
		return serverUrl;
	}
	
	/**
	 * Get a list of all pagelets from Ensemble Rest API by sending a Rest Request to Ensemble service  
	 * 
	 * @param session
	 * @return
	 */
	public EnsemblePageletRestData[] GetAllPageletsFromRest(IPTSession session)
	{
		return GetAllPageletsFromRest(-1, session);

	}
	
	/**
	 * 
	 * @param maxResults	Max results to return in the response
	 * @param session
	 * @return	null if response is not valid and no data was retrieved
	 */
	public EnsemblePageletRestData[] GetAllPageletsFromRest(int maxResults, IPTSession session)
	{
		XPStringBuilder url = new XPStringBuilder();
		url.Append(GetEnsembleBaseUrl(session)).Append(BASEURI+GETPAGELETS);
		if ( maxResults > 0 ) {
			url.Append(LIMITRESULTS).Append(maxResults);
		}
		
		if (log.IsInfoEnabled()) { 
			log.Info("Sending Rest Request to: "+url.ToString());
		}
		RestRequest req = RestRequest.GetInstance(url.ToString(), OpenHTTPServiceHelper.GetOpenHTTPService(), OpenHTTPServiceHelper.GetOpenHTTPSession());
		
		EnsembleAuthenticate(req, session);
		
		IRestResponse response;
		try {
			response = req.Submit();
		} catch (Exception e) {
			throw new RestConnectFailedException(e);
		}
		
		boolean valid = VerifyEnsemblePageletListResponse(response);
		if ( !valid ) {
			throw new XPIllegalStateException("Invalid Rest API response received from Ensemble. ");
			//return null;
		}
		
		EnsemblePageletRestData[] pagelets = GetPageletListData(response);
		return pagelets;
	}
	
	/**
	 * Returns a single pagelet entry specified by its name
	 * 
	 * ?? Pagelet with same name? Partial search on name?
	 * 
	 * @param session
	 * @return m_ensembleServerValid = false;
	 */
	public EnsemblePageletRestData[] GetPageletWithNameFromRest(String pageletName, IPTSession session)
	{
		if ( null == pageletName || "".equals(pageletName) ) {
			throw new XPIllegalArgumentException("Provided pagelet name cannot be empty");
		}
		
		if (log.IsInfoEnabled()) { 
			log.Info("Sending Rest Request to: "+GetEnsembleBaseUrl(session)+BASEURI+GETPAGELETS+SEARCHNAME+pageletName);
		}
		RestRequest req = RestRequest.GetInstance(GetEnsembleBaseUrl(session)+BASEURI+GETPAGELETS+SEARCHNAME+pageletName, OpenHTTPServiceHelper.GetOpenHTTPService(), OpenHTTPServiceHelper.GetOpenHTTPSession());
		
		EnsembleAuthenticate(req, session);
		
		IRestResponse response = req.Submit();
		
		EnsemblePageletRestData[] result = null; 
		
		if ( VerifyEnsemblePageletListResponse(response) ) {
			result = GetPageletListData(response);
		} else if ( VerifyEnsemblePageletEntryResponse(response) ) {
			EnsemblePageletRestData data = GetSinglePageletData(response);
			result = new EnsemblePageletRestData[] {data};
		} else {
			// invalid result type
			throw new XPIllegalStateException("Retrieved invalid REST data response from Ensemble REST API!");
		}
		return result;
	}
	
	/**
	 * 
	 * @param id
	 * @param session
	 * @return
	 */
	public EnsemblePageletRestData GetPageletWithIdFromRest(int id, IPTSession session)
	{
		if ( id < 0 ) {
			throw new XPIllegalArgumentException("Provided pagelet id cannot be empty");
		}
		
		if (log.IsInfoEnabled()) { 
			log.Info("Sending Rest Request to: "+GetEnsembleBaseUrl(session)+BASEURI+GETPAGELETS+SEARCHID+id);
		}
		RestRequest req = RestRequest.GetInstance(GetEnsembleBaseUrl(session)+BASEURI+GETPAGELETS+SEARCHID+id, OpenHTTPServiceHelper.GetOpenHTTPService(), OpenHTTPServiceHelper.GetOpenHTTPSession());
		
		EnsembleAuthenticate(req, session);
		
		IRestResponse response = req.Submit();
		
		EnsemblePageletRestData result = null;
		
		if ( VerifyEnsemblePageletEntryResponse(response) ) {
			result = GetSinglePageletData(response);
		} else {
			// invalid result type
			throw new XPIllegalStateException("Retrieved invalid REST data response from Ensemble REST API!");
		}
		return result;
	}
	
	/**
	 * NOTE: Ensemble moving to use basic auth instead of csp login tokens
	 * 
	 * Set authentication 
	 * 
	 * @param request
	 * @param session
	 */
	protected void EnsembleAuthenticate(RestRequest request, IPTSession session)
	{
		//
		//String token = session.GetToken(3);
		//request.SetCSPLoginToken(token);
	}
	
	
	/**
	 * Helper method to verify Ensemble Rest API response with a list of pagelet entries 
	 * 
	 * @param response
	 * @return
	 */
	protected boolean VerifyEnsemblePageletListResponse(IRestResponse response)
	{
		if ( response == null ) {
			log.Error("No response received from Ensemble REST API call.");
			return false;
		}
		
		String rootname = response.GetRootElementName();
		if ( !ROOTNAME_PAGELETLIST.equals(rootname) && !ROOTNAME_SINGLEPAGELET.equals(rootname)) {
			log.Error("Found unexpected root element in Rest API response: "+rootname+", expected root element with name: "+ROOTNAME_SINGLEPAGELET+" or " +ROOTNAME_PAGELETLIST);
			return false;
		}
		
		// Doesnt work for single pagelet response
//		String listnodename = response.GetListName();
//		if ( !DATAENTRY_PAGELET.equals(listnodename) ) {
//			log.Error("Expected pagelet list element with name: "+DATAENTRY_PAGELET+", instead found element with name: "+listnodename+" in response." );
//			return false;
//		}
		
		return true;
	}

	/**
	 * Helper method to verify Ensemble Rest API response with a single pagelet entry
	 * Calling method should perform validation that the response contains a single pagelet data entry should  
	 * 
	 * @param response
	 * @return false if response is not valid 
	 */
	protected boolean VerifyEnsemblePageletEntryResponse(IRestResponse response)
	{
		if ( response == null ) {
			return false;
		}
		
		String rootname = response.GetRootElementName();
		if ( !ROOTNAME_SINGLEPAGELET.equals(rootname) ) {
			return false;
		}
		
		return true;
	}
	
	/**
	 * Internal helper method to get pagelet data in Ensemble Rest API response with single pagelet
	 * Calling method should perform validation that the response contains a list of pagelet data entries
	 * 
	 * @param response
	 * @return null if response is not valid and no data was retrieved
	 */
	protected EnsemblePageletRestData GetSinglePageletData(IRestResponse response)
	{
		if ( response == null ) {
			return null;
		}
		
		XPHashtable data = response.GetSingleDataItem();
		EnsemblePageletRestData pagelet = new EnsemblePageletRestData(); 
		if ( pagelet.Init(data) ) {
			return pagelet;
		} else {
			return null;
		}
	}
	
	/**
	 * Internal helper method to get pagelet data in Ensemble Rest API response with a list of pagelets 
	 * 
	 * @param response
	 * @return null if response is not valid and no data was retrieved
	 */
	protected EnsemblePageletRestData[] GetPageletListData(IRestResponse response)
	{
		if ( response == null ) {
			return null;
		}
		
		String rootEleName = response.GetRootElementName();
		boolean pageletList = ROOTNAME_PAGELETLIST.equals( rootEleName );
		boolean singlePagelet = ROOTNAME_SINGLEPAGELET .equals( rootEleName );
		
		EnsemblePageletRestData[] pagelets = null;
		if ( pageletList ) 
		{
			XPArrayList list = response.GetResponseList();
			
			pagelets = new EnsemblePageletRestData[list.GetSize()];
			
			for ( int i = 0; i < list.GetSize(); i++ ) {
				XPHashtable xmldata = (XPHashtable)list.GetElement(i);
				EnsemblePageletRestData data = new EnsemblePageletRestData(); 
				if ( data.Init(xmldata) ) {
					pagelets[i] = data;
				}
			}
			
		} else if ( singlePagelet ) { // Single pagelet
			XPHashtable item = response.GetSingleDataItem();
			pagelets = new EnsemblePageletRestData[1];
			
			EnsemblePageletRestData data = new EnsemblePageletRestData(); 
			if ( data.Init(item) ) {
				pagelets[0] = data;
			}
			
		} 
		if ( !singlePagelet && !pageletList ) {
			throw new XPIllegalStateException("Unknown root element name encountered while parsing response to get all Pagelets: "+rootEleName);
		}
		return pagelets;
	}

}
