package com.plumtree.portaluiinfrastructure.restconsumerframework;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.xml.dom.*;
import com.plumtree.openkernel.http.*;

/**
 * Helper class to process responses containing XML
 *   
 * 
 * @author rozhang
 */
public class RestXMLResponse implements IRestResponse
{
	protected XPDocument m_doc = null;  
	
	protected String m_toplevelnodenames = null;
	
	protected boolean m_notoplevelnodes = false;
	
	/**
	 * 
	 * @param response
	 */
	public RestXMLResponse(IOKHttpResponse response)
	{
		if ( response == null ) {
			throw new XPIllegalArgumentException("Valid response required, cannot be null.");
		}
		
		XPDocument doc = XPBuilder.LoadDocumentFromByteArray(response.GetBodyAsBinary());
		if ( doc != null ) {
			m_doc = doc;
		}
	}
	
	/**
	 * Mainly for unit tests 
	 * 
	 * @param response
	 */
	public RestXMLResponse(String  responseContent)
	{
		if ( responseContent == null ) {
			throw new XPIllegalArgumentException("Valid response content required, cannot be null.");
		}
		
		XPDocument doc = XPBuilder.LoadDocumentFromString(responseContent);
		if ( doc != null ) {
			m_doc = doc;
		}
	}
	
	/**
	 * Gets the name of the top level element
	 * 
	 * @return
	 */
	public String GetRootElementName()
	{
		if ( m_doc == null ) {
			throw new XPIllegalStateException("This object has not been properly initialized. No valid response was provided");
		}
		
		XPElement root = m_doc.GetDocumentElement();
		String name = root.GetNodeName();
		return name;
	}
	
	/**
	 * Returns the name of a node list under the root node. All the nodes
	 * under the root node has to share the same name to be considered a valid
	 * node list. 
	 * 
	 * @return null if the response content doesn't contain a list 
	 */
	public String GetListName()
	{
		if ( m_notoplevelnodes || m_toplevelnodenames != null ) {
			return m_toplevelnodenames;
		}
		
		XPElement root = m_doc.GetDocumentElement();
		XPNodeList list = root.GetChildNodes();
		String listnodenames = null;

		for ( int i = 0; i < list.GetLength(); i++ ) 
		{
			XPNode node = list.Item(i);
			if ( node.IsTextNode() || node.IsCommentNode() ) { continue; }
			
			if ( node.GetNodeType() == XPNode.ELEMENT_NODE ) {
				// first node encountered, store the name
				if ( listnodenames == null ) 
				{
					listnodenames = node.GetNodeName();
				} 
				// we have node name stored, check if it is the same
				else if (!listnodenames.equals(node.GetNodeName())) 
				{
					m_notoplevelnodes = true;
					m_toplevelnodenames = null;
				} else {
					m_notoplevelnodes = false;
					m_toplevelnodenames = listnodenames;
				}
			}
		}
		return listnodenames;
	}
	
	/**
	 * Exclusive from GetResponseList method, only one is valid in any one response
	 * 
	 * @return null if the response is actually a list of items
	 */
	public XPHashtable GetSingleDataItem()
	{
		XPElement root = m_doc.GetDocumentElement();
		XPHashtable datamap = ExtractNodeChildData(root);
		
		return datamap;
	}
	
	/**
	 * Gets the element name of the root element.
	 * @return 
	 */
	public String GetRootNodeName()
	{
		XPElement root = m_doc.GetDocumentElement();
		return root.GetNodeName();
	}
	
	/**
	 * Exclusive from GetSingleDataItem method, only one is valid in any one response
	 * 
	 * @return null if the current response does not contain a top level list, XPArrayList contains 
	 * XPHashtables
	 */
	public XPArrayList GetResponseList()
	{
		// first check that we have a top level list in the response (compared to single item)
		if ( GetListName() == null ) {
			// log
			return null;
		}
		XPArrayList outVal = new XPArrayList();
		
		XPElement root = m_doc.GetDocumentElement();
		XPNodeList list = root.GetChildNodes();

		for ( int i = 0; i < list.GetLength(); i++ ) 
		{
			XPNode node = list.Item(i);
			if ( node.IsTextNode() || node.IsCommentNode() ) { continue; }
			
			if ( node.GetNodeType() == XPNode.ELEMENT_NODE ) {
				XPHashtable datamap = ExtractNodeChildData(node);
				outVal.Add(datamap);
			}
		}
		return outVal;
		
	}
	
	/**** INTERNAL METHODS *****/
	
	protected XPElement GetRootNode()
	{
		if ( m_doc == null ) {
			throw new XPIllegalStateException("This object has not been properly initialized. No valid response was provided");
		}
		
		XPElement root = m_doc.GetDocumentElement();
		return root;
	}
	
	/**
	 * Helper method to get the text value in a node
	 * 
	 * @param node
	 * @return
	 */
	protected String GetNodeTextBody(XPNode node)
	{
		if ( node == null ) {
			throw new XPIllegalArgumentException("Provided XML node element is not valid");
		}
		
		// empty node, just treat it as empty string 
		if ( node.GetFirstChild() == null ) {
			return "";
		}
		
		String textValue = node.GetFirstChild().GetNodeValue();
		if ( "".equals( textValue ) ) {
			return "";
		} else {
			return textValue;
		}
	}
	
	/**
	 * Fills a hashtable with the child node names and node body text of the provided node.
	 * Multiple child nodes with the same names are inserted into the same hash key, and the multiple
	 * values are added to an XPArrayList
	 * Nested child nodes are inserted as nested XPHashTable    
	 * 
	 * 
	 * @param topnode
	 * @return null if the provided node is null or does not have any child nodes.
	 */
	protected XPHashtable ExtractNodeChildData(XPNode topnode)
	{
		XPNodeList list = topnode.GetChildNodes();
		if ( (list == null) || (list.GetLength() == 0) ) {
			return null;
		}
		XPHashtable hashtable = new XPHashtable();

		// iterate over all the child nodes and add the values for the nodes in there
		for ( int i = 0; i < list.GetLength(); i++ ) 
		{
			XPNode node = list.Item(i);
			if ( node.IsTextNode() || node.IsCommentNode() ) { continue; }
			if ( node.GetNodeType() == XPNode.ELEMENT_NODE ) {
				String name = node.GetNodeName();
				XPNodeList children = node.GetChildNodes();
				// value will be empty string for nodes with no body since we need to put in a non-null
				// value into the hashtable
				Object value = GetNodeTextBody(node); // @TODO should we trim the string here?
				
				//P2/portalui/main/ptwebui/portaluiinfrastructure/java/prod/src/com/plumtree/portaluiinfrastructure/restconsumerframework/RestXMLResponse.java
									
				/*  bug fix 9410440 - IF ONLY ONE TAG IS AVAILABLE FOR A PORTAL OBJECT, IT IS NOT VISIBLE
				 *  It seems in the .net Node, node.GetFirstChild().GetNodeValue(), results in a null value and that's a key difference with the java 
				 *  node.  B/c of this, a single item in the <tags> node will result in a null value.  This is the same result for multiple items in the 
				 *  <tags> node, except that we traverse the children of the child nodes in below if statement:  if (children.GetLength() >1) so the tag 
				 *  items are actually getting extracted.  In a single tag value, we skip the extraction of the node.
				 *  To fix this bug, I will check if the system is running in .net, we're traversing the <tags> node, and there is an item length of 1.
				*/
				String TAGS_NODE = "tags";
				if ( XPSystem.IsRunningInDotNet() && name.equals(TAGS_NODE) && (children.GetLength() == 1))
				{
					value = ExtractNodeChildData(node);
				}				

				// we got a complex node with child elements, populate a hashtable with the child nodes
				if ( children.GetLength() > 1 ) {
					value = ExtractNodeChildData(node);
				}
				
				// check if that value already exists, if so add values to a list
				if ( hashtable.ContainsKey(name) ) {
					Object o = hashtable.GetElement(name);
					// just add to existing list if one has already been created
					if ( o instanceof XPArrayList ) {
						((XPArrayList)o).Add(value);
						
					} else {
						// create a list and put the old value in there before the current value
						XPArrayList bucket = new XPArrayList();
						bucket.Add(o);
						bucket.Add(value);
						hashtable.PutElement(name, bucket);
					}
				} else {
					hashtable.PutElement(name, value);
				}
			}
		}
		
		return hashtable;
	}
	
}
