package com.plumtree.portaluiinfrastructure.restconsumerframework.pathways;

import com.plumtree.server.IPTSession;
import com.plumtree.uiinfrastructure.activityspace.AActivitySpace;
import com.plumtree.portaluiinfrastructure.search.pathways.PathwaysQueryArguments;

/**
 * @author BobbyT
 */
public interface IPathwaysRestAPIService 
{
	/**
	 * Checks if Pathways service is enabled from the config settings.  
	 * 
	 * @return true if Pathways is enabled, otherwise false.
	 */
	public boolean IsPathwaysEnabled();
	
	/**
	 * Gets the Pathways API base URL from the Pathways API Web Service object
	 * 
	 * @param session the current user session.
	 * @return the URL to the Pathways API Web Service, null if service is not installed.
	 */
	public String GetPathwaysAPIBaseURL(IPTSession session);
	
	/**
	 * Gets the Pathways API Gatewayed URL from the Pathways API Web Service object
	 * This is used when making Pathways API calls on the client side via JSON.
	 * 
	 * @param session the current user session.
	 * @return the gatewayed URL to the Pathways API Web Service, null if service is not installed.
	 */
	public String GetPathwaysAPIGatewayedURL(AActivitySpace space, IPTSession session);

	/**
	 * Gets the Pathways Version from Pathways Rest API.
	 * 
	 * Note: this method is disabled for now as it is not being used and causes extra
	 * trips to the Pathways service. Re-enable when there's a need to detect Pathways 
	 * versioning in case of backward compatibility.
	 *  
	 * @return String representing the Pathways version.
	 */
//	public String GetAPIVersion();
	
	/**
	 * Returns a list of Pathways Views from Pathways Rest API.
	 * 
	 * @return a list of PathwaysViewsRestData
	 */
	public PathwaysViewsRestData[] GetViews();
	
	/**
	 * Returns a Tag Cloud from a Pathways search query.
	 * 
	 * @param view the Pathways view to use when retrieving the Tag Cloud
	 * @param format the payload format of the response
	 * @param query the Pathways search query
	 * @param tagCount the maximum number of tags to return for the Tag Cloud
	 * @return
	 */
	public PathwaysTagRestData[] GetTagCloud(String view, String format, String query, String tagCount);
	
	/**
	 * Returns a list of search results from a Pathways search query.
	 * 
	 * @param view the Pathways view to use when retrieving the Tag Cloud
	 * @param format the payload format of the response
	 * @param query the Pathways search query
	 * @param tagCount the maximum number of tags to return for the Tag Cloud
	 * @param objectTypes restrict the objects to search for in pathways
	 * @return array of PathwaysSearchRestData
	 */
	public PathwaysSearchRestData[] DoSearch(PathwaysQueryArguments pathwaysQueryArguments);

	/**
	 * This Method is used to Add/Remove a Pathways Tag on a particular document. This uses 
	 * Pathways REST API to connect to Pathways and execute the requested action.
	 * @param tname Name of the Tag on Which to take action
	 * @param fullId Complete ID (CLASSID_DOCUMENTID) of the Document the tag is related to 
	 * @param action Action Operation to perform on the tag, 0 = Add, 1 = Delete, 2 = Edit (delete old tag and add new tag)
	 * @param ptsource Source of the Object, for Portal it is 'PTPORTAL' and collab it is 'PTCOLLAB'. More sources are in PathwaysConstants. 
	 */
	public void AddRemoveTagRest(String tname, String fullId, int action, String ptsource);

	/**
	 * This Method is used to Rename a Pathways Tag on a particular document. This uses 
	 * Pathways REST API to connect to Pathways and execute the requested action.
	 * @param tname the Source Tag Name to be used for renaming
	 * @param fullId Complete ID (CLASSID_DOCUMENTID) of the Document the tag is related to 
	 * @param ptsource Source of the Object, for Portal it is 'PTPORTAL' and collab it is 'PTCOLLAB'. More sources are in PathwaysConstants.
	 * @param renametag the Target Tag Name to be used for renaming 
	 */
	public void RenameTagRest(String tname, String fullId, String ptsource, String renametag);
	
	/**
	 * Returns the total count of search results returned 
	 */
	public int GetTotalCount();

	/**
	 * Returns if the current user experience supports adaptive layout mode
	 * @return
	 */
	public boolean IsAdaptiveLayoutMode();
}
