package com.plumtree.portaluiinfrastructure.restconsumerframework.pathways;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.helpers.*;
import com.plumtree.portaluiinfrastructure.layout.*;
import com.plumtree.portaluiinfrastructure.pathways.tagging.*;
import com.plumtree.portaluiinfrastructure.restconsumerframework.*;
import com.plumtree.portaluiinfrastructure.search.pathways.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.statichelpers.*;


/**
 * Provides methods to simplify and abstract making Pathways specific Rest API requests. 
 * Consumes either a RestXMLParser or RestJSONParser.
 * 
 * @author BobbyT
 * 
 * 
 */
public class PathwaysRestAPIService implements IPathwaysRestAPIService
{
	private static OpenLogger logger = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PATHWAYS),
		"com.plumtree.portaluiinfrastructure.restconsumerframework.pathways.PathwaysRestAPIService");
	
	//this is test instance 
	private static IPathwaysRestAPIService m_testInstance;
	
	// This stores whether or not the factory is in test mode.
	private static boolean m_bIsProduction = false;

	private String m_strBaseURL;
	private AActivitySpace m_asOwner;
	private boolean m_bIsPathwaysEnabled;
	private IPTSession m_session;
	protected int m_totalCount;	
	
	private static final String opAND = "AND";
	private static final String opOR = "OR";
	
	// Protected constructor to ensure controlled access to the singleton.
	protected PathwaysRestAPIService(AActivitySpace space) 
	{
		m_asOwner = space;
		
		// Getting Pathways Settings		
		ConfigVarPack vpConfig = (ConfigVarPack) m_asOwner.GetApplication()
									.GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);

		// Getting the UserSession
		m_session = (IPTSession) m_asOwner.GetUserSession();

		try 
		{
			String strFlag = vpConfig.GetConfigValueAsString(
					ConfigConstants.SERVICE_URLS_SECTION, ConfigConstants.PATHWAYS_REST_FLAG,
					ConfigConstants.VALUE_ATTRIBUTE);
			
			m_bIsPathwaysEnabled = XPConvert.ToBoolean(strFlag);
		}
		catch (Exception ex)
		{
			m_bIsPathwaysEnabled = false;
			logger.Info("Unable to retrieve Pathways config settings, Pathways will be disabled");
		}
		
		if (m_bIsPathwaysEnabled)
		{
			m_strBaseURL = GetPathwaysAPIBaseURL(m_session);
			
			// Trim off the end slash of the base URL if there is any.
			if (m_strBaseURL != null && m_strBaseURL.endsWith("/"))
			{
				m_strBaseURL = m_strBaseURL.substring(0, m_strBaseURL.length() - 1);
			}
		}
		else
		{
			m_strBaseURL = null;
		}
	}
	
	/**
	 * @return The singleton PathwaysRestAPIService.
	 */
	public static IPathwaysRestAPIService GetInstance(AActivitySpace space) {

		if(m_testInstance == null){
			m_bIsProduction = true;
			return new PathwaysRestAPIService(space);
		} else {
			return m_testInstance;
		}
	}
	
	/**
	 * Unit tests should use this method to overwrite/teardown existing helper singleton.
	 * DO NOT USE for production code.
	 * 
	 * NOTE: When finished with a unit test, this should be set back to null so as not
	 * to interfere with other tests.
	 */
	public static void setTestInstance(IPathwaysRestAPIService testHelper)
	{
		if (m_bIsProduction)
		{
			String strMessage = "Can't override PathwaysRestAPIService instance for testing... PathwaysRestAPIService has already been initialized.";
			logger.Error(strMessage);
			throw new XPException(strMessage);
		}

		m_testInstance = testHelper;
	}	
	
	/**
	 * Checks if Pathways service is enabled from the config settings.  
	 * 
	 * @return true if Pathways is enabled, otherwise false.
	 */
	public boolean IsPathwaysEnabled()
	{
		return m_bIsPathwaysEnabled;
	}
	
	/**
	 * Gets the Pathways API base URL from the Pathways API Web Service object
	 * 
	 * @param session the current user session.
	 * @return the URL to the Pathways API Web Service, null if service is not installed.
	 */
	public String GetPathwaysAPIBaseURL(IPTSession session)
	{
		int pathwaysServiceId = PathwaysHelperService.GetInstance().GetAPIWebServiceID(session);
		int remoteServerId = ((IPTWebService) session.GetWebServices().Open(pathwaysServiceId, false)).GetRemoteServerID();
		IPTObjectManager spServers = session.GetGadgetContentServers();
		IPTGadgetContentServer server = (IPTGadgetContentServer)spServers.Open(remoteServerId, false);
		String webServiceURL = server.GetURL() + "api/";
		
		return webServiceURL;
	}
	
	/**
	 * Gets the Pathways API Gatewayed URL from the Pathways API Web Service object
	 * This is used when making Pathways API calls on the client side via JSON.
	 * 
	 * @param session the current user session.
	 * @return the gatewayed URL to the Pathways API Web Service, null if service is not installed.
	 */
	public String GetPathwaysAPIGatewayedURL(AActivitySpace space, IPTSession session)
	{
		int pathwaysServiceId = PathwaysHelperService.GetInstance().GetAPIWebServiceID(session);
		String pathwaysAPIBaseURL = GetPathwaysAPIBaseURL(session);
		
		/* contruct the gateway url to pathways access */
        String pathwaysAPIGatewayedURL = GatewayHelpers.ConstructGatewayURL(
        		space.GetCurrentRequestURL(), 
        		pathwaysServiceId, 0, 0, 0, PT_CLASSIDS.PT_WEBSERVICE_ID, 
        		pathwaysAPIBaseURL);
        
        return pathwaysAPIGatewayedURL;
	}
	
	/**
	 * Queries the Pathways API service and returns the current version of the service.
	 * 
	 * Note: this method is disabled for now as it is not being used and causes extra
	 * trips to the Pathways service. Re-enable when there's a need to detect Pathways 
	 * versioning in case of backward compatibility.
	 * 
	 * @return String representing the Pathways API version.
	 */
//	public String GetAPIVersion()
//	{
//		// TODO: need to be cached
//		// Example Pathways Views Request:
//		// http://se652dnet-pw15.devnet.plumtree.com:11960/graffiti/api/version?format=XML
//		<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
//		<soc:PVersion xmlns:soc="http://social.bea.com/pathways">
//	    	<version>Pathways REST API V.1.5</version>
//		</soc:PVersion>	
//		RestRequest req = RestRequest.GetInstance(m_strBaseURL + "/" + PathwaysConstants.VERSION_RESOURCE + "?" + PathwaysConstants.KEY_XML);
//		String version = null;
//		
//		PathwaysAuthenticate(req);
//		IRestResponse response = req.Submit();
//		
//		XPHashtable nodeData = (XPHashtable)response.GetSingleDataItem();
//		version = XPConvert.ToString(nodeData.GetElement(PathwaysConstants.API_VERSION));
//		
//		// TODO: error checking
//		return version;
//	}
	
	/**
	 * Queries the Pathways API service and returns an array of Pathways Views.
	 * 
	 * return PathwaysViewsRestData[] containing list of Pathways views.
	 */
	public PathwaysViewsRestData[] GetViews()
	{
		// Example Pathways Views Request:
		// http://se652dnet-pw15.devnet.plumtree.com:11960/graffiti/api/views?format=XML
//		<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
//		<soc:PViewsList xmlns:soc="http://social.bea.com/pathways">
//		    <view>
//		        <id>my</id>
//		        <lastModified>2009-02-03T15:40:18.011-08:00</lastModified>
//		        <name>My View</name>
//		    </view>
//		    <view>
//		        <id>ev</id>
//		        <lastModified>2009-02-03T15:40:18.042-08:00</lastModified>
//		        <name>Everyone View</name>
//		    </view>
//		    <view>
//		        <id>favorites</id>
//		        <lastModified>2009-02-03T15:40:18.027-08:00</lastModified>
//		        <name>Favorites</name>
//		    </view>
//		</soc:PViewsList>
		RestRequest req = RestRequest.GetInstance(m_strBaseURL + "/" + PathwaysConstants.VIEWS_RESOURCE + "?" + PathwaysConstants.KEY_XML,
					OpenHTTPServiceHelper.GetOpenHTTPService(), OpenHTTPServiceHelper.GetOpenHTTPSession());
		AddPathwaysAuthenticationToken(req);
		IRestResponse response = req.Submit();
		
		PathwaysViewsRestData[] result = null; 
		
		// TODO: Verify Response?
		result = GetViewsData(response);

		// TODO: error checking
		return result;
	}
	
	/**
	 * Returns a Tag Cloud from a Pathways search query.
	 * 
	 * @param view the Pathways view to use when retrieving the Tag Cloud
	 * @param format the payload format of the response
	 * @param query the Pathways search query
	 * @param tagCount the maximum number of tags to return for the Tag Cloud
	 * @return PathwaysTagRestData[] containing a list of tags forming the Tag Cloud.
	 */
	public PathwaysTagRestData[] GetTagCloud(String view, String format, String query, String resultCount)
	{
		// TODO: use XPStringUtility, use helper method for arg checking
		if ( null == view || "".equals(view) ) 
		{
			throw new XPIllegalArgumentException("Provided Pathways view cannot be empty");
		}
		
		if ( null == format || "".equals(format) ) 
		{
			throw new XPIllegalArgumentException("Provided request format cannot be empty");
		}
		
		if ( null == query || "".equals(query) ) 
		{
			throw new XPIllegalArgumentException("Provided query cannot be empty");
		}
		
		if ( null == resultCount || "".equals(resultCount) ) 
		{
			throw new XPIllegalArgumentException("Provided tag count cannot be empty");
		}
		
		// Example Tag Cloud request:
		// http://se652dnet-pw15.devnet.plumtree.com:11960/graffiti/api/tags/ev?format=XML&query=*&tagCount=20
//		<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
//		<soc:PTagList xmlns:soc="http://social.bea.com/pathways">
//		    <tag>
//		        <count>1</count>
//		        <normalizedCount>0</normalizedCount>
//		        <value>another tag</value>
//		    </tag>
//		    <tag>
//		        <count>1</count>
//		        <normalizedCount>0</normalizedCount>
//		        <value>www</value>
//		    </tag>
//		    <tag>
//		        <count>1</count>
//		        <normalizedCount>0</normalizedCount>
//		        <value>yahoo</value>
//		    </tag>
//		</soc:PTagList>		
		RestRequest req = RestRequest.GetInstance(m_strBaseURL + "/" + PathwaysConstants.TAGS_RESOURCE + "/" + view +
				"?" + format + "&" + PathwaysConstants.QUERY_ARG + "=" + query + "&" + PathwaysConstants.TAG_COUNT_ARG + "=" + resultCount,
				OpenHTTPServiceHelper.GetOpenHTTPService(), OpenHTTPServiceHelper.GetOpenHTTPSession());
		
		AddPathwaysAuthenticationToken(req);
		
		IRestResponse response = req.Submit();
		
		PathwaysTagRestData[] result = null; 
		
		// TODO: Verify Response?
		result = GetTagListData(response);
		
		//TODO: error checking

		return result;
	}
	
	/**
	 * Returns a Tag Cloud from a Pathways search query.
	 * 
	 * TODO: Use a Search Query Argument collection instead of passing in everything by arguments
	 * 
	 * @param view the Pathways view to use when retrieving the Tag Cloud
	 * @param format the payload format of the response
	 * @param query the Pathways search query
	 * @param tagCount the maximum number of tags to return for the Tag Cloud
	 * @param objectTypes is the object types to limit the search
	 * @return PathwaysSearchRestData[] containing a collection of search results generated from Pathways API service.
	 */
/*	public PathwaysSearchRestData[] DoSearch(String format, String startAt, String viewId, String query, 
			String resultCount, String order, String orderBy, XPArrayList objectTypes)
*/
	public PathwaysSearchRestData[] DoSearch(PathwaysQueryArguments pathwaysQueryArgs)
	{
		if (pathwaysQueryArgs.equals(null))
		{
			throw new XPIllegalArgumentException("Provided Pathways arguments cannot be null");
		}

		// Example Pathways Search Request:
		// http://bt-pathways-dev.devnet.plumtree.com:11960//graffiti/api/items?format=XML&startAt=1&viewId=ev&query=Layout%20Header%20Portlet&resultCount=10&order=DES&orderby=RANK&
		// objectTypes=PortalDocuments,DocumentFolders,Users,Communities,Portlets,CommunityPages,CollabDiscussion,CollabDocument,CollabEvent,CollabMessage,CollabProject,CollabTask,PublisherItems,
//		<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
//		<soc:PResultList xmlns:soc="http://social.bea.com/pathways">
//		    <item>
//		        <activityRank>0.0</activityRank>
//		        <classId>43</classId>
//		        <description>Layout Header showing Adaptive Tags</description>
//		        <folderClassId>20</folderClassId>
//		        <folderId>201</folderId>
//		        <folderName>\Admin Objects Directory\Portal Resources</folderName>
//		        <iconLink>plumtree/portal/public/img/icon_portlet.gif</iconLink>
//		        <isFavorite>false</isFavorite>
//		        <lastModified>2008-08-14T17:51:00-07:00</lastModified>
//		        <name>Layout Header Portlet</name>
//		        <objectId>35</objectId>
//		        <path>\Admin Objects Directory\Portal Resources</path>
//		        <rank>1</rank>
//		        <resultScore>42.2</resultScore>
//		        <source>PTPORTAL</source>
//		        <type>Portlet</type>
//		        <URL>http://bt-trinity-dev.devnet.plumtree.com/portal/server.pt?open=43&amp;objID=35&amp;mode=2&amp;in_hi_userid=1</URL>
//		        <viewDetailURL>http://bt-trinity-dev.devnet.plumtree.com/portal/server.pt?objID=35&amp;open=43&amp;mode=3&amp;subspace=mySubspaceKey&amp;space=Opener</viewDetailURL>
//		    </item>
//		    <totalCount>1</totalCount>
//		</soc:PResultList>	

		//we must now set the object restrictions in pathways, pathways takes a comma delimited string
		String toSearchObjectTypes = "";
		String pathwaysQuery = "";
		String adminFolders = "";
		String dirFolders = "";
		String isSpellCheck = "";

		//we must check for advanced search, b/c if we do, we need to add additional control arguments to the REST request
		boolean isAdvancedSearch = pathwaysQueryArgs.isAdvancedSearch();
		
		if (!isAdvancedSearch)
		{
			pathwaysQuery = pathwaysQueryArgs.GetQuery();
			
		}
		//else this is advanced search
		else
		{
			//need to construct the more complex search criteria
			XPArrayList advancedCriteria = pathwaysQueryArgs.GetAdvancedPropertyContraints();
			String topLevelOperator = pathwaysQueryArgs.GetAdvancedSearchTopPropertyOperator();
			
			//search terms could be empty for advanced search, if this is the case, we need to handle this case.
			String searchTerm = pathwaysQueryArgs.GetQuery();
			if ( null == pathwaysQueryArgs.GetQuery())
			{
				//if we have a null search term, we'll change it to empty string
				searchTerm = "*";
			}
			
			//user has selected that all search criteria must be used
			if (opAND.equals(topLevelOperator))
			{   
				pathwaysQuery = SetPathwaysQueryWithANDOperator(advancedCriteria, searchTerm);
			}
			//else the operator is OR and we have to construct multiple strings for each OR
			//for example:  test AND (Author:"John Hill" OR Keyword:"Miami") ===> test AND Author:"John Hill" OR test AND Keyword:"Miami"
			else {
				pathwaysQuery = SetPathwaysQueryWithOROperator(advancedCriteria, searchTerm);
			}
			
			//getting adminFolders to restrain to
			adminFolders = pathwaysQueryArgs.GetStringAdminFolders();
			if (null == adminFolders)
			{
				adminFolders = "";
			}
			
			//getting dirFolders to restrain to
			dirFolders = pathwaysQueryArgs.GetStringDirFolders();
			if (null == dirFolders)
			{
				dirFolders = "";
			}			
		}
		
		//look for spellcheck now
		isSpellCheck = XPConvert.ToString(pathwaysQueryArgs.GetSpellCheck());

		
		XPArrayList objectTypes = pathwaysQueryArgs.GetObjectTypes();
		for (int i=0; i<objectTypes.GetSize(); i++)
		{
			toSearchObjectTypes = toSearchObjectTypes + XPConvert.ToString(objectTypes.GetElement(i)) + ",";
		}
		
		// Bug 10201138 - Ensure query string is URL encoded before making the rest request
		pathwaysQuery = XPHttpUtility.UrlEncode(pathwaysQuery, LinkConstants.UTF_8_ENCODING);
		
		RestRequest req = RestRequest.GetInstance(m_strBaseURL + "/" + PathwaysConstants.ITEMS_RESOURCE + "?" + 
				PathwaysConstants.FORMAT_ARG + "=" + pathwaysQueryArgs.GetFormat() + 
				"&" + PathwaysConstants.START_AT_ARG + "=" + pathwaysQueryArgs.GetStartAt() + 
				"&" + PathwaysConstants.VIEW_ID_ARG + "=" + pathwaysQueryArgs.GetViewId() + 
				"&" + PathwaysConstants.QUERY_ARG + "=" + pathwaysQuery + 
				"&" + PathwaysConstants.RESULT_COUNT_ARG + "=" + pathwaysQueryArgs.GetResultCount() + 
				"&" + PathwaysConstants.ORDER_ARG + "=" + pathwaysQueryArgs.GetOrder() + 
				"&" + PathwaysConstants.ORDER_BY_ARG + "=" + pathwaysQueryArgs.GetOrderBy() + 
				"&" + PathwaysConstants.OBJECT_TYPES_ARG + "=" + toSearchObjectTypes + 
				"&" + PathwaysConstants.ADMINFOLDERS_ARG + "=" + adminFolders + 
				"&" + PathwaysConstants.DIRFOLDERS_ARG + "=" + dirFolders + 
				"&" + PathwaysConstants.SPELLCHECK_ARG + "=" + isSpellCheck,
				OpenHTTPServiceHelper.GetOpenHTTPService(), 
				OpenHTTPServiceHelper.GetOpenHTTPSession());
		
		AddPathwaysAuthenticationToken(req);
		
		IRestResponse response = req.Submit();
		
		PathwaysSearchRestData[] result = null; 
		
		// TODO: Verify Response?
		result = GetSearchResultsData(response);
		
		//check for null searchData value
		
		//getting total count value from this request
		m_totalCount = GetTotalCountSearchResults(response);

		return result;
	}
	
	
	/**
	 * This Method is used to Add/Remove a Pathways Tag on a particular document. This uses 
	 * Pathways REST API to connect to Pathways and execute the requested action.
	 * @param tname Name of the Tag on Which to take action
	 * @param fullId Complete ID (CLASSID_DOCUMENTID) of the Document the tag is related to 
	 * @param action Operation to perform on the tag, 0 = Add, 1 = Delete, 2 = Edit (delete old tag and add new tag)
	 * @param ptsource Source of the Object, for Portal it is 'PTPORTAL' and collab it is 'PTCOLLAB'. More sources are in PathwaysConstants.
	 */
	public void AddRemoveTagRest(String tname, String fullId, int action, String ptsource) 
	{					
		String strUrl = m_strBaseURL + "/" + PathwaysConstants.KEY_ITEM +  "/"	+ ptsource 
						+ "/" + fullId + "/" + PathwaysConstants.KEY_TAGS;
			
		logger.Info("REST Request URL Used: " + strUrl);
		RestRequest req = RestRequest.GetInstance(strUrl, OpenHTTPServiceHelper.GetOpenHTTPService(), OpenHTTPServiceHelper.GetOpenHTTPSession());
			
		// Process Tag Operation here.
		if (action == TaggingControl.ACTION_ADD_TAG) 
		{
			// Set Request to Add this New Tag
			req.SetRequestMethod("POST");
		} 
		else
		{
			// Set Request to remove this New Tag
			req.SetRequestMethod("DELETE");
		}
		ExecuteAddRemoveTagAction(req, tname);
	}
	
	/**
	 * This Method is used to Rename a Pathways Tag on a particular document. This uses 
	 * Pathways REST API to connect to Pathways and execute the requested action.
	 * @param tname the Source Tag Name to be used for renaming
	 * @param fullId Complete ID (CLASSID_DOCUMENTID) of the Document the tag is related to 
	 * @param ptsource Source of the Object, for Portal it is 'PTPORTAL' and collab it is 'PTCOLLAB'. More sources are in PathwaysConstants.
	 * @param renametag the Target Tag Name to be used for renaming 
	 */
	public void RenameTagRest(String tname, String fullId, String ptsource, String renametag) 
	{					
		String strUrl = m_strBaseURL + "/" + PathwaysConstants.KEY_ITEM +  "/"	+ ptsource 
						+ "/" + fullId + "/" + PathwaysConstants.KEY_TAGS;
			
		logger.Info("REST Request URL Used: " + strUrl);
		RestRequest req = RestRequest.GetInstance(strUrl, OpenHTTPServiceHelper.GetOpenHTTPService(), OpenHTTPServiceHelper.GetOpenHTTPSession());
		req.SetRequestMethod("DELETE");
		ExecuteAddRemoveTagAction(req, tname);
		req.SetRequestMethod("POST");
		ExecuteAddRemoveTagAction(req, renametag);
	}
	
	private void ExecuteAddRemoveTagAction(RestRequest req, String tname)
	{
		//Set Request Body for the Tag			
		req.SetPostBody(GetTagPostBody(tname));
			
		//Set Content-Type Header, to utf-8 so that we can also handle international strings.
		req.SetContentTypeHeader("text/html; charset=utf-8");
		
		//Set Header and Send Request
		AddPathwaysAuthenticationToken(req);

		IRestResponse objResponse =  req.Submit();
		// TODO: If Request Fails throws error.
		// Process Response and return success/failure.
		if(null == objResponse) {
			throw new XPException("Failed to submit request properly, Verify Pathways is configured properly! ");
		}
		XPHashtable resItem = objResponse.GetSingleDataItem();
		if(null != resItem) {
			XPHashtable tagItem = (XPHashtable) resItem.GetElement(PathwaysConstants.KEY_TAG);
			
			if(null != tagItem){
				String respTagName = (String) tagItem.GetElement(PathwaysConstants.TAG_VALUE);
				
				if(XPStringUtility.CompareIgnoreCase(tname, respTagName) != 0){
					throw new XPException("Pahtways should remove tag: " + tname + " it removed tag: " + respTagName );
				}
				if(null == respTagName ) {
					throw new XPException("Failed to get Valid Response from Pathways ");
				}
			}
		}		
	}
	
	/**
	 * Set authentication 
	 * 
	 * @param request
	 */
	protected void AddPathwaysAuthenticationToken(RestRequest request)
	{
		String token = m_session.GetToken(5);
		request.SetCSPLoginToken(token);
	}
	
	protected PathwaysTagRestData[] GetTagListData(IRestResponse response)
	{
		IXPList nodesList = response.GetResponseList();		
		PathwaysTagRestData[] tags = new PathwaysTagRestData[nodesList.GetSize()];
		
		for ( int i = 0; i < nodesList.GetSize(); i++ ) 
		{
			XPHashtable tagNode = (XPHashtable)nodesList.GetElement(i);
			PathwaysTagRestData data = new PathwaysTagRestData();
			if ( data.Init(tagNode) ) 
			{
				tags[i] = data;
			}
		}
		
		return tags;
	}
	
	/**
	 * 
	 * @param response
	 * @return
	 */
	protected PathwaysSearchRestData[] GetSearchResultsData(IRestResponse response)
	{
		// TODO: error checking
		XPArrayList responseList = null;
		
		XPHashtable itemHash = response.GetSingleDataItem();
		Object tempResponse = itemHash.GetElement(PathwaysConstants.KEY_ITEM);
		if (tempResponse instanceof XPArrayList)
		{
			responseList = (XPArrayList)itemHash.GetElement(PathwaysConstants.KEY_ITEM);

		}
		//if a single <item> is returned, the object type is of XPHashtable and not XPArrayList, so need to handle this case
		else if (tempResponse instanceof XPHashtable) 
		{
			responseList = new XPArrayList();
			responseList.Add( (XPHashtable)itemHash.GetElement(PathwaysConstants.KEY_ITEM) );
		}
		//it's null or invalid
		else 
		{
			logger.Info("PathwaysRestAPIService.GetSearchResultsData:  could not find an item in the search response of type XPArrayList or XPHashtable");
			return null;
		}
		
		int responseSize = responseList.GetSize();

		PathwaysSearchRestData[] searchResults = new PathwaysSearchRestData[responseSize];

		// TODO: comment
		for ( int i = 0; i < responseSize; i++ ) 
		{	
			XPHashtable tagNode = (XPHashtable)responseList.GetElement(i);			
			PathwaysSearchRestData data = new PathwaysSearchRestData();
			if ( data.Init(tagNode) ) 
			{
				searchResults[i] = data;
			}
		}

		return searchResults;
	}
	
	/*
	 * this will return the totalcount of the search results of the pathways search
	 */
	protected int GetTotalCountSearchResults(IRestResponse response)
	{
		int totalCount = -1;
		XPHashtable totalCountNode = response.GetSingleDataItem();
		
		if (null == totalCountNode)
		{
			logger.Info("PathwaysRestAPIService.GetTotalCountSearchResults:  totalCountNode returned a null");
			return totalCount;
		}

		if (totalCountNode.ContainsKey(PathwaysConstants.SEARCH_TOTAL_COUNT))
		{
			totalCount = XPConvert.ToInteger(totalCountNode.GetElement(PathwaysConstants.SEARCH_TOTAL_COUNT));
		}

		return totalCount;
	}
	
	/*
	 * GET TOTAL COUNT
	 */
	public int GetTotalCount()
	{
		return m_totalCount;
	}
	
	
	protected PathwaysViewsRestData[] GetViewsData(IRestResponse response)
	{
		IXPList nodesList = response.GetResponseList();		
		PathwaysViewsRestData[] views = new PathwaysViewsRestData[nodesList.GetSize()];
		
		for ( int i = 0; i < nodesList.GetSize(); i++ ) 
		{
			XPHashtable tagNode = (XPHashtable)nodesList.GetElement(i);
			PathwaysViewsRestData data = new PathwaysViewsRestData();
			if ( data.Init(tagNode) ) 
			{
				views[i] = data;
			}
		}
		
		// TODO: validation, error checking
		return views;
	}
	
	/**
	 * This method builds HTTP POST Body that is needed by pathways
	 * to Add/Remove Tag. This also creates post body for comma seperated
	 * list of tags given input by user to add multiple tags.
	 * @param name Takes Input the name of the tag, or comma seperated list of tags.
	 * @return Post body XML for Pathways Tagging.
	 */
	
	private String GetTagPostBody(String name) {
		
		XPStringBuilder strBody = new XPStringBuilder();
		
		strBody.Append("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>");
		strBody.Append("<soc:PTagList xmlns:soc=\"http://social.bea.com/pathways\">");
		
		if(XPStringUtility.IndexOf(name, ",") >= 0)
		{
			XPStringTokenizer strTok = new XPStringTokenizer(name, ",");
			while(strTok.HasMoreTokens()) {
				String tagItem = XPStringUtility.Trim(strTok.GetNextToken());
				if(null != tagItem && !XPStringUtility.EqualsIgnoreCase(tagItem, ""))
				{
					strBody.Append("<tag><value>" + tagItem  + "</value></tag>");
				}				
			}
		}else {
			strBody.Append("<tag><value>" + name + "</value></tag>");
		}
		
		strBody.Append("</soc:PTagList>");
		
		return strBody.ToString();
	}
	
	private String SetPathwaysQueryWithANDOperator(XPArrayList advancedCriteria, String searchTerm)
	{
		XPStringBuilder strQuery = new XPStringBuilder(searchTerm);
		
		for (int i=0; i<advancedCriteria.GetSize(); i++)
		{
			strQuery.Append(" ").Append(opAND).Append(" ");
			strQuery.Append(XPConvert.ToString(advancedCriteria.GetElement(i)));
		}
		return strQuery.toString();
	}
	
	private String SetPathwaysQueryWithOROperator(XPArrayList advancedCriteria, String searchTerm)
	{
		XPStringBuilder strQuery = new XPStringBuilder();

		for (int i=0; i<advancedCriteria.GetSize(); i++)
		{
			strQuery.Append(searchTerm);
			strQuery.Append(" ").Append(opAND).Append(" ");
			strQuery.Append(XPConvert.ToString(advancedCriteria.GetElement(i)));
			strQuery.Append(" ").Append(opOR).Append(" ");
			String deleteOR= " " + opOR + " ";
			int endindex = strQuery.GetLength() - deleteOR.length();
			return strQuery.Substring(0, endindex);
		}
		strQuery.Append(searchTerm);
		return strQuery.toString();
	}	
	
	  //this function is checking if the current access style is STANDARD and the layout mode is enabled, if true
	  //adaptive layouts are supported.  If either one is false, then we're not in adaptive layout mode.

	public boolean IsAdaptiveLayoutMode()
	{
		boolean standardStyle = m_asOwner.GetIsAccessStyleStandard();
		boolean isLayoutMode = LayoutService.getChooserInstance().GetPageLayoutsEnabled(m_asOwner);
		IPTSession userSess = (IPTSession)m_asOwner.GetUserSession();
		boolean isExpLayoutMode = userSess.GetSessionInfo().GetCurrentUserCachedSubPortalInfo().GetLayoutEnabled();
		return (standardStyle && isLayoutMode && isExpLayoutMode);
	}
}
