package com.plumtree.portaluiinfrastructure.resultwrapper;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;

/**
 * PTObjectManagerQueryWrapper: The PTObjectManagerQueryWrapper class wraps the
 * IPTTurboQueryResult  Server object that's returned by the various Query
 * methods of IPTObjectManager. This wrapper handles the details of retrieving
 * a subset of the data, sorting on the client when necessary, etc.
 *
 * @author Michael Smedber
 */
public class APTQueryWrapper extends PTSortedArrayWrapper
	implements IASQueryResult
{
	/** MES- By default, we only show the first 1000 rows. */
	public static final int PTTQRW_DEFAULT_MAX_ROWS = 1000;

	/**
	 * MES- In this mode, all the data is on the client, we can sort on the
	 * client.
	 */
	public static final int PTTQRW_MODE_CLIENT = 0;

	/**
	 * MES- In this mode, the data we hold is truncated, we have to go to the
	 * Server for sorts.
	 */
	public static final int PTTQRW_MODE_TRUNCATED = 1;

	/** MES- The maximum number of rows we'll return */
	protected int m_iMaxRows = 0;

	/**
	 * MES- Our mode- we have to treat large result sets special.  Value should
	 * be PTTQRW_MODE_CLIENT or PTTQRW_MODE_TRUNCATED
	 */
	protected int m_iMode = 0;

	public APTQueryWrapper()
	{
		super();
		m_iMaxRows = PTTQRW_DEFAULT_MAX_ROWS;
		m_iMode = PTTQRW_MODE_CLIENT;
	}

	/**
	 * MES- GetFields emulates the IPTQueryResult call of the same name. It's
	 * just a tiny wrapper around PTSortedArrayWrapper::GetItem, but it offers
	 * the name expected by IPTQueryResult callers.
	 *
	 * @param iRow is the row desired
	 * @param iPropID is the PT_PROPIDs id of the Property desired
	 *
	 * @return the object from the array
	 */
	public Object GetFields(int iRow, int iPropID)
	{
		return GetItem(iRow, iPropID);
	}

	/**
	 * MES- IsComplete returns a boolean indicating if the rows we can return
	 * are the complete rowset (i.e. have we truncated the rowset because it's
	 * too large to bring to the client.)
	 *
	 * @return A boolean indicating completeness.
	 */
	public boolean IsComplete()
	{
		if (PTTQRW_MODE_CLIENT == m_iMode)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * MES- MaxRows returns the maximum number of rows an instance may contain
	 * before entering PTTQRW_MODE_TRUNCATED mode.
	 *
	 * @return an int containing the maximum rows.
	 */
	public int MaxRows()
	{
		return m_iMaxRows;
	}

	/**
	 * MES- SetMaxRows allows clients to set the maximum number of rows that
	 * this     wrapper will hold before entering PTTQRW_MODE_TRUNCATED mode.
	 * Call this     function BEFORE querying.
	 *
	 * @param iMaxRows is the maximum number of rows desired, which must be > 0
	 *
	 * @throws XPException 
	 */
	public void SetMaxRows(int iMaxRows)
	{
		if (0 < iMaxRows)
		{
			m_iMaxRows = iMaxRows;
		}
		else
		{
			throw new XPException("Error in APTQueryWrapper::SetMaxRows, " +
				XPConvert.ToString(iMaxRows) +
				" is not an allowed value for Max Rows.  Value must be greater than 0.");
		}
	}
}

