package com.plumtree.portaluiinfrastructure.resultwrapper;

import com.plumtree.openfoundation.util.*;
import com.plumtree.server.*;

/**
 * PTObjectManagerQueryWrapper: The PTObjectManagerQueryWrapper class wraps the
 * IPTTurboQueryResult  Server object that's returned by the various Query
 * methods of IPTObjectManager. This wrapper handles the details of retrieving
 * a subset of the data, sorting on the client when necessary, etc.
 *
 * @author Michael Smedberg
 */
public class PTObjectManagerQueryWrapper extends APTQueryWrapper
{
	/** MES- The ObjectManager we're wrapping */
	protected IPTObjectManager m_ObjectManager;

	/** MES- The Query Filter we're using, if we need to requery to resort */
	protected Object[][] m_objQueryFilter;

	/** MES- The Folder ID we're using, if we need to requery to resort */
	protected int m_iFolderID;

	/** MES- The Property ID mask we're using, if we need to requery to resort */
	protected int m_iPropIDMask;

	/**
	 * MES- The constructor is used by clients to attach this instance to an
	 * Object Manager
	 *
	 * @param om is the Object Manager we should be querying
	 */
	public PTObjectManagerQueryWrapper(IPTObjectManager om)
	{
		m_ObjectManager = om;
	}

	/**
	 * MES- This version of Initialize masks the Initialize of our superclass-
	 * we want to  make sure that clients don't call the wrong version of
	 * Initialize.
	 *
	 * @param arrColumnIDs is not used
	 * @param arrData is not used
	 * @param arrSort is not used
	 * @param bRequiresSort is not used
	 *
	 * @throws XPException is not used
	 */
	public void Initialize(int[] arrColumnIDs, Object[][] arrData,
		int[][] arrSort, boolean bRequiresSort)
	{
		throw new XPException(
			"APTQueryWrapper::Initialize(int[] arrColumnIDs, Object[][] arrData, int[][] arrSort, boolean bRequiresSort) is NOT supported.");
	}

	/**
	 * MES- Query wraps the ObjectManager call of the same name
	 *
	 * @param iPropIDMask is a bitmask of the properties to be returned as
	 *        columns
	 * @param iFolderID is the FolderID to query in, or -1 for all folders
	 * @param arrSort is a 2D int array of PropertyIDs to sort by.  It should
	 *        contain     a column for each sort.  The first row holds the
	 *        PropertyID, and the second row     holds the sort direction,
	 *        PTSAW_SORT_ASCENDING or PTSAW_SORT_DESCENDING
	 * @param objQueryFilter is a Filter to include in the query, or null to
	 *        skip filtering
	 *
	 * @return boolean indicating success or failure
	 */
	public boolean Query(int iPropIDMask, int iFolderID, int[][] arrSort,
		Object[][] objQueryFilter)
	{
		return Query(iPropIDMask, iFolderID, arrSort, 0, objQueryFilter);
	}

	/**
	 * MES- Query wraps the ObjectManager call of the same name
	 *
	 * @param iPropIDMask is a bitmask of the properties to be returned as
	 *        columns
	 * @param iFolderID is the FolderID to query in, or -1 for all folders
	 * @param arrSort is a 2D int array of PropertyIDs to sort by.  It should
	 *        contain     a column for each sort.  The first row holds the
	 *        PropertyID, and the second row     holds the sort direction,
	 *        PTSAW_SORT_ASCENDING or PTSAW_SORT_DESCENDING
	 * @param iSkipRows is the index of the first row to start the query on
	 * @param objQueryFilter is a Filter to include in the query, or null to
	 *        skip filtering
	 *
	 * @return boolean indicating success or failure
	 */
	public boolean Query(int iPropIDMask, int iFolderID, int[][] arrSort,
		int iSkipRows, Object[][] objQueryFilter)
	{
		//MES- If the sort is null, we'll sort by name, ascending, which
		//	is the default in the server
		if (null == arrSort)
		{
			arrSort									 = new int[1][2];
			arrSort[0][PTSAW_SORT_ROW_PROPID]		 = PT_PROPIDS.PT_PROPID_NAME;
			arrSort[0][PTSAW_SORT_ROW_DIRECTION]     = PTSAW_SORT_ASCENDING;
		}

		//MES- We'd like to just pass the arguments through to the Query server method, 
		//	but ObjectManager::Query doesn't support sorting on more than 3 columns, so 
		//	we may have to truncate our sort array, then sort on the client.
		int[][] arrInternalSort;
		boolean bRequiresClientSort;

		if (3 < arrSort.length)
		{
			arrInternalSort = new int[3][2];

			for (int iSortIndex = 0; iSortIndex < 3; ++iSortIndex)
			{
				arrInternalSort[iSortIndex][PTSAW_SORT_ROW_PROPID]		  = arrSort[iSortIndex][PTSAW_SORT_ROW_PROPID];
				arrInternalSort[iSortIndex][PTSAW_SORT_ROW_DIRECTION]     = arrSort[iSortIndex][PTSAW_SORT_ROW_DIRECTION];
			}

			//MES- The server can't do all of the sorting, so we should try to do it on the client
			bRequiresClientSort = true;
		}
		else
		{
			//MES- The sort array is fine the way it is, don't monkey with it.
			arrInternalSort     = arrSort;

			//MES- The server may be able to do the sort, we may not be required to do it
			bRequiresClientSort = false;
		}

		//MES- Perform the query		
		IPTQueryResult qr = m_ObjectManager.Query(iPropIDMask, iFolderID,
			arrInternalSort, iSkipRows, /*iSkipRows + */m_iMaxRows, objQueryFilter);

		//MES- Retrieve the data
		int[] cols	    = (int[]) qr.Columns();
		Object[][] data = (Object[][]) qr.Data();

		//MES- Did we get a complete data set?  If not, we can't sort on the client, and
		//	every time the user of this class asks for a sort, we have to requery the Server.
		int iRowsReturned = qr.RowCount();

		if (iRowsReturned >= m_iMaxRows)
		{
			//MES- This is "large" mode- don't sort on client, since we don't have all the data
			m_iMode     = PTTQRW_MODE_TRUNCATED;

			//MES- Record the query data, so that we can requery on sort
			m_iPropIDMask		 = iPropIDMask;
			m_iFolderID			 = iFolderID;
			m_objQueryFilter     = objQueryFilter;

			//MES- Initialize the superclass, SKIPPING client sorting, and we're done.
			//	NOTE: We may have truncated the sort above, so the data may not be
			//	sorted right, but we can't correct it, since we don't have all the data.
			//	You might argue, "OK, but we should STILL sort it on the client as best
			//	we can."  That's sorta OK, but it has a problem.  Suppose that the result
			//	from the server was sorted by Name, and that Name is localized.  Sorting
			//	of localized data can't be done on the Server in 5.0, so we're getting
			//	back data that's not sorted.  For example, the names might be A, Z, B.
			//	If we sort this on the client, we'd be giving the user the impression that
			//	they're seeing all the names through Z.  It's kinda ugly to show A, Z, B,
			//	but it's even worse to show A, B, Z, when the next item might be BB.
			//	We'd rather present an unsorted list than fool the user.  Fortunately, this
			//	shouldn't come up often, since we should usually get the whole result set.
			//MES- Note:  We're initalizing with our INTERNAL sort, since that's what
			//	was really used.  If the client specified more sort columns, but the data
			//	set is too large, we just can't do it.
			super.Initialize(cols, data, arrInternalSort, false);

			//MES- OK, we're ready for use
			return true;
		}
		else
		{
			//MES- The result set isn't large, so we CAN sort on the client, but
			//	we may not need to, because the result may already be sorted by the
			//	server.
			m_iMode = PTTQRW_MODE_CLIENT;

			//MES- If we truncated the sort above, then we definitely need to sort
			//	on the client, but if we did NOT truncate, we may STILL need to
			//	sort on the client, since the server may have failed to sort correctly.
			if (!bRequiresClientSort)
			{
				int iServerSorted = qr.IsOrdered();

				if (0 == iServerSorted)
				{
					bRequiresClientSort = true;
				}
			}

			//MES- Initialize ourself with the REAL sort!
			super.Initialize(cols, data, arrSort, bRequiresClientSort);

			//MES- OK, we're ready for use
			return true;
		}
	}

	/**
	 * MES- QuerySingleObject wraps the ObjectManager call of the same name
	 *
	 * @param iObjectID is the ObjectID of the object to be queried
	 *
	 * @return boolean value indicating success or failure
	 */
	public boolean QuerySingleObject(int iObjectID)
	{
		IPTQueryResult qr = m_ObjectManager.QuerySingleObject(iObjectID);
		int[] cols		  = (int[]) qr.Columns();
		Object[][] data   = (Object[][]) qr.Data();

		//MES- QuerySingleObject sorts by ObjectID ascending, not that it really matters
		int[][] sort = new int[1][2];
		sort[0][PTSAW_SORT_ROW_PROPID]		  = PT_PROPIDS.PT_PROPID_OBJECTID;
		sort[0][PTSAW_SORT_ROW_DIRECTION]     = PTSAW_SORT_ASCENDING;

		//MES- Initialize ourself, we don't need a sort, since we have at most 1 row.
		super.Initialize(cols, data, sort, false);

		return true;
	}

	/**
	 * MES- SetMaxRows allows clients to set the maximum number of rows that
	 * this     wrapper will hold before entering PTTQRW_MODE_TRUNCATED mode.
	 * Call this     function BEFORE querying.
	 *
	 * @param iMaxRows is the maximum number of rows desired, which must be > 0
	 *
	 * @throws XPException 
	 */
	public void SetMaxRows(int iMaxRows)
	{
		if (0 < iMaxRows)
		{
			m_iMaxRows = iMaxRows;
		}
		else
		{
			throw new XPException(
				"Error in PTObjectManagerQueryWrapper::SetMaxRows, " +
				XPConvert.ToString(iMaxRows) +
				" is not an allowed value for Max Rows.  Value must be greater than 0.");
		}
	}

	/**
	 * MES- SimpleQuery wraps the ObjectManager call of the same name     This
	 * function queries all folders, and orders by name.
	 *
	 * @return boolean indicating success or failure
	 */
	public boolean SimpleQuery()
	{
		return SimpleQuery(-1, PT_PROPIDS.PT_PROPID_NAME);
	}

	/**
	 * MES- SimpleQuery wraps the ObjectManager call of the same name.     This
	 * function orders results by name.
	 *
	 * @param iFolderID indicates a folder to be queried, or -1 for all folders
	 *
	 * @return boolean indicating success or failure
	 */
	public boolean SimpleQuery(int iFolderID)
	{
		return SimpleQuery(iFolderID, PT_PROPIDS.PT_PROPID_NAME);
	}

	/**
	 * MES- SimpleQuery wraps the ObjectManager call of the same name.
	 *
	 * @param iFolderID indicates a folder to be queried, or -1 for all folders
	 * @param iPropToOrderBy indicates the PropertyID to sort by.  The sort
	 *        will be ascending.
	 *
	 * @return boolean indicating success or failure
	 */
	public boolean SimpleQuery(int iFolderID, int iPropToOrderBy)
	{
		//MES- SimpleQuery sorts ascending
		int[][] arrSort = new int[1][2];
		arrSort[0][PTSAW_SORT_ROW_PROPID]		 = iPropToOrderBy;
		arrSort[0][PTSAW_SORT_ROW_DIRECTION]     = PTSAW_SORT_ASCENDING;

		return SimpleQuery(iFolderID, arrSort);
	}

	/**
	 * MES- SimpleQuery wraps the ObjectManager call of the same name.
	 *
	 * @param iFolderID indicates a folder to be queried, or -1 for all folders
	 * @param arrSort is a 2D int array of PropertyIDs to sort by.  It should
	 *        contain     a column for each sort.  The first row holds the
	 *        PropertyID, and the second row     holds the sort direction,
	 *        PTSAW_SORT_ASCENDING or PTSAW_SORT_DESCENDING
	 *
	 * @return boolean indicating success or failure
	 */
	public boolean SimpleQuery(int iFolderID, int[][] arrSort)
	{
		//MES- You'd think that we'd just call SimpleQuery on the ObjectManager.
		//	However, this wrapper is supposed to limit return arrays to a
		//	max number of rows- we don't want to copy 100,000 rows to the client!
		//	Therefore, we're not going to call SimpleQuery, we're going to call Query instead
		//MES- Pass through to the Query method, so we can centralize
		//	the code.
		//MES- SimpleQuery selects all columns except Locked
		int iPropIDMask = PT_PROPIDS.PT_PROPID_ALL &
			~PT_PROPIDS.PT_PROPID_LOCKED;

		//MES- SimpleQuery doesn't support filtering
		Object[][] objFilter = null;

		return Query(iPropIDMask, iFolderID, arrSort, objFilter);
	}

	/**
	 * MES- Sort is used to resort the data.  Sort will do an in-memory sort,
	 * or     will requery the server, as needed.  If the server is re-queried,
	 * the skipped rows from the original query will be ignored.
	 *
	 * @param arrSort is a 2D int array of PropertyIDs to sort by.  It should
	 *        contain     a column for each sort.  The first row holds the
	 *        PropertyID, and the second row     holds the sort direction,
	 *        PTSAW_SORT_ASCENDING or PTSAW_SORT_DESCENDING
	 *
	 * @throws XPException 
	 */
	public void Sort(int[][] arrSort)
	{
		//MES- We could check the query type here, and only allow 
		//	sorting in the way the Server would allow sorting,
		//	but that seems unnecessary.  Why shouldn't we be more
		//	flexible than the server?
		//MES- If we have all the data, simply sort
		if (PTTQRW_MODE_CLIENT == m_iMode)
		{
			//MES- We have the data, it's a client sort.
			//	Pass the sort up to the superclass.
			super.Sort(arrSort);
		}
		else if (PTTQRW_MODE_TRUNCATED == m_iMode)
		{
			//MES- We don't have all the data, so we can't do a client sort.
			//	Just redo the query with the new sort.
			if (!Query(m_iPropIDMask, m_iFolderID, arrSort, m_objQueryFilter))
			{
				throw new XPException(
					"Error in PTObjectManagerQueryWrapper::Sort, requery failed.");
			}
		}
		else
		{
			//MES- ERROR! Unknown mode!
			throw new XPException(
				"Error in PTObjectManagerQueryWrapper::Sort, unknown mode: " +
				XPConvert.ToString(m_iMode) + ".");
		}
	}
}

