package com.plumtree.portaluiinfrastructure.resultwrapper;

import com.plumtree.openfoundation.util.*;
import com.plumtree.server.*;

/**
 * APTSortedQueryWrapper: The PTSortedDocumentsQueryWrapper class wraps the
 * IPTTurboQueryResult Server object that's returned by the various Query
 * methods of IPTFolder. This wrapper handles the details of retrieving
 * a subset of the data, sorting on the client when necessary, etc.
 *
 * @author rupap
 */
public class PTSortedDocumentsQueryWrapper extends APTQueryWrapper
{
	/** MES- The Folder ID we're using, if we need to requery to resort */
	protected IPTFolder m_ptFolder;

	/** DH- The query filter we're using, if we need to requery to resort */
	protected Object[][] m_ptqf = null;

	/** MES- The Property ID mask we're using, if we need to requery to resort */
	protected int m_iPropIDMask;

	/** RP- false to show only approved docs, true to show unapproved docs also */
	protected boolean m_bShowUnapproved;

	/** SCC - Number of rows to skip (used when doing displaying ranges of Documents */
	protected int m_nSkippedRows = 0;

	/**
	 * RP- The constructor is used by clients to attach this instance to a
	 * Catalog Folder.
	 *
	 * @param ptFolder is the folder we should be querying
	 */
	public PTSortedDocumentsQueryWrapper(IPTFolder ptFolder)
	{
		m_ptFolder = ptFolder;
	}

	/**
	 * MES- This version of Initialize masks the Initialize of our superclass-
	 * we want to  make sure that clients don't call the wrong version of
	 * Initialize.
	 *
	 * @param arrColumnIDs is not used
	 * @param arrData is not used
	 * @param arrSort is not used
	 * @param bRequiresSort is not used
	 *
	 * @throws XPException is not used
	 */
	public void Initialize(int[] arrColumnIDs, Object[][] arrData,
		int[][] arrSort, boolean bRequiresSort)
	{
		throw new XPException(
			"APTQueryWrapper::Initialize(int[] arrColumnIDs, Object[][] arrData, int[][] arrSort, boolean bRequiresSort) is NOT supported.  Call Initialize(IPTObjectManager om) instead.");
	}

	/**
	 * MES- Query wraps the IPTFolder call of the same name
	 *
	 * @param iPropIDMask is a bitmask of the properties to be returned as
	 *        columns
	 * @param iShowUnapproved
	 * @param arrSort is a 2D int array of PropertyIDs to sort by.  It should
	 *        contain     a column for each sort.  The first row holds the
	 *        PropertyID, and the second row     holds the sort direction,
	 *        PTSAW_SORT_ASCENDING or PTSAW_SORT_DESCENDING
	 * @param queryFilter is a 2D Object array that is a query filter
	 *
	 * @return boolean indicating success or failure
	 */
	public boolean QueryCards(int iPropIDMask, boolean bShowUnapproved,
		int[][] arrSort, Object[][] queryFilter)
	{
		m_ptqf = queryFilter;

		//MES- If the sort is null, we'll sort by name, ascending, which
		//	is the default in the server
		if (null == arrSort)
		{
			arrSort									 = new int[1][2];
			arrSort[0][PTSAW_SORT_ROW_PROPID]		 = PT_PROPIDS.PT_PROPID_NAME;
			arrSort[0][PTSAW_SORT_ROW_DIRECTION]     = PTSAW_SORT_ASCENDING;
		}

		//MES- We'd like to just pass the arguments through to the Query server method, 
		//	but ObjectManager::Query doesn't support sorting on more than 3 columns, so 
		//	we may have to truncate our sort array, then sort on the client.
		int[][] arrInternalSort;
		boolean bRequiresClientSort;

		if (3 < arrSort.length)
		{
			arrInternalSort = new int[3][2];

			for (int iSortIndex = 0; iSortIndex < 3; ++iSortIndex)
			{
				arrInternalSort[iSortIndex][PTSAW_SORT_ROW_PROPID]		  = arrSort[iSortIndex][PTSAW_SORT_ROW_PROPID];
				arrInternalSort[iSortIndex][PTSAW_SORT_ROW_DIRECTION]     = arrSort[iSortIndex][PTSAW_SORT_ROW_DIRECTION];
			}

			//MES- The server can't do all of the sorting, so we should try to do it on the client
			bRequiresClientSort = true;
		}
		else
		{
			//MES- The sort array is fine the way it is, don't monkey with it.
			arrInternalSort     = arrSort;

			//MES- The server may be able to do the sort, we may not be required to do it
			bRequiresClientSort = false;
		}

		//MES- Perform the query		
		IPTQueryResult qr = m_ptFolder.QueryCards(iPropIDMask, bShowUnapproved,
				arrSort[0][0], m_nSkippedRows, m_iMaxRows, m_ptqf);

		//MES- Retrieve the data
		int[] cols	    = (int[]) qr.Columns();
		Object[][] data = (Object[][]) qr.Data();

		//MES- Did we get a complete data set?  If not, we can't sort on the client, and
		//	every time the user of this class asks for a sort, we have to requery the Server.
		int iRowsReturned = qr.RowCount();

		if (iRowsReturned >= m_iMaxRows)
		{
			//MES- This is "large" mode- don't sort on client, since we don't have all the data
			m_iMode     = PTTQRW_MODE_TRUNCATED;

			//MES- Record the query data, so that we can requery on sort
			m_iPropIDMask		  = iPropIDMask;
			m_bShowUnapproved     = bShowUnapproved;

			//MES- Initialize the superclass, SKIPPING client sorting, and we're done.
			//	NOTE: We may have truncated the sort above, so the data may not be
			//	sorted right, but we can't correct it, since we don't have all the data.
			//	You might argue, "OK, but we should STILL sort it on the client as best
			//	we can."  That's sorta OK, but it has a problem.  Suppose that the result
			//	from the server was sorted by Name, and that Name is localized.  Sorting
			//	of localized data can't be done on the Server in 5.0, so we're getting
			//	back data that's not sorted.  For example, the names might be A, Z, B.
			//	If we sort this on the client, we'd be giving the user the impression that
			//	they're seeing all the names through Z.  It's kinda ugly to show A, Z, B,
			//	but it's even worse to show A, B, Z, when the next item might be BB.
			//	We'd rather present an unsorted list than fool the user.  Fortunately, this
			//	shouldn't come up often, since we should usually get the whole result set.
			//MES- Note:  We're initalizing with our INTERNAL sort, since that's what
			//	was really used.  If the client specified more sort columns, but the data
			//	set is too large, we just can't do it.
			super.Initialize(cols, data, arrInternalSort, false);

			//MES- OK, we're ready for use
			return true;
		}
		else
		{
			//MES- The result set isn't large, so we CAN sort on the client, but
			//	we may not need to, because the result may already be sorted by the
			//	server.
			m_iMode = PTTQRW_MODE_CLIENT;

			//MES- If we truncated the sort above, then we definitely need to sort
			//	on the client, but if we did NOT truncate, we may STILL need to
			//	sort on the client, since the server may have failed to sort correctly.
			if (!bRequiresClientSort)
			{
				int iServerSorted = qr.IsOrdered();

				if (0 == iServerSorted)
				{
					bRequiresClientSort = true;
				}
			}

			//MES- Initialize ourself with the REAL sort!
			super.Initialize(cols, data, arrSort, bRequiresClientSort);

			//MES- OK, we're ready for use
			return true;
		}
	}

	/**
	 * Set the number of rows to skip when Querying Cards
	 */
	public void SetSkippedRows(int nSkippedRows)
	{
		m_nSkippedRows = nSkippedRows;
	}

	/**
	 * MES- Sort is used to resort the data.  Sort will do an in-memory sort,
	 * or     will requery the server, as needed.
	 *
	 * @param arrSort is a 2D int array of PropertyIDs to sort by.  It should
	 *        contain     a column for each sort.  The first row holds the
	 *        PropertyID, and the second row     holds the sort direction,
	 *        PTSAW_SORT_ASCENDING or PTSAW_SORT_DESCENDING
	 *
	 * @throws XPException 
	 */
	public void Sort(int[][] arrSort)
	{
		//MES- We could check the query type here, and only allow 
		//	sorting in the way the Server would allow sorting,
		//	but that seems unnecessary.  Why shouldn't we be more
		//	flexible than the server?
		//MES- If we have all the data, simply sort
		if (PTTQRW_MODE_CLIENT == m_iMode)
		{
			//MES- We have the data, it's a client sort.
			//	Pass the sort up to the superclass.
			super.Sort(arrSort);
		}
		else if (PTTQRW_MODE_TRUNCATED == m_iMode)
		{
			//MES- We don't have all the data, so we can't do a client sort.
			//	Just redo the query with the new sort.
			if (!QueryCards(m_iPropIDMask, m_bShowUnapproved, arrSort, m_ptqf))
			{
				throw new XPException(
					"Error in PTSortedDocumentsQueryWrapper::Sort, requery failed.");
			}
		}
		else
		{
			//MES- ERROR! Unknown mode!
			throw new XPException(
				"Error in PTSortedDocumentsQueryWrapper::Sort, unknown mode: " +
				XPConvert.ToString(m_iMode) + ".");
		}
	}
}

