package com.plumtree.portaluiinfrastructure.search;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author CraigS
 */
public class PaginationControl extends SearchResultControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.search.PaginationControl");

	public static final String STR_MVC_CLASS_NAME = "page";

	public static final String FIRST_RESULT_DATA = "first";

	public static final String PAGE_SIZE_DATA = "num";

	public static final String ORDER_BY_DATA = "order";

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IControl#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 *  Factory method to make a pagination link for some search results.
	 *   @param model     Current batch of search results
	 *   @param parent    ActivitySpace we're living in
	 *   @param urlText   Desired text within URL
	 *   @param orderBy   Sort order within which newFirst is to be interpreted
	 *   @param newFirst  Desired first result on new page
	 *   @param pageSize  Desired number of results on page
	 */
	public static String makeURLAsString(IBaseSearchResultModelRO model,
		AActivitySpace parent, String urlText, String orderBy, int newFirst,
		int pageSize)
	{
		ASURL url = (ASURL)makeURL(model, parent, urlText, orderBy, newFirst, false);
		url.AddControlArgument(PaginationControl.PAGE_SIZE_DATA, pageSize);
		return url.GetURLAsString();
	}

	/**
	 *  Factory method to make a pagination link for some search results.
	 *   @param model     Current batch of search results
	 *   @param parent    ActivitySpace we're living in
	 *   @param urlText   Desired text within URL
	 *   @param orderBy   Sort order within which newFirst is to be interpreted
	 *   @param newFirst  Desired first result on new page
	 *   @param pageSize  Desired number of results on page
	 */
	public static HTMLElement makeURL(IBaseSearchResultModelRO model,
		AActivitySpace parent, String urlText, String orderBy, int newFirst,
		int pageSize)
	{
		ILinkElement url = makeURL(model, parent, urlText, orderBy, newFirst);
		url.AddControlArgument(PaginationControl.PAGE_SIZE_DATA, pageSize);
		return url.GetURLAsHTMLElement();
	}

	/**
	 *  Factory method to make a pagination link for some search results.
	 *   @param model     Current batch of search results
	 *   @param parent    ActivitySpace we're living in
	 *   @param urlText   Desired text within URL
	 *   @param orderBy   Sort order within which newFirst is to be interpreted
	 *   @param newFirst  Desired first result on new page
	 */
	public static ILinkElement makeURL(IBaseSearchResultModelRO model,
			AActivitySpace parent, String urlText, String orderBy, int newFirst)	
	{
		boolean embedInForm = SpaceNeedsPassthruURL(parent);

		return makeURL(model, parent, urlText, orderBy, newFirst, embedInForm);
	}
	
	/**
	 *  Factory method to make a pagination link for some search results.
	 *   @param model     Current batch of search results
	 *   @param parent    ActivitySpace we're living in
	 *   @param urlText   Desired text within URL
	 *   @param orderBy   Sort order within which newFirst is to be interpreted
	 *   @param newFirst  Desired first result on new page
	 *   @param bEmbed    Whether or not this URL should be embedded in a form
	 */
	public static ILinkElement makeURL(IBaseSearchResultModelRO model,
		AActivitySpace parent, String urlText, String orderBy, int newFirst,
		boolean bEmbed)	
	{
		// Create a URL that directs into the current Activity Space
		ILinkElement url = null;

		if (bEmbed)
		{
			url = new PassthruURL();
		}
		else
		{
			url = new ASURL(parent);
			url.SetLinkGetCachedSpace(parent);
			url.SetDisplayPage(parent.GetCurrentPage().GetName());
		}

		// Create a PaginationControl within the search space
		url.SetControl(PaginationControl.STR_MVC_CLASS_NAME);

		// Set superclass data
		// NOTE: We know we're always passed a fullblown
		// ISearchResultModelRO, since there's only one
		// implementation of either interface (SearchResultModel).
		addCommonData((ISearchResultModelRO) model, url);

		// Set link text
		url.AddInnerHTMLString(urlText);

		// Set other data
		url.AddControlArgument(PaginationControl.ORDER_BY_DATA, orderBy);
		url.AddControlArgument(PaginationControl.FIRST_RESULT_DATA,
			XPConvert.ToString(newFirst));

		return url;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new PaginationControl();
	}

	/**
	 * @see com.plumtree.portalpages.search.SearchResultControl#ExecuteSearchControl()
	 */
	protected Redirect ExecuteSearchControl(XPHashtable args, String key)

	{
		int newFirstResult;
		int newPageSize;
		String orderBy;

		// First result
		String[] values = (String[]) args.GetElement(FIRST_RESULT_DATA);

		if (null == values)
		{
			log.Error("could not find first result data in query string.");

			return null;
		}

		try
		{
			newFirstResult = XPConvert.ToInteger(values[0]);
		}
		catch (Exception e)
		{
			log.Error(e, "could not parse first result data in query string: " +
				values[0]);

			return null;
		}

		// Page size
		values = (String[]) args.GetElement(PAGE_SIZE_DATA);

		if (null == values)
		{
			log.Error("could not find page size in query string.");

			return null;
		}

		try
		{
			newPageSize = XPConvert.ToInteger(values[0]);
		}
		catch (Exception e)
		{
			log.Error(e, "could not parse new page size data in query string: " +
				values[0]);

			return null;
		}

		// Sort order
		values = (String[]) args.GetElement(ORDER_BY_DATA);

		if (null != values)
		{
			orderBy = values[0];
		}
		else
		{
			log.Error("could not find order by data in query string");

			return null;
		}

		// Tell the model to go to the new page
		_model.GoToPage(orderBy, newFirstResult, newPageSize, key);

		// Don't need to redirect to a different page or space; just
		// redisplay the page
		return null;
	}
}