package com.plumtree.portaluiinfrastructure.search;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author CraigS
 *
 *  Simple view to return pagination for an IBaseSearchResultModelRO:
 *   << Prev 10   Results 11-20 of 57  Next 10 >>
 *
 *  You can either make this its own Section in your space, or
 *  just call the static DisplayPagination() method,
 *  passing in a model interface.
 */
public class PaginationView implements IView
{
	
	public static final String STR_MVC_CLASS_NAME = "PaginationView";
	protected AActivitySpace m_asOwner		  = null;
	protected IBaseSearchResultModelRO _model = null;

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IView#Display()
	 */
	public HTMLElement Display()
	{
		try
		{
			return DisplayPagination(_model, m_asOwner);
		}
		catch (Exception e)
		{
			throw XPException.GetInstance(e);
		}
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IView#DisplayJavascript()
	 */
	public HTMLScript DisplayJavascript()
	{
		// nothing needed
		return null;
	}

	/**
	 *  Returns an HTMLElement for pagination of the results table.
	 */
	public static HTMLElement DisplayPagination(IBaseSearchResultModelRO model, AActivitySpace parent)
	{
		HTMLElementCollection group = new HTMLElementCollection();
		HTMLSpan span = new HTMLSpan();
		span.SetStyleClass(PTStyleClass.OBJECTSEARCH_TEXT);
		group.AddInnerHTMLElement(span);

		final int PAGES_TO_DISPLAY = 10;

		try
		{
			String sortOrder = model.GetSortOrder();

			int currFirst  = model.GetFirstDisplayableItem();
			int totResults = model.GetTotalItems();
			int pageSize   = model.GetItemsPerPage();

			

			boolean shownOneItem = false;

			// If there is only one page
			if (totResults <= pageSize)
			{
				// Don't bother
				return group;
			}

			// Add a space at the beginning so we don't run into a cell border
			span.AddInnerHTMLString(CommonHTMLStrings.SPACE);

			// First, check whether we can display a "previous" link.
			int prevFirst = 0;

			if (currFirst > pageSize)
			{
				// we're at least one page beyond the beginning;
				// we can back up one page
				prevFirst = currFirst - pageSize;
			}
			else if (currFirst > 1)
			{
				// odd case: we're partway thru the first page.
				// how did we get here?
				// display a link to go back to the first result
				prevFirst = 1;
			}

			// If we can display a "previous" link
			if (0 < prevFirst)
			{
				// Display "prev" link
				span.AddInnerHTMLElement(PaginationControl.makeURL(model,
						parent, 
						
				parent.GetString(207, "ptmsgs_portalinfrastructure"),
						sortOrder, prevFirst, pageSize));
			}
			else
			{
				// Display a nonclickable link
				span.AddInnerHTMLString(parent.GetString(207,
						"ptmsgs_portalinfrastructure"));
			}

			shownOneItem = true;

			int currPage = (currFirst - 1) / pageSize;

			int firstPageToDisplay = (currPage / PAGES_TO_DISPLAY); // integer divide!
			firstPageToDisplay = (firstPageToDisplay * PAGES_TO_DISPLAY) - 1;

			int lastPageToDisplay = firstPageToDisplay + PAGES_TO_DISPLAY + 1;

			for (int page = firstPageToDisplay; page <= lastPageToDisplay;
					page++)
			{
				if (page < 0)
				{
					continue;
				}

				int firstResultInPage = (page * pageSize) + 1;
				int lastResultInPage  = (firstResultInPage + pageSize) - 1;

				// If we're over the total number of results
				if (firstResultInPage > totResults)
				{
					// we're all done
					break;
				}

				// If this is the last page
				if (lastResultInPage > totResults)
				{
					// show the exact number of results we'll be displaying
					lastResultInPage = totResults;
				}

				String linkText = "...";

				// If we're not showing the first or last link
				if ((page > firstPageToDisplay) && (page < lastPageToDisplay))
				{
					// show the actual result range
					linkText = XPConvert.ToString(firstResultInPage) + "-" +
						XPConvert.ToString(lastResultInPage);
				}

				if (shownOneItem)
				{
					span.AddInnerHTMLString("&nbsp;|&nbsp;");
				}

				shownOneItem = true;

				// If this isn't the current page
				if (page != currPage)
				{
					// make a link
					span.AddInnerHTMLElement(PaginationControl.makeURL(model,
							parent, linkText, sortOrder,
							firstResultInPage, pageSize));
				}
				else
				{
					// else make a nonclickable thing
					span.AddInnerHTMLString(linkText);
				}
			}

			// Finally, handle the "next" link
			// Calculate start of next page
			int nextFirst = currFirst + pageSize;

			// Check that "next" won't go beyond the end of
			// the result set
			if (nextFirst > totResults)
			{
				nextFirst = 0;
			}

			if (shownOneItem)
			{
				span.AddInnerHTMLString("&nbsp;|&nbsp;");
			}

			shownOneItem = true;

			// if displaying "next" link
			if (0 != nextFirst)
			{
//				int numOnNextPage = totResults - nextFirst + 1;
//
//				if (numOnNextPage > pageSize)
//				{
//					numOnNextPage = pageSize;
//				}

				span.AddInnerHTMLElement(PaginationControl.makeURL(model, parent, 

				// Should we really be using the numOnNextPage instead of pageSize?
				parent.GetString(208, "ptmsgs_portalinfrastructure"),
						sortOrder, nextFirst, pageSize));
			}
			else
			{
				// Show a nonclickable "Next"
				span.AddInnerHTMLString(parent.GetString(208,
						"ptmsgs_portalinfrastructure"));
			}

			// Finally, add a space on the end
			span.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		}
		catch (Exception e)
		{
			throw XPException.GetInstance(e);
		}

		return group;
	}

	/**
	 *  Returns an alternative version of the pagination bar that works well with
         *  screenreading software.
	 */
        public static HTMLElement Display508StylePagination(ISearchResultModelRO model, AActivitySpace parent) {

          HTMLElementCollection group = new HTMLElementCollection();
          HTMLSpan span = new HTMLSpan();
          group.AddInnerHTMLElement(span);

          final int PAGES_TO_DISPLAY = 10;

          try {
            String sortOrder = model.GetSortOrder();

            int currFirst  = model.GetFirstDisplayableItem();
            int totResults = model.GetTotalItems();
            int pageSize   = model.GetItemsPerPage();
            int currPage = (currFirst - 1) / pageSize;

            // If there is only one page
            if (totResults <= pageSize) {
              // Don't bother
              return group;
            }

            // Show the current page
            span.AddInnerHTMLString(XPResourceManager.FormatMessage(parent.GetString(492,"ptmsgs_portalinfrastructure"),
                                                                    XPConvert.ToString(currPage+1)));
            span.AddInnerHTMLString("  ");

            // Check whether we can display a "previous" link.
            int prevFirst = 0;

            if (currFirst > pageSize)
              {
                // we're at least one page beyond the beginning;
                // we can back up one page
                prevFirst = currFirst - pageSize;
              }
            else if (currFirst > 1)
              {
                // odd case: we're partway thru the first page.
                // how did we get here?
                // display a link to go back to the first result
                prevFirst = 1;
              }

            // If we can display a "previous" link
            if (0 < prevFirst)
              {
                // Display "prev" link

                HTMLElement prevLink = PaginationControl.makeURL(model,
                                                                 parent, 
                                                                 parent.GetString(494, "ptmsgs_portalinfrastructure"),
                                                                 sortOrder, prevFirst, pageSize);

                span.AddInnerHTMLElement(prevLink);
                span.AddInnerHTMLString(".  ");
              }

            // Handle the "next" link
            // Calculate start of next page
            int nextFirst = currFirst + pageSize;

            // Check that "next" won't go beyond the end of
            // the result set
            if (nextFirst > totResults)
              {
                nextFirst = 0;
              }

            // if displaying "next" link
            if (0 != nextFirst)
              {
                int numOnNextPage = totResults - nextFirst + 1;

                if (numOnNextPage > pageSize)
                  {
                    numOnNextPage = pageSize;
                  }

                span.AddInnerHTMLElement(PaginationControl.makeURL(model,
                                                                   parent, 
                                                                   parent.GetString(493, "ptmsgs_portalinfrastructure"),
                                                                   sortOrder, nextFirst, pageSize));
                span.AddInnerHTMLString(".  ");
              }

            // Now, handle "jump to page" links

            int firstPageToDisplay = (currPage / PAGES_TO_DISPLAY); // integer divide!
            firstPageToDisplay = (firstPageToDisplay * PAGES_TO_DISPLAY) - 1;

            int lastPageToDisplay = firstPageToDisplay + PAGES_TO_DISPLAY + 1;

            boolean shownOneLink = false;

            for (int page = firstPageToDisplay; page <= lastPageToDisplay;
                 page++)
              {
                if (page < 0)
                  {
                    continue;
                  }

                int firstResultInPage = (page * pageSize) + 1;
                int lastResultInPage  = (firstResultInPage + pageSize) - 1;

                // If we're over the total number of results
                if (firstResultInPage > totResults)
                  {
                    // we're all done
                    break;
                  }

                // If this is the last page
                if (lastResultInPage > totResults)
                  {
                    // show the exact number of results we'll be displaying
                    lastResultInPage = totResults;
                  }

                String linkText = "";

                // If we're not showing the last link
                if (page < lastPageToDisplay)
                  {
                    // show the actual page number
                    linkText = XPConvert.ToString(page+1);
                  }
                else
                  {
                    // use the "more" text
                    linkText = parent.GetString(496, "ptmsgs_portalinfrastructure");
                  }

                if (shownOneLink)
                  {
                    // Add a comma to separate page numbers
                    span.AddInnerHTMLString(", ");
                  }
                else
                  {
                    // Show the "Jump to page:" text
                    span.AddInnerHTMLString(parent.GetString(495, "ptmsgs_portalinfrastructure"));
                    span.AddInnerHTMLString(" ");

                    // For the rest, just show commas
                    shownOneLink = true;
                  }

                // If this isn't the current page
                if (page != currPage)
                  {
                    // make a pagination link
                    span.AddInnerHTMLElement(PaginationControl.makeURL(model,
                                                                       parent, linkText, sortOrder,
                                                                       firstResultInPage, pageSize));
                  }
                else 
                  {
                    // If this is the current page, show plain text, not a link
                    span.AddInnerHTMLString(linkText);
                  }
              }

            if (shownOneLink) {
              span.AddInnerHTMLString(".");
            }
          }
          catch (Exception e) {
            throw XPException.GetInstance(e);
          }

          return group;
        }

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new PaginationView();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IView#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	
	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IView#Init(IModelRO, AActivitySpace)
	 */
	public void Init(IModelRO model, AActivitySpace parent)
	{
		_model		  = (IBaseSearchResultModelRO) model;
		m_asOwner     = parent;
	}
}

