package com.plumtree.portaluiinfrastructure.search;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.filter.*;
import com.plumtree.portaluiinfrastructure.friendlyurl.FriendlyURLConstants;
import com.plumtree.portaluiinfrastructure.friendlyurl.FriendlyURLParserManager;
import com.plumtree.portaluiinfrastructure.friendlyurl.IFriendlyURLParserManager;
import com.plumtree.portaluiinfrastructure.friendlyurl.SearchFriendlyURLParser;
import com.plumtree.portaluiinfrastructure.helpers.PTConfigService;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 *  Factory class that manufactures a
 *  com.plumtree.xpshared.htmlelements.HTMLForm
 *  that encodes search requests and submits them to any
 *  ActivitySpace that includes a SearchResultModel.
 *
 *  Note: If your form is used to find certain types of
 *   administrative objects, optionally within certain folders,
 *   you should probably call the static method GetObjectSearchForm();
 *   this is the easiest way to make a form that works and
 *   satisfies portal UI standards.  If you're making any
 *   other type of search form, you need to instantiate a
 *   SearchFormFactory object and add constraints and HTML
 *   elements to it, one at a time.
 *
 *  Sample usage:  see package com.plumtree.portalpagessample.search
 *
 *  @author CraigS
 */
public class SearchFormFactory
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.search.SearchFormFactory");

	private static final String SLASH = "/";
	
	/**
	 * This is prefix for the Form Name when it is rendered in the Banner Search.
	 */
	public static final String BANNER_NAME_PREFIX = "ptbanner";

	/**
	 *  Owning ActivitySpace, needed for string I18N.
	 */
	private AActivitySpace m_asOwner;

	/**
	 *  The inner elements of the form
	 */
	private HTMLElementCollection _elements;

	/**
	 *  The form we're building
	 */
	private HTMLForm _form;

	/** Form ID */
	private String _formID;

	/** Form name */
	private String _name;

	/** Target DisplayPage */
	private String _targetPage;

	/** Target ActivitySpace */
	private String _targetSpace;

	/** Target ActivitySpace ID */
	private String _targetSpaceID;

	/** Javascript validation function */
	private String _validationFunction;

	IFriendlyURLParserManager fUrlMgr = FriendlyURLParserManager.getInstance();
	
	private boolean bUseFriendlySearch = PTConfigService.getPTInstance().UseFriendlyURLS(FriendlyURLConstants.FRIENDLY_SEARCH_ID, "");
	
	private String _inputBoxName = StartSearchControl.USER_QUERY;
	

	/**
	 *  Whether we are constructing a complete form, or elements that will
	 *  be inserted into a display page form (a subclass of AFormDP)
	 */
	private boolean _completeForm;

	/**
	 *  Special flag to force banner search defaults.
	 */
	private boolean _forceBannerSearch;
	
	// DCD: Adding this for top best bet searches, which require a special control
	private boolean _isTopBestBetSearch;
	
	/**
	 * Flag for partial form case to control whether an alert is displayed when
	 * submitting without a search string.
	 */
	private boolean _validateAlert = false;
		
	
	/**
	 *  Yet another hack special flag.  If true, and we're embedded within another form,
	 *  (_completeForm==false), then force the form to be submitted through
	 *  javascript when the user hits Enter in the search box.
	 *  *THIS IS DANGEROUS*; some browsers also submit the form automatically;
	 *  make sure you don't get double searches!!
	 */
	private boolean _forceSubmitEnclosingForm;

	/**
	 *  Page mode.  The form simply passes this on to the target page.
	 */
	private String _pageMode;

	/**
	 *  Special flag (== "hack").  If true, and we're embedded within another form,
	 *  (_completeForm==false), then before submitting the search form, override
	 *  the parent form's in_hi_space and associated arguments to force the user
	 *  to go to the search results page.
	 */
	private boolean _overrideEnclosingFormAndSubmitToSearchResultPage;

	/** Number of selects generated so far */
	private int _selectCount;

	/** Number of statements added to user filter. */
	private int _filterStatementCount = 0;

	/** User filter. See AddFilterStatement() */
	private IPTFilter _userFilter = null;

	/** Value controlling whether filter statements are anded or ored together */
	private int _filterClauseOperator = PT_BOOLOPS.PT_BOOLOP_AND;

	/** User added filter clause. */
	private IPTPropertyFilterClauses _filterClause = null;
	  
	private static final String EMPTY_STRING = "";

	private static final String SINGLE_QUOTE = "\"";

	/** boolean flag to verify if the SearchForm generated is for Administration UI
	 * If it is for Administration then we generate UI by turning off friendly URLs
	 * Since search friendly URLs are only for BannerSearch. 
	 */
	private boolean bIsAdminSearch = true;
	
	/**
	 *  Constructor.  Note "name" must be unique for the current page.
	 */
	public SearchFormFactory(String name, String formID, AActivitySpace owner)

	{
		this(name, formID, owner, /* completeForm= */
			true);
	}

	/**
	 *  Constructor.  Note "name" must be unique for the current page.
	 */
	public SearchFormFactory(String name, String formID, AActivitySpace owner,
		boolean buildCompleteForm)

	{
		_name												  = name;
		_formID												  = formID;
		m_asOwner											  = owner;
		_completeForm										  = buildCompleteForm;
		_forceBannerSearch									  = false;
		_isTopBestBetSearch									  = false;
		_pageMode										  = null;
		_overrideEnclosingFormAndSubmitToSearchResultPage     = false;
		_forceSubmitEnclosingForm							  = false;

		_selectCount		    = 0;
		_targetSpace		    = null;
		_targetSpaceID		    = null;
		_targetPage			    = null;
		_validationFunction     = null;

		
		bIsAdminSearch = (m_asOwner.GetName() == "AdminObjSearch");
		
		if (Is508Mode() && (!_completeForm))
		{
			throw new XPException(
				"SearchFormFactory cannot embed a search box in a larger form in 508 mode, since this would require Javascript.");
		}

		InitForm();
	}

	/**
	 *  Add an optional setting that will be set only if the indicated
	 *  optionName is selected
	 */
	public void AddOptionalSetting(String optionName, int settingId,
		String settingValue)

	{
		if (-1 != optionName.indexOf('_'))
		{
			throw new HTMLException(
				"optionName may not contain an underscore (_) character.");
		}

		AddInputs(StartSearchControl.OPTIONAL_CONSTRAINT_PREFIX + optionName +
			"_", settingId, settingValue);
	}

	/**
	 *  Add an optional setting that will be set only if the indicated
	 *  optionName is selected
	 */
	public void AddOptionalSetting(String optionName, int settingId,
		boolean settingValue)

	{
		if (-1 != optionName.indexOf('_'))
		{
			throw new HTMLException(
				"optionName may not contain an underscore (_) character.");
		}

		AddInputs(StartSearchControl.OPTIONAL_CONSTRAINT_PREFIX + optionName +
			"_", settingId, new Boolean(settingValue));
	}

	/**
	 *  Add an optional setting that will be set only if the indicated
	 *  optionName is selected
	 */
	public void AddOptionalSetting(String optionName, int settingId,
		int settingValue)

	{
		if (-1 != optionName.indexOf('_'))
		{
			throw new HTMLException(
				"optionName may not contain an underscore (_) character.");
		}

		AddInputs(StartSearchControl.OPTIONAL_CONSTRAINT_PREFIX + optionName +
			"_", settingId, new Integer(settingValue));
	}

	/**
	 *  Add an optional setting that will be set only if the indicated
	 *  optionName is selected
	 */
	public void AddOptionalSetting(String optionName, int settingId,
		int[] settingValue)

	{
		if (-1 != optionName.indexOf('_'))
		{
			throw new HTMLException(
				"optionName may not contain an underscore (_) character.");
		}

		AddInputs(StartSearchControl.OPTIONAL_CONSTRAINT_PREFIX + optionName +
			"_", settingId, settingValue);
	}

	/**
	 *  Add a group of required settings (constraint) that will be added to
	 *  all searches generated by this form.
	 */
	public void AddRequiredSettings(SearchSettingCollection c)

	{
    IXPEnumerator e = c.GetSettingIDs();

    while (e.MoveNext()) {
      int key = XPConvert.ToInteger(e.GetCurrent());
      Object val = c.GetSettingValue(key);
      
      // If the set of additional properties to be returned are empty then 
      // we skip adding that setting to input.
      if(key == PT_SEARCH_SETTING.PT_SEARCHSETTING_RET_PROPS)
      {
    	  int[] props = (int[])val;    	  
    	  if(props.length == 0)
    	  {
    		  continue;
    	  }
      }
      
      AddInputs(StartSearchControl.REQUIRED_CONSTRAINT_PREFIX, key, val);
    }
	}

	/**
	 *  Add a required setting (constraint) that will be added to
	 *  all searches generated by this form.
	 */
	public void AddRequiredSetting(int settingId, boolean settingValue)

	{
		AddInputs(StartSearchControl.REQUIRED_CONSTRAINT_PREFIX, settingId,
			new Boolean(settingValue));
	}

	/**
	 *  Add a required setting (constraint) that will be added to
	 *  all searches generated by this form.
	 */
	public void AddRequiredSetting(int settingId, int settingValue)

	{
		AddInputs(StartSearchControl.REQUIRED_CONSTRAINT_PREFIX, settingId,
			new Integer(settingValue));
	}

	/**
	 *  Add a required setting (constraint) that will be added to
	 *  all searches generated by this form.
	 */
	public void AddRequiredSetting(int settingId, int[] settingValue)

	{
		AddInputs(StartSearchControl.REQUIRED_CONSTRAINT_PREFIX, settingId,
			settingValue);
	}

	/**
	 *  Special flag that forces this search box to have all the user's
	 *  banner search defaults applied to it.  Should only be used by the
	 *  banner search box itself and the "search again" box on the
	 *  search results page, otherwise you can't be sure what you get.
	 */
	public void ForceBannerSearchDefaults()

	{
		_forceBannerSearch = true;

		// Let banner search (only) use the GET method, since we know
		// it works and makes the resulting page more bookmark-friendly.
		_form.SetMethod("get");
	}
	
	/**
	 *  Special flag for designating the form to be a top best bet search,
	 *  which requires a special control (the activity space was alterable
	 *  through other methods, but this was added for setting the control).
	 *
	 */
	public void ForceTopBestBet()
	{
	    _isTopBestBetSearch = true;
	    _form.SetMethod("get");
	}
	

	/**
	 *  Yet another hack special flag.  If called, and we're embedded within another form,
	 *  then force the form to be submitted through
	 *  javascript when the user hits Enter in the search box.
	 *  *THIS IS DANGEROUS*; some browsers also submit the form automatically;
	 *  make sure you don't get double searches!!
	 */
	public void ForceSubmitOfEnclosingForm()
	{
		if (_completeForm)
		{
			throw new XPException(
				"Cannot call ForceSubmitOfEnclosingForm if you're building a complete form.");
		}

		_forceSubmitEnclosingForm = true;
	}

	public void SetValidateAlert(boolean validateAlert)
	{
		_validateAlert = validateAlert;
	}

	/**
	 *  Returns the onclick javascript for the HTMLAnchor used as a "submit"
	 *  button or link
	 */
	public final String GetAnchorOnClick()
	{
		if (Is508Mode())
		{
			return null;
		}
		else
		{
			return "submitSearch" + _name + "(); return false;";
		}
	}

	/**
	 *  Convenience method.  Same as GetResult(), but simply casts
	 *  the result to type HTMLForm.  Do not call if
	 *  buildCompleteForm=false was passed to the constructor.
	 */
	public HTMLForm GetForm()

	{
		if (!_completeForm)
		{
			throw new XPException(
				"Cannot call GetForm if you're not building a complete form.");
		}

		return (HTMLForm) GetResult();
	}
		

	/**
	 *  Returns the name callers must use for the input box, if it
	 *  is a banner search then it returns 'q' since it is friendly url
	 *  parameter for search name.
	 */
	public final String GetInputBoxName()	
	{				
		if(bUseFriendlySearch && XPStringUtility.StartsWith(_name, BANNER_NAME_PREFIX)) {
			return SearchFriendlyURLParser.SEARCH_QS;
		}
		return _inputBoxName;
	}

	/**
	 *  Add a required setting (constraint) that will be added to
	 *  all searches generated by this form.
	 */
	public void AddRequiredSetting(int settingId, String settingValue)

	{
		AddInputs(StartSearchControl.REQUIRED_CONSTRAINT_PREFIX, settingId,
			settingValue);
	}

	/**
	 *  Returns the javascript callers should use as the "onKeyDown" attribute of
	 *  the input box
	 */
	public String GetInputBoxOnKeyDown()
	{
		if (Is508Mode())
		{
			return null;
		}
		else
		{
			return "return handleKeyDown" + _name + "(event);";
		}
	}

	/**
	 *  Get a new name for a pull-down HTMLSelect.
	 */
	public String GetNextSelectName()
	{
		_selectCount++;

		return StartSearchControl.SELECTED_PREFIX +
		XPConvert.ToString(_selectCount);
	}

	/**
	 *  Returns the search form (if building a complete form)
	 *  or inputs to be inserted into another form (otherwise).
	 */
	public HTMLElement GetResult()
	{
    	if(_filterStatementCount != 0)
		{
			CreateFilterElements();
		}
		if (_completeForm)
		{
			// Add target inputs
			// RT - This is needed only if friendly URL is turned off for search or if it is 
			// Adminsearch we do it in regular way.
			if(!bUseFriendlySearch || bIsAdminSearch) {
				 AddTargetInputs();
			}
			
		}
		
		// RT - This is needed only if friendly URL is turned off for search or if it is 
		// Adminsearch we do it in regular way.
		if(!bUseFriendlySearch || bIsAdminSearch) {
			AddUserIDInput();
		}

		// Handle the "page mode" argument
		if ((null != _pageMode) && (0 < _pageMode.length()))
		{
			HTMLInput pageMode = new HTMLInput(HTMLInputTypes.HIDDEN,
					SearchResultModel.MODEL_MODE_INPUT,
					SearchResultModel.MODEL_MODE_INPUT, "");
			pageMode.SetValue(_pageMode);
			_elements.AddInnerHTMLElement(pageMode);
		}

		// Add form submission and validation javascript
		if (!Is508Mode())
		{
			AddJavaScript();
		}

		if (_completeForm)
		{
			// Add the inputs to the form
			_form.AddInnerHTMLElement(_elements);

			// .. and return the form
			return _form;
		}
		else
		{
			// just return the innards
			return _elements;
		}
	}

	/**
	 *  Static convenience method to return the standard portal
	 *  search button
	 */
	public static HTMLImg GetSearchButton(AActivitySpace m_asOwner)
	{
		PTConstruct searchButton = null;

		try
		{
			searchButton = UIConstructFactory.GetConstruct(PTImageType.ACTION_SEARCH_BUTTON);
		}
		catch (Exception e)
		{
		}

		if (null == searchButton)
		{
			return new HTMLImg("", m_asOwner.GetString(293, "ptmsgs_portalinfrastructure"));
		}

		HTMLImg searchButtonImg = (HTMLImg) searchButton.GetHTMLElement();
		searchButtonImg.SetAlt(m_asOwner.GetString(293, "ptmsgs_portalinfrastructure"));
    //    searchButtonImg.SetWidth("16");
    //    searchButtonImg.SetHeight("16");

		return searchButtonImg;
	}

	/**
	 *  A special "mode" argument thta is passed along to the results page.
	 *  Different results page
	 *  implementations may interpret this flag in different ways.
	 */
	public void SetPageMode(String pageMode)
	{
		_pageMode = pageMode;
	}

	/**
	 *  This is public so it can be used for the "search again" form on the
	 *  results page; most users of this class should NOT call this method.
	 */
	public static HTMLScript MakeSearchFormJavascript(String name, 
		AActivitySpace m_asOwner, String inputBoxName)

	{
		return MakeSearchFormJavascript(name, m_asOwner, true, false, false, false,
			null, inputBoxName, null, null, null);
	}

	/**
	 *  Special flag (== "hack"), buildCompleteForm (passed to the constructor) must
	 *  be false.
	 *  Before submitting the search form, override the parent form's in_hi_space and
	 *  associated arguments to force the user to go to the search results page.
	 */
	public void OverrideEnclosingFormAndSubmitToSearchResultPage()
	{
		if (_completeForm)
		{
			throw new XPException(
				"Cannot call OverrideEnclosingFormAndSubmitToSearchResultPage if you're building a complete form.");
		}

		_overrideEnclosingFormAndSubmitToSearchResultPage = true;
	}

	/**
	 *  Sets the target for this form.
	 */
	public void SetTarget(String space, String spaceID, String page)
	{
		/* MATT
		if (!_completeForm)
		{
			throw new XPException(
				"Cannot call SetTarget if you're not building a complete form.");
		}
		*/
		_targetSpace	   = space;
		_targetSpaceID     = spaceID;
		_targetPage		   = page;
	}

	/**
	 *  Sets the javascript validation function for this form.
	 *
	 *  This javascript function must be declared elsewhere on the page
	 *  and must not include the final '();'.
	 */
	public void SetValidationFunction(String strFunctionName)
	{
		if ((strFunctionName.indexOf('(') == -1) &&
				(strFunctionName.indexOf(')') == -1) &&
				(strFunctionName.indexOf(';') == -1))
		{
			_validationFunction = strFunctionName;
		}
		else
		{
			
				log.Error("Submit method cannot contain ();");
			
		}
	}

	/**
	 *  Helper method to actually add the setting hidden inputs, called by
	 *  AddRequiredSetting and AddOptionalSetting
	 */
	protected void AddInputs(String inputNamePrefix, int settingId,
		Object settingValue)

	{
		String settingSuffix = SearchSettingCollection.getStringFromSettingId(settingId);

		if (null == settingSuffix)
		{
			throw new HTMLException("The search setting with id " + settingId +
				" either does not exist or may not be attached to a search box.");
		}

		String inputName = inputNamePrefix + settingSuffix;

		String[] values = SearchSettingCollection.stringifySettingValue(settingId,
				settingValue);

		if (null != values)
		{
			for (int j = 0; j < values.length; j++)
			{
				// Create the setting input
				HTMLInput input = new HTMLInput(HTMLInputTypes.HIDDEN,
						inputName, inputName, "");
				input.SetValue(values[j]);

				// and add it to the form in progress
				_elements.AddInnerHTMLElement(input);
			}
		}
		else
		{
			throw new HTMLException("Invalid value for search setting with id " +
				settingId);
		}
	}

	/**
	 * 
	 *
	 * @return 
	 */
	private final String GetCheckSearchFunction()
	{
		return "checkSearch" + _name + "();";
	}

	/**
	 * 
	 *
	 * @throws HTMLException 
	 */
	private void AddJavaScript()
	{
		
		HTMLScript script = MakeSearchFormJavascript(_name, m_asOwner,
				_completeForm,
				_overrideEnclosingFormAndSubmitToSearchResultPage,
				_forceSubmitEnclosingForm, _validateAlert, _validationFunction,
				GetInputBoxName(), _targetSpace, _targetSpaceID, _targetPage);

		_elements.AddInnerHTMLElement(script);
	}

	/**
	 * 
	 *
	 * @throws HTMLException 
	 */
	private void AddTargetInputs()

	{
		// Set target space (default is com.plumtree.portalpages.browsing.search.results.SearchResultAS)
		HTMLInput targetSpace = new HTMLInput(HTMLInputTypes.HIDDEN,
				ASConstants.SPACENAME_INPUT, ASConstants.SPACENAME_INPUT, "");

		if (null == _targetSpace)
		{
			targetSpace.SetValue("SearchResult");
		}
		else
		{
			targetSpace.SetValue(_targetSpace);
		}

		_elements.AddInnerHTMLElement(targetSpace);

		// Set target space ID, if provided
		if (null != _targetSpaceID)
		{
			HTMLInput targetSpaceID = new HTMLInput(HTMLInputTypes.HIDDEN,
					ASConstants.SPACEID_INPUT, ASConstants.SPACEID_INPUT, "");
			targetSpaceID.SetValue(_targetSpaceID);
			_elements.AddInnerHTMLElement(targetSpaceID);
		}

		// Set target page, if provided
		if (null != _targetPage)
		{
			HTMLInput targetPage = new HTMLInput(HTMLInputTypes.HIDDEN,
					ASConstants.PAGE_INPUT, ASConstants.PAGE_INPUT, "");
			targetPage.SetValue(_targetPage);
			_elements.AddInnerHTMLElement(targetPage);
		}

		// Target control is almost always StartSearchControl, except for the
		// special case of banner search
		HTMLInput targetControl = new HTMLInput(HTMLInputTypes.HIDDEN,
				ASConstants.CONTROL_INPUT, ASConstants.CONTROL_INPUT, "");

		if (_forceBannerSearch)
		{
			targetControl.SetValue("bannerstart");
		}
		else if(_isTopBestBetSearch)
		{
		    // DCD: Defined in PoralPages as TopBestBetHandlerCT.CLASS_NAME
		    targetControl.SetValue("TopBestBetHandlerCT");
		}
		else
		{
			targetControl.SetValue(StartSearchControl.STR_MVC_CLASS_NAME);
		}

		_elements.AddInnerHTMLElement(targetControl);
	}
	
	private void AddUserIDInput()
	{
		// Set target space (default is com.plumtree.portalpages.browsing.search.results.SearchResultAS)
		HTMLInput useridInput = new HTMLInput(HTMLInputTypes.HIDDEN,
				ASConstants.USERID_INPUT, ASConstants.USERID_INPUT, "");

		useridInput.SetValue(XPConvert.ToString(((IPTSession)m_asOwner.GetUserSession()).GetSessionInfo().GetCurrentUserID()));
		
		_elements.AddInnerHTMLElement(useridInput);
	}

	/**
	 *  Helper method to initialize _form with the parameters that must
	 *  always be defined for any search
	 */
	private void InitForm()

	{
		_elements = new HTMLElementCollection();

		if (_completeForm)
		{
			_form = new HTMLForm(_formID);
			_form.SetMethod("post");
			_form.SetName(_name);
			
			
			String strURL = PTConfigHelper.GetApplicationBaseURLToAS(m_asOwner.GetCurrentNormalizedBaseURL(), 
					m_asOwner.GetName(), m_asOwner.IsSubSpace());
			
			if(null == strURL ) {
				throw new XPNullPointerException("Application base URL obtained is null!");
			}
			
			
			// RT - If friendly URLs is turned off for search then we do it 
			// regular way, else we get base url and append friendly url keyword
			// and set action.
			
			// We also verify if it is AdminSearch and we do not support friendly 
			// urls for Admin searches and process the request as if it is non friendly url
			
			if(!bUseFriendlySearch || bIsAdminSearch) {
				 _form.SetAction(strURL);
			} else {
				if( XPStringUtility.EndsWith(strURL, "?")) {
					strURL = XPStringUtility.Substring(strURL, 0, strURL.indexOf("?") );
				}				
				_form.SetAction(strURL + SLASH + fUrlMgr.FindKeywordForType(FriendlyURLConstants.FRIENDLY_SEARCH_URL));				
			}
			

			// There shouldn't be a "real" submit button displayed (the "go" button
			// is actually an anchor tag), but the form may be submitted by the browser
			// anyway if the user presses Enter in the input box without typing anything
			// first.  This call prevents the form from being submitted without going
			// through the validation method:
			if (!Is508Mode())
			{
				_form.SetOnSubmit("return " + GetCheckSearchFunction());
			}
		}
	}

	/**
	 * 
	 *
	 * @return 
	 */
	private boolean Is508Mode()
	{
		return (m_asOwner.GetAccessStyle() != AccessStyles.STANDARD);
	}

	/**
	 * 
	 *
	 * @param name 
	 * @param m_asOwner 
	 * @param completeForm 
	 * @param overrideEnclosing 
	 * @param forceSubmitEnclosingForm 
	 * @param validationFunction 
	 * @param inputBoxName 
	 *
	 * @return 
	 *
	 * @throws HTMLException 
	 */
	private static HTMLScript MakeSearchFormJavascript(String name,	
		AActivitySpace m_asOwner, boolean completeForm,
		boolean overrideEnclosing, boolean forceSubmitEnclosingForm, boolean validateAlert,
		String validationFunction, String inputBoxName, String space, String spaceID, String page)

	{
		XPStringBuilder scriptText = new XPStringBuilder(300);
		// Add Javascript to check that the query is not empty
		scriptText.Append("function checkSearch").Append(name).Append("()\n{\n");

		scriptText.Append("  var searchForm = document.").Append(name).Append(";\n");

		// Look for the input box element and make sure it's not empty
		scriptText.Append(
			"  for (var i = 0; i < searchForm.elements.length; i++) {\n")
				.Append("    if (searchForm.elements[i].name == \"").Append(inputBoxName)
		 		.Append("\") {\n");
		scriptText.Append("     if (searchForm.elements[i].value == \"\") {\n");
		scriptText.Append("      return false;\n");
		scriptText.Append("     }\n");

		if (completeForm)
		{
			// Just return true; the query is ok
			scriptText.Append("    return true;\n");
		}
		else
		{
			// Add the validation function, if there is one.
			if (validationFunction != null)
			{
				scriptText.Append("     if (").Append(validationFunction).Append("()) {\n  ");
			}

			// If part of another form, may need to override the parent form's targets
			if (overrideEnclosing)
			{
				String spaceString = (space == null) ? "SearchResult" : space;
				scriptText.Append("      searchForm.")
				.Append(ASConstants.SPACENAME_INPUT).Append(".value=\"" + spaceString + "\";\n");
				String spaceIDString = (spaceID == null) ? "" : spaceID;
				scriptText.Append("      searchForm.")
				.Append(ASConstants.SPACEID_INPUT).Append(".value=\"" + spaceIDString + "\";\n");
				String pageString = (page == null) ? "SearchResult" : page;
				scriptText.Append("      searchForm.")
				.Append(ASConstants.PAGE_INPUT).Append(".value=\"" + pageString + "\";\n");
				scriptText.Append("      searchForm.")
				.Append(ASConstants.CONTROL_INPUT).Append(".value=\"")
				.Append(StartSearchControl.STR_MVC_CLASS_NAME).Append("\";\n");
			}

			scriptText.Append("      return true;\n");

			if (validationFunction != null)
			{
				scriptText.Append("     }\n");
			}
		}

		// Close loop over elements
		scriptText.Append("    }\n  }\n");

		// If we got through the loop without validating, don't let the form be submitted
		scriptText.Append("  return false;\n");
		scriptText.Append("}\n");

		// Next, add submitSearch
		scriptText.Append("function submitSearch").Append(name).Append("() {\n");

		// If checkSearch returns true
		scriptText.Append("  if (checkSearch").Append(name).Append("()) {\n");

		scriptText.Append("    var searchForm = document.").Append(name).Append(";\n");

		if (completeForm)
		{
			// If building a complete form, just submit the form
			scriptText.Append("    searchForm.submit();\n");
		}
		else
		{
			// Call the Javascript function (defined in AFormDP) that will submit the form
			// and then pass through to StartSearchControl.
			scriptText.Append("    ")
			.Append(AFormDP.JAVASCRIPT_SUBMIT_TO_CONTROL).Append("(\"")
			.Append(StartSearchControl.STR_MVC_CLASS_NAME).Append("\", \"\");");
		}

    // If a complete form, popup a message if the user clicks the button
    // without typing anything.

		if (completeForm || validateAlert) 
		{
      		scriptText.Append("  } else {\n").
        	Append("    alert('").
       		Append(m_asOwner.GetString(468, "ptmsgs_portalinfrastructure")).
        	Append("');\n");
    	}

		scriptText.Append("  }\n"); // Close if statement
		scriptText.Append("}\n"); // End of submitSearch

		// Next, add the onKeyDown event handler..
		scriptText.Append("function handleKeyDown").Append(name).Append("(evt) {\n");

		// If there is only one text box in the form (which is always true if we are
		// building a complete form), the form will be submitted immediately
		// if we return true.  So we have to do all the setup for submission here,
		// even stuff normally handled in postToSelf.  Alternately, if there are other
		// textboxes around, the form does not automatically get submitted, so we have
		// to submit() it ourselves through javascript.  Ugh!!
		if (!completeForm)
		{
			// Use either IE or Netscape event object (trick from Dynamic HTML book)
			scriptText.Append(
				" evt = (evt) ? evt : ((event) ? event : null);\n");

			scriptText.Append(" if (evt.keyCode == 13){\n");
			//scriptText.Append(" ")
			//.Append(AFormDP.JAVASCRIPT_SETUP_FOR_SUBMIT_TO_CONTROL).Append("(\"")
			//.Append(StartSearchControl.STR_MVC_CLASS_NAME).Append("\", \"\");\n");
			scriptText.Append(" ").Append("submitSearch").Append(name).Append("();\n");

			//if (forceSubmitEnclosingForm)
			//{
			//	scriptText.Append("    document.").Append(name).Append(".submit();\n");
			//}
			scriptText.Append("   return false;\n");
			scriptText.Append(" }\n");
			scriptText.Append(" return true;\n"); // otherwise, go to normal onkeydown action
		}
		else
		{
			scriptText.Append(" evt = (evt) ? evt : ((event) ? event : null);\n");
			scriptText.Append(" if (evt.keyCode == 13){\n");
			scriptText.Append("	  if (checkSearch").Append(name).Append("()) {\n");
			scriptText.Append("		   return true;\n");
			scriptText.Append("	  } else {\n");
			scriptText.Append("	       alert('").Append(m_asOwner.GetString(468, "ptmsgs_portalinfrastructure")).Append("');\n");
			scriptText.Append("	       return false;\n");
			scriptText.Append("	  }\n");
			scriptText.Append(" }\n");
			scriptText.Append(" return true;\n");
			//riptText.Append(" return checkSearch").Append(name).Append("();\n");
		}		
		
		scriptText.Append("}\n");

		HTMLScript script = new HTMLScript("text/javaScript");
		script.AddInnerHTMLString(scriptText.ToString());

		return script;
	}

	/**
	 * Set the operator for the set of statements in the filter.
	 * 
	 * @param oper Should be PT_BOOLOP_AND or PT_BOOLOP_OR
	 */
	public void SetFilterClauseOperator(int oper)
	{
	    _filterClauseOperator = oper;
	    GetClause().SetOperator(_filterClauseOperator);
	}

	/**
	 * Return the filter being created by AddFilterStatement()
	 * 
	 * @return User filter under construction 
	 */
	private IPTFilter GetUserFilter()
	{
	    if (null == _userFilter)
	    {
		_userFilter = PortalObjectsFactory.CreateSearchFilter();
	    }			
	    return _userFilter;
	}

	/**
	 * 
	 * @return Filter clause under construction
	 */
	private IPTPropertyFilterClauses GetClause()
	{
	    if(null == _filterClause)
	    {
		_filterClause = (IPTPropertyFilterClauses) GetUserFilter().GetNewFilterItem(PT_FILTER_ITEM_TYPES.PT_FILTER_ITEM_CLAUSES);
		GetUserFilter().SetPropertyFilter(_filterClause);
		_filterClause.SetOperator(_filterClauseOperator);
	    }
	    return _filterClause;
	}

	private void clearFilterComponents()
	{
	    _filterStatementCount = 0;
	    _userFilter = null;
	    _filterClause = null;
	}
	

	/**
	 * Add an advanced search-style filter to the search form. For example:
	 * 
	 * 	factory.AddFilterStatement(PT_INTRINSICS.PT_PROPERTY_PROVIDERCLSID,
	 *    				     PT_FILTEROPS.PT_FILTEROP_CONTAINS, "61266621");
	 * This will add a filter for things with a provider classid that contains "61266621".
	 * Note that you can make multiple calls to this method to produce a set of filter statements.
	 * The default is for these to be 'and-ed' together, but you can change that by calling
	 * SetFilterClauseOperator. You should call this before calling GetForm.
	 * 
	 * @param operand Any integer property id
	 * @param operatorId Something from PT_FILTEROPS
	 * @param value A string value.
	 * 
	 * @exception HTMLException
	 */
	public void AddFilterStatement(int operand, int operatorId, String value)
	{

	    try
	    {
  		IPTPropertyFilterStatement statement = (IPTPropertyFilterStatement) GetUserFilter().GetNewFilterItem(PT_FILTER_ITEM_TYPES.PT_FILTER_ITEM_STATEMENT);
		statement.SetOperand(operand);
		statement.SetOperator(operatorId);
		statement.SetValue(SINGLE_QUOTE + value + SINGLE_QUOTE);

		GetClause().AddItem(statement, _filterStatementCount);
		_filterStatementCount++;
	    }
	    catch(Exception exe)
	    {
	    	// NOTE: Should do something like the following:
	    	// throw new XPException("msg", exe);

		throw new HTMLException("The call to AddFilterStatement passing (operand, operatorId, value) = (" + operand +
			", " + operatorId + ", " + value + ") failed with the following exception: " + exe.getMessage());
	    }

	    
		log.Debug("Leaving AddFilterStatement in SearchFormFactory");
	    
	}

	/**
	 * Create the hidden inputs for the search form to encode the filter.
	 */
	protected void CreateFilterElements()
	{
	    if(_filterStatementCount == 0)
		return;

	    // Change our inputboxname
	    _inputBoxName = FilterForm.FULL_TEXT;


    	    // Process the filter and create the dictionary of key/values
	    XPHashtable filterEls = new XPHashtable();
	    FilterForm.EncodeFilterAsControlArgs((IPTSession)m_asOwner.GetUserSession(),m_asOwner,GetUserFilter(),filterEls,""); //FilterForm.FILTER_PREFIX); 
	    

	    // Iterate through the dictionary creating inputs.
	    IXPEnumerator e = filterEls.GetKeys();
	    
		log.Debug("In CreateFilterElement creating form inputs:");
	    

	    while(e.MoveNext())
	    {
		try
		{
		    String keyStr = (String)e.GetCurrent();
		    	// Skip in_hi_fulltext as this will be added elsewhere.
		    if(FilterForm.FULL_TEXT.equals(keyStr))
		    	continue;
		    Object value = filterEls.GetElement(keyStr);

		    String valStr = EMPTY_STRING;
		    if(value != null)
		    {
			valStr = ((String [])value)[0];
		    }
		    
			HTMLInput myInput = new HTMLInput( HTMLInputTypes.HIDDEN_TYPE, EMPTY_STRING );
			myInput.SetValue(valStr);
			myInput.SetValidateName( false );
			myInput.SetName( keyStr );

		    _elements.AddInnerHTMLElement(myInput);

		    
			log.Debug("    Input name: " + keyStr + " value: " + valStr);
		    
		}
		catch(Exception exe)
		{
		    log.Error("    Got exception in CreateFilterElements: " + exe.getMessage());
		}
	    }

	    clearFilterComponents();

	    
		log.Debug("Leaving CreateFilterElement.");
	    
	}
	
	// This hack of a method is here to fix bug 24662. What we are doing with this
	// javascript (used in TreeSearchBoxView) is disabling the advanced search filter
	// for all but a single classid.
	public String GetSingleClassFilterJavascript(String optionValue)
	{
		XPStringBuilder scriptText = new XPStringBuilder(300);
		scriptText.Append("  var searchForm = document.").Append(_name).Append(";\n");
		//simpleForm true means don't use the filter
		scriptText.Append("  var simpleForm = false;\n");
		// Loop through inputs. We want to use the simpleForm if we are not 
		// on the option passed in for the in_se_sel_1 select.
		scriptText.Append(
			"  for (var i = 0; i < searchForm.elements.length; i++) {\n")
			.Append("    if (searchForm.elements[i].name == \"in_se_sel_1\"){\n");
		scriptText.Append("        simpleForm = (searchForm.elements[i].value != \"" + optionValue + "\");");
		scriptText.Append("    }\n");
		scriptText.Append("  }\n");

		// If we want the simple form, we do two things to change this to 
		// a basic search form: change the name of in_tx_fulltext input to
		// in_tx_query, and set the value of in_ra_topoperator to an empty string.
		// No top operator and search will not try to build an advanced filter out of this.
		scriptText.Append("if(simpleForm) { \n");
		scriptText.Append(
			"  for (var i = 0; i < searchForm.elements.length; i++) {\n")
			.Append("    if (searchForm.elements[i].name == \"in_tx_fulltext\"){\n");
		scriptText.Append("         searchForm.elements[i].name = \"in_tx_query\";");
		scriptText.Append("}\n");
		scriptText.Append("    else if (searchForm.elements[i].name == \"in_ra_topoperator\")\n")
		.Append("{\n");
		scriptText.Append("         searchForm.elements[i].value = \"\";");
		scriptText.Append("    }\n");
		scriptText.Append("  }\n");
		scriptText.Append("  }\n");
		return scriptText.toString();

	}
}

