package com.plumtree.portaluiinfrastructure.search;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.filter.*;
import com.plumtree.portaluiinfrastructure.friendlyurl.*;
import com.plumtree.portaluiinfrastructure.helpers.*;
import com.plumtree.portaluiinfrastructure.search.exceptions.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.form.*;

/**
 * @author CraigS
 *
 *  Superclass for all search result activity space controls.
 */
public abstract class SearchResultControl implements IControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.search.SearchResultControl");

	
	public static final String KEY_DATA = "in_hi_key";

	
	public static final String RECONSTRUCTION_ARGS_PREFIX = "r_";

	
	public static final int RECONSTRUCTION_PREFIX_LENGTH = 2;

	
	public static final String USER_QUERY   = "in_tx_query";
	
	public static final String PARTIAL_SEARCH   = "in_hi_partial";
	
	public static final String BASIC_FIELDS = "in_tx_bfields";
	
	protected AActivitySpace _parent;

	protected ISearchResultModel _model;

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IControl#Execute()
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable args)
	{
		Redirect r = null;

		// Clear any errors from previous searches.
		// (Is there a nicer way to do this?)
		{
			IXPEnumerator errors = UIErrorHandler.GetErrors(_parent.GetSubSession(),
					_parent.GetApplication(), _parent, true, false);

			while (errors.MoveNext())
			{
				UIError pte = (UIError) errors.GetCurrent();
				UIErrorHandler.ClearError(pte, _parent.GetSubSession(),
					_parent.GetApplication());
			}
		}

		String key = null;

		// Figure out where we were (i.e. the value of the key)
		// when the model was generated
		String[] values = (String[]) args.GetElement(KEY_DATA);

		if (values != null)
		{
			key = values[0];
		}

		try
		{
			// If the subclass expects the model to be initialized,
			// but it is not..
			if (expectModelInitialized() && (!_model.IsInitialized()))
			{
				// Try to reconstruct the model from the provided args
				reconstructModel(args);
			}

			// At this point the model ought to be ok.
			// Run the real Execute method.
			return ExecuteSearchControl(args, key);
		}
		catch (SearchExecutionWarning e)
		{
			log.Warn(e, e.GetMessage());

			_model.SetWarning(e);
		}
		catch (SearchExecutionError e)
		{
			log.Error(e, e.GetMessage());

			_model.SetError(e);

			// Raise this error through the framework so it displays on the page
			UIError ptError = new UIError(UIErrorLevel.SERIOUS,
					UIErrorScope.CURRENT_USER, e, e.GetMessage(), null);
			ptError.SetActivitySpace(_parent.GetName());
			ptError.SetClearWithActivitySpaceChange(true);

			UIErrorHandler.RaiseError(ptError, _parent.GetSubSession(),
				_parent.GetApplication());
		}
		catch (SearchExecutionException e)
		{
			// shouldn't happen; all instances should be a Warning or an Error
		}

		return r;
	}

	/**
	  * This method is used by the ASManager to return new instances of managed
	  * objects.
	  *
	  * @return A new instance of the managed class (i.e. return new Foo();)
	  */
	abstract public Object Create();

	/**
	 * @see com.plumtree.xpshared.activityspace.IControl#GetName()
	 */
	public abstract String GetName();

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IControl#Init(IModel, AActivitySpace)
	 */
	public void Init(IModel model, AActivitySpace space)
	{
		_model	    = (ISearchResultModel) model;
		_parent     = space;
	}

	/**
	 *  Most subclasses of SearchResultControl have a makeURL() method
	 *  to generate a link that activates the control; makeURL() should
	 *  call this method to set common superclass data.
	 */
	protected static void addCommonData(ISearchResultModelRO model, ILink url)

	{
		String modelKey = model.GetCurrentKey();

		// Record where in a drilldown sequence this URL was generated
		url.AddControlArgument(KEY_DATA, modelKey);

		// If no drilldown has occurred, we can reconstruct the search
		// if it gets flushed from the cache.  Save enough info to do
		// this.
		if (1 >= model.GetDrilldownDepth())
		{
			XPHashtable reconstructArgs = model.GetReconstructionArgs();

			IXPEnumerator keys = reconstructArgs.GetKeys();

			while (keys.MoveNext())
			{
				String key	    = (String) keys.GetCurrent();
				String[] values = (String[]) reconstructArgs.GetElement(key);

				for (int j = 0; j < values.length; j++)
				{
					url.AddControlArgument(key, values[j]);
				}
			}
		}
	}

	/**
	 *  Execute method that subclasses should implement; unlike Execute()
	 *  it's allowed to throw a SearchExecutionException.  The key argument
	 *  indicates the page on which this control's URL was located; this
	 *  lets us handle the case where the user hits BACK before clicking
	 *  on something, and (for example) drills down into a different set
	 *  of categories than the set being displayed on the current page.
	 */
	abstract protected Redirect ExecuteSearchControl(XPHashtable args,
		String key);

	/**
	 *  Subclasses should override this method and return false if they don't
	 *  mind the model being uninitialized on entry.
	 */
	protected boolean expectModelInitialized()
	{
		return true;
	}

	/**
	 *  Helper method to grab query parameters out of the provided
	 *  arguments and return them in a temporary QueryArguments structure.
	 *  The argument names may be prepended with the provided prefix.
	 *
	 *  If no args were found or the query could not be reconstructed,
	 *  this method should return null.
	 */
	protected QueryArguments parseQueryArgs(XPHashtable args, String prefix)
	{
		QueryArguments result = new QueryArguments();

		String[] values;

		values = (String[]) args.GetElement(prefix + USER_QUERY);
		
		//RT - If user switches search from Administration UI to Banner then
		// Initialy it is not processed by the Friendly URL code, so handling that case.
		if( (null == values) && PTConfigService.getPTInstance().UseFriendlyURLS(FriendlyURLConstants.FRIENDLY_SEARCH_ID, "")) {
			values = (String[]) args.GetElement(prefix + SearchFriendlyURLParser.SEARCH_QS);
		}

		if ((null != values) && (0 < values.length))
		{
			result.userQuery = values[0];
			
			// DCA - check if we want partial search
			values = (String[]) args.GetElement(PARTIAL_SEARCH);
			if ((null != values) && (0 < values.length) && XPConvert.ToBoolean(values[0])) {
				// only add wildcard if it doesn't end w/ it
				if (!XPStringUtility.EndsWith(result.userQuery, "*")) {
					result.userQuery += "*";
				}
			}
		}
		else
		{
			// No basic query.  See whether this is an advanced search.
			result.advancedFilter = FilterForm.BuildFilterFromForm(_parent, args,
                                                             prefix); 

			if (null == result.advancedFilter)
			{
				// No query at all
				return null;
			}
		}

		// Check for fields argument as well; may be empty
		values = (String[]) args.GetElement(prefix + BASIC_FIELDS);

		if ((null != values) && (0 < values.length))
		{
			result.basicFields = values[0];
		}
		else
		{
			result.basicFields = null;
		}

		return result;
	}

	/**
	 *  Attempt to reconstruct a model that's been flushed from the
	 *  ActivitySpace cache, using the provided arguments.
	 *  Throws a SearchExecutionException if unsuccessful.
	 */
	private void reconstructModel(XPHashtable args)
	{
		// Parse user's original query
		QueryArguments query = parseQueryArgs(args, RECONSTRUCTION_ARGS_PREFIX);

		if (null == query)
		{
			throw new CacheFlushedException(_parent);
		}

		// Parse settings
		SearchSettingCollection settings = new SearchSettingCollection();

		IXPEnumerator keys = args.GetKeys();

		while (keys.MoveNext())
		{
			String keyString = (String) keys.GetCurrent();

			if (keyString.startsWith(RECONSTRUCTION_ARGS_PREFIX))
			{
				String[] valueStrings = (String[]) args.GetElement(keyString);

				int settingId = SearchSettingCollection.getSettingIdFromString(keyString.substring(
							RECONSTRUCTION_PREFIX_LENGTH));

				// If it's a real setting
				if (0 != settingId)
				{
					// parse its value
					Object settingValue = SearchSettingCollection.parseURLString(settingId,
							valueStrings);

					// and save it for later
					settings.add(settingId, settingValue);
				}
			}
		}

		// Now reconstruct the old results
		_model.Search(query.userQuery, query.basicFields, query.advancedFilter,
			settings);
	}

  /**
   *  Utility function to figure out whether we're in a space that requires
   *  a "passthru url" (i.e. a link that is actually javascript to direct
   *  the request through the repost control, so that the page gets saved
   *  before the search link is followed).
   */
  public static boolean SpaceNeedsPassthruURL(AActivitySpace space) {
    return ((space.GetAccessStyle() == AccessStyles.STANDARD) &&    // not in 508 and
            (space instanceof AFormAS) &&                           // a forms page that
            (null != ((AFormAS) space).GetRepostControlName()));    // uses a repost control
  }
}

