package com.plumtree.portaluiinfrastructure.sso;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;

/**
 * Interface for classes which integrate Plumtree with SSO products.  Classes
 * implementing this interface must include a default constructor in order for
 * Plumtree to instantiate them.
 * 
 * Methods can return <code>null</code> if they are unimplemented, encounter an
 * error, or have no return value.
 * 
 * Implementations are used in the following manner: first, a single static
 * instance is instantiated using reflection.  <code>Initialize</code> is called
 * on this instance, passing in settings from the <code>SSOSettings</code>
 * section of <code>x_config.xml</code>.  <code>GetSecureHeaders</code>,
 * <code>GetSecureCookies</code>, and <code>GetSSOProductName</code> are each
 * called once on the static instance, and the results cached.
 * 
 * Each time a user logs in, <code>GetLoginInfo</code> is called on the static
 * instance to produce a <code>SSOLoginInfo</code> instance with information
 * about the current user. <code>GetLoginInfo</code> may be called concurrently
 * by several threads, so it must be thread safe.
 * 
 * When a user clicks logout, <code>OnLogout</code> is called to end the SSO
 * session.  Note that this method is not called every time a session ends, for
 * instance if a user closes the browser window or a session times out.  This
 * method may also be called concurrently by various threads, so it must also be
 * thread safe.
 *
 * @author Mike Jahr
 * @see com.plumtree.portaluiinfrastructure.sso.SSOLoginInfo
 */
public interface ISSOIntegration
{
    /**
     * Initializes the object with settings from the <code>sso.xml</code> config
     * file.  The map is from <code>String</code> to <code>XPArrayList</code>.
     *
     * @param mapSettings an <code>XPHashtable</code> of settings
     *
     * @return a <code>boolean</code> indicating success
     */
    public boolean Initialize(XPHashtable mapSettings);

	/**
	 * Returns a brief description of the SSO product.  This method will only be
	 * called once, and the result cached.
	 *
	 * @return a description
	 */
	public String GetSSOProductName();

	/**
	 * Returns the names of secure cookies which should be forwarded to portlets
	 * for authentication.  This method will only be called once, and the result
	 * cached.
	 *
	 * @return a <code>String[]</code> array of cookie names
	 */
	public String[] GetSecureCookies();

	/**
	 * Returns an array of header prefixes which should not be forwarded in
	 * requests to portlets.  For example, if "PT_" is in the array, no headers
	 * beginning with "PT_" will be included in HTTP requests to portlets.  This
	 * list is case-sensitive.  This method will only be called once, and the
	 * result cached.
	 *
	 * @return a <code>String[]</code> array of header names
	 */
	public String[] GetSecureHeaders();

    /**
     * Returns login information extracted from the headers of the given
     * request.  This method is called whenever a user logs in to Plumtree.
     * 
     * This method must be thread safe.
     *
     * @param request an <code>IXPRequest</code> value
     *
     * @return a <code>SSOLoginInfo</code> value
     *
     * @see com.plumtree.portaluiinfrastructure.sso.SSOLoginInfo
     */
    public SSOLoginInfo GetLoginInfo(IXPRequest request);

	/**
	 * Performs any logout actions required by the SSO product.  This can
	 * include clearing cookies or redirecting to another web page that performs
	 * logout actions.  If a redirect is performed, the other web page should
	 * redirect back to <code>sReturnURL</code> after performing the SSO logout.
	 *
	 * Note that this function is only called if the user clicks the logout link
	 * in the portal, not if their session times out or they close the browser
	 * window.
     * 
     * This method must be thread safe.
	 *
	 * @param response an <code>IXPResponse</code> value
     * 
     * @return <code>true</code> if a redirect is required, <code>false</code> otherwise
	 */
	public boolean OnLogout(IXPResponse response, String sReturnURL);
}

