package com.plumtree.portaluiinfrastructure.sso;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;


/**
 * Simple class which contains a username and password.
 * 
 * @author Mike Jahr
 */
public class SSOLoginInfo
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_SSO),
		"com.plumtree.portaluiinfrastructure.sso.SSOLoginInfo");

	private String m_sPassword;
	private String m_sPrefixAndName;

	/**
	 * Constructs a <code>SSOLoginInfo</code> with the given information.  The
	 * given prefix and username are conca  If no password is available, <code>null</code>
	 * should be passed, or the simplest constructor can be used.
	 * 
	 * @param sPrefix the user prefix
	 * @param sName the username without the prefix
	 * @param sPassword the password of the user
	 */
	public SSOLoginInfo(String sPrefix, String sName, String sPassword)
	{
		m_sPassword		 = sPassword;
		
		if (sPrefix != null && sPrefix.length() > 0)
		{
			m_sPrefixAndName = sPrefix + '\\' + sName;
		}
		else
		{
			SetPrefixAndName(sName);
		}
	}

	/**
	 * Constructs a <code>SSOLoginInfo</code> with the given information.  If
	 * the prefix of the user is unavailable, sPrefixAndName should contain only
	 * the username without a "\\"; in this case the default prefix from
	 * x_config.xml will be added.  If no password is available, <code>null</code>
	 * should be passed, or the simplest constructor can be used.
	 * 
	 * @param sPrefixAndName the username in the form <code>"prefix\\username"</code>
	 * @param sPassword the password of the user
	 */
	public SSOLoginInfo(String sPrefixAndName, String sPassword)
	{
		m_sPassword		 = sPassword;
		
		SetPrefixAndName(sPrefixAndName);
	}

	/**
	 * Constructs a <code>SSOLoginInfo</code> with the given username and an
	 * empty password.	Identical to 
	 * <code>SSOLoginInfo(sPrefixAndName, null)</code>.
	 * 
	 * @param sPrefixAndName the username in the form <code>"prefix\\username"</code>
	 */
	public SSOLoginInfo(String sPrefixAndName)
	{
		this(sPrefixAndName, null);
	}

	/**
	 * Returns the user's password.	 Only necessary if it is to be passed to
	 * portlets in basic auth headers.
	 *
	 * @return the password
	 */
	public String GetPassword()
	{
		return m_sPassword;
	}

	/**
	 * Returns the prefix and name of this user in the form
	 * <code>"prefix\\username"</code>.
	 *
	 * @return the qualified username
	 */
	public String GetPrefixAndName()
	{
		return m_sPrefixAndName;
	}
	
	/**
	 * Sets the password;
	 * 
	 * @param sPassword the user's password
	 */
	public void SetPassword(String sPassword)
	{
		m_sPassword = sPassword;
	}
	
	/**
	 * Sets the username in the form <code>"prefix\\username"</code>.  If the
	 * prefix of the user is unavailable, sPrefixAndName should contain only the
	 * username without a "\\"; in this case the default prefix from x_config.xml
	 * will be added.
	 *
	 * @param sName the username in the form <code>"prefix\\username"</code>
	 */
	public void SetPrefixAndName(String sName)
	{
		if ((sName == null) || (sName.length() == 0))
		{
			m_sPrefixAndName = null;
		}
		else if (sName.indexOf('\\') == -1)
		{
			// mj: if m_sPrefixAndName doesn't contain the domain, we must add it
			String sDomain = SSOIntegrator.GetDefaultPrefix();

			if ((sDomain != null) && (sDomain.length() > 0))
			{
				log.Info("SSOLoginInfo: Found default prefix: {0}", sDomain);
				m_sPrefixAndName = new XPStringBuilder(sDomain.length() +
					sName.length() + 1).Append(sDomain).Append('\\')
					.Append(sName).ToString();
			}
			else
			{
				log.Info("SSOLoginInfo: No domain or default prefix was found.");
				m_sPrefixAndName = sName;
			}
		}
		else
		{
			m_sPrefixAndName = sName;
		}
		
		log.Info("SSOLoginInfo username: {0}", m_sPrefixAndName);		
	}
}

