package com.plumtree.portaluiinfrastructure.sso;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.authsource.sso.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.activityspace.utils.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.helpers.*;
import com.plumtree.uiinfrastructure.interpreter.*;
import com.plumtree.uiinfrastructure.interpreter.filter.utils.*;
import com.plumtree.uiinfrastructure.login.*;
import com.plumtree.uiinfrastructure.opener.*;
import com.plumtree.uiinfrastructure.opener.types.classtypes.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * <code>SSOLoginPage</code> is a servlet that retrieves SSO login information for 
 * integration with SSO products.  It must be a separate servlet from the main portal
 * application
 *
 * @author Mike Jahr
 *
 * @version 1.0
 */
public class SSOLoginPage extends XPPage implements IXPPage
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_SSO),
		"com.plumtree.portaluiinfrastructure.sso.SSOLoginPage");

    private static final String EMPTY_STRING = "";
    // open=space&name=Login
    private static final String LOGIN_SPACE_MARTOK_STYLE = IOpenerConstants.QS_OPEN
                                                           + LinkConstants.EQUALS
                                                           + ASConstants.SPACENAME
                                                           + LinkConstants.AMPERSAND
                                                           + UIActivitySpaces.QS_ACTIVITY_SPACE_NAME
                                                           + LinkConstants.EQUALS
                                                           + LoginHandlers.LOGINSPACE;
    // space=Login
    private static final String LOGIN_SPACE_OLD = ASConstants.SPACENAME
                                                  + LinkConstants.EQUALS
                                                  + LoginHandlers.LOGINSPACE;
    // Old style: space=Login&dljr=
    // We use the Martok style: open=space&name=Login&dljr=
    private static final String LOGIN_SPACE_REDIRECT_QUERY_STRING = LOGIN_SPACE_MARTOK_STYLE
                                                           + LinkConstants.AMPERSAND
                                                           + LoginHelper.PARAM_DONT_LOGIN_JUST_REDIRECT
                                                           + LinkConstants.EQUALS;

	// mj: static instance of the SSOLoginPage
	private static final IXPPage m_smPage = new SSOLoginPage();

	/**
	 * Performs the SSO login, if necessary, and then redirects back to the
	 * standard servlet.
	 *
	 * @param request an <code>IXPRequest</code> value
	 * @param response an <code>IXPResponse</code> value
	 * @param session an <code>ISessionManager</code> value
	 * @param application an <code>IApplication</code> value
	 * @see com.plumtree.uiinfrastructure.web.IXPPage#DoService(IXPRequest, IXPResponse, ISessionManager, IApplication)
	 */
	public void DoService(IXPRequest request, IXPResponse response,
		ISessionManager session, IApplication application)
	{
		// mj: construct the redirect url back to the main portal page.
		//     we will use the url stored on the session combined with the query string to this page.
		String sRedirectURL = null;
		String sOriginalURL = null;
		String sFriendlyURLPart = "";
		String sGatewayURL = "";
		String sQS = request.GetQueryString();
		
		// Get the original URL that was requested.
		// redirect url stored in cookie instead of session variable since session is lost in the redirect
		// to sso login page in dotnet for session fixation protection
		// @TODO Should really pass redirect in querystring but there is some old and complex handling 
		// of querystring in ssologin page so less risky to pass redirect in cookie for nightrain release
		//sOriginalURL = (String) session.GetAttribute(SSOIntegrator.REQUEST_URL_BEFORE_SSO_LOGIN);
		sOriginalURL = GetCookieValue(request, SSOIntegrator.REQUEST_URL_BEFORE_SSO_LOGIN);
		if ( log.IsDebugEnabled() ) { 
			log.Debug("Value of Request URL before SSO Login cookie : "+sOriginalURL);
		}
		
		if (sOriginalURL != null)
		{
			
			// Base URL to redirect to
			// mj: we must process the url to make sure we use a client-friendly redirect url
			sRedirectURL = PTConfigHelper.GetApplicationBaseURL(ConfigHelper.GetNormalizedBaseURL(sOriginalURL));
			// Bug 9352261: Change from single colons to double colons to avoid conflict with colons in absolute urls
			// Semi-colons in the url were replaced with colons before set in the cookie since cookie values are deliminated with semi-colons
			// change back
			sOriginalURL = sOriginalURL.replace("::", ";");
			
			log.Debug("SSO login found after login redirect URL : "+sOriginalURL);
			
			if (sRedirectURL.indexOf('?') != -1)
			{
				sRedirectURL = sRedirectURL.substring(0, sRedirectURL.length() - 1);
			}
			log.Debug("SSO login redirect base URL: "+sRedirectURL);
			

			// Gateway arguments
			int intGatewayIndex = sOriginalURL.indexOf(GatewayHandlers.GATEWAY_UNIQUE);
			if (intGatewayIndex != -1)
			{
				int intQSIndex = sOriginalURL.indexOf("?");
				if (intQSIndex == -1)
				{
					intQSIndex = sOriginalURL.length();
				}
				sGatewayURL = sOriginalURL.substring(intGatewayIndex, intQSIndex);
				log.Debug("Redirect URL is a gateway URL: "+sGatewayURL);
			}

            // mel: Fix for 38605, when accessing a community link without the gateway, user is redirected to the login page, even though they have SSO running. This is due to sQS (query string) being null (see request.GetQueryString() above).
            // For some reason request.GetQueryString is returning null even when there's a valid query string after the ?
            //
            // mel: Additional fix for 50816: this was previously an "else if" with the gateway if above.
            // Forced the query string to always be processed, even if it's part of a gateway arg.
            //
            if(sQS == null || sQS.length() == 0)
            {
                int queryStringIndex = sOriginalURL.indexOf("?");
                if(queryStringIndex > 0 && sOriginalURL.length() > queryStringIndex)
                {
                    sQS = sOriginalURL.substring(queryStringIndex+1);
                    log.Debug("Setting redirect URL query string from query string from original requested URL: "+sQS);
                }
            }
            
            String httpEntryPoint = ConfigService.getInstance().GetHTTPEntryPoint();
            int friendlyUrlBeginIndex = sOriginalURL.indexOf(httpEntryPoint) + httpEntryPoint.length();
            log.Debug("SSOLogin Page values - sQS: "+sQS);
			log.Debug("SSOLogin Page values - sGatewayURL: "+sGatewayURL);
			log.Debug("SSOLogin Page values - friendlyUrlBeginIndex: "+friendlyUrlBeginIndex);
			log.Debug("SSOLogin Page values - sOriginalURL: "+sOriginalURL);
            
            // rob: fix for #8535406 - handle friendly url redirects
            // look for the slash after server.pt, ex http://portal.plumtree.com/server.pt/comm/500
			log.Debug("Checking if Redirect URL is a friendly URL.");
            if ( friendlyUrlBeginIndex > 0 && friendlyUrlBeginIndex < sOriginalURL.length() && sOriginalURL.charAt(friendlyUrlBeginIndex) == '/' ) 
            {
            	// include the slash in the friendly url part
            	sFriendlyURLPart = sOriginalURL.substring(friendlyUrlBeginIndex, sOriginalURL.length());
            	// strip off query string from friendly url part since it is added to the redirect url in the sQs variable
            	if (sFriendlyURLPart.indexOf('?') != -1)
            	{
            		// but keep the '?' character in the friendly url part since query string doesnt include it
            		sFriendlyURLPart = sFriendlyURLPart.substring(0, sFriendlyURLPart.indexOf('?')+1 );
            	}
            	log.Debug("Redirecting to friendly Url after SSO login: "+sFriendlyURLPart);
            }
		}
		else
		{
			// No original request URL was found on the session.
			// Redirect to the main page.

			log.Error("The SSO Login Page was unable to retrieve the request URL from the session. " +
					"Will use a relative redirect to return to the main page.");

            // mel: Fix for bug 45335, when SSOServlet is hit directly via http in security mode 3, the redirect needs to be coerced into https
            if(request.IsSecureConnection()
               || PTConfigHelper.GetSecurityMode() == 2
               || PTConfigHelper.GetSecurityMode() == 3)
			{
				sRedirectURL = PTConfigHelper.GetMainServerApplicationURL(true);
			}
			else
			{
				sRedirectURL = PTConfigHelper.GetMainServerApplicationURL(false);
			}
            
            
		}

		// If there is no query string, take off the ending '?'.
		// If there is a query string, make sure the redirectURL has a '?'.
		if (sQS == null || sQS.length() == 0)
		{
			sQS = "";
			int nIndex = sRedirectURL.indexOf('?');
			if (nIndex > 0)
				sRedirectURL = sRedirectURL.substring(0, nIndex);
		}
		else
		{
			// SE: 54255: if sRedirectURL already has a '?' then remove it
			int nIndex = sRedirectURL.indexOf('?');
			if (nIndex > 0)
				sRedirectURL = sRedirectURL.substring(0, nIndex);
			
				sGatewayURL += '?';
		}
		
		//log.Info("--Redirect URL: " + sRedirectURL + sFriendlyURLPart + sGatewayURL + sQS);
		
		try
		{
			RequestData tempData = new RequestData(request, response, session,
					application);

			// mj: if sso is disabled, redirect back immediately
			if (!SSOAuthenticator.GetInstance().UseThisAuth(tempData))
			{
			
					log.Error("SSO is disabled. Exiting SSOLoginPage.");
			

				response.Redirect(sRedirectURL + sGatewayURL + sQS);
				return;
			}
		
			// mj: SSOIntegrator is a helper class that handles all the SSO-specific stuff
			SSOLoginInfo info = SSOIntegrator.GetLoginInfo(request);
			String sUserName = info.GetPrefixAndName();
	
			if (sUserName == null)
			{
				log.Error("SSO integration returned a null username. Exiting SSOLoginPage.");
				
				response.Redirect(sRedirectURL + sGatewayURL + sQS);
				return;
			}
	
			// mj: get the AActivitySpace cache and cachelist
			tempData.m_smCache		    = ASCache.GetASCache(session, tempData.m_smSandbox);
			tempData.m_xpmCacheList     = ASCache.GetCacheList(session);

			// mj: set the request url on the tempData so that the login code
			// constructs the expected url for gateway clickthroughs.
			// the login code expects that we're hitting the Interpreter page,
			// so we must use give it the request url it's expecting.
			tempData.SetRequestURL(sOriginalURL);
	
			if (tempData.GetUserSession() != null)
			{
				String strCurrentUserLogin = ((IPTSession) tempData.GetUserSession()).GetSessionInfo().GetCurrentUserLoginName(); 
				log.Info("SSOLoginPage: User session found for {0}", strCurrentUserLogin);
	
				// mj: unless they're logging in as a different user, redirect back
				//     to the main servlet.
				if ((sUserName != null) &&
						XPStringUtility.EqualsIgnoreCase(sUserName, strCurrentUserLogin))
				{
					log.Info("The user " + sUserName + " already has a valid session. " + "Exiting SSOLoginPage.");
					
	
					response.Redirect(sRedirectURL + sGatewayURL + sQS);
					return;
				}
				
				// Logging in as a different user, so first logout the current one.
				Redirect r = LoginHelper.INSTANCE.DoTasksBeforeLogout(tempData.GetUserSession(), 
					tempData.m_xpRequest, tempData.m_wdPageData, application, tempData.m_smSandbox, 
					tempData.m_smPersistentSandbox);
					
				if (r != null && log.IsWarnEnabled())
				{
					log.Warn("SSO logout is pre-empting the Logout PEI redirect to " +
									(r.GetIsExternalURL() ? r.GetExternalURL() : r.GetQueryString()));
				}

				// mj: logout code goes here.
				LoginHandlers.DoLogout(tempData);
				// Should probably use ClearUserSession()
				tempData.SetUserSession(null);
			}
	
			// Attempt portal login for the SSO user
			if (sUserName != null)
			{
				// mj: we need an SSO auth token
				IPTSSOAuthTokenGenerator generator = PortalObjectsFactory.CreateSSOAuthTokenGenerator();
				String sPassword				   = "";
				String sToken					   = generator.GetSecureToken();

				LoginResult loginResult = LoginHelper.INSTANCE.AttemptLogin
					(sUserName, sPassword, sToken, false, tempData.m_smSandbox,
					tempData.m_smPersistentSandbox, application, 
					tempData.GetRequestURL(), tempData.m_xpRequest,
					tempData.m_wdPageData);

				if (loginResult.m_bSuccess)
				{
					tempData.SetUserSession(loginResult.m_UserSession);

					XPCookie cookie = null;

					if (( tempData.m_wdPageData != null ) && (!loginResult.m_bRedirectedBeforeLogin))
					{
						// Not setting an expires value on a cookie uses the
						// default behaviour in both Java and .NET (deleted
						// when browser is closed).
						cookie = new XPCookie(LoginHandlers.LOGIN_OCCURED_COOKIE_NAME,
							XPConvert.ToString(true));
						cookie.SetPath("/");

						tempData.m_wdPageData.AddCookie(cookie);

					}

					LoginHandlers.DoLogin(tempData);
						
					log.Info("SSO login succeeded for the user " + sUserName);

					// mj: login succeeded. set SSOInfo on the AS cache.
					IPTSSOInfo ptSSOInfo = SSOIntegrator.GetSSOInfo(request, info);
					tempData.m_smSandbox.SetAttribute(PlumtreeExtensibility.PT_SSOINFO_ON_STATE, ptSSOInfo);
					session.SetAttribute(SSOIntegrator.SSOSTATEATTRIBUTE, new Integer(SSOIntegrator.SSOSTATE_LOGGEDIN));
					
					// Session fixation vunerability protection
					// In java we set a flag on the session to abandon session on the next request. The flag is checked in
					// XPPage.ProcessUseNewHTTPSession, which will migrate data on the current session to a new session after
					// succesful sso login
					// Session abandon works different in dotnet where we clear asp session id cookie when redirecting to sso
					// login page. IIS will assign a new session id to the session when hitting this page 
					if (!XPSystem.IsRunningInDotNet()) {
						session.SetAttribute(XPPage.USE_NEW_SESSION_FLAG, "");
						log.Debug("Setting flag to use new http session on next request for session fixation vunerability protection.");
					}
						
				} else
				{
					
						log.Error("SSO login failed for the user " + sUserName +
							": " + loginResult.m_strError);
					

					// mj: raise the error
					// We don't set the activity space name, because we don't know it.
					UIError myError = new UIError(UIErrorLevel.FRIENDLY,
							UIErrorScope.CURRENT_USER, null, loginResult.m_strError,
							loginResult.m_strError);
					myError.SetClearWithActivitySpaceChange(true);
		
					UIErrorHandler.RaiseError(myError, session, application);

					// Should probably use ClearUserSession()
					tempData.SetUserSession(null);
				}

				if (loginResult.m_Redirect != null)
				{
					String strRedirectURL;
					// A login PEI returned a redirect that we need to go to.
					if (loginResult.m_Redirect.GetIsExternalURL())
					{
						strRedirectURL = loginResult.m_Redirect.GetExternalURL();
					}
					else
					{
						String strBaseURL = ConfigHelper.GetApplicationBaseURLToAS(ConfigHelper.GetNormalizedBaseURL(sOriginalURL),
							loginResult.m_Redirect.GetSpaceName(), ConfigHelper.IsGoingToSubSpace(loginResult.m_Redirect));
						strRedirectURL = strBaseURL + loginResult.m_Redirect.GetQueryString();
					}
					
					log.Info("SSOLoginPage redirecting user to {0}", strRedirectURL);
					tempData.m_xpResponse.Redirect(strRedirectURL);

					// mj: our work is done. redirect back to the main servlet.
					return;
				}
			}
	
			// mj: check whether login was successful
			if (tempData.GetUserSession() != null)
			{
				// If this is not a gatewayed URL and there's no activity space in the query string,
				// use the default post-login space
				// rob: Fix for #8535406  don't redirect to login space if we have a friendly url
                if (ShouldRedirectToLoginSpace(sGatewayURL, sQS) && sFriendlyURLPart.length() == 0 )
				{
                    // If the redirect does not end with ? and the gateway URL does not have ?, then we need to add it
					if (!sRedirectURL.endsWith(LinkConstants.QUESTION)
                        && (sGatewayURL == null || sGatewayURL.indexOf(LinkConstants.QUESTION) < 0))
                    {
						sRedirectURL += LinkConstants.QUESTION;
                    }
					sQS = LOGIN_SPACE_REDIRECT_QUERY_STRING;
				}
			}
		}
		catch (Exception e)
		{
			log.Error(e, "SSO login encountered an error.");
			// @TODO shouldn't we redirect to some default page if this fails?
		}

		String redirectUrl;
		log.Debug("SSO Final Redirect URL: "+ sRedirectURL);
		log.Debug("SSO Final Redirect URL FriendlyURL Part: "+sFriendlyURLPart);
		log.Debug("SSO Final Redirect URL GatewayURL Part: "+sGatewayURL);
		log.Debug("SSO Final Redirect URL Querystring: "+sQS);
		if ( sFriendlyURLPart.length() == 0 ) {
			redirectUrl = sRedirectURL + sGatewayURL + sQS;
		} else {
			redirectUrl = sRedirectURL + sFriendlyURLPart + sQS;
		}
		// mj: our work is done. redirect back to the main servlet.
		log.Info("SSO Login redirecting to: " + redirectUrl);
		
		response.Redirect(redirectUrl);
	}

	/**
	 * @param request
	 * @param sOriginalURL
	 * @return
	 */
	protected String GetCookieValue(IXPRequest request, String cookieName)
	{
		if ( cookieName == null ) { return null; }
		String cookieValue = "";
		// request only has getCookies method, so have to iterate over
		// all cookies and look for the one we want
		XPCookie[] cookies = request.GetCookies();
		for ( int i = 0; i < cookies.length; i++ ) {
			if ( cookieName.equals( cookies[i].GetName() ) ) {
				cookieValue = cookies[i].GetValue();
				break;
			}
		}
		return cookieValue;
	}

    // mel: Fix for 40320, when sso fails and there is no query string, need to redirect user to "space=Login"
    /**
     * @return true if the gatewayURL is unspecified and the queryString has
     * a space specified that indicates login.
     * false if those conditions are not met,
     * and false if the queryString indicates this is a redirect only (and should not login).
     */
    private boolean ShouldRedirectToLoginSpace(String gatewayURL, String queryString)
    {
        // PRE: sQS != null because it was set to ""
        // in DoService around the part: if (sQS == null || sQS.length() == 0)
        if(queryString == null)
        {
            return false;
        }

        boolean rval = false;

        // Is the gateway URL unspecified, "" or "?" match
        boolean gatewayURLIsUnspecified = (gatewayURL == null || gatewayURL.equals(EMPTY_STRING));
        if(!gatewayURLIsUnspecified && LinkConstants.QUESTION.equals(gatewayURL))
        {
            gatewayURLIsUnspecified = true;
        }

        if(gatewayURLIsUnspecified)
        {
            // If it has space or open= then it's using the common opener and should not redirect to login.
            if(queryString.indexOf(ASConstants.SPACENAME) < 0
               && queryString.indexOf(IOpenerConstants.QS_OPEN + LinkConstants.EQUALS) < 0)
            {
                // No space was specified in the query string, so we will adjust
                // the sQS (query string) below to redirect to login
                rval = true;
            }
            // If this is a redirect only and not a login, we should not rewrite the query string
            else if(queryString.indexOf(LoginHelper.PARAM_DONT_LOGIN_JUST_REDIRECT) < 0)
            {
                // TODO Can we eliminate LOGIN_SPACE_OLD since Martok uses the common opener?
                // space=Login
                boolean isOldStyleFullLoginSpace = queryString.indexOf(LOGIN_SPACE_OLD) >= 0;

                // open=space&name=Login
                boolean isMartokStyleFullLoginSpace = queryString.indexOf(LOGIN_SPACE_MARTOK_STYLE) >= 0;

                rval = (isOldStyleFullLoginSpace || isMartokStyleFullLoginSpace);
            }
        }
        return rval;
    }


	/**
	 * Returns the shared static instance of SSOLoginPage.
	 *
	 * @return an <code>IXPPage</code> value
	 * @see com.plumtree.xpsupport.web.XPPage#GetXPPage()
	 */
	public IXPPage GetXPPage()
	{
		return m_smPage;
	}
}

