package com.plumtree.portaluiinfrastructure.statichelpers;

import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;

/**
 * Provide security related configuration information.
 * This class contains helper methods to get variables from the ConfigVarPack.
 * This class accompanies ConfigHelper and PTConfigHelper.
 */
public class SecurityConfigHelper
{
    private static final String OUR_NAME = "SecurityConfigHelper";

    private static OpenLogger log = OpenLogService.GetLogger(
        OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
        "com.plumtree.portaluiinfrastructure.statichelpers.SecurityConfigHelper");

    private String ssoAbsoluteURL = null;

    private SecurityConfigHelper()
    {
        InitializeSSOAbsoluteURL();
        log.Debug(OUR_NAME + ".ctor completed.");
    }

    // mel: Avoid the double checked locking problem by lazy initalization of a static instance.
    // The JVM spec (and the .NET runtime) make guarantees that a static member
    // will be initialized only once upon first active use of the class.
    // See http://www.cs.umd.edu/~pugh/java/memoryModel/DoubleCheckedLocking.html
    // http://java.sun.com/docs/books/vmspec/html/Concepts.doc.html#19075
    private static class SecurityConfigHelperLatch
    {
        public static SecurityConfigHelper singleton = new SecurityConfigHelper();
    }

    // Since SecurityConfigHelper uses lazy initialization, this should only be called
    // when an absolute URL is needed by calling GetInstance().GetSSOAbsoluteURL().
    private void InitializeSSOAbsoluteURL()
    {
        String ssoBaseURL = PTConfigHelper.GetSSOBaseURL();

        try
        {
            // ssoBaseURL=/portal/SSOServlet

            boolean useSecureURL = false;
            if(ConfigHelper.GetSecurityMode() == 1
               || ConfigHelper.GetSecurityMode() == 2
               || ConfigHelper.GetSecurityMode() == 3)
            {
                useSecureURL = true;
            }
            String portalMainServerURL = ConfigHelper.GetMainServerApplicationURL(useSecureURL);

            if(portalMainServerURL == null || portalMainServerURL.indexOf("*") == 0)
            {
                log.Warn(OUR_NAME + ".InitializeSSOAbsoluteURL failed to determine an absolute URL. The main server application URL (secure=" + XPConvert.ToString(useSecureURL) + ") returned was: " + portalMainServerURL);
            }
            else
            {
            // portalMainServerURL=https://hostname:8080/portal/server.pt

            IApplication application = ApplicationManager.GetInstance()
                .GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
            ConfigVarPack vpConfig = (ConfigVarPack) application.GetVarPackManager()
                .GetVariablePackage(ConfigVarPack.VARPACK_ID);

            String portalVirtualDirectoryPath
                = vpConfig.GetConfigValueAsString(
                    ConfigConstants.SYSTEM_PROPERTIES_SECTION,
                    ConfigConstants.VIRTUAL_DIRECTORY_PATH_SUBSECTION,
                    ConfigConstants.VALUE_ATTRIBUTE);
            // portalVirtualDirectoryPath=/portal/

            ssoAbsoluteURL = portalMainServerURL;
            int indexOfPortalVirtualDir = ssoAbsoluteURL.indexOf(portalVirtualDirectoryPath);
            if(indexOfPortalVirtualDir >= 0)
            {
                ssoAbsoluteURL
                = ssoAbsoluteURL.substring(0, indexOfPortalVirtualDir);
                // ssoAbsoluteURL=https://hostname:8080
            }
            if(!ssoAbsoluteURL.endsWith("/") && !ssoBaseURL.startsWith("/"))
            {
                ssoAbsoluteURL += "/";
            }
            ssoAbsoluteURL += ssoBaseURL;
            // ssoAbsoluteURL=https://hostname:8080/portal/SSOServlet
            }
        }
        catch(Exception anyException)
        {
            ssoAbsoluteURL = ssoBaseURL;
            log.Error(anyException, OUR_NAME + ".InitializeSSOAbsoluteURL failed. SSO login redirects in Security Mode 1 and 2 may use http instead of https.");
        }
        log.Debug(OUR_NAME + ".InitializeSSOAbsoluteURL assigned ssoAbsoluteURL: " + ssoAbsoluteURL);
    }

    /**
     * GetInstance must be called after the ConfigVarPack has loaded.
     * Calling GetInstance prior to warmup or prior to loading ConfigVarPack
     * will result in an error.
     *
     * Any errors will be logged and a best attempt is made to continue initialization.
     *
     * @return the singleton instance of SecurityConfigHelper.
     */
    public static SecurityConfigHelper GetInstance()
    {
        return SecurityConfigHelperLatch.singleton;
    }

    /**
     * Concatenate GetSSOBaseURL with the application or secure application URL.
     * The additional virtual directory and endpoint (e.g. /portal/server.pt)
     * that would normally be present is removed.
     *
     * For security mode 1, 2, and 3 the secure URL is used.
     * For security mode 0 the application URL is used.
     *
     * @return the SSO base URL in absolute form, e.g.
     * https://hostname:8080/ssoVirtualDir/SSOServlet
     */
    public String GetSSOAbsoluteURL()
    {
        return ssoAbsoluteURL;
    }
}
