package com.plumtree.portaluiinfrastructure.tags;

import com.plumtree.openfoundation.reflect.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * This class manages variable storage and retrieval for
 * tags.  It allows you to get and set variables for
 * various contexts (session, request, tag, etc...).
 */
public interface ITagState
{
	/**
	 * Sets a variable in memory in the appropriate scope.
	 * <p>
	 * The scope determines who can see this variable and
	 * how long it stays in memory.
	 * <p>
	 * For instance, a variable stored in Tag Scope can only
	 * be seen by children of the current tag and will be
	 * removed from memory when the tag is finished.
	 * <p>
	 * A variable stored in Portlet Request Scope will
	 * be visible to all tags in the same portlet as the
	 * current tag, and will be removed from memory when
	 * the portlet is finished displaying.  Tags in other
	 * portlets on the same page will not be able to see
	 * the variable.
	 * <p>
	 * Standard variables (as opposed to shared variables)
	 * can only be accessed by tags from the same tag
	 * library as the tag that originally stored the
	 * variable.
	 * <p>
	 * Note: Displaying an HTMLElement in a tag and then
	 * caching it so that another tag can add more HTML
	 * to the original tag later is not supported.
	 * HTMLElement trees can be generated and stored for
	 * later use, as long as they are self-contained
	 * trees and used in a read only way.  It is safest
	 * to make clones of a cached HTMLElement tree before
	 * trying to display it again to make sure there are
	 * no threading problems.
	 * 
	 * @param String _strKey The key used to store the data.
	 * The key cannot contain the reserved character '.'.
	 * @param Object _oValue The data to be stored. Note: setting
	 * the value to null is the same as never having set the variable.
	 * @param Scope _scope The scope to store the data in.
	 * @throws XPIllegalArgumentException if the key is null or
	 * contains the reserved character '.', or if the scope
	 * is null or unknown.
	 */
	public void SetVariable(String _strKey, Object _oValue, Scope _scope);

	/**
	 * Sets a variable in memory in the appropriate scope.
	 * <p>
	 * The scope determines who can see this variable and
	 * how long it stays in memory.
	 * <p>
	 * For instance, a variable stored in Tag Scope can only
	 * be seen by children of the current tag and will be
	 * removed from memory when the tag is finished.
	 * <p>
	 * A variable stored in Portlet Request Scope will
	 * be visible to all tags in the same portlet as the
	 * current tag, and will be removed from memory when
	 * the portlet is finished displaying.  Tags in other
	 * portlets on the same page will not be able to see
	 * the variable.
	 * <p>
	 * Shared variables (as opposed to standard variables)
	 * can be accessed by tags from any library.  If
	 * bOwnerEditOnly is set to true when a shared variable
	 * is first stored, then any tag can read the variable,
	 * but only tags from the same library as the tag that
	 * originally stored the variable can edit it (and
	 * replace the value in memory).
	 * <p>
	 * Note: Displaying an HTMLElement in a tag and then
	 * caching it so that another tag can add more HTML
	 * to the original tag later is not supported.
	 * HTMLElement trees can be generated and stored for
	 * later use, as long as they are self-contained
	 * trees and used in a read only way.  It is safest
	 * to make clones of a cached HTMLElement tree before
	 * trying to display it again to make sure there are
	 * no threading problems.
	 * 
	 * @param String _strKey The key used to store the data.
	 * The key cannot contain the reserved character '.'.
	 * @param Object _oValue The data to be stored. Note: setting
	 * the value to null is the same as never having set the variable.
	 * @param Scope _scope The scope to store the data in.
	 * @param bOwnerEditOnly True implies that only tags from
	 * the same tag library should be able to store the value
	 * for this tag.  If this variable has already been saved,
	 * then the original bOwnerEditOnly value will be retained
	 * and this argument will be ignored.
	 * @throws XPIllegalArgumentException if the key is null or
	 * contains the reserved character '.', or if the scope
	 * is null or unknown.
	 * @throws XPIllegalAccessException if the variable is being
	 * set by a tag from a different library than the original
	 * tag that stored the variable with bOwnerEditOnly set to
	 * true.
	 */
	public void SetSharedVariable(String _strKey, Object _oValue, Scope _scope, boolean bOwnerEditOnly);

	/**
	 * Gets a variable from memory in the appropriate scope.
	 * <p>
	 * The scope determines who can see this variable and
	 * how long it stays in memory.
	 * <p>
	 * For instance, a variable can only be retrieved from
	 * Tag Scope if a tag that includes the current tag
	 * added the variable to Tag Scope.
	 * <p>
	 * A variable can only be retrieved from Portlet
	 * Request Scope if another tag in the portlet stored
	 * the variable in memory.
	 * <p>
	 * Standard variables (as opposed to shared variables)
	 * can only be accessed by tags from the same tag
	 * library as the tag that originally stored the
	 * variable.
	 * <p>
	 * Note: Retrieving an HTMLElement from memory that
	 * has already been displayed in another tag and then
	 * adding more HTML to the original tag is not
	 * supported.  HTMLElement trees can be retrieved from
	 * memory and re-used, as long as they are not modified.
	 * The safest way to do this is to make a clone of the
	 * cached HTMLElement tree before trying to display it
	 * again to make sure there are no threading problems.
	 * 
	 * @param String _strKey The key used to store the data.
	 * The key cannot contain the reserved character '.'.
	 * @param Scope _scope The scope used to store the data.
	 * @return Object The requested data. Null if the variable
	 * has never been set.
	 * @throws XPIllegalArgumentException if the key is null or
	 * contains the reserved character '.', or if the scope
	 * is null or unknown.
	 */
	public Object GetVariable(String _strKey, Scope _scope);

	/**
	 * Gets a shared variable from memory in the appropriate
	 * scope.
	 * <p>
	 * The scope determines who can see this variable and
	 * how long it stays in memory.
	 * <p>
	 * For instance, a variable can only be retrieved from
	 * Tag Scope if a tag that includes the current tag
	 * added the variable to Tag Scope.
	 * <p>
	 * A variable can only be retrieved from Portlet
	 * Request Scope if another tag in the portlet stored
	 * the variable in memory.
	 * <p>
	 * Shared variables can be accessed by tags from any
	 * tag library (as opposed to standard variables, which
	 * can only be accessed by tags from the same tag
	 * library).
	 * <p>
	 * Note: Retrieving an HTMLElement from memory that
	 * has already been displayed in another tag and then
	 * adding more HTML to the original tag is not
	 * supported.  HTMLElement trees can be retrieved from
	 * memory and re-used, as long as they are not modified.
	 * The safest way to do this is to make a clone of the
	 * cached HTMLElement tree before trying to display it
	 * again to make sure there are no threading problems.
	 * 
	 * @param String _strKey The key used to store the data.
	 * The key cannot contain the reserved character '.'.
	 * @param Scope _scope The scope used to store the data.
	 * @return Object The requested data. Null if the variable
	 * has never been set.
	 * @throws XPIllegalArgumentException if the key is null or
	 * contains the reserved character '.', or if the scope
	 * is null or unknown.
	 */
	public Object GetSharedVariable(String _strKey, Scope _scope);

	/**
	 * Gets a variable from memory from all scopes.
	 * <p>
	 * The scope determines who can see this variable and
	 * how long it stays in memory.
	 * <p>
	 * For instance, a variable can only be retrieved from
	 * Tag Scope if a tag that includes the current tag
	 * added the variable to Tag Scope.
	 * <p>
	 * A variable can only be retrieved from Portlet
	 * Request Scope if another tag in the portlet stored
	 * the variable in memory.
	 * <p>
	 * Standard variables (as opposed to shared variables)
	 * can only be accessed by tags from the same tag
	 * library as the tag that originally stored the
	 * variable.
	 * <p>
	 * This method will first look for standard variables,
	 * and if not found will then look for shared variables.
	 * <p>
	 * Note: Retrieving an HTMLElement from memory that
	 * has already been displayed in another tag and then
	 * adding more HTML to the original tag is not
	 * supported.  HTMLElement trees can be retrieved from
	 * memory and re-used, as long as they are not modified.
	 * The safest way to do this is to make a clone of the
	 * cached HTMLElement tree before trying to display it
	 * again to make sure there are no threading problems.
	 * 
	 * @param String _strKey The key used to store the data.
	 * @return Object The requested data. Null if the variable
	 * has never been set.
	 * @throws XPIllegalArgumentException if the key is null.
	 */
	public Object GetAnyVariable(String _strKey);
	
	/**********************
	 * Data Object Methods*
	 **********************/

	/**
	 * Return the Application for use in storing data
	 * on the HTTP Application.
	 * <p>
	 * Most developers should use the Get/SetVariable methods instead.
	 *
	 * @return The application
	 */
	public IApplication GetApplication();

	/**
	 * Returns the subsession that can be used to store data
	 * on the HTTP Session.
	 *<p>
	 * This SubSession is cleared of all data on logout.
	 * <p>
	 * Most developers should use the Get/SetVariable methods instead.
	 *
	 * @return ISessionManager The subsession that can be used to store
	 * data on the HTTP Session.
	 */
	public ISessionManager GetSubSession();

	/**
	 * Returns the persistent subsession that can be used to store data on
	 * the HTTP Session for use across multiple login/logout cycles.
	 *<p>
	 * This SubSession is not cleared of its data on user logout, so be sure not
	 * to cache anything on this session that could be considered a security risk
	 * if it was leaked to another user.
	 * <p>
	 * Most developers should use the normal subsession (GetSubSession()) for HTTP
	 * Session data storage, and the Get/SetVariable methods instead.
	 * <p>
	 * @deprecated The Persistent Session is no longer available in tags.  The normal Session scope should be used instead.
	 * 
	 * @return ISessionManager The persistent subsession that can be used to store
	 * data on the HTTP Session across multiple login/logout cycles.
	 */
	public ISessionManager GetPersistentSubSession();
	
	public boolean AddCacheEntry(String cacheKey, Object cacheValue, Scope scope, int duration);
	
	public Object GetCacheEntryData(String cacheKey, Scope scope);
	
	public void StoreInParentTagScope(boolean useParent);
}
