package com.plumtree.portaluiinfrastructure.tags.displaytree;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.helper.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This class contains multiple Display Elements.
 * 
 * This is a framework only class and should not be
 * used by external developers.
 * 
 * @author donh
 */
public class CompositeDisplayElement implements IDisplayElement
{
	/**
	 * The child display elements of this element.
	 */
	protected XPArrayList m_DisplayElements = null;

	/**
	 * The immediate parent tag of this element.
	 */
	protected TagDisplayElement m_ParentTag = null;

	/**
	 * The container that this element is stored in.  This
	 * may be an internal child container of the parent tag
	 * element.
	 */
	protected CompositeDisplayElement m_ParentContainer = null;

	/**
	 * Constructor for CompositeDisplayElement.  This should
	 * only be called by the tag framework.
	 * 
	 * @param _parentTag The immediate parent tag of this element.
	 * @param _parentContainer The container that this element is
	 * stored in.  This may be an internal child container of the
	 * parent tag element.
	 */
	public CompositeDisplayElement(TagDisplayElement _parentTag,
		CompositeDisplayElement _parentContainer)
	{
		// Note: We only need to initialize this array if this
		// tag will contain child Display Elements.  We could
		// do a lazy initialization of this array if necessary.
		m_DisplayElements = new XPArrayList();
		m_ParentTag = _parentTag;
		m_ParentContainer = _parentContainer;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.displaytree.IDisplayElement#Display(TagVisitor)
	 */
	public HTMLElement Display(TagVisitor _tagVisitor)
	{
		HTMLElementCollection result = new HTMLElementCollection();

		IXPEnumerator elements = m_DisplayElements.GetEnumerator();

		while (elements.MoveNext())
		{
			result.AddInnerHTMLElement(((IDisplayElement)elements.GetCurrent()).Display(_tagVisitor));
		}

		return result;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.displaytree.IDisplayElement#Release()
	 */
	public void Release()
	{
		IXPEnumerator elements = m_DisplayElements.GetEnumerator();

		while (elements.MoveNext())
		{
			((IDisplayElement)elements.GetCurrent()).Release();
		}
	}

	/**
	 * Add a child display element to this container.
	 * 
	 * @param IDisplayElement _child The child to add
	 */
	public void AddChildElement(IDisplayElement _child)
	{
		m_DisplayElements.Add(_child);
	}

	/**
	 * Get the parent tag of this container.
	 * 
	 * The parent tag is the next TagDisplayElement
	 * above this element in the display tree.
	 * 
	 * @return CompositeDisplayElement The parent tag.
	 */
	public TagDisplayElement GetParentTag()
	{
		return m_ParentTag;
	}

	/**
	 * Get the parent container of this container.
	 * 
	 * The parent container is the next CompositeDisplayElement
	 * above this element in the display tree, not the next
	 * element (which will often be a TagDisplayElement), but
	 * rather the composite element above that one.
	 * 
	 * @return CompositeDisplayElement The parent container.
	 */
	public CompositeDisplayElement GetParentContainer()
	{
		return m_ParentContainer;
	}
	
	/**
	 * Returns the unprocessed display elements
	 * Only used for Tag definitions.  
	 * @return
	 */
	public XPArrayList GetDisplayElements()
	{
		return m_DisplayElements;
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.displaytree.IDisplayElement#HasChildElements(ITagMetaData[], Object[], TagLibraryMetaData, int)
	 */
	public boolean HasChildElements(ITagMetaData[] _childTags, Object[] _results, TagLibraryMetaData _library, int _nCurrentDepth)
	{
		IXPEnumerator children = m_DisplayElements.GetEnumerator();

		boolean bCompleted = false;

		while (children.MoveNext() && (!bCompleted))
		{
			IDisplayElement child = (IDisplayElement) children.GetCurrent();

			// This allows us to short-circuit the search if we have found
			// all of the child tags.
			bCompleted = child.HasChildElements(_childTags, _results, _library, _nCurrentDepth);
		}

		return bCompleted;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.displaytree.IDisplayElement#HasParentElement(ITagMetaData, TagLibraryMetaData)
	 */
	public int HasParentElement(ITagMetaData _parentTag, TagLibraryMetaData _library)
	{
		if (null == m_ParentTag)
		{
			return 0;
		}

		// Don't change the depth information here.  We only count PT Tags.
		return m_ParentTag.HasParentElement(_parentTag, _library);
	}
}
