package com.plumtree.portaluiinfrastructure.tags.helper;

import com.plumtree.openfoundation.reflect.*;
import com.plumtree.openfoundation.util.*;

/**
 * This class wraps shared state variables and
 * tracking information to provide edit access
 * checks.
 * 
 * @author DonH
 */
public class SharedStateVariable
{
	/**
	 * The actual value to be stored.
	 */
	protected Object m_oValue;

	/**
	 * The name of the library that has edit access
	 * to this object.  If this is null, it means
	 * all libraries have edit access.
	 */
	protected String m_strOwningLibraryName;

	/**
	 * Construct a new SharedStateVariable for an
	 * object with no edit access restrictions.
	 * 
	 * @param _oValue The actual value to be stored.
	 */
	public SharedStateVariable(Object _oValue)
	{
		m_strOwningLibraryName = null;
		m_oValue = _oValue;
	}

	/**
	 * Construct a new SharedStateVariable for an
	 * object with edit access restricted to the
	 * owning library only.
	 * 
	 * @param __strOwningLibraryName The name of the
	 * owning library with edit access.
	 * @param _oValue The actual value to be stored.
	 */
	public SharedStateVariable(String _strOwningLibraryName, Object _oValue)
	{
		m_strOwningLibraryName = _strOwningLibraryName;
		m_oValue = _oValue;
	}

	/**
	 * Check whether or not the specified library
	 * has edit access to this object.
	 * 
	 * @param _strCurrentLibraryName The name of the
	 * library to check security for.
	 * @param _strKey The key used to access this variable.
	 * This is used for error reporting purposes only.
	 * @param __strCurrentTagName The name of the current tag.
	 * This is used for error reporting purposes only.
	 * @return true implies that the library has edit access
	 * @throws XPIllegalAccessException if the specified library
	 * does not have edit access to this variable.
	 */
	public void CheckHasEditAccess(String _strCurrentLibraryName,
		String _strKey, String _strCurrentTagName)
	{
		if ((null != m_strOwningLibraryName) &&
			(0 != XPStringUtility.Compare(m_strOwningLibraryName, _strCurrentLibraryName)))
		{
			// Does not have edit access to this variable
			throw new XPIllegalAccessException
				("Illegal attempt to replace the shared state variable " + _strKey +
				" by tag " + _strCurrentTagName + ".  The library " + _strCurrentLibraryName +
				" does not have edit access.  Access restricted to the creating library: " +
				m_strOwningLibraryName + ".");
		}
	}

	/**
	 * Check whether or not access is restricted
	 * to a particular library in this shared state
	 * variable.
	 * 
	 * @return boolean Whether or not access is restricted.
	 */
	public boolean IsAccessRestricted()
	{
		return (null != m_strOwningLibraryName);
	}

	/**
	 * Gets the value of the stored object.
	 * 
	 * @return The value of the stored object.
	 */
	public Object GetValue()
	{
		return m_oValue;
	}

	/**
	 * Updates the value of the stored object.
	 * 
	 * @param The new value for the stored object.
	 */
	public void SetValue(Object _value)
	{
		m_oValue = _value;
	}
}
