package com.plumtree.portaluiinfrastructure.tags.helper;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This class manages variable storage and retrieval for
 * tags.  It allows you to get and set variables for
 * various contexts (session, request, tag, etc...).
 */
public class TagState implements ITagState
{
	/** The tag library used to store the cache map. Use a non-existant tag lib to avoid conflicts with regular tag variables. Public so that it is accessible from unit tests **/
	public static final String CACHE_LIB = "TAGCACHE";
	/** Name of the cache map entry. Public so that it is accessible from unit tests  **/
	public static final String CACHE_MAPENTRY_NAME = "$CacheMap$";
	/** Lock for creating the cache map entry **/
	private static final Object CACHEMAP_LOCK = new Object();

	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		"com.plumtree.portaluiinfrastructure.tags.helper.TagState");

	/**
	 * This key is used to store an http request level list that
	 * stores whether or not a particular class of tags has been
	 * displayed on the page yet.
	 */
	public static final String SHARED_JS_VAR_LIST = "DisplayedTagClasses";

	/**
	 * This key is used to store an http request level HTMLElementCollection
	 * that contains all of the shared javascript for the tags in the HTTP
	 * request.
	 */
	public static final String SHARED_JS_COLLECTION = "SharedJSCollection";

	/**
	 * This key is used to store an http request level Boolean that marks
	 * whether or not the current shared javascript collection for this HTTP
	 * request has been retrieved for displayed or not.
	 */
	public static final String SHARED_JS_COLLECTION_DISPLAYED = "SharedJSCollectionDisplayed";

	/**
	 * The separator between library and tag name in the combined key.
	 */
	protected static final String COMBINED_KEY_SEPARATOR = ".";

	/**
	 * This key is used to store HTTP request data collections on
	 * the session using the request as a key.
	 */
	public static final String HTTP_REQUEST_DATA_KEY = "httprequestdata";

	/**
	 * The Tag Stack.  This keeps track of which Tags
	 * have been entered. It is a list of ATags.
	 */
	protected XPArrayList m_TagStack;

	/**
	 * The Tag Var Stack.  This keeps track of what
	 * Data has been stored by which tag in tag scope.
	 * It is a list of XPArrayLists.
	 */
	protected XPArrayList m_TagVarStack;

	/**
	 * The index into the top of the Tag Stack.
	 */
	protected int m_nTagStackTop;

	/**
	 * The Tag Data container.  This cannot be re-used
	 * between portlet requests and must be reset.
	 */
	protected XPHashtable m_TagScopeData;

	/**
	 * The Portlet Request Data container.  This
	 * cannot be re-used between portlet requests
	 * and must be reset.
	 */
	protected XPHashtable m_PortletRequestScopeData;

	/**
	 * The HTTP Request Data container.  This hashtable
	 * is stored in the session using the request as a
	 * key and is cached in the TagState object.
	 */
	protected XPHashtable m_HTTPRequestScopeData;

	/**
	 * The HTTP Application.
	 */
	protected IApplication m_Application;

	/**
	 * The sub-session.
	 */
	protected ISessionManager m_Session;

	/**
	 * The persistent sub-session.
	 */
	protected ISessionManager m_PersistentSession;

	/**
	 * The current tag.
	 */
	protected ATag m_CurrentTag = null;
	/** Whether to use parent tag scope or tag scope inside current tag when setting variables **/
	protected boolean m_useParentTagScope = false;

	/**
	 * This static variable can be used to supply a pre-populated
	 * hashtable for the portlet request data.  This will be used
	 * during TagState construction only.  This should only be used
	 * in unit tests and not in production code.
	 */
	public static XPHashtable TEST_PORTLET_REQUEST_DATA;

	/**
	 * Creates a TagState object.
	 * 
	 * The TagState object contains portlet request
	 * specific data and must either be reset between
	 * requests or re-created.
	 * 
	 * @param _application The HTTP Application
	 * @param _session The sub-session
	 * @param _persistentSession The persistent sub-session
	 */
	public TagState(IApplication _application, ISessionManager _session,
		ISessionManager _persistentSession, XPHashtable _httpRequestData)
	{
		m_TagStack = new XPArrayList();
		m_TagVarStack = new XPArrayList();
		m_nTagStackTop = -1;
		m_TagScopeData = new XPHashtable();

		// If we are in unit test mode, use the supplied hash table if it exists
		if (TagState.TEST_PORTLET_REQUEST_DATA != null)
		{
			m_PortletRequestScopeData = TagState.TEST_PORTLET_REQUEST_DATA;
			// Clear this so unit tests won't overwrite each other.
			TagState.TEST_PORTLET_REQUEST_DATA = null;
		} else
		{
			m_PortletRequestScopeData = new XPHashtable();
		}
		m_HTTPRequestScopeData = _httpRequestData;

		m_Application = _application;
		m_Session = _session;
		m_PersistentSession = _persistentSession;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ITagState#SetVariable(String, Object, Scope)
	 */
	public void SetVariable(String _strKey, Object _oValue, Scope _scope)
	{
		CheckValidVariableName(_strKey);

		String strLibName = m_CurrentTag.GetTagMetaDataHelper().GetTagLibraryMetaData().GetName();

		// This stores the variable with a key composed of the lib name
		// and key, meaning that this var is unique for this library.
		// Another library could set the same variable and they would
		// not conflict.
		InternalSetVariable(strLibName, _strKey, _oValue, _scope);
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ITagState#SetSharedVariable(String, Object, Scope, boolean)
	 */
	public void SetSharedVariable(String _strKey, Object _oValue, Scope _scope, boolean bOwnerEditOnly)
	{
		CheckValidVariableName(_strKey);

		String strLibName = m_CurrentTag.GetTagMetaDataHelper().GetTagLibraryMetaData().GetName();

		// Check for appropriate edit access
		SharedStateVariable sharedVariable = (SharedStateVariable) InternalGetVariable(null, _strKey, _scope);

		if (null != sharedVariable)
		{
			// This will throw an exception if this library does not
			// have edit access.
			sharedVariable.CheckHasEditAccess(strLibName, _strKey, m_CurrentTag.GetName());

			// Update the pre-existing variable with the new value.
			sharedVariable.SetValue(_oValue);
		} else
		{
			// Create a new variable.
			if (bOwnerEditOnly)
			{
				sharedVariable = new SharedStateVariable(strLibName, _oValue);
			} else
			{
				sharedVariable = new SharedStateVariable(_oValue);
			}
		}

		if (null == _oValue)
		{
			// Setting a variable to null means that we should remove the
			// variable from the appropriate collection.
			sharedVariable = null;
		}

		// This stores the variable with a key composed of just the
		// key, meaning that this var is shared and can be accessed
		// by other libraries.
		InternalSetVariable(null, _strKey, sharedVariable, _scope);
	}

	/**
	 * This method checks that the variable name does not contain
	 * any reserved characters (".").
	 * 
	 * @param String _strVariableName The variable name to be checked.
	 * @throws XPIllegalArgumentException if the variable name contains
	 * illegal characters (".").
	 */
	protected void CheckValidVariableName(String _strVariableName)
	{
		// NOTE: This method is called approximately 2500 times for
		// an empty mypage, so it's pretty expensive.
		if (XPStringUtility.IndexOf(_strVariableName, TagHelper.ATTRIBUTE_VARIABLE_SEPARATOR) >= 0)
		{
			throw new XPIllegalArgumentException("Illegal variable name " +
				_strVariableName + ".  Contains illegal character \".\".");
		}
	}

	/**
	 * Helper method to set variables in memory.
	 * 
	 * @param _strLibrary The current library.  May be null.
	 * @param _strKey The key used to store the variable.
	 * @param _oValue The variable.
	 * @param _scope The scope used to store the variable.
	 */
	protected void InternalSetVariable(String _strLibrary, String _strKey, Object _oValue, Scope _scope)
	{
		if (null == _strKey)
		{
			throw new XPIllegalArgumentException("Null key argument in tag " + GetTagName() +
				" while setting variable to value " + _oValue + ".  Scope: " + _scope);
		}

		if (null == _scope)
		{
			throw new XPIllegalArgumentException("Null scope argument in tag " + GetTagName() +
				" while setting variable " + _strKey + " to value " + _oValue + ".");
		}

		String strCombinedKey;

		if (null != _strLibrary)
		{
			strCombinedKey = _strLibrary + COMBINED_KEY_SEPARATOR + _strKey;
		} else
		{
			strCombinedKey = COMBINED_KEY_SEPARATOR + _strKey;
		}

		if (null == _oValue)
		{
			// Setting a variable to null means that we should remove the
			// variable from the appropriate collection.
			InternalRemoveVariable(strCombinedKey, _scope);

			return;
		}

		if (Scope.TAG == _scope )
		{
			if (!m_TagScopeData.ContainsKey(strCombinedKey))
			{
				// use current tag stack index if scope is tag, otherwise get the previous index unless we are the top index already 
				int tagStackIndex = (!m_useParentTagScope) ? m_nTagStackTop : (m_nTagStackTop >= 1) ? (m_nTagStackTop-1) : 0;
				// TODO: check if m_nTagStackTop >= m_TagStack.GetSize() or -1?  (Both invalid)
				XPArrayList varList = (XPArrayList) m_TagVarStack.GetElement(tagStackIndex);

				if (null == varList)
				{
					// Create a new list if this is the first variable
					// associated with this tag
					varList = new XPArrayList();
					m_TagVarStack.SetElement(tagStackIndex, varList);
				}

				// Associate the new tag scope variable with the current tag 
				varList.Add(strCombinedKey);
			}

			m_TagScopeData.PutElement(strCombinedKey, _oValue);
		} else if (Scope.PORTLET_REQUEST == _scope)
		{
			m_PortletRequestScopeData.PutElement(strCombinedKey, _oValue);
		} else if (Scope.HTTP_REQUEST == _scope)
		{
			m_HTTPRequestScopeData.PutElement(strCombinedKey, _oValue);
		} else if (Scope.SESSION == _scope)
		{
			m_Session.SetAttribute(strCombinedKey, _oValue);
		} else if (Scope.PERSISTENT_SESSION == _scope)
		{
			m_PersistentSession.SetAttribute(strCombinedKey, _oValue);
		} else if (Scope.APPLICATION == _scope)
		{
			m_Application.GetCachingManager().SetEntry(strCombinedKey, _oValue);
		} else
		{
			// Unknown scope constant
			throw new XPIllegalArgumentException("Unknown scope argument in tag " + GetTagName() +
				" while setting variable " + _strKey + " to value " + _oValue + ".  Scope: " + _scope);
		}
	}

	/**
	 * This method removes a variable from the underlying collection.
	 * 
	 * @param _strCombinedKey The key used to access the variable.
	 * @param _scope The scope used to store the variable. Assumed to be non-null.
	 */
	protected void InternalRemoveVariable(String _strCombinedKey, Scope _scope)
	{
		if (Scope.TAG == _scope)
		{
			if (!m_TagScopeData.ContainsKey(_strCombinedKey))
			{
				// TODO: check if m_nTagStackTop >= m_TagStack.GetSize() or -1?  (Both invalid)
				XPArrayList varList = (XPArrayList) m_TagVarStack.GetElement(m_nTagStackTop);
	
				if ((null == varList) || (!varList.Contains(_strCombinedKey)))
				{
					// Ignore - variable not present, so does not need to be removed.
				} else
				{
					// Remove the variable
					varList.Remove(_strCombinedKey);
				}
			}

			m_TagScopeData.RemoveElement(_strCombinedKey);
		} else if (Scope.PORTLET_REQUEST == _scope)
		{
			m_PortletRequestScopeData.RemoveElement(_strCombinedKey);
		} else if (Scope.HTTP_REQUEST == _scope)
		{
			m_HTTPRequestScopeData.RemoveElement(_strCombinedKey);
		} else if (Scope.SESSION == _scope)
		{
			m_Session.RemoveAttribute(_strCombinedKey);
		} else if (Scope.PERSISTENT_SESSION == _scope)
		{
			m_PersistentSession.RemoveAttribute(_strCombinedKey);
		} else if (Scope.APPLICATION == _scope)
		{
			m_Application.GetCachingManager().RemoveEntry(_strCombinedKey);
		} else
		{
			// Unknown scope constant
			throw new XPIllegalArgumentException("Unknown scope argument in tag " + GetTagName() +
				" while attempting to remove variable " + _strCombinedKey + ".  Scope: " + _scope);
		}
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ITagState#GetVariable(String, Scope)
	 */
	public Object GetVariable(String _strKey, Scope _scope)
	{
		String strLibName = m_CurrentTag.GetTagMetaDataHelper().GetTagLibraryMetaData().GetName();

		return InternalGetVariable(strLibName, _strKey, _scope);
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ITagState#GetSharedVariable(String, Scope)
	 */
	public Object GetSharedVariable(String _strKey, Scope _scope)
	{
		Object oSharedResult = InternalGetVariable(null, _strKey, _scope);

		SharedStateVariable sharedResult;

		try
		{
			sharedResult = (SharedStateVariable) oSharedResult;
		} catch (Exception e)
		{
			throw new XPException("Unable to convert value stored in memory to SharedStateVariable object.", e);
		}

		if (null == sharedResult)
		{
			return null;
		}

		return sharedResult.GetValue();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ITagState#GetAnyVariableFromAllScopes(String, Scope)
	 */
	public Object GetAnyVariable(String _strKey)
	{
		int nIndexOfDot = XPStringUtility.IndexOf(_strKey, '.');
		if(nIndexOfDot > 0)
		{
			String strObjectContainer = XPStringUtility.Substring(_strKey, 0, nIndexOfDot);
			String strRestOfVariable = XPStringUtility.Substring(_strKey, nIndexOfDot+1);

			IDataObject dataObject = (IDataObject) InternalGetAnyVariable(strObjectContainer);
			
			if(dataObject != null)
			{
				return dataObject.GetData(strRestOfVariable);
			}
		}
		else
		{
			Object oResult = null;
			oResult = InternalGetAnyVariable(_strKey);
			if (null != oResult)
			{
				return oResult;
			}
		}

		return null;
	}
	
	/**
	 * Helper method to get variables in memory from any scope.
	 * 
	 * @param _strKey The key used to store the variable.
	 * @return Object the requested variable
	 */
	private Object InternalGetAnyVariable(String _strKey)
	{
		Object oResult;
		Scope[] scopeSearchOrder = Scope.GetScopeSearchOrder();
		
		// Check variables
		for (int i = 0; i < scopeSearchOrder.length; i++)
		{
			oResult = GetVariable(_strKey, scopeSearchOrder[i]);

			if (null != oResult)
			{
				return oResult;
			}
		}

		// Check shared variables
		for (int i = 0; i < scopeSearchOrder.length; i++)
		{
			oResult = GetSharedVariable(_strKey, scopeSearchOrder[i]);

			if (null != oResult)
			{
				return oResult;
			}
		}

		return null;
	}

	/**
	 * Helper method to get variables in memory.
	 * 
	 * @param _strLibrary The current library.  May be null.
	 * @param _strKey The key used to store the variable.
	 * @param _scope The scope used to store the variable.
	 * @return Object the requested variable (may be a SharedStateVariable object that
	 * needs to be processed before being returned to the tag.
	 */
	public Object InternalGetVariable(String _strLibrary, String _strKey, Scope _scope)
	{
		CheckValidVariableName(_strKey);

		if (null == _strKey)
		{
			throw new XPIllegalArgumentException("Null key argument in tag " + GetTagName() +
				" while getting variable.  Scope: " + _scope);
		}

		if (null == _scope)
		{
			throw new XPIllegalArgumentException("Null scope argument in tag " + GetTagName() +
				" while getting variable " + _strKey + ".");
		}

		String strCombinedKey;
		if (null != _strLibrary)
		{
			strCombinedKey = _strLibrary + COMBINED_KEY_SEPARATOR + _strKey;
		} else
		{
			strCombinedKey = COMBINED_KEY_SEPARATOR + _strKey;
		}

		Object oResult;

		if (Scope.TAG == _scope)
		{
			oResult = m_TagScopeData.GetElement(strCombinedKey);
		} else if (Scope.PORTLET_REQUEST == _scope)
		{
			oResult = m_PortletRequestScopeData.GetElement(strCombinedKey);
		} else if (Scope.HTTP_REQUEST == _scope)
		{
			oResult = m_HTTPRequestScopeData.GetElement(strCombinedKey);
		} else if (Scope.SESSION == _scope)
		{
			oResult = m_Session.GetAttribute(strCombinedKey);
		} else if (Scope.PERSISTENT_SESSION == _scope)
		{
			oResult = m_PersistentSession.GetAttribute(strCombinedKey);
		} else if (Scope.APPLICATION == _scope)
		{
			oResult = m_Application.GetCachingManager().GetEntry(strCombinedKey);
		} else
		{
			// Unknown scope constant
			throw new XPIllegalArgumentException("Unknown scope argument in tag " + GetTagName() +
				" while getting variable " + _strKey + ".  Scope: " + _scope);
		}

		return oResult;
	}
	
	/**
	 * Add a cached tag data entry
	 * Cache entries are stored in the same scope as the other tag variables, but cache entries are prefixed with a special
	 * string.
	 * 
	 * @param cacheKey
	 * @param cacheValue
	 * @param scope 
	 * @param duration number of minutes the cache entry is valid
	 * @return
	 */
	public boolean AddCacheEntry(String cacheKey, Object cacheValue, Scope scope, int duration)
	{
		// make sure we still need to add the entry by checking if the entry already exists and it is not expired
		TagCacheEntry existing = InternalGetCacheEntry(cacheKey, scope);
		if ( existing == null || existing.IsEntryExpired() ) {
			XPDateTime now = new XPDateTime();
			long entryExpirationTime =  now.GetTimeInMilliSeconds() + duration * 1000 * 60;
			// create cache entry
			TagCacheEntry entry = new TagCacheEntry(cacheValue, new XPDateTime(entryExpirationTime));
			
			//String cacheEntryKey = GetCacheMetaDataEntryKey(cacheKey); // create the cache entry string
			XPHashtable cacheMap = InternalGetCacheMap(scope);
			if ( cacheMap == null ) {
				synchronized ( CACHEMAP_LOCK ) {
					// want a synchronized hashtable
					if ( cacheMap == null ) {
						cacheMap = XPHashtable.GetSynchronized(new XPHashtable(2000)); // set initial size to something big so that the hashtable doesn't keep expanding
					}
					InternalSetVariable(CACHE_LIB, CACHE_MAPENTRY_NAME, cacheMap, scope);
				}
			}
			cacheMap.PutElement(cacheKey, entry);
			
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Get tag data entry from cache
	 * @param cacheKey
	 * @param scope Scope.APPLICATION or Scope.Session
	 * @return The cached entry or null if no entry was found or the entry expired.
	 */
	public Object GetCacheEntryData(String cacheKey, Scope scope)
	{
		if ( !Scope.APPLICATION.equals(scope) && !Scope.SESSION.equals(scope) ) {
			throw new XPIllegalArgumentException("Caching only supported on Application and Session scope. Invalid scope: "+scope.toString());
		}
		
		Object retData = null;
	
		TagCacheEntry cacheEntry = InternalGetCacheEntry(cacheKey, scope);
		
		if ( cacheEntry == null || cacheEntry.IsEntryExpired() ) { // if no entry found in cache, or entry expired, return null
			return null;
		} else {
			retData = cacheEntry.GetCacheData(); // otherwise return cache value
		}
			
		return retData;
	}
	/**
	 * Internal method to retrieve the Cache map object
	 * @param scope
	 * @return
	 */
	protected XPHashtable InternalGetCacheMap(Scope scope)
	{
		XPHashtable cacheMap = (XPHashtable)InternalGetVariable(CACHE_LIB, CACHE_MAPENTRY_NAME, scope);
		return cacheMap;
	}
	/**
	 * Gets cache entry from the cache map
	 * @param cacheKey
	 * @param scope
	 * @return
	 */
	protected TagCacheEntry InternalGetCacheEntry(String cacheKey, Scope scope)
	{
		XPHashtable cacheMap = InternalGetCacheMap(scope);
		if ( null == cacheMap ) { // map hasn't been created yet, just return null. Map is created in AddCacheEntry
			return null;
		}
		Object entry = cacheMap.GetElement(cacheKey);
		if ( entry == null ) {
			return null;
		}
		if ( entry instanceof TagCacheEntry ) {
			return (TagCacheEntry)entry;
		} else {
			throw new XPIllegalStateException("Invalid cache entry metadata type found with name: "+cacheKey+". Entry: "+entry);
		}
	}
	
	
	/**********************
	 * Data Object Methods*
	 **********************/

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ITagState#GetApplication()
	 */
	public IApplication GetApplication()
	{
		return m_Application;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ITagState#GetSubSession()
	 */
	public ISessionManager GetSubSession()
	{
		return m_Session;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ITagState#GetPersistentSubSession()
	 */
	public ISessionManager GetPersistentSubSession()
	{
		return m_PersistentSession;
	}

	/********************
	 * Framework Methods*
	 ********************/

	/**
	 * The tag container calls this method every time a
	 * new tag is displayed.
	 */
	public void EnterTag(ATag _currentTag)
	{
		m_CurrentTag = _currentTag;
		m_TagStack.Add(_currentTag);

		m_nTagStackTop++;
		m_TagVarStack.Add(null);
	}
	/**
	 * The tag container calls this method when a tag is
	 * finished displaying.  This will clean up data
	 * stored in the tag scope.
	 */
	public void ExitTag()
	{
		XPArrayList varList = (XPArrayList) m_TagVarStack.GetElement(m_nTagStackTop);

		if (null != varList)
		{
			IXPEnumerator vars = varList.GetEnumerator();
			while (vars.MoveNext())
			{
				m_TagScopeData.RemoveElement(vars.GetCurrent());				
			}
		}

		// Error checking
		if (m_nTagStackTop != (m_TagVarStack.GetSize() - 1))
		{
			log.Error("Invalid TagState while calling ExitTag().  TagStackTop " +
				m_nTagStackTop + " not equal to top of stack " +
				(m_TagVarStack.GetSize() - 1));

			// Reset to top of stack.
			m_nTagStackTop = m_TagVarStack.GetSize() - 1;
		}

		m_TagVarStack.RemoveAt(m_nTagStackTop);

		m_TagStack.RemoveAt(m_nTagStackTop);

		m_nTagStackTop--;

		// Restore the current tag variable to the previous tag
		if (m_nTagStackTop >= 0)
		{
			m_CurrentTag = (ATag) m_TagStack.GetElement(m_nTagStackTop);
		} else
		{
			m_CurrentTag = null;
		}
	}

	/**
	 * This method is called at the end of every HTTP request to
	 * reset the page request data.  This is a framework method
	 * and should not be called by tag developers.
	 * 
	 * @param ISessionManager the current session manager.
	 */
	public static void ResetPageRequestData(AActivitySpace _aSpace)
	{
		// SE: Bug 61632 - Fix for Samsung. Backport of DonH's fix - CL 258931.
		// TODO: This method shouldn't be static.
		// TODO: Bug#61730 This method isn't really relevant anymore,
		// except possibly for unit tests. Maybe it should be removed.

		IXPRequest request = _aSpace.GetCurrentHTTPRequest();

		// This can be null if the page display operation times out.
		if (request != null)
		{
			// Remove the current request from the HTTP request data set
			request.SetAttribute(HTTP_REQUEST_DATA_KEY, null);
		}
	}

	/**
	 * The tag container calls this method to reset the
	 * portlet request data for each portlet on a page
	 * and to verify that the TagState is in the correct
	 * state after having finished displaying all of the
	 * tags in the current portlet request.
	 */
	public void ResetPortletRequestData()
	{
		m_PortletRequestScopeData = new XPHashtable();

		// This hashtable should be empty.
		if (m_TagScopeData.GetSize() > 0)
		{
			log.Error("Tag Scope Data not empty at end of portlet request.");

			// Clear it if it is not empty
			m_TagScopeData.Clear();
		}

		if ((-1 != m_nTagStackTop) ||
			(0 != m_TagStack.GetSize()) ||
			(0 != m_TagVarStack.GetSize()))
		{
			log.Error("Tag Stack not empty at end of portlet request.");

			m_nTagStackTop = -1;
			m_TagStack.Clear();
			m_TagVarStack.Clear();
		}

		m_CurrentTag = null;
	}

	/**
	 * This method extracts the HTTP request data from the
	 * Activity Space.
	 * 
	 * @param aSpace The current AActivitySpace.
	 * @return the current HTTP Request data object.
	 */
	public static XPHashtable GetHTTPRequestData(AActivitySpace aSpace)
	{
		// SE: Bug 61632 - Fix for Samsung. Backport of DonH's fix - CL 258931.
		// TODO: Bug#61730 This no longer requires the session manager.
		// Refactor this to remove it from here, and other places where it
		// can be removed, like the GetSharedJavascriptCollection method.

		// Get the HTTP Request Scope Data
		Object oRequestDataStorage = aSpace.GetCurrentHTTPRequest().GetAttribute(HTTP_REQUEST_DATA_KEY);

		if (null == oRequestDataStorage)
		{
			aSpace.GetCurrentHTTPRequest().SetAttribute(HTTP_REQUEST_DATA_KEY, new XPHashtable());

			return GetHTTPRequestData(aSpace);
		}

		return (XPHashtable) oRequestDataStorage;
	}

	/**
	 * This method gets the shared JavaScript collection for this HTTP request,
	 * and initializes it if it has not been initialized yet.
	 * 
	 * @param asOwner The current Activity Space.
	 * @return The shared JavaScript collection for this HTTP request.
	 */
	public static HTMLElementCollection GetSharedJavascriptCollection(AActivitySpace asOwner)
	{
		XPHashtable HTTPRequestData = TagState.GetHTTPRequestData(asOwner);

		// Check and set if SharedJSCollection has been called before
		// NOTE: What happens if a custom tag overwrites this variable or someone stores
		// some random request data here using the variable tag?
		Object oSharedJSCollection = HTTPRequestData.GetElement(TagState.SHARED_JS_COLLECTION);

		if (null == oSharedJSCollection)
		{
			HTTPRequestData.PutElement(TagState.SHARED_JS_COLLECTION, new HTMLElementCollection());

			return GetSharedJavascriptCollection(asOwner);
		}

		return (HTMLElementCollection) oSharedJSCollection;
	}

	/**
	 * This method marks the shared JavaScript collection for this HTTP request as
	 * retrieved for display.
	 * 
	 * @param asOwner The current Activity Space.
	 * @throws XPException if this method is called more than once in an HTTP request.
	 */
	public static void SetSharedJavascriptCollectionAsDisplayed(AActivitySpace asOwner)
	{
		XPHashtable HTTPRequestData = TagState.GetHTTPRequestData(asOwner);

		// Check and set if SharedJSCollection has been called before
		// NOTE: What happens if a custom tag overwrites this variable or someone stores
		// some random request data here using the variable tag?
		Object oSharedJSCollectionDisplayed = HTTPRequestData.GetElement(TagState.SHARED_JS_COLLECTION_DISPLAYED);

		if (null != oSharedJSCollectionDisplayed)
		{
			throw new XPException("Tag Markup Shared JavaScript Collection already marked for display.  " +
				"The collection should only be displayed once per HTTP request.");
		}

		HTTPRequestData.PutElement(TagState.SHARED_JS_COLLECTION, new Boolean(true));

		return;
	}

	/*****************
	 * Helper Methods*
	 *****************/

	/**
	 * Get the name of the current tag.
	 * 
	 * @return String the name of the current tag
	 */
	protected String GetTagName()
	{
		if (null != m_CurrentTag)
		{
			return m_CurrentTag.GetName();
		}

		return "";
	}
	
	public void StoreInParentTagScope(boolean useParent)
	{
		m_useParentTagScope = useParent;
	}
}
