package com.plumtree.portaluiinfrastructure.tags.manager;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;

public interface ITagLibManager {

	/**
	 * Create the requested Tag. (Use this method where the library version is specified
	 * where possible as it is more efficient.)
	 *
	 * This method should not be called by developers.  It is only for
	 * the Tag Framework.
	 *
	 * @param String _strTagName The name of the requested tag
	 * @param String _strLibraryName The name of the library that
	 * 		contains the requested tag
	 * @param _dLibraryVersion The version of the library that contains
	 * 		the requested tag
	 * @return ATag The requested tag, null on failure
	 */
	public ATag GetTag(String _strTagName, String _strLibraryName,
			double _dLibraryVersion);

	/**
	 * Create the requested Tag. (Avoid this method where the library version is 
	 * not specified where possible as it is less efficient.)
	 *
	 * This method should not be called by developers.  It is only for
	 * the Tag Framework.
	 *
	 * @param String _strTagName The name of the requested tag
	 * @param String _strLibraryName The name of the library that
	 * 		contains the requested tag
	 * @return ATag The requested tag, null on failure
	 */
	public ATag GetTag(String _strTagName, String _strLibraryName);

	/**
	 * Return true if the tag specified by the parameters exists otherwise return false.
	 * Use this method specifying the library version rather than not where possible
	 * as it is much more efficient.
	 * 
	 * @param String _strTagName The name of the requested tag
	 * @param String _strLibraryName The name of the library that
	 * 		contains the requested tag
	 * @param _dLibraryVersion The version of the library that contains
	 * 		the requested tag
	 * @return true if the tag exists otherwise false.
	 */
	public boolean HasTag(String _strTagName, String _strLibraryName,
			double _dLibraryVersion);

	/**
	 * Return true if the tag specified by the parameters exists otherwise return false.
	 * Use the method specifying the library version rather than this where possible
	 * as it is much more efficient.
	 * 
	 * @param String _strTagName The name of the requested tag
	 * @param String _strLibraryName The name of the library that
	 * 		contains the requested tag
	 * @return true if the tag exists otherwise false.
	 */
	public boolean HasTag(String _strTagName, String _strLibraryName);

	/**
	 * This method initializes the TagLibManager and should only be called by
	 * the Application Warmup.  This method can only be called once.
	 *
	 * @param strLibDir The file directory where jars and dlls can be found
	 * @param strConfigFile the name of the config file
	 */
	public void LoadClasses(String strLibDir, String strConfigFile);

	/**
	 * This method initializes the TagLibManager with custom tags
	 * to overwrite the original ones and should only be called by the
	 * the Application Warmup.
	 *
	 * @param strLibDir The file directory where jars and dlls can be found
	 * @param strConfigFile the name of the custom config file
	 */
	public void LoadCustomClasses(String strLibDir, String strConfigFile);

	/**
	 * Return a data structure holding name and version number of all libraries
	 * loaded.
	 * 
	 * TODO this may not be the form we really want to return this data in
	 * 		that's still being figured out - CVN
	 */
	public XPHashtable GetLibraries();

	/**
	 * Return library meta data on a specific library and version.
	 * 
	 * TODO this may not be the form we really want to return this data in
	 * 		that's still being figured out - CVN
	 */
	public TagLibraryMetaData GetTagLibMetadata(String _sLibname,
			double _dVersion);

	/**
	 * Return tag info for a specific library
	 * TODO this may not be the form we really want to return this data in
	 * 		that's still being figured out - CVN
	 */
	public XPArrayList GetLibraryTagList(String _sLibname, double _dVersion);

	/**
	 * Return all tags in library
	 * TODO this may not be the form we really want to return this data in
	 * 		that's still being figured out - CVN
	 */
	public XPArrayList GetLibraryTags(String _sLibname, double _dVersion);

	/**
	 * Return metadata or something on all tags in library
	 * TODO not yet implemented cvn
	 */
	public void GetTagMetadata(String _sLibname, double _dVersion,
			String _sTagname);

	/**
	 * This method reloads a particular version of a tag library from
	 * the same library file (i.e. jar or dll) that it was originally
	 * loaded from.
	 *  
	 * @param _sLibname String The name of the library to reload (not display name).
	 * @param _dVersion double The version number of the library to reload.
	 */
	public void ReloadTagLibrary(String _sLibname, double _dVersion);

	/**
	 * This method reloads both the standard and custom tag libraries
	 * from the config files.
	 */
	public void ReloadAllTagLibraries();

	/**
	 * This method returns the number of custom Tag Libraries
	 * currently loaded.  This number is reset every time
	 * custom objects are loaded.
	 *  
	 * @return The number of custom Tag Libraries currently loaded.
	 */
	public int GetNumCustomTagLibraries();

	/**
	 * This method returns the number of custom Tags
	 * currently loaded.  This number is reset every time
	 * custom objects are loaded.
	 *  
	 * @return The number of custom Tagss currently loaded.
	 */
	public int GetNumCustomTags();

}