package com.plumtree.portaluiinfrastructure.tags.metadata;

import com.plumtree.openfoundation.util.*;

/**
 * This base class provides programmatic access to the
 * tag meta data for attributes, including name,
 * description, type, requirement, and default values.
 * <p>
 * Adding public static final member variables of
 * the various ATagAttribute classes turns on
 * validation that the tag attributes are being used
 * correctly, as well as providing useful documentation
 * information.
 */
public abstract class ATagAttribute
{
	// TODO: Should this be converted to lowercase?
	protected String m_strName;
	protected String m_strDescription;
	protected final AttributeType m_Type;

	/**
	 * Creates a new tag attribute.
	 * 
	 * @param _strName The attribute name.
	 * @param _strDescription The attribute description.
	 * @param _type The attribute type.
	 * @throws XPFormatException if the name contains any
	 * whitespace.
	 * @throws XPIllegalArgumentException if the AttributeType
	 * is null.
	 */
	protected ATagAttribute(String _strName, String _strDescription,
		AttributeType _type)
	{
		m_strName = _strName;
		m_strDescription = _strDescription;
		m_Type = _type;

		if (null == m_Type)
		{
			throw new XPIllegalArgumentException(
				"Cannot create Tag Attribute with null AttributeType.  Tag: " +
				m_strName + " Description: " + m_strDescription + ".");
		}

		CheckNameSafety();
	}

	/**
	 * Gets whether or not this attribute is required to be
	 * present every time the tag is used.  If it is
	 * required, the tag will fail if it is not present.
	 * 
	 * @return boolean True means that this attribute is required
	 */
	abstract public boolean GetIsRequired();

	/**
	 * The default value for an optional attribute
	 * if the attribute is not present or is of the
	 * incorrect type.  This will return null for
	 * required attributes.
	 * 
	 * @return String The default value for this
	 * attribute.  Null implies there is no default
	 * value.
	 */
	abstract public String GetDefaultValue();

	/**
	 * The name of the attribute.
	 * 
	 * @return String The attribute name
	 */
	public String GetName()
	{
		return m_strName;
	}

	/**
	 * The description of the attribute and
	 * it's behavior.
	 * 
	 * @return String The attribute description
	 */
	public String GetDescription()
	{
		return m_strDescription;
	}

	/**
	 * The data type of the attribute.
	 * E.G. int, string, etc...
	 * <p>
	 * Specifying the attribute type will
	 * force the conversion of the attribute
	 * to that type and will cause an error
	 * if it cannot be converted.
	 * <p>
	 * Required attributes will be converted
	 * before the tag is displayed and will
	 * cause the tag to fail if they cannot
	 * be converted correctly.
	 * 
	 * @return AttributeType The type of the attribute
	 */
	public AttributeType GetType()
	{
		return m_Type;
	}

	/**
	 * This helper method checks whether or not the
	 * attribute name contains any illegal characters,
	 * such as whitespace.
	 * 
	 * @throws XPFormatException if the name contains any
	 * whitespace.
	 */
	protected void CheckNameSafety()
	{
		// Not the most efficient way to do this, but this
		// method should only be called at startup.  This
		// can be optimized if need be.
		if ((XPStringUtility.IndexOf(m_strName, "\r") >= 0) ||
			(XPStringUtility.IndexOf(m_strName, "\n") >= 0) ||
			(XPStringUtility.IndexOf(m_strName, "\t") >= 0) ||
			(XPStringUtility.IndexOf(m_strName, " ") >= 0))
		{
			throw new XPFormatException("TagAttribute name cannot contain whitespace.  Tag: " +
				m_strName + " Description: " + m_strDescription + ".");
		}
	}
}