package com.plumtree.portaluiinfrastructure.tags.metadata;

import com.plumtree.openfoundation.util.*;

/**
 * This enumeration specifies the type of an attribute.
 * If the input value of a required attribute cannot be
 * converted to the correct type, an error will be
 * raised and the tag will not be displayed (for required
 * attributes only).
 * <p>
 * Information about valid inputs for each type is provided
 * with each enumeration value.  In general, the Tag Framework
 * uses the XPConvert class to convert the string attribute
 * value into the appropriate type.
 *
 * @see com.plumtree.openfoundation.util.XPConvert
 */
public class AttributeType
{
	/**
	 * Human readable name of the AttributeType.
	 */
	private String m_strName;

	/**
	 * String
	 * <p>
	 * Valid input for this type is any string.
	 */
	public static final AttributeType STRING = new AttributeType("string");

	/**
	 * int
	 * <p>
	 * Valid input for this type is a string representation of a
	 * 32-bit signed integer.
	 * <p>
	 * E.G. "7", "42".
	 */
	public static final AttributeType INT = new AttributeType("int");

	/**
	 * boolean
	 * <p>
	 * Valid input for this type is a string that contains the value of
	 * "true", "True", "false", "False", "1", or "0".
	 */
	public static final AttributeType BOOLEAN = new AttributeType("boolean");

	/**
	 * double
	 * <p>
	 * Valid input for this type is a string representation of double-precision
	 * floating point number.  This conversion always uses the default locale
	 * representation of the server ("." for decimal point, et cetera).
	 * <p>
	 * E.G. "4.2", "37", "987.45".
	 */
	public static final AttributeType DOUBLE = new AttributeType("double");

	/**
	 * long
	 * <p>
	 * Valid input for this type is a string representation of a 64-bit signed
	 * integer (I.E. a long).
	 * <p>
	 * E.G. "-5465", "12". 
	 */
	public static final AttributeType LONG = new AttributeType("long");

	/**
	 * char
	 * <p>
	 * Valid input for this type is a string that contains a single character.
	 * <p>
	 * E.G. "c", "q".
	 */
	public static final AttributeType CHAR = new AttributeType("char");

	/**
	 * Private constructor to ensure that instances of
	 * this class form a specific enumeration of values.
	 */
	private AttributeType(String _strName)
	{
		m_strName = _strName;
	}

	/**
	 * Human readable name of the AttributeType.
	 * 
	 * @return String Human readable name of the AttributeType
	 */
	public String toString()
	{
		return m_strName;
	}

	/**
	 * This helper method is used to ensure that the input supplied
	 * is valid for this attribute type.
	 * 
	 * @param String _strValue The input value.
	 * @param AttributeType The current attribute type.
	 * @throws XPFormatException The input cannot be converted
	 * to the appropriate type.
	 */
	public static void ValidateTypeSafety(String _strValue, AttributeType _Type)
	{
		if (AttributeType.BOOLEAN == _Type)
		{
			XPConvert.ToBoolean(_strValue);
		} else if (AttributeType.CHAR == _Type)
		{
			if (_strValue.length() == 0)
			{
				throw new XPFormatException("Unable to convert zero length string to char for attribute value.");
			} else if (_strValue.length() > 1)
			{
				throw new XPFormatException("Attempt to convert string of more than one character (" +
					_strValue + ") to char for attribute value.");
			}

			XPStringUtility.CharAt(_strValue, 0);
		} else if (AttributeType.DOUBLE == _Type)
		{
			XPConvert.ToDouble(_strValue);
		} else if (AttributeType.INT == _Type)
		{
			XPConvert.ToInteger(_strValue);
		} else if (AttributeType.LONG == _Type)
		{
			XPConvert.ToLong(_strValue);
//		} else if (AttributeType.SCOPE == _Type)
//		{
//			if (null == Scope.GetScope(_strValue))
//			{
//				throw new XPFormatException("Attempt to convert string (" +
//					_strValue + ") to Scope for attribute value.  Not a valid Scope.");
//			}
		}
	}
}
