package com.plumtree.portaluiinfrastructure.tags.metadata;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;

/**
 * This class provides programmatic access to the
 * tag meta data for optional attributes, including name,
 * description, type, and default values.
 * <p>
 * Tag classes need to have one public static final
 * OptionalTagAttribute member variable for every attribute
 * that this tag supports, but does not require.
 * <p>
 * E.G.
 * <br>
 * &lt;pt:thistag pt:firstattribute="foo" pt:secondattribute="bar"/&gt;
 * <br>
 * would need
 * <br>
 * public static final OptionalTagAttribute = new OptionalTagAttribute(
 * 		"firstattribute", "This optional attribute is used to ...",
 * 		AttributeType.STRING, "first default value");
 * <br>
 * public static final OptionalTagAttribute = new OptionalTagAttribute(
 * 		"secondattribute", "This optional attribute is used to ...",
 * 		AttributeType.STRING, "second default value");
 * <p>
 * The OptionalTagAttribute objects are used for programmatic
 * access to tag meta data, as well as to pre-process tags
 * for tag attributes (presence, correct type, and default
 * values).  If the attribute values are not correct, the default
 * values will be used instead.  An error will be logged, but the
 * tag and its children will still be displayed.
 * 
 * @see com.plumtree.portaluiinfrastructure.tags.metadata.RequiredTagAttribute
 */
public class OptionalTagAttribute extends ATagAttribute
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.tags.metadata.OptionalTagAttribute");

	protected String m_strDefaultValue;

	/**
	 * Creates a new optional tag attribute.
	 * 
	 * @param _strName The attribute name.
	 * @param _strDescription The attribute description.
	 * @param _type The attribute type.
	 * @param _strDefaultValue The attribute default value.
	 * @throws XPFormatException if the name contains any
	 * whitespace or if the default value cannot be
	 * converted to the appropriate type.
	 * @throws XPIllegalArgumentException if the AttributeType
	 * is null.
	 */
	public OptionalTagAttribute(String _strName, String _strDescription,
		AttributeType _type, String _strDefaultValue)
	{
		super(_strName, _strDescription, _type);

		m_strDefaultValue = _strDefaultValue;

		CheckDefaultValue();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.metadat.ATagAttribute#GetIsRequired()
	 */
	public boolean GetIsRequired()
	{
		return false;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.metadat.ATagAttribute#GetDefaultValue()
	 */
	public String GetDefaultValue()
	{
		return m_strDefaultValue;
	}

	/**
	 * This method is used to ensure that default values
	 * are of the appropriate type.
	 * 
	 * @throws XPFormatException if the default value cannot
	 * be converted to the appropriate type.
	 */
	protected void CheckDefaultValue()
	{
		try
		{
			AttributeType.ValidateTypeSafety(m_strDefaultValue, m_Type);
		} catch (Exception e)
		{
			String strError = "Illegal optional attribute. Unable to convert default value " +
				m_strDefaultValue + " to type " + m_Type +
				".  Tag: " + m_strName + " Description: " + m_strDescription + ".";

			log.Error(e, strError);

			throw new XPFormatException(strError, e);
		}
	}
}
