package com.plumtree.portaluiinfrastructure.tags.metadata;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;

/**
 * This class provides programmatic access to the
 * tag meta data for required attributes, including name,
 * description, and type.
 * <p>
 * Tag classes need to have one public static final
 * RequiredTagAttribute member variable for every attribute
 * that this tag requires.
 * <p>
 * E.G.
 * <br>
 * &lt;pt:thistag pt:firstattribute="foo" pt:secondattribute="bar"/&gt;
 * <br>
 * would need
 * <br>
 * public static final RequiredTagAttribute = new RequiredTagAttribute(
 * 		"firstattribute", "This attribute is used to ...");
 * <br>
 * public static final RequiredTagAttribute = new RequiredTagAttribute(
 * 		"secondattribute", "This attribute is used to ...");
 * <p>
 * The RequiredTagAttribute objects are used for programmatic
 * access to tag meta data, as well as to pre-process tags
 * for required attributes (presence and correct type).  If the
 * attributes are not correct (missing required attribute or bad
 * value), an error will be logged and the tag and its children
 * will be skipped and not displayed.  An HTML Comment describing
 * the tag and error will be displayed instead.
 * 
 * @see com.plumtree.portaluiinfrastructure.tags.metadata.OptionalTagAttribute
 */
public class RequiredTagAttribute extends ATagAttribute
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.tags.metadata.RequiredTagAttribute");

	/**
	 * Creates a new required tag attribute of type String.
	 * <p>
	 * If a required attribute is missing from a tag, the
	 * tag will fail.
	 * 
	 * @param _strName The attribute name
	 * @param _strDescription The attribute description
	 * @throws XPFormatException if the name contains any
	 * whitespace.
	 */
	public RequiredTagAttribute(String _strName, String _strDescription)
	{
		super(_strName, _strDescription, AttributeType.STRING);
	}

	/**
	 * Creates a new required tag attribute.
	 * <p>
	 * If a required attribute is missing from a tag, the
	 * tag will fail.
	 * 
	 * @param _strName The attribute name
	 * @param _strDescription The attribute description
	 * @param _type The attribute type
	 * @throws XPFormatException if the name contains any
	 * whitespace.
	 * @throws XPIllegalArgumentException if the AttributeType
	 * is null.
	 */
	public RequiredTagAttribute(String _strName, String _strDescription,
		AttributeType _type)
	{
		super(_strName, _strDescription, _type);
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.metadat.ATagAttribute#GetIsRequired()
	 */
	public boolean GetIsRequired()
	{
		return true;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.metadat.ATagAttribute#GetDefaultValue()
	 */
	public String GetDefaultValue()
	{
		// Required attributes do not have default values.
		return null;
	}

	/**
	 * This method is used to ensure that the input supplied
	 * for this attribute is valid.
	 * 
	 * @param String _strValue The supplied input value.
	 * @throws XPFormatException The input cannot be converted
	 * to the appropriate type.
	 */
	public void CheckValue(String _strValue)
	{
		try
		{
			AttributeType.ValidateTypeSafety(_strValue, m_Type);
		} catch (Exception e)
		{
			String strError = "Illegal value for required attribute.  Unable to convert supplied input " +
				_strValue + " to type " + m_Type + ".  Tag: " + m_strName +
				" Description: " + m_strDescription + ".";

			log.Error(e, strError);

			throw new XPFormatException(strError, e);
		}
	}
}