package com.plumtree.portaluiinfrastructure.tree;


/**
 * @author Don Hayler
 */
public interface IObjectTreeModel extends IObjectTreeModelRO
{
	/**
	 * This method sets the access level that the user needs to have
	 * to view items in the tree (default is SELECT).
	 *
	 * This must be called before SetQueryFilter().
	 *
	 * @param int the instance of PT_ACCESS_LEVELS to use
	 *             This should be either ADMIN, EDIT, SELECT, or READ otherwise
	 *             it will log an error
	 */
	public void SetAccessLevel(int nLevel);

	/**
	 * This method sets the community mode for this tree.
	 * @see IObjectTreeModelRO
	 *
	 * @param int nMode the community mode
	 * @param int nID the community folder ID (for THISCOMMUNITY mode)
	 */
	public void SetCommunityMode(int nMode, int nID);

	/**
	 * This method sets a filter on a specific class.  This is used for
	 * things like distinguishing gadgets from gadget templates (which
	 * share the same classID).  Hiding objects should be done with the
	 * ClassIDsToHide QS argument.
	 *
	 * This can only be called for filtering on gadget alignments or widths.
	 *
	 * This will create a Query Filter as follows:
	 * [0][0] will be equal to nPropID
	 * [1][0] will be equal to PT_FILTEROPS.PT_FILTEROP_IN
	 * [2][0] will be equal to nValues
	 *
	 * Filters will only be applied to objects of type nClassID.
	 *
	 * This must be called after SetAccessLevel() and SetClassIDsToDisplay().
	 *
	 * @param int nClassID the classID of the class of objects to apply
	 *             this filter to
	 * @param Integer the property ID to filter on
	 * @param int[] the values to filter with.
	 */
	public void SetQueryFilter(int nClassID, int nPropID, int[] nValues);

	/**
	 * This method sets a filter on a specific class.  This is used for
	 * things like distinguishing gadgets from gadget templates (which
	 * share the same classID).  Hiding objects should be done with the
	 * ClassIDsToHide QS argument.
	 *
	 * This can only be called for filtering on data source or web service provider clsids.
	 *
	 * This will create a Query Filter as follows:
	 * [0][0] will be equal to nPropID
	 * [1][0] will be equal to PT_FILTEROPS.PT_FILTEROP_IN
	 * [2][0] will be equal to nValues
	 *
	 * Filters will only be applied to objects of type nClassID.
	 *
	 * This must be called after SetAccessLevel() and SetClassIDsToDisplay().
	 *
	 * @param int nClassID the classID of the class of objects to apply
	 *             this filter to
	 * @param Integer the property ID to filter on
	 * @param String[] the values to filter with.
	 */
	public void SetQueryFilter(int nClassID, int nPropID, String[] sValues);

	/**
	 * This method de/selects the specified tree node
	 *
	 * @param nID The ID of the node to de/select
	 * @param bSelect true implies select, false deselect
	 */
	public void SelectNode(int nID, boolean bSelect);

	/**
	 * This method resets the tree view to display the browsing tree
	 * rather than the search tree.
	 */
	public void ResetToBrowsingMode();

	/**
	 * This method sets the root id for object type trees
	 *
	 * @param nID The ID of the root folder/catalog/etc...
	 */
	public void SetRootFolderID(int nID);
	
	/**
	 * This method returns the class ID to be filtered.
	 *
	 * @return the class ID to apply the above filter to
	 */
	public int GetFilterClassID();
}

