package com.plumtree.portaluiinfrastructure.tree.sitemap;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.treeiterator.*;

/**
 * This class allows you to iterate through a Site Map Folder hierarchy.
 *
 * @author Bridget Frey
 */
public class SiteMapTreeIterator implements IPTTreeIterator
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.tree.sitemap.SiteMapTreeIterator");

	/** Site Map Folder object manager */
	protected IPTObjectManager m_objOM;

	/** User's session. */
	protected IPTSession m_objSession;

	/** Current folder; can be NULL if we move to a folder that is inaccessible. */
	protected IPTSiteMapFolder m_objFolder;

	/** Root of the iteration. */
	protected IPTSiteMapFolder m_objRoot;

	/** Has the class been initialized. */
	protected boolean m_bInitialized = false;

	/** Only 0 is recognized. */
	protected int m_nStyle = 0;

	/**
	 * Constructor for SiteMapTreeIterator.
	 */
	public SiteMapTreeIterator()
	{
		super();
	}

	/**
	 * @see com.plumtree.server.treeiterator.IPTTreeIterator#Children(int, int)
	 */
	public Object[][] Children(int arg0, int arg1)
	{
		if (!CheckInitialized())
		{
			return null;
		}

		if (m_objFolder == null)
		{
			return null;
		}

		IPTQueryResult qrFolders = m_objFolder.QuerySubfolders(false);
		int nNumChildren		 = qrFolders.RowCount();

		Object[][] objChildren = new Object[10][nNumChildren];

		for (int i = 0; i < nNumChildren; i++)
		{
			String strName     = qrFolders.ItemAsString(i,
					PT_PROPIDS.PT_PROPID_NAME);
			int lObjectID	   = qrFolders.ItemAsInt(i,
					PT_PROPIDS.PT_PROPID_OBJECTID);
			String strObjectID = XPConvert.ToString(lObjectID);

			objChildren[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_NAME][i] = strName;
			objChildren[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_IDENTIFIER][i] = strObjectID;
			objChildren[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_TYPE][i] = new Integer(PT_TREE_ITERATOR_CHILD_TYPES.PT_TREE_ITERATOR_CHILD_TYPE_CONTAINER);

			// -1 signifies that we don't know how many children we have	
			objChildren[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_CHILDCOUNT][i] = new Integer(-1);
		}

		return objChildren;
	}

	/**
	 * @see com.plumtree.server.treeiterator.IPTTreeIterator#CurrentLocInfo()
	 */
	public Object[][] CurrentLocInfo()
	{
		if (!CheckInitialized())
		{
			return null;
		}

		if (m_objFolder == null)
		{
			return null;
		}

		IPTServerContext objSC = (IPTServerContext) m_objFolder.GetInterfaces(
				"IPTServerContext");
		String strObjectID     = XPConvert.ToString(objSC.GetObjectID());

		Object[][] objCurrentLocInfo = new Object[10][1];
		objCurrentLocInfo[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_NAME][0]			 = m_objFolder.GetLocalizedName();
		objCurrentLocInfo[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_IDENTIFIER][0]     = strObjectID;
		objCurrentLocInfo[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_TYPE][0]			 = new Integer(PT_TREE_ITERATOR_CHILD_TYPES.PT_TREE_ITERATOR_CHILD_TYPE_CONTAINER);
		objCurrentLocInfo[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_CHILDCOUNT][0]     = new Integer(GetChildCount());

		return objCurrentLocInfo;
	}

	/**
	 * @see com.plumtree.server.treeiterator.IPTTreeIterator#GetAtRoot()
	 */
	public boolean GetAtRoot()
	{
		if (!CheckInitialized())
		{
			return false;
		}

		if (m_objFolder == null)
		{
			return false;
		}

		IPTServerContext scFolder = (IPTServerContext) m_objFolder.GetInterfaces(
				"IPTServerContext");
		IPTServerContext scRoot   = (IPTServerContext) m_objRoot.GetInterfaces(
				"IPTServerContext");

		return scFolder.GetObjectID() == scRoot.GetObjectID();
	}

	/**
	 * @see com.plumtree.server.treeiterator.IPTTreeIterator#GetChildCount()
	 */
	public int GetChildCount()
	{
		if (!CheckInitialized())
		{
			return 0;
		}

		if (m_objFolder == null)
		{
			return 0;
		}

		IPTQueryResult qrFolders = m_objFolder.QuerySubfolders(false);

		return qrFolders.RowCount();
	}

	/**
	 * @see com.plumtree.server.treeiterator.IPTTreeIterator#GetLocation()
	 */
	public String GetLocation()
	{
		if (!CheckInitialized())
		{
			return "";
		}

		if (m_objFolder == null)
		{
			return "0";
		}

		IPTServerContext objSC = (IPTServerContext) m_objFolder.GetInterfaces("IPTServerContext");
		String strObjectID     = XPConvert.ToString(objSC.GetObjectID());

		return strObjectID;
	}

	/**
	 * @see com.plumtree.server.treeiterator.IPTTreeIterator#GetStyle()
	 */
	public int GetStyle()
	{
		return m_nStyle;
	}

	/**
	 * @see com.plumtree.server.treeiterator.IPTTreeIterator#InitializeIterator(java.lang.Object)
	 */
	public void InitializeIterator(Object arg0)
	{
	}

	/**
	 * @see com.plumtree.server.treeiterator.IPTTreeIterator#MoveTo(java.lang.String)
	 */
	public void MoveTo(String arg0)
	{
		if (!CheckInitialized())
		{
			return;
		}

		int nMoveToID = XPConvert.ToInteger(arg0);
		m_objFolder = (IPTSiteMapFolder) m_objOM.Open(nMoveToID, false);

		if (m_objFolder == null)
		{
			
				log.Warn("SiteMapTreeIterator: Attempted to MoveTo a folder that does not exist, or this user does not have access to the folder.  Will set folder to null.");
				m_objFolder = null;
			

			return;
		}
	}

	/**
	 * @see com.plumtree.server.treeiterator.IPTTreeIterator#MoveToParent()
	 */
	public void MoveToParent()
	{
		if (!CheckInitialized())
		{
			return;
		}

		if (m_objFolder == null)
		{
			// this can happen if someone moved us to a folder we don't have access to, or that
			// failed to open.  so in this case, just send them to the root.
			m_objFolder = m_objRoot;

			return;
		}

		m_objFolder = (IPTSiteMapFolder) m_objOM.Open(m_objFolder.GetParentFolderID(), false);
	}

	/**
	 * @see com.plumtree.server.treeiterator.IPTTreeIterator#MoveToRoot()
	 */
	public void MoveToRoot()
	{
		if (!CheckInitialized())
		{
			return;
		}

		m_objFolder = m_objRoot;
	}

	/**
	 * Use this function to initialize the Iterator.  Do not use
	 * InitializeIterator, which is on IPTTreeIterator, but does not meet the
	 * needs of this class.
	 *
	 * @param nSiteMapFolderID: ID of SiteMapFolder to iterate.
	 * @param objSession: User's session.
	 */
	public void RealInitializeIterator(int nSiteMapFolderID,
		IPTSession objSession)
	{
		m_objSession     = objSession;
		m_objOM			 = objSession.GetSiteMapFolders();

		m_objRoot     = (IPTSiteMapFolder) m_objOM.Open(nSiteMapFolderID, false);

		// set the current folder equal to the root
		m_objFolder = m_objRoot;

		if ((m_objRoot == null) || (m_objSession == null) || (m_objOM == null))
		{
			
				log.Warn("SiteMapTreeIterator: Iterator was initialized with null values.");
			

			return;
		}

		m_bInitialized = true;
	}

	/**
	 * @see com.plumtree.server.treeiterator.IPTTreeIterator#SetStyle(int)
	 */
	public void SetStyle(int nStyle)
	{
		m_nStyle = nStyle;
	}

	/**
	 * Test function.
	 *
	 * @param ptTreeIterator: IPTTreeIterator to iterate.
	 * @param strIndent: This function is called recursively, and adds spaces to
	 * strIndent for every level of recursion.  Pass in "" for the first call to
	 * the function.
	 */
	public static void printTree(IPTTreeIterator ptTreeIterator,
		String strIndent)
	{
		Object[][] currentLocation = (Object[][]) ptTreeIterator.CurrentLocInfo();

		String parentName = (String) currentLocation[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_NAME][0];
		log.Info(strIndent + parentName);

		Object[][] childNodes = (Object[][]) ptTreeIterator.Children(0, -1);

		if (null == childNodes)
		{
			return;
		}

		int numChildren = childNodes[0].length;

		int parentType = XPConvert.ToInteger(currentLocation[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_TYPE][0]);

		if (PT_TREE_ITERATOR_CHILD_TYPES.PT_TREE_ITERATOR_CHILD_TYPE_CONTAINER == parentType)
		{
			for (int i = 0; i < numChildren; i++)
			{
				String childLocation = (String) childNodes[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_IDENTIFIER][i];
				ptTreeIterator.MoveTo(childLocation);
				printTree(ptTreeIterator, strIndent + "  ");
			}
		}
	}

	/**
	 * Check whether Initialized was called.
	 *
	 * @return True if initialized, False if not initialized.
	 */
	private boolean CheckInitialized()
	{
		if (m_bInitialized == false)
		{
			
				log.Warn("SiteMapTreeIterator: Iterator has not been initialized.");
			

			return false;
		}
		else
		{
			return true;
		}
	}
}

