package com.plumtree.portaluiinfrastructure.mock.restapi;

import com.plumtree.openfoundation.util.*;

/**
 * Takes the XPHashtable generated by RestXMLResponse and converts it to a 
 * human readable string representation of the hashtable content. 
 * 
 * Use the string representation to validate the expected output from the parsed
 * xml.
 * 
 * Format of the string representation:
 * 
 * Some rules:
 * 1. Each level adds 2 spaces (not tabs) as indentation
 * 2. Arrays and nested child nodes has '+' instead of the '-' used by simple text data
 * 3. Array elements are denoted by '[n]' where n is the array index of the element 
 *  
 * ROOT
 * - name1: value2 				"Simple name/value pairs"
 * - name2: value2				"Simple name/value pairs"
 * + name3:						"Nested node "
 *   - name4: value4
 * 	 - name5: value5
 * 	 + name6:				"Nested nested node"
 * 	   - name6: value7
 * + name8: 					"List node (nodes with the same name)
 *   [0] value10
 * 	 [1] value11
 * 	 [2] value12
 * + name9						"List node with nested nodes and nested list nodes
 *   - name20: value20
 * 	 - name21: value21	
 * 	 + name22:					"List of nested child nodes"
 * 	 [0]:						
 * 	   [0]: value22A			"List in a List
 * 	   [1]: value22B
 * 	   [2]: value22C
 * 	 [1]: 						"Nested child node in array"
 * 	   - name30 : value30A
 * 	   - name31 : value30B
 * 	   - name32 : value30C
 * 
 * 
 * @author rozhang
 *
 */
public class XMLParseResultVisualizer 
{
	protected XPHashtable m_result = null; 
	
	public XMLParseResultVisualizer(XPHashtable result)
	{
		m_result = result;
	}
	
	/**
	 * Main parsing function 
	 * @return
	 */
	public String GenerateStringRepresentation()
	{
		XPStringBuilder buf = new XPStringBuilder();
		buf.Append("ROOT\n");
		buf.Append( ParseHashTable(m_result, 0) );
		
		return buf.ToString();
	}
	
	/**
	 * Generate String representation for node with child nodes
	 * 
	 * @param map
	 * @param level	The depth from the top where top is 0 
	 * @return
	 */
	protected String ParseHashTable(XPHashtable map, int level)
	{
		XPStringBuilder buf = new XPStringBuilder();
		// iterate over the xphashtable
		IXPEnumerator keys = map.GetKeys();
		while ( keys.MoveNext() ) 
		{
			String curName = (String)keys.GetCurrent();
			Object val = map.GetElement(curName);
			
			
			if ( val instanceof XPHashtable ) 
			{
				buf.Append(AddPadding(level)).Append("+ ").Append(curName).Append(":");
				buf.Append("\n");
				buf.Append( ParseHashTable((XPHashtable)val, level+1) );
			} 
			else if ( val instanceof XPArrayList ) 
			{
				buf.Append(AddPadding(level)).Append("+ ").Append(curName).Append(":");
				buf.Append("\n");
				buf.Append(  ParseArray((XPArrayList)val, level ));
				
			} 
			else 
			{
				buf.Append(AddPadding(level)).Append("- ").Append(curName).Append(":");
				buf.Append(" ").Append(val).Append("\n");
			} 
		}
		return buf.ToString();
	}
	
	/**
	 * Generate String representation for Array node
	 * 
	 * @param values
	 * @param level
	 * @return
	 */
	protected String ParseArray(XPArrayList values, int level)
	{
		XPStringBuilder buf = new XPStringBuilder();

		for (int i = 0; i < values.GetSize(); i++ ) {
			Object val = values.GetElement(i);
			
			if ( val instanceof XPHashtable ) {
				buf.Append(AddPadding(level)).Append("["+i+"]:\n");
				buf.Append( ParseHashTable((XPHashtable)val, level+1) );
			} 
			else if ( val instanceof XPArrayList ) 
			{
				buf.Append("\n");
				buf.Append(  ParseArray((XPArrayList)val, level+1 ) );
			} 
			else 
			{
				buf.Append(AddPadding(level)).Append("  ["+i+"]: ").Append(val).Append("\n");
			}
		}
		return buf.ToString();
	}
	
	/**
	 * Helper for adding the proper amount of indentation
	 * @param level
	 * @return
	 */
	protected String AddPadding(int level)
	{
		XPStringBuilder buf = new XPStringBuilder();
		for (int i = 0; i < level; i++ ) {
			buf.Append("  "); // 2 spaces
		}
		return buf.ToString();
	}

}
