package com.plumtree.portaluiinfrastructure.test.restapi;

import junit.framework.*;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.mock.restapi.*;
import com.plumtree.portaluiinfrastructure.restconsumerframework.*;

/**
 *  Test for RestXMLResponse
 *  
 *  This test takes inline XML responses in Strings that are used by the mock RestRequest object. 
 *  
 *  For validation of the parsed XML, a string representation is generated from the resulting hashtable. The generated string
 *  is a visual representation of the content in hashtable. So instead of manually code comparison of an expected and generated
 *  hashtable, validation is just a string comparison of the generated string representation.
 *  
 *  The most efficient way to create a test is
 *  
 *  1. Copy an existing test method and change the name.
 *  
 *  2. Put the XML to test in a Java string.
 *  
 *  3. Comment out the validation code in the method and run the test method and output the string representation 
 *     (from  parser.GenerateStringRepresentation(); ) to console or some other way.
 *     
 *  4. Visually verify that the string representation looks correct based on input XML. Refer to XMLParseResultVisualizer
 *  	for the rules of how the string representation is generated. NOTE: The order of the nodes in the string representation
 *  	is probably different than from the XML. This is OK. The order will always be the same in the string representation.
 *    
 *  5. If the output looks correct, put it into a String. Avoid tabs to create indentation and remember to put in a carriage return
 *  	'\n' after each line. The whitespace matters during String comparison.
 *  
 *  6. Uncomment the validation section of the method and run the test again. You might get a test error if the string representation 
 *  	isn't properly formatted. Look in the console for what the test expects and what is returned. Any whitespace issues should
 *  	be visible in the console. Probably due to tabs in the string instead of spaces.
 *  	
 *  
 * 
 * @author rozhang
 */
public class RestXMLParserTest extends TestCase
{
	/** Use later if we want to load xml from file system **/
	public static String base_path = XPSystem.GetCurrentDirectory();
	
	
	/**
	 */
	public RestXMLParserTest(String name)
   	{
		super(name);
   	}
   
   	/**
	 * Setup if any is required.
	 */
   	public void setUp() throws Exception
   	{
   		super.setUp();
   		//XPSystem.Out.WriteLine("Base PATH: "+base_path);
   	}
   	
   	/**
	 * @see TestCase#tearDown()
	 */
	public void tearDown() throws Exception
	{
		super.tearDown();
	}

	
	public void testNestedArrayData()
	{
		String content = 
			"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>"+
			"<soc:PResultList xmlns:soc=\"http://social.bea.com/pathways\">"+
			"<item>"+
	        "<activityRank>0.0</activityRank>"+
	        "<classId>18</classId>"+
	        "<resultScore>7.35571</resultScore>"+
	        "<source>PTPORTAL</source>"+
	        "<tags>"+
	        "    <tag>"+
	        "        <count>1</count>"+
	        "        <isAppliedByCurrentUser>false</isAppliedByCurrentUser>"+
	        "        <normalizedCount>9</normalizedCount>"+
	        "        <value>cheap</value>"+
	        "    </tag>"+
	        "    <tag>"+
	        "        <count>1</count>"+
	        "        <isAppliedByCurrentUser>false</isAppliedByCurrentUser>"+
	        "        <normalizedCount>9</normalizedCount>"+
	        "        <value>superstore</value>"+
	        "    </tag>"+
	        "</tags>  "+
			"</item>"+
			"</soc:PResultList>";
		
		String answer = 
			"ROOT\n"+
			"+ item:\n"+
			"  - classId: 18\n"+
			"  + tags:\n"+
			"    + tag:\n"+
			"    [0]:\n"+
			"      - value: cheap\n"+
			"      - normalizedCount: 9\n"+
			"      - isAppliedByCurrentUser: false\n"+
			"      - count: 1\n"+
			"    [1]:\n"+
			"      - value: superstore\n"+
			"      - normalizedCount: 9\n"+
			"      - isAppliedByCurrentUser: false\n"+
			"      - count: 1\n"+
			"  - source: PTPORTAL\n"+
			"  - activityRank: 0.0\n"+
			"  - resultScore: 7.35571\n";

		
		RestRequest req = new MockRestRequest(content, false);
		IRestResponse response = req.Submit();
		
		XMLParseResultVisualizer parser = new XMLParseResultVisualizer(response.GetSingleDataItem());
		String result = parser.GenerateStringRepresentation();
		//XPSystem.Out.WriteLine("testNestedArrayData:\n"+result);
		
		if ( !answer.equals(result) ) {
			// print out to console since the eclipse view doesn't show the full string
			XPSystem.Out.WriteLine("testNestedArrayData failed!");
			XPSystem.Out.WriteLine("Expected Result:");
			XPSystem.Out.WriteLine(answer);
			XPSystem.Out.WriteLine("Recieved Result:");
			XPSystem.Out.WriteLine(result);
			assertEquals(answer, result);
		}
		
		
	}
	
	public void testSimpleValuesOnly() 
	{
		String content =
			"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>"+
			"<soc:PResultList xmlns:soc=\"http://social.bea.com/pathways\">"+
	        "  <activityRank>0.0</activityRank>"+
	        "  <classId>18</classId>"+
	        "  <resultScore>7.35571</resultScore>"+
	        "  <source>PTPORTAL</source>"+
	        "</soc:PResultList>";
		
		String answer =
			"ROOT\n"+
			"- classId: 18\n"+
			"- source: PTPORTAL\n"+
			"- activityRank: 0.0\n"+
			"- resultScore: 7.35571\n";
		
		
		RestRequest req = new MockRestRequest(content, false);
		IRestResponse response = req.Submit();
		
		XMLParseResultVisualizer parser = new XMLParseResultVisualizer(response.GetSingleDataItem());
		String result = parser.GenerateStringRepresentation();
		//XPSystem.Out.WriteLine(result);
		
		if ( !answer.equals(result) ) {
			// print out to console since the eclipse view doesn't show the full string
			XPSystem.Out.WriteLine("testSimpleValuesOnly failed!");
			XPSystem.Out.WriteLine("Expected Result:");
			XPSystem.Out.WriteLine(answer);
			XPSystem.Out.WriteLine("Recieved Result:");
			XPSystem.Out.WriteLine(result);
			assertEquals(answer, result);
		}
	}
	
	public void testArrayInArrayData()
	{
		String content = 
			"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>"+
			"<soc:PResultList xmlns:soc=\"http://social.bea.com/pathways\">"+
			"<item>"+
	        "<tags>"+
	        "    <tag>"+
	        "        <count>3</count>"+
	        "        <tags2>arrayinarray1</tags2>"+
	        "        <tags2>arrayinarray2</tags2>"+
	        "        <tags2>arrayinarray3</tags2>"+
	        "    </tag>"+
	        "    <tag>"+
	        "        <tags2>arrayinarray1</tags2>"+
	        "        <tags2>arrayinarray2</tags2>"+
	        "        <tags2>arrayinarray3</tags2>"+
	        "    </tag>"+
	        "</tags>  "+
			"</item>"+
			"</soc:PResultList>";
		
		String answer = 
			"ROOT\n"+
			"+ item:\n"+
			"  + tags:\n"+
			"    + tag:\n"+
			"    [0]:\n"+
			"      - count: 3\n"+
			"      + tags2:\n"+
			"        [0]: arrayinarray1\n"+
			"        [1]: arrayinarray2\n"+
			"        [2]: arrayinarray3\n"+
			"    [1]:\n"+
			"      + tags2:\n"+
			"        [0]: arrayinarray1\n"+
			"        [1]: arrayinarray2\n"+
			"        [2]: arrayinarray3\n"
			;
		
		RestRequest req = new MockRestRequest(content, false);
		IRestResponse response = req.Submit();
		
		XMLParseResultVisualizer parser = new XMLParseResultVisualizer(response.GetSingleDataItem());
		String result = parser.GenerateStringRepresentation();
		//XPSystem.Out.WriteLine(result);
		
		if ( !answer.equals(result) ) {
			// print out to console since the eclipse view doesn't show the full string
			XPSystem.Out.WriteLine("testArrayInArrayData failed!");
			XPSystem.Out.WriteLine("Expected Result:");
			XPSystem.Out.WriteLine(answer);
			XPSystem.Out.WriteLine("Recieved Result:");
			XPSystem.Out.WriteLine(result);
			assertEquals(answer, result);
		}
	}
	
	public void testListWithNestedData()
	{
		String content = 
			"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>"+
	        "<tags>"+
	        "    <tag>"+
	        "        <count>3</count>"+
	        "        <tags2>arrayinarray1</tags2>"+
	        "        <tags2>arrayinarray2</tags2>"+
	        "        <tags2>arrayinarray3</tags2>"+
	        "        <moredata>"+
	        "        	<data1>1</data1>"+
	        "        	<data1>a</data1>"+
	        "        	<data1>c</data1>"+
	        "        </moredata>"+
	        "    </tag>"+
	        "    <tag>"+
	        "        <tags2>arrayinarray1</tags2>"+
	        "        <tags2>arrayinarray2</tags2>"+
	        "        <tags2>arrayinarray3</tags2>"+
	        "        <moredata>"+
	        "        	<data1>1</data1>"+
	        "        	<data1>a</data1>"+
	        "        	<data1>c</data1>"+
	        "        </moredata>"+
	        "    </tag>"+
	        "</tags>"
        ;
		
		String answer =
			"ROOT\n"+
			"+ tag:\n"+
			"[0]:\n"+
			"  - count: 3\n"+
			"  + moredata:\n"+
			"    + data1:\n"+
			"      [0]: 1\n"+
			"      [1]: a\n"+
			"      [2]: c\n"+
			"  + tags2:\n"+
			"    [0]: arrayinarray1\n"+
			"    [1]: arrayinarray2\n"+
			"    [2]: arrayinarray3\n"+
			"[1]:\n"+
			"  + moredata:\n"+
			"    + data1:\n"+
			"      [0]: 1\n"+
			"      [1]: a\n"+
			"      [2]: c\n"+
			"  + tags2:\n"+
			"    [0]: arrayinarray1\n"+
			"    [1]: arrayinarray2\n"+
			"    [2]: arrayinarray3\n"
			;
		
		RestRequest req = new MockRestRequest(content, false);
		IRestResponse response = req.Submit();
		
		XMLParseResultVisualizer parser = new XMLParseResultVisualizer(response.GetSingleDataItem());
		String result = parser.GenerateStringRepresentation();

		//XPSystem.Out.WriteLine(visString);
		
		if ( !answer.equals(result) ) {
			// print out to console since the eclipse view doesn't show the full string
			XPSystem.Out.WriteLine("testListWithNestedData failed!");
			XPSystem.Out.WriteLine("Expected Result:");
			XPSystem.Out.WriteLine(answer);
			XPSystem.Out.WriteLine("Recieved Result:");
			XPSystem.Out.WriteLine(result);
			assertEquals(answer, result);
		}
	}
	
	

}
