package com.plumtree.portaluiinfrastructure.test.tags.helper;

import junit.framework.*;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.helper.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.portaluiinfrastructure.test.tags.*;
import com.plumtree.server.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Test for the TagHelper class.
 * 
 * @author donh
 */
public class TagHelperTest extends TestCase
{
	public static final String STRING_VALUE = "value";
	public static final String INT_VALUE_STRING = "72";

	protected TestTag m_TestTag;

	/**
	 * Constructor.
	 */
	public TagHelperTest(String name)
	{
		super(name);

		// Initialize the meta data for the test tags.
		// This is normally done by the TagLibManager.

		m_TestTag = new TestTag();
		TagMetaDataHelper testMetaDataHelper = new TagMetaDataHelper();
		testMetaDataHelper.Init(m_TestTag,
			new TagLibraryMetaData("Test Tag Library",
			"testtaglibrary", "Test tag library description", 1.0));
		m_TestTag.SetTagMetaDataHelper(testMetaDataHelper);
	}

	/**
	 * This tests the helper functions for attribute
	 * type safety.
	 */
	public void testTagAttributeTypes()
	{
		// Can't put this directly as an argument because of jump issues.
		Object[][] objXML = new Object[0][0];

		TagHelper helper = new TagHelper(objXML, null, GetNewTestTagInstance());

		String strTest;

		strTest = helper.GetAttributeAsString(TestTag.TESTATTRIBUTE);
		Assert.assertNull("Expected empty String for test attribute.", strTest);

		boolean bFailed = false;
		boolean bTest = true;

		try
		{
			bTest = helper.GetAttributeAsBoolean(TestTag.TESTATTRIBUTE);
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Got empty string test attribute as boolean: " +
				bTest + ", expected XPFormatException.");
		}

		bFailed = false;
		char cTest = 'a';

		try
		{
			cTest = helper.GetAttributeAsChar(TestTag.TESTATTRIBUTE);
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Got empty string test attribute as char: " +
				cTest + ", expected XPFormatException.");
		}

		bFailed = false;
		double dTest = -1;

		try
		{
			dTest = helper.GetAttributeAsDouble(TestTag.TESTATTRIBUTE);
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Got empty string test attribute as double: " +
				dTest + ", expected XPFormatException.");
		}

		bFailed = false;
		int nTest = -1;

		try
		{
			nTest = helper.GetAttributeAsInt(TestTag.TESTATTRIBUTE);
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Got empty string test attribute as int: " +
				nTest + ", expected XPFormatException.");
		}

		bFailed = false;
		long lTest = -1;

		try
		{
			lTest = helper.GetAttributeAsLong(TestTag.TESTATTRIBUTE);
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Got empty string test attribute as long: " +
				lTest + ", expected XPFormatException.");
		}
	}

	/**
	 * This tests the attributes to make sure they return
	 * appropriate values.
	 */
	public void testTagAttributeValues()
	{
		// Can't put this directly as an argument because of jump issues.
		Object[][] objXML = {
			{ new Integer(PT_XML_TYPES.PT_XML_TYPES_PTUI_ATT), new Integer(PT_XML_TYPES.PT_XML_TYPES_PTUI_ATT), new Integer(PT_XML_TYPES.PT_XML_TYPES_PTUI_ATT) },
			{ "testattr", "testintattr", "testbadintattr" },
			{ STRING_VALUE, INT_VALUE_STRING , "garbageint" } };

		TagHelper helper = new TagHelper(objXML,
			null, GetNewTestTagInstance());

		String strTest;

		strTest = helper.GetAttributeAsString(TestTag.TESTATTRIBUTE);
		Assert.assertEquals("Getting value for test attribute.", STRING_VALUE, strTest);

		strTest = helper.GetAttributeAsString(TestTag.TEST2ATTRIBUTE);
		Assert.assertEquals("Getting default value for test2 attribute.", TestTag.DEFAULT_STRING_VALUE, strTest);

		boolean bTest;

		bTest = helper.HasAttribute(TestTag.TEST2ATTRIBUTE);
		Assert.assertFalse("Checking for non-existent test 2 attribute.", bTest);

		int nTest;

		int nIntValue = XPConvert.ToInteger(INT_VALUE_STRING);

		nTest = helper.GetAttributeAsInt(TestTag.TESTINTATTRIBUTE);
		Assert.assertEquals("Getting int test attribute.", nIntValue, nTest);

		bTest = helper.HasAttribute(TestTag.TESTINT2ATTRIBUTE);
		Assert.assertEquals("Checking for non-existent test 2 int attribute.", false, bTest);

		int nDefaultIntValue = XPConvert.ToInteger(TestTag.DEFAULT_INT_VALUE_STRING);

		nTest = helper.GetAttributeAsInt(TestTag.TESTINT2ATTRIBUTE);
		Assert.assertEquals("Getting default value for int test 2 attribute.", nDefaultIntValue, nTest);

		// Check for proper error handling with non-convertable attributes
		bTest = helper.HasAttribute(TestTag.TESTBADINTATTRIBUTE);
		Assert.assertEquals("Checking for existence of test bad int attribute.", true, bTest);

		nTest = helper.GetAttributeAsInt(TestTag.TESTBADINTATTRIBUTE);
		Assert.assertEquals("Getting default value for bad int test attribute.", nDefaultIntValue, nTest);
	}

	/**
	 * This tests the XML attributes to make sure they return
	 * appropriate values.
	 */
	public void testTagXMLAttributeValues()
	{
		// Can't put this directly as an argument because of jump issues.
		Object[][] objXML = {
			{ new Integer(PT_XML_TYPES.PT_XML_TYPES_XHTML_ATT), new Integer(PT_XML_TYPES.PT_XML_TYPES_PTUI_ATT), new Integer(PT_XML_TYPES.PT_XML_TYPES_XHTML_ATT) },
			{ "xhtmlattr", "testintattr", "xhtmlattr2" },
			{ STRING_VALUE, INT_VALUE_STRING , "garbagestring" } };

		TagHelper helper = new TagHelper(objXML,
			null, GetNewTestTagInstance());

		Assert.assertTrue("Checking for existence of xhtml attribute.", helper.HasXMLAttribute("xhtmlattr"));

		String strTest;

		strTest = helper.GetXMLAttribute("xhtmlattr");
		Assert.assertEquals("Getting value for test xhtml attribute.", STRING_VALUE, strTest);

		strTest = helper.GetXMLTagAttributesAsString();
		Assert.assertEquals("Getting xhtml attribute output string.",
			"xhtmlattr=\"" + STRING_VALUE + "\" xhtmlattr2=\"garbagestring\"", strTest);
	}

	/**
	 * This tests the helper function that returns a uniqued ID for
	 * each instance of a tag.
	 * 
	 * NOTE: This does not test if the unique ID is unique across all
	 * portlets in a a page, just one.
	 */
	public void testGetUniqueID()
	{
		TagVisitor visitor = new TagVisitor(new TagState(null, null, null, new XPHashtable()),
		        	null, new HTMLElementCollection());

		// Can't put this directly as an argument because of jump issues.
		Object[][] objXML = new Object[0][0];

		// Create a new Tag and test the Unique ID function in it.
		TestTag testTag = GetNewTestTagInstance();

		TagHelper firstHelper = new TagHelper(objXML, null, testTag);

		firstHelper.SetTagVisitor(visitor);

		firstHelper.GetState().EnterTag(testTag);

		String strFirstUniqueID = firstHelper.GetUniqueID();

		Assert.assertNotNull("Getting unique ID for the first tag.", strFirstUniqueID);

		Assert.assertFalse("Unique ID should not be the empty string",
			XPStringUtility.EqualsIgnoreCase("", strFirstUniqueID));

		Assert.assertEquals("Unique ID should be the same for the same tag instance.",
			strFirstUniqueID, firstHelper.GetUniqueID());

		// Create a second Tag and test the Unique ID function in it.

		testTag = GetNewTestTagInstance();

		TagHelper secondHelper = new TagHelper(objXML, null, testTag);

		secondHelper.SetTagVisitor(visitor);

		secondHelper.GetState().EnterTag(testTag);

		String strSecondUniqueID = secondHelper.GetUniqueID();

		Assert.assertNotNull("Getting unique ID for the second tag.", strSecondUniqueID);

		Assert.assertFalse("Unique ID should not be the empty string",
			XPStringUtility.EqualsIgnoreCase("", strSecondUniqueID));

		Assert.assertEquals("Unique ID should be the same for the same tag instance.",
		strSecondUniqueID, secondHelper.GetUniqueID());

		// Make sure the two unique IDs are different.
		Assert.assertTrue("Two tags should have different unique IDs.",
			(0 != XPStringUtility.CompareIgnoreCase(strFirstUniqueID, strSecondUniqueID)));

		// Make sure the first unique ID is still returned correctly.
		Assert.assertEquals("Unique ID for the first tag should be the same after running another tag.",
			strFirstUniqueID, firstHelper.GetUniqueID());

		firstHelper.GetState().ExitTag();

		firstHelper.ReleaseTagVisitor();

		// Make sure the second unique ID is still returned correctly.
		Assert.assertEquals("Unique ID for the second tag should be the same after running another tag.",
			strSecondUniqueID, secondHelper.GetUniqueID());

		secondHelper.GetState().ExitTag();

		secondHelper.ReleaseTagVisitor();
	}

	/**
	 * This helper method creates and initializes a new TestTag instance.
	 * This would normally be handled by the TagLibManager.
	 * 
	 * @return  A new TestTag instance.
	 */
	protected TestTag GetNewTestTagInstance()
	{
		TestTag result = new TestTag();
		result.SetTagMetaDataHelper(m_TestTag.GetTagMetaDataHelper());

		return result;
	}
}
