package com.plumtree.portaluiinfrastructure.test.tags.helper;

import junit.framework.*;

import com.plumtree.openfoundation.reflect.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.helper.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.portaluiinfrastructure.test.tags.*;

/**
 * Tests for the Tag State class.
 * 
 * @author donh
 */
public class TagStateTest extends TestCase
{
	public static final String EDITABLE_KEY = "testeditable";
	public static final String UNEDITABLE_KEY = "testuneditable";

	protected TagState m_TagState;

	protected TestTag m_TestTag;
	protected SecondTestTag m_SecondTestTag;

	/**
	 * Constructor.
	 */
	public TagStateTest(String name)
	{
		super(name);

		// Initialize the meta data for the test tags.
		// This is normally done by the TagLibManager.

		m_TestTag = new TestTag();
		TagMetaDataHelper testMetaDataHelper = new TagMetaDataHelper();
		testMetaDataHelper.Init(m_TestTag,
			new TagLibraryMetaData("Test Tag Library",
			"testtaglibrary", "Test tag library description", 1.0));
		m_TestTag.SetTagMetaDataHelper(testMetaDataHelper);

		m_SecondTestTag = new SecondTestTag();
		TagMetaDataHelper secondTestMetaDataHelper = new TagMetaDataHelper();
		secondTestMetaDataHelper.Init(m_SecondTestTag,
			new TagLibraryMetaData("Second Test Tag Library",
			"secondtesttaglibrary", "Second test tag library description", 1.0));
		m_SecondTestTag.SetTagMetaDataHelper(secondTestMetaDataHelper);
	}

	/**
	 * Setup if any is required.
	 */
	public void setUp() throws Exception
	{
		super.setUp();

		m_TagState = new TagState(null, null, null, new XPHashtable());
	}

	public void tearDown() throws Exception
	{
		super.tearDown();

		m_TagState.ResetPortletRequestData();
		m_TagState = null;
	}

	/**
	 * This tests resetting the Tag State object.
	 */
	public void testResetPortletRequestData()
	{
		testTagScope();
		m_TagState.ResetPortletRequestData();

		testPortletScope();
		m_TagState.ResetPortletRequestData();

		testTagScope();
		m_TagState.ResetPortletRequestData();

		testPortletScope();
		m_TagState.ResetPortletRequestData();
	}

	/**
	 * This tests the argument checking for
	 * accessing variables.
	 */
	public void testIllegalArguments()
	{
		// Tag State cannot be used without entering a tag first.
		m_TagState.EnterTag(GetNewTestTagInstance());

		boolean bFailed = false;

		try
		{
			m_TagState.SetVariable(null, "test", Scope.TAG);
		} catch (XPIllegalArgumentException xpiae)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Setting Variable with null key argument did not fail.");
		}

		bFailed = false;

		try
		{
			m_TagState.SetVariable("test.test", "test", Scope.SESSION);
		} catch (XPIllegalArgumentException xpiae)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Setting Variable with illegal character (\".\") in key did not fail.");
		}

		bFailed = false;

		try
		{
			m_TagState.SetVariable("test", "test", null);
		} catch (XPIllegalArgumentException xpiae)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Setting Variable with null scope argument did not fail.");
		}

		bFailed = false;

		try
		{
			m_TagState.GetVariable(null, Scope.SESSION);
		} catch (XPIllegalArgumentException xpiae)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Getting Variable with null key argument did not fail.");
		}

		bFailed = false;

		try
		{
			m_TagState.GetVariable("test", null);
		} catch (XPIllegalArgumentException xpiae)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Getting Variable with null scope argument did not fail.");
		}

		m_TagState.ExitTag();
	}

	/**
	 * This tests Tag Scope variables.
	 */
	public void testTagScope()
	{
		Boolean bTest;
		Object oTest;

		m_TagState.EnterTag(GetNewTestTagInstance());  // tag 1

		m_TagState.EnterTag(GetNewTestTagInstance());  // tag 2

		// Use an object, since this is null, and booleans
		// are not objects in C#.
		oTest = m_TagState.GetVariable("test", Scope.TAG);
		Assert.assertNull("Getting empty variable test before it has been set.", oTest);

		m_TagState.SetVariable("test", new Boolean(true), Scope.TAG);

		bTest = (Boolean) m_TagState.GetVariable("test", Scope.TAG);
		Assert.assertTrue("Getting variable test that has been set to true in this tag.", XPConvert.ToBoolean(bTest));

		m_TagState.EnterTag(GetNewTestTagInstance()); // tag 3

		bTest = (Boolean) m_TagState.GetVariable("test", Scope.TAG);
		Assert.assertTrue("Getting variable test that has been set to true in a parent tag.", XPConvert.ToBoolean(bTest));

		m_TagState.SetVariable("test", new Boolean(false), Scope.TAG);

		bTest = (Boolean) m_TagState.GetVariable("test", Scope.TAG);
		Assert.assertFalse("Getting variable test that has been set to false in this tag.", XPConvert.ToBoolean(bTest));

		m_TagState.ExitTag(); // tag 3

		bTest = (Boolean) m_TagState.GetVariable("test", Scope.TAG);
		Assert.assertFalse("Getting variable test that has been set to false in another tag.", XPConvert.ToBoolean(bTest));

		m_TagState.SetVariable("test", new Boolean(true), Scope.TAG);

		bTest = (Boolean) m_TagState.GetVariable("test", Scope.TAG);
		Assert.assertTrue("Getting variable test that has been set to true in this tag.", XPConvert.ToBoolean(bTest));

		m_TagState.ExitTag(); // tag 2

		// Use an object, since we have just set this to null, and booleans
		// are not objects in C#.
		oTest = m_TagState.GetVariable("test", Scope.TAG);
		Assert.assertNull("Getting empty variable test before it has been set.", oTest);

		m_TagState.ExitTag(); // tag 1
	}

	/**
	 * This tests removing Tag Scope variables.
	 */
	public void testTagScopeRemoval()
	{
		Boolean bTest;
		Object oTest;

		// Test setting tags to null
		// Setting a tag to null should be the same as never having set the 
		// tag in the first place.

		m_TagState.EnterTag(GetNewTestTagInstance());  // tag 1

		m_TagState.SetVariable("test", new Boolean(true), Scope.TAG);

		m_TagState.EnterTag(GetNewTestTagInstance());  // tag 2

		// Make sure setting it to null and resetting it cause the
		// tag association to be changed (i.e. which tag is it in scope for)
		m_TagState.SetVariable("test", null, Scope.TAG);

		// Replace the reset tag using the current tag scope.
		m_TagState.SetVariable("test", new Boolean(true), Scope.TAG);

		m_TagState.ExitTag(); // tag 3

		// Use an object, since this variable has just passed out of tag scope,
		// and booleans are not objects in C#.
		oTest = m_TagState.GetVariable("test", Scope.TAG);
		Assert.assertNull("Getting empty variable test after it has passed out of tag scope.", oTest);
	}

	/**
	 * This tests Portlet Scope variables.
	 */
	public void testPortletScope()
	{
		// This method does not enter a tag so that it can be called from
		// within other test methods.

		Object oTest;
		Boolean bTest;

		m_TagState.ResetPortletRequestData();

		// Tag State cannot be used without entering a tag first.
		m_TagState.EnterTag(GetNewTestTagInstance());

		// Use an object, since this is null, and booleans
		// are not objects in C#.
		oTest =  m_TagState.GetVariable("test", Scope.PORTLET_REQUEST);
		Assert.assertNull("Getting empty variable test before it has been set.", oTest);

		m_TagState.SetVariable("test", new Boolean(true), Scope.PORTLET_REQUEST);

		bTest = (Boolean) m_TagState.GetVariable("test", Scope.PORTLET_REQUEST);
		Assert.assertTrue("Getting variable test that has been set to true in this tag.", XPConvert.ToBoolean(bTest));

		m_TagState.ExitTag();

		m_TagState.ResetPortletRequestData();

		// Tag State cannot be used without entering a tag first.
		m_TagState.EnterTag(GetNewTestTagInstance());

		// Use an object, since this is null, and booleans
		// are not objects in C#.
		oTest = m_TagState.GetVariable("test", Scope.PORTLET_REQUEST);
		Assert.assertNull("Getting empty variable test after portlet request has been cleared.", oTest);

		m_TagState.ExitTag();

		m_TagState.ResetPortletRequestData();
	}

	/**
	 * This tests HTTP Request Scope variables.
	 */
	public void testHTTPRequestScope()
	{
		// This method does not enter a tag so that it can be called from
		// within other test methods.

		Object oTest;
		Boolean bTest;

		m_TagState.ResetPortletRequestData();

		// Tag State cannot be used without entering a tag first.
		m_TagState.EnterTag(GetNewTestTagInstance());

		// Use an object, since this is null, and booleans
		// are not objects in C#.
		oTest =  m_TagState.GetVariable("test", Scope.HTTP_REQUEST);
		Assert.assertNull("Getting empty variable test before it has been set.", oTest);

		m_TagState.SetVariable("test", new Boolean(true), Scope.HTTP_REQUEST);

		bTest = (Boolean) m_TagState.GetVariable("test", Scope.HTTP_REQUEST);
		Assert.assertTrue("Getting variable test that has been set to true in this tag.", XPConvert.ToBoolean(bTest));

		m_TagState.ExitTag();

		m_TagState.ResetPortletRequestData();

		// Tag State cannot be used without entering a tag first.
		m_TagState.EnterTag(GetNewTestTagInstance());

		bTest = (Boolean) m_TagState.GetVariable("test", Scope.HTTP_REQUEST);
		Assert.assertTrue("Getting variable test that has been set to true in previous portlet.", XPConvert.ToBoolean(bTest));

		m_TagState.ExitTag();

		m_TagState.ResetPortletRequestData();
	}

	/**
	 * This tests shared variable edit access protection.
	 */
	public void testSharedVariableProtection()
	{
		m_TagState.EnterTag(GetNewTestTagInstance());

		m_TagState.SetSharedVariable(UNEDITABLE_KEY, new Boolean(true), Scope.PORTLET_REQUEST, true);

		Boolean bTest = (Boolean) m_TagState.GetSharedVariable(UNEDITABLE_KEY, Scope.PORTLET_REQUEST);
		Assert.assertTrue("Getting shared variable test that has been set to true in this tag.", XPConvert.ToBoolean(bTest));

		m_TagState.SetSharedVariable(EDITABLE_KEY, new Boolean(true), Scope.PORTLET_REQUEST, false);

		bTest = (Boolean) m_TagState.GetSharedVariable(EDITABLE_KEY, Scope.PORTLET_REQUEST);
		Assert.assertTrue("Getting shared variable test that has been set to true in this tag.", XPConvert.ToBoolean(bTest));

		m_TagState.ExitTag();

		m_TagState.EnterTag(GetNewSecondTestTagInstance());

		bTest = (Boolean) m_TagState.GetSharedVariable(UNEDITABLE_KEY, Scope.PORTLET_REQUEST);
		Assert.assertTrue("Getting uneditable shared variable test that has been set to true in another tag.", XPConvert.ToBoolean(bTest));

		boolean bFailed = false;

		try
		{
			m_TagState.SetSharedVariable(UNEDITABLE_KEY, new Boolean(false), Scope.PORTLET_REQUEST, false);
		} catch (XPIllegalAccessException xpiae)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Setting Shared Variable as editable without proper edit access did not fail.");
		}

		try
		{
			m_TagState.SetSharedVariable(UNEDITABLE_KEY, new Boolean(false), Scope.PORTLET_REQUEST, true);
		} catch (XPIllegalAccessException xpiae)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Setting Shared Variable as uneditable without proper edit access did not fail.");
		}

		bTest = (Boolean) m_TagState.GetSharedVariable(EDITABLE_KEY, Scope.PORTLET_REQUEST);
		Assert.assertTrue("Getting editable shared variable test that has been set to true in another tag.", XPConvert.ToBoolean(bTest));

		m_TagState.SetSharedVariable(EDITABLE_KEY, new Boolean(false), Scope.PORTLET_REQUEST, false);

		bTest = (Boolean) m_TagState.GetSharedVariable(EDITABLE_KEY, Scope.PORTLET_REQUEST);
		Assert.assertFalse("Getting editable shared variable test that has been set to false in this tag.", XPConvert.ToBoolean(bTest));

		m_TagState.ExitTag();
	}

	/**
	 * This helper method creates and initializes a new TestTag instance.
	 * This would normally be handled by the TagLibManager.
	 * 
	 * @return  A new TestTag instance.
	 */
	protected TestTag GetNewTestTagInstance()
	{
		TestTag result = new TestTag();
		result.SetTagMetaDataHelper(m_TestTag.GetTagMetaDataHelper());

		return result;
	}

	/**
	 * This helper method creates and initializes a new TestSecondTestTagTag
	 * instance.  This would normally be handled by the TagLibManager.
	 * 
	 * @return  A new SecondTestTag instance.
	 */
	protected SecondTestTag GetNewSecondTestTagInstance()
	{
		SecondTestTag result = new SecondTestTag();
		result.SetTagMetaDataHelper(m_SecondTestTag.GetTagMetaDataHelper());

		return result;
	}
}
