package com.plumtree.portaluiinfrastructure.test.tags.metadata;

import junit.framework.*;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;

/**
 * Tests for the Tag Attribute meta data classes.
 * 
 * @author donh
 */
public class AttributeTest extends TestCase
{
	/**
	 * Constructor.
	 */
	public AttributeTest(String name)
	{
		super(name);
	}

	/**
	 * This tests invalid values for required tag attribute names.
	 */
	public void testInvalidRequiredAttributeNames()
	{
		// Test to make sure safe name works
		new RequiredTagAttribute("test", "description");

		boolean bFailed = false;

		try
		{
			new RequiredTagAttribute("test test", "description");
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Required Tag Attribute with a space in the name did not fail.");
		}

		bFailed = false;
		
		try
		{
			new RequiredTagAttribute("test	test", "description", AttributeType.CHAR);
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Required Tag Attribute with a tab in the name did not fail.");
		}

		bFailed = false;
		
		try
		{
			new RequiredTagAttribute("test\rtest", "description", AttributeType.BOOLEAN);
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Required Tag Attribute with a carriage return in the name did not fail.");
		}

		bFailed = false;
		
		try
		{
			new RequiredTagAttribute("test\ntest", "description", AttributeType.LONG);
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Required Tag Attribute with a new line in the name did not fail.");
		}
	}

	/**
	 * This tests invalid values for optional tag attribute names.
	 */
	public void testInvalidOptionalAttributeNames()
	{
		// Test to make sure safe name works
		new OptionalTagAttribute("test", "description", AttributeType.STRING, "test");

		boolean bFailed = false;

		try
		{
			new OptionalTagAttribute("test test", "description", AttributeType.STRING, "test");
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Optional Tag Attribute with a space in the name did not fail.");
		}

		bFailed = false;
		
		try
		{
			new OptionalTagAttribute("test	test", "description", AttributeType.STRING, "test");
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Optional Tag Attribute with a tab in the name did not fail.");
		}

		bFailed = false;
		
		try
		{
			new OptionalTagAttribute("test\rtest", "description", AttributeType.STRING, "test");
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Optional Tag Attribute with a carriage return in the name did not fail.");
		}

		bFailed = false;
		
		try
		{
			new OptionalTagAttribute("test\ntest", "description", AttributeType.STRING, "test");
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Optional Tag Attribute with a new line in the name did not fail.");
		}
	}

	/**
	 * This tests for invalid tag attribute types.
	 */
	public void testInvalidAttributeType()
	{
		boolean bFailed = false;

		try
		{
			new OptionalTagAttribute("test", "description", null, "test");
		} catch (XPIllegalArgumentException xpiae)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Optional Tag Attribute with null type did not fail.");
		}

		bFailed = false;
		
		try
		{
			new RequiredTagAttribute("test", "description", null);
		} catch (XPIllegalArgumentException xpiae)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Required Tag Attribute with null type did not fail.");
		}
	}

	/**
	 * This tests invalid default values for optional tag attributes.
	 */
	public void testInvalidOptionalAttributes()
	{
		boolean bFailed = false;

		try
		{
			new OptionalTagAttribute("test", "description", AttributeType.BOOLEAN, "1.0");
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Optional Tag Attribute of type boolean with non-boolean default value did not fail.");
		}

		bFailed = false;
		
		try
		{
			new OptionalTagAttribute("test", "description", AttributeType.CHAR, "1.0");
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Optional Tag Attribute of type char with non-char default value did not fail.");
		}

		bFailed = false;
		
		try
		{
			new OptionalTagAttribute("test", "description", AttributeType.DOUBLE, "test");
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Optional Tag Attribute of type double with non-double default value did not fail.");
		}

		bFailed = false;
		
		try
		{
			new OptionalTagAttribute("test", "description", AttributeType.INT, "1.0");
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Optional Tag Attribute of type int with non-int default value did not fail.");
		}

		bFailed = false;
		
		try
		{
			new OptionalTagAttribute("test", "description", AttributeType.LONG, "test");
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Creating Optional Tag Attribute of type long with non-long default value did not fail.");
		}

		// Default values of type String cannot fail because they would
		// fail at compile time.
	}

	/**
	 * This tests valid default values for optional tag attributes.
	 */
	public void testValidOptionalAttributes()
	{
		new OptionalTagAttribute("test", "description", AttributeType.BOOLEAN, "true");
		new OptionalTagAttribute("test", "description", AttributeType.CHAR, "c");
		new OptionalTagAttribute("test", "description", AttributeType.DOUBLE, "1.0");
		new OptionalTagAttribute("test", "description", AttributeType.INT, "1");
		new OptionalTagAttribute("test", "description", AttributeType.LONG, "1");
		new OptionalTagAttribute("test", "description", AttributeType.STRING, "test");
	}

	/**
	 * This tests valid values for type safety validation.
	 */
	public void testValidAttributeTypeSafetyValidation()
	{
		AttributeType.ValidateTypeSafety("true", AttributeType.BOOLEAN);
		AttributeType.ValidateTypeSafety("c", AttributeType.CHAR);
		AttributeType.ValidateTypeSafety("1.0", AttributeType.DOUBLE);
		AttributeType.ValidateTypeSafety("1", AttributeType.INT);
		AttributeType.ValidateTypeSafety("1", AttributeType.LONG);
//		AttributeType.ValidateTypeSafety("tag", AttributeType.SCOPE);
	}

	/**
	 * This tests invalid values for type safety validation.
	 */
	public void testInValidAttributeTypeSafetyValidation()
	{
		boolean bFailed = false;
		
		try
		{
			AttributeType.ValidateTypeSafety("truesdf", AttributeType.BOOLEAN);
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Type safety check for invalid boolean did not fail.");
		}

		bFailed = false;
		
		try
		{
			AttributeType.ValidateTypeSafety("cd", AttributeType.CHAR);
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Type safety check for invalid char did not fail.");
		}

		bFailed = false;
		
		try
		{
			AttributeType.ValidateTypeSafety("1.0sdf", AttributeType.DOUBLE);
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Type safety check for invalid double did not fail.");
		}

		bFailed = false;
		
		try
		{
			AttributeType.ValidateTypeSafety("1.0", AttributeType.INT);
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Type safety check for invalid int did not fail.");
		}

		bFailed = false;
		
		try
		{
			AttributeType.ValidateTypeSafety("1.0", AttributeType.LONG);
		} catch (XPFormatException xpfe)
		{
			// Expected result
			bFailed = true;
		}

		if (!bFailed)
		{
			fail("Type safety check for invalid long did not fail.");
		}

		bFailed = false;
		
//		try
//		{
//			AttributeType.ValidateTypeSafety("invalid scope", AttributeType.SCOPE);
//		} catch (XPFormatException xpfe)
//		{
//			// Expected result
//			bFailed = true;
//		}
//
//		if (!bFailed)
//		{
//			fail("Type safety check for invalid Scope did not fail.");
//		}
	}
}