/**
 * 
 */
package com.plumtree.portaluiinfrastructure.utils;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.mock.tags.*;
import com.plumtree.portaluiinfrastructure.tags.helper.*;

/**
 * Helper classes to access tag variables from unit tests
 * @author rozhang
 *
 */
public class TagStateTestHelper
{
	protected XPHashtable m_portletRequestMap = new XPHashtable();
	
	public TagStateTestHelper()
	{
		// Seed the test variable in Tag state
		TagState.TEST_PORTLET_REQUEST_DATA = m_portletRequestMap;
	}
	
	/**
	 * Get tag variable from Portlet Request Scope
	 * Portlet Request Scope only exists within one execution of a DisplayTree.ProcessTagMarkup call
	 * 
	 * @param varName Tag variable name, if the tag variable name contains a '.' this function assumes that the taglibrary name is
	 *                already included in the name
	 * @param env
	 * @return null if the variable is not found
	 */
	public Object GetTagPortletRequestVar(String varName)
	{
		// tag variable names are prefixed with their tag library in the scopedata map, but most tags store in the shared scope
		// so they are prefixed with a '.'. In case the variable name contains a dot, assume that the caller already included
		// it so don't add it again
		String adjustedTagVarName = (varName.indexOf(".") < 0) ? "."+varName : varName;
		SharedStateVariable stateVar = (SharedStateVariable) m_portletRequestMap.GetElement(adjustedTagVarName);

		if ( stateVar == null ) {	
			return null;
		} else {
			return stateVar.GetValue();
		}
	}
	
	/**
	 * Set tag variable from Portlet Request Scope
	 * Portlet Request Scope only exists within one execution of a DisplayTree.ProcessTagMarkup call
	 * @param varLibName
	 * @param varName
	 * @param varValue
	 * @return
	 */
	public Object SetTagPortletRequestVar(String varLibName, String varName, Object varValue)
	{
		String finalVarName = (varLibName==null||"".equals(varLibName))? "."+varName : varLibName+"."+varName;
		SharedStateVariable stateVar = new SharedStateVariable(varValue);
		return m_portletRequestMap.PutElement(finalVarName, stateVar);
	}
	
	/**
	 * Get tag variable in the HTTP Request Scope
	 * HTTP Request Scope variables persists between portlets runs (Portlet Request Scope)
	 * 
	 * @param varName
	 * @param env
	 * @return
	 */
	public Object GetTagHTTPRequestVar(String varName, MockEnvironment env)
	{
		XPHashtable scopeData = TagState.GetHTTPRequestData(env);

		// tag variable names are prefixed with their tag library in the scopedata map, but most tags store in the shared scope
		// so they are prefixed with a '.'. In case the variable name contains a dot, assume that the caller already included
		// it so don't add it again
		String adjustedTagVarName = (varName.indexOf(".") < 0) ? "."+varName : varName;
		SharedStateVariable stateVar = (SharedStateVariable) scopeData.GetElement(adjustedTagVarName);
		
		if ( stateVar == null ) {	
			return null;
		} else {
			return stateVar.GetValue();
		}
	}
	/**
	 * Set tag variable in the HTTP Request Scope
	 * HTTP Request Scope variables persists between portlets runs (Portlet Request Scope)
	 * @param varLibName
	 * @param varName
	 * @param varValue
	 * @param env
	 * @return
	 */
	public Object SetTagHTTPRequestVar(String varLibName, String varName, Object varValue, MockEnvironment env)
	{
		String finalVarName = (varLibName==null||"".equals(varLibName))? "."+varName : varLibName+"."+varName;
		XPHashtable httpReqScopeData = TagState.GetHTTPRequestData(env);
		SharedStateVariable tagVarValue = new SharedStateVariable(varValue);
		return httpReqScopeData.PutElement(finalVarName, tagVarValue);
	}
	
	/**
	 * Get Tag variable on session
	 * Session variables aren't stored as SharedStateVariable type like HTTP and PORTLET REQUEST tag variables
	 * 
	 * @param varName
	 * @param env
	 * @return
	 */
	public Object GetTagSessionVar(String varName, MockEnvironment env)
	{
		String adjustedTagVarName = (varName.indexOf(".") < 0) ? "."+varName : varName;
		Object varValue = env.GetSubSession().GetAttribute(adjustedTagVarName);
		return varValue;
	}
	
	/**
	 * Set Tag variable on session
	 * Session variables aren't stored as SharedStateVariable type like HTTP and PORTLET REQUEST tag variables
	 * 
	 * @param varLibName
	 * @param varName
	 * @param varValue
	 * @param env
	 */
	public void SetTagSessionVar(String varLibName, String varName, Object varValue, MockEnvironment env)
	{
		String finalVarName = (varLibName==null||"".equals(varLibName))? "."+varName : varLibName+"."+varName;
		env.GetSubSession().SetAttribute(finalVarName, varValue);
	}
	
	
	/**
	 * Get Tag variable from Application
	 * Session variables aren't stored as SharedStateVariable type like HTTP and PORTLET REQUEST tag variables
	 * 
	 * @param varName
	 * @param env
	 * @return
	 */
	public Object GetTagApplicationVar(String varName, MockEnvironment env)
	{
		String adjustedTagVarName = (varName.indexOf(".") < 0) ? "."+varName : varName;
		Object varValue = env.GetApplication().GetCachingManager().GetEntry(adjustedTagVarName);
		return varValue;
	}
	
	/**
	 * Set Tag variable from Application
	 * Session variables aren't stored as SharedStateVariable type like HTTP and PORTLET REQUEST tag variables
	 * 
	 * @param varLibName
	 * @param varName
	 * @param varValue
	 * @param env
	 */
	public void SetTagApplicationVar(String varLibName, String varName, Object varValue, MockEnvironment env)
	{
		String finalVarName = (varLibName==null||"".equals(varLibName))? "."+varName : varLibName+"."+varName;
		env.GetApplication().GetCachingManager().SetEntry(finalVarName, varValue);
	}
}