using System;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.common
{
  public class HeadFilterUtil
  {
    // These patterns search for script or link elements that include URLs.
    // They also match the three basic patterns of HTML elements:
    // <el>, <el></el>, and <el/>
    public static readonly XPRegex JS_PATTERN = new XPRegex("(<script [^>]*src=[\"\']([^\"\']*)[\"\'][^>]*(>\\s*</script>|>))",
     XPRegex.IGNORE_CASE);
    public const String JS_START = "src=";
    public static readonly XPRegex CSS_PATTERN = new XPRegex("(<link [^>]*href=[\"\']([^\"\']*)[\"\'][^>]*(>\\s*</link>|>))",
     XPRegex.IGNORE_CASE);
    public const String CSS_START = "href=";
    // This List contains JS URLs that are displayed in this content.
    protected XPArrayList m_newJavaScript;
    // This List contains CSS URLs that are displayed in this content.
    protected XPArrayList m_newCSS;
    protected HTMLElement m_displayContent;
    
    /// <summary>
    /// This method filters the content for .js and .css URLS and
    /// removes them if necessary.  It also adds new .js and .css
    /// files to lists.
    /// </summary>
    /// <param name="_content"></param>
    /// <param name="_javaScriptForRemoval"></param>
    /// <param name="_cssForRemoval"></param>
    /// <returns>false implies failure.</returns>
    public virtual bool Filter(HTMLElement _content, XPArrayList _javaScriptForRemoval,
     XPArrayList _cssForRemoval)
    {
      m_newJavaScript = new XPArrayList();
      m_newCSS = new XPArrayList();
      m_displayContent = new HTMLElementCollection();
      if (_javaScriptForRemoval == null)
      {
        _javaScriptForRemoval = new XPArrayList();
      }
      if (_cssForRemoval == null)
      {
        _cssForRemoval = new XPArrayList();
      }
      String strContent = _content.GetDisplayString();
      // NOTE: This algorithm makes a lot of substring objects.
      // This could probably be optimized by copying the content
      // from strContent to the filteredContent SB char by char.
      // Search for JavaScript
      XPMatch[] jsMatches = JS_PATTERN.Matches(strContent);
      XPStringBuilder filteredContent = new XPStringBuilder();
      int nEndOfLastMatch = 0;
      for (int i = 0;i < jsMatches.Length;i++)
      {
        String strMatch = jsMatches[i].GetValue();
        // Find the URL in the match
        int nSrcIndex = XPStringUtility.IndexOf(strMatch, JS_START);
        // Skip over the (single or double) quote
        String strURL = XPStringUtility.Substring(strMatch, nSrcIndex + JS_START.Length + 1);
        int nURLEndIndex = XPStringUtility.IndexOf(strURL, "\"");
        if (nURLEndIndex < 0)
        {
          // Attributes can be either single or double quoted.
          nURLEndIndex = XPStringUtility.IndexOf(strURL, "\'");
        }
        strURL = XPStringUtility.Substring(strURL, 0, nURLEndIndex);
        int nStartOfMatchIndex = jsMatches[i].GetIndex();
        if (_javaScriptForRemoval.Contains(strURL))
        {
          // If we are supposed to remove this script,
          // just append the section before it to the result.
          filteredContent.Append(XPStringUtility.Substring(strContent, nEndOfLastMatch,
           nStartOfMatchIndex));
        } else
        {
          // If we are supposed to display this script,
          // append it and the section before it to the result.
          filteredContent.Append(XPStringUtility.Substring(strContent, nEndOfLastMatch,
           nStartOfMatchIndex + strMatch.Length));
          m_newJavaScript.Add(strURL);
        }
        nEndOfLastMatch = nStartOfMatchIndex + strMatch.Length;
      }
      // append the rest
      filteredContent.Append(XPStringUtility.Substring(strContent, nEndOfLastMatch));
      strContent = filteredContent.ToString();
      // Search for CSS
      XPMatch[] cssMatches = CSS_PATTERN.Matches(strContent);
      filteredContent = new XPStringBuilder();
      nEndOfLastMatch = 0;
      for (int i = 0;i < cssMatches.Length;i++)
      {
        String strMatch = cssMatches[i].GetValue();
        // Find the URL in the match
        int nSrcIndex = XPStringUtility.IndexOf(strMatch, CSS_START);
        // Skip over the (single or double) quote
        String strURL = XPStringUtility.Substring(strMatch, nSrcIndex + CSS_START.Length + 1);
        int nURLEndIndex = XPStringUtility.IndexOf(strURL, "\"");
        if (nURLEndIndex < 0)
        {
          // Attributes can be either single or double quoted.
          nURLEndIndex = XPStringUtility.IndexOf(strURL, "\'");
        }
        strURL = XPStringUtility.Substring(strURL, 0, nURLEndIndex);
        int nStartOfMatchIndex = cssMatches[i].GetIndex();
        if (_cssForRemoval.Contains(strURL))
        {
          // If we are supposed to remove this script,
          // just append the section before it to the result.
          filteredContent.Append(XPStringUtility.Substring(strContent, nEndOfLastMatch,
           nStartOfMatchIndex));
        } else
        {
          // If we are supposed to display this script,
          // append it and the section before it to the result.
          filteredContent.Append(XPStringUtility.Substring(strContent, nEndOfLastMatch,
           nStartOfMatchIndex + strMatch.Length));
          m_newCSS.Add(strURL);
        }
        nEndOfLastMatch = nStartOfMatchIndex + strMatch.Length;
      }
      // append the rest
      filteredContent.Append(XPStringUtility.Substring(strContent, nEndOfLastMatch));
      strContent = filteredContent.ToString();
      m_displayContent.AddInnerHTMLString(strContent);
      return true;
    }
    
    /// <summary>
    /// This method returns a List of new JS URLs.
    /// </summary>
    /// <returns></returns>
    public virtual XPArrayList GetNewJavaScript()
    {
      return m_newJavaScript;
    }
    
    /// <summary>
    /// This method returns a List of new CSS URLs.
    /// </summary>
    /// <returns></returns>
    public virtual XPArrayList GetNewCSS()
    {
      return m_newCSS;
    }
    public virtual HTMLElement GetDisplayContent()
    {
      return m_displayContent;
    }
  }
}