using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.common
{
  /// <summary>
  /// This marks JavaScript & CSS information to be included in the
  /// Head element of the HTML page by the headincludes tag.
  /// </summary>
  public class IncludeInHeadTag : ATag
  {
    public const String INCLUDE_IN_HEAD_EXAMPLE = "<![CDATA[&lt;pt:common.includeinhead&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&nbsp;&nbsp;&lt;script type=\"text/javascript\"&gt;&lt;!-- JavaScript --&gt;&lt;/script&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&nbsp;&nbsp;&lt;script type=\"text/javascript\" src=\"http://test.com/test.js\"&gt;&lt;/script&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&nbsp;&nbsp;&lt;link type=\"text/css\" rel=\"stylesheet\" href=\"http://test.com/test.css\"&gt;&lt;/link&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;/pt:common.includeinhead&gt;]]>";
    
    /// <summary>
    /// This key is used to store a List<String> containing the previously
    /// included JavaScript file names. This will be stored in HTTP Request scope.
    /// </summary>
    public const String JAVASCRIPT_LIST = "JavaScriptList";
    
    /// <summary>
    /// This key is used to store a List<String> containing the previously
    /// included CSS file names. This will be stored in HTTP Request scope.
    /// </summary>
    public const String CSS_LIST = "CSSList";
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    static IncludeInHeadTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("includeinhead", "This marks JavaScript and CSS information to be included in the " + "Head element of the HTML page by the headincludes tag.  " + HeadIncludesTag.AUTOMATIC_HEAD_INSERTION + "  If a .js or .css " + "file is marked for inclusion multiple times, possibly in multiple " + "files, it will only be included once.  This tag will be ignored " + "during automatic in-place refresh requests.  Custom in-place " + "refresh solutions will need to ensure that their JavaScript " + "gets included correctly. ",
       INCLUDE_IN_HEAD_EXAMPLE);
    }
    public override HTMLElement DisplayTag()
    {
      XPArrayList filterJavaScript = GetDisplayedJavaScript();
      XPArrayList filterCSS = GetDisplayedCSS();
      // This parses the body for JS & CSS files.
      HeadFilterUtil util = new HeadFilterUtil();
      util.Filter(ProcessTagBody(), filterJavaScript, filterCSS);
      // Add all new JS to the list of included URLs.
      filterJavaScript.AddRange(util.GetNewJavaScript());
      // Add all new CSS to the list of included URLs.
      filterCSS.AddRange(util.GetNewCSS());
      // Add the remaining content from the body.
      HTMLScriptCollection scripts = new HTMLScriptCollection();
      scripts.AddInnerHTMLElement(util.GetDisplayContent());
      AddJavascript(scripts);
      return null;
    }
    public virtual XPArrayList GetDisplayedJavaScript()
    {
      XPArrayList result;
      Object oList = GetStateVariable(JAVASCRIPT_LIST, Scope.HTTP_REQUEST);
      if (oList == null)
      {
        result = new XPArrayList();
        SetStateVariable(JAVASCRIPT_LIST, result, Scope.HTTP_REQUEST);
      } else
      {
        result = ConvertObjectToList(oList);
      }
      return result;
    }
    public virtual XPArrayList GetDisplayedCSS()
    {
      XPArrayList result;
      Object oList = GetStateVariable(CSS_LIST, Scope.HTTP_REQUEST);
      if (oList == null)
      {
        result = new XPArrayList();
        SetStateVariable(CSS_LIST, result, Scope.HTTP_REQUEST);
      } else
      {
        result = ConvertObjectToList(oList);
      }
      return result;
    }
    
    /// <summary>
    /// This method is used to convert an object to a String list
    /// without displaying a compiler warning.  This should only be
    /// used for converting the JS & CSS lists in the IncludeInHead Tag.
    /// This is ported from java 1.5 code and doesn't really need to be here.
    /// </summary>
    /// <param name="_oList"></param>
    /// <returns>The converted list</returns>
    public static XPArrayList ConvertObjectToList(Object _oList)
    {
      return (XPArrayList) _oList;
    }
    public override ATag Create()
    {
      return new IncludeInHeadTag();
    }
  }
}