package com.plumtree.taglib.common;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.server.marshalers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag displays the current error text on the page so that
 * it can be placed and formatted as desired. Only the first error
 * message will be displayed.  Other errors, as well as exception
 * stack traces and extended error messages will be ignored.  If
 * this tag is displayed on a page, errors will no longer be
 * displayed in the normal error location.
 */
public class ErrorCodeTag extends ATag
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		"com.plumtree.taglib.common.ErrorCodeTag");

	public static final String ERROR_CODE_EXAMPLE =
		"<![CDATA[&lt;pt:common.error&gt;]]><![CDATA[<BR>]]>" +
		"<![CDATA[&lt;pt:common.errortext/&gt;]]><![CDATA[<BR>]]>" +
		"<![CDATA[&lt;pt:common.errorcode pt:key=\"myerrorcode\"/&gt;]]><![CDATA[<BR>]]>" +
		"<![CDATA[&lt;/pt:common.error&gt;]]>";

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute KEY;
	public static final OptionalTagAttribute SCOPE;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("errorcode",
			"This tag stores a collection of the current error codes in memory. " +
			"If the " + ErrorTag.TAG.GetName() + " has been displayed, no error " +
			"codes will be available.  These error codes can be accessed using " +
			"the logic.foreach tag.", ERROR_CODE_EXAMPLE);

		KEY = new RequiredTagAttribute("key",
			"The key used to store the error code collection in memory.");

		SCOPE = new OptionalTagAttribute("scope",
			"The scope used to store the error code collection.  " +
			CommonTagLibraryData.SCOPE_REFERENCE,
			AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
	}

	public HTMLElement DisplayTag()
	{
		Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));

		if (null == scope)
		{
			log.Info("Unable to convert scope attribute " +
				GetTagAttributeAsString(SCOPE) + " to a valid scope in Tag " +
				TAG.GetName() + ".");

			scope = Scope.PORTLET_REQUEST;
		}

		AActivitySpace currentAS = (AActivitySpace)GetEnvironment();

		// Get errors associated with this activity space
		IXPEnumerator eErrors = UIErrorHandler.GetErrors(currentAS.GetSubSession(),
			currentAS.GetApplication(), currentAS, true,
			UIErrorHandler.GetDisplayAdminErrors(currentAS));

		XPArrayList errorCodeCollection = new XPArrayList();

		// Copy the error codes into a new array
		while (eErrors.MoveNext())
		{
			Exception e = ((UIError)eErrors.GetCurrent()).GetException();
			if (e instanceof PTException)
			{
				errorCodeCollection.Add(new Integer(((PTException) e).GetErrorCode()));
			}
		}

		// Store the error code using the specified name.
		SetStateSharedVariable(GetTagAttributeAsString(KEY),
			errorCodeCollection, scope, false);

		return ProcessTagBody();
	}

	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// This tag does not display any content by itself, and therefore
		// supports all access styles.
		return true;
	}

	public ATag Create()
	{
		return new ErrorCodeTag();
	}
}