package com.plumtree.taglib.common;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag displays errors on the page so that they can be
 * placed and formatted as desired.  If this tag is displayed
 * on a page, errors will no longer be displayed in the normal
 * error location.
 */
public class ErrorTag extends ATag
{
	public static final String ERROR_EXAMPLE =
		"<![CDATA[&lt;pt:common.error/&gt;]]><![CDATA[<br>]]>";

	public static final String ERROR_TEXT_EXAMPLE =
		"<![CDATA[&lt;pt:common.error&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;p&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;pt:common.errortext/&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;/p&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;/pt:common.error&gt;]]>";

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RelatedChildTag ERROR_TEXT;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("error",
			"This tag displays errors on the page so that they can be " +
			"placed and formatted as desired.  If the " + ErrorTextTag.TAG.GetName() + " " +
			"tag is included inside this tag, the contents of this tag will " +
			"only be processed if there is an error.  If the child tag is not " +
			"present, then the error messages will be formatted and displayed " +
			"from this tag in the standard style.  If this tag is displayed " +
			"on a page, errors will no longer be displayed in the normal error " +
			"location or available after this tag has been displayed.",
			ERROR_EXAMPLE + "or<![CDATA[<br>]]>" + ERROR_TEXT_EXAMPLE);

		ERROR_TEXT = new RelatedChildTag(ErrorTextTag.TAG);
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		AActivitySpace currentAS = (AActivitySpace)GetEnvironment();

		// Get errors associated with this activity space
		IXPEnumerator eErrors = UIErrorHandler.GetErrors(currentAS.GetSubSession(),
			currentAS.GetApplication(), currentAS, true,
			UIErrorHandler.GetDisplayAdminErrors(currentAS));

		// If we have a child error text tag
		if (HasChildTag(ErrorTextTag.TAG))
		{
			// and we have errors
			if (eErrors.MoveNext())
			{
				// Display the inner contents of this tag
				HTMLElement result = ProcessTagBody();

				// Clear all errors
				UIErrorHandler.ClearErrors(currentAS.GetSubSession(),
					currentAS.GetApplication());

				return result;
			}
		} else
		{
			HTMLElement result = new HTMLElementCollection();

			// Generate HTML comments with the errors
			HTMLElementCollection comments = new HTMLElementCollection();

			while (eErrors.MoveNext())
			{
				UIError error = (UIError) eErrors.GetCurrent();

				String strFullException = "";
				if (error.GetException() != null)
				{
					strFullException = XPException.ExceptionToString(error.GetException());
				}

				// We put the remaining information in HTMLComments.
				comments.AddInnerHTMLElement(
						new HTMLComment(TaskAPIUIHTML.HTMLEncode(	error.GetMessage() + " " + strFullException) ) ); 
			}

			// Get a new enumerator for the errors associated with this activity space
			eErrors = UIErrorHandler.GetErrors(currentAS.GetSubSession(),
				currentAS.GetApplication(), currentAS, true,
				UIErrorHandler.GetDisplayAdminErrors(currentAS));

			// We want to display the full list of errors formatted in HTML.
			ErrorDisplayHelper helper = new ErrorDisplayHelper(currentAS, eErrors);

			// NOTE: this will clear all UI Errors for this user.
			result.AddInnerHTMLElement(helper.DisplayErrors());
			// Add errors in HTML comments.
			result.AddInnerHTMLElement(comments);

			return result;
		}

		// otherwise don't display anything (no errors)
		return null;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// This tag does not display any content by itself, and therefore
		// supports all access styles.
		return true;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new ErrorTag();
	}
}