package com.plumtree.taglib.common;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag displays the current error text on the page so that
 * it can be placed and formatted as desired. Only the first error
 * message will be displayed.  Other errors, as well as exception
 * stack traces and extended error messages will be ignored.  If
 * this tag is displayed on a page, errors will no longer be
 * displayed in the normal error location.
 */
public class ErrorTextTag extends ATag
{
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final OptionalTagAttribute TEXT;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("errortext",
			"This tag displays the current error text on the page so that " +
			"it can be placed and formatted as desired. Only the first error " +
			"message will be displayed.  Other errors, as well as exception " +
			"stack traces and extended error messages will be ignored.  If " +
			"this tag is displayed on a page, errors will no longer be " +
			"displayed in the normal error location.", ErrorTag.ERROR_TEXT_EXAMPLE);

		TEXT = new OptionalTagAttribute(
			"text", "This attribute allows you to override the error text with " +
			"text of your own choosing.  For instance, this could be used to " +
			"provide custom error messages on multiple login pages. It is probably " +
			"not appropriate for use on pages that can have many possible error " +
			"messages.", AttributeType.STRING, "");
	}

	public HTMLElement DisplayTag()
	{
		HTMLElement result = new HTMLElementCollection();

		AActivitySpace currentAS = (AActivitySpace)GetEnvironment();

		// Get errors associated with this activity space
		IXPEnumerator eErrors = UIErrorHandler.GetErrors(currentAS.GetSubSession(),
			currentAS.GetApplication(), currentAS, true,
			UIErrorHandler.GetDisplayAdminErrors(currentAS));

		HTMLElementCollection comments = new HTMLElementCollection();
		String strFirstError = null;

		while (eErrors.MoveNext())
		{
			UIError error = (UIError) eErrors.GetCurrent();

			if (strFirstError == null)
			{
				// Store the first error message for later display
				strFirstError = error.GetMessage();
			}

			String strFullException = "";
			if (error.GetException() != null)
			{
				strFullException = XPException.ExceptionToString(error.GetException());
			}

			// We put the remaining information in HTMLComments.
			comments.AddInnerHTMLElement(new HTMLComment(error.GetMessage() + " " +
				strFullException));
		}

		String strCustomErrorText = GetTagAttributeAsString(TEXT);

		// If we don't have a custom error message
		if ((null == strCustomErrorText) ||
			"".equals(strCustomErrorText))
		{
			// We want to display the first error only
			result.AddInnerHTMLElement(new HTMLGenericElement(strFirstError));
		} else
		{
			result.AddInnerHTMLEncodedString(strCustomErrorText);
		}

		// Display remaining errors in HTML comments
		result.AddInnerHTMLElement(comments);

		return result;
	}

	public TagType GetTagType()
	{
		return TagType.NO_BODY;
	}

	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// This tag does not display any content by itself, and therefore
		// supports all access styles.
		return true;
	}

	public ATag Create()
	{
		return new ErrorTextTag();
	}
}