package com.plumtree.taglib.common;

import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlelements.*;

public class HeadFilterUtil
{
	// These patterns search for script or link elements that include URLs.
	// They also match the three basic patterns of HTML elements:
	// <el>, <el></el>, and <el/>

	public static final XPRegex JS_PATTERN =
		new XPRegex("(<script [^>]*src=[\"']([^\"']*)[\"'][^>]*(>\\s*</script>|>))",
		XPRegex.IGNORE_CASE);
	public static final String JS_START = "src=";

	public static final XPRegex CSS_PATTERN =
		new XPRegex("(<link [^>]*href=[\"']([^\"']*)[\"'][^>]*(>\\s*</link>|>))",
		XPRegex.IGNORE_CASE);
	public static final String CSS_START = "href=";

	// This List contains JS URLs that are displayed in this content.
	protected XPArrayList m_newJavaScript;

	// This List contains CSS URLs that are displayed in this content.
	protected XPArrayList m_newCSS;

	protected HTMLElement m_displayContent;

	/**
	 * This method filters the content for .js and .css URLS and
	 * removes them if necessary.  It also adds new .js and .css
	 * files to lists.
	 * 
	 * @param _content
	 * @param _javaScriptForRemoval
	 * @param _cssForRemoval
	 * @return false implies failure.
	 */
	public boolean Filter(HTMLElement _content, XPArrayList _javaScriptForRemoval,
		XPArrayList _cssForRemoval)
	{
		m_newJavaScript = new XPArrayList();
		m_newCSS = new XPArrayList();
		m_displayContent = new HTMLElementCollection();

		if (_javaScriptForRemoval == null)
		{
			_javaScriptForRemoval = new XPArrayList();
		}

		if (_cssForRemoval == null)
		{
			_cssForRemoval = new XPArrayList();
		}

		String strContent = _content.GetDisplayString();

		// NOTE: This algorithm makes a lot of substring objects.
		// This could probably be optimized by copying the content
		// from strContent to the filteredContent SB char by char.

		// Search for JavaScript

		XPMatch[] jsMatches = JS_PATTERN.Matches(strContent);

		XPStringBuilder filteredContent = new XPStringBuilder();
		int nEndOfLastMatch = 0;

		for (int i = 0; i < jsMatches.length; i++)
		{
			String strMatch = jsMatches[i].GetValue();

			// Find the URL in the match
			int nSrcIndex = XPStringUtility.IndexOf(strMatch, JS_START);
			// Skip over the (single or double) quote
			String strURL = XPStringUtility.Substring(strMatch, nSrcIndex + JS_START.length() + 1);
			int nURLEndIndex = XPStringUtility.IndexOf(strURL, "\"");
			if (nURLEndIndex < 0)
			{
				// Attributes can be either single or double quoted.
				nURLEndIndex = XPStringUtility.IndexOf(strURL, "'");
			}
			strURL = XPStringUtility.Substring(strURL, 0, nURLEndIndex);

			int nStartOfMatchIndex = jsMatches[i].GetIndex();

			if (_javaScriptForRemoval.Contains(strURL))
			{
				// If we are supposed to remove this script,
				// just append the section before it to the result.
				filteredContent.Append(XPStringUtility.Substring(strContent,
					nEndOfLastMatch, nStartOfMatchIndex));
			} else
			{
				// If we are supposed to display this script,
				// append it and the section before it to the result.
				filteredContent.Append(XPStringUtility.Substring(strContent,
					nEndOfLastMatch, nStartOfMatchIndex + strMatch.length()));

			    m_newJavaScript.Add(strURL);
			}

			nEndOfLastMatch = nStartOfMatchIndex + strMatch.length();
		}

		 // append the rest
		filteredContent.Append(XPStringUtility.Substring(strContent, nEndOfLastMatch));

		strContent = filteredContent.ToString();

		// Search for CSS
		XPMatch[] cssMatches = CSS_PATTERN.Matches(strContent);

		filteredContent = new XPStringBuilder();

		nEndOfLastMatch = 0;

		for (int i = 0; i < cssMatches.length; i++)
		{
			String strMatch = cssMatches[i].GetValue();

			// Find the URL in the match
			int nSrcIndex = XPStringUtility.IndexOf(strMatch, CSS_START);
			// Skip over the (single or double) quote
			String strURL = XPStringUtility.Substring(strMatch, nSrcIndex + CSS_START.length() + 1);
			int nURLEndIndex = XPStringUtility.IndexOf(strURL, "\"");
			if (nURLEndIndex < 0)
			{
				// Attributes can be either single or double quoted.
				nURLEndIndex = XPStringUtility.IndexOf(strURL, "'");
			}
			strURL = XPStringUtility.Substring(strURL, 0, nURLEndIndex);

			int nStartOfMatchIndex = cssMatches[i].GetIndex();

			if (_cssForRemoval.Contains(strURL))
			{
				// If we are supposed to remove this script,
				// just append the section before it to the result.
				filteredContent.Append(XPStringUtility.Substring(strContent,
					nEndOfLastMatch, nStartOfMatchIndex));
			} else
			{
				// If we are supposed to display this script,
				// append it and the section before it to the result.
				filteredContent.Append(XPStringUtility.Substring(strContent,
					nEndOfLastMatch, nStartOfMatchIndex + strMatch.length()));

			    m_newCSS.Add(strURL);
			}

			nEndOfLastMatch = nStartOfMatchIndex + strMatch.length();
		}

		 // append the rest
		filteredContent.Append(XPStringUtility.Substring(strContent, nEndOfLastMatch));

		strContent = filteredContent.ToString();

		m_displayContent.AddInnerHTMLString(strContent);

		return true;
	}

	/**
	 * This method returns a List of new JS URLs.
	 * 
	 * @return
	 */
	public XPArrayList GetNewJavaScript()
	{
		return m_newJavaScript;
	}

	/**
	 * This method returns a List of new CSS URLs.
	 * 
	 * @return
	 */
	public XPArrayList GetNewCSS()
	{
		return m_newCSS;
	}

	public HTMLElement GetDisplayContent()
	{
		return m_displayContent;
	}
}