using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.helpers;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.core
{
  /// <summary>
  /// This tag allows use of attribute value replacment with standard
  /// HTML tags.  Simply use the tag PT attribute to specify the name
  /// of the HTML tag to display, and then insert the HTML tag attributes
  /// as normal XML attributes.  They will get processed and added to the
  /// HTML tag.
  /// I.E.
  /// <pt:core.html pt:tag="a" href="$url" title="My title">My link</pt:core.html>
  /// would get transformed into
  /// <a href="[the data stored in the $url variable]" title="My title>My link</a>
  /// </summary>
  public class HTMLTag : ATag
  {
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly RequiredTagAttribute TAG_NAME;
    static HTMLTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("html", "This tag displays an HTML tag, allowing for attribute value replacement.  " + "Simply use the pt:tag attribute to specify the name of the HTML tag to " + "display, and then insert the HTML tag attributes as normal XML attributes.  " + "They will get processed and added to the HTML tag.  If an attribute starts with " + "the pt://images URI constant it will be replaced with the full image server URL " + "when used in the portal.",
       "<![CDATA[&lt;pt:core.html pt:tag=\"a\" href=\"$url\" title=\"My title\"&gt;My link&lt;/pt:core.html&gt;]]><![CDATA[<br>]]>" + "would get transformed into<![CDATA[<br>]]>" + "<![CDATA[&lt;a href=\"[the data stored in the $url variable]\" title=\"My title\"/&gt;My link&lt;/a&gt;]]>");
      TAG_NAME = new RequiredTagAttribute("tag", "The name of the HTML tag to display.");
      }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()"></seealso>
    public override HTMLElement DisplayTag()
    {
      String strXMLAttributes = GetXMLAttributes();
      String strHTMLTagName = GetTagAttributeAsString(TAG_NAME);
      String strOpenTag;
      if ((strXMLAttributes != null) && (strXMLAttributes.Length > 0))
      {
        strOpenTag = "<" + strHTMLTagName + " " + strXMLAttributes + ">";
        } else
      {
        strOpenTag = "<" + strHTMLTagName + ">";
      }
      String strCloseTag = "</" + strHTMLTagName + ">";
      HTMLElementCollection result = new HTMLElementCollection();
      result.AddInnerHTMLString(strOpenTag);
      result.AddInnerHTMLElement(ProcessTagBody());
      result.AddInnerHTMLString(strCloseTag);
      return result;
    }
    
    /// <summary>
    /// This helper method puts all the XML attributes into
    /// a single string, and also processes the PT URI constants.
    /// </summary>
    /// <returns>the XML attributes.</returns>
    public virtual String GetXMLAttributes()
    {
      // NOTE: This is a limited solution to the problem of URI constants and
      // URL transformation in Adaptive Tags.  This only addresses pt://images,
      // and if we need to support more cases in the future, we should probably
      // rethink how this works.  The problem is that the transformer knows that
      // the "src" attribute of the "img" tag is a URL, and transforms it, but
      // doesn't know that the "src" attribute of the "pt:core.html" tag with a
      // "pt:tag='img'" attribute is also a URL and needs to be transformed.  This
      // fix solves the limited problem of adding images into layouts that point to
      // the image server and have a localized "alt" string.
      // NOTE: This is basically a copy of the GetXMLTagAttributesAsString() method,
      // except that it handles pt://images.
      // TODO: Evaluate this guess in terms of the expected size of XML attributes;
      XPStringBuilder result = new XPStringBuilder(5 * 10);
      IXPEnumerator keys = GetXMLTagAttributeNames();
      while (keys.MoveNext())
      {
        Object oKey = keys.GetCurrent();
        result.Append(oKey);
        result.Append("=\"");
        String strValue = GetXMLTagAttribute((String) oKey);
        // If this attribute starts with the pt://images URI constant ...
        if ((strValue != null) && (XPStringUtility.StartsWith(strValue, PlumtreeExtensibility.PT_IMAGESERVER_BASE_URI)))
        {
          // Replace the constant with the full image server URL
          strValue = GetImageServerURI() + XPStringUtility.Substring(strValue,
           PlumtreeExtensibility.PT_IMAGESERVER_BASE_URI.Length);
        }
        result.Append(strValue);
        result.Append("\" ");
      }
      if (result.GetLength() > 0)
      {
        // Remove the trailing space
        result.RemoveCharAt(result.GetLength() - 1);
      }
      return result.ToString();
    }
    
    /// <returns>the absolute image server URI.</returns>
    public virtual String GetImageServerURI()
    {
      String url = GetEnvironment().GetCurrentRequestURL();
      AActivitySpace space = (AActivitySpace) GetEnvironment();
      return PTConfigService.getPTInstance().GetAbsoluteImageServerRootURL(space,
       url);
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)"></seealso>
    public override bool SupportsAccessStyle(AccessStyles _style)
    {
      // This tag does not display any JavaScript by itself, and therefore
      // supports all access styles.
      return true;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#Create()"></seealso>
    public override ATag Create()
    {
      return new HTMLTag();
    }
  }
}