package com.plumtree.taglib.core;

import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.portaluiinfrastructure.tags.tasks.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag turns tag debug mode on and off.
 */
public class DebugModeTag extends ATag
{
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final OptionalTagAttribute MODE;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("debugmode",
			"This tag turns tag debug mode on and off.  When debug mode is on, tags " +
			"will display HTML comments before and after they are executed to aid in " +
			"debugging.  Tags may also have custom behavior in debug mode.  This tag "+
			"uses an editable shared variable for debug mode (see " +
			"com.plumtree.portaluiinfrastructure.tags.tasks.TagStateTasks.GetIsDebugMode " +
			"for more details).");

		MODE = new OptionalTagAttribute("mode",
			"Turns debug mode on and off.", AttributeType.BOOLEAN, "true");
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		boolean bPreviousDebugMode = TagStateTasks.GetIsDebugMode(GetState());

		boolean bNewDebugMode = GetTagAttributeAsBoolean(MODE);

		// Set debug mode to the value of the MODE tag attribute.
		TagStateTasks.SetIsDebugMode(bNewDebugMode, GetState());

		// Do not add comment if debug mode was off and we kept it turned off.
		if (bPreviousDebugMode || bNewDebugMode)
		{
			if (bNewDebugMode)
			{
				return new HTMLComment("Turning Debug Mode On.");
			} else
			{
				return new HTMLComment("Turning Debug Mode Off.");
			}
		}

		return null;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetTagType()
	 */
	public TagType GetTagType()
	{
		return TagType.NO_BODY;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// This tag does not display any content, and therefore supports
		// all access styles.
		return true;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new DebugModeTag();
	}
}
