using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.logic
{
  /// <summary>
  /// This tag evaluates a boolean expression and stores
  /// the result as a boolean in memory.  It is designed
  /// to work with the logic.if tag.
  /// </summary>
  public class BooleanExprTag : ATag
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
     "com.plumtree.taglib.logic.BooleanExprTag");
    public const String BOOL_EXAMPLE_SETUP = "<![CDATA[&lt;pt:logic.collectionlength pt:data=\"testcollection\" pt:key=\"length\"/&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;pt:logic.intexpr pt:expr=\"($length) > 5\" pt:key=\"greater5\"/&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;pt:logic.intexpr pt:expr=\"($length) < 10\" pt:key=\"less10\"/&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;pt:logic.boolexpr pt:expr=\"($greater5) && ($less10)\" pt:key=\"boolvalue\"/&gt;]]><![CDATA[<br>]]>";
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly RequiredTagAttribute EXPR;
    public static readonly RequiredTagAttribute KEY;
    public static readonly OptionalTagAttribute SCOPE;
    static BooleanExprTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("boolexpr", "This tag evaluates a boolean expression and stores " + "the result as a boolean in memory.  It is designed " + "to work with the logic.if tag.",
       BOOL_EXAMPLE_SETUP + IfTag.IF_EXAMPLE_IF);
      EXPR = new RequiredTagAttribute("expr", "A boolean value or a tag variable, followed by a " + "boolean comparator, and then another boolean value or " + "tag variable.  Spaces can be added around the comparator " + "for legibility if desired.  The tag variables must evaluate to an " + "boolean and be in $ attribute notation surrounded by " + "parentheses.  The following operators are allowed: <![CDATA[\'" + BooleanOperators.OP_AND + "\', \'" + BooleanOperators.OP_OR + "\', \'" + BooleanOperators.OP_EQUAL + "\', and \'" + BooleanOperators.OP_NOT_EQUAL + "\']]>.");
      KEY = new RequiredTagAttribute("key", "The key used to store the result in memory.");
      SCOPE = new OptionalTagAttribute("scope", "The scope used to store the result in memory.  " + LogicTagLibraryData.SCOPE_REFERENCE,
       AttributeType.STRING, Scope.PORTLET_REQUEST.ToString());
    }
    public override HTMLElement DisplayTag()
    {
      String strExpr = GetTagAttributeAsString(EXPR);
      BooleanOperators op = FindOperator(strExpr);
      int nIndex = XPStringUtility.IndexOf(strExpr, op.ToString());
      String strFirstOperand = XPStringUtility.Substring(strExpr, 0, nIndex);
      String strSecondOperand = XPStringUtility.Substring(strExpr, nIndex + op.ToString().Length,
       strExpr.Length);
      // Strip whitespace from operands
      strFirstOperand = XPStringUtility.Trim(strFirstOperand);
      strSecondOperand = XPStringUtility.Trim(strSecondOperand);
      bool bFirstOperand;
      if (XPStringUtility.StartsWith(strFirstOperand, IfTag.OPERAND_VARIABLE_PREFIX))
      {
        if (IfTag.OPERAND_VARIABLE_SUFFIX != XPStringUtility.CharAt(strFirstOperand,
         strFirstOperand.Length - 1))
        {
          return new HTMLComment("Invalid first operand in " + EXPR.GetName() + " attribute in " + TAG.GetName() + " tag.  Variable operands must be " + "enclosed in parentheses and start with a \'$\': " + strFirstOperand);
          }
        String strVariable = XPStringUtility.Substring(strFirstOperand, IfTag.OPERAND_VARIABLE_PREFIX.Length,
         strFirstOperand.Length - 1);
        String strValue = XPConvert.ToString(GetAnyStateVariable(strVariable));
        if ((null == strValue) || (strValue.Length < 1))
        {
          return new HTMLComment("Empty value returned for first operand variable in " + EXPR.GetName() + " attribute in " + TAG.GetName() + " tag.  First operand: " + strFirstOperand);
          }
        bFirstOperand = XPConvert.ToBoolean(GetAnyStateVariable(strVariable));
        } else
      {
        bFirstOperand = XPConvert.ToBoolean(strFirstOperand);
      }
      bool bSecondOperand;
      if (XPStringUtility.StartsWith(strSecondOperand, IfTag.OPERAND_VARIABLE_PREFIX))
      {
        if (IfTag.OPERAND_VARIABLE_SUFFIX != XPStringUtility.CharAt(strSecondOperand,
         strSecondOperand.Length - 1))
        {
          return new HTMLComment("Invalid second operand in " + EXPR.GetName() + " attribute in " + TAG.GetName() + " tag.  Variable operands must be " + "enclosed in parentheses and start with a \'$\': " + strFirstOperand);
          }
        String strVariable = XPStringUtility.Substring(strSecondOperand, 2,
         strSecondOperand.Length - 1);
        String strValue = XPConvert.ToString(GetAnyStateVariable(strVariable));
        if ((null == strValue) || (strValue.Length < 1))
        {
          return new HTMLComment("Empty value returned for second operand variable in " + EXPR.GetName() + " attribute in " + TAG.GetName() + " tag.  Second operand: " + strSecondOperand);
          }
        bSecondOperand = XPConvert.ToBoolean(GetAnyStateVariable(strVariable));
        } else
      {
        bSecondOperand = XPConvert.ToBoolean(strSecondOperand);
      }
      bool bResult;
      if (op == BooleanOperators.OP_AND)
      {
        bResult = (bFirstOperand && bSecondOperand);
      } else
      if (op == BooleanOperators.OP_OR)
      {
        bResult = (bFirstOperand || bSecondOperand);
      } else
      if (op == BooleanOperators.OP_EQUAL)
      {
        bResult = (bFirstOperand == bSecondOperand);
      } else
      if (op == BooleanOperators.OP_NOT_EQUAL)
      {
        bResult = (bFirstOperand != bSecondOperand);
      } else
      {
        return new HTMLComment("Invalid operator in " + EXPR.GetName() + " attribute in " + TAG.GetName() + " tag.  Valid operators include: " + BooleanOperators.OP_EQUAL + ", " + BooleanOperators.OP_NOT_EQUAL + ", " + BooleanOperators.OP_AND + ", and " + BooleanOperators.OP_OR);
        }
      // Determine the appropriate scope
      Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));
      if (null == scope)
      {
        log.Warn("Unable to convert scope attribute " + GetTagAttributeAsString(SCOPE) + " to a valid scope in Tag " + TAG.GetName() + ".");
        scope = Scope.PORTLET_REQUEST;
      }
      // Store the filled out data collection in its final location.
      SetStateSharedVariable(GetTagAttributeAsString(KEY), XPConvert.ToString(bResult),
       scope, false);
      return ProcessTagBody();
    }
    
    /// <summary>
    /// This helper method finds a boolean operator in an
    /// expression string.
    /// </summary>
    /// <param name="_strExpr"></param>
    /// <returns></returns>
    public virtual BooleanOperators FindOperator(String _strExpr)
    {
      BooleanOperators resultOp = BooleanOperators.OP_AND;
      bool bFoundResult = false;
      BooleanOperators[] ops = BooleanOperators.ops;
      for (int i = 0;i < ops.Length;i++)
      {
        if (XPStringUtility.IndexOf(_strExpr, ops[i].ToString()) >= 0)
        {
          if (bFoundResult)
          {
            throw new XPFormatException("Invalid expression in " + EXPR.GetName() + " attribute in " + TAG.GetName() + " tag.  Can only include one operator.");
            }
          resultOp = ops[i];
          bFoundResult = true;
        }
      }
      if (bFoundResult)
      {
        return resultOp;
      }
      throw new XPIllegalArgumentException("Unable to locate valid " + "boolean operator in expression: " + _strExpr);
      }
    public override bool SupportsAccessStyle(AccessStyles style)
    {
      // This tag does not display any JavaScript, and therefore
      // supports all access styles.
      return true;
    }
    public override ATag Create()
    {
      return new BooleanExprTag();
    }
  }
}