using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.logic
{
  /// <summary>
  /// This tag evaluates the length of a collection and stores the result in memory.
  /// </summary>
  public class CollectionLengthTag : ATag
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
     "com.plumtree.taglib.logic.CollectionLengthTag");
    public const String COLLECTION_LENGTH_EXAMPLE = "<![CDATA[&lt;pt:logic.collectionlength pt:data=\"testcollection\" pt:key=\"length\"/&gt;]]><![CDATA[<br>]]>" + "would store the length of the \'testcollection\' collection as an " + "editable shared variable using the key \'length\'.";
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly RequiredTagAttribute DATA;
    public static readonly RequiredTagAttribute KEY;
    public static readonly OptionalTagAttribute SCOPE;
    static CollectionLengthTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("collectionlength", "This tag evaluates the length of a collection and stores the result in memory.",
       COLLECTION_LENGTH_EXAMPLE);
      DATA = new RequiredTagAttribute("data", "The key used to retrieve the XPArrayList data collection as a " + "shared variable from Tag or PortletRequest scope memory.");
      KEY = new RequiredTagAttribute("key", "The key used to store the result value in memory.");
      SCOPE = new OptionalTagAttribute("scope", "The scope used to store the result value.  " + LogicTagLibraryData.SCOPE_REFERENCE,
       AttributeType.STRING, Scope.PORTLET_REQUEST.ToString());
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()"></seealso>
    public override HTMLElement DisplayTag()
    {
      String strDataKey = GetTagAttributeAsString(DATA);
      Object oData = GetStateSharedVariable(strDataKey, Scope.TAG);
      if (null == oData)
      {
        oData = GetStateSharedVariable(strDataKey, Scope.PORTLET_REQUEST);
        }
      int nLength = 0;
      HTMLElement result;
      // NOTE: We can't tell the difference between a variable with a null value
      // and not finding the variable.  Therefore, add an HTMLComment and set
      // the length to 0.
      if (null == oData)
      {
        String strError = "Unable to locate data collection " + strDataKey + " in " + GetName() + " tag, or data collection was null.";
        log.Error(strError);
        nLength = 0;
        result = new HTMLComment(strError);
      } else
      {
        XPArrayList dataList;
        try
        {
          dataList = (XPArrayList) oData;
        }
        catch (Exception e)
        {
          log.Error(e, "Unable to cast data collection " + strDataKey + " to an XPArrayList in " + GetName() + " tag.");
          return new HTMLComment("Unable to cast data collection " + strDataKey + " to an XPArrayList in " + GetName() + " tag: " + e.Message);
          }
        nLength = dataList.GetSize();
        result = ProcessTagBody();
      }
      Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));
      if (null == scope)
      {
        log.Warn("Unable to convert scope attribute " + GetTagAttributeAsString(SCOPE) + " to a valid scope in Tag " + TAG.GetName() + ".");
        scope = Scope.PORTLET_REQUEST;
      }
      // Store the data collection length using the specified name.
      SetStateSharedVariable(GetTagAttributeAsString(KEY), XPConvert.ToString(nLength),
       scope, false);
      return result;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)"></seealso>
    public override bool SupportsAccessStyle(AccessStyles _style)
    {
      // This tag does not display any content by itself, and therefore
      // supports all access styles.
      return true;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#Create()"></seealso>
    public override ATag Create()
    {
      return new CollectionLengthTag();
    }
  }
}