using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.logic
{
  /// <summary>
  /// This tag creates a data collection and stores it in memory.
  /// </summary>
  public class CollectionTag : ATag
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
     "com.plumtree.taglib.logic.CollectionTag");
    
    /// <summary>
    /// This key is used to store the array list in Tag Scope so that the
    /// child tags can find it.
    /// </summary>
    public const String ARRAY_LIST_KEY = "collection";
    public const String COLLECTION_EXAMPLE1 = "<![CDATA[&lt;pt:logic.collection pt:key=\"testcollection\"&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;pt:logic.data url=\"http://www.myco.com\" name=\"My company\"/&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;pt:logic.data url=\"http://www.otherco.com\" name=\"Other company\"/&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;/pt:logic.collection&gt;]]>";
    public const String COLLECTION_EXAMPLE2 = "<![CDATA[&lt;pt:logic.collection pt:key=\"teststringcollection\"&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;pt:logic.variable pt:value=\"my string data\"/&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;pt:logic.variable pt:value=\"my other string data\"/&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;/pt:logic.collection&gt;]]>";
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly RequiredTagAttribute KEY;
    public static readonly OptionalTagAttribute SCOPE;
    public static readonly RelatedChildTag DATA_OBJECT;
    public static readonly RelatedChildTag VARIABLE;
    static CollectionTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("collection", "This tag creates a data collection (a collection of variables, " + "such as data objects) and stores it as an editable shared variable " + "using the key supplied in the tag attributes.  It is stored in in " + "Portlet Request scope by default.  The variable is only stored " + "after this tag is finished processing it\'s contents.  " + "The collection tag does not append to a pre-existing collection, " + "but will just overwrite it instead. To clear a collection, simply " + "use this tag with no data or variable tags inside it. " + "This collection variable can be used with the foreach tag.  This tag " + "will process nested tags to populate the data collection, " + "but will not output any HTML.  This tag also creates a temporary " + "editable shared variable stored using the key " + ARRAY_LIST_KEY + ". " + "Be careful not to create a collection with different types of variables, " + "such as string variables and data objects.",
       COLLECTION_EXAMPLE1 + "<![CDATA[<p>]]>" + COLLECTION_EXAMPLE2);
      KEY = new RequiredTagAttribute("key", "The key used to store the data collection in memory.");
      SCOPE = new OptionalTagAttribute("scope", "The scope used to store the data collection.  " + LogicTagLibraryData.SCOPE_REFERENCE,
       AttributeType.STRING, Scope.PORTLET_REQUEST.ToString());
      DATA_OBJECT = new RelatedChildTag(DataObjectTag.TAG);
      VARIABLE = new RelatedChildTag(VariableTag.TAG);
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()"></seealso>
    public override HTMLElement DisplayTag()
    {
      XPArrayList dataCollection = new XPArrayList();
      // Determine the appropriate scope
      Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));
      if (null == scope)
      {
        log.Warn("Unable to convert scope attribute " + GetTagAttributeAsString(SCOPE) + " to a valid scope in Tag " + TAG.GetName() + ".");
        scope = Scope.PORTLET_REQUEST;
      }
      // Cache the original value of the array list variable.
      Object oOriginalDataItem = GetStateSharedVariable(ARRAY_LIST_KEY, Scope.TAG);
      // Store the current array list.
      SetStateSharedVariable(ARRAY_LIST_KEY, dataCollection, Scope.TAG, false);
      // The child tags will add variables to the array list.
      // Throw away any resulting HTML (there shouldn't be any).
      HTMLElement result = ProcessTagBody();
      // Store the filled out data collection in its final location.
      SetStateSharedVariable(GetTagAttributeAsString(KEY), dataCollection,
       scope, false);
      // Restore the original value of the array list variable.
      SetStateSharedVariable(ARRAY_LIST_KEY, oOriginalDataItem, Scope.TAG,
       false);
      return result;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)"></seealso>
    public override bool SupportsAccessStyle(AccessStyles _style)
    {
      // This tag does not display any content by itself, and therefore
      // supports all access styles.
      return true;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#Create()"></seealso>
    public override ATag Create()
    {
      return new CollectionTag();
    }
  }
}