using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.data;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.logic
{
  /// <summary>
  /// This tag evaluates if a value is null or not  and stores
  /// the result as a boolean in memory.  It is designed
  /// to work with the logic.if tag.
  /// </summary>
  /// <author>robertz</author>
  public class ContainsExprTag : ATag
  {
    public const String OPERATOR_CONTAINS = "contains";
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
     "com.plumtree.taglib.logic.ContainsExprTag");
    public const String CONTAINSEXPR_EXAMPLE_SETUP1A = "<![CDATA[&lt;pt:ptdata.mycommunitiesdata pt:id=\"comms\"/&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;pt:logic.foreach pt:data=\"comms\" pt:var=\"curr\"&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&nbsp;&lt;pt:logic.containsexpr pt:expr=\"($curr) contains currpage\" pt:key=\"boolvalue\"/&gt;]]><![CDATA[<br><br>]]>";
    public const String CONTAINSEXPR_EXAMPLE_SETUP1B = "<![CDATA[<br><br>]]><![CDATA[&lt;/pt:logic.foreach&gt;]]><![CDATA[<br>]]>";
    public const String CONTAINSEXPR_EXAMPLE_SETUP2 = "<![CDATA[&lt;pt:logic.data pt:key=\"data\" name=\"abc\"/&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;pt:logic.containsexpr pt:expr=\"($data) contains name\" pt:key=\"boolvalue\" /&gt;]]><![CDATA[<br><br>]]>";
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly RequiredTagAttribute KEY;
    public static readonly OptionalTagAttribute SCOPE;
    public static readonly RequiredTagAttribute EXPR;
    static ContainsExprTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("containsexpr", "This tag checks if an collection contains a specific data " + "element and sets a specified variable to true or false. " + "It is designed to work with the logic.if tag.",
       CONTAINSEXPR_EXAMPLE_SETUP1A + IfTag.IF_EXAMPLE_IF + CONTAINSEXPR_EXAMPLE_SETUP1B + "<![CDATA[<br><br>]]>or ...<![CDATA[<br><br>]]>" + CONTAINSEXPR_EXAMPLE_SETUP2 + IfTag.IF_EXAMPLE_IF);
      KEY = new RequiredTagAttribute("key", "The key used to store the result in memory.");
      EXPR = new RequiredTagAttribute("expr", "A string expression that requires three components separated by a space: " + "1. A variable reference for a data object in parentheses " + "(entry in a pt:data list, or a variable created from the logic.data tag) " + "2. The word \"" + OPERATOR_CONTAINS + "\" " + "3. The key of an element in the data to check for. Example:  " + " expr=\"($mydata) " + OPERATOR_CONTAINS + " myelement\" ");
      SCOPE = new OptionalTagAttribute("scope", "The scope used to store the result in memory.  " + LogicTagLibraryData.SCOPE_REFERENCE,
       AttributeType.STRING, Scope.PORTLET_REQUEST.ToString());
    }
    public override HTMLElement DisplayTag()
    {
      bool bResult = false;
      try
      {
        bResult = EvaluateExpressionString();
      }
      catch (XPException e)
      {
        return new HTMLComment("Error encountered in the " + EXPR.GetName() + " attribute in the " + TAG.GetName() + " tag. " + e.GetMessage());
        }
      // Store the expression result to memory
      // Determine the appropriate scope
      Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));
      if (null == scope)
      {
        log.Warn("Unable to convert scope attribute " + GetTagAttributeAsString(SCOPE) + " to a valid scope in Tag " + TAG.GetName() + ".");
        scope = Scope.PORTLET_REQUEST;
      }
      // Store the filled out data collection in its final location.
      SetStateSharedVariable(GetTagAttributeAsString(KEY), XPConvert.ToString(bResult),
       scope, false);
      return ProcessTagBody();
    }
    protected virtual bool EvaluateExpressionString()
    {
      String expr = GetTagAttributeAsString(EXPR);
      XPStringTokenizer tok = new XPStringTokenizer(expr, " ");
      if (tok == null || tok.GetTokenCount() != 3)
      {
        throw new XPException("Expression malformed. Needs to be of the format \"($[collection variable name]) contains  [collection element]\" Found expression: " + expr);
        }
      // get the expression components
      String collectionRef = tok.GetNextToken();
      String conoperator = tok.GetNextToken();
      String elemName = tok.GetNextToken();
      // check the operator
      if (XPStringUtility.IndexOf(conoperator, OPERATOR_CONTAINS) == -1)
      {
        throw new XPException("Expression is missing the " + OPERATOR_CONTAINS + " word. Found expression: " + expr);
        }
      // check the collection variable name
      if (!XPStringUtility.StartsWith(collectionRef, IfTag.OPERAND_VARIABLE_PREFIX) && !(IfTag.OPERAND_VARIABLE_SUFFIX != XPStringUtility.CharAt(collectionRef,
       collectionRef.Length - 1)))
      {
        throw new XPException("Malformed collection variable name. Needs to be in the following format: \"$(collvarname)\"  Found expression: " + expr);
        }
      // we got the variable name, now we can look it up
      String varName = XPStringUtility.Substring(collectionRef, IfTag.OPERAND_VARIABLE_PREFIX.Length,
       collectionRef.Length - 1);
      Object coll = GetAnyStateVariable(varName);
      if (null == coll)
      {
        throw new XPException("Unable to find the specified collection variable name:  " + varName);
        }
      if (coll is IDataObject)
      {
        // check if the elements exists in the collection
        return ((IDataObject) coll).GetData(elemName) != null;
      } else
      if (coll is IXPList)
      {
        // check if the list contains the string element (only works for string lists, not list of collections like what ptdata tags return) 
        return ((IXPList) coll).Contains(elemName);
      } else
      {
        throw new XPException("Found collection variable is not of the expected types IDataObject or IXPList:  " + collectionRef.ToString());
        }
    }
    public override bool SupportsAccessStyle(AccessStyles style)
    {
      // This tag does not display any JavaScript, and therefore
      // supports all access styles.
      return true;
    }
    public override ATag Create()
    {
      return new ContainsExprTag();
    }
  }
}