using System;
using com.plumtree.openfoundation.reflect;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.logic
{
  /// <summary>
  /// This tag allows you to loop over data collections.
  /// </summary>
  public class ForEachTag : ATag
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
     "com.plumtree.taglib.logic.ForEachTag");
    public const String LOOP_EXAMPLE = "<![CDATA[&lt;pt:logic.foreach pt:data=\"testcollection\" pt:var=\"co\"&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;pt:core.html pt:tag=\"a\" href=\"$co.url\"&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;pt:logic.value pt:value=\"$co.name\"/&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;/pt:core.html&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;pt:logic.separator&gt;&lt;br&gt;-----&lt;br&gt;&lt;/pt:logic.separator&gt;]]><![CDATA[<br>]]>" + "<![CDATA[&lt;/pt:logic.foreach&gt;]]><![CDATA[<br>]]>";
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly RequiredTagAttribute DATA;
    public static readonly OptionalTagAttribute VAR;
    public static readonly OptionalTagAttribute ACCEPTNULLDATA;
    public static readonly RelatedChildTag SEPARATOR;
    static ForEachTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("foreach", "This tag allows you to loop over data collections stored in memory as XPArrayLists. " + "The data collection should not be modified while looping to avoid unpredictable side " + "effects. The index value in a foreach loop is key+index (i.e. $varindex).",
       LOOP_EXAMPLE);
      DATA = new RequiredTagAttribute("data", "The key used to retrieve the XPArrayList data collection " + "as a shared variable from Tag, PortletRequest, or Session scope memory.");
      VAR = new OptionalTagAttribute("var", "The key used to store the current IDataObject in Tag scope memory as an " + "editable shared variable.  This tag will restore the original value of " + "this variable after the tag is finished processing, meaning that multiple " + "foreach loops can share the same variable name, although it is considered " + "a best practice to use unique names for the variable attribute for each " + "foreach tag.  If this variable already exists, it must be editable.",
       AttributeType.STRING, "foreachvar");
      ACCEPTNULLDATA = new OptionalTagAttribute("acceptnulldata", "Specifies whether to raise an error or not when null data is used." + "If not, a log message to PTSpy and an HTML comment is generated when " + "null data is passed in to the data attribute. Default value is true. ",
       AttributeType.BOOLEAN, "true");
      SEPARATOR = new RelatedChildTag(SeparatorTag.TAG);
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()"></seealso>
    public override HTMLElement DisplayTag()
    {
      String strDataKey = GetTagAttributeAsString(DATA);
      Object oData = GetStateSharedVariable(strDataKey, Scope.TAG);
      if (null == oData)
      {
        oData = GetStateSharedVariable(strDataKey, Scope.PORTLET_REQUEST);
        }
      if (null == oData)
      {
        oData = GetStateSharedVariable(strDataKey, Scope.SESSION);
      }
      // NOTE: We can't tell the difference between a variable with a null value
      // and not finding the variable.  Therefore, just display an HTMLComment and return.
      // Don't generate any errors if we accept null data
      if (null == oData)
      {
        log.Info("Tag foreach skipping iterating over datavar: \'{0}\' since it was empty.",
         strDataKey);
        if (GetTagAttributeAsBoolean(ACCEPTNULLDATA))
        {
          // don't do anything if we accept null data, just continue gracefully
          return null;
        } else
        {
          String strError = "Unable to locate data collection " + strDataKey + " in " + GetName() + " tag, or data collection was null.";
          log.Warn(strError);
          return new HTMLComment(strError);
        }
      }
      IXPList dataList;
      try
      {
        dataList = (IXPList) oData;
      }
      catch (Exception e)
      {
        log.Warn(e, "Unable to cast data collection " + strDataKey + " to an XPArrayList in " + GetName() + " tag.");
        return new HTMLComment("Unable to cast data collection " + strDataKey + " to an XPArrayList in " + GetName() + " tag: " + e.Message);
        }
      String strVarItemKey = GetTagAttributeAsString(VAR);
      String strIndexVarName = strVarItemKey + "index";
      log.Debug("Tag foreach iterating over datavar: \'{0}\' collection with {1} entries using iterator var: \'{2}\'.",
       strDataKey, dataList.GetSize(), strVarItemKey);
      // Cache original var item value
      Object oOriginalVarItem = GetStateSharedVariable(strVarItemKey, Scope.TAG);
      Object oOriginalIndexVarItem = GetStateSharedVariable(strIndexVarName,
       Scope.TAG);
      try
      {
        // Make sure we have edit access to this variable
        SetStateSharedVariable(strVarItemKey, null, Scope.TAG, false);
      }
      catch (XPIllegalAccessException xpiae)
      {
        String strError = GetName() + " Tag needs edit access to the variable " + "specified in the var attribute (" + strVarItemKey + ").";
        log.Warn(xpiae, strError);
        return new HTMLComment(strError);
      }
      // Cache the previous value of the separator display variable (in case we are in a
      // loop inside a loop).
      Object oOriginalSeparatorDisplayVar = GetStateVariable(SeparatorTag.DISPLAY,
       Scope.TAG);
      SetStateVariable(SeparatorTag.DISPLAY, true, Scope.TAG);
      HTMLElementCollection result = new HTMLElementCollection();
      for (int i = 0;i < dataList.GetSize();i++)
      {
        SetStateSharedVariable(strIndexVarName, i, Scope.TAG, false);
        Object oDataItem = dataList.GetElement(i);
        SetStateSharedVariable(strVarItemKey, oDataItem, Scope.TAG, false);
        // Don't display the separator on the last element
        if (i == (dataList.GetSize() - 1))
        {
          SetStateVariable(SeparatorTag.DISPLAY, false, Scope.TAG);
        }
        result.AddInnerHTMLElement(ProcessTagBody());
      }
      // Restore the original var item value (usually null)
      SetStateSharedVariable(strVarItemKey, oOriginalVarItem, Scope.TAG, false);
      // Restore the original separator display value (usually null)
      SetStateVariable(SeparatorTag.DISPLAY, oOriginalSeparatorDisplayVar,
       Scope.TAG);
      // Restore the original index value (usually null)
      SetStateSharedVariable(strIndexVarName, oOriginalIndexVarItem, Scope.TAG,
       false);
      if (result != null)
      {
        // No need for end log if the foreach is missing body 
        log.Debug("End of foreach tag with datavar: \'{0}\' and iterator var: \'{1}\'.",
         strDataKey, strVarItemKey);
      }
      return result;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#GetTagType()"></seealso>
    public override TagType GetTagType()
    {
      // This tag can call ProcessTagBody() multiple times. 
      return TagType.LOOPING;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)"></seealso>
    public override bool SupportsAccessStyle(AccessStyles _style)
    {
      // This tag does not display any content by itself, and therefore
      // supports all access styles.
      return true;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#Create()"></seealso>
    public override ATag Create()
    {
      return new ForEachTag();
    }
  }
}