using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.logic
{
  /// <summary>
  /// This tag evaluates an integer expression and stores
  /// the result as an integer in memory. 
  /// </summary>
  public class IntOpsTag : ATag
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
     "com.plumtree.taglib.logic.IntOpsTag");
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly RequiredTagAttribute EXPR;
    public static readonly RequiredTagAttribute KEY;
    public static readonly OptionalTagAttribute SCOPE;
    static IntOpsTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("intops", "This tag evaluates an integer expression and stores " + "the result as an integer in memory.");
      EXPR = new RequiredTagAttribute("expr", "An integer value or a tag variable, followed by an " + "arithmetic operator, and then another integer value or " + "tag variable.  Spaces can be added around the comparator " + "for legibility if desired.  The tag variables must evaluate to an " + "integer and be in $ attribute notation surrounded by " + "parentheses.  The following operators are allowed: <![CDATA[\'" + ArithmeticOperators.OP_ADD + "\', \'" + ArithmeticOperators.OP_SUBTRACT + "\', \'" + ArithmeticOperators.OP_MULTIPLY + "\',\'" + ArithmeticOperators.OP_DIVIDE + "\', and \'" + ArithmeticOperators.OP_MOD + "\']]>.");
      KEY = new RequiredTagAttribute("key", "The key used to store the result in memory.");
      SCOPE = new OptionalTagAttribute("scope", "The scope used to store the result in memory.  " + LogicTagLibraryData.SCOPE_REFERENCE,
       AttributeType.STRING, Scope.PORTLET_REQUEST.ToString());
    }
    public override HTMLElement DisplayTag()
    {
      String strExpr = GetTagAttributeAsString(EXPR);
      ArithmeticOperators op = FindOperator(strExpr);
      int nIndex = XPStringUtility.IndexOf(strExpr, op.ToString());
      String strFirstOperand = XPStringUtility.Substring(strExpr, 0, nIndex);
      String strSecondOperand = XPStringUtility.Substring(strExpr, nIndex + op.ToString().Length,
       strExpr.Length);
      // Strip whitespace from operands
      strFirstOperand = XPStringUtility.Trim(strFirstOperand);
      strSecondOperand = XPStringUtility.Trim(strSecondOperand);
      int nFirstOperand;
      if (XPStringUtility.StartsWith(strFirstOperand, IfTag.OPERAND_VARIABLE_PREFIX))
      {
        if (IfTag.OPERAND_VARIABLE_SUFFIX != XPStringUtility.CharAt(strFirstOperand,
         strFirstOperand.Length - 1))
        {
          return new HTMLComment("Invalid first operand in " + EXPR.GetName() + " attribute in " + TAG.GetName() + " tag.  Variable operands must be " + "enclosed in parentheses and start with a \'$\': " + strFirstOperand);
          }
        String strVariable = XPStringUtility.Substring(strFirstOperand, 2,
         strFirstOperand.Length - 1);
        String strValue = XPConvert.ToString(GetAnyStateVariable(strVariable));
        if ((null == strValue) || (strValue.Length < 1))
        {
          return new HTMLComment("Empty value returned for first operand variable in " + EXPR.GetName() + " attribute in " + TAG.GetName() + " tag.  First operand: " + strFirstOperand);
          }
        nFirstOperand = XPConvert.ToInteger(GetAnyStateVariable(strVariable));
        } else
      {
        nFirstOperand = XPConvert.ToInteger(strFirstOperand);
      }
      int nSecondOperand;
      if (XPStringUtility.StartsWith(strSecondOperand, IfTag.OPERAND_VARIABLE_PREFIX))
      {
        if (IfTag.OPERAND_VARIABLE_SUFFIX != XPStringUtility.CharAt(strSecondOperand,
         strSecondOperand.Length - 1))
        {
          return new HTMLComment("Invalid second operand in " + EXPR.GetName() + " attribute in " + TAG.GetName() + " tag.  Variable operands must be " + "enclosed in parentheses and start with a \'$\': " + strFirstOperand);
          }
        String strVariable = XPStringUtility.Substring(strSecondOperand, 2,
         strSecondOperand.Length - 1);
        String strValue = XPConvert.ToString(GetAnyStateVariable(strVariable));
        if ((null == strValue) || (strValue.Length < 1))
        {
          return new HTMLComment("Empty value returned for second operand variable in " + EXPR.GetName() + " attribute in " + TAG.GetName() + " tag.  Second operand: " + strSecondOperand);
          }
        nSecondOperand = XPConvert.ToInteger(GetAnyStateVariable(strVariable));
        } else
      {
        nSecondOperand = XPConvert.ToInteger(strSecondOperand);
      }
      int nResult;
      if (op == ArithmeticOperators.OP_ADD)
      {
        nResult = (nFirstOperand + nSecondOperand);
      } else
      if (op == ArithmeticOperators.OP_SUBTRACT)
      {
        nResult = (nFirstOperand - nSecondOperand);
      } else
      if (op == ArithmeticOperators.OP_MULTIPLY)
      {
        nResult = (nFirstOperand * nSecondOperand);
      } else
      if (op == ArithmeticOperators.OP_DIVIDE)
      {
        nResult = (nFirstOperand / nSecondOperand);
      } else
      if (op == ArithmeticOperators.OP_MOD)
      {
        nResult = (nFirstOperand % nSecondOperand);
      } else
      {
        return new HTMLComment("Invalid operator in " + EXPR.GetName() + " attribute in " + TAG.GetName() + " tag.  Valid operators include: " + ArithmeticOperators.OP_ADD + ", " + ArithmeticOperators.OP_SUBTRACT + ", " + ArithmeticOperators.OP_MULTIPLY + ", " + ArithmeticOperators.OP_DIVIDE + ", and " + ArithmeticOperators.OP_MOD);
        }
      // Determine the appropriate scope
      Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));
      if (null == scope)
      {
        log.Warn("Unable to convert scope attribute " + GetTagAttributeAsString(SCOPE) + " to a valid scope in Tag " + TAG.GetName() + ".");
        scope = Scope.PORTLET_REQUEST;
      }
      // Store the filled out data collection in its final location.
      SetStateSharedVariable(GetTagAttributeAsString(KEY), XPConvert.ToString(nResult),
       scope, false);
      return ProcessTagBody();
    }
    
    /// <summary>
    /// This helper method finds an arithmetic operator in an
    /// expression string.
    /// </summary>
    /// <param name="_strExpr"></param>
    /// <returns></returns>
    public virtual ArithmeticOperators FindOperator(String _strExpr)
    {
      ArithmeticOperators resultOp = null;
      bool bFoundResult = false;
      ArithmeticOperators[] ops = ArithmeticOperators.ops;
      for (int i = 0;i < ops.Length;i++)
      {
        if (XPStringUtility.IndexOf(_strExpr, ops[i].ToString()) >= 0)
        {
          if (bFoundResult)
          {
            throw new XPFormatException("Invalid expression in " + EXPR.GetName() + " attribute in " + TAG.GetName() + " tag.  Can only include one operator.");
            }
          resultOp = ops[i];
          bFoundResult = true;
        }
      }
      if (bFoundResult)
      {
        return resultOp;
      }
      throw new XPIllegalArgumentException("Unable to locate valid int " + "operator in expression: " + _strExpr);
      }
    public override bool SupportsAccessStyle(AccessStyles style)
    {
      // This tag does not display any JavaScript, and therefore
      // supports all access styles.
      return true;
    }
    public override ATag Create()
    {
      return new IntOpsTag();
    }
  }
}