/**
 * 
 */
package com.plumtree.taglib.logic;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author rozhang
 */
public class AppendHierarchicalDataTag extends AHierIndexDataTag
{
	public static final ITagMetaData TAG;
	
	public static final RequiredTagAttribute ATTRIB_DATAID;
	public static final OptionalTagAttribute ATTRIB_DATASCOPE;
	
	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		
		TAG = new TagMetaData("appendhierdata", 
				"This tag is used to create hierarchical data by appending child data or adding new child data if none exist. "+
				"Use this tag to append data from the data attribute to the target location specified by the index attribute. If no children exists, the data is added as children. "+
				"An index attribute with no value creates a new hierarchical data structure.<br/>"+
				"The example below appends the data named \"communityPages\" to \"topCommunities\" at index 0,2. If \"topCommunities\" contains a list of "+
				"Communities and their Subcommunities and communityPages contains a list of Community Pages this tag adds Community Pages in \"communityPages\""+
				"to the 3rd Subcommunity (2 is the 3rd element in 0 based index) of the first top level Community. In the example, the pt:childkey attribute specifies that you are adding data to "+
				"Community Pages; this attribute is not needed when appending a list of subcommunities as it is the default child type for communities.",
				EncodeForTagDoc(
						"<pt:logic.appendhierdata pt:id=\"topCommunities\" pt:dataid=\"communityPages\" pt:index=\"0,2\" pt:childkey=\"commpages\" /><br/>"+
						"// append  a community to second top level community<br/>"+
						"<pt:logic.appendhierdata pt:id=\"topCommunities\" pt:dataid=\"newcomm\" pt:index=\"1\" /><br/>"
						));  	
		
		ATTRIB_DATAID = new RequiredTagAttribute("data", "Name of tag variable with data to append to target hierarchical structure.", AttributeType.STRING);
		ATTRIB_DATASCOPE = new OptionalTagAttribute("datascope", "Scope of the data tag variable name.", AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
		
		ATTRIB_INDEX = new RequiredTagAttribute("index", 
					"Comma separated list of numbers specifying the position in the hierarchical data structure to append the data to. "+
					INDEX_ATTRIB_DESC+
					EncodeForTagDoc("<br/>Examples:<br/>"+
							"\"\" - No index, create a new data structure<br/>"+
							"\"0\" - Append to children in the first top level element<br/>"+
							"\"1,3\" - Append children to the 4th child object of the 2nd top level data object <br/>"),
					AttributeType.STRING);
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		// destination hierarchical data
		String destDataName = GetTagAttributeAsString(ATTRIB_ID);
		Object destHierData = GetTagAttributeAsTagVariable(ATTRIB_ID, GetTagAttributeAsScope(ATTRIB_SCOPE) );
		String hierDataId = GetTagAttributeAsString(ATTRIB_HIERID);
		
		
		IHierarchicalDataObject destParent = null; // destination parent data containing the children to append to 
		XPArrayList destList = null; // target list to append the data too
		//
		if ( destHierData instanceof XPArrayList ) {
			destList = (XPArrayList)destHierData;
			destParent = GetObjectFromIndex(destList);
			if ( destParent != null ) { 
				destList = destParent.GetHierData(hierDataId);
			}
			if ( destList == null ) { // if the parent data object doesn't already contain children
				destList = new XPArrayList(); // create 
				destParent.SetHierData(hierDataId, destList);
			}
			// if destParent is null then there is no parent data object, i.e. we are adding to the top level list of data and destList already contains reference to the top list
		} else if ( destHierData == null) {
			// we are creating a new hier data structure, don't need to do anything here
		} else {
			throw new XPIllegalArgumentException("Unknown type found for Tag variable with name: '"+destDataName+"'. Excpected a list but found: "+destHierData);
		}

		// data to add (could be hierarchical too)
		String srcTagDataName = GetTagAttributeAsString(ATTRIB_DATAID);
		Object srcTagData = GetTagAttributeAsTagVariable(ATTRIB_DATAID, GetTagAttributeAsScope(ATTRIB_DATASCOPE));
		
		// either add list or data
		XPArrayList srcDataList = null; 
		IDataObject srcData = null; 
		if ( srcTagData instanceof IDataObject ) {
			srcDataList = new XPArrayList();
			srcDataList.Add(srcTagData);
		} else if ( srcTagData instanceof XPArrayList ) {
			srcDataList = (XPArrayList)srcTagData;
		} else {
			throw new XPIllegalStateException("Tag variable '"+srcTagDataName+"'specified in the 'data' attribute is of an invalid type:'"+srcTagData+"'. Expecting data or list types.");
		}
		
		// means no index was passed in, getobjectfromindex throws exception for invalid indices
		// we are adding new data  
		if ( destParent == null && destList == null) {
			if ( srcDataList == null ) { // data needs to be in a list
				srcDataList = new XPArrayList();
				srcDataList.Add(srcData);
			}
			GetState().SetSharedVariable(destDataName, srcDataList, GetTagAttributeAsScope(ATTRIB_SCOPE), false);
		} 
		else 
		{
			if ( srcDataList == destList ) { // make sure we don't add from the same list we are adding too, will cause infinite loop
				throw new XPIllegalStateException("Source and destination data are the same. Source tag variable name: '"+srcDataList+"' Destination tag variable name: '"+destDataName+"'");
			}
			// this tag (appends) the source data to the child list
			if ( srcDataList != null ) {
					destList.AddRange ( srcDataList );
			} else {
				destList.Add( srcData );
			}
		}
		
		return ProcessTagBody();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new AppendHierarchicalDataTag();
	}

}
