package com.plumtree.taglib.logic;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag evaluates a boolean expression and stores
 * the result as a boolean in memory.  It is designed
 * to work with the logic.if tag.
 */
public class BooleanExprTag extends ATag
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		"com.plumtree.taglib.logic.BooleanExprTag");

	public static final String BOOL_EXAMPLE_SETUP =
		"<![CDATA[&lt;pt:logic.collectionlength pt:data=\"testcollection\" pt:key=\"length\"/&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;pt:logic.intexpr pt:expr=\"($length) > 5\" pt:key=\"greater5\"/&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;pt:logic.intexpr pt:expr=\"($length) < 10\" pt:key=\"less10\"/&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;pt:logic.boolexpr pt:expr=\"($greater5) && ($less10)\" pt:key=\"boolvalue\"/&gt;]]><![CDATA[<br>]]>";

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute EXPR;
	public static final RequiredTagAttribute KEY;
	public static final OptionalTagAttribute SCOPE;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("boolexpr",
			"This tag evaluates a boolean expression and stores " +
			"the result as a boolean in memory.  It is designed " +
			"to work with the logic.if tag.", BOOL_EXAMPLE_SETUP + IfTag.IF_EXAMPLE_IF);

		EXPR = new RequiredTagAttribute("expr",
			"A boolean value or a tag variable, followed by a " +
			"boolean comparator, and then another boolean value or " +
			"tag variable.  Spaces can be added around the comparator " +
			"for legibility if desired.  The tag variables must evaluate to an " +
			"boolean and be in $ attribute notation surrounded by " +
			"parentheses.  The following operators are allowed: <![CDATA['" +
			BooleanOperators.OP_AND + "', '" + BooleanOperators.OP_OR + "', '" +
			BooleanOperators.OP_EQUAL + "', and '" + BooleanOperators.OP_NOT_EQUAL + "']]>.");

		KEY = new RequiredTagAttribute("key",
			"The key used to store the result in memory.");

		SCOPE = new OptionalTagAttribute("scope",
			"The scope used to store the result in memory.  " + LogicTagLibraryData.SCOPE_REFERENCE,
			AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
	}

	public HTMLElement DisplayTag()
	{
		String strExpr = GetTagAttributeAsString(EXPR);

		BooleanOperators op = FindOperator(strExpr);

		int nIndex = XPStringUtility.IndexOf(strExpr, op.toString());

		String strFirstOperand = XPStringUtility.Substring(strExpr, 0, nIndex);
		String strSecondOperand = XPStringUtility.Substring(strExpr,
			nIndex + op.toString().length(), strExpr.length());

		// Strip whitespace from operands
		strFirstOperand = XPStringUtility.Trim(strFirstOperand);
		strSecondOperand = XPStringUtility.Trim(strSecondOperand);

		boolean bFirstOperand;

		if (XPStringUtility.StartsWith(strFirstOperand, IfTag.OPERAND_VARIABLE_PREFIX))
		{
			if (IfTag.OPERAND_VARIABLE_SUFFIX !=
				XPStringUtility.CharAt(strFirstOperand, strFirstOperand.length() - 1))
			{
				return new HTMLComment("Invalid first operand in " +
					EXPR.GetName() + " attribute in " +
					TAG.GetName() + " tag.  Variable operands must be " +
					"enclosed in parentheses and start with a '$': " + strFirstOperand);
			}

			String strVariable = XPStringUtility.Substring(strFirstOperand, IfTag.OPERAND_VARIABLE_PREFIX.length(), strFirstOperand.length() - 1);

			String strValue = XPConvert.ToString(GetAnyStateVariable(strVariable));

			if ((null == strValue) || (strValue.length() < 1))
			{
				return new HTMLComment("Empty value returned for first operand variable in " +
					EXPR.GetName() + " attribute in " +
					TAG.GetName() + " tag.  First operand: " + strFirstOperand );
			}

			bFirstOperand = XPConvert.ToBoolean(GetAnyStateVariable(strVariable));
		} else
		{
			bFirstOperand = XPConvert.ToBoolean(strFirstOperand);
		}

		boolean bSecondOperand;

		if (XPStringUtility.StartsWith(strSecondOperand, IfTag.OPERAND_VARIABLE_PREFIX))
		{
			if (IfTag.OPERAND_VARIABLE_SUFFIX !=
				XPStringUtility.CharAt(strSecondOperand, strSecondOperand.length() - 1))
			{
				return new HTMLComment("Invalid second operand in " +
					EXPR.GetName() + " attribute in " +
					TAG.GetName() + " tag.  Variable operands must be " +
					"enclosed in parentheses and start with a '$': " + strFirstOperand);
			}

			String strVariable = XPStringUtility.Substring(strSecondOperand, 2, strSecondOperand.length() - 1);

			String strValue = XPConvert.ToString(GetAnyStateVariable(strVariable));

			if ((null == strValue) || (strValue.length() < 1))
			{
				return new HTMLComment("Empty value returned for second operand variable in " +
					EXPR.GetName() + " attribute in " +
					TAG.GetName() + " tag.  Second operand: " + strSecondOperand );
			}

			bSecondOperand = XPConvert.ToBoolean(GetAnyStateVariable(strVariable));
		} else
		{
			bSecondOperand = XPConvert.ToBoolean(strSecondOperand);
		}

		boolean bResult;

		if (op == BooleanOperators.OP_AND) {
			bResult = (bFirstOperand && bSecondOperand);
		} else if (op == BooleanOperators.OP_OR) {
			bResult = (bFirstOperand || bSecondOperand);
		} else if (op == BooleanOperators.OP_EQUAL) {
			bResult = (bFirstOperand == bSecondOperand);
		} else if (op == BooleanOperators.OP_NOT_EQUAL) {
			bResult = (bFirstOperand != bSecondOperand);
		} else {
			return new HTMLComment("Invalid operator in " +
				EXPR.GetName() + " attribute in " +
				TAG.GetName() + " tag.  Valid operators include: " +
				BooleanOperators.OP_EQUAL + ", " + BooleanOperators.OP_NOT_EQUAL + ", " +
				BooleanOperators.OP_AND + ", and " + BooleanOperators.OP_OR);
		}

		// Determine the appropriate scope
		Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));

		if (null == scope)
		{
			log.Warn("Unable to convert scope attribute " +
				GetTagAttributeAsString(SCOPE) + " to a valid scope in Tag " +
				TAG.GetName() + ".");

			scope = Scope.PORTLET_REQUEST;
		}

		// Store the filled out data collection in its final location.
		SetStateSharedVariable(GetTagAttributeAsString(KEY),
			XPConvert.ToString(bResult), scope, false);
		
		return ProcessTagBody();
	}

    /**
     * This helper method finds a boolean operator in an
     * expression string.
     * 
     * @param _strExpr
     * @return
     */
	public BooleanOperators FindOperator(String _strExpr)
	{
		BooleanOperators resultOp = BooleanOperators.OP_AND;
		boolean bFoundResult = false;

		final BooleanOperators[] ops = BooleanOperators.ops;

        for (int i = 0; i < ops.length; i++)
        {
            if (XPStringUtility.IndexOf(_strExpr, ops[i].toString()) >= 0)
            {
            	if (bFoundResult)
            	{
            		throw new XPFormatException("Invalid expression in " +
            			EXPR.GetName() + " attribute in " +
            			TAG.GetName() + " tag.  Can only include one operator.");
            	}
            	resultOp = ops[i];
            	bFoundResult = true;
            }
        }

        if (bFoundResult)
        {
        	return resultOp;
        }

        throw new XPIllegalArgumentException("Unable to locate valid " +
        	"boolean operator in expression: " + _strExpr);
	}

	public boolean SupportsAccessStyle(AccessStyles style)
	{
		// This tag does not display any JavaScript, and therefore
		// supports all access styles.
		return true;
	}

	public ATag Create()
	{
		return new BooleanExprTag();
	}
}