package com.plumtree.taglib.logic;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag creates a data object and stores it in memory.
 */
public class DataObjectTag extends ATag
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		"com.plumtree.taglib.logic.DataObjectTag");

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final OptionalTagAttribute KEY;
	public static final OptionalTagAttribute SCOPE;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("data",
			"This tag creates a data object (a collection of name value pairs) " +
			"and stores it as an editable shared variable using the key supplied " +
			"in the tag attributes.  This tag can also be used inside the collection " +
			"tag, in which case the key attribute is not necessary and the name " +
			"value pairs will be stored directly in the parent collection.  " +
			"The key attribute is required otherwise.  The XML attributes " +
			"(non pt: attributes) for this tag are used as name value pairs for " +
			"the data object.  It is stored in in Portlet Request scope by default.  " +
			"This variable can be used in attribute value replacement or with the value tag.",
			CollectionTag.COLLECTION_EXAMPLE1);

		KEY = new OptionalTagAttribute("key",
			"The key used to store the data object in memory.  This attribute is required unless " +
			"this tag is being used inside a collection tag.  If this attribute is present and " +
			"this tag is being used inside a collection tag, this attribute will take precedence " +
			"and the data will be stored in the collection specified in the attribute.",
			AttributeType.STRING, null);

		SCOPE = new OptionalTagAttribute("scope",
			"The scope used to store the data object.  " + LogicTagLibraryData.SCOPE_REFERENCE,
			AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		// Create the data hashtable
		XPHashtable hashtable = new XPHashtable();

		IXPEnumerator names = GetXMLTagAttributeNames();

		while (names.MoveNext())
		{
			hashtable.PutElement(names.GetCurrent(), GetXMLTagAttribute((String) names.GetCurrent()));
		}

		// Convert the data hashtable to an IDataObject.
		IDataObject dataObject = new HashtableDataObject(hashtable);

		if (HasParentTag(CollectionTag.TAG) &&
			(null == GetTagAttributeAsString(KEY)))
		{
			// Store the data object in the collection.
			XPArrayList collection = (XPArrayList) GetStateSharedVariable(CollectionTag.ARRAY_LIST_KEY, Scope.TAG);

			collection.Add(dataObject);
		} else
		{
			// Store the data object using the specified name.

			String strName = GetTagAttributeAsString(KEY);

			if (null == strName)
			{
				throw new XPException(TAG.GetName() +
					" tag cannot be displayed unless it is nested inside a " +
					"collection tag or has a valid " + KEY.GetName() +
					" attribute value.");
			}

			// Determine the appropriate scope
			Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));

			if (null == scope)
			{
				log.Warn("Unable to convert scope attribute " +
					GetTagAttributeAsString(SCOPE) + " to a valid scope in Tag " +
					TAG.GetName() + ".");

				scope = Scope.PORTLET_REQUEST;
			}

			// Store the data object
			SetStateSharedVariable(strName, dataObject, scope, false);
		}

		return ProcessTagBody();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// This tag does not display any content by itself, and therefore
		// supports all access styles.
		return true;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new DataObjectTag();
	}
}