package com.plumtree.taglib.logic;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag evaluates an integer expression and stores
 * the result as an integer in memory. 
 */
public class IntOpsTag extends ATag
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		"com.plumtree.taglib.logic.IntOpsTag");

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute EXPR;
	public static final RequiredTagAttribute KEY;
	public static final OptionalTagAttribute SCOPE;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("intops",
			"This tag evaluates an integer expression and stores " +
			"the result as an integer in memory.");

		EXPR = new RequiredTagAttribute("expr",
			"An integer value or a tag variable, followed by an " +
			"arithmetic operator, and then another integer value or " +
			"tag variable.  Spaces can be added around the comparator " +
			"for legibility if desired.  The tag variables must evaluate to an " +
			"integer and be in $ attribute notation surrounded by " +
			"parentheses.  The following operators are allowed: <![CDATA['" +
			ArithmeticOperators.OP_ADD + "', '" + ArithmeticOperators.OP_SUBTRACT + "', '" +
			ArithmeticOperators.OP_MULTIPLY + "','" + ArithmeticOperators.OP_DIVIDE + "', and '" + 
			ArithmeticOperators.OP_MOD + "']]>.");

		KEY = new RequiredTagAttribute("key",
			"The key used to store the result in memory.");

		SCOPE = new OptionalTagAttribute("scope",
			"The scope used to store the result in memory.  " + LogicTagLibraryData.SCOPE_REFERENCE,
			AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
	}

	public HTMLElement DisplayTag()
	{
		String strExpr = GetTagAttributeAsString(EXPR);

		ArithmeticOperators op = FindOperator(strExpr);

		int nIndex = XPStringUtility.IndexOf(strExpr, op.toString());

		String strFirstOperand = XPStringUtility.Substring(strExpr, 0, nIndex);
		String strSecondOperand = XPStringUtility.Substring(strExpr,
			nIndex + op.toString().length(), strExpr.length());

		// Strip whitespace from operands
		strFirstOperand = XPStringUtility.Trim(strFirstOperand);
		strSecondOperand = XPStringUtility.Trim(strSecondOperand);

		int nFirstOperand;

		if (XPStringUtility.StartsWith(strFirstOperand, IfTag.OPERAND_VARIABLE_PREFIX))
		{
			if (IfTag.OPERAND_VARIABLE_SUFFIX !=
				XPStringUtility.CharAt(strFirstOperand, strFirstOperand.length() - 1))
			{
				return new HTMLComment("Invalid first operand in " +
					EXPR.GetName() + " attribute in " +
					TAG.GetName() + " tag.  Variable operands must be " +
					"enclosed in parentheses and start with a '$': " + strFirstOperand);
			}

			String strVariable = XPStringUtility.Substring(strFirstOperand, 2, strFirstOperand.length() - 1);

			String strValue = XPConvert.ToString(GetAnyStateVariable(strVariable));

			if ((null == strValue) || (strValue.length() < 1))
			{
				return new HTMLComment("Empty value returned for first operand variable in " +
					EXPR.GetName() + " attribute in " +
					TAG.GetName() + " tag.  First operand: " + strFirstOperand );
			}

			nFirstOperand = XPConvert.ToInteger(GetAnyStateVariable(strVariable));
		} else
		{
			nFirstOperand = XPConvert.ToInteger(strFirstOperand);
		}

		int nSecondOperand;

		if (XPStringUtility.StartsWith(strSecondOperand, IfTag.OPERAND_VARIABLE_PREFIX))
		{
			if (IfTag.OPERAND_VARIABLE_SUFFIX !=
				XPStringUtility.CharAt(strSecondOperand, strSecondOperand.length() - 1))
			{
				return new HTMLComment("Invalid second operand in " +
					EXPR.GetName() + " attribute in " +
					TAG.GetName() + " tag.  Variable operands must be " +
					"enclosed in parentheses and start with a '$': " + strFirstOperand);
			}

			String strVariable = XPStringUtility.Substring(strSecondOperand, 2, strSecondOperand.length() - 1);

			String strValue = XPConvert.ToString(GetAnyStateVariable(strVariable));

			if ((null == strValue) || (strValue.length() < 1))
			{
				return new HTMLComment("Empty value returned for second operand variable in " +
					EXPR.GetName() + " attribute in " +
					TAG.GetName() + " tag.  Second operand: " + strSecondOperand );
			}

			nSecondOperand = XPConvert.ToInteger(GetAnyStateVariable(strVariable));
		} else
		{
			nSecondOperand = XPConvert.ToInteger(strSecondOperand);
		}

		int nResult;

		if (op == ArithmeticOperators.OP_ADD) {
			nResult = (nFirstOperand + nSecondOperand);
		} else if (op == ArithmeticOperators.OP_SUBTRACT) {
			nResult = (nFirstOperand - nSecondOperand);
		} else if (op == ArithmeticOperators.OP_MULTIPLY) {
			nResult = (nFirstOperand * nSecondOperand);
		} else if (op == ArithmeticOperators.OP_DIVIDE) {
			nResult = (nFirstOperand / nSecondOperand);
		} else if (op == ArithmeticOperators.OP_MOD) {
			nResult = (nFirstOperand % nSecondOperand);
		} else {
			return new HTMLComment("Invalid operator in " +
				EXPR.GetName() + " attribute in " +
				TAG.GetName() + " tag.  Valid operators include: " +
				ArithmeticOperators.OP_ADD + ", " + ArithmeticOperators.OP_SUBTRACT + ", " +
				ArithmeticOperators.OP_MULTIPLY + ", " + ArithmeticOperators.OP_DIVIDE + 
				", and " + ArithmeticOperators.OP_MOD);
		}

		// Determine the appropriate scope
		Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));

		if (null == scope)
		{
			log.Warn("Unable to convert scope attribute " +
				GetTagAttributeAsString(SCOPE) + " to a valid scope in Tag " +
				TAG.GetName() + ".");

			scope = Scope.PORTLET_REQUEST;
		}

		// Store the filled out data collection in its final location.
		SetStateSharedVariable(GetTagAttributeAsString(KEY),
			XPConvert.ToString(nResult), scope, false);
		
		return ProcessTagBody();
	}

    /**
     * This helper method finds an arithmetic operator in an
     * expression string.
     * 
     * @param _strExpr
     * @return
     */
	public ArithmeticOperators FindOperator(String _strExpr)
	{
		ArithmeticOperators resultOp = null;
		boolean bFoundResult = false;

		final ArithmeticOperators[] ops = ArithmeticOperators.ops;

        for (int i = 0; i < ops.length; i++)
        {
            if (XPStringUtility.IndexOf(_strExpr, ops[i].toString()) >= 0)
            {
            	if (bFoundResult)
            	{
            		throw new XPFormatException("Invalid expression in " +
            			EXPR.GetName() + " attribute in " +
            			TAG.GetName() + " tag.  Can only include one operator.");
            	}
            	resultOp = ops[i];
            	bFoundResult = true;
            }
        }

        if (bFoundResult)
        {
        	return resultOp;
        }

        throw new XPIllegalArgumentException("Unable to locate valid int " +
        	"operator in expression: " + _strExpr);
	}

	public boolean SupportsAccessStyle(AccessStyles style)
	{
		// This tag does not display any JavaScript, and therefore
		// supports all access styles.
		return true;
	}

	public ATag Create()
	{
		return new IntOpsTag();
	}
}
