/**
 * 
 */
package com.plumtree.taglib.logic;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag is used to replace child data in hierarchical data
 * 
 * @author rozhang
 */
public class ReplaceHierarchicalDataTag extends AHierIndexDataTag
{
	public static final ITagMetaData TAG;
	
	public static final RequiredTagAttribute ATTRIB_DATAID;
	public static final OptionalTagAttribute ATTRIB_DATASCOPE;
	
	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("replacehierdata", "This tag is used to create or modify hierarchical data by replacing child nodes with new data collection.<br/>"+
				"Use appendhierdata tag to create new children or append data to existing children.<br/>"+
				"The example below replaces children of \"topCommunities\" at location 0,2 with the data in \"communityPages\" tag variable. If topCommunities contains a list of "+
				"communities and their Subcommunities and \"communityPages\" contains a list of Community pages, the tag replaces existing Community Pages "+
				"in the 3rd Subcommunity (2 is the 3rd element in 0 based index) of the first top level Community. In the example, the pt:childkey attribute specifies that you are inserting data to "+
				"Community Pages; this attribute is not needed when inserting a list of subcommunities, because that is the default child type for Communities.",
				EncodeForTagDoc(
						"<pt:logic.replacehierdata pt:id=\"topCommunities\" pt:dataid=\"communityPages\" pt:index=\"0,2\" pt:childkey=\"commpages\" /><br/>"+
						"// replace children of the second top level parent<br/>"+
						"<pt:logic.replacehierdata pt:id=\"topCommunities\" pt:dataid=\"communityPages\" pt:index=\"1\" pt:childkey=\"commpages\" /><br/>"
						)); 
		
		ATTRIB_DATAID = new RequiredTagAttribute("data", "Name of tag variable that contains data with which to replace the target data.", AttributeType.STRING);
		
		ATTRIB_DATASCOPE = new OptionalTagAttribute("datascope", "Scope of the data tag variable id.", AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
		
		ATTRIB_INDEX = new RequiredTagAttribute("index", 
					"Comma separated list of numbers specifying the position in the hierarchical data structure at which to replace the data. "+
					INDEX_ATTRIB_DESC+
					EncodeForTagDoc("<br/>Examples:<br/>"+
							"\"0\" - Insert data list as  children to the first top level entry<br/>"+
							"\"1,3\" - Inserth children as the 4th child object of the 2nd top level data object<br/>"),
					AttributeType.STRING);
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		// destination hierarchical data
		String destDataName = GetTagAttributeAsString(ATTRIB_ID);
		Object destHierData = GetTagAttributeAsTagVariable(ATTRIB_ID, GetTagAttributeAsScope(ATTRIB_SCOPE) );
		String hierDataId = GetTagAttributeAsString(ATTRIB_HIERID);
		
		if ( destHierData == null ) {
			throw new XPIllegalStateException("No hierarchical data tag variable with name: '"+destDataName+"' passed in with the id attribute was found.");
		}

		IHierarchicalDataObject destData = null;
		if ( destHierData instanceof IXPList ) {
			destData = GetObjectFromIndex((XPArrayList)destHierData);
//		} 
//		else if ( destHierData instanceof IHierarchicalDataObject ) {
//			// @TODO is this a case we need to support? 
		} else {
			throw new XPIllegalArgumentException("Unknown type found for Tag variable with name: '"+destDataName+"'. Excpected hierarchical data or a list: "+destHierData);
		}
		
		// data to add (could be hierarchical too)
		String srcTagDataName = GetTagAttributeAsString(ATTRIB_DATAID);
		Object srcTagData = GetTagAttributeAsTagVariable(ATTRIB_DATAID, GetTagAttributeAsScope(ATTRIB_DATASCOPE));
		
		XPArrayList srcDataList = null; // the value to add to the hierarchical data structure
		// make sure the source data is an xparraylist
		if ( srcTagData instanceof IDataObject ) {
			srcDataList = new XPArrayList();
			srcDataList.Add(srcTagData);
		} else if ( srcTagData instanceof XPArrayList ) {
			srcDataList = (XPArrayList)srcTagData;
		} else {
			throw new XPIllegalStateException("Tag variable '"+srcTagDataName+"'specified in the 'data' attribute is of an invalid type:'"+srcTagData+"'. Expecting data or list types.");
		}

		// means no index was passed in, getobjectfromindex throws exception for invalid indices
		// set the data to add for the passed in tag variable name 
		if ( destData == null ) {
			GetState().SetSharedVariable(destDataName, srcDataList, GetTagAttributeAsScope(ATTRIB_SCOPE), false);
		} else {
			// this tag (attach) replaces existing children, the append tag adds to existing list
			destData.SetHierData(hierDataId, srcDataList);
		}
		
		
		return ProcessTagBody();
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new ReplaceHierarchicalDataTag();
	}

}
