package com.plumtree.taglib.logic;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlelements.*;

//this tag sorts a list provided by ptdata tags and stores it in memory

/**
 * CR ROB: Add proper javadocs comments for this class
 * 
 */
public class SortTag extends ATag {
	
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
			"com.plumtree.taglib.logic.SortTag");

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute DATA;
	public static final RequiredTagAttribute SORTEDDATA;
	public static final OptionalTagAttribute PROPERTY;
	public static final OptionalTagAttribute SCOPE;
	public static final OptionalTagAttribute SORTEDSCOPE;
	
	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("sort",
			"This tag allows you sort data collections stored in memory as XPArrayLists. " +
			"The property attribute will be used to construct the sorted list.  If the property doesn't exist " +
			"for certain entries in the list, those entries will then be ordered at the end of the list in alphabetical " +
			"order based on their name.");
			// CR ROB: Add an example of how to use it, look in the portletpage tags for example

		DATA = new RequiredTagAttribute("data",
			// CR ROB: don't mention XPArrayList in the tag, it is an implementation detail that tag users don't necessarily
			// know about. Just say a list of collections. Shared data is also implementation specific, no need to mention it.
			"The key used to retrieve the XPArrayList data collection " +
			"as a shared variable from Tag or PortletRequest scope memory.");
		
		// CR ROB: again just say a list of sorted collection. Don't need to mention shared data.
		SORTEDDATA = new RequiredTagAttribute("sorteddata",
				"The key used to store the sorted XPArrayList data collection as a shared variable from PortletRequest scope memory. " +
				"If the data tag passed in is null, the tag will return a copy of the original data tag.");

		PROPERTY = new OptionalTagAttribute("property",
			"The attribute used to sort the list items. The default is set to \"title\"." +
			"NOTE:  For collections defined by logic.collection, you must ensure that the property you are sorting by exists." +
			"A logic.collection containing only a string value is not supported.", AttributeType.STRING, "title");
		
		SCOPE = new OptionalTagAttribute("scope",
				"The scope used to store the variable.  " + LogicTagLibraryData.SCOPE_REFERENCE,
				AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
		
		SORTEDSCOPE = new OptionalTagAttribute("sortedscope",
				"The scope used to store the variable.  " + LogicTagLibraryData.SCOPE_REFERENCE,
				AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag() {

		//we are getting our list, this is our original list
		String dataKey = GetTagAttributeAsString(DATA);
		String sorteddataKey = GetTagAttributeAsString(SORTEDDATA);
		Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));
		Scope sortedscope = Scope.GetScope(GetTagAttributeAsString(SORTEDSCOPE));

		Object oData = GetStateSharedVariable(dataKey, scope);
		
		if (oData == null)
		{
			//return a copy of the original datalist for sorting, response to bug64650
			XPArrayList nullData = (XPArrayList)oData;
			SetStateSharedVariable(sorteddataKey, nullData, sortedscope, false);
			return null;
			
		}
		
		XPArrayList dataList;
		
		try {
			dataList = (XPArrayList)oData;
		}
		catch (Exception ex)
		{
			// CR ROB: Throw a new XPIllegalArgumentException with the original exception and use the error text
			// below. Tag fw will generate an html comment on the page
			log.Error(ex, "Unable to cast data collection " +
					dataKey + " to an XPArrayList in " + GetName() + " tag.");

			return new HTMLComment("Unable to cast data collection " +
					dataKey + " to an XPArrayList in " + GetName() + " tag: " +
					ex.getMessage());	
		}
		
		String propertyString = GetTagAttributeAsString(PROPERTY);

		//Optional tag, if missing, set to default
		// CR ROB: you don't have to do this check, the framework takes care of it,
		if (propertyString.equals("") || propertyString.equals(null))
		{ 
			propertyString = "title";
		}

		int length = dataList.GetSize();
		IDataObject temp = null;
		boolean nopropflag = false;
		int amount = 0;
		XPHashtable tmp_hash = new XPHashtable();
		String[] tempstrArray = new String[length];
		String noprop = "noprop";
		IXPList tmp_list = null;
		for (int i = 0; i < length; i++)
		{
			try {
				temp = (IDataObject)dataList.GetElement(i);
			} catch (XPClassCastException e) {
				throw new XPIllegalArgumentException("Tag collection with id '"+dataKey+"' does not contain valid IDataObject objects."+
						" Verify that the datavar attribute refers to a variable set by a portlet data tag.");
			} 

			//we are converting all elements to lower case to sort, this will be universal regardless of property
			String tmp = "";

			//We have a datalist object that does not contain a value for the property being searched
			// CR ROB: no need to check for the title property if they wanted to sort on another, it might cause
			// confusion since the order is not as expected and no error is displayed. We can just remove this check
			if ( (temp.GetData(propertyString) == null) || (temp.GetData(propertyString).equals("")) )
			{
				nopropflag = true;
				//SEARCHING FOR THE DEFAULT TITLE PROPERTY
				
				try {
					tmp = XPStringUtility.ToLowerCase(temp.GetData("title"));
				}
				catch (XPClassCastException ex)
				{
					throw new XPClassCastException("There was an error retrieving the property in the datalist item.\n");
				}				

				//list entries that do not have the PROPERTY to be sorted on, tagging them to be stored and retrieved later
				// DCA - add entries to an array list before inserting to hashtable in case there are multiple entries w/
				// the same key
				String tag = noprop + tmp;
				if (tmp_hash.ContainsKey(tag)) {
					tmp_list = (IXPList) tmp_hash.GetElement(tag);
				} else {
					tmp_list = new XPArrayList();
					tmp_hash.PutElement(tag, tmp_list);
				}
				tmp_list.Add(dataList.GetElement(i));
				tempstrArray[i] = tag;
				// CR ROB: I don't think you need this variable, you can just check the length of the tempstrArray later
				amount = amount + 1;
			}
			// CR ROB: what if the hashtable doesn't have that entry? you'll get an ugly null pointer exception
			else if ( (!temp.GetData(propertyString).equals("")) )
			{
				// DCA - add entries to an array list before inserting to hashtable in case there are multiple entries w/
				// the same key
				tmp = XPStringUtility.ToLowerCase(temp.GetData(propertyString));
				if (tmp_hash.ContainsKey(tmp)) {
					tmp_list = (IXPList) tmp_hash.GetElement(tmp);
				} else {
					tmp_list = new XPArrayList();
					tmp_hash.PutElement(tmp, tmp_list);
				}
				tmp_list.Add(dataList.GetElement(i));
				tempstrArray[i] = tmp;
			}	
		}
		
		try {
			XPArray.Sort(tempstrArray);	
		}
		catch (Exception ex) {	
			// CR ROB: this shouldnt throw classcast exception, something like XPIllegalState exception is more appropriate
			// CR ROB: you also need to include the original exception to facilitate debugging of the root cause, pass it in
			// to the new exception
				throw new XPClassCastException("There was an error sorting the data entries.\n");
		}
		
		//temp array to store variables with no property values
		String[] npstrArray = new String[amount];

		XPArrayList sortedArrayList = new XPArrayList(length);
		String searchvalue = "";		
		int counter = 0;
		
		// CR ROB: add some high level description of what you are doing here
		for (int i=0; i<tempstrArray.length; i++)
		{
			// CR ROB: don't need a try catch block here, no value in rethrowing any exception in this block
			try {
			
				searchvalue = tempstrArray[i];
				//checking for tagging
				// CR ROB: don't really understand what you are doing here, add some description
				boolean defaultsort = searchvalue.startsWith(noprop);
				tmp_list = (IXPList) tmp_hash.GetElement(searchvalue);
				
				if (defaultsort) {
					npstrArray[counter] = searchvalue;
					counter = counter + 1;
				}
				else {
					// DCA - always remove the first one, that way we go through the list if there's more than 1
					sortedArrayList.Add(tmp_list.RemoveAt(0));
				}
			}
			catch (Exception ex) 
			{
				
				throw new XPClassCastException("There was an error grabbing the sorted data values\n");
			}
		}
		// CR ROB: add description of what is happening here
		if (nopropflag)
		{
			//here's the end index of array to start at
			XPArray.Sort(npstrArray);
			for (int j=0; j<amount; j++)
			{
				searchvalue = npstrArray[j];
				tmp_list = (IXPList) tmp_hash.GetElement(searchvalue);
				sortedArrayList.Add(tmp_list.RemoveAt(0));
			}
		}
		
		SetStateSharedVariable(sorteddataKey, sortedArrayList, sortedscope, false);
		return null;
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetTagType()
	 */
	public TagType GetTagType() {
		return TagType.NO_BODY;
	}


	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles _style) {
		return true;
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create() {
		return new SortTag();
	}


}
