package com.plumtree.taglib.logic;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlelements.*;

public class StringExprTag extends ATag
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		"com.plumtree.taglib.logic.StringExprTag");

	public static final String STRING_EXAMPLE_SETUP =
		"<![CDATA[&lt;pt:logic.variable pt:key=\"title\" pt:value=\"Administrator\"/&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;pt:logic.stringexpr pt:expr=\"($title) == Administrator\" pt:key=\"boolvalue\"/&gt;]]><![CDATA[<br>]]>";

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute EXPR;
	public static final RequiredTagAttribute KEY;
	public static final OptionalTagAttribute SCOPE;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("stringexpr",
			"This tag evaluates whether or not two strings are equal and " +
			"stores the result as a boolean in memory.  Case must match.  " +
			"It is designed to work with the logic.if tag.",
			STRING_EXAMPLE_SETUP + IfTag.IF_EXAMPLE_IF);

		EXPR = new RequiredTagAttribute("expr",
			"A tag variable, followed by a space and a string comparator, and then " +
			"another space and a string value or another tag variable.  The tag " +
			"variable must be in $ attribute notation surrounded by " +
			"parentheses.  The following operators are allowed: <![CDATA['" +
			StringOperators.OP_EQUAL + "' and '" + StringOperators.OP_NOT_EQUAL + "']]>.");

		KEY = new RequiredTagAttribute("key",
			"The key used to store the result in memory.");

		SCOPE = new OptionalTagAttribute("scope",
			"The scope used to store the result in memory.  " + LogicTagLibraryData.SCOPE_REFERENCE,
			AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
	}

	public HTMLElement DisplayTag()
	{
		String strExpr = GetTagAttributeAsString(EXPR);

		StringOperators op = FindOperator(strExpr);

		int nIndex = XPStringUtility.IndexOf(strExpr, op.toString());

		String strFirstOperand = XPStringUtility.Substring(strExpr, 0, nIndex);
		String strSecondOperand = XPStringUtility.Substring(strExpr,
			nIndex + op.toString().length(), strExpr.length());

		// Strip whitespace from operands
		strFirstOperand = XPStringUtility.Trim(strFirstOperand);
		// remove leading (but not trailing) whitespace from second operand
		int nWhitespace = 0;
		for (nWhitespace = 0; nWhitespace < strSecondOperand.length(); nWhitespace++)
		{
			if (XPStringUtility.CharAt(strSecondOperand, nWhitespace) != ' ')
			{
				// found first non-whitespace character
				break;
			}
		}
		strSecondOperand = XPStringUtility.Substring(strSecondOperand, nWhitespace, strSecondOperand.length());
 
		// Get first operand
		if (!XPStringUtility.StartsWith(strFirstOperand, IfTag.OPERAND_VARIABLE_PREFIX) ||
			IfTag.OPERAND_VARIABLE_SUFFIX !=
			XPStringUtility.CharAt(strFirstOperand, strFirstOperand.length() - 1))
		{
			return new HTMLComment("Invalid first operand in " +
					EXPR.GetName() + " attribute in " +
					TAG.GetName() + " tag.  First operand must be a variable and be " +
					"enclosed in parentheses and start with a '$': " + strFirstOperand);
		}

		String strVariable = XPStringUtility.Substring(strFirstOperand, 2, strFirstOperand.length() - 1);

		String strFirstValue = XPConvert.ToString(GetAnyStateVariable(strVariable));

		if (null == strFirstValue)
		{
			return new HTMLComment("Empty value returned for first operand variable in " +
				EXPR.GetName() + " attribute in " +
				TAG.GetName() + " tag.  First operand: " + strFirstOperand );
		}

		// get second operand
		String strSecondValue;

		if (XPStringUtility.StartsWith(strSecondOperand, IfTag.OPERAND_VARIABLE_PREFIX))
		{
			strSecondOperand = XPStringUtility.Trim(strSecondOperand);

			if (IfTag.OPERAND_VARIABLE_SUFFIX !=
				XPStringUtility.CharAt(strSecondOperand, strSecondOperand.length() - 1))
			{
				return new HTMLComment("Invalid second operand in " +
					EXPR.GetName() + " attribute in " +
					TAG.GetName() + " tag.  Variable operands must be " +
					"enclosed in parentheses and start with a '$': " + strFirstOperand);
			}

			strVariable = XPStringUtility.Substring(strSecondOperand, 2, strSecondOperand.length() - 1);

			strSecondValue = XPConvert.ToString(GetAnyStateVariable(strVariable));

			if (null == strSecondValue)
			{
				return new HTMLComment("Empty value returned for second operand variable in " +
					EXPR.GetName() + " attribute in " +
					TAG.GetName() + " tag.  Second operand: " + strSecondOperand );
			}
		} else
		{
			strSecondValue = strSecondOperand;
		}

		boolean bResult;

		// Calculate result
		if (op == StringOperators.OP_EQUAL)
		{
			bResult = (XPStringUtility.Compare(strFirstValue, strSecondValue) == 0);
		} else if (op == StringOperators.OP_NOT_EQUAL)
		{
			bResult = (XPStringUtility.Compare(strFirstValue, strSecondValue) != 0);
		} else
		{
			return new HTMLComment("Invalid operator in " +
				EXPR.GetName() + " attribute in " +
				TAG.GetName() + " tag.  Valid operators include: " +
				StringOperators.OP_EQUAL + " and " + StringOperators.OP_NOT_EQUAL);
		}

		// Determine the appropriate scope
		Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));

		if (null == scope)
		{
			log.Warn("Unable to convert scope attribute " +
				GetTagAttributeAsString(SCOPE) + " to a valid scope in Tag " +
				TAG.GetName() + ".");

			scope = Scope.PORTLET_REQUEST;
		}

		// Store the filled out data collection in its final location.
		SetStateSharedVariable(GetTagAttributeAsString(KEY),
			XPConvert.ToString(bResult), scope, false);
		
		return ProcessTagBody();
	}

    /**
     * This helper method finds a string operator in an
     * expression string.
     * 
     * @param _strExpr
     * @return
     */
	public StringOperators FindOperator(String _strExpr)
	{
		int nFirstSpace = XPStringUtility.IndexOf(_strExpr, ' ');
		// first space must be after ($x) 
		if (nFirstSpace < (IfTag.OPERAND_VARIABLE_PREFIX.length() + 2))
		{
    		throw new XPFormatException("Invalid expression '"+
        			_strExpr + "' in " + EXPR.GetName() + " attribute in " +
        			TAG.GetName() + " tag.  Operator must be surrounded by spaces.");
		}

		String strOperator = XPStringUtility.Substring(_strExpr, nFirstSpace + 1, _strExpr.length());

		strOperator = XPStringUtility.Trim(strOperator);

		int nSecondSpace = XPStringUtility.IndexOf(strOperator, ' ');
		// must have space after the operand
		if (nSecondSpace < 1)
		{
    		throw new XPFormatException("Invalid expression '"+
        			_strExpr + "' in " + EXPR.GetName() + " attribute in " +
        			TAG.GetName() + " tag.  Operator must be surrounded by spaces.");
		}

		strOperator = XPStringUtility.Substring(strOperator, 0, nSecondSpace);

		final StringOperators[] ops = StringOperators.ops;

		for (int i = 0; i < ops.length; i++)
        {
            if (XPStringUtility.Compare(strOperator, ops[i].toString()) >= 0)
            {
            	return ops[i];
            }
        }

        throw new XPIllegalArgumentException("Unable to locate valid " +
        	"string operator in expression: " + _strExpr);
	}

	public boolean SupportsAccessStyle(AccessStyles style)
	{
		// This tag does not display any JavaScript, and therefore
		// supports all access styles.
		return true;
	}

	public ATag Create()
	{
		return new StringExprTag();
	}
}