/**
 * 
 */
package com.plumtree.taglib.logic;

import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Caches data tag on the session scope
 * 
 * @author rozhang
 */
public class TagDataSessionCacheTag extends ATag
{
	public static final ITagMetaData TAG;
	
	public static final OptionalTagAttribute ATTRIB_CACHEID;
	public static final OptionalTagAttribute ATTRIB_CONTEXTID;
	public static final RequiredTagAttribute ATTRIB_EXPIRATION;
	public static final RequiredTagAttribute ATTRIB_DATAID;
	public static final OptionalTagAttribute ATTRIB_DATASCOPE;
	
	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("sessioncache", EncodeForTagDoc(
				"This tag provides caching of data set by data tags. "+
				"The difference between this tag the other cache tag, appcache tag, is that this tag caches data on a per user basis and the data is cleared after a user logs off, " +
				"while the appcache tag caches data shared between multiple users and persists across user sessions. <br/>"+ 
				"Typical use of this tag would be to cache user specific data such as User Info data or custom properties added to users. <br/>"+
				"Note that most data tags do not need to be cached. Data tags in the ptdata tag library already use server-side cached data. "+
				"Suitable data tags to cache are ones that execute database or search queries every time they run. "+
				"For example custom properties on a Community that every user has access to. <br/>"+
				"To cache a data tag, define it normally and then nest it inside of the cache tag. "+
				"Then set the data and datascope attributes of the cache tag with the data key and data scope of the data tag. "+
				"On the cache tag, define expiration duration for the cache entries and a contextid. "+
				"To cache data in a Community, pass in current Community id as the contextid value. For a page, use the Page id."
				), 
				EncodeForTagDoc(
						"<pt:logic.sessioncache pt:cachekey=\"cacheddata\" pt:expiration=\"20\" pt:data=\"datatocache\" ><br/>"+
						"&nbsp;&nbsp;<pt:custom.customquery pt:key=\"datatocache\"/> // this tag is only run when cached data expired or none is found<br/>"+
						"</pt:logic.sessioncache> <br/><br/>"+
						"<pt:logic.value pt:value=\"$cacheddata\" pt:scope=\"session\"> // accessing the cached data")); 
		
		ATTRIB_CACHEID 		= new OptionalTagAttribute("cachekey", "Specify a custom cache key. By default the dataid is used as cachekey. "+
				" Custom cachekeys is only needed to prevent cachekey conflicts.", AttributeType.STRING, null);
		ATTRIB_CONTEXTID 	= new OptionalTagAttribute("contextid", "A value representing the current context. "+
				"Current Community id to cache data for a specific Community, or current Page id to cache Page data for a user. "+
				"It is also possible to combine two or more ids as a contextid to create custom context. "+
				"For example combining current Community id and Experience Definition id caches data only for users in this Community and Experience Definition. "+
				"Do not include User id in the contextid; the logic.sessioncache tag caches per user by definition.", AttributeType.STRING, null);
		ATTRIB_EXPIRATION 	= new RequiredTagAttribute("expiration", "Number of minutes to cache an entry before rerunning the data tag to refresh the data.", AttributeType.INT);
		ATTRIB_DATAID 		= new RequiredTagAttribute("data", "Name of the tag variable where the data tag to cache sets its data. "+
				"Should be same as the pt:key attribute in the enclosed data tag.", AttributeType.STRING);
		ATTRIB_DATASCOPE 	= new OptionalTagAttribute("datascope", "Scope of the data tag variable id.", AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
		
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		String dataId 	= GetTagAttributeAsString(ATTRIB_DATAID);
		Scope dataScope = GetTagAttributeAsScope(ATTRIB_DATASCOPE);
		String cacheId 	= GetTagAttributeAsString(ATTRIB_CACHEID);
		String contextId= GetTagAttributeAsString(ATTRIB_CONTEXTID);
		int duration	= GetTagAttributeAsInt(ATTRIB_EXPIRATION);
		
		// use the tagvarid as the key if no cacheid was explicitly set
		String entryId = (cacheId==null) ? dataId : cacheId; 
		// combine the context id and the entry id to form the final cache key
		String combinedCacheKey = ( (contextId==null)?"":contextId ) +"-"+ entryId; 
		
		ITagState state = GetState();
		Object cacheData = state.GetCacheEntryData(combinedCacheKey, Scope.SESSION);
		HTMLElement cacheBody = null;
		if ( cacheData == null ) { // either nothing is cached or the entry expired
			cacheBody = ProcessTagBody(); // run the data tag to get the data set in tag variables
			Object data = GetTagAttributeAsTagVariable(ATTRIB_DATAID, dataScope);
			if ( data == null ) {
				// didn't find the tag variable. Output HTML comment. This isn't an error case since some data tags don't always a value. For example a community tag when not in a community  
				return new HTMLGenericElement("<!-- logic.appcache: No data to cache. No data found in tag variable: '"+dataId+"' with scope: '"+dataScope+"'.  -->");
			}
			state.AddCacheEntry(combinedCacheKey, data, Scope.SESSION, duration);
			cacheData = data;
		} 
		state.SetSharedVariable(entryId, cacheData, dataScope, false);
		
		
		return cacheBody;
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new TagDataSessionCacheTag();
	}
	
}
