package com.plumtree.taglib.logic;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag stores a shared variable in memory.
 */
public class VariableTag extends AEncodeTag
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		"com.plumtree.taglib.logic.VariableTag");

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final OptionalTagAttribute KEY;
	public static final RequiredTagAttribute VALUE;
	public static final OptionalTagAttribute SCOPE;
	public static final OptionalTagAttribute ENCODE;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("variable",
			"This tag stores an editable shared variable using the key and " +
			"value supplied in the tag attributes.  This tag can also be " +
			"used inside the collection tag, in which case the key attribute " +
			"is not necessary and the name value pairs will be stored directly " +
			"in the parent collection.  The key attribute is required otherwise. " +
			"It is stored in Portlet Request scope by default.  This variable can " +
			"be used in attribute value replacement or with the value tag.",

			"<![CDATA[&lt;pt:logic.variable pt:key=\"test\" pt:value=\"my value\"/&gt;]]><![CDATA[<br>]]>" +
			"would store the value 'my value' as an editable shared variable using " +
			"the key 'test'.<![CDATA[<p>]]>" + CollectionTag.COLLECTION_EXAMPLE2 +
			ATag.EncodeForTagDoc("<pt:logic.variable pt:key\"test\" pt:value=\"my value\" pt:encode=\"1\"/><br/>"+
			"would html encode and store the value 'my value' as an editable shared variable using " +
			"the key 'test'."));

		KEY = new OptionalTagAttribute("key", "The key used to store the variable in memory. " +
			"This attribute is required unless this tag is being used inside a collection tag.  " +
			"If this attribute is present and this tag is being used inside a collection tag, " +
			"this attribute will take precedence and the data will be stored using the key " +
			"specified in the attribute.",
			AttributeType.STRING, null);

		VALUE = new RequiredTagAttribute("value", "The variable value.");

		SCOPE = new OptionalTagAttribute("scope",
			"The scope used to store the variable.  " + LogicTagLibraryData.SCOPE_REFERENCE,
			AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
		
		ENCODE = new OptionalTagAttribute(ENCODE_ATTR_NAME, ENCODE_ATTR_DESC,
				AttributeType.INT, XPConvert.ToString(ENCODE_NONE));
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));

		if (null == scope)
		{
			log.Warn("Tag variable unable to convert scope attribute {0} to a valid scope with key '{1}'",
				GetTagAttributeAsString(SCOPE), GetTagAttributeAsString(KEY));

			scope = Scope.PORTLET_REQUEST;
		}

		String value = GetTagAttributeAsString(VALUE);
		if (null != value) {
			// DCA - encode value if necessary
			value = EncodeString(value);
		}
		
		if (HasParentTag(CollectionTag.TAG) && (null == GetTagAttributeAsString(KEY)))
		{
			// Store the data object in the collection.
			XPArrayList collection = (XPArrayList) GetStateSharedVariable(CollectionTag.ARRAY_LIST_KEY, Scope.TAG);
			collection.Add(value);
		} 
		else 
		{
			String strKey = GetTagAttributeAsString(KEY);

			if (null == strKey)
			{
				throw new XPException(TAG.GetName() +
					" tag cannot be displayed unless it is nested inside a " +
					"collection tag or has a valid " + KEY.GetName() +
					" attribute value.");
			}

			// Store the data object using the specified name.
			log.Debug("Tag variable setting tag variable key: '{0}' and value: '{1}' in scope: {2}.", 
					strKey, value, scope);
			SetStateSharedVariable(strKey, value, scope, false);
		}

		return ProcessTagBody();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// This tag does not display any content by itself, and therefore
		// supports all access styles.
		return true;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new VariableTag();
	}
	
	/**
	 * @see com.plumtree.taglib.logic.AEncodeTag#GetEncoding()
	 */
	protected int GetEncoding() {
		return GetTagAttributeAsInt(ENCODE);
	}
}
