using System;
using junit.framework;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.mock.tags;
using com.plumtree.portaluiinfrastructure.utils;
using com.plumtree.taglib.ptconditionals;
using com.plumtree.taskapi.portalui;
using com.plumtree.taskapi.test.portalui.mock;
using com.plumtree.xpshared.htmlelements;
namespace com.bea.alui.taglib.ptconditionals.test
{
  /// <author>rozhang</author>
  public class IsCurrentCommunityTagTest : TestCase
  {
    internal static readonly String TAG_LIBNAME = PTConditionalsTagLibraryData.LIBRARY.GetName();
    internal static readonly String TAG_NAME = IsCurrentCommunityTag.TAG.GetName();
    internal static readonly String TAG_ATTRIB_ID = IsCurrentCommunityTag.ATTRIB_COMMID.GetName();
    internal static readonly String TAG_ATTRIB_NAME = IsCurrentCommunityTag.ATTRIB_COMMNAME.GetName();
    internal static readonly String TAG_ATTRIB_DATA = IsCurrentCommunityTag.ATTRIB_COMMIDDATA.GetName();
    internal const String TESTHTML = "<p>HTML in Tag body</p>";
    internal MockEnvironment m_mockEnv = null;
    internal MockTaskAPIUICommunityService m_mockCommAPI = null;
    internal int m_currCommId = 500;
    internal String m_currCommName = "Unit Test Community";
    
    /// <seealso cref="junit.framework.TestCase#setUp()"></seealso>
    public override void setUp()
    {
      m_mockEnv = TagTestHelper.SetupTagTestEnvironment();
      TagTestHelper.CreateTestTag(TAG_LIBNAME, TAG_NAME, new IsCurrentCommunityTag());
      m_mockCommAPI = new MockTaskAPIUICommunityService();
      TaskAPIUICommunityService.SetMockInstanceForDebug(m_mockCommAPI);
      m_mockCommAPI.m_currCommId = m_currCommId;
      m_mockCommAPI.MockAddCommunity(m_currCommId, m_currCommName, null, null);
      }
    
    /// <summary>
    /// Test that tag does not return any content when not in a community
    /// </summary>
    public virtual void testNotInCommunity()
    {
      // simulate not in a community
      m_mockCommAPI.m_currCommId = -1;
      XPHashtable args = new XPHashtable();
      args.PutElement(TAG_ATTRIB_ID, "600");
      HTMLElement result = TagTestHelper.RunSingleTagWithNestedHTML(TAG_LIBNAME,
       TAG_NAME, args, TESTHTML, m_mockEnv);
      // not in a community so tag should not return anything 
      assertTrue("Unexpected content was returned: " + result.GetDisplayString(),
       TagTestHelper.VerifyNoTagContent(result));
    }
    
    /// <summary>
    /// Test Error case when no attributes are passed to the tag
    /// </summary>
    public virtual void testInvalidArguments()
    {
      XPHashtable args = new XPHashtable();
      HTMLElement result = TagTestHelper.RunSingleTagWithNestedHTML(TAG_LIBNAME,
       TAG_NAME, args, TESTHTML, m_mockEnv);
      // not in a community so tag should not return anything 
      assertTrue("Expected exception from tag is not displayed!", !TagTestHelper.VerifyNoTagContent(result) && XPStringUtility.IndexOf(result.GetDisplayString(),
       "XPIllegalArgumentException") != -1);
    }
    
    /// <summary>
    /// Tests community id attribute with values that DOES NOT match that of the current community
    /// </summary>
    public virtual void testIdAttributeNotInCurrentCommunity()
    {
      XPHashtable args = new XPHashtable();
      args.PutElement(TAG_ATTRIB_ID, "600");
      HTMLElement result = TagTestHelper.RunSingleTagWithNestedHTML(TAG_LIBNAME,
       TAG_NAME, args, TESTHTML, m_mockEnv);
      // we didn't pass in current comm id so the content inside the tag body should display 
      assertTrue("Unexpected content was returned: " + result.GetDisplayString(),
       TagTestHelper.VerifyNoTagContent(result));
    }
    
    /// <summary>
    /// Test community id attribute with values that DOES match that of the current community
    /// </summary>
    public virtual void testIdAttributeInCurrentCommunity()
    {
      XPHashtable args = new XPHashtable();
      args.PutElement(TAG_ATTRIB_ID, m_currCommId + "");
      HTMLElement result = TagTestHelper.RunSingleTagWithNestedHTML(TAG_LIBNAME,
       TAG_NAME, args, TESTHTML, m_mockEnv);
      // we didn't pass in current comm id so the content inside the tag body should display 
      assertTrue("Expected tag return content was not found: " + result.GetDisplayString(),
       TagTestHelper.VerifyTagReturnedContent(result, TESTHTML));
    }
    
    /// <summary>
    /// Test commnity name attribute with values that DOES NOT match that of the current community
    /// </summary>
    public virtual void testNameAttributeNotInCurrentCommunity()
    {
      String[] testNames = new String[]{"Different Name", "^Test Community",
       "Unit Test^", "^Community", "2 Unit", "Test Community 4"};
      for (int i = 0;i < testNames.Length;i++)
      {
        XPHashtable args = new XPHashtable();
        args.PutElement(TAG_ATTRIB_NAME, testNames[i]);
        HTMLElement result = TagTestHelper.RunSingleTagWithNestedHTML(TAG_LIBNAME,
         TAG_NAME, args, TESTHTML, m_mockEnv);
        // we didn't pass in current comm name so the content inside the tag body should display 
        assertTrue("Found tag content when none was expected for name: " + testNames[i] + ". Return content: " + result.GetDisplayString(),
         TagTestHelper.VerifyNoTagContent(result));
      }
    }
    
    /// <summary>
    /// Test community name attribute with value that DOES match that of the current community
    /// </summary>
    public virtual void testNameAttributeInCurrentCommunity()
    {
      String[] testNames = new String[]{"Unit Test", "Test Community", "Test Community^",
       "^Unit Test", "^Unit Test Community^", "Unit Test Community^", "^Unit Test Community",
       "Test", "Commu"};
      for (int i = 0;i < testNames.Length;i++)
      {
        XPHashtable args = new XPHashtable();
        args.PutElement(TAG_ATTRIB_NAME, testNames[i]);
        HTMLElement result = TagTestHelper.RunSingleTagWithNestedHTML(TAG_LIBNAME,
         TAG_NAME, args, TESTHTML, m_mockEnv);
        // expect the test html back
        assertTrue("Expected tag return content was not found for name: " + testNames[i] + ". Return content: " + result.GetDisplayString(),
         TagTestHelper.VerifyTagReturnedContent(result, TESTHTML));
      }
    }
    
    /// <summary>
    /// Test tag var data attribute with values that DOES NOT match that of the current community
    /// </summary>
    public virtual void testDataAttributeNotInCurrentCommunity()
    {
      // Test Case 1: Tag var with string value
      TagStateTestHelper mockState = TagTestHelper.GetMockTagStateInstance();
      String tagVarName = "testTagVar";
      // not current comm id
      mockState.SetTagPortletRequestVar(null, tagVarName, "200");
      XPHashtable args = new XPHashtable();
      args.PutElement(TAG_ATTRIB_DATA, tagVarName);
      HTMLElement result = TagTestHelper.RunSingleTagWithNestedHTML(TAG_LIBNAME,
       TAG_NAME, args, TESTHTML, m_mockEnv);
      assertTrue("Found tag content when none was expected for tag variable: " + tagVarName + ". Return content: " + result.GetDisplayString(),
       TagTestHelper.VerifyNoTagContent(result));
      // Test Case 2: Tag var with a list of ids
      mockState = TagTestHelper.GetMockTagStateInstance();
      tagVarName = "testTagVar2";
      // none of these community ids are valid
      XPArrayList idList = new XPArrayList();
      idList.Add("200");
      idList.Add("201");
      idList.Add("202");
      idList.Add("203");
      mockState.SetTagPortletRequestVar(null, tagVarName, idList);
      args = new XPHashtable();
      args.PutElement(TAG_ATTRIB_DATA, tagVarName);
      result = TagTestHelper.RunSingleTagWithNestedHTML(TAG_LIBNAME, TAG_NAME,
       args, TESTHTML, m_mockEnv);
      assertTrue("Found tag content when none was expected for tag variable: " + tagVarName + ". Return content: " + result.GetDisplayString(),
       TagTestHelper.VerifyNoTagContent(result));
    }
    
    /// <summary>
    /// Test tag var data attribute with values that DOES match that of the current community
    /// </summary>
    public virtual void testDataAttributeInCurrentCommunity()
    {
      // Test Case 1: Tag var with string value
      TagStateTestHelper mockState = TagTestHelper.GetMockTagStateInstance();
      String tagVarName = "testTagVar";
      mockState.SetTagPortletRequestVar(null, tagVarName, m_currCommId + "");
      XPHashtable args = new XPHashtable();
      args.PutElement(TAG_ATTRIB_DATA, tagVarName);
      HTMLElement result = TagTestHelper.RunSingleTagWithNestedHTML(TAG_LIBNAME,
       TAG_NAME, args, TESTHTML, m_mockEnv);
      assertTrue("Expected tag return content was not found. Return content:" + result.GetDisplayString(),
       TagTestHelper.VerifyTagReturnedContent(result, TESTHTML));
      // Test Case 2: Tag var with a list of ids
      mockState = TagTestHelper.GetMockTagStateInstance();
      tagVarName = "testTagVar2";
      XPArrayList idList = new XPArrayList();
      idList.Add("200");
      idList.Add("201");
      // current id
      idList.Add(m_currCommId + "");
      idList.Add("203");
      mockState.SetTagPortletRequestVar(null, tagVarName, idList);
      args = new XPHashtable();
      args.PutElement(TAG_ATTRIB_DATA, tagVarName);
      result = TagTestHelper.RunSingleTagWithNestedHTML(TAG_LIBNAME, TAG_NAME,
       args, TESTHTML, m_mockEnv);
      assertTrue("Expected tag return content was not found for name: " + tagVarName,
       TagTestHelper.VerifyTagReturnedContent(result, TESTHTML));
    }
    
    /// <summary>
    /// Test with multiple attributes together with values that DOES match that of the current community
    /// </summary>
    public virtual void testMultipleAttributesInCurrentCommunity()
    {
      // Test 1: With id and name attributes
      XPHashtable args = new XPHashtable();
      args.PutElement(TAG_ATTRIB_ID, m_currCommId + "");
      args.PutElement(TAG_ATTRIB_NAME, "Not current Community Name");
      HTMLElement result = TagTestHelper.RunSingleTagWithNestedHTML(TAG_LIBNAME,
       TAG_NAME, args, TESTHTML, m_mockEnv);
      // expect test html back
      assertTrue("Expected tag return content was not found for Community id: " + m_currCommId + ". Returned content: " + result.GetDisplayString(),
       TagTestHelper.VerifyTagReturnedContent(result, TESTHTML));
      // Test 2: With all 3 attributes
      TagStateTestHelper mockState = TagTestHelper.GetMockTagStateInstance();
      String tagVarName = "testTagVar";
      mockState.SetTagPortletRequestVar(null, tagVarName, "700");
      args = new XPHashtable();
      args.PutElement(TAG_ATTRIB_ID, "600");
      args.PutElement(TAG_ATTRIB_DATA, tagVarName);
      args.PutElement(TAG_ATTRIB_NAME, m_currCommName);
      result = TagTestHelper.RunSingleTagWithNestedHTML(TAG_LIBNAME, TAG_NAME,
       args, TESTHTML, m_mockEnv);
      // expect test html back
      assertTrue("Expected tag return content was not found for Community with name: " + m_currCommName,
       TagTestHelper.VerifyTagReturnedContent(result, TESTHTML));
    }
    
    /// <summary>
    /// Test with multiple attributes together with values that DOESN'T match that of the current community
    /// </summary>
    public virtual void testMultipleAttributesNotInCurrentCommunity()
    {
      // Test 1: With id and name attributes
      XPHashtable args = new XPHashtable();
      args.PutElement(TAG_ATTRIB_ID, "600");
      args.PutElement(TAG_ATTRIB_NAME, "Not current Community Name");
      HTMLElement result = TagTestHelper.RunSingleTagWithNestedHTML(TAG_LIBNAME,
       TAG_NAME, args, TESTHTML, m_mockEnv);
      // we didn't pass in current comm id so the content inside the tag body should display 
      assertTrue("Unexpected content was returned: " + result.GetDisplayString(),
       TagTestHelper.VerifyNoTagContent(result));
      // Test 2: With all 3 attributes
      TagStateTestHelper mockState = TagTestHelper.GetMockTagStateInstance();
      String tagVarName = "testTagVar";
      mockState.SetTagPortletRequestVar(null, tagVarName, "700");
      args = new XPHashtable();
      args.PutElement(TAG_ATTRIB_ID, "600");
      args.PutElement(TAG_ATTRIB_DATA, tagVarName);
      args.PutElement(TAG_ATTRIB_NAME, "Not current Community Name");
      result = TagTestHelper.RunSingleTagWithNestedHTML(TAG_LIBNAME, TAG_NAME,
       args, TESTHTML, m_mockEnv);
      // we didn't pass in current comm id so the content inside the tag body should display 
      assertTrue("Unexpected content was returned: " + result.GetDisplayString(),
       TagTestHelper.VerifyNoTagContent(result));
    }
  }
}