/**
 * 
 */
package com.plumtree.taglib.ptconditionals;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Conditional Tag to check if in a Community
 * 
 * @author rozhang
 */
public class IsCurrentCommunityTag extends ATag
{
	public static final ITagMetaData TAG;
	public static final OptionalTagAttribute ATTRIB_COMMID;
	public static final OptionalTagAttribute ATTRIB_COMMNAME;
	public static final OptionalTagAttribute ATTRIB_COMMIDDATA;
	public static final OptionalTagAttribute ATTRIB_DATASCOPE;
	
	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("iscurrcommunity", 
				"This is a conditional tag used to output HTML and Adaptive Tags when on a specified Community page. Nothing is displayed if not in a Community or in the specified Community. "+
				"The specific Community is determined by passing in the Community Id or name. The Community Id is specified either by directly passing the id value in the "+
				"'id' attribute or passing in the name of a tag variable with a list of ids. "+
				"Comparison against Community name will check if the current Community name contains the passed in value. Exact match is done by using the begin of line and end of line markers."+
				"In the case when both ids and name is specified the tag body is displayed if one (or more) of the checks pass. "+
				"Nest two or more tags to perform checks where both id and name need to match.",
				EncodeForTagDoc(
						"<pt:ptcond.iscurrcommunity pt:id=\"350\"/><br/>You are in Community with id 350<br/></pt:ptcond.iscurrcommunity><br/>"+
						"<pt:ptcond.iscurrcommunity pt:name=\"^My Community^\"/><br/>You are in Community with the exact name 'My Community'<br/></pt:ptcond.iscurrcommunity><br/>"+
						"<pt:ptcond.iscurrcommunity pt:name=\"^My Community\"/><br/>You are in Community with a name starting with 'My Community'<br/></pt:ptcond.iscurrcommunity><br/>"+
						"<pt:ptcond.iscurrcommunity pt:name=\"^My Community\"/><br/>"+
						"&nbsp;<pt:ptcond.iscurrcommunity pt:id=\"350\"/><br/>You are in Community with id 350 and name starting with 'My Community'</pt:ptcond.iscurrcommunity><br/>"+
						"</pt:logic.iscurrcommunity><br/>")); 
		
		ATTRIB_COMMID 		= new OptionalTagAttribute("id", 		"Community id to display the tag body in.", AttributeType.INT, "-1");
		ATTRIB_COMMNAME 	= new OptionalTagAttribute("name", 		"Community name. The '^' character is used as begin of string marker when placed in the beginning of the string and end of string at the end.", AttributeType.STRING, null);
		ATTRIB_COMMIDDATA 	= new OptionalTagAttribute("iddata", 	"Id of data tag variable with a list of Community ids to display the tag body in.", AttributeType.STRING, null);
		ATTRIB_DATASCOPE 	= new OptionalTagAttribute("datascope", "The tag scope of the iddata tag variable if it is not in the portlet request scope.", AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		int inputCommId = GetTagAttributeAsInt(ATTRIB_COMMID);
		String inputCommName = GetTagAttributeAsString(ATTRIB_COMMNAME);
		String tagVarName = GetTagAttributeAsString(ATTRIB_COMMIDDATA);
		// verify that at least one of the attributes were passed in
		if ( inputCommId == -1 && inputCommName == null && tagVarName == null ) {
			throw new XPIllegalArgumentException("None of the required attributes were defined. This tag requires at least one of the id, name or iddata.");
		}
		
		AActivitySpace env = (AActivitySpace)GetEnvironment();
		ITaskAPIUICommunity commAPI = TaskAPIUICommunityService.GetInstance();
		
		if ( !commAPI.IsCurrentPageCommunityPage(env) ) { // just quit if we aren't on a community page
			return null;
		}
		// get current community info
		int currId = commAPI.GetCurrentCommunityID(env);
		String currName = commAPI.GetCurrentCommunityName(env);
		
		boolean idMatch = false;
		boolean nameMatch = false;
		boolean idTagVarMatch = false;
		// community id match?
		if ( inputCommId > 0 ) {
			if ( currId == inputCommId ) {
				idMatch = true;
			}
		}
		// communtiy name match?
		if ( inputCommName != null && !"".equals(inputCommName) ) {
			nameMatch = GeneralHelpers.CompareStringWithWildCard(inputCommName, currName);
		}
		// community id match with values in the tag variable?
		if ( tagVarName != null )
		{
			Object inputIdVar = GetTagAttributeAsTagVariable(ATTRIB_COMMIDDATA, GetTagAttributeAsScope(ATTRIB_DATASCOPE));
			if ( inputIdVar instanceof IXPList ) { // compare tag var with list value
				IXPList idList = (IXPList)inputIdVar;
				idTagVarMatch = idList.Contains(currId+"");
			} else if ( inputIdVar instanceof String ) { // compare tag var with string value
				idTagVarMatch = (currId+"").equals(inputIdVar);
			} else {
				throw new XPIllegalArgumentException("Unexpected type for tag variable with name: '"+tagVarName+"'. Expected string or IXPList, was: "+inputIdVar);
			}
		} 
		// process tag body if one of the checks pass
		if ( idMatch || nameMatch || idTagVarMatch ) {
			return ProcessTagBody();
		} else {
			return null;
		}
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new IsCurrentCommunityTag();
	}

}
