using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.ptlink;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.data;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.taskapi.portalui;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.ptdata
{
  /// <summary>
  /// Tag for creating a data structure with Communities, Subcommunities and Community Pages
  /// </summary>
  /// <author>rozhang</author>
  public class CommunityHierarchyDataTag : APTURLDODataTag
  {
    
    /// <summary>
    /// Hierarchical data child key *
    /// </summary>
    public const String HIERDATA_PAGESKEY = "commpages";
    // constants for error strings (also used by unit tests 
    public const String ERRORMSG_INVALIDTAGVARTYPE = "The tag variable specified in commdataid attribute is of an unknown type. Only String and IXPList types are recognized: ";
    public const String ERRORMSG_INVALIDTAGVARLISTENTRYTYPE = "The list in tag variable specified in commdataid attribute contained element of unknown type: ";
    public const String ERRORMSG_INVALIDTAGVAR_COMMID = "The tag variable specified in commdataid attribute contains an invalid Community Id: ";
    public const String ERRORMSG_NONNUMERICCOMMID = "Provided Community id value is a non-numeric value: ";
    public const String ERRORMSG_INVALIDCOMMID_INLIST = "The list of Community ids values contained non-numeric value: ";
    public const String ERRORMSG_BOTHCOMMIDTYPES = "Both commid and commdataid attributes contain values. This tag requires valid value from one and only one of the attributes.";
    public const String ERRORMSG_NOCOMMIDS = "Missing valid community ids from both commid and commdataid attributes. This tag requires valid value from one of the attributes.";
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly OptionalTagAttribute ATTRIB_COMMUNITYID;
    public static readonly OptionalTagAttribute ATTRIB_COMMUNITYDATAID;
    public static readonly OptionalTagAttribute ATTRIB_DATAIDSCOPE;
    public static readonly OptionalTagAttribute ATTRIB_INCLUDEPAGES;
    public static readonly OptionalTagAttribute ATTRIB_MAXDEPTH;
    static CommunityHierarchyDataTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      // @TODO add note that you might want to cache this data tag
      // @TODO add example
      TAG = new TagMetaData("communityhierdata", "This tag takes one or more community ids and creates a hierarchical data collection of Subcommunities and Community pages for the specified communities. " + "Hierarchical data tags are used with display tags with nested data such as a tree or multi-level dropdown menus. " + COMMON_TAGVAR_DESC + COMMON_COLLECTION_DESC + COMMON_PROPLIST_HEADER + COMMON_POSTTAGMETADATA_DESC);
      ATTRIB_COMMUNITYID = new OptionalTagAttribute("commid", "A single Community id or comma separated list of Community ids values (as opposed to the commdataid attribute which takes the id of a tag variable containing the Community ids). " + "A valid value for this or the commdataid attribute is required.",
       AttributeType.STRING, null);
      ATTRIB_COMMUNITYDATAID = new OptionalTagAttribute("commdataid", "Takes a tag variable id to a data tag with either a single or multiple Community Ids. " + "A valid value for this or the commid attribute is required.",
       AttributeType.STRING, null);
      ATTRIB_DATAIDSCOPE = new OptionalTagAttribute("commdatascope", "Tag scope for commdataid attribute. Only used if there is a value for commdataid.",
       AttributeType.STRING, Scope.PORTLET_REQUEST.ToString());
      ATTRIB_INCLUDEPAGES = new OptionalTagAttribute("includepages", "Include Community pages in the hierarchy or not.",
       AttributeType.BOOLEAN, "true");
      ATTRIB_MAXDEPTH = new OptionalTagAttribute("maxdepth", "The maximum levels of sub-Communities to create structure for.",
       AttributeType.INT, "3");
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()"></seealso>
    public override HTMLElement DisplayTag()
    {
      bool includePages = GetTagAttributeAsBoolean(ATTRIB_INCLUDEPAGES);
      // parse the attribute to get parent communities
      int[] parentCommIds = GetTopLevelCommunityIds();
      // the value to set in tag variable
      XPArrayList parentComms = new XPArrayList();
      // for each parent community 1. create a data entry and 2. list of sub-communities and comm pages 
      for (int i = 0;i < parentCommIds.Length;i++)
      {
        // @TODO refactor this with the recursive method?
        int currCommId = parentCommIds[i];
        // 1. data entry for parent community
        IPTURL curr = CreateCommunityDataEntry(currCommId);
        // get subcommunities (and sub-subcommunities, and sub-sub-subcomms, etc) 
        XPArrayList children = GetSubCommunities(currCommId);
        // set the subcomms as default children
        curr.SetHierData(HashtableDataObject.HIERDATA_DEFAULTKEY, children);
        if (includePages)
        {
          // 2. get community pages
          XPArrayList pages = GetCommunityPages(currCommId);
          // set as page child nodes
          curr.SetHierData(HIERDATA_PAGESKEY, pages);
        }
        // add parent community to return list
        parentComms.Add(curr);
      }
      SetPTURLDataCollection(parentComms);
      return ProcessTagBody();
    }
    
    /// <summary>
    /// Method that gets the subcommunities and any subcommunities of the subcommunities, etc 
    /// </summary>
    /// <param name="commId"></param>
    /// <param name="includePages"></param>
    /// <returns></returns>
    protected virtual XPArrayList GetSubCommunities(int commId)
    {
      int maxDepth = GetTagAttributeAsInt(ATTRIB_MAXDEPTH);
      XPArrayList urls = RecursiveGetSubCommunityURLs(commId, maxDepth, 0);
      return urls;
    }
    
    /// <summary>
    /// Recursive method to get subcommunities and subcommunities of each subcommunity
    /// </summary>
    /// <param name="commId"></param>
    /// <param name="maxDepth"></param>
    /// <param name="currDepth"></param>
    /// <returns></returns>
    private XPArrayList RecursiveGetSubCommunityURLs(int commId, int maxDepth,
     int currDepth)
    {
      AActivitySpace space = (AActivitySpace) GetEnvironment();
      ITaskAPIUICommunity commAPI = TaskAPIUICommunityService.GetInstance();
      XPArrayList subComms = commAPI.GetSubCommununitiesAsURLs(space, commId);
      // did we reach the max depth?
      if (currDepth < maxDepth)
      {
        IXPEnumerator scEnum = subComms.GetEnumerator();
        while (scEnum.MoveNext())
        {
          IPTURL curr = (IPTURL) scEnum.GetCurrent();
          int subCommId = commAPI.GetCommunityIDFromPTURL(curr);
          if (subCommId < 1)
          {
            throw new XPIllegalStateException("Illegal Community Id found in PTURL while building tag data for Communities hierarchy. Id: " + subCommId + " is invalid");
            }
          // recurse through each sub community
          XPArrayList subsubComms = RecursiveGetSubCommunityURLs(subCommId,
           maxDepth, currDepth + 1);
          curr.SetHierData(HashtableDataObject.HIERDATA_DEFAULTKEY, subsubComms);
          // also get community pages if enabled
          bool includePages = GetTagAttributeAsBoolean(ATTRIB_INCLUDEPAGES);
          if (includePages)
          {
            // 2. get community pages
            XPArrayList pages = GetCommunityPages(subCommId);
            // set as page child nodes
            curr.SetHierData(HIERDATA_PAGESKEY, pages);
          }
        }
      }
      return subComms;
    }
    
    /// <summary>
    /// Get community pages for provided community id
    /// </summary>
    /// <param name="commId"></param>
    /// <returns></returns>
    protected virtual XPArrayList GetCommunityPages(int commId)
    {
      AActivitySpace space = (AActivitySpace) GetEnvironment();
      ITaskAPIUICommunity commAPI = TaskAPIUICommunityService.GetInstance();
      XPArrayList urls = commAPI.GetCommunityPagesAsURLs(space, commId);
      return urls;
    }
    
    /// <summary>
    /// Creates url data entry for community without child data 
    /// </summary>
    /// <param name="commId"></param>
    /// <returns></returns>
    protected virtual IPTURL CreateCommunityDataEntry(int commId)
    {
      AActivitySpace space = (AActivitySpace) GetEnvironment();
      ITaskAPIUICommunity commAPI = TaskAPIUICommunityService.GetInstance();
      String commName = commAPI.GetCommunityName(space, commId);
      IPTURL commUrl = commAPI.CreateCommunityCommonOpenerURL(space, commId,
       commName);
      return commUrl;
    }
    
    /// <summary>
    /// Creates url data entry for community page
    /// </summary>
    /// <param name="commId"></param>
    /// <param name="commPageId"></param>
    /// <param name="commPageName"></param>
    /// <returns></returns>
    protected virtual IPTURL CreateCommunityPageDataEntry(int commId, int commPageId,
     String commPageName)
    {
      AActivitySpace space = (AActivitySpace) GetEnvironment();
      ITaskAPIUICommunity task = TaskAPIUICommunityService.GetInstance();
      IPTURL pageUrl = task.CreateCommunityPageCommonOpenerURL(space, commId,
       commPageId, commPageName);
      return pageUrl;
    }
    
    /// <summary>
    /// Parses the attributes to get a list of top level community ids
    /// </summary>
    /// <returns></returns>
    protected virtual int[] GetTopLevelCommunityIds()
    {
      String commValue = GetTagAttributeAsString(ATTRIB_COMMUNITYID);
      String commRef = GetTagAttributeAsString(ATTRIB_COMMUNITYDATAID);
      // need something from one of the attributes
      if (commValue == null && commRef == null)
      {
        throw new XPIllegalArgumentException(ERRORMSG_NOCOMMIDS);
      }
      // but not both
      if (commValue != null && commRef != null)
      {
        throw new XPIllegalArgumentException(ERRORMSG_BOTHCOMMIDTYPES);
      }
      // return array, should never return null. Either valid value is returned or exception is thrown
      int[] returnIds = null;
      // we got community id value
      if (commValue != null)
      {
        // check if it is a single value or comma separated lists of community ids
        if (XPStringUtility.IndexOf(commValue, ",") != -1)
        {
          XPStringTokenizer idsTok = new XPStringTokenizer(commValue, ",");
          returnIds = new int[idsTok.GetTokenCount()];
          int count = 0;
          try
          {
            while (idsTok.HasMoreTokens())
            {
              String currId = idsTok.GetNextToken();
              returnIds[count++] = XPConvert.ToInteger(currId);
            }
          }
          catch (XPException e)
          {
            throw new XPIllegalArgumentException(ERRORMSG_INVALIDCOMMID_INLIST + commValue);
            }
        } else
        // single value
        {
          returnIds = new int[1];
          try
          {
            int singleId = XPConvert.ToInteger(commValue);
            returnIds[0] = singleId;
          }
          catch (XPException e)
          {
            throw new XPIllegalArgumentException(ERRORMSG_NONNUMERICCOMMID + commValue);
            }
        }
      } else
      // we got reference to tag variable with community ids
      if (commRef != null)
      {
        Object tagVarVal = GetTagAttributeAsTagVariable(ATTRIB_COMMUNITYDATAID,
         GetTagAttributeAsScope(ATTRIB_SCOPE));
        // tag variable with single string value
        if (tagVarVal is String)
        {
          returnIds = new int[1];
          try
          {
            int singleCommId = XPConvert.ToInteger(tagVarVal);
            returnIds[0] = singleCommId;
          }
          catch (XPException e)
          {
            throw new XPIllegalArgumentException(ERRORMSG_INVALIDTAGVAR_COMMID + tagVarVal);
            }
        } else
        // tag variable with list of values
        if (tagVarVal is IXPList)
        {
          IXPList listCommIds = (IXPList) tagVarVal;
          returnIds = new int[listCommIds.GetSize()];
          // conver the list to int array
          for (int i = 0;i < listCommIds.GetSize();i++)
          {
            Object ele = null;
            try
            {
              ele = listCommIds.GetElement(i);
              returnIds[i] = XPConvert.ToInteger(ele);
            }
            catch (XPException e)
            {
              throw new XPIllegalArgumentException(ERRORMSG_INVALIDTAGVARLISTENTRYTYPE + ele);
              }
          }
        } else
        {
          throw new XPIllegalArgumentException(ERRORMSG_INVALIDTAGVARTYPE + tagVarVal);
          }
      }
      return returnIds;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#Create()"></seealso>
    public override ATag Create()
    {
      return new CommunityHierarchyDataTag();
    }
  }
}