/**
 * 
 */
package com.plumtree.taglib.ptdata;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Tag for creating a data structure with Communities, Subcommunities and Community Pages
 * 
 * @author rozhang
 */
public class CommunityHierarchyDataTag extends APTURLDODataTag
{
	/** Hierarchical data child key **/
	public static final String HIERDATA_PAGESKEY = "commpages"; 
	
	// constants for error strings (also used by unit tests 
	public static final String ERRORMSG_INVALIDTAGVARTYPE = "The tag variable specified in commdataid attribute is of an unknown type. Only String and IXPList types are recognized: ";
	public static final String ERRORMSG_INVALIDTAGVARLISTENTRYTYPE = "The list in tag variable specified in commdataid attribute contained element of unknown type: ";
	public static final String ERRORMSG_INVALIDTAGVAR_COMMID = "The tag variable specified in commdataid attribute contains an invalid Community Id: ";
	public static final String ERRORMSG_NONNUMERICCOMMID = "Provided Community id value is a non-numeric value: ";
	public static final String ERRORMSG_INVALIDCOMMID_INLIST = "The list of Community ids values contained non-numeric value: ";
	public static final String ERRORMSG_BOTHCOMMIDTYPES = "Both commid and commdataid attributes contain values. This tag requires valid value from one and only one of the attributes.";
	public static final String ERRORMSG_NOCOMMIDS = "Missing valid community ids from both commid and commdataid attributes. This tag requires valid value from one of the attributes.";

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final OptionalTagAttribute ATTRIB_COMMUNITYID;
	public static final OptionalTagAttribute ATTRIB_COMMUNITYDATAID;
	public static final OptionalTagAttribute ATTRIB_DATAIDSCOPE;
	
	public static final OptionalTagAttribute ATTRIB_INCLUDEPAGES;
	public static final OptionalTagAttribute ATTRIB_MAXDEPTH;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		
		// @TODO add note that you might want to cache this data tag
		// @TODO add example
		TAG = new TagMetaData("communityhierdata", 	
			"This tag takes one or more community ids and creates a hierarchical data collection of Subcommunities and Community pages for the specified communities. "+
			"Hierarchical data tags are used with display tags with nested data such as a tree or multi-level dropdown menus. "+
			COMMON_TAGVAR_DESC+
			COMMON_COLLECTION_DESC+
			COMMON_PROPLIST_HEADER+
			COMMON_POSTTAGMETADATA_DESC);
		
		ATTRIB_COMMUNITYID = new OptionalTagAttribute("commid",
			"A single Community id or comma separated list of Community ids values (as opposed to the commdataid attribute which takes the id of a tag variable containing the Community ids). "+
			"A valid value for this or the commdataid attribute is required.",	
			AttributeType.STRING, null);
		
		ATTRIB_COMMUNITYDATAID = new OptionalTagAttribute("commdataid",
			"Takes a tag variable id to a data tag with either a single or multiple Community Ids. "+
			"A valid value for this or the commid attribute is required.",	
			AttributeType.STRING, null);
		
		ATTRIB_DATAIDSCOPE = new OptionalTagAttribute("commdatascope",
			"Tag scope for commdataid attribute. Only used if there is a value for commdataid.",
			AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
		
		ATTRIB_INCLUDEPAGES = new OptionalTagAttribute("includepages",
			"Include Community pages in the hierarchy or not.",
			AttributeType.BOOLEAN, "true");
		
		ATTRIB_MAXDEPTH = new OptionalTagAttribute("maxdepth",
			"The maximum levels of sub-Communities to create structure for.",
			AttributeType.INT, "3");
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		boolean includePages = GetTagAttributeAsBoolean(ATTRIB_INCLUDEPAGES);
		
		int[] parentCommIds = GetTopLevelCommunityIds(); // parse the attribute to get parent communities
		
		XPArrayList parentComms = new XPArrayList(); // the value to set in tag variable
		// for each parent community 1. create a data entry and 2. list of sub-communities and comm pages 
		for ( int i = 0; i < parentCommIds.length; i++ ) {
			// @TODO refactor this with the recursive method?
			int currCommId = parentCommIds[i];
			IPTURL curr = CreateCommunityDataEntry(currCommId); // 1. data entry for parent community
			
			XPArrayList children = GetSubCommunities(currCommId); // get subcommunities (and sub-subcommunities, and sub-sub-subcomms, etc) 
			curr.SetHierData(HashtableDataObject.HIERDATA_DEFAULTKEY, children); // set the subcomms as default children
			if ( includePages ) {
				XPArrayList pages = GetCommunityPages(currCommId); // 2. get community pages
				curr.SetHierData(HIERDATA_PAGESKEY, pages); // set as page child nodes
			}
			
			parentComms.Add(curr);  // add parent community to return list
		}
		SetPTURLDataCollection(parentComms);
		
		return ProcessTagBody();
	}
	
	/**
	 * Method that gets the subcommunities and any subcommunities of the subcommunities, etc 
	 * @param commId
	 * @param includePages
	 * @return
	 */
	protected XPArrayList GetSubCommunities(int commId)
	{
		int maxDepth = GetTagAttributeAsInt(ATTRIB_MAXDEPTH);
		XPArrayList urls = RecursiveGetSubCommunityURLs(commId, maxDepth, 0);
		
		return urls;
	}
	
	/**
	 * Recursive method to get subcommunities and subcommunities of each subcommunity
	 * @param commId
	 * @param maxDepth
	 * @param currDepth
	 * @return
	 */
	private XPArrayList RecursiveGetSubCommunityURLs(int commId, int maxDepth, int currDepth)
	{
		AActivitySpace space = (AActivitySpace)GetEnvironment();
		ITaskAPIUICommunity commAPI = TaskAPIUICommunityService.GetInstance();
		XPArrayList subComms = commAPI.GetSubCommununitiesAsURLs(space, commId);
		
		if ( currDepth < maxDepth ) { // did we reach the max depth?
			IXPEnumerator scEnum = subComms.GetEnumerator();
			while (scEnum.MoveNext()) {
				IPTURL curr = (IPTURL)scEnum.GetCurrent();
				int subCommId =  commAPI.GetCommunityIDFromPTURL(curr);
				if ( subCommId < 1 ) {
					throw new XPIllegalStateException("Illegal Community Id found in PTURL while building tag data for Communities hierarchy. Id: "+subCommId+" is invalid");
				}
				XPArrayList subsubComms = RecursiveGetSubCommunityURLs(subCommId, maxDepth, currDepth+1); // recurse through each sub community
				curr.SetHierData(HashtableDataObject.HIERDATA_DEFAULTKEY, subsubComms);
				// also get community pages if enabled
				boolean includePages = GetTagAttributeAsBoolean(ATTRIB_INCLUDEPAGES);
				if ( includePages ) {
					XPArrayList pages = GetCommunityPages(subCommId); // 2. get community pages
					curr.SetHierData(HIERDATA_PAGESKEY, pages); // set as page child nodes
				}
			}
		}
		return subComms;
	}
	
	/**
	 * Get community pages for provided community id
	 * 
	 * @param commId
	 * @return
	 */
	protected XPArrayList GetCommunityPages(int commId)
	{
		AActivitySpace space = (AActivitySpace)GetEnvironment();
		
		ITaskAPIUICommunity commAPI = TaskAPIUICommunityService.GetInstance();
		XPArrayList urls = commAPI.GetCommunityPagesAsURLs(space, commId);
		
		return urls;
	}

	/**
	 * Creates url data entry for community without child data 
	 * 
	 * @param commId
	 * @return
	 */
	protected IPTURL CreateCommunityDataEntry(int commId)
	{
		AActivitySpace space = (AActivitySpace)GetEnvironment();
		
		ITaskAPIUICommunity commAPI = TaskAPIUICommunityService.GetInstance();
		String commName = commAPI.GetCommunityName(space, commId);
		IPTURL commUrl  = commAPI.CreateCommunityCommonOpenerURL(space, commId, commName);
		
		
		return commUrl;
	}
	
	/**
	 * Creates url data entry for community page
	 * @param commId
	 * @param commPageId
	 * @param commPageName
	 * @return
	 */
	protected IPTURL CreateCommunityPageDataEntry(int commId, int commPageId, String commPageName)
	{
		AActivitySpace space = (AActivitySpace)GetEnvironment();
		
		ITaskAPIUICommunity task = TaskAPIUICommunityService.GetInstance();
		IPTURL pageUrl = task.CreateCommunityPageCommonOpenerURL(space, commId, commPageId, commPageName);
		
		return pageUrl;
		
	}
	
	/**
	 * Parses the attributes to get a list of top level community ids
	 * @return
	 */
	protected int[] GetTopLevelCommunityIds()
	{
		String commValue 	= GetTagAttributeAsString(ATTRIB_COMMUNITYID);
		String commRef 		= GetTagAttributeAsString(ATTRIB_COMMUNITYDATAID);
		
		if ( commValue == null && commRef == null ) { // need something from one of the attributes
			throw new XPIllegalArgumentException(ERRORMSG_NOCOMMIDS);
		}
		
		if ( commValue != null && commRef != null ) { // but not both
			throw new XPIllegalArgumentException(ERRORMSG_BOTHCOMMIDTYPES);
		}
		
		int[] returnIds = null; // return array, should never return null. Either valid value is returned or exception is thrown
		if ( commValue != null ) // we got community id value
		{
			// check if it is a single value or comma separated lists of community ids
			if ( commValue.indexOf(",") != -1) {
				XPStringTokenizer idsTok = new XPStringTokenizer(commValue, ",");
				returnIds = new int[idsTok.GetTokenCount()];
				int count = 0;
				
				try {
					while (idsTok.HasMoreTokens()) {
						String currId = idsTok.GetNextToken();
						returnIds[count++] = XPConvert.ToInteger(currId);
					}
				} catch (XPException e) {
					throw new XPIllegalArgumentException(ERRORMSG_INVALIDCOMMID_INLIST+commValue);
				}
			} else { // single value
				returnIds = new int[1];
				try {
					int singleId = XPConvert.ToInteger(commValue);
					returnIds[0] = singleId;
				} catch (XPException e) {
					throw new XPIllegalArgumentException(ERRORMSG_NONNUMERICCOMMID+commValue);
				}
			}
		} 
		else if ( commRef != null ) // we got reference to tag variable with community ids
		{
			Object tagVarVal = GetTagAttributeAsTagVariable(ATTRIB_COMMUNITYDATAID, GetTagAttributeAsScope(ATTRIB_SCOPE));
			if ( tagVarVal instanceof String ) { // tag variable with single string value
				returnIds = new int[1];
				try {
					int singleCommId = XPConvert.ToInteger(tagVarVal);
					returnIds[0] = singleCommId;
				} catch (XPException e) {
					throw new XPIllegalArgumentException(ERRORMSG_INVALIDTAGVAR_COMMID+tagVarVal);
				}
				
			} else if ( tagVarVal instanceof IXPList ) { // tag variable with list of values
				IXPList listCommIds = (IXPList)tagVarVal;
				returnIds = new int [listCommIds.GetSize()];
				
				for ( int i = 0; i < listCommIds.GetSize(); i++ ) { // conver the list to int array
					Object ele = null;
					try {
						ele = listCommIds.GetElement(i);
						returnIds[i] = XPConvert.ToInteger( ele );
					} catch (XPException e) {
						throw new XPIllegalArgumentException(ERRORMSG_INVALIDTAGVARLISTENTRYTYPE+ele); 
					}
				}
			} else {
				throw new XPIllegalArgumentException(ERRORMSG_INVALIDTAGVARTYPE+ tagVarVal);
			}
		}
		
		return returnIds;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new CommunityHierarchyDataTag();
	}
}
