package com.plumtree.taglib.ptdata;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.helpers.*;
import com.plumtree.uiinfrastructure.uitasks.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag will display a link back to the current page or to
 * the base URL of the current page (without the ?).  The HTTP
 * entry point portion of the link can be modified.
 */
public class PageLinkTag extends ATag
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		"com.plumtree.taglib.ptdata.PageLinkTag");
	
	public static final String EMPTY_STRING 		= "EMPTY_STRING";
	public static final String HOST_NAME			= "hostname";
	public static final String PORT_NUMBER			= "port";
	public static final String BASE_URL      		= "baseurl";
	public static final String HTTP_ENTRY_POINT 	= "httpentrypoint";
	public static final String FILE_PATH_NAME		= "filepathname";
	public static final String QUERY_STRING			= "querystring";

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final OptionalTagAttribute KEY;
	public static final OptionalTagAttribute SCOPE;
	public static final OptionalTagAttribute ENTRY_POINT;
	public static final OptionalTagAttribute USE_BASE_URL;
	public static final OptionalTagAttribute CONTROL_NAME;
	public static final OptionalTagAttribute PARSE_URL;
	
	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		KEY = new OptionalTagAttribute("key", "The key used to store the variable in memory.",
			AttributeType.STRING, "pagelink");

		SCOPE = new OptionalTagAttribute("scope",
			"The scope used to store the variable.  See the Logic Tag Library " +
			"description for more details about valid scope arguments.",
			AttributeType.STRING, Scope.PORTLET_REQUEST.toString());

		ENTRY_POINT = new OptionalTagAttribute("entrypoint", "The new HTTP entry point of the URL that will " +
			"replace the main HTTP entry point (i.e. /portal/server.pt).", AttributeType.STRING, "");

		USE_BASE_URL = new OptionalTagAttribute("usebaseurl", "Use the normalized base URL of the current " +
			"page, rather than the full URL.  This will cause the query string, and any portion of the " +
			"URL after the entry point (i.e. friendly URL sections) to be removed.",
			AttributeType.BOOLEAN, "false");
		
		CONTROL_NAME = new OptionalTagAttribute("control", "This optional atttribute takes the name of " +
				"the control and returns the base url that has current activity space and points to the control" +
				"the generated url can be used to invoke the control with in the current activity space.", 
				AttributeType.STRING, null);
		
		PARSE_URL = new OptionalTagAttribute("parseurl", "This optional atttribute parses the link of the current " +
				"page and generates a data collection that includes the hostname, baseurl (URL up to the entry point), " +
				"HTTP entry point, and query string.",
				AttributeType.BOOLEAN, "false");
		
		TAG = new TagMetaData("pagelink",
				"This tag will display a link back to the current page or to the base URL of the " +
				"current page (without the ?).  The HTTP entry point portion of the link can be modified.",
				ATag.EncodeForTagDoc("<pt:ptdata.pagelink pt:usebaseurl=\"true\"/><br>" +
				"<pt:logic.value pt:value=\"$pagelink\"/><br>" +
				"or<br>" +
				"<pt:ptdata.pagelink pt:entrypoint=\"/portal/test.pt\"/><br>" +
				"<pt:logic.value pt:value=\"$pagelink\"/>" +
				"If the \"" + PARSE_URL.GetName() + "\" attribute is specified, a DataObject will be returned. " +
				"When used in conjunction with the \"" + ENTRY_POINT.GetName() + "\" attribute, it will parse the URL " +
				"with the modified entry point." +
				"The DataObject contains the following variables:<br/>" +
				"<ul>" +
				"<li> <code>" + HOST_NAME + "</code> - The hostname in the URL.</li>" +
				"<li> <code>" + PORT_NUMBER + "</code> - The port number in the URL.</li>" +
				"<li> <code>" + BASE_URL + "</code> - The URI up to the HTTP entry point in the URL.</li>" +
				"<li> <code>" + HTTP_ENTRY_POINT + "</code> - The HTTP entry point in the URL.</li>" +
				"<li> <code>" + FILE_PATH_NAME + "</code> - The file path between the HTTP entry point and the query string if any.</li>" +
				"<li> <code>" + QUERY_STRING + "</code> - The query string including the '?' in the URL if there is one. Otherwise it will " +
				"not be stored. Use the logic.existexpr tag to check if variable exists.</li>" +
				"</ul>"));
	}

	public HTMLElement DisplayTag()
	{		
		String strURL = GetEnvironment().GetCurrentRequestURL();
		String strCurrentEntryPoint = ConfigService.getInstance().GetHTTPEntryPoint();
		int nMethodIndex;
		int nHostIndex;
		int nPortIndex;
		int nEntryPointIndex;
		int nQueryStringIndex;
		HashtableDataObject htDataObject = new HashtableDataObject();
		
		String strControl = GetTagAttributeAsString(CONTROL_NAME);
		if( null != strControl ) {
			IEnvironment env = GetEnvironment();
			AActivitySpace owner = (AActivitySpace)TaskAPIUICommon.GetEnvTypeObject(env);
			//TODO: Get Friendly URL if possible
			
			
			ASURL ctrlURL = new ASURL(owner);
			ctrlURL.SetLinkGetCachedSpace(owner);
			ctrlURL.SetControl(strControl);				
			strURL = ctrlURL.GetURLAsString();
		}

		if (GetTagAttributeAsBoolean(USE_BASE_URL))
		{
			// Use the base URL instead of the full URL.
			strURL = GetEnvironment().GetCurrentNormalizedBaseURL();
			if (strURL.endsWith("?"))
			{
				strURL = strURL.substring(0, strURL.length() - 1);
			}
		}

		String strNewEntryPoint = GetTagAttributeAsString(ENTRY_POINT);

		// If we have a new entry point, modify the URL.
		if ((null != strNewEntryPoint) && (XPStringUtility.Compare("", strNewEntryPoint) != 0))
		{
			nMethodIndex = XPStringUtility.IndexOf(strURL, "//");
			nHostIndex = XPStringUtility.IndexOf(strURL, "/", nMethodIndex + 2);
			nEntryPointIndex = XPStringUtility.IndexOf(strURL, strCurrentEntryPoint, nHostIndex + 1);

			// Replace the old entry point with the new one
			strURL = XPStringUtility.Substring(strURL, 0, nHostIndex) + strNewEntryPoint +
				XPStringUtility.Substring(strURL, nEntryPointIndex + strCurrentEntryPoint.length(), strURL.length());
		}
		
		// Split up the URL and return the various pieces as a DataObject
		if (GetTagAttributeAsBoolean(PARSE_URL))
		{
			String strEntryPoint;
			
			// Search for the correct entry point depending if the ENTRY_POINT attribute was used or not
			if ((null != strNewEntryPoint) && (XPStringUtility.Compare("", strNewEntryPoint) != 0))
			{
				strEntryPoint = strNewEntryPoint;
			}
			else
			{
				strEntryPoint = strCurrentEntryPoint;
			}

			// Deconstruct the URL and return it as a data collection.
			nMethodIndex = XPStringUtility.IndexOf(strURL, "//");
			nHostIndex = XPStringUtility.IndexOf(strURL, "/", nMethodIndex + 2);
			nEntryPointIndex = XPStringUtility.IndexOf(strURL, strEntryPoint, nHostIndex);
			nQueryStringIndex = XPStringUtility.IndexOf(strURL, "?", nEntryPointIndex + 1);
			
			String strBaseURL = XPStringUtility.Substring(strURL, 0, nEntryPointIndex + strEntryPoint.length());
			String strHostname = XPStringUtility.Substring(strURL, nMethodIndex + 2, nHostIndex);
			String strPortNumber = null;
			String strFilePathname = "";
			String strQueryString = null;
			
			// Split up the hostname and port number if any
			nPortIndex = XPStringUtility.IndexOf(strHostname, ":");
			if (nPortIndex > -1)
			{
				strPortNumber = XPStringUtility.Substring(strHostname, nPortIndex + 1, strHostname.length());
				strHostname = XPStringUtility.Substring(strHostname, 0, nPortIndex);
			}
			
			// Split up the friendly URI part and the query string.
			if (nQueryStringIndex > -1)
			{
				// Get the Friendly URI (between the entry point and the start of the query string)
				strFilePathname = XPStringUtility.Substring(strURL, nEntryPointIndex + strEntryPoint.length(), nQueryStringIndex);
				// Rest of URL which should be the query string (starting from ? and on).
				strQueryString = XPStringUtility.Substring(strURL, nQueryStringIndex, strURL.length());
			}
			else
			{
				// No query string, but get the Friendly URI part if any
				strFilePathname = XPStringUtility.Substring(strURL, nEntryPointIndex + strEntryPoint.length(), strURL.length());
			}
			
			htDataObject.SetData(HOST_NAME, strHostname);
			htDataObject.SetData(BASE_URL, strBaseURL);
			htDataObject.SetData(HTTP_ENTRY_POINT, strEntryPoint);
			htDataObject.SetData(FILE_PATH_NAME, strFilePathname);
			
			if (null != strPortNumber)
			{
				htDataObject.SetData(PORT_NUMBER, strPortNumber);
			}
			
			if (null != strQueryString)
			{
				htDataObject.SetData(QUERY_STRING, strQueryString);
			}
		}
		
		Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));

		if (null == scope)
		{
			log.Warn("Tag variable unable to convert scope attribute {0} to a valid scope with key '{1}'",
				GetTagAttributeAsString(SCOPE), GetTagAttributeAsString(KEY));

			scope = Scope.PORTLET_REQUEST;
		}

		if (GetTagAttributeAsBoolean(PARSE_URL))
		{
			SetStateSharedVariable(GetTagAttributeAsString(KEY), htDataObject, scope, false);
		}
		else
		{
			SetStateSharedVariable(GetTagAttributeAsString(KEY), strURL, scope, false);
		}

		return null;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetTagType()
	 */
	public TagType GetTagType()
	{
		return TagType.NO_BODY;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// This tag does not display any content by itself, and therefore
		// supports all access styles.
		return true;
	}

	public ATag Create()
	{
		return new PageLinkTag();
	}
}