using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.taskapi.portalui;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.uitasks;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.ptui
{
  /// <summary>
  /// The JSIncludeTag is a wrapper for the JSIncluder class (@see com.plumtree.jsincluder.JSIncluder), which
  /// replaces the old JSRegistry.  The JSIncluder generates HTML "script" and "link" tags used for the inclusion
  /// of the JavaScript, String, and CSS files that make up the JSComponents.  The JSIncluder may be used
  /// outside the context of this tag.
  /// In addition to generating includes for a specific JSComponent version and language, the JSIncludeTag
  /// also allows users to specify a "basepath" attribute that can point to any location on the imageserver.
  /// This JSIncluder will then look for the included files in the specified directory.  The JSIncluder also
  /// has a "debug" attribute that loads debug-enabled versions of the JSComponent librarires.
  /// </summary>
  /// <author>Andrew Lorek</author>
  public class JSIncludeTag : ATag
  {
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly RequiredTagAttribute NAME;
    public static readonly OptionalTagAttribute LOCALE;
    public static readonly OptionalTagAttribute DEBUG;
    static JSIncludeTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("include", "Tag for including JSComponent javascripts, string files, and css files.");
      NAME = new RequiredTagAttribute("name", "Name of the JSComponent", AttributeType.STRING);
      LOCALE = new OptionalTagAttribute("locale", "ISO 639-1 short locale code in the form language-country",
       AttributeType.STRING, XPLocale.GetDefault().ToString());
      DEBUG = new OptionalTagAttribute("debug", "Flag for loading debug version of the JSComponent",
       AttributeType.BOOLEAN, "false");
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()"></seealso>
    public override HTMLElement DisplayTag()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      String name = GetTagAttributeAsString(NAME);
      if ((name == null) || XPStringUtility.Trim(name).Equals(""))
      {
        return null;
      }
      String locale = GetTagAttributeAsString(LOCALE);
      bool debug = GetTagAttributeAsBoolean(DEBUG);
      String strComponentName = GetTagAttributeAsString(NAME);
      XPLocale localeFull = new XPLocale(locale);
      result.AddInnerHTMLElement(ConfigHelper.GetJSIncluderJSComponentInclude(strComponentName,
       debug, TaskAPIUICommon.GetEnvTypeObject(GetEnvironment()), localeFull));
      return result;
    }
    
    /// <summary>
    /// Adds the PTIncluder object to the client.  This object is used for
    /// retrieving JSComponent client classes from a page.
    /// </summary>
    public override HTMLScriptCollection DisplaySharedJavascript()
    {
      HTMLScriptCollection result = new HTMLScriptCollection();
      HTMLScript script = new HTMLScript("text/javascript");
      result.AddInnerHTMLElement(script);
      script.SetID("pt-includer");
      // use static JSIncluder method for generating the PTIncluder object on the client.
      //		script.AddInnerHTMLString(JSIncluder.GeneratePTIncluder());
      return result;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)"></seealso>
    public override bool SupportsAccessStyle(AccessStyles _style)
    {
      // This tag displays JavaScript, but should still support
      // low bandwidth mode.
      if (_style == AccessStyles.SECTION_508)
      {
        return false;
      }
      return true;
    }
    /*
     * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
     */
     
    public override ATag Create()
    {
      return new JSIncludeTag();
    }
  }
}