package com.plumtree.taglib.ptui;

import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This base class provides common functionality for all
 * Plumtree UI tags.  Tags that extend this class must
 * add the USE_SPAN attribute as follows:
 *
 * 	public static final OptionalTagAttribute USE_SPAN = new OptionalTagAttribute(
 * 		USE_SPAN_NAME, COMMON_PRE_USE_SPAN_DESC + "of myStyleID " +
 * 		COMMON_POST_USE_SPAN_DESC, USE_SPAN_TYPE, USE_SPAN_DEFAULT);
 */
public abstract class APTUITag extends ATag
{
	public static final String USE_SPAN_NAME = "usespan";

	public static final String COMMON_PRE_USE_SPAN_DESC = 
		"This attribute controls whether or not this tag wraps " +
		"the output in a span with an ID ";
	
	public static final String COMMON_POST_USE_SPAN_DESC =
		"for use in DHTML manipulation, or just displays the text.";

	public static final AttributeType USE_SPAN_TYPE = AttributeType.BOOLEAN;

	public static final String USE_SPAN_DEFAULT = "true";

	/**
	 * This method is used to access the USE_SPAN attribute in
	 * the base APTUITag class.
	 * 
	 * @return OptionalTagAttribute The USE_SPAN attribute.
	 */
	abstract public OptionalTagAttribute GetUseSpanAttribute();

	/**
	 * This helper method checks to see if the tag output needs to be wrapped
	 * in a Span and returns the final output.
	 * 
	 * @param _tagOutput The preliminary output from the tag.
	 * @param _strStyleID The style ID to use in the Span, if necessary.  This may be null.
	 * @return HTMLElement The final output of the tag (possibly wrapped in a Span).
	 */
	protected HTMLElement AddSpanToFinalOutput(HTMLElement _tagOutput, String _strStyleID)
	{
		return AddSpanToFinalOutput(_tagOutput, _strStyleID, null);
	}

	/**
	 * This helper method checks to see if the tag output needs to be wrapped
	 * in a Span and returns the final output.
	 * 
	 * @param _tagOutput The preliminary output from the tag.
	 * @param _strStyleID The style ID to use in the Span, if necessary.  This may be null.
	 * @param _styleClass The style class to use in the Span, if necessary.  This may be null.
	 * @return HTMLElement The final output of the tag (possibly wrapped in a Span).
	 */
	protected HTMLElement AddSpanToFinalOutput(HTMLElement _tagOutput,
		String _strStyleID, StyleClass _styleClass)
	{
		HTMLElement result;

		if (GetTagAttributeAsBoolean(GetUseSpanAttribute()))
		{
			result = new HTMLSpan();

			if (null != _strStyleID)
			{
				result.SetID(_strStyleID);
			}

			if (null != _styleClass)
			{
				result.SetStyleClass(_styleClass);
			}

			result.AddInnerHTMLElement(_tagOutput);
		} else
		{
			result = _tagOutput;
		}

		return result;
	}
}
