package com.plumtree.taglib.ptui;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag displays errors on the page so that they can be
 * placed and formatted as desired.  If this tag is displayed
 * on a page, errors will no longer be displayed in the normal
 * error location.
 */
public class ErrorTag extends ATag
{
	public static final String ERROR_EXAMPLE =
		"<![CDATA[&lt;pt:ptui.error/&gt;]]><![CDATA[<br>]]>" +
		"or<![CDATA[<br>]]>" +
		"<![CDATA[&lt;pt:ui.error&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;p&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;pt:ptui.errortext/&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;p/&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;pt:ptui.errorextendedtext/&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;p/&gt;]]><![CDATA[<br>]]>" +
		"<![CDATA[&lt;/pt:ui.error&gt;]]>";

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RelatedChildTag ERROR_TEXT;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("error",
			"This tag displays errors on the page so that they can be " +
			"placed and formatted as desired.  If the " + ErrorTextTag.TAG.GetName() + " " +
			"tag is included inside this tag, the contents of this tag will " +
			"only be processed if there is an error.  If the child tag is not " +
			"present, then the error messages will be formatted and displayed " +
			"from this tag in the same style as used by the portal.  If this tag " +
			"is displayed on a page, errors will no longer be displayed in the " +
			"normal error location.", ERROR_EXAMPLE);

		ERROR_TEXT = new RelatedChildTag(ErrorTextTag.TAG);		
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		AActivitySpace currentAS = (AActivitySpace)GetEnvironment();

		// Get errors associated with this activity space
		IXPEnumerator eErrors = UIErrorHandler.GetErrors(currentAS.GetSubSession(),
			currentAS.GetApplication(), currentAS, true,
			UIErrorHandler.GetDisplayAdminErrors(currentAS));

		// If we have a child error text tag
		if (HasChildTag(ErrorTextTag.TAG) || HasChildTag(ErrorExtendedMessageTag.TAG))
		{
			// and we have errors
			if (eErrors.MoveNext())
			{
				// Display the inner contents of this tag
				HTMLElement result = ProcessTagBody();
				
				// We clear all UI Errors for this user. 
				UIErrorHandler.ClearErrors(currentAS.GetSubSession(),
						currentAS.GetApplication());
				
				return result;
			}

			// otherwise don't display anything (no errors)
			return null;
		} else
		{
			// We want to display the full list of errors formatted in HTML.
			ErrorDisplayHelper helper = new ErrorDisplayHelper(currentAS, eErrors);

			// NOTE: this will clear all UI Errors for this user.
			return helper.DisplayErrors();
		}
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// This tag does not display any content by itself, and therefore
		// supports all access styles.
		return true;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new ErrorTag();
	}
}