package com.plumtree.taglib.ptui;

import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag conditionally processes inner content based on the option
 * parameter and PTLoginHelper settings. So if "option" is set to 
 * "remembermypassword" and PTLoginHelper.ShowRememberMyPassword() returns
 * true the tag body is processed.
 */
public class LoginOptionsEnabledTag extends ATag
{
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute OPTION;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("loginoptionsenabled",
			"Conditionally processes inner content based on the option parameter " +
			"and Portal settings. For example, if the option attribute is set to " +
			"remembermypassword and the Portal setting for remembermypassword is true, " +
			"then the HTML inside the tag will be displayed.<![CDATA[<p/>]]>" +
			"The remembermypassword setting is configured by the AllowAutoConnect parameter " +
			"in portalconfig.xml (setting AllowAutoConnect to true sets remembermypassword to true.) " +
			"The createaccount setting is set with the Allow creation of Self Registered Users " +
			"in the Portal Administration's Portal Settings.<![CDATA[<p/>]]>" +
			"The authsource setting is set with AllowDefaultLoginPageAuthSournce in portalconfig.xml.");

		OPTION =  new RequiredTagAttribute("option", "A login option to check. This attribute " +
			"must be set to one of the following: " +
			"authsource (processes tag body if the authsource setting is set to true)," +
			"remembermypassword (processes tag body if the remembermypassword setting is set to true), " +
			"createaccount (processes tag body if the createaccount setting is set to true)",
			AttributeType.STRING);
	}

	private static final String REMEMBER_MY_PASSWORD = "remembermypassword";
	private static final String CREATE_ACCOUNT = "createaccount";
	private static final String AUTHSOURCE = "authsource";
	private static final String ERR_MSG = " is an invalid value for the loginoptionsenabled tag option attribute.";
	
	/**
     * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
     */
    public HTMLElement DisplayTag()
    {
        HTMLElementCollection myCollection = new HTMLElementCollection();
        String myOption = GetTagAttributeAsString(OPTION);
        if (null != myOption && myOption.equals(REMEMBER_MY_PASSWORD))
        {
            if (PTLoginHelper.ShowRememberMyPassword())
            {
                myCollection.AddInnerHTMLElement(ProcessTagBody());
            }
        }
        else if (null != myOption && myOption.equals(CREATE_ACCOUNT))
        {
            if (PTLoginHelper.GetAllowUserCreation( (AActivitySpace) GetEnvironment(), ((AActivitySpace) GetEnvironment()).GetApplication() ))
            {
                myCollection.AddInnerHTMLElement(ProcessTagBody());
            }
        }
        else if (null != myOption && myOption.equals(AUTHSOURCE))
        {
            if (PTLoginHelper.ShowAuthDropdown())
            {
                myCollection.AddInnerHTMLElement(ProcessTagBody());
            }
        }
        else
        {
            myCollection.AddInnerHTMLElement(new HTMLComment(myOption + ERR_MSG));
        }
		return myCollection;
    }

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// This tag does not display any JavaScript, and therefore
		// supports all access styles.
		return true;
	}

	/**
     * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
     */
    public ATag Create()
    {
        return new LoginOptionsEnabledTag();
    }
}