using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.restconsumerframework.pathways;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.common;
using HashtableDataObject=com.plumtree.portaluiinfrastructure.tags.data.HashtableDataObject;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using AActivitySpace=com.plumtree.uiinfrastructure.activityspace.AActivitySpace;
using HTMLElement=com.plumtree.xpshared.htmlelements.HTMLElement;
namespace com.plumtree.taglib.pathways
{
  /// <summary>
  /// This class implements the pathways.viewsdata tag. This tag can be used in any adaptive layout.
  /// It fills in a list of Pathways Views retrieved from the Pathways REST API.
  /// </summary>
  /// <author>BobbyT</author>
  public class ViewsDataTag : ATag
  {
    private static OpenLogger logger = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PATHWAYS),
     "com.plumtree.taglib.pathways.ViewsDataTag");
    private const String TAG_NAME = "viewsdata";
    public const String NAME = "name";
    public const String VIEW_ID = "viewid";
    public const String LAST_MODIFIED = "lastmodified";
    public const String EVERYONE_VIEW = "Everyone View";
    public const String MY_VIEW = "My View";
    public const String FAVORITES = "Favorites";
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly RequiredTagAttribute ATTRIB_ID;
    public static readonly OptionalTagAttribute ATTRIB_SCOPE;
    public static readonly String DATATAG_DESCRIPTION = ATag.EncodeForTagDoc("Generates the data collection needed to show a list of views defined in Pathways, and stores it in memory " + "using the variable name specified by the " + "<code>" + CommonTagAttributeConstants.ATTRIB_ID_NAME + "</code> attribute. " + "Each result is a DataObject with the following variables:<br/>" + "<ul>" + "<li> <code>" + NAME + "</code> - The name of the view.</li>" + "<li> <code>" + VIEW_ID + "</code> - The id of the view.</li>" + "<li> <code>" + LAST_MODIFIED + "</code> - The date and time of when the view was last modified.</li>" + "</ul>" + "Use tags from the Logic library to iterate over the list and display results.");
    public static readonly String DATATAG_EXAMPLE = ATag.EncodeForTagDoc("This simple example loops through results and shows the view name and view id for each view.<br/><br/>" + "<code>" + "<pt:" + PathwaysTagLibraryData.LIBRARY.GetName() + "." + TAG_NAME + " pt:id=\"pathwaysviews\"/><br/>" + "<pt:logic.foreach pt:data=\"pathwaysviews\" pt:var=\"view\"><br/>" + "<pt:logic.value pt:value=\"$view." + NAME + "\"/> - " + "<pt:logic.value pt:value=\"$view." + VIEW_ID + "\"/><br/>" + "</pt:logic.foreach><br/>" + "</code>");
    static ViewsDataTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData(TAG_NAME, DATATAG_DESCRIPTION, DATATAG_EXAMPLE);
      ATTRIB_ID = new RequiredTagAttribute(CommonTagAttributeConstants.ATTRIB_ID_NAME,
       CommonTagAttributeConstants.ATTRIB_ID_DESC, AttributeType.STRING);
      ATTRIB_SCOPE = new OptionalTagAttribute(CommonTagAttributeConstants.ATTRIB_SCOPE_NAME,
       CommonTagAttributeConstants.ATTRIB_SCOPE_DESC, AttributeType.STRING,
       Scope.PORTLET_REQUEST.ToString());
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#Create()"></seealso>
    public override ATag Create()
    {
      return new ViewsDataTag();
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()"></seealso>
    public override HTMLElement DisplayTag()
    {
      AActivitySpace space = (AActivitySpace) GetEnvironment();
      IPathwaysRestAPIService service = PathwaysRestAPIService.GetInstance(space);
      Scope tagScope = Scope.GetScope(GetTagAttributeAsString(ATTRIB_SCOPE));
      String strDataReference = GetTagAttributeAsString(ATTRIB_ID);
      bool bPathwaysEnabled = service.IsPathwaysEnabled();
      if (bPathwaysEnabled)
      {
        // Pathways Request API should look something like this:
        // http://[Pathways REST API Base URL/views?format=XML
        PathwaysViewsRestData[] views = service.GetViews();
        IXPList objViews = new XPArrayList(views.Length);
        for (int i = 0;i < views.Length;i++)
        {
          HashtableDataObject htObject = new HashtableDataObject();
          //views returned by pathways are always in english, to support localization, I will add a check
          //for the view names and use the appropriate translated one in the i18n messages.
          String viewName = "";
          if ((views[i].name).Equals(EVERYONE_VIEW))
          {
            viewName = space.GetString(2018, "ptmsgs_portalbrowsingmsgs");
            } else
          if ((views[i].name).Equals(MY_VIEW))
          {
            viewName = space.GetString(2019, "ptmsgs_portalbrowsingmsgs");
            } else
          if ((views[i].name).Equals(FAVORITES))
          {
            viewName = space.GetString(2020, "ptmsgs_portalbrowsingmsgs");
            } else
          //else we could have custom views that we cannot anticipate for, the user will have to localize
          //the names of this string
          //TODO document how to localize the view names?
          {
            viewName = views[i].name;
          }
          htObject.SetData(NAME, viewName);
          htObject.SetData(VIEW_ID, views[i].id);
          htObject.SetData(LAST_MODIFIED, views[i].lastModified);
          objViews.Add(htObject);
          logger.Debug("View: " + views[i].name + ", Id: " + views[i].id + ", Last Modified: " + views[i].lastModified);
          }
        // Add tag result list to memory
        GetState().SetSharedVariable(strDataReference, objViews, tagScope,
         true);
      } else
      {
        logger.Info("Pathways is disabled. Enable Pathways access in Portal Settings");
        IXPList objViews = new XPArrayList(0);
        // Just add an empty list to memory if Pathways is disabled.
        GetState().SetSharedVariable(strDataReference, objViews, tagScope,
         true);
      }
      return ProcessTagBody();
    }
  }
}