package com.plumtree.taglib.pathways;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.restconsumerframework.pathways.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.common.*;
import com.plumtree.portaluiinfrastructure.tags.data.HashtableDataObject;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.activityspace.AActivitySpace;
import com.plumtree.xpshared.htmlelements.HTMLElement;

/**
 * This class implements the pathways.tagclouddata tag. This tag can be used in any adaptive layout.
 * It fills in a list of Pathways Tags and their weights based on the current
 * search query. If there is no current search query, it will do a wildcard search to grab all tags up
 * to the specified maximum or 20 tags by default.
 * 
 * @author BobbyT
 */
public class TagCloudDataTag extends ATag
{
	private static OpenLogger logger = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PATHWAYS),
			"com.plumtree.taglib.pathways.TagCloudDataTag");
	
	private static final String TAG_NAME = "tagclouddata";
	private static final String EVERYONE_TAG_VIEW_ATTRIBUTE = "ev";
	
	public static final String TAG_VIEW_ATTRIBUTE_NAME = "tagview";
	public static final String TAG_COUNT_ATTRIBUTE_NAME = "tagcount";
	
	public static final String NAME      			= "name";
	public static final String COUNT    			= "count";
	public static final String NORMALIZED_COUNT		= "normalizedcount";
	
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute ATTRIB_ID;
	public static final OptionalTagAttribute ATTRIB_SCOPE;
	public static final OptionalTagAttribute ATTRIB_TAGVIEW;
	public static final OptionalTagAttribute ATTRIB_TAGCOUNT;
	
	public static final String DATATAG_DESCRIPTION = ATag.EncodeForTagDoc(
			"Generates a data collection needed to show a tag cloud, and stores it in memory " +
			"using the variable name specified by the " +
			"<code>" + CommonTagAttributeConstants.ATTRIB_ID_NAME + "</code> attribute. " +
			"By default the tag cloud retrieved will use the Everyone View (\"ev\"). The view " +
			"can be specified using the <code>" + TAG_VIEW_ATTRIBUTE_NAME + "</code> attribute. " +
			"The <code>" + TAG_COUNT_ATTRIBUTE_NAME + "</code> attribute specifies the maximum number " +
			"of tags to return. By default it will return 20 tags." +
			"Each result is a DataObject with the following variables:<br/>" +
			"<ul>" +
			"<li> <code>" + NAME + "</code> - The name of the result.</li>" +
			"<li> <code>" + COUNT + "</code> - The number of times this tag has been applied on items.</li>" +
			"<li> <code>" + NORMALIZED_COUNT + "</code> - The weight of the result.</li>" +
			"</ul>" +
			"Use tags from the Logic library to iterate over the list and display results.");
	
	public static final String DATATAG_EXAMPLE = ATag.EncodeForTagDoc(
			"This simple example loops through results and shows the name and weight (as a clickable link) for each tag.<br/><br/>" +
			  "<code>" +
			  "<pt:" + PathwaysTagLibraryData.LIBRARY.GetName() + "." + TAG_NAME + " pt:id=\"tagcloudresults\"/><br/>" +
			  "<pt:logic.foreach pt:data=\"tagcloudresults\" pt:var=\"tag\"><br/>" +
			  "<pt:logic.value pt:value=\"tag." + NORMALIZED_COUNT + "\"/>. " +
			  	"<pt:core.html pt:tag=\"a\" " + "href=\"[TAG_SEARCH_URL]\" title=\"tag." + NAME + "\">" +
			  							"<pt:logic.value " + "pt:value=\"tag." + NAME + "\"/>" +
			  	"</pt:core.html><br/><br/>" +
			  "</pt:logic.foreach><br/>" +
			  "</code>"
			  );
	
	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData(TAG_NAME, DATATAG_DESCRIPTION, DATATAG_EXAMPLE);

		ATTRIB_ID = new RequiredTagAttribute(CommonTagAttributeConstants.ATTRIB_ID_NAME,
				CommonTagAttributeConstants.ATTRIB_ID_DESC,	
				AttributeType.STRING);

		ATTRIB_SCOPE = new OptionalTagAttribute(CommonTagAttributeConstants.ATTRIB_SCOPE_NAME,
				CommonTagAttributeConstants.ATTRIB_SCOPE_DESC,
				AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
		
		ATTRIB_TAGVIEW = new OptionalTagAttribute(TAG_VIEW_ATTRIBUTE_NAME, "This attribute specifies the Pathways view to use when " +
				"retrieving the tag cloud.",
				AttributeType.STRING, EVERYONE_TAG_VIEW_ATTRIBUTE);

		ATTRIB_TAGCOUNT = new OptionalTagAttribute(TAG_COUNT_ATTRIBUTE_NAME, "This attribute specifies the number of tags returned " +
			"for the tag cloud.",
			AttributeType.STRING, "20");
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create() 
	{
		return new TagCloudDataTag();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag() 
	{
		AActivitySpace space = (AActivitySpace) GetEnvironment();
		IPathwaysRestAPIService service = PathwaysRestAPIService.GetInstance(space);
		String strTagView = GetTagAttributeAsString(ATTRIB_TAGVIEW);
		String strTagCount = GetTagAttributeAsString(ATTRIB_TAGCOUNT);
		Scope tagScope = Scope.GetScope( GetTagAttributeAsString(ATTRIB_SCOPE) );
		String strDataReference = GetTagAttributeAsString(ATTRIB_ID);
		boolean bPathwaysEnabled = service.IsPathwaysEnabled();
		
		if (bPathwaysEnabled) 
		{
			ISearchResultModelRO model = null;
			String queryString = null;
			IXPList objTags = null;
			
			if (space.GetModelExists(SearchResultModel.STR_MVC_CLASS_NAME)) 
			{
				// We're in the SearchResults activity space. Get the original search query string
				// to use for the Pathways Tag Cloud query request.
				model = (ISearchResultModelRO) space.GetModel(SearchResultModel.STR_MVC_CLASS_NAME);
				queryString = model.GetSearchString();
				
				// Check if query string is null or empty and default to "*" search query.
				if (null == queryString || queryString.length() == 0)
				{
					queryString = "*";
				}
				
				// The following was originally used to get the Tag Cloud from the Search Result
				// model to save an extra trip to Pathways/Search. However, MikeS has stated that
				// when the counts are returned for an item search it's really just a count of how 
				// many duplicate tags have been applied to that same item and does not represent 
				// realistic weights to generate a valid search cloud.
//				XPHashtable htTagList = model.GetTagCloud();
//				objTags = new XPArrayList(htTagList.GetSize());
//				
//				IXPEnumerator enumTagList = htTagList.GetElements();
//				while (enumTagList.MoveNext())
//				{
//					PathwaysTagRestData tag = (PathwaysTagRestData) enumTagList.GetCurrent();					
//					objTags.Add(getSingleTag(tag));
//				}
			} 
			else 
			{
				// Non SearchResult Activity Space. We have to make a call to Pathways
				// to get the TagCloud. Use wildcard "*" search for the search query.
				queryString = "*";
			}
			
			// Make a request to Pathways to get the Tag Cloud.
			// Pathways Request API should look something like this:
			// http://[Pathways REST API Base URL/tags/ev?format=XML&query=*&tagCount=20
			PathwaysTagRestData[] tags = service.GetTagCloud(strTagView, PathwaysConstants.KEY_XML, queryString, strTagCount);
			objTags = new XPArrayList(tags.length);
			
			for (int i = 0; i < tags.length; i++)
			{
				// Guard against null values that we get from Pathways Requests
				if (null != tags[i].name)
				{
					objTags.Add(getSingleTag(tags[i], bPathwaysEnabled));
				}
			}

		    // Add tag result list to memory
			GetState().SetSharedVariable(strDataReference, objTags, tagScope, true);
		} 
		else
		{
			logger.Info("Pathways is disabled. Enable Pathways access in Portal Settings");
			IXPList objTags = new XPArrayList();
		    
			// Just add empty tag result list to memory if Pathways is disabled.
			GetState().SetSharedVariable(strDataReference, objTags, tagScope, true);
		}
		
		return ProcessTagBody();
	}	
	
	private HashtableDataObject getSingleTag(PathwaysTagRestData tag, boolean pathwaysEnabled)
	{
		// Create the data object.
		HashtableDataObject htObject = new HashtableDataObject();

		htObject.SetData(NAME, tag.name);
		htObject.SetData(COUNT, tag.count);
		htObject.SetData(NORMALIZED_COUNT, tag.normalizedCount);
		
		logger.Debug("Tag Name: " + tag.name + " with count: " + tag.count + ", normalizedcount: " + tag.normalizedCount);
		
		return htObject;
	}
}
