using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.login;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.standard
{
  /// <summary>
  /// The choose, when and otherwise tags allow you to insert
  /// content on a page based on conditional statements of user
  /// and group membership. The choose tag denotes the start of a
  /// secured content section, and the when tags include a test
  /// condition that defines who has access to the enclosed content.
  /// The otherwise tags include content that should be displayed
  /// as default.
  /// </summary>
  public class WhenTag : ATag
  {
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly RequiredParentTag CHOOSE;
    public static readonly RequiredTagAttribute TEST;
    static WhenTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("when", ChooseTag.CHOOSE_DESCRIPTION, ChooseTag.CHOOSE_EXAMPLE_SYNTAX + ChooseTag.CHOOSE_EXAMPLE_1 + ChooseTag.CHOOSE_EXAMPLE_2);
      CHOOSE = new RequiredParentTag(ChooseTag.TAG);
      TEST = new RequiredTagAttribute("test", "description");
    }
    private const String ACL_GROUP_STRING_PREFIX = "stringToACLGroup(\'";
    private const String ACL_GROUP_STRING_SUFFIX = "\').isMember($currentuser)";
    private const String GUEST_STRING = "isGuest($currentuser)";
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()"></seealso>
    public override HTMLElement DisplayTag()
    {
      HTMLElement result = new HTMLElementCollection();
      bool bHasAccess = false;
      // syntax: stringToACLGroup('id1,id2, id3').isMember($currentuser)
      // or
      // syntax: isGuest($currentuser)
      String strTestAttribute = GetTagAttributeAsString(TEST);
      int nACLGroupIndex = XPStringUtility.IndexOf(strTestAttribute, ACL_GROUP_STRING_PREFIX);
      int nGuestIndex = XPStringUtility.IndexOf(strTestAttribute, GUEST_STRING);
      // If they are trying to do a user/group check
      if (-1 != nACLGroupIndex)
      {
        // Get the list of user and group IDs
        String strTestCase = XPStringUtility.Substring(strTestAttribute, nACLGroupIndex + ACL_GROUP_STRING_PREFIX.Length,
         XPStringUtility.IndexOf(strTestAttribute, ACL_GROUP_STRING_SUFFIX));
        // first check groups
        bHasAccess = WhenTag.CheckAccess(strTestCase, (IPTSession) GetEnvironment().GetUserSession(),
         result, true);
        // next check users
        if (!bHasAccess)
        {
          bHasAccess = WhenTag.CheckAccess(strTestCase, (IPTSession) GetEnvironment().GetUserSession(),
           result, false);
        }
      } else
      // If they are trying to do a guest check
      if (-1 != nGuestIndex)
      {
        // Check if they are guest.
        bHasAccess = LoginHelper.INSTANCE.GetIsGuestUser(GetEnvironment().GetUserSession());
        }
      if (bHasAccess)
      {
        // Set that we have access so the otherwise tag won't display.
        SetStateVariable(ChooseTag.HAS_ACCESS_VARIABLE, true, Scope.PORTLET_REQUEST);
        result.AddInnerHTMLElement(ProcessTagBody());
      }
      return result;
    }
    
    /// <summary>
    /// Check to see if the current user is a member of one of the groups in the checkGroups
    /// array.
    /// </summary>
    /// <param name="checkGroups"></param>
    /// <param name="ptSession"></param>
    /// <returns>
    /// True if the current user is a member of one of the groups in the array;
    /// false if not.
    /// </returns>
    protected static bool CheckAccess(String testCase, IPTSession ptSession,
     HTMLElement result, bool isGroupTest)
    {
      // TODO: update this code so that it is easier to understand. 
      bool hasAccess = false;
      int groupStart;
      int groupEnd;
      String groupCase;
      if (isGroupTest)
      {
        groupStart = XPStringUtility.IndexOf(testCase, "group=");
      } else
      {
        groupStart = XPStringUtility.IndexOf(testCase, "user=");
      }
      if (groupStart != -1)
      {
        groupEnd = XPStringUtility.IndexOf(testCase, ";", groupStart);
        if (groupEnd == -1)
        {
          HTMLComment myComment = new HTMLComment("Bad syntax: Missing semi-colon in test case.");
          result.AddInnerHTMLElement(myComment);
        } else
        {
          if (isGroupTest)
          {
            groupCase = XPStringUtility.Substring(testCase, groupStart + 6,
             groupEnd);
          } else
          {
            groupCase = XPStringUtility.Substring(testCase, groupStart + 5,
             groupEnd);
          }
          XPStringTokenizer checkGroups = new XPStringTokenizer(groupCase,
           ",");
          while (checkGroups.HasMoreTokens())
          {
            String strGroupToken = XPStringUtility.Trim(checkGroups.GetNextToken());
            if (isGroupTest)
            {
              Object[] userGroupIDs = (Object[]) ptSession.GetSessionInfo().GetCurrentUserGroupIDs();
              if (userGroupIDs != null)
              {
                for (int j = 0;j < userGroupIDs.Length;j++)
                {
                  int compGroup = XPConvert.ToInteger(userGroupIDs[j]);
                  if (compGroup == XPConvert.ToInteger(strGroupToken))
                  {
                    hasAccess = true;
                    break;
                  } else
                  {
                    hasAccess = false;
                  }
                }
                if (hasAccess)
                {
                  break;
                }
              }
            } else
            {
              int currUser = ptSession.GetSessionInfo().GetCurrentUserID();
              if (currUser == XPConvert.ToInteger(strGroupToken))
              {
                hasAccess = true;
                break;
              } else
              {
                hasAccess = false;
              }
            }
          }
        }
      }
      return hasAccess;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)"></seealso>
    public override bool SupportsAccessStyle(AccessStyles _style)
    {
      // This tag does not display any JavaScript, and therefore
      // supports all access styles.
      return true;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#Create()"></seealso>
    public override ATag Create()
    {
      return new WhenTag();
    }
  }
}