using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.server;
using com.plumtree.taglib.standard;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
namespace com.plumtree.taglib.standard.basetags
{
  /// <summary>
  /// Base class for the GatewayLinkTag which prints a Gateway anchor tag.
  /// </summary>
  public abstract class AGatewayLinkTag : ALinkTag
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.taglib.standard.basetags.AGatewayLinkTag");
    // The public static final ITagMetaData cannot be in this
    // class, otherwise dynamic discovery will get confused if
    // we try to override it.
    public const String GATEWAY_LINK_TAG_DESCRIPTION = "This tag allows you to build gatewayed links to remote pages and " + "include references to associated portal objects, usually a portlet or community. " + "When the link is executed, the portal sends any preferences " + "associated with the referenced object to the remote server.";
    // This example follows lower case tag usage.  This is not correct
    // for the legacy version of this tag, but we do not generate tagdocs
    // for the legacy tags, so this is okay.
    public static readonly String GATEWAY_LINK_TAG_EXAMPLES = "The code below creates a link to a remote page associated with the portlet with ID 201.<![CDATA[<p>]]>" + "<![CDATA[&lt;pt:standard.gatewaylink class=\"myStyle\" pt:" + AttributeConstants.OBJECT_ID_ATTRIBUTE + "=\"201\" pt:href=\"doStuff.aspx?ID=5\"&gt;Click here&lt;/pt:standard.gatewaylink&gt;]]><![CDATA[<p>]]>" + "The arguments in the resulting URL tell the portal to send the preferences " + "associated with the portlet to the remote server. The code is replaced with " + "a link of the following form:<![CDATA[<p>]]>" + "<![CDATA[&lt;a href=\"http://portal.plumtree.com/portal/server.pt/gateway/PTARGS_0_0_201_0_0_43/doStuff.aspx?ID=5\" class=\"myStyle\"&gt;Click here&lt;/a&gt;]]><![CDATA[<p>]]>" + "The code below creates a link to a page associated with the Web Service with ID 200, " + "and also sends the Community preferences from the Community with ID 301 to the remote server.<![CDATA[<p>]]>" + "<![CDATA[&lt;pt:standard.gatewaylink pt:href=\"http://myRemoteServer/myTestPage.jsp\" pt:" + AttributeConstants.OBJECT_ID_ATTRIBUTE + "=\"200\" pt:" + AttributeConstants.CLASS_ID_ATTRIBUTE + "=\"47\" pt:communityid=\"301\"&gt;Click here&lt;/pt:standard.gatewaylink&gt;]]><![CDATA[<p>]]>" + "You can use the pt:gatewayLink tag to gateway documents that have not been crawled or uploaded to " + "the portal using the ID for the associated WWW Data Source, as shown in the code below.<![CDATA[<p>]]>" + "<![CDATA[&lt;pt:standard.gatewaylink pt:href=\"http://myRemoteServer/mydocs/WhitePaper2002.doc\" pt:" + AttributeConstants.OBJECT_ID_ATTRIBUTE + "=\"202\" pt:" + AttributeConstants.CLASS_ID_ATTRIBUTE + "=\"35\"&gt;WhitePaper2002&lt;/pt:standard.gatewaylink&gt;]]>";
    // The TagAttributes cannot go in the base class because they
    // are all lower case for the standard tags and camelCase for
    // the legacy tags.
    public abstract ATagAttribute GetCommunityIDAttribute();
    public abstract ATagAttribute GetPageIDAttribute();
    public abstract ATagAttribute GetObjectIDAttribute();
    public abstract ATagAttribute GetClassIDAttribute();
    public static readonly String CLASS_ID_DESCRIPTION = "The portal object type. The default is portlet (" + PT_CLASSIDS.PT_GADGET_ID + "). This tag also supports cards (" + PT_CLASSIDS.PT_CATALOGCARD_ID + "), Data Sources (" + PT_CLASSIDS.PT_DATASOURCE_ID + "), and Web Services (" + PT_CLASSIDS.PT_WEBSERVICE_ID + ").";
    public const String COMMUNITY_ID_DESCRIPTION = "The ID of the associated Community. " + "The Community preferences from this Community will be sent to the remote server.";
    public const String PAGE_ID_DESCRIPTION = "The ID of the associated page (can be positive or negative).";
    // This attribute can be in the base class because it is the same for both
    // the standard and legacy tags (all lower case).
    public static readonly RequiredTagAttribute HREF = new RequiredTagAttribute("href",
     "The URL to the remote page. If you pass in a relative URL, " + "the portal will use the configuration settings for the referenced portal " + "object to create the gatewayed URL.");
    
    /// <seealso cref="com.plumtree.taglib.standard.ALinkTag#GetHREF()"></seealso>
    public override String GetHREF()
    {
      // Make sure our new URL matches the popup flag of the current page.
      // This will return 0 (no flag) if the current page is not a gateway page.
      int nPrefType = GetGatewayPopUpFlag(GetEnvironment().GetCurrentRequestURL());
      // Generate the base gateway url
      String strGatewayLink = GatewayHelpers.ConstructPrefPageLink((AActivitySpace) GetEnvironment(),
       GetTagAttributeAsInt(GetObjectIDAttribute()), GetTagAttributeAsInt(GetCommunityIDAttribute()),
       GetTagAttributeAsInt(GetPageIDAttribute()), nPrefType, GetTagAttributeAsInt(GetClassIDAttribute()));
      // Add the gatewayed url to the base gateway url.
      strGatewayLink+=GatewayHelpers.CreateGatewayFriendlyURL(GetTagAttributeAsString(HREF));
      return strGatewayLink;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)"></seealso>
    public override bool SupportsAccessStyle(AccessStyles _style)
    {
      // This tag does not display any JavaScript, and therefore
      // supports all access styles.
      return true;
    }
    
    /// <summary>
    /// This method retrieves the gateway popup flag from the provided URL.
    /// </summary>
    /// <param name="_strURL">A URL.</param>
    /// <returns>
    /// int The gateway popup flag.  This will return 0 if the URL
    /// is not a gateway URL.
    /// </returns>
    public static int GetGatewayPopUpFlag(String _strURL)
    {
      // try to find "/gateway/PAGE_SPECIAL_ARGS_" substring
      int nStartIndex = XPStringUtility.IndexOf(_strURL, GatewayHelpers.GATEWAY_URL_PREFIX);
      if (-1 == nStartIndex)
      {
        // Not a gateway URL.
        return GatewayHelpers.NOFLAGS;
      }
      nStartIndex+=GatewayHelpers.GATEWAY_URL_PREFIX.Length;
      int nArgEndIndex = XPStringUtility.IndexOf(_strURL, GatewayHelpers.GATEWAY_ARG_DELIMITER,
       nStartIndex);
      if (-1 == nArgEndIndex)
      {
        // Malformed gateway URL.
        return GatewayHelpers.NOFLAGS;
      }
      int nPrefArg;
      try
      {
        nPrefArg = XPConvert.ToInteger(XPStringUtility.Substring(_strURL,
         nStartIndex, nArgEndIndex));
      }
      catch (Exception e)
      {
        // Malformed gateway URL
        log.Warn(e, "Malformed gateway URL.  Unable to retrieve popup flag to generate gateway link. URL: " + _strURL);
        return GatewayHelpers.NOFLAGS;
      }
      return nPrefArg;
    }
  }
}