using System;
using com.plumtree.portaluiinfrastructure.opener;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.taglib.standard;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
namespace com.plumtree.taglib.standard.basetags
{
  /// <summary>
  /// Base Class for the OpenerLinkTag which prints an opener link.
  /// </summary>
  public abstract class AOpenerLinkTag : ALinkTag
  {
    // The public static final ITagMetaData cannot be in this
    // class, otherwise dynamic discovery will get confused if
    // we try to override it.
    public const String OPENERLINKTAG_DESCRIPTION = "The pt:openerLink tag creates a link that can open " + "or view an object (or properties of an object) that " + "already exists within the portal.";
    // This example follows lower case tag usage.  This is not correct
    // for the legacy version of this tag, but we do not generate tagdocs
    // for the legacy tags, so this is okay.
    public static readonly String OPENERLINKTAG_EXAMPLE = "For example, the code below is transformed into a link " + "that brings users to a specific page within a Community.<![CDATA[<p>]]>" + "<![CDATA[&lt;pt:standard.openerlink pt:" + AttributeConstants.OBJECT_ID_ATTRIBUTE + "=\"219\" pt:" + AttributeConstants.CLASS_ID_ATTRIBUTE + "=\"514\" pt:mode=\"2\" target=\"myWindow\" onclick=\"window.top.open" + "(\'\',\'myWindow\',\'height=800,width=700,status=no,toolbar=no,menubar=no, " + "location=no\');\"&gt;view community page&lt;/pt:standard.openerlink&gt;]]>";
    // The TagAttributes cannot go in the base class because they
    // are all lower case for the standard tags and camelCase for
    // the legacy tags.
    public abstract ATagAttribute GetObjectIDAttribute();
    public abstract ATagAttribute GetClassIDAttribute();
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    // This attribute can be in the base class because it is the same for both
    // the standard and legacy tags (all lower case).
    public static readonly RequiredTagAttribute OPENER_MODE;
    static AOpenerLinkTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      OPENER_MODE = new RequiredTagAttribute("mode", "The action of the link (Opener mode). 1 = Open, 2 = View, 3 =ViewMeta.",
       AttributeType.INT);
    }
    
    /// <seealso cref="com.plumtree.taglib.standard.ALinkTag#GetHREF()"></seealso>
    public override String GetHREF()
    {
      return GetOpenerURL();
    }
    
    /// <seealso cref="com.plumtree.taglib.standard.ALinkTag#GetOnClick()"></seealso>
    public override String GetOnClick()
    {
      // Popup mode is used by Collab 3.x to open the selected
      // item (user profile) in the parent (portal) page rather
      // than the popup collab page.  This is not documented
      // behavior and should not be used outside of Collab 3.x.
      // define popup modes as 100+ regular modes -jas
      if (GetTagAttributeAsInt(OPENER_MODE) > 100)
      {
        String strOpenerURL = GetOpenerURL();
        return "if (window.opener) { window.opener.location=&quot;" + strOpenerURL + "&quot;;window.opener.focus();} else { document.location=&quot;" + strOpenerURL + "&quot;;}";
        }
      // Use a normal link
      return null;
    }
    protected virtual String GetOpenerURL()
    {
      int nOpenerMode = GetTagAttributeAsInt(OPENER_MODE);
      // define popup modes as 100+ regular modes -jas
      if (nOpenerMode > 100)
      {
        nOpenerMode-=100;
      }
      // use the common opener framework -jas
      return PTOpenerLinks.GetOpenerURLOpenObjID(GetTagAttributeAsInt(GetClassIDAttribute()),
       GetTagAttributeAsInt(GetObjectIDAttribute()), nOpenerMode, (AActivitySpace) GetEnvironment());
      }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)"></seealso>
    public override bool SupportsAccessStyle(AccessStyles _style)
    {
      // Popup mode is used by Collab 3.x to open the selected
      // item (user profile) in the parent (portal) page rather
      // than the popup collab page.  This is not documented
      // behavior and should not be used outside of Collab 3.x.
      // define popup modes as 100+ regular modes -jas
      if (GetTagAttributeAsInt(OPENER_MODE) > 100)
      {
        if (_style == AccessStyles.STANDARD)
        {
          return true;
        }
        return false;
      }
      // This tag does not display any JavaScript outside of the popup mode,
      // and therefore supports all access styles.
      return true;
    }
  }
}