package com.plumtree.taglib.standard;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.login.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * The choose, when and otherwise tags allow you to insert
 * content on a page based on conditional statements of user
 * and group membership. The choose tag denotes the start of a
 * secured content section, and the when tags include a test
 * condition that defines who has access to the enclosed content.
 * The otherwise tags include content that should be displayed
 * as default.
 */
public class WhenTag extends ATag
{
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredParentTag CHOOSE;
	public static final RequiredTagAttribute TEST;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("when",
			ChooseTag.CHOOSE_DESCRIPTION, ChooseTag.CHOOSE_EXAMPLE_SYNTAX +
				ChooseTag.CHOOSE_EXAMPLE_1 + ChooseTag.CHOOSE_EXAMPLE_2);

		CHOOSE = new RequiredParentTag(ChooseTag.TAG);

		TEST = new RequiredTagAttribute("test", "description");
	}

	private static final String ACL_GROUP_STRING_PREFIX = "stringToACLGroup('";
	private static final String ACL_GROUP_STRING_SUFFIX = "').isMember($currentuser)";

	private static final String GUEST_STRING = "isGuest($currentuser)";

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		HTMLElement result = new HTMLElementCollection();

		boolean bHasAccess = false;

		// syntax: stringToACLGroup('id1,id2, id3').isMember($currentuser)
		// or
		// syntax: isGuest($currentuser)

		String strTestAttribute = GetTagAttributeAsString(TEST);

		int nACLGroupIndex = strTestAttribute.indexOf(ACL_GROUP_STRING_PREFIX);
		int nGuestIndex = strTestAttribute.indexOf(GUEST_STRING);

		// If they are trying to do a user/group check
		if (-1 != nACLGroupIndex)
		{
			// Get the list of user and group IDs
			String strTestCase = strTestAttribute.substring(nACLGroupIndex + ACL_GROUP_STRING_PREFIX.length(),
				strTestAttribute.indexOf(ACL_GROUP_STRING_SUFFIX));
								
			// first check groups
			bHasAccess = WhenTag.CheckAccess(strTestCase, (IPTSession) GetEnvironment().GetUserSession(), result, true);
								
			// next check users
			if (!bHasAccess)
			{
				bHasAccess = WhenTag.CheckAccess(strTestCase, (IPTSession) GetEnvironment().GetUserSession(), result, false);
			}
		} else
		// If they are trying to do a guest check
		if (-1 != nGuestIndex)
		{
			// Check if they are guest.
			bHasAccess = LoginHelper.INSTANCE.GetIsGuestUser(GetEnvironment().GetUserSession());
		}

		if (bHasAccess)
		{
			// Set that we have access so the otherwise tag won't display.
			SetStateVariable(ChooseTag.HAS_ACCESS_VARIABLE, new Boolean(true), Scope.PORTLET_REQUEST);

			result.AddInnerHTMLElement(ProcessTagBody());
		}

		return result;
	}

	/**
	 * Check to see if the current user is a member of one of the groups in the checkGroups
	 * array.
	 * 
	 * @param checkGroups
	 * @param ptSession
	 * @return True if the current user is a member of one of the groups in the array;
	 * false if not.
	 */
	protected static boolean CheckAccess(String testCase, IPTSession ptSession, HTMLElement result, boolean isGroupTest)
	{
		// TODO: update this code so that it is easier to understand. 

		boolean hasAccess = false;
		
		int groupStart, groupEnd;
		String groupCase;
		
		if (isGroupTest)
		{
			groupStart = testCase.indexOf("group=");
		}
		else
		{
			groupStart = testCase.indexOf("user=");
		}

		if (groupStart != -1)
		{
			groupEnd = testCase.indexOf(";", groupStart);
			if (groupEnd == -1)
			{
				HTMLComment myComment = new HTMLComment( "Bad syntax: Missing semi-colon in test case." );
				result.AddInnerHTMLElement(myComment);
			}
			else
			{
				if (isGroupTest)
				{
					groupCase = testCase.substring(groupStart + 6, groupEnd);					
				}
				else
				{
					groupCase = testCase.substring(groupStart + 5, groupEnd);					
				}
				
				XPStringTokenizer checkGroups = new XPStringTokenizer(groupCase, ",");
				while (checkGroups.HasMoreTokens()) 
				{
					String strGroupToken = checkGroups.GetNextToken().trim();

					if (isGroupTest)
					{
						Object[] userGroupIDs = (Object[])ptSession.GetSessionInfo().GetCurrentUserGroupIDs();
						if (userGroupIDs != null) 
						{
							for (int j = 0; j < userGroupIDs.length; j++) 
							{
								int compGroup = XPConvert.ToInteger(userGroupIDs[j]);
								if (compGroup == XPConvert.ToInteger(strGroupToken)) 
								{
									hasAccess = true;
									break;
								} 
								else 
								{
									hasAccess = false;
								}
							}
							if (hasAccess) 
							{
								break;
							}
						}
					}
					else
					{
						int currUser = ptSession.GetSessionInfo().GetCurrentUserID();
						if (currUser == XPConvert.ToInteger(strGroupToken))
						{
							hasAccess = true;
							break;
						} else {
							hasAccess = false;
						}
					}
				}
			}
		}

		return hasAccess;		
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// This tag does not display any JavaScript, and therefore
		// supports all access styles.
		return true;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new WhenTag();
	}
}