package com.plumtree.taglib.standard.basetags;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.gateway.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag is used in a gateway page (as opposed to a MyPage or Community
 * page) to set whether or not the page should be displayed in
 * Hosted Mode.  If the page is displayed in Hosted Mode, it will
 * be surrounded by the Portal banner (header and footer).  It can
 * also be used to set the page title and subtitle, as well as the
 * values returned by the pagename and realmname tags on a gateway
 * page.
 */
public abstract class ADisplayModeTag extends ATag
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.taglib.standard.basetags.ADisplayModeTag");

	// The public static final ITagMetaData cannot be in this
	// class, otherwise dynamic discovery will get confused if
	// we try to override it.

	public static final String DISPLAY_MODE_TAG_DESCRIPTION =
		"This tag is used in a gateway page (as opposed to a MyPage or Community " +
		"page) to set whether or not the page should be displayed in " +
		"Hosted Mode.  If the page is displayed in Hosted Mode, it will " +
		"be surrounded by the Portal banner (header and footer).  It can " +
		"also be used to set the page title and subtitle, as well as the " +
		"values returned by the pagename and realmname tags on a gateway " +
		"page.";

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final OptionalTagAttribute MODE;
	public static final OptionalTagAttribute TITLE;
	public static final OptionalTagAttribute SUBTITLE;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		MODE = new OptionalTagAttribute("mode",
			"This controls whether or not the page should be displayed in " +
			"Hosted Mode.  If the page is displayed in Hosted Mode, it will " +
			"be surrounded by the Portal banner (header and footer). To turn " +
			"Hosted Mode on, this attribute should be \"Hosted\". The default " +
			"value for this attribute, \"\", will display the page in Plain Mode.",
			AttributeType.STRING, "");

		TITLE = new OptionalTagAttribute("title",
			"This sets the title used for the Hosted Mode page, as well as the " +
			"value returned by the realmname tag on a gateway page.", AttributeType.STRING, "");

		SUBTITLE = new OptionalTagAttribute("subtitle",
			"This sets the subtitle used for the Hosted Mode page, as well as the " +
			"value returned by the pagename tag on a gateway page.", AttributeType.STRING, "");
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		AActivitySpace asOwner = (AActivitySpace) GetEnvironment();

		if (asOwner.GetModelExists(GatewayModel.strModelName))
		{
			GatewayModel gatewayModel = (GatewayModel) asOwner.GetModel(GatewayModel.strModelName);

			// Hosted Display Mode
			String strHostedMode = GetTagAttributeAsString(MODE);

			boolean bHostedModePTTagPresent = ((!"".equals(strHostedMode)) && 
				XPStringUtility.EqualsIgnoreCase(strHostedMode.trim(), ("Hosted")));

			gatewayModel.SetIsHostedModePTTagPresent(bHostedModePTTagPresent);

			// Title
			String strTitle = GetTagAttributeAsString(TITLE);

			if (!"".equals(strTitle))
			{
				gatewayModel.SetTitle(strTitle);
			}

			// Subtitle
			String strSubtitle = GetTagAttributeAsString(SUBTITLE);

			if (!"".equals(strSubtitle))
			{
				gatewayModel.SetSubtitle(strSubtitle);
			}

			log.Debug("DisplayModeTag: Stored " +
				"DISPLAYMODEKEY=[" + strHostedMode + "]\r\n" +
				"TITLEKEY=[" + strTitle + "]\r\n" +
				"SUBTITLEKEY=[" + strSubtitle + "]\r\n" +
				" on Gateway Model.");
		}

		return null;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetTagType()
	 */
	public TagType GetTagType()
	{
		return TagType.NO_BODY;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// This tag does not display any content by itself, and therefore
		// supports all access styles.
		return true;
	}
}