package com.plumtree.taglib.standard.basetags;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tree.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag outputs a URL to a tree that allows users to
 * select options from a structured list of Plumtree objects.
 * This URL is suitable for use in JavaScript.
 */
public abstract class ATreeURLTag extends ATag
{
	// The public static final ITagMetaData cannot be in this
	// class, otherwise dynamic discovery will get confused if
	// we try to override it.

	public static final String TREE_URL_TAG_DESCRIPTION =
		"This tag outputs a URL to a tree that allows users to " +
		"select options from a structured list of Plumtree objects. " +
		"This URL is suitable for use in JavaScript.";

	public static final String TREE_URL_TAG_EXAMPLES =
		"The code below creates a URL to a tree that can be used " +
		"to select a user. See the DevCenter documentation for more details.<![CDATA[<p>]]>" +
		"<![CDATA[&lt;pt:standard.treeurl pt:RootID=\"1\" pt:Multi=\"false\" pt:SelectMd=\"2\" pt:SubmitMd=\"2\" pt:Submit=\"PickerSubmit\" pt:Title=\"User\" pt:SubTitle=\"Pick a user\" pt:Class=\"1\"/&gt;]]>";

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		HTMLElement result = new HTMLElementCollection();

		// Output just the url (for use in javascript).
		result.AddInnerHTMLString(ATreeURLTag.BuildTreeURL(this).GetURLAsString());

		return result;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#GetTagType()
	 */
	public TagType GetTagType()
	{
		return TagType.NO_BODY;
	}

	/**
	 * Build a URL that will open the standard tree control, using attributes from a PT XML tag
	 *
	 * @param ATag the current tag.
	 *
	 * @return ASURL for the tree.
	 */
	public static ASURL BuildTreeURL(ATag _tag)
	{
		AActivitySpace asOwner = (AActivitySpace) _tag.GetEnvironment();
		boolean isCached = false;
		
		// The URL to open a tree -jas
		ASURL newUserTree = new ASURL(asOwner);
		newUserTree.SetLinkCreateNewSpace(TreeAS.STR_MVC_CLASS_NAME, asOwner);
		// JF- We need to set the sub space key, this way in security mode 1, 
		// the tree will have the same security as the parent page
		// (it's important, as the tree often calls JavaScripts on the parent page, see bug 24726)  
		newUserTree.SetSubSpaceKey("GatewayTree");

		IXPEnumerator names = _tag.GetTagAttributeNames();

		// Pass the PT attributes through as query string args -jas
		while (names.MoveNext())
		{
			String strName = (String) names.GetCurrent();
			String strValue   =  _tag.GetTagAttribute(strName);

			newUserTree.AddControlArgument(strName, strValue);

			if (0 == strName.compareTo(ASConstants.SPACEID)) {
				isCached = true;
			}
		}

		if (!isCached) {
			newUserTree.SetControl(StartTreeControl.STR_MVC_CLASS_NAME);
		}

		return newUserTree;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// This tag does not display any JavaScript, and therefore
		// supports all access styles.
		return true;
	}
}