package com.bea.alui.taglib.kdpage;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portalpages.browsing.directory.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.server.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag only works with the Knowledge Directory activity space. It stores the bread crumb path to the current folder in memory.
 * Each bread crumb data object will have the following variables: 'name' for the bread crumbs' folder name, 'id' for the folder's Id,
 * and 'url' for the URL to access that folder. The current folder will not contain the URL, instead it will have the constant 'EMPTY_STRING'.
 * 
 * @author Dustin Ang
 */
public class BreadCrumbsDataTag extends AKDPageTag 
{	
	private static final String TAG_NAME = "breadcrumbsdata";
	
	public static final String NAME = "name";
	public static final String ID = "id";
	public static final String URL = "url";

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData(TAG_NAME, ATag.EncodeForTagDoc(KD_PAGE_ONLY_CONSTANT +
			"Stores the breadcrumb path to the current folder in memory. " +
			"Each breadcrumb data object will have the following variables: <br/>" + 
			"<ul>" + 
			"<li> <code>" + NAME +"</code> - The breadcrumbs folder name.</li>" + 
			"<li> <code>" + ID + "</code> - The folder ID.</li>" + 
			"<li> <code>" + URL + "</code> - The URL to access that folder. The current folder will not contain the URL, instead it will have the constant '"
			+ EMPTY_STRING +"'.</li>"
			+ "</ul>")
		);
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create() {
		return new BreadCrumbsDataTag();
	}

	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag() 
	{
		IXPList breadCrumbsList = new XPArrayList();

		IDirModelRO dirModel = GetDirModel();

		// If we are in the root folder, no need to query for the path.  Legacy
		// UI doesn't even show the breadcrumbs in the root folder.
		if (dirModel.GetCurrentFolderID() == PT_INTRINSICS.PT_FOLDER_ROOTFOLDER)
		{
			// DCA - populate data object
			HashtableDataObject htObject = new HashtableDataObject();
			htObject.SetData(NAME, dirModel.GetCurrentFolderName());
			htObject.SetData(ID, XPConvert.ToString(dirModel.GetCurrentFolderID()));
			// This is the current folder, so do not add the URL
			htObject.SetData(URL, EMPTY_STRING);

			breadCrumbsList.Add(htObject);
		} else
		{
			// DCA - get bread crumb path from model
			IASQueryResult qrFolders = dirModel.GetBreadCrumbPath();
			for (int i=0; i<qrFolders.GetCount(); i++) {
				// DCA - get name and id from query result
				String strName     = (String) qrFolders.GetFields(i, PT_PROPIDS.PT_PROPID_NAME);
				int nFolderID	   = XPConvert.ToInteger(qrFolders.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID));
	
				// DCA - get URL
				IPTURL subFolderURL  = TaskAPIUIDirectory.CreateCommonOpenerDirectoryFolderURL((AActivitySpace) GetEnvironment(), 
							strName, dirModel.GetDirectoryMode(), nFolderID, strName);
				
				// DCA - populate data object
				HashtableDataObject htObject = new HashtableDataObject();
				htObject.SetData(NAME, strName);
				htObject.SetData(ID, XPConvert.ToString(nFolderID));
				// DCA - do not add the URL for the current folder
				if (nFolderID == dirModel.GetCurrentFolderID()) {
					htObject.SetData(URL, EMPTY_STRING);
				} else {
					htObject.SetData(URL, subFolderURL.GetFullURL());
				}
				breadCrumbsList.Add(htObject);
			}
		}

		// DCA - add list to memory
		Scope tagScope = Scope.GetScope( GetTagAttributeAsString(ATTRIB_SCOPE));
		String strDataReference = GetTagAttributeAsString(ATTRIB_ID);
		GetState().SetSharedVariable(strDataReference, breadCrumbsList, tagScope, true);
		
		return ProcessTagBody();
	}
}
