package com.bea.alui.taglib.kdpage;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.portalpages.browsing.directory.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.server.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag only works with the Knowledge Directory activity space. It contains information about the current folder.
 * The current folder data object contains the following variables: 'name' returns the name of the current folder.
 * 'id' has the Id of the current folder, 'level' indicates the level of the current folder with the root folder being 
 * level 1, and its subfolders being level 2 and so on, 'parenturl' returns the URL to access the parent folder, or the constant 
 * 'EMPTY_STRING' if the current folder doesn't have a parent folder, 'editurl' returns the URL to switch the Directory Mode to 
 * Edit if the user has rights to edit the directory, 'EMPTY_STRING' if they don't, 'canaddexpert' returns true or false on whether 
 * the current user can add him or herself as an expert. If the user is already an expert, 'canaddexpert' will return false.
 * 
 * @author Dustin Ang
 */
public class CurrentFolderDataTag extends AKDPageTag 
{	
	private static final String TAG_NAME = "currentfolderdata";
	
	public static final String NAME = "name";
	public static final String ID = "id";
	public static final String LEVEL = "level";
	public static final String PARENT_URL = "parenturl";
	public static final String EDIT_URL = "editurl";
	public static final String CAN_ADD_EXPERT = "canaddexpert";
	
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData(TAG_NAME,
			// TODO CR DLH: Shouldn't we also include the description, in case they want to use that?
			// DCA - I'm not sure how useful the description would be. We've never had an instance where we displayed an object's
			// description while in view mode.
			ATag.EncodeForTagDoc(KD_PAGE_ONLY_CONSTANT +
			"Contains information about the current folder. " +
			"The current folder data object contains the following variables:<br/>" + 
			"<ul>" +
			"<li> <code>" + NAME +"</code> - The name of the current folder.</li>" +
			"<li> <code>" + ID +"</code> - The ID of the current folder.</li>" + 
			"<li> <code>" + LEVEL +"</code> - Indicates the level of the current folder with the root folder being " +
			"level 1 and its subfolders being level 2 and so on.</li>" + 
			"<li> <code>" + PARENT_URL +"</code> - The URL to access the parent folder or the constant '" + 
			EMPTY_STRING +"' if the current folder does not have a parent folder.</li>" + 
			"<li> <code>" + EDIT_URL + "</code> - The URL to switch the Directory Mode to Edit if the user has rights to edit the directory or '" + EMPTY_STRING +
			"' if they do not.</li>" + 
			"<li> <code>" + CAN_ADD_EXPERT +"</code> True or false depending on whether the current user can add him or herself as an expert. " +
			"If the user is already an expert, returns false.</li>" +
			"</ul>")
		);
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create() {
		return new CurrentFolderDataTag();
	}

	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag() {
		IDirModelRO dirModel = GetDirModel();
		String strFolderName = dirModel.GetCurrentFolderName();
		String strFolderId = XPConvert.ToString(dirModel.GetCurrentFolderID());
		String strFolderLevel = XPConvert.ToString(dirModel.GetCurrentFolderLevel());
		int nParentFolderId = dirModel.GetParentFolderID();
		IPTURL parentURL = null;
		// DCA - check whether the current folder has a parent folder
		if (nParentFolderId > 0) {
			// DCA - it does have a parent folder, get URL for it
			parentURL = TaskAPIUIDirectory.CreateCommonOpenerDirectoryFolderURL((AActivitySpace) GetEnvironment(), 
						"", dirModel.GetDirectoryMode(), nParentFolderId);
		}
		IPTURL editModeURL = TaskAPIUIDirectory.CreateCommonOpenerDirectoryFolderURL((AActivitySpace) GetEnvironment(), 
				"", DirConstants.EDIT_MODE, dirModel.GetCurrentFolderID());
		
		// DCA - populate data object
		HashtableDataObject htObject = new HashtableDataObject();
		htObject.SetData(NAME, XPHttpUtility.HtmlEncode(strFolderName));
		htObject.SetData(ID, strFolderId);
		htObject.SetData(LEVEL, strFolderLevel);
		if (null == parentURL) {
			htObject.SetData(PARENT_URL, EMPTY_STRING);
		} else {
			htObject.SetData(PARENT_URL, parentURL.GetFullURL());
		}
		boolean bCanEditFolder;
		if(dirModel.InRootFolder()){
			bCanEditFolder = (dirModel.HasAccessToEditFolder() || dirModel.CheckForActivityRight(PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_EDIT_DOCUMENT_DIRECTORY));
		}else{
			bCanEditFolder = dirModel.HasAccessToEditFolder();
		}
		
		if (bCanEditFolder) {
			htObject.SetData(EDIT_URL, editModeURL.GetFullURL());
		} else {
			htObject.SetData(EDIT_URL, EMPTY_STRING);
		}
		boolean bCanAddExpert = dirModel.HasSelfSelectExpertRightOnThisFolder() && !dirModel.IsRelatedExpert();
		htObject.SetData(CAN_ADD_EXPERT, XPConvert.ToString(bCanAddExpert));
		
		String strDataReference = GetTagAttributeAsString(ATTRIB_ID);
		Scope tagScope = Scope.GetScope( GetTagAttributeAsString(ATTRIB_SCOPE));
		GetState().SetSharedVariable(strDataReference, htObject, tagScope, true);
		
		return ProcessTagBody();
	}
}
