package com.bea.alui.taglib.kdpage;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.portalpages.browsing.directory.*;
import com.plumtree.portaluiinfrastructure.opener.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag only works with the Knowledge Directory activity space. It stores a list of related resources of the current folder in memory. " +
 * Each related resource data object will have the following variables: 'name' for the resource's name, 'id' for the resource's Id, " +
 * 'description' for the resource's description, and 'url' for the URL to view the resource.
 * 
 * @author Dustin Ang
 */
public class RelatedResourcesDataTag extends AKDPageTag 
{
	private static final String TAG_NAME = "relatedresourcesdata";
	private static final String HAS_MORE_PREFIX = "hasmore";
	
	public static final String NAME = "name";
	public static final String ID = "id";
	public static final String URL = "url";
	public static final String DESC = "description";
	
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final RequiredTagAttribute ATTRIB_TYPE;
	public static final OptionalTagAttribute ATTRIB_LIMIT;

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData(TAG_NAME, ATag.EncodeForTagDoc(KD_PAGE_ONLY_CONSTANT +
			"Stores a list of related resources of the current folder in memory. " +
			"Each related resource data object will have the following variables: <br/>" +
			"<ul>" + 
			"<li> <code>" + NAME +"</code> - The name of the resource.</li>" + 
			"<li> <code>" + ID + "</code> - The resource ID.</li>" + 
			"<li> <code>" + DESC +"</code> - The resource description.</li>" + 
			"<li> <code>" + URL +"</code> - The URL to view the resource.</li>" +
			"</ul>")
		);
		
		ATTRIB_TYPE = new RequiredTagAttribute("type",
			"The type of related resources to retrieve. There are five different types of related resources. Use " + PT_INTRINSICS.PT_RELATED_RESOURCE_COMMUNITIES +
			" for related communities, " + PT_INTRINSICS.PT_RELATED_RESOURCE_FOLDERS + " for related folders, " + PT_INTRINSICS.PT_RELATED_RESOURCE_GADGETS + 
			" for related portlets, " + PT_INTRINSICS.PT_RELATED_RESOURCE_EXPERTS + " for related experts, and " + PT_INTRINSICS.PT_RELATED_RESOURCE_CONTENT_MANAGERS +
			" for related content managers. Other values will be ignored.",
			AttributeType.INT);

		ATTRIB_LIMIT = new OptionalTagAttribute("limit",
				"Usede to limit the number of resources displayed. If no limit is defined, the entire list will be returned. " +
				"If a limit is set, a boolean of whether there are more links than the limit will be stored in memory using the key " +
				"'hasmore' plus the ID from the " + ATTRIB_ID.GetName() +
				" attribute. (i.e. if the id attribute uses 'myresources' as the ID, then the key would be 'hasmoremyresources'.",
				AttributeType.INT, "-1");
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create() {
		return new RelatedResourcesDataTag();
	}

	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag() {
		IDirModelRO dirModel = GetDirModel();
		IXPList resourceList = new XPArrayList();
		boolean bHasMore = false;

		// DCA - get the related resource indicated by the type attribute
		int type = GetTagAttributeAsInt(ATTRIB_TYPE);
		IASQueryResult qrRelatedResources = dirModel.GetRelatedResource(type);
		if (null != qrRelatedResources && qrRelatedResources.GetCount() > 0) {
			int limit = GetTagAttributeAsInt(ATTRIB_LIMIT);
			int nNumToDisplay = qrRelatedResources.GetCount();
			// DCA - check to see if we're limiting the number of related resources to display
			if (limit > -1 && nNumToDisplay > limit)
			{
				// limit is > 1, set the number to display to the limit
				nNumToDisplay     = limit;
				bHasMore		  = true;
			}
			for (int i=0; i<nNumToDisplay; i++) {
				String strName = (String) qrRelatedResources.GetFields(i, PT_PROPIDS.PT_PROPID_NAME);
				int nObjectID   = XPConvert.ToInteger(qrRelatedResources.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID));
				String strDesc = (String) qrRelatedResources.GetFields(i, PT_PROPIDS.PT_PROPID_DESCRIPTION);
				if (null == strDesc) {
					strDesc = EMPTY_STRING;
				}
				String strResourceURL = GetResourceURL(type, nObjectID, strName);
				
				HashtableDataObject htObject = new HashtableDataObject();
				htObject.SetData(NAME, strName);
				htObject.SetData(DESC, XPHttpUtility.HtmlEncode(strDesc));
				htObject.SetData(ID, XPConvert.ToString(nObjectID));
				htObject.SetData(URL, strResourceURL);
				resourceList.Add(htObject);
			}
		}
		
		// DCA - add to memory
		Scope tagScope = Scope.GetScope( GetTagAttributeAsString(ATTRIB_SCOPE));
		String strDataReference = GetTagAttributeAsString(ATTRIB_ID);
		GetState().SetSharedVariable(strDataReference, resourceList, tagScope, true);

		GetState().SetSharedVariable(HAS_MORE_PREFIX + strDataReference, XPConvert.ToString(bHasMore), tagScope, true);
		
		return ProcessTagBody();
	}

	private String GetResourceURL(int type, int nObjectID, String strName) {
		int nClassId = 0;
		switch (type)
		{
			case PT_INTRINSICS.PT_RELATED_RESOURCE_COMMUNITIES:
				nClassId = PT_CLASSIDS.PT_COMMUNITY_ID;					
				break;

			case PT_INTRINSICS.PT_RELATED_RESOURCE_FOLDERS:
				nClassId = PT_CLASSIDS.PT_CATALOGFOLDER_ID;		
				break;

			case PT_INTRINSICS.PT_RELATED_RESOURCE_GADGETS:
				nClassId = PT_CLASSIDS.PT_GADGET_ID;	
				break;

			case PT_INTRINSICS.PT_RELATED_RESOURCE_EXPERTS:
			case PT_INTRINSICS.PT_RELATED_RESOURCE_CONTENT_MANAGERS:
				nClassId = PT_CLASSIDS.PT_USER_ID;	
				break;
		}
		return PTOpenerLinks.GetOpenerURLOpenObjID(nClassId, nObjectID, strName, IPTOpenerConstants.OPENER_MODE_VIEW, (AActivitySpace) GetEnvironment());	
	}
}
