package com.bea.alui.taglib.kdpage;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portalpages.browsing.directory.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.server.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag only works with the Knowledge Directory activity space. It stores a list of subfolders of the current folder in memory.
 * Each sub folder data object will have the following variables: 'name' for the sub folder's name, 'id' for the sub folder's Id,
 * 'description' for the sub folder's description, and 'url' for the URL to access that folder. Each sub folder will have a list of 
 * its sub folders stored in memory and can be accessed by using the concatenation of 'subfoldersof' and the subfolder's id.
 * 
 * @author Dustin Ang
 */
public class SubFoldersDataTag extends AKDPageTag 
{	
	private static final String TAG_NAME = "subfoldersdata";
	private static final String SUBFOLDERS_OF_PREFIX = "subfoldersof";
	
	public static final String NAME = "name";
	public static final String ID = "id";
	public static final String URL = "url";
	public static final String DESC = "description";
	public static final String LEVEL = "level";
	public static final String SUBSUBFOLDERS_KEY = "subsubfolderskey";

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData(TAG_NAME, ATag.EncodeForTagDoc(KD_PAGE_ONLY_CONSTANT +
			"Stores a list of subfolders of the current folder in memory. " +
			"Each subfolder data object has the following variables: <br/>" + 
			"<ul>" + 
			"<li> <code>" + NAME +"</code> - The name of the subfolder.</li>" + 
			"<li> <code>" + ID + "</code> - The subfolder ID.</li>" + 
			"<li> <code>" + DESC + "</code> - The subfolder description.</li>" + 
			"<li> <code>" + URL + "</code> - The URL to access the folder.</li>" +
			"Each subfolder will have a list of its subfolders stored in memory and can be accessed by using the concatenation of '" + 
			SUBFOLDERS_OF_PREFIX + "' and the subfolder ID.</li>" +
			"</ul>")
		);
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create() {
		return new SubFoldersDataTag();
	}

	/*
	 * (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag() {
		HashtableDataObject htObject = null;
		IXPList subFoldersList = new XPArrayList();
		IDirModelRO dirModel = GetDirModel();

		// DCA - get sub folders
		if (dirModel.RetrieveSubFoldersThroughSearch()) {
			IPTSearchResponse srFolders = dirModel.GetFolderSearchResponse();
			for (int i=0; i<srFolders.GetTotalMatches(); i++) {
				htObject = GetFolderDataObjectFromSearch(i, srFolders, dirModel.GetCurrentFolderLevel());
				// DCA - if it's not null, it must be a sub folder
				// since we are the querying folder, we don't have to worry about checking for folders that are the same level as we are
				if (null != htObject) {
					subFoldersList.Add(htObject);
				}
			}
		} else {
			IASQueryResult qrFolders = dirModel.GetFolderQueryResult();
			for (int i=0; i<qrFolders.GetCount(); i++) {
				htObject = GetFolderDataObjectFromQuery(i, qrFolders, dirModel.GetCurrentFolderLevel());
				// DCA - if it's not null, it must be a sub folder
				// since we are the querying folder, we don't have to worry about checking for folders that are the same level as we are
				if (null != htObject) {
					subFoldersList.Add(htObject);
				}
			}
		}
		
		// DCA - add to memory
		Scope tagScope = Scope.GetScope( GetTagAttributeAsString(ATTRIB_SCOPE));
		String strDataReference = GetTagAttributeAsString(ATTRIB_ID);
		GetState().SetSharedVariable(strDataReference, subFoldersList, tagScope, true);
		
		return ProcessTagBody();
	}

	/**
	 * Gets the information of the folder in row: index from the query result creates the data object
	 * out of it. This method also creates the list of the folder's sub folders and stores it in tag 
	 * memory.
	 * 
	 * @param index			 		the index of the folder wanted
	 * @param qrFolders				the query result containing folders
	 * @param nParentFolderLevel	the folder's (the folder specified by index) parent folder level
	 * @return	HashtableDataObject
	 */
	private HashtableDataObject GetFolderDataObjectFromQuery(int index, IASQueryResult qrFolders, int nParentFolderLevel) {
		int nLevel	   = XPConvert.ToInteger(qrFolders.GetFields(index, PT_PROPIDS.PT_PROPID_FOLDER_LEVEL));

		if ((nLevel - 1) == nParentFolderLevel) 
		{
			// DCA - we are a sub folder, get properties
			String strName     = (String) qrFolders.GetFields(index, PT_PROPIDS.PT_PROPID_NAME);
			int nFolderID	   = XPConvert.ToInteger(qrFolders.GetFields(index, PT_PROPIDS.PT_PROPID_OBJECTID));
			String strDesc     = (String) qrFolders.GetFields(index, PT_PROPIDS.PT_PROPID_DESCRIPTION);
			
			// DCA - get this folders' sub folders if there are any
			HashtableDataObject htObject = null;
			IXPList subFoldersList = new XPArrayList();
			// DCA - start with folder next to us, we know that the folders before us aren't our sub folders
			for (int i=index+1; i<qrFolders.GetCount(); i++) {
				htObject = GetFolderDataObjectFromQuery(i, qrFolders, nLevel);
				//  DCA - if it's not null and it is 1 level below us, it must be a sub folder
				if (null != htObject) {
					if ((XPConvert.ToInteger(htObject.GetData(LEVEL))-1) == nLevel) {
						subFoldersList.Add(htObject);
					} else if (XPConvert.ToInteger(htObject.GetData(LEVEL)) == nLevel) {
						// DCA - this folder must be the same level as we are which means we don't have any more subfolders and we can break
						break;
					}
				}
			}
			// DCA - add to memory as subfoldersof + nFolderID
			Scope tagScope = Scope.GetScope( GetTagAttributeAsString(ATTRIB_SCOPE));
			String strDataReference = SUBFOLDERS_OF_PREFIX + nFolderID;
			GetState().SetSharedVariable(strDataReference, subFoldersList, tagScope, true);
			
			return PopulateFolderDataObject(strName, nFolderID, nLevel, strDesc, strDataReference);
		} 
		else if (nLevel == nParentFolderLevel) 
		{
			// DCA - if we are on the same level as the parent folder, return only our level 
			// to let them know we are not a sub folder
			HashtableDataObject htObject = new HashtableDataObject();
			htObject.SetData(LEVEL, XPConvert.ToString(nLevel));
			
			return htObject;
		} 
		else 
		{
			// DCA - we are a sub sub folder, return null
			return null;
		}
	}

	/**
	 * Gets the information of the folder in row: index from the search result creates the data object
	 * out of it. This method also creates the list of the folder's sub folders and stores it in tag 
	 * memory.
	 * 
	 * @param index			 		the index of the folder wanted
	 * @param qrFolders				the query result containing folders
	 * @param nParentFolderLevel	the folder's (the folder specified by index) parent folder level
	 * @return	HashtableDataObject
	 */
	private HashtableDataObject GetFolderDataObjectFromSearch(int index, IPTSearchResponse srFolders, int nParentFolderLevel) {
		int nLevel	   = srFolders.GetFieldsAsInt(index, PT_INTRINSICS.PT_PROPERTY_FOLDERLEVEL);

		if ((nLevel - 1) == nParentFolderLevel) 
		{
			//  DCA - we are a sub folder, get properties
			String strName     = srFolders.GetFieldsAsString(index, PT_INTRINSICS.PT_PROPERTY_OBJECTNAME);
			int nFolderID	   = srFolders.GetFieldsAsInt(index, PT_INTRINSICS.PT_PROPERTY_OBJECTID);
			String strDesc     = srFolders.GetFieldsAsString(index, PT_INTRINSICS.PT_PROPERTY_OBJECTSUMMARY);
			
			// DCA - get this folders' sub folders if there are any
			HashtableDataObject htObject = null;
			IXPList subFoldersList = new XPArrayList();
			// DCA - start with folder next to us, we know that the folders before us aren't our sub folders
			for (int i=index+1; i<srFolders.GetTotalMatches(); i++) {
				htObject = GetFolderDataObjectFromSearch(i, srFolders, nLevel);
				//  DCA - if it's not null and it is 1 level below us, it must be a sub folder
				if (null != htObject) {
					if ((XPConvert.ToInteger(htObject.GetData(LEVEL))-1) == nLevel) {
						subFoldersList.Add(htObject);
					} else if (XPConvert.ToInteger(htObject.GetData(LEVEL)) == nLevel) {
						// DCA - this folder must be the same level as we are which means we don't have any more subfolders and we can break
						break;
					}
				}
			}
			// DCA - add to memory as subfoldersof + nFolderID
			Scope tagScope = Scope.GetScope( GetTagAttributeAsString(ATTRIB_SCOPE));
			String strDataReference = SUBFOLDERS_OF_PREFIX + nFolderID;
			GetState().SetSharedVariable(strDataReference, subFoldersList, tagScope, true);
	
			return PopulateFolderDataObject(strName, nFolderID, nLevel, strDesc, strDataReference);
		} 
		else if (nLevel == nParentFolderLevel) 
		{
			// DCA - if we are on the same level as the parent folder, return only our level 
			// to let them know we are not a sub folder
			HashtableDataObject htObject = new HashtableDataObject();
			htObject.SetData(LEVEL, XPConvert.ToString(nLevel));
			
			return htObject;
		} 
		else 
		{
			// DCA - we are a sub sub folder, return null
			return null;
		}
	}

	/**
	 * Helper method that populates the data object using the parameters specified.
	 * 
	 * @param strName
	 * @param nFolderID
	 * @param nLevel
	 * @param strDesc
	 * @param strSubSubFoldersKey
	 * @return	HashtableDataObject
	 */
	private HashtableDataObject PopulateFolderDataObject(String strName, int nFolderID, int nLevel, String strDesc, String strSubSubFoldersKey) {
		HashtableDataObject htObject = new HashtableDataObject();
		if (null == strDesc || strDesc.equals(DirConstants.DIR_FOLD_DESC_NONE)) {
			strDesc = EMPTY_STRING;
		}
		IPTURL subFolderURL = GetFolderURL(nFolderID, strName);
		
		htObject.SetData(NAME, strName);
		htObject.SetData(DESC, strDesc);

		htObject.SetData(ID, XPConvert.ToString(nFolderID));
		htObject.SetData(URL, subFolderURL.GetFullURL());
		htObject.SetData(LEVEL, XPConvert.ToString(nLevel));
		htObject.SetData(SUBSUBFOLDERS_KEY, strSubSubFoldersKey);
		
		return htObject;
	}
	
	/**
	 * Helper method that returns an IPTURL object that points to the folder with nFolderID
	 * 
	 * @param nFolderID
	 * @param strName
	 * @return IPTURL
	 */
	private IPTURL GetFolderURL(int nFolderID, String strName) {
		IDirModelRO dirModel = GetDirModel();
		
		IPTURL subFolderURL = TaskAPIUIDirectory.CreateCommonOpenerDirectoryFolderURL((AActivitySpace) GetEnvironment(), 
					strName, dirModel.GetDirectoryMode(), nFolderID, strName);
		return subFolderURL;
	}
}
