package com.bea.alui.taglib.kdpage;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portalpages.browsing.directory.*;
import com.plumtree.portalpages.browsing.directory.documentsubmitsimple.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.common.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.server.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.taskapi.portalui.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag generates data describing the data sources
 * that can be used to submit data to the Knowledge Directory.
 * 
 * It can be used on any page or portlet.
 */
public class SubmitSourceDataTag extends ATag
{
	private static final String TAG_NAME = "submitsourcedata";

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute ATTRIB_ID;
	public static final OptionalTagAttribute ATTRIB_SCOPE;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData(TAG_NAME, ATag.EncodeForTagDoc(
			"Stores a list of data sources in memory that can be used to " +
			"submit documents to the Knowledge Directory. " +
			"Each data source data object has the following variables: <br/>" + 
			"<ul>" + 
			"<li> <code>" + SubmitDataSourceConstants.NAME +"</code> - The name of the data source.</li>" + 
			"<li> <code>" + SubmitDataSourceConstants.ID + "</code> - The data source ID.</li>" +
			"<li> <code>" + SubmitDataSourceConstants.TYPE + "</code> - The type of the data source. This can be either " +
			"\"" + SubmitDataSourceConstants.TYPE_WEB + "\" or \"" + SubmitDataSourceConstants.TYPE_FILE + "\".</li>" + 
			"<li> <code>" + SubmitDataSourceConstants.URL + "</code> - The remote file upload URL. " +
			"This is only present for remote file upload data sources of type \"" + SubmitDataSourceConstants.TYPE_FILE + "\".</li>" +
			"</ul>")
		);

		ATTRIB_ID = new RequiredTagAttribute(CommonTagAttributeConstants.ATTRIB_ID_NAME,
			CommonTagAttributeConstants.ATTRIB_ID_DESC,	
			AttributeType.STRING);

		ATTRIB_SCOPE = new OptionalTagAttribute(CommonTagAttributeConstants.ATTRIB_SCOPE_NAME,
			CommonTagAttributeConstants.ATTRIB_SCOPE_DESC,
			AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
	}

	@Override
	public HTMLElement DisplayTag()
	{
		AActivitySpace owner = TaskAPIUICommon.GetEnvTypeObject(GetEnvironment());
		IXPList dataSourcesList = TaskAPIUIDirectoryService.GetInstance().GetSubmitDataSources(owner);

		dataSourcesList = TaskAPIUIDirectoryService.GetInstance().AddUploadData(owner, dataSourcesList);

		for (int i = 0; i < dataSourcesList.GetSize(); i++)
		{
			HashtableDataObject source = (HashtableDataObject) dataSourcesList.GetElement(i);
			if (XPStringUtility.Compare(SubmitDataSourceConstants.TYPE_FILE, source.GetData(SubmitDataSourceConstants.TYPE)) == 0)
			{
				// This will take the web service URL, gateway it, and add the return URL.
				ModifyWebServiceURL(source);
			}
		}
		
		// add to memory
		Scope tagScope = Scope.GetScope( GetTagAttributeAsString(ATTRIB_SCOPE));
		String strDataReference = GetTagAttributeAsString(ATTRIB_ID);
		GetState().SetSharedVariable(strDataReference, dataSourcesList, tagScope, true);
		
		return ProcessTagBody();
	}

	/**
	 * This method takes the web service URL, gateways it, and adds the return URL.
	 * 
	 * @param _source The current data source item to modify
	 */
	protected void ModifyWebServiceURL(HashtableDataObject _source)
	{
		AActivitySpace owner = TaskAPIUICommon.GetEnvTypeObject(GetEnvironment());

		String strWebServiceURL = _source.GetData(SubmitDataSourceConstants.URL);

		if ((strWebServiceURL == null) || (strWebServiceURL.length() == 0))
		{
			// This data source does not have valid web service information.  Skip it.
			return;
		}
		
		String strWebServiceID = _source.GetData(SubmitDataSourceConstants.WEB_SERVICE_ID);
		int nWebServiceID = XPConvert.ToInteger(strWebServiceID);

		String gwUrl =
			GatewayHelpers.ConstructPrefPageLink(
				owner,
				nWebServiceID,
				0,
				0,
				0,
				PT_CLASSIDS.PT_WEBSERVICE_ID);

		// Remote file upload services require a URL to the portal to be included in
		// the query string.  This URL is then used in a 302 redirect to cause the
		// portal to create the card corresponding to the uploaded file.
		ASURL myCardSubmitURL = new ASURL(owner);
		myCardSubmitURL.SetLinkGetSpaceIfCached(DirAS.STR_MVC_CLASS_NAME, owner);
		myCardSubmitURL.SetDisplayPage(
			DirCardSubmitSimpleDP.STR_MVC_CLASS_NAME);
		myCardSubmitURL.SetControl(
			DirRepostControl.STR_MVC_CLASS_NAME);

		String strUploadUrl =
			gwUrl
				+ GatewayHelpers.CreateGatewayFriendlyURL(
					strWebServiceURL
					+ "?"
					+ DirCardSubmitSimpleView.HTMLINPUT_ReturnURL
					+ "="
					+ myCardSubmitURL.GetURLAsString());

		_source.SetData(SubmitDataSourceConstants.URL, strUploadUrl);
	}

	@Override
	public ATag Create() {
		return new SubmitSourceDataTag();
	}
}