package com.bea.alui.taglib.portletpage;


import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.myportal.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This data tag gets all portlets in a specific portlet region 
 * 
 * @author rozhang
 */
public class PortletRegionDataTag extends ATag 
{
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
			"com.bea.alui.taglib.portletpage.PortletRegionDataTag");

	/** The data parameters **/
	public static final String KEY_PORTLETNAME 				= "name";
	public static final String KEY_PORTLETOBJECTID 			= "objid";
	public static final String KEY_PORTLETINDEX 			= "index";
	public static final String KEY_PORTLETIDSTRING			= "portletidstring";
	public static final String KEY_PORTLETADMINPREFURL		= "adminprefurl";
	public static final String KEY_PORTLETCOMMPREFURL		= "commprefurl";
	public static final String KEY_PORTLETUSERPREFURL		= "userprefurl";
	public static final String KEY_PORTLETHELPURL			= "helpurl";
	public static final String KEY_PORTLETCOLLAPSEEXPANDURL	= "collapseexpandurl";
	public static final String KEY_PORTLETREMOVEURL			= "removeurl";
	public static final String KEY_PORTLETHASTITLEBAR		= "hastitlebar";
	public static final String KEY_PORTLETISCOLLAPSED		= "iscollapsed";
	public static final String KEY_PORTLETISMANDATORY		= "ismandatory";
	
	/**
	 *  Mapping between regions and the classic portlet layout column values where array
	 *  index represent the region index and the value at that index contains the column value 
	 *  in the classic portlet layout
	 *  
	 *  Not really needed for 6.5 since the values are identical, but might be needed in the future
	 */
	public static final int[] REGION_TOCOLUMN_MAP = new int[] {0, 1, 2};
	
	public static final String DATATAG_EXAMPLE = ATag.EncodeForTagDoc(
			"<pt:portletpage.portletregiondata pt:key=\"var1\" pt:region=\"1\"/><br/>"
			);
	
	public static final String DATATAG_PROPERTIES = ATag.EncodeForTagDoc(
		"The following properties can be accessed from this tag:<br/>" +
		"<ul>" +
		"<li> <code>" + KEY_PORTLETNAME + "</code> - The name of the portlet.</li>"+
		"<li> <code>" + KEY_PORTLETOBJECTID + "</code> - The object ID of the portlet.</li>"+
		"<li> <code>" + KEY_PORTLETINDEX + " </code> - The index of the portlet in the portlet array.</li>"+
		"<li> <code>" + KEY_PORTLETIDSTRING + "</code> - The string identifier of the portlet.</li>"+
		"<li> <code>" + KEY_PORTLETADMINPREFURL + "</code> - The administrative preference page link for the portlet if one exists.</li>"+
		"<li> <code>" + KEY_PORTLETCOMMPREFURL + "</code> - The community preference page link for the portlet if one exists.</li>"+
		"<li> <code>" + KEY_PORTLETUSERPREFURL + "</code> - The user preference page link for the portlet if one exists.</li>"+
		"<li> <code>" + KEY_PORTLETHELPURL + "</code> - The help link for the portlet if one exists.</li>"+
		"<li> <code>" + KEY_PORTLETCOLLAPSEEXPANDURL + "</code> - The link for collapsing or expanding the portlet depending "+
		"on the collapse state of the portlet.</li>" +
		"<li> <code>" + KEY_PORTLETREMOVEURL + "</code> - The link for removing the portlet.</li>" +
		"<li> <code>" + KEY_PORTLETHASTITLEBAR + "</code> - True if the portlet title bar is not suppressed, false otherwise.</li>" +
		"<li> <code>" + KEY_PORTLETISCOLLAPSED + "</code> - True if the portlet is collapsed, false if it is expanded.</li>" +
		"</ul>"
	);

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute KEY;
	public static final RequiredTagAttribute REGIONINDEX;
	public static final OptionalTagAttribute SCOPE;
	
	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("portletregiondata",
			"Creates a tag data variable with a list of portlets. "+
			"Other display tags, such as portletpage.portletdisplay use this data to display the portlet content. "+
			"This tag works on MyPages and Community pages. "+
			DATATAG_PROPERTIES+DATATAG_EXAMPLE+PortletDisplayTag.DISPLAYTAG_EXAMPLE);

		KEY = new RequiredTagAttribute("key",
			"The ID of the tag data variable where the portlet information is set.");

		REGIONINDEX = new RequiredTagAttribute("region",
		"The portlet region index to use with this data tag. Valid region values are 1, 2 or 3.", AttributeType.INT);
		
		SCOPE = new OptionalTagAttribute("scope",
			"The scope used to store the tag data variable. "+
			"Only tag and portlet request scopes are valid for this tag as it relies on the Community Page Model. "+
			"See the Logic tag library for further information on scope.",
			AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
	}

	/**
	 * 
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag() 
	{
		/** Environment validation begin **/

		
		// first get the model which should exist if we are on a community page
		AActivitySpace space = (AActivitySpace)GetEnvironment();
		MyPortalModel model = null;
		
		model = (MyPortalModel)space.GetModel(MyPortalModel.STR_MVC_CLASS_NAME);
		if ( model == null)
			throw new XPException("Could not find MyPortalContentModel in the current ActivitySpace. This tag only works on Community Pages.");
		
		/** Attribute validation begin **/

		int regionIndex = GetTagAttributeAsInt(REGIONINDEX);
		if ( regionIndex < 1 || regionIndex > 3 ) {
			throw new XPIllegalArgumentException("Region index attribute containedinvalid value of "+regionIndex+". Valid regions range from 1 to 3");
		}
		// subtract one for the value since the mapping array is 0 based
		regionIndex--;
		
		String variableKey = null;
		variableKey = GetTagAttributeAsString(KEY);
		log.Debug("Tag portletregiondata getting portlet data for region: {0}", regionIndex);
		// iterate over all the portlets and get the ones in the region we are looking for
		XPArrayList portlets = new XPArrayList();
		for ( int i = 0; i < model.GetPortletCount(); i++ ) {
			int currCol = model.GetPortletColumnFromIndex(i);

			// Aren't you just being fancy here?  The region index is 1 based, but the mapping array
			// is 0 based, and so is the column value.  Can't you just say currCol == regionIndex
			// (assuming you don't do regionIndex-- above)?
			if ( currCol == REGION_TOCOLUMN_MAP[regionIndex] ) {
				// for each portlet, create an IDataObject so that the values are accessible from tag expression language
				HashtableDataObject portletDO = new HashtableDataObject();
				// CR DLH: Shouldn't you just use XPConvert.ToString so that we don't create
				// an extra String object (although I'm sure the jvm has a shared empty string somewhere)?
				// ROB: JVM has global available instances for all one character strings and this makes the code
				// more legible
				portletDO.SetData(KEY_PORTLETINDEX, i+"");
				portletDO.SetData(KEY_PORTLETOBJECTID, model.GetPortletIDFromIndex(i)+"");
				portletDO.SetData(KEY_PORTLETNAME, model.GetPortletNameFromIndex(i));
				portletDO.SetData(KEY_PORTLETIDSTRING, model.GetIDStringForPortlet(i));
				portletDO.SetData(KEY_PORTLETADMINPREFURL, model.GetPortletAdminPrefsURL(i));
				portletDO.SetData(KEY_PORTLETCOMMPREFURL, model.GetPortletCommunityPrefsURL(i));
				portletDO.SetData(KEY_PORTLETUSERPREFURL, model.GetPortletUserPrefsURL(i));
				portletDO.SetData(KEY_PORTLETHELPURL, model.GetPortletHelpURL(i));
				portletDO.SetData(KEY_PORTLETISMANDATORY, model.GetPortletIsMandatory(i)+"");
				
				if (model.GetPortletCollapsed(i))
				{
					portletDO.SetData(KEY_PORTLETCOLLAPSEEXPANDURL, model.GetPortletCollapseExpandURL(i, CollapsePortletControl.STR_EXPAND_CONTROL));
				}
				else if (!model.GetPortletCollapsed(i))
				{
					portletDO.SetData(KEY_PORTLETCOLLAPSEEXPANDURL, model.GetPortletCollapseExpandURL(i, CollapsePortletControl.STR_COLLAPSE_CONTROL));
				}				
				portletDO.SetData(KEY_PORTLETISCOLLAPSED, XPConvert.ToString(model.GetPortletCollapsed(i)));
				portletDO.SetData(KEY_PORTLETREMOVEURL, model.GetPortletRemoveURL(i));
				portletDO.SetData(KEY_PORTLETHASTITLEBAR, XPConvert.ToString(model.GetPortletHasTitleBar(i)));
				
				log.Debug("- Adding portlet data to collection index: {0} "+
						"portlet id: {1} portlet name: '{2}' mandatory: {3} collapsed: {4}", 
						i, model.GetPortletIDFromIndex(i)+"", model.GetPortletNameFromIndex(i)+"",
						model.GetPortletIsMandatory(i)+"", model.GetPortletCollapsed(i)+"");
				
				portlets.Add(portletDO);
			}
		}
		Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));
		log.Debug("- Setting {0} portlets with variable name: '{1}' in scope: {2}", portlets.GetSize(), variableKey, scope);
		GetState().SetSharedVariable(variableKey, portlets, scope, false);
		return ProcessTagBody();
	}
	
	/**
	 * 
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create() {
		return new PortletRegionDataTag();
	}
}