package com.bea.alui.taglib.portletpageeditor;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portalpages.browsing.objectselection.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.common.*;
import com.plumtree.uiinfrastructure.helpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag generates the form and hidden inputs necessary to search for portlets.
 * It does not generate the text input or search button
 */
public class PortletSearchFormTag extends ATag
{
	private static final String TAG_NAME = "portletsearchform";

	public static final String QUERY_INPUT = "in_tx_query";

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final OptionalTagAttribute FORM_NAME;
	public static final OptionalTagAttribute FORM_ID;
	public static final OptionalTagAttribute VALIDATE_FUNCTION;
	public static final OptionalTagAttribute SUBMIT_FUNCTION;
	public static final OptionalTagAttribute KEY_DOWN_FUNCTION;
	public static final RequiredTagAttribute FLYOUT_ID;
	public static final RequiredTagAttribute DEFAULT_TEXT;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData(TAG_NAME, "Generates the form and " +
			"hidden inputs necessary to search for portlets. It does not " +
			"generate the text input or search button.  The text input must " +
			"be named " + QUERY_INPUT + ". " + PortletPageEditorTagLibraryData.FLYOUT_ONLY);
		
		FORM_NAME = new OptionalTagAttribute("name",
			"The name to use for the search form.",
			AttributeType.STRING, "portletSearch");

		FORM_ID = new OptionalTagAttribute("id",
			"The ID to use for the search form.",
			AttributeType.STRING, "portletSearch");

		SUBMIT_FUNCTION = new OptionalTagAttribute("submit",
			"The name to use for the javascript submit function. This function " +
			"will need to be called when the user searches for portlets.",
			AttributeType.STRING, "submitPortletSearch");

		VALIDATE_FUNCTION = new OptionalTagAttribute("validate",
			"The name to use for the javascript search validation function used in the form.",
			AttributeType.STRING, "checkPortletSearch");

		KEY_DOWN_FUNCTION = new OptionalTagAttribute("keydown",
			"The name to use for the javascript key down event handler function. " +
			"This function should be added to the 'onkeydown' attribute of the search " +
			"text input box.  The function takes the key down event as an argument (i.e. " +
			"onkeydown='handleSearchEvent(event);').",
			AttributeType.STRING, "handleSearchEvent");

		FLYOUT_ID = PortletPageEditorTagLibraryData.ATTRIB_FLYOUTID;

		DEFAULT_TEXT = new RequiredTagAttribute("defaulttext",
			"The default text displayed in the search box.  This will be used " +
			"by the search validation javascript to convert the default text " +
			"to a wildcard query (i.e. *) if the user searches for it.",
			AttributeType.STRING);
	}

	@Override
	public HTMLElement DisplayTag()
	{
        AActivitySpace asOwner = (AActivitySpace) GetEnvironment();

		IObjectSelectionModelRO objectModel = (IObjectSelectionModelRO)
			asOwner.GetModel(ObjectSelectionModel.STR_MVC_CLASS_NAME);

		String formName = GetTagAttributeAsString(FORM_NAME);
		String formId = GetTagAttributeAsString(FORM_ID);

		XPStringBuilder strb = new XPStringBuilder();
		
		// Add the <form> tag.
		strb.Append("<form name=\"").Append(formName);
		strb.Append("\" id=\"").Append(formId);
		strb.Append("\" onsubmit=\"return ").Append(GetTagAttributeAsString(VALIDATE_FUNCTION)).Append("();");
		strb.Append("\" action=\"").Append(getFormAction(asOwner));
		strb.Append("\" method=\"POST\">");

		// Add the required hidden input tags.
    	addHiddenInputHTML(strb, "in_hi_space", asOwner.GetName());
    	addHiddenInputHTML(strb, "in_hi_spaceID", asOwner.GetSpaceID());

    	// If we are in browse mode and switching to search mode, we need to
    	// switch the object selection model to display search.  This is normally
    	// done by a full form repost, but since we don't implement the object selection
    	// form correctly, we need to have a control do it for us.  We use the pass-thru
    	// control to first switch to search, and second start the search.
    	addHiddenInputHTML(strb, "in_hi_control", PassthruControl.STR_MVC_CLASS_NAME);
    	// Switch to search
    	addHiddenInputHTML(strb, PassthruControl.QS_PASSTHRU_CONTROL, ObjectSelectionActionControl.STR_MVC_CLASS_NAME);
    	addHiddenInputHTML(strb, ObjectSelectionActionControl.SEARCH_MODE, XPConvert.ToString(true));
    	// Brand new search
    	addHiddenInputHTML(strb, PassthruControl.QS_MAIN_CONTROL, StartSearchControl.STR_MVC_CLASS_NAME);

		SearchSettingCollection scc = objectModel.GetSearchSettingCollection();

    	// Reuse the search settings from the object model
		IXPEnumerator settings = scc.GetSettingIDs();
		while (settings.MoveNext())
		{
			int nKey = (Integer) settings.GetCurrent();
			String strKey = SearchSettingCollection.getStringFromSettingId(nKey);
			Object oValue = scc.GetSettingValue(nKey);

			// Search values are either int[]s or things that can be converted
			// to strings
			if (oValue instanceof int[]) {
				int[] nValues = (int[]) oValue;

				for (int i = 0; i < nValues.length; i++)
				{
					addHiddenInputHTML(strb, "in_hi_req_" + strKey, nValues[i]);
				}
			} else
			{
				if (nKey == PT_SEARCH_SETTING.PT_SEARCHSETTING_GADGET_FIELDS)
				{
					// HACK: The gadget field setting in the search settings collection
					// is a boolean (true), which is required when starting the initial
					// search from inside the model, but when that is put in a form
					// and posted, it doesn't work.  The docs say a non-zero integer
					// value means true, so using integers instead.
					if (oValue instanceof Boolean)
					{
						Boolean bValue = (Boolean) oValue;
						oValue = (bValue ? "1" : "0");
					}
				}

				addHiddenInputHTML(strb, "in_hi_req_" + strKey, XPConvert.ToString(oValue));
			}
		}

    	// Add the Javascript
    	addFormJavascript(strb, formName, QUERY_INPUT, asOwner);

		HTMLElementCollection result = new HTMLElementCollection();
		result.AddInnerHTMLString(strb.ToString());
		result.AddInnerHTMLElement(ProcessTagBody());
		result.AddInnerHTMLString("</form>");

		return result;
	}

	private void addFormJavascript(XPStringBuilder strb, String formName, String inputTextName, AActivitySpace asOwner)
	{
		// **********************************************************************************************
		// This javascript somewhat replicates the javascript in:
		// com.bea.alui.taglib.searchpage.FollowupFormTag.addFormJavascript()
		//
		// That is the method responsible for displaying the javascript in the normal search form tag.
		// **********************************************************************************************

		strb.Append("<script type=\"text/javaScript\">\n");
		// DCA - we need to define the javascript functions this way so that calling eval() on it works
		strb.Append(GetTagAttributeAsString(VALIDATE_FUNCTION)).Append(" = function").Append("() {\n");
		strb.Append("  var searchForm = document.").Append(formName).Append(";\n");
		strb.Append("  for (var i = 0; i < searchForm.elements.length; i++) {\n");
		strb.Append("    if (searchForm.elements[i].name == \"").Append(inputTextName).Append("\") {\n");
		// If they try to search for blank or the default text,
		// just replace it with a valid wildcard query (*)
		strb.Append("      if ((searchForm.elements[i].value == \"\") || ")
		.Append("(searchForm.elements[i].value == \"").Append(GetTagAttributeAsString(DEFAULT_TEXT)).Append("\")) {\n");
		strb.Append("        searchForm.elements[i].value = \"*\";\n");
		strb.Append("        return true;\n");
		strb.Append("      }\n");
		strb.Append("      return true;\n");
		strb.Append("    }\n");
		strb.Append("  }\n");
		strb.Append("  return false;\n");
		strb.Append("}\n");

		strb.Append(GetTagAttributeAsString(SUBMIT_FUNCTION)).Append(" = function").Append("() {\n");
		strb.Append("  if (").Append(GetTagAttributeAsString(VALIDATE_FUNCTION)).Append("()) {\n");
		strb.Append("    var searchForm = document.").Append(formName).Append(";\n");
		String strFlyoutId = GetTagAttributeAsString(FLYOUT_ID);
		if (null == strFlyoutId || strFlyoutId.equals("")) {
			strb.Append("    searchForm.submit();\n");
		} else {
			strb.Append("    ").Append(FlyoutJSTag.FLYOUT_OBJECT).Append(strFlyoutId).Append(".cursorWait();\n");
			strb.Append("    var request = new PTHTTPPOSTRequest(searchForm.action, searchForm, ").Append(FlyoutJSTag.FLYOUT_OBJECT).Append(strFlyoutId).Append(".").Append(FlyoutJSTag.ADD_CONTENT).Append(");\n");
			strb.Append("    request.invoke();\n");
		}
		strb.Append("  } else {\n");
		strb.Append("    alert('");
		strb.Append(asOwner.GetString(468, "ptmsgs_portalinfrastructure"));
		strb.Append("');\n");
		strb.Append("  }\n");
		strb.Append("  return false;\n");
		strb.Append("}\n");
						
		strb.Append(GetTagAttributeAsString(KEY_DOWN_FUNCTION)).Append(" = function").Append("(evt) {\n");
		strb.Append("  evt = (evt) ? evt : ((event) ? event : null);\n");
		strb.Append("  if (evt.keyCode == 13){\n");
		strb.Append("    return ").Append(GetTagAttributeAsString(SUBMIT_FUNCTION)).Append("();\n");
		strb.Append("  }\n");
		strb.Append("  return true;\n");
		strb.Append("}\n");
		strb.Append("</script>\n");
	}

	private void addHiddenInputHTML(XPStringBuilder strb,
		String name, int value)
	{
		addHiddenInputHTML(strb, name, XPConvert.ToString(value));
	}

	private void addHiddenInputHTML(XPStringBuilder strb,
		String name, String value)
	{
		strb.Append("<input type=\"hidden\" ");
		strb.Append("name=\"").Append(name).Append("\" ");
		strb.Append("value=\"").Append(value).Append("\"");
		strb.Append("/>");
	}

	private String getFormAction(AActivitySpace asOwner)
	{
		String action = ConfigService.getInstance().GetApplicationBaseURLToAS(asOwner.GetCurrentNormalizedBaseURL(), 
			asOwner.GetName(), asOwner.IsSubSpace());
		return action;
	}

	@Override
	public ATag Create()
	{
		return new PortletSearchFormTag();
	}
}