package com.bea.alui.taglib.portletpageeditor;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.portalpages.common.constants.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.common.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.uitasks.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag returns a list of order by property entries. Custom order by properties are specified
 * with the {@link PortletSortPropertyEntry} tag 
 *  
 * @author rozhang
 */
public class PortletSortPropertyDataTag extends ATag 
{
	// This needs to be out side the static block so we can reference it
	// properly in the PortletDataTag.
	public static final String TAG_NAME = "sortpropertiesdata";

	public static final String DATAVARNAME_PROPENTRIES = "sortPropEntries";
	
	public static final String DATA_OBJECTNAME = "title";
	public static final String DATA_OBJECTID = "id";
	public static final String DATA_DEFAULTPROP = "default";
	public static final String DATA_CUSTOMPROP = "custom";
	public static final String DATA_DEFAULTORDERBY = "defaultsort";
	
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute ATTRIB_ID;
	public static final OptionalTagAttribute ATTRIB_SCOPE;
	public static final OptionalTagAttribute ATTRIB_INCLUDEDEFAULTPROPS;
	public static final OptionalTagAttribute ATTRIB_DEFAULTPROPSFIRST;
	public static final OptionalTagAttribute ATTRIB_DEFAULTSORTPROPID;

	
	public static final String EXAMPLE = ATag.EncodeForTagDoc(
						"<pt:portletpageeditor.sortpropertiesdata pt:id=\"sortprops\" ><br/>"+
						"  <pt:portletpageeditor.sortpropertyentry pt:propid=\"200\" pt:proptitle=\"Custom Sort Ascending\"><br/>"+
						"  <pt:portletpageeditor.sortpropertyentry pt:propid=\"-200\" pt:proptitle=\"Custom Sort Descending\"><br/>"+
						"</pt:portletpageeditor.sortpropertiesdata><br/>"
				);

	public static final String INIT_WARNING = "NOTE: This tag and the " +
		PortletDataTag.TAG.GetName() + " tag must be displayed before " +
		"the other portletpageeditor tags so the page can be initialized properly.";

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData(TAG_NAME, 
				"Sets a collection of properties used to specify sort order. "+
				"Default properties include object name and last modified date in ascending and descending order. "+
				"Additional order by properties are defined by adding sortpropertyentry tags in the body. " +
				INIT_WARNING, EXAMPLE
			);

		ATTRIB_ID = new RequiredTagAttribute("id",
			CommonTagAttributeConstants.ATTRIB_ID_DESC,	
			AttributeType.STRING);

		ATTRIB_SCOPE = new OptionalTagAttribute("scope",
			CommonTagAttributeConstants.ATTRIB_SCOPE_DESC,
			AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
			
		ATTRIB_INCLUDEDEFAULTPROPS = new OptionalTagAttribute("usedefprop",
			"Defined whether or not to include the default sort properties in the list. "+
			"Only set the value to false if additional custom properties havs been specified. "+
			"True by default.",
			AttributeType.BOOLEAN,
			"true"
			);
		
		ATTRIB_DEFAULTPROPSFIRST = new OptionalTagAttribute("defpropfirst",
			"Defines whether or not to display the default sort properties before or after custom properties. "+
			"This attribute is not used if no custom properties has been specified or if "+
			"default properties are not included. True by default.",
			AttributeType.BOOLEAN,
			"true"
			);
		
		ATTRIB_DEFAULTSORTPROPID = new OptionalTagAttribute("defaultsort",
			"Specifies which custom properties values (extended object data) to retrieve with each associated portlet. " +
			"The attribute takes a comma separated list of property object IDs.",
			AttributeType.INT, 
			-PT_INTRINSICS.PT_PROPERTY_OBJECTLASTMODIFIED+"" 
			);
	}
	
	@Override
	public HTMLElement DisplayTag() 
	{
		AActivitySpace space = (AActivitySpace) GetEnvironment();
		
		
		// the list of entries to set
		XPArrayList dataList = new XPArrayList();
		
		Scope tagScope = Scope.GetScope( GetTagAttributeAsString(ATTRIB_SCOPE) );
		String strDataReference = GetTagAttributeAsString(ATTRIB_ID);
		GetState().SetSharedVariable(strDataReference, dataList, tagScope, true);
		
		XPArrayList propEntries = new XPArrayList();
		
		// set the list of sort property entries for the potential child tags to set
		GetState().SetVariable(DATAVARNAME_PROPENTRIES, propEntries, Scope.TAG);
		
		// process any child tags that set prop entries
		ProcessTagBody();
		
		boolean defPropsFirst = GetTagAttributeAsBoolean(ATTRIB_DEFAULTPROPSFIRST);
		boolean includeDefProps = GetTagAttributeAsBoolean(ATTRIB_INCLUDEDEFAULTPROPS);
		
		if ( defPropsFirst && includeDefProps ) {
			dataList.AddRange(GetDefaultSortProperties(space));
		}
		int defPropId = GetTagAttributeAsInt(ATTRIB_DEFAULTSORTPROPID);
		// add entry custom sort property entries to data list 
		for ( int i = 0; i < propEntries.GetSize(); i++ ) 
		{
			HashtableDataObject entry = (HashtableDataObject) propEntries.GetElement(i);
			entry.SetData(DATA_CUSTOMPROP, "true");
			if ( defPropId == XPConvert.ToInteger( entry.GetData(DATA_OBJECTID)) ) {
				entry.SetData(DATA_DEFAULTORDERBY, "true");
			}
			
			dataList.Add(entry);
		}
		
		if ( !defPropsFirst && includeDefProps ) {
			dataList.AddRange(GetDefaultSortProperties(space));
		}
		
		return null;
	}
	
	/**
	 * Helper method to create the default sort properties
	 * 
	 * @return
	 */
	public XPArrayList GetDefaultSortProperties(IEnvironment env)
	{
		AActivitySpace space = (AActivitySpace) env;
		
		XPArrayList list = new XPArrayList(); 
		
		int defPropId = GetTagAttributeAsInt(ATTRIB_DEFAULTSORTPROPID);
		
		String name = space.GetString(1927, ResourceConstants.PORTALBROWSINGMSGS);
		String modified = space.GetString(1928, ResourceConstants.PORTALBROWSINGMSGS);
		String asc = space.GetString(1929, ResourceConstants.PORTALBROWSINGMSGS);
		String desc = space.GetString(1930, ResourceConstants.PORTALBROWSINGMSGS);
		
		HashtableDataObject nameAsc = new HashtableDataObject();
		nameAsc.SetData(DATA_OBJECTNAME, XPResourceManager.FormatMessage(asc, name));
		nameAsc.SetData(DATA_OBJECTID, PT_INTRINSICS.PT_PROPERTY_OBJECTNAME+"");
		nameAsc.SetData(DATA_DEFAULTPROP, "true");
		if ( defPropId == PT_INTRINSICS.PT_PROPERTY_OBJECTNAME ) {
			nameAsc.SetData(DATA_DEFAULTORDERBY, "true");
		}
		
		HashtableDataObject nameDesc = new HashtableDataObject();
		nameDesc.SetData(DATA_OBJECTNAME, XPResourceManager.FormatMessage(desc, name));
		nameDesc.SetData(DATA_OBJECTID, -PT_INTRINSICS.PT_PROPERTY_OBJECTNAME+"");
		nameDesc.SetData(DATA_DEFAULTPROP, "true");
		if ( defPropId == -PT_INTRINSICS.PT_PROPERTY_OBJECTNAME ) {
			nameDesc.SetData(DATA_DEFAULTORDERBY, "true");
		}
		
		HashtableDataObject modAsc = new HashtableDataObject();
		modAsc.SetData(DATA_OBJECTNAME, XPResourceManager.FormatMessage(asc, modified));
		modAsc.SetData(DATA_OBJECTID, PT_INTRINSICS.PT_PROPERTY_OBJECTLASTMODIFIED+"");
		modAsc.SetData(DATA_DEFAULTPROP, "true");
		if ( defPropId == PT_INTRINSICS.PT_PROPERTY_OBJECTLASTMODIFIED ) {
			modAsc.SetData(DATA_DEFAULTORDERBY, "true");
		}
		
		HashtableDataObject modDesc = new HashtableDataObject();
		modDesc.SetData(DATA_OBJECTNAME, XPResourceManager.FormatMessage(desc, modified));
		modDesc.SetData(DATA_OBJECTID, -PT_INTRINSICS.PT_PROPERTY_OBJECTLASTMODIFIED+"");
		modDesc.SetData(DATA_DEFAULTPROP, "true");
		if ( defPropId == -PT_INTRINSICS.PT_PROPERTY_OBJECTLASTMODIFIED ) {
			modDesc.SetData(DATA_DEFAULTORDERBY, "true");
		}
		
		
		list.Add(nameAsc);
		list.Add(nameDesc);
		list.Add(modAsc);
		list.Add(modDesc);
		
		return list;
	}
	
	/**
	 * Parses a comma separated string with integer values and returns the integer values
	 * in an array
	 * 
	 * @param attrib	Tag Attribute with the comma separated string
	 * @return
	 */
	protected int[] ParsePropertyIDString(ATagAttribute attrib)
	{
		String ids = GetTagAttributeAsString(attrib);
		XPStringTokenizer list = new XPStringTokenizer(ids, ",", false);
		int numIds = list.GetTokenCount();
		int[] idList = new int[numIds];
		int index = 0;
		while ( list.HasMoreTokens() ) {
			String strId = list.GetNextToken();
			int id = XPConvert.ToInteger(strId);
			idList[index] = id;
			index++;
		}
		return idList;
	}
	
	/**
	 * Helper method to compare two integer values and returns a string with "true"
	 * if the two values are identical, "false" otherwise
	 * 
	 * @param propId1
	 * @param propId2
	 * @return
	 */
	public String GetSelectedStatus(String propId1, int propId2) 
	{
		return ( XPConvert.ToInteger(propId1) == propId2) ? "true" : "false";
	}
	
	@Override
	public ATag Create() 
	{
		return new PortletSortPropertyDataTag();
	}

}
